/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         Agl.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;
import com.memoire.script.*;

import java.util.*;
import java.io.*;
import java.net.*;

public class Agl
{
  private String  _sortie_    ="-";
  private String  _error_     ="";
  private String  _repertoire_=System.getProperty("user.dir");
  private boolean _generation_=false;
  private boolean _progres_   =false;
  private boolean _coloration_=false;

  private ScriptInterpreterInterface _si_=null;
  private String[]                   args_;

  public Agl(String[] _args)
  {
    args_=_args;
  }

  public void run() throws Exception
  {
    if(AglOptions.terminal==null)
    {
      AglOptions.terminal=new AglTerminalSystem();
      AglOptions.parse="Java";//AglParse.langage("Java");
      AglOptions.code ="Java";//AglCode .langage("Java");
      _generation_=true;
    }

    AglProjet.reinit(false);

    // AglOptions.trace=true;
    // Class.forName("AglPatronType");

    if(args_.length==0)
    {
      AglOptions.terminal.sortie("-------------------------------------------------------------------------------");
      AglOptions.terminal.sortie("Alma version "+AglOptions.VERSION+" ["+AglOptions.DATE+"]");
      AglOptions.terminal.sortie("Distribu sous les termes de la GNU Public License v2");
      AglOptions.terminal.sortie("Guillaume Desnoix (c)1998-2001 <guillaume@desnoix.com>");
      AglOptions.terminal.sortie("Tous droits rservs par l'auteur et les contributeurs");
      AglOptions.terminal.sortie("http://www.memoire.com/guillaume-desnoix/alma/");
      AglOptions.terminal.sortie("--------------------------------------------------------------------------------");
      AglOptions.terminal.sortie("Syntaxe: java com.memoire.agl.Agl [options] [<fichier-entree> [...]]");
      AglOptions.terminal.sortie("  -a                      slectionne le mode applet");
      AglOptions.terminal.sortie("  -f                      charge les classes fondamentales");
      AglOptions.terminal.sortie("  -fm <module>            dfinit le filtre de module");
      AglOptions.terminal.sortie("  -fc <classe>            dfinit le filtre de classe");
      AglOptions.terminal.sortie("  -i  <format>            slectionne le format des images Gif Jpeg Ppm");
      AglOptions.terminal.sortie("  -lc                     rappelle les classes utilises");
      AglOptions.terminal.sortie("  -lp                     rappelle les patrons");
      AglOptions.terminal.sortie("  -lr                     rappelle les relations");
      AglOptions.terminal.sortie("  -o                      slectionne le fichier de sortie");
      AglOptions.terminal.sortie("  -p <language-entree>    slectionne le langage d'analyse");
      AglOptions.terminal.sortie("                          Cpp Fortran Java JavaP Jsp Mdl Pascal Taglib Xmi Xsd");
      AglOptions.terminal.sortie("  -g <language-sortie>    slectionne le langage de gnration");
      AglOptions.terminal.sortie("                          Autotest Bdl C Cpp Delphi DjaPaquets DocHtml Eiffel");
      AglOptions.terminal.sortie("                          Gui H Hpp Idl Jasmin Java JniH JniC Lisp Python");
      AglOptions.terminal.sortie("                          Naturel Odl Qualite Resume Sql2 Sql3 Tom UmlHierarchie");
      AglOptions.terminal.sortie("                          UmlRelations Uxf Xmi XmlProto Xsd");
      AglOptions.terminal.sortie("  -s <clef>=<valeur>      dfinit une proprit");
      AglOptions.terminal.sortie("  -v -vv                  slectionne le mode (trs) verbeux");
      AglOptions.terminal.sortie("  -V                      affiche la version");
      AglOptions.terminal.sortie("  --coloration            colore la sortie");
      AglOptions.terminal.sortie("  --comments              analyse des commentaires");
      AglOptions.terminal.sortie("  --literals              dfinit les litraux");
      AglOptions.terminal.sortie("  --header                affiche l'entte");
      AglOptions.terminal.sortie("  --progress              affiche la progression");
      AglOptions.terminal.sortie("  --regexp                applique les expressions rgulires");
      AglOptions.terminal.sortie("  --no-<option>           positionne l'option  faux");
      AglOptions.terminal.sortie("  --css <url>             choisit la feuille de style");
      AglOptions.terminal.sortie("  --script <interpreteur> lance un interpreteur");
      AglOptions.terminal.sortie("  --eval <file>           value un script");
      AglOptions.terminal.sortie("  --load <file>           charge un projet");
      AglOptions.terminal.sortie("  --save <file>           sauve un projet");
      AglOptions.terminal.sortie("");
      return;
    }

    for(int i=0;i<args_.length;i++)
    {
      AglOptions.terminal.progres(5+95*i/args_.length);

      if(args_[i]==null) continue;

      if(args_[i].startsWith("-")&&!args_[i].startsWith("--")&&(args_[i].length()>3))
	  args_[i]="-"+args_[i];

      if(args_[i].startsWith("--no")&&!args_[i].startsWith("--no-"))
	  args_[i]="--no-"+args_[i].substring(4);

      if(  args_[i].equals("-m")
	 ||args_[i].equals("-r")
	 ||args_[i].equals("-u")
	 ||args_[i].equals("-w")
	 ||args_[i].equals("-expreg"))
	AglOptions.terminal.erreur("L'option "+args_[i]+" est obsolte");

           if(args_[i].equals("-a" )||args_[i].equals("--applet"        )) AglOptions.applet         =true;
      else if(args_[i].equals("-v" )||args_[i].equals("--verbose"       )) AglOptions.suivi          =true;
      else if(args_[i].equals("-lc")||args_[i].equals("--list-classes"  )) AglOptions.rappelClasses  =true;
      else if(args_[i].equals("-lp")||args_[i].equals("--list-patterns" )) AglOptions.rappelPatrons  =true;
      else if(args_[i].equals("-lr")||args_[i].equals("--list-relations")) AglOptions.rappelRelations=true;

      else if(args_[i].equals("--no-applet"                            )) AglOptions.applet         =false;
      else if(args_[i].equals("--no-list-classes"                      )) AglOptions.rappelClasses  =false;
      else if(args_[i].equals("--no-list-patterns"                     )) AglOptions.rappelPatrons  =false;
      else if(args_[i].equals("--no-list-relations"                    )) AglOptions.rappelRelations=false;

      else if(args_[i].equals("-vv")||args_[i].equals("--very-verbose"))
	  AglOptions.suivi=AglOptions.trace=true;
      else if(args_[i].equals("--no-verbose"))
	  AglOptions.suivi=AglOptions.trace=false;

      else if(args_[i].equals("-V")||args_[i].equals("--version"))
      {
	AglOptions.terminal.sortie("Alma version "+AglOptions.VERSION);
      }
      else if(args_[i].equals("-s" )||args_[i].equals("--set"))
      {
	i++;
	int j=args_[i].indexOf('=');
	if(j<0)
	  AglOptions.terminal.erreur("Le format de l'option -s a chang.");

	String k=args_[i].substring(0,j);
	String v=args_[i].substring(j+1);
        AglOptions.terminal.trace("\nProprit: "+k+"="+v);
	AglPreferences.AGL.putStringProperty(k,v);
      }
      else if(args_[i].equals("--no-set")||args_[i].equals("--unset"))
      {
	i++;
        AglOptions.terminal.trace("\nProprit: "+args_[i]+" supprime");
	AglPreferences.AGL.removeProperty(args_[i]);
      }
      else if(args_[i].equals("-i" )||args_[i].equals("--image-format"))
      {
	i++;
        AglOptions.terminal.trace("\nFormat des images: "+args_[i]);
	AglOptions.formatImage=args_[i];
      }
      else if(args_[i].equals("--css"))
      {
	i++;
        AglOptions.terminal.trace("\nFeuille de style: "+args_[i]);
	AglOptions.styleHtml=args_[i];
      }
      else if(args_[i].equals("-fm")||args_[i].equals("--filter-module"))
      {
	i++;
        AglOptions.terminal.trace("\nFiltre de module: "+args_[i]);
	AglOptions.module=args_[i];
      }
      else if(args_[i].equals("-fc")||args_[i].equals("--filter-class"))
      {
	i++;
        AglOptions.terminal.trace("\nFiltre de classe: "+args_[i]);
	AglOptions.classe=args_[i];
      }
      else if(args_[i].equals("-bc")||args_[i].equals("--build-conditions"))
      {
        AglOptions.terminal.trace("\nConstruction des conditions");
	AglCondition.construireConditions();
      }
      else if(args_[i].equals("-br")||args_[i].equals("--build-relations"))
      {
        AglOptions.terminal.trace("\nConstruction des relations");
	AglRelation.detruireRelationsGenerees();
	AglRelation.construireRelationsHeritage();
	AglRelation.construireRelationsImplantation();
	AglRelation.construireRelationsContenance();
	AglRelation.construireRelationsUtilisation();
      }
      else if(args_[i].equals("-bp")||args_[i].equals("--build-patterns"))
      {
        AglOptions.terminal.trace("\nConstruction des patrons");
	AglPatron.construirePatrons();
	/*
	AglOptions.terminal.erreur
	  ("L'option --build-patterns n'est pas encore implante");
	*/
      }
      else
      if(args_[i].equals("-d")||args_[i].equals("--directory"))
      {
        i++;
        AglOptions.terminal.trace
	  ("\nSlection du rpertoire de sortie: "+args_[i]);
	_repertoire_=args_[i];
	if(!new File(_repertoire_).exists())
	  AglOptions.terminal.erreur
	    ("Le rpertoire "+_repertoire_+" n'existe pas");
      }
      else
      if(args_[i].equals("-o")||args_[i].equals("--output"))
      {
        i++;
        AglOptions.terminal.trace
	  ("\nSlection du fichier de sortie: "+args_[i]);
	_sortie_=args_[i];
      }
      else
      if(args_[i].equals("-p")||args_[i].equals("--parser"))
      {
        i++;
        AglOptions.terminal.trace
	  ("\nSlection du langage d'analyse: "+args_[i]);
	AglOptions.parse=args_[i];
      }
      else
      if(args_[i].equals("-g")||args_[i].equals("--generator"))
      {
        i++;
        AglOptions.terminal.trace
	  ("\nSlection du langage de gnration: "+args_[i]);
	AglOptions.code=args_[i];
      }
      else
      if(args_[i].equals("--comments"))
      {
        AglOptions.analyserCommentaires=true;
      }
      else
      if(args_[i].equals("--no-comments"))
      {
        AglOptions.analyserCommentaires=false;
      }
      else
      if(args_[i].equals("--literals"))
      {
        AglOptions.definirLiteraux=true;
      }
      else
      if(args_[i].equals("--no-literals"))
      {
        AglOptions.definirLiteraux=false;
      }
      else
      if(args_[i].equals("--regexp"))
      {
        AglOptions.appliquerExpReg=true;
      }
      else
      if(args_[i].equals("--no-regexp"))
      {
        AglOptions.appliquerExpReg=false;
      }
      else
      if(args_[i].equals("--header"))
      {
	AglOptions.entete=true;
      }
      else
      if(args_[i].equals("--no-header" ))
      {
	AglOptions.entete=false;
      }
      else
      if(args_[i].equals("--progress"))
      {
        _progres_=true;
      }
      else
      if(args_[i].equals("--no-progress"))
      {
        _progres_=false;
      }
      else
      if(args_[i].equals("--coloration"))
      {
        _coloration_=true;
      }
      else
      if(args_[i].equals("--no-coloration"))
      {
        _coloration_=false;
      }
      else
      if(args_[i].equals("--script"))
      {
        i++;

	if(AglOptions.application!=null)
	{
	  AglOptions.terminal.trace
	    ("--script ignored");
	}
	else
	{
	  AglOptions.terminal.trace
	    ("\nSlection du langage de script: "+args_[i]);
	  _generation_=false;

	  /*GCJ-BEGIN*/
	  _si_=ScriptInterpreter.language(args_[i]);
	  if(_si_==null)
	    AglOptions.terminal.erreur
	      (args_[i]+" n'est pas un langage de script");
	  
	  _si_.start();
	  _si_.addVariable("interpreter",_si_);
	  /*GCJ-END*/
	}
      }
      else
      if(args_[i].equals("--eval"))
      {
        i++;
	
	if(AglOptions.application!=null)
	{
	  AglOptions.terminal.trace
	    ("--eval ignored");
	}
	else
	{
	  if(_si_==null)
	    AglOptions.terminal.erreur
	      ("Le langage de script n'a pas t dfini");
	  else
	  {
	    String nom=args_[i];
	    _generation_=true;

	    try
	    {
	      Reader in =null;

	      if(AglOptions.applet)
		in=new InputStreamReader
		  ((new URL(nom)).openStream());
	      else
		in=new FileReader(nom);

	      LineNumberReader rin=new LineNumberReader(in);
	      String           s  ="";
	      while(rin.ready())
	      {
		String l=rin.readLine();
		if(l==null) break;
		s+=l+'\n';
	      }

	      _si_.eval(s);
	    }
	    catch (IOException ex)
	    {
	      AglOptions.terminal.erreur("Le script "+nom+" est introuvable");
	      return;
	    }
	  }
	}
      }
      else
      if(args_[i].equals("-f")||args_[i].equals("--load-fundamental"))
      {
	AglOptions.classesFondamentales=true;

	String nom=AglOptions.parse;//.langage();
        AglOptions.terminal.suivi
	  ("\nChargement des classes fondamentales "+nom+" ...");
        try
        {
          Reader in=null;
          if(AglOptions.applet)
	    in=new InputStreamReader
	      ((new URL(nom+"ClassesFondamentales.txt")).openStream());
          else
	    in=new FileReader(nom+"ClassesFondamentales.txt");

          LineNumberReader rin=new LineNumberReader(in);
          while(rin.ready())
          {
            String l=rin.readLine();
            String nc=l.substring(0,l.indexOf(' '));
            String nm=l.substring(l.indexOf(' ')+1,l.length());

            AglClasse c=new AglClasse(new AglNom(nc));
            AglType   m=AglType.cherche(nm);
            if(m==null) m=new AglClasse(new AglNom(nm));
            c.meres.noms.addElement(m.nom);
          }
        }
        catch (Exception ex)
	{
	  AglOptions.terminal.erreur
	    (""+ex.getMessage()+" [Agl.load-fundamental]");
	  return;
	}
      }
      else
      if(args_[i].equals("-c")||args_[i].equals("--config"))
      {
	String acf=args_[i+1];
        AglOptions.terminal.suivi
	  ("\nChargement de la configuration "+acf+" ...");

	Vector vs=new Vector();
	for(int j=0;j<i;j++)
	    vs.addElement(args_[j]);

        try
        {
          Reader in=null;
          if(AglOptions.applet)
	    in=new InputStreamReader
	      ((new URL(acf)).openStream());
          else
	    in=new FileReader(acf);

	  StreamTokenizer st=new StreamTokenizer(in);

	  st.resetSyntax();
	  st.eolIsSignificant(false);
	  /*
	  st.slashSlashComments(true);
	  st.slashStarComments (true);
	  */
	  st.wordChars(0,255);
	  st.quoteChar('"');
	  st.quoteChar('\'');
	  st.commentChar('#');
	  st.whitespaceChars(' ',' ');
	  st.whitespaceChars('\t','\t');
	  st.whitespaceChars('\n','\n');
	  st.whitespaceChars('\r','\r');
	  st.whitespaceChars((char)12,(char)12);

	  while(true)
	  {
	    String a=null;
	    st.nextToken();
	    switch(st.ttype)
	    {
	    case StreamTokenizer.TT_EOF   : a=null;       break;
	    case StreamTokenizer.TT_WORD  : a=st.sval;    break;
	    //case StreamTokenizer.TT_NUMBER: a=""+st.nval; break;
	    case '"':
	    if(st.sval!=null) a=st.sval;
	    else              a="\"";
	    break;
	    case '\'':
		if(st.sval!=null) a=st.sval;
		else              a="'";
		break;
	    default: a=""+(char)st.ttype; break;
	    }
	    if(a==null) break;
	    vs.addElement(a);
	    AglOptions.terminal.trace("<<< "+a);
	  }
	}
        catch (Exception ex)
	{
	  AglOptions.terminal.erreur
	    (""+ex.getMessage()+" [Agl.config]");
	  return;
	}

	for(int j=i+2;j<args_.length;j++)
	    vs.addElement(args_[j]);

	int ls=vs.size();
	args_=new String[ls];
	for(int j=0;j<ls;j++)
	  args_[j]=(String)vs.elementAt(j);
	i--;
      }
      else
      if(args_[i].equals("--load"))
      {
        i++;
	AglProjet.load(args_[i]);
      }
      else
      if(args_[i].equals("--save"))
      {
        i++;
	AglProjet p=new AglProjet();
	p.nom=args_[i];
	AglProjet.save(p,args_[i]);
      }
      else
      if(!args_[i].equals("-")&&args_[i].startsWith("-"))
      {
	AglOptions.terminal.erreur("L'option "+args_[i]+" n'existe pas");
      }
      else
      {
        if(AglOptions.application!=null)
	  AglOptions.terminal.message("Analyse de "+args_[i]);
	else
        if(_progres_)
	{
	    String n=args_[i];
	    int    m=n.lastIndexOf(File.separator);
	    if(m>=0) n=n.substring(m+1);
	    System.err.print("\r"+(5+85*i/args_.length)+"% "+
			     (n+"                                        ").
			     substring(0,40)+"\r");
	}
	else
	  AglOptions.terminal.trace("\nAnalyse de "+args_[i]+" ...");

	InputStream sin=null;
        try
        {
          /*
	  if(AglOptions.applet) sin=(new URL(args_[i])).openStream();
          else                  sin=new FileInputStream(args_[i]);
	  */

	  if(args_[i].equals("-"))
	  {
	    sin=System.in;
	  }
	  else
	  {
	    URL url=null;
	    try { url=new URL(args_[i]); }
	    catch(MalformedURLException ex1)
	    {
	      try
	      {
		sin=new FileInputStream(args_[i]);
	      }
	      catch(IOException ex2)
	      {
		AglOptions.terminal.avertissement
		  ("Open file stream failed: "+args_[i]);
	      }
	      /*
	      try
	      {
		url=new URL("file","",new File(args_[i]).getAbsolutePath());
	      }
	      catch(MalformedURLException ex2)
	      {
		AglOptions.terminal.avertissement
		  ("URL invalide: "+new File(args_[i]).getAbsolutePath());
		url=null;
	      }
	      */
	    }
	    
	    if((sin==null)&&(url!=null))
	    {
	      try { sin=url.openStream(); }
	      catch(Exception ex)
	      {
		AglOptions.terminal.avertissement
		  ("Open URL stream failed: "+url);
	      }
	    }
	  }

	  if(sin!=null)
	  {
	    AglParse p=AglOptions.parse();
	    p.initStream(sin,args_[i]);

	    try { p.parseFichier(); }
	    catch(RuntimeException re)
	    {
	      _error_+="[ An error occured when analysing "+args_[i]+" ]\n";
	      re.printStackTrace();
	    }

	    sin.close();
	  }
	  else
	  {
	    AglOptions.terminal.erreur("Impossible de lire "+args_[i]);
	  }  
        }
        catch (Exception ex1)
	{
	  if(sin!=null)
	  {
	    try { sin.close(); }
	    catch(IOException ex2) { }
	  }
	  ex1.printStackTrace();
	  AglOptions.terminal.erreur(""+ex1+" [Agl]");
        }
      }
    }

    AglOptions.terminal.progres(100);
    if(_progres_)
      System.err.print("                                             \r");

    if(_generation_)
    {
      AglOptions.terminal.progres(0);
      if(!_sortie_.equals("null"))
      {
	AglOptions.terminal.trace("\nGnration ...");
	AglProjet p=new AglProjet();

	/*
	AglProjet.saveYapodXml(p,"essai.xml");
	AglProjet.reinit(true);
	p=AglProjet.loadYapodXml("essai.xml");
	*/

	AglCode     c=AglOptions.code();
	AglSortie[] s=c.code(p);

	for(int k=0;k<s.length;k++)
	{
	  if(s[k].donnee instanceof String)
	  {
	    String r=(String)s[k].donnee;

	    if(AglOptions.appliquerExpReg)
	    {
	      AglOptions.terminal.trace("\nRecherche/remplacement ...");
	      r=c.appliqueExpReg(r);
	    }

	    if(_coloration_)
	    {
	      r=AglCodeSrcHtml.coloration(r,c.syntaxe(),p.nom,false,false,true);
	    }

	    if(!"".equals(_error_))
	    {
	      if("Html".equals(c.syntaxe())||_coloration_)
		_error_=FuLib.replace(_error_,"\n","<BR>\n");
	      r=_error_+r;
	      _error_="";
	    }

	    if(_sortie_.equals("-"))
	    {
	      if(!"".equals(s))
		AglOptions.terminal.sortie(r);
	    }
	    else
	    if(!AglOptions.applet)
	    {
	      File f=null;

	      if("+".equals(_sortie_))
	      {
		if(s[k].fichier!=null) f=new File(s[k].fichier);
		else                   f=new File("output"+k);
	      }
	      else
		f=new File(_sortie_);

	      if(!f.isAbsolute())
		f=new File(_repertoire_,f.getCanonicalPath());

	      String a=f.getCanonicalPath();
	      //System.err.println("A="+a);

	      if(f.exists())
	        AglOptions.terminal.erreur
		  ("Le fichier "+a+" existe dj");

	      /*
	      if(_sortie_.startsWith("file:"))
	      {
	      _sortie_=_sortie_.substring(5);
	      _sortie_=_sortie_.substring(_sortie_.indexOf('/'));
	      }
	      */

	      AglOptions.terminal.suivi("\nEcriture de "+a);

	      FileWriter sout=null;
	      try
	      {
		sout=new FileWriter(a,true); //_sortie_);
		sout.write(r);
		sout.flush();
		sout.close();
	      }
	      catch(IOException ex1)
	      {
		if(sout!=null)
	        {
	          try { sout.close(); }
		  catch(IOException ex2) { }
		}
		ex1.printStackTrace();
		//AglOptions.terminal.erreur("Impossible d'crire "+a);
		AglOptions.terminal.erreur(""+ex1+" [Agl]");
	      }
	    }
	  }
	  else
	  {
	    AglOptions.terminal.avertissement
		("Save binary object: not yet implemented\n"
		 +s[k].toString());
	  }
	}
      }
    }
    else
    if(_si_!=null)
    {
      _si_.run();
    }

    AglOptions.terminal.progres(0);
  }

  public final static void main(String _args[])
  {
    /*
    Thread.currentThread().setName("Agl main thread");
    Thread.currentThread().setPriority(Thread.NORM_PRIORITY-1);
    */

    try
    {
      new Agl(_args).run();
    }
    catch(Throwable th)
    {
      th.printStackTrace();
      System.err.println("Please report this bug to alma-bug@memoire.com");
    }
    finally
    {
      if(AglOptions.application==null)
	System.exit(0); // swing thread
    }
  }
}
