/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglClasse.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglClasse extends AglType
{
  public AglListeNoms meres;
  public AglListeNoms interfaces;
  public AglListeNoms exceptions;

  public AglTableOrdonnee attributs;
  public AglTableOrdonnee constructeurs;
  public AglTableOrdonnee methodes;

  public AglVecteur    invariants;

  public AglVisibilite visibilite;
  public boolean       elementaire;
  public boolean       normale;     // faux si interface
  public boolean       complexe;    // type complexe comme dans Xsd 

  public AglBloc       obinit;
  public AglBloc       clinit;

  public AglClasse() // yapod
  {
    this(new AglNom(AglLib.genereNom("classe")));
  }

  public AglClasse(AglNom _nom)
  {
    super(_nom);

    meres     =new AglListeNoms();
    interfaces=new AglListeNoms();
    exceptions=new AglListeNoms();

    attributs    =new AglTableOrdonnee();
    constructeurs=new AglTableOrdonnee();
    methodes     =new AglTableOrdonnee();

    invariants   =new AglVecteur();

    visibilite   =new AglVisibilite();
    elementaire  =true;
    normale      =true;
    complexe     =true;

    obinit       =new AglBloc();
    clinit       =new AglBloc();
  }

  public AglAttribut chercheAttributParNom(String _nom)
  {
    AglAttribut r=null;

    for(Enumeration e=attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut a=(AglAttribut)e.nextElement();
      if(_nom.equals(a.nom))
	{ r=a; break; }
    }

    return r;
  }

  public AglAttribut chercheAttributParSignature(String _signature)
  {
    AglAttribut r=null;

    for(Enumeration e=attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut a=(AglAttribut)e.nextElement();
      if(_signature.equals(a.signature()))
	{ r=a; break; }
    }

    return r;
  }

  public AglConstructeur chercheConstructeurParSignature(String _signature)
  {
    AglConstructeur r=null;

    for(Enumeration e=constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur c=(AglConstructeur)e.nextElement();
      if(_signature.equals(c.signature()))
	{ r=c; break; }
    }

    return r;
  }

  public AglMethode chercheMethodeParSignature(String _signature)
  {
    AglMethode r=null;

    for(Enumeration e=methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode m=(AglMethode)e.nextElement();
      if(_signature.equals(m.signature()))
	{ r=m; break; }
    }

    return r;
  }

  public String toString()
  {
    return "CLASSE("+nom+")";
  }
}
