/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCode.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;
import com.memoire.re.*;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

public abstract class AglCode
{
  protected boolean LIGNES;
  protected String  TABULATION;
  protected boolean COMMENTAIRES;
  protected boolean PBCFULL;
  protected String  PBCROOT;
  protected int     indentation;

  protected AglCode()
  {
    String l=langage().toLowerCase();
    LIGNES=AglPreferences.AGL.getBooleanProperty
	(l+".code.lines",AglPreferences.AGL.getBooleanProperty
	 ("code.lines",false));
    TABULATION=decodeTabulation(AglPreferences.AGL.getStringProperty
	(l+".code.tabulation",AglPreferences.AGL.getStringProperty
	 ("code.tabulation","  ")));
    COMMENTAIRES=AglPreferences.AGL.getBooleanProperty
	(l+".code.comments",AglPreferences.AGL.getBooleanProperty
	 ("code.comments",true));
    PBCFULL=AglPreferences.AGL.getBooleanProperty
	(l+".code.pbcfull",
	 AglPreferences.AGL.getBooleanProperty("code.pbcfull",true));
    PBCROOT=AglPreferences.AGL.getStringProperty
	(l+".code.pbcroot",
	 AglPreferences.AGL.getStringProperty
	 ("code.pbcroot","com.memoire.pbc.PbcLib."));
    indentation=0;
  }

  private static String decodeTabulation(String _s)
  {
    String r=_s;

    if(!"".equals(r))
    {
      r=FuLib.replace(r,"\\n","\n");
      r=FuLib.replace(r,"\\t","\t");
      if(Character.isDigit(r.charAt(0)))
      {
	try
	{
	  int n=Integer.parseInt(r);
	  r="";
	  for(int i=0;i<n;i++) r+=" ";
	}
	catch(Exception ex) { }
      }
      else
      if(r.indexOf("%")>=0) r=URLDecoder.decode(r);
    }

    /*
      System.err.println("TABULATION="+_s+"|"+r);
      System.err.println(getClass());
      try { throw new Exception();}
      catch(Exception ex) { ex.printStackTrace(); }
    */

    return r;
  }

  public abstract String   extension();
  public abstract String[] extensions();

  public static final void avertissement(String _s)
  { AglOptions.terminal.avertissement(_s); }

  public static final void syntaxe(String _s)
  { AglOptions.terminal.syntaxe(_s); }

  public static final void erreur(String _s)
  { AglOptions.terminal.erreur(_s); }

  public static final void suivi(String _s)
  { AglOptions.terminal.suivi(_s); }

  public static final void trace(String _s)
  { AglOptions.terminal.trace(_s); }

  public static final void sortie(String _s)
  { AglOptions.terminal.sortie(_s); }

  public static final void progres(int _v)
  { AglOptions.terminal.progres(_v); }

  public final String langage()
  {
    String r=getClass().getName();
    int    i=r.indexOf("AglCode");
    r=r.substring(i+7);
    return r;
  }

  public final static boolean disponible(String n)
  {
    boolean r=true;
    try
    {
      String p=AglCode.class.getName();
      int    i=p.indexOf("AglCode");

      p=p.substring(0,i+7);
      Class.forName(p+n);
    }
    catch(Throwable ex) { r=false; }

    return r;
  }

  public final static AglCode langage(String n)
  {
    if(n==null) return null;

    AglCode r=null;

    String p=AglCode.class.getName();
    int    i=p.indexOf("AglCode");
    p=p.substring(0,i+7);

    try
    {
      Class c=Class.forName(p+n);
      //Class c=FuClassLoaderDedicated.load(p+n,AglCode.class);
      r=(AglCode) c.newInstance();
    }
    catch(ClassNotFoundException ex)
    {
      AglCode.avertissement("Classe manquante: "+p+n+".class");
      AglCode.erreur(n+" n'est pas un langage pour la gnration");
    }
    catch(Throwable th)
    {
      th.printStackTrace();
      AglCode.erreur(n+" n'est pas disponible pour la gnration");
    }

    return r;
  }

  protected String indente()
  {
    String r="";
    for(int i=0;i<indentation;i++) r+=TABULATION;
    return r;
  }

  protected final boolean global(AglType _type)
  {
    return _type==AglType.cherche("Global");
  }

  protected final boolean filtre(AglNom _nom)
  {
    boolean r=true;

    if(r&&optionModule()&&!"".equals(AglOptions.module))
      r=_nom.formeJava().startsWith(AglOptions.module);

    if(r&&optionClasse()&&!"".equals(AglOptions.classe))
      r=_nom.dernier().equals(AglOptions.classe);

    return r;
  }

  // Surchargeables

  public String syntaxe()
  {
    return langage();
  }

  protected final String pbc() { return ""; }

  protected String debutCS  () { return indente()+"// "; }
  protected String finCS    () { return "";              }
  protected String debutCM  () { return indente()+"/*";  }
  protected String specialCM() { return "*";             }
  protected String milieuCM () { return indente()+" * "; }
  protected String finCM    () { return indente()+" */"; }

  // Options

  public boolean optionFormatImage()
    { return false; }

  public boolean optionStyleHtml()
    { return false; }

  public boolean optionModule()
    { return true; }

  public boolean optionClasse()
    { return true; }

  public final boolean optionAppliquerExpReg()
  {
    return getClass().getResourceAsStream
      (langage()+"ExpReg.txt")!=null;
  }

  /**
   * charge les expressions regulieres.
   * @param  _fichier contenant les expregs
   * @return tableau de expregs
   */
  protected String[][] chargeExpReg(String _fichier)
  {
    AglVecteur mf=new AglVecteur(11,11);
    AglVecteur mr=new AglVecteur(11,11);

    try
    {
      suivi("Chargement des expressions rationnelles: "+_fichier);

      LineNumberReader rin=
	new LineNumberReader
	(new InputStreamReader
	 (new BufferedInputStream
	  (getClass().getResourceAsStream(_fichier))));

      String l,k,v;

      k=null;
      v=null;

      while(rin.ready())
      {
	l=rin.readLine();
	if(l==null) break;

	if(l.equals("")&&(k!=null)&&(v!=null))
	{
	  // System.err.println(k+"\n"+v);
	  mf.addElement(k);
	  mr.addElement(v);
	  k=null;
	  v=null;
	}
	else if(l.equals("")&&(k==null)&&(v==null)) ;
	else if(l.startsWith("#")) ;
	else if(k==null) k=l;
	else if(v==null) v=l;
	else // v+="\n"+l;
	  erreur("Mauvais format de fichier: "+_fichier);
      }

      if((k!=null)&&(v!=null))
      {
	// System.err.println(k+"\n"+v);
	mf.addElement(k);
	mr.addElement(v);
      }

    }
    catch(Exception ex) { trace(_fichier+" not found"); }

    String[][] r=new String[mf.size()][2];
    for(int i=0;i<mf.size(); i++)
    {
      r[i][0]=(String)mf.elementAt(i);
      r[i][1]=(String)mr.elementAt(i);
    }
    return r;
  }

  protected String[][] expreg_=null;

  /**
   * Cette expression renvoie un tableau d'expressions regulieres
   * de recherche/remplacement.
   * @return tableau 2D de motif/remplacement
   */
  public String[][] getExpReg()
  {
    if(expreg_==null)
      expreg_=chargeExpReg(langage()+"ExpReg.txt");

    return expreg_;
  }

  /**
   * Applique les expressions regulieres du langage.
   * @param s une chaine de caracteres
   * @return la chaine modifiee
   */
  public final String appliqueExpReg(String s)
  {
    RE         expreg=null;
    String[][] motifs=getExpReg();

    if(motifs==null) return s;
    
    /*GCJ-BEGIN*/
    for(int i=0;i<motifs.length;i++)
    {
      try
      {
	trace("ExpReg: "+motifs[i][0]+" --> "+motifs[i][1]);
	expreg=new RE(motifs[i][0],
		      RE.REG_MULTILINE,RESyntax.RE_SYNTAX_PERL5);
	s=expreg.substituteAll(s,motifs[i][1]);
      }
      catch(REException ex)
	{ erreur("Expression rgulire n"+i+" incorrecte: "+motifs[i][0]); }
    }
    /*GCJ-END*/

    return s;
  }

  protected final String entete(String _unite)
  {
    return entete(_unite,null);
  }

  protected final String entete(String _unite, String _lien)
  {
    return entete(_unite,"Guillaume Desnoix","guillaume@desnoix.com",_lien);
  }

  protected final String entete(String _unite, String _auteur, String _mel)
  {
    return entete(_unite,_auteur,_mel,null);
  }

  protected final String entete
    (String _unite, String _auteur, String _mel, String _lien)
  {
    if(!AglOptions.entete) return "";

    String date=new SimpleDateFormat
      ("yyyy-MM-dd",Locale.getDefault()).format(new Date());

    String r=debutCM()+"\n";
    r+=milieuCM()+"Entte : Alma\n";
    if(_unite!=null) r+=milieuCM()+"Unit  : "+_unite+"\n";
    r+=milieuCM()+"Date   : "+date+"\n";
    r+=milieuCM()+"Cible  : "+langage()+" ("+extension()+")\n";
    if(_lien !=null) r+=milieuCM()+"Lien   : "+_lien+"\n";
    r+=milieuCM()+"Auteur : "+_auteur+"\n";
    r+=milieuCM()+"Mel    : "+_mel+"\n";
    r+=milieuCM()+"Produit: "+AglOptions.PRODUIT+"\n";
    r+=milieuCM()+"Version: "+AglOptions.VERSION+"\n";
    r+=milieuCM()+"Site   : "+AglOptions.SITE+"\n";
    r+=finCM()+"\n";

    return r;
  }

  public String code(AglCommentaireSimple o)
  {
    if(  !COMMENTAIRES
       ||(o==null)
       ||(o.texte==null)
       ||o.texte.equals("")) return "";

    return debutCS()+o.texte+finCS()+"\n";
  }

  private final String coupeCommentaire(String _s)
  {
    String r="";
    while(_s.length()>74)
    {
      int i=_s.lastIndexOf(" ",72);
      if(i<0) break;
      r+=milieuCM()+_s.substring(0,i)+"\n";
      _s=_s.substring(i+1);
      }
    r+=milieuCM()+_s+"\n";
    return r;
  }

  public String code(AglCommentaireMulti o)
  {
    if(  !COMMENTAIRES
       ||(o==null)
       ||(o.clefs.size()==0)) return "";

    String mv=(String)o.clefs.get("@main");

    if(  (mv!=null)
       &&mv.startsWith("Entte : Alma")
       &&(o.clefs.size()==1)) return "";

    String r=debutCM();
    if(o.special) r+=specialCM();
    r+="\n";

    if(mv!=null)
      r+=coupeCommentaire(mv);

    for(Enumeration e=o.clefs.keys(); e.hasMoreElements(); )
    {
      String k=(String)e.nextElement();
      if(!"@main".equals(k))
      {
	AglVecteur l=(AglVecteur)o.clefs.get(k);
	for(int i=0;i<l.size();i++)
	  r+=coupeCommentaire(k+" "+(String)l.elementAt(i));
      }
    }

    r+=finCM()+"\n";
    return r;
  }

  public String code(AglPaquet o)
    { return code(o.nom); }

  public String code(AglParametreMoule o)
    { return code(o.type)+" "+code(o.nom); }

  public String code(AglMoule o)
  {
    String  r=debutCS()+"PATRON "+o.nom.dernier()+" <";
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglParametreMoule)e.nextElement());
    }
    r+=">"+finCS()+"\n";

    r+=code(o.declaration);
    return r;
  }

  public String code(AglCondition o)
  {
    String r=debutCM()+specialCM()+" ";

         if(o.pre&&o.post) r+="@invariant";
    else if(o.pre)         r+="@require";
    else if(o.post)        r+="@ensure";
    else                   r+="@check";

    if((o.langage!=null)&&!langage().equals(o.langage))
	r+="["+o.langage+"]";

    r+=" "+code(o.test)+finCM();
    return r;
  }

  public String code(AglInstructionCondition o)
  {
    String r=indente();
    if(PBCFULL)
    {
      String n=PBCROOT;
      AglCondition c=o.condition;
           if(c.pre&&c.post) n+="invariant";
      else if(c.pre)         n+="require";
      else if(c.post)        n+="ensure";
      else                   n+="check";

      String t=code(o.condition.test);
      String s=t;
      s=FuLib.replace(s,"\\","\\\\");
      s=FuLib.replace(s,"\"","\\\"");

      String f=null;
      if(c.unite()!=null) f=c.unite().fichier;
 
      r+=n+"(("+t+"),\""+s+"\","+f+","+c.ligne+");\n";
    }
    else
    {
      r+="assert("+code(o.condition.test)+");\n";
    }
    return r;
  }

  public final String code(AglPatron o)
  {
    String r=debutCM()+specialCM()+"\n";
    r+=milieuCM()+"@pattern "+o.type.nom;

    if(o.roles.size()>0)
    {
      r+="\n"+milieuCM()+"         (";
      boolean premier=true;
      for(Enumeration e=o.roles.keys(); e.hasMoreElements(); )
      {
	if(premier) premier=false;
	else        r+=",";
	r+="\n";
	r+=milieuCM()+"           ";
	String  k=(String)e.nextElement();
	Object  v=o.roles.get(k);
	String  t="";
	if(v instanceof AglType) t=code(((AglType)v).nom);
	else                     t=code(v);
	r+=t+":"+k;
      }
      r+="\n";
      r+=milieuCM()+"         )\n";
    }
    else
      r+="()\n";

    r+=milieuCM()+"         {\n";
    r+=milieuCM()+"           name="+o.nom+";\n";
    r+=milieuCM()+"         }\n";
    r+=finCM()+"\n";
    return r;
  }

  public String code(AglRelation o)
  {
    String r=debutCS()+o.getIntituleLong()+finCS()+"\n";
    return r;
  }

  public AglSortie[] code(AglProjet o)
  {
    Vector r=new Vector();
    int i,l;

    progres(10);
    i=0;
    l=o.unites.size();

    for(Enumeration e=o.unites.elements(); e.hasMoreElements(); )
    {
      AglUnite au=(AglUnite)e.nextElement();

      if(!au.cible)
      {
	/*
	  if(AglOptions.application==null)
	  sortie(code(au));
	  else
	*/
	String f;
	try
	{
	  f=new File(au.fichier).getName();
	  if(f==null) f=o.nom;
	  int k=f.lastIndexOf(".");
	  if(k>0) f=f.substring(0,k);
	}
	catch(Exception ex) { f=o.nom; }

	f+="."+extension();
	AglSortie s=new AglSortie("text/plain",syntaxe(),f,code(au));
	r.addElement(s);
	i++;
	progres(10+80*i/l);

	//System.err.println("GENERATION: "+(10+80*i/l)+"% "+au.fichier);
      }
    }

    if(AglOptions.rappelClasses)
    {
      String s=rappelleClasses();
      if(!"".equals(s))
	r.addElement(new AglSortie("text/plain",syntaxe(),null,s));
    }
    progres(93);

    if(AglOptions.rappelPatrons)
    {
      String s=rappellePatrons();
      if(!"".equals(s))
	r.addElement(new AglSortie("text/plain",syntaxe(),null,s));
    }
    progres(96);

    if(AglOptions.rappelRelations)
    {
      String s=rappelleRelations();
      if(!"".equals(s))
	r.addElement(new AglSortie("text/plain",syntaxe(),null,s));
    }
    progres(100);

    progres(0);

    AglSortie[] s=new AglSortie[r.size()];
    for(int j=0;j<s.length;j++) s[j]=(AglSortie)r.elementAt(j);
    return s;
  }

  protected String rappelleClasses()
  {
    String r="";

    r+="\n"+debutCM()+" Classes:\n"+milieuCM();

    String s="    ";
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
	if(ac.elementaire)
        {
          // if(ac.paquet!="") s+=ac.paquet;
          s+=code(ac.nom)+" ";
	}
	if(s.length()>60)
	{
          r+=s+"\n"+milieuCM();
	  s="    ";
	}
      }
    }

    r+=s+"\n";
    r+=finCM()+"\n";

    return r;
  }

  protected String rappellePatrons()
  {
    String r="";

    for(Enumeration e=AglPatron.table.elements(); e.hasMoreElements(); )
    {
      AglPatron ap=(AglPatron)e.nextElement();
      r+="\n"+code(ap);
    }

    return r;
  }

  protected String rappelleRelations()
  {
    String r="";

    r+="\n"+debutCS()+" Relations:"+finCS()+"\n";
    for(Enumeration e=AglRelation.table.elements(); e.hasMoreElements(); )
    {
      AglRelation ar=(AglRelation)e.nextElement();
      r+=code(ar);
    }

    return r;
  }

  public final String code(AglVecteur o)
  {
    if(o==null) return "";

    String r="";
    for(Enumeration e=o.elements(); e.hasMoreElements(); )
    {
      r+=code(e.nextElement());
      while(r.endsWith("\n\n")) r=r.substring(0,r.length()-1);
      if(!r.endsWith("\n")) r+="\n";

      /*
      String s=""+e.nextElement();
      int i; 
      while((i=s.indexOf('('))>=0) s=s.substring(i+1,s.length()-1);
      r+=s.trim()+'\n';
      */
    }
    return r;
  }

  public final String code(AglTableOrdonnee o)
  {
    if(o==null) return "";

    String r="";
    for(Enumeration e=o.elements(); e.hasMoreElements(); )
    {
      r+=code(e.nextElement());
      while(r.endsWith("\n\n")) r=r.substring(0,r.length()-1);
      if(!r.endsWith("\n")) r+="\n";

      /*
      String s=""+e.nextElement();
      int i; 
      while((i=s.indexOf('('))>=0) s=s.substring(i+1,s.length()-1);
      r+=s.trim()+'\n';
      */
    }
    return r;
  }

  public abstract String code(AglNom          o);
  public abstract String code(AglListeNoms    o);
  public abstract String code(AglUnite        o);

  public abstract String code(AglAttribut     o);
  public abstract String code(AglBloc         o);
  public abstract String code(AglClasse       o);
  public abstract String code(AglConstructeur o);
  public abstract String code(AglImport       o);
  public abstract String code(AglMethode      o);
  public abstract String code(AglParametre    o);
  public abstract String code(AglStruct       o);
  public abstract String code(AglType         o);
  public abstract String code(AglVisibilite   o);

  public abstract String code(AglExpressionGenerique     o);
  public abstract String code(AglExpressionAffectation   o);
  public abstract String code(AglExpressionAttribut      o);
  public abstract String code(AglExpressionBinaire       o);
  public abstract String code(AglExpressionDefVar        o);
  public abstract String code(AglExpressionEstUn         o);
  public abstract String code(AglExpressionFonction      o);
  public abstract String code(AglExpressionIndice        o);
  public abstract String code(AglExpressionInitArray     o);
  public abstract String code(AglExpressionMethode       o);
  public abstract String code(AglExpressionMultiple      o);
  public abstract String code(AglExpressionNew           o);
  public abstract String code(AglExpressionNewArray      o);
  public abstract String code(AglExpressionParenthesee   o);
  public abstract String code(AglExpressionTernaire      o);
  public abstract String code(AglExpressionTypage        o);
  public abstract String code(AglExpressionUnaireDroite  o);
  public abstract String code(AglExpressionUnaireGauche  o);

  public abstract String code(AglInstructionGenerique    o);
  public abstract String code(AglInstructionVide         o);
  public abstract String code(AglInstructionAffectation  o);
  public abstract String code(AglInstructionBloc         o);
  public abstract String code(AglInstructionBreak        o);
  public abstract String code(AglInstructionCatch        o);
  public abstract String code(AglInstructionContinue     o);
  public abstract String code(AglInstructionDefVar       o);
  public abstract String code(AglInstructionDoWhile      o);
  public abstract String code(AglInstructionExpression   o);
  public abstract String code(AglInstructionFinally      o);
  public abstract String code(AglInstructionFor          o);
  public abstract String code(AglInstructionGoto         o);
  public abstract String code(AglInstructionIf           o);
  public abstract String code(AglInstructionLabel        o);
  public abstract String code(AglInstructionProcedure    o);
  public abstract String code(AglInstructionReturn       o);
  public abstract String code(AglInstructionSwitch       o);
  public abstract String code(AglInstructionSynchronized o);
  public abstract String code(AglInstructionThrow        o);
  public abstract String code(AglInstructionTry          o);
  public abstract String code(AglInstructionWhile        o);

  public final String code(Object o)
  {
    String r="";
         if(o instanceof AglAttribut)          r=code((AglAttribut)         o);
    else if(o instanceof AglBloc)              r=code((AglBloc)             o);
    else if(o instanceof AglClasse)            r=code((AglClasse)           o);
    else if(o instanceof AglConstructeur)      r=code((AglConstructeur)     o);
    else if(o instanceof AglExpression)        r=code((AglExpression)       o);
    else if(o instanceof AglImport)            r=code((AglImport)           o);
    else if(o instanceof AglInstruction)       r=code((AglInstruction)      o);
    else if(o instanceof AglMethode)           r=code((AglMethode)          o);
    else if(o instanceof AglPaquet)            r=code((AglPaquet)           o);
    else if(o instanceof AglParametre)         r=code((AglParametre)        o);
    else if(o instanceof AglParametreMoule)    r=code((AglParametreMoule)   o);
    else if(o instanceof AglMoule)             r=code((AglMoule)            o);
    else if(o instanceof AglStruct)            r=code((AglStruct)           o);
    else if(o instanceof AglType)              r=code((AglType)             o);
    else if(o instanceof AglVisibilite)        r=code((AglVisibilite)       o);

    else if(o instanceof AglCommentaireSimple) r=code((AglCommentaireSimple)o);
    else if(o instanceof AglCommentaireMulti)  r=code((AglCommentaireMulti) o);
    else if(o instanceof AglCondition)         r=code((AglCondition)        o);
    else if(o instanceof AglPatron)            r=code((AglPatron)           o);
    else if(o instanceof AglRelation)          r=code((AglRelation)         o);

    else if(o instanceof AglNom)               r=code((AglNom)              o);
    else if(o instanceof AglListeNoms)         r=code((AglListeNoms)        o);
    else if(o instanceof AglUnite)             r=code((AglUnite)            o);
    else if(o instanceof AglVecteur)           r=code((AglVecteur)          o);
    else if(o instanceof AglTableOrdonnee)     r=code((AglTableOrdonnee)    o);

    else if(o instanceof String)               r=(String)o;
    else if(o==null) r="";
    else r=indente()+"Unknown Object : "+o.getClass().getName()+"\n";

    return r;
  }

  public String code(AglExpression o)
  {
    String r="";
         if(o instanceof AglExpressionGenerique)     r=code((AglExpressionGenerique)     o);
    else if(o instanceof AglExpressionAffectation)   r=code((AglExpressionAffectation)   o);
    else if(o instanceof AglExpressionAttribut)      r=code((AglExpressionAttribut)      o);
    else if(o instanceof AglExpressionBinaire)       r=code((AglExpressionBinaire)       o);
    else if(o instanceof AglExpressionDefVar)        r=code((AglExpressionDefVar)        o);
    else if(o instanceof AglExpressionEstUn)         r=code((AglExpressionEstUn)         o);
    else if(o instanceof AglExpressionFonction)      r=code((AglExpressionFonction)      o);
    else if(o instanceof AglExpressionIndice)        r=code((AglExpressionIndice)        o);
    else if(o instanceof AglExpressionInitArray)     r=code((AglExpressionInitArray)     o);
    else if(o instanceof AglExpressionMethode)       r=code((AglExpressionMethode)       o);
    else if(o instanceof AglExpressionMultiple)      r=code((AglExpressionMultiple)      o);
    else if(o instanceof AglExpressionNew)           r=code((AglExpressionNew)           o);
    else if(o instanceof AglExpressionNewArray)      r=code((AglExpressionNewArray)      o);
    else if(o instanceof AglExpressionParenthesee)   r=code((AglExpressionParenthesee)   o);
    else if(o instanceof AglExpressionTernaire)      r=code((AglExpressionTernaire)      o);
    else if(o instanceof AglExpressionTypage)        r=code((AglExpressionTypage)        o);
    else if(o instanceof AglExpressionUnaireDroite)  r=code((AglExpressionUnaireDroite)  o);
    else if(o instanceof AglExpressionUnaireGauche)  r=code((AglExpressionUnaireGauche)  o);
    else if(o==null) r="";
    else r=" Unknown Expression  : "+o.getClass().getName()+" ";
    return r;
  }

  public String code(AglInstruction o)
  {
    String r="";
         if(o instanceof AglInstructionGenerique)    r=code((AglInstructionGenerique)    o);
    else if(o instanceof AglInstructionCondition)    r=code((AglInstructionCondition)    o);
    else if(o instanceof AglInstructionVide)         r=code((AglInstructionVide)         o);
    else if(o instanceof AglInstructionAffectation)  r=code((AglInstructionAffectation)  o);
    else if(o instanceof AglInstructionBloc)         r=code((AglInstructionBloc)         o);
    else if(o instanceof AglInstructionBreak)        r=code((AglInstructionBreak)        o);
    else if(o instanceof AglInstructionCatch)        r=code((AglInstructionCatch)        o);
    else if(o instanceof AglInstructionContinue)     r=code((AglInstructionContinue)     o);
    else if(o instanceof AglInstructionDefVar)       r=code((AglInstructionDefVar)       o);
    else if(o instanceof AglInstructionDoWhile)      r=code((AglInstructionDoWhile)      o);
    else if(o instanceof AglInstructionExpression)   r=code((AglInstructionExpression)   o);
    else if(o instanceof AglInstructionFinally)      r=code((AglInstructionFinally)      o);
    else if(o instanceof AglInstructionFor)          r=code((AglInstructionFor)          o);
    else if(o instanceof AglInstructionGoto)         r=code((AglInstructionGoto)         o);
    else if(o instanceof AglInstructionIf)           r=code((AglInstructionIf)           o);
    else if(o instanceof AglInstructionLabel)        r=code((AglInstructionLabel)        o);
    else if(o instanceof AglInstructionProcedure)    r=code((AglInstructionProcedure)    o);
    else if(o instanceof AglInstructionReturn)       r=code((AglInstructionReturn)       o);
    else if(o instanceof AglInstructionSwitch)       r=code((AglInstructionSwitch)       o);
    else if(o instanceof AglInstructionSynchronized) r=code((AglInstructionSynchronized) o);
    else if(o instanceof AglInstructionThrow)        r=code((AglInstructionThrow)        o);
    else if(o instanceof AglInstructionTry)          r=code((AglInstructionTry)          o);
    else if(o instanceof AglInstructionWhile)        r=code((AglInstructionWhile)        o);
    else if(o==null) r="";
    else r=indente()+"Unknown Instruction : "+o.getClass().getName()+"\n";
    return r;
  }
}

