/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeAutotest.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public  class AglCodeAutotest extends AglCode
{
  private AglClasse _classe;
  private AglUnite  _unite;

  public String extension()
  {
    return "java";
  }

  public String[] extensions()
  {
    return new String[] { "java" };
  }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=".";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
         if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau) r+=code(((AglTypeTableau)o).type)+"[]";
    else r+=code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    String r="";

    if(o.visibilite.vpublic && !o.visibilite.vfinal)
    {
      if(o.type==AglTypePrimitif.tboolean)
      {
	r+=indente()+"boolean tmp_"+o.nom+"=o."+o.nom+"();\n";
	r+=indente()+"o."+o.nom+"=true;\n";
	r+=indente()+"if(o."+o.nom+"!=true) erreur(\""+o.nom+"\\n\");\n";
	r+="\n";
      }

      if(o.type==AglTypePrimitif.tchar)
      {
	r+=indente()+"char tmp_"+o.nom+"=o."+o.nom+"();\n";
	r+=indente()+"o."+o.nom+"=' ';\n";
	r+=indente()+"if(o."+o.nom+"!=' ') erreur(\""+o.nom+"\\n\");\n";
	r+="\n";
      }

      if(o.type==AglTypePrimitif.tbyte)
      {
	r+=indente()+"byte tmp_"+o.nom+"=o."+o.nom+"();\n";
	r+=indente()+"o."+o.nom+"=2;\n";
	r+=indente()+"if(o."+o.nom+"!=2) erreur(\""+o.nom+"\\n\");\n";
	r+="\n";
      }

      if(o.type==AglTypePrimitif.tinteger)
      {
	r+=indente()+"int tmp_"+o.nom+"=o."+o.nom+"();\n";
	r+=indente()+"o."+o.nom+"=2;\n";
	r+=indente()+"if(o."+o.nom+"!=2) erreur(\""+o.nom+"\\n\");\n";
	r+="\n";
      }

      if(o.type==AglTypePrimitif.tlong)
      {
	r+=indente()+"long tmp_"+o.nom+"=o."+o.nom+"();\n";
	r+=indente()+"o."+o.nom+"=2;\n";
	r+=indente()+"if(o."+o.nom+"!=2) erreur(\""+o.nom+"\\n\");\n";
	r+="\n";
      }

      if(o.type==AglTypePrimitif.tfloat)
      {
	r+=indente()+"float tmp_"+o.nom+"=o."+o.nom+"();\n";
	r+=indente()+"o."+o.nom+"=2.1;\n";
	r+=indente()+"if(o."+o.nom+"!=2.1) erreur(\""+o.nom+"\\n\");\n";
	r+="\n";
      }

      if(o.type==AglTypePrimitif.tdouble)
      {
	r+=indente()+"double tmp_"+o.nom+"=o."+o.nom+"();\n";
	r+=indente()+"o."+o.nom+"=2.2;\n";
	r+=indente()+"if(o."+o.nom+"!=2.2) erreur(\""+o.nom+"\\n\");\n";
	r+="\n";
      }
    }

    return r;
  }

  public String code(AglBloc o)
  {
    String r=indente()+"{\n";
    indentation++;

    int n=1;
    for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
    {
      AglInstruction ai=(AglInstruction)e.nextElement();
      r+=code(ai);
      n++;
    }

    indentation--;
    r+=indente()+"}\n";
    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;
    String r="/* ";
    // if(o.paquet!="") r+=o.paquet;
    r+=code(o.nom)+" ["+o.ligne+"] */\n\n";

    /*
    r+=code(o.visibilite);
    r+="class "+code(o.nom);

    r+="\n{\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation=0;
    r+="}\n";
    */

    return r;
  }

  public String code(AglClasse o)
  {
    _classe=o;
    
    indentation=1;
    String r=indente()+"/* ";
    r+=code(o.nom)+" ["+o.ligne+"] */\n\n";
    r+=indente()+"public static boolean teste_"+code(o.nom).replace('.','_')+"()\n";
    r+=indente()+"{\n";

    indentation=2;
    r+=indente()+"test(\""+code(o.nom)+"\");\n\n";
    r+=indente()+code(o.nom)+" o=null;\n\n";  
  
    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    r+=indente()+"return true;\n";
    indentation=1;
    r+=indente()+"}\n";
    return r;
  }

  public String code(AglConstructeur o)
  {
    indentation=2;
    String r="";

    if(o.parametres.size()==0)
    {
      r+=indente()+"try { o=new "+code(_classe.nom)+"(); }\n";
      r+=indente()+"catch(Exception ex) { erreur(\""+code(o.nom)+"()\"); }\n\n";
    }

    return r;
  }

  public String code(AglImport o)
  {
    indentation=0;
    String r="import "+code(o.nom)+";\n";
    return r;
  }

  public String code(AglMethode o)
  {
    if(!o.visibilite.vpublic) return "";

    indentation=2;
    String r="";

    AglType      ot=o.type;

    while(ot instanceof AglTypeAlias)
      ot=((AglTypeAlias)ot).type;
    
    String[] args=null;

    if(o.parametres.size()==0)
    {
      args=new String[] { "" };
    }
    else
    if(o.parametres.size()==1)
    {
      AglParametre ap=(AglParametre)o.parametres.elementAt(0);
      AglType      at=ap.type;

      while(at instanceof AglTypeAlias)
	at=((AglTypeAlias)at).type;

      if(at==AglTypePrimitif.tboolean)  args=new String[] { "true","false" };
      if(at==AglTypePrimitif.tchar)     args=new String[] { "' '" ,"'\\t'" };
      if(at==AglTypePrimitif.tbyte)     args=new String[] { "0","1","127","128","255" };
      if(at==AglTypePrimitif.tinteger)  args=new String[] { "0","1","10","1000","-1","-10","-1000" };
      if(at==AglTypePrimitif.tlong)     args=new String[] { "0l","1l","10l","1000l","-1l","-10l","-1000l" };
      if(at==AglTypePrimitif.tfloat)    args=new String[] { "0.f","1.f","10.f","1000.f","-1.f","-10.f","-1000.f" };
      if(at==AglTypePrimitif.tdouble)   args=new String[] { "0.d","1.d","10.d","1000.d","-1.d","-10.d","-1000.d" };

      if(at==AglType.cherche("String")) args=new String[] { "\"abcdef\"","\"\"" };
      if(at==AglType.cherche("Object")) args=new String[] { "new java.lang.Object()" };
      if(at==AglType.cherche("IObjet")) args=new String[] { "new fr.stcpmvn.dodico.objet.CObjet()" };

      if(at instanceof AglTypeTableau)
      {
	AglType et=((AglTypeTableau)at).type;

	while(et instanceof AglTypeAlias)
	  et=((AglTypeAlias)et).type;

	if(et==AglTypePrimitif.tboolean)  args=new String[] { "new boolean[] { true,false }" };
	if(et==AglTypePrimitif.tchar)     args=new String[] { "new char   [] { ' ','\\t'  }" };
	if(et==AglTypePrimitif.tbyte)     args=new String[] { "new byte   [] { 0,1,127,128,255 }" };
	if(et==AglTypePrimitif.tinteger)  args=new String[] { "new int    [] { 0,1,10,1000,-1,-10,-1000 }" };
	if(et==AglTypePrimitif.tlong)     args=new String[] { "new long   [] { 0l,1l,10l,1000l,-1l,-10l,-1000l }" };
	if(et==AglTypePrimitif.tfloat)    args=new String[] { "new floar  [] { 0.f,1.f,10.f,1000.f,-1.f,-10.f,-1000.f }" };
	if(et==AglTypePrimitif.tdouble)   args=new String[] { "new double [] { 0.d,1.d,10.d,1000.d,-1.d,-10.d,-1000.d }" };

	if(et==AglType.cherche("String"))
	  args=new String[] { "new String[] { \"abcdef\",\"\" }" };
	if(et==AglType.cherche("Object"))
	  args=new String[] { "new java.lang.Object[] { new java.lang.Object() }" };
	if(et==AglType.cherche("IObjet"))
	  args=new String[] { "new fr.stcpmvn.dodico.objet.CObjet[] { new fr.stcpmvn.dodico.objet.CObjet() }" };
      }

      if(args==null) args=new String[] { "null" };
    }

    if(args!=null)
    {
      for(int i=0;i<args.length;i++)
      {
	String t=args[i].replace('\"','\'');

	if(ot==AglTypePrimitif.tvoid)
	{
	  r+=indente()+"try { trace(\""
	    +(o.nom+"("+t+")                                 ").substring(0,30)
	    +" . \"); o."+o.nom+"("+args[i]+"); }\n";
	  r+=indente()+"catch(Exception ex) { exception(ex); erreur(\""+o.nom+"("+t+")\"); }\n\n";
	}
	else
	if(ot instanceof AglTypeTableau)
	{
	  r+=indente()+"try { trace(\""
	    +(o.nom+"("+t+")                                 ").substring(0,30)
	    +" ? \"+o."+o.nom+"("+args[i]+").length+\" elements\"); }\n";
	  r+=indente()+"catch(Exception ex) { exception(ex); erreur(\""+o.nom+"("+t+")\"); }\n\n";
	}
	else
	{
	  r+=indente()+"try { trace(\""
	    +(o.nom+"("+t+")                                 ").substring(0,30)
	    +" ? \"+o."+o.nom+"("+args[i]+")); }\n";
	  r+=indente()+"catch(Exception ex) { exception(ex); erreur(\""+o.nom+"("+t+")\"); }\n\n";
	}
      }
    }

    return r;
  }

  public String code(AglParametre o)
  {
    String r="";
    r+=code(o.type)+" "+o.nom;
    // if(o.defaut!="") r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vpublic)       r+="public ";
    if(o.vprivate)      r+="private ";
    if(o.vprotected)    r+="protected ";
    // if(o.vinline)       r+="inline ";
    if(o.vabstract)     r+="abtract ";
    if(o.vstatic)       r+="static ";
    if(o.vfinal)        r+="final ";
    if(o.vnative)       r+="native ";
    if(o.vtransient)    r+="transient ";
    if(o.vsynchronized) r+="synchronized ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique     o) { return ""; }
  public String code(AglExpressionAffectation   o) { return ""; }
  public String code(AglExpressionAttribut      o) { return ""; } 
  public String code(AglExpressionBinaire       o) { return ""; }
  public String code(AglExpressionDefVar        o) { return ""; }
  public String code(AglExpressionEstUn         o) { return ""; }
  public String code(AglExpressionFonction      o) { return ""; }
  public String code(AglExpressionIndice        o) { return ""; }
  public String code(AglExpressionInitArray     o) { return ""; }
  public String code(AglExpressionMethode       o) { return ""; }
  public String code(AglExpressionMultiple      o) { return ""; }
  public String code(AglExpressionNew           o) { return ""; }
  public String code(AglExpressionNewArray      o) { return ""; }
  public String code(AglExpressionParenthesee   o) { return ""; }
  public String code(AglExpressionTernaire      o) { return ""; }
  public String code(AglExpressionTypage        o) { return ""; }
  public String code(AglExpressionUnaireDroite  o) { return ""; }
  public String code(AglExpressionUnaireGauche  o) { return ""; }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=indente();
    r+=o.body;
    if(r.charAt(r.length()-1)!='\n') r+="\n";
    r=r.substring(0,r.length()-1)+" /* Instruction "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=indente()+"; /* Vide "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=indente()+code(o.gauche)+"="+code(o.droite);
    r+="; /* Affectation "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=indente();
    r+="/* Bloc "+o.ligne+" */\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=indente()+"break";
    if(o.nom!=null) r+=" "+o.nom;
    r+="; /* Break "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=indente()+"catch("+code(o.signal)+") /* Catch "+o.ligne+" */\n";
    r+=code(o.reprise);
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=indente()+"continue";
    if(o.nom!=null) r+=" "+o.nom;
    r+="; /* Continue "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=indente()+code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    r+="; /* DefVar "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r=indente()+"do /* DoWhile "+o.ligne+" */\n";
    r+=code(o.corps);
    r+=indente()+"while("+code(o.condition)+"); /* DoWhile "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=indente()+code(o.expression)+"; /* InsExpr "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=indente()+"finally /* Finally "+o.ligne+" */\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r=indente()+"for(";
    r+=code(o.debut);
    r+=indente()+"    "+code(o.test)+"; ";
    r+=code(o.fin);
    r+=") /* For "+o.ligne+" */\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=indente()+"continue "+o.nom;
    r+="; /* Goto "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=indente()+"if("+code(o.condition)+") /* If "+o.ligne+" */\n";
    r+=code(o.sioui);
    if(o.sinon!=null)
    {
      r+=indente()+"else /* Else "+o.ligne+" */\n";
      r+=code(o.sinon);
    }

    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=" "+o.nom+":";
    r+=" /* Label "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=indente()+o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+="); /* Procedure "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=indente()+"return "+code(o.valeur);
    r+="; /* Return "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=indente()+"switch("+code(o.condition)+") /* Switch "+o.ligne+" */\n";
    r+=indente()+"{\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"case "+code(ce)+":\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      r+=indente()+"break;\n";
    }

    r+=indente()+"default:\n";
    r+=code(o.defaut);
    // r+=indente()+"break;\n";

    indentation--;
    r+=indente()+"}\n";

    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=indente()+"synchronized("+code(o.objet)+
	") /* Synchronized "+o.ligne+" */\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=indente()+"throw "+code(o.valeur);
    r+="; /* Throw "+o.ligne+" */\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=indente()+"try /* Try "+o.ligne+" */\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=indente()+"while("+code(o.condition)+
	") /* While "+o.ligne+" */\n";
    r+=code(o.corps);
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    _unite=o;

    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";

    // IHM :: if(AglImport.table.get("java.awt.*")==null)
    //   AglImport.tAwt=new AglImport("java.awt.*");

    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }
    r+="\n";

    String fu=o.fichier;
    int i=fu.lastIndexOf('/');
    if(i>=0) fu=fu.substring(i+1);
    int j=fu.lastIndexOf('.');
    if(j>=0) fu=fu.substring(0,j);
    
    r+="public class Autotest_"+fu.replace('.','_')+"\n";
    r+="{\n";
        
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    r+="  public static void test(String s)\n";
    r+="  {\n";
    r+="    System.out.println(\"TST|\"+s);\n";
    r+="  }\n\n";

    r+="  public static void trace(String s)\n";
    r+="  {\n";
    r+="    System.out.println(\"TRC|\"+s);\n";
    r+="  }\n\n";

    r+="  public static void exception(Exception e)\n";
    r+="  {\n";
    r+="    System.out.println(\"EXC|\"+e);\n";
    r+="  }\n\n";

    r+="  public static void erreur(String s)\n";
    r+="  {\n";
    r+="    System.out.println(\"ERR|\"+s);\n";
    r+="  }\n\n";

    r+="  public static void main(String[] args)\n";
    r+="  {\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+="    teste_"+code(ac.nom).replace('.','_')+"();\n";
        }
      }
    }

    r+="  }\n";
    r+="}\n";
    return r;
  }
}
