/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeBdl.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglCodeBdl extends AglCode
{
  public String extension()
  {
    return "bdl";
  }

  public String[] extensions()
  {
    return new String[] { "bdl", "BDL" };
  }

  public String code(AglNom o)
  {
    String r="";

    /*
    boolean p=true;

    if(o.identificateurs.size()>1) r+="::";

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="::";
      r+=(String)e.nextElement();
    }
    */

    r=o.dernier();
    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    //o=AglTypeAlias.simplifie(o,"Idl");

    String r="";
    if(o instanceof AglTypeTableau)
    {
      AglTypeTableau t=(AglTypeTableau)o;
      if(t.dimension==null) r+=code(t.type)+"[]";
      else                  r+=code(t.type)+"["+code(t.dimension)+"]";
    }
    else r+=code(o.nom);

    return r;
  }

  // PROPRIETE

  public String nomPropriete(String n)
  {
    String r=n;
    if(r.startsWith("get")) r=r.substring(3);
    if(r.startsWith("set")) r=r.substring(3);
    if(r.endsWith  ("_"))   r=r.substring(0,r.length()-1);
    return r;
  }

  public boolean estUnePropriete(AglClasse o, String n)
  {
    boolean batt,bget,bset;

    batt=bget=bset=false;

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(aa.nom.equalsIgnoreCase(n))     batt=true;
      if(aa.nom.equalsIgnoreCase(n+"_")) batt=true;
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.nom.equalsIgnoreCase("get"+n)) bget=true;
      if(am.nom.equalsIgnoreCase("set"+n)) bset=true;
    }

    return (batt && bget) || (bget && bset);
  }

  public String findReader(AglClasse o, String n)
  {
    String r="none";

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.nom.equalsIgnoreCase("get"+n)) r=am.nom;
    }

    return r;
  }

  public String findWriter(AglClasse o, String n)
  {
    String r="none";

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.nom.equalsIgnoreCase("set"+n)) r=am.nom;
    }

    return r;
  }

  public String findSpecificSupport(AglClasse o, String n)
  {
    String r="herite";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(aa.nom.equalsIgnoreCase(n+"Listeners"))  r=aa.nom;
      if(aa.nom.equalsIgnoreCase(n+"Listeners_")) r=aa.nom;
    }

    return r;
  }

  public String proprietes(AglClasse o)
  {
    String r="";
    indentation++;

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      
      String s=nomPropriete(aa.nom);
      if(estUnePropriete(o,s))
      {
	r+=indente()+"propriete\n";
	r+=indente()+"{\n";
	indentation++;

	r+=indente()+"nom       : "+s+";\n";
	r+=indente()+"type      : "+code(aa.type)+";\n";

	if(aa.initiale!=null)
	  r+=indente()+"initiale  : "+code(aa.initiale)+";\n";
	
	if(AglPreferences.AGL.getBooleanProperty("bdl.code.simple",false))
	{
	  // r+=indente()+"visibilite: "+code(aa.visibilite)+";\n";
	  r+=indente()+"champ     : "+aa.nom                  +";\n";
	  r+=indente()+"lecture   : "+findReader(o,s)         +";\n";
	  r+=indente()+"ecriture  : "+findWriter(o,s)         +";\n";
	  r+=indente()+"support   : "+findSpecificSupport(o,s)+";\n";
	}

	indentation--;
	r+=indente()+"}\n\n";
      }
    }

    indentation--;
    return r;
  }

  // EVT

  public String nomEvenement(String n)
  {
    String r=n;
    if(r.startsWith("fire"))     r=r.substring(4);
    if(r.startsWith("add"))      r=r.substring(3);
    if(r.startsWith("remove"))   r=r.substring(6);
    if(r.endsWith  ("Listener")) r=r.substring(0,r.length()-8);
    return r;
  }

  public boolean estUnEvenement(AglClasse o, String n)
  {
    boolean bfire,badd,brem;

    bfire=badd=brem=false;

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.nom.equalsIgnoreCase("fire"+n)) bfire=true;
      if(am.nom.equalsIgnoreCase("add"+n+"Listener")) badd=true;
      if(am.nom.equalsIgnoreCase("remove"+n+"Listener")) brem=true;
    }

    return (badd && brem) || bfire;
  }

  public String findFirer(AglClasse o, String n)
  {
    String r="none";

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.nom.equalsIgnoreCase("fire"+n)) r=am.nom;
    }

    return r;
  }

  public String findAdder(AglClasse o, String n)
  {
    String r="none";

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.nom.equalsIgnoreCase("add"+n+"Listener")) r=am.nom;
    }

    return r;
  }

  public String findRemover(AglClasse o, String n)
  {
    String r="none";

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.nom.equalsIgnoreCase("remove"+n+"Listener")) r=am.nom;
    }

    return r;
  }

  public String findEventListeners(AglClasse o, String n)
  {
    String r="none";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(aa.nom.equalsIgnoreCase(n+"Listeners"))  r=aa.nom;
      if(aa.nom.equalsIgnoreCase(n+"Listeners_")) r=aa.nom;
    }

    return r;
  }

  public String evenements(AglClasse o)
  {
    AglVecteur v=new AglVecteur(11,11);
    String r="";
    indentation++;

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();

      String s=nomEvenement(am.nom);
      if(!v.contains(s)&&estUnEvenement(o,s))
      {
	r+=indente()+"evenement\n";
	r+=indente()+"{\n";
	indentation++;
	
	r+=indente()+"type      : "+s                      +";\n";
	r+=indente()+"emission  : "+findFirer(o,s)         +";\n";
	r+=indente()+"ajout     : "+findAdder(o,s)         +";\n";
	r+=indente()+"retrait   : "+findRemover(o,s)       +";\n";
	r+=indente()+"support   : "+findEventListeners(o,s)+";\n";

	indentation--;
	r+=indente()+"}\n";
	v.addElement(s);
      }
    }

    indentation--;
    return r;
  }

  public String code(AglAttribut o)
    { return ""; }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    indentation=0;
    String r="/* ";
    // if(o.paquet!="") r+=o.paquet;
    r+=code(o.nom)+" ["+o.ligne+"] */\n\n";

    // r+=code(o.visibilite);
    r+="struct "+code(o.nom);

    r+="\n{\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation=0;
    return r+"};\n";
  }

  public String code(AglClasse o)
  {
    String r="";

    if(o.visibilite.vpublic)
    {

      // r+="// "+code(o.nom)+"."+extension()+" ["+o.ligne+"]\n\n";
      // r+=code(o.visibilite);

      r+="composant "+code(o.nom);
      r+="\n{\n";

      if(o.meres.noms.size()>0)
	r+="  heritage        : "+code(o.meres)+";\n";

      String lp="";
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	
	String s=nomPropriete(aa.nom);
	if(estUnePropriete(o,s))
	{
	  if(lp.equals("")) lp+=s;
	  else lp+=", "+s;
	}
      }
      r+="  proprietes      : "+lp+";\n";
      
      String le="";
      AglVecteur ve=new AglVecteur(11,11);
      if(!lp.equals("")) le+="PropertyChangeEvent";
      
      for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
      {
	AglMethode am=(AglMethode)e.nextElement();
	
	String s=nomEvenement(am.nom);
	if(!ve.contains(s)&&estUnEvenement(o,s))
	{
	  if(le.equals("")) le+=s;
	  else le+=", "+s;
	  ve.addElement(s);	
	}
      }
      r+="  evenements emis : "+le+";\n";

      if(o.interfaces.noms.size()>0)
      {
	String li="";
	String lr="";
	String s;
	
	for(Enumeration e=o.interfaces.noms.elements(); e.hasMoreElements(); )
	{
	  AglNom n=(AglNom)e.nextElement();
	  if(n.dernier().endsWith("Listener"))
	  {
	    if(!lr.equals("")) lr+=", ";
	    s=code(n);
	    s=s.substring(0,s.length()-8);
	    if((s.indexOf("Evt")<0)&&(s.indexOf("Event")<0)) s+="Event";
	    lr+=s;
	  }
	  else
	  {
	    if(!li.equals("")) li+=", ";
	    li+=code(n);
	  }
	}

	if(!lr.equals("")) r+="  evenements recus: "+lr+";\n";
	if(!li.equals("")) r+="  interfaces      : "+li+";\n";
      }

      if(o.exceptions.noms.size()>0)
	r+="  exceptions      : "+code(o.exceptions)+";\n";

      r+="\n";

      r+=proprietes(o);
      if(AglPreferences.AGL.getBooleanProperty("bdl.code.simple",false))
	r+=evenements(o);

      /*
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
      }
      */

      /*
      for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
      {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
      }
      */

      r+="}\n";
    }

    return r;
  }

  public String code(AglConstructeur o)
  {
    String r="  ";
    /*
    r+=code(o.visibilite);
    r+="init"+code(o.nom)+o.parametres.size()+"(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }
    */
    r+=");";
    return r;
  }

  public String code(AglImport o)
  {
    String r="#include <"+o.nom.dernier();
    r+=".idl>\n";
    return r;
  }

  public String code(AglMethode o)
    { return ""; }

  public String code(AglParametre o)
  {
    String r="";

    switch(o.passage)
    {
      case AglParametre.IN   : r+="in";    break;
      case AglParametre.OUT  : r+="out";   break;
      case AglParametre.INOUT: r+="inout"; break;
    }

    r+=" "+code(o.type)+" "+o.nom;
    // if(!o.defaut.equals("")) r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vfinal&&o.vstatic) r+="const ";
    if(o.voneway)           r+="oneway ";
    if(o.vreadonly)         r+="readonly ";

    if(r.equals("")) r+="default ";
    r=r.substring(0,r.length()-1);
    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="new "+code(o.type)+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="new "+code(o.type)+"[";
    r+=code(o.nombre);
    r+="]";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }

  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);

    // IHM :: if(AglImport.table.get("java.awt.*")==null)
    //   AglImport.tAwt=new AglImport("java.awt.*");

    /*
    for(Enumeration e=AglImport.table.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      System.out.print(code(ai));
    }
    System.out.println("");
    */

    /*
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglTypeAlias)
      {
        AglTypeAlias aa=(AglTypeAlias)at;
	if(!aa.langage.equals("Idl"))
	  r+="typedef "+code(aa.type)+" "+code(aa.nom)+";\n";
      }
    }
    r+="\n";
    */

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
}


