/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeC.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import java.util.*;

public class AglCodeC extends AglCode
{
  AglClasse classe_         =null;
  String    arg_            =null;
  int       nbconstructeurs_=0;

  public String extension()
    { return "c"; }

  public String[] extensions()
  {
    return new String[] { "c", "C" };
  }

  private String removeP(String _s)
  {
    if((_s.length()>2)&&_s.startsWith("(")&&_s.endsWith(")"))
      _s=_s.substring(1,_s.length()-1);
    return _s;
  }

  public String code(AglNom o)
  {
    String r=o.dernier();
    if(r.equals("String")) r="char*";
    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";

    if(o instanceof AglTypeAlias)
      r+=code(((AglTypeAlias)o).type);
    else
    if(o instanceof AglTypeTableau)
    {
      r+=code(((AglTypeTableau)o).type);

      AglExpression t=((AglTypeTableau)o).dimension;
      /*
      if(t==null) r+="*";
      else        r+="["+code(t)+"]";
      */
      r+="*";
    }
    else r+=code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  { return ""; }

  public String code(AglBloc o)
  {
    return code(o,false);
  }

  public String code(AglBloc o,boolean _reduced)
  {
    String r="";

    if(_reduced&&(o.instructions.size()==1))
    {
      r+=indente()+code((AglInstruction)o.instructions.elementAt(0))+"\n";
    }
    else
    {
      r+=indente()+"{\n";
      indentation++;

      //int n=1;
      for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
      {
	AglInstruction ai=(AglInstruction)e.nextElement();
	r+=code(ai);
	//n++;
      }
      indentation--;
      r+=indente()+"}\n";
    }

    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;

    String r="/* ";
    // if(!o.paquet.equals("")) r+=o.paquet.replace('.','/');
    r+=code(o.nom)+"."+extension()+" ["+o.ligne+"] */\n\n";

    r+="#include \""+o.nom.dernier().toLowerCase()+".h\"\n\n";
    return r;
  }

  public String code(AglClasse o)
  {
    classe_=o;
    indentation=0;

    String r="/* ";
    // if(!o.paquet.equals("")) r+=o.paquet.replace('.','/');
    r+=code(o.nom)+"."+extension()+" ["+o.ligne+"] */\n\n";

    r+="#include \""+o.nom.dernier().toLowerCase()+".h\"\n\n";

    if(!o.constructeurs.isEmpty())
      for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
      {
	AglConstructeur am=(AglConstructeur)e.nextElement();
	r+="\n"+code(am);
      }
    else
    {
      AglConstructeur am=new AglConstructeur(o.nom);
      r+="\n"+code(am);
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+="\n"+code(am);
    }

    // classe_=null;
    return r;
  }

  public String code(AglConstructeur o)
  {
    if(global(classe_)) return "";

    indentation=0;
    String r="";
    r+=code(classe_.nom)+"* new_"+code(o.nom); // classe_.nom.dernier();
    if(o.parametres.size()>0)
    {
      nbconstructeurs_++;
      r+=nbconstructeurs_; // o.parametres.size();
    }
    r+="(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=")\n";
    r+="{\n";
    r+="  "+code(classe_.nom)+"* this=("+
      code(classe_.nom)+"*)malloc(1,sizeof("+
      code(classe_.nom)+"));\n";

    if(!o.corps.instructions.isEmpty())
    {
      indentation++;
      r+=code(o.corps);
      indentation--;
    }

    r+="  return this;\n";
    r+="}\n";
    return r;
  }

  public String code(AglImport o)
    { return ""; }

  public String code(AglMethode o)
  {
    indentation=0;
    String r="";

    AglType at=o.type;
    while(at instanceof AglTypeAlias)
      at=((AglTypeAlias)at).type;

    if(at instanceof AglTypePrimitif)
      r+=code(at);
    else
    if(at instanceof AglTypeTableau)
      r+=code(at);
    else
    if(at==AglType.cherche("String"))
      r+=code(at);
    else
      r+=code(at)+"*";

    r+=" ";
    if(!global(classe_))
      r+=code(classe_.nom)+"_";
    r+=o.nom+"(";

    boolean p=true;

    if(!o.visibilite.vstatic&&!global(classe_))
    {
      r+=code(classe_.nom)+"* this";
      p=false;
    }

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=")\n";
    r+=code(o.corps);

    r+="\n";
    return r;
  }

  public String code(AglParametre o)
  {
    String r="";

    AglType at=o.type;
    while(at instanceof AglTypeAlias)
      at=((AglTypeAlias)at).type;

    if((at instanceof AglTypePrimitif)&&(o.passage==AglParametre.IN))
      r+=code(at);
    else
    if(at instanceof AglTypeTableau)
      r+=code(at);
    else
    if(at==AglType.cherche("String"))
      r+=code(at);
    else
      r+=code(at)+"*";

    r+=" "+o.nom;
    return r;
  }

  public String code(AglVisibilite o)
  { return ""; }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;

    if((classe_!=null)&&!global(classe_))
      for(Enumeration e=classe_.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(r.equals(aa.nom))
	{
	  r="this->"+r;
	  break;
	}
      }

    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean premier=true;
    if(!"".equals(arg_))
      { r+=arg_; premier=false; }

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!premier) r+=", ";
      else premier=false;
      r+=code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
         if(o.objet!=null)     arg_=code(o.objet);
    else if(global(classe_)) arg_="";
    else                       arg_="this";

    if(!global(classe_)) r+=code(classe_.nom)+"_";
    r+=code(o.fonction);
    arg_="";
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="new_"+o.type.nom.dernier();
    if(o.arguments.size()>0) r+=o.arguments.size();
    r+="(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r=" ("+code(o.type)+"*)malloc("+code(o.nombre)+",sizeof("+code(o.type)+")))";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="("+removeP(code(o.expression))+")";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r="";
    r+=o.body;
    if(r.charAt(r.length()-1)!='\n') r+="\n";
    r=r.substring(0,r.length()-1)+"\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=indente()+";\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=indente()+code(o.gauche)+"="+code(o.droite)+";\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r="";
    if(!o.virgule)
	r+=code(o.corps);
    else
    {
      boolean p=true;
      for(Enumeration e=o.corps.instructions.elements(); e.hasMoreElements(); )
      {
	if(!p) r+=",";
	AglInstruction ai=(AglInstruction)e.nextElement();
	String s=code(ai).trim();
	int    i=s.lastIndexOf(";");
	s=s.substring(0,i)+s.substring(i+1);
	if((!p)&&(ai instanceof AglInstructionDefVar))
	{
	  i=s.indexOf(" ")+1;
	  s=s.substring(i);
	}
	r+=s;
	p=false;
      }
      r+=";";
    }
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=indente()+"break";
    if(o.nom!=null) r+=" "+o.nom;
    r+=";\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=indente()+"catch("+code(o.signal)+")\n";
    r+=code(o.reprise);
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=indente()+"continue";
    if(o.nom!=null) r+=" "+o.nom;
    r+=";\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=indente()+code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    r+=";\n";
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r=indente()+"do\n";
    r+=code(o.corps);
    r+=indente()+"while("+code(o.condition)+");\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=code(o.expression);
    if(r.startsWith("(")&&(r.endsWith(")"))) r=r.substring(1,r.length()-1);
    r=indente()+r+";\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=indente()+"// finally\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r=code(o.commentaires)+indente()+"for(";
    String d=code(o.debut).trim();
    String t=code(o.test).trim();
    String f=code(o.fin).trim();

    if(f.endsWith(";")) f=f.substring(0,f.length()-1);

    r+=d+" "+t+"; "+f+")";
    r+="\n";
    r+=code(o.corps,true);
    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=indente()+"goto "+o.nom+";\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=indente()+"if("+code(o.condition)+")\n";
    r+=code(o.sioui,true);
    if(o.sinon!=null)
    {
      r+=indente()+"else\n";
      r+=code(o.sinon,true);
    }

    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=" "+o.nom+":\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=indente()+o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=");\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=indente()+"return "+code(o.valeur)+";\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=indente()+"switch("+code(o.condition)+")";
    r+=indente()+"{\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"case "+code(ce)+":\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      // r+=indente()+"break;\n";
    }

    r+=indente()+"default:\n";
    r+=code(o.defaut);
    // r+=indente()+"break;\n";

    indentation--;
    r+=indente()+"}\n";

    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=indente()+"synchronized("+removeP(code(o.objet))+")\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=indente()+"throw "+code(o.valeur);
    r+=";\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=indente()+"try\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=indente()+"while("+removeP(code(o.condition))+")\n";
    r+=code(o.corps);
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
        {
          r+=code(as);
          r+="\n";
        }
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
}
