/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeCpp.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;

public class AglCodeCpp extends AglCodeCppAbstract
{
  public String extension()
  {
    return "cpp";
  }

  public String[] extensions()
  {
    return new String[] { "cpp", "CPP", "cc", "CC" };
  }

  public String code(AglAttribut o)
  {
    String r="";

    r+=code(o.visibilite);
    r+=code(o.type)+" ";
    if(!global(classe_)) r+=code(classe_.nom)+"::";
    r+=o.nom;
    if((o.initiale!=null)&&o.visibilite.vstatic)
      r+="="+code(o.initiale);
    r+=";\n";

    return r;
  }

  private String attributes0(AglAttribut o)
  {
    String r="";

    if((o.initiale!=null)&&!o.visibilite.vstatic)
      r+=indente()+o.nom+"="+code(o.initiale)+";\n";

    return r;
  }

  public String code(AglStruct o)
  {
    return "";
  }

  public String avantClasse(AglClasse o)
  {
    String r="";

    if(!global(o))
    {
      r+="\n/* ";
      // if(!o.paquet.equals("")) r+=o.paquet.replace('.','/');
      r+=code(o.nom,NAMESPACE)+"."+extension()+" ["+o.ligne+"] */\n\n";
      r+="#include \""+code(o.nom,NAMESPACE)+".hpp\"\n";
    }

    return r;
  }

  public String avantMethodes(AglClasse o)
  {
    return "";
  }

  public String code(AglClasse o)
  {
    classe_=o;
    indentation=0;
    String r="";

    r+=avantClasse(o);

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(aa.visibilite.vstatic)
	r+="\n"+code(aa);
    }

    if(JAVALIKE&&o.normale)
    {
      r+="\nvoid "+code(o.nom)+"::attributes0()";
      if(RETURNAFTER) r+="\n";
      else            r+=" ";
      r+="{\n";
      indentation++;
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vstatic)
	  r+=attributes0(aa);
      }
      indentation--;
      r+="}\n";
    }

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+="\n"+code(am);
    }

    r+=avantMethodes(o);
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+="\n"+code(am);
    }

    return r;
  }

  public String code(AglConstructeur o)
  {
    indentation=0;
    String r="";

    r+=code(classe_.nom)+"::";
    if(o.destructeur) r+="~";
    r+=code(classe_.nom,true)+"(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=")";
    //if(o.visibilite.vfinal) r+=" const";

    boolean saut=false;

    if(JAVALIKE&&!o.destructeur)
    {
      if(o.corps.instructions.size()>0)
      {
	Object ie=o.corps.instructions.elementAt(0);
	if(ie instanceof AglInstructionExpression)
	{
	  AglExpression em=((AglInstructionExpression)ie).expression;
	  if(em instanceof AglExpressionMethode)
	  {
	    AglExpressionFonction ef=(AglExpressionFonction)
	      ((AglExpressionMethode)em).fonction;
	    if("super".equals(ef.nom))
	    {
	      r+=" : "+code(ef);
	      if(RETURNAFTER) r+="\n";
	      else            r+=" ";
	      saut=true;
	    }
	  }
	}
      }

      if(!saut)
      {
	if(RETURNAFTER) r+="\n";
	else            r+=" ";
      }

      r+="{\n";
      indentation++;
      r+=indente()+"attributes0();\n";
      r+=indente()+"initialize0(";

      p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)e.nextElement();
	if(!p) r+=","+SPACEAFTER;
	r+=ap.nom;
	p=false;
      }

      r+=");\n";
      indentation--;
      r+="}\n";
      // init_this

      r+="\n";
      indentation=0;
      //String r="";

      r+="void "+code(classe_.nom)+"::initialize0(";

      p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)e.nextElement();
	if(!p) r+=", ";
	r+=code(ap);
	p=false;
      }

      r+=")";
    }

    r+=code(o.corps,false,saut);
    return r;
  }

  public String code(AglImport o)
    { return ""; }

  public String code(AglMethode o)
  {
    indentation=0;
    String r="";

    if(o.visibilite.vabstract||!classe_.normale) return "";

    if(!global(classe_))
      r+=code(classe_.nom)+"::";

    if(JAVALIKE&&"finalize".equals(o.nom))
      r+="~"+code(classe_.nom,true);
    else
      r=code(o.type)+" "+r+operator(o);

    String v=code(o.visibilite);
    v=FuLib.replace(v,"const ","");
    r=v+r+"(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=")";
    if(o.visibilite.vfinal) r+=" const";
    r+=code(o.corps);
    r+="\n";

    if(MAIN&&!global(classe_)&&"main".equals(o.nom))
    {
      r+="\n";
      r+="void main(int _argc,char** _argv)";
      if(RETURNAFTER) r+="\n";
      else            r+=" ";
      r+="{\n";
      indentation++;
      if(STL)
      {
	/*
	r+=indente()+"string* args=new string[_argc];\n";
	r+=indente()+"for(int i=0;i<_argc;i++)\n";
	indentation++;
	r+=indente()+"args[i]=string(_argv[i]);\n";
	indentation--;
	r+=indente()+code(classe_.nom)+"::main(args);\n";
	*/
	r+=indente()+"vector<string>* args=new vector<string>(_argc);\n";
	r+=indente()+"for(int i=0;i<_argc;i++)\n";
	indentation++;
	r+=indente()+"args->assign(i,string(_argv[i]));\n";
	indentation--;
	r+=indente()+code(classe_.nom)+"::main(args);\n";
      }
      else
      {
	r+=indente()+code(classe_.nom)+"::main(_argv);\n";
      }
      indentation--;
      r+="}\n";
    }

    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vinline)    r+="inline ";
    //if(o.vstatic)    r+="static ";
    if(o.vfinal)     r+="const ";
    if(o.vtransient) r+="register ";

    return r;
  }

// Expressions

// Instructions

// Unite

}
