/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeCppAbstract.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;
import java.util.*;

public abstract class AglCodeCppAbstract extends AglCode
{
  protected boolean MAIN=
    AglPreferences.AGL.getBooleanProperty("cpp.code.main",true);

  protected boolean GC=
    AglPreferences.AGL.getBooleanProperty("cpp.code.gc",true);

  protected boolean STL=
    AglPreferences.AGL.getBooleanProperty("cpp.code.stl",true);

  protected boolean SINGLECLASSROOT=
    AglPreferences.AGL.getBooleanProperty("cpp.code.singleclassroot",false);

  protected boolean JAVALIKE=
    AglPreferences.AGL.getBooleanProperty("cpp.code.javalike",true);

  protected String SPACEAFTER=
    (AglPreferences.AGL.getBooleanProperty("cpp.code.spaceafter",true)
     ? " " : "");

  protected boolean RETURNAFTER=
    AglPreferences.AGL.getBooleanProperty("cpp.code.returnafter",false);

  protected boolean NAMESPACE=
    AglPreferences.AGL.getBooleanProperty("cpp.code.namespace",false);

  protected boolean SHORTNAME=
    AglPreferences.AGL.getBooleanProperty("cpp.code.shortname",false);

  protected String BOOLEANTYPE=
    AglPreferences.AGL.getStringProperty("cpp.code.type.boolean","bool");

  protected AglCodeCppAbstract()
  {
    TABULATION=AglPreferences.AGL.getStringProperty
      ("cpp.code.tabulation",TABULATION);
  }

  protected AglClasse classe_=null;

  public String syntaxe()
  {
    return "Cpp";
  }

  protected String operator(AglMethode o)
  {
    String r="";

    if(!o.nom.startsWith("operator"))
      r+=o.nom;
    else
    {
      String s=o.nom.substring(9);
      r+="operator ";

           if(s.equals("set"      )) r+="=";
      else if(s.equals("addset"   )) r+="+=";
      else if(s.equals("subset"   )) r+="-=";
      else if(s.equals("mulset"   )) r+="*=";
      else if(s.equals("divset"   )) r+="/=";
      else if(s.equals("incr"     )) r+="++";
      else if(s.equals("decr"     )) r+="--";
      else if(s.equals("add"      )) r+="+";
      else if(s.equals("sub"      )) r+="-";
      else if(s.equals("mul"      )) r+="*";
      else if(s.equals("div"      )) r+="/";
      else if(s.equals("element"  )) r+="[]";
      else if(s.equals("call"     )) r+="()";
      else if(s.equals("eq"       )) r+="==";
      else if(s.equals("ne"       )) r+="!=";
      else if(s.equals("lt"       )) r+="<";
      else if(s.equals("gt"       )) r+=">";
      else if(s.equals("le"       )) r+="<=";
      else if(s.equals("ge"       )) r+=">=";
      else if(s.equals("band"     )) r+="&&";
      else if(s.equals("bor"      )) r+="||";
      else if(s.equals("and"      )) r+="&";
      else if(s.equals("or"       )) r+="|";
      else if(s.equals("cast"     )) r+="cast";
    }

    return r;
  }

  // abstract

  public abstract String code(AglStruct o);
  public abstract String code(AglClasse o);

  // common

  public final String code(AglNom o)
  {
    return code(o,false);
  }

  protected final String code(AglNom o,boolean _short)
  {
    String r="";

    if(_short||(SHORTNAME&&!NAMESPACE))
    {
      r=o.dernier();
    }
    else
    {
      //if(NAMESPACE) r+="::";
      String s=(NAMESPACE ? "::" : "_");
      boolean p=true;

      for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
      {
	if(p) p=false; else r+=s;
	r+=(String)e.nextElement();
      }
    }

    return r;
  }

  public final String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    return code(o,false);
  }

  protected final String code(AglType o,boolean _cru)
  {
    String r="";
         
    o=AglTypeAlias.simplifie(o,"Cpp");

    if(STL&&(o==AglType.cherche("String")))
      r+="string";
    else
    if(o==AglTypePrimitif.tboolean)
      r+=BOOLEANTYPE;
    else
    if(o instanceof AglTypePrimitif)
      r+=code(o.nom,NAMESPACE);
    else
    if(o instanceof AglTypeTableau)
    {
      if(STL) r+="vector<";
      r+=code(((AglTypeTableau)o).type);
      if(STL) r+=">";

      //if(r.endsWith("&"))
      //r=r.substring(0,r.length()-1);

      if(!STL&&((AglTypeTableau)o).dimension!=null)
	r+="["+code(((AglTypeTableau)o).dimension)+"]";
      else
	r+="*";

      //if(r.endsWith("[][]"))
      //r=r.substring(0,r.length()-4)+"*[]";
    }
    else
    if(o instanceof AglTypePointeur)
      r+=code(((AglTypePointeur)o).type)+"*";
    else
    if(o instanceof AglTypeReference)
      r+=code(((AglTypeReference)o).type)+"&";
    else
    {
      r+=code(o.nom);
      if(!_cru) r+="*";
    }

    return r;
  }

  public final String code(AglBloc o)
  {
    return code(o,false,false);
  }

  protected final String code(AglBloc o,boolean _reduit)
  {
    return code(o,_reduit,false);
  }

  protected final String code
    (AglBloc o,boolean _reduit,boolean _saut)
  {
    String r="";
    
    if(_reduit&&(o.instructions.size()==1))
    {
      r+="\n";
      indentation++;
      r+=code(o.instructions.elementAt(0));
      indentation--;
    }
    else
    {
      if(RETURNAFTER) r+="\n"+indente()+"{\n";
      else            r+=" {\n";
      indentation++;

      Enumeration e=o.instructions.elements();
      if(_saut&&e.hasMoreElements()) e.nextElement();
      while(e.hasMoreElements())
      {
	AglInstruction ai=(AglInstruction)e.nextElement();
	r+=code(ai);
      }

      indentation--;
      r+=indente()+"}\n";
    }

    return r;
  }

  public abstract String avantClasse(AglClasse o);
  public abstract String avantMethodes(AglClasse o);

  public final String code(AglParametre o)
  {
    String r="";
    r+=code(o.type)+" "+o.nom;
    return r;
  }

// Expressions

  public final String code(AglExpressionGenerique o)
  {
    String r=o.body;

    if(STL)
    {
      int    l=r.length();
      if((l>=2)&&(r.charAt(0)=='"')&&(r.charAt(l-1)=='"'))
	r="string("+r+")";
    }

    return r;
  }

  public final String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public final String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null)
    {
      r+=code(o.objet);

      /*
      AglType t=AglType.cherche(r);
      System.err.println("$$$A "+r+" - "+t+" - "+o.attribut);
      if(t!=null) r+="::";
      else        r+="->";
      */

      if(o.statique) r+="::";
      else           r+="->";
    }
    //if(r.equals("this.")) r="this->";
    r+=o.attribut;
    return r;
  }

  public final String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public final String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public final String code(AglExpressionEstUn o)
  {
    //String r="instanceof("+code(o.expression)+","+code(o.type)+")";
    String r="dynamic_cast<"+code(o.type)+">"+
             "("+code(o.expression)+")!=null";
    return r;
  }

  public final String code(AglExpressionFonction o)
  {
    String r="";

    if(JAVALIKE&&"this" .equals(o.nom))
      r+="initialize0";
    else
    if(JAVALIKE&&"super".equals(o.nom))
    {
      if(classe_.meres.noms.size()>0)
	r+=code(classe_.meres.noms.elementAt(0));
      else
	r+="// super";
    }
    else
      r+=o.nom;

    r+="(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=","+SPACEAFTER;
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public final String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public final String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public final String code(AglExpressionMethode o)
  {
    String r="";

    if(o.objet!=null)
    {
      String c=code(o.objet);

      if("this" .equals(c))
	c="";
      else
      if("super".equals(c))
      {
	if(classe_.meres.noms.size()>0)
	  c=code(classe_.meres.noms.elementAt(0))+"::";
	else
	  c="/*super*/ ";
      }
      else
      {
	/*
	  AglType t=AglType.cherche(c);
	  System.err.println("$$$M "+c+" - "+t+" - "+code(o.fonction));
	  if(t!=null) c+="::";
	  else        c+="->";
	  */

	if(o.statique) c+="::";
	else           c+="->";
      }

      r+=c;
    }

    r+=code(o.fonction);
    return r;
  }

  public final String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public final String code(AglExpressionNew o)
  {
    String r=code(o.type,true);
    r="new "+r+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=","+SPACEAFTER;
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public final String code(AglExpressionNewArray o)
  {
    String r="";
    String t="";
    String n="";

    if(o.nombre!=null)
    {
      t=code(o.type,false);
      n=code(o.nombre);
    }
    else
    if(o.type instanceof AglTypeTableau)
    {
      t=code(((AglTypeTableau)o.type).type,false);
      if(((AglTypeTableau)o.type).dimension!=null)
	n=code(((AglTypeTableau)o.type).dimension);
      if(o.initiale instanceof AglExpressionInitArray)
	n=""+((AglExpressionInitArray)o.initiale).initiales.size();
    }
    else
      t=code(o.type,true);

    //System.err.println("#### TYPE R="+r);
    if(STL)
    {
      r="new vector<"+t+">("+n+")";
      if(o.initiale!=null)
	r="("+r+SPACEAFTER+" = "+SPACEAFTER+
	  "(new "+t+"["+n+"] = "+code(o.initiale)+"))";
    }
    else
    {
      r="new "+t+"["+n+"]";
      if(o.initiale!=null)
	r="("+r+SPACEAFTER+" = "+SPACEAFTER+code(o.initiale)+")";
    }

    return r;
  }

  public final String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public final String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public final String code(AglExpressionTypage o)
  {
    //String r="("+code(o.type)+")"+code(o.expression);
    String r="static_cast<"+code(o.type)+">"+
             "("+code(o.expression)+")";
    return r;
  }

  public final String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public final String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public final String code(AglInstructionGenerique o)
  {
    String r=indente();
    r+=o.body;
    if((r.length()>0)&&(r.charAt(r.length()-1)!='\n')) r+="\n";
    // r=r.substring(0,r.length()-1)+"\n";
    return r;
  }

  public final String code(AglInstructionVide o)
  {
    String r=indente()+";\n";
    return r;
  }

  public final String code(AglInstructionAffectation o)
  {
    String r=indente()+code(o.gauche)+"="+code(o.droite)+";\n";
    return r;
  }

  public final String code(AglInstructionBloc o)
  {
    String r="";
    if(!o.virgule)
	r+=code(o.corps);
    else
    {
      boolean p=true;
      for(Enumeration e=o.corps.instructions.elements(); e.hasMoreElements(); )
      {
	if(!p) r+=",";
	AglInstruction ai=(AglInstruction)e.nextElement();
	String s=code(ai).trim();
	int    i=s.lastIndexOf(";");
	s=s.substring(0,i)+s.substring(i+1);
	if((!p)&&(ai instanceof AglInstructionDefVar))
	{
	  i=s.indexOf(" ")+1;
	  s=s.substring(i);
	}
	r+=s;
	p=false;
      }
      r+=";";
    }
    return r;
  }

  public final String code(AglInstructionBreak o)
  {
    String r=indente()+"break";
    if(o.nom!=null) r+=" "+o.nom;
    r+=";\n";
    return r;
  }

  public final String code(AglInstructionCatch o)
  {
    String r=indente()+"catch"+SPACEAFTER+"(";

    if(  (o.reprise.instructions.size()==0)
       &&(o.signal instanceof AglExpressionDefVar))
      r+=code(((AglExpressionDefVar)o.signal).type);
    else
      r+=code(o.signal);

    r+=")";
    r+=code(o.reprise);
    return r;
  }

  public final String code(AglInstructionContinue o)
  {
    String r=indente()+"continue";
    if(o.nom!=null) r+=" "+o.nom;
    r+=";\n";
    return r;
  }

  public final String code(AglInstructionDefVar o)
  {
    String r=indente()+code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    r+=";\n";
    return r;
  }

  public final String code(AglInstructionDoWhile o)
  {
    String r=indente()+"do";
    r+=code(o.corps);

    if(RETURNAFTER)
      r+=indente();
    else
    {
      if(r.endsWith("\n")) r=r.substring(0,r.length()-1);
      r+=" ";
    }

    r+="while"+SPACEAFTER+"("+code(o.condition)+");\n";
    return r;
  }

  public final String code(AglInstructionExpression o)
  {
    String r=code(o.expression);
    if(r.startsWith("(")&&(r.endsWith(")"))) r=r.substring(1,r.length()-1);
    r=indente()+r+";\n";
    return r;
  }

  public final String code(AglInstructionFinally o)
  {
    String r=indente()+"// finally\n";
    r+=code(o.corps);
    return r;
  }

  public final String code(AglInstructionFor o)
  {
    String r=code(o.commentaires)+indente()+"for"+SPACEAFTER+"(";
    String d=code(o.debut).trim();
    String t=code(o.test).trim();
    String f=code(o.fin).trim();

    if(f.endsWith(";")) f=f.substring(0,f.length()-1);

    r+=d+" "+t+"; "+f+")";
    r+=code(o.corps,true);
    return r;
  }

  public final String code(AglInstructionGoto o)
  {
    String r=indente()+"goto "+o.nom+";\n";
    return r;
  }

  public final String code(AglInstructionIf o)
  {
    String r=indente()+"if"+SPACEAFTER+"("+code(o.condition)+")";
    r+=code(o.sioui,true);
    if(o.sinon!=null)
    {
      r+=indente()+"else";
      r+=code(o.sinon,true);
    }
    return r;
  }

  public final String code(AglInstructionLabel o)
  {
    String r=" "+o.nom+":\n";
    return r;
  }

  public final String code(AglInstructionProcedure o)
  {
    String r=indente()+o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=","+SPACEAFTER;
      r+=code(ae);
      p=false;
    }

    r+=");\n";
    return r;
  }

  public final String code(AglInstructionReturn o)
  {
    String r=indente()+"return "+code(o.valeur)+";\n";
    return r;
  }

  public final String code(AglInstructionSwitch o)
  {
    String r=indente()+"switch"+SPACEAFTER+"("+code(o.condition)+")";
    if(RETURNAFTER) r+="\n"+indente();
    else            r+=" ";
    r+="{\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"case "+code(ce)+":\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));

      indentation++;
      Enumeration ec=cb.instructions.elements();
      while(ec.hasMoreElements())
      {
	AglInstruction ai=(AglInstruction)ec.nextElement();
	r+=code(ai);
      }
      indentation--;
    }

    if(o.defaut.instructions.size()!=0)
    {
      r+=indente()+"default:\n";

      indentation++;
      Enumeration ed=o.defaut.instructions.elements();
      while(ed.hasMoreElements())
      {
	AglInstruction ai=(AglInstruction)ed.nextElement();
	r+=code(ai);
      }
      indentation--;
    }

    indentation--;
    r+=indente()+"}\n";

    return r;
  }

  public final String code(AglInstructionSynchronized o)
  {
    String r=code(o.commentaires)+indente()+
	"synchronized("+code(o.objet)+")\n";
    r+=code(o.corps,true);
    return r;
  }

  public final String code(AglInstructionThrow o)
  {
    String r=indente()+"throw "+code(o.valeur)+";\n";
    return r;
  }

  public final String code(AglInstructionTry o)
  {
    String r=indente()+"try";
    r+=code(o.corps);
    return r;
  }

  public final String code(AglInstructionWhile o)
  {
    String r=indente()+"while"+SPACEAFTER+"("+code(o.condition)+")";
    r+=code(o.corps,true);
    return r;
  }

// Unite

  protected String avantUnite(AglUnite o)
  {
    return "";
  }

  protected String apresUnite(AglUnite o)
  {
    return "";
  }

  public final String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=avantUnite(o);

    r+=entete(o.fichier);
    r+="\n";

    r+="#include <stdio.h>\n";
    if(GC)
    {
      r+="#include <gc/gc_cpp.h>\n";
    }
    if(STL)
    {
      r+="#include <string>\n";
      r+="#include <vector>\n";
    }

    if(NAMESPACE)
    {
      r+="\n";
      if(JAVALIKE)
	r+="using namespace java::lang;\n";
      for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
      {
        AglImport ai=(AglImport)e.nextElement();
	if("*".equals(ai.nom.dernier()))
          r+="using namespace "+code(ai.nom.parent())+";\n";
      }
    }

    String[] ss=null;
    if(NAMESPACE)
    {
      AglVecteur simplifications=new AglVecteur(11,11);
      if(JAVALIKE)
        simplifications.addElement("java::lang::");

      for(Enumeration ei=o.imports.elements(); ei.hasMoreElements(); )
      {
        AglImport ai=(AglImport)ei.nextElement();
        AglNom    ni=ai.nom;
        if("*".equals(ni.dernier()))
        {
	  ni=ni.parent();
	  simplifications.addElement(ni.formeIdl()+"::");
        }
      }
      ss=new String[simplifications.size()];
      simplifications.copyInto(ss);
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
	{
	  String ras=code(as);
	  if(NAMESPACE)
	    for(int i=ss.length-1;i>=0;i--)
	      ras=FuLib.replace(ras,ss[i],"");
	  r+=ras;
	}
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
	{
	  String rac=code(ac);
	  if(NAMESPACE)
	    for(int i=ss.length-1;i>=0;i--)
	      rac=FuLib.replace(rac,ss[i],"");
	  r+=rac;
	}
      }
    }

    r+=apresUnite(o);

    return r;
  }
}
