/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeDelphi.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
// Auteur : Arona Kane
// Mel    : a.kane@maulde.com
 * @license      GNU General Public License 2 (GPL2)
//          Copyright (c)1998-2001 Arona Kane.
 */

package com.memoire.agl;
import com.memoire.agl.*;
import java.io.*;
import java.util.*;

/**
 * <French>Cette classe est une adapatation  Delphi inspire
 * des modules AglCodeHpp, AglCodeCpp et AglCodeJava.
 * Se rfrer au Copyright mentionn.</French>
 *
 * <English>This class is an adaptation of Guillaume Desnoix's  
 * AglCodeHpp, AglCodeCpp and AglCodeJava to Delphi Object Pascal. 
 * Please refer to the included Copyright.</English>
 *
 * @author Arona Kane
 * @email  a.kane@maulde.com
 */

public class AglCodeDelphi extends AglCode
{
  AglClasse _classe=null;
  boolean   IN_INTERFACE=true;

  public String extension()
  {
    return "pas";
  }

  public String[] extensions()
  {
    return new String[] { "pas", "PAS", "Pas" };
  }

  protected String debutCS  () { return indente()+"// "; }
  protected String finCS    () { return "";              }
  protected String debutCM  () { return indente()+"(*";  }
  protected String specialCM() { return "*";             }
  protected String milieuCM () { return indente()+" * "; }
  protected String finCM    () { return indente()+" *)"; }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="_";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
    if(o instanceof AglClasse){
      String nc=o.nom.dernier(); 
      if(((AglClasse)o).normale)
        r+="T"+nc;
      else
        r+="I"+nc;
    }
    else
    if(o instanceof AglTypeAlias)
      r+=code(((AglTypeAlias)o).type);
    else
    if(o instanceof AglTypeTableau)
    {
     if(((AglTypeTableau)o).dimension!=null)
       r="array[1.." +code(((AglTypeTableau)o).dimension)+"] of ";
     else
       r="array of ";
     r+=code(((AglTypeTableau)o).type);
    }
    else if(o instanceof AglTypePrimitif)
    {
      if (code(o.nom).equals("int")) r+="integer"; else
      if (code(o.nom).equals("short")) r+="smallint"; else
      if (code(o.nom).equals("long")) r+="longint"; else
      r+=code(o.nom);
    }
    else r+=o.nom.dernier();
    return r;
  }

  public String code(AglAttribut o)
    {
    String r="";
    if(IN_INTERFACE)
    {
      r+=code(o.commentaires);
      r+=code(o.nom)+": "+code(o.type)+";\n";
    }
    else
    {
      if(o.visibilite.vpublic)
      {
        r+="function T"+code(_classe.nom)+".Get_"+o.nom+": "+code(o.type)+";\n";
        r+="begin\n  result := F"+code(o.nom)+";\nend;\n\n";
        if(!o.visibilite.vreadonly)
        {
          r+="procedure T"+code(_classe.nom)+".Set_"+o.nom+";";
          r+="(value: "+code(o.type)+");\n";
          r+="begin\n";
          r+="  F"+code(o.nom)+" := value;\n";
          r+="end;\n";

        }
      }
    }
    return r;
 }

  public String code(AglBloc o)
  {
    return code(o,false,false);
  }

  public String code(AglBloc o, boolean _forced)
  {
    return code(o,_forced,false);
  }

  public String code(AglBloc o, boolean _forced, boolean _ignored)
  {
     if(IN_INTERFACE) return "";
     String r=code(o.commentaires);
     if(_forced||!_ignored)
     {
       r+=indente()+"begin\n";
       indentation++;
     }
     for(Enumeration e=o.instructions.elements(); e.hasMoreElements() ; )
     {
       AglInstruction ai=(AglInstruction)e.nextElement();
       if (!(ai instanceof AglInstructionDefVar))
         r+=code(ai);
       else if (((AglInstructionDefVar)ai).initiale != null)
       {
           r+=indente()+code(((AglInstructionDefVar)ai).nom);
           r+=" := "+code(((AglInstructionDefVar)ai).initiale)+";\n";
       } 
	 
     }
     if(_forced||!_ignored)
     {
       indentation--;
       r+=indente()+"end;\n";
     }
     return r;
  }

  public String code(AglStruct o)
  {
    if(!IN_INTERFACE) return "";

    String r=code(o.commentaires);
    boolean AllPublic=true;

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(!aa.visibilite.vpublic)
      {
        AllPublic=false;
        break;
      }
    }

    if(AllPublic)
    {
      r+="type "+o.nom.dernier()+" = record\n"; 
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
        AglAttribut aa=(AglAttribut)e.nextElement();
        r+="    "+code(aa)+"\n";
      }
    }
    else
    {
      r+="type "+o.nom.dernier()+" class\n"; 
      if(o.attributs.size()>0)
      {
        r+="private\n";
        for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
        {
          AglAttribut aa=(AglAttribut)e.nextElement();
          if(aa.visibilite.vprivate)
          r+="    "+code(aa);
          if(aa.visibilite.vpublic)
            r+="    F"+aa.nom +": "+code(aa.type)+";\n";
	}

	r+="protected\n";
	for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
	{
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  if(!aa.visibilite.vpublic&&!aa.visibilite.vprivate)
	    r+="    "+code(aa);
	}
	r+="  public\n";
        for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
        {
          AglAttribut aa=(AglAttribut)e.nextElement();
          if(aa.visibilite.vpublic)
            r+="    property "+aa.nom+": "+code(aa.type)+" read F"+aa.nom;
          if(!aa.visibilite.vreadonly)
            r+=" write F"+aa.nom;
          if(aa.initiale!=null)
            r+=" default "+code(aa.initiale);
          r+=";\n";
        }

      }
    }
    r+="end;\n\n";
    return r;
  }


  public String interfaceProcess(AglClasse o)
  {
    String r="";
    r+="type ";
    if(o.normale) r+="T"+o.nom.dernier()+" = class";          
    else          r+="I"+o.nom.dernier()+" = interface";

    if((o.meres.noms.size()>0)||(o.interfaces.noms.size()>0))
    {
      
      r+="(";
      boolean p=true;
      for(Enumeration e=o.meres.noms.elements(); e.hasMoreElements(); )
      {
	if(p) p=false; else r+=", ";
	r+="T"+((AglNom)e.nextElement()).dernier();
      }
      for(Enumeration e=o.interfaces.noms.elements(); e.hasMoreElements(); )
      {
	if(p) p=false; else r+=", ";
	r+="I"+((AglNom)e.nextElement()).dernier();
      }
      r+=")\n";
    }
    if(o.meres.noms.size()>1) 
      r+="  (* Warning: multiple class inheritance is not supported in Delphi 4 and former versions.*)\n";
    r+="  private\n";
    r+="  { Private Declarations }\n";
    if(o.attributs.size()>0)
    {
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
        AglAttribut aa=(AglAttribut)e.nextElement();
        if(aa.visibilite.vprivate)
          r+="    "+code(aa);
        if(aa.visibilite.vpublic)
          r+="    F"+code(aa);
      }
    }
    if(o.methodes.size()>0)
    {
      for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
      {
        AglMethode am=(AglMethode)e.nextElement();
        if(am.visibilite.vprivate)
          r+=code(am)+"\n";
      }
    }
    r+="  protected\n";
    r+="  { Protected Declarations }\n";
    if(o.attributs.size()>0)
    {
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
        AglAttribut aa=(AglAttribut)e.nextElement();
        if(!aa.visibilite.vpublic&&!aa.visibilite.vprivate)
          r+="    "+code(aa);
      }
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
        AglAttribut aa=(AglAttribut)e.nextElement();
        if(aa.visibilite.vpublic)
        {
	  r+="    function  Get_"+code(aa.nom)+": "+code(aa.type)+";\n";
	  r+="    procedure Set_"+code(aa.nom)+"(value: "+code(aa.type)+");\n";
        }
      }
    }
    if(o.methodes.size()>0)
    {
      for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
      {
        AglMethode am=(AglMethode)e.nextElement();
        if(!am.visibilite.vpublic&&!am.visibilite.vprivate)
          r+=code(am)+"\n";
      }
    }
    r+="  public\n";
    r+="  { Public Declarations }\n";
    if(o.methodes.size()>0)
    {
      for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
      {
        AglMethode am=(AglMethode)e.nextElement();
        if(am.visibilite.vpublic)
          r+=code(am)+"\n";
      }
    }
    if(o.attributs.size()>0)
    {
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
        AglAttribut aa=(AglAttribut)e.nextElement();
        if(aa.visibilite.vpublic)
        {
          r+="    property "+code(aa.nom)+": "+code(aa.type)+" read Get_"+code(aa.nom);
          if(!aa.visibilite.vreadonly)
            r+=" write Set_"+code(aa.nom);
          if(aa.initiale!=null)
            r+=" default "+code(aa.initiale);
          r+=";\n";
        }
      }
    }
    if(o.constructeurs.size()>0)
    {
      for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
      {
        AglConstructeur ac=(AglConstructeur)e.nextElement();
        r+=code(ac)+"\n";
      }
    }
    r+="end;\n\n";
    return r;
  }

 
  public String implementationProcess(AglClasse o)
  {
    indentation=0;
    String r="";

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+="\n"+code(am);
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+="\n"+code(am);
    }

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
        AglAttribut aa=(AglAttribut)e.nextElement();
        if(aa.visibilite.vpublic)
          r+="\n"+code(aa);
    }
 
    return r;
  }


  public String code(AglClasse o)
  {
    _classe=o;
    indentation=0;
    String r="";
    if(IN_INTERFACE)
    {
      r+=code(o.commentaires);
      r+=interfaceProcess(o);
    }
    else
    {
      r=implementationProcess(o);
    }
    return r.replace('"','\'');
  }


  public String code(AglConstructeur o)
  {
    indentation=0;
    String r="";
    if(!IN_INTERFACE)
    {
      r+="constructor T" +_classe.nom.dernier()+".create"; 
      if(o.parametres.size()>0)
      {
        r+="(";
        boolean p=true;
        for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
        {
          AglParametre ap=(AglParametre)e.nextElement();
          if(!p) r+="; ";
          r+=code(ap);
          p=false;
        }
        r+=")";
      }
      r+=";\n"+code(o.corps);
    }
    else
    {
      r+=code(o.commentaires);
      r+="    constructor create";
      if(o.parametres.size()>0)
      {
        r+="(";
        boolean p=true;
        for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
        {
          AglParametre ap=(AglParametre)e.nextElement();
          if(!p) r+="; ";
          r+=code(ap);
          p=false;
        }
        r+=")"; 
      }
      r+=";";
    }
      return r;
  }
 
  public String code(AglImport o)
  {
    String r="";
    return r;
 }

 
  public String code(AglMethode o)
  {
    indentation=0;
    String r="";

    if(IN_INTERFACE) r+=code(o.commentaires);

    String t="";
    r+=((IN_INTERFACE)?"    ":"")+"function  ";
    AglType at=o.type;
    while(at instanceof AglTypeAlias)
      at=((AglTypeAlias)at).type;
    if(at instanceof AglTypePrimitif)
    {
      t=code(at);
      if(t.equals("void")) r=((IN_INTERFACE)?"    ":"")+"procedure ";
    }
    else
      t=code(at);

    if(!IN_INTERFACE)
      r+=" T"+_classe.nom.dernier()+"."; 
    r+=o.nom;
    boolean p=true;
    if (o.parametres.size()>0) 
    {
      r+="(";  
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
        AglParametre ap=(AglParametre)e.nextElement();
        if(!p) r+="; ";
        r+=code(ap);
        p=false;
      }
      r+=")";
    }
    if(!t.equals("void"))
      r+=": "+t;
    r+=";";
    if(o.visibilite.vabstract&&IN_INTERFACE) 
      r+="virtual;abstract;";
    if(!IN_INTERFACE)
    {
     indentation++;
     p=true;
     for(Enumeration e=o.corps.instructions.elements(); e.hasMoreElements() ; )
     {
       AglInstruction ai=(AglInstruction)e.nextElement();
       if((ai instanceof AglInstructionDefVar))
       {
         if(p){ r+="\nvar\n"; p=false;}
         r+=indente()+((AglInstructionDefVar)ai).nom+": "+code(((AglInstructionDefVar)ai).type)+";\n";
       }
     }
     indentation--;
     if(p) r+="\n";
     r+=code(o.corps);
    }
    return r;
  }


  public String code(AglParametre o)
  {
    String r="";
    if(o.passage!=AglParametre.IN) r+="var ";
    r+=code(o.nom) + ": " + code(o.type);
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";
    if(o.vabstract)     r+="abtract; virtual;";
    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="";
    if (o.droite instanceof AglExpressionTernaire)
    {
      r+="if " + code(((AglExpressionTernaire)o.droite).gauche) + " then\n";
      indentation++;
	r+=indente()+code(o.gauche)+" := "+code(((AglExpressionTernaire)o.droite).milieu) +"\n";
      indentation--; 
 	r+=indente()+"else\n";
      indentation++;
      r+=indente()+code(o.gauche)+" := "+code(((AglExpressionTernaire)o.droite).droite);
      indentation--;
      return r;
    } 
    return "("+code(o.gauche)+" := "+code(o.droite)+")";
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    if(r.equals("this.")) r="self.";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r="";
    String g=code(o.gauche);
    String d=code(o.droite);
    String op=o.operateur;

    if(op.equals("+=")) 
    {
      r+=g+" := "+g+" + "+d; 
      return r;
    }  
    if(op.equals("/=")) 
    {
      r+=g+" := "+g+" div "+d; 
      return r;
    } 
    if(op.equals("*=")) 
    {
      r+=g+" := "+g+"*"+d; 
      return r;
    } 

    if(op.equals("%=")) 
    {
      r+=g+" := "+g+" mod "+d; 
      return r;
    } 
    if(op.equals("-=")) 
    {
      r+=g+" := "+g+" - "+d; 
      return r;
    }
    if(op.equals("^=")) 
    {
      r+=g+" := "+g+" xor "+d; 
      return r;
    }
    if(op.equals("|=")) 
    {
      r+=g+" := "+g+" or "+d; 
      return r;
    }
    if(op.equals("&=")) 
    {
      r+=g+" := "+g+" and "+d; 
      return r;
    }
    if(op.equals("<<=")) 
    {
      r+=g+" := "+g+" shl "+d; 
      return r;
    }
    if(op.equals(">>=")) 
    {
      r+=g+" := "+g+" shr "+d; 
      return r;
    }

    r+=g;
    if(op.equals("!=")) r+=" <> ";  else
    if(op.equals("==")) r+=" = ";   else
    if(op.equals("||")) r+=" or ";  else
    if(op.equals("&&")) r+=" and "; else
    if(op.equals("<<")) r+=" shl "; else
    if(op.equals(">>")) r+=" shr "; else 
    if(op.equals("/"))  r+=" div "; else
    if(op.equals("%"))  r+=" mod "; else
    if(op.equals("|"))  r+=" or ";  else
    if(op.equals("&"))  r+=" and "; else
    if(op.equals("^"))  r+=" xor "; else 
    if(op.equals("="))  r+=" := "; else 
      r+=op;
    r+=d;
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r="(*var "+o.nom+": "+code(o.type)+";\n*)";
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" is "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r="";
    if("this".equals(o.nom)) r+="self"; else r+=o.nom;
    if("super".equals(o.nom)) r="inherited create";
    if (o.arguments.size()>0)
    {
      r+="(";
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
        AglExpression ae=(AglExpression)e.nextElement();
        if(!p) r+=", ";
        r+=(code(ae).equals("this")?"self":code(ae));
        p=false;
      }
      r+=")";
    }
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  
  public String code(AglExpressionInitArray o)
  { String r="[";
    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }
    return r+"]";
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    if ("this.".equals(r)) r="self.";
    r+=code(o.fonction);
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+="; ";
      r+=code(ae);
      p=false;
    }

    return r;
  }


  public String code(AglExpressionNew o)
  {
    String r=code(o.type)+".create";
    if(o.arguments.size()>0)
    {
      r+="(";
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
        AglExpression ae=(AglExpression)e.nextElement();
        if(!p) r+=", ";
          r+=code(ae);
        p=false;
      }
      r+=")";
    }
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    return "";
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="("+code(o.expression)+")";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=
      code(o.gauche)+o.operateur1+
      code(o.milieu)+o.operateur2+
      code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r=code(o.type)+"("+code(o.expression)+")";
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r="";
    String op=o.operateur.trim();
    String co=code(o.droite);
    if(op.equals("++")) r+="inc("+co+")"; else
    if(op.equals("--")) r+="dec("+co+")"; else
    r+=o.operateur + code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r="";
    String op=o.operateur.trim();
    String co=code(o.gauche);
    if (op.equals("!")||op.equals("~"))
      r+=" not "+co; else 
    if(op.equals("++")) r+="inc("+co+")"; else
    if(op.equals("--")) r+="dec("+co+")"; else
    r+=code(o.gauche) + o.operateur ;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=code(o.commentaires)+indente();
    r+=o.body;
    if((r.length()>0)&&(r.charAt(r.length()-1)!='\n')) r+="\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=code(o.commentaires);
    //r+=indente()+";\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=code(o.commentaires)+indente();
    r+=code(o.gauche)+" := "+code(o.droite)+";\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=code(o.commentaires);
    if(!o.virgule)
      r+=code(o.corps);
    else
      r+=code(o.corps,false,true);
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=code(o.commentaires);
    r+=indente()+"break";
    if(o.nom!=null) r+=" "+o.nom;
    r+=";\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=code(o.commentaires);
    r+=indente()+"except\n";
    indentation++;
    String sc=code(o.signal);
    int k=sc.lastIndexOf(' ');
    r+=indente()+"on "+sc.substring(k)+ ": "+sc.substring(0,k)+" do \n"+code(o.reprise);
    indentation--;
    r+=indente()+"end;\n";
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=code(o.commentaires);
    r+=indente()+"continue";
    if(o.nom!=null) r+=" "+o.nom;
    r+=";\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=code(o.commentaires);
    r+=indente()+ o.nom+": "+code(o.type);
    if(o.initiale!=null&&o.visibilite.vpublic&&IN_INTERFACE) r+=" = "+code(o.initiale);
    r+=";\n";
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r=code(o.commentaires);
    r+=indente()+"repeat\n";
    r+=code(o.corps);
    r+=indente()+"until(not ("+code(o.condition)+"));\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=code(o.commentaires);
    String s=code(o.expression).trim();
    if(s.startsWith("(")&&s.endsWith(")")) s=s.substring(1,s.length()-1);
    r+=indente()+s+";\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=code(o.commentaires);
    r+=indente()+"finally\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r=code(o.commentaires);

    r+=code(o.debut);
    r+=indente()+"while ";
    if(o.test!=null) r+="("+code(o.test).trim()+")";
    else             r+="true";
    r+=" do\n";
    r+=indente()+"begin\n";
    indentation++;
    r+=code(o.corps,false,true);
    r+=code(o.fin);
    indentation--;
    r+=indente()+"end;\n";

    /*
    if(o.test instanceof AglExpressionBinaire)
    {
      r+=indente()+"for ";
      AglExpressionBinaire t=(AglExpressionBinaire)o.test;
      String to="";  
      String downto=""; 
      if(t.operateur.equals("<"))  to=code(t.droite)+"-1";
      if(t.operateur.equals("<=")) to=code(t.droite); 
      if(t.operateur.equals(">"))  downto=code(t.droite)+"+1";
      if(t.operateur.equals(">=")) downto=code(t.droite);

      String sd=code(o.debut).trim();
      int    ld=sd.length();
      if((ld>0)&&(sd.charAt(ld-1)==';'))
	sd=sd.substring(0,ld-1);
      if((ld>1)&&(sd.charAt(0)=='(')&&(sd.charAt(ld-2)==')'))
	sd=sd.substring(1,ld-2);
      r+=sd.trim();

      if(!to.equals("")) r+=" to "+to+" do\n";
      if(!downto.equals("")) r+=" downto "+downto+" do\n";
     	r+=code(o.corps);
    } 
    else
    {
      int k;
      indentation--;
      r+=indente();
      indentation++;
      r+=code(o.debut);
      r+=indente()+"while ("+code(o.test).trim()+") do\n";
      r+=code(o.corps);
      k=r.lastIndexOf(indente()+"end;");
      r=r.substring(0,k-1)+"\n";
      indentation++;
      r+=code(o.fin);
      indentation--;
      r+=indente()+"end;\n";
    }
    */
    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=code(o.commentaires);
    r+=indente()+"goto "+o.nom+";\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=code(o.commentaires);
    r+=indente()+"if("+code(o.condition)+") then\n";
    r+=code(o.sioui);
    if(o.sinon!=null)
    { 
     r+=r.substring(0,r.length()-2)+"\n";
     r+=indente()+"else\n";
     r+=code(o.sinon);
    }
    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=code(o.commentaires);
    r+=" "+o.nom+":\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=code(o.commentaires);
    r+=indente()+o.nom;
    if (o.arguments.size()>0)
      r+="(";

    boolean p=true;

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=("this".equals(code(ae))?"self":code(ae));
      p=false;
    }
    if(o.arguments.size()>0)
      r+=");\n";
    else
      r+=";\n";
    return r;
  }

   public String code(AglInstructionReturn o)
  {
    String r=code(o.commentaires);
    if(o.valeur!=null)
    { 
      r+=indente()+"result := "+code(o.valeur)+";\n";
      r+=indente()+"exit;\n";
    }
    else r+=indente()+"exit;\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=code(o.commentaires);
    r+=indente()+"case("+code(o.condition)+") of";

    r+="\n";
    indentation++;
    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      
      r+=indente()+code(ce)+":\n";
      indentation++;
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      r+=indente()+"break;\n";
      indentation--;
    }
    if (o.defaut!=null){
      r=r.substring(0,r.lastIndexOf(";"))+"\n";
      r+=indente()+"else\n";
      indentation++;
      r+=code(o.defaut);
      indentation--;
    }
    indentation--; 
    r+=indente()+"end;\n";

    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=code(o.commentaires);
    r+=indente()+"synchronized("+code(o.objet)+") do\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=code(o.commentaires);
    r+=indente()+"raise "+code(o.valeur)+";\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=code(o.commentaires);
    r+=indente()+"try\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=code(o.commentaires);
    r+=indente()+"while("+code(o.condition)+") do\n";
    r+=code(o.corps);
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    boolean p=true;
    indentation=0;
    String u=o.fichier;
    String ul=u;
    ul=ul.substring(0,ul.lastIndexOf('.')+1)+extension();

    r+=entete(ul,"Arona Kane","a.kane@maulde.com");

    int      i=u.lastIndexOf(File.separator);
    if(i>=0) u=u.substring(i+1);
    int      j=u.lastIndexOf('.');
    if(j>=0) u=u.substring(0,j);
    
    r+="\nunit "+u+";\n\n";

    r+="interface\n\n";
    IN_INTERFACE=true;
    if(o.imports.size()>0)
    {
      r+="uses ";
      for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
      {
        AglImport ai=(AglImport)e.nextElement();
        String imp=code(ai.nom).trim();
        int k=imp.lastIndexOf('_');
        if(imp.endsWith("*"))
        {
          imp=imp.substring(0,k);
          k=imp.lastIndexOf("_"); 
        }
        imp=imp.substring(k+1).trim();
        if(!p) r+=", ";
          r+=imp;
        p=false;
      }
      r+=";\n";
    }
    
    r+="\n";
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    r+="\nimplementation\n\n";
    IN_INTERFACE=false;
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    r+="\nend.\n\n"; 
    IN_INTERFACE=true;
    return r;
  }
}
