/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeDependance.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import java.io.*;
import java.util.*;

public class AglCodeDependance extends AglCode
{
  protected AglTableTriee unites_ =new AglTableTriee();
  protected AglTableTriee paquets_=new AglTableTriee();
  protected AglTableTriee types_  =new AglTableTriee();

  protected AglUnite      unite_  =null;
  protected AglPaquet     paquet_ =null;
  protected AglType       type_   =null;

  protected boolean       projet_ =false;

  public String extension()
  {
    return "txt";
  }

  public String[] extensions()
  {
    return new String[] { "txt", "TXT" };
  }

  protected String indente  () { return ""; }
  protected String debutCS  () { return ""; }
  protected String finCS    () { return ""; }
  protected String debutCM  () { return ""; }
  protected String specialCM() { return ""; }
  protected String milieuCM () { return ""; }
  protected String finCM    () { return ""; }

  public String code(AglNom o)
  {
    if(unite_!=null)
    {
      AglType at=AglType.cherche(o);
      if((at instanceof AglClasse)||(at instanceof AglStruct))
      {
        AglVecteur v;

	if(unite_!=null)
	{
	  v=(AglVecteur)unites_.get(unite_);
          if(v==null)
	  {
	    v=new AglVecteur(1,1);
            unites_.put(unite_,v);
	  }
          if(!v.contains(o)) v.addElement(o);
	}

	if(paquet_!=null)
	{
	  v=(AglVecteur)paquets_.get(paquet_);
          if(v==null)
	  {
	    v=new AglVecteur(1,1);
            paquets_.put(paquet_,v);
	  }
          if(!v.contains(o)) v.addElement(o);
	}

	if(type_!=null)
	{
	  v=(AglVecteur)types_.get(type_);
          if(v==null)
	  {
	    v=new AglVecteur(1,1);
            types_.put(type_,v);
	  }
          if(!v.contains(o)) v.addElement(o);
	}
      }
    }

    return "";
  }

  public String code(AglListeNoms o)
  {
    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
      code((AglNom)e.nextElement());
    return "";
  }

  public String code(AglType o)
  {
    o=AglTypeAlias.simplifie(o);

    if(o instanceof AglTypeTableau) code(((AglTypeTableau)o).type);
    else code(o.nom);

    return "";
  }

  public String code(AglAttribut o)
  {
    code(o.visibilite);
    code(o.type);
    if(o.initiale!=null) code(o.initiale);
    return "";
  }

  public String code(AglBloc o)
  {
    for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
    {
      AglInstruction ai=(AglInstruction)e.nextElement();
      code(ai);
    }
    return "";
  }

  public String code(AglStruct o)
  {
    type_=o;

    code(o.visibilite);
    code(o.nom);

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      code(aa);
    }

    type_=null;
    return "";
  }

  public String code(AglClasse o)
  {
    type_=o;

    code(o.visibilite);
    code(o.nom);

    if(o.meres.noms.size()>0)      code(o.meres);
    if(o.interfaces.noms.size()>0) code(o.interfaces);
    if(o.exceptions.noms.size()>0) code(o.exceptions);

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      code(aa);
    }

    if(o.clinit.instructions.size()>0) code(o.clinit);
    if(o.obinit.instructions.size()>0) code(o.obinit);

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur ac=(AglConstructeur)e.nextElement();
      code(ac);
    }
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      code(am);
    }

    type_=null;
    return "";
  }

  public String code(AglConstructeur o)
  {
    code(o.visibilite);
    code(o.nom);

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      code(ap);
    }

    if(!o.corps.instructions.isEmpty()) code(o.corps);

    return "";
  }

  public String code(AglPaquet o)
  {
    return "";
  }

  public String code(AglImport o)
  {
    return "";
  }

  public String code(AglMethode o)
  {
    code(o.visibilite);
    code(o.type);

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      code(ap);
    }

    if(!o.corps.instructions.isEmpty()) code(o.corps);

    return "";
  }

  public String code(AglParametre o)
  {
    code(o.type);
    return "";
  }

  public String code(AglVisibilite o)
  {
    return "";
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    return "";
  }

  public String code(AglExpressionAffectation o)
  {
    code(o.gauche);
    code(o.droite);
    return "";
  }

  public String code(AglExpressionAttribut o)
  {
    if(o.objet!=null) code(o.objet);
    return "";
  }

  public String code(AglExpressionBinaire o)
  {
    code(o.gauche);
    code(o.droite);
    return "";
  }

  public String code(AglExpressionDefVar o)
  {
    code(o.type);
    if(o.initiale!=null) code(o.initiale);
    return "";
  }

  public String code(AglExpressionEstUn o)
  {
    code(o.expression);
    code(o.type);
    return "";
  }

  public String code(AglExpressionFonction o)
  {
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglExpressionIndice o)
  {
    code(o.objet);
    code(o.indice);
    return "";
  }

  public String code(AglExpressionInitArray o)
  {
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglExpressionMethode o)
  {
    if(o.objet!=null) code(o.objet);
    code(o.fonction);
    return "";
  }

  public String code(AglExpressionMultiple o)
  {
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglExpressionNew o)
  {
    code(o.type);
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglExpressionNewArray o)
  {
    code(o.type);
    code(o.nombre);
    if(o.initiale!=null) code(o.initiale);
    return "";
  }

  public String code(AglExpressionParenthesee o)
  {
    code(o.expression);
    return "";
  }

  public String code(AglExpressionTernaire o)
  {
    code(o.gauche);
    code(o.milieu);
    code(o.droite);
    return "";
  }

  public String code(AglExpressionTypage o)
  {
    code(o.type);
    code(o.expression);
    return "";
  }

  public String code(AglExpressionUnaireDroite o)
  {
    code(o.droite);
    return "";
  }

  public String code(AglExpressionUnaireGauche o)
  {
    code(o.gauche);
    return "";
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    return "";
  }

  public String code(AglInstructionVide o)
  {
    return "";
  }

  public String code(AglInstructionAffectation o)
  {
    code(o.gauche);
    code(o.droite);
    return "";
  }

  public String code(AglInstructionBloc o)
  {
    code(o.corps);
    return "";
  }

  public String code(AglInstructionBreak o)
  {
    return "";
  }

  public String code(AglInstructionCatch o)
  {
    code(o.signal);
    code(o.reprise);
    return "";
  }

  public String code(AglInstructionContinue o)
  {
    return "";
  }

  public String code(AglInstructionDefVar o)
  {
    code(o.type);
    if(o.initiale!=null) code(o.initiale);
    return "";
  }

  public String code(AglInstructionDoWhile o)
  {
    code(o.corps);
    code(o.condition);
    return "";
  }

  public String code(AglInstructionExpression o)
  {
    code(o.expression);
    return "";
  }

  public String code(AglInstructionFinally o)
  {
    code(o.corps);
    return "";
  }

  public String code(AglInstructionFor o)
  {
    code(o.debut);
    code(o.test);
    code(o.fin);
    code(o.corps);
    return "";
  }

  public String code(AglInstructionGoto o)
  {
    return "";
  }

  public String code(AglInstructionIf o)
  {
    code(o.condition);
    code(o.sioui);
    if(o.sinon!=null) code(o.sinon);
    return "";
  }

  public String code(AglInstructionLabel o)
  {
    return "";
  }

  public String code(AglInstructionProcedure o)
  {
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglInstructionReturn o)
  {
    code(o.valeur);
    return "";
  }

  public String code(AglInstructionSwitch o)
  {
    code(o.condition);

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      code(ce);
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      if(cb!=null) code(cb);
      else
      {
	System.err.println("BLOC NULL: "+ce+
			   " ["+ce.ligne+"]");
	if(ce instanceof AglExpressionAttribut)
	  System.err.println("  "+((AglExpressionAttribut)ce).attribut);
	for(Enumeration f=o.cas.elements(); f.hasMoreElements(); )
        {
	    System.err.println("  "+f.nextElement());
	}
      }
    }

    code(o.defaut);
    return "";
  }

  public String code(AglInstructionSynchronized o)
  {
    code(o.objet);
    code(o.corps);
    return "";
  }

  public String code(AglInstructionThrow o)
  {
    code(o.valeur);
    return "";
  }

  public String code(AglInstructionTry o)
  {
    code(o.corps);
    return "";
  }

  public String code(AglInstructionWhile o)
  {
    code(o.condition);
    code(o.corps);
    return "";
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";

    if(!projet_) r+=entete(o.fichier);

    unite_=o;
    paquet_=o.paquet;

    // Ajout imports
    // TMP
    if(unite_!=null)
    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglNom ni=((AglImport)e.nextElement()).nom.parent();
      if(ni==null) continue;

      AglVecteur v=(AglVecteur)unites_.get(unite_);
      if(v==null)
      {
	v=new AglVecteur(1,1);
	unites_.put(unite_,v);
      }
      if(!v.contains(o)) v.addElement(ni);

      if(paquet_!=null)
      {
        v=(AglVecteur)paquets_.get(paquet_);
        if(v==null)
        {
	  v=new AglVecteur(1,1);
	  paquets_.put(paquet_,v);
        }
        if(!v.contains(o)) v.addElement(ni);
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
	  code(as);
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
	  code(ac);
      }
    }

    if(!projet_) r+=dependanceUnite(o);

    unite_ =null;
    paquet_=null;
    return r;
  }

  public final AglSortie[] superCode(AglProjet o)
  {
    return super.code(o);
  }

  public AglSortie[] code(AglProjet o)
  {
    projet_=true;
    AglSortie[] s1=superCode(o);
    AglSortie[] s2=dependances();
    projet_=false;

    AglSortie[] r=new AglSortie[s1.length+s2.length];
    int i;
    for(i=0;i<s1.length;i++) r[i          ]=s1[i];
    for(i=0;i<s2.length;i++) r[i+s1.length]=s2[i];
    return r;
  }

  protected String rappelleClasses  () { return ""; }
  protected String rappellePatrons  () { return ""; }
  protected String rappelleRelations() { return ""; }

  protected AglSortie[] dependances()
  {
    AglSortie[] r=new AglSortie[3];
    String      s;

    s="";
    for(Enumeration e=paquets_.keys(); e.hasMoreElements(); )
      s+=dependancePaquet((AglPaquet)e.nextElement());
    r[0]=new AglSortie("text/plain",null,null,s);

    s="";
    for(Enumeration e=types_.keys(); e.hasMoreElements(); )
      s+=dependanceType((AglType)e.nextElement());
    r[1]=new AglSortie("text/plain",null,null,s);

    s="";
    for(Enumeration e=unites_.keys(); e.hasMoreElements(); )
      s+=dependanceUnite((AglUnite)e.nextElement());
    r[2]=new AglSortie("text/plain",null,null,s);

    return r;
  }

  protected String dependanceUnite(AglUnite o)
  {
    String r="U "+o.fichier+"\n";

    AglVecteur v=(AglVecteur)unites_.get(o);
    if(v!=null)
    {
      AglTableTriee t=new AglTableTriee();

      for(Enumeration e=v.elements(); e.hasMoreElements(); )
      {
        AglNom   an=(AglNom)e.nextElement();
	AglType  at=AglType.cherche(an);
	if(at!=null)
	{
	  AglObjet ap=at.proprietaire;
	  if(ap instanceof AglUnite)
	  {
	    AglUnite au=(AglUnite)ap;
	    if(au!=o)
	      t.put(au.fichier,Boolean.TRUE);
	  }
	}
      }

      for(Enumeration e=t.keys(); e.hasMoreElements(); )
	r+="U |_ "+e.nextElement()+"\n";
    }

    r+="\n";
    return r;
  }

  protected String dependancePaquet(AglPaquet o)
  {
    String r="P "+o.nom.formeJava()+"\n";

    AglVecteur v=(AglVecteur)paquets_.get(o);
    if(v!=null)
    {
      AglTableTriee t=new AglTableTriee();

      for(Enumeration e=v.elements(); e.hasMoreElements(); )
      {
        AglNom   an=(AglNom)e.nextElement();
	AglType  at=AglType.cherche(an);
	if(at!=null)
	{
	  AglNom ap=at.nom.parent();
	  if((ap!=null)&&!ap.equals(o.nom)&&(AglPaquet.table.get(ap)!=null))
	    t.put(ap.formeJava(),Boolean.TRUE);
	}
      }

      for(Enumeration e=t.keys(); e.hasMoreElements(); )
	r+="P |_ "+e.nextElement()+"\n";
    }

    r+="\n";
    return r;
  }

  protected String dependanceType(AglType o)
  {
    String r="C "+o.nom.formeJava()+"\n";

    AglVecteur v=(AglVecteur)types_.get(o);
    if(v!=null)
    {
      AglTableTriee t=new AglTableTriee();

      for(Enumeration e=v.elements(); e.hasMoreElements(); )
      {
        AglNom   an=(AglNom)e.nextElement();
	AglType  at=AglType.cherche(an);
	if((at!=null)&&(at!=o))
	  t.put(at.nom.formeJava(),Boolean.TRUE);
      }

      for(Enumeration e=t.keys(); e.hasMoreElements(); )
	r+="C |_ "+e.nextElement()+"\n";
    }

    r+="\n";
    return r;
  }
}
