/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeDocHtml.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
// Auteur : Guillaume Desnoix,Axel von Arnim
// Mel    : guillaume@desnoix.com,axel.vonarnim@equipement.gouv.fr
 * @license      GNU General Public License 2 (GPL2)
//          Copyright (c)1998-2001 Guillaume Desnoix,Axel von Arnim.
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.net.*;
import java.text.*;
import java.util.*;

/**
 * Generation de documentation sous forme de pages HTML.
 * La premiere version a ete ecrite par Axel.
 * @author Guillaume Desnoix
 * @email  guillaume@desnoix.com
 * @author Axel von Arnim
 * @email  axel.vonarnim@equipement.gouv.fr
 */

public class AglCodeDocHtml extends AglCode
{
  protected boolean FRAMES=
    AglPreferences.AGL.getBooleanProperty("dochtml.code.frames",true);

  protected boolean JAVADOC=
    AglPreferences.AGL.getBooleanProperty("dochtml.code.javadoc",false);

  protected boolean COLORATION=
    AglPreferences.AGL.getBooleanProperty("dochtml.code.coloration",true);

  protected String  BASE=
    AglPreferences.AGL.getStringProperty("dochtml.code.base","");

  protected String  SOURCE=
    AglPreferences.AGL.getStringProperty("dochtml.code.source","");

  AglUnite unite_;
  String   en_tete;
  int      classes=0;
  int      attributs=0;
  int      constructeurs=0;
  int      methodes=0;
  //String classeCourante;
  String   langage;

  public String extension()
  {
    return "html";
  }

  public String[] extensions()
  {
    return new String[] { "html", "HTML", "htm", "HTM" };
  }

  public boolean optionStyleHtml()
  {
    return true;
  }

  public String syntaxe()
  {
    return "Html";
  }

  protected String debutCS  () { return indente()+"<!-- "; }
  protected String finCS    () { return " -->";            }
  protected String debutCM  () { return indente()+"<!-- "; }
  protected String specialCM() { return "";                }
  protected String milieuCM () { return indente()+"  ";    }
  protected String finCM    () { return indente()+"-->";   }

  private String transcode(String _s)
  {
    String r=_s.trim();

    r=FuLib.replaceIgnoreCase(r,". *",".");
    r=FuLib.replaceIgnoreCase(r,"&nbsp;"," ");
    r=FuLib.replaceIgnoreCase(r,"<font size=\"-2\"><sup>TM</sup></font>","");

    r=FuLib.replaceIgnoreCase(r,"&lt;"  ,"<");
    r=FuLib.replaceIgnoreCase(r,"&gt;"  ,">");

    r=FuLib.replaceIgnoreCase(r,"&" ,"&amp;" );
    r=FuLib.replaceIgnoreCase(r,"\"","&quot;");
    r=FuLib.replaceIgnoreCase(r,">" ,"&gt;"  );
    r=FuLib.replaceIgnoreCase(r,"<" ,"&lt;"  );

    r=FuLib.replaceIgnoreCase(r,"&lt;!--"      ,"<!--"   );
    r=FuLib.replaceIgnoreCase(r,"--&gt;"       ,"-->"    );
    r=FuLib.replaceIgnoreCase(r,"&lt;i&gt;"    ,"<I>"    );
    r=FuLib.replaceIgnoreCase(r,"&lt;/i&gt;"   ,"</I>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;b&gt;"    ,"<B>"    );
    r=FuLib.replaceIgnoreCase(r,"&lt;/b&gt;"   ,"</B>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;tt&gt;"   ,"<TT>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/tt&gt;"  ,"</TT>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;br&gt;"   ,"<BR>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;p&gt;"    ,"<BR>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/p&gt;"   ,""       );
    r=FuLib.replaceIgnoreCase(r,"&lt;P&gt;"    ,"<BR>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/P&gt;"   ,""       );
    r=FuLib.replaceIgnoreCase(r,"&lt;em&gt;"   ,"<EM>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/em&gt;"  ,"</EM>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;ul&gt;"   ,"<UL>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/ul&gt;"  ,"</UL>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;li&gt;"   ,"<LI>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/li&gt;"  ,"</LI>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;pre&gt;"        ,"\n<CODE>"       );
    r=FuLib.replaceIgnoreCase(r,"&lt;/pre&gt;"       ,"</CODE>\n"      );
    r=FuLib.replaceIgnoreCase(r,"&lt;code&gt;"       ,"<CODE>"         );
    r=FuLib.replaceIgnoreCase(r,"&lt;/code&gt;"      ,"</CODE>"        );
    r=FuLib.replaceIgnoreCase(r,"&lt;blockquote&gt;" ,"\n<BLOCKQUOTE>" );
    r=FuLib.replaceIgnoreCase(r,"&lt;/blockquote&gt;","</BLOCKQUOTE>\n");
    r=FuLib.replaceIgnoreCase(r,"&lt;table&gt;"      ,"\n<TABLE>"      );
    r=FuLib.replaceIgnoreCase(r,"&lt;/table&gt;"     ,"</TABLE>\n"     );
    r=FuLib.replaceIgnoreCase(r,"&lt;td&gt;"   ,"<TD>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/td&gt;"  ,"</TD>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;th&gt;"   ,"<TH>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/th&gt;"  ,"</TH>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;tr&gt;"   ,"<TR>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/tr&gt;"  ,"</TR>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;dl&gt;"   ,"<TABLE>"   );
    r=FuLib.replaceIgnoreCase(r,"&lt;/dl&gt;"  ,"</TABLE>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;dt&gt;"   ,"<TR><TH>"  );
    r=FuLib.replaceIgnoreCase(r,"&lt;/dt&gt;"  ,"</TH>"     );
    r=FuLib.replaceIgnoreCase(r,"&lt;dd&gt;"   ,"<TD>"      );
    r=FuLib.replaceIgnoreCase(r,"&lt;/dd&gt;"  ,"</TD></TR>");

    r=FuLib.replaceIgnoreCase(r,"<BR> *","<BR>");

    r=FuLib.replace(r,"@author"    ,"<B>Auteur:</B>");
    r=FuLib.replace(r,"@email"     ,"<B>M&eacute;l:</B>");
    r=FuLib.replace(r,"@version"   ,"<B>Version:</B>");
    r=FuLib.replace(r,"@serial"    ,"<B>Num&eacute;ro de s&eacute;rie:</B>");
    r=FuLib.replace(r,"@since"     ,"<B>Depuis:</B>");
    r=FuLib.replace(r,"@deprecated","<B>Obsol&egrave;te:</B>");
    r=FuLib.replace(r,"@see #"     ,"<B>Voir:</B>");
    r=FuLib.replace(r,"@see"       ,"<B>Voir:</B>");
    r=FuLib.replace(r,"@param"     ,"<B>Param&egrave;tre:</B>");
    r=FuLib.replace(r,"@return"    ,"<B>Retour:</B>");
    r=FuLib.replace(r,"@exception" ,"<B>Exception:</B>");

    r=FuLib.replace(r,"@invariant","<B>Invariant:</B>");
    r=FuLib.replace(r,"@require"  ,"<B>Requis:</B>");
    r=FuLib.replace(r,"@ensure"   ,"<B>Garanti:</B>");

    while(r.startsWith("<BR>")) r=r.substring(4).trim();
    while(r.endsWith  ("<BR>")) r=r.substring(0,r.length()-4).trim();

    int i;
    while((i=r.indexOf("{@link "))>=0)
    {
      int j=r.indexOf("}",i);
      if(j>=0)
      {
	String a=r.substring(i+7,j).trim();
	a=base_link(a);
	r=r.substring(0,i)+a+r.substring(j+1);
      }
    }

    return r;
  }

  protected String source(String _fichier)
  {
    String r=" - <A HREF=\""+SOURCE;
    r+="Source-"+_fichier.replace('/','.')+".html\"";
    if(FRAMES) r+=" TARGET=\"sourceFrame\"";
    r+=">";
    //r+="<IMG ALIGN=BASE BORDER=0 SRC=\"dochtml-source.gif\">&nbsp;";
    r+="code source</A>";
    return r;
  }

  protected String base_paquet(String _fichier)
  {
    String r=" <A HREF=\""+BASE;
    if(!JAVADOC) r+=langage+"-module-";
    r+=_fichier+".html\"";
    if(FRAMES) r+=" TARGET=\"packageFrame\"";
    r+=">";
    //r+="<IMG ALIGN=BASE BORDER=0 SRC=\"dochtml-base.gif\">&nbsp;";
    r+=_fichier+"</A>";
    return r;
  }

  protected String base_classe(String _fichier)
  {
    String r=" <A HREF=\""+BASE;
    if(!JAVADOC) r+=langage+"-";
    r+=_fichier+".html\"";
    if(FRAMES) r+=" TARGET=\"classFrame\"";
    r+=">";
    //r+="<IMG ALIGN=BASE BORDER=0 SRC=\"dochtml-base.gif\">&nbsp;";
    r+=_fichier+"</A>";
    return r;
  }

  protected String base_link(String _link)
  {
    String r   ="";
    String lang=langage+"-";
    int    j;

    _link=_link.trim();

    j=_link.indexOf('-');
    if(j>=0)
    {
      lang =_link.substring(0,j+1);
      _link=_link.substring(j+1);
    }

    j=_link.indexOf('#');
    if(j==0)
    {
      r=_link.substring(1);
    }
    else
    if(j>0)
    {
      r=_link.substring(0,j);
      AglType t=AglType.cherche(r);
      if(t!=null)
	r="<A HREF=\""+BASE+lang+code(t.nom)+".html\">"+
	  r+"</A>."+_link.substring(j+1);
      else
	r=_link.replace('#','.');
    }
    else
    {
      r=_link;
      AglType t=AglType.cherche(r);
      if(t!=null)
	r="<A HREF=\""+BASE+lang+code(t.nom)+".html\">"+
	  r+"</A>";
      else
	r=_link;
    }

    return "<CODE>"+r+"</CODE>";
  }

  public String liste(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      AglNom n=(AglNom)e.nextElement();
      AglType t=AglType.cherche(n);
      if(t!=null) r+="<CODE>"+code(t)+"</CODE>";
      else        r+=code(n);
    }

    return r;
  }

  public String prototype(AglObjet o)
  {
    String r="";

    if(AglCode.disponible(langage))
    {
      AglCode l=AglCode.langage(langage);
      int     i;

      l.COMMENTAIRES=false;
      r=l.code(o);
      //i=r.indexOf('/'); if(i>=0) r=r.substring(0,i);
      i=r.indexOf('{'); if(i>=0) r=r.substring(0,i);
      i=r.indexOf(';'); if(i>=0) r=r.substring(0,i);

      r=r.trim();
      r=FuLib.replace(r,"\"" ,"&quot;");
      r=FuLib.replace(r,"\n" ," ");
      //r=FuLib.replace(r,"   "," ");
      while(r.indexOf("  ")>=0) r=FuLib.replace(r,"  " ," ");
      //r=FuLib.replace(r," "  ,"&nbsp;");

      r="<TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=0 CELLPADDING=0>\n"+
	"<TR BGCOLOR=\"#000000\"><TD>\n"+
	"<TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=1 CELLPADDING=2>\n"+
	"<TR BGCOLOR=\"#FFFFFF\"><TD ALIGN=LEFT><SMALL>\n"+
	r+
	"</SMALL></TD></TR>\n"+
	"</TABLE>\n"+
	"</TD></TR>\n"+
	"</TABLE>\n";
      //r="&nbsp;<IMG SRC=\"dochtml-proto.gif\" ALT=\""+r+"\">";
    }

    return r;
  }

  public String declaration(AglVisibilite o)
  {
    String r="<TT>";

         if(o.vpublic)    r+="+";
    else if(o.vprivate)   r+="-";
    else if(o.vprotected) r+="#";
    else                  r+="&nbsp;";

	 //     if(o.vstatic)    r+="$";
	 //else                  r+="&nbsp;";

    r+="&nbsp;</TT>";
    return r;
  }

  public String declaration(AglType o)
  {
    String r="";
    o=AglTypeAlias.simplifie(o,"Idl");

    if(o instanceof AglTypeTableau)
	r+=declaration(((AglTypeTableau)o).type)+"[]";
    else r+=o.nom.dernier();

    return r;
  }

  public String declaration(AglAttribut o)
  {
    String  r="&nbsp;:&nbsp;"+declaration(o.type);
    return r;
  }

  public String declaration(AglConstructeur o)
  {
    String  r="";
    r+="&nbsp;(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(p) p=false; else r+=",&nbsp;";
      r+=declaration(ap.type);
    }

    r+=")";
    return r;
  }

  public String declaration(AglMethode o)
  {
    String  r="";
    r+="&nbsp;(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(p) p=false; else r+=",&nbsp;";
      r+=declaration(ap.type);
    }

    r+=")";

    if(AglTypeAlias.simplifie(o.type)!=AglTypePrimitif.tvoid)
      r+="&nbsp;:&nbsp;"+declaration(o.type);

    return r;
  }

  public String code(AglCommentaireSimple o)
  {
    /*
    if((o.texte==null)||o.texte.equals("")) return "";
    return "<SMALL>"+transcode(o.texte)+"</SMALL><BR>\n";
    */
    return "";
  }

  public String code(AglCommentaireMulti o)
  {
    if((o==null)||(o.clefs.size()==0)||!o.special) return "";

    String r="";

    String mv=(String)o.clefs.get("@main");
    if(mv!=null) r+=transcode(mv)+"<BR><BR>\n";

    for(Enumeration e=o.clefs.keys(); e.hasMoreElements(); )
    {
      String k=(String)e.nextElement();
      if(!"@main".equals(k))
      {
	AglVecteur l=(AglVecteur)o.clefs.get(k);
	for(int i=0;i<l.size();i++)
	{
	  String s=transcode((String)l.elementAt(i)).trim();

	  if(k.startsWith("@see"))
	  {
	    String a=base_link(s);
	    r+=transcode(k)+" "+a+"<BR>\n";

	    /*
	    String a=s;
	    int j=a.indexOf('#');
	    if(j>0)
	    {
	      a=a.substring(0,j);
	      r+=transcode(k)+" ";
	      r+="<A HREF=\""+a+".html\"><CODE>"+s+"</CODE></A>";
	      r+="<BR>\n";
	    }
	    */
	  }
	  else
	  if(k.startsWith("@param"))
	      ;
	  else
	  if(k.startsWith("@return"))
	      ;
	  else
	  if(k.startsWith("@exception"))
	  {
	    r+=transcode(k)+" ";

	    int j=s.indexOf(' ');
	    if(j>0)
	      r+="<CODE>"+s.substring(0,j)+"</CODE>"+s.substring(j);
	    else
	     r+=s;

	    r+="<BR>\n";
	  }
	  else
	    r+=transcode(k)+" "+s+"<BR>\n";
	}
      }
    }

    if(!"".equals(r)) r="<P>"+r+" </P>\n";
    //r="<SMALL>"+r+"<BR></SMALL>\n";
    return r;
  }

  public String commentaireParam(String _nom, AglVecteur _v)
  {
    if((_v==null)||(_v.size()==0)) return "";

    String r="";

    Enumeration f=_v.elements();
    while(f.hasMoreElements())
    {
      Object o=f.nextElement();
      if(o instanceof AglCommentaireMulti)
      {
	AglCommentaireMulti c=(AglCommentaireMulti)o;
	if((c.clefs.size()==0)||!c.special) continue;

	for(Enumeration e=c.clefs.keys(); e.hasMoreElements(); )
        {
	  String k=(String)e.nextElement();
	  if("@param".equals(k))
          {
	    AglVecteur l=(AglVecteur)c.clefs.get(k);
	    for(int i=0;i<l.size();i++)
	    {
	      String s=transcode((String)l.elementAt(i)).trim();
	  
	      int j=s.indexOf(' ');
	      if(j>0)
	      {
	        if(_nom.equals(s.substring(0,j)))
	        r+="<BR>"+s.substring(j);
	      }
	    }
	  }
	}
      }
    }

    return r;
  }

  public String commentaireReturn(AglVecteur _v)
  {
    if((_v==null)||(_v.size()==0)) return "";

    String r="<UL>\n";

    Enumeration f=_v.elements();
    while(f.hasMoreElements())
    {
      Object o=f.nextElement();
      if(o instanceof AglCommentaireMulti)
      {
	AglCommentaireMulti c=(AglCommentaireMulti)o;
	if((c.clefs.size()==0)||!c.special) continue;

	for(Enumeration e=c.clefs.keys(); e.hasMoreElements(); )
        {
	  String k=(String)e.nextElement();
	  if("@return".equals(k))
          {
	    AglVecteur l=(AglVecteur)c.clefs.get(k);
	    for(int i=0;i<l.size();i++)
	    {
	      String s=transcode((String)l.elementAt(i)).trim();
	      r+="<LI>"+s+"</LI>\n";
	    }
	  }
	}
      }
    }

    r+="</UL>\n";
    if("<UL>\n</UL>\n".equals(r)) r="";
    return r;
  }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=".";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
    if(o instanceof AglTypeAlias) o=AglTypeAlias.simplifie(o,langage);

    if(o instanceof AglTypeTableau)
      r+=code(((AglTypeTableau)o).type)+"["+code(((AglTypeTableau)o).dimension)+"]";
    else if(AglTypeAlias.simplifie(o) instanceof AglClasse)
    {
      if("Java".equals(langage))    r+=base_classe(code(o.nom));
      else if(o.nom.parent()!=null) r+=base_paquet(code(o.nom.parent()))+"."+o.nom.dernier();
      else                          r+=o.nom.dernier();
    }
    else r+=code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    String r="";
    r+="<FONT COLOR=\"#00A000\"><CODE><B>"+o.nom+"</B></CODE></FONT><BR>\n";
    //r+=prototype(o);
    r+="<B>Modifieurs : </B>"+code(o.visibilite)+"<BR>\n";
    r+="<B>Type : </B><CODE>"+code(o.type)+"</CODE><BR>\n";

    /*
    if((o.initiale!=null)&&AglCode.disponible(langage))
    {
      AglCode l=AglCode.langage(langage);
      String  s="";
      int     i;

      l.COMMENTAIRES=false;
      s=l.code(o.initiale);
      //i=s.indexOf('/'); if(i>=0) s=s.substring(0,i);
      i=s.indexOf('{'); if(i>=0) s=s.substring(0,i);
      i=s.indexOf(';'); if(i>=0) s=s.substring(0,i);
      
      s=s.trim();
      s=FuLib.replace(s,"\"","&quot;");
      s=FuLib.replace(s," " ,"&nbsp;");
      s=FuLib.replace(s,"\n"," ");
      
      r+="&nbsp;&nbsp;valeur initiale : <CODE>"+
	s+"</CODE><BR>\n";
    }
    */

    r+=code(o.commentaires);
    return r;
  }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    String    listeNoms[];
    Hashtable tableNoms;

    String r="";

    //classeCourante=code(o.nom);

    /*
    String nom_reduit =code(o.nom);//classeCourante;
    String nom_paquet="";

    int i=nom_reduit.lastIndexOf('.');
    if(i>=0)
    {
      nom_paquet=nom_reduit.substring(0,i);
      nom_reduit =nom_reduit.substring(i+1,nom_reduit.length());
    }
    */

    // WIDTH=\"98%\" 
    en_tete+="<BR><CENTER><TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0>\n";
    en_tete+="<TR BGCOLOR=\"#000000\"><TD>\n";
    en_tete+="<TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=1 CELLPADDING=2>\n";
    en_tete+="<TR BGCOLOR=\"#FFFFFF\"><TD ALIGN=MIDDLE><SMALL>\n";
    en_tete+="&lt;&lt;struct&gt;&gt;<BR>\n";
    en_tete+=/*declaration(o.visibilite)+*/
      "<A HREF=\"#"+URLEncoder.encode(code(o.nom))+"\">"+
      o.nom.dernier()+"</A><BR>\n";
    en_tete+="</SMALL></TD></TR>\n";

    r+="<A NAME=\""+URLEncoder.encode(code(o.nom))+"\"></A>\n";
    r+="<H2>";
    r+="Structure";
    r+=" "+o.nom.dernier()+"</H2>\n";

    /*
    if(o.meres.noms.size()>0)
      r+="<B>d&eacute;rive de : </B>"+liste(o.meres)+"<BR>\n";
    if(o.interfaces.noms.size()>0)
      r+="<B>impl&eacute;mente : </B>"+liste(o.interfaces)+"<BR>\n";
    if(o.exceptions.noms.size()>0)
      r+="<B>lance : </B>"+liste(o.exceptions)+"<BR>\n";
    r+="<B>modifieurs : </B>"+code(o.visibilite)+"<BR>\n";
    */
    r+="<BR>\n";

    r+=code(o.commentaires);

    en_tete+="<TR BGCOLOR=\"#FFFFFF\"><TD><SMALL>\n";

    if(o.attributs.size()>0)
    {
      r+="<UL>\n";

      int i=0;
      listeNoms=new String[o.attributs.size()];
      tableNoms=new Hashtable();
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
        AglAttribut aa=(AglAttribut)e.nextElement();
        tableNoms.put(aa.signature(),aa);
        listeNoms[i++]=aa.signature();
      }
      sort(listeNoms);

      for(i=0; i<listeNoms.length; i++)
      {
        AglAttribut aa=(AglAttribut)tableNoms.get(listeNoms[i]);

	en_tete+=declaration(aa.visibilite)+"<A HREF=\"#"+
	    URLEncoder.encode(aa.signature())+"\">";
	if(aa.visibilite.vstatic) en_tete+="<U>";
	en_tete+=aa.nom;
	if(aa.visibilite.vstatic) en_tete+="</U>";
	en_tete+="</A>"+declaration(aa)+"<BR>\n";

	r+="<A NAME=\""+URLEncoder.encode(aa.signature())+"\"></A>\n";
        r+="<LI>"+code(aa)+"</LI>\n";
      }   

      r+="</UL>\n";
    }
    else
      en_tete+="&nbsp; ";

    /*
    en_tete+="</SMALL></TD></TR>\n";
    en_tete+="<TR BGCOLOR=\"#FFFFFF\"><TD><SMALL>\n";

    if(o.constructeurs.size()>0)
    {
      r+="<H3>Constructeurs</H3>\n";
      r+="<UL>\n";

      int i=0;
      listeNoms=new String[o.constructeurs.size()];
      tableNoms=new Hashtable();
      for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
      {
        AglConstructeur ac=(AglConstructeur)e.nextElement();
        tableNoms.put(ac.signature(),ac);
        listeNoms[i]=ac.signature();
	i++;
      }
      sort(listeNoms);

      for(i=0; i<listeNoms.length; i++)
      {
        AglConstructeur ac=(AglConstructeur)tableNoms.get(listeNoms[i]);

	en_tete+=declaration(ac.visibilite)+"<A HREF=\"#"+ac.signature()+"\">";
	if(ac.visibilite.vstatic) en_tete+="<U>";
	en_tete+=ac.nom.dernier();
	if(ac.visibilite.vstatic) en_tete+="</U>";
	en_tete+="</A>"+declaration(ac)+"<BR>\n";

	r+="<A NAME=\""+ac.signature()+"\"></A>\n";
        r+="<LI>"+code(ac)+"</LI>\n";
      }

      r+="</UL>\n";
    }

    if(o.methodes.size()>0)
    {
      r+="<H3>M&eacute;thodes</H3>\n";
      r+="<UL>\n";

      int i=0;
      listeNoms=new String[o.methodes.size()];
      tableNoms=new Hashtable();
      for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
      {
        AglMethode am=(AglMethode)e.nextElement();
        tableNoms.put(am.signature(),am);
        listeNoms[i++]=am.signature();
      }
      sort(listeNoms);
      for(i=0; i<listeNoms.length; i++)
      {
        AglMethode am=(AglMethode)tableNoms.get(listeNoms[i]);

	en_tete+=declaration(am.visibilite)+"<A HREF=\"#"+am.signature()+"\">";
	if(am.visibilite.vstatic) en_tete+="<U>";
	en_tete+=am.nom;
	if(am.visibilite.vstatic) en_tete+="</U>";
	en_tete+="</A>"+declaration(am)+"<BR>\n";

	r+="<A NAME=\""+am.signature()+"\"></A>\n";
        r+="<LI>"+code(am)+"</LI>\n";
      }   

      r+="</UL>\n";
    }

    if((o.constructeurs.size()==0)&&(o.methodes.size()==0))
      en_tete+="&nbsp; ";
    */

    en_tete+="</SMALL></TD></TR>";
    en_tete+="</TABLE>\n</TD></TR>\n</TABLE></CENTER>\n";
    en_tete+="<BR>\n";

    return r+"<BR>\n";
  }

  public String code(AglClasse o)
  {
    String    listeNoms[];
    Hashtable tableNoms;

    String r="";

    //classeCourante=code(o.nom);

    /*
    String nom_reduit =code(o.nom);//classeCourante;
    String nom_paquet="";

    int i=nom_reduit.lastIndexOf('.');
    if(i>=0)
    {
      nom_paquet=nom_reduit.substring(0,i);
      nom_reduit =nom_reduit.substring(i+1,nom_reduit.length());
    }
    */

    // WIDTH=\"98%\" 
    en_tete+="<BR><CENTER><TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0>\n";
    en_tete+="<TR BGCOLOR=\"#000000\"><TD>\n";
    en_tete+="<TABLE WIDTH=\"100%\" BORDER=0 CELLSPACING=1 CELLPADDING=2>\n";
    en_tete+="<TR BGCOLOR=\"#FFFFFF\"><TD ALIGN=MIDDLE><SMALL>\n";
    if(o.normale) en_tete+="&lt;&lt;classe&gt;&gt;<BR>\n";
    else          en_tete+="&lt;&lt;interface&gt;&gt;<BR>\n";
    en_tete+=/*declaration(o.visibilite)+*/
      "<A HREF=\"#"+code(o.nom)+"\">"+
      o.nom.dernier()+"</A><BR>\n";
    en_tete+="</SMALL></TD></TR>\n";

    r+="<A NAME=\""+URLEncoder.encode(code(o.nom))+"\"></A>\n";
    r+="<H2>";
    if(o.normale) r+="Classe";
    else          r+="Interface";
    r+=" "+o.nom.dernier()+"</H2>\n";

    if(o.meres.noms.size()>0)
      r+="<B>D&eacute;rive de : </B>"+liste(o.meres)+"<BR>\n";
    if(o.interfaces.noms.size()>0)
      r+="<B>Implante : </B>"+liste(o.interfaces)+"<BR>\n";
    if(o.exceptions.noms.size()>0)
      r+="<B>Lance : </B>"+liste(o.exceptions)+"<BR>\n";
    r+="<B>Modifieurs : </B>"+code(o.visibilite)+"<BR>\n";
    r+="<BR>\n";

    r+=code(o.commentaires);

    en_tete+="<TR BGCOLOR=\"#FFFFFF\"><TD><SMALL>\n";

    if(o.attributs.size()>0)
    {
      r+="<UL>\n";

      int i=0;
      listeNoms=new String[o.attributs.size()];
      tableNoms=new Hashtable();
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
        AglAttribut aa=(AglAttribut)e.nextElement();
        tableNoms.put(aa.signature(),aa);
        listeNoms[i++]=aa.signature();
      }
      sort(listeNoms);

      for(i=0; i<listeNoms.length; i++)
      {
        AglAttribut aa=(AglAttribut)tableNoms.get(listeNoms[i]);

	en_tete+=declaration(aa.visibilite)+"<A HREF=\"#"+
	    URLEncoder.encode(aa.signature())+"\">";
	if(aa.visibilite.vstatic) en_tete+="<U>";
	en_tete+=aa.nom;
	if(aa.visibilite.vstatic) en_tete+="</U>";
	en_tete+="</A>"+declaration(aa)+"<BR>\n";

	r+="<A NAME=\""+URLEncoder.encode(aa.signature())+"\"></A>\n";
        r+="<LI>"+code(aa)+"</LI>\n";
      }   

      r+="</UL>\n";
    }
    else
      en_tete+="&nbsp; ";

    en_tete+="</SMALL></TD></TR>\n";
    en_tete+="<TR BGCOLOR=\"#FFFFFF\"><TD><SMALL>\n";

    if(o.constructeurs.size()>0)
    {
      r+="<H3>Constructeurs</H3>\n";
      r+="<UL>\n";

      int i=0;
      listeNoms=new String[o.constructeurs.size()];
      tableNoms=new Hashtable();
      for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
      {
        AglConstructeur ac=(AglConstructeur)e.nextElement();
        tableNoms.put(ac.signature(),ac);
        listeNoms[i]=ac.signature();
	i++;
      }
      sort(listeNoms);

      for(i=0; i<listeNoms.length; i++)
      {
        AglConstructeur ac=(AglConstructeur)tableNoms.get(listeNoms[i]);

	en_tete+=declaration(ac.visibilite)+"<A HREF=\"#"+
	    URLEncoder.encode(ac.signature())+"\">";
	if(ac.visibilite.vstatic) en_tete+="<U>";
	en_tete+=ac.nom.dernier();
	if(ac.visibilite.vstatic) en_tete+="</U>";
	en_tete+="</A>"+declaration(ac)+"<BR>\n";

	r+="<A NAME=\""+URLEncoder.encode(ac.signature())+"\"></A>\n";
        r+="<LI>"+code(ac)+"</LI>\n";
      }

      r+="</UL>\n";
    }

    if(o.methodes.size()>0)
    {
      r+="<H3>M&eacute;thodes</H3>\n";
      r+="<UL>\n";

      int i=0;
      listeNoms=new String[o.methodes.size()];
      tableNoms=new Hashtable();
      for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
      {
        AglMethode am=(AglMethode)e.nextElement();
        tableNoms.put(am.signature(),am);
        listeNoms[i++]=am.signature();
      }
      sort(listeNoms);
      for(i=0; i<listeNoms.length; i++)
      {
        AglMethode am=(AglMethode)tableNoms.get(listeNoms[i]);

	en_tete+=declaration(am.visibilite)+"<A HREF=\"#"+
	    URLEncoder.encode(am.signature())+"\">";
	if(am.visibilite.vstatic) en_tete+="<U>";
	en_tete+=am.nom;
	if(am.visibilite.vstatic) en_tete+="</U>";
	en_tete+="</A>"+declaration(am)+"<BR>\n";

	r+="<A NAME=\""+URLEncoder.encode(am.signature())+"\"></A>\n";
        r+="<LI>"+code(am)+"</LI>\n";
      }   

      r+="</UL>\n";
    }

    if((o.constructeurs.size()==0)&&(o.methodes.size()==0))
      en_tete+="&nbsp; ";

    en_tete+="</SMALL></TD></TR>";
    en_tete+="</TABLE>\n</TD></TR>\n</TABLE></CENTER>\n";
    en_tete+="<BR>\n";

    return r+"<BR>\n";
  }

  public String code(AglPaquet o)
  {
    String r="";
    if(o!=null) r=base_paquet(code(o.nom));
    return r;
  }

  public String code(AglImport o)
  {
    String r="";
    if("*".equals(o.nom.dernier()))
      r=base_paquet(code(o.nom.parent()));
    else
      r=base_classe(code(o.nom));
    return r;
  }

  public String code(AglConstructeur o)
  {
    String r="";
    r+="<FONT COLOR=\"#FF0000\"><CODE><B>"+code(o.nom)+
      "</B></CODE></FONT><BR>\n";
    //r+=prototype(o);
    r+="<B>Modifieurs : </B>"+code(o.visibilite)+"<BR>\n";

    r+="<B>Param&egrave;tres :</B>\n";
    if(o.parametres.size()==0)
    {
      r+=" aucun<BR>\n";
    }
    else
    {
      r+="<UL>\n";
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
        AglParametre ap=(AglParametre)e.nextElement();
        //r+="&nbsp;&nbsp;&nbsp;&nbsp;"+code(ap)+"<BR>\n";
        r+="<LI>"+code(ap);
	r+=commentaireParam(ap.nom,o.commentaires);
	r+="</LI>\n";
      }
      r+="</UL>\n";
    }

    r+=code(o.commentaires);
    return r;
  }

  public String code(AglMethode o)
  {
    String r="";
    r+="<FONT COLOR=\"#0000FF\"><CODE><B>"+o.nom+"</B></CODE></FONT><BR>\n";
    //r+=prototype(o);
    r+="<B>Modifieurs : </B>"+code(o.visibilite)+"<BR>\n";
    r+="<B>Retour : </B><CODE>"+code(o.type)+"</CODE><BR>\n";
    r+=commentaireReturn(o.commentaires);
    r+="<B>Param&egrave;tres :</B>\n";
    if(o.parametres.size()==0)
    {
      r+=" aucun<BR>\n";
    }
    else
    {
      r+="<UL>\n";
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
        AglParametre ap=(AglParametre)e.nextElement();
        //r+="&nbsp;&nbsp;&nbsp;&nbsp;"+code(ap)+"<BR>\n";
        r+="<LI>"+code(ap);
	r+=commentaireParam(ap.nom,o.commentaires);
	r+="</LI>\n";
      }
      r+="</UL>\n";
    }

    r+=code(o.commentaires);
    return r;
  }

  public String code(AglParametre o)
  {
    String r="";
    r+="<CODE>"+o.nom+"</CODE><B> : </B><CODE>"+code(o.type)+"</CODE>\n";
    // if(!o.defaut.equals("")) r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vabstract)     r+="abstrait ";
    if(o.vinline)       r+="en-ligne ";
    if(o.vpublic)       r+="public ";
    if(o.vprivate)      r+="priv&eacute; ";
    if(o.vprotected)    r+="prot&eacute;g&eacute; ";
    if(o.vstatic)       r+="statique ";
    if(o.vfinal)        r+="final ";
    if(o.vnative)       r+="natif ";
    if(o.vtransient)    r+="transitoire ";
    if(o.vsynchronized) r+="synchronis&eacute; ";
    if(o.voneway)       r+="asynchrone ";
    if(o.vreadonly)     r+="lecture ";
    if(o.vregister)     r+="registre ";
    if(o.vvirtual)      r+="virtuel ";
    //if(r.equals(""))    r+="paquet ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="new "+code(o.type)+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="new "+code(o.type)+"[";
    r+=code(o.nombre);
    r+="]";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }

  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    String fichier=o.fichier;
    langage=o.langage;
    unite_=o;

    int i;
    i=fichier.lastIndexOf('/');
    if(i>=0) fichier=fichier.substring(i+1,fichier.length());
    i=fichier.lastIndexOf('\\');
    if(i>=0) fichier=fichier.substring(i+1,fichier.length());
    //i=fichier.lastIndexOf('.');
    //if(i>=0) fichier=fichier.substring(0,i);

    en_tete ="<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n<HTML>\n";

    en_tete+=entete(o.fichier,"Guillaume Desnoix",
		    "guillaume@desnoix.com");

    en_tete+="<HEAD>\n";
    //en_tete+="  <META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=iso-8859-1\">\n";
    en_tete+="  <TITLE>"+fichier+"</TITLE>\n";
    en_tete+="  <LINK REL=\"StyleSheet\" HREF=\""+AglOptions.styleHtml+"\" TITLE=\"Style\">\n";
    en_tete+="</HEAD>\n";

    en_tete+="<BODY>\n";
    en_tete+="<H1>Unit&eacute; "+fichier+"</H1>\n";
    en_tete+="<UL>\n";

    en_tete+="<LI><B>Langage : </B>"+langage;
    if(o.paquet!=null)
      en_tete+=source(code(o.paquet.nom).replace('.','/')+"/"+fichier);
    else
      en_tete+=source(o.fichier);
    en_tete+="</LI>\n";

    if(o.paquet!=null)
      en_tete+="<LI><B>Paquet : </B>"+code(o.paquet)+"</LI>\n";

    en_tete+="</UL><UL>\n";

    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      en_tete+="<LI><B>Import : </B>"+code(ai)+"</LI>\n";
    }

    en_tete+="</UL>\n";
    //en_tete+="<H2>Index</H2>\n";

    if(AglOptions.classe.equals(""))
    {
      for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
      {
	AglType at=(AglType)e.nextElement();
	if(at instanceof AglClasse)
	{
	  AglClasse ac=(AglClasse)at;
	  if(!ac.elementaire)
	  {
	    attributs    +=ac.attributs.size();
	    constructeurs+=ac.constructeurs.size();
	    methodes     +=ac.methodes.size();
	  }
	}
      }
    }
    else
    {
      AglType at=AglType.cherche(AglOptions.classe);
      if(at instanceof AglClasse)
      {
	AglClasse ac=(AglClasse)at;
	if(!ac.elementaire)
	{
	  attributs    +=ac.attributs.size();
	  constructeurs+=ac.constructeurs.size();
	  methodes     +=ac.methodes.size();
	}
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
	  r+=code(ac);
      }
    }

    /*
    if(AglOptions.classe.equals(""))
    {
      for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
      {
	AglType at=(AglType)e.nextElement();
	if(at instanceof AglClasse)
	{
	  AglClasse ac=(AglClasse)at;
	  if(!ac.elementaire)
	  {
	    r+=code(ac);
	    r+="\n";
	  }
	}
      }
    }
    else
    {
      AglType at=AglType.cherche(AglOptions.classe);
      if(at instanceof AglClasse)
      {
	AglClasse ac=(AglClasse)at;
	if(!ac.elementaire)
	{
	  r+=code(ac);
	  r+="\n";
	}
      }
    }
    */

    r+="<BR>\n</BODY>\n</HTML>";
    unite_=null;

    return en_tete+r;
  }
  
  void sort(String[] tab)
  {
    Collator coll  =Collator.getInstance();
    boolean  encore=true;
    String   tmp;

    while(encore)
    {
      encore=false;
      for(int i=1; i<tab.length; i++)
      {
        if(coll.compare(tab[i-1],tab[i])>0)
	{
          tmp     =tab[i-1];
          tab[i-1]=tab[i];
          tab[i]  =tmp;
          encore  =true;
        }
      }
    }
  }
}
