/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeEiffel.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;

public class AglCodeEiffel extends AglCode
{
  boolean LIGNES=AglPreferences.AGL.getBooleanProperty("eiffel.code.lines",false);

  public String extension()
  {
    return "e";
  }

  public String[] extensions()
  {
    return new String[] { "e", "E" };
  }

  protected String debutCS  () { return indente()+"-- ";    }
  protected String finCS    () { return "";                 }
  protected String debutCM  () { return indente()+"------"; }
  protected String specialCM() { return "";                 }
  protected String milieuCM () { return indente()+"-- ";    }
  protected String finCM    () { return indente()+"------"; }

  public String code(AglCondition o)
  {
    String r=code(o.test);
    r=FuLib.replace(r,"!=","/="   );
    r=FuLib.replace(r,"==","="    );
    r=FuLib.replace(r,"!" ," not ");
    r=FuLib.replace(r,"&&"," and ");
    r=FuLib.replace(r,"||"," or " );
    r=FuLib.replace(r,"  "," "    );
    return r;
  }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=".";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
         if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau) r+="ARRAY["+code(((AglTypeTableau)o).type)+"]";

    else if(o==AglTypePrimitif.tchar   ) r+="CHARACTER";
    else if(o==AglTypePrimitif.tinteger) r+="INTEGER";
    else if(o==AglTypePrimitif.tfloat )  r+="REAL";
    else if(o==AglTypePrimitif.tdouble)  r+="DOUBLE";

    else r+=o.nom.dernier().toUpperCase();
    return r;
  }

  public String code(AglAttribut o)
  {
    indentation=1;
    String r=indente();

    r+=code(o.visibilite);
    r+=o.nom+": "+code(o.type);
    if(o.initiale!=null) r+=" is "+code(o.initiale);
    if(LIGNES) r+=" -- "+o.ligne;
    r+="\n";

    return r;
  }

  public String code(AglBloc o)
  {
    String r=indente()+"do\n";
    indentation++;

    int n=1;
    for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
    {
      AglInstruction ai=(AglInstruction)e.nextElement();
      r+=code(ai);
      n++;
    }

    indentation--;
    r+=indente()+"end\n";
    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;
    String r="";

    if(LIGNES)
    {
      r+="/* ";
      // if(o.paquet!="") r+=o.paquet;
      r+=code(o.nom)+" ["+o.ligne+"] */\n\n";
    }

    if(o.visibilite.vabstract) r+="deferred ";
    r+="class "+o.nom.dernier().toUpperCase();

    r+="\n{\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation=0;
    return r+"}\n";
  }

  public String code(AglClasse o)
  {
    indentation=0;
    String r="";

    if(LIGNES)
    {
      r+="-- ";
      // if(o.paquet!="") r+=o.paquet;
      r+=code(o.nom)+" ["+o.ligne+"]\n\n";
    }

    if(o.visibilite.vabstract) r+="deferred ";
    r+="class "+o.nom.dernier().toUpperCase();

    if((o.meres.noms.size()>0)&&(o.interfaces.noms.size()>0))
      r+=" inherit "+code(o.meres).toUpperCase()+","+code(o.interfaces).toUpperCase();
    else
      if(o.meres.noms.size()>0)
	r+=" inherit "+code(o.meres).toUpperCase();
    else
      if(o.interfaces.noms.size()>0)
	r+=" inherit "+code(o.interfaces).toUpperCase();

    /*
    if(o.exceptions.noms.size()>0)
      r+=" throws "+code(o.exceptions);
    */

    r+="\n{\n";

    if(o.constructeurs.size()>0)
    {
      r+=indente()+"creation\n";
      for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
      {
	AglConstructeur am=(AglConstructeur)e.nextElement();
	r+=indente()+"make\n"; // code(am);
      }
      r+="\n";
    }

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    r+="\n";
    if(o.clinit.instructions.size()>0) { r+=code(o.clinit); r+="\n"; }
    if(o.obinit.instructions.size()>0) { r+=code(o.obinit); r+="\n"; }

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    indentation=0;
    return r+"}\n";
  }

  public String code(AglConstructeur o)
  {
    indentation=1;
    String r=indente();

    r+=code(o.visibilite)+"\n";
    indentation++;
    r+=indente()+"make"; // o.nom.dernier()

    if(o.parametres.size()>0)
    {
      r+="(";
      boolean p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)e.nextElement();
	if(!p) r+=", ";
	r+=code(ap);
	p=false;
      }
      r+=")";
    }

    r+=" is";

    if(  (o.visibilite.vabstract||o.visibilite.vnative)
       && o.corps.instructions.isEmpty()) r+="deferred end";
    if(LIGNES) r+=" -- "+o.ligne;
    r+="\n";
    if(!o.corps.instructions.isEmpty()) r+=code(o.corps);

    r+="\n";
    indentation--;
    return r;
  }

  public String code(AglImport o)
  {
    indentation=0;
    String r="import "+code(o.nom)+";\n";
    return r;
  }

  public String code(AglMethode o)
  {
    indentation=1;
    String r=indente();
    r+=code(o.visibilite)+"\n";
    indentation++;
    r+=indente()+o.nom;

    if(o.parametres.size()>0)
    {
      r+="(";
      boolean p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)e.nextElement();
	if(!p) r+=", ";
	r+=code(ap);
	p=false;
      }
      r+=")";
    }

    if(o.type!=AglTypePrimitif.tvoid)
      r+=": "+code(o.type);

    r+=" is\n";

    if(  (o.visibilite.vabstract||o.visibilite.vnative)
       && o.corps.instructions.isEmpty())
    {
      indentation++;
      r+=indente()+"deferred\n";
      indentation--;
    }

    if(o.preconditions.size()>0)
    {
      r+=indente()+"require\n";
      indentation++;
      for(Enumeration e=o.preconditions.elements(); e.hasMoreElements(); )
      {
	AglCondition c=(AglCondition)e.nextElement();
	r+=indente()+code(c)+"\n";
      }
      indentation--;
    }

    if(o.postconditions.size()>0)
    {
      r+=indente()+"ensure\n";
      indentation++;
      for(Enumeration e=o.postconditions.elements(); e.hasMoreElements(); )
      {
	AglCondition c=(AglCondition)e.nextElement();
	r+=indente()+code(c)+"\n";
      }
      indentation--;
    }

    if(LIGNES) r+=" -- "+o.ligne+"\n";
    if(!o.corps.instructions.isEmpty()) r+=code(o.corps);
    else                                r+=indente()+"end";

    r+="\n";
    indentation--;
    return r;
  }

  public String code(AglParametre o)
  {
    String r="";
    r+=o.nom+": "+code(o.type);
    // if(o.defaut!="") r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="feature ";

    if(o.vpublic)       r+="{ANY} ";
    else                r+="{NONE} ";

    // if(o.vprivate)      r+="{NONE} ";
    // if(o.vprotected)    r+="{NONE} ";

    /*
    if(o.vinline)       r+="inline ";
    if(o.vabstract)     r+="abtract ";
    if(o.vstatic)       r+="static ";
    if(o.vfinal)        r+="final ";
    if(o.vnative)       r+="native ";
    if(o.vtransient)    r+="transient ";
    if(o.vsynchronized) r+="synchronized ";
    */

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="(";

    if(o.droite instanceof AglExpressionNew)
    {
      r+="!!"+code(o.gauche);
      String p=code(o.droite);
      int    i=p.indexOf('(');
      if(i>=0)
      {
	p=p.substring(i);
	if(!p.equals("()")) r+=p;
      }
    }
    else
      r+=code(o.gauche)+" := "+code(o.droite);

    r+=")";
    if(LIGNES) r+=" -- Affectation "+o.ligne;
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    if(LIGNES) r+=" -- Attribut "+o.ligne;
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String p=o.operateur;
    if(p.equals("==")) p="=";
    if(p.equals("!=")) p="/=";
    if(p.equals("!" )) p="not";
    if(p.equals("&&")) p="and";
    if(p.equals("||")) p="or";

    String r=code(o.gauche)+p+code(o.droite);
    if(LIGNES) r+=" -- Binaire "+o.ligne;
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r="local "+o.nom+": "+code(o.type);
    if(o.initiale!=null) r+=" is "+code(o.initiale);
    if(LIGNES) r+=" -- Var "+o.ligne;
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    if(LIGNES) r+=" -- EstUn "+o.ligne;
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom;

    if(o.arguments.size()>0)
    {
      r+="(";
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
	AglExpression ae=(AglExpression)e.nextElement();
	if(!p) r+=",";
	r+=code(ae);
	p=false;
      }
      r+=")";
    }

    if(LIGNES) r+=" -- Fonction "+o.ligne;
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    if(LIGNES) r+=" -- Indice "+o.ligne;
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="<< ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" >>";
    if(LIGNES) r+=" -- InitArray "+o.ligne;
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    if(LIGNES) r+=" -- Methode "+o.ligne;
    r+=code(o.fonction);
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";
    if(LIGNES) r+=" -- Multiple "+o.ligne;

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="!!"+code(o.type)+".make";

    if(o.arguments.size()>0)
    {
      r+="(";
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
	AglExpression ae=(AglExpression)e.nextElement();
	if(!p) r+=",";
	r+=code(ae);
	p=false;
      }
      r+=")";
    }

    if(LIGNES) r+=" -- New "+o.ligne;
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="!!";

    if(o.initiale==null)
    {
      r+=code(o.type)+"[";
      if(o.nombre!=null) r+=code(o.nombre);
      r+="]";
      if(LIGNES) r+=" -- New "+o.ligne;
    }
    else
    {
      r+=code(o.initiale);
      if(LIGNES) r+=" -- New "+o.ligne;
    }

    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="("+code(o.expression)+")";
    if(LIGNES) r+=" -- Parenthesee "+o.ligne;
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)+" "
      +o.operateur1+" "+code(o.milieu)+" "
      +o.operateur2+" "+code(o.droite);
    if(LIGNES) r+=" -- Ternaire "+o.ligne;
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    if(LIGNES) r+=" -- Typage "+o.ligne;
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    if(LIGNES) r+=" -- UnaireDroite "+o.ligne;
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    if(LIGNES) r+=" -- UnaireGauche "+o.ligne;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=indente();
    r+=o.body;
    if((r.length()>0)&&(r.charAt(r.length()-1)!='\n')) r+="\n";
    r=r.substring(0,r.length()-1);
    if(LIGNES) r+=" -- Instruction "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=indente()+";\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=indente();

    if(o.droite instanceof AglExpressionNew)
    {
      r+="!!"+code(o.gauche);
      String p=code(o.droite);
      p=p.substring(p.indexOf('('));
      r+=p;
    }
    else
      r+=code(o.gauche)+" := "+code(o.droite);

    if(LIGNES) r+=" -- Affectation "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r="";
    if(LIGNES) r+=indente()+"/* Bloc "+o.ligne+" */\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=indente()+"break";
    if(o.nom!=null) r+=" "+o.nom;
    if(LIGNES) r+=" -- Break "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=indente()+"catch("+code(o.signal)+")";
    if(LIGNES) r+=" -- Catch "+o.ligne;
    r+="\n";
    r+=code(o.reprise);
    return r;
  }

  public String code(AglInstructionCondition o)
  {
    /*
    String r=indente()+code(o.condition.test);
    if(LIGNES) r+=" -- Condition "+o.ligne;
    r+="\n";
    return r;
    */

    return "";
  }

  public String code(AglInstructionContinue o)
  {
    String r=indente()+"continue";
    if(o.nom!=null) r+=" "+o.nom;
    if(LIGNES) r+=" -- Continue "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=indente()+"local "+o.nom+": "+code(o.type);
    if(o.initiale!=null) r+=" is "+code(o.initiale);
    if(LIGNES) r+=" -- DefVar "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r=indente()+"do";
    if(LIGNES) r+=" -- DoWhile "+o.ligne;
    r+="\n";
    r+=code(o.corps);
    r+=indente()+"while "+code(o.condition);
    if(LIGNES) r+=" -- DoWhile "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=code(o.expression);
    if(r.startsWith("(")&&(r.endsWith(")"))) r=r.substring(1,r.length()-1);
    r=indente()+r;
    if(LIGNES) r+=" -- InsExpr "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=indente()+"finally";
    if(LIGNES) r+=" -- Finally "+o.ligne;
    r+="\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r="";

    String d=code(o.debut).trim();
    if(d.endsWith(";")) d=d.substring(0,d.length()-1);

    r+=indente()+"from "+d+" until "+code(o.test).trim();
    if(LIGNES) r+=" -- For "+o.ligne;
    r+="\n";
    r+=indente()+"loop\n";

    indentation++;
    if(o.corps!=null)
      for(Enumeration e=o.corps.instructions.elements(); e.hasMoreElements(); )
	r+=code((AglInstruction)e.nextElement());
    /*
    if(o.fin!=null)
      for(Enumeration e=o.fin.instructions.elements(); e.hasMoreElements(); )
	r+=code((AglInstruction)e.nextElement());
    */
    r+=code(o.fin);
    indentation--;

    r+=indente()+"end\n";
    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=indente()+"goto "+o.nom;
    if(LIGNES) r+=" -- Goto "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=indente()+"if "+code(o.condition)+" then";
    if(LIGNES) r+=" -- If "+o.ligne;
    r+="\n";
    indentation++;
    r+=code(o.sioui);
    indentation--;
    if(o.sinon!=null)
    {
      r+=indente()+"else";
      if(LIGNES) r+=" -- Else "+o.ligne;
      r+="\n";
      indentation++;
      r+=code(o.sinon);
      indentation--;
    }

    r+=indente()+"end\n";
    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=" "+o.nom+":";
    if(LIGNES) r+=" -- Label "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=indente()+o.nom;

    if(o.arguments.size()>0)
    {
      r+="(";
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
	AglExpression ae=(AglExpression)e.nextElement();
	if(!p) r+=", ";
	r+=code(ae);
	p=false;
      }
      r+=")";
    }

    if(LIGNES) r+=" -- Procedure "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=indente()+"Result := "+code(o.valeur);
    if(LIGNES) r+=" -- Return "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=indente()+"inspect "+code(o.condition);
    if(LIGNES) r+=" -- Switch "+o.ligne;
    r+="\n";
    r+=indente()+"{\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"when "+code(ce)+" then\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      r+=indente()+"\n";
    }

    r+=indente()+"else\n";
    r+=code(o.defaut);

    indentation--;
    r+=indente()+"end\n";

    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=indente()+"synchronized "+code(o.objet);
    if(LIGNES) r+=" -- Synchronized "+o.ligne;
    r+="\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=indente()+"throw "+code(o.valeur);
    if(LIGNES) r+=" -- Throw "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=indente()+"try";
    if(LIGNES) r+=" -- Try "+o.ligne;
    r+="\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=indente()+"while "+code(o.condition);
    if(LIGNES) r+=" -- While "+o.ligne;
    r+="\n";
    r+=code(o.corps);
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
}
