/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeFormel.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;
import java.io.*;
import java.util.*;

public class AglCodeFormel extends AglCodeJava
{
  private int       idbase_=0;
  private int       niveau_=0;
  private Hashtable table_ =new Hashtable();
  private String    loe_   ="";

  private boolean   formel_=true;
  private boolean   projet_=false;

  public String extension()
  {
    return "txt";
  }

  public String[] extensions()
  {
    return new String[] { "txt", "TXT" };
  }

  protected String indente  () { return ""; }
  protected String debutCS  () { return ""; }
  protected String finCS    () { return ""; }
  protected String debutCM  () { return ""; }
  protected String specialCM() { return ""; }
  protected String milieuCM () { return ""; }
  protected String finCM    () { return ""; }

  private String getId(String _s)
  {
    String r="";

    if(formel_)
    {
      r=(String)table_.get(_s);

      if(r==null)
      {
	idbase_++;
        r="@"+idbase_;
        table_.put(_s,r);
      }
    }
    else r=_s;

    return r;
  }

  public String code(AglCommentaireSimple o)
  {
    return "";
  }

  public String code(AglCommentaireMulti o)
  {
    return "";
  }

  public String code(AglBloc o, boolean _forced, boolean _ignored)
  {
    niveau_++;

    String r="";
    
    if(_forced||(o.instructions.size()!=1))
    {
      if(!_ignored) r+="{\n";

      for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
      {
	if(niveau_<=1)
	{
	  table_.clear();
	  idbase_=0;
	}

	AglInstruction ai=(AglInstruction)e.nextElement();
	String s=code(ai);
	if((niveau_<=1)&&(s.indexOf('@')>=0))
	{
	  formel_=false;
	  s+=code(ai);
	  formel_=true;
	}
	r+=s;
      }

      if(!_ignored) r+="}\n";
    }
    else
    {
      if(niveau_<=1)
      {
	table_.clear();
	idbase_=0;
      }

      AglInstruction ai=(AglInstruction)o.instructions.elementAt(0);

      String s=code(ai);
      if((niveau_<=1)&&(s.indexOf('@')>=0))
      {
	formel_=false;
	s+=code(ai);
	formel_=true;
      }
      r+=s;
    }

    niveau_--;
    return r;
  }

  public String code(AglStruct o)
  {
    String r="";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    return r;
  }

  public String code(AglClasse o)
  {
    String r="";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    if(o.clinit.instructions.size()>0)
      r+=code(o.clinit,false,true);

    if(o.obinit.instructions.size()>0)
      r+=code(o.obinit,false,true);

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    return r;
  }

  public String code(AglAttribut o)
  {
    niveau_=0;
    String r="";
    r+=code(o.visibilite);
    r+=code(o.type)+" "+getId(o.nom);
    if(o.initiale!=null) r+="="+code(o.initiale);
    r+=";\n";
    return r;
  }

  public String code(AglConstructeur o)
  {
    niveau_=0;
    return code(o.corps,false,true);
  }

  public String code(AglMethode o)
  {
    niveau_=0;
    return code(o.corps,false,true);
  }

  public String code(AglVisibilite o)
  {
    String r="";
    if(o.vabstract)     r+="abstract ";
    if(o.vinline)       r+="inline ";
    if(o.vpublic)       r+="public ";
    if(o.vprivate)      r+="private ";
    if(o.vprotected)    r+="protected ";
    if(o.vstatic)       r+="static ";
    if(o.vfinal)        r+="final ";
    if(o.vnative)       r+="native ";
    if(o.vtransient)    r+="transient ";
    if(o.vsynchronized) r+="synchronized ";
    if(o.voneway)       r+="oneway ";
    if(o.vreadonly)     r+="readonly ";
    if(o.vregister)     r+="register ";
    if(o.vvirtual)      r+="virtual ";

    return r;
  }

// Expressions

  public String code(AglExpression o)
  {
    String r=super.code(o);

    if(projet_&&!(o instanceof AglExpressionGenerique))
    {
      String  s=r.replace('\n',' ').trim();
      if(s.startsWith("("))
      {
	boolean p=true;
        int     n=0;
	for(int i=0;i<s.length()-1;i++)
        {
	  if(s.charAt(i)=='(') n++;
	  if(s.charAt(i)==')') n--;
	  if(n==0) p=false;
        }
	if(p)
	  while(s.startsWith("(")&&s.endsWith(")"))
	    s=s.substring(1,s.length()-1);
      }
      if(s.length()>2) loe_+=s+"\n";
    }

    return r;
  }

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;

    if(r.length()>0)
      if(Character.isJavaIdentifierStart(r.charAt(0))
	 &&(AglType.cherche(r)==null))
	  r=getId(r);

    return r;
  }

// Instructions

  public String code(AglInstruction o)
  {
    String r=super.code(o);
    r=r.replace('\n',' ').trim()+"\n";
    return r;
  }

  public String code(AglInstructionGenerique o)
  {
    return o.body;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=code(o.type)+" "+getId(o.nom);
    if(o.initiale!=null) r+="="+code(o.initiale);
    r+=";";
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglStruct))
      {
        AglStruct ac=(AglStruct)at;
        if(!ac.elementaire&&filtre(ac.nom))
	  r+=code(ac);
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
	  r+=code(ac);
      }
    }

    if(!projet_)
      r=entete(o.fichier)+sort(r);

    return r;
  }

  public AglSortie[] code(AglProjet o)
  {
    projet_=true;
    String r=entete(null);
    loe_="";

    String t="";
    AglSortie[] s=super.code(o);
    for(int i=0;i<s.length;i++)
      if(s[0].donnee instanceof String) t+=(String)s[0].donnee;

    r+=sort(t);
    r+="\n";
    r+=sort(loe_);
    projet_=false;

    return new AglSortie[]
    { new AglSortie("text/plain",null,null,r) };
  }

  protected String rappelleClasses  () { return ""; }
  protected String rappellePatrons  () { return ""; }
  protected String rappelleRelations() { return ""; }

  private String sort(String _s)
  {
    StringTokenizer st=new StringTokenizer(_s,"\n");
    Hashtable       tb=new Hashtable();
    while(st.hasMoreTokens())
    {
      String  k=st.nextToken();
      Integer v=(Integer)tb.get(k);
      if(v==null) v=new Integer(1);
      else        v=new Integer(v.intValue()+1);
      tb.put(k,v);
    }

    int         i =0;
    int         l =tb.size();
    String[]    vr=new String[l];
    Enumeration ek=tb.keys();
    while(ek.hasMoreElements())
    {
      String  k=(String)ek.nextElement();
      int     v=((Integer)tb.get(k)).intValue();

      if(v>1)
      {
        String  s="          "+v;
        s=s.substring(s.length()-10);
        vr[i]=s+" "+k+"\n";
        i++;
      }
    }
    tb=null;
    FuSort.sort(vr,0,i);

    String r="";
    for(int j=i-1;j>=0;j--) r+=vr[j];
    return r;
  }
}
