/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeGenerique.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.io.*;
import java.text.*;
import java.util.*;

public class AglCodeGenerique extends AglCode
{
  private Hashtable formats=new Hashtable(31);

  public AglCodeGenerique()
  {
    chargePhrases();
    getExpReg();
  }

  public final String[][] getExpReg()
  {
    if(expreg_==null)
    {
      String f=AglPreferences.AGL.getStringProperty("generique.code.format");
      if("".equals(f))
      {
        expreg_=new String[0][0];
      }
      else
      {
        expreg_=chargeExpReg(f+"ExpReg.txt");
      }
    }

    return expreg_;
  }

  private void chargePhrases()
  {
    String f=AglPreferences.AGL.getStringProperty("generique.code.format");
    if("".equals(f)) return;

    f+="Generique.txt";

    suivi("Chargement des phrases: "+f);

    try
    {
      int n=0;

      LineNumberReader rin=
	new LineNumberReader
	(new InputStreamReader
	 (new BufferedInputStream
	  (getClass().getResourceAsStream(f))));

      String l,k,v;

      k=null;
      v=null;

      while(rin.ready())
      {
	l=rin.readLine();
	if(l==null) break;

	n++;
	if(l.equals("")&&(k!=null)&&(v!=null))
	{
	  //System.err.println("$$$$$$ "+k+"\n"+v);
	  formats.put(k,v);
	  k=null;
	  v=null;
	}
	else if(l.equals("")&&(k==null)&&(v==null)) ;
	else if(l.startsWith("#")) ;
	else if(k==null)
	{
	  if(!l.endsWith(":"))
	    erreur("Mauvais format de clef: "+l+" ["+f+":"+n+"]");
	  k=l.substring(0,l.length()-1);
	}
	else if(v==null)
	{
	  if(!l.startsWith("\"")||!l.startsWith("\""))
	    erreur("Mauvais format de valeur: "+l+" "+k+" ["+f+":"+n+"]");
	  v=l.substring(1,l.length()-1);
 	  v=FuLib.replace(v,"\\n","\n");
 	  v=FuLib.replace(v,"\\t","\t");
	  try { new MessageFormat(v); }
	  catch(IllegalArgumentException ex)
	  {
	    erreur("Format de message non reconnu: "+k+" ["+f+":"+n+"]");
	  }
	}
	else // v+="\n"+l;
	  erreur("Mauvais format de fichier: "+f+" ["+n+"]");
      }

      if((k!=null)&&(v!=null))
      {
	//System.err.println(k+"\n"+v);
	formats.put(k,v);
      }

    }
    catch(Exception ex)
      { erreur("Resource ou fichier introuvable: "+f); }
  }

  private String get(String k)
  {
    return (String)formats.get(k);
  }

  private String get(String k, String d)
  {
    String r=(String)formats.get(k);
    if(r==null) r=d;
    return r;
  }

  private String normalise(String s)
  {
    int i=s.lastIndexOf('.');
    if(i>=0) s=s.substring(i+1);

    if(s.startsWith("Agl")) s=s.substring(3);

    String r="";
    for(i=0; i<s.length(); i++)
    {
      char c=s.charAt(i);
      if(Character.isUpperCase(c))
      {
	if(i>0) r+="-";
	c=Character.toLowerCase(c);
      }
      r+=c;
    }

    return r;
  }

  protected String format(AglObjet _o)
  {
    return format(_o,null);
  }

  protected String format(AglObjet _o, String _d)
  {
    String r=null;
    if(_o!=null) r=format(_o.getClass());
    if(r ==null) r=_d;
    return r;
  }

  protected String format(Class c)
  {
    String n=c.getName();
    String r=get(n);
    if(r!=null) return r;

    n=normalise(n);
    r=get(n);
    if(r!=null) return r;

    c=c.getSuperclass();
    if(c==null) return null;

    return format(c);
  }

  protected String formate(String _f, Object[] _s)
  {
    if(_f==null)
    {
      _f="";
      int l=_s.length;
      for(int i=0;i<l;i++) _f+="{"+i+"} ";
      if(l>1) _f+="\n";
    }
    else
    {
      _f=FuLib.replace(_f,"\t",indente());
    }

    return MessageFormat.format(_f,_s);
  }

  public String extension()
  {
    return "txt";
  }

  public String[] extensions()
  {
    return new String[] { "txt", "TXT" };
  }

  protected String debutCS  () { return formate(get("debut-commentaire-simple" ,"\t// "),new String[0]); }
  protected String finCS    () { return formate(get("fin-commentaire-simple"   ,""     ),new String[0]); }
  protected String debutCM  () { return formate(get("debut-commentaire-multi"  ,"\t/*" ),new String[0]); }
  protected String specialCM() { return formate(get("special-commentaire-multi","*"    ),new String[0]); }
  protected String milieuCM () { return formate(get("milieu-commentaire-multi" ,"\t * "),new String[0]); }
  protected String finCM    () { return formate(get("fin-commentaire-multi"    ,"\t */"),new String[0]); }

  protected String remplaceParEt(String _s)
  {
    String n=get("derniere-virgule");
    if(n==null) return _s;

    String r;
    int i=_s.lastIndexOf(",");
    if(i>=0) r=_s.substring(0,i)+n+_s.substring(i+1);
    else     r=_s;
    return r;
  }

  public String code(AglNom o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=get("separateur-nom",".");
      r+=(String)e.nextElement();
    }

    return formate(format(o,"{0}"),new String[] { r,o.dernier() } );
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=get("separateur-liste",", ");
      r+=code((AglNom)e.nextElement());
    }

    return formate(format(o,"{0}"),new String[] { r } );
  }

  public String code(AglType o)
  {
    o=AglTypeAlias.simplifie(o);

    String r="";
         if(o==AglTypePrimitif.tvoid   ) r+=get("type-void"   ,"void"    );
    else if(o==AglTypePrimitif.tboolean) r+=get("type-boolean","boolean" );
    else if(o==AglTypePrimitif.tbyte   ) r+=get("type-byte"   ,"byte"    );
    else if(o==AglTypePrimitif.tchar   ) r+=get("type-char"   ,"char"    );
    else if(o==AglTypePrimitif.tshort  ) r+=get("type-short"  ,"short"   );
    else if(o==AglTypePrimitif.tinteger) r+=get("type-int"    ,"int"     );
    else if(o==AglTypePrimitif.tlong   ) r+=get("type-long"   ,"long"    );
    else if(o==AglTypePrimitif.tfloat  ) r+=get("type-float"  ,"float"   );
    else if(o==AglTypePrimitif.tdouble ) r+=get("type-double" ,"double"  );
    else if(o==AglType.cherche("String")) r+=get("type-string",code(o.nom));
    else
    if(o instanceof AglTypeTableau)
    {
      String dim="";
      if(((AglTypeTableau)o).dimension!=null)
	dim=code(((AglTypeTableau)o).dimension);

      r+=formate(format(o,"{0}[{1}]"), new String[]
	  { code(((AglTypeTableau)o).type), dim }); 
    }
    else
      r+=code(o.nom);

    return formate(get("type","{0}"),new String[] { r } );
  }

  public String code(AglAttribut o)
  {
    String vi=code(o.initiale);
    vi=formate(get("valeur-initiale","{1,choice,0#|1#={0}}"),
	       new Object[] { vi, new Integer((o.initiale==null) ? 0 : 1) });

    return formate(format(o,"\t{0}{1} {2}{3}; // {4}\n"),new String[]
	{ code(o.visibilite), code(o.type), o.nom,
	  vi, (""+o.ligne) } );
  }

  public String code(AglBloc o)
  {
    String r="";

    indentation++;
    if((o!=null)&&!o.instructions.isEmpty())
      for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
      {
        AglInstruction ai=(AglInstruction)e.nextElement();
        r+=code(ai);
      }
    indentation--;

    return formate(format(o,"\t'{'\n{0}\t'}'\n"), new String[] { r });
  }

  public String code(AglStruct o)
  {
    indentation++;

    String la="";
    int    na=o.attributs.size();
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      la+=code(aa);
    }
    la=formate(get("liste-attributs","{1,choice,0#|1#{0}\n}"),
	       new Object[] { la, new Integer(na) });

    indentation--;

    return formate(format(o,"\t{0}class {1}\n\t'{'{2}\t'}'"),new String[]
	{ code(o.visibilite), code(o.nom), la, (""+o.ligne) } );
  }

  public String code(AglClasse o)
  {
    String ls=code(o.meres);
    int    ns=o.meres.noms.size();
    ls=remplaceParEt(ls);
    ls=formate(get("liste-meres","{1,choice,0#|1#\textends {0}\n}"),
	       new Object[] { ls, new Integer(ns) });

    String li=code(o.interfaces);
    int    ni=o.interfaces.noms.size();
    li=remplaceParEt(li);
    li=formate(get("liste-interfaces","{1,choice,0#|1#\timplements {0}\n}"),
	       new Object[] { li, new Integer(ni) });

    String le=code(o.exceptions);
    int    ne=o.exceptions.noms.size();
    le=remplaceParEt(le);
    le=formate(get("liste-exceptions","{1,choice,0#|1#\tthrows {0}\n}"),
	       new Object[] { le, new Integer(ne) });

    indentation++;

    String la="";
    int    na=o.attributs.size();
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      la+=code(aa);
    }
    la=formate(get("liste-attributs","{1,choice,0#|1#{0}\n}"),
	       new Object[] { la, new Integer(na) });

    String lc="";
    int    nc=o.constructeurs.size();
    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur ac=(AglConstructeur)e.nextElement();
      lc+=code(ac);
    }
    lc=formate(get("liste-constructeurs","{1,choice,0#|1#{0}\n}"),
	       new Object[] { lc, new Integer(nc) });

    String lm="";
    int    nm=o.methodes.size();
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      lm+=code(am);
    }
    lm=formate(get("liste-methodes","{1,choice,0#|1#{0}\n}"),
	       new Object[] { lm, new Integer(nm) });

    indentation--;

    return formate
      (format(o,"\t{0}class {1}\n"
	      +"{2}{3}{4}"
	      +"\t'{'\n{5}{6}{7}\t'}'"),
       new String[]
	  { code(o.visibilite), code(o.nom), ls, li, le,
	    la, lc, lm, (""+o.ligne) } );
  }

  public String code(AglConstructeur o)
  {
    String lp="";
    int    np=o.parametres.size();
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(p) p=false; else lp+=get("separateur-parametre",", ");
      lp+=code(ap);
    }
    lp=remplaceParEt(lp);
    lp=formate(get("liste-parametres","{1,choice,0#()|1#({0})}"),
	       new Object[] { lp, new Integer(np) });

    String le=code(o.exceptions);
    int    ne=o.exceptions.noms.size();
    le=formate(get("liste-exceptions","{1,choice,0#|1#\n\tthrows {0}\n}"),
	       new Object[] { le, new Integer(ne) });

    String sc="";
    if(!o.corps.instructions.isEmpty())
	sc=code(o.corps);

    return formate
	(format(o,"\t{0}{1}{2}{3} // {5}\n{4}"),
	 new String[]
	    { code(o.visibilite), code(o.nom), lp, le, sc, (""+o.ligne) } );
  }

  public String code(AglMethode o)
  {
    String lp="";
    int    np=o.parametres.size();
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(p) p=false; else lp+=get("separateur-parametre",", ");
      lp+=code(ap);
    }
    lp=remplaceParEt(lp);
    lp=formate(get("liste-parametres","{1,choice,0#()|1#({0})}"),
	       new Object[] { lp, new Integer(np) });

    String le=code(o.exceptions);
    int    ne=o.exceptions.noms.size();
    le=formate(get("liste-exceptions","{1,choice,0#|1#\n\tthrows {0}\n}"),
	       new Object[] { le, new Integer(ne) });

    String sc="";
    if(!o.corps.instructions.isEmpty())
	sc=code(o.corps);

    return formate
	(format(o,"\t{0}{1} {2}{3}{4} // {6}\n{5}"),
	 new String[]
	    { code(o.visibilite), code(o.type), code(o.nom),
	      lp, le, sc, (""+o.ligne) } );
  }

  public String code(AglParametre o)
  {
    String vi=code(o.defaut);
    vi=formate(get("valeur-initiale","{1,choice,0#|1#={0}}"),
	       new Object[] { vi, new Integer((o.defaut==null) ? 0 : 1) });

    return formate(format(o,"{0}{1} {2}{3}"),new String[]
	{ code(o.visibilite), code(o.type), o.nom, vi, (""+o.ligne) } );
  }

  public String code(AglPaquet o)
  {
    return formate(format(o,"\tpackage {0};\n"), new String[]
	{ code(o.nom), (""+o.ligne) });
  }

  public String code(AglImport o)
  {
    return formate(format(o,"\timport {0};\n"),new String[]
	{ code(o.nom), (""+o.ligne) });
  }

  public String code(AglVisibilite o)
  {
    String r="";
    String s=get("separateur-visibilite"," ");
    if(o.vabstract)     r+=get("visibilite-abstract"    ,"abstract"    )+s;
    if(o.vpublic)       r+=get("visibilite-public"      ,"public"      )+s;
    if(o.vprivate)      r+=get("visibilite-private"     ,"private"     )+s;
    if(o.vprotected)    r+=get("visibilite-protected"   ,"protected"   )+s;
    if(o.vinline)       r+=get("visibilite-inline"      ,"inline"      )+s;
    if(o.vstatic)       r+=get("visibilite-static"      ,"static"      )+s;
    if(o.vfinal)        r+=get("visibilite-final"       ,"final"       )+s;
    if(o.vnative)       r+=get("visibilite-native"      ,"native"      )+s;
    if(o.vtransient)    r+=get("visibilite-transient"   ,"transient"   )+s;
    if(o.vsynchronized) r+=get("visibilite-synchronized","synchronized")+s;
    
    if(r.endsWith(s)) r=r.substring(0,r.length()-s.length());
    r=remplaceParEt(r);
    //if(r.equals("")) r="normal ";
    //if((r.length()>0)&&(r.charAt(r.length()-1)==' ')) r=r.substring(0,r.length()-1);
    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    return formate(format(o,"{0}"), new String[] { o.body });
  }

  public String code(AglExpressionAffectation o)
  {
    return formate(format(o,"{0}={1}"), new String[] { code(o.gauche), code(o.droite) });
  }

  public String code(AglExpressionAttribut o)
  {
    String so="";
    if(o.objet!=null) so=code(o.objet);
    return formate(format(o,"{0}.{1}"),new String[] { so, o.attribut });
  }

  public String code(AglExpressionBinaire o)
  {
    String t=get("b"+o.operateur,"{0}"+o.operateur+"{1}");

    indentation++;
    String sg=code(o.gauche);
    String sd=code(o.droite);
    indentation--;

    return formate(t,new String[] { sg,sd });
  }

  public String code(AglExpressionDefVar o)
  {
    String vi=code(o.initiale);
    vi=formate(get("valeur-initiale","{1,choice,0#|1#={0}}"),
	       new Object[] { vi, new Integer((o.initiale==null) ? 0 : 1) });

    return formate(format(o,"{0} {1}{2}"), new String[]
		   { code(o.type), o.nom, vi } );
  }

  public String code(AglExpressionEstUn o)
  {
    return formate(format(o,"{0} instanceof {1}"), new String[]
	{ code(o.expression), code(o.type) } );
  }

  public String code(AglExpressionFonction o)
  {
    String la="";
    indentation++;
    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(p) p=false; else la+=get("separateur-argument",", ");
      la+=code(ae);
    }
    la=remplaceParEt(la);
    indentation--;

    return formate(format(o,"{0}({1})"),new String[] { o.nom,la });
  }

  public String code(AglExpressionIndice o)
  {
    return formate(format(o,"{0}[{1}]"), new String[]
	{ code(o.objet), code(o.indice) });
  }

  public String code(AglExpressionInitArray o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=get("separateur-valeur",", ");
      r+=code(ae);
      p=false;
    }
    r=remplaceParEt(r);

    return formate(format(o,"'{'{0}'}'"), new String[] { r });
  }

  public String code(AglExpressionMethode o)
  {
    String la="";
    indentation++;
    boolean p=true;
    for(Enumeration e=o.fonction.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(p) p=false; else la+=get("separateur-argument",", ");
      la+=code(ae);
    }
    la=remplaceParEt(la);
    indentation--;

    return formate(format(o,"{0}.{1}({2})"),
		   new String[] { code(o.objet), o.fonction.nom, la });
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=get("separateur-valeur",", ");
      r+=code(ae);
      p=false;
    }

    return formate(format(o,"{0}"),new String[] { r });
  }

  public String code(AglExpressionNew o)
  {
    String la="";
    if(o.arguments.size()>0)
    {
      indentation++;
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
        AglExpression ae=(AglExpression)e.nextElement();
        if(p) p=false; else la+=get("separateur-argument",", ");
	la+=code(ae);
      }
      la=remplaceParEt(la);
      indentation--;
    }

    return formate(format(o,"new {0}({1})"), new String[]
		   { code(o.type), la });
  }

  public String code(AglExpressionNewArray o)
  {
    String sn="";
    if(o.nombre!=null) sn+=code(o.nombre);

    return formate(format(o,"new {0}[{1}]"),new String[]
	{ code(o.type), sn });
  }

  public String code(AglExpressionParenthesee o)
  {
    return formate(format(o,"({0})"), new String[] { code(o.expression) });
  }

  public String code(AglExpressionTernaire o)
  {
    return formate(format(o,"{0} ? {1} : {2}"), new String[]
	{ code(o.gauche), code(o.milieu), code(o.droite) });
  }

  public String code(AglExpressionTypage o)
  {
    return formate(format(o,"({0}){1}"), new String[]
		   { code(o.type), code(o.expression) });
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r="";
    String t=get("d"+o.operateur,o.operateur+"{0}");

    indentation++;
    String sd=code(o.droite);
    indentation--;

    r+=formate(t,new String[] { sd });
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r="";
    String t=get("g"+o.operateur,"{0}"+o.operateur);
                                                                                           
    indentation++;
    String sg=code(o.gauche);
    indentation--;

    r+=formate(t,new String[] { sg });
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=o.body;
    if(r.charAt(r.length()-1)!='\n') r+="\n";
    r=r.substring(0,r.length()-1)+"\n";
    return formate(format(o,"\t{0}"), new String[] { r });
  }

  public String code(AglInstructionVide o)
  {
    return formate(format(o,"\t;\n"), new String[0]);
  }

  public String code(AglInstructionAffectation o)
  {
    return formate(format(o,"\t{0}={1};\n"), new String[]
	{ code(o.gauche), code(o.droite) });
  }

  public String code(AglInstructionBloc o)
  {
    return formate(format(o,"\t{0}"), new String[]
	{ code(o.corps) });
  }

  public String code(AglInstructionBreak o)
  {
    String sl="";
    if(o.nom!=null) sl=o.nom;

    return formate(format(o,"\t{0}:\n"), new String[] { sl });
  }

  public String code(AglInstructionCatch o)
  {
    return formate(format(o,"\tcatch({0})\n{1}"), new String[]
	{ code(o.signal), code(o.reprise) });
  }

  public String code(AglInstructionContinue o)
  {
    String sl="";
    if(o.nom!=null) sl+=o.nom;

    return formate(format(o,"\tcontinue {0};\n"), new String[] { sl });
  }

  public String code(AglInstructionDefVar o)
  {
    String vi=code(o.initiale);
    vi=formate(get("valeur-initiale","{1,choice,0#|1#={0}}"),
	       new Object[] { vi, new Integer((o.initiale==null) ? 0 : 1) });

    return formate(format(o,"\t{0} {1}{2};\n"), new String[]
		   { code(o.type), o.nom, vi } );
  }

  public String code(AglInstructionDoWhile o)
  {
    return formate(format(o,"\tdo\n{0}\twhile({1});\n"), new String[]
	{ code(o.corps), code(o.condition) });
  }

  public String code(AglInstructionExpression o)
  {
    return formate(format(o,"\t{0};\n"),
	 new String[] { code(o.expression) });
  }

  public String code(AglInstructionFinally o)
  {
    return formate(format(o,"\tfinally\n{0}"),
		   new String[] { code(o.corps) });
  }

  public String code(AglInstructionFor o)
  {
    return formate(format(o,"\tfor({0} {1}; {2})\n{3}"), new String[]
	{ code(o.debut), code(o.test), code(o.fin), code(o.corps) });
  }

  public String code(AglInstructionGoto o)
  {
    return formate(format(o,"\tgoto {0};\n"), new String[]
	{ o.nom });
  }

  public String code(AglInstructionIf o)
  {
    String ssn="";
    int    nsn=0;
    if(o.sinon!=null) { ssn=code(o.sinon); nsn=1; }

    return formate
	(format(o,"\tif({0})\n{1}{3,choice,0#|1#\telse {2}}"),
	 new Object[] { code(o.condition), code(o.sioui),
			ssn, new Integer(nsn) });
  }

  public String code(AglInstructionLabel o)
  {
    return formate(format(o,"\t{0}:\n"),new String[] { o.nom });
  }

  public String code(AglInstructionProcedure o)
  {
    String la="";
    indentation++;
    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(p) p=false; else la+=get("separateur-argument",", ");
      la+=code(ae);
    }
    la=remplaceParEt(la);
    indentation--;

    return formate(format(o,"\t{0}({1});\n"),
		   new String[] { o.nom, la });
  }

  public String code(AglInstructionReturn o)
  {
    return formate(format(o,"\treturn {0};\n"),
		   new String[] { code(o.valeur) });
  }

  public String code(AglInstructionSwitch o)
  {
    String lc="";
    indentation++;
    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      lc+=formate(get("case","\tcase {0}:\n{1}\n"), new String[]
	  { code(ce), code(cb) });
    }
    indentation--;

    return formate
	(format(o,"\tswitch({0})\n\t'{'\n{1}{2}\t'}'\n"),
	 new String[] { code(o.condition), lc, code(o.defaut) });
  }

  public String code(AglInstructionSynchronized o)
  {
    return formate(format(o,"\tsynchronized({0})\n{1}"), new String[]
	{ code(o.objet), code(o.corps) });
  }

  public String code(AglInstructionThrow o)
  {
    return formate(format(o,"\tthrow {0};\n"), new String[]
	{ code(o.valeur) });
  }

  public String code(AglInstructionTry o)
  {
    return formate(format(o,"\ttry\n{0}"), new String[]
			     { code(o.corps) });
  }

  public String code(AglInstructionWhile o)
  {
    return formate(format(o,"\twhile({0})\n{1}"), new String[]
	{ code(o.condition), code(o.corps) });
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
        
    if(o.paquet!=null) r+="\n"+code(o.paquet);

    if(o.imports.size()>0) r+="\n";
    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }

    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
        {
	  r+=code(as);
          r+="\n";
        }
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
	{
	  r+=code(ac);
	  r+="\n";
	}
      }
    }

    /*
    r=FuLib.replace(r," que en "    ," qu'en "     );
    r=FuLib.replace(r," de le "     ," du "        );
    r=FuLib.replace(r," de e"       ," d'e"        );
    r=FuLib.replace(r," de o"       ," d'o"        );
    r=FuLib.replace(r," un chane " ," une chane ");
    */

    return r;
  }
}
