/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeH.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import java.util.*;

public class AglCodeH extends AglCode
{
  AglClasse classe_=null;
  int       nbconstructeurs_=0;

  public String extension()
  {
    return "h";
  }

  public String[] extensions()
  {
    return new String[] { "h", "H" };
  }

  public String syntaxe()
  {
    return "C";
  }

  public String code(AglNom o)
  {
    String r=o.dernier();
    if(r.equals("String")) r="char[]";
    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
         if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau) r+=code(((AglTypeTableau)o).type)+"["+code(((AglTypeTableau)o).dimension)+"]";
    else r+=code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    String r="";
    r+=code(o.visibilite);
    r+=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    r+=";\n";
    return r;
  }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    String r="/* ";
    r+=code(o.nom)+"."+extension();
    if(o.ligne>0) r+=" ["+o.ligne+"]";
    r+=" */\n\n";

    r+="typedef struct\n";
    r+="{\n";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+="  "+code(aa);
    }

    r+="} "+code(o.nom)+";\n\n";
    return r+"\n\n";
  }

  protected void ajouteAttributs(AglClasse c, AglVecteur v)
  {
    for(Enumeration ea=c.attributs.elements(); ea.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)ea.nextElement();
      v.addElement(aa);
    }

    for(Enumeration em=c.meres.noms.elements(); em.hasMoreElements(); )
    {
      AglNom    nm=(AglNom)em.nextElement();
      AglClasse cm=(AglClasse)AglType.cherche(nm);
      if(cm!=null) ajouteAttributs(cm,v);
    }
  }

  public String code(AglClasse o)
  {
    classe_=o;

    String r="/* ";
    r+=code(o.nom)+"."+extension();
    if(o.ligne>0) r+=" ["+o.ligne+"]";
    r+=" */\n\n";

    if(!global(o))
    {
      r+="typedef struct\n";
      r+="{\n";

      AglVecteur va=new AglVecteur(11,11);
      ajouteAttributs(o,va);
      for(Enumeration e=va.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	r+="  "+code(aa);
      }

      r+="} "+code(o.nom)+";\n\n";

      /*
	r+=code(o.visibilite);
	r+="class "+code(o.nom);
	
	if(o.meres.noms.size()>0)
	r+=" : "+code(o.meres);
	if(o.interfaces.noms.size()>0)
	r+=" implements "+code(o.interfaces);
	if(o.exceptions.noms.size()>0)
	r+=" throws "+code(o.exceptions);
      */
    
      for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
      {
	AglConstructeur am=(AglConstructeur)e.nextElement();
	r+="\n"+code(am);
      }
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+="\n"+code(am);
    }

    classe_=null;
    return r+"\n\n";
  }

  public String code(AglConstructeur o)
  {
    if(global(classe_)) return "";

    String r="";
    r+=code(o.visibilite);
    r+=code(classe_.nom)+"* new_"+code(o.nom);
    if(o.parametres.size()>0)
    {
      nbconstructeurs_++;
      r+=nbconstructeurs_; // o.parametres.size();
    }
    r+="(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=");";
    return r;
  }

  public String code(AglImport o)
  {
    String r="#include <"+o.nom.dernier();
    r+=".hpp>\n";
    return r;
  }

  public String code(AglMethode o)
  {
    String r="";
    r+=code(o.visibilite);

    AglType at=o.type;
    while(at instanceof AglTypeAlias)
      at=((AglTypeAlias)at).type;

    if(at instanceof AglTypePrimitif)
      r+=code(at);
    else
    if(at instanceof AglTypeTableau)
      r+=code(at);
    else
    if(at==AglType.cherche("String"))
      r+=code(at);
    else
      r+=code(at)+"*";

    r+=" ";
    if(!global(classe_))
      r+=code(classe_.nom)+"_";
    r+=o.nom+"(";

    boolean p=true;

    if(!o.visibilite.vstatic&&!global(classe_))
    {
      r+=code(classe_.nom)+"* this";
      p=false;
    }

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=");";
    return r;
  }

  public String code(AglParametre o)
  {
    String r="";

    AglType at=o.type;
    while(at instanceof AglTypeAlias)
      at=((AglTypeAlias)at).type;

    if((at instanceof AglTypePrimitif)&&(o.passage==AglParametre.IN))
      r+=code(at);
    else
    if(at instanceof AglTypeTableau)
      r+=code(at);
    else
    if(at==AglType.cherche("String"))
      r+=code(at);
    else
      r+=code(at)+"*";

    r+=" "+o.nom;
    // if(!o.defaut.equals("")) r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    // if(o.vpublic)       r+="public ";
    // if(o.vprivate)      r+="private ";
    // if(o.vprotected)    r+="protected ";
    // if(o.vabstract)     r+="virtual ";
    // if(o.vinline)       r+="inline ";
    // if(o.vstatic)       r+="static ";
    // if(o.vfinal)        r+="final ";
    // if(o.vnative)       r+="native ";
    // if(o.vtransient)    r+="register ";
    // if(o.vsynchronized) r+="synchronized ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="new_"+o.type.nom.dernier();
    if(o.arguments.size()>0) r+=o.arguments.size();
    r+="(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r=" ("+code(o.type)+"*)malloc("+code(o.nombre)+",sizeof("+code(o.type)+")))";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }

  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
        {
          r+=code(as);
          r+="\n";
        }
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
}
