/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeHpp.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.io.*;
import java.util.*;

public class AglCodeHpp extends AglCodeCppAbstract
{
  protected boolean VIRTUALMOTHER=
    AglPreferences.AGL.getBooleanProperty("hpp.code.virtualmother",false);

  protected boolean UNITPROTECTOR=
    AglPreferences.AGL.getBooleanProperty("hpp.code.unitprotector",true);

  protected boolean UPPERCASEPROTECTOR=
    AglPreferences.AGL.getBooleanProperty("hpp.code.uppercaseprotector",true);

  public String extension()
  {
    return "hpp";
  }

  public String[] extensions()
  {
    return new String[] { "hpp", "HPP", "hh", "HH", "h", "H" };
  }

  public String code(AglAttribut o)
  {
    String r="";
    r+=code(o.visibilite);
    r+=code(o.type)+" "+o.nom;
    //if(o.initiale!=null) r+=" ="+code(o.initiale);
    r+=";\n";
    return r;
  }

  public String avantStruct(AglStruct o)
  {
    return "\n";
  }

  public String apresStruct(AglStruct o)
  {
    return "";
  }

  public String code(AglStruct o)
  {
    String r="";

    r+=avantStruct(o);
    /*
    String macro="__"+code(o.nom,NAMESPACE).toUpperCase()+"_HPP__";
    r+="\n";
    r+="#ifndef "+macro+"\n";
    r+="#define "+macro+"\n";
    r+="\n";
    r+="// "+code(o.nom,NAMESPACE)+"."+extension()+" ["+o.ligne+"]\n\n";
    */

    // r+=code(o.visibilite);
    r+="struct "+code(o.nom,true);

    r+="\n{\n";
    if(o.attributs.size()>0)
    {
      r+="\n// Attributes\n";
      r+="public:\n";
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(aa.visibilite.vpublic)
	  r+="  "+code(aa);
      }
      r+="protected:\n";
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vpublic&&!aa.visibilite.vprivate)
	  r+="  "+code(aa);
      }
      r+="private:\n";
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(aa.visibilite.vprivate)
	  r+="  "+code(aa);
      }
    }

    r+="};\n";
    r+=apresStruct(o);
    return r;
  }

  public String avantClasse(AglClasse o)
  {
    return "\n";
  }

  public String apresClasse(AglClasse o)
  {
    return "";
  }

  public String avantMethodes(AglClasse o)
  {
    return "";
  }

  public String code(AglClasse o)
  {
    classe_=o;
    String r="";

    r+=avantClasse(o);

    if(!global(o))
    {
      // r+=code(o.visibilite);
      r+="class "+code(o.nom,true);

      boolean derived=false;

      if(  (o.meres.noms.size()>0)
	 &&(  SINGLECLASSROOT
	    ||!(new AglNom("java.lang.Object")
		.equals(o.meres.noms.elementAt(0)))))
      {
	r+=" : ";
	derived=true;

	boolean p=true;
	
	for(Enumeration e=o.meres.noms.elements(); e.hasMoreElements(); )
	{
	  if(p) p=false; else r+=", ";
	  if(VIRTUALMOTHER) r+="virtual ";
	  r+="public ";
	  r+=code((AglNom)e.nextElement());
	}
      }
      else
      if(GC&&o.normale)
      {
	r+=" : ";
	if(VIRTUALMOTHER) r+="virtual ";
	r+="public ";
	r+="gc_cleanup";
	derived=true;
      }
      
      if(o.interfaces.noms.size()>0)
      {
	if(!derived) r+=" : ";
	else         r+=", ";
	boolean p=true;
	
	for(Enumeration e=o.interfaces.noms.elements(); e.hasMoreElements(); )
	{
	  if(p) p=false; else r+=", ";
	  r+="virtual public "+code((AglNom)e.nextElement());
	}
      }

      /*
	if(o.exceptions.noms.size()>0)
	r+=" throws "+code(o.exceptions);
      */

      r+="\n{";
      indentation++;

      if(o.attributs.size()>0)
      {
	r+="\n// Attributes\n";
	r+="public:\n";
	for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
	{
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  if(aa.visibilite.vpublic)
	    r+=indente()+code(aa);
	}
	r+="protected:\n";
	for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
	{
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  if(!aa.visibilite.vpublic&&!aa.visibilite.vprivate)
	    r+=indente()+code(aa);
	}
	r+="private:\n";
	for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
	{
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  if(aa.visibilite.vprivate)
	    r+=indente()+code(aa);
	}
      }

      if(o.constructeurs.size()>0)
      {
	r+="\n// Constructors\n";
	r+="public:\n";
	for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
	{
	  AglConstructeur ac=(AglConstructeur)e.nextElement();
	  if(ac.visibilite.vpublic)
	    r+=code(ac)+"\n";
	}
	r+="protected:\n";
	for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
	{
	  AglConstructeur ac=(AglConstructeur)e.nextElement();
	  if(!ac.visibilite.vpublic&&!ac.visibilite.vprivate)
	    r+=code(ac)+"\n";
	}

	if(JAVALIKE)
	{
	  Enumeration ec=o.constructeurs.elements();
	  while(ec.hasMoreElements())
	  {
	    AglConstructeur ac=(AglConstructeur)ec.nextElement();
	    r+=indente()+"void initialize0(";
	    boolean p=true;
	    Enumeration ep=ac.parametres.elements();
	    while(ep.hasMoreElements())
	    {
	      AglParametre ap=(AglParametre)ep.nextElement();
	      if(!p) r+=", ";
	      r+=code(ap);
	      p=false;
	    }
	    r+=");\n";
	  }
	}

	r+="private:\n";
	for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
	{
	  AglConstructeur ac=(AglConstructeur)e.nextElement();
	  if(ac.visibilite.vprivate)
	    r+=code(ac)+"\n";
	}
      }
      else
      {
	if(JAVALIKE&&o.normale)
	{
	  r+="\npublic:\n";
	  r+=indente()+"inline "+code(o.nom,true)+"() { ";
	  r+="attributes0(); }\n";
	  r+="private:\n";
	}
      }

      if(JAVALIKE&&o.normale)
	r+=indente()+"void attributes0();\n";

      if(o.methodes.size()>0)
      {
	r+="\n// Methods\n";
	r+="public:\n";
	r+=avantMethodes(o);
	for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
	{
	  AglMethode am=(AglMethode)e.nextElement();
	  if(am.visibilite.vpublic)
	    r+=code(am)+"\n";
	}
	r+="protected:\n";
	for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
	{
	  AglMethode am=(AglMethode)e.nextElement();
	  if(!am.visibilite.vpublic&&!am.visibilite.vprivate)
	    r+=code(am)+"\n";
	}
	r+="private:\n";
	for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
	{
	  AglMethode am=(AglMethode)e.nextElement();
	  if(am.visibilite.vprivate)
	    r+=code(am)+"\n";
	}
      }

      indentation--;
      r+="};\n";
    }
    else // Global
    {
      if(o.attributs.size()>0)
      {
	r+="\n// Attributes\n";
	for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
	{
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  //if(aa.visibilite.vpublic)
	    r+="extern "+code(aa.type)+" "+aa.nom+";\n";
	}
      }
      if(o.methodes.size()>0)
      {
	r+="\n// Methods\n";
	for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
	{
	  AglMethode am=(AglMethode)e.nextElement();
	  if(!am.visibilite.vprivate)
	  {
	    String s=code(am).trim();
	    s=FuLib.replace(s,"static " ,"");
	    s=FuLib.replace(s,"virtual ","");
	    r+=s+"\n";
	  }
	}
      }
    }

    r+=apresClasse(o);
    return r;
  }

  public String code(AglConstructeur o)
  {
    String r=indente();
    r+=code(o.visibilite);
    r=FuLib.replace(r,"const ","");

    if(o.destructeur) r+="virtual ~";

    if(classe_!=null) r+=code(classe_.nom,true);
    else              r+=code(o.nom,true);

    r+="(";
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=")";
    if(o.visibilite.vfinal) r+=" const";

    if(!o.visibilite.vinline)
      r+=";";
    else
      r+=code(o.corps);

    return r;
  }

  public String code(AglImport o)
  {
    String r="#include <"+o.nom.dernier();
    r+=".hpp>\n";
    return r;
  }

  public String code(AglMethode o)
  {
    String r="";
    r+=code(o.visibilite);
    r=FuLib.replace(r,"const ","");

    if(!o.visibilite.vstatic&&!o.visibilite.vfinal) r+="virtual ";

    if(JAVALIKE&&"finalize".equals(o.nom))
      r=indente()+r+"~"+code(classe_.nom,true);
    else
      r=indente()+r+code(o.type)+" "+operator(o);

    r+="(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=")";
    if(o.visibilite.vfinal) r+=" const";

    if(!o.visibilite.vinline)
    {
      if(o.visibilite.vabstract||!classe_.normale) r+=" =0";
      r+=";";
    }
    else
      r+=code(o.corps);

    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vinline)    r+="inline ";
    if(o.vstatic)    r+="static ";
    if(o.vfinal)     r+="const ";
    //if(o.vtransient) r+="register ";

    return r;
  }

// Expressions

// Instructions

// Unite

  protected String avantUnite(AglUnite o)
  {
    String r="";

    if(UNITPROTECTOR)
    {
      String m=o.fichier;

      int i;
      i=m.lastIndexOf(File.separator);
      if(i>=0) m=m.substring(i+1);
      i=m.indexOf('.');
      if(i>=0) m=m.substring(0,i);
      if(UPPERCASEPROTECTOR)
	  m="__"+m.toUpperCase()+"_HPP__";
      else
	  m="_"+m+"_hpp_";

      r+="\n";
      r+="#ifndef "+m+"\n";
      r+="#define "+m+"\n";
      r+="\n";
    }

    return r;
  }

  protected String apresUnite(AglUnite o)
  {
    String r="";

    if(UNITPROTECTOR)
    {
      r+="\n";
      r+="#endif\n";
      r+="\n";
    }

    return r;
  }
}
