/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeJasmin.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;

public class AglCodeJasmin extends AglCode
{
  private Hashtable parametres_=new Hashtable();
  private int       stack_ =0,maxstack_=0;
  private AglClasse classe_=null;

  private int   cpt_cmp=0;
  private int   cpt_ter=0;
  private int   cpt_dowhile=0;
  private int   cpt_for=0;
  private int   cpt_if=0;
  private int   cpt_switch=0;
  private int   cpt_while=0;
  private Stack boucles_=new Stack();

  public String extension()
  {
    return "j";
  }

  public String[] extensions()
  {
    return new String[] { "j", "J" };
  }

  protected String indente()
  {
    return (indentation==0 ? "" : "    ");
  }

  private void definit(String _nom, int _taille)
  {
    int i=parametres_.size();
    if(_taille==1)
      parametres_.put(_nom,new Integer(i));
    else
    if(_taille==2)
    {
      parametres_.put(_nom        ,new Integer(i  ));
      parametres_.put(_nom+"<bis>",new Integer(++i));
    }
    else
      erreur("ERREUR DEFINITION "+_nom+" "+_taille);
  }

  private int numero(String _nom)
  {
    int r=-1;
    Integer i=(Integer)parametres_.get(_nom);

    if(i==null)
      erreur("ERREUR NUMERO "+_nom);
    else
      r=i.intValue();

    return r;
  }

  private void entreeBoucle(String _s)
  {
    boucles_.push(_s);
  }

  private void sortieBoucle(String _s)
  {
    Object o=boucles_.pop();
    if(!o.equals(_s))
      erreur("ERREUR BOUCLE "+_s);
  }

  private String debutBoucle()
  {
    return (String)boucles_.peek();
  }

  private String finBoucle()
  {
    return "END"+(String)boucles_.peek();
  }

  protected String debutCS  () { return indente()+";";   }
  protected String finCS    () { return "";              }
  protected String debutCM  () { return indente()+";;;"; }
  protected String specialCM() { return "";              }
  protected String milieuCM () { return indente()+";  "; }
  protected String finCM    () { return indente()+";;;"; }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="/";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String codeListeTypes(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      AglNom n=(AglNom)e.nextElement();
      AglType t=AglType.cherche(n);
      if(t!=null)
      {
	t=AglTypeAlias.simplifie(t,langage());
	r+=code(t);
      }
      else r+=code(n);
    }

    return r;
  }

  public String code(AglType o)
  {
    if(o instanceof AglTypeAlias) o=AglTypeAlias.simplifie(o,langage());
    return AglType.signature(o);
  }

  public String code(AglMoule o)
  {
    return super.code(o);
  }

  public String code(AglAttribut o)
  {
    indentation=0;
    String r=code(o.commentaires);
    if(LIGNES) r+="; ["+o.ligne+"]\n";

    r+=".field ";
    r+=code(o.visibilite);
    r+=o.nom+" ";
    r+=code(o.type);
    r+="\n";

    indentation=0;
    return r;
  }

  public final String code(AglBloc o)
  {
    return code(o,false,false);
  }

  public final String code(AglBloc o, boolean _forced)
  {
    return code(o,_forced,false);
  }

  public String code(AglBloc o, boolean _forced, boolean _ignored)
  {
    String r=code(o.commentaires);
    
    if(_forced||(o.instructions.size()!=1))
    {
      if(!_ignored) r+=indente()+"\n";
      indentation++;

      // int n=1;
      for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
      {
	AglInstruction ai=(AglInstruction)e.nextElement();
	r+=code(ai);
	// n++;
      }

      indentation--;
      if(!_ignored) r+=indente()+"\n";
    }
    else
    {
      indentation++;
      r+=code(o.instructions.elementAt(0));
      indentation--;
    }

    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;
    String r=code(o.commentaires);
    if(LIGNES) r+="; ["+o.ligne+"]\n";

    r+=".class ";
    r+=code(o.visibilite);
    r+=code(o.nom);
    r+="\n";
    r+=".super java/lang/Object\n\n";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    r+="\n";
    if(LIGNES) r+="; generated default construtor\n";
    r+=".method public <init>()V\n";
    r+=".limit locals 1\n";
    r+=".limit stack  1\n";
    r+="\n";
    indentation++;
    r+=indente()+"aload_0\n";
    r+=indente()+"invokespecial java/lang/Object/<init>()V\n";
    r+=indente()+"return\n";
    indentation--;
    r+="\n";
    r+=".end method\n";

    indentation=0;
    return r;
  }

  public String code(AglClasse o)
  {
    indentation=0;
    classe_=o;

    String r=code(o.commentaires);
    if(LIGNES) r+="; ["+o.ligne+"]\n";

    if(o.normale) r+=".class ";
    else          r+=".interface ";
    r+=code(o.visibilite);
    r+=code(o.nom);
    r+="\n";

    if(o.meres.noms.size()>0)
      r+=".super "+codeListeTypes(o.meres)+"\n";
    if(o.interfaces.noms.size()>0)
      r+=".implements "+codeListeTypes(o.interfaces)+"\n";
    if(o.exceptions.noms.size()>0)
      r+=";.throws "+codeListeTypes(o.exceptions)+"\n";

    if(o.attributs.size()>0) r+="\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    if(o.clinit.instructions.size()>0)
    {
      indentation++;
      r+=indente()+"static\n";
      r+=code(o.clinit,true);
      r+="\n"; 
      indentation--;
    }
    if(o.obinit.instructions.size()>0)
    {
      indentation++;
      r+=code(o.obinit,true);
      r+="\n";
      indentation--;
    }

    if(o.constructeurs.size()>0)
      for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
      {
        AglConstructeur am=(AglConstructeur)e.nextElement();
        r+="\n"+code(am);
      }
    else
      {
	r+="\n";
	if(LIGNES) r+="; generated default construtor\n";
	r+=".method public <init>()V\n";
	r+=".limit locals 1\n";
	r+=".limit stack  1\n";
	r+="\n";
	indentation++;
	r+=indente()+"aload_0\n";
	r+=indente()+"invokespecial ";
	if(o.meres.noms.size()>0) r+=code(o.meres.noms.elementAt(0));
	else                      r+="java/lang/Object";
	r+="/<init>()V\n";
	r+=indente()+"return\n";
	indentation--;
	r+="\n";
	r+=".end method\n";
      }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+="\n"+code(am);
    }

    classe_=null;
    indentation=0;
    return r;
  }

  public String code(AglConstructeur o)
  {
    indentation=0;
    String r=code(o.commentaires);
    if(LIGNES) r+="; ["+o.ligne+"]\n";

    r+=".method ";
    r+=code(o.visibilite);
    if(o.destructeur) r+="finalize";
    else              r+=code("<init>");
    r+="(";

    maxstack_=stack_=0;
    parametres_.clear();
    if(!o.visibilite.vstatic)
    {
      maxstack_=1;
      definit("this",1);
    }
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      definit(ap.nom,taille(ap.type));
      r+=code(ap);
      maxstack_+=taille(ap.type);
    }

    r+=")";
    if(o.destructeur) r+="V";
    r+="\n";

    if(o.exceptions.noms.size()>0)
      r+=";.throws "+code(o.exceptions)+"\n";

    r+=".limit locals 20\n";
    r+=".limit stack  "+(o.parametres.size()+1)+"\n";
    r+=code(o.corps);

    indentation++;
    r+=indente()+"return\n";
    indentation--;
    r+=".end method\n";

    r=FuLib.replace(r,
		    ".limit locals 20\n",
		    ".limit locals "+parametres_.size()+"\n");
    r=FuLib.replace(r,
		    ".limit stack  20\n",
		    ".limit stack  "+maxstack_+"\n");

    return r;
  }

  public String code(AglMethode o)
  {
    indentation=0;
    String r=code(o.commentaires);
    if(LIGNES) r+="; ["+o.ligne+"]\n";

    r+=".method ";
    r+=code(o.visibilite);
    r+=code(o.nom);
    r+="(";

    maxstack_=stack_=0;
    parametres_.clear();
    if(!o.visibilite.vstatic)
    {
      maxstack_=1;
      definit("this",1);
    }
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      definit(ap.nom,taille(ap.type));
      r+=code(ap);
      maxstack_+=taille(ap.type);
    }

    r+=")";
    r+=code(o.type);
    r+="\n";

    if(o.exceptions.noms.size()>0)
      r+=";.throws "+code(o.exceptions)+"\n";

    r+=".limit locals 20\n";
    r+=".limit stack  20\n";
    r+=code(o.corps);

    if(o.type==AglTypePrimitif.tvoid)
    {
      indentation++;
      r+=indente()+"return\n";
      indentation--;
    }
    r+=".end method\n";

    r=FuLib.replace(r,
		    ".limit locals 20\n",
		    ".limit locals "+parametres_.size()+"\n");
    r=FuLib.replace(r,
		    ".limit stack  20\n",
		    ".limit stack  "+maxstack_+"\n");

    return r;
  }

  public String code(AglPaquet o)
  {
    indentation=0;
    String r=code(o.commentaires);
    r+=";.package "+code(o.nom)+"\n";
    return r;
  }

  public String code(AglImport o)
  {
    indentation=0;
    String r=code(o.commentaires);
    r+=";.import "+code(o.nom)+"\n";
    return r;
  }

  public String code(AglParametre o)
  {
    String r="";
    r+=code(o.type);
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vpublic)       r+="public ";
    if(o.vprivate)      r+="private ";
    if(o.vprotected)    r+="protected ";
    // if(o.vinline)       r+="inline ";
    if(o.vabstract)     r+="abstract ";
    if(o.vstatic)       r+="static ";
    if(o.vfinal)        r+="final ";
    if(o.vnative)       r+="native ";
    if(o.vtransient)    r+="transient ";
    if(o.vstrictfp)     r+="strictfp ";
    if(o.vsynchronized) r+="synchronized ";

    return r;
  }

// Expressions

  private String prefix(AglExpression o)
  {
    if(o==null) return "x";
    return prefix(o.guessType());
  }

  private String prefix(AglLiteral o)
  {
    if(o==null) return "x";
    return prefix(o.guessType());
  }

  private String prefix(AglType o)
  {
    if(o==null) return "x";
    String  r;
         if(o==AglTypePrimitif.tdouble)  r="d";
    else if(o==AglTypePrimitif.tfloat )  r="f";
    else if(o==AglTypePrimitif.tlong)    r="l";
    else if(o==AglTypePrimitif.tinteger) r="i";
    else if(o==AglTypePrimitif.tshort)   r="i";
    else if(o==AglTypePrimitif.tbyte)    r="i";
    else if(o==AglTypePrimitif.tchar)    r="i";
    else if(o==AglTypePrimitif.tboolean) r="i";
    else                                 r="a";
    return r;
  }

  private int taille(AglExpression o)
  {
    if(o==null) return 1;
    return taille(o.guessType());
  }

  private int taille(AglLiteral o)
  {
    if(o==null) return 1;
    return taille(o.guessType());
  }

  private int taille(AglType o)
  {
    int r=1;
         if(o==AglTypePrimitif.tvoid  ) r=0;
    else if(o==AglTypePrimitif.tdouble) r=2;
    else if(o==AglTypePrimitif.tlong  ) r=2;
    return r;
  }

  private String constant(AglLiteral o)
  {
    AglType t=o.guessType();
    String  r;
         if(t==AglTypePrimitif.tdouble) r="ldc2_w";
    else if(t==AglTypePrimitif.tfloat ) r="ldc";
    else if(t==AglTypePrimitif.tlong)   r="ldc2_w";
    else                                r="ldc";
    return r;
  }

  private String pop(AglExpression o)
  {
    String r="";
    int    n=taille(o);

    switch(n)
    {
    case  0: break;
    case  1: r+=indente()+"pop\n"; break;
    case  2: r+=indente()+"pop2\n"; break;
    default: for(int i=0;i<n;i++) r+=indente()+"pop\n"; break;
    }

    maxstack_=Math.max(stack_,maxstack_);
    //r+=";;; S-="+n+"\n";
    stack_-=n;
    return r;
  }

  public String code(AglExpression o)
  {
    String r="";
    //r+="  ;;; AVANT S="+stack_+" M="+maxstack_+"\n";
    r+=super.code(o);
    //r+="  ;;; APRES S="+stack_+" M="+maxstack_+" "+o+"\n";
    return r;
  }

  public String code(AglExpressionGenerique o)
  {
    String r="";
    //if(LIGNES) r+=indente()+"; Generique ["+o.ligne+"]\n";
    if(o.literal==null)
      r+=indente()+"; push "+o.body+"\n";
    else
    {
      r+=indente();
      if(o.literal instanceof AglLiteralThis)
      {
	r+="iload_0";
	stack_++;
      }
      else
      if(o.literal instanceof AglLiteralSuper)
      {
	r+="iload_0";
	stack_++;
      }
      else
      if(o.literal instanceof AglLiteralNull)
      {
	r+="aconst_null";
	stack_++;
      }
      else
      if(o.literal instanceof AglLiteralBooleen)
      {
	if(((AglLiteralBooleen)o.literal).valeur)
	  r+="iconst_1";
	else
	  r+="iconst_0";
	stack_++;
      }
      else
      if(o.literal instanceof AglLiteralEntier)
      {
	//if(((AglLiteralEntier)o.literal).simple) r+="i";
	//else                                     r+="l";
        r+=constant(o.literal)+" "+((AglLiteralEntier)o.literal).valeur;
	stack_+=taille(o.literal);
      }
      else
      if(o.literal instanceof AglLiteralReel)
      {
	//if(((AglLiteralReel)o.literal).simple) r+="f";
	//else                                   r+="d";
        r+=constant(o.literal)+" "+((AglLiteralReel)o.literal).valeur;
	stack_+=taille(o.literal);
      }
      else
      if(o.literal instanceof AglLiteralChaine)
      {
	r+=constant(o.literal)+" \""+((AglLiteralChaine)o.literal).valeur+"\"";
	stack_+=taille(o.literal);
      }
      else
      if(o.literal instanceof AglLiteralParametre)
      {
	String n=((AglLiteralParametre)o.literal).parametre.nom;
	r+=prefix(o.literal)+"load "+numero(n)+" ; param "+n;
	stack_+=taille(o.literal);
      }
      else
      if(o.literal instanceof AglLiteralVariable)
      {
	 AglLiteralVariable lv=(AglLiteralVariable)o.literal;
	 String n="<unknown>";
	      if(lv.expression !=null) n=lv.expression.nom;
	 else if(lv.instruction!=null) n=lv.instruction.nom;
	 r+=prefix(o.literal)+"load "+numero(n)+" ; var "+n;
	 stack_+=taille(o.literal);
      }
      else
	r+="; push "+o.body;
      r+="\n";
    }
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Exp. Affectation ["+o.ligne+"]\n";

    if(o.gauche instanceof AglExpressionGenerique)
    {
      r+=code(o.gauche);
      r+=code(o.droite);
      r+=indente()+prefix(o.gauche)+"store "
	  +numero(((AglExpressionGenerique)o.gauche).body)
	  +"\n";
      maxstack_=Math.max(stack_,maxstack_);
      stack_-=taille(o.droite);
    }
    else
    if(o.gauche instanceof AglExpressionAttribut)
    {
      r+=code(((AglExpressionAttribut)o.gauche).objet);
      r+=code(o.droite);
      r+=indente()+"putfield "
	  +code(((AglExpressionAttribut)o.gauche).objet.guessType().nom)
	  +"/"
	  +((AglExpressionAttribut)o.gauche).attribut
	  +" "
	  +code(((AglExpressionAttribut)o.gauche).guessType())
	  +"\n";
      maxstack_=Math.max(stack_,maxstack_);
      stack_-=taille(o.droite);
      stack_-=taille(((AglExpressionAttribut)o.gauche).objet);
    }
    else
    {
      r+=code(o.gauche);
      r+=code(o.droite);
      maxstack_=Math.max(stack_,maxstack_);
      stack_-=taille(o.droite);
    }

    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Attribut ["+o.ligne+"]\n";

    if(o.statique)
    {
      AglType t=null;
      if(o.objet!=null)	t=o.objet.guessType();
      r+=";;; T="+t+" "+o+"\n";
      if(t!=null)
      r+=indente()+"getstatic "+code(t.nom)+"/"+o.attribut+" ";
      t=o.guessType();
      if(t!=null) r+=code(t);
      else        r+="<unknown>";
      r+="\n";
    }
    else
    {
      String p="";
      if(o.objet!=null) p=code(o.objet);
      else              p="this";
      if("this".equals(p))
        r+=indente()+"aload_0\n";
      else
        r+=p;

      String  s;
      AglType t;
      if("this".equals(p)) t=classe_;
      else                 t=o.objet.guessType();
      if(t!=null)
        s=code(t.nom)+"/"+o.attribut+" "+code(t);
      else
        s=o.attribut;

      r+=indente()+"getfield "+s+"\n";
    }

    //stack_+=taille(o.guessType());
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Binaire ["+o.ligne+"]\n";
    r+=code(o.gauche);
    r+=code(o.droite);

    String s=indente();
    s+=prefix(o.guessType());
         if("+"  .equals(o.operateur)) s+="add";
    else if("-"  .equals(o.operateur)) s+="sub";
    else if("*"  .equals(o.operateur)) s+="mul";
    else if("/"  .equals(o.operateur)) s+="div";
    else if("%"  .equals(o.operateur)) s+="rem";
    else if("<<" .equals(o.operateur)) s+="ishl";
    else if(">>" .equals(o.operateur)) s+="ishr";
    else if(">>>".equals(o.operateur)) s+="iushr";
    else if("&"  .equals(o.operateur)) s+="and";
    else if("|"  .equals(o.operateur)) s+="or" ;
    else if("^"  .equals(o.operateur)) s+="xor";
    else if("&&" .equals(o.operateur)) s+="and";
    else if("||" .equals(o.operateur)) s+="or" ;
    else if("^^" .equals(o.operateur)) s+="xor";
    else
    {
      s=indente();
           if("<"  .equals(o.operateur)) s+="iflt";
      else if("<=" .equals(o.operateur)) s+="ifle";
      else if(">"  .equals(o.operateur)) s+="ifgt";
      else if(">=" .equals(o.operateur)) s+="ifge";
      else s+="if_"+o.operateur;

      cpt_cmp++;
      int n=cpt_cmp;
      s+=" CMP"+n+"T\n";
      s+=indente()+"iconst_0\n";
      s+=indente()+"goto CMP"+n+"E\n";
      s+="CMP"+n+"T:\n";
      s+=indente()+"iconst_1\n";
      s+="CMP"+n+"E:";
    }
    r+=s;
    r+="\n";

    maxstack_=Math.max(stack_,maxstack_);
    stack_-=taille(o.gauche);
    stack_-=taille(o.droite);
    stack_+=taille(o.guessType());
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Exp. Var ["+o.ligne+"]\n";
    definit(o.nom,taille(o.type));
    int i=numero(o.nom);
    r+=indente()+"; "+code(o.type)+" "+o.nom+"->"+i+"\n";
    if(o.initiale!=null)
    {
      r+=code(o.initiale);
      r+=indente()+prefix(o.initiale)+"store "+i+"\n";
      maxstack_=Math.max(stack_,maxstack_);
      stack_-=taille(o.initiale);
    }
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r="";
    if(LIGNES) r+=indente()+" EstUn ["+o.ligne+"]\n";
    r+=code(o.expression);
    r+=indente()+"instanceof "+code(o.type.nom)+"\n";
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Fonction ["+o.ligne+"]\n";

    String t="";
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=code(ae);
      maxstack_=Math.max(stack_,maxstack_);
      //r+=";;; S="+stack_+" M="+maxstack_+"\n";
      t+=code(ae.guessType());
      stack_-=taille(ae);
    }

    r+=indente()+"invokevirtual "+o.nom
	+"("+t+")\n";
    stack_+=taille(o.guessType());
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Indice ["+o.ligne+"]\n";
    r+=code(o.objet);
    r+=code(o.indice);
    r+=indente()+"aaload\n";
    maxstack_=Math.max(stack_,maxstack_);
    stack_--;
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; InitArray ["+o.ligne+"]\n";

    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=code(ae);
    }

    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Methode ["+o.ligne+"]\n";

    String p="";
         if(o.objet!=null) p=code(o.objet);
    else if(o.statique)    p="static";
    else                   p="this";

    if(o.objet!=null)
    {
      r+=p;
    }
    else
    if(p.equals("this"))
    {
      r+=indente()+"aload_0\n";
      stack_++;
    }

    //r+=";;; S="+stack_+" M="+maxstack_+"\n";
    String  s=code(o.fonction);
    AglType t;
    if(o.objet!=null)    t=o.objet.guessType();
    else                 t=classe_;

    if(t!=null)
	s=FuLib.replace(s,
			"invokevirtual "+o.fonction.nom,
			"invokevirtual "+code(t.nom)+"/"+o.fonction.nom);
    else
	s=FuLib.replace(s,
			"invokevirtual "+o.fonction.nom,
			"invokevirtual <unknown>/"+o.fonction.nom);
    
    if(o.statique)
      s=FuLib.replace(s,"invokevirtual","invokestatic");

    t=o.guessType();
    if(t!=null)
	s=FuLib.replace(s,")\n",")"+code(t)+"\n");
    else
	s=FuLib.replace(s,")\n",")<unknown>\n");

    r+=s;

    maxstack_=Math.max(maxstack_,stack_);
         if(o.objet!=null)    stack_-=taille(o.objet);
    else if("this".equals(p)) stack_--;
    else r+="; STATIC\n";

    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Multiple ["+o.ligne+"]\n";

    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=code(ae);
      stack_++;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; New ["+o.ligne+"]\n";

    r+=indente()+"new "+code(o.type.nom)+"\n";
    r+=indente()+"dup\n";
    stack_+=2;

    String t="";
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=code(ae);
      maxstack_=Math.max(stack_,maxstack_);
      //r+=";;; S="+stack_+" M="+maxstack_+"\n";
      t+=code(ae.guessType());
      stack_-=taille(ae);
    }

    r+=indente()+"invokespecial "+code(o.type.nom)
	+"/<init>("+t+")V\n";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; NewArray ["+o.ligne+"]\n";

    if(o.nombre!=null) r+=code(o.nombre);
    else               r+=indente()+"iconst_0\n";
    r+=indente()+"anewarray "+code(o.type)+"\n";

    if(o.initiale!=null) r+=code(o.initiale);
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Parenthesee ["+o.ligne+"]\n";
    r+=code(o.expression);
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Ternaire ["+o.ligne+"]\n";

    cpt_ter++;
    int n=cpt_ter;

    r+=code(o.gauche);
    r+=indente()+"iconst_0\n";
    r+=indente()+"ifeq TER"+n+"F\n";
    r+=code(o.milieu);
    r+=indente()+"goto TER"+n+"E\n";
    r+="TER"+n+"F:\n";
    r+=code(o.droite);
    r+="TER"+n+"E:\n";
    //+o.operateur1
    //+o.operateur2
    maxstack_=Math.max(stack_,maxstack_);
    stack_-=2;
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; Typage ["+o.ligne+"]\n";
    r+=code(o.expression);
    r+=indente()+"checkcast "+code(o.type.nom)+"\n";
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; UnaireDroite ["+o.ligne+"]\n";
    r+=code(o.droite);

    if(!"+".equals(o.operateur))
    {
      r+=indente();
      r+=prefix(o.guessType());
           if("-"  .equals(o.operateur)) r+="neg";
      else if("!"  .equals(o.operateur)) r+="xor 0xffffffff";
      else r+=o.operateur+" ; unknown operator "+o;
      r+="\n";

      maxstack_=Math.max(stack_,maxstack_);
      stack_-=taille(o.droite);
      stack_+=taille(o.guessType());
    }

    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r="";
    if(LIGNES) r+=indente()+"; UnaireGauche ["+o.ligne+"]\n";
    r+=code(o.gauche);
    r+=indente();

    if("++".equals(o.operateur))
      r+="iinc "+numero(((AglExpressionGenerique)o.gauche).body)
	 +" 1";
    else
    if("--".equals(o.operateur))
      r+="iinc "+numero(((AglExpressionGenerique)o.gauche).body)
	 +" -1";
    else
      r+=o.operateur+" ; unknown operator "+o;
    r+="\n";

    maxstack_=Math.max(stack_,maxstack_);
    stack_-=taille(o.gauche);
    stack_+=taille(o.guessType());
    return r;
  }

// Instructions

  public String code(AglInstruction o)
  {
    String r="";
    //r+=";;; S="+stack_+" M="+maxstack_+"\n";
    int    os=stack_;
    r+=super.code(o);
    int    ns=stack_;
    if(ns<os)
      r+=";;; ERROR STACK O="+os+" N="+ns+"\n";
    else
    {
      if(ns!=os) r+=";;; POP S="+stack_+" M="+maxstack_+" "+o+"\n";
      for(int i=os;i<ns;i++)
	r+=indente()+"pop\n";
      stack_=os;
    }
    return r;
  }


  public String code(AglInstructionGenerique o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Instruction ["+o.ligne+"]\n";
    r+=indente();
    r+=o.body;
    if((r.length()>0)&&(r.charAt(r.length()-1)!='\n')) r+="\n";
    r=r.substring(0,r.length()-1);
    r+="\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Vide ["+o.ligne+"]\n";
    r+=indente()+"nop\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Ins. Affectation ["+o.ligne+"]\n";

    r+=code(o.droite);
    if(o.gauche instanceof AglExpressionGenerique)
    {
      maxstack_=Math.max(stack_,maxstack_);
      stack_-=taille(o.droite);
      r+=indente()+prefix(o.gauche)+"store "
	  +numero(((AglExpressionGenerique)o.gauche).body)
	  +"\n";
      //r+=code(o.gauche);
    }
    else
    {
      r+=code(o.gauche);
      maxstack_=Math.max(stack_,maxstack_);
      stack_-=taille(o.droite);
    }

    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Bloc ["+o.ligne+"]\n";
    if(!o.virgule)
	r+=code(o.corps);
    else
    {
      boolean p=true;
      for(Enumeration e=o.corps.instructions.elements(); e.hasMoreElements(); )
      {
	if(!p) r+=",";
	AglInstruction ai=(AglInstruction)e.nextElement();
	String s=code(ai).trim();
	int    i=s.lastIndexOf(";");
	s=s.substring(0,i)+s.substring(i+1);
	if((!p)&&(ai instanceof AglInstructionDefVar))
	{
	  i=s.indexOf(" ")+1;
	  s=s.substring(i);
	}
	r+=s;
	p=false;
      }
      r+=";";
    }
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Break ["+o.ligne+"]\n";
    r+=indente()+"goto ";
    if(o.nom!=null) r+=o.nom.toUpperCase();
    else            r+=finBoucle();
    r+="\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Catch ["+o.ligne+"]\n";
    r+="CATCH:\n";
    r+=indente()+".catch "+code(o.signal).trim();
    r+=" from TRY to CATCH unsing HANDLER\n";
    r+=indente()+"goto AFTER_CATCH\n";
    r+="HANDLER:\n";
    r+=code(o.reprise,true);
    r+="AFTER_CATCH:\n";
    return r;
  }

  public String code(AglInstructionCondition o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Condition ["+o.ligne+"]\n";
    r+=indente()+"; assert("+
	new AglCodeJava().code(o.condition.test)+")\n";
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Continue ["+o.ligne+"]\n";
    r+=indente()+"goto ";
    if(o.nom!=null) r+=o.nom.toUpperCase();
    else            r+=debutBoucle();
    r+="\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Ins. DefVar ["+o.ligne+"]\n";
    definit(o.nom,taille(o.type));
    int i=numero(o.nom);
    r+=indente()+"; "+code(o.type)+" "
	+o.nom+"->"+i+"\n";
    if(o.initiale!=null)
    {
      r+=code(o.initiale);
      r+=indente()+prefix(o.initiale)+"store "+i+"\n";
      maxstack_=Math.max(stack_,maxstack_);
      stack_-=taille(o.initiale);
    }
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; DoWhile ["+o.ligne+"]\n";

    cpt_dowhile++;
    int n=cpt_dowhile;

    entreeBoucle("DOWHILE"+n);
    r+="DOWHILE"+n+":\n";
    r+=code(o.corps);
    r+=indente()+"iconst_0\n";
    r+=indente()+"ifne DOWHILE"+n+"\n";
    r+="ENDDOWHILE"+n+":\n";
    sortieBoucle("DOWHILE"+n);

    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; InsExpr ["+o.ligne+"]\n";
    r+=code(o.expression);
    r+=pop(o.expression);
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Finally ["+o.ligne+"]\n";
    r+=code(o.corps,true);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; For ["+o.ligne+"]\n";

    r+=code(o.debut);

    cpt_for++;
    int n=cpt_for;

    entreeBoucle("FOR"+n);
    r+="FOR"+n+":\n";
    r+=code(o.test);
    r+=indente()+"iconst_0\n";
    r+=indente()+"ifeq ENDFOR"+n+"\n";
    r+=code(o.corps);
    r+=code(o.fin);
    r+=indente()+"goto FOR"+n+"\n";
    r+="ENDFOR"+n+":\n";
    sortieBoucle("FOR"+n);

    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Goto ["+o.ligne+"]\n";
    r+=indente()+"goto "+o.nom+"\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; If ["+o.ligne+"]\n";

    cpt_if++;
    int n=cpt_if;

    r+=code(o.condition);
    r+=indente()+"iconst_0\n";
    if(o.sinon!=null)
      r+=indente()+"ifeq ELSE"+n+"\n";
    else
      r+=indente()+"ifeq ENDIF"+n+"\n";

    r+=code(o.sioui);
    if(o.sinon!=null)
    {
      r+=indente()+"goto ENDIF"+n+"\n";
      r+="ELSE"+n+":\n";
      r+=code(o.sinon);
    }
    r+="ENDIF"+n+":\n";

    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Label ["+o.ligne+"]\n";
    r+=o.nom.toUpperCase()+":\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Procedure ["+o.ligne+"]\n";

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=code(ae);
    }

    r+=indente()+"invokevirtual "+o.nom;
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Return ["+o.ligne+"]\n";
    if(o.valeur!=null)
    {
      r+=code(o.valeur);
      r+=indente()+prefix(o.valeur)+"return\n";
      stack_-=taille(o.valeur);
    }
    else
    {
      r+=indente()+"return\n";
    }
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Switch ["+o.ligne+"]\n";

    cpt_switch++;
    int n=cpt_switch;

    entreeBoucle("SWITCH"+n);
    r+=code(o.condition);
    r+=indente()+"lookupswitch\n";
    maxstack_=Math.max(stack_,maxstack_);
    stack_--;
    indentation++;

    int i=1;
    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); i++)
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente();

      if(ce instanceof AglExpressionGenerique)
	   r+=((AglExpressionGenerique)ce).body;
      else r+=indente()+"   "+code(ce);

      r+=" : SWITCH"+n+"C"+i+"\n";
    }
    r+=indente()+"default : SWITCH"+n+"DEFAULT\n";

    r+=indente()+"goto "+finBoucle()+"\n";
    i=1;
    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); i++)
    {
      AglExpression ce=(AglExpression)e.nextElement();
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+="SWITCH"+n+"C"+i+":\n";
      r+=code(cb,false,true);
      //r+=indente()+"goto "+finBoucle()+"\n";
    }

    r+="SWITCH"+n+"DEFAULT:\n";
    r+=code(o.defaut);

    indentation--;
    r+=finBoucle()+":\n";
    sortieBoucle("SWITCH"+n);
    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Synchronized ["+o.ligne+"]\n";
    r+=indente()+"; synchronized("+code(o.objet)+")";
    r+=code(o.corps,true);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Throw ["+o.ligne+"]\n";
    r+=code(o.valeur);
    r+=indente()+"athrow\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; Try ["+o.ligne+"]\n";
    r+="TRY:\n";
    r+=code(o.corps,true);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"; While ["+o.ligne+"]\n";

    cpt_while++;
    int n=cpt_while;

    entreeBoucle("WHILE"+n);
    r+="WHILE"+n+":\n";
    r+=code(o.condition);
    r+=indente()+"iconst_0\n";
    r+=indente()+"ifeq ENDWHILE"+n+"\n";
    r+=code(o.corps);
    r+=indente()+"goto WHILE"+n+"\n";
    r+="ENDWHILE"+n+":\n";
    sortieBoucle("WHILE"+n);

    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+=code(o.commentaires);

    if(o.paquet!=null) r+="\n"+code(o.paquet);

    if(o.imports.size()>0) r+="\n";
    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }

    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglMoule)
      {
        AglMoule ap=(AglMoule)at;
        if(!ap.elementaire)
        {
          r+=code(ap);
          r+="\n";
        }
      }
    }

    AglVecteur simplifications=new AglVecteur(11,11);
    /*
    simplifications.addElement("java.lang.");
    for(Enumeration ei=o.imports.elements(); ei.hasMoreElements(); )
    {
      AglImport ai=(AglImport)ei.nextElement();
      AglNom    ni=ai.nom;
      if("*".equals(ni.dernier()))
      {
	ni=ni.parent();
	simplifications.addElement(ni.formeJava()+".");
      }
    }
    */
    String[] ss=new String[simplifications.size()];
    simplifications.copyInto(ss);

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
        {
          String ras=code(as);
	  for(int i=ss.length-1;i>=0;i--)
	    ras=FuLib.replace(ras,ss[i],"");
	  r+=ras;
          r+="\n";
        }
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
        {
          String rac=code(ac);
	  for(int i=ss.length-1;i>=0;i--)
	    rac=FuLib.replace(rac,ss[i],"");
	  r+=rac;
          r+="\n";
        }
      }
    }

    return r;
  }
}
