/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeJniH.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglCodeJniH extends AglCode
{
  protected AglClasse classe_=null;

  public String extension()
  {
    return "h";
  }

  public String[] extensions()
  {
    return new String[] { "h", "H" };
  }

  public String syntaxe()
  {
    return "C";
  }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="_";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";

         if(o instanceof AglTypeAlias   ) r+=code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau ) r+=code(((AglTypeTableau)o).type)+"Array";
    else if(o instanceof AglTypePrimitif) r+="j"+o.nom.dernier();
    else if(o.nom.equals(new AglNom("java.lang.String"))) r+="jstring";
    else if(o.nom.equals(new AglNom("java.lang.Class" ))) r+="jclass";
    else r+="jobject";

    return r;
  }

  public String code(AglAttribut o)
  { return ""; }

  public String code(AglBloc o)
  { return ""; }

  public String code(AglStruct o)
  { return ""; }

  public String code(AglClasse o)
  {
    String r="";
    classe_=o;

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.visibilite.vnative) r+="\n"+code(am)+"\n";
    }

    classe_=null;
    return r;
  }

  public String code(AglConstructeur o)
  { return ""; }

  public String code(AglImport o)
  { return ""; }

  public String code(AglMethode o)
  {
    String r="";

    r+=debutCM()+"\n";
    r+=milieuCM()+"Class:     "+classe_.nom.dernier()+"\n";
    r+=milieuCM()+"Method:    "+o.nom+"\n";

    String s=o.signature();
    int    i=s.indexOf(':');
    if(i>=0) s=s.substring(i+1);
    r+=milieuCM()+"Signature: "+s+"\n";
    r+=finCM()+"\n";

    r+="JNIEXPORT "+code(o.type)+" JNICALL ";
    r+=" Java_"+code(classe_.nom)+"_"+o.nom+"\n";
    r+="  (JNIEnv *";

    if(o.visibilite.vstatic) r+=", jclass  _class";
    else                     r+=", jobject _this";

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      r+=", ";
      r+=code(ap);
    }

    r+=");";
    return r;
  }

  public String code(AglParametre o)
  {
    return code(o.type);
  }

  public String code(AglVisibilite o)
  {
    return "";
  }

// Expressions

  public String code(AglExpressionGenerique o)
  { return ""; }
  
  public String code(AglExpressionAffectation o)
  { return ""; }
  
  public String code(AglExpressionAttribut o)
  { return ""; }
  
  public String code(AglExpressionBinaire o)
  { return ""; }
  
  public String code(AglExpressionDefVar o)
  { return ""; }
  
  public String code(AglExpressionEstUn o)
  { return ""; }
  
  public String code(AglExpressionFonction o)
  { return ""; }
  
  public String code(AglExpressionIndice o)
  { return ""; }
  
  public String code(AglExpressionInitArray o)
  { return ""; }
  
  public String code(AglExpressionMethode o)
  { return ""; }
  
  public String code(AglExpressionMultiple o)
  { return ""; }
  
  public String code(AglExpressionNew o)
  { return ""; }
  
  public String code(AglExpressionNewArray o)
  { return ""; }
  
  public String code(AglExpressionParenthesee o)
  { return ""; }
  
  public String code(AglExpressionTernaire o)
  { return ""; }
  
  public String code(AglExpressionTypage o)
  { return ""; }
  
  public String code(AglExpressionUnaireDroite o)
  { return ""; }
  
  public String code(AglExpressionUnaireGauche o)
  { return ""; }
  

// Instructions

  public String code(AglInstructionGenerique o)
  { return ""; }
  
  public String code(AglInstructionVide o)
  { return ""; }
  
  public String code(AglInstructionAffectation o)
  { return ""; }
  
  public String code(AglInstructionBloc o)
  { return ""; }
  
  public String code(AglInstructionBreak o)
  { return ""; }
    
  public String code(AglInstructionCatch o)
  { return ""; }
  
  public String code(AglInstructionContinue o)
  { return ""; }
  
  public String code(AglInstructionDefVar o)
  { return ""; }
  
  public String code(AglInstructionDoWhile o)
  { return ""; }
  
  public String code(AglInstructionExpression o)
  { return ""; }
  
  public String code(AglInstructionFinally o)
  { return ""; }
  
  public String code(AglInstructionGoto o)
  { return ""; }
  
  public String code(AglInstructionFor o)
  { return ""; }
  
  public String code(AglInstructionIf o)
  { return ""; }
  
  public String code(AglInstructionLabel o)
  { return ""; }

  public String code(AglInstructionProcedure o)
  { return ""; }
  
  public String code(AglInstructionReturn o)
  { return ""; }

  public String code(AglInstructionSwitch o)
  { return ""; }

  public String code(AglInstructionSynchronized o)
  { return ""; }

  public String code(AglInstructionThrow o)
  { return ""; }

  public String code(AglInstructionTry o)
  { return ""; }

  public String code(AglInstructionWhile o)
  { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";
    r+="/* DO NOT EDIT THIS FILE - it is machine generated */\n";
    r+="#include <jni.h>\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
	  String nc=code(ac.nom);
	  r+="/* Header for class "+nc+" */\n";
	  r+="\n";
	  r+="#ifndef _Included_"+nc+"\n";
	  r+="#define _Included_"+nc+"\n";
	  r+="#ifdef __cplusplus\n";
	  r+="extern \"C\" {\n";
	  r+="#endif";

          r+=code(ac);

	  r+="\n";
	  r+="#ifdef __cplusplus\n";
	  r+="}\n";
	  r+="#endif\n";
	  r+="#endif\n";
        }
      }
    }

    return r;
  }
}
