/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeLisp.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;

import com.memoire.fu.*;

import java.util.*;

public  class AglCodeLisp extends AglCode
{
  AglClasse classe_=null;
  String    main_  ="";

  public String extension()
  {
    return "lsp";
  }

  public String[] extensions()
  {
    return new String[] { "lsp", "LSP", "l", "L", "ll", "LL", "lisp", "LISP" };
  }

  protected String debutCS  () { return indente()+";";   }
  protected String finCS    () { return "";              }
  protected String debutCM  () { return indente()+";;";  }
  protected String specialCM() { return ";";             }
  protected String milieuCM () { return indente()+"; ";  }
  protected String finCM    () { return indente()+";;";  }

  protected String traduit(String s)
  {
    String r="";

    if((s==null)||(s.equals("")))
      ;
    else
    if(s.equals(s.toLowerCase()))
      r+=s;
    else
    if(Character.isUpperCase(s.charAt(0))) //s.equals(s.toUpperCase()))
      r+=s.toLowerCase();
    else
    for(int i=0; i<s.length(); i++)
    {
      char c=s.charAt(i);
      if(Character.isUpperCase(c))
      {
	if(i>0) r+="-";
	c=Character.toLowerCase(c);
      }
      r+=c;
    }

    return r;
  }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false;
      else  r+=":";

      r+=traduit((String)e.nextElement());
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=" ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
         if(o==AglTypePrimitif.tinteger) r="integer";
    else if(o instanceof AglTypeTableau) r="vector";
    else r=code(o.nom);
    return r;
  }

  public String code(AglMoule o)
  {
    return super.code(o);
  }

  public String code(AglAttribut o)
  {
    String n=traduit(o.nom);
    String r="("+n;
    r+=" :type '"+code(o.type);
    r+=" :initarg :"+n;
    r+=" :accessor "+n;
    if(o.initiale!=null)
	r+=" :initform "+code(o.initiale);
    r+=")";
    return r;
  }

  public String code(AglBloc o)
  {
    indentation++;
    String r="";

    for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
    {
      AglInstruction ai=(AglInstruction)e.nextElement();
      if(ai instanceof AglInstructionDefVar)
      {
	AglInstructionDefVar dv=(AglInstructionDefVar)ai;
	if(dv.initiale!=null) r+="("+dv.nom+" "+code(dv.initiale)+") ";
	else                  r+=dv.nom+" ";
      }
    }

    if(r.equals("")) r=indente()+"(progn\n";
    else             r=indente()+"(let* ("+r.trim()+")\n";
    indentation++;

    for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
    {
      AglInstruction ai=(AglInstruction)e.nextElement();
      if(!(ai instanceof AglInstructionDefVar)) r+=code(ai);
    }

    indentation--;
    r+=indente()+")\n";
    indentation--;
    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;

    String r=";;; ";
    r+=code(o.nom)+" ["+o.ligne+"]\n";

    r+="\n(defstruct ";
    r+=o.nom.dernier()+"\n  ";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }
    r+=")\n\n";

    r+="(defabbrev "+o.nom.dernier()+" ";
    r+="#:"+o.nom.dernier();
    r+=")\n\n";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+="(deffield {"+o.nom.dernier()+"} "+code(aa)+")\n";
    }

    indentation=0;
    return r+"\n";
  }

  public String code(AglClasse o)
  {
    classe_=o;
    indentation=0;

    String r=";;; ";
    r+=code(o.nom)+" ["+o.ligne+"]\n";

    if(o.interfaces.noms.size()>0)
      r+="; interfaces: "+code(o.interfaces)+"\n";
    if(o.exceptions.noms.size()>0)
      r+="; exceptions: "+code(o.exceptions)+"\n";

    r+="\n(defclass ";
    r+=code(o.nom); //o.nom.dernier().toLowerCase();
    r+=" ("+code(o.meres)+")";

    if(o.attributs.size()>0)
    {
      indentation++;
      boolean p=true;
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vstatic)
	{
	  r+="\n"+indente();
	  if(p) r+="("; else r+=" ";
	  p=false;
	  r+=code(aa);
	}
      }
      r+=")";
      indentation--;
    }
    else r+=" ()";

    r+=")\n\n";

    /*
    r+="(defabbrev "+o.nom.dernier()+" ";
    if(o.meres.noms.size()>0) r+="{"+code(o.meres)+"}:"+o.nom.dernier();
    else r+="#:"+o.nom.dernier();
    r+=")\n\n";
    */

    /*
    if(o.attributs.size()>0)
    {
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(aa.visibilite.vstatic)
	  r+="(defvariable {"+o.nom.dernier()+"} "+
	    code(aa)+" "+code(aa.initiale)+")\n";
      }
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vstatic)
	  r+="(deffield {"+o.nom.dernier()+"} "+code(aa)+")\n";
      }
      r+="\n";
    }
    */

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am)+"\n";
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am)+"\n";
    }

    indentation=0;
    classe_=null;

    return r+"\n";
  }

  public String code(AglConstructeur o)
  {
    indentation=0;
    String r="(defmethod init-"+code(o.nom)
	+" ((this "+code(classe_.nom)+")";

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      r+=" ";
      r+=code(ap);
    }

    r+=")\n";

    if((classe_!=null)&&!o.visibilite.vstatic)
    {
      indentation++;
      r+=indente()+"(with-slots (";
      boolean p=true;
      for(Enumeration e=classe_.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vstatic)
	{
	  if(!p) r+=" ";
	  r+=code(aa.nom);
	  p=false;
	}
      }
      r+=") this\n";
    }

    if(!o.corps.instructions.isEmpty()) r+=code(o.corps);

    if((classe_!=null)&&!o.visibilite.vstatic)
    {
      r+=indente()+")\n";
      indentation--;
    }

    indentation++;
    r+=indente()+"this\n";
    indentation--;

    r+=")\n";
    return r;
  }

  public String code(AglPaquet o)
  {
    String r="";
    r="(setf *package* "+code(o.nom)+")";
    return r;
  }

  public String code(AglImport o)
  {
    indentation=0;
    String r="";

    if(!o.nom.dernier().equals("*"))
	r+="(import '"+code(o.nom)+")";
    else
	r+="(use-package '"+code(o.nom.parent())+")";

    //(loadfile "+code(o.nom)+".ll)\n";

    r+="\n";
    return r;
  }

  public String code(AglMethode o)
  {
    indentation=0;
    String r="";

    String n=traduit(o.nom);

    if(o.visibilite.vstatic)
    {
      r+="(defmethod ";
      if(classe_!=null)
      {
	r+=code(classe_.nom)+"-";
	if("main".equals(n)) main_=code(classe_.nom)+"-"+n;
      }
      r+=n+" (";
    }
    else
    {
      r+="(defmethod ";
      if(classe_!=null)
        r+=n+" ((this "+code(classe_.nom)+") ";
      else
        r+=n+" (this ";
    }

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(p) p=false; else r+=" ";
      r+=code(ap);
    }

    r+=")\n";

    if((classe_!=null)&&!o.visibilite.vstatic)
    {
      indentation++;
      r+=indente()+"(with-slots (";
      p=true;
      for(Enumeration e=classe_.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vstatic)
	{
	  if(!p) r+=" ";
	  r+=code(aa.nom);
	  p=false;
	}
      }
      r+=") this\n";
    }

    if(!o.corps.instructions.isEmpty()) r+=code(o.corps);

    if((classe_!=null)&&!o.visibilite.vstatic)
    {
      r+=indente()+")\n";
      indentation--;
    }

    r+=")\n";
    return r;
  }

  public String code(AglParametre o)
  {
    String r="";
    if(o.type!=null)
      r+="("+traduit(o.nom)+" "+code(o.type)+")";
    else
      r+=traduit(o.nom);
    return r;
  }

  public String code(AglVisibilite o)
  {
    return "";
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body.toLowerCase();
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="(setf "+code(o.gauche)+" "+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";

    String n=traduit(o.attribut);

    if(o.statique)
    {
      if(o.objet!=null) r+=code(o.objet)+"-"+n;
    }
    else
    {
      if(o.objet!=null) r+="("+n+" "+code(o.objet)+")";
      else              r+=n;
    }

    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String  r=o.operateur;
    boolean n=false;

         if(r.equals("==")) r="equals";
    else if(r.equals("&&")) r="and";
    else if(r.equals("||")) r="or";
    else if(r.equals("!=")) { n=true; r="equals"; }

    r="("+r+" ";
    r+=code(o.gauche);
    r+=" ";
    r+=code(o.droite);
    r+=")";

    if(n) r="(not "+r+")";
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=o.nom;
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r="(typep "+code(o.expression)+" "+code(o.type)+")";
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r="("+traduit(o.nom);

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r="(svref "+code(o.objet)+" "+code(o.indice)+")";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="(vector ";

    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="(";

    String n;
    if(o.fonction.nom.equals("this"))
      n="init-"+traduit(classe_.nom.dernier());
    else
    if(o.fonction.nom.equals("super"))
      n="init-"+traduit
	  (((AglNom)classe_.meres.noms.elementAt(0)).dernier());
    else
      n=traduit(o.fonction.nom);

    if(o.statique)
    {
      if(o.objet!=null) r+=traduit(code(o.objet))+"-"+n;
      else              r+=n;
    }
    else
    {
      r+=n+" ";
      if(o.objet!=null) r+=code(o.objet);
      else              r+="this";
    }

    for(Enumeration e=o.fonction.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="(progn";

    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="(init-"+traduit(o.type.nom.dernier());

    r+=" (make-instance '"+code(o.type)+")";

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="(";
    if(o.nombre!=null) r+="make-array "+code(o.nombre);
    else               r+="vector";
    r+=")";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r=code(o.expression);
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r="(if "+code(o.gauche)
      +" "+code(o.milieu)
      +" "+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r=code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r="("+o.operateur+" "+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r="("+o.operateur+" "+code(o.gauche)+")";
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=indente()+"(";
    r+=o.body.trim();
    if(r.charAt(r.length()-1)!='\n') r+="\n";
    r=r.substring(0,r.length()-1)+")\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    return indente()+"(progn)\n";
  }

  public String code(AglInstructionAffectation o)
  {
    String r=indente()+"(setf "+code(o.gauche)+" "+code(o.droite)+")\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=code(o.corps);
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=indente()+"(return-from ";
    if(o.nom!=null) r+=traduit(o.nom);
    else            r+="()";
    r+=" ())\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=indente()+"; catch "+code(o.signal)+"\n";
    r+="#|\n"+code(o.reprise)+"|#\n";
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=indente()+"(continue";
    if(o.nom!=null) r+=" "+o.nom;
    r+=")\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r="";
    if(o.initiale!=null) r+="("+o.nom+" "+code(o.initiale)+") ";
    else r+=o.nom;
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r="";
    r+=indente()+"(let ((b t))\n";
    indentation++;

    r+=indente()+"(while (or b "+code(o.condition)+")\n";
    r+=indente()+"  (setq b ())\n";
    r+=code(o.corps);
    r+=indente()+")\n";

    indentation--;
    r+=indente()+")\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=indente()+code(o.expression)+"\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=indente()+"; finally\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r="";

    if(o.debut instanceof AglInstructionDefVar)
    {
      r+=indente()+"(let* ("+code(o.debut).trim()+")\n";
      indentation++;
    }
    else
      r+=code(o.debut);

    r+=indente()+"(while ";
    if(o.test!=null) r+=code(o.test);
    else             r+="t";
    r+="\n";
    r+=code(o.corps);

    indentation++;
    r+=code(o.fin);
    indentation--;

    r+=indente()+")\n";

    if(o.debut instanceof AglInstructionDefVar)
    {
      indentation--;
      r+=indente()+")\n";
    }

    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=indente()+"(goto "+o.nom;
    r+=")\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=indente()+"(if "+code(o.condition)+"\n";
    r+=code(o.sioui);
    if(o.sinon!=null) r+=code(o.sinon);
    r+=indente()+")\n";
    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=indente()+traduit(o.nom)+"\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=indente()+"("+o.nom;

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=indente()+code(o.valeur)+"\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=indente()+"(select "+code(o.condition)+"\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"("+code(ce)+"\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      r+=indente()+")\n";
    }

    r+=indente()+"(t\n"+code(o.defaut)+indente()+")\n";

    indentation--;
    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=indente()+"(synchronized "+code(o.objet)+"\n";
    indentation++;
    r+=code(o.corps);
    indentation--;
    r+=indente()+")\n";
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=indente()+"(throw 'error \""+code(o.valeur)+"\")\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=indente()+"(catch 'error\n";
    indentation++;
    r+=code(o.corps);
    indentation--;
    r+=indente()+")\n";
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=indente()+"(while "+code(o.condition)+"\n";
    r+=code(o.corps);
    r+=indente()+")\n";
    return r;
  }

// Unite

  /*
  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
  */

  public String code(AglUnite o)
  {
    main_="";

    String r="";
    indentation=0;

    r+=entete(o.fichier);

    r+="\n(unless (boundp 'object) (defclass object () ()))\n";

    r+=code(o.commentaires);

    if(o.paquet!=null) r+="\n"+code(o.paquet)+"\n";

    if(o.imports.size()>0) r+="\n";
    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }

    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglMoule)
      {
        AglMoule ap=(AglMoule)at;
        if(!ap.elementaire)
        {
          r+=code(ap);
          r+="\n";
        }
      }
    }

    AglVecteur simplifications=new AglVecteur(11,11);
    simplifications.addElement("java:lang:");
    for(Enumeration ei=o.imports.elements(); ei.hasMoreElements(); )
    {
      AglImport ai=(AglImport)ei.nextElement();
      AglNom    ni=ai.nom;
      if("*".equals(ni.dernier()))
      {
	ni=ni.parent();
	simplifications.addElement(code(ni)+":");
      }
    }
    String[] ss=new String[simplifications.size()];
    simplifications.copyInto(ss);

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
        {
          String ras=code(as);
	  for(int i=ss.length-1;i>=0;i--)
	    ras=FuLib.replace(ras,ss[i],"");
	  r+=ras;
          r+="\n";
        }
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
        {
          String rac=code(ac);
	  for(int i=ss.length-1;i>=0;i--)
	    rac=FuLib.replace(rac,ss[i],"");
	  r+=rac;
          r+="\n";
        }
      }
    }

    if(!"".equals(main_))
      r+="("+main_+" (make-array 0))\n";

    return r;
  }
}
