/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeLisp.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public  class AglCodeLispOld extends AglCode
{
  AglClasse _classe=null;

  public String extension()
  {
    return "lsp";
  }

  public String[] extensions()
  {
    return new String[] { "lsp", "LSP", "l", "L", "ll", "LL", "lisp", "LISP" };
  }

  protected String debutCS  () { return indente()+";";   }
  protected String finCS    () { return "";              }
  protected String debutCM  () { return indente()+";;";  }
  protected String specialCM() { return ";";             }
  protected String milieuCM () { return indente()+"; ";  }
  protected String finCM    () { return indente()+";;";  }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    if(o.identificateurs.size()>1)
    {
      for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
      {
        if(p) { p=false; r+="#:"; }
        else  r+=":";
        r+=(String)e.nextElement();
      }
    }
    else r+=o.dernier();

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=" ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
    return r;
  }

  public String code(AglAttribut o)
  {
    String r=o.nom;
    return r;
  }

  public String code(AglBloc o)
  {
    indentation++;
    String r="";

    for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
    {
      AglInstruction ai=(AglInstruction)e.nextElement();
      if(ai instanceof AglInstructionDefVar)
      {
	AglInstructionDefVar dv=(AglInstructionDefVar)ai;
	if(dv.initiale!=null) r+="("+dv.nom+" "+code(dv.initiale)+") ";
	else                  r+=dv.nom+" ";
      }
    }

    if(r.equals("")) r=indente()+"(progn\n";
    else             r=indente()+"(let ("+r.trim()+")\n";
    indentation++;

    for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
    {
      AglInstruction ai=(AglInstruction)e.nextElement();
      if(!(ai instanceof AglInstructionDefVar)) r+=code(ai);
    }

    indentation--;
    r+=indente()+")\n";
    indentation--;
    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;

    String r=";;; ";
    r+=code(o.nom)+" ["+o.ligne+"]\n";

    r+="\n(defstruct ";
    r+=o.nom.dernier()+"\n  ";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }
    r+=")\n\n";

    r+="(defabbrev "+o.nom.dernier()+" ";
    r+="#:"+o.nom.dernier();
    r+=")\n\n";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+="(deffield {"+o.nom.dernier()+"} "+code(aa)+")\n";
    }

    indentation=0;
    return r+"\n";
  }

  public String code(AglClasse o)
  {
    _classe=o;
    indentation=0;

    String r=";;; ";
    r+=code(o.nom)+" ["+o.ligne+"]\n";

    if(o.interfaces.noms.size()>0)
      r+=";;; interfaces: "+code(o.interfaces)+"\n";
    if(o.exceptions.noms.size()>0)
      r+=";;; exceptions: "+code(o.exceptions)+"\n";

    r+="\n(defclass ";
    if(o.meres.noms.size()>0) r+="{"+code(o.meres)+"}:";
    r+=o.nom.dernier();

    if(o.attributs.size()>0)
    {
      r+="\n   ";

      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vstatic) r+=" "+code(aa);
      }
    }

    r+=")\n\n";

    r+="(defabbrev "+o.nom.dernier()+" ";
    if(o.meres.noms.size()>0) r+="{"+code(o.meres)+"}:"+o.nom.dernier();
    else r+="#:"+o.nom.dernier();
    r+=")\n\n";

    if(o.attributs.size()>0)
    {
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(aa.visibilite.vstatic)
	  r+="(defvariable {"+o.nom.dernier()+"} "+
	    code(aa)+" "+code(aa.initiale)+")\n";
      }
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vstatic)
	  r+="(deffield {"+o.nom.dernier()+"} "+code(aa)+")\n";
      }
      r+="\n";
    }

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am)+"\n";
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am)+"\n";
    }

    indentation=0;
    _classe=null;

    return r+"\n";
  }

  public String code(AglConstructeur o)
  {
    indentation=0;
    String r="(deffunction "+code(o.nom)+o.parametres.size()+" (";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=" ";
      r+=code(ap);
      p=false;
    }

    r+=")\n";
    if(!o.corps.instructions.isEmpty()) r+=code(o.corps);
    r+=")\n";
    return r;
  }

  public String code(AglImport o)
  {
    indentation=0;
    String r="(loadfile "+code(o.nom)+".ll)\n";
    return r;
  }

  public String code(AglMethode o)
  {
    indentation=0;
    String r="(defmethod ";

    if(_classe!=null) r+="{"+_classe.nom.dernier()+"}:";
    r+=o.nom+" (this";

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      r+=" "+code(ap);
    }

    if(_classe!=null)
    {
      r+=") (";
      boolean p=true;
      for(Enumeration e=_classe.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	if(!aa.visibilite.vstatic)
	{
	  if(!p) r+=" ";
	  r+=code(aa);
	  p=false;
	}
      }
    }

    r+=")\n";

    if(!o.corps.instructions.isEmpty()) r+=code(o.corps);
    r+=")\n";
    return r;
  }

  public String code(AglParametre o)
  {
    String r=o.nom;
    return r;
  }

  public String code(AglVisibilite o)
  {
    return "";
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="(setq "+code(o.gauche)+" "+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+="(sendq "+o.attribut+" "+code(o.objet)+")";
    else              r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r="("+o.operateur+" ";
    r+=code(o.gauche);
    r+=" ";
    r+=code(o.droite);
    r+=")";
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=o.nom;
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r="(isa-p '"+code(o.type)+" "+code(o.expression)+")";
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r="("+o.nom;

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r="(vref "+code(o.objet)+" "+code(o.indice)+")";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="(vector ";

    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="(sendq "+o.fonction.nom+" ";

    if(o.objet!=null) r+=code(o.objet);
    else              r+="this";

    for(Enumeration e=o.fonction.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="(progn";

    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="("+o.type.nom.dernier()+o.arguments.size();

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="(makevector "+code(o.nombre)+")";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r=code(o.expression);
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r="(if "+code(o.gauche)
      +" "+code(o.milieu)
      +" "+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r=code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r="("+o.operateur+" "+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r="("+o.operateur+" "+code(o.gauche)+")";
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=indente()+"(";
    r+=o.body.trim();
    if(r.charAt(r.length()-1)!='\n') r+="\n";
    r=r.substring(0,r.length()-1)+")\n";
    return r;
  }

  public String code(AglInstructionVide o)
    { return indente()+"(progn)\n"; }

  public String code(AglInstructionAffectation o)
  {
    String r=indente()+"(setq "+code(o.gauche)+" "+code(o.droite)+")\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=code(o.corps);
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=indente()+"(break";
    if(o.nom!=null) r+=" "+o.nom;
    r+=")\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=indente()+";;; catch "+code(o.signal)+"\n";
    r+="#|\n"+code(o.reprise)+"|#\n";
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=indente()+"(continue";
    if(o.nom!=null) r+=" "+o.nom;
    r+=")\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r="";
    if(o.initiale!=null) r+="("+o.nom+" "+code(o.initiale)+") ";
    else r+=o.nom;
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r="";
    r+=indente()+"(let ((b t))\n";
    indentation++;

    r+=indente()+"(while (or b "+code(o.condition)+")\n";
    r+=indente()+"  (setq b ())\n";
    r+=code(o.corps);
    r+=indente()+")\n";

    indentation--;
    r+=indente()+")\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=indente()+code(o.expression)+"\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=indente()+";;; finally\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r="";

    if(o.debut instanceof AglInstructionDefVar)
    {
      r+=indente()+"(let ("+code(o.debut).trim()+")\n";
      indentation++;
    }
    else
      r+=code(o.debut);

    r+=indente()+"(while ";
    if(o.test!=null) r+=code(o.test);
    else             r+="t";
    r+="\n";
    r+=code(o.corps);
    r+=code(o.fin);
    r+=indente()+")\n";

    if(o.debut instanceof AglInstructionDefVar)
    {
      indentation--;
      r+=indente()+")\n";
    }

    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=indente()+"(goto "+o.nom;
    r+=")\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=indente()+"(if "+code(o.condition)+"\n";
    r+=code(o.sioui);
    if(o.sinon!=null) r+=code(o.sinon);
    r+=indente()+")\n";
    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=indente()+"'"+o.nom+"\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=indente()+"("+o.nom;

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" "+code(ae);
    }

    r+=")\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=indente()+code(o.valeur)+"\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=indente()+"(select "+code(o.condition)+"\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"("+code(ce)+"\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      r+=indente()+")\n";
    }

    r+=indente()+"(t\n"+code(o.defaut)+indente()+")\n";

    indentation--;
    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=indente()+"(synchronized "+code(o.objet)+"\n";
    indentation++;
    r+=code(o.corps);
    indentation--;
    r+=indente()+")\n";
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=indente()+"(throw 'error \""+code(o.valeur)+"\")\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=indente()+"(catch 'error\n";
    indentation++;
    r+=code(o.corps);
    indentation--;
    r+=indente()+")\n";
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=indente()+"(while "+code(o.condition)+"\n";
    r+=code(o.corps);
    r+=indente()+")\n";
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";

    /*
    for(Enumeration e=AglImport.table.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      System.out.print(code(ai));
    }
    System.out.println("");
    */
        
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
}
