/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeNaturel.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:05 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.text.*;
import java.util.*;

public class AglCodeNaturel extends AglCode
{
  static Hashtable operateurs;

  static
  {
    operateurs=new Hashtable();

    operateurs.put("d+" ,"{0}");
    operateurs.put("d-" ,"l''oppos de {0}");
    operateurs.put("d!" ,"la ngation de {0}");
    operateurs.put("d++","la pr-incrmentation de {0}");
    operateurs.put("d--","la pr-dcrmentation de {0}");
    operateurs.put("g++","la post-incrmentation de {0}");
    operateurs.put("g--","la post-dcrmentation de {0}");

    operateurs.put("b==","{0} est gal a {1}");
    operateurs.put("b!=","{0} est different de {1}");
    operateurs.put("b<" ,"{0} est strictement infrieur  {1}");
    operateurs.put("b>" ,"{0} est strictementsuprieur  {1}");
    operateurs.put("b<=","{0} est infrieur ou egal  {1}");
    operateurs.put("b>=","{0} est suprieur ou egal  {1}");

    operateurs.put("b+" ,"l''addition de {0} et de {1}");
    operateurs.put("b-" ,"la soustraction de {1}  {0}");
    operateurs.put("b*" ,"la multiplication de {0} par {1}");
    operateurs.put("b/" ,"la division de {0} par {1}");
    operateurs.put("b%" ,"le reste de la division de {0} par {1}");

    operateurs.put("b&" ,"{0} et {1}");
    operateurs.put("b|" ,"{0} ou {1}");
    operateurs.put("b^" ,"{0} ou sinon que {1}");
    operateurs.put("b&&","{0} et {1}");
    operateurs.put("b||","{0} ou {1}");
    operateurs.put("b^^","{0} ou sinon que {1}");

    operateurs.put("binstanceof","{0} est un {1}");
    operateurs.put("b<<"        ,"le dcalage a gauche de {0} de {1} positions");
    operateurs.put("b>>"        ,"le dcalage a droite de {0} de {1} positions");
    operateurs.put("b<<<"       ,"le dcalage a gauche de {0} de {1} positions");
    operateurs.put("b>>>"       ,"le dcalage a droite de {0} de {1} positions");
  }

  public String extension()
  {
    return "txt";
  }

  public String[] extensions()
  {
    return new String[] { "txt", "TXT" };
  }

  protected String debutCS  () { return indente(); }
  protected String finCS    () { return "";        }
  protected String debutCM  () { return indente(); }
  protected String specialCM() { return "";        }
  protected String milieuCM () { return indente(); }
  protected String finCM    () { return indente(); }

  protected String remplaceParEt(String _s)
  {
    String r;
    int i=_s.lastIndexOf(", ");
    if(i>=0) r=_s.substring(0,i)+" et "+_s.substring(i+2);
    else     r=_s;
    return r;
  }

  public String code(AglNom o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="/";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    o=AglTypeAlias.simplifie(o);

    String r="";
         if(o==AglTypePrimitif.tboolean) r+="boolen";
    else if(o==AglTypePrimitif.tbyte   ) r+="octet";
    else if(o==AglTypePrimitif.tchar   ) r+="caractre";
    else if(o==AglTypePrimitif.tshort  ) r+="entier court";
    else if(o==AglTypePrimitif.tinteger) r+="entier";
    else if(o==AglTypePrimitif.tlong   ) r+="entier long";
    else if(o==AglTypePrimitif.tfloat  ) r+="rel court";
    else if(o==AglTypePrimitif.tdouble ) r+="rel";
    else if(o==AglType.cherche("String")) r+="chane";
    else
    if(o instanceof AglTypeTableau)
    {
      r+="tableau de "+code(((AglTypeTableau)o).type)+"s";
      if(((AglTypeTableau)o).dimension!=null)
	  r+=" de dimension "+code(((AglTypeTableau)o).dimension);
    }
    else r+=code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    indentation=0;
    String r=indente();

    r+="Soit "+o.nom+", un attribut de type "+code(o.type);
    r+=", "+code(o.visibilite);
    if(o.initiale!=null) r+=", de valeur initiale "+code(o.initiale);
    r+=".\n\n";
    return r;
  }

  public String code(AglBloc o)
  {
    String r="";
    indentation++;

    if(!o.instructions.isEmpty())
      for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
      {
        AglInstruction ai=(AglInstruction)e.nextElement();
        r+=code(ai);
      }
    else r+=indente()+"rien\n";

    indentation--;
    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;
    String r="";

    r+="Soit l'enregistrement "+code(o.nom);
    r+=" "+code(o.visibilite);
    r+=". ";

    r+="Il est dfini  la ligne "+o.ligne+".\n\n";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation=0;
    return r;
  }

  public String code(AglClasse o)
  {
    indentation=0;
    String r="";

    r+="Soit la catgorie "+code(o.nom);

    if(!o.meres.noms.isEmpty())
      r+=" hritant de "+code(o.meres);

    r+=", "+code(o.visibilite);
    r+=".";

    if(!o.interfaces.noms.isEmpty())
      r+=" Elle propose les interfaces "+code(o.interfaces)+".";
    if(!o.exceptions.noms.isEmpty())
      r+=" Elle peut lancer des exceptions "+code(o.exceptions)+".";

    r+=" Elle est dfinie  la ligne "+o.ligne+".\n\n";

    //if(!o.attributs.isEmpty()) r+="\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    //if(!o.constructeurs.isEmpty()) r+="\n";
    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }

    //if(!o.methodes.isEmpty()) r+="\n";
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    indentation=0;
    return r;
  }

  public String code(AglConstructeur o)
  {
    indentation=0;
    String r=indente();

    r+="Soit le constructeur "+code(o.nom);
    r+=", "+code(o.visibilite);

    if(!o.parametres.isEmpty())
    {
      r+=", prenant comme paramtres ";
      boolean p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
        AglParametre ap=(AglParametre)e.nextElement();
        if(!p) r+=", ";
        r+=code(ap);
        p=false;
      }
    }
    else r+=", ne prenant pas de paramtres";

    if(!o.corps.instructions.isEmpty())
    {
      r+=", se droulant ainsi :\n";
      r+=code(o.corps);
      r+="\n";
    }
    else r+=indente()+".\n";

    return r;
  }

  public String code(AglMethode o)
  {
    indentation=0;
    String r=indente();

    r+="Soit la mthode "+o.nom;
    r+=", "+code(o.visibilite);
    r+=" renvoyant un "+code(o.type);

    if(!o.parametres.isEmpty())
    {
      r+=", prenant comme paramtres ";
      boolean p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
        AglParametre ap=(AglParametre)e.nextElement();
        if(!p) r+=", ";
        r+=code(ap);
        p=false;
      }
    }
    else r+=", ne prenant pas de paramtres";

    if(!o.corps.instructions.isEmpty())
    {
      r+=", se droulant ainsi :\n";
      r+=code(o.corps);
      r+="\n";
    }
    else r+=indente()+".\n";

    return r;
  }

  public String code(AglParametre o)
  {
    String r="";
    r+=o.nom+" un "+code(o.type);
    // if(!o.defaut.equals("")) r+=" initalise a "+code(o.defaut);
    return r;
  }

  public String code(AglImport o)
  {
    indentation=0;
    String r="On utilise "+code(o.nom)+".\n";
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";
    if(o.vabstract)     r+="abstrait ";
    if(o.vpublic)       r+="public ";
    if(o.vprivate)      r+="priv ";
    if(o.vprotected)    r+="protg ";
    if(o.vinline)       r+="en-ligne ";
    if(o.vstatic)       r+="statique ";
    if(o.vfinal)        r+="final ";
    if(o.vnative)       r+="natif ";
    if(o.vtransient)    r+="transcendant ";
    if(o.vsynchronized) r+="synchronis ";
    if(r.equals("")) r="normal ";

    if((r.length()>0)&&(r.charAt(r.length()-1)==' ')) r=r.substring(0,r.length()-1);
    r="d'accs "+r;
    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r=code(o.gauche)+" affect de "+code(o.droite);
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="l'attribut "+o.attribut;
    if(o.objet!=null) r+=" de "+code(o.objet);
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r="";
    String t=(String)operateurs.get("b"+o.operateur);

    if(t!=null)
    {
      indentation++;
      String sg=code(o.gauche);
      String sd=code(o.droite);
      indentation--;

      r+=MessageFormat.format(t,new String[] { sg,sd });

      /*
      t=FuLib.replace(t,"{0}",sg);
      t=FuLib.replace(t,"{1}",sd);
      r+=t;
      */
      /*
      int i,j;

      i=t.indexOf("{0}");
      j=t.indexOf("{1}");

      indentation++;
      if(i<j) r+= t.substring(0,i)+code(o.gauche)+t.substring(i+1,j)
                 +code(o.droite)+t.substring(j+1,t.length());
      else    r+= t.substring(0,j)+code(o.droite)+t.substring(j+1,i)
                 +code(o.gauche)+t.substring(i+1,t.length());
      indentation--;
      */
    }
    else
    {
      r+=o.operateur;
      r+=code(o.gauche)+" "+o.operateur+" "+code(o.droite);
    }

    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r="soit "+o.nom+ " un "+code(o.type);
    if(o.initiale!=null) r+=" de valeur initiale "+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r="si "+code(o.expression)+" est un "+code(o.type)+", ";
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r="le rsultat de l'appel de la fonction "+o.nom;

    if(o.arguments.size()==1)
    {
      r+=" avec l'argument ";
      AglExpression ae=(AglExpression)o.arguments.elementAt(0);
      r+=code(ae);
    }
    else
    if(o.arguments.size()>0)
    {
      r+=" avec les arguments ";
      indentation++;
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
        AglExpression ae=(AglExpression)e.nextElement();
        if(p) p=false; else r+=", ";
	r+=code(ae);
      }
      r=remplaceParEt(r);
      indentation--;
    }
    else r+=" sans argument";

    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r="l'lment "+code(o.indice)+" de "+code(o.objet);
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="de valeurs initiales ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r=remplaceParEt(r);
    r+=", ";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="le rsultat de l'appel de la mthode "+o.fonction.nom;
    if(o.objet!=null) r+=" de "+code(o.objet);

    if(o.fonction.arguments.size()==1)
    {
      r+=" avec l'argument ";
      AglExpression ae=(AglExpression)o.fonction.arguments.elementAt(0);
      r+=code(ae);
    }
    else
    if(o.fonction.arguments.size()>0)
    {
      r+=" avec les arguments ";
      indentation++;
      boolean p=true;
      for(Enumeration e=o.fonction.arguments.elements(); e.hasMoreElements(); )
      {
        AglExpression ae=(AglExpression)e.nextElement();
        if(p) p=false; else r+=", ";
	r+=code(ae);
      }

      r=remplaceParEt(r);
      indentation--;
    }
    else r+=" sans argument";

    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="un nouveau "+code(o.type);

    if(o.arguments.size()>0)
    {
      r+=" cr avec les arguments ";
      indentation++;
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
        AglExpression ae=(AglExpression)e.nextElement();
        if(p) p=false; else r+=", ";
	r+=code(ae);
      }
      r=remplaceParEt(r);
      indentation--;
    }

    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="un nouveau "+code(o.type);
    if(o.nombre!=null) r+=" de taille "+code(o.nombre);
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    /*
    String r="( "+code(o.expression)+" )";
    return r;
    */
    return code(o.expression);
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.milieu)
      +" si "+code(o.gauche)
      +" et "+code(o.droite)+" sinon";
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r=code(o.expression)+" considr comme un "+code(o.type);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r="";
    String t=(String)operateurs.get("d"+o.operateur);

    if(t!=null)
    {
      indentation++;
      String sd=code(o.droite);
      indentation--;

      r+=MessageFormat.format(t,new String[] { sd });

      /*
      int i=t.indexOf("{0}");
      t=t.substring(0,i)+code(o.droite)+t.substring(i+3,t.length());
      r+=t;
      */
    }
    else
    {
      r+=o.operateur;
      r+=" "+code(o.droite);
    }

    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r="";
    String t=(String)operateurs.get("g"+o.operateur);
                                                                                           
    if(t!=null)
    {
      indentation++;
      String sg=code(o.gauche);
      indentation--;

      r+=MessageFormat.format(t,new String[] { sg });

      /*
      int i=t.indexOf("{0}");
      t=t.substring(0,i)+code(o.gauche)+t.substring(i+3,t.length());
      r+=t;
      */
    }
    else
    {
      r+=o.operateur;
      r+=" "+code(o.gauche);
    }

    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=indente();
    r+=o.body;
    if(r.charAt(r.length()-1)!='\n') r+="\n";
    r=r.substring(0,r.length()-1)+"\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=indente()+"On ne fait rien.\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=indente()+"On n'affecte  "+code(o.gauche)+" "+code(o.droite)+".\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=indente();
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=indente()+"On sort";
    if(o.nom!=null) r+=" en "+o.nom;
    r+=".\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=indente()+"En cas d'chec, "+code(o.signal);
    r+=" et on excute :\n"+code(o.reprise);
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=indente()+"Continue";
    if(o.nom!=null) r+=" en "+o.nom;
    r+=".\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=indente()+"Soit "+o.nom+", un "+code(o.type);
    if(o.initiale!=null) r+=" de valeur initiale "+code(o.initiale);
    r+=".\n";
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r=indente()+"On rpte\n";
    r+=code(o.corps);
    r+=indente()+"tant que "+code(o.condition)+".\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=indente()+"On ";

    if(o.expression instanceof AglExpressionAffectation)
      r+="affecte  "+FuLib.replace(code(o.expression)," affect de "," ");
    else
      r+="value "+code(o.expression);

    r+=".\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=indente()+"Dans tous les cas,\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r=indente()+"Pour ";
    r+=code(o.debut);
    r+=indente()+"  et tant que "+code(o.test)+", "+code(o.fin)+"\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=indente()+"On va en "+o.nom+".\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=indente()+"Si "+code(o.condition)+",";
    r+=" alors :\n"+code(o.sioui);
    if(o.sinon!=null) r+=indente()+"Sinon "+code(o.sinon);
    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=" "+o.nom+":\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=indente()+"On excute la procdure "+o.nom;

    if(o.arguments.size()==1)
    {
      r+=" avec l'argument ";
      AglExpression ae=(AglExpression)o.arguments.elementAt(0);
      r+=code(ae);
    }
    else
    if(o.arguments.size()>0)
    {
      r+=" avec les arguments ";
      boolean p=true;
      for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
      {
        AglExpression ae=(AglExpression)e.nextElement();
        if(!p) r+=", ";
        r+=code(ae);
        p=false;
      }
    }
    else r+=" sans argument";

    r+=".\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=indente()+"On retourne "+code(o.valeur)+".\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=indente()+"Selon "+code(o.condition)+" :\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"- cas "+code(ce)+" :\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb)+"\n";
    }

    r+=indente()+"- par dfaut :\n";
    r+=code(o.defaut)+"\n";

    indentation--;
    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=indente()+"En synchronizant sur "+code(o.objet)+" :\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=indente()+"On lance "+code(o.valeur)+".\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=indente()+"On essaye :\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=indente()+"Tant que "+code(o.condition)+", on excute :\n";
    r+=code(o.corps);
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";

    /*
    for(Enumeration e=AglImport.table.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      System.out.print(code(ai));
    }
    System.out.println("");
    */
        
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
	{
	  r+=code(ac);
	  r+="\n";
	}
      }
    }

    /*
    r=FuLib.replace(r," que en "    ," qu'en "     );
    r=FuLib.replace(r," de le "     ," du "        );
    r=FuLib.replace(r," de e"       ," d'e"        );
    r=FuLib.replace(r," de o"       ," d'o"        );
    r=FuLib.replace(r," un chane " ," une chane ");
    */

    return r;
  }
}
