/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeOdl.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglCodeOdl extends AglCode
{
  public String extension()
  {
    return "odl";
  }

  public String[] extensions()
  {
    return new String[] { "odl", "ODL" };
  }

  public String code(AglNom o)
  {
    if(o.equals(new AglNom("java.lang.Object"))) return "Object";
    if(o.equals(new AglNom("java.lang.String"))) return "string";

    String r="";
    boolean p=true;

    if(o.identificateurs.size()>1) r+="::";

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="::";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    o=AglTypeAlias.simplifie(o,"Odl");

    String r="";
    if(o instanceof AglTypeTableau)
    {
      AglTypeTableau t=(AglTypeTableau)o;
      if(t.dimension==null) r+="set<"+code(t.type)+">";
      else                  r+=code(t.type)+"["+code(t.dimension)+"]";
    }
    else r+=code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    String r="";
    r+=code(o.visibilite)+"attribute ";
    r+=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+=" ="+code(o.initiale);
    r+=";\n";
    return r;
  }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    indentation=0;
    String r="/* ";
    // if(o.paquet!="") r+=o.paquet;
    r+=code(o.nom)+" ["+o.ligne+"] */\n\n";

    // r+=code(o.visibilite);
    r+="struct "+code(o.nom);

    r+="\n{\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation=0;
    return r+"};\n";
  }

  public String code(AglClasse o)
  {
    String r="// ";
    r+=code(o.nom)+"."+extension()+" ["+o.ligne+"]\n\n";

    // r+=code(o.visibilite);
    r+="class "+code(o.nom)+"\n";
    r+="(     extent "+o.nom.dernier().toLowerCase()+"s)";
    
    /*
      if(o.meres.noms.size()>0)
      r+=" : "+code(o.meres);
    */

    /*
    if(o.interfaces.noms.size()>0)
      r+=" implements "+code(o.interfaces);
    if(o.exceptions.noms.size()>0)
      r+=" throws "+code(o.exceptions);
    */

    r+="\n{\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+="  "+code(aa);
    }
    if((o.attributs.size()>0)&&(o.methodes.size()>0)) r+="\n";

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am)+"\n";
    }
    return r+"};\n";
  }

  public String code(AglConstructeur o)
  {
    String r="  ";
    r+=code(o.visibilite);
    r+="init"+code(o.nom)+o.parametres.size()+"(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=");";
    return r;
  }

  public String code(AglImport o)
  {
    String r="#include <"+o.nom.dernier();
    r+=".odl>\n";
    return r;
  }

  public String code(AglMethode o)
  {
    String r="  ";
    r+=code(o.visibilite);
    r+=code(o.type)+" "+o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }
    r+=")";

    if(o.exceptions.noms.size()>0)
    {
      r+="\n"+indente()+"  raises (";
      r+=code(o.exceptions);
      r+=")";
    }

    r+=";";
    return r;
  }

  public String code(AglParametre o)
  {
    String r="";

    switch(o.passage)
    {
      case AglParametre.IN   : r+="in";    break;
      case AglParametre.OUT  : r+="out";   break;
      case AglParametre.INOUT: r+="inout"; break;
    }

    r+=" "+code(o.type)+" "+o.nom;
    // if(!o.defaut.equals("")) r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vfinal&&o.vfinal) r+="const ";
    if(o.voneway)          r+="oneway ";
    if(o.vreadonly)        r+="readonly ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="new "+code(o.type)+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="new "+code(o.type)+"[";
    r+=code(o.nombre);
    r+="]";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }

  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier,"http://www.odmg.org/");

    /*
    for(Enumeration e=AglImport.table.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }
    */

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglTypeAlias)
      {
        AglTypeAlias aa=(AglTypeAlias)at;
	if(!aa.langage.equals("Odl"))
	  r+="typedef "+code(aa.type)+" "+code(aa.nom)+";\n";
      }
    }
    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
}
