/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodePython.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglCodePython extends AglCode
{
  protected AglClasse classe_;
  protected Hashtable redirect_;

  public AglCodePython()
  {
    TABULATION="\t";
    classe_   =null;
    redirect_ =new Hashtable();
  }

  public String extension()
  {
    return "py";
  }

  public String[] extensions()
  {
    return new String[] { "py", "PY" };
  }

  protected String debutCS  () { return indente()+"# ";  }
  protected String finCS    () { return "";              }
  protected String debutCM  () { return indente()+"###"; }
  protected String specialCM() { return "";              }
  protected String milieuCM () { return indente()+"# ";  }
  protected String finCM    () { return indente()+"###"; }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=".";
      r+=(String)e.nextElement();
    }
    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
         if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau) r+=code(((AglTypeTableau)o).type)+"[]";
    else r+=o.nom.dernier(); // code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    String r=code(o.commentaires);

    r+=indente();
    if((classe_!=null)&&o.visibilite.vstatic) r+=classe_.nom.dernier()+".";
    r+=o.nom+"=";
    if(o.initiale!=null) r+=code(o.initiale);
    else                 r+="None";

    r+=" # "+code(o.type)+"\n";
    return r;
  }

  public String code(AglBloc o)
  {
    indentation++;
    String r=code(o.commentaires);
    
    switch(o.instructions.size())
    {
    case 0:
      r+=indente()+"pass\n";
      break;
    case 1:
      r+=code(o.instructions.elementAt(0));
      break;
    default:
      // int n=1;
      for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
      {
	AglInstruction ai=(AglInstruction)e.nextElement();
	r+=code(ai);
	// n++;
      }
      break;
    }

    indentation--;
    return r;
  }

  public String code(AglStruct o)
  {
    String r="";

    {
      r+=indente()+"# ";
      // if(o.paquet!="") r+=o.paquet;
      r+=code(o.nom)+" ["+o.ligne+"]\n\n";
    }

    // r+=code(o.visibilite);
    r+=indente()+"class "+o.nom.dernier()+":\n";

    indentation++;

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    r+=indente()+"pass\n\n";
    indentation--;

    return r;
  }

  public String code(AglClasse o)
  {
    String r="";

    {
      r+=indente()+"# ";
      // if(o.paquet!="") r+=o.paquet;
      r+=code(o.nom)+" ["+o.ligne+"]\n\n";
    }

    r+=code(o.commentaires);
    // r+=code(o.visibilite);
    r+=indente()+"class "+o.nom.dernier();

    if(o.meres.noms.size()>=1)
    {
      String cm=code(o.meres);
      if(!cm.equals("Object"))
	r+="("+cm+")";
    }

    /*
    if(o.interfaces.noms.size()>0)
      r+=" implements "+code(o.interfaces);
    if(o.exceptions.noms.size()>0)
      r+=" throws "+code(o.exceptions);
    */

    r+=":\n";

    indentation++;
    classe_=o;

    r+=indente()+"# instance attributes\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(!aa.visibilite.vstatic) r+=code(aa);
    }

    if(o.clinit.instructions.size()>0) r+=code(o.clinit);
    if(o.obinit.instructions.size()>0) r+=code(o.obinit); 

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }

    r+=indente()+"# instance methods\n";
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(!am.visibilite.vstatic) r+=code(am);
    }

    r+=indente()+"pass\n\n";
    indentation--;

    r+=indente()+"# static attributes\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(aa.visibilite.vstatic) r+=code(aa);
    }
    r+=indente()+"# static methods\n";
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      if(am.visibilite.vstatic) r+=code(am);
    }

    classe_=null;
    return r;
  }

  public String code(AglConstructeur o)
  {
    String r=code(o.commentaires);

    r+=indente();
    // r+=code(o.visibilite);
    r+="def __init__(self";

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      r+=","+code(ap);
    }

    r+="):\n";

    r+=code(o.corps);

    /*
    if(o.visibilite.vabstract&&o.corps.instructions.isEmpty()) r+=";";
    r+="\n";
    if(!o.corps.instructions.isEmpty())
    {
      if(o.corps.instructions.size()>1) r+=code(o.corps);
      else
	r+=indente()+"{\n"+code(o.corps)+indente()+"}\n";
      r+="\n";
    }
    */

    return r;
  }

  public String code(AglPaquet o)
  {
    String r="# package "+code(o.nom)+"\n";
    return r;
  }

  public String code(AglImport o)
  {
    String r=code(o.nom);
    int    i=r.lastIndexOf('.');
    if(i>=0) r=r.substring(0,i);
    r="import "+r+"\n";
    return r;
  }

  public String code(AglMethode o)
  {
    String r=code(o.commentaires);

    r+=indente();
    // r+=code(o.visibilite);
    r+="def ";

    boolean redirected=false;

    if(!"main".equals(o.nom)&&o.visibilite.vstatic&&classe_!=null)
      { r+=classe_.nom.dernier()+"_"; redirected=true; }

    r+=o.nom+"(";

    if(!o.visibilite.vstatic) r+="self,";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=",";
      p=false;
      r+=code(ap);
    }

    r+="):\n";

    r+=code(o.corps);

    /*
    if(redirected)
    {
      r+=classe_.nom.dernier()+"."+o.nom+"=Redirect"+o.parametres.size()+
	"(\""+classe_.nom.dernier()+"_"+o.nom+"\")\n";
      redirect_.put(new Integer(o.parametres.size()),Boolean.TRUE);
    }
    */

    return r;
  }

  public String code(AglParametre o)
  {
    String r=o.nom;
    // r+=code(o.type)+" "+o.nom;
    // if(o.defaut!="") r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vpublic)       r+="public ";
    if(o.vprivate)      r+="private ";
    if(o.vprotected)    r+="protected ";
    // if(o.vinline)       r+="inline ";
    if(o.vabstract)     r+="abtract ";
    if(o.vstatic)       r+="static ";
    if(o.vfinal)        r+="final ";
    if(o.vnative)       r+="native ";
    if(o.vtransient)    r+="transient ";
    if(o.vsynchronized) r+="synchronized ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    if(classe_!=null)
    {
      AglAttribut a=(AglAttribut)classe_.chercheAttributParNom(o.body);
      if((a!=null)&&(a.visibilite.vstatic))
	r=classe_.nom.dernier()+"."+a.nom;
    }
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=",";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"] ";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=code(o.fonction);
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r=code(o.type)+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=",";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="new "+code(o.type)+"[";
    if(o.nombre!=null) r+=code(o.nombre);
    r+="]";
    if(o.initiale!=null) r+=code(o.initiale);
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="("+code(o.expression)+")";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
     String r=
      code(o.gauche)+o.operateur1+
      code(o.milieu)+o.operateur2+
      code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r=code(o.expression);
    // code(o.type)+"("+code(o.expression)+")";
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r="";
    String op=o.operateur.trim();
    String co=code(o.droite);
         if(op.equals("++")) r+=co+"="+co+"+1";
    else if(op.equals("--")) r+=co+"="+co+"-1";
    else r+=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r="";
    String op=o.operateur.trim();
    String co=code(o.gauche);
         if(op.equals("!")||op.equals("~")) r+="~"+co;
    else if(op.equals("++")) r+=co+"="+co+"+1";
    else if(op.equals("--")) r+=co+"="+co+"-1";
    else r+=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=code(o.commentaires)+indente();
    r+=o.body;
    if((r.length()>0)&&(r.charAt(r.length()-1)!='\n')) r+="\n";
    r=r.substring(0,r.length()-1);
    r+="\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=code(o.commentaires)+indente();
    r+="pass";
    r+="\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=code(o.commentaires)+indente();
    r+=code(o.gauche)+"="+code(o.droite);
    r+="\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=code(o.commentaires);
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=code(o.commentaires)+indente()+"break"; // " "+o.nom;
    r+="\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=code(o.commentaires)+indente();
    r+="except "+code(o.signal)+":\n";
    r+=code(o.reprise);
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=code(o.commentaires)+indente()+"continue"; // " "+o.nom;
    r+="\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=code(o.commentaires);
    if(o.initiale!=null) r+=indente()+o.nom+"="+code(o.initiale)+"\n";
    return r;
  }

  static int loop=0;

  public String code(AglInstructionDoWhile o)
  {
    String r=code(o.commentaires);
    r+=indente()+"_first"+loop+"_=1\n";
    r+=indente()+"while _first"+loop+"_ or ("+code(o.condition)+"):\n";
    indentation++;
    r+=indente()+"_first"+loop+"_=0\n";
    indentation--;
    loop++;
    r+=code(o.corps);
    loop--;
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=code(o.expression);
    if(r.startsWith("(")&&(r.endsWith(")"))) r=r.substring(1,r.length()-1);
    r=code(o.commentaires)+indente()+r;
    r+="\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=code(o.commentaires)+indente()+"finally:\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r=code(o.commentaires);
    if(   o.debut instanceof AglInstructionDefVar
       && o.test instanceof AglExpressionBinaire)
    {
      r+=indente()+"for ";
      AglInstructionDefVar d=(AglInstructionDefVar)o.debut;
      AglExpressionBinaire t=(AglExpressionBinaire)o.test;
      String to="";  
      String downto=""; 
      if(t.operateur.equals("<"))  to=code(t.droite)+"-1";
      if(t.operateur.equals("<=")) to=code(t.droite); 
      if(t.operateur.equals(">"))  downto=code(t.droite)+"+1";
      if(t.operateur.equals(">=")) downto=code(t.droite);

      String sd=code(d.initiale).trim();
      int    ld=sd.length();
      if((ld>0)&&(sd.charAt(ld-1)==';'))
	sd=sd.substring(0,ld-1);
      if((ld>1)&&(sd.charAt(0)=='(')&&(sd.charAt(ld-2)==')'))
	sd=sd.substring(1,ld-2);

      r+=d.nom;
      r+=" in range("+sd.trim()+","+to+"):\n";

      // if(!to.equals("")) r+=" to "+to+" do\n";
      if(!downto.equals("")) r+=" downto "+downto+" do\n";
     	r+=code(o.corps);
    } 
    else
    {
      // int k;
      String t=code(o.test).trim();
      if("".equals(t)) t="1";
      r+=code(o.debut);
      r+=indente()+"while "+t+":\n";
      r+=code(o.corps);
      r+=code(o.fin);
    }
    return r;

    /*
    String r=code(o.commentaires)+indente()+"for(";
    String d=code(o.debut).trim()+" "+code(o.test).trim()+"; ";
    String f="";

    if(o.fin.instructions.size()>1)
      f=code(o.fin).trim();
    else
    if(o.fin.instructions.size()==1)
      f=code(o.fin.instructions.elementAt(0)).trim();

    if(f.endsWith(";")) f=f.substring(0,f.length()-1);

    r+=d+f+")";
    r+="\n";
    r+=code(o.corps);
    return r;
    */
  }

  public String code(AglInstructionGoto o)
  {
    String r=code(o.commentaires)+indente()+"continue"; // " "+o.nom;
    r+="\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=code(o.commentaires)+indente()+"if "+code(o.condition)+":\n";
    r+=code(o.sioui);
    if(o.sinon!=null)
    {
      r+=indente()+"else:\n";
      r+=code(o.sinon);
    }

    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=code(o.commentaires);
    r+=indente()+"# ";
    r+=o.nom+":";
    r+="\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=code(o.commentaires)+indente()+o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=code(o.commentaires);

    AglExpression e=o.valeur;
    while(e instanceof AglExpressionParenthesee)
      e=((AglExpressionParenthesee)e).expression;

    if(e instanceof AglExpressionTernaire)
    {
      AglExpressionTernaire v=(AglExpressionTernaire)e;
      r+=indente()+"if "+code(v.gauche)+": result="+code(v.milieu)+"\n";
      r+=indente()+"else: result="+code(v.droite)+"\n";
      r+=indente()+"return result\n";
      return r;
    }
    else
      r+=indente()+"return "+code(o.valeur)+"\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=code(o.commentaires)+indente()+"switch "+code(o.condition)+":\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"case "+code(ce)+":\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      r+=indente()+"break\n";
    }

    r+=indente()+"default:\n";
    r+=code(o.defaut);
    r+=indente()+"break\n";

    indentation--;

    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=code(o.commentaires);
    r+=indente()+"synchronized "+code(o.objet)+":\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=code(o.commentaires)+indente()+"raise "+code(o.valeur);
    r+="\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=code(o.commentaires)+indente()+"try:\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=code(o.commentaires);
    r+=indente()+"while "+code(o.condition)+":\n";
    r+=code(o.corps);
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier,"http://www.python.org/");

    if(o.paquet!=null) r+=code(o.paquet);

    if(o.imports.size()>0) r+="\n";
    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }

    String s="\n";
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          s+=code(ac);
          s+="\n";
        }
      }
    }

    if(redirect_.size()>0)
    {
      for(Enumeration e=redirect_.keys(); e.hasMoreElements(); )
      {
	int n=((Integer)e.nextElement()).intValue();
	r+="class Redirect"+n+":\n";
	indentation++;
	r+=indente()+"nm=None\n";
	r+=indente()+"def __init__(self,name):\n";
	indentation++;
	r+=indente()+"self.nm=name\n";
	indentation--;
	r+=indente()+"def __call__(self";
	for(int i=1;i<=n;i++)
	  r+=",p"+i;
	r+="):\n";
	indentation++;
	r+=indente()+"exec self.nm+\"(";
	for(int i=1;i<=n;i++)
	{
	  if(i>1) r+=",";
	  r+="p"+i;
	}
	r+=")\"\n";
	indentation--;
	indentation--;
      }

      r+="\n";
    }

    r+=s;
    r+="\n";
    r+="from sys import argv\n";
    r+="main(argv[1:])\n";

    return r;
  }
}
