/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeQualite.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import java.io.*;
import java.util.*;

public class AglCodeQualite extends AglCode
{
  private boolean derniere_;
  private int     CX;

  public String extension()
  {
    return "txt";
  }

  public String[] extensions()
  {
    return new String[] { "txt", "TXT" };
  }

  protected String indente  () { return ""; }
  protected String debutCS  () { return ""; }
  protected String finCS    () { return ""; }
  protected String debutCM  () { return ""; }
  protected String specialCM() { return ""; }
  protected String milieuCM () { return ""; }
  protected String finCM    () { return ""; }

  public String code(AglNom o)
  {
    CX++;
    if(o.identificateurs.size()>1) CX++;
    return "";
  }

  public String code(AglListeNoms o)
  {
    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
      code((AglNom)e.nextElement());
    return "";
  }

  public String code(AglType o)
  {
         if(o instanceof AglTypeAlias)   code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau) { CX++; code(((AglTypeTableau)o).type); }
    else code(o.nom);
    return "";
  }

  public String code(AglAttribut o)
  {
    code(o.visibilite);
    code(o.type);
    if(o.initiale!=null) { CX+=2; code(o.initiale); }
    return "";
  }

  public String code(AglBloc o)
  {
    for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
    {
      AglInstruction ai=(AglInstruction)e.nextElement();
      code(ai);
    }
    return "";
  }

  public String code(AglStruct o)
  {
    int CY=CX;

    code(o.visibilite);
    code(o.nom);

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      code(aa);
    }

    String cx="          "+(CX-CY);
    cx=cx.substring(cx.length()-10);
    return "             |\n"+
	   cx+" S |_ "+o.nom.dernier()+"\n";
  }

  public String code(AglClasse o)
  {
    int CY=CX;
    String r="";

    code(o.visibilite);
    code(o.nom);

    if(o.meres.noms.size()>0)      { CX+=1; code(o.meres);      }
    if(o.interfaces.noms.size()>0) { CX+=2; code(o.interfaces); }
    if(o.exceptions.noms.size()>0) { CX+=3; code(o.exceptions); }

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      code(aa);
    }

    if(o.clinit.instructions.size()>0) { CX+= 6; code(o.clinit); }
    if(o.obinit.instructions.size()>0) { CX+=12; code(o.obinit); }

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur ac=(AglConstructeur)e.nextElement();
      r+=code(ac);
    }
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    String cx="          "+(CX-CY);
    cx=cx.substring(cx.length()-10);
    return "             |\n"+
	   cx+" C |_ "+o.nom.dernier()+"\n"+r;
  }

  public String code(AglConstructeur o)
  {
    int CY=CX;

    code(o.visibilite);
    code(o.nom);

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      code(ap);
    }

    if(!o.corps.instructions.isEmpty()) code(o.corps);

    String cx="          "+(CX-CY);
    cx=cx.substring(cx.length()-10);
    return cx+" N "+(derniere_ ? " " : "|")+"  |_ "+o.nom.dernier()+"()\n";
  }

  public String code(AglImport o)
  {
    CX++;
    return "";
  }

  public String code(AglMethode o)
  {
    int CY=CX;

    code(o.visibilite);
    code(o.type);

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      code(ap);
    }

    if(!o.corps.instructions.isEmpty()) code(o.corps);

    String cx="          "+(CX-CY);
    cx=cx.substring(cx.length()-10);
    return cx+" M "+(derniere_ ? " " : "|")+"  |_ "+o.nom+"()\n";
  }

  public String code(AglParametre o)
  {
    code(o.type);
    return "";
  }

  public String code(AglVisibilite o)
  {
    if(o.vpublic)       ;
    if(o.vprivate)      ;
    if(o.vprotected)    CX++;
    if(o.vinline)       CX+=2;
    if(o.vabstract)     CX++;
    if(o.vstatic)       CX++;
    if(o.vfinal)        CX++;
    if(o.vnative)       CX+=9;
    if(o.vtransient)    CX+=3;
    if(o.vsynchronized) ; //CX+=2;

    return "";
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    CX+=(""+o.body).length()/5+3;
    return "";
  }

  public String code(AglExpressionAffectation o)
  {
    code(o.gauche);
    code(o.droite);
    CX++;
    return "";
  }

  public String code(AglExpressionAttribut o)
  {
    if(o.objet!=null) code(o.objet);
    CX++;
    return "";
  }

  public String code(AglExpressionBinaire o)
  {
    code(o.gauche);
    code(o.droite);
    CX++;
    return "";
  }

  public String code(AglExpressionDefVar o)
  {
    code(o.type);
    CX++;
    if(o.initiale!=null) { CX+=3; code(o.initiale); }
    return "";
  }

  public String code(AglExpressionEstUn o)
  {
    code(o.expression);
    code(o.type);
    CX+=3;
    return "";
  }

  public String code(AglExpressionFonction o)
  {
    CX+=2;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglExpressionIndice o)
  {
    code(o.objet);
    code(o.indice);
    CX++;
    return "";
  }

  public String code(AglExpressionInitArray o)
  {
    CX+=3;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglExpressionMethode o)
  {
    if(o.objet!=null) code(o.objet);
    code(o.fonction);
    return "";
  }

  public String code(AglExpressionMultiple o)
  {
    CX+=3;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglExpressionNew o)
  {
    CX++;
    code(o.type);
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglExpressionNewArray o)
  {
    CX++;
    code(o.type);
    code(o.nombre);
    if(o.initiale!=null) { CX+=2; code(o.initiale); }
    return "";
  }

  public String code(AglExpressionParenthesee o)
  {
    code(o.expression);
    return "";
  }

  public String code(AglExpressionTernaire o)
  {
    CX+=4;
    code(o.gauche);
    code(o.milieu);
    code(o.droite);
    return "";
  }

  public String code(AglExpressionTypage o)
  {
    CX+=2;
    code(o.type);
    code(o.expression);
    return "";
  }

  public String code(AglExpressionUnaireDroite o)
  {
    CX+=2;
    code(o.droite);
    return "";
  }

  public String code(AglExpressionUnaireGauche o)
  {
    CX++;
    code(o.gauche);
    return "";
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    CX+=(""+o.body).length()/5+3;
    return "";
  }

  public String code(AglInstructionVide o)
  {
    CX+=3;
    return "";
  }

  public String code(AglInstructionAffectation o)
  {
    CX++;
    code(o.gauche);
    code(o.droite);
    return "";
  }

  public String code(AglInstructionBloc o)
  {
    code(o.corps);
    return "";
  }

  public String code(AglInstructionBreak o)
  {
    CX+=2;
    if(o.nom!=null) CX+=3;
    return "";
  }

  public String code(AglInstructionCatch o)
  {
    CX+=3;
    code(o.signal);
    code(o.reprise);
    return "";
  }

  public String code(AglInstructionContinue o)
  {
    CX+=3;
    if(o.nom!=null) CX+=3;
    return "";
  }

  public String code(AglInstructionDefVar o)
  {
    CX++;
    code(o.type);
    if(o.initiale!=null) { CX+=2; code(o.initiale); }
    return "";
  }

  public String code(AglInstructionDoWhile o)
  {
    CX+=3;
    code(o.corps);
    code(o.condition);
    return "";
  }

  public String code(AglInstructionExpression o)
  {
    code(o.expression);
    return "";
  }

  public String code(AglInstructionFinally o)
  {
    CX+=7;
    code(o.corps);
    return "";
  }

  public String code(AglInstructionFor o)
  {
    code(o.debut);
    code(o.test);
    code(o.fin);
    code(o.corps);
    return "";
  }

  public String code(AglInstructionGoto o)
  {
    CX+=5;
    return "";
  }

  public String code(AglInstructionIf o)
  {
    CX++;
    code(o.condition);
    code(o.sioui);
    if(o.sinon!=null)
    {
      CX++;
      code(o.sinon);
    }
    return "";
  }

  public String code(AglInstructionLabel o)
  {
    CX+=3;
    return "";
  }

  public String code(AglInstructionProcedure o)
  {
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      code(ae);
    }
    return "";
  }

  public String code(AglInstructionReturn o)
  {
    CX++;
    code(o.valeur);
    return "";
  }

  public String code(AglInstructionSwitch o)
  {
    CX+=5;
    code(o.condition);

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      CX+=2;
      AglExpression ce=(AglExpression)e.nextElement();
      code(ce);
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      code(cb);
    }

    code(o.defaut);
    return "";
  }

  public String code(AglInstructionSynchronized o)
  {
    CX+=5;
    code(o.objet);
    code(o.corps);
    return "";
  }

  public String code(AglInstructionThrow o)
  {
    CX++;
    code(o.valeur);
    return "";
  }

  public String code(AglInstructionTry o)
  {
    CX++;
    code(o.corps);
    return "";
  }

  public String code(AglInstructionWhile o)
  {
    CX++;
    code(o.condition);
    code(o.corps);
    return "";
  }

// Unite

  public String code(AglUnite o)
  {
    int CY=CX;
    String r="";

    r+=entete(o.fichier);
    r+="\n";

    String t="";

    /*
    for(Enumeration e=AglImport.table.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      System.out.print(code(ai));
    }
    System.out.println("");
    */

    AglVecteur v=new AglVecteur();

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
	    v.addElement(as);//t+=code(as);
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((o==at.proprietaire)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
	    v.addElement(ac); //t+=code(ac);
      }
    }

    int l=v.size();
    for(int i=0;i+1<l;i++)
      t+=code(v.elementAt(i));

    if(0<l)
    {
      derniere_=true;
      t+=code(v.elementAt(l-1));
      derniere_=false;
    }

    String f=o.fichier.substring(1+o.fichier.lastIndexOf(File.separator));
    String cx="          "+(CX-CY);
    cx=cx.substring(cx.length()-10);
    r+=cx+" U "+f+"\n"+t;

    return r;
  }
}

