/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeRebol.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;

import java.io.*;
import java.text.*;
import java.util.*;

public class AglCodeRebol extends AglCode
{
  public String extension()
  {
    return "r";
  }

  public String[] extensions()
  {
    return new String[] { "r", "R" };
  }

  protected String debutCS  () { return indente()+"; ";   }
  protected String finCS    () { return "";               }
  protected String debutCM  () { return indente()+";;; "; }
  protected String specialCM() { return "";               }
  protected String milieuCM () { return indente()+"; ";   } 
  protected String finCM    () { return indente()+";;; "; }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="/";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="[";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=" ";
      r+=code((AglNom)e.nextElement());
    }

    r+="]";
    return r;
  }

  public String code(AglType o)
  {
    String r="";
         if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
 //else if(o instanceof AglTypeTableau) r+=code(((AglTypeTableau)o).type)+"[]";
    else r+=code(o.nom);
    return r;
  }

  public String code(AglMoule o)
  {
    return super.code(o);
  }

  public String code(AglAttribut o)
  {
    indentation=1;
    String r=code(o.commentaires);

    r+=indente();
    r+=code(o.visibilite);

    // System.err.println("Nom="+o.nom+", type="+o.type);

    r+=o.nom+": ";//code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+=code(o.initiale);
    else                 r+="none";
    if(LIGNES) r+=" ;-- "+o.ligne;
    r+="\n";

    return r;
  }

  public String code(AglBloc o)
  {
    return code(o,false);
  }

  public String code(AglBloc o, boolean _forced)
  {
    String r=code(o.commentaires);
    
    if(_forced||(o.instructions.size()!=1))
    {
      r+=" [\n";
      indentation++;

      // int n=1;
      for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
      {
	AglInstruction ai=(AglInstruction)e.nextElement();
	r+=code(ai);
	// n++;
      }

      indentation--;
      r+=indente()+"]\n";
    }
    else
    {
      indentation++;
      r+=code(o.instructions.elementAt(0));
      indentation--;
    }

    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;
    String r="";

    if(LIGNES)
    {
      r+=";-- ";
      // if(o.paquet!="") r+=o.paquet;
      r+=code(o.nom)+" ["+o.ligne+"]\n\n";
    }

    //r+=code(o.visibilite);
    //r+="class "+code(o.nom);
    r+=code(o.nom)+": make object! [";

    r+="\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation=0;
    return r+"]\n";
  }

  public String code(AglClasse o)
  {
    indentation=0;
    String r="";

    if(LIGNES)
    {
      r+=";-- ";
      // if(o.paquet!="") r+=o.paquet;
      r+=code(o.nom)+" ["+o.ligne+"]\n\n";
    }

    r+=code(o.commentaires);
    //r+=code(o.visibilite);
    //if(o.normale) r+="class";
    //else          r+="interface";
    //r+=" "+code(o.nom);

    r+=code(o.nom)+": make ";

    if(o.meres.noms.size()>0)
      r+=code(o.meres);
    else
    if(o.interfaces.noms.size()>0)
      r+=code(o.interfaces);
    else
      r+="object!";

    r+=" [\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    r+="\n";
    if(o.clinit.instructions.size()>0) { r+=code(o.clinit); r+="\n"; }
    if(o.obinit.instructions.size()>0) { r+=code(o.obinit); r+="\n"; }

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    indentation=0;
    return r+"]\n";
  }

  public String code(AglConstructeur o)
  {
    indentation=1;
    String r=code(o.commentaires);

    r+=indente();
    r+=code(o.visibilite);

    /*
    if(o.destructeur) r+="void finalize";
    else              r+=code(o.nom);
    */

    if(o.destructeur) r+="create: ";
    else              r+="destroy: ";

    r+="func [";
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=" ";
      r+=code(ap);
      p=false;
    }
    r+="]";

    if(o.corps.instructions.isEmpty()) r+="none";
    if(LIGNES) r+=" ;-- "+o.ligne;
    if(!o.corps.instructions.isEmpty())
    {
      if(o.corps.instructions.size()>1) r+=code(o.corps);
      else
	r+=" [\n"+code(o.corps)+indente()+"]\n";
      r+="\n";
    }

    return r;
  }

  public String code(AglPaquet o)
  {
    indentation=0;
    String r=""; //"package "+code(o.nom)+";\n";
    return r;
  }

  public String code(AglImport o)
  {
    indentation=0;
    String r=""; //"import "+code(o.nom)+";\n";
    return r;
  }

  public String code(AglMethode o)
  {
    indentation=1;
    String r=code(o.commentaires);

    r+=indente();
    //r+=code(o.visibilite);
    //r+=code(o.type)+" "+o.nom+"(";
    r+=o.nom+": func [";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=" ";
      r+=code(ap);
      p=false;
    }

    r+="]";

    if(  // (o.visibilite.vabstract||o.visibilite.vnative)&&
       o.corps.instructions.isEmpty()) r+=" []";
    if(LIGNES) r+=" ;-- "+o.ligne;
    if(!o.corps.instructions.isEmpty())
    {
      if(o.corps.instructions.size()>1) r+=code(o.corps);
      else
	r+=" [\n"+code(o.corps)+indente()+"]\n";
      r+="\n";
    }

    return r;
  }

  public String code(AglParametre o)
  {
    String r="";
    //r+=code(o.type)+" "+o.nom;
    r+=o.nom;
    // if(o.defaut!="") r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    /*
    if(o.vpublic)       r+="public ";
    if(o.vprivate)      r+="private ";
    if(o.vprotected)    r+="protected ";
    // if(o.vinline)       r+="inline ";
    if(o.vabstract)     r+="abtract ";
    if(o.vstatic)       r+="static ";
    if(o.vfinal)        r+="final ";
    if(o.vnative)       r+="native ";
    if(o.vtransient)    r+="transient ";
    if(o.vsynchronized) r+="synchronized ";
    */

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="[";

    if(  (o.droite instanceof AglExpressionBinaire)
       &&(((AglExpressionBinaire)o.droite).gauche==o.gauche))
      r+=code(o.gauche)+((AglExpressionBinaire)o.droite).operateur+
	": "+code(((AglExpressionBinaire)o.droite).droite);
    else
      r+=code(o.gauche)+": "+code(o.droite);

    r+="]";
    if(LIGNES) r+=" ;-- Affectation "+o.ligne;
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+"/";
    if(LIGNES) r+=" ;-- Attribut "+o.ligne;
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r="["+code(o.gauche)+" "+o.operateur+" "+code(o.droite)+"]";
    if(LIGNES) r+=" ;-- Binaire "+o.ligne;
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=o.nom+": ";
    if(o.initiale!=null) r+=code(o.initiale);
    else                 r+="none";
    if(LIGNES) r+=" ;-- Var "+o.ligne;
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    if(LIGNES) r+=" ;-- EstUn "+o.ligne;
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom;

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" ";
      r+=code(ae);
    }

    if(LIGNES) r+=" ;-- Fonction "+o.ligne;
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"/"+code(o.indice);
    if(LIGNES) r+=" ;-- Indice "+o.ligne;
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="[";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=" ";
      r+=code(ae);
      p=false;
    }

    r+="]";
    if(LIGNES) r+=" ;-- InitArray "+o.ligne;
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+"/";
    if(LIGNES) r+=" ;-- Methode "+o.ligne;
    r+=code(o.fonction);
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";
    if(LIGNES) r+=" ;-- Multiple "+o.ligne;

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=" ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r=code(o.type)+"/create [";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=" ";
      r+=code(ae);
      p=false;
    }

    r+="]";
    if(LIGNES) r+=" ;-- New "+o.ligne;
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r=code(o.type)+"/create [";
    if(o.nombre!=null) r+=code(o.nombre);
    r+="]";
    if(LIGNES) r+=" ;-- New "+o.ligne;
    if(o.initiale!=null) r+=code(o.initiale);
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="["+code(o.expression)+"]";
    if(LIGNES) r+=" ;-- Parenthesee "+o.ligne;
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)+" "
      +o.operateur1+" "+code(o.milieu)+" "
      +o.operateur2+" "+code(o.droite);
    if(LIGNES) r+=" ;-- Ternaire "+o.ligne;
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r=/*"("+code(o.type)+")"+*/code(o.expression);
    if(LIGNES) r+=" ;-- Typage "+o.ligne;
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    if(LIGNES) r+=" ;-- UnaireDroite "+o.ligne;
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    if(LIGNES) r+=" ;-- UnaireGauche "+o.ligne;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=code(o.commentaires)+indente();
    r+=o.body;
    if((r.length()>0)&&(r.charAt(r.length()-1)!='\n')) r+="\n";
    r=r.substring(0,r.length()-1);
    if(LIGNES) r+=" ;-- Instruction "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=code(o.commentaires)+indente();
    r+="none";
    if(LIGNES) r+=" ;-- Vide "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=code(o.commentaires)+indente();

    if(  (o.droite instanceof AglExpressionBinaire)
       &&(((AglExpressionBinaire)o.droite).gauche==o.gauche))
      r+=code(o.gauche)+((AglExpressionBinaire)o.droite).operateur+
	": "+code(((AglExpressionBinaire)o.droite).droite);
    else
      r+=code(o.gauche)+": "+code(o.droite);

    if(LIGNES) r+=" ;-- Affectation "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+";-- Bloc "+o.ligne+"\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=code(o.commentaires)+indente()+"break";
    if(o.nom!=null) r+=" "+o.nom;
    if(LIGNES) r+=" ;-- Break "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=code(o.commentaires)+indente();
    r+="catch "+code(o.signal);
    if(LIGNES) r+=" ;-- Catch "+o.ligne;
    r+="\n";
    r+=code(o.reprise,true);
    return r;
  }

  public String code(AglInstructionCondition o)
  {
    String r=code(o.commentaires)+indente()+
      "assert "+code(o.condition.test);
    r+=";";
    if(LIGNES) r+=" ;-- Condition "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=code(o.commentaires)+indente()+"continue";
    if(o.nom!=null) r+=" "+o.nom;
    if(LIGNES) r+=" ;-- Continue "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=code(o.commentaires)+indente()+o.nom+": ";
    if(o.initiale!=null) r+=code(o.initiale);
    else                 r+="none";
    if(LIGNES) r+=" ;-- DefVar "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r=code(o.commentaires)+indente()+"do";
    if(LIGNES) r+=" ;-- DoWhile "+o.ligne;
    r+="\n";
    r+=code(o.corps);
    r+=indente()+"while "+code(o.condition);
    if(LIGNES) r+=" ;-- DoWhile "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=code(o.expression);
    if(r.startsWith("[")&&(r.endsWith("]"))) r=r.substring(1,r.length()-1);
    r=code(o.commentaires)+indente()+r;
    if(LIGNES) r+=" ;-- InsExpr "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=code(o.commentaires)+indente()+"finally";
    if(LIGNES) r+=" ;-- Finally "+o.ligne;
    r+="\n";
    r+=code(o.corps,true);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r=code(o.commentaires)+indente()+"foreach ";
    String d=code(o.debut).trim();
    String t=code(o.test).trim();
    String f=code(o.fin).trim();

    r+=d+" "+t+" "+f+" ";
    if(LIGNES) r+=" ;-- For "+o.ligne;
    r+="\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=code(o.commentaires)+indente()+"continue "+o.nom;
    if(LIGNES) r+=" ;-- Goto "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=code(o.commentaires)+indente()+"if "+code(o.condition)+" ";
    if(LIGNES) r+=" ;-- If "+o.ligne;
    r+="\n";
    r+=code(o.sioui);
    if(o.sinon!=null)
    {
      r+=indente()+"else";
      if(LIGNES) r+=" ;-- Else "+o.ligne;
      r+="\n";
      r+=code(o.sinon);
    }

    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=code(o.commentaires);
    indentation--;
    r+=indente()+" ";
    indentation++;
    r+=o.nom+"::";
    if(LIGNES) r+=" ;-- Label "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=code(o.commentaires)+indente()+o.nom+"[";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=" ";
      r+=code(ae);
      p=false;
    }

    r+="]";
    if(LIGNES) r+=" ;-- Procedure "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=code(o.commentaires)+indente()+"return ";
    if(o.valeur!=null) r+=code(o.valeur);
    else               r+="none";
    if(LIGNES) r+=" ;-- Return "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=code(o.commentaires)+indente()+"switch "+code(o.condition);
    if(LIGNES) r+=" ;-- Switch "+o.ligne;
    r+="\n";
    r+=indente()+"[\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"case "+code(ce)+":\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      r+=indente()+"break;\n";
    }

    r+=indente()+"default:\n";
    r+=code(o.defaut);
    // r+=indente()+"break;\n";

    indentation--;
    r+=indente()+"]\n";

    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=code(o.commentaires)+indente()+"synchronized "+code(o.objet);
    if(LIGNES) r+=" ;-- Synchronized "+o.ligne;
    r+="\n";
    r+=code(o.corps,true);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=code(o.commentaires)+indente()+"throw "+code(o.valeur);
    r+=";";
    if(LIGNES) r+=" ;-- Throw "+o.ligne;
    r+="\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=code(o.commentaires)+indente()+"try";
    if(LIGNES) r+=" ;-- Try "+o.ligne;
    r+="\n";
    r+=code(o.corps,true);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=code(o.commentaires)+indente()+"while "+code(o.condition);
    if(LIGNES) r+=" ;-- While "+o.ligne;
    r+="\n";
    r+=code(o.corps);
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    String date=new SimpleDateFormat
      ("yyyy-MM-dd",Locale.getDefault()).format(new Date());

    r+="REBOL [\n";
    r+="    Title: %"+new File(o.fichier).getName()+"\n";
    r+="    Date: "+date+"\n";
    r+="]\n\n";

    r+=entete(o.fichier,"http://www.rebol.com/");

    if(o.paquet!=null) r+="\n"+code(o.paquet);

    if(o.imports.size()>0) r+="\n";
    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }

    r+="\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglMoule)
      {
        AglMoule ap=(AglMoule)at;
        if(!ap.elementaire)
        {
          r+=code(ap);
          r+="\n";
        }
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglStruct)
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire)
        {
          r+=code(as);
          r+="\n";
        }
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
}
