/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeResume.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglCodeResume extends AglCode
{
  private String formate(String _s, int _t)
  {
    String r=_s;
    while(r.length()<_t) r+=' ';
    return r.substring(0,_t);
  }

  public String extension()
  {
    return "txt";
  }

  public String[] extensions()
  {
    return new String[] { "txt", "TXT" };
  }

  protected String debutCS  () { return "";        }
  protected String finCS    () { return "";        }
  protected String debutCM  () { return indente(); }
  protected String specialCM() { return "";        }
  protected String milieuCM () { return indente(); }
  protected String finCM    () { return "";        }

  public String code(AglNom o)
  {
    // if(o.equals(new AglNom("java.lang.Object"))) return "Object";
    // if(o.equals(new AglNom("java.lang.String"))) return "string";

    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=".";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
    if(o instanceof AglTypeAlias)
    {
      AglTypeAlias a=(AglTypeAlias)o;
      r+=code(a.type);
    }
    else if(o instanceof AglTypeTableau)
    {
      AglTypeTableau t=(AglTypeTableau)o;
      if(t.dimension==null) r+=code(t.type)+"[]";
      else                  r+=code(t.type)+"["+code(t.dimension)+"]";
    }
    else r+=o.nom.dernier(); // code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    String r=indente();
    r+=formate(o.nom             ,17)+" ";
    r+=formate(code(o.type)      ,24)+" ";
    r+=formate(code(o.visibilite),14)+" ";

    if(o.initiale!=null) r+=formate("="+code(o.initiale),38);
    r+="\n";
    return r;
  }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    String r=indente()+"Structure "+o.nom.dernier()+"\n";

    r+=indente()+"  ";
    r+=formate(code(o.nom)       ,44)+" ";
    r+=formate(code(o.visibilite),14)+" ";
    r+="\n\n";

    indentation++;
    if(o.attributs.size()>0) r+=indente()+"Attributs     \n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+="  "+code(aa);
    }
    indentation--;

    return r;
  }

  public String code(AglClasse o)
  {
    String r=indente()+"Classe "+o.nom.dernier()+"\n";

    r+=indente()+"  ";
    r+=formate(code(o.nom)       ,44)+" ";
    r+=formate(code(o.visibilite),14)+" ";
    r+="\n\n";

    indentation++;

    if(o.meres.noms.size()>0)
      r+=indente()+"Mres               "+code(o.meres)+"\n";
    if(o.interfaces.noms.size()>0)
      r+=indente()+"Interfaces          "+code(o.interfaces)+"\n";
    if(o.exceptions.noms.size()>0)
      r+=indente()+"Exceptions          "+code(o.exceptions)+"\n";

    if(o.attributs.size()>0) r+="\n"+indente()+"Attributs     \n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+="  "+code(aa);
    }

    if(o.constructeurs.size()>0) r+="\n"+indente()+"Constructeurs \n";
    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur ac=(AglConstructeur)e.nextElement();
      r+="  "+code(ac);
    }

    if(o.methodes.size()>0) r+="\n"+indente()+"Mthodes      \n";
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+="  "+code(am);
    }

    indentation--;

    return r;
  }

  public String code(AglConstructeur o)
  {
    String r=indente();

    r+=formate(code(o.nom)       ,17)+" ";
    r+=formate(""                ,24)+" ";
    r+=formate(code(o.visibilite),14)+" ";

    String s="(";
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) s+=",";
      s+=code(ap);
      p=false;
    }
    s+=")";
    r+=formate(s,38)+"\n";
    return r;
  }

  public String code(AglImport o)
  {
    return indente()+code(o.nom)+"\n";
  }

  public String code(AglMethode o)
  {
    String r=indente();

    r+=formate(o.nom             ,17)+" ";
    r+=formate(code(o.type)      ,24)+" ";
    r+=formate(code(o.visibilite),14)+" ";

    String s="(";
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) s+=",";
      s+=code(ap);
      p=false;
    }
    s+=")";
    r+=formate(s,38)+"\n";
    return r;
  }

  public String code(AglParametre o)
  {
    String r="";

    /*
    switch(o.passage)
    {
      case AglParametre.IN   : r+="in";    break;
      case AglParametre.OUT  : r+="out";   break;
      case AglParametre.INOUT: r+="inout"; break;
    }
    */

    r+=code(o.type);
    // " "+o.nom;
    // if(!o.defaut.equals("")) r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vabstract)        r+="A ";
    if(o.vfinal)           r+="F ";
    if(o.vnative)          r+="N ";
    if(o.vprivate)         r+="- ";
    if(o.vprotected)       r+="# ";
    if(o.vpublic)          r+="+ ";
    if(o.vsynchronized)    r+="S ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="new "+code(o.type)+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="new "+code(o.type)+"[";
    r+=code(o.nombre);
    r+="]";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }

  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);

    if(o.imports.size()>0) r+=indente()+"Imports\n";
    indentation++;
    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }
    indentation--;

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglStruct))
      {
        AglStruct as=(AglStruct)at;
        if(!as.elementaire&&filtre(as.nom))
	  r+="\n"+code(as)+"\n";
      }
    }

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if((at.proprietaire==o)&&(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire&&filtre(ac.nom))
          r+="\n"+code(ac)+"\n";
      }
    }

    return r;
  }
}
