/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeRuby.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
// Auteur : Jan Arne Petersen
// Mel    : jpetersen@uni-bonn.de
 * @license      GNU General Public License 2 (GPL2)
//          Copyright (c)2001 Jan Arne Petersen
 */

package com.memoire.agl;
import java.util.*;

public class AglCodeRuby extends AglCode {
	
    /**
     * @see AglCode#extensions()
     */
    public String[] extensions() {
        return new String[] { "rb", "RB" };
    }

    /**
     * @see AglCode#extension()
     */
    public String extension() {
        return "rb";
    }

    protected String debutCS() {
        return indente() + "# ";
    }

    protected String finCS() {
        return "";
    }

    protected String debutCM() {
        return indente() + "#";
    }

    protected String specialCM() {
        return "";
    }

    protected String milieuCM() {
        return indente() + "# ";
    }

    protected String finCM() {
        return indente() + "#";
    }

    /**
     * @see AglCode#code(AglInstructionWhile)
     */
    public String code(AglInstructionWhile o) {
        return code(o.commentaires)
            + indente()
            + "while "
            + code(o.condition)
            + "\n"
            + code(o.corps);
    }

    /**
     * @see AglCode#code(AglInstructionTry)
     */
    public String code(AglInstructionTry o) {
        return code(o.commentaires) + indente() + "begin\n" + code(o.corps, false);
    }

    /**
     * @see AglCode#code(AglInstructionThrow)
     */
    public String code(AglInstructionThrow o) {
        return code(o.commentaires) + indente() + "raise " + code(o.valeur) + "\n";
    }

    /**
     * @see AglCode#code(AglInstructionSynchronized)
     */
    public String code(AglInstructionSynchronized o) {
        return "";
    }

    /**
     * @see AglCode#code(AglInstructionSwitch)
     */
    public String code(AglInstructionSwitch o) {
        String r =
            code(o.commentaires) + indente() + "case " + code(o.condition) + "\n";
        indentation++;

        for (Enumeration e = o.cas.keys(); e.hasMoreElements();) {
            AglExpression ce = (AglExpression) e.nextElement();
            r += indente() + "when " + code(ce) + "\n";
            AglBloc cb = (AglBloc) (o.cas.get(ce));
            r += code(cb, false);
        }

        if (o.defaut != null) {
            r += indente() + "else\n";
            r += code(o.defaut, false);
        }

        indentation--;
        r += indente() + "end\n";

        return r;
    }

    /**
     * @see AglCode#code(AglInstructionReturn)
     */
    public String code(AglInstructionReturn o) {
        return code(o.commentaires)
            + indente()
            + "return"
            + ((o.valeur != null) ? " " + code(o.valeur) : "")
            + "\n";
    }

    /**
     * @see AglCode#code(AglInstructionProcedure)
     */
    public String code(AglInstructionProcedure o) {
        String r = code(o.commentaires) + indente() + o.nom;

        if (o.arguments.size() > 0) {
            r += "(";

            boolean p = true;
            for (Enumeration e = o.arguments.elements(); e.hasMoreElements();) {
                AglExpression ae = (AglExpression) e.nextElement();
                if (!p) {
                    r += ", ";
                }
                r += code(ae);
                p = false;
            }

            r += ")";
        }
        r += "\n";
        return r;
    }

    /**
     * @see AglCode#code(AglInstructionLabel)
     */
    public String code(AglInstructionLabel o) {
        return code(o.commentaires) + "# Label: " + o.nom + "\n";
    }

    /**
    * @see AglCode#code(AglInstructionIf)
    */
    public String code(AglInstructionIf o) {
        return code(o, false);
    }
    /**
     * @see AglCode#code(AglInstructionIf)
     */
    public String code(AglInstructionIf o, boolean elsif) {
        String r =
            (!elsif ? code(o.commentaires) + indente() + "if " : "")
                + code(o.condition)
                + "\n";
        r += code(o.sioui, false);
        if (o.sinon != null) {
            r += indente();

            // test pour aligner les if/else
            boolean a = false;
            if ((o.sinon.instructions.size() == 1)
                && (o.sinon.instructions.elementAt(0) instanceof AglInstructionIf)) {
                r += "elsif ";
                indentation--;
                code((AglInstructionIf) o.sinon.instructions.elementAt(0), false);
                indentation++;
            } else {
                r += "else\n";
                indentation--;
                code(o.sinon, false);
                indentation++;
            }
        }
        if (!elsif) {
            r += indente() + "end\n";
        }
        return r;
    }

    /**
     * @see AglCode#code(AglInstructionGoto)
     */
    public String code(AglInstructionGoto o) {
        return "";
    }

    /**
     * @see AglCode#code(AglInstructionFor)
     */
    public String code(AglInstructionFor o) {
        String r = code(o.commentaires) + indente() + "begin\n";
        indentation++;
        r += indente() + "while " + code(o.test) + "\n";
        r += code(o.corps, false);
        r += code(o.debut);
        r += indente() + "end\n";
        indentation--;
        r += indente() + "end\n";
        return r;
    }

    /**
     * @see AglCode#code(AglInstructionFinally)
     */
    public String code(AglInstructionFinally o) {
        return "";
    }

    /**
     * @see AglCode#code(AglInstructionExpression)
     */
    public String code(AglInstructionExpression o) {
        String exp = code(o.expression);

        if (exp.startsWith("(") && (exp.endsWith(")"))) {
            exp = exp.substring(1, exp.length() - 1);
        }

        return code(o.commentaires) + indente() + exp + "\n";
    }

    /**
     * @see AglCode#code(AglInstructionDoWhile)
     */
    public String code(AglInstructionDoWhile o) {
        return "";
    }

    /**
     * @see AglCode#code(AglInstructionDefVar)
     */
    public String code(AglInstructionDefVar o) {
        if (o.initiale != null) {
            return code(o.commentaires) + indente() + o.nom + "=" + code(o.initiale) + "\n";
        } else {
            return code(o.commentaires);
        }
    }

    /**
     * @see AglCode#code(AglInstructionContinue)
     */
    public String code(AglInstructionContinue o) {
        return code(o.commentaires) + indente() + "continue\n";
    }

    /**
     * @see AglCode#code(AglInstructionCatch)
     */
    public String code(AglInstructionCatch o) {
        return "";
    }

    /**
     * @see AglCode#code(AglInstructionBreak)
     */
    public String code(AglInstructionBreak o) {
        return code(o.commentaires) + indente() + "break\n";
    }

    /**
     * @see AglCode#code(AglInstructionBloc)
     */
    public String code(AglInstructionBloc o) {
        String r = code(o.commentaires);
        if (!o.virgule) {
            r += code(o.corps);
        } else {
            boolean p = true;
            for (Enumeration e = o.corps.instructions.elements(); e.hasMoreElements();) {
                if (!p)
                    r += ",";
                AglInstruction ai = (AglInstruction) e.nextElement();
                String s = code(ai).trim();
                int i = s.lastIndexOf("\n");
                s = s.substring(0, i) + s.substring(i + 1);
                if ((!p) && (ai instanceof AglInstructionDefVar)) {
                    i = s.indexOf(" ") + 1;
                    s = s.substring(i);
                }
                r += s;
                p = false;
            }
            r += "\n";
        }
        return r;
    }

    /**
     * @see AglCode#code(AglInstructionAffectation)
     */
    public String code(AglInstructionAffectation o) {
        String r = code(o.commentaires) + indente();

        if ((o.droite instanceof AglExpressionBinaire)
            && (((AglExpressionBinaire) o.droite).gauche == o.gauche)) {
            r += code(o.gauche)
                + ((AglExpressionBinaire) o.droite).operateur
                + "="
                + code(((AglExpressionBinaire) o.droite).droite);
        } else {
            r += code(o.gauche) + "=" + code(o.droite);
        }
        r += "\n";
        return r;
    }

    /**
     * @see AglCode#code(AglInstructionVide)
     */
    public String code(AglInstructionVide o) {
        return "";
    }

    /**
     * @see AglCode#code(AglInstructionGenerique)
     */
    public String code(AglInstructionGenerique o) {
        String r = code(o.commentaires) + indente() + o.body;
        //	if((r.length()>0)&&(r.charAt(r.length()-1)!='\n')) r+="\n";
        //    r=r.substring(0,r.length()-1);
        r += "\n";
        return r;
    }

    /**
     * @see AglCode#code(AglExpressionUnaireGauche)
     */
    public String code(AglExpressionUnaireGauche o) {
        return code(o.gauche) + o.operateur;
    }

    /**
     * @see AglCode#code(AglExpressionUnaireDroite)
     */
    public String code(AglExpressionUnaireDroite o) {
        return o.operateur + code(o.droite);
    }

    /**
     * @see AglCode#code(AglExpressionTypage)
     */
    public String code(AglExpressionTypage o) {
        return code(o.expression);
    }

    /**
     * @see AglCode#code(AglExpressionTernaire)
     */
    public String code(AglExpressionTernaire o) {
        return code(o.gauche)
            + " "
            + o.operateur1
            + " "
            + code(o.milieu)
            + " "
            + o.operateur2
            + " "
            + code(o.droite);
    }

    /**
     * @see AglCode#code(AglExpressionParenthesee)
     */
    public String code(AglExpressionParenthesee o) {
        return "(" + code(o.expression) + ")";
    }

    /**
     * @see AglCode#code(AglExpressionNewArray)
     */
    public String code(AglExpressionNewArray o) {
        if (o.initiale != null) {
            return " = " + code(o.initiale);
        } else if (o.nombre != null) {
            return "[nil] * " + code(o.nombre);
        } else {
            return "[]";
        }
    }

    /**
     * @see AglCode#code(AglExpressionNew)
     */
    public String code(AglExpressionNew o) {
        String r = code(o.type) + ".new(";

        boolean p = true;
        for (Enumeration e = o.arguments.elements(); e.hasMoreElements();) {
            AglExpression ae = (AglExpression) e.nextElement();
            if (!p) {
                r += ",";
            }
            r += code(ae);
            p = false;
        }

        r += ")";
        return r;
    }

    /**
     * @see AglCode#code(AglExpressionMultiple)
     */
    public String code(AglExpressionMultiple o) {
        String r = "";

        boolean p = true;
        for (Enumeration e = o.expressions.elements(); e.hasMoreElements();) {
            AglExpression ae = (AglExpression) e.nextElement();
            if (!p) {
                r += ", ";
            }
            r += code(ae);
            p = false;
        }

        return r;
    }

    /**
     * @see AglCode#code(AglExpressionMethode)
     */
    public String code(AglExpressionMethode o) {
        String r = "";
        if (o.objet != null) {
            r += code(o.objet) + ".";
        }
        r += code(o.fonction);
        return r;
    }

    /**
     * @see AglCode#code(AglExpressionInitArray)
     */
    public String code(AglExpressionInitArray o) {
        String r = "[ ";

        boolean p = true;
        for (Enumeration e = o.initiales.elements(); e.hasMoreElements();) {
            AglExpression ae = (AglExpression) e.nextElement();
            if (!p) {
                r += ", ";
            }
            r += code(ae);
            p = false;
        }

        r += " ]";
        return r;
    }

    /**
     * @see AglCode#code(AglExpressionIndice)
     */
    public String code(AglExpressionIndice o) {
        return code(o.objet) + "[" + code(o.indice) + "]";
    }

    /**
     * @see AglCode#code(AglExpressionFonction)
     */
    public String code(AglExpressionFonction o) {
        String r = o.nom + "(";

        boolean p = true;
        for (Enumeration e = o.arguments.elements(); e.hasMoreElements();) {
            AglExpression ae = (AglExpression) e.nextElement();
            if (!p) {
                r += ",";
            }
            r += code(ae);
            p = false;
        }

        r += ")";
        return r;
    }

    /**
     * @see AglCode#code(AglExpressionEstUn)
     */
    public String code(AglExpressionEstUn o) {
        return code(o.expression) + ".kind_of? " + code(o.type);
    }

    /**
     * @see AglCode#code(AglExpressionDefVar)
     */
    public String code(AglExpressionDefVar o) {
        if (o.initiale != null) {
            String nom = o.nom;
            if (nom.startsWith("this.")) {
                nom = "@" + nom.substring(5);
            }
            return nom + " = " + code(o.initiale);
        } else {
            return "";
        }
    }

    /**
     * @see AglCode#code(AglExpressionBinaire)
     */
    public String code(AglExpressionBinaire o) {
        return code(o.gauche) + o.operateur + code(o.droite);
    }

    /**
     * @see AglCode#code(AglExpressionAttribut)
     */
    public String code(AglExpressionAttribut o) {
        if (o.objet != null) {
            return code(o.objet) + "." + o.attribut;
        } else {
            return "@" + o.attribut;
        }
    }

    /**
     * @see AglCode#code(AglExpressionAffectation)
     */
    public String code(AglExpressionAffectation o) {
        String r = "(";

        if ((o.droite instanceof AglExpressionBinaire)
            && (((AglExpressionBinaire) o.droite).gauche == o.gauche)) {
            String gauche = code(o.gauche);
            if (gauche.startsWith("this.")) {
                gauche = "@" + gauche.substring(5);
            }
            r += gauche
                + ((AglExpressionBinaire) o.droite).operateur
                + "="
                + code(((AglExpressionBinaire) o.droite).droite);
        } else {
            String gauche = code(o.gauche);
            if (gauche.startsWith("this.")) {
                gauche = "@" + gauche.substring(5);
            }
            r += gauche + "=" + code(o.droite);
        }
        r += ")";

        return r;
    }

    /**
     * @see AglCode#code(AglExpressionGenerique)
     */
    public String code(AglExpressionGenerique o) {
        return o.body;
    }

    /**
     * @see AglCode#code(AglVisibilite)
     */
    public String code(AglVisibilite o) {
        if (o.vpublic) {
            return ""; //public\n";
        } else if (o.vprivate) {
            return "private\n";
        } else if (o.vprotected) {
            return "protected\n";
        } else {
            return "";
        }
    }

    /**
     * @see AglCode#code(AglType)
     */
    public String code(AglType o) {
        return code(o.nom);
    }

    /**
     * @see AglCode#code(AglStruct)
     */
    public String code(AglStruct o) {
        String r = code(o.commentaires);
        r += o.nom.dernier() + " = Struct.new( \"" + o.nom.dernier() + "\"";
        for (Enumeration e = o.attributs.elements(); e.hasMoreElements();) {
            AglAttribut aa = (AglAttribut) e.nextElement();
            r += ", :" + aa.nom;
        }

        r += ")\n";
        return r;
    }

    /**
     * @see AglCode#code(AglParametre)
     */
    public String code(AglParametre o) {
        return o.nom;
    }

    /**
     * @see AglCode#code(AglMethode)
     */
    public String code(AglMethode o) {
        if((o.proprietaire instanceof AglClasse)
	   &&!((AglClasse)o.proprietaire).normale) {
            String r = code(o.commentaires);

            r += indente() + "def " + o.nom + " (";

            boolean p = true;
            for (Enumeration e = o.parametres.elements(); e.hasMoreElements();) {
                AglParametre ap = (AglParametre) e.nextElement();
                if (!p) {
                    r += ", ";
                }
                r += code(ap);
                p = false;
            }

            r += ")\n";

            indentation++;
            r += indente()
                + "raise \"You have to override the abstract method \\\""
                + o.nom
                + "\\\"\"\n";
            indentation--;

            r += indente() + "end\n";

            return r;
        } else if (
            o.nom.startsWith("get")
                && o.parametres.size() == 0
                && o.corps.instructions.size() == 1) {
            return indente()
                + "attr_reader :"
                + o.nom.substring(3, 4).toLowerCase()
                + o.nom.substring(4)
                + "\n";
        } else if (
            o.nom.startsWith("is")
                && o.parametres.size() == 0
                && o.corps.instructions.size() == 1) {
            return indente()
                + "attr_reader :"
                + o.nom.substring(2, 3).toLowerCase()
                + o.nom.substring(3)
                + "\n";
        } else if (
            o.nom.startsWith("set")
                && o.parametres.size() == 1
                && o.corps.instructions.size() == 1) {
            return indente()
                + "attr_writer :"
                + o.nom.substring(3, 4).toLowerCase()
                + o.nom.substring(4)
                + "\n";
        } else {
            String r = code(o.commentaires);

            String visible = code(o.visibilite);
            if (visible.length() > 0) {
                r += indente();
                r += code(o.visibilite);
            }
            r += indente() + "def " + o.nom + " (";

            boolean p = true;
            for (Enumeration e = o.parametres.elements(); e.hasMoreElements();) {
                AglParametre ap = (AglParametre) e.nextElement();
                if (!p) {
                    r += ", ";
                }
                r += code(ap);
                p = false;
            }

            r += ")\n";
            r += code(o.corps);

            return r;
        }
    }

    /**
     * @see AglCode#code(AglImport)
     */
    public String code(AglImport o) {
        return "require " + code(o.nom) + "\n";
    }

    /**
     * @see AglCode#code(AglConstructeur)
     */
    public String code(AglConstructeur o) {
        if (o.destructeur) {
            return "";
        }

        indentation = 1;
        String r = code(o.commentaires);

        String visibilite = code(o.visibilite);
        if (visibilite.length() > 0) {
            r += indente();
            r += visibilite;
        }
        r += indente() + "def new(";

        boolean p = true;
        for (Enumeration e = o.parametres.elements(); e.hasMoreElements();) {
            AglParametre ap = (AglParametre) e.nextElement();
            if (!p) {
                r += ", ";
            }
            r += code(ap);
            p = false;
        }

        r += ")\n";
        r += code(o.corps);

        return r;
    }

    /**
     * @see AglCode#code(AglClasse)
     */
    public String code(AglClasse o) {
        indentation = 0;
        String r = code(o.commentaires);

        String is = "";
        indentation++;
        is = indente();

        if (o.normale)
            r += "class";
        else
            r += "module";
        r += " " + o.nom.dernier();

        if (o.meres.noms.size() > 0) {
            String name = ((AglNom) o.meres.noms.elementAt(0)).dernier();
            r += " < " + name.substring(name.lastIndexOf(".") + 1);
        }

        r += "\n";

        for (int i = 0; i < o.interfaces.noms.size(); i++) {
            String name = ((AglNom) o.interfaces.noms.elementAt(i)).dernier();
            r += indente() + "include " + name + "\n";
        }

        r += "\n";

        AglVecteur methodesVecteur = new AglVecteur();
        AglVecteur attributsReaderVecteur = new AglVecteur();
        AglVecteur attributsWriterVecteur = new AglVecteur();
        for (Enumeration e = o.methodes.elements(); e.hasMoreElements();) {
            AglMethode am = (AglMethode) e.nextElement();
            String code = code(am);
            if (code.indexOf("attr_reader :") != -1) {
                attributsReaderVecteur.addElement(code);
            } else if (code.indexOf("attr_writer :") != -1) {
                attributsWriterVecteur.addElement(code);
            } else {
                methodesVecteur.addElement(code);
            }
        }

        for (Enumeration e = o.attributs.elements(); e.hasMoreElements();) {
            AglAttribut aa = (AglAttribut) e.nextElement();
            if (aa.visibilite.vpublic) {
                if (aa.visibilite.vstatic && aa.visibilite.vfinal) {
                    // Constant
                    r += indente() + code(o.nom) + " = " + code(aa.initiale) + "\n";
                } else {
                    r += indente() + "attr_accessor :" + aa.nom + "\n";
                }
            }
        }

        // First convert attr_reader and attr_writer to attr_accessor if possible
        for (Enumeration e = attributsReaderVecteur.elements(); e.hasMoreElements();) {
            String code = (String) e.nextElement();
            int c = code.indexOf("attr_reader");
            String code2 = code.substring(0, c) + "attr_writer" + code.substring(c + 11);
            if (attributsWriterVecteur.indexOf(code2) != -1) {
                r += indente() + "attr_accessor" + code.substring(c + 11);

                attributsReaderVecteur.removeElement(code);
                attributsWriterVecteur.removeElement(code2);
            }
        }

        // Ouput only readable or writeable attributs
        for (Enumeration e = attributsWriterVecteur.elements(); e.hasMoreElements();) {
            r += e.nextElement();
        }

        for (Enumeration e = attributsReaderVecteur.elements(); e.hasMoreElements();) {
            r += e.nextElement();
        }

        if (o.obinit.instructions.size() > 0) {
            r += "\n";
            r += code(o.obinit, true);
        }

        if (o.constructeurs.size() > 0) {
            r += "\n";
            for (Enumeration e = o.constructeurs.elements(); e.hasMoreElements();) {
                AglConstructeur am = (AglConstructeur) e.nextElement();
                r += code(am);
            }
        }

        for (Enumeration e = methodesVecteur.elements(); e.hasMoreElements();) {
            r += e.nextElement();
            r += "\n";
        }

        indentation--;
        return r + "end\n";
    }

    /**
     * @see AglCode#code(AglBloc)
     */
    public String code(AglBloc o) {
        return code(o, true);
    }

    /**
     * @see AglCode#code(AglBloc)
     */
    public String code(AglBloc o, boolean addEnd) {
        indentation++;

        String r = "";
        // int n=1;
        for (Enumeration e = o.instructions.elements(); e.hasMoreElements();) {
            AglInstruction ai = (AglInstruction) e.nextElement();
            r += code(ai);
            // n++;
        }
        indentation--;
        if (addEnd) {
            r += indente() + "end\n";
        }
        return r;
    }

    /**
     * @see AglCode#code(AglAttribut)
     */
    public String code(AglAttribut o) {
        return o.nom;
    }

    /**
     * @see AglCode#code(AglUnite)
     */
    public String code(AglUnite o) {
        String r = "";
        indentation = 0;

        r += entete(o.fichier);
        r += code(o.commentaires);

        // if(o.paquet!=null) r+="\n"+code(o.paquet);

        if (o.imports.size() > 0)
            r += "\n";
        for (Enumeration e = o.imports.elements(); e.hasMoreElements();) {
            AglImport ai = (AglImport) e.nextElement();
            r += code(ai);
        }

        r += "\n";

        for (Enumeration e = AglType.table.elements(); e.hasMoreElements();) {
            AglType at = (AglType) e.nextElement();
            if (at instanceof AglMoule) {
                AglMoule ap = (AglMoule) at;
                if (!ap.elementaire) {
                    r += code((AglType) ap);
                    r += "\n";
                }
            }
        }

        for (Enumeration e = AglType.table.elements(); e.hasMoreElements();) {
            AglType at = (AglType) e.nextElement();
            if ((o == at.proprietaire) && (at instanceof AglStruct)) {
                AglStruct as = (AglStruct) at;
                if (!as.elementaire && filtre(as.nom)) {
                    r += code(as);
                    r += "\n";
                }
            }
        }

        for (Enumeration e = AglType.table.elements(); e.hasMoreElements();) {
            AglType at = (AglType) e.nextElement();
            if ((o == at.proprietaire) && (at instanceof AglClasse)) {
                AglClasse ac = (AglClasse) at;
                if (!ac.elementaire && filtre(ac.nom)) {
                    r += code(ac);
                    r += "\n";
                }
            }
        }

        return r;
    }

    /**
     * @see AglCode#code(AglListeNoms)
     */
    public String code(AglListeNoms o) {
        String r = "";
        boolean p = true;

        for (Enumeration e = o.noms.elements(); e.hasMoreElements();) {
            if (p) {
                p = false;
            } else {
                r += ", ";
            }
            r += code((AglNom) e.nextElement());
        }

        return r;
    }

    /**
     * @see AglCode#code(AglNom)
     */
    public String code(AglNom o) {
        return o.dernier();
    }
}
