/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeSql3.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglCodeSql3 extends AglCode
{
  AglClasse _classe;

  public String extension()
  {
    return "sql";
  }

  public String[] extensions()
  {
    return new String[] { "sql", "SQL" };
  }

  protected String debutCS  () { return indente()+"-- ";    }
  protected String finCS    () { return "";                 }
  protected String debutCM  () { return indente()+"------"; }
  protected String specialCM() { return "";                 }
  protected String milieuCM () { return indente()+"-- ";    }
  protected String finCM    () { return indente()+"------"; }

  public String code(AglNom o)
  {
    return o.dernier();
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";

         if(o==AglTypePrimitif.tboolean)  r+="BOOL";
    else if(o==AglTypePrimitif.tchar   )  r+="CHAR";
    else if(o==AglTypePrimitif.tbyte   )  r+="BYTE";
    else if(o==AglTypePrimitif.tinteger)  r+="INTEGER";
    else if(o==AglTypePrimitif.tlong   )  r+="INTEGER";
    else if(o==AglTypePrimitif.tfloat  )  r+="REAL";
    else if(o==AglTypePrimitif.tdouble )  r+="DOUBLE PRECISION";
    else if(o==AglType.cherche("String")) r+="VARCHAR(80)";
    else if(o==AglType.cherche("Date"  )) r+="DATE";

    else if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau) r+="["+code(((AglTypeTableau)o).type)+"("+code(((AglTypeTableau)o).dimension)+")]";
    else r+="REF(t_"+code(o.nom).toLowerCase()+")";
    return r;
  }

  public String code(AglAttribut o)
  {
    String r="";
    r+=code(o.visibilite);
    r+=o.nom.toLowerCase()+" "+code(o.type);
    if(o.initiale!=null) r+=" DEFAULT "+code(o.initiale);
    return r;
  }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    String r="";
    r+=debutCS()+code(o.nom)+" ["+o.ligne+"]"+finCS()+"\n\n";

    r+="CREATE TYPE t_"+code(o.nom).toLowerCase()+"\n";
    r+="(\n";

    boolean p=true;
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(!p) r+=",\n";
      r+="  "+code(aa);
      p=false;
    }

    r+="\n);\n";

    return r;
  }

  protected void ajouteAttributs(AglClasse c, AglVecteur v)
  {
    for(Enumeration ea=c.attributs.elements(); ea.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)ea.nextElement();
      v.addElement(aa);
    }

    for(Enumeration em=c.meres.noms.elements(); em.hasMoreElements(); )
    {
      AglNom    nm=(AglNom)em.nextElement();
      AglClasse cm=(AglClasse)AglType.cherche(nm);
      if(cm!=null) ajouteAttributs(cm,v);
    }
  }

  public String code(AglClasse o)
  {
    _classe=o;

    String r="";
    r+=debutCS()+code(o.nom)+" ["+o.ligne+"]"+finCS()+"\n\n";

    r+="CREATE TYPE t_"+code(o.nom).toLowerCase()+"\n";
    r+="(\n";
    // r+="  id_"+o.nom.dernier().toLowerCase()+" INTEGER PRIMARY KEY";

    AglVecteur va=new AglVecteur(11,11);
    ajouteAttributs(o,va);

    boolean p=true;
    for(Enumeration e=va.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      if(!p) r+=",\n";
      r+="  "+code(aa);
      p=false;
    }

    r+="\n";

/*
    r+=code(o.visibilite);
    r+="class "+code(o.nom);

    if(o.meres.noms.size()>0)
      r+=" : "+code(o.meres);
    if(o.interfaces.noms.size()>0)
      r+=" implements "+code(o.interfaces);
    if(o.exceptions.noms.size()>0)
      r+=" throws "+code(o.exceptions);
*/

/*
    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+="\n"+code(am);
    }
*/

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+="\n  "+code(am);
    }

    r+="\n);\n";
    return r;
  }

  public String code(AglConstructeur o)
  {
    String r="";
    r+=code(o.visibilite);
    r+=code(_classe.nom)+"* new"+code(o.nom)+o.parametres.size()+"(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=");";
    return r;
  }

  public String code(AglImport o)
  {
    String r="@import "+o.nom.dernier();
    r+=".sql\n";
    return r;
  }

  public String code(AglMethode o)
  {
    String r="";
    r+=code(o.visibilite)+"FUNCTION "+o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=", ";
      r+=code(ap);
      p=false;
    }

    r+=")";
    if(o.type!=AglTypePrimitif.tvoid) r+=" RETURNS "+code(o.type);
    r+=";";
    return r;
  }

  public String code(AglParametre o)
  {
    String r=o.nom+" "+code(o.type);
    // if(!o.defaut.equals("")) r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vpublic)       r+="PUBLIC ";
    if(o.vprivate)      r+="PRIVATE ";
    // if(o.vprotected)    r+="protected ";
    // if(o.vabstract)     r+="virtual ";
    // if(o.vinline)       r+="inline ";
    // if(o.vstatic)       r+="static ";
    // if(o.vfinal)        r+="final ";
    // if(o.vnative)       r+="native ";
    // if(o.vtransient)    r+="register ";
    // if(o.vsynchronized) r+="synchronized ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r=" ("+code(o.type)+"*)malloc(1,sizeof("+code(o.type)+"))";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r=" ("+code(o.type)+"*)malloc("+code(o.nombre)+",sizeof("+code(o.type)+")))";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }

  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier);
    r+="\n";
        
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    return r;
  }
}
