/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeTom.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import java.util.*;

public class AglCodeTom extends AglCode
{
  private String unite_="";

  public String extension()
  {
    return "t";
  }

  public String[] extensions()
  {
    return new String[] { "t", "T", "tom", "TOM" };
  }

  public String code(AglNom o)
  {
    String r="";
    boolean p=true;
    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=".";
      r+=(String)e.nextElement();
    }

    if(r.startsWith(unite_+'.'))
      r=r.substring(unite_.length()+1);
    return r;
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
         if(o==AglTypePrimitif.tshort)   r+="int";
    else if(o instanceof AglTypeTableau) r+="Array";
    else if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
    else r+=code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    indentation=1;
    String r=code(o.commentaires);

    r+=indente();
    r+=code(o.visibilite);
    r+=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+=" ="+code(o.initiale);
    r+=";";
    if(LIGNES) r+=" /* "+o.ligne+" */";
    r+="\n";

    return r;
  }

  public String code(AglBloc o)
  {
    return code(o,false);
  }

  protected final String code(AglBloc o,boolean _reduced)
  {
    String r=code(o.commentaires);
    
    if(_reduced&&(o.instructions.size()==1))
    {
      indentation++;
      r+=code(o.instructions.elementAt(0));
      indentation--;
    }
    else
    {
      r+=indente()+"{\n";
      indentation++;

      //int n=1;
      for(Enumeration e=o.instructions.elements(); e.hasMoreElements(); )
      {
	AglInstruction ai=(AglInstruction)e.nextElement();
	r+=code(ai);
	//n++;
      }

      indentation--;
      r+=indente()+"}\n";
    }

    return r;
  }

  public String code(AglStruct o)
  {
    indentation=0;
    String r="";

    if(LIGNES)
    {
      r+="/* ";
      // if(o.paquet!="") r+=o.paquet;
      r+=code(o.nom)+" ["+o.ligne+"] */\n\n";
    }

    r+=code(o.visibilite);
    r+="class "+code(o.nom);

    r+="\n{\n";
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation=0;
    return r+"}\n";
  }

  public String code(AglClasse o)
  {
    indentation=0;
    String r="";

    if(LIGNES)
    {
      r+="/* ";
      // if(o.paquet!="") r+=o.paquet;
      r+=code(o.nom)+" ["+o.ligne+"] */\n\n";
    }

    r+=code(o.commentaires);
    // r+=code(o.visibilite);
    if(o.normale) r+="implementation class";
    else          r+="implementation interface";
    r+=" "+code(o.nom);

    if(o.meres.noms.size()>0)
      r+=": "+code(o.meres);

    /*
    if(o.interfaces.noms.size()>0)
      r+=" implements "+code(o.interfaces);
    if(o.exceptions.noms.size()>0)
      r+=" throws "+code(o.exceptions);
    */

    r+="\n\n";
    indentation++;

    if(o.clinit.instructions.size()>0) { r+=code(o.clinit); r+="\n"; }
    if(o.obinit.instructions.size()>0) { r+=code(o.obinit); r+="\n"; }

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }
    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    indentation--;
    r+="\nend;\n";

    r+="\n";
    if(o.normale)
    {
      r+="implementation instance "+code(o.nom)+"\n";
      indentation++;
      for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	r+=code(aa);
      }
      indentation--;
      r+="end;\n";
    }

    indentation=0;
    return r+"\n";
  }

  public String code(AglConstructeur o)
  {
    indentation=1;
    String r=code(o.commentaires);

    r+=indente();
    r+=code(o.visibilite);

    if(!o.destructeur)
      r+="id init";
    else
      r+="void dealloc";

    if(o.parametres.size()>0)
    {
      r+=o.parametres.size();
      r+=" (";

      boolean p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)e.nextElement();
	if(!p) r+=",";
	r+=code(ap.type);
	p=false;
      }
      r+=") (";

      p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)e.nextElement();
	if(!p) r+=" ";
	r+=code(ap.nom);
	p=false;
      }
      r+=")";
    }

    if(o.visibilite.vabstract&&o.corps.instructions.isEmpty()) r+=";";
    if(LIGNES) r+=" /* "+o.ligne+" */";
    r+="\n";
    if(!o.corps.instructions.isEmpty())
    {
      if(o.corps.instructions.size()>1) r+=code(o.corps);
      else
	r+=indente()+"{\n"+code(o.corps)+indente()+"}\n";
      r+="\n";
    }

    return r;
  }

  public String code(AglPaquet o)
  {
    indentation=0;
    unite_=code(o.nom);
    String r="unit "+unite_+"\n";
    return r;
  }

  public String code(AglImport o)
  {
    indentation=0;
    String r=code(o.nom);
    int i=r.lastIndexOf('.');
    if(i>=0) r="uses "+r.substring(0,i)+";\n";
    else     r="";
    return r;
  }

  public String code(AglMethode o)
  {
    indentation=1;
    String r=code(o.commentaires);

    r+=indente();
    r+=code(o.visibilite);
    r+=code(o.type)+" "+o.nom;

    if(o.parametres.size()>0)
    {
      r+=" (";

      boolean p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)e.nextElement();
	if(!p) r+=",";
	r+=code(ap.type);
	p=false;
      }
      r+=") (";

      p=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)e.nextElement();
	if(!p) r+=" ";
	r+=code(ap.nom);
	p=false;
      }
      r+=")";
    }

    if(  // (o.visibilite.vabstract||o.visibilite.vnative)&&
       o.corps.instructions.isEmpty()) r+=";";
    if(LIGNES) r+=" /* "+o.ligne+" */";
    r+="\n";
    if(!o.corps.instructions.isEmpty())
    {
      if(o.corps.instructions.size()>1) r+=code(o.corps);
      else
	r+=indente()+"{\n"+code(o.corps)+indente()+"}\n";
      r+="\n";
    }

    return r;
  }

  public String code(AglParametre o)
  {
    String r="";
    r+=code(o.type)+" "+o.nom;
    // if(o.defaut!="") r+=" ="+o.defaut;
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

    if(o.vpublic)       r+="public ";
    if(o.vprivate)      r+="private ";
    if(o.vprotected)    r+="protected ";
    // if(o.vinline)       r+="inline ";
    // if(o.vabstract)     r+="abtract ";
    if(o.vstatic)       r+="static ";
    // if(o.vfinal)        r+="final ";
    // if(o.vnative)       r+="native ";
    // if(o.vtransient)    r+="transient ";
    // if(o.vsynchronized) r+="synchronized ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    if(LIGNES) r+=" /* Affectation "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    if(LIGNES) r+=" /* Attribut "+o.ligne+" */ ";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    if(LIGNES) r+=" /* Binaire "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    if(LIGNES) r+=" /* Var "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    if(LIGNES) r+=" /* EstUn "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=",";
      r+=code(ae);
      p=false;
    }

    r+=")";
    if(LIGNES) r+=" /* Fonction "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]"; // at
    if(LIGNES) r+=" /* Indice "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    if(LIGNES) r+=" /* InitArray "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="[";
    if(o.objet!=null) r+=code(o.objet);
    else              r+="self";

    r+=" "+o.fonction.nom;

    for(Enumeration e=o.fonction.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" ";
      r+=code(ae);
    }

    r+="]";
    if(LIGNES) r+=" /* Methode "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";
    if(LIGNES) r+=" /* Multiple "+o.ligne+" */ ";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r="[["+code(o.type)+" alloc] init";
    if(o.arguments.size()>0) r+=o.arguments.size();

    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      r+=" ";
      r+=code(ae);
    }

    r+="]";
    if(LIGNES) r+=" /* New "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r="[Array alloc]";
    // if(o.nombre!=null) r+=code(o.nombre);
    // r+="]";
    if(LIGNES) r+=" /* NewArray "+o.ligne+" */ ";
    if(o.initiale!=null) r="("+r+code(o.initiale)+")";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="("+code(o.expression)+")";
    if(LIGNES) r+=" /* Parenthesee "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)+" "
      +o.operateur1+" "+code(o.milieu)+" "
      +o.operateur2+" "+code(o.droite);
    if(LIGNES) r+=" /* Ternaire "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r=code(o.type)+"("+code(o.expression)+")";
    if(LIGNES) r+=" /* Typage "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String op=o.operateur;
    if("--".equals(op)) op="-";
    String r=op+code(o.droite);
    if(LIGNES) r+=" /* UnaireDroite "+o.ligne+" */ ";
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String op=o.operateur;
    if("--".equals(op)) op="-";
    String r=op+code(o.gauche);
    if(LIGNES) r+=" /* UnaireGauche "+o.ligne+" */ ";
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
  {
    String r=code(o.commentaires)+indente();
    r+=o.body;
    if((r.length()>0)&&(r.charAt(r.length()-1)!='\n')) r+="\n";
    r=r.substring(0,r.length()-1);
    if(LIGNES) r+=" /* Instruction "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionVide o)
  {
    String r=code(o.commentaires)+indente();
    r+=";";
    if(LIGNES) r+=" /* Vide "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionAffectation o)
  {
    String r=code(o.commentaires)+indente();
    r+=code(o.gauche)+"="+code(o.droite)+";";
    if(LIGNES) r+=" /* Affectation "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionBloc o)
  {
    String r=code(o.commentaires);
    if(LIGNES) r+=indente()+"/* Bloc "+o.ligne+" */\n";
    if(!o.virgule)
	r+=code(o.corps);
    else
    {
      boolean p=true;
      for(Enumeration e=o.corps.instructions.elements(); e.hasMoreElements(); )
      {
	if(!p) r+=",";
	AglInstruction ai=(AglInstruction)e.nextElement();
	String s=code(ai).trim();
	int    i=s.lastIndexOf(";");
	s=s.substring(0,i)+s.substring(i+1);
	if((!p)&&(ai instanceof AglInstructionDefVar))
	{
	  i=s.indexOf(" ")+1;
	  s=s.substring(i);
	}
	r+=s;
	p=false;
      }
      r+=";";
    }
    return r;
  }

  public String code(AglInstructionBreak o)
  {
    String r=code(o.commentaires)+indente()+"break";
    if(o.nom!=null) r+=" "+o.nom;
    r+=";";
    if(LIGNES) r+=" /* Break "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionCatch o)
  {
    String r=code(o.commentaires)+indente();
    r+="catch("+code(o.signal)+")";
    if(LIGNES) r+=" /* Catch "+o.ligne+" */";
    r+="\n";
    r+=code(o.reprise);
    return r;
  }

  public String code(AglInstructionContinue o)
  {
    String r=code(o.commentaires)+indente()+"continue";
    if(o.nom!=null) r+=" "+o.nom;
    r+=";";
    if(LIGNES) r+=" /* Continue "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionDefVar o)
  {
    String r=code(o.commentaires)+indente()+code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    r+=";";
    if(LIGNES) r+=" /* DefVar "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionDoWhile o)
  {
    String r=code(o.commentaires)+indente()+"do";
    if(LIGNES) r+=" /* DoWhile "+o.ligne+" */";
    r+="\n";
    r+=code(o.corps);
    r+=indente()+"while("+code(o.condition)+");";
    if(LIGNES) r+=" /* DoWhile "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionExpression o)
  {
    String r=code(o.expression);
    if(r.startsWith("(")&&(r.endsWith(")"))) r=r.substring(1,r.length()-1);
    r=code(o.commentaires)+indente()+r+";";
    if(LIGNES) r+=" /* InsExpr "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionFinally o)
  {
    String r=code(o.commentaires)+indente()+"finally";
    if(LIGNES) r+=" /* Finally "+o.ligne+" */";
    r+="\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionFor o)
  {
    String r=code(o.commentaires)+indente()+"for(";
    String d=code(o.debut).trim();
    String t=code(o.test).trim();
    String f=code(o.fin).trim();

    if(f.endsWith(";")) f=f.substring(0,f.length()-1);

    r+=d+" "+t+"; "+f+")";
    if(LIGNES) r+=" /* For "+o.ligne+" */";
    r+="\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionGoto o)
  {
    String r=code(o.commentaires)+indente()+"continue "+o.nom;
    r+=";";
    if(LIGNES) r+=" /* Goto "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionIf o)
  {
    String r=code(o.commentaires)+indente()+"if ("+code(o.condition)+")";
    if(LIGNES) r+=" /* If "+o.ligne+" */";
    r+="\n";
    r+=code(o.sioui);
    if(o.sinon!=null)
    {
      r+=indente()+"else";
      if(LIGNES) r+=" /* Else "+o.ligne+" */";
      r+="\n";
      r+=code(o.sinon);
    }

    return r;
  }

  public String code(AglInstructionLabel o)
  {
    String r=code(o.commentaires);
    indentation--;
    r+=indente()+" ";
    indentation++;
    r+=o.nom+":";
    if(LIGNES) r+=" /* Label "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionProcedure o)
  {
    String r=code(o.commentaires)+indente()+o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=");";
    if(LIGNES) r+=" /* Procedure "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionReturn o)
  {
    String r=code(o.commentaires)+indente()+"return "+code(o.valeur);
    r+=";";
    if(LIGNES) r+=" /* Return "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionSwitch o)
  {
    String r=code(o.commentaires)+indente()+"switch("+code(o.condition)+")";
    if(LIGNES) r+=" /* Switch "+o.ligne+" */";
    r+="\n";
    r+=indente()+"{\n";
    indentation++;

    for(Enumeration e=o.cas.keys(); e.hasMoreElements(); )
    {
      AglExpression ce=(AglExpression)e.nextElement();
      r+=indente()+"case "+code(ce)+":\n";
      AglBloc cb=(AglBloc)(o.cas.get(ce));
      r+=code(cb);
      r+=indente()+"break;\n";
    }

    r+=indente()+"default:\n";
    r+=code(o.defaut);
    // r+=indente()+"break;\n";

    indentation--;
    r+=indente()+"}\n";

    return r;
  }

  public String code(AglInstructionSynchronized o)
  {
    String r=code(o.commentaires)+indente()+
	"synchronized("+code(o.objet)+")";
    if(LIGNES) r+=" /* Synchronized "+o.ligne+" */";
    r+="\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionThrow o)
  {
    String r=code(o.commentaires)+indente()+"throw "+code(o.valeur);
    r+=";";
    if(LIGNES) r+=" /* Throw "+o.ligne+" */";
    r+="\n";
    return r;
  }

  public String code(AglInstructionTry o)
  {
    String r=code(o.commentaires)+indente()+"try";
    if(LIGNES) r+=" /* Try "+o.ligne+" */";
    r+="\n";
    r+=code(o.corps);
    return r;
  }

  public String code(AglInstructionWhile o)
  {
    String r=code(o.commentaires)+indente()+"while("+code(o.condition)+")";
    if(LIGNES) r+=" /* While "+o.ligne+" */";
    r+="\n";
    r+=code(o.corps);
    return r;
  }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+=entete(o.fichier,"http://www.gerbil.org/tom/");

    if(o.paquet!=null) r+=code(o.paquet);

    if(o.imports.size()>0) r+="\n";
    for(Enumeration e=o.imports.elements(); e.hasMoreElements(); )
    {
      AglImport ai=(AglImport)e.nextElement();
      r+=code(ai);
    }
        
    r+="\n";
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
          r+=code(ac);
          r+="\n";
        }
      }
    }

    if(o.paquet!=null) r+="end;\n";

    return r;
  }
}
