/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeUmlAbstract.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public abstract class AglCodeUmlAbstract extends AglCode
{
  protected boolean NAMESPACE=
    AglPreferences.AGL.getBooleanProperty("uml.code.namespace",false);

  protected boolean SHORTNAME=
    AglPreferences.AGL.getBooleanProperty("uml.code.shortname",true);

  protected boolean ATTRIBUTETYPE=
    AglPreferences.AGL.getBooleanProperty("uml.code.attribute.type",false);

  protected boolean METHODTYPE=
    AglPreferences.AGL.getBooleanProperty("uml.code.method.type",false);

  protected boolean PARAMETERNAME=
    AglPreferences.AGL.getBooleanProperty("uml.code.parameter.name",false);

  protected boolean PARAMETERTYPE=
    AglPreferences.AGL.getBooleanProperty("uml.code.parameter.type",false);

  protected String debutCS  () { return ""; }
  protected String finCS    () { return ""; }
  protected String debutCM  () { return ""; }
  protected String specialCM() { return ""; }
  protected String milieuCM () { return ""; }
  protected String finCM    () { return ""; }

  public final String code(AglNom o)
  {
    return code(o,false);
  }

  protected final String code(AglNom o,boolean _short)
  {
    String r="";

    if(_short||(SHORTNAME&&!NAMESPACE))
    {
      r=o.dernier();
    }
    else
    {
      //if(NAMESPACE) r+="::";
      String s=(NAMESPACE ? "::" : "_");
      boolean p=true;

      for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
      {
	if(p) p=false; else r+=s;
	r+=(String)e.nextElement();
      }
    }

    return r;
  }

  public final String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public final String code(AglType o)
  {
    String r="";
    /*
      if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
      else
    */
    if(o instanceof AglTypeTableau)
      r+=code(((AglTypeTableau)o).type)+
	"["+code(((AglTypeTableau)o).dimension)+"]";
    else
      r+=code(o.nom);

    return r;
  }

  public final String code(AglAttribut o)
  {
    String r=o.nom;
    if(ATTRIBUTETYPE) r+=" : "+code(o.type);
    return r;
  }

  public final String code(AglBloc o)
  {
    return "";
  }

  public final String code(AglImport o)
  {
    return "";
  }

  public final String code(AglConstructeur o)
  {
    String r="";
    r+=o.nom+" (";

    if(o.parametres.size()>0) r+=o.parametres.size();
    else r+=" ";

    /*
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=",";
      r+=code(ap);
      p=false;
    }
    */

    r+=")";
    r+=" <<constructeur>>";
    return r;
  }

  public final String code(AglMethode o)
  {
    String r="";
    r+=o.nom+" (";

    if(o.parametres.size()>0) r+=o.parametres.size();
    else r+=" ";

    /*
    boolean p=true;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      if(!p) r+=",";
      r+=code(ap);
      p=false;
    }
    */

    r+=")";
    if(METHODTYPE) r+=" : "+code(o.type);
    return r;
  }

  public final String code(AglParametre o)
  {
    String r="";
    if(PARAMETERTYPE) r+=code(o.type);
    if(PARAMETERNAME) r+=(PARAMETERTYPE ? " " : "")+o.nom;
    return r;
  }

  public final String code(AglVisibilite o)
  {
    String r="";
    if(o.vpublic)    r+="+";
    if(o.vprivate)   r+="-";
    if(o.vprotected) r+="#";
    if(o.vabstract)  r+="$";
    return r;
  }

  // Expressions

  public final String code(AglExpressionGenerique o)
  { return ""; }

  public final String code(AglExpressionAffectation o)
  { return ""; }

  public final String code(AglExpressionAttribut o)
  { return ""; }

  public final String code(AglExpressionBinaire o)
  { return ""; }

  public final String code(AglExpressionDefVar o)
  { return ""; }

  public final String code(AglExpressionEstUn o)
  { return ""; }

  public final String code(AglExpressionFonction o)
  { return ""; }

  public final String code(AglExpressionIndice o)
  { return ""; }

  public final String code(AglExpressionInitArray o)
  { return ""; }

  public final String code(AglExpressionMethode o)
  { return ""; }

  public final String code(AglExpressionMultiple o)
  { return ""; }

  public final String code(AglExpressionNew o)
  { return ""; }

  public final String code(AglExpressionNewArray o)
  { return ""; }

  public final String code(AglExpressionParenthesee o)
  { return ""; }

  public final String code(AglExpressionTernaire o)
  { return ""; }

  public final String code(AglExpressionTypage o)
  { return ""; }

  public final String code(AglExpressionUnaireDroite o)
  { return ""; }

  public final String code(AglExpressionUnaireGauche o)
  { return ""; }

  // Instructions

  public final String code(AglInstructionGenerique o)
  { return ""; }
  
  public final String code(AglInstructionVide o)
  { return ""; }

  public final String code(AglInstructionAffectation o)
  { return ""; }
  
  public final String code(AglInstructionBloc o)
  { return ""; }
  
  public final String code(AglInstructionBreak o)
  { return ""; }
  
  public final String code(AglInstructionCatch o)
  { return ""; }

  public final String code(AglInstructionContinue o)
  { return ""; }
  
  public final String code(AglInstructionDefVar o)
  { return ""; }
  
  public final String code(AglInstructionDoWhile o)
  { return ""; }
  
  public final String code(AglInstructionExpression o)
  { return ""; }
  
  public final String code(AglInstructionFinally o)
    { return ""; }

  public final String code(AglInstructionGoto o)
  { return ""; }
  
  public final String code(AglInstructionFor o)
  { return ""; }
  
  public final String code(AglInstructionIf o)
  { return ""; }
  
  public final String code(AglInstructionLabel o)
  { return ""; }

  public final String code(AglInstructionProcedure o)
  { return ""; }

  public final String code(AglInstructionReturn o)
  { return ""; }

  public final String code(AglInstructionSwitch o)
    { return ""; }

  public final String code(AglInstructionSynchronized o)
    { return ""; }

  public final String code(AglInstructionThrow o)
    { return ""; }

  public final String code(AglInstructionTry o)
  { return ""; }

  public final String code(AglInstructionWhile o)
  { return ""; }

  public final String code(AglUnite o)
  { return ""; }
}
