/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeUmlRelations.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;
//import com.memoire.alma.*;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;

import javax.swing.*;
import javax.swing.border.*;

class AglCodeUmlRelationsXY
{
  AglClasse classe;
  int       x,y;
  boolean   in;

  AglCodeUmlRelationsXY(AglClasse _c, int _x, int _y, boolean _in)
  {
    classe=_c;
    x=_x;
    y=_y;
    in=_in;
  }
}

class AglCodeUmlRelationsFrame extends JFrame implements Runnable
{
  //Thread blinker;
  AglCodeUmlRelations       a;
  AglCodeUmlRelationsCanvas c;

  private JComponent    content;
  private BorderLayout  layout;
  private JScrollPane   spGraphe;

  public AglCodeUmlRelationsFrame(AglCodeUmlRelations _a)
  {
    a=_a;
    if(!"".equals(AglOptions.classe))
      setTitle("UmlRelations: "+AglOptions.classe);
    else
      setTitle("UmlRelations: "+a.projet.nom);

    content=(JComponent)getContentPane();
    layout=new BorderLayout();
    content.setLayout(layout);
    content.setBorder(new EmptyBorder(0,0,0,0));

    setSize(200,200);

    c=new AglCodeUmlRelationsCanvas(a);
    c.setBorder(new EmptyBorder(0,0,0,0));
    spGraphe=new JScrollPane(c);
    content.add("Center",spGraphe);

    setVisible(true);
    start();
  }

  public void start()
  {
    setLocation(0,128);
    new Thread(this,"Agl code uml relations").start();
  }

  /*  
  public void stop()
  {
    blinker.stop();
    blinker=null;
    // System.exit(0);
  }
  */

  public void run()
  {
    if(AglOptions.application==null)
    {
      repaint(0);

      while(!a.done)
      {
	System.err.println("WAITING...");
	try { Thread.sleep(1500); }
	catch(Exception ex) { }
      }

      save();

      if(AglOptions.application==null)
      {
	dispose();
	System.exit(0);
      }
    }
  }
  
  void save()
  {
    Image    img=createImage(a.xmax,a.ymax);
    Graphics g  =img.getGraphics();

    g.setColor(Color.white);
    g.fillRect(0,0,a.xmax,a.ymax);

    synchronized (a)
    {
      a.g=g;
      a.paintImage();
    }

    // c.update(g);

    AglImageEncoder.save(img,"UmlRelations",AglOptions.formatImage);
  }
}

public class AglCodeUmlRelations extends AglCodeUmlAbstract
{
  final static int WCI=150;
  final static int HCI=12;

  public AglProjet projet;

  protected Graphics  g;

  Hashtable table       =new Hashtable();
  Vector    classes     =new Vector();
  Hashtable heritages   =new Hashtable();
  Hashtable derivations =new Hashtable();
  Hashtable contenances =new Hashtable();
  Hashtable agregats    =new Hashtable();
  Hashtable utilisations=new Hashtable();
  Hashtable appels      =new Hashtable();

  int       x,y;
  int       xmax,ymax;
  boolean   done;

  public String extension()
  {
    return "html";
  }

  public String[] extensions()
  {
    return new String[] { "html", "htm" };
  }

  public String syntaxe()
  {
    return "Html";
  }

  public boolean optionFormatImage()
    { return true; }

  public boolean optionStyleHtml()
    { return true; }

  public synchronized Image createImage(Component _c)
  {
    Image img;

    /*
    img=_c.createImage(xmax,ymax);
    g=img.getGraphics();
    g.setColor(Color.red);
    g.fillRect(0,0,xmax,ymax);
    paintImage();
    */

    done=false;

    img=_c.createImage(xmax,ymax);
    g=img.getGraphics();
    g.setColor(Color.white);
    g.fillRect(0,0,xmax,ymax);
    paintImage();

    return img;
  }

  public synchronized void paintImage()
  {
    table.clear();
    x=10;
    y=10;
    xmax=Math.max(xmax,20);
    ymax=Math.max(ymax,20);

    paint();
  }

  private AglType simplifyType(AglType at)
  {
    boolean encore=true;

    while(encore)
    {
      encore=false;
      //if(at instanceof AglTypeAlias)
      //{ at=((AglTypeAlias)at).type; encore=true; }
      if(at instanceof AglTypeTableau)
	{ at=((AglTypeTableau)at).type; encore=true; }
    }

    return at;
  }

  protected boolean isInside(AglClasse o)
  {
    return filtre(o.nom);

    /*
    boolean inside=false;

    if(AglOptions.module.equals(""))
      inside=true;
    else
    if(o.nom.formeJava().startsWith(AglOptions.module))
      inside=true;

    return inside;
    */
  }

  static void degrade(Graphics g, Color c, int x, int y, int w, int h)
  {
    int   r1=c.getRed();
    int   v1=c.getGreen();
    int   b1=c.getBlue();

    int   r2=r1*8/10;
    int   v2=v1*8/10;
    int   b2=b1*9/10;

    for(int j=0;j<h;j++)
    {
      Color d=c;
      if(j<=6)     d=new Color(r2+(r1-r2)*j/6,
			       v2+(v1-v2)*j/6,
			       b2+(b1-b2)*j/6);
      if(h-j-1<=6) d=new Color(r2+(r1-r2)*(h-j-1)/6,
			       v2+(v1-v2)*(h-j-1)/6,
			       b2+(b1-b2)*(h-j-1)/6);
      g.setColor(d);
      g.drawLine(x,y+j,x+w-1,y+j);
    }
  }

  public int min(int i1, int i2)
  {
    return i1>i2 ? i2 : i1;
  }

  public int min(int i1, int i2, int i3)
  {
    return min(min(i1,i2),i3);
  }

  public int max(int i1, int i2)
  {
    return i1<i2 ? i2 : i1;
  }

  public int max(int i1, int i2, int i3)
  {
    return max(max(i1,i2),i3);
  }

  public void paint()
  {
    g.setFont(new Font("SansSerif",Font.PLAIN,10));
    g.setColor(Color.black);

    AglClasse o=(AglClasse)AglType.cherche(AglOptions.classe);
    if(o!=null)
    {

      int y0; // herits
      int y1; // utils / appels
      int y2; // cont. / deriv.

      int x0; // utils
      int x1; // herits / cont. / deriv.
      int x2; // appels

      int xo,yo,x,y,n;

      y0  =10;
      y1  =y0+max(  heritages.size(),agregats.size())*(HCI+10);
      y2  =y1+max(3,utilisations.size(),appels.size())*(HCI+10);

      ymax=y2+max(  contenances.size(),derivations.size())*(HCI+10);

      x0  =10;
      x1  =x0+(utilisations.size()>0 ? 1 : 0)*(WCI+10);
      x2  =x1+2*(WCI+10)+60;
      xmax=x2+(appels.size()>0 ? 1 : 0)*(WCI+10);

      // g.setColor(Color.white);
      // g.fillRect(0,0,xmax,ymax);
      g.setColor(Color.black);

      xo=(x1+x2-WCI)/2;
      yo=(y1+y2-HCI)/2;
      g.drawString(o.nom.dernier(),xo+2,yo+10);
      g.drawRect(xo,yo,WCI,HCI);

      classes.addElement(new AglCodeUmlRelationsXY(o,xo,yo,true));

      x=xo-WCI/2-30;
      y=y0;
      for(Enumeration e=heritages.keys(); e.hasMoreElements(); )
      {
	AglClasse ac=(AglClasse)e.nextElement();
	g.drawString(ac.nom.dernier(),x+2,y+10);
	g.drawRect(x,y,WCI,HCI);

	classes.addElement(new AglCodeUmlRelationsXY(ac,x,y,true));

	g.drawLine(x+WCI+6,y+HCI/2,xo+WCI/2-10,y+HCI/2);
	g.drawLine(xo+WCI/2-10,y+HCI/2,xo+WCI/2-10,yo);

	g.drawLine(x+WCI+1,y+HCI/2,x+WCI+6,y+HCI/2-5);
	g.drawLine(x+WCI+1,y+HCI/2,x+WCI+6,y+HCI/2+5);
	g.drawLine(x+WCI+6,y+HCI/2-5,x+WCI+6,y+HCI/2+5);

	y+=HCI+10;
      }

      x=xo+WCI/2+30;
      y=y0;
      for(Enumeration e=agregats.keys(); e.hasMoreElements(); )
      {
	AglClasse ac=(AglClasse)e.nextElement();
	g.drawString(ac.nom.dernier(),x+2,y+10);
	g.drawRect(x,y,WCI,HCI);

	classes.addElement(new AglCodeUmlRelationsXY(ac,x,y,true));

	g.drawLine(x-11,y+HCI/2,xo+WCI/2+10,y+HCI/2);
	g.drawLine(xo+WCI/2+10,y+HCI/2,xo+WCI/2+10,yo);

	g.drawLine(x-11,y+HCI/2,x-6,y+HCI/2-3);
	g.drawLine(x-11,y+HCI/2,x-6,y+HCI/2+3);
	g.drawLine(x-6,y+HCI/2-3,x-1,y+HCI/2);
	g.drawLine(x-6,y+HCI/2+3,x-1,y+HCI/2);

	y+=HCI+10;
      }

      x=xo-WCI/2-30;
      y=y2;
      for(Enumeration e=contenances.keys(); e.hasMoreElements(); )
      {
	AglClasse ac=(AglClasse)e.nextElement();
	g.drawString(ac.nom.dernier(),x+2,y+10);
	g.drawRect(x,y,WCI,HCI);

	classes.addElement(new AglCodeUmlRelationsXY(ac,x,y,true));

	g.drawLine(x+WCI,y+HCI/2,xo+WCI/2-10,y+HCI/2);
	g.drawLine(xo+WCI/2-10,y+HCI/2,xo+WCI/2-10,yo+HCI+11);

	g.drawLine(xo+WCI/2-10,yo+HCI+11,xo+WCI/2-13,yo+HCI+6);
	g.drawLine(xo+WCI/2-10,yo+HCI+11,xo+WCI/2- 7,yo+HCI+6);
	g.drawLine(xo+WCI/2-13,yo+HCI+ 6,xo+WCI/2-10,yo+HCI+1);
	g.drawLine(xo+WCI/2- 7,yo+HCI+ 6,xo+WCI/2-10,yo+HCI+1);

	y+=HCI+10;
      }

      x=xo+WCI/2+30;
      y=y2;
      for(Enumeration e=derivations.keys(); e.hasMoreElements(); )
      {
	AglClasse ac=(AglClasse)e.nextElement();
	g.drawString(ac.nom.dernier(),x+2,y+10);
	g.drawRect(x,y,WCI,HCI);

	classes.addElement(new AglCodeUmlRelationsXY(ac,x,y,true));

	g.drawLine(x,y+HCI/2,xo+WCI/2+10,y+HCI/2);
	g.drawLine(xo+WCI/2+10,y+HCI/2,xo+WCI/2+10,yo+HCI+6);

	g.drawLine(xo+WCI/2+ 5,yo+HCI+6,xo+WCI/2+10,yo+HCI+1);
	g.drawLine(xo+WCI/2+15,yo+HCI+6,xo+WCI/2+10,yo+HCI+1);
	g.drawLine(xo+WCI/2+ 5,yo+HCI+6,xo+WCI/2+15,yo+HCI+6);

	y+=HCI+10;
      }

      x=x0;
      y=yo-(utilisations.size()-1)*(HCI+10)/2;
      for(Enumeration e=utilisations.keys(); e.hasMoreElements(); )
      {
	AglClasse ac=(AglClasse)e.nextElement();
	g.drawString(ac.nom.dernier(),x+2,y+10);
	g.drawRect(x,y,WCI,HCI);

	classes.addElement(new AglCodeUmlRelationsXY(ac,x,y,true));

	g.drawOval(x+WCI+1,y+HCI/2-3,7,7);

	g.drawLine(x+WCI+9,y+HCI/2,x+WCI+20,y+HCI/2);
	g.drawLine(x+WCI+20,y+HCI/2,x+WCI+20,yo+HCI/2);
	g.drawLine(x+WCI+20,yo+HCI/2,xo,yo+HCI/2);
	y+=HCI+10;
      }

      x=x2;
      y=yo-(appels.size()-1)*(HCI+10)/2;
      for(Enumeration e=appels.keys(); e.hasMoreElements(); )
      {
	AglClasse ac=(AglClasse)e.nextElement();
	g.drawString(ac.nom.dernier(),x+2,y+10);
	g.drawRect(x,y,WCI,HCI);

	classes.addElement(new AglCodeUmlRelationsXY(ac,x,y,true));

	g.drawLine(x,y+HCI/2,x-10,y+HCI/2);
	g.drawLine(x-10,y+HCI/2,x-10,yo+HCI/2);
	g.drawLine(x-10,yo+HCI/2,xo+WCI+9,yo+HCI/2);

	g.drawOval(xo+WCI+1,yo+HCI/2-3,7,7);
	y+=HCI+10;
      }
    }

    done=true;
  }

  public String code(AglStruct o)
  {
    return "";
  }

  public String code(AglClasse o)
  {
    return "";
  }

  public AglSortie[] code(AglProjet o)
  {
    projet=o;

    indentation=0;
    String r="";

    r+="<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n";
    r+="<HTML>\n";

    if(AglOptions.entete)
    {
      r+="<!--\n";
      r+="  Entte : Alma\n";
      r+="  Projet : "+o.nom+"\n";
      r+="  Classe : "+AglOptions.classe+"\n";
      r+="  Date   : "+(new Date())+"\n";
      r+="  Cible  : UmlRelations ("+extension()+")\n";
      r+="  Auteur : Guillaume Desnoix\n";
      r+="  Mel    : guillaume@desnoix.com\n";
      r+="  Produit: "+AglOptions.PRODUIT+"\n";
      r+="  Version: "+AglOptions.VERSION+"\n";
      r+="  Site   : "+AglOptions.SITE+"\n";
      r+="-->\n";
    }

    classes.removeAllElements();
    heritages.clear();
    derivations.clear();
    contenances.clear();
    agregats.clear();
    utilisations.clear();

    if("".equals(AglOptions.classe))
      erreur("Le filtre de classe n'est pas dfini.");

    AglClasse cc=(AglClasse)AglType.cherche(AglOptions.classe);

    if(cc==null)
      erreur("Le filtre de classe ne spcifie pas une classe existante.");

    AglOptions.terminal.trace("Classe      : "+cc.nom.dernier());

    for(Enumeration e=cc.meres.noms.elements(); e.hasMoreElements(); )
    {
      AglNom  an=(AglNom)e.nextElement();
      AglType at=(AglType)AglType.cherche(an);

      if((at!=null)&&(at instanceof AglClasse))
      {
	AglClasse ac=(AglClasse)at;
	heritages.put(ac,cc);
	AglOptions.terminal.trace("Hritage    : "+ac.nom.dernier());
      }
    }
    if(heritages.size()==0)
    {
      AglClasse ac=(AglClasse)AglType.cherche("Object");
      if(ac==null)
      {
	ac=new AglClasse(new AglNom("Object"));
	ac.elementaire=true;
      }
      heritages.put(ac,cc);
    }

    for(Enumeration et=AglType.table.elements(); et.hasMoreElements(); )
    {
      AglType xt=(AglType)et.nextElement();
      if(xt instanceof AglClasse)
      {
	AglClasse ac=(AglClasse)xt;

	for(Enumeration e=ac.meres.noms.elements(); e.hasMoreElements(); )
	{
	  AglNom  an=(AglNom)e.nextElement();
	  if(an.dernier().equals(cc.nom.dernier()))
	  {
	    derivations.put(ac,cc);
	    AglOptions.terminal.trace("Drivation  : "+ac.nom.dernier());
	  }
	}

	for(Enumeration e=ac.attributs.elements(); e.hasMoreElements(); )
	{
	  AglAttribut  aa=(AglAttribut)e.nextElement();
	  if(aa.type==cc)
	  {
	    agregats.put(ac,cc);
	    AglOptions.terminal.trace("Aggrgat    : "+ac.nom.dernier());
	  }
	}
      }
    }

    for(Enumeration e=cc.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      AglType     at=simplifyType(aa.type);

      if(at instanceof AglClasse)
      {
	AglClasse ac=(AglClasse)at;
	contenances.put(ac,cc);
	AglOptions.terminal.trace("Contenance  : "+ac.nom.dernier());
      }
    }

    for(Enumeration e=cc.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();

      {
	AglType at=simplifyType(am.type);
	if(at instanceof AglClasse)
	{
	  AglClasse ac=(AglClasse)at;
	  utilisations.put(ac,cc);
	  AglOptions.terminal.trace("Utilisation : "+ac.nom.dernier());
	}
      }

      for(Enumeration ep=am.parametres.elements(); ep.hasMoreElements(); )
      {
	AglParametre ap=(AglParametre)ep.nextElement();
	AglType      at=simplifyType(ap.type);

	if(at instanceof AglClasse)
	{
	  AglClasse cp=(AglClasse)at;
	  utilisations.put(cp,cc);
	  AglOptions.terminal.trace("Utilisation : "+cp.nom.dernier());
	}
      }
    }
    utilisations.remove(cc);

    for(Enumeration et=AglType.table.elements(); et.hasMoreElements(); )
    {
      AglType xt=(AglType)et.nextElement();
      if(xt instanceof AglClasse)
      {
	AglClasse ac=(AglClasse)xt;
	for(Enumeration e=ac.methodes.elements(); e.hasMoreElements(); )
	{
	  AglMethode am=(AglMethode)e.nextElement();

	  {
	    AglType at=simplifyType(am.type);
	    if(at==cc)
	    {
	      appels.put(ac,cc);
	      AglOptions.terminal.trace("Appel       : "+ac.nom.dernier());
	    }
	  }

	  for(Enumeration ep=am.parametres.elements(); ep.hasMoreElements(); )
	  {
	    AglParametre ap=(AglParametre)ep.nextElement();
	    AglType      at=simplifyType(ap.type);

	    if(at==cc)
	    {
	      appels.put(ac,cc);
	      AglOptions.terminal.trace("Appel       : "+ac.nom.dernier());
	    }
	  }
	}
      }
    }
    appels.remove(cc);

    if(AglOptions.application==null)
    {
      AglCodeUmlRelationsFrame f=new AglCodeUmlRelationsFrame(this);
      try { Thread.sleep(2000); }
      catch (InterruptedException ex) { }
    }
    else
    {
      //AlmaCodeUmlRelationsFrame f=new AlmaCodeUmlRelationsFrame(this);
      //((AlmaImplementation)AglOptions.application).addInternalFrame(f);

      ((AglTransfert)AglOptions.application).transfert(this);
    }

    r+="<HEAD>\n";
    //r+="  <META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=iso-8859-1\">\n";
    r+="  <TITLE>"+cc.nom.dernier()+" (Relations)</TITLE>\n";
    r+="  <LINK REL=\"StyleSheet\" HREF=\""+AglOptions.styleHtml+"\" TITLE=\"Style\">\n";
    r+="</HEAD>\n";
    r+="<BODY>\n";

    r+="<MAP NAME=\""+cc.nom.dernier()+"_map\">\n";
    for(Enumeration e=classes.elements(); e.hasMoreElements(); )
    {
      AglCodeUmlRelationsXY pc=(AglCodeUmlRelationsXY)e.nextElement();

      if((pc!=null)&&(pc.classe!=cc)&&(!pc.classe.elementaire))
      {
	r+="  <AREA SHAPE=\"rect\" ";
	r+="COORDS=\""+pc.x+","+pc.y+",";
	r+=(pc.x+WCI)+","+(pc.y+HCI)+"\" ";
	r+="HREF=\"_R_"+pc.classe.nom.dernier()+".html\">\n";
      }
    }
    r+="</MAP>\n";

    r+="<IMG SRC=\"_R_"+cc.nom.dernier()+".gif\" ";
    r+="BORDER=0 USEMAP=\"#"+cc.nom.dernier()+"_map\">\n";
    
    r+="</BODY>\n";
    r+="</HTML>\n";

    return new AglSortie[]
    {
      new AglSortie("text/html","Html","_R_"+AglOptions.module+".html",r),
   //new AglSortie("image/x-java-object","_R_"+AglOptions.module+".gif",image),
    };
  }
}
