/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeUxf.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import java.io.*;
import java.util.*;

public class AglCodeUxf extends AglCode
{
  public String extension()
  {
    return "xml";
  }

  public String[] extensions()
  {
    return new String[] { "xml", "XML" };
  }

  public String syntaxe()
  {
    return "Xml";
  }

  protected String debutCS  () { return indente()+"<!--";  }
  protected String finCS    () { return "-->";             }
  protected String debutCM  () { return indente()+"<!--";  }
  protected String specialCM() { return "-";               }
  protected String milieuCM () { return indente()+"  -- "; }
  protected String finCM    () { return indente()+"  -->"; }

  public String code(AglNom o)
  {
    return o.dernier();

/*
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="_";
      r+=(String)e.nextElement();
    }

    return r;
*/
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";
         if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau) r+=code(((AglTypeTableau)o).type)+"["+code(((AglTypeTableau)o).dimension)+"]";
    else r+=code(o.nom);
    return r;
  }

  public String code(AglAttribut o)
  {
    String r=indente()+"<Attribute>\n";
    indentation++;

    r+=indente()+"<Name>"+o.nom       +"</Name>\n";
    r+=indente()+"<Type>"+code(o.type)+"</Type>\n";;
    r+=code(o.visibilite);

    if(o.initiale!=null)
      r+=indente()+"<InitialValue>"+code(o.initiale)+"</InitialValue>\n";

    indentation--;
    r+=indente()+"</Attribute>\n";
    return r;
  }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    /*
    String r="";
    r+=code(o.nom)+"."+extension()+" ["+o.ligne+"]";

    r+="typedef struct\n";
    r+="{\n";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+="  "+code(aa);
    }

    r+="} "+code(o.nom)+";\n\n";
    return r+"\n\n";
    */

    return "";
  }

  /*
  protected void ajouteAttributs(AglClasse c, AglVecteur v)
  {
    for(Enumeration ea=c.attributs.elements(); ea.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)ea.nextElement();
      v.addElement(aa);
    }

    for(Enumeration em=c.meres.noms.elements(); em.hasMoreElements(); )
    {
      AglNom    nm=(AglNom)em.nextElement();
      AglClasse cm=(AglClasse)AglType.cherche(nm);
      if(cm!=null) ajouteAttributs(cm,v);
    }
  }
  */

  public String code(AglClasse o)
  {
    String r="";

    r+=indente()+"<Class>\n";
    indentation++;

    r+=indente()+"<Name>"+o.nom.dernier()+"</Name>\n";
    if(o.visibilite.vabstract)
      r+=indente()+"<IsAbstract>true</IsAbstract>\n";

    /*
      if(o.meres.noms.size()>0)
      r+=indente()+"<Generalization FROM = \""+code(o.meres)+"\"/>\n";
      if(o.interfaces.noms.size()>0)
      r+=" implements "+code(o.interfaces);
      if(o.exceptions.noms.size()>0)
      r+=" throws "+code(o.exceptions);
    */

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    indentation--;
    r+=indente()+"</Class>\n";
    return r;
  }

  public String code(AglConstructeur o)
  {
    return "";
  }

  public String code(AglImport o)
  {
    /*
    String r="#include <"+o.nom.dernier();
    r+=".hpp>\n";
    return r;
    */

    return "";
  }

  public String code(AglMethode o)
  {
    String r=indente()+"<Operation>\n";
    indentation++;

    r+=indente()+"<Name>"+o.nom+"</Name>\n";
    r+=code(o.visibilite);
    r+=indente()+"<ReturnType>"+code(o.type)+"</ReturnType>\n";;

    indentation++;
    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      r+=code(ap);
    }
    indentation--;

    indentation--;
    r+=indente()+"</Operation>\n";
    return r;
  }

  public String code(AglParametre o)
  {
    String r=indente()+"<Parameter>\n";
    indentation++;

    r+=indente()+"<Name>"+o.nom       +"</Name>\n";
    r+=indente()+"<Type>"+code(o.type)+"</Type>\n";;

    indentation--;
    r+=indente()+"</Parameter>\n";
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";

         if(o.vprivate)      r+="private";
    else if(o.vprotected)    r+="protected";
    else r+="public";
    r=indente()+"<Visibility>"+r+"</Visibility>\n";

    // if(o.vabstract)     r+="virtual ";
    // if(o.vinline)       r+="inline ";
    // if(o.vstatic)       r+="static ";
    // if(o.vfinal)        r+="final ";
    // if(o.vnative)       r+="native ";
    // if(o.vtransient)    r+="register ";
    // if(o.vsynchronized) r+="synchronized ";

    return r;
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r=" ("+code(o.type)+"*)malloc(1,sizeof("+code(o.type)+"))";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r=" ("+code(o.type)+"*)malloc("+code(o.nombre)+",sizeof("+code(o.type)+")))";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }
  
  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+="<?xml version=\"1.0\"?>\n";
    r+="<!DOCTYPE UXF SYSTEM \"uxf/uml.dtd\">\n";

    String titre=o.fichier;
    int    ii=titre.lastIndexOf(File.separator);
    titre=titre.substring(ii+1);

    r+=entete(o.fichier,"http://www.yy.cs.keio.ac.jp/~suzuki/project/uxf/");

    indentation=0;
    r+=indente()+"<UXF Version=\"2.0\"\n";
    r+=indente()+"     xmlns:UXF=\"http://www.yy.cs.keio.ac.jp/~suzuki/project/uxf/\">\n";
    indentation++;

    r+=indente()+"<UXF:TaggedValue>\n";
    r+=indente()+"  <UXF:Tag>Model Name</UXF:Tag><UXF:Value>Model "+titre+"</UXF:Value>\n";
    r+=indente()+"  <UXF:Tag>Author</UXF:Tag><UXF:Value>"+System.getProperty("user.name")+"</UXF:Value>\n";
    r+=indente()+"  <UXF:Tag>Date</UXF:Tag><UXF:Value>"+(new Date())+"</UXF:Value>\n";
    r+=indente()+"  <UXF:Tag>Version</UXF:Tag><UXF:Value>1.0</UXF:Value>\n";
    r+=indente()+"</UXF:TaggedValue>\n";

    r+=indente()+"<Package>\n";
    indentation++;
    r+=indente()+"<Name>Package "+titre+"</Name>\n";

    r+=indente()+"<ClassDiagram>\n";
    indentation++;
    r+=indente()+"<Name>Diagram "+titre+"</Name>\n";

    // Classes
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
          r+=code(ac);
      }
    }

    // Generalisations
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire)
        {
	  for(Enumeration m=ac.meres.noms.elements(); m.hasMoreElements(); )
	  {
	    r+=indente()+"<Generalization>\n";
	    indentation++;
	    r+=indente()+"<Parent>"+code((AglNom)m.nextElement())+"</Parent>\n";
	    r+=indente()+"<Child>" +code(ac.nom)                 +"</Child>\n";
	    indentation--;
	    r+=indente()+"</Generalization>\n\n";
	  }
	}
      }
    }

    indentation--;
    r+=indente()+"</ClassDiagram>\n";
    indentation--;
    r+=indente()+"</Package>\n";
    indentation--;
    r+=indente()+"</UXF>\n";
    indentation--;

    return r;
  }
}
