/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeXmi.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

/**
 * Generateur XMI plus ou moins conforme.
 * Dependant de uml.dtd fournie par IBM, version 1.24, 8/10/1998.
 */
public class AglCodeXmi extends AglCode
{
  public String extension()
  {
    return "xmi";
  }

  public String[] extensions()
  {
    return new String[] { "xmi", "XMI" };
  }

  public String syntaxe()
  {
    return "Xml";
  }

  protected String debutCS  () { return indente()+"<!--";  }
  protected String finCS    () { return "-->";             }
  protected String debutCM  () { return indente()+"<!--";  }
  protected String specialCM() { return "-";               }
  protected String milieuCM () { return indente()+"  -- "; }
  protected String finCM    () { return indente()+"  -->"; }

  private Hashtable ids_=new Hashtable();
  private String id(Object _o)
  {
    String r=(String)ids_.get(_o);
    if(r==null)
    {
      r="00000000"+(ids_.size()+1);
      r="A"+r.substring(r.length()-8);
      ids_.put(_o,r);
      
    }
    return r;
  }

  public String code(AglNom o)
  {
    return o.dernier();

/*
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="_";
      r+=(String)e.nextElement();
    }

    return r;
*/
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";

    if(o instanceof AglTypeAlias)
      r+=code(((AglTypeAlias)o).type);
    else
    if(o instanceof AglTypeTableau)
      r+=code(((AglTypeTableau)o).type)+
	"["+code(((AglTypeTableau)o).dimension)+"]";
    else
    //if(o instanceof AglTypePrimitif) r+="<"+o.nom.dernier()+"/>";
    //else
      r+="<XMI.reference target=\""+id(o)+"\"/>";

    return r;
  }

  public String code(AglAttribut o)
  {
    String r="";

    r+=indente()+"<feature>\n";
    indentation++;
    r+=indente()+"<Attribute>\n";

    indentation++;
    r+=indente()+"<name>"+o.nom+"</name>\n";
    r+=indente()+codeVisibility(o.visibilite)+"\n";

    indentation++;
    r+=indente()+"<ownerScope XMI.value=\"";
    r+=(o.visibilite.vstatic ? "classifier" : "instance");
    r+="\"/>\n";

    r+=indente()+"<multiplicity>1</multiplicity>\n";
    
    r+=indente()+"<changeable XMI.value=\"none\"/>\n";
    //    r+=indente()+"<targetScope XMI.value=\"instance\"/>\n";
    indentation--;

    r+=indente()+"<type>"+code(o.type)+"</type>\n";;

    if(o.initiale!=null)
      r+=indente()+"<initialValue>"+code(o.initiale)+"</initialValue>\n";

    indentation--;
    r+=indente()+"</Attribute>\n";
    indentation--;
    r+=indente()+"</feature>\n";
    return r;
  }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    String r="";

    r+=indente()+"<Class XMI.id=\""+id(o)+"\">\n";
    indentation++;

    r+=indente()+"<name>"+code(o.nom)+"</name>\n";
    r+=code(o.visibilite);
    r+=indente()+"<isRoot XMI.value=\"true\"/>\n";
    r+=indente()+"<isLeaf XMI.value=\"true\"/>\n";
    r+=indente()+"<isActive XMI.value=\"false\"/>\n";

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation--;
    r+=indente()+"</Class>\n";
    return r;
  }

  /*
  protected void ajouteAttributs(AglClasse c, AglVecteur v)
  {
    for(Enumeration ea=c.attributs.elements(); ea.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)ea.nextElement();
      v.addElement(aa);
    }

    for(Enumeration em=c.meres.noms.elements(); em.hasMoreElements(); )
    {
      AglNom    nm=(AglNom)em.nextElement();
      AglClasse cm=(AglClasse)AglType.cherche(nm);
      if(cm!=null) ajouteAttributs(cm,v);
    }
  }
  */

  public String code(AglClasse o)
  {
    String r="";

    r+=indente()+"<";
    if(o.normale) r+="Class";
    else          r+="Interface";

    r+=" XMI.id=\""+id(o)+"\">\n";
    indentation++;

    r+=indente()+"<name>"+code(o.nom)+"</name>\n";
    r+=code(o.visibilite);

    r+=indente()+"<isRoot XMI.value=\"";
    r+=(o.meres.noms.size()==0 ? "true" : "false");
    r+="\"/>\n";

    r+=indente()+"<isLeaf XMI.value=\"";
    r+=(o.visibilite.vfinal ? "true" : "false");
    r+="\"/>\n";

    r+=indente()+"<isActive XMI.value=\"false\"/>\n";

    if(o.meres.noms.size()>0)
    {
      Enumeration em=o.meres.noms.elements();
      while(em.hasMoreElements())
      {
	AglType mt=AglType.cherche((AglNom)em.nextElement());
	  
	if(mt!=null)
	{
	  r+=indente()+"<generalization>";
	  r+="<XMI.reference target=\""+
	      id(mt)+"\"/></generalization>\n";
	  r+=indente()+"<!-- generalization is: "+code(mt.nom)+" -->\n";
	}
      }
    }

    // realization is missing.

    if(o.interfaces.noms.size()>0)
    {
      Enumeration ei=o.meres.noms.elements();
      while(ei.hasMoreElements())
      {
	AglType it=AglType.cherche((AglNom)ei.nextElement());
	  
	if(it!=null)
	{
	  r+=indente()+"<specification>";
	  r+="<XMI.reference target=\""+
	      id(it)+"\"/></specification>\n";
	  r+=indente()+"<!-- specification is: "+code(it.nom)+" -->\n";
	}
      }
    }

    /*
      if(o.exceptions.noms.size()>0)
      r+=" throws "+code(o.exceptions);
    */

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    for(Enumeration e=o.constructeurs.elements(); e.hasMoreElements(); )
    {
      AglConstructeur am=(AglConstructeur)e.nextElement();
      r+=code(am);
    }

    for(Enumeration e=o.methodes.elements(); e.hasMoreElements(); )
    {
      AglMethode am=(AglMethode)e.nextElement();
      r+=code(am);
    }

    indentation--;
    r+=indente()+"</Class>\n";
    return r;
  }

  public String code(AglConstructeur o)
  {
    return "";
  }

  public String code(AglImport o)
  {
    /*
    String r="#include <"+o.nom.dernier();
    r+=".hpp>\n";
    return r;
    */

    return "";
  }

  public String code(AglMethode o)
  {
    String r="";

    r+=indente()+"<feature>\n";
    indentation++;
    r+=indente()+"<Operation>\n";
    indentation++;

    r+=indente()+"<name>"+o.nom+"</name>\n";
    r+=code(o.visibilite);

    // et static (class) ?
    r+=indente()+"<ownerScope XMI.value=\"instance\"/>\n";

    // champe requete dans Methode ou commentaire
    r+=indente()+"<isQuery XMI.value=\"false\"/>\n";

    // concurrent ?
    r+=indente()+"<concurrency XMI.value=\"";
    r+=(o.visibilite.vsynchronized ? "guarded" : "sequential");
    r+="\"/>\n";

    r+=indente()+"<isPolymorphic XMI.value=\""+
      o.visibilite.vvirtual+"\"/>\n";

    // r+=indente()+"<specification/>\n";
    // operationSpecification ???

    r+=indente()+"<parameter>\n";
    indentation++;

    for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
    {
      AglParametre ap=(AglParametre)e.nextElement();
      r+=code(ap);
    }

    // return type
    r+=indente()+"<Parameter XMI.id=\""+id(o)+"\">\n";
    indentation++;
    r+=indente()+"<name>return</name>\n";
    r+=indente()+"<visibility XMI.value=\"public\"/>\n";
    r+=indente()+"<defaultValue>\n";
    r+=indente()+"</defaultValue>\n";
    r+=indente()+"<kind XMI.value=\"return\"/>\n";
    r+=indente()+"<type>"+code(o.type)+"</type>\n";;
    indentation--;
    r+=indente()+"</Parameter>\n";

    indentation--;
    r+=indente()+"</parameter>\n";

    indentation--;
    r+=indente()+"</Operation>\n";
    indentation--;
    r+=indente()+"</feature>\n";
    return r;
  }

  public String code(AglParametre o)
  {
    String r=indente()+"<Parameter XMI.id=\""+id(o)+"\">\n";
    indentation++;
    r+=indente()+"<name>"+o.nom+"</name>\n";
    r+=indente()+codeVisibility(o.visibilite)+"\n";

    r+=indente()+"<defaultValue>\n";
    indentation++;
    if(o.defaut!=null) r+=indente()+code(o.defaut)+"\n";
    indentation--;
    r+=indente()+"</defaultValue>\n";

    r+=indente()+"<kind XMI.value=\"";
    switch(o.passage)
    {
    case AglParametre.OUT  : r+="in";    break;
    case AglParametre.INOUT: r+="inout"; break;
    default:                 r+="in";    break;
    }
    r+="\"/>\n";

    r+=indente()+"<type>"+code(o.type)+"</type>\n";;

    indentation--;
    r+=indente()+"</Parameter>\n";
    return r;
  }

  public String code(AglVisibilite o)
  {
    String r="";
    r+=indente()+codeVisibility(o)+"\n";
    r+=indente()+codeIsAbstract(o)+"\n";
    return r;
  }

  public String codeVisibility(AglVisibilite o)
  {
    String r="<visibility XMI.value=\"";
         if(o.vprivate)   r+="private";
    else if(o.vprotected) r+="protected";
    else r+="public";
    r+="\"/>";
    return r;
  }

  public String codeIsAbstract(AglVisibilite o)
  {
    String r="<isAbstract XMI.value=\"";
         if(o.vabstract)  r+="true";
    else r+="false";
    r+="\"/>";
    return r;
  }


// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r=" ("+code(o.type)+"*)malloc(1,sizeof("+code(o.type)+"))";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r=" ("+code(o.type)+"*)malloc("+code(o.nombre)+",sizeof("+code(o.type)+")))";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }
  
  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+="<?xml version=\"1.0\"  encoding=\"ISO-8859-1\"?>\n";
    r+="<!DOCTYPE XMI SYSTEM \"uml.dtd\">\n";

    r+=entete(o.fichier,"http://www.software.ibm.com/ad/features/xmi.html");

    r+="<XMI xmi-version=\"1.0\">\n";
    indentation++;

    r+=indente()+"<XMI.header>\n";
    indentation++;
    r+=indente()+"<XMI.documentation>\n";
    indentation++;
    r+=indente()+"<XMI.exporter>Alma</XMI.exporter>\n";
    r+=indente()+"<XMI.exporterVersion>"+
      AglOptions.VERSION+"</XMI.exporterVersion>\n";
    indentation--;
    r+=indente()+"</XMI.documentation>\n";
    r+=indente()+"<XMI.metamodel name=\"uml\" version=\"1.1\"/>\n";
    indentation--;
    r+=indente()+"</XMI.header>\n";

    r+=indente()+"<XMI.content>\n";
    indentation++;

    r+=indente()+"<Model XMI.id=\""+id(o)+"\">\n";

    indentation++;
    r+=indente()+"<name>"+o.fichier+"</name>\n";
    r+=indente()+"<visibility XMI.value=\"public\"/>\n";
    r+=indente()+"<isRoot     XMI.value=\"false\"/>\n";
    r+=indente()+"<isLeaf     XMI.value=\"false\"/>\n";
    r+=indente()+"<isAbstract XMI.value=\"false\"/>\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(/*(o==at.proprietaire)&&*/(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        //if(!ac.elementaire&&filtre(ac.nom))
	{
	  r+=indente()+"<ownedElement>\n";
	  indentation++;
          r+=code(ac);
	  indentation--;
	  r+=indente()+"</ownedElement>\n";
	}
      }
      // DataType missing...
    }

    /*
    Enumeration ec=AglType.table.elements();
    while(ec.hasMoreElements())
    {
      AglType at=(AglType)ec.nextElement();
      if(at instanceof AglClasse)
      {
        AglClasse ac=(AglClasse)at;

	Enumeration em=ac.meres.noms.elements();
	while(em.hasMoreElements())
	{
	  AglType st=AglType.cherche((AglNom)em.nextElement());
	  
	  if(st!=null)
	  {
	    r+=indente()+"<ownedElement>\n";
	    indentation++;
	    r+=indente()+"<Generalization>\n";
	    indentation++;
	    r+=indente()+"<subtype><XMI.reference target=\""+
	      id(ac)+"\"/></subtype>\n";
	    r+=indente()+"<!-- subtype is: "+code(ac.nom)+" -->\n";
	    r+=indente()+"<supertype><XMI.reference target=\""+
	      id(st)+"\"/></supertype>\n";
	    r+=indente()+"<!-- supertype is: "+code(st.nom)+" -->\n";
	    indentation--;
	    r+=indente()+"</Generalization>\n";
	    indentation--;
	    r+=indente()+"</ownedElement>\n";
	  }
	}
      }
    }
    */

    indentation--;
    r+=indente()+"</Model>\n";

    indentation--;
    r+=indente()+"</XMI.content>\n";
    indentation--;
    r+=indente()+"</XMI>\n";

    return r;
  }
}
