/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCodeXsd.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

/**
 * Generateur XSD plus ou moins conforme.
 * Dependant de uml.dtd fournie par IBM, version 1.24, 8/10/1998.
 */
public class AglCodeXsd extends AglCode
{
  public String extension()
  {
    return "xsd";
  }

  public String[] extensions()
  {
    return new String[] { "xsd", "XSD" };
  }

  public String syntaxe()
  {
    return "Xml";
  }

  protected String debutCS  () { return indente()+"<!--";  }
  protected String finCS    () { return "-->";             }
  protected String debutCM  () { return indente()+"<!--";  }
  protected String specialCM() { return "-";               }
  protected String milieuCM () { return indente()+"  -- "; }
  protected String finCM    () { return indente()+"  -->"; }

  /*
  private Hashtable ids_=new Hashtable();
  private String id(Object _o)
  {
    String r=(String)ids_.get(_o);
    if(r==null)
    {
      r="00000000"+(ids_.size()+1);
      r="A"+r.substring(r.length()-8);
      ids_.put(_o,r);
      
    }
    return r;
  }
  */

  public String code(AglNom o)
  {
    return o.dernier();

    /*
    String r="";
    boolean p=true;

    for(Enumeration e=o.identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="_";
      r+=(String)e.nextElement();
    }

    return r;
    */
  }

  public String code(AglListeNoms o)
  {
    String r="";
    boolean p=true;

    for(Enumeration e=o.noms.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=", ";
      r+=code((AglNom)e.nextElement());
    }

    return r;
  }

  public String code(AglType o)
  {
    String r="";

    if(o instanceof AglTypeAlias)
	o=AglTypeAlias.simplifie(o,langage());

    if(o instanceof AglTypePrimitif)
      r+=code("xsd:"+code(o.nom));
    else
    if(o==AglType.cherche("String"))
	r+="xsd:string";
    else
    if(o instanceof AglTypeTableau)
      r+=code(((AglTypeTableau)o).type)+
	"["+code(((AglTypeTableau)o).dimension)+"]";
    else
      r+=code(o.nom).replace('_',':');

    return r;
  }

  public String code(AglAttribut o)
  {
    String r="";

    if(!o.visibilite.vstatic)
    {
      r+=indente()+"<xsd:element"+
	" name=\""+o.nom+"\""+
	" type=\""+code(o.type)+"\"";
      if(o.initiale!=null)
        r+=" fixed="+code(o.initiale);
      r+=" />\n";
    }
    else
    {
      r+=indente()+"<xsd:"+o.nom.toLowerCase();
      if(o.initiale!=null)
        r+=" value="+code(o.initiale);
      r+=" />\n";
    }

    return r;
  }

  public String code(AglBloc o)
    { return ""; }

  public String code(AglStruct o)
  {
    String r="";

    r+=indente()+"<xsd:complexType";
    r+=" name=\""+code(o.nom)+"\">\n";
    indentation++;
    r+=indente()+"<xsd:sequence>\n";
    indentation++;

    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }

    indentation--;
    r+=indente()+"</xsd:sequence>\n";
    indentation--;
    r+=indente()+"</xsd:complexType>\n";
    return r;
  }

  /*
  protected void ajouteAttributs(AglClasse c, AglVecteur v)
  {
    for(Enumeration ea=c.attributs.elements(); ea.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)ea.nextElement();
      v.addElement(aa);
    }

    for(Enumeration em=c.meres.noms.elements(); em.hasMoreElements(); )
    {
      AglNom    nm=(AglNom)em.nextElement();
      AglClasse cm=(AglClasse)AglType.cherche(nm);
      if(cm!=null) ajouteAttributs(cm,v);
    }
  }
  */

  public String code(AglClasse o)
  {
    String r="";

    r+=indente()+"<xsd:";
    if(o.complexe) r+="complexType";
    else           r+="simpleType";
    r+=" name=\""+code(o.nom)+"\">\n";

    indentation++;
    if(o.complexe)
      r+=indente()+"<xsd:sequence>\n";
    else
    if(o.meres.noms.size()>0)
    {
      String n=code(o.meres.noms.elementAt(0));
      n=n.replace('_',':');
      r+=indente()+"<xsd:restriction base=\""+n+"\">\n";
    }

    indentation++;
    for(Enumeration e=o.attributs.elements(); e.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)e.nextElement();
      r+=code(aa);
    }
    indentation--;

    if(o.complexe) r+=indente()+"</xsd:sequence>\n";
    else           r+=indente()+"</xsd:restriction>\n";
    indentation--;

    r+=indente()+"</xsd:";
    if(o.complexe) r+="complexType";
    else           r+="simpleType";
    r+=">\n";
    return r;
  }

  public String code(AglConstructeur o)
  {
    return "";
  }

  public String code(AglImport o)
  {
    /*
    String r="#include <"+o.nom.dernier();
    r+=".hpp>\n";
    return r;
    */

    return "";
  }

  public String code(AglMethode o)
  {
    return "";
  }

  public String code(AglParametre o)
  {
    return "";
  }

  public String code(AglVisibilite o)
  {
    return "";
  }

// Expressions

  public String code(AglExpressionGenerique o)
  {
    String r=o.body;
    return r;
  }

  public String code(AglExpressionAffectation o)
  {
    String r="("+code(o.gauche)+"="+code(o.droite)+")";
    return r;
  }

  public String code(AglExpressionAttribut o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.attribut;
    return r;
  }

  public String code(AglExpressionBinaire o)
  {
    String r=code(o.gauche)+o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionDefVar o)
  {
    String r=code(o.type)+" "+o.nom;
    if(o.initiale!=null) r+="="+code(o.initiale);
    return r;
  }

  public String code(AglExpressionEstUn o)
  {
    String r=code(o.expression)+" instanceof "+code(o.type);
    return r;
  }

  public String code(AglExpressionFonction o)
  {
    String r=o.nom+"(";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionIndice o)
  {
    String r=code(o.objet)+"["+code(o.indice)+"]";
    return r;
  }

  public String code(AglExpressionInitArray o)
  {
    String r="{ ";

    boolean p=true;
    for(Enumeration e=o.initiales.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=" }";
    return r;
  }

  public String code(AglExpressionMethode o)
  {
    String r="";
    if(o.objet!=null) r+=code(o.objet)+".";
    r+=o.fonction;
    return r;
  }

  public String code(AglExpressionMultiple o)
  {
    String r="";

    boolean p=true;
    for(Enumeration e=o.expressions.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    return r;
  }

  public String code(AglExpressionNew o)
  {
    String r=" ("+code(o.type)+"*)malloc(1,sizeof("+code(o.type)+"))";

    boolean p=true;
    for(Enumeration e=o.arguments.elements(); e.hasMoreElements(); )
    {
      AglExpression ae=(AglExpression)e.nextElement();
      if(!p) r+=", ";
      r+=code(ae);
      p=false;
    }

    r+=")";
    return r;
  }

  public String code(AglExpressionNewArray o)
  {
    String r=" ("+code(o.type)+"*)malloc("+code(o.nombre)+",sizeof("+code(o.type)+")))";
    return r;
  }

  public String code(AglExpressionParenthesee o)
  {
    String r="( "+code(o.expression)+" )";
    return r;
  }

  public String code(AglExpressionTernaire o)
  {
    String r=code(o.gauche)
      +" "+o.operateur1+" "+code(o.milieu)
      +" "+o.operateur2+" "+code(o.droite);
    return r;
  }

  public String code(AglExpressionTypage o)
  {
    String r="("+code(o.type)+")"+code(o.expression);
    return r;
  }

  public String code(AglExpressionUnaireDroite o)
  {
    String r=o.operateur+code(o.droite);
    return r;
  }

  public String code(AglExpressionUnaireGauche o)
  {
    String r=code(o.gauche)+o.operateur;
    return r;
  }

// Instructions

  public String code(AglInstructionGenerique o)
    { return ""; }

  public String code(AglInstructionVide o)
    { return ""; }

  public String code(AglInstructionAffectation o)
    { return ""; }

  public String code(AglInstructionBloc o)
    { return ""; }

  public String code(AglInstructionBreak o)
    { return ""; }

  public String code(AglInstructionCatch o)
    { return ""; }

  public String code(AglInstructionContinue o)
    { return ""; }

  public String code(AglInstructionDefVar o)
    { return ""; }

  public String code(AglInstructionDoWhile o)
    { return ""; }

  public String code(AglInstructionExpression o)
    { return ""; }

  public String code(AglInstructionFinally o)
    { return ""; }

  public String code(AglInstructionGoto o)
    { return ""; }

  public String code(AglInstructionFor o)
    { return ""; }

  public String code(AglInstructionIf o)
    { return ""; }

  public String code(AglInstructionLabel o)
    { return ""; }

  public String code(AglInstructionProcedure o)
    { return ""; }

  public String code(AglInstructionReturn o)
    { return ""; }

  public String code(AglInstructionSwitch o)
    { return ""; }

  public String code(AglInstructionSynchronized o)
    { return ""; }

  public String code(AglInstructionThrow o)
    { return ""; }

  public String code(AglInstructionTry o)
    { return ""; }

  public String code(AglInstructionWhile o)
    { return ""; }

// Unite

  public String code(AglUnite o)
  {
    String r="";
    indentation=0;

    r+="<?xml version=\"1.0\"  encoding=\"ISO-8859-1\"?>\n";

    r+=entete(o.fichier,"http://www.w3.org/2001/XMLSchema");

    r+="<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"\n"
	+"  targetNamespace=\"http://www.memoire.com/XSD\">\n";
    indentation++;

    r+=indente()+"<xsd:annotation>\n";
    indentation++;
    r+=indente()+"<xsd:documentation>\n";
    indentation++;
    r+=indente()+o.fichier+"\n";
    indentation--;
    r+=indente()+"</xsd:documentation>\n";
    indentation--;
    r+=indente()+"</xsd:annotation>\n";

    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(/*(o==at.proprietaire)&&*/(at instanceof AglClasse))
      {
        AglClasse ac=(AglClasse)at;
        if(!ac.elementaire) //&&filtre(ac.nom))
	{
          r+=code(ac);
	}
      }
      // DataType missing...
    }

    indentation--;
    r+=indente()+"</xsd:schema>\n";

    return r;
  }
}
