/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglCondition.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import java.util.*;

// Preconditions, postconditions et invariants.

public class AglCondition extends AglObjet
{
  public AglExpression test;
  public String        langage;
  public boolean       pre;
  public boolean       post;

  public AglCondition() // yapod
  {
    this(null);
  }

  public AglCondition(AglExpression _test)
  {
    AglOptions.terminal.trace("Create condition: "+_test);
    test=_test;
  }

  public String toString()
  {
    return "CONDITION("+test+")";
  }

  public static void construireConditions()
  {
    int p=0;
    for(Enumeration e1=AglType.table.elements(); e1.hasMoreElements(); )
    {
      p++;
      AglOptions.terminal.progres(100*p/AglType.table.size());

      Object oc=e1.nextElement();
      if(oc instanceof AglClasse)
      {
	AglClasse c=(AglClasse)oc;
	if(c.normale)
	{
	  // enleve toutes les conditions existantes
	  c.invariants.removeAllElements();
	  enleveConditions(c.clinit);
	  enleveConditions(c.obinit);

	  // ajoute invariants
	  if(c.commentaires!=null)
	  for(Enumeration e5=c.commentaires.elements();
	      e5.hasMoreElements(); )
	  {
	    Object ocm=e5.nextElement();
	    if(ocm instanceof AglCommentaireMulti)
	    {
	      AglCommentaireMulti cm=(AglCommentaireMulti)ocm;
	      AglVecteur pc=(AglVecteur)cm.clefs.get("@invariant");
	      if(pc!=null)
	      {
		for(int j=0;j<pc.size();j++)
		{
		  String sc=(String)pc.elementAt(j);;
		  sc=sc.trim();
		  AglExpression ec=new AglExpressionGenerique(sc);
		  ec.ligne=cm.ligne;
		  ec.proprietaire=cm.proprietaire;

		  AglCondition  cc=new AglCondition(ec);
		  cc.pre =true;
		  cc.post=true;
		  cc.ligne=ec.ligne;
		  cc.proprietaire=ec.proprietaire;
		  System.err.println(cc);
		  c.invariants.addElement(cc);
		}
	      }
	    }
	  }

	  for(Enumeration e2=c.methodes.elements(); e2.hasMoreElements(); )
	  {
	    AglMethode m=(AglMethode)e2.nextElement();

	    // enleve toutes les conditions existantes
	    m.preconditions.removeAllElements();
	    m.postconditions.removeAllElements();
	    enleveConditions(m.corps);
	    /*
	    if(m.corps!=null)
	    {
	      for(int j=m.corps.instructions.size()-1;j>=0;j--)
	      {
		AglInstruction i=(AglInstruction)
		  m.corps.instructions.elementAt(j);
		if(i instanceof AglInstructionCondition)
		{
		  // System.err.println("remove "+i);
		  m.corps.instructions.removeElementAt(j);
		}
	      }
	    }
	    */

	    // ajoute preconditions
	    if(m.corps!=null)
	    {
	      if(m.commentaires!=null)
	      for(Enumeration e3=m.commentaires.elements();
		  e3.hasMoreElements(); )
	      {
		Object ocm=e3.nextElement();
		if(ocm instanceof AglCommentaireMulti)
		{
		  AglCommentaireMulti cm=(AglCommentaireMulti)ocm;
		  AglVecteur pc=(AglVecteur)cm.clefs.get("@require");
		  if(pc!=null)
		  {
		    for(int j=0;j<pc.size();j++)
		    {
		      String sc=(String)pc.elementAt(j);;
		      sc=sc.trim();
		      AglExpression ec=new AglExpressionGenerique(sc);
		      ec.ligne=cm.ligne;
		      ec.proprietaire=cm.proprietaire;

		      AglCondition cc=new AglCondition(ec);
		      cc.pre=true;
		      cc.ligne=ec.ligne;
		      cc.proprietaire=ec.proprietaire;
		      m.preconditions.addElement(cc);

		      AglInstructionCondition ic=
			  new AglInstructionCondition(cc);
		      ic.ligne=cc.ligne;
		      ic.proprietaire=cc.proprietaire;
		      int i=0;
		      /*
			while(  (i<m.corps.instructions.size())
			      &&(m.corps.instructions.elementAt(i)
			       instanceof AglInstructionDefVar)) i++;
		      */
		      m.corps.instructions.insertElementAt(ic,i);
		    }
		  }
		}
	      }

	      if(!m.visibilite.vstatic)
		for(Enumeration e6=c.invariants.elements();
		    e6.hasMoreElements(); )
		{
		  int i=0;
		  /*
		  while(  (i<m.corps.instructions.size())
			&&(m.corps.instructions.elementAt(i)
			   instanceof AglInstructionDefVar)) i++;
		  */
		  AglCondition  cc=(AglCondition)e6.nextElement();
		  AglInstructionCondition ic=new AglInstructionCondition(cc);
		  ic.ligne=cc.ligne;
		  ic.proprietaire=cc.proprietaire;
		  m.corps.instructions.insertElementAt(ic,i);
	      }
	    }

	    // ajoute postconditions
	    if(m.corps!=null)
	    {
	      if(m.commentaires!=null)
	      for(Enumeration e4=m.commentaires.elements();
		  e4.hasMoreElements(); )
	      {
		Object ocm=e4.nextElement();
		if(ocm instanceof AglCommentaireMulti)
		{
		  AglCommentaireMulti cm=(AglCommentaireMulti)ocm;
		  AglVecteur pc=(AglVecteur)cm.clefs.get("@ensure");
		  if(pc!=null)
		  {
		    for(int j=0;j<pc.size();j++)
		    {
		      String sc=(String)pc.elementAt(j);;
		      sc=sc.trim();
		      AglExpression ec=new AglExpressionGenerique(sc);
		      ec.ligne=cm.ligne;
		      ec.proprietaire=cm.proprietaire;

		      AglCondition  cc=new AglCondition(ec);
		      cc.post=true;
		      cc.ligne=ec.ligne;
		      cc.proprietaire=ec.proprietaire;
		      m.postconditions.addElement(cc);

		      AglInstructionCondition ic=
			new AglInstructionCondition(cc);
		      int i=m.corps.instructions.size();
		      while((i>0)&&(m.corps.instructions.elementAt(i-1)
				    instanceof AglInstructionReturn)) i--;
		      m.corps.instructions.insertElementAt(ic,i);
		    }
		  }
		}
	      }

	      if(!m.visibilite.vstatic)
		for(Enumeration e7=c.invariants.elements();
		    e7.hasMoreElements(); )
		{
		  int i=m.corps.instructions.size();
		  while((i>0)&&(m.corps.instructions.elementAt(i-1)
				instanceof AglInstructionReturn)) i--;
		  AglCondition cc=(AglCondition)e7.nextElement();
		  AglInstructionCondition ic=new AglInstructionCondition(cc);
		  ic.ligne=cc.ligne;
		  ic.proprietaire=cc.proprietaire;
		  m.corps.instructions.insertElementAt(ic,i);
		}
	    }
	  }
	}
      }
    }
  }

  private static void enleveConditions(AglBloc _bloc)
  {
    if(_bloc!=null)
    {
      for(int j=_bloc.instructions.size()-1;j>=0;j--)
      {
	AglInstruction i=(AglInstruction)
	    _bloc.instructions.elementAt(j);
	if(i instanceof AglInstructionCondition)
	{
	  // System.err.println("remove "+i);
	  _bloc.instructions.removeElementAt(j);
	}
	else
	if(i instanceof AglInstructionBloc)
	  enleveConditions(((AglInstructionBloc)i).corps);
	else
	if(i instanceof AglInstructionCatch)
	  enleveConditions(((AglInstructionCatch)i).reprise);
	else
	if(i instanceof AglInstructionDoWhile)
	  enleveConditions(((AglInstructionDoWhile)i).corps);
	else
	if(i instanceof AglInstructionFinally)
	  enleveConditions(((AglInstructionFinally)i).corps);
	else
	if(i instanceof AglInstructionFor)
	  enleveConditions(((AglInstructionFor)i).corps);
	else
	if(i instanceof AglInstructionIf)
	{
	  enleveConditions(((AglInstructionIf)i).sioui);
	  enleveConditions(((AglInstructionIf)i).sinon);
	}
	else
	if(i instanceof AglInstructionSwitch)
	{
	  Enumeration e=((AglInstructionSwitch)i).cas.elements();
	  while(e.hasMoreElements())
	    enleveConditions((AglBloc)e.nextElement());
	  enleveConditions(((AglInstructionSwitch)i).defaut);
	}
	else
	if(i instanceof AglInstructionSynchronized)
	  enleveConditions(((AglInstructionSynchronized)i).corps);
	else
	if(i instanceof AglInstructionTry)
	  enleveConditions(((AglInstructionTry)i).corps);
	else
	if(i instanceof AglInstructionWhile)
	  enleveConditions(((AglInstructionWhile)i).corps);
      }
    }
  }
}
