/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglNom.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglNom extends AglDeclaration
{
  public AglVecteur identificateurs;

  public AglNom()
  {
    identificateurs=new AglVecteur(0,1);
  }

  public AglNom(String _nom)
  {
    this(_nom,".");
  }

  public AglNom(String _nom, String _separateur)
  {
    this();

    if(  !"." .equals(_separateur)
       &&!"/" .equals(_separateur)
       &&!"::".equals(_separateur)
       &&!":" .equals(_separateur))
      throw new RuntimeException("namespaces should use . or / or :: or :");

    if(_nom!=null)
      while(!_nom.equals(""))
      {
	int i=_nom.indexOf(_separateur);
	if(i==0)
        {
	  _nom=_nom.substring(i+_separateur.length());
	}
	else
	if(i>0)
	{
	  identificateurs.addElement(_nom.substring(0,i));
	  _nom=_nom.substring(i+_separateur.length());
	}
	else
	{
	  identificateurs.addElement(_nom); _nom="";
	}
      }
  }

  public AglNom copie()
  {
    AglNom r=new AglNom();

    for(Enumeration e=identificateurs.elements(); e.hasMoreElements(); )
      r.identificateurs.addElement(e.nextElement());

    return r;
  }

  public AglNom parent()
  {
    AglNom r=copie();

    int l=r.identificateurs.size();
    if(l<=1) r=null;
    else     r.identificateurs.removeElementAt(l-1);

    return r;
  }

  public AglNom(AglNom p, String n)
  {
    this();

    if(p!=null)
    for(Enumeration e=p.identificateurs.elements(); e.hasMoreElements(); )
      identificateurs.addElement(e.nextElement());

    identificateurs.addElement(n);
  }

  public AglNom(AglNom p, AglNom n)
  {
    this();

    if(p!=null)
    for(Enumeration e=p.identificateurs.elements(); e.hasMoreElements(); )
      identificateurs.addElement(e.nextElement());

    for(Enumeration e=n.identificateurs.elements(); e.hasMoreElements(); )
      identificateurs.addElement(e.nextElement());
  }

  public String dernier()
  {
    String r="";
    if(identificateurs.size()>=1)
      r=(String)(identificateurs.elementAt(identificateurs.size()-1));
    return r;
  }

  public String avant_dernier()
  {
    String r="";
    if(identificateurs.size()>=2)
      r=(String)(identificateurs.elementAt(identificateurs.size()-2));
    return r;
  }

  public String formeJava()
  {
    String r="";
    boolean p=true;

    for(Enumeration e=identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+=".";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public String formeIdl()
  {
    String r="";
    boolean p=true;

    for(Enumeration e=identificateurs.elements(); e.hasMoreElements(); )
    {
      if(p) p=false; else r+="::";
      r+=(String)e.nextElement();
    }

    return r;
  }

  public final String signature()
  {
    return signature(this);
  }

  public static String signature(AglNom o)
  {
    String r="";
    if(o!=null) r=o.formeJava().replace('.','/');
    return r;
  }

  public String toString()
  {
    return "NOM("+formeJava()+")";
  }

  public int hashCode()
  {
    /*
    int r=0;
    for(Enumeration e=identificateurs.elements(); e.hasMoreElements(); )
    r^=((String)e.nextElement()).hashCode();
    return r;
    */

    return dernier().hashCode();
  }

  public boolean equals(Object o)
  {
    if(!(o instanceof AglNom)) return false;

    AglNom n=(AglNom)o;

    if(  (  identificateurs.size()==1)
       ||(n.identificateurs.size()==1))
      return dernier().equals(n.dernier());

    if(identificateurs.size()!=n.identificateurs.size()) return false;

    for(Enumeration e=identificateurs.elements(),
	            f=n.identificateurs.elements();
        e.hasMoreElements(); )
      if(!(e.nextElement().equals(f.nextElement()))) return false;

    return true;
  }

  public static void main(String[] args)
  {
    AglNom a=new AglNom(args[0]);
    AglNom b=new AglNom(args[1]);
    System.err.println("a==b->"+a.equals(b));
    System.err.println("b==a->"+b.equals(a));
    System.err.println("a.include(c)->"+a.identificateurs.contains(args[2]));
  }
}


