/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParse.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;
import java.io.*;
import java.net.*;

public abstract class AglParse
{
  public int MaxLines=0;

  public abstract String   extension();
  public abstract String[] extensions();
  public abstract String   get();
  public abstract void     put(String _s);
  public abstract int      ligne();
  public abstract String   fichier();

  public final String indente(int n)
  {
    String r="";
    for(int i=0;i<n;i++) r+="  ";
    return r;
  }

  public final String peek()
  {
    String t=get();
    put(t);
    return t;
  }

  public final String ascii2C(String s)
  {
    String r="";
    for(int i=0;i<s.length();i++)
    {
      char c=s.charAt(i);
      switch(c)
      {
        case    0: r+="\\0";  break;
        case    8: r+="\\b";  break;
        case    9: r+="\\t";  break;
        case   10: r+="\\n";  break;
        case   11: r+="\\v";  break;
        case   12: r+="\\f";  break;
        case   13: r+="\\r";  break;
        case '\\': r+="\\\\"; break;
        case '\'': r+="\\\'"; break;
        case '\"': r+="\\\""; break;
	default:
	  /*
	  if((s.charAt(i)==117)&&(i+4<s.length()))
	  {
	    System.err.println("\\U="+s.charAt(i));
	    System.err.println("117="+(char)117);
	    r+="\\u";
	    String h="0000"+Integer.toHexString
	      ( (Character.getNumericValue(s.charAt(i+1))<<12)
	       +(Character.getNumericValue(s.charAt(i+2))<< 8)
	       +(Character.getNumericValue(s.charAt(i+3))<< 4)
	       +(Character.getNumericValue(s.charAt(i+4))    ));
	    r+=h.substring(h.length()-4);
	    i+=4;
	  }
	  else
	  */
	  if(Character.isISOControl(c))
	  {
	    r+="\\";
	    String h="000"+Integer.toOctalString((int)c);
	    r+=h.substring(h.length()-3);
	  }
	  else
	  if((c>255)||(c<0))
	  {
	    r+="\\u";
	    String h="0000"+Integer.toHexString((int)c);
	    r+=h.substring(h.length()-4);
	  }
	  else
	    r+=c;
	break;
      }
    }
    return r;
  }

  private final String getLocation()
  {
    String r=" [";
    int    l=ligne();
    String f=fichier();
    if(l>=0) r+="ligne "+l;
    if(!"".equals(f)&&!"-".equals(f))
    {
      if(l>=0) r+=" ";
      r+="fichier "+f;
    }
    r+="]";
    if(" []".equals(r)) r="";
    return r;
  }

  public final void avertissement(String _s)
  {
    if(!_s.endsWith("]")) _s+=getLocation();
    AglOptions.terminal.avertissement(_s);
  }

  public final void syntaxe(String _s)
  {
    if(!_s.endsWith("]")) _s+=getLocation();
    AglOptions.terminal.syntaxe(_s);
  }

  public final void erreur(String _s)
  {
    if(!_s.endsWith("]")) _s+=getLocation(); 
    AglOptions.terminal.erreur(_s);
  }

  public final void suivi(String _s)
  {
    if(!_s.endsWith("]")) _s+=getLocation();
    AglOptions.terminal.suivi(_s);
  }

  public final void trace(String _s)
  {
    if(!_s.endsWith("]")) _s+=getLocation();
    AglOptions.terminal.trace(_s);
  }

  public final void sortie(String _s)
  { AglOptions.terminal.sortie (_s); }

  public final void progres(int _v)
  { AglOptions.terminal.progres(_v); }

  public final String langage()
  {
    String r=getClass().getName();
    int    i=r.indexOf("AglParse");
    r=r.substring(i+8);
    return r;
  }

  public String syntaxe()
  {
    return langage();
  }

  // Options

  public boolean optionTrace()
    { return true; }

  public boolean optionAnalyserCommentaires()
    { return false; }

  public final boolean optionClassesFondamentales()
  {
    return getClass().getResourceAsStream
      (langage()+"ClassesFondamentales.txt")!=null;
  }

  // Analyse

  public final static boolean disponible(String n)
  {
    boolean r=true;
    try
    {
      String p=AglParse.class.getName();
      int    i=p.indexOf("AglParse");

      p=p.substring(0,i+8);
      Class.forName(p+n);
    }
    catch(Throwable ex) { r=false; }

    return r;
  }

  public final static AglParse langage(String n)
  {
    if(n==null) return null;

    AglParse r=null;

    String p=AglParse.class.getName();
    int    i=p.indexOf("AglParse");
    p=p.substring(0,i+8);

    try
    {
      /*
	boolean gcj=true;
        GCJ-BEGIN
	gcj=false;
	GCJ-END

      if(gcj)
      {
	if("Java"   .equals(n)) r=new AglParseJava();
	if("Jsp"    .equals(n)) r=new AglParseJsp();
      }
      else
      */

      {
	Class c=Class.forName(p+n);
	//Class c=FuClassLoaderDedicated.load(p+n,AglParse.class);
	r=(AglParse) c.newInstance();
      }
    }
    catch(ClassNotFoundException ex)
    {
      AglOptions.terminal.avertissement("Classe manquante: "+p+n+".class");
      AglOptions.terminal.erreur(n+" n'est pas un langage d'analyse");
    }
    catch(Throwable th)
    {
      th.printStackTrace();
      AglCode.erreur(n+" n'est pas disponible pour l'analyse");
    }

    return r;
  }

// Parse

  /** @deprecated use initStream() instead */
  public final void parseStream(InputStream sin, String f) throws Exception
  { throw new Exception("AglParse.parseStream() is deprecated"); }

  public abstract void initStream(InputStream _input, String _fichier) throws Exception;

// Elements

  public abstract void            parseFichier()      throws Exception;
  public abstract AglUnite        parseUnite()        throws Exception;
  public abstract AglVisibilite   parseVisibilite()   throws Exception;
  public abstract AglNom          parseNom()          throws Exception;
  public abstract AglListeNoms    parseListeNoms()    throws Exception;
  public abstract AglType         parseType()         throws Exception;
  public abstract AglPaquet       parsePaquet()       throws Exception;
  public abstract AglImport       parseImport()       throws Exception;
  public abstract AglClasse       parseClasse()       throws Exception;
  public abstract AglListeNoms    parseMothers()      throws Exception;
  public abstract AglListeNoms    parseInterfaces()   throws Exception;
  public abstract AglListeNoms    parseExceptions()   throws Exception;
  public abstract AglConstructeur parseConstructeur() throws Exception;
  public abstract AglMethode      parseMethode()      throws Exception;
  public abstract AglVecteur      parseAttribut()     throws Exception;
  public abstract AglVecteur      parseParametres()   throws Exception;
  public abstract AglVecteur      parseArgs()         throws Exception;
  public abstract AglBloc         parseBloc()         throws Exception;
  public abstract AglBloc         parseBlocMultiple() throws Exception;
  public abstract AglBloc         parseBlocSimple()   throws Exception;
  public abstract AglBloc         parseBlocVirgule()  throws Exception;

// Expressions

  public abstract AglExpression              parseExpression()              throws Exception;
  public abstract AglExpressionGenerique     parseExpressionGenerique()     throws Exception;
  public abstract AglExpressionUnaireDroite  parseExpressionUnaireDroite()  throws Exception;
  public abstract AglExpressionBinaire       parseExpressionBinaire()       throws Exception;
  public abstract AglExpressionFonction      parseExpressionFonction()      throws Exception;
  public abstract AglExpression              parseExpressionNew()           throws Exception;
  public abstract AglExpression              parseExpressionTypage()        throws Exception;

// Instructions

  public abstract AglInstructionGenerique    parseInstructionGenerique()    throws Exception;
  public abstract AglInstructionBloc         parseInstructionBloc()         throws Exception;
  public abstract AglVecteur                 parseInstructionDefVar()       throws Exception;
  public abstract AglInstructionCatch        parseInstructionCatch()        throws Exception;
  public abstract AglInstructionDoWhile      parseInstructionDoWhile()      throws Exception;
  public abstract AglInstructionIf           parseInstructionIf()           throws Exception;
  public abstract AglInstructionProcedure    parseInstructionProcedure()    throws Exception;
  public abstract AglInstructionReturn       parseInstructionReturn()       throws Exception;
  public abstract AglInstructionThrow        parseInstructionThrow()        throws Exception;
  public abstract AglInstructionSwitch       parseInstructionSwitch()       throws Exception;
  public abstract AglInstructionTry          parseInstructionTry()          throws Exception;
  public abstract AglInstructionWhile        parseInstructionWhile()        throws Exception;

  public abstract AglInstructionAffectation  parseInstructionAffectation(AglExpression eg) throws Exception;
  public abstract AglInstructionExpression   parseInstructionExpression(AglExpression eg)  throws Exception;
}
