/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseCpp.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;
import java.io.*;

public class AglParseCpp extends AglParse
{
  FuStreamTokenizer st;
  Stack             pile;

  AglUnite unite_;
  AglNom   nom_;
  AglType  type_;

//  AglPaquet  paquet;
//  AglClasse     classe;
//  AglMethode    methode;
//  AglAttribut   attribut;

  public AglParseCpp()
  {
    if(AglType.cherche("FILE")==null)
      new AglTypePrimitif("FILE");
  }

  public String extension()
  {
    return "hpp";
  }

  public String[] extensions()
  {
    return new String[] { "hpp", "cpp", "HPP", "CPP", "hh", "cc", "HH", "CC" };
  }

  public String syntaxe()
  {
    return "Cpp";
  }

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    st=new FuStreamTokenizer(_input,_fichier);
    st.resetSyntax();
    st.eolIsSignificant(false);
    st.quoteChar('"');
    st.quoteChar('\'');
    st.slashSlashComments(true);
    st.slashStarComments(true);
    st.wordChars('a','z');
    st.wordChars('A','Z');
    st.wordChars('0','9');
    st.wordChars('_','_');
    st.whitespaceChars(' ',' ');
    st.whitespaceChars('\t','\t');
    st.whitespaceChars('\n','\n');
    st.whitespaceChars('\r','\r');

    pile=new Stack();
  }

  public String get()
  {
    String r="";

    if(pile.empty())
    {
      try
      {
        st.nextToken();

        switch(st.ttype)
        {
          case StreamTokenizer.TT_EOF   : r="";                break;
          case StreamTokenizer.TT_WORD  : r=st.sval;           break;
          case StreamTokenizer.TT_NUMBER: r=""+st.nval;        break;
          case '"'                      : r="\""+ascii2C(st.sval)+"\""; break;
          case '\''                     : r="'" +ascii2C(st.sval)+"'" ; break;
          default                       : r=""+(char)st.ttype; break;
        }
      }
      catch(Exception e) { erreur(""+e+" [AglParse.get]"); }

      if(r.equals("#"))
      {
	st.eolIsSignificant(true);
	while(!r.equals("\n")) r=get();
	st.eolIsSignificant(false);
	r=get();
      }

      if((!r.equals(""))&&Character.isDigit(r.charAt(0)))
      {
        if(peek().equals("."))
        {
          r+=get();
          if(Character.isDigit(peek().charAt(0)))
          {
            r+=get();
            if((peek().equals("E"))||(peek().equals("e")))
            {
	      r+=get();
              if((peek().equals("+"))||(peek().equals("-")))
                r+=get();
              r+=get();
            }
          }
        }
      }

      // notation .25 (sans 0)
      if((r.equals("("))&&(peek().equals("."))) put("0");
      if((r.equals(")"))&&(peek().equals("."))) put("0");
      if((r.equals(","))&&(peek().equals("."))) put("0");
      if((r.equals("+"))&&(peek().equals("."))) put("0");
      if((r.equals("-"))&&(peek().equals("."))) put("0");
      if((r.equals("*"))&&(peek().equals("."))) put("0");
      if((r.equals("/"))&&(peek().equals("."))) put("0");

      if((r.equals(":" ))&&(peek().equals(":"))) r+=get();
      if((r.equals("[" ))&&(peek().equals("]"))) r+=get();
      if((r.equals("-" ))&&(peek().equals(">"))) r+=get();

      if((r.equals("=" ))&&(peek().equals("="))) r+=get();
      if((r.equals("!" ))&&(peek().equals("="))) r+=get();

      if((r.equals("+" ))&&(peek().equals("="))) r+=get();
      if((r.equals("-" ))&&(peek().equals("="))) r+=get();
      if((r.equals("*" ))&&(peek().equals("="))) r+=get();
      if((r.equals("/" ))&&(peek().equals("="))) r+=get();
      if((r.equals("%" ))&&(peek().equals("="))) r+=get();

      if((r.equals("+" ))&&(peek().equals("+"))) r+=get();
      if((r.equals("-" ))&&(peek().equals("-"))) r+=get();

      if((r.equals(">" ))&&(peek().equals(">"))) r+=get();
      if((r.equals("<" ))&&(peek().equals("<"))) r+=get();
      if((r.equals(">" ))&&(peek().equals("="))) r+=get();
      if((r.equals("<" ))&&(peek().equals("="))) r+=get();
      if((r.equals(">>"))&&(peek().equals(">"))) r+=get();
      if((r.equals("<<"))&&(peek().equals("<"))) r+=get();

      if((r.equals("&" ))&&(peek().equals("&"))) r+=get();
      if((r.equals("|" ))&&(peek().equals("|"))) r+=get();

      if((r.equals("&" ))&&(peek().equals("="))) r+=get();
      if((r.equals("|" ))&&(peek().equals("="))) r+=get();
      if((r.equals("^" ))&&(peek().equals("="))) r+=get();

      if((r.equals("public"   ))&&(peek().equals(":"))) r+=get();
      if((r.equals("protected"))&&(peek().equals(":"))) r+=get();
      if((r.equals("private"  ))&&(peek().equals(":"))) r+=get();

      if(r.equals("operator")) r=operator();

      if(r.startsWith("\"")&&r.endsWith("\""))
      {
	// System.err.println("R="+r);
	// System.err.println("P="+peek());
	if(peek().startsWith("\"")&&peek().endsWith("\""))
	  r=r.substring(0,r.length()-1)+get().substring(1);
      }

      trace(">>> "+r);
    }
    else r=(String)pile.pop();

    return r;
  }

  public void put(String _s)
  {
    pile.push(_s);
  }

  private int pp=0;
  public int ligne()
  {
    int r=st.lineno();
    if(MaxLines!=0)
    {
      int p=100*r/MaxLines;
      if(p!=pp) progres(p);
      pp=p;
    }
    return r;
  }

  public String fichier()
  {
    String r=st.origin();
    int i=r.lastIndexOf(File.separator);
    if(i>=0) r=r.substring(i+1);
    return r;
  }

  // Parse

  private boolean isGlobal(AglClasse _classe)
  {
    return _classe==AglType.cherche("Global");
  }

  private boolean isConstructeur()
  {
    String p1=get(); // class
    String p2=get(); // ::
    String p3=get(); // class

    boolean r=true;

    if(!p2.equals("::"))    r=false;
    if(!peek().equals("(")) r=false;
    if(!p1.equals(p3)) r=false;

    put(p3);
    put(p2);
    put(p1);

    if(r&&(AglType.cherche(p3)==null))
      new AglClasse(new AglNom(p3)).elementaire=false;

    // trace("isConstructeur: peek="+p1+p2+p3);
    // trace("---> "+r);
    return r;
  }

  private boolean isDestructeur()
  {
    String p1=get(); // class
    String p2=get(); // ::
    String p3=get(); // ~
    String p4=get(); // class

    boolean r=true;

    if(!p2.equals("::"))    r=false;
    if(!p3.equals("~"))     r=false;
    if(!peek().equals("(")) r=false;
    if(!p1.equals(p4))      r=false;

    put(p4);
    put(p3);
    put(p2);
    put(p1);

    if(r&&(AglType.cherche(p4)==null))
      new AglClasse(new AglNom(p4)).elementaire=false;

    // trace("isDestructeur: peek="+p1+p2+p3+p4);
    // trace("---> "+r);
    return r;
  }

  private boolean isOperator()
  {
    String p1=get(); // class
    String p2=get(); // ::
    String p3=get(); // operator_???

    boolean r=true;

    if(!p2.equals    ("::"       )) r=false;
    if(!p3.startsWith("operator_")) r=false;
    if(!peek().equals("("))         r=false;

    put(p3);
    put(p2);
    put(p1);

    if(r&&(AglType.cherche(p1)!=null))
      new AglClasse(new AglNom(p1)).elementaire=false;

    // trace("isOperator: peek="+p1+p2+p3);
    // trace("---> "+r);
    return r;
  }

  /*
  private String couldBeType()
  {
    if("".equals(peek())||!Character.isJavaIdentifierStart(peek().charAt(0)))
      return null;

    String r=get();
    AglVecteur v=new AglVecteur();
    v.addElement(r);
    while(peek().equals("::"))
    {
      get();
      r=get();
      if("".equals(r)||!Character.isJavaIdentifierStart(r.charAt(0))) r=null;
      v.addElement(r);
    }

    if(v.size()>1)
    {
      if(v.elementAt(v.size()-2).equals(v.elementAt(v.size()-1)))
	r=null; // Type::Type
    }

    for(int i=v.size()-1;i>=0;i--)
    {
      put((String)v.elementAt(i));
      if(i>0) put("::");
    }

    return r;
  }
  */

  public void parseFichier() throws Exception
  {
    parseUnite();
  }

  public AglUnite parseUnite() throws Exception
  {
    AglUnite au=new AglUnite(st.origin());
    au.langage="Cpp";

    unite_=au;
    String t="",v="";

    do
    {
      AglVisibilite visibilite=parseVisibilite();

      t=get();
      v=peek();
      put(t);
      trace("WU>"+t+", "+v);

      /*
      if(t.equals("#"))
      {
	st.eolIsSignificant(true);
	while(!t.equals("\n")) t=get();
	st.eolIsSignificant(false);
      }
      else
      */

      if(t.equals("{"))
	{ get(); }
      else
      if(t.equals("}"))
	{ get(); }
      else
      if(t.equals(";"))
	{ get(); }
      else
      if(t.equals("extern"))
      {
	get();
	if(!peek().startsWith("\""))
	  while(!t.equals(";")) t=get();
	else
	{
	  get(); // "C"
	}
      }
      else
      if(t.equals("template"))
      {
        AglMoule ap=parseMoule();
        if(ap.declaration instanceof AglClasse)
	{
	  AglClasse ac=(AglClasse)ap.declaration;
	  ac.visibilite=visibilite;
	}
        if(ap.declaration instanceof AglStruct)
	{
	  AglStruct as=(AglStruct)ap.declaration;
	  as.visibilite=visibilite;
	}
        au.types.put(""+ap,ap);
      }
      else
      if(t.equals("class"))
      {
        AglClasse ac=parseClasse();
        ac.visibilite=visibilite;
        au.types.put(""+ac,ac);
      }
      else
      /*
      if(t.equals("enum"))
      {
        AglClasse ac=parseEnum();
        ac.visibilite=visibilite;
        au.types.put(""+ac,ac);
      }
      else
      if(t.equals("struct"))
      {
        AglStruct ac=parseStruct();
        ac.visibilite=visibilite;
        au.types.put(""+ac,ac);
      }
      else
      if(t.equals("union"))
      {
        AglStruct ac=parseUnion();
        ac.visibilite=visibilite;
        au.types.put(""+ac,ac);
      }
      else
      */
      if(t.equals("typedef"))
      {
        AglTypeAlias at=parseTypeDef();
        au.types.put(""+at,at);
      }
      else
      if(isConstructeur())
      // (AglType.cherche(t) instanceof AglClasse)&&v.equals("::"))
      {
	AglClasse ac=(AglClasse)AglType.cherche(get());
	get(); // ::
	
	AglConstructeur am=parseConstructeur();
	am.visibilite=visibilite;
	ac.constructeurs.put(am.signature(),am);
      }
      else
      if(isDestructeur())
      // (AglType.cherche(t) instanceof AglClasse)&&v.equals("::"))
      {
	AglClasse ac=(AglClasse)AglType.cherche(get());
	get(); // ::
	get(); // ~
	AglConstructeur am=parseDestructeur();
	am.visibilite=visibilite;
	ac.constructeurs.put(am.signature(),am);
      }
      else
      if(isOperator())
      {
	nom_=parseNom();
	AglClasse ac=(AglClasse)AglType.cherche(nom_.avant_dernier());

             if(nom_.dernier().equals("operator_cast")) type_=parseType();
	else if(ac!=null) type_=ac;
	else type_=AglTypePrimitif.tvoid;

	AglMethode am=parseMethode();
	am.visibilite=visibilite;
	am.proprietaire=ac;
	ac.methodes.put(am.signature(),am);
      }
      else
      if(!t.equals("")) // TYPE
      {
	AglClasse ac;
	AglType   at;

	if(v.equals("(")&&(AglType.cherche(t)==null))
	  put("int");

        type_=parseType();

	if(peek().equals(";"))
	{
	    get();
	}
	else
	{
	  nom_ =parseNom();

	  at=AglType.cherche(nom_.avant_dernier());

	  if(at instanceof AglClasse)
	  {
	    ac=(AglClasse)at;
	  }
	  else
	  {
	    ac=(AglClasse)AglType.cherche("Global");
	    if(ac==null)
	    {
	      ac=new AglClasse(new AglNom("Global"));
	      ac.elementaire=false;
	      ac.proprietaire=unite_;
	      ac.visibilite.vpublic=true;
	      ac.visibilite.vfinal =true;
	    }
	  }

	  if(peek().equals("("))
          {
            AglMethode am=parseMethode();
            am.visibilite=visibilite;
	    am.proprietaire=ac;
            ac.methodes.put(am.signature(),am);
	    if(isGlobal(ac)) am.visibilite.vstatic=true;
          }
          else
          {
            AglVecteur ava=parseAttribut();
            for(Enumeration e=ava.elements(); e.hasMoreElements(); )
            {
              AglAttribut aa=(AglAttribut)e.nextElement();
	      aa.visibilite=visibilite;
	      aa.proprietaire=ac;
	      ac.attributs.put(aa.signature(),aa);
	      if(isGlobal(ac)) aa.visibilite.vstatic=true;
	    }
	  }
	}
      }

      else if(!t.equals("")) syntaxe("unknown keyword: "+t);

    } while(!t.equals(""));

    return au;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    AglVisibilite av=new AglVisibilite();
    String        t ="";
    boolean       e =false;

    do
    {
      e=false;
      t=get();
      trace("WV>"+t);

      /*
         if(t.equals("public"      )) e=av.vpublic      =true;
      else
      */
           if(t.equals("static"      )) e=av.vstatic      =true;
      else if(t.equals("const"       )) e=av.vreadonly    =true;
      else if(t.equals("inline"      )) e=av.vinline      =true;
      else if(t.equals("virtual"     )) e=av.vvirtual     =true;
      else if(t.equals("register"    )) e=av.vregister    =true;
      else if(t.equals("volatile"    )) e=av.vtransient   =true;

    } while(e);

    put(t);
    return av;
  }

  private String operator()
  {
    boolean op=false;
    String t=peek();

    if(t.equals("=" )) { op=true; t="set";     }
    if(t.equals("+=")) { op=true; t="addset";  }
    if(t.equals("-=")) { op=true; t="subset";  }
    if(t.equals("*=")) { op=true; t="mulset";  }
    if(t.equals("/=")) { op=true; t="divset";  }
    if(t.equals("++")) { op=true; t="incr";    }
    if(t.equals("--")) { op=true; t="decr";    }
    if(t.equals("+" )) { op=true; t="add";     }
    if(t.equals("-" )) { op=true; t="sub";     }
    if(t.equals("*" )) { op=true; t="mul";     }
    if(t.equals("/" )) { op=true; t="div";     }
    if(t.equals("[]")) { op=true; t="element"; }
    if(t.equals("(" )) { op=true; t="call"; get(); }
    if(t.equals("==")) { op=true; t="eq";      }
    if(t.equals("!=")) { op=true; t="ne";      }
    if(t.equals("<" )) { op=true; t="lt";      }
    if(t.equals(">" )) { op=true; t="gt";      }
    if(t.equals("<=")) { op=true; t="le";      }
    if(t.equals(">=")) { op=true; t="ge";      }
    if(t.equals("&&")) { op=true; t="band";    }
    if(t.equals("||")) { op=true; t="bor";     }
    if(t.equals("&" )) { op=true; t="and";     }
    if(t.equals("|" )) { op=true; t="or";      }

    if(op) get();
    else   t="cast";

    return "operator_"+t;
  }

  /*
  AglType type=parseType();
  String s="operator_cast_"+new AglCodeHpp().code(type);
  int i;
  while((i=s.indexOf(' ' ))>=0) s=s.substring(0,i)+"__spc__"+s.substring(i+1);
  while((i=s.indexOf('*' ))>=0) s=s.substring(0,i)+"__ptr__"+s.substring(i+1);
  while((i=s.indexOf('&' ))>=0) s=s.substring(0,i)+"__ref__"+s.substring(i+1);
  while((i=s.indexOf("[]"))>=0) s=s.substring(0,i)+"__arr__"+s.substring(i+2);
  */

  public AglNom parseNom() throws Exception
  {
    AglNom an=new AglNom();

    String t=peek();

    if(t.equals("::"))
    {
      get();
      an.identificateurs.addElement("Global");
    }

    do
    {
      t=get();
      trace("WN>"+t);
      an.identificateurs.addElement(t);
      t=peek();
      if(t.equals("::")) get();
    }
    while(t.equals("::"));

    an.ligne=st.lineno();
    return an;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    AglListeNoms al=new AglListeNoms();

    String t="";
    do
    {
      trace("WLN>"+peek());
      al.noms.addElement(parseNom());
      t=peek();
      if(t.equals(",")) get();
    }
    while(t.equals(","));

    al.ligne=st.lineno();
    return al;
  }

  public AglType parseType() throws Exception
  {
    trace("WT>"+peek());
    boolean b=false;
    String  attribut=null;

    if(peek().equals("const")) get();

    if(peek().equals("unsigned"  )) { b=true; get(); }
    if(peek().equals("signed"    )) { b=true; get(); }
    if(peek().equals("__signed__")) { b=true; get(); }
    if(peek().equals("short"     )) { b=true; get(); }
    if(peek().equals("long"      )) { b=true; get(); }
    if(peek().equals("long"      )) { b=true; get(); }

    // if(peek().equals("int"       )) { b=true; get(); }

    if(peek().equals("operator")) return AglTypePrimitif.tvoid;

    AglType r=null;

    if(peek().equals("enum"))
      r=parseEnum();
    else
    if(peek().equals("struct"))
      r=parseStruct();
    else
    if(peek().equals("union"))
      r=parseUnion();
    else
    if(peek().equals(":")) // bitfield
    {
      if(b) r=AglTypePrimitif.tinteger;
      else  syntaxe(": defines a bitfield only for integers.");

      get(); get();
      if(peek().equals(";")) attribut="bitfield_anonyme";
    }
    else
    {
      if(b)
      {
	trace("WT2>"+peek());
	r=AglType.cherche(peek());
	if((r==null)||!(r instanceof AglTypePrimitif))
	  r=AglTypePrimitif.tinteger;
	else
	  get();
      }
      else
      {
	AglNom n=parseNom();
	trace("WT2>"+n);

	r=AglType.cherche(n);
	if(r==null) r=new AglClasse(n);
      }

      // TMP
      if(peek().equals("("))
      {
	get();
	while(peek().equals("*"))
	  { trace("WT1>"+peek()); r=new AglTypeTableau(r);  get(); }
	if(peek().equals("&"))
	  { trace("WT1>"+peek()); get(); }
	attribut=get();
	if(!peek().equals(")"))
	    syntaxe(") needed and not "+peek());
	else get();
      }
    }

    while(peek().equals("<"))
    {
      get();
      if(peek().equals("const")) get();
      AglType type=parseType();
      String s=new AglCodeHpp().code(r)+"_template_"+new AglCodeHpp().code(type);
      int i;
      while((i=s.indexOf(' ' ))>=0) s=s.substring(0,i)+"__spc__"+s.substring(i+1);
      while((i=s.indexOf('*' ))>=0) s=s.substring(0,i)+"__ptr__"+s.substring(i+1);
      while((i=s.indexOf('&' ))>=0) s=s.substring(0,i)+"__ref__"+s.substring(i+1);
      while((i=s.indexOf("[]"))>=0) s=s.substring(0,i)+"__arr__"+s.substring(i+2);
      r=AglType.cherche(s);
      if(r==null) r=new AglClasse(new AglNom(s));

      s=get();
      if(s.equals(",")) // Plus d'un parametre
	  put("<");
      else
      {
	if(!s.startsWith(">"))
	  syntaxe("> needed and not "+s);

	if(!s.equals(">")) // >> or >>>
	  put(s.substring(1));
      }
    }

    while(peek().equals("*"))
      { trace("WT>"+peek()); r=new AglTypePointeur(r); get(); }
    if(peek().equals("&"))
      { trace("WT>"+peek()); r=new AglTypeReference(r); get(); }

    // parametres de fonction
    if(peek().equals("("))
    {
      AglVecteur tp=parseParametres();
      suivi("### "+tp.size()+
	    " parameters ignored in function type.");
    }

    while(peek().equals("const"))
      { trace("WT>"+peek()); get(); }

    if(attribut!=null) put(attribut);

    return r;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    erreur("Pas de paquet en C/C++");
    return null;

    /*
    String t="";

    t=get();
    if(!t.equals("package")) syntaxe("package needed and not "+t);

    AglNom np=parseNom();
    trace("Paquet   : "+np);

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglPaquet ap=new AglPaquet(np);
    ap.ligne=st.lineno();
    return ap;
    */
  }
  
  public AglImport parseImport() throws Exception
  {
    erreur("Pas d'import en C/C++");
    return null;

    /*
    String t="";

    t=get();
    if(!t.equals("import")) syntaxe("import needed and not "+t);

    AglNom ni=parseNom();
    trace("Import      : "+ni);

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglImport ai=new AglImport(ni);
    ai.ligne=st.lineno();
    return ai;
    */
  }
  
  public AglTypeAlias parseTypeDef() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("typedef")) syntaxe("typedef needed and not "+t);

    AglType at=null;
    if(peek().equals("enum"))
      at=parseEnum();
    else
    if(peek().equals("struct"))
      at=parseStruct();
    else
    if(peek().equals("union"))
      at=parseUnion();
    else
      at=parseType();

    while(peek().equals("*"))
      { trace("WT>"+peek()); at=new AglTypeTableau(at); get(); }
    if(peek().equals("&"))
      { trace("WT>"+peek()); get(); }

    if(peek().equals("("))
    {
      get();
      while(peek().equals("*"))
	{ trace("WT>"+peek()); at=new AglTypeTableau(at); get(); }
    }

    AglNom  an=parseNom();
    AglTypeAlias al=new AglTypeAlias(an.dernier(),at,"Cpp");

    if(peek().equals(")"))
      get();

    if(peek().equals("("))
    {
      AglVecteur tp=parseParametres();
      trace("### "+tp.size()+" parameters ignored in function type.");
    }

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);
    return al;
  }

  public AglClasse parseEnum() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("enum")) syntaxe("enum needed and not "+t);

    trace("WENUM>"+t);

    AglNom nc=new AglNom("enum_anonyme");

    if(!peek().equals("{"))
    {
      nc=parseNom();
      if(unite_.paquet!=null) nc=new AglNom(unite_.paquet.nom,nc);
    }

    trace("Enum        : "+nc);

    AglClasse ac=(AglClasse)AglType.cherche(nc);
    if(ac==null) ac=new AglClasse(nc);
    ac.ligne=st.lineno();
    // ac.elementaire=false;

    if(peek().equals(";"))
      return ac;
    
    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    // Body

    do
    {
      t=peek();
      trace("WBE>"+t);

      if(!t.equals("}"))
      {
	String        nom   =get();
	AglExpression valeur=null;

	if(peek().equals("="))
	{
	  get();
	  valeur=parseExpression();
	}

	AglAttribut aa=new AglAttribut(nom,AglTypePrimitif.tinteger);
	aa.ligne=st.lineno();
	aa.visibilite=new AglVisibilite();
	aa.visibilite.vpublic=true;
	aa.initiale=valeur;
	ac.attributs.put(aa.signature(),aa);

	if(peek().equals(",")) get();
      }
    } while(!peek().equals("}"));

    get();
    // if(peek().equals(";")) get();
    return ac;
  }

  public AglStruct parseStruct() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("struct")) syntaxe("struct needed and not "+t);

    AglNom nc=new AglNom("struct_anonyme");

    if(!peek().equals("{"))
    {
      nc=parseNom();
      if(unite_.paquet!=null) nc=new AglNom(unite_.paquet.nom,nc);
    }

    trace("Struct      : "+nc);
    AglStruct ac=(AglStruct)AglType.cherche(nc);
    boolean creee=false;

    if(ac==null)
    {
      ac=new AglStruct(nc);
      ac.ligne=st.lineno();
      ac.elementaire=true;
      creee=true;
    }

    if(peek().equals(";")) // pre-declaration
      return ac;
    
    if(!peek().equals("{"))
    {
      // syntaxe("{ needed and not "+t);
      return ac;
    }

    get(); // "{"
    if(creee)
    {
      if(!ac.visibilite.vstatic)
      {
	ac.visibilite.vpublic=true;
	ac.visibilite.vfinal =true;
      }
      ac.elementaire=false;
      ac.proprietaire=unite_;
    }

    // Body

    String ppp="public:";

    do
    {
      t=peek();
      trace("WBS>"+t);

      if(!t.equals("}"))
      {
	if(peek().equals("public:"   )) { ppp=get(); continue; }
	if(peek().equals("protected:")) { ppp=get(); continue; }
	if(peek().equals("private:"  )) { ppp=get(); continue; }

	AglVisibilite visibilite=parseVisibilite();
	     if(ppp.equals("public:"   )) visibilite.vpublic   =true;
	else if(ppp.equals("protected:")) visibilite.vprotected=true;
	else if(ppp.equals("private:"  )) visibilite.vprivate  =true;

	type_=parseType();
	nom_ =parseNom();

	AglVecteur ava=parseAttribut();
	for(Enumeration e=ava.elements(); e.hasMoreElements(); )
	{
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  aa.visibilite=visibilite;
	  ac.attributs.put(aa.signature(),aa);
        }
      }
    } while(!peek().equals("}"));

    get();
    //if(peek().equals(";")) get();
    return ac;
  }


  public AglStruct parseUnion() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("union")) syntaxe("union needed and not "+t);
    trace("WUNION>"+t);

    put("struct");

    AglStruct ac=parseStruct();

    if(ac.nom.equals("struct_anonyme"))
      ac.nom=new AglNom("union_anonyme");

    trace(" --> Union");
    return ac;
  }

  public AglMoule parseMoule() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("template")) syntaxe("template needed and not "+t);

    t=get();
    if(!t.equals("<")) syntaxe("< needed and not "+t);

    AglVecteur v=new AglVecteur();

    int i=1;
    while(!peek().equals(">"))
    {
      String type=get();
      String nom ="_p"+i;
      if(!peek().equals(",")&&!peek().equals(">"))
	nom=get();
      AglParametreMoule pp=new AglParametreMoule(nom);
      pp.type=type;
      v.addElement(pp);
      if(peek().equals(",")) get();
      i++;
    }

    t=get();
    if(!t.equals(">")) syntaxe("> needed and not "+t);

    AglClasse c=parseClasse();
    AglMoule r=new AglMoule(c.nom,c);
    r.parametres=v;

    return r;
  }

  public AglClasse parseClasse() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("class")) syntaxe("class needed and not "+t);

    trace("WC>"+t);
    AglNom nc=parseNom();
    if(unite_.paquet!=null) nc=new AglNom(unite_.paquet.nom,nc);
    trace("Classe      : "+nc);
    
    AglClasse ac=(AglClasse)AglType.cherche(nc);
    if(ac==null) ac=new AglClasse(nc);
    ac.proprietaire=unite_;
    ac.ligne       =st.lineno();
    ac.elementaire =false;
    
    if(peek().equals(";"))
    {
      get();
      return ac;
    }
    
    do
    {
      t=get();
      trace("WCB>"+t);
           if(t.equals(":"))    ac.meres     =parseMothers();
	   // else if(t.equals("implements")) ac.interfaces=parseInterfaces();
	   // else if(t.equals("throws"))     ac.exceptions=parseExceptions();
    } while(!t.equals("{"));

  // Body

    String ppp="private:";

    do
    {
      if(peek().equals("}")) break;

      if(peek().equals("typedef"   )) { parseTypeDef(); continue; }

      if(peek().equals("public:"   )) { ppp=get(); continue; }
      if(peek().equals("protected:")) { ppp=get(); continue; }
      if(peek().equals("private:"  )) { ppp=get(); continue; }

      AglVisibilite visibilite=parseVisibilite();
           if(ppp.equals("public:"   )) visibilite.vpublic   =true;
      else if(ppp.equals("protected:")) visibilite.vprotected=true;
      else if(ppp.equals("private:"  )) visibilite.vprivate  =true;

      
      if(peek().equals("}")) break;

      trace("WBC>"+peek());

      t=get();

      if(t.startsWith("operator"))
      {
	nom_=new AglNom(t);

	     if(t.equals("operator_cast")) type_=parseType();
	else if(ac!=null) type_=ac;
	else type_=AglTypePrimitif.tvoid;

	AglMethode am=parseMethode();
	ac.methodes.put(am.signature(),am);
      }
      else
      if(t.equals(ac.nom.dernier())&&peek().equals("(")&&!visibilite.vstatic)
      {
        put(t);

        AglConstructeur am=parseConstructeur();
        am.visibilite=visibilite;
        ac.constructeurs.put(am.signature(),am);
      }
      else
      if(t.equals("~")&&peek().equals(ac.nom.dernier()))
      {
        // put(t);
        AglConstructeur am=parseDestructeur();
        am.visibilite=visibilite;
        ac.constructeurs.put(am.signature(),am);
      }
      else
      if(t.equals("enum"))
      {
	put(t); //@GDX
        AglClasse ic=parseEnum();
        ic.visibilite=visibilite;
	ic.nom=new AglNom(ac.nom,ic.nom.dernier());
        unite_.types.put(""+ic,ic);
      }
      else
      if(t.equals("struct")&&peek().equals("{"))
      {
	put(t);
	AglStruct ic=parseStruct();
	ic.visibilite=visibilite;
	ic.nom=new AglNom(ac.nom,ic.nom.dernier());
	unite_.types.put(""+ic,ic);
      }
      else
      if(t.equals("union")&&peek().equals("{"))
      {
	put(t);
	AglStruct ic=parseUnion();
	ic.visibilite=visibilite;
	ic.nom=new AglNom(ac.nom,ic.nom.dernier());
	unite_.types.put(""+ic,ic);
      }
      else
      if(t.equals("class"))
      {
        put(t);
	AglClasse ic=parseClasse();
	ic.nom=new AglNom(ac.nom,ic.nom.dernier());
        unite_.types.put(""+ic,ic);
      }
      else if(!t.equals("}"))
      {
        put(t);

        type_=parseType();
        nom_ =parseNom();
        String v=peek();

        if(v.equals("("))
        {
          AglMethode am=parseMethode();
          am.visibilite=visibilite;
          ac.methodes.put(am.signature(),am);
        }
        else
        {
          AglVecteur ava=parseAttribut();
          for(Enumeration e=ava.elements(); e.hasMoreElements(); )
          {
            AglAttribut aa=(AglAttribut)e.nextElement();
            aa.visibilite=visibilite;
            ac.attributs.put(aa.signature(),aa);
          }
        }
      }

    } while(!peek().equals("}"));

    get();
    if(peek().equals(";")) get();
    return ac;
  }

  public AglListeNoms parseMothers() throws Exception
  {
    AglListeNoms ln=new AglListeNoms();

    String t=peek();
    while(!t.equals("{")&&!t.equals(";"))
    {
      parseVisibilite();
      ln.noms.addElement(parseNom());
      if(peek().equals(",")) get();
      t=peek();
    }

    trace("Mere        : "+ln);
    return ln;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    trace("Interfaces  : "+ln);
    return ln;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    trace("Exceptions  : "+ln);
    return ln;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    AglNom nm=parseNom();
    trace("Constructeur: "+nm);

    AglConstructeur m=new AglConstructeur(nm);
    m.ligne=st.lineno();

    m.parametres=parseParametres();

    if(peek().equals(":"))
    {
      get();
      while(!peek().equals(";")&&!peek().equals("{"))
	get();
      // @GDX should manage init of classes and fields
    }

         if(peek().equals(";")) get();
    else if(peek().equals("{")) m.corps=parseBlocMultiple();

    return m;
  }

  public AglConstructeur parseDestructeur() throws Exception
  {
    AglNom nm=parseNom();
    trace("Destructeur : "+nm);

    AglConstructeur m=new AglConstructeur(nm);
    m.ligne=st.lineno();
    m.destructeur=true;

    m.parametres=parseParametres();

         if(peek().equals(";")) get();
    else if(peek().equals("{")) m.corps=parseBlocMultiple();

    return m;
  }

  public AglMethode parseMethode() throws Exception
  {
    trace("Methode     : "+nom_+" ["+type_+"]");

    AglMethode m=new AglMethode(nom_.dernier(),type_);
    m.ligne=st.lineno();

    m.parametres=parseParametres();
    // if(peek().equals("throws")) { get(); m.exceptions=parseExceptions(); }

    if(peek().equals("const")) get();

    if(peek().equals("="))
    {
      get();
      if(!peek().equals("0"))
	syntaxe("0 needed for abstract and not "+peek());
      else get();
      m.visibilite.vabstract=true;
    }

    if(peek().equals(";"    )) get();
    else m.corps=parseBlocMultiple();

    return m;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    AglVecteur r=new AglVecteur();
    String t="";

    do
    {
      trace("Attribut    : "+nom_+" ["+type_+"]");

      AglType tm=type_;

      if(peek().equals(":")) // bitfield
      {
	get();
	get();
      }

      while(peek().equals("["))
      {
	trace("WA>"+peek());
	tm=new AglTypeTableau(tm);
	get();
	((AglTypeTableau)tm).dimension=parseExpression();
	get();
      }
      while(peek().equals("[]")) { get(); tm=new AglTypeTableau(tm); }

      if(  (tm instanceof AglStruct)
	 &&(new AglNom("struct_anonyme").equals(tm.nom)))
	tm.setNom(new AglNom("struct_"+nom_.dernier()));

      if(  (tm instanceof AglStruct)
	 &&(new AglNom("union_anonyme").equals(tm.nom)))
	tm.setNom(new AglNom("union_"+nom_.dernier()));

      if(  (tm instanceof AglClasse)
	 &&(new AglNom("enum_anonyme").equals(tm.nom)))
	tm.setNom(new AglNom("enum_"+nom_.dernier()));

      AglAttribut a=new AglAttribut(nom_.dernier(),tm);
      a.ligne=st.lineno();
      r.addElement(a);

      t=get();
      if(t.equals("="))
      {
	if(peek().equals("{")) a.initiale=parseExpressionInitArray();
	else                   a.initiale=parseExpression();
        trace("Initiale    : "+a.initiale);
        t=get();
      }

      if(t.equals(","))
      {
	if(peek().equals("*")) get();
	nom_=parseNom();
      }
      else
      if(!t.equals(";"))
        syntaxe(", or ; needed and not "+t);

    } while(!t.equals(";"));

    return r;
  }

  public AglVecteur parseParametres() throws Exception
  {
    int        np=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    do 
    {
      if(!peek().equals(")"))
      {
	AglVisibilite visibilite=parseVisibilite();

        AglType type=parseType();

	// cas: (void)
	if(type==AglTypePrimitif.tvoid) break;

        np++;

        String nom=null;

	if(peek().equals("("))
	{
	  get();
	  if(peek().equals("*")) get();
	  nom=get();
	  get(); // )
	  if(peek().equals("("))
	  {
	    AglVecteur tp=parseParametres();
	    trace("### "+tp.size()+" parameters ignored in function type.");
	  }
	  type=new AglTypePrimitif("function");
	}	  
	else
	{
	  t=peek();
	  if(!t.equals(",")&&!t.equals(")")) nom=get();
	  else                               nom="p"+np;
	}

	while(peek().equals("["))
	{
	  trace("WP>"+peek());
	  type=new AglTypeTableau(type);
	  get();
	  ((AglTypeTableau)type).dimension=parseExpression();
	  get();
	}
        while(peek().equals("[]")) { get(); type=new AglTypeTableau(type); }

        trace("Parametre "+np+" : "+nom+" ["+type+"]");

        AglParametre p=new AglParametre(nom,type);
        r.addElement(p);

	if(peek().equals("="))
	{
	  get();
	  p.defaut=parseExpression();
	}

             if(peek().equals(",")) get();
        else if(!peek().equals(")")) syntaxe(", or ; needed and not "+t);
      }

    } while(!peek().equals(")"));

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    return r;
  }

  public AglVecteur parseArgs() throws Exception
  {
    int        na=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    t=get();
    if(!(t.equals("("))) syntaxe("( needed and not "+t);

    do 
    {
      t=peek();

      if(!t.equals(")"))
      {
        AglExpression ae=parseExpression();
        na++;
        trace("Argument  "+na+" : "+ae);
        r.addElement(ae);
      }

      t=get();
      if(!(t.equals(",")||t.equals(")"))) syntaxe(", or ) needed and not "+t);

    } while(!t.equals(")"));

    return r;
  }

  public AglBloc parseBloc() throws Exception
  {
    return peek().equals("{") ? parseBlocMultiple() : parseBlocSimple();
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t="";

    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    while(!peek().equals("}"))
    {
      AglVisibilite visibilite=parseVisibilite();

      t=get();
      trace("WBM>>> "+t);

      if( (  t.equals("struct")
	   ||t.equals("union")
	   ||(AglType.cherche(t)!=null))
	 &&!peek().equals(".")
	 &&!peek().equals("("))
      {
        put(t);
        for(Enumeration e=parseInstructionDefVar().elements(); e.hasMoreElements(); )
        {
          AglInstructionDefVar ai=(AglInstructionDefVar) e.nextElement();
          ai.visibilite=visibilite;
          ab.instructions.addElement(ai);
        }
      }
      else
      {
        put(t);
        AglInstruction ai=null;
        
             if(t.equals("{"))      ai=parseInstructionBloc       ();
        else if(t.equals("catch"))  ai=parseInstructionCatch      ();
        else if(t.equals("do"))     ai=parseInstructionDoWhile    ();
        else if(t.equals("for"))    ai=parseInstructionFor        ();
        else if(t.equals("if"))     ai=parseInstructionIf         ();
        else if(t.equals("return")) ai=parseInstructionReturn     ();
        else if(t.equals("switch")) ai=parseInstructionSwitch     ();
        else if(t.equals("try"))    ai=parseInstructionTry        ();
        else if(t.equals("while"))  ai=parseInstructionWhile      ();
        else
        {
          AglExpression gauche=parseExpression();

          t=peek();
               if(t.equals("="))      ai=parseInstructionAffectation(gauche);
       // else if(t.equals("("))      ai=parseInstructionProcedure  (gauche);
          else if(t.equals(";"))      ai=parseInstructionExpression (gauche);
          else if(  t.equals("+=")||t.equals("-=")
		  ||t.equals("*=")||t.equals("/=")||t.equals("%=")
                  ||t.equals("&=")||t.equals("|=")||t.equals("^="))
          {
            t=get();
            AglExpression ae=new AglExpressionBinaire(t.substring(0,1),gauche,parseExpression());
            ae.ligne=st.lineno();

            ai=new AglInstructionAffectation(gauche,ae);
            ai.ligne=st.lineno();

            t=get();
            if(!t.equals(";")) syntaxe("; needed and not "+t);
          }
          else
	  {
	    ai=parseInstructionGenerique();

	    if(gauche instanceof AglExpressionGenerique)
	      ((AglInstructionGenerique)ai).body=
		((AglExpressionGenerique)gauche).body+" "+
		((AglInstructionGenerique)ai).body.trim();
	    else
	    {
	      AglInstructionExpression aie=
		new AglInstructionExpression(gauche);
	      aie.ligne=gauche.ligne;
	      ab.instructions.addElement(aie);
	    }
	  }
        }

        ab.instructions.addElement(ai);
      }
    }

    t=get();
    if(!t.equals("}")) syntaxe("} needed and not "+t);

    t=peek();
    if(t.equals(";")) get();

    return ab;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t=get();
    String  u=peek();
    trace("WBS>>> "+t+" | "+u);

    /*
    if( (  t.equals("struct")
	 ||t.equals("union")
	 ||(AglType.cherche(t)!=null))
       &&(!u.equals(".")))
    {
      put(t);
      for(Enumeration e=parseInstructionDefVar().elements();
	  e.hasMoreElements(); )
      {
	AglInstructionDefVar ai=(AglInstructionDefVar) e.nextElement();
	//ai.visibilite=visibilite;
	ab.instructions.addElement(ai);
      }
    }
    else
    */

    {
      put(t);
      AglInstruction ai=null;
        
           if(t.equals("{"))      syntaxe("{ is not wanted there");
      else if(t.equals("catch"))  ai=parseInstructionCatch      ();
      else if(t.equals("do"))     ai=parseInstructionDoWhile    ();
      else if(t.equals("for"))    ai=parseInstructionFor        ();
      else if(t.equals("if"))     ai=parseInstructionIf         ();
      else if(t.equals("return")) ai=parseInstructionReturn     ();
      else if(t.equals("switch")) ai=parseInstructionSwitch     ();
      else if(t.equals("try"))    ai=parseInstructionTry        ();
      else if(t.equals("while"))  ai=parseInstructionWhile      ();
      else
      {
	AglExpression gauche=parseExpression();

	t=peek();
             if(t.equals("="))      ai=parseInstructionAffectation(gauche);
     // else if(t.equals("("))      ai=parseInstructionProcedure  (gauche);
        else if(t.equals(";"))      ai=parseInstructionExpression (gauche);
        else if(  t.equals("+=")||t.equals("-=")
	        ||t.equals("*=")||t.equals("/=")||t.equals("%=")
                ||t.equals("&=")||t.equals("|=")||t.equals("^="))
        {
	  t=get();
	  AglExpression ae=new AglExpressionBinaire
	    (t.substring(0,1),gauche,parseExpression());
	  ae.ligne=st.lineno();

	  ai=new AglInstructionAffectation(gauche,ae);
	  ai.ligne=st.lineno();

	  t=get();
	  if(!t.equals(";")) syntaxe("; needed and not "+t);
	}
	else ai=parseInstructionGenerique();
      }
	   
      ab.instructions.addElement(ai);
    }
  
    return ab;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t =get();
    String  u =peek();

    trace("WBV>>> "+t+" | "+u);

    put(t);

    while(!t.equals(";")&&!t.equals(")"))
    {
      AglInstruction ai=null;
      AglExpression gauche=parseExpression();

      t=peek();
      if(t.equals(",")||t.equals(";")||t.equals(")"))
      {
	ai=new AglInstructionExpression(gauche);
	ai.ligne=st.lineno();
      }
      else if(t.equals("="))
      {
	t=get();
	AglExpression ae=parseExpression();

	ai=new AglInstructionAffectation(gauche,ae);
	ai.ligne=st.lineno();
      }
      else if(  t.equals("+=")||t.equals("-=")
	      ||t.equals("*=")||t.equals("/=")||t.equals("%=")
	      ||t.equals("&=")||t.equals("|=")||t.equals("^="))
      {
	t=get();
	AglExpression ae=new AglExpressionBinaire(t.substring(0,1),gauche,parseExpression());
	ae.ligne=st.lineno();

	ai=new AglInstructionAffectation(gauche,ae);
	ai.ligne=st.lineno();
      }
      else ai=parseInstructionGenerique();

      ab.instructions.addElement(ai);

      t=peek();	   
      if(!t.equals(";")&&!t.equals(",")&&!t.equals(")"))
	syntaxe("; or , or ) needed and not "+t);

      if(t.equals(",")) get();
    }

    return ab;
  }

// Expressions

  public AglExpression parseExpression() throws Exception
  {
    String p=peek();
    trace("WE>>> "+p);
    if(p.equals(";")||p.equals(",")) return null;

    AglExpression ae;

         if(p.equals("new"))    ae=parseExpressionNew();
    else if(p.equals("+" ))     ae=parseExpressionUnaireDroite();
    else if(p.equals("-" ))     ae=parseExpressionUnaireDroite();
    else if(p.equals("++"))     ae=parseExpressionUnaireDroite();
    else if(p.equals("--"))     ae=parseExpressionUnaireDroite();
    else if(p.equals("!" ))     ae=parseExpressionUnaireDroite();
    else if(p.equals("*" ))     ae=parseExpressionUnaireDroite();
    else if(p.equals("&" ))     ae=parseExpressionUnaireDroite();
    else if(p.equals("sizeof")) ae=parseExpressionGenerique();
    else if(p.equals("("))
    {
      String t,u,v;
      t=get();
      u=get();
      v=peek();
      put(u);
      put(t);
      trace("WE>>> "+t+" | "+u+" | "+v);

      if(  !v.equals("::")
	 &&!v.equals("->")
	 &&(  u.equals("struct")
	    ||u.equals("union")
	    ||u.equals("unsigned")
	    ||u.equals("const")
	    ||(AglType.cherche(u)!=null)
	    ||(Character.isLetter(u.charAt(0))&&v.equals(")"))))
	      // TMP @GDX probablement cast
        ae=parseExpressionTypage();
      else
	ae=parseExpressionParenthesee();
    }
    else
    {
      String  t,u;
      t=get();

      if(t.equals("::"))
	t="Global."+get();

      u=peek();
      put(t);

      trace("WE>>> "+t+" | "+u);

      if(u.equals("("))
	ae=new AglExpressionMethode(null,parseExpressionFonction());
      else
      if(u.equals(".")||u.equals("->")) // TMP @GDX
	ae=new AglExpressionAttribut(null,get());
      else
      if(u.equals("::"))
        // attribut de classe
	ae=new AglExpressionAttribut(null,parseNom().dernier());
      else
      if(u.equals("["))
      {
	ae=new AglExpressionGenerique(get());
	u=get(); if(!u.equals("[")) syntaxe("[ needed and not "+u);
	ae=new AglExpressionIndice(ae,parseExpression());
	u=get(); if(!u.equals("]")) syntaxe("] needed and not "+u);
      }
      else
      {
	trace("EGX>>> "+t);
	ae=new AglExpressionGenerique(get());
      }

      ae.ligne=st.lineno();
    }

boolean encore=true;
while(encore)
{
    encore=false;
    String u;

    trace("Expression  : "+ae+" ["+ae.getClass()+"]");
    u=peek();
    trace("WEA>>> "+u);

    if(u.equals(";")||u.equals(","))
      encore=false;
    else
    if(u.equals("("))
    {
      trace("--- CALL");
      AglVecteur le=parseArgs();
      AglExpressionFonction af=new AglExpressionFonction("function");
      af.arguments=le;
      af.ligne=st.lineno();

      String nf=null;
      try
      {
	AglExpression ag=((AglExpressionParenthesee)ae).expression;
	ag=((AglExpressionUnaireDroite)ag).droite;
	nf=((AglExpressionGenerique)ag).body;
      }
      catch(Exception ex) { }

      if(nf==null) ae=new AglExpressionMethode(ae,af);
      else         { ae=af; af.nom=nf; }

      ae.ligne=st.lineno();
      encore=true;
    }
    else
    if(u.equals("?"))
    {
      String        op1=get();
      AglExpression em =parseExpression();
      String        op2=get();
      if(!op2.equals(":")) syntaxe(": needed and not "+op2);
      AglExpression ed =parseExpression();

      ae=new AglExpressionTernaire(ae,op1,em,op2,ed);
      ae.ligne=st.lineno();
      encore=true;
    }
    else
    if(u.equals("="))
    {
      get();
      ae=new AglExpressionAffectation(ae,parseExpression());
      ae.ligne=st.lineno();
      encore=true;
    }
    else if(  u.equals("+=")||u.equals("-=")
	    ||u.equals("*=")||u.equals("/=")||u.equals("%=")
	    ||u.equals("&=")||u.equals("|=")||u.equals("^="))
    {
      get();
      AglExpression gauche=ae;
      ae=new AglExpressionBinaire
	(u.substring(0,1),gauche,parseExpression());
      ae.ligne=st.lineno();

      ae=new AglExpressionAffectation(gauche,ae);
      ae.ligne=st.lineno();
    }
    else
    if(  (u.equals("+"  ))||(u.equals("-"))
       ||(u.equals("*"  ))||(u.equals("/"))
       ||(u.equals("%"  ))
       ||(u.equals("==" ))||(u.equals("!="))
       ||(u.equals("<"  ))||(u.equals(">"))
       ||(u.equals("<=" ))||(u.equals(">="))
       ||(u.equals("<<" ))||(u.equals(">>"))
       ||(u.equals("<<<"))||(u.equals(">>>"))
       ||(u.equals("&&"))||(u.equals("||"))
       ||(u.equals("&"))||(u.equals("|"))||(u.equals("^"))
       ||(u.equals("->")))
    {
      ae=new AglExpressionBinaire(get(),ae,parseExpression());
      ae.ligne=st.lineno();
      encore=true;
    }
    else
    if(u.equals("["))
    {
      get();
      ae=new AglExpressionIndice(ae,parseExpression());
      get();
      ae.ligne=st.lineno();
      encore=true;
    }
    else
    if((u.equals("++"))||(u.equals("--")))
    {
      ae=new AglExpressionUnaireGauche(get(),ae);
      ae.ligne=st.lineno();
      encore=true;
    }
    else
    if(u.equals(".")||u.equals("->")) // TMP @GDX
    {
      get();
      String v=get();
      String w=peek();
      put(v);

      if(w.equals("("))
	ae=new AglExpressionMethode(ae,parseExpressionFonction());
      else
	ae=new AglExpressionAttribut(ae,get());
      ae.ligne=st.lineno();
      encore=true;
    }
}

    return ae;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    String t="";
    int    np=0;
    int    nt=0;

    String body="";

    do
    {
      t=get();
      // trace("WEG>>> "+t);

      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      if(t.equals("]")) nt--;
      if(t.equals("[")) nt++;
      
           if((np<0)&&(t.equals(")"))) ;
      else if((nt<0)&&(t.equals("]"))) ;
      else if((np==0)&&(nt==0)&&(t.equals(";")||t.equals(","))) ;
      else
      {
        if(  (body.length()>0)
           &&(  Character.isJavaIdentifierPart(body.charAt(body.length()-1))
	      ||(body.charAt(body.length()-1)==')'))
	   &&(  Character.isJavaIdentifierStart(t.charAt(0))
	      ||(t.charAt(0)=='\"'))) body+=" ";
	body+=t;
      }
    } while(  (np>0)
            ||(nt>0)
	    ||((np==0)&&(nt==0)&&(!(t.equals(";")||t.equals(",")))));

    if(t.equals(";")||t.equals(",")) put(t);
    if((np<0)&&t.equals(")")) put(t);
    if((nt<0)&&t.equals("]")) put(t);

    AglExpressionGenerique ae=new AglExpressionGenerique(body);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite() throws Exception
  {
    String        op=get();

    trace("Unaire     : "+op);
    AglExpression ed=parseExpression();

    AglExpressionUnaireDroite ae=new AglExpressionUnaireDroite(op,ed);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    AglExpression eg=new AglExpressionGenerique(get());
    String        op=get();

    trace("Binaire    : "+op);
    AglExpression ed=parseExpression();

    AglExpressionBinaire ae=new AglExpressionBinaire(op,eg,ed);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    String t="";

    String nom=get();
    AglVecteur le =parseArgs();
    trace("Fonction    : "+nom+" ("+le.size()+" arguments)");

    AglExpressionFonction ae=new AglExpressionFonction(nom);
    ae.ligne=st.lineno();
    for(Enumeration e=le.elements(); e.hasMoreElements(); )
      ae.arguments.addElement(e.nextElement());
    return ae;
  }

  public AglExpression parseExpressionInitArray() throws Exception
  {
    if(!peek().equals("{")) syntaxe("{ needed and not "+peek());
    get();

    AglExpressionInitArray ae=new AglExpressionInitArray();
    do
    {
      if(peek().equals("{")) // multi-dim array
	ae.initiales.addElement(parseExpressionInitArray());
      else
	ae.initiales.addElement(parseExpression());

      if(!peek().equals(",")&&!peek().equals("}"))
	syntaxe("} or , needed and not "+peek());
      if(peek().equals(",")) get();

    } while(!peek().equals("}"));

    if(!peek().equals("}")) syntaxe("} needed and not "+peek());
    get();

    return ae;
  }


  public AglExpression parseExpressionNew() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("new")) syntaxe("new needed and not "+t);

    t=get();
    AglExpression ae;

    if(peek().equals("("))
    {
      put(t);
      AglType c =parseType();
      AglVecteur  le=parseArgs();
      trace("New         : "+t+" ("+le.size()+" arguments)");

      AglExpressionNew aen=new AglExpressionNew(c);
      aen.ligne=st.lineno();
      for(Enumeration e=le.elements(); e.hasMoreElements(); )
        aen.arguments.addElement(e.nextElement());
      ae=aen;
    }
    else
    {
      put(t);
      AglType c =parseType();
      trace("NewArray    : "+t);

      AglExpressionNewArray aen=new AglExpressionNewArray(c);
      aen.ligne=st.lineno();
      ae=aen;

      if(peek().equals("{"))
      {
	aen.initiale=parseExpressionInitArray();
      }
      else
      {
	t=get(); if(!t.equals("[")) syntaxe("[ needed and not "+t);
	AglExpression ne=parseExpression();
	t=get(); if(!t.equals("]")) syntaxe("] needed and not "+t);
	aen.nombre=ne;
      }
    }

    return ae;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpressionParenthesee ae=new AglExpressionParenthesee(parseExpression());

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglType type=parseType();
    trace("Typage    : "+type);

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglExpressionTypage ae=new AglExpressionTypage(type,parseExpression());
    ae.ligne=st.lineno();
    return ae;
  }

// Instructions

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    String  ident="                                         ";
    String  t="";
    int     na=0;
    int     np=0;
    boolean pv=true;

    String body="";

    do
    {
      t=get();
      // trace("WI>>> "+t);

      if(t.equals("}")) na--;
      if(t.equals("{")) na++;
      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      // if(na>0)
      {
             if(t.equals("{"))
	       { body+="\n" +ident.substring(0,Math.max(0,2*na))+"{\n"; pv=true; }
        else if(t.equals("}"))
	  { body+=ident.substring(0,Math.max(0,2*na+2))+"}\n"; pv=true; }
        else if(t.equals(";"))
	  { if(np==0) { body+=";\n"; pv=true; } else body+="; "; }
        else if(pv)
	  { body+=ident.substring(0,Math.max(0,2*na+2))+t; pv=false; }
        else
	{
          if(  (body.length()>0)
	     &&(  Character.isJavaIdentifierPart(body.charAt(body.length()-1))
		||(body.charAt(body.length()-1)==')'))
	     &&(  Character.isJavaIdentifierStart(t.charAt(0))
		||(t.charAt(0)=='\"'))) body+=" ";
	  body+=t;
          pv=false;
        }
      }
    } while((na>0)||(!t.equals(";")));

    trace("Instruction : "+body);
    AglInstructionGenerique ai=new AglInstructionGenerique(body);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionAffectation parseInstructionAffectation(AglExpression eg) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("=")) syntaxe("= needed and not "+t);

    AglExpression ed=parseExpression();

    t=get();
    while(t.equals("="))
    {
      ed=new AglExpressionAffectation(ed,parseExpression());
      t=get();
    }
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    trace("Affectation: "+eg+" <-- "+ed);

    AglInstructionAffectation ai=new AglInstructionAffectation(eg,ed);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    AglBloc ab=parseBlocMultiple();
    AglInstructionBloc ai=new AglInstructionBloc(ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("catch")) syntaxe("catch needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglType type=parseType();
    AglExpression ae=new AglExpressionGenerique(type.nom.dernier()+" "+get());

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();

    AglInstructionCatch ai=new AglInstructionCatch(ae,ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglVecteur parseInstructionDefVar() throws Exception
  {
    String     t ="";
    int        nv=0;
    AglVecteur r =new AglVecteur();

    AglType type=parseType();

    do 
    {
      t=get();

      if(!t.equals(";"))
      {
	AglType at=type;

	if(t.equals("*")) t=get();

	if(peek().equals(":")) // bitfield
	{
	  get();
	  get();
	}

	while(peek().equals("["))
	{
	  trace("WBK>"+peek());
	  at=new AglTypeTableau(at);
	  get();
	  ((AglTypeTableau)at).dimension=parseExpression();
	  get();
	}
        while(peek().equals("[]")) { get(); at=new AglTypeTableau(at); }

        nv++;
        trace("Variable  "+nv+" : "+t+" ["+at+"]");
        AglInstructionDefVar ai=new AglInstructionDefVar(t,at);
        ai.ligne=st.lineno();

	// Type a(1) equiv Type a=new Type(1);
	if(peek().equals("("))
	{
	  put(type.nom.dernier());
	  put("new");
	  put("=");
	}

	t=get();

        if(t.equals("="))
        {
          if(peek().equals("{")) ai.initiale=parseExpressionInitArray();
	  else                   ai.initiale=parseExpression();
	  trace("Initiale   : "+ai.initiale);
	  t=get();
        }

        r.addElement(ai);

        if(!(t.equals(",")||t.equals(";")))
	  syntaxe(", or ; needed and not "+t);
      }

    } while(!t.equals(";"));

    return r;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("do")) syntaxe("do needed and not "+t);

    AglBloc ab=parseBloc();

    t=get();
    if(!t.equals("while")) syntaxe("while needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);
    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglInstructionDoWhile ai=new AglInstructionDoWhile(ae,ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    trace("InsExpr   : "+eg);

    AglInstructionExpression ai=new AglInstructionExpression(eg);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionFor parseInstructionFor() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("for")) syntaxe("for needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);
    AglInstruction ad=parseInstructionGenerique();

    // (AglInstruction)parseBlocSimple().instructions.elementAt(0);
    // t=get();
    // if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglExpression at=null;
    if(!peek().equals(";")) at=parseExpression();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglInstruction af=new AglInstructionBloc(parseBlocVirgule(),true);

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();
    AglInstructionFor ai=new AglInstructionFor(ad,at,af,ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("if")) syntaxe("if needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);
    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc aoui=parseBloc();
    AglBloc anon=null;

    t=peek();
    if(t.equals("else"))
    {
      t=get();
      anon=parseBloc();
    }
    
    AglInstructionIf ai=new AglInstructionIf(ae,aoui,anon);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    syntaxe("Il n'y a pas de procedure en Hpp ("+get()+")");
    return null;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("return")) syntaxe("return needed and not "+t);

    AglExpression ae=parseExpression();

    AglInstructionReturn ai=new AglInstructionReturn(ae);
    ai.ligne=st.lineno();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    return ai;
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("switch")) syntaxe("switch needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    AglTableOrdonnee ac=new AglTableOrdonnee();
    AglBloc             ad=new AglBloc();

    while(peek().equals("default")||peek().equals("case"))
    {
      t=get();
      if(!t.equals("default")&&!t.equals("case"))
	syntaxe("default or case needed and not "+t);

      AglExpression ce=null;
      if(t.equals("case")) ce=parseExpression();

      t=get();
      if(!t.equals(":")) syntaxe(": needed and not "+t);

      AglBloc cb=null;
      t=peek();

      cb=new AglBloc();
      while(  !t.equals("case" )&&!t.equals("default")
	    &&!t.equals("break")&&!t.equals("}"))
      {
	if(t.equals("{"))
	  cb.instructions.addElement(parseInstructionBloc());
	else
	{
	  AglBloc bi=parseBlocSimple();
	  cb.instructions.addElement(bi.instructions.elementAt(0));
	}
	t=peek();
      }

      if(t.equals("break"))
      {
	get();

	t=get();
	if(!t.equals(";"))
	{
	  // BUG @GDX
	  trace("!!! label for break is ignored.");
	  trace("!!! This bug will be corrected later.");
	  t=get();
	}

	if(!t.equals(";")) syntaxe("; needed and not "+t);
      }

      if(ce!=null) ac.put(ce,cb);
      else         ad=cb;
    }

    t=get();
    if(!t.equals("}")) syntaxe("} needed and not "+t);

    AglInstructionSwitch ai=new AglInstructionSwitch(ae,ac,ad);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("throw")) syntaxe("throw needed and not "+t);

    AglExpression ae=parseExpression();

    AglInstructionThrow ai=new AglInstructionThrow(ae);
    ai.ligne=st.lineno();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    return ai;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("try")) syntaxe("try needed and not "+t);

    AglBloc ab=parseBloc();

    AglInstructionTry ai=new AglInstructionTry(ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("while")) syntaxe("while needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();
    AglInstructionWhile ai=new AglInstructionWhile(ae,ab);
    ai.ligne=st.lineno();
    return ai;
  }
}

