/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseFortran.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;
import java.io.*;

public class AglParseFortran extends AglParse
{
  FuStreamTokenizer st;
  Stack             pile;

  AglUnite   _unite;
  AglNom     _nom;
  AglType    _type;
  AglVecteur _parametres;

  public AglParseFortran()
  {
    if(AglType.cherche("CHARACTER")==null)
      new AglTypeAlias("CHARACTER",AglTypePrimitif.tchar,"Fortran");
    if(AglType.cherche("INTEGER")==null)
      new AglTypeAlias("INTEGER",AglTypePrimitif.tinteger,"Fortran");
    if(AglType.cherche("LOGICAL")==null)
      new AglTypeAlias("LOGICAL",AglTypePrimitif.tboolean,"Fortran");
    if(AglType.cherche("REAL"   )==null)
      new AglTypeAlias("REAL"   ,AglTypePrimitif.tdouble ,"Fortran");
  }

  private AglType devineType(String _n)
  {
    AglType r=AglType.cherche("REAL");

    char c=Character.toUpperCase(_n.charAt(0));
    if((c>='I')&&(c<='N'))
      r=AglType.cherche("INTEGER");

    return r;
  }

  public String extension()
  {
    return "f";
  }

  public String[] extensions()
  {
    return new String[] { "f", "for", "F", "FOR" };
  }

  short[] ll=new short[20000];

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    String sr="";
    LineNumberReader lnr=new LineNumberReader(new InputStreamReader(_input));
    int n=-1;

    while(lnr.ready())
    {
      String t=lnr.readLine();
      t=FuLib.replace(t,"\t","        ");

      if((t.length()>6)&&(t.charAt(0)!='C')&&(t.charAt(0)!='c'))
      {
	boolean space=true;
        if(t.charAt(5)==' ')
	{
	  sr+="\n";
	  space=false;
	  n++;
	  if(n<20000) ll[n]=(short)lnr.getLineNumber();
	}

        int nd=0,nf=4;
        while((nd<6)&&(t.charAt(nd)==' ')) nd++;
        while((nf>0)&&(t.charAt(nf)==' ')) nf--;
        if(nd<=nf)
        {
          //trace("N="+t.substring(nd,nf+1));
          int etiquette=Integer.parseInt(t.substring(nd,nf+1));
          sr+="LABEL_"+etiquette+" ";
        }

	if(space) sr+=' ';
        sr+=t.substring(6).trim();
      }
    }
    lnr.close();

    //trace(sr);
    //trace("___________________");

    st=new FuStreamTokenizer(new StringReader(sr),_fichier);
    st.resetSyntax();
    st.eolIsSignificant(true);
    st.quoteChar('"');
    st.quoteChar('\'');
    st.slashSlashComments(false);
    st.slashStarComments(false);
    st.wordChars('a','z');
    st.wordChars('A','Z');
    st.wordChars('0','9');
    st.wordChars('_','_');
    st.whitespaceChars(' ',' ');
    st.whitespaceChars('\t','\t');
    st.whitespaceChars('\n','\n');
    st.whitespaceChars('\r','\r');

    pile=new Stack();
  }

  public String get()
  {
    String r="";

    if(pile.empty())
    {
      try
      {
        st.nextToken();
        // trace("TOKEN="+st.ttype);

        switch(st.ttype)
        {
          case StreamTokenizer.TT_EOF   : r="";                         break;
          case StreamTokenizer.TT_WORD  : r=st.sval.toUpperCase();      break;
          case StreamTokenizer.TT_NUMBER: r=""+st.nval;                 break;
          case '"'                      : r="\""+ascii2C(st.sval)+"\""; break;
          case '\''                     : r="\""+ascii2C(st.sval)+"\""; break;
          default                       : r=""+(char)st.ttype;          break;
        }
      }
      catch(Exception e) { erreur(""+e+" [AglParse.get]"); }

      if(r.equals("."))
      {
        String op=get();
             if(peek().equals(".")) r="."+op+get();
        else put(op);
      }

      if((!r.equals(""))&&Character.isDigit(r.charAt(0)))
      {
        if(peek().equals("."))
        {
          r+=get();
          if(Character.isDigit(peek().charAt(0)))
            r+=get();
        }

        String m=peek();
	if(  (m.startsWith("E"))||(m.startsWith("e"))
	   ||(m.startsWith("D"))||(m.startsWith("d")))
        {
	  get();
	  r+="E"+m.substring(1);
          if(m.length()==1)
	  {
	    if((peek().equals("+"))||(peek().equals("-"))) r+=get();
	    r+=get();
	  }
        }
	/*
        if(  (peek().equals("E"))||(peek().equals("e"))
	   ||(peek().equals("D"))||(peek().equals("d")))
        {
	  r+="E"; //get();
          if((peek().equals("+"))||(peek().equals("-"))) r+=get();
          r+=get();
        }
	*/
      }

      if(r.startsWith("\"")&&r.endsWith("\"")&&peek().startsWith("\""))
	r=r.substring(0,r.length()-1)+"'"+get().substring(1);

      if(r.equals(".EQ."  )) r="==";
      if(r.equals(".NE."  )) r="!=";
      if(r.equals(".EQV." )) r="==";
      if(r.equals(".NEQV.")) r="!=";

      if(r.equals(".LT.")) r="<";
      if(r.equals(".GT.")) r=">";
      if(r.equals(".LE.")) r="<=";
      if(r.equals(".GE.")) r=">=";

      if(r.equals(".AND.")) r="&&";
      if(r.equals(".OR." )) r="||";
      if(r.equals(".NOT.")) r="!";

      if(r.equals(".TRUE." )) r="true";
      if(r.equals(".FALSE.")) r="false";

      if(r.equals("END")&&peek().equals("DO")) r+=get();
      if(r.equals("END")&&peek().equals("IF")) r+=get();
      if(r.equals("GO" )&&peek().equals("TO")) r+=get();

      if(r.equals("*")&&peek().equals("*")) r+=get();
      if(r.equals("/")&&peek().equals("/")) { get(); r="+"; }
      if(r.equals(":"))                     r=",";

      if(r.equals("ABS"  )) r="_abs";
      if(r.equals("ACOS" )) r="_acos";
      if(r.equals("AIMAG")) r="_aimag";
      if(r.equals("AINT" )) r="_aint";
      if(r.equals("AMAX0")) r="_amax0";
      if(r.equals("AMIN0")) r="_amin0";
      if(r.equals("AMAX1")) r="_amax1";
      if(r.equals("AMIN1")) r="_amin1";
      if(r.equals("ANINT")) r="_anint";
      if(r.equals("ASIN" )) r="_asin";
      if(r.equals("ATAN" )) r="_atan";
      if(r.equals("ATAN2")) r="_atan2";
      if(r.equals("CONJG")) r="_conjg";
      if(r.equals("COS"  )) r="_cos";
      if(r.equals("COSH" )) r="_cosh";
      if(r.equals("DIM"  )) r="_dim";
      if(r.equals("DPROD")) r="_dprod";
      if(r.equals("EXP"  )) r="_exp";
      if(r.equals("INDEX")) r="_index";
      if(r.equals("LEN"  )) r="_len";
      if(r.equals("LGE"  )) r="_lge";
      if(r.equals("LGT"  )) r="_lgt";
      if(r.equals("LLE"  )) r="_lle";
      if(r.equals("LLT"  )) r="_llt";
      if(r.equals("LOG"  )) r="_log";
      if(r.equals("LOG10")) r="_log10";
      if(r.equals("MAX"  )) r="_max";
      if(r.equals("MAX0" )) r="_max0";
      if(r.equals("MAX1" )) r="_max1";
      if(r.equals("MIN"  )) r="_min";
      if(r.equals("MIN0" )) r="_min0";
      if(r.equals("MIN1" )) r="_min1";
      if(r.equals("MOD"  )) r="_mod";
      if(r.equals("NINT" )) r="_nint";
      if(r.equals("SIGN" )) r="_sign";
      if(r.equals("SIN"  )) r="_sin";
      if(r.equals("SINH" )) r="_sinh";
      if(r.equals("SQRT" )) r="_sqrt";
      if(r.equals("TAN"  )) r="_tan";
      if(r.equals("TANH" )) r="_tanh";

      if(r.equals("CHAR" )&&peek().equals("(")) r="_char";
      if(r.equals("CMPLX")&&peek().equals("(")) r="_complex";
      if(r.equals("ICHAR")&&peek().equals("(")) r="_ichar";
      if(r.equals("DBLE" )&&peek().equals("(")) r="_double";
      if(r.equals("FLOAT")&&peek().equals("(")) r="_float";
      if(r.equals("INT"  )&&peek().equals("(")) r="_int";
      if(r.equals("REAL" )&&peek().equals("(")) r="_real";

      if(r.equals("CLOSE"  )) r="_close";
      if(r.equals("INQUIRE")) r="_inquire";
      if(r.equals("PRINT"  )) r="_print";
      if(r.equals("READ"   )) r="_read";
      if(r.equals("REWIND" )) r="_rewind";
      if(r.equals("WRITE"  )) r="_write";

      // trace(">>> "+r);
    }
    else r=(String)pile.pop();

    return r;
  }

  public void put(String ih)
  {
    pile.push(ih);
  }

  public String getNoCR()
  {
    String r="";
    do { r=get(); } while(r.equals("\n"));
    return r;
  }

  public String peekNoCR()
  {
    String r=getNoCR();
    put(r);
    return r;
  }

  private int pp=0;
  public int ligne()
  {
    if(st==null) return 0;

    int r=st.lineno();
    if(MaxLines!=0)
    {
      int p=100*r/MaxLines;
      if(p!=pp) progres(p);
      pp=p;
    }

    if(r>=20000) r=-1;
    else r=ll[r];

    return r;
  }

  public String fichier()
  {
    String r=st.origin();
    int i=r.lastIndexOf(File.separator);
    if(i>=0) r=r.substring(i+1);
    return r;
  }

  // Parse

  public void parseFichier() throws Exception
  {
    parseUnite();
  }

  public AglUnite parseUnite() throws Exception
  {
    AglUnite au=new AglUnite(st.origin());
    au.langage="Fortran";

    _unite=au;

    AglClasse ac=parseClasse();
    au.types.put(""+ac,ac);

    return au;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    erreur("Il n'y a pas de visibilite en Fortran ("+get()+")");
    return null;
  }

  public AglNom parseNom() throws Exception
  {
    AglNom an=new AglNom();

    trace("WN>"+peek());
    an.identificateurs.addElement(getNoCR());

    an.ligne=st.lineno();
    return an;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    AglListeNoms al=new AglListeNoms();

    String t="";
    do
    {
      trace("WLN>"+peek());
      al.noms.addElement(parseNom());
      t=peek();
      if(t.equals(",")) get();
    }
    while(t.equals(","));

    al.ligne=st.lineno();
    return al;
  }

  public AglType parseType() throws Exception
  {
    trace("WT>"+peekNoCR());

    String  t=getNoCR();
    AglType r=AglType.cherche(t);

    if(t.equals("CHARACTER")&&peek().equals("*"))
    {
      get(); r=new AglTypeTableau(r,new AglExpressionGenerique(get()));
    }

    if(r==null) syntaxe("Unknown type : "+t);
    return r;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    erreur("Il n'y a pas de paquet en Fortran ("+get()+")");
    return null;
  }
  
  public AglImport parseImport() throws Exception
  {
    erreur("Il n'y a pas d'import en Fortran ("+get()+")");
    return null;
  }

  public AglClasse parseClasse() throws Exception
  {
    AglClasse ac=(AglClasse) AglType.cherche("Global");
    if(ac==null) ac=new AglClasse(new AglNom("Global"));
    ac.visibilite.vpublic=true;
    ac.visibilite.vstatic=true;

    String t="";

    suivi("Classe      : "+ac.nom);
    ac.proprietaire=_unite;
    ac.ligne       =st.lineno();
    ac.elementaire =false;
    
  // Body

    do
    {
      trace("WBC>"+peekNoCR());
      t=getNoCR();

      //System.err.println("$$$$$ "+t);

      if(t.equals("EXTERNAL"))
      {
        _nom=parseNom();
        trace("Externe   : "+_nom);
      }
      else
      if(t.equals("PROGRAM"))
      {
        _type=AglTypePrimitif.tvoid;
        _nom=parseNom();
        AglMethode am=parseProgramme();
        ac.methodes.put(am.signature(),am);
      }
      else
      if(t.equals("FUNCTION"))
      {
        _type=AglTypePrimitif.tvoid;
        _nom =parseNom();
        AglMethode am=parseMethode();
        ac.methodes.put(am.signature(),am);
      }
      else
      if(t.equals("SUBROUTINE"))
      {
        _type=AglTypePrimitif.tvoid;
        _nom =parseNom();
        AglMethode am=parseMethode();
        ac.methodes.put(am.signature(),am);
      }
      else
      {
	// implicit program
	_type=AglTypePrimitif.tvoid;
        _nom=new AglNom("main_program");
        AglMethode am=parseProgramme();
        ac.methodes.put(am.signature(),am);
      }

    } while(!peekNoCR().equals(""));

    return ac;
  }

  public AglListeNoms parseMothers() throws Exception
  {
    erreur("Il n'y a pas de meres en Fortran ("+get()+")");
    return null;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    erreur("Il n'y a pas d'interfaces en Fortran ("+get()+")");
    return null;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    erreur("Il n'y a pas d'exceptions ("+get()+")");
    return null;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    erreur("Il n'y a pas de constructeurs en Fortran ("+get()+")");
    return null;
  }

  public AglMethode parseMethode() throws Exception
  {
    suivi("Methode     : "+_nom+" ["+_type+"]");

    AglMethode m=new AglMethode(_nom.dernier().toLowerCase(),_type);
    m.visibilite.vpublic=true;
    m.visibilite.vstatic=true;
    m.ligne=st.lineno();

    m.parametres=parseParametres();
    m.corps=parseBloc();

    if(!peekNoCR().equals("END")) syntaxe("END needed and not "+peekNoCR());
    getNoCR();

    return m;
  }

  public AglMethode parseProgramme() throws Exception
  {
    suivi("Programme   : "+_nom+" ["+_type+"]");

    AglMethode m=new AglMethode(_nom.dernier().toLowerCase(),_type);
    m.visibilite.vpublic=true;
    m.visibilite.vstatic=true;
    m.ligne=st.lineno();

    m.corps=parseBloc();

    /*
    if(peekNoCR().equals("")) avertissement("Assuming END of program");
    else
    */

    if(!peekNoCR().equals("END")) syntaxe("END needed and not "+peekNoCR());
    getNoCR();

    return m;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    AglVecteur r=new AglVecteur();
    String     t="";

    do
    {
      trace("Attribut    : "+_nom+" ["+_type+"]");

      AglAttribut a=new AglAttribut(_nom.dernier().toLowerCase(),_type);
      a.ligne=st.lineno();

      t=get();
      if(t.equals("="))
      {
        a.initiale=parseExpression();
        trace("Initiale    : "+a.initiale);
      }
      else
      if(t.equals(","))
        _nom=parseNom();
      else
      if(!t.equals(";"))
        syntaxe(", or ; needed and not "+t);

    } while(!t.equals(";"));

    return r;
  }

  public AglVecteur parseParametres() throws Exception
  {
    int        np=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    //t=getNoCR();
    //if(!t.equals("(")) syntaxe("( needed and not "+t);

    if(peekNoCR().equals("("))
    {
      getNoCR();
      while(!(t=peekNoCR()).equals(")"))
      {
	String nom=getNoCR().toLowerCase();
	np++;
	trace("Parametre "+np+" : "+nom);
	AglParametre p=new AglParametre(nom,devineType(nom));
	r.addElement(p);

	t=peekNoCR();
	     if( t.equals(",")) getNoCR();
	else if(!t.equals(")")) syntaxe(") or , needed and not "+t);
      }
      getNoCR();
    }

    _parametres=r;
    return r;
  }

  public AglVecteur parseArgs() throws Exception
  {
    int        na=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    //t=peek();
    //if(!(t.equals("("))) syntaxe("( needed and not "+t);

    if(peek().equals("("))
    {
      get();
      while(!(t=peekNoCR()).equals(")"))
      {
	AglExpression ae=parseExpression();
	na++;
	trace("Argument  "+na+" : "+ae);
	r.addElement(ae);
	
	t=peekNoCR();
	     if( t.equals(",")) getNoCR();
	else if(!t.equals(")")) syntaxe(") or , needed and not "+t);
      }
      getNoCR();
    }

    return r;
  }

  public AglBloc parseBloc() throws Exception
  {
    String t=peekNoCR();

    if(t.equals("IMPLICIT"))
    {
      while(peekNoCR().equals("DOUBLE"))
      {
	getNoCR();
	while(!get().equals("\n")) ; // trace("IMPLICIT: "+get());
      }
    }
    else
    if(t.equals("INCLUDE"))
    {
      trace("Include   : "+getNoCR()+" (ignored)");
    }

    return parseBlocMultiple("","");
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    return parseBlocMultiple("","");
  }

  public AglBloc parseBlocMultiple(String ss1, String ss2) throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t="";

    while(  (!peekNoCR().equals(ss1))
          &&(!peekNoCR().equals(ss2))
          &&(!peekNoCR().equals("END")))
    {
      trace("WBM>>> "+peek());
      t=get();

      if(t.equals("\n")) ;
      else
      if(t.startsWith("LABEL_"))
      {
        AglInstructionLabel ai=new AglInstructionLabel("L"+t.substring(6));
        ai.ligne=st.lineno();
        ab.instructions.addElement(ai);
      }
      else
      if(AglType.table.get(t)!=null)
      {
        put(t);
        for(Enumeration e=parseInstructionDefVar().elements(); e.hasMoreElements(); )
          ab.instructions.addElement((AglInstructionDefVar)e.nextElement());
      }
      else
      {
        put(t);
        AglInstruction ai=null;
        
             if(t.equals("CALL"))        ai=parseInstructionProcedure  ();
        else if(t.equals("CONTINUE"))    ai=parseInstructionContinue   ();
        else if(t.equals("COMMON"))
        {
          for(Enumeration e=parseInstructionCommon().elements(); e.hasMoreElements(); )
            ab.instructions.addElement((AglInstructionDefVar)e.nextElement());
        }
        else if(t.equals("DIMENSION"))
        {
          for(Enumeration e=parseInstructionDimension().elements(); e.hasMoreElements(); )
            ab.instructions.addElement((AglInstructionDefVar)e.nextElement());
        }
        else if(t.equals("DATA"))
        {
          for(Enumeration e=parseInstructionData().elements(); e.hasMoreElements(); )
            ab.instructions.addElement((AglInstructionAffectation)e.nextElement());
        }
        else if(t.equals("DO"))          ai=parseInstructionFor        ();
        else if(t.equals("ENTRY"))       ai=parseInstructionGenerique  ();
        else if(t.equals("EQUIVALENCE")) ai=parseInstructionGenerique  ();
        else if(t.equals("FORMAT"))      ai=parseInstructionGenerique  ();
        else if(t.equals("GOTO"))        ai=parseInstructionGoto       ();
        else if(t.equals("IF"))          ai=parseInstructionIf         ();
        else if(t.equals("IMPLICIT"))    ai=parseInstructionGenerique  ();
        else if(t.equals("INTRINSIC"))   ai=parseInstructionGenerique  ();
        else if(t.equals("PARAMETER"))   ai=parseInstructionGenerique  ();
        else if(t.equals("RETURN"))      ai=parseInstructionReturn     ();
        else if(t.equals("SAVE"))        ai=parseInstructionGenerique  ();
        else if(t.equals("STOP"))        ai=parseInstructionGenerique  ();
        else if(t.equals("WHILE"))       ai=parseInstructionWhile      ();
        else if(t.equals("WRITE"))       ai=parseInstructionGenerique  ();
        else
        {
          AglExpression gauche=parseExpression();

          t=peek();
               if(t.equals("="))   ai=parseInstructionAffectation(gauche);
          else if(t.equals("\n"))  ai=parseInstructionExpression (gauche);
          else if(  t.equals("+=")||t.equals("-=")
		  ||t.equals("*=")||t.equals("/=")||t.equals("%=")
                  ||t.equals("&=")||t.equals("|=")||t.equals("^="))
          {
            t=get();
            AglExpression ae=new AglExpressionBinaire(t.substring(0,1),gauche,parseExpression());
            ae.ligne=st.lineno();

            ai=new AglInstructionAffectation(gauche,ae);
            ai.ligne=st.lineno();

            t=get();
            if(!t.equals("\n")) syntaxe("\\n needed and not "+t);
          }
          else ai=parseInstructionGenerique  ();
        }

        if(ai!=null) ab.instructions.addElement(ai);
      }
    }

    peekNoCR();
    return ab;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t=get();

    trace("WBS>>> "+t+" | "+peek());

        String u=peek();
        put(t);
        AglInstruction ai=null;
        
             if(t.equals("{"))           syntaxe("{ is not wanted there");
        else if(t.equals("CALL"))        ai=parseInstructionProcedure  ();
        else if(t.equals("CONTINUE"))    ai=parseInstructionContinue   ();
        else if(t.equals("COMMON"))
        {
          for(Enumeration e=parseInstructionCommon().elements(); e.hasMoreElements(); )
            ab.instructions.addElement((AglInstructionDefVar)e.nextElement());
        }
        else if(t.equals("DIMENSION"))
        {
          for(Enumeration e=parseInstructionDimension().elements(); e.hasMoreElements(); )
            ab.instructions.addElement((AglInstructionDefVar)e.nextElement());
        }
        else if(t.equals("DATA"))
        {
          for(Enumeration e=parseInstructionData().elements(); e.hasMoreElements(); )
            ab.instructions.addElement((AglInstructionAffectation)e.nextElement());
        }
        else if(t.equals("DO"))          ai=parseInstructionFor        ();
        else if(t.equals("ENTRY"))       ai=parseInstructionGenerique  ();
        else if(t.equals("EQUIVALENCE")) ai=parseInstructionGenerique  ();
        else if(t.equals("FORMAT"))      ai=parseInstructionGenerique  ();
        else if(t.equals("GOTO"))        ai=parseInstructionGoto       ();
        else if(t.equals("IF"))          ai=parseInstructionIf         ();
        else if(t.equals("IMPLICIT"))    ai=parseInstructionGenerique  ();
        else if(t.equals("INTRINSIC"))   ai=parseInstructionGenerique  ();
        else if(t.equals("PARAMETER"))   ai=parseInstructionGenerique  ();
        else if(t.equals("RETURN"))      ai=parseInstructionReturn     ();
        else if(t.equals("SAVE"))        ai=parseInstructionGenerique  ();
        else if(t.equals("STOP"))        ai=parseInstructionGenerique  ();
        else if(t.equals("WHILE"))       ai=parseInstructionWhile      ();
        else if(t.equals("WRITE"))       ai=parseInstructionGenerique  ();
        else
        {
          AglExpression gauche=parseExpression();

          t=peek();
               if(t.equals("="))   ai=parseInstructionAffectation(gauche);
          else if(t.equals("\n"))  ai=parseInstructionExpression (gauche);
          else if(  t.equals("+=")||t.equals("-=")
		  ||t.equals("*=")||t.equals("/=")||t.equals("%=")
                  ||t.equals("&=")||t.equals("|=")||t.equals("^="))
          {
            t=get();
            AglExpression ae=new AglExpressionBinaire(t.substring(0,1),gauche,parseExpression());
            ae.ligne=st.lineno();

            ai=new AglInstructionAffectation(gauche,ae);
            ai.ligne=st.lineno();

            t=get();
            if(!t.equals("\n")) syntaxe("\\n needed and not "+t);
          }
          else ai=parseInstructionGenerique();
        }

        if(ai!=null) ab.instructions.addElement(ai);

    return ab;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    erreur("Il n'y a pas de bloc-virgule en Fortran ("+get()+")");
    return null;
  }

// Expressions

  public AglExpression parseExpression() throws Exception
  {
    AglExpression ae;

    trace("WE>>> "+peek());

         if(peek().equals("+" ))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("-" ))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("++"))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("--"))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("!" ))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("("))      ae=parseExpressionParenthesee ();
    else
    {
      String  t,u;
      t=get();
      u=peek();
      put(t);

      trace("WE>>> "+t+" | "+u);

      if(u.equals("("))
      {
        if(t.startsWith("_"))
        {
          ae=parseExpressionFonction();
        }
        else
        {
          t=get();
          u=get();
          if(!u.equals("(")) syntaxe("( needed and not "+u);
          ae=new AglExpressionIndice(new AglExpressionGenerique(t.toLowerCase()), parseExpression());
          ae.ligne=st.lineno();
          while(peek().equals(","))
          {
            get();
            ae=new AglExpressionIndice(ae, parseExpression());
            ae.ligne=st.lineno();
          }

          u=get();
          if(!u.equals(")")) syntaxe(") needed and not "+u);
        }
      }
      else
      {
        t=get();
        if(t.equals(".")&&(u.length()>0)&&Character.isDigit(u.charAt(0)))
          { t="0."+u; get(); }
        ae=new AglExpressionGenerique(t.toLowerCase());
        ae.ligne=st.lineno();
      }
    }

    boolean encore=true;
    while(encore)
    {
      encore=false;
      String u;

      trace("Expression  : "+ae+" ["+ae.getClass()+"]");
      u=peek();
      trace("WEA>>> "+u);
      
      if(u.equals("\n")||u.equals(","))
	encore=false;
      else
      if(u.equals("="))
      {
	get();
	ae=new AglExpressionAffectation(ae,parseExpression());
	ae.ligne=st.lineno();
	encore=true;
      }
      else
      if(  (u.equals("+"  ))||(u.equals("-"))
         ||(u.equals("*"  ))||(u.equals("/"))
         ||(u.equals("%"  ))||(u.equals("**"))
         ||(u.equals("==" ))||(u.equals("!="))
         ||(u.equals("<"  ))||(u.equals(">"))
         ||(u.equals("<=" ))||(u.equals(">="))
         ||(u.equals("<<" ))||(u.equals(">>"))
         ||(u.equals("<<<"))||(u.equals(">>>"))
         ||(u.equals("&&"))||(u.equals("||"))
         ||(u.equals("&"))||(u.equals("|"))||(u.equals("^"))
         ||(u.equals("instanceof")))
      {
	ae=new AglExpressionBinaire(get(),ae,parseExpression());
	ae.ligne=st.lineno();
	encore=true;
      }
    }

    return ae;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    String t="";
    String body="";
    int    np=0;
    int    nt=0;

    do
    {
      t=get();
      trace("WEG>>> "+t);

      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      if(t.equals("]")) nt--;
      if(t.equals("[")) nt++;
      
           if((np<0)&&(t.equals(")"))) ;
      else if((nt<0)&&(t.equals("]"))) ;
      else if((np==0)&&(nt==0)&&(t.equals("\n")||t.equals(","))) ;
      else
      {
        if(!body.equals("")) body+=" ";
        body+=t;
      }
    } while(  (np>0)
            ||(nt>0)
	    ||((np==0)&&(nt==0)&&(!(t.equals("\n")||t.equals(",")))));

    if(t.equals("\n")||t.equals(",")) put(t);
    if((np<0)&&t.equals(")")) put(t);
    if((nt<0)&&t.equals("]")) put(t);

    body=body.toLowerCase();
    AglExpressionGenerique ae=new AglExpressionGenerique(body);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite() throws Exception
  {
    String        op=get();

    trace("Unaire     : "+op);
    AglExpression ed=parseExpression();

    AglExpressionUnaireDroite ae=new AglExpressionUnaireDroite(op,ed);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    AglExpression eg=new AglExpressionGenerique(get());
    String        op=get();

    trace("Binaire    : "+op);
    AglExpression ed=parseExpression();

    AglExpressionBinaire ae=new AglExpressionBinaire(op,eg,ed);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    String t="";

    String nom=get();
    AglVecteur le =parseArgs();
    trace("Fonction    : "+nom+" ("+le.size()+" arguments)");

    AglExpressionFonction ae=new AglExpressionFonction(nom);
    ae.ligne=st.lineno();
    for(Enumeration e=le.elements(); e.hasMoreElements(); )
      ae.arguments.addElement(e.nextElement());
    return ae;
  }

  public AglExpression parseExpressionNew() throws Exception
  {
    erreur("Il n'y a pas de new en Fortran ("+get()+")");
    return null;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpressionParenthesee ae=new AglExpressionParenthesee(parseExpression());

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    erreur("Il n'y a pas de (cast) en Fortran ("+get()+")");
    return null;
  }

// Instructions

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    String  t="";
    String body="";

    do
    {
      t=get();

      if(!t.equals("\n"))
      {
	trace("WIG>>> "+t);
        if(!body.equals("")) body+=" ";
        body+=t;
      }
    } while(!t.equals("\n"));

    body=body.toLowerCase().trim();
    trace("Instruction : "+body);
    AglInstructionGenerique ai=new AglInstructionGenerique(body);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionAffectation parseInstructionAffectation(AglExpression eg) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("=")) syntaxe("= needed and not "+t);

    AglExpression ed=parseExpression();

    t=get();
    if(!t.equals("\n")) syntaxe("\\n needed and not "+t);

    trace("Affectation: "+eg+" <-- "+ed);

    AglInstructionAffectation ai=new AglInstructionAffectation(eg,ed);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    AglBloc ab=parseBloc();
    AglInstructionBloc ai=new AglInstructionBloc(ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    erreur("Il n'y a pas d'instruction catch en Fortran ("+get()+")");
    return null;
  }

  public AglInstructionContinue parseInstructionContinue() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("CONTINUE")) syntaxe("CONTINUE needed and not "+t);

    trace("Continue");

    t=get();
    if(!t.equals("\n")) syntaxe("\\n needed and not "+t);

    AglInstructionContinue ai=new AglInstructionContinue("");
    ai.ligne=st.lineno();
    return ai;
  }  

  public AglVecteur parseInstructionCommon() throws Exception
  {
    String     t="";
    AglVecteur r=new AglVecteur();

    t=get();
    if(!t.equals("COMMON")) syntaxe("COMMON needed and not "+t);

    t=get();
    if(!t.equals("/")) syntaxe("COMMON needed and not "+t);

    String classe=get();
    classe=classe.substring(0,1).toUpperCase()+classe.substring(1).toLowerCase();

    t=get();
    if(!t.equals("/")) syntaxe("/ needed and not "+t);

    do 
    {
      String  nom =get().toLowerCase();
      AglType type=devineType(nom);

      if(peek().equals("("))
      {
        trace("WCOM>"+peek());
        get();
        type=new AglTypeTableau(type,parseExpression());

        while(peek().equals(","))
        {
          get();
          type=new AglTypeTableau(type,parseExpression());
        }

        if(!peek().equals(")")) syntaxe(") needed and not "+peek());
        get();
      }

      trace("Common     : "+nom+" ["+type+"]");
      AglExpression        ae=new AglExpressionGenerique(classe+"."+nom);
      ae.ligne=st.lineno();
      AglInstructionDefVar ai=new AglInstructionDefVar(nom,type,ae);
      ai.ligne=st.lineno();
      r.addElement(ai);

      t=get();
      if(!(t.equals(",")||t.equals("\n")))
        syntaxe(", or \\n needed and not "+t);
   
   } while(!t.equals("\n"));

    trace("$$$$$$$$$$$ common : "+r.size());
    return r;
  }


  public AglVecteur parseInstructionData() throws Exception
  {
    String     t="";
    AglVecteur r=new AglVecteur();

    t=get();
    if(!t.equals("DATA")) syntaxe("DATA needed and not "+t);

    do 
    {
      String nom=get().toLowerCase();

      if(peek().equals("/"))
      {
        get();
        String valeur=get();

        if(peek().equals("/"))
        {
          AglExpressionGenerique eg=new AglExpressionGenerique(nom);
          AglExpressionGenerique ed=new AglExpressionGenerique(valeur);

          AglInstructionAffectation ai=new AglInstructionAffectation(eg,ed);
          ai.ligne=st.lineno();
          r.addElement(ai);
          get();
        }
        else
        {
          while(!peek().equals("/")) valeur+=get();
          valeur="{ "+valeur+" }";
          AglExpressionGenerique eg=new AglExpressionGenerique(nom);
          AglExpressionGenerique ed=new AglExpressionGenerique(valeur);

          AglInstructionAffectation ai=new AglInstructionAffectation(eg,ed);
          ai.ligne=st.lineno();
          r.addElement(ai);
          get();
        }
      }
      else
      {
        AglExpressionGenerique eg=new AglExpressionGenerique(nom);
        AglExpressionGenerique ed=new AglExpressionGenerique("???");

        AglInstructionAffectation ai=new AglInstructionAffectation(eg,ed);
        ai.ligne=st.lineno();
        r.addElement(ai);
      }

      t=peek();
      if(t.equals(",")) t=get();

    } while(!t.equals("\n"));

    trace("$$$$$$$$$$$ data : "+r.size());
    return r;
  }


  public AglVecteur parseInstructionDefVar() throws Exception
  {
    String     t ="";
    int        nv=0;
    AglVecteur r =new AglVecteur();

    AglType type=parseType();

    do 
    {
      t=get().toLowerCase();

      if(!t.equals("\n"))
      {

        if(peek().equals("("))
        {
          trace("WTT>"+peek());
          get();
          type=new AglTypeTableau(type,parseExpression());

          while(peek().equals(","))
          {
            get();
            type=new AglTypeTableau(type,parseExpression());
          }

          if(!peek().equals(")")) syntaxe(") needed and not "+peek());
          get();
        }

        if(peek().equals("*"))
        {
          get();
          type=new AglTypeTableau(type,parseExpression());
        }

        nv++;

        boolean isaparam=false;

        if(_parametres!=null)
          for(Enumeration e=_parametres.elements(); e.hasMoreElements(); )
          {
	    AglParametre p=(AglParametre)e.nextElement();
            if(p.nom.equals(t))
            {
              isaparam=true;
              p.type=type;
              break;
            }
          }

        if(!isaparam)
        {
          trace("Variable  "+nv+" : "+t+" ["+type+"]");
          AglInstructionDefVar ai=new AglInstructionDefVar(t,type);
          ai.ligne=st.lineno();
          r.addElement(ai);
        }

        t=get();
        if(!(t.equals(",")||t.equals("\n")))
	  syntaxe("DefVar: , or \\n needed and not "+t);
      }

    } while(!t.equals("\n"));

    trace("$$$$$$$$$$$ defvar : "+r.size());
    return r;
  }

  public AglVecteur parseInstructionDimension() throws Exception
  {
    String     t ="";
    int        nv=0;
    AglVecteur r =new AglVecteur();

    t=get();
    if(!t.equals("DIMENSION")) syntaxe("DIMENSION needed and not "+t);

    do 
    {

      t=get().toLowerCase();
      AglType type=devineType(t);

      boolean isaparam=false;
      AglParametre param=null;

      if(_parametres!=null)
        for(Enumeration e=_parametres.elements(); e.hasMoreElements(); )
        {
	  AglParametre p=(AglParametre)e.nextElement();
          if(p.nom.equals(t))
          {
            isaparam=true;
            param=p;
            type=p.type;
            break;
          }
        }

      if(!t.equals("\n"))
      {

        if(peek().equals("("))
        {
          trace("WDIM>"+peek());
          get();
          type=new AglTypeTableau(type,parseExpression());

          while(peek().equals(","))
          {
            get();
            type=new AglTypeTableau(type,parseExpression());
          }

          if(!peek().equals(")")) syntaxe(") needed and not "+peek());
          get();
        }

        if(peek().equals("*"))
        {
          get();
          type=new AglTypeTableau(type,parseExpression());
        }

        if(!isaparam)
        {
          nv++;

          trace("Variable  "+nv+" : "+t+" ["+type+"]");
          AglInstructionDefVar ai=new AglInstructionDefVar(t,type);
          ai.ligne=st.lineno();
          r.addElement(ai);
        }
        else
        {
          param.type=type;
          trace("Dimension    : "+param.nom+" ["+type+"]");
        }

        t=get();
        if(!(t.equals(",")||t.equals("\n")))
	  syntaxe("DefVar: , or \\n needed and not "+t);
      }

    } while(!t.equals("\n"));

    trace("$$$$$$$$$$$ dimension : "+r.size());
    return r;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction try/catch en Fortran ("+get()+")");
    return null;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("\n")) syntaxe("\\n needed and not "+t);

    trace("InsExpr   : "+eg);

    AglInstructionExpression ai=new AglInstructionExpression(eg);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionFor parseInstructionFor() throws Exception
  {
    String t="";
    String v="";
    String l="";
    int    nv=0;

    t=get();
    if(!t.equals("DO")) syntaxe("DO needed and not "+t);


    v=get().toLowerCase();

    // test if it is a label
    try
    {
      new Integer(v);
      l=v;
      v=get().toLowerCase();
    }
    catch(Exception ex) { }

    t=get();
    if(!t.equals("=")) syntaxe("= needed and not "+t);

    AglExpression binf=parseExpression();

    t=get();
    if(!t.equals(",")) syntaxe(", needed and not "+t);

    AglExpression bsup=parseExpression();

    AglExpression binc=new AglExpressionGenerique("1");
    if(peek().equals(","))
    {
      get();
      if(peek().equals("-")) binc=new AglExpressionUnaireDroite(get(),parseExpression());
      else binc=parseExpression();
    }

    AglInstruction aivd=new AglInstructionDefVar(v,AglTypePrimitif.tinteger,binf);
    AglExpression  aevt=new AglExpressionBinaire("!=",new AglExpressionGenerique(v),bsup);
    AglInstruction aivf=new AglInstructionExpression(new AglExpressionGenerique(v+"+="+binc));

    AglBloc ab;

    if(l.equals(""))
    {
      ab=parseBlocMultiple("ENDDO","");
      t=get();
      if(!t.equals("ENDDO")) syntaxe("ENDDO needed and not "+t);
    }
    else
    {
      ab=parseBlocMultiple("LABEL_"+l,"");
      t=get();
      if(!t.equals("LABEL_"+l)) syntaxe("LABEL_"+l+" needed and not "+t);
      t=get();
      if(!t.equals("CONTINUE")) syntaxe("CONTINUE needed and not "+t);
    }

    AglInstructionFor ai=new AglInstructionFor(aivd,aevt,aivf,ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionGoto parseInstructionGoto() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("GOTO")) syntaxe("GOTO needed and not "+t);

    String nom="";

    if(!peek().equals("("))
    {
      nom="L"+get();
    }
    else
    {
      // goto calcule
      while(!peek().equals("\n"))
	nom+=get();      
    }

    trace("Goto       : "+nom);
    t=get();
    if(!t.equals("\n")) syntaxe("\\n needed and not "+t);

    AglInstructionGoto ai=new AglInstructionGoto(nom);
    ai.ligne=st.lineno();
    return ai;
  }  

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("IF")) syntaxe("IF needed and not "+t);

    int line_if=st.lineno();
    if(line_if>=20000) line_if=0;

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);
    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc aoui=null;
    AglBloc anon=null;

    if(peek().equals("THEN"))
    {
      get();
      peekNoCR();
      aoui=parseBlocMultiple("ELSE","ENDIF");

      if(peekNoCR().equals("ELSE"))
      {
        getNoCR();
        if(peekNoCR().equals("IF"))
	  anon=parseBlocSimple();
	else
	  anon=parseBlocMultiple("ELSE","ENDIF");
      }
      else
      if(peekNoCR().equals("ENDIF"))
	getNoCR();
      else
	syntaxe("ENDIF needed and not "+peekNoCR()+" (^"+ll[line_if]+")");
    }
    else aoui=parseBlocSimple();
    
    AglInstructionIf ai=new AglInstructionIf(ae,aoui,anon);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("CALL")) syntaxe("CALL needed and not "+t);

    String     nom=get().toLowerCase();
    AglVecteur le =parseArgs();
    trace("Procedure   : "+nom+" ("+le.size()+" arguments)");

    t=get();
    if(!t.equals("\n")) syntaxe("\\n needed and not "+t);

    AglInstructionProcedure ae=new AglInstructionProcedure(nom);
    ae.ligne=st.lineno();
    for(Enumeration e=le.elements(); e.hasMoreElements(); )
      ae.arguments.addElement(e.nextElement());
    return ae;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("RETURN")) syntaxe("RETURN needed and not "+t);

    t=get();
    if(!t.equals("\n")) syntaxe("\\n needed and not "+t);

    return new AglInstructionReturn();
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    erreur("Il n'y a pas de switch en Fortran ("+get()+")");
    return null;
  }

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    erreur("Il n'y a pas de throw en Fortran ("+get()+")");
    return null;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    erreur("Il n'y a pas d'instruction try en Fortran ("+get()+")");
    return null;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("WHILE")) syntaxe("WHILE needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();
    AglInstructionWhile ai=new AglInstructionWhile(ae,ab);
    ai.ligne=st.lineno();
    return ai;
  }
}
