/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseIdl.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;
import java.io.*;

public class AglParseIdl extends AglParse
{
  private final static String CR="<CR>";

  private FuStreamTokenizer st_;
  private Stack             pile_;
  private AglUnite          _unite;
  private AglNom            _nom;
  private AglType           _type;
  private AglClasse         _classe;
  private AglPaquet         _paquet;
  private AglVecteur        commentaires_;

  public AglParseIdl()
  {
    // considere IDL long as Java int
    new AglTypeAlias("long"     ,AglTypePrimitif.tinteger,"Idl");
    new AglTypeAlias("long long",AglTypePrimitif.tlong   ,"Idl");

    if(AglType.cherche("String")==null)
      (new AglClasse(new AglNom("java.lang.String"))).elementaire=true;

    if(AglType.cherche("string")==null)
      new AglTypeAlias("string",AglType.cherche("String"),"Idl");
    if(AglType.cherche("wstring")==null)
      new AglTypeAlias("wstring",AglType.cherche("String"),"Idl");
  }

  public String extension()
  {
    return "idl";
  }

  public String[] extensions()
  {
    return new String[] { "idl", "IDL" };
  }

  public boolean optionAnalyserCommentaires()
    { return true; }

  private void setSourceSyntax()
  {
    st_.resetSyntax();
    st_.eolIsSignificant(false);
    st_.quoteChar('"');
    st_.quoteChar('\'');
    st_.slashSlashComments(!AglOptions.analyserCommentaires);
    st_.slashStarComments (!AglOptions.analyserCommentaires);

    // Directives are ignored
    st_.commentChar('#'); 

    st_.wordChars('a','z');
    st_.wordChars('A','Z');
    st_.wordChars('0','9');
    st_.wordChars('_','_');
    st_.wordChars(129,255);
    st_.whitespaceChars(' ',' ');
    st_.whitespaceChars('\t','\t');
    st_.whitespaceChars('\n','\n');
    st_.whitespaceChars('\r','\r');
  }

  private void setCommentSyntax()
  {
    st_.resetSyntax();
    st_.eolIsSignificant(true);
    st_.wordChars(0,255);
    st_.whitespaceChars(0  ,32);
    st_.whitespaceChars(127,160);
    st_.ordinaryChar('*');
    st_.ordinaryChar('/');
  }

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    st_          =new FuStreamTokenizer(_input,_fichier);
    pile_        =new Stack();
    //anonymes_  =0;
    commentaires_=new AglVecteur();

    setSourceSyntax();

    /*
    st=new FuStreamTokenizer(_input,_fichier);
    st.resetSyntax();
    st.eolIsSignificant(false);
    st.quoteChar('"');
    st.quoteChar('\'');
    st.commentChar('#');
    st.slashSlashComments(true);
    st.slashStarComments(true);
    st.wordChars('a','z');
    st.wordChars('A','Z');
    st.wordChars('0','9');
    st.wordChars('_','_');
    st.whitespaceChars(' ',' ');
    st.whitespaceChars('\t','\t');
    st.whitespaceChars('\n','\n');
    st.whitespaceChars('\r','\r');
    */
  }

  private static void addCommentKey(AglTableTriee t, String k, String v)
  {
    v=v.trim();
    v=FuLib.replace(v,CR+"*","");
    v=FuLib.replace(v,CR    ,"");
    v=FuLib.replace(v,"  "," ");
    //while(v.endsWith(" "+CR)) v=v.substring(0,v.length()-CR.length()-1);

    if(k.startsWith("@alma_")||k.startsWith("@alma-"))
	k="@"+k.substring(6);

    if("@pre"          .equals(k)) k="@require";
    if("@precondition" .equals(k)) k="@require";
    if("@post"         .equals(k)) k="@ensure";
    if("@postcondition".equals(k)) k="@ensure";
    if("@inv"          .equals(k)) k="@invariant";
    if("@cond"         .equals(k)) k="@check";
    if("@condition"    .equals(k)) k="@check";

    if("@main".equals(k))
    {
      t.put(k,v);
    }
    else
    {
      AglVecteur l=(AglVecteur)t.get(k);
      if(l==null) l=new AglVecteur();
      l.addElement(v);
      t.put(k,l);
    }
  }

  private void addCommentaire(Object _c)
  {
    commentaires_.addElement(_c);
  }

  private AglVecteur getCommentaires()
  {
    AglVecteur r=commentaires_;
    commentaires_=new AglVecteur();
    return r;
  }

  private boolean hook_number=false;
  private boolean hook_comment=false;

  public String get()
  {
    String r="";

    if(pile_.empty())
    {
      try
      {
        st_.nextToken();

        switch(st_.ttype)
        {
          case StreamTokenizer.TT_EOL   : r=CR;                break;
          case StreamTokenizer.TT_EOF   : r="";                 break;
          case StreamTokenizer.TT_WORD  : r=st_.sval;           break;
          case StreamTokenizer.TT_NUMBER: r=""+st_.nval;       break;
	  case '"':
	    if(st_.sval!=null) r="\""+ascii2C(st_.sval)+"\"";
	    else               r="\"";
	    break;
          case '\'':
	    if(st_.sval!=null) r="'" +ascii2C(st_.sval)+"'";
	    else               r="'";
	    break;
          default: r=""+(char)st_.ttype; break;
        }
      }
      catch(Exception ex)
      {
        ex.printStackTrace();
	erreur(""+ex+" [AglParse.get]");
      }

      trace(">>> "+r);

    // Commentaire uni-ligne
    if(  AglOptions.analyserCommentaires
       &&!hook_comment
	 //&&r.equals("//"))
       &&(r.equals("/"))&&(peek().equals("/")))
      {
	hook_comment=true;
	setCommentSyntax();

	int l=st_.lineno();
	get();
	r="";

	while(!peek().equals(CR)&&!peek().equals(""))
	{
	  String t=get();
	  if(  !"/".equals(t)&&!r.endsWith("/") 
	     &&!"*".equals(t)&&!r.endsWith("*")) r+=" ";
	  r+=t;
	}

	hook_comment=false;
	setSourceSyntax();

	r=r.trim();
	// System.err.println("CS: "+r);

	if(!"".equals(r))
	{
	  AglCommentaireSimple cs=new AglCommentaireSimple(r);
	  cs.ligne=l;
	  addCommentaire(cs);
	}

	while(peek().equals(CR)) get();
	r=get();
      }

    // Commentaire multi-lignes
    if(  AglOptions.analyserCommentaires
       &&!hook_comment
	 //&&r.equals("/*"))
       &&(r.equals("/"))&&(peek().equals("*")))
      {
	hook_comment=true;
	setCommentSyntax();

	int l=st_.lineno();
	get();
	r="";
	AglTableTriee clefs=new AglTableTriee();

	boolean special=false;
	while(peek().equals("*" )) { special=true; get(); }
	if(special&&peek().equals("/")) put("*");

	String k="@main";
	String v="";

	while(true)
	{
	  String t=get();

	  if(t.equals(""))
	    erreur("Unclosed comment");
	  if(t.equals("/")&&peek().equals("*"))
	    avertissement("Nested comment /*");
	  if(t.equals("*")&&peek().equals("/"))
	    break;

	  if(t.startsWith("@"))
	  {
	    if(!"".equals(v)) addCommentKey(clefs,k,v);
	    k=t;
	    if("@".equals(k)) k+=get();
	    v="";
	  }
	  else
	  {
	    if(  !"/".equals(t)&&!v.endsWith("/")
	       &&!"*".equals(t)&&!v.endsWith("*")) v+=" ";
	    v+=t;
	  }
	}

	get();
	if(!"".equals(v)) addCommentKey(clefs,k,v);
	hook_comment=false;
	setSourceSyntax();

	/*
	for(Enumeration e=clefs.keys(); e.hasMoreElements(); )
	{
	  k=(String)e.nextElement();
	  v=(String)clefs.get(k);
	  System.err.println("CM: "+k+"="+v+"=");
	}
	*/

	AglCommentaireMulti cm=new AglCommentaireMulti(clefs,special);
	cm.ligne=l;
	addCommentaire(cm);

	while(peek().equals(CR)) get();
	r=get();
      }

      if(!hook_comment)
      {
      // Nombres reels
      if((!r.equals(""))&&Character.isDigit(r.charAt(0)))
      {
	hook_number=true;
        if(peek().equals("."))
        {
          r+=get();
          if(Character.isDigit(peek().charAt(0)))
          {
            r+=get();
            if((peek().equals("E"))||(peek().equals("e")))
            {
	      r+=get();
              if((peek().equals("+"))||(peek().equals("-")))
                r+=get();
              r+=get();
            }
          }
        }
	hook_number=false;
      }

      if((r.equals(":" ))&&(peek().equals(":"))) r+=get();
      if((r.equals("}" ))&&(peek().equals(";"))) r+=get();

      if((r.equals("[" ))&&(peek().equals("]"))) r+=get();

      if((r.equals("=" ))&&(peek().equals("="))) r+=get();
      if((r.equals("!" ))&&(peek().equals("="))) r+=get();

      if((r.equals("+" ))&&(peek().equals("="))) r+=get();
      if((r.equals("-" ))&&(peek().equals("="))) r+=get();
      if((r.equals("*" ))&&(peek().equals("="))) r+=get();
      if((r.equals("/" ))&&(peek().equals("="))) r+=get();
      if((r.equals("%" ))&&(peek().equals("="))) r+=get();

      if((r.equals("+" ))&&(peek().equals("+"))) r+=get();
      if((r.equals("-" ))&&(peek().equals("-"))) r+=get();

      if((r.equals(">" ))&&(peek().equals(">"))) r+=get();
      if((r.equals("<" ))&&(peek().equals("<"))) r+=get();
      if((r.equals(">" ))&&(peek().equals("="))) r+=get();
      if((r.equals("<" ))&&(peek().equals("="))) r+=get();
      if((r.equals(">>"))&&(peek().equals(">"))) r+=get();
      if((r.equals("<<"))&&(peek().equals("<"))) r+=get();

      if((r.equals("&" ))&&(peek().equals("&"))) r+=get();
      if((r.equals("|" ))&&(peek().equals("|"))) r+=get();

      if((r.equals("&" ))&&(peek().equals("="))) r+=get();
      if((r.equals("|" ))&&(peek().equals("="))) r+=get();
      if((r.equals("^" ))&&(peek().equals("="))) r+=get();

      if((r.equals("long"))&&(peek().equals("long")))
	  r+=" "+get();

      if((r.equals("%"))&&(peek().equals("{"))) r+=get();
      if((r.equals("%"))&&(peek().equals("}"))) r+=get();

      if(r.equals("%{"))
      {
	while(!get().equals("%}")) ;
	r=get();
      }
      }
    }
    else r=(String)pile_.pop();

    return r;
  }

  public void put(String ih)
  {
    pile_.push(ih);
  }

  private int pp=0;
  public int ligne()
  {
    int r=st_.lineno();
    if(MaxLines!=0)
    {
      int p=100*r/MaxLines;
      if(p!=pp) progres(p);
      pp=p;
    }
    return r;
  }

  public String fichier()
  {
    String r=st_.origin();
    int i=r.lastIndexOf(File.separator);
    if(i>=0) r=r.substring(i+1);
    return r;
  }

  // Parse

  public void parseFichier() throws Exception
  {
    parseUnite();
  }

  public AglUnite parseUnite() throws Exception
  {
    peek(); // recupere les commentaires

    AglUnite au=new AglUnite(st_.origin());
    au.langage="Idl";
    au.commentaires=getCommentaires();

    _unite=au;
    String t="";

    AglVisibilite  visibilite;
    AglVecteur commentaires;

    do
    {
      visibilite=new AglVisibilite(); // parseVisibilite();
      visibilite.vpublic=true;
      commentaires=getCommentaires();

      t=peek();
      trace("WU>"+t);

      if(t.equals("#"))
      {
	st_.eolIsSignificant(true);
	while(!t.equals("\n")) t=get();
	st_.eolIsSignificant(false);
      }
      else
      if(t.equals("module"))
      {
        AglPaquet ap=parsePaquet();
	ap.commentaires=commentaires;
        _paquet=ap;
	t=get();
	if(!t.equals("{")) syntaxe("{ needed and not "+t);
      }
      else
      if(t.equals("};"))
      {
	if(_paquet==null) syntaxe("} but not in a module");
	t=get();
	AglNom np=_paquet.nom.parent();
	if(np==null) _paquet=null;
	else         _paquet=AglPaquet.cherche(np);
      }
      else
      if(t.equals("interface"))
      {
        AglClasse ac=parseClasse();
	ac.commentaires=commentaires;
        ac.visibilite=visibilite;
        au.types.put(""+ac,ac);
      }
      else
      if(t.equals("const"))
      {
        parseConst();
      }
      else
      if(t.equals("enum"))
      {
        AglClasse ac=parseEnum();
	ac.commentaires=commentaires;
        ac.visibilite=visibilite;
        au.types.put(""+ac,ac);
      }
      else
      if(t.equals("exception"))
      {
        AglClasse ac=parseException();
	ac.commentaires=commentaires;
        ac.visibilite=visibilite;
        au.types.put(""+ac,ac);
      }
      else
      if(t.equals("struct"))
      {
        AglStruct ac=parseStruct();
	ac.commentaires=commentaires;
        ac.visibilite=visibilite;
        au.types.put(""+ac,ac);
      }
      else
      if(t.equals("union"))
      {
        parseUnion();
      }
      else
      if(t.equals("typedef"))
      {
        AglTypeAlias at=parseTypeDef();
	at.commentaires=commentaires;
        au.types.put(""+at,at);
      }
      else if(!t.equals("")) syntaxe("unknown keyword: "+t);

    } while(!t.equals(""));

    return au;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    AglVisibilite av=new AglVisibilite();
    String        t ="";
    boolean       e =false;

    av.vpublic=true;
    do
    {
      e=false;
      t=get();
      trace("WV>"+t);
           if(t.equals("const"))    e=av.vfinal=av.vstatic=true;
      else if(t.equals("readonly")) e=av.vreadonly=true;
      else if(t.equals("oneway"  )) e=av.voneway=true;
    } while(e);

    put(t);
    return av;
  }

  public AglNom parseNom() throws Exception
  {
    AglNom an=new AglNom();

    String t="";

    t=peek();
    if(t.equals("::")) get();

    do
    {
      trace("WN>"+peek());
      an.identificateurs.addElement(get());
      t=peek();
      if(t.equals("::")) get();
    }
    while(t.equals("::"));

    an.ligne=st_.lineno();
    return an;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    AglListeNoms al=new AglListeNoms();

    String t="";
    do
    {
      trace("WLN>"+peek());
      al.noms.addElement(parseNom());
      t=peek();
      if(t.equals(",")) get();
    }
    while(t.equals(","));

    al.ligne=st_.lineno();
    return al;
  }

  public AglType parseType() throws Exception
  {
    boolean tableau=false;
    String t="";

    if(peek().equals("sequence"))
    {
      tableau=true;
      get();
      t=get();
      if(!t.equals("<")) syntaxe("< needed and not "+t);
    }

    if(peek().equals("unsigned")) get();

    trace("WT>"+peek());
    AglNom base=parseNom();

    AglType r=AglType.cherche(base);
    if(r==null) r=new AglClasse(base);

    if(tableau)
    {
      r=new AglTypeTableau(r);
      t=get();
      if(!t.equals(">")) syntaxe("> needed and not "+t);
    }

    // while(peek().equals("[]")) { trace("WT>"+peek()); r=new AglTypeTableau(r); get(); }
    return r;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("module")) syntaxe("module needed and not "+t);

    AglNom np=parseNom();
    trace("Module      : "+np);

    if(_paquet!=null) np=new AglNom(_paquet.nom,np);
    // t=get();
    // if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglPaquet ap=AglPaquet.cherche(np);
    if(ap==null)
    {
      ap=new AglPaquet(np);
      ap.ligne=st_.lineno();
    }
    return ap;
  }
  
  public AglImport parseImport() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("include")) syntaxe("include needed and not "+t);

    AglNom ni=parseNom();
    trace("#include    : "+ni);

    // t=get();
    // if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglImport ai=new AglImport(ni);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglTypeAlias parseTypeDef() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("typedef")) syntaxe("typedef needed and not "+t);

    AglType at=parseType();
    AglNom  an=parseNom();

    while(peek().equals("["))
    {
      get();
      AglExpression dim=parseExpression();
      t=get();
      if(!t.equals("]")) syntaxe("] needed and not "+t);
      at=new AglTypeTableau(at,dim);
    }

    AglTypeAlias al=new AglTypeAlias(an.dernier(),at); // "Idl"

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);
    return al;
  }

  public void parseConst() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("const")) syntaxe("const needed and not "+t);

    while(!t.equals(";")) t=get();
  }

  public AglClasse parseEnum() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("enum")) syntaxe("enum needed and not "+t);

    trace("WENUM>"+t);
    AglNom nc=parseNom();
         if(_classe!=null) nc=new AglNom(_classe.nom,nc);
    else if(_paquet!=null) nc=new AglNom(_paquet.nom,nc);
    trace("Enum        : "+nc);

    AglClasse ac=(AglClasse)AglType.cherche(nc);
    if(ac==null) ac=new AglClasse(nc);
    ac.ligne=st_.lineno();
    // ac.elementaire=false;

    if(peek().equals(";"))
    {
      get();
      return ac;
    }
    
    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    // Body

    do
    {
      t=peek();
      trace("WBENUM>"+t);

      if(!t.equals("};"))
      {
	// _type=AglTypePrimitif.tinteger;
	_nom =parseNom();

	// AglAttribut aa=new AglAttribut(_nom.dernier(),_type);
	// aa.ligne=st_.lineno();
	// aa.visibilite=new AglVisibilite();
	// aa.visibilite.vpublic=true;
	// aa.visibilite.vstatic=true;
	// aa.visibilite.vfinal=true;
	// ac.attributs.addElement(aa);

	if(peek().equals(",")) get();
      }
    } while(!peek().equals("};"));

    AglAttribut aa=new AglAttribut("value",AglTypePrimitif.tinteger);
    aa.ligne=st_.lineno();
    aa.visibilite=new AglVisibilite();
    aa.visibilite.vpublic=true;
    ac.attributs.put(aa.signature(),aa);

    get();
    return ac;
  }

  public AglClasse parseException() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("exception")) syntaxe("exception needed and not "+t);

    trace("WEXC>"+t);
    AglNom nc=parseNom();
         if(_classe!=null) nc=new AglNom(_classe.nom,nc);
    else if(_paquet!=null) nc=new AglNom(_paquet.nom,nc);
    trace("Exception   : "+nc);

    AglClasse ac=(AglClasse)AglType.cherche(nc);
    if(ac==null) ac=new AglClasse(nc);
    ac.ligne=st_.lineno();
    // ac.elementaire=false;

    if(peek().equals(";"))
    {
      get();
      return ac;
    }
    
    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    // Body

    do
    {
      t=peek();
      trace("WBEXC>"+t);

      if(!t.equals("};"))
      {
	_type=parseType();
	_nom =parseNom();

	// trace(" --- "+_nom.dernier()+" "+_type.nom.dernier());
	AglAttribut aa=new AglAttribut(_nom.dernier(),_type);
	aa.ligne=st_.lineno();
	aa.visibilite=new AglVisibilite();
	aa.visibilite.vpublic=true;
	// aa.visibilite.vstatic=true;
	// aa.visibilite.vfinal=true;
	ac.attributs.put(aa.signature(),aa);

	if(peek().equals(";")) get();
      }
    } while(!peek().equals("};"));

    get();
    return ac;
  }

  public void parseUnion() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("union")) syntaxe("union needed and not "+t);

    while(!t.equals("};")) t=get();
  }

  public AglStruct parseStruct() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("struct")) syntaxe("struct needed and not "+t);

    trace("WS>"+t);
    AglNom nc=parseNom();
    if(_paquet!=null) nc=new AglNom(_paquet.nom,nc);
    trace("Struct      : "+nc);

    AglStruct ac=(AglStruct)AglType.cherche(nc);
    if(ac==null)
    {
      ac=new AglStruct(nc);
      ac.visibilite.vfinal=true;
      ac.elementaire=true;
    }
    ac.ligne=st_.lineno();

    if(peek().equals(";"))
      {
	get();
	return ac;
      }
    
    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    // Body

    ac.elementaire=false;
    ac.attributs.clear();

    do
    {
      t=peek();
      trace("WBS>"+t);

      if(!t.equals("};"))
      {
	_type=parseType();
	_nom =parseNom();

	AglVecteur ava=parseAttribut();
	for(Enumeration e=ava.elements(); e.hasMoreElements(); )
	{
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  aa.visibilite=new AglVisibilite();
	  aa.visibilite.vpublic=true;
	  ac.attributs.put(aa.signature(),aa);
        }
      }
    } while(!peek().equals("};"));

    get();
    return ac;
  }

  public AglClasse parseClasse() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("interface")) syntaxe("interface needed and not "+t);

    trace("WC>"+t);
    AglNom nc=parseNom();
    if(_paquet!=null) nc=new AglNom(_paquet.nom,nc);
    trace("Classe      : "+nc);

    AglClasse ac=(AglClasse)AglType.cherche(nc);
    if(ac==null) ac=new AglClasse(nc);

    _classe=ac;

    ac.normale=false;
    ac.proprietaire=_unite;
    ac.ligne=st_.lineno();
    ac.elementaire=true;

    if(peek().equals(";"))
      {
	get();
	return ac;
      }

    ac.elementaire=false;
    
    do
    {
      t=get();
      trace("WCB>"+t);
           if(t.equals(":"))    ac.meres     =parseMothers();
	   // else if(t.equals("implements")) ac.interfaces=parseInterfaces();
      else if(t.equals("raises"))     ac.exceptions=parseExceptions();
    } while(!t.equals("{"));

  // Body

    ac.attributs.clear();
    ac.methodes .clear();

    do
    {
      if(peek().equals("};")) break;

      if(peek().equals("const"))
      {
        while(!peek().equals(";")) get();
	get();
	continue;
      }

      if(peek().equals("const"    )) { parseConst();     continue; }
      if(peek().equals("typedef"  )) { parseTypeDef();   continue; }
      if(peek().equals("enum"     )) { parseEnum();      continue; }
      if(peek().equals("exception")) { parseException(); continue; }
      if(peek().equals("union"    )) { parseUnion();     continue; }

      AglVecteur    commentaires=getCommentaires();
      AglVisibilite visibilite  =parseVisibilite();

      trace("WBC>"+peek());

      t=get();

      if(t.equals(ac.nom.dernier())&&peek().equals("(")&&!visibilite.vstatic)
      {
        put(t);

        AglConstructeur am=parseConstructeur();
	am.commentaires=commentaires;
        am.visibilite=visibilite;
        ac.constructeurs.put(am.signature(),am);
      }
      else if(t.equals("attribute"))
      {
        _type=parseType();
        _nom =parseNom();

	AglVecteur ava=parseAttribut();
	for(Enumeration e=ava.elements(); e.hasMoreElements(); )
        {
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  aa.commentaires=commentaires;
	  aa.visibilite=visibilite;
	  ac.attributs.put(aa.signature(),aa);
        }
      }
      else if(!t.equals("};"))
      {
        put(t);

        _type=parseType();
        _nom =parseNom();
        String v=peek();

        if(!v.equals("(")) syntaxe("( needed and not "+t);

	AglMethode am=parseMethode();
	am.commentaires=commentaires;
	am.visibilite=visibilite;
	am.visibilite.vvirtual=true;
	ac.methodes.put(am.signature(),am);
      }

    } while(!peek().equals("};"));

    get();
    _classe=null;
    return ac;
  }

  public AglListeNoms parseMothers() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    trace("Mere        : "+ln);
    return ln;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    trace("Interfaces  : "+ln);
    return ln;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);
    
    AglListeNoms ln=parseListeNoms();
    trace("Exceptions  : "+ln);

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);
    
    return ln;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    erreur("Il n'y a pas de constructeurs en Idl ("+get()+")");
    return null;
  }

  public AglMethode parseMethode() throws Exception
  {
    trace("Methode     : "+_nom+" ["+_type+"]");

    AglMethode m=new AglMethode(_nom.dernier(),_type);
    m.ligne=st_.lineno();

    m.parametres=parseParametres();
    if(peek().equals("raises")) { get(); m.exceptions=parseExceptions(); }

    String t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    // m.corps=parseBlocMultiple();

    return m;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    AglVecteur r=new AglVecteur(0,1);
    String t="";

    // t=get();
    // if(!t.equals("attribute")) syntaxe("attribute needed and not "+t);

    do
    {
      trace("Attribut    : "+_nom+" ["+_type+"]");

      AglAttribut a=new AglAttribut(_nom.dernier(),_type);
      a.ligne=st_.lineno();
      r.addElement(a);

      t=get();
      if(t.equals("="))
      {
        a.initiale=parseExpression();
        trace("Initiale    : "+a.initiale);
      }
      else
      if(t.equals(","))
        _nom=parseNom();
      else
      if(!t.equals(";"))
        syntaxe(", or ; needed and not "+t);

    } while(!t.equals(";"));

    return r;
  }

  public AglVecteur parseParametres() throws Exception
  {
    int    np=0;
    String t ="";
    AglVecteur r =new AglVecteur(0,1);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    do 
    {
      if(!peek().equals(")"))
      {
	String io=get();
	int passage=0;
             if(io.equals("in"   )) passage=AglParametre.IN;
        else if(io.equals("out"  )) passage=AglParametre.OUT;
        else if(io.equals("inout")) passage=AglParametre.INOUT;
	else syntaxe("in/out needed and not "+io);

        AglType type=parseType();
        String  nom =get();
        np++;

        // ancienne forme
        while(peek().equals("[]")) { get(); type=new AglTypeTableau(type); }

        trace("Parametre "+np+" : "+nom+" ["+type+"]");

        AglParametre p=new AglParametre(nom,type);
	p.passage=passage;
        r.addElement(p);

             if(peek().equals(",")) get();
        else if(!peek().equals(")")) syntaxe(", or ; needed and not "+t);
      }

    } while(!peek().equals(")"));

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    return r;
  }

  public AglVecteur parseArgs() throws Exception
  {
    int    na=0;
    String t ="";
    AglVecteur r =new AglVecteur(0,1);

    t=get();
    if(!(t.equals("("))) syntaxe("( needed and not "+t);

    do 
    {
      t=peek();

      if(!t.equals(")"))
      {
        AglExpression ae=parseExpression();
        na++;
        trace("Argument  "+na+" : "+ae);
        r.addElement(ae);
      }

      t=get();
      if(!(t.equals(",")||t.equals(")"))) syntaxe(", or ) needed and not "+t);

    } while(!t.equals(")"));

    return r;
  }

  public AglBloc parseBloc() throws Exception
  {
    return peek().equals("{") ? parseBlocMultiple() : parseBlocSimple();
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t="";

    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    while(!peek().equals("};"))
    {
      AglVisibilite visibilite=parseVisibilite();

      trace("WBM>>> "+peek());

      t=get();
      if((AglType.cherche(t)!=null)&&(!peek().equals(".")))
      {
        put(t);
        for(Enumeration e=parseInstructionDefVar().elements(); e.hasMoreElements(); )
        {
          AglInstructionDefVar ai=(AglInstructionDefVar) e.nextElement();
          ai.visibilite=visibilite;
          ab.instructions.addElement(ai);
        }
      }
      else
      {
        put(t);
        AglInstruction ai=null;
        
	if(t.equals("{")) ai=parseInstructionBloc ();
        else
        {
          AglExpression gauche=parseExpression();

          t=peek();
               if(t.equals("="))      ai=parseInstructionAffectation(gauche);
       // else if(t.equals("("))      ai=parseInstructionProcedure  (gauche);
          else if(t.equals(";"))      ai=parseInstructionExpression (gauche);
          else if(  t.equals("+=")||t.equals("-=")
		  ||t.equals("*=")||t.equals("/=")||t.equals("%=")
                  ||t.equals("&=")||t.equals("|=")||t.equals("^="))
          {
            t=get();
            AglExpression ae=new AglExpressionBinaire(t.substring(0,1),gauche,parseExpression());
            ae.ligne=st_.lineno();

            ai=new AglInstructionAffectation(gauche,ae);
            ai.ligne=st_.lineno();

            t=get();
            if(!t.equals(";")) syntaxe("; needed and not "+t);
          }
          else ai=parseInstructionGenerique  ();
        }

        ab.instructions.addElement(ai);
      }
    }

    t=get();
    if(!t.equals("};")) syntaxe("} needed and not "+t);

    return ab;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t=get();

    trace("WBS>>> "+t+" | "+peek());

        String u=peek();
        put(t);
        AglInstruction ai=null;
        
	if(t.equals("{")) syntaxe("{ is not wanted there");
        else
        {
          AglExpression gauche=parseExpression();

          t=peek();
               if(t.equals("="))      ai=parseInstructionAffectation(gauche);
       // else if(t.equals("("))      ai=parseInstructionProcedure  (gauche);
          else if(t.equals(";"))      ai=parseInstructionExpression (gauche);
          else if(  t.equals("+=")||t.equals("-=")
		  ||t.equals("*=")||t.equals("/=")||t.equals("%=")
                  ||t.equals("&=")||t.equals("|=")||t.equals("^="))
          {
            t=get();
            AglExpression ae=new AglExpressionBinaire(t.substring(0,1),gauche,parseExpression());
            ae.ligne=st_.lineno();

            ai=new AglInstructionAffectation(gauche,ae);
            ai.ligne=st_.lineno();

            t=get();
            if(!t.equals(";")) syntaxe("; needed and not "+t);
          }
          else ai=parseInstructionGenerique  ();
        }

        ab.instructions.addElement(ai);

    return ab;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    erreur("Il n'y a pas de bloc-virgule en Idl ("+get()+")");
    return null;
  }

// Expressions

  public AglExpression parseExpression() throws Exception
  {
    AglExpression ae;

    trace("WE>>> "+peek());

         if(peek().equals("new"))    ae=parseExpressionNew();
    else if(peek().equals("+" ))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("-" ))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("++"))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("--"))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("!" ))     ae=parseExpressionUnaireDroite();
    else if(peek().equals("("))
    {
      String  t,u;
      t=get();
      u=peek();
      put(t);
      trace("WE>>> "+t+" | "+u);

      AglType c=AglType.cherche(u);
      if(c==null) ae=parseExpressionParenthesee();
      else        ae=parseExpressionTypage();
    }
    else
    {
      String  t,u;
      t=get();
      u=peek();
      put(t);

      trace("WE>>> "+t+" | "+u);

           if(u.equals("("))      ae=new AglExpressionMethode(null,parseExpressionFonction());
      else if(u.equals("."))      ae=new AglExpressionAttribut(null,get());
      else                        ae=new AglExpressionGenerique(get());
      ae.ligne=st_.lineno();
    }

boolean encore=true;
while(encore)
{
    encore=false;
    String u;

    trace("Expression  : "+ae+" ["+ae.getClass()+"]");
    u=peek();
    trace("WEA>>> "+u);

    if(u.equals(";")||u.equals(","))
      encore=false;
    else
    if(  (u.equals("+"  ))||(u.equals("-"))
       ||(u.equals("*"  ))||(u.equals("/"))
       ||(u.equals("%"  ))
       ||(u.equals("==" ))||(u.equals("!="))
       ||(u.equals("<"  ))||(u.equals(">"))
       ||(u.equals("<=" ))||(u.equals(">="))
       ||(u.equals("<<" ))||(u.equals(">>"))
       ||(u.equals("<<<"))||(u.equals(">>>"))
       ||(u.equals("&&"))||(u.equals("||"))
       ||(u.equals("&"))||(u.equals("|"))||(u.equals("^"))
       ||(u.equals("instanceof")))
    {
      ae=new AglExpressionBinaire(get(),ae,parseExpression());
      ae.ligne=st_.lineno();
      encore=true;
    }
    else
    if(u.equals("."))
    {
      get();
      String v=get();
      String w=peek();
      put(v);

      if(w.equals("(")) ae=new AglExpressionMethode(ae,parseExpressionFonction());
      else ae=new AglExpressionAttribut(ae,get());
      ae.ligne=st_.lineno();
      encore=true;
    }
}

    return ae;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    String t="";
    int    np=0;
    int    nt=0;

    String body="";

    do
    {
      t=get();
      // trace("WEG>>> "+t);

      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      if(t.equals("]")) nt--;
      if(t.equals("[")) nt++;
      
           if((np<0)&&(t.equals(")"))) ;
      else if((nt<0)&&(t.equals("]"))) ;
      else if((np==0)&&(nt==0)&&(t.equals(";")||t.equals(","))) ;
      else
      {
        if(  (body.length()>0)
           &&(  Character.isJavaIdentifierPart(body.charAt(body.length()-1))
	      ||(body.charAt(body.length()-1)==')'))
	   &&(  Character.isJavaIdentifierStart(t.charAt(0))
	      ||(t.charAt(0)=='\"'))) body+=" ";
	body+=t;
      }
    } while(  (np>0)
            ||(nt>0)
	    ||((np==0)&&(nt==0)&&(!(t.equals(";")||t.equals(",")))));

    if(t.equals(";")||t.equals(",")) put(t);
    if((np<0)&&t.equals(")")) put(t);
    if((nt<0)&&t.equals("]")) put(t);

    AglExpressionGenerique ae=new AglExpressionGenerique(body);
    ae.ligne=st_.lineno();
    return ae;
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite() throws Exception
  {
    String        op=get();

    trace("Unaire     : "+op);
    AglExpression ed=parseExpression();

    AglExpressionUnaireDroite ae=new AglExpressionUnaireDroite(op,ed);
    ae.ligne=st_.lineno();
    return ae;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    AglExpression eg=new AglExpressionGenerique(get());
    String        op=get();

    trace("Binaire    : "+op);
    AglExpression ed=parseExpression();

    AglExpressionBinaire ae=new AglExpressionBinaire(op,eg,ed);
    ae.ligne=st_.lineno();
    return ae;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    String t="";

    String nom=get();
    AglVecteur le =parseArgs();
    trace("Fonction    : "+nom+" ("+le.size()+" arguments)");

    AglExpressionFonction ae=new AglExpressionFonction(nom);
    ae.ligne=st_.lineno();
    for(Enumeration e=le.elements(); e.hasMoreElements(); )
      ae.arguments.addElement(e.nextElement());
    return ae;
  }

  public AglExpression parseExpressionNew() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("new")) syntaxe("new needed and not "+t);

    t=get();
    AglExpression ae;

    if(peek().equals("("))
    {
      put(t);
      AglType c =parseType();
      AglVecteur  le=parseArgs();
      trace("New         : "+t+" ("+le.size()+" arguments)");

      AglExpressionNew aen=new AglExpressionNew(c);
      aen.ligne=st_.lineno();
      for(Enumeration e=le.elements(); e.hasMoreElements(); )
        aen.arguments.addElement(e.nextElement());
      ae=aen;
    }
    else
    {
      put(t);
      ae=parseExpression();
    }

    return ae;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpressionParenthesee ae=new AglExpressionParenthesee(parseExpression());

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    ae.ligne=st_.lineno();
    return ae;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglType type=parseType();
    trace("Typage    : "+type);

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglExpressionTypage ae=new AglExpressionTypage(type,parseExpression());
    ae.ligne=st_.lineno();
    return ae;
  }

// Instructions

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    String  ident="                                         ";
    String  t="";
    int     na=0;
    int     np=0;
    boolean pv=true;

    String body="";

    do
    {
      t=get();
      // trace("WI>>> "+t);

      if(t.equals("};")) na--;
      if(t.equals("{")) na++;
      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      // if(na>0)
      {
             if(t.equals("{")) { body+="\n" +ident.substring(0,2*na)+"{\n"; pv=true; }
        else if(t.equals("};")) { body+=ident.substring(0,2*na+2)+"}\n"; pv=true; }
        else if(t.equals(";")) { if(np==0) { body+=";\n"; pv=true; } else body+="; "; }
        else if(pv) { body+=ident.substring(0,2*na+2)+t; pv=false; }
        else
	{
          if(  (body.length()>0)
	     &&(  Character.isJavaIdentifierPart(body.charAt(body.length()-1))
		||(body.charAt(body.length()-1)==')'))
	     &&(  Character.isJavaIdentifierStart(t.charAt(0))
		||(t.charAt(0)=='\"'))) body+=" ";
	  body+=t;
          pv=false;
        }
      }
    } while((na>0)||(!t.equals(";")));

    trace("Instruction : "+body);
    AglInstructionGenerique ai=new AglInstructionGenerique(body);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionAffectation parseInstructionAffectation(AglExpression eg) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("=")) syntaxe("= needed and not "+t);

    AglExpression ed=parseExpression();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    trace("Affectation: "+eg+" <-- "+ed);

    AglInstructionAffectation ai=new AglInstructionAffectation(eg,ed);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    AglBloc ab=parseBlocMultiple();
    AglInstructionBloc ai=new AglInstructionBloc(ab);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    erreur("Il n'y a pas d'instruction catch en Idl ("+get()+")");
    return null;
  }

  public AglVecteur parseInstructionDefVar() throws Exception
  {
    String     t ="";
    int        nv=0;
    AglVecteur     r =new AglVecteur(0,1);

    AglType type=parseType();

    do 
    {
      t=get();

      if(!t.equals(";"))
      {
        // ancienne forme
        while(peek().equals("[]")) { get(); type=new AglTypeTableau(type); }

        nv++;
        trace("Variable  "+nv+" : "+t+" ["+type+"]");
        AglInstructionDefVar ai=new AglInstructionDefVar(t,type);
        ai.ligne=st_.lineno();

        t=get();
        if(t.equals("="))
        {
          ai.initiale=parseExpression();
          trace("Initiale   : "+ai.initiale);
          t=get();
        }
        r.addElement(ai);

        if(!(t.equals(",")||t.equals(";")))
	  syntaxe(", or ; needed and not "+t);
      }

    } while(!t.equals(";"));

    return r;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction do/while en Idl ("+get()+")");
    return null;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction expressive en Idl ("+get()+")");
    return null;
  }

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    erreur("Il n'y a pas d'instruction if en Idl ("+get()+")");
    return null;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    erreur("Il n'y a pas de procedure en Java ("+get()+")");
    return null;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    erreur("Il n'y a pas d'instruction return en Idl ("+get()+")");
    return null;
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    erreur("Il n'y a pas d'instruction switch en Idl ("+get()+")");
    return null;
  }  

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    erreur("Il n'y a pas d'instruction throw en Idl ("+get()+")");
    return null;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    erreur("Il n'y a pas d'instruction try en Idl ("+get()+")");
    return null;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction while en Idl ("+get()+")");
    return null;
  }
}

