/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseJava.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import java.net.*;

public class AglParseJava extends AglParse
{
  protected boolean VERIFYCLASS=
    AglPreferences.AGL.getBooleanProperty("java.parse.verifyclass",false);

  protected boolean SKIPINSTRUCTIONS=
    AglPreferences.AGL.getBooleanProperty("java.parse.skipinstructions",false);

  private final static String CR="<CR>";

  private final static AglCode CODE=new AglCodeJava();

  private FuStreamTokenizer st_;
  private Stack             pile_;
  private AglPileChamps     champs_;
  private AglVecteur        inconnus_;

  private AglUnite          unite_;
  private AglClasse         classe_;
  private AglNom            nom_;
  private AglType           type_;
  private int               anonymes_;
  private AglVecteur        commentaires_;

  //  AglPaquet     paquet;
  //  AglClasse     classe;
  //  AglMethode    methode;
  //  AglAttribut   attribut;

  public AglParseJava()
  {
    if(AglType.cherche("Object")==null)
      creeClasse("java.lang.Object");
  }

  public String extension()
  {
    return "java";
  }

  public String[] extensions()
  {
    return new String[] { "java", "jav", "JAV" };
  }

  public boolean optionAnalyserCommentaires()
    { return true; }

  private void setSourceSyntax()
  {
    st_.resetSyntax();
    st_.eolIsSignificant(false);
    st_.quoteChar('"');
    st_.quoteChar('\'');
    st_.slashSlashComments(!AglOptions.analyserCommentaires);
    st_.slashStarComments (!AglOptions.analyserCommentaires);
    st_.wordChars('a','z');
    st_.wordChars('A','Z');
    st_.wordChars('0','9');
    st_.wordChars('_','_');
    st_.wordChars(129,255);
    st_.whitespaceChars(' ',' ');
    st_.whitespaceChars('\t','\t');
    st_.whitespaceChars('\n','\n');
    st_.whitespaceChars('\r','\r');
    st_.whitespaceChars((char)12,(char)12);
  }

  private void setCommentSyntax()
  {
    st_.resetSyntax();
    st_.eolIsSignificant(true);
    st_.wordChars(0,255);
    st_.whitespaceChars(0  ,32);
    st_.whitespaceChars(127,160);
    st_.ordinaryChar('*');
    st_.ordinaryChar('/');
  }

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    st_          =new FuStreamTokenizer(_input,_fichier);
    pile_        =new Stack();
    anonymes_    =0;
    commentaires_=new AglVecteur();

    if(AglOptions.definirLiteraux)
    {
      champs_  =new AglPileChamps();
      inconnus_=new AglVecteur();
    }

    setSourceSyntax();
  }

  private static void addCommentKey(AglTableTriee t, String k, String v)
  {
    v=v.trim();
    v=FuLib.replace(v,CR+"*","");
    v=FuLib.replace(v,CR    ,"");
    v=FuLib.replace(v,"  "," ");
    //while(v.endsWith(" "+CR)) v=v.substring(0,v.length()-CR.length()-1);

    if(k.startsWith("@alma_")||k.startsWith("@alma-"))
	k="@"+k.substring(6);

    if("@pre"          .equals(k)) k="@require";
    if("@precondition" .equals(k)) k="@require";
    if("@post"         .equals(k)) k="@ensure";
    if("@postcondition".equals(k)) k="@ensure";
    if("@inv"          .equals(k)) k="@invariant";
    if("@cond"         .equals(k)) k="@check";
    if("@condition"    .equals(k)) k="@check";

    if("@main".equals(k))
    {
      t.put(k,v);
    }
    else
    {
      AglVecteur l=(AglVecteur)t.get(k);
      if(l==null) l=new AglVecteur();
      l.addElement(v);
      t.put(k,l);
    }
  }

  private void addCommentaire(AglCommentaire _c)
  {
    commentaires_.addElement(_c);
  }

  private AglVecteur getCommentaires()
  {
    AglVecteur r=commentaires_;
    commentaires_=new AglVecteur();
    if(r.size()==0) r=null;
    return r;
  }

  private AglVecteur getCommentairesNormaux()
  {
    AglVecteur r=new AglVecteur();
    while(commentaires_.size()>0)
    {
      Object c=commentaires_.elementAt(0);
      if(  (c instanceof AglCommentaireSimple)
	 ||(   (c instanceof AglCommentaireMulti)
	    &&!((AglCommentaireMulti)c).special))
      {
	r.addElement(c);
	commentaires_.removeElementAt(0);
      }
      else break;
    }
    if(r.size()==0) r=null;
    return r;
  }

  private boolean hook_number=false;
  private boolean hook_comment=false;

  public String get()
  {
    String r="";

    if(pile_.empty())
    {
      try
      {
        st_.nextToken();

        switch(st_.ttype)
        {
          case StreamTokenizer.TT_EOL   : r=CR;                break;
          case StreamTokenizer.TT_EOF   : r="";                break;
          case StreamTokenizer.TT_WORD  : r=st_.sval;          break;
          case StreamTokenizer.TT_NUMBER: r=""+st_.nval;       break;
	  case '"':
	    if(st_.sval!=null) r="\""+ascii2C(st_.sval)+"\"";
	    else               r="\"";
	    break;
          case '\'':
	    if(st_.sval!=null) r="'" +ascii2C(st_.sval)+"'";
	    else               r="'";
	    break;
          default: r=""+(char)st_.ttype; break;
        }
      }
      catch(Exception ex)
      {
        ex.printStackTrace();
	erreur(""+ex+" [AglParse.get]");
      }

      trace(">>> "+r);

    // Commentaire uni-ligne
    if(  AglOptions.analyserCommentaires
       &&!hook_comment
	 //&&r.equals("//"))
       &&(r.equals("/"))&&(peek().equals("/")))
      {
	hook_comment=true;
	setCommentSyntax();

	int l=st_.lineno();
	get();
	r="";

	while(!peek().equals(CR)&&!peek().equals(""))
	{
	  String t=get();
	  if(  !"/".equals(t)&&!r.endsWith("/") 
	     &&!"*".equals(t)&&!r.endsWith("*")) r+=" ";
	  r+=t;
	}

	hook_comment=false;
	setSourceSyntax();

	r=r.trim();
	// System.err.println("CS: "+r);

	if(!"".equals(r))
	{
	  AglCommentaireSimple cs=new AglCommentaireSimple(r);
	  cs.ligne=l;
	  addCommentaire(cs);
	}

	while(peek().equals(CR)) get();
	r=get();
      }

    // Commentaire multi-lignes
    if(  AglOptions.analyserCommentaires
       &&!hook_comment
	 //&&r.equals("/*"))
       &&(r.equals("/"))&&(peek().equals("*")))
      {
	hook_comment=true;
	setCommentSyntax();

	int l=st_.lineno();
	get();
	r="";
	AglTableTriee clefs=new AglTableTriee();

	boolean special=false;
	while(peek().equals("*" )) { special=true; get(); }
	if(special&&peek().equals("/")) put("*");

	String k="@main";
	String v="";

	while(true)
	{
	  String t=get();

	  if(t.equals(""))
	    erreur("Unclosed comment");
	  if(t.equals("/")&&peek().equals("*"))
	    avertissement("Nested comment /*");
	  if(t.equals("*")&&peek().equals("/"))
	    break;

	  if(t.startsWith("@"))
	  {
	    if(!"".equals(v)) addCommentKey(clefs,k,v);
	    k=t;
	    if("@".equals(k)) k+=get();
	    v="";
	  }
	  else
	  {
	    if(  !"/".equals(t)&&!v.endsWith("/")
	       &&!"*".equals(t)&&!v.endsWith("*")) v+=" ";
	    v+=t;
	  }
	}

	get();
	if(!"".equals(v)) addCommentKey(clefs,k,v);
	hook_comment=false;
	setSourceSyntax();

	/*
	for(Enumeration e=clefs.keys(); e.hasMoreElements(); )
	{
	  k=(String)e.nextElement();
	  v=(String)clefs.get(k);
	  System.err.println("CM: "+k+"="+v+"=");
	}
	*/

	AglCommentaireMulti cm=new AglCommentaireMulti(clefs,special);
	cm.ligne=l;
	addCommentaire(cm);

	while(peek().equals(CR)) get();
	r=get();
      }

      if(!hook_comment)
      {
      // Nombres reels
      if((!r.equals(""))&&Character.isDigit(r.charAt(0)))
      {
	  hook_number=true;

          if(peek().equals("."))
          {
            r+=get();
	    if(  Character.isDigit(peek().charAt(0))
	       ||peek().equals("d")
	       ||peek().equals("f"))
	      r+=get();
	  }
	  if(   (r.endsWith("E")||r.endsWith("e"))
	     &&!(r.startsWith("0x")||r.startsWith("0X")))
	  {
	    if((peek().equals("+"))||(peek().equals("-")))
	      r+=get();
	    r+=get();
	  }

	  r=r.toLowerCase();
	  hook_number=false;
      }
      else
      // .####
      if(  (r.equals("."))
	 &&(!hook_number)
	 &&(!peek().equals(""))
	 &&(Character.isDigit(peek().charAt(0))))
      {
	// System.err.println("~~~ R="+r);
	// System.err.println("~~~ P="+peek());
	r="0."+get();
      }
      else
      {
	if((r.equals("["))&&(peek().equals("]" ))) r+=get();

	if((r.equals("="))&&(peek().equals("=" ))) r+=get();
	if((r.equals("!"))&&(peek().equals("=" ))) r+=get();

	if((r.equals("+"))&&(peek().equals("=" ))) r+=get();
	if((r.equals("-"))&&(peek().equals("=" ))) r+=get();
	if((r.equals("*"))&&(peek().equals("=" ))) r+=get();
	if((r.equals("/"))&&(peek().equals("=" ))) r+=get();
	if((r.equals("%"))&&(peek().equals("=" ))) r+=get();
	
	if((r.equals("+"))&&(peek().equals("+" ))) r+=get();
	if((r.equals("-"))&&(peek().equals("-" ))) r+=get();
	
	if((r.equals(">"))&&(peek().equals(">"  ))) r+=get();
	if((r.equals("<"))&&(peek().equals("<"  ))) r+=get();
	if((r.equals(">"))&&(peek().equals("="  ))) r+=get();
	if((r.equals("<"))&&(peek().equals("="  ))) r+=get();
	if((r.equals(">"))&&(peek().equals(">>" ))) r+=get();
	if((r.equals("<"))&&(peek().equals("<<" ))) r+=get();
	if((r.equals(">"))&&(peek().equals(">=" ))) r+=get();
	if((r.equals("<"))&&(peek().equals("<=" ))) r+=get();
	if((r.equals(">"))&&(peek().equals(">>="))) r+=get();
	if((r.equals("<"))&&(peek().equals("<<="))) r+=get();

	if((r.equals("&"))&&(peek().equals("&" ))) r+=get();
	if((r.equals("|"))&&(peek().equals("|" ))) r+=get();

	if((r.equals("&"))&&(peek().equals("=" ))) r+=get();
	if((r.equals("|"))&&(peek().equals("=" ))) r+=get();
	if((r.equals("^"))&&(peek().equals("=" ))) r+=get();
      }
      }
    }
    else r=(String)pile_.pop();

    return r;
  }

  public void put(String _s)
  {
    pile_.push(_s);
  }

  private int pp=0;
  public int ligne()
  {
    int r=((st_==null) ? -1 : st_.lineno());

    if(MaxLines!=0)
    {
      int p=100*r/MaxLines;
      if(p!=pp) progres(p);
      pp=p;
    }

    return r;
  }

  public String fichier()
  {
    String r=((st_==null) ? "" : st_.origin());

    int i=r.lastIndexOf(File.separator);
    if(i>=0) r=r.substring(i+1);

    return r;
  }

  private URL getSystemResource(ClassLoader _cl, String _sr)
  {
    URL r=null;
    /*GCJ-BEGIN*/
    r=_cl.getSystemResource(_sr);
    /*GCJ-END*/
    return r;    
  }

  public String chercheClasse(String _s)
  {
    String r=chercheClasseSysteme(_s);
    if(r==null) r=chercheClasseNormale(_s);
    if(r==null) trace("### "+_s+" introuvable");
    return r;
  }

  private boolean verifieClasse(ClassLoader _cl, String _s)
  {
    boolean r=true;

    /*GCJ-BEGIN*/
    if(VERIFYCLASS)
    {
      trace("### verifie classe "+_s);
      Class c=null;
      try { c=_cl.loadClass(_s); }
      catch(Throwable t)
      {
	r=false;
	trace("### "+_s+" n'est pas une classe valide");
      }
    }
    /*GCJ-END*/

    return r;
  }

  public String chercheClasseNormale(String _s)
  {
    ClassLoader cl=getClass().getClassLoader();
    trace("### cherche classe normale "+_s);

    // Sans package
    {
      String sr ="/"+_s.replace('.','/')+".class";
      URL    url=getSystemResource(cl,sr);
      // trace("=== "+sr);

      if(url!=null)
      {
	String r=_s;
	if(verifieClasse(cl,r))
	{
	  trace("### "+_s+" est une classe ("+r+").");
	  return r;
	}
      }
    }

    // Package standard java.lang
    {
      String sr ="/java/lang/"+_s+".class";
      URL    url=getSystemResource(cl,sr);
      // trace("=== "+sr);

      if(url!=null)
      {
	String r="java.lang."+_s;
	if(verifieClasse(cl,r))
	{
	  trace("### "+_s+" est une classe ("+r+").");
	  return r;
	}
      }
    }

    if(unite_!=null)
    {

      // Package de l'unite courante
      {
	AglPaquet ap=unite_.paquet;
	
	String r="";
	//boolean p=false;
      
	  // if(ap==null)
	  // r="."; @GDX problem ?

	if(ap!=null)
	  for(Enumeration e=ap.nom.identificateurs.elements();
	      e.hasMoreElements(); )
	  {
	    /*if(p) p=false; else */
	    r+="/";
	    r+=(String)e.nextElement();
	  }

	String sr =r+"/"+_s+".class";
	URL    url=getSystemResource(cl,sr);
	// trace("=== "+sr);

	if(url!=null)
	{
	  r=r.replace('/','.')+"."+_s;
	  if(r.startsWith(".")) r=r.substring(1);

	  if(verifieClasse(cl,r))
	  {
	    trace("### "+_s+" est une classe("+r+").");
	    return r;
	  }
	}
      }

      //Imports
      for(Enumeration ei=unite_.imports.elements(); ei.hasMoreElements(); )
      {
	AglImport ai=(AglImport)ei.nextElement();
	
	String r="";
	boolean p=false; // true;
      
	for(Enumeration e=ai.nom.identificateurs.elements();
	    e.hasMoreElements(); )
	{
	  if(p) p=false; else r+="/";
	  r+=(String)e.nextElement();
	}

	if(r.endsWith("/*"))
	{
	  String sr =r.substring(0,r.length()-2)+"/"+_s+".class";
	  URL    url=getSystemResource(cl,sr);
	  // trace("=== "+sr);

	  if(url!=null)
	  {
	    r=r.substring(1,r.length()-2).replace('/','.')+"."+_s;
	    if(verifieClasse(cl,r))
	    {
	      trace("### "+_s+" est une classe ("+r+").");
	      return r;
	    }
	  }
	}
      }
    }

    return null;
  }

  /*
   * A cause du JDK 1.2 il faut aussi regarder le SystemClassLoader.
   * Mais pour maintenir la compatibilite, on utilise la reflection.
   */

  public String chercheClasseSysteme(String _s)
  {
    if(System.getProperty("java.version").compareTo("1.2")<0) return null;

    ClassLoader cl=null;

    /*GCJ-BEGIN*/
    try
    {
      Method m=ClassLoader.class.getMethod("getSystemClassLoader",
					   new Class[0]);
      cl=(ClassLoader)m.invoke(null,new Object[0]);
    }
    catch(Exception ex) { }
    /*GCJ-END*/

    if(cl==null) return null;
    // trace("### can't find SystemClassLoader");

    // System.out.println("ClassPath="+System.getProperty("java.class.path"));
    // URL url=getSystemResource(cl,_s);
    // if(url!=null) return true;
    trace("### cherche classe systeme: "+_s);

    // Sans package
    {
      String sr =_s.replace('.','/')+".class";
      URL    url=getSystemResource(cl,sr);
      // trace("=== "+sr);

      if(url!=null)
      {
	String r=_s;
	if(verifieClasse(cl,r))
	{
	  trace("### "+_s+" est une classe ("+r+").");
	  return r;
	}
      }
    }

    // Package standard java.lang
    {
      String sr ="java/lang/"+_s+".class";
      URL    url=getSystemResource(cl,sr);
      // trace("=== "+sr);

      if(url!=null)
      {
	String r="java.lang."+_s;
	if(verifieClasse(cl,r))
	{
	  trace("### "+_s+" est une classe ("+r+").");
	  return r;
	}
      }
    }

    if(unite_!=null)
    {

      // Package de l'unite courante
      {
	AglPaquet ap=unite_.paquet;
	
	String r="";
	boolean p=true;
      
	if(ap==null)
	  r=".";
	else
	  for(Enumeration e=ap.nom.identificateurs.elements();
	      e.hasMoreElements(); )
	  {
	    if(p) p=false; else r+="/";
	    r+=(String)e.nextElement();
	  }

	String sr =r+"/"+_s+".class";
	URL    url=getSystemResource(cl,sr);
	// trace("=== "+sr);

	if(url!=null)
	{
	  r=r.replace('/','.')+"."+_s;
	  if(verifieClasse(cl,r))
	  {
	    trace("### "+_s+" est une classe("+r+").");
	    return r;
	  }
	}
      }

      //Imports
      for(Enumeration ei=unite_.imports.elements(); ei.hasMoreElements(); )
      {
	AglImport ai=(AglImport)ei.nextElement();
	
	String r="";
	boolean p=true;
      
	for(Enumeration e=ai.nom.identificateurs.elements();
	    e.hasMoreElements(); )
	{
	  if(p) p=false; else r+="/";
	  r+=(String)e.nextElement();
	}

	if(r.endsWith("/*"))
	{
	  String sr =r.substring(0,r.length()-2)+"/"+_s+".class";
	  URL    url=getSystemResource(cl,sr);
	  // trace("=== "+sr);

	  if(url!=null)
	  {
	    r=r.substring(0,r.length()-2).replace('/','.')+"."+_s;
	    if(verifieClasse(cl,r))
	    {
	      trace("### "+_s+" est une classe ("+r+").");
	      return r;
	    }
	  }
	}
      }
    }

    return null;
  }

  public AglType chercheType(String _s)
  {
    return chercheType(new AglNom(_s));
  }

  public AglType chercheType(AglNom _s)
  {
    AglType r=AglType.cherche(_s);

    if(r==null)
    {
      String c=chercheClasse(_s.formeJava());
      if(c!=null) r=creeClasse(c);
    }

    return r;
  }

  public AglClasse creeClasse(String _c)
  {
    return creeClasse(new AglNom(_c));
  }

  public AglClasse creeClasse(AglNom _c)
  {
    // Inner class: Map.Entry
    AglNom parent=_c.parent();
    if(parent!=null)
    {
      AglType t=chercheType(parent);
      if(t!=null) _c=new AglNom(t.nom,_c.dernier());
    }

    AglClasse r=new AglClasse(_c);

    AglNom no=new AglNom("java.lang.Object");
    if(!r.nom.equals(no))
      r.meres.noms.addElement(no);

    return r;
  }

  private final static boolean estDeclaration(String _t, String _u)
  {
    return (  !_t.equals("new")
	    &&(_t.length()>0)
	    &&Character.isJavaIdentifierStart(_t.charAt(0))
	    &&(_u.length()>0)
	    &&(  Character.isJavaIdentifierStart(_u.charAt(0))
	       ||("[]".equals(_u))));
  }

  // Parse

  public void parseFichier() throws Exception
  {
    parseUnite();

    if(AglOptions.definirLiteraux)
    {
      Hashtable t=new Hashtable(11);
      for(Enumeration ei=inconnus_.reverseElements(); ei.hasMoreElements(); )
      {
        AglExpressionGenerique ae=(AglExpressionGenerique)ei.nextElement();
	if((ae.literal==null)&&(t.get(ae.body)==null))
        {
	  t.put(ae.body,Boolean.TRUE);
	  avertissement("Unreconized id "+ae.body+
			" [ligne "+ae.ligne+" fichier "+fichier()+"]");
	}
      }
    }
  }

  public AglUnite parseUnite() throws Exception
  {
    peek(); // recupere les commentaires
    AglUnite au=new AglUnite(st_.origin());
    au.langage="Java";
    au.commentaires=getCommentairesNormaux();

    unite_=au;
    String t="";

    AglVisibilite  visibilite;
    AglVecteur commentaires;

    do
    {
      visibilite  =parseVisibilite();
      commentaires=getCommentaires();

      t=peek();
      trace("WU>"+t);
      
      if(t.equals("package"))
      {
        AglPaquet ap=parsePaquet();
	ap.commentaires=commentaires;
        au.paquet=ap;
      }
      else
      if(t.equals("import"))
      {
        AglImport ai=parseImport();
	ai.commentaires=commentaires;
        au.imports.put(ai.nom.formeJava(),ai);
      }
      else
      if(t.equals("class")||t.equals("interface"))
      {
        AglClasse ac=parseClasse();
	ac.commentaires=commentaires;
        ac.visibilite  =visibilite;
        au.types.put(""+ac,ac);
      }
      else if(!t.equals("")) syntaxe("unknown keyword: "+t);

    } while(!t.equals(""));

    // au.commentaires=commentaires;
    return au;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    AglVisibilite av=new AglVisibilite();
    String        t ="";
    boolean       e =false;

    av.ligne=st_.lineno();
    do
    {
      e=false;
      t=get();
      trace("WV>"+t);
           if(t.equals("abstract"    )) e=av.vabstract    =true;
      else if(t.equals("public"      )) e=av.vpublic      =true;
      else if(t.equals("private"     )) e=av.vprivate     =true;
      else if(t.equals("protected"   )) e=av.vprotected   =true;
      else if(t.equals("static"      )) e=av.vstatic      =true;
      else if(t.equals("final"       )) e=av.vfinal       =true;
      else if(t.equals("native"      )) e=av.vnative      =true;
      else if(t.equals("transient"   )) e=av.vtransient   =true;
      else if(t.equals("strictfp"    )) e=av.vstrictfp    =true;
      else if(t.equals("synchronized")) e=av.vsynchronized=true;
    } while(e);

    put(t);
    return av;
  }

  public AglNom parseNom() throws Exception
  {
    AglNom an=new AglNom();

    String t="";
    do
    {
      trace("WN>"+peek());
      an.identificateurs.addElement(get());
      t=peek();
      if(t.equals(".")) get();
    }
    while(t.equals("."));

    an.ligne=st_.lineno();
    return an;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    AglListeNoms al=new AglListeNoms();

    String t="";
    do
    {
      trace("WLN>"+peek());
      al.noms.addElement(parseNom());
      t=peek();
      if(t.equals(",")) get();
    }
    while(t.equals(","));

    al.ligne=st_.lineno();
    return al;
  }

  public AglType parseType() throws Exception
  {
    trace("WT>"+peek());

    AglNom  base=parseNom();
    AglType r=chercheType(base);
    if(r==null) r=creeClasse(base);

    while(peek().equals("[]"))
      { trace("WT>"+peek()); r=new AglTypeTableau(r); get(); }
    return r;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("package")) syntaxe("package needed and not "+t);

    AglNom np=parseNom();
    suivi("Paquet   : "+np);

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglPaquet ap=AglPaquet.cherche(np);
    if(ap==null) ap=new AglPaquet(np);
    ap.elementaire=false;
    ap.ligne=st_.lineno();
    return ap;
  }
  
  public AglImport parseImport() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("import")) syntaxe("import needed and not "+t);

    AglNom ni=parseNom();
    suivi("Import      : "+ni);

    if(!ni.dernier().equals("*"))
    {
      /*
      AglType ti=AglType.cherche(ni);
      if(ti==null) 
      {
	AglClasse ci=creeClasse(ni);
	ci.ligne=st_.lineno();
      }
      */
      chercheType(ni);
    }

    AglNom np=ni.parent();
    if(  (np!=null)
       &&(AglPaquet.cherche(np)==null)
       &&(chercheType(np)==null))
    {
      AglPaquet ap=new AglPaquet(np);
      ap.ligne=st_.lineno();
    }

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglImport ai=new AglImport(ni);
    ai.ligne=st_.lineno();
    return ai;
  }
  
  public AglClasse parseClasse() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("class")&&!t.equals("interface"))
      syntaxe("class or interface needed and not "+t);

    boolean normale=t.equals("class");

    trace("WC>"+t);
    AglNom nc=parseNom();
    if(unite_.paquet!=null) nc=new AglNom(unite_.paquet.nom,nc);

    if(normale) suivi("Classe      : "+nc);
    else        suivi("Interface   : "+nc);
    
    AglClasse ac=creeClasse(nc);
    ac.ligne       =st_.lineno();
    ac.proprietaire=unite_;
    ac.elementaire =false;
    ac.normale     =normale;

    if(!normale) ac.meres=new AglListeNoms();

    do
    {
      t=get();
      trace("WCB>"+t);
           if(t.equals("extends"))    ac.meres     =parseMothers();
      else if(t.equals("implements")) ac.interfaces=parseInterfaces();
      else if(t.equals("throws"))     ac.exceptions=parseExceptions();
    } while(!t.equals("{"));

    // Ajout des champs herites
    if(AglOptions.definirLiteraux)
    for(Enumeration em=ac.meres.noms.elements(); em.hasMoreElements(); )
    {
      AglNom  nm=(AglNom)em.nextElement();
      AglType tm=AglType.cherche(nm);

      if(tm instanceof AglClasse)
      {
	AglClasse cm=(AglClasse)tm;
	for(Enumeration ea=cm.attributs.elements(); ea.hasMoreElements(); )
	{
          AglAttribut aa=(AglAttribut)ea.nextElement();
	  champs_.ajoute(aa.nom,aa);
	}
      }
    }

    // Body
    put(t);

    AglClasse old=classe_;
    classe_=ac;
    parseBodyClass(ac);
    classe_=old;

    while(peek().equals(";")) get();

    // Retrait des champs herites
    if(AglOptions.definirLiteraux)
    for(Enumeration em=ac.meres.noms.reverseElements();
	em.hasMoreElements(); )
    {
      AglNom  nm=(AglNom)em.nextElement();
      AglType tm=AglType.cherche(nm);

      if(tm instanceof AglClasse)
      {
	AglClasse cm=(AglClasse)tm;
	for(Enumeration ea=cm.attributs.reverseElements();
	    ea.hasMoreElements(); )
	{
          AglAttribut aa=(AglAttribut)ea.nextElement();
	  champs_.enleve(aa.nom,aa);
	}
      }
    }

    return ac;
  }

  private void parseBodyClass(AglClasse ac) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    ac.constructeurs.clear();
    ac.attributs    .clear();
    ac.methodes     .clear();

    while(!peek().equals("}"))
    {
      AglVecteur    commentaires=getCommentaires();
      AglVisibilite visibilite  =parseVisibilite();

      trace("WBC>"+peek());
      t=get();

      trace("### current class: "+ac.nom);
      if(t.equals(ac.nom.dernier())&&peek().equals("(")&&!visibilite.vstatic)
      {
        put(t);

        AglConstructeur am=parseConstructeur();
	am.commentaires=commentaires;
        am.visibilite=visibilite;
	am.proprietaire=ac;
        ac.constructeurs.put(am.signature(),am);
      }
      else if(t.equals("{"))
      {
	put(t);

	// <clinit> et <init>

	AglBloc b;
	if(visibilite.vstatic)
	  { b=ac.clinit; suivi("Clinit        : "+ac.nom.dernier()); }
	else
	  { b=ac.obinit; suivi("Obinit        : "+ac.nom.dernier()); }

	AglBloc bi=parseBlocMultiple();
	for(Enumeration e=bi.instructions.elements(); e.hasMoreElements(); )
        {
	  AglInstruction ai=(AglInstruction)e.nextElement();
	  b.instructions.addElement(ai);
	}

	while(peek().equals(";")) get();
      }
      else if(t.equals("class")||t.equals("interface"))
      {
        put(t);
	AglClasse ic=parseClasse();
	ic.commentaires=commentaires;
	ic.setNom(new AglNom(ac.nom,ic.nom.dernier()));
	//System.err.println("OUTER CLASS "+ac);
	//System.err.println("INNER CLASS "+ic);
        unite_.types.put(""+ic,ic);
      }
      else if(!t.equals("}"))
      {
        put(t);

        type_=parseType();
        nom_ =parseNom();

	if(nom_.dernier().equals(ac.nom.dernier()))
	  syntaxe("no return type for a constructor");

        String v=peek();
        if(v.equals("("))
        {
          AglMethode am=parseMethode();
	  am.commentaires=commentaires;
	  am.proprietaire=ac;
          am.visibilite=visibilite;
	  am.visibilite.vvirtual=true;
	  if(!ac.normale)
	  {
	    am.visibilite.vpublic=true;
	  }
	  ac.methodes.put(am.signature(),am);
        }
        else
        {
          AglVecteur ava=parseAttribut();
          for(Enumeration ea=ava.elements(); ea.hasMoreElements(); )
          {
            AglAttribut aa=(AglAttribut)ea.nextElement();
	    aa.commentaires=commentaires;
            aa.visibilite=visibilite;
	    if(!ac.normale)
	    {
	      aa.visibilite.vpublic=true; 
	      aa.visibilite.vfinal =true; 
	      aa.visibilite.vstatic=true; 
	    }
	    aa.proprietaire=ac;
            ac.attributs.put(aa.signature(),aa);

	    if(AglOptions.definirLiteraux)
	      champs_.ajoute(aa.nom,aa);
          }
        }
      }
    }

    get();

    if(AglOptions.definirLiteraux)
    for(Enumeration ei=inconnus_.reverseElements(); ei.hasMoreElements(); )
    {
      AglExpressionGenerique ae=(AglExpressionGenerique)ei.nextElement();
      if(ae.literal==null) definitLiteral(ae);
      if(ae.literal!=null)
      {
	inconnus_.removeElement(ae);
	//System.err.println("Found id "+ae.body);
      }
      else
      {
	//System.err.println("Unreconized id "+ae.body);
      }
    }

    if(AglOptions.definirLiteraux)
    for(Enumeration ea=ac.attributs.reverseElements(); ea.hasMoreElements(); )
    {
      AglAttribut aa=(AglAttribut)ea.nextElement();
      champs_.enleve(aa.nom,aa);
    }
  }

  public AglListeNoms parseMothers() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    suivi("Mere        : "+ln);

    for(Enumeration e=ln.noms.elements(); e.hasMoreElements(); )
    {
      AglNom  an=(AglNom)e.nextElement();
      AglType at=chercheType(an);
      //if(at==null) creeClasse(an);
    }

    return ln;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    suivi("Interfaces  : "+ln);

    for(Enumeration e=ln.noms.elements(); e.hasMoreElements(); )
    {
      AglNom  an=(AglNom)e.nextElement();
      AglType at=chercheType(an);

      if(at instanceof AglClasse)
	((AglClasse)at).normale=false;
      //if(at==null) creeClasse(an).normale=false;
    }

    return ln;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    suivi("Exceptions  : "+ln);

    for(Enumeration e=ln.noms.elements(); e.hasMoreElements(); )
    {
      AglNom  an=(AglNom)e.nextElement();
      AglType at=chercheType(an);
      //if(at==null) creeClasse(an);
    }

    return ln;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    AglNom nm=parseNom();
    suivi("Constructeur: "+nm);

    AglConstructeur m=new AglConstructeur(nm);
    m.ligne=st_.lineno();

    m.parametres=parseParametres();

    if(peek().equals("throws")) { get(); m.exceptions=parseExceptions(); }

    Enumeration ep;

    if(AglOptions.definirLiteraux)
    {
      ep=m.parametres.reverseElements();
      while(ep.hasMoreElements())
      {
        AglParametre p=(AglParametre)ep.nextElement();
        champs_.ajoute(p.nom,p);
      }
    }

    if(SKIPINSTRUCTIONS)
    {
      skipInstructions();
      m.corps=new AglBloc();
    }
    else
    {
      m.corps=parseBlocMultiple();
    }

    while(peek().equals(";")) get();

    if(AglOptions.definirLiteraux)
    {
      ep=m.parametres.elements();
      while(ep.hasMoreElements())
      {
        AglParametre p=(AglParametre)ep.nextElement();
	champs_.enleve(p.nom,p);
      }
    }

    return m;
  }

  public AglMethode parseMethode() throws Exception
  {
    suivi("Methode     : "+nom_+" ["+type_+"]");

    AglMethode m=new AglMethode(nom_.dernier(),type_);
    m.ligne=st_.lineno();

    m.parametres=parseParametres();

    while(peek().equals("[]"))
    {
      get();
      m.type=new AglTypeTableau(m.type);
    }

    if(peek().equals("throws")) { get(); m.exceptions=parseExceptions(); }

    Enumeration ep;

    if(AglOptions.definirLiteraux)
    {
      ep=m.parametres.reverseElements();
      while(ep.hasMoreElements())
      {
        AglParametre p=(AglParametre)ep.nextElement();
	champs_.ajoute(p.nom,p);
      }
    }

    if(!peek().equals(";"))
    {
      if(SKIPINSTRUCTIONS)
      {
	skipInstructions();
	m.corps=new AglBloc();
      }
      else
      {
	m.corps=parseBlocMultiple();
      }
    }

    while(peek().equals(";")) get();

    if(AglOptions.definirLiteraux)
    {
      ep=m.parametres.elements();
      while(ep.hasMoreElements())
      {
        AglParametre p=(AglParametre)ep.nextElement();
        champs_.enleve(p.nom,p);
      }
    }

    return m;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    AglVecteur r  =new AglVecteur();
    AglNom     nom=nom_;
    String     t  ="";
    
    do
    {
      AglType type=type_;

      while(peek().equals("[]"))
      {
	get();
	type=new AglTypeTableau(type);
      }

      suivi("Attribut    : "+nom+" ["+type+"]");

      AglAttribut a=new AglAttribut(nom.dernier(),type);
      a.ligne=st_.lineno();
      r.addElement(a);

      t=get();
      if(t.equals("="))
      {
	if(peek().equals("{")) a.initiale=parseExpressionInitArray();
	else                   a.initiale=parseExpression();
        suivi("Initiale    : "+a.initiale);
	t=get();
      }

      if(t.equals(","))
        nom=parseNom();
      else
      if(!t.equals(";"))
        syntaxe(", or ; needed and not "+t);

    } while(!t.equals(";"));

    while(peek().equals(";")) get();

    return r;
  }

  public AglVecteur parseParametres() throws Exception
  {
    int    np=0;
    String t ="";
    AglVecteur r =new AglVecteur();

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    do 
    {
      if(!peek().equals(")"))
      {
	AglVisibilite visibilite=parseVisibilite();
        AglType type=parseType();
        String  nom =get();
        np++;

        // ancienne forme
        while(peek().equals("[]")) { get(); type=new AglTypeTableau(type); }

        suivi("Parametre "+np+" : "+nom+" ["+type+"]");

        AglParametre p=new AglParametre(nom,type);
	p.visibilite=visibilite;
	p.ligne=st_.lineno();
        r.addElement(p);

             if(peek().equals(",")) get();
        else if(!peek().equals(")")) syntaxe(", or ; needed and not "+t);
      }

    } while(!peek().equals(")"));

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    return r;
  }

  public AglVecteur parseArgs() throws Exception
  {
    int        na=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    t=get();
    if(!(t.equals("("))) syntaxe("( needed and not "+t);

    do 
    {
      t=peek();

      if(!t.equals(")"))
      {
        AglExpression ae=parseExpression();
        na++;
        trace("Argument  "+na+" : "+ae);
        r.addElement(ae);
      }

      t=get();
      if(!(t.equals(",")||t.equals(")"))) syntaxe(", or ) needed and not "+t);

    } while(!t.equals(")"));

    return r;
  }

  public AglBloc parseBloc() throws Exception
  {
    return peek().equals("{") ? parseBlocMultiple() : parseBlocSimple();
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    AglBloc ab=new AglBloc();
    ab.commentaires=getCommentaires();

    AglVecteur vars=null;
    if(AglOptions.definirLiteraux)
      vars=new AglVecteur();

    String  t="";
    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    while(!peek().equals("}"))
    {
      trace("WBM>>> "+peek());
      t=get();

      while(peek().equals(":"))
      {
	AglVecteur commentaires=getCommentaires();
	get();
	ab.instructions.addElement(new AglInstructionLabel(t));
	t=get();
      }

      // Essai @GDX
      AglVisibilite visibilite=new AglVisibilite();
      if(t.equals("final"))
      {
	visibilite.vfinal=true;
	t=get();
      }

      // classdef inside a block ;-((( cf. EventQueue 537
      if(t.equals("class")||t.equals("interface"))
      {
        put(t);
	AglVecteur commentaires=getCommentaires();
	AglClasse  ic=parseClasse();
	ic.visibilite=visibilite;
	ic.commentaires=commentaires;
	//ic.setNom(new AglNom(ac.nom,ic.nom.dernier()));
        unite_.types.put(""+ic,ic);
      }
      else
      /*
      if((chercheType(t)!=null)&&(!peek().equals(".")))
      {
        put(t);
        for(Enumeration e=parseInstructionDefVar().elements();
	    e.hasMoreElements(); )
        {
          AglInstructionDefVar ai=(AglInstructionDefVar) e.nextElement();
          ai.visibilite=visibilite;
          ab.instructions.addElement(ai);

	  vars.addElement(ai);
	  champs_.ajoute(ai.nom,ai);
        }
      }
      else
      */
      {
	String u=peek(); //TMP

        put(t);
        AglInstruction ai=null;
	AglVecteur commentaires=getCommentaires();
        
             if(t.equals("{"))            ai=parseInstructionBloc        ();
        else if(t.equals(";"))            ai=parseInstructionVide        ();
        else if(t.equals("break"))        ai=parseInstructionBreak       ();
        else if(t.equals("catch"))        ai=parseInstructionCatch       ();
        else if(t.equals("continue"))     ai=parseInstructionContinue    ();
        else if(t.equals("do"))           ai=parseInstructionDoWhile     ();
        else if(t.equals("finally"))      ai=parseInstructionFinally     ();
        else if(t.equals("for"))          ai=parseInstructionFor         ();
        else if(t.equals("if"))           ai=parseInstructionIf          ();
        else if(t.equals("return"))       ai=parseInstructionReturn      ();
        else if(t.equals("switch"))       ai=parseInstructionSwitch      ();
	else if(t.equals("synchronized")) ai=parseInstructionSynchronized();
	else if(t.equals("throw"))        ai=parseInstructionThrow       ();
        else if(t.equals("try"))          ai=parseInstructionTry         ();
        else if(t.equals("while"))        ai=parseInstructionWhile       ();

	// BEGIN TMP @GDX PROBABLY A TYPE
        else if(estDeclaration(t,u))
	{
	  boolean p=true;
          for(Enumeration e=parseInstructionDefVar().elements();
	      e.hasMoreElements(); )
          {
            AglInstructionDefVar aidv=(AglInstructionDefVar) e.nextElement();
	    if(p) aidv.commentaires=commentaires;
	    aidv.visibilite=visibilite;
	    ab.instructions.addElement(aidv);
	    p=false;

	    if(AglOptions.definirLiteraux)
	    {
	      vars.addElement(aidv);
	      champs_.ajoute(aidv.nom,aidv);
	    }
	  }
	}
	// END TMP @GDX PROBABLY A TYPE

        else
        {
          AglExpression gauche=parseExpression();

          t=peek();
               if(t.equals("="))      ai=parseInstructionAffectation(gauche);
       // else if(t.equals("("))      ai=parseInstructionProcedure  (gauche);
          else if(t.equals(";"))      ai=parseInstructionExpression (gauche);
	  else if(  t.equals("+="  )||t.equals("-=")
		  ||t.equals("*="  )||t.equals("/=")||t.equals("%=")
                  ||t.equals("&="  )||t.equals("|=")||t.equals("^=")
                  ||t.equals(">>=" )||t.equals("<<=")
		  ||t.equals(">>>=")||t.equals("<<<="))
          {
            t=get();
            AglExpression ae=
	      new AglExpressionBinaire(t.substring(0,1),gauche,parseExpression());
            ae.ligne=st_.lineno();

            ai=new AglInstructionAffectation(gauche,ae);
            ai.ligne=st_.lineno();

            t=get();
            if(!t.equals(";")) syntaxe("; needed and not "+t);
          }
	  else
	  {
	    try
	    {
	      String s=CODE.code(gauche)+" ";
	      //if(s.startsWith("(")&&s.endsWith(")"))
	      //  s=s.substring(1,s.length()-1);
	      put(s);
	    }
	    catch(Throwable th) { }
	    ai=parseInstructionGenerique();
	    System.exit(0);
	  }
        }

	if(ai!=null)
	{
	  ai.commentaires=commentaires;
	  ab.instructions.addElement(ai);
	}
      }
    }

    t=get();
    if(!t.equals("}")) syntaxe("} needed and not "+t);

    if(AglOptions.definirLiteraux)
    {
      Enumeration ev=vars.reverseElements();
      while(ev.hasMoreElements())
      {
        AglInstructionDefVar ai=(AglInstructionDefVar)ev.nextElement();
        champs_.enleve(ai.nom,ai);
      }
    }

    return ab;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t=get();

    trace("WBS>>> "+t+" | "+peek());

    String u=peek();
    put(t);

    AglInstruction ai=null;
    AglVecteur commentaires=getCommentaires();

    /*
      if((chercheType(t)!=null)&&(!u.equals(".")))
      {
      for(Enumeration e=parseInstructionDefVar().elements();
      e.hasMoreElements(); )
      {
      ai=(AglInstructionDefVar) e.nextElement();
      ai.commentaires=commentaires;
      commentaires=getCommentaires();
      ab.instructions.addElement(ai);
      }
      }
      else
    */
    {
             if(t.equals("{"))            syntaxe("{ is not wanted there");
        else if(t.equals(";"))            ai=parseInstructionVide        ();
        else if(t.equals("break"))        ai=parseInstructionBreak       ();
        else if(t.equals("catch"))        ai=parseInstructionCatch       ();
        else if(t.equals("continue"))     ai=parseInstructionContinue    ();
        else if(t.equals("do"))           ai=parseInstructionDoWhile     ();
        else if(t.equals("finally"))      ai=parseInstructionFinally     ();
        else if(t.equals("for"))          ai=parseInstructionFor         ();
        else if(t.equals("if"))           ai=parseInstructionIf          ();
        else if(t.equals("return"))       ai=parseInstructionReturn      ();
        else if(t.equals("switch"))       ai=parseInstructionSwitch      ();
        else if(t.equals("synchronized")) ai=parseInstructionSynchronized();
        else if(t.equals("throw"))        ai=parseInstructionThrow       ();
        else if(t.equals("try"))          ai=parseInstructionTry         ();
        else if(t.equals("while"))        ai=parseInstructionWhile       ();

	// BEGIN TMP @GDX PROBABLY A TYPE
        else if(estDeclaration(t,u))
	{
	  boolean p=true;
	  for(Enumeration e=parseInstructionDefVar().elements();
	      e.hasMoreElements(); )
          {
            AglInstructionDefVar aidv=(AglInstructionDefVar) e.nextElement();
	    if(p) aidv.commentaires=commentaires;
	    ab.instructions.addElement(aidv);
	    p=false;
	  }
	}
	// END TMP @GDX PROBABLY A TYPE

        else
        {
          AglExpression gauche=parseExpression();

          t=peek();
               if(t.equals("="))      ai=parseInstructionAffectation(gauche);
       // else if(t.equals("("))      ai=parseInstructionProcedure  (gauche);
          else if(t.equals(";"))      ai=parseInstructionExpression (gauche);
          else if(  t.equals("+=")||t.equals("-=")
		  ||t.equals("*=")||t.equals("/=")||t.equals("%=")
                  ||t.equals("&=")||t.equals("|=")||t.equals("^=")
                  ||t.equals(">>=" )||t.equals("<<=")
		  ||t.equals(">>>=")||t.equals("<<<="))
          {
            t=get();
            AglExpression ae=new AglExpressionBinaire(t.substring(0,1),gauche,parseExpression());
            ae.ligne=st_.lineno();

            ai=new AglInstructionAffectation(gauche,ae);
            ai.ligne=st_.lineno();

            t=get();
            if(!t.equals(";")) syntaxe("; needed and not "+t);
          }
          else
	  {
	    try { put(CODE.code(gauche)+" "); }
	    catch(Throwable th) { }
	    ai=parseInstructionGenerique();
	  }
        }

	if(ai!=null)
	{
	  ai.commentaires=commentaires;
	  ab.instructions.addElement(ai);
	}
      }

    return ab;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t =get();
    String  u =peek();

    trace("WBV>>> "+t+" | "+u);

    put(t);

    while(!t.equals(";")&&!t.equals(")"))
    {
      AglInstruction ai=null;
      AglExpression gauche=parseExpression();

      t=peek();
      if(t.equals(",")||t.equals(";")||t.equals(")"))
      {
	ai=new AglInstructionExpression(gauche);
	ai.ligne=st_.lineno();
      }
      else if(t.equals("="))
      {
	t=get();
	AglExpression ae=parseExpression();

	ai=new AglInstructionAffectation(gauche,ae);
	ai.ligne=st_.lineno();
      }
      else if(  t.equals("+=")||t.equals("-=")
	      ||t.equals("*=")||t.equals("/=")||t.equals("%=")
	      ||t.equals("&=")||t.equals("|=")||t.equals("^=")
	      ||t.equals(">>=" )||t.equals("<<=")
	      ||t.equals(">>>=")||t.equals("<<<="))
      {
	t=get();
	AglExpression ae=new AglExpressionBinaire
	    (t.substring(0,1),gauche,parseExpression());
	ae.ligne=st_.lineno();

	ai=new AglInstructionAffectation(gauche,ae);
	ai.ligne=st_.lineno();
      }
      else ai=parseInstructionGenerique();

      ab.instructions.addElement(ai);

      t=peek();	   
      if(!t.equals(";")&&!t.equals(",")&&!t.equals(")"))
	syntaxe("; or , or ) needed and not "+t);

      if(t.equals(",")) get();
    }

    return ab;
  }

// Expressions

  public AglExpression parseExpression() throws Exception
  {
    String p=peek();
    trace("WE1>>> "+p);
    if(p.equals(";")||p.equals(",")) return null;

    AglExpression ae;

         if(p.equals("new")) ae=parseExpressionNew();
    else if(p.equals("+" ))  ae=parseExpressionUnaireDroite();
    else if(p.equals("-" ))  ae=parseExpressionUnaireDroite();
    else if(p.equals("~" ))  ae=parseExpressionUnaireDroite();
    else if(p.equals("++"))  ae=parseExpressionUnaireDroite();
    else if(p.equals("--"))  ae=parseExpressionUnaireDroite();
    else if(p.equals("!" ))  ae=parseExpressionUnaireDroite();
    else if(p.equals("("))
    {
      Stack buffer=new Stack();

      String t=get();
      String v="";

      while(  !peek().equals("(")
            &&!peek().equals(")")
            &&!peek().equals("instanceof"))
      {
	if(!peek().equals("[]")) v+=peek();
	buffer.push(get());
      }

      boolean typage=peek().equals(")");
      AglType type=chercheType(v);
      typage=(typage&&(type!=null));
      //System.err.println("==================== "+v+" "+typage);

      while(!buffer.empty())
	put((String)buffer.pop());
      put("(");

      if(typage)
	ae=parseExpressionTypage();
      else
	ae=parseExpressionParenthesee();

      /*
      String  t,u,v;
      t=get();
      u=get();
      v=peek();
      put(u);
      put(t);
      trace("WE2>>> "+t+" | "+u+" | "+v);

      AglType c=chercheType(u);
      if((c!=null)&&((v.equals(")"))||(v.equals("[]"))))
	ae=parseExpressionTypage();
      else
	ae=parseExpressionParenthesee();
      */
    }
    else
    {
      String  t,u;
      t=get();
      u=peek();
      put(t);

      trace("WE3>>> "+t+" | "+u);

      if(u.equals("("))
      {
	boolean statique=false;
	if(classe_!=null)
	{
	  for(Enumeration em=classe_.methodes.elements();
	      em.hasMoreElements(); )
	  {
	    AglMethode am=(AglMethode)em.nextElement();
	    if(am.nom.equals(t)&&am.visibilite.vstatic)
	      statique=true;
	  }
	}

	if(AglOptions.definirLiteraux)
	{
	  if(statique) ae=creeExpressionGenerique(classe_.nom.formeJava());
	  else         ae=creeExpressionGenerique("this");
	}
	else ae=null;

	ae=new AglExpressionMethode(ae,parseExpressionFonction(),statique);
      }
      /*
	else if(u.equals("."))
	ae=new AglExpressionAttribut(null,get());
      */
      else if(u.equals("["))
      {
	ae=creeExpressionGenerique(get());
	u=get(); if(!u.equals("[")) syntaxe("[ needed and not "+u);
	ae=new AglExpressionIndice(ae,parseExpression());
	u=get(); if(!u.equals("]")) syntaxe("] needed and not "+u);
      }
      else
      {
	t=get();
        while(peek().equals("[]")) t+=get();
	ae=creeExpressionGenerique(t);
      }

      ae.ligne=st_.lineno();
    }

    boolean encore=true;
    while(encore)
    {
      encore=false;
      String u;

      trace("Expression  : "+ae+" ["+ae.getClass()+"]");
      u=peek();
      trace("WE4>>> "+u);

      if(u.equals(";")||u.equals(","))
	encore=false;
      else
      if(u.equals("?"))
      {
	String        op1=get();
	AglExpression em =parseExpression();
	String        op2=get();
	if(!op2.equals(":")) syntaxe(": needed and not "+op2);
	AglExpression ed =parseExpression();
	
	ae=new AglExpressionTernaire(ae,op1,em,op2,ed);
	ae.ligne=st_.lineno();
	encore=true;
      }
      else
      if(u.equals("="))
      {
	get();
	ae=new AglExpressionAffectation(ae,parseExpression());
	ae.ligne=st_.lineno();
	encore=true;
      }
      else if(  u.equals("+="  )||u.equals("-="  )
	      ||u.equals("*="  )||u.equals("/="  )||u.equals("%=")
	      ||u.equals("&="  )||u.equals("|="  )||u.equals("^=")
              ||u.equals(">>=" )||u.equals("<<=" )
	      ||u.equals(">>>=")||u.equals("<<<="))
      {
	get();
	AglExpression gauche=ae;
	ae=new AglExpressionBinaire
	  (u.substring(0,1),gauche,parseExpression());
	ae.ligne=st_.lineno();

	ae=new AglExpressionAffectation(gauche,ae);
	ae.ligne=st_.lineno();
      }
      else
      if(  (u.equals("+"  ))||(u.equals("-"))
         ||(u.equals("*"  ))||(u.equals("/"))
         ||(u.equals("%"  ))
         ||(u.equals("==" ))||(u.equals("!="))
         ||(u.equals("<"  ))||(u.equals(">"))
         ||(u.equals("<=" ))||(u.equals(">="))
         ||(u.equals("<<" ))||(u.equals(">>"))
         ||(u.equals("<<<"))||(u.equals(">>>"))
         ||(u.equals("&&" ))||(u.equals("||"))
         ||(u.equals("&"  ))||(u.equals("|"))||(u.equals("^")))
      {
	ae=new AglExpressionBinaire(get(),ae,parseExpression());
	ae.ligne=st_.lineno();
	encore=true;
	
	// detour temporaire pour (1<2)?3:4
	if(((AglExpressionBinaire)ae).droite instanceof AglExpressionTernaire)
	{
	  AglExpressionBinaire  eb=(AglExpressionBinaire)ae;
	  AglExpressionTernaire et=(AglExpressionTernaire)eb.droite;
	  eb.droite=et.gauche;
	  et.gauche=eb;
	  ae=et;
	}
      }
      else
      if(u.equals("instanceof"))
      {
	get();
	ae=new AglExpressionEstUn(ae,parseType());
	ae.ligne=st_.lineno();
	encore=true;
      }
      else
      if(u.equals("["))
      {
	u=get(); if(!u.equals("[")) syntaxe("[ needed and not "+u);
	ae=new AglExpressionIndice(ae,parseExpression());
	u=get(); if(!u.equals("]")) syntaxe("] needed and not "+u);
	ae.ligne=st_.lineno();
	encore=true;
      }
      else
      if((u.equals("++"))||(u.equals("--")))
      {
	ae=new AglExpressionUnaireGauche(get(),ae);
	ae.ligne=st_.lineno();
	encore=true;
      }
      else
      if(u.equals("."))
      {
        get();
	String v=get();
	String w;
	while((w=peek()).equals("[]"))
	    v+=get();
	put(v);
	
	if(w.equals("("))
	{
	  ae=new AglExpressionMethode
	    (ae,parseExpressionFonction(),
	     chercheType(CODE.code(ae))!=null);
	}
	else
	{
	  //System.err.println("V="+v+" W="+w+" ["+ligne()+"]");

	  ae=new AglExpressionAttribut
	    (ae,get(),
	     chercheType(CODE.code(ae))!=null);

	  if(  !w.equals("")
	     &&!w.equals("instanceof")
	     &&Character.isJavaIdentifierStart(w.charAt(0)))
	  {
	    // declaration
	    AglNom an=new AglNom(CODE.code(ae));
	    //System.err.println(an);
	    AglType at=chercheType(an);
	    if(at==null) at=creeClasse(an);
	    ae=new AglExpressionDefVar(get(),at);
	    while(peek().equals("[]"))
	    {
	      get();
	      ((AglExpressionDefVar)ae).type=
		new AglTypeTableau(((AglExpressionDefVar)ae).type);
	    }
	    if(peek().equals("="))
	    {
	      get();
	      ((AglExpressionDefVar)ae).initiale=parseExpression();
	    }
	  }
	}

	ae.ligne=st_.lineno();
	encore=true;
      }

      // Test... FAUX
      /*
      else
      {
        get();
        String v=peek();
        put(u);

        if("=".equals(v)&&ae instanceof AglExpressionGenerique)
        {
          AglClasse c=new AglClasse
	      (new AglNom(((AglExpressionGenerique)ae).body));
	  AglExpressionAffectation ea=
	      (AglExpressionAffectation)parseExpression();
	  ae=new AglExpressionDefVar(u,c);
	  ((AglExpressionDefVar)ae).initiale=ea.droite;
        }
      }
      */

    }

    return ae;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    String t="";
    int    np=0;
    int    nt=0;

    String body="";

    do
    {
      t=get();
      //trace("WEG>>> "+t);

      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      if(t.equals("]")) nt--;
      if(t.equals("[")) nt++;
      
           if((np<0)&&(t.equals(")"))) ;
      else if((nt<0)&&(t.equals("]"))) ;
      else if((np==0)&&(nt==0)&&(t.equals(";")||t.equals(","))) ;
      else
      {
        if(  (body.length()>0)
           &&(  Character.isJavaIdentifierPart(body.charAt(body.length()-1))
	      ||(body.charAt(body.length()-1)==')'))
	   &&(  Character.isJavaIdentifierStart(t.charAt(0))
	      ||(t.charAt(0)=='\"'))) body+=" ";
	body+=t;
      }
    } while(  (np>0)
            ||(nt>0)
	    ||((np==0)&&(nt==0)&&(!(t.equals(";")||t.equals(",")))));

    if(t.equals(";")||t.equals(",")) put(t);
    if((np<0)&&t.equals(")")) put(t);
    if((nt<0)&&t.equals("]")) put(t);

    AglExpressionGenerique ae=creeExpressionGenerique(body);
    ae.ligne=st_.lineno();
    return ae;
  }

  protected AglExpressionGenerique creeExpressionGenerique(String _body)
  {
    AglExpressionGenerique r=new AglExpressionGenerique(_body);
    if(AglOptions.definirLiteraux)
    {
      definitLiteral(r);
      if(r.literal==null) inconnus_.addElement(r);
    }
    return r;
  }

  protected void definitLiteral(AglExpressionGenerique _expr)
  {
    String s=_expr.body;

    int  l=s.length();
    char c0=((l>0) ? s.charAt(0)   : ' ');
    char c1=((l>1) ? s.charAt(1)   : ' ');
    char cN=((l>0) ? s.charAt(l-1) : ' ');

    if("null".equals(s))
    {
      //System.err.println("Literal Null");
      _expr.literal=new AglLiteralNull();
    }
    else
    if("this".equals(s))
    {
      //System.err.println("Literal Null");
      _expr.literal=new AglLiteralThis(classe_);
    }
    else
    if("super".equals(s))
    {
      //System.err.println("Literal Null");
      AglType t=null;
      if((classe_!=null)&&(classe_.meres.noms.size()>0))
	t=chercheType((AglNom)classe_.meres.noms.elementAt(0));
      _expr.literal=new AglLiteralSuper(t);
    }
    else
    if("true".equals(s))
    {
      //System.err.println("Literal Boolean: "+s);
      _expr.literal=new AglLiteralBooleen(true);
    }
    else
    if("false".equals(s))
    {
      //System.err.println("Literal Boolean: "+s);
      _expr.literal=new AglLiteralBooleen(false);
    }
    else
    if(  (l>=2)
       &&(c0=='"')
       &&(cN=='"')
	 //&&(s.substring(1,l-1).indexOf('"')<0)
	 )
    {
      //System.err.println("Literal Chaine: "+s.substring(1,l-1));
      _expr.literal=new AglLiteralChaine(s.substring(1,l-1));
    }
    else
    if(  (l>=2)
       &&(c0=='\'')
       &&(cN=='\'')
	 //&&(s.substring(1,l-1).indexOf('\'')<0)
	 )
    {
      //System.err.println("Literal Caractere: "+s.substring(1,l-1));
      _expr.literal=new AglLiteralCaractere(s.substring(1,l-1));
    }
    else
    if(AglType.cherche(s)!=null)
    {
      //System.err.println("Literal Type: "+s);
      //avertissement("class "+s+" used before its declaration");
      _expr.literal=new AglLiteralType(AglType.cherche(s));
    }
    else
    if((l>0)&&Character.isJavaIdentifierStart(c0))
    {
      AglObjet o=champs_.objet(s);

      if(o instanceof AglParametre)
      {
	//System.err.println("Literal Parametre: "+s);
	_expr.literal=new AglLiteralParametre((AglParametre)o);
      }
      else
      if(o instanceof AglExpressionDefVar)
      {
	//System.err.println("Literal Variable: "+s);
	_expr.literal=new AglLiteralVariable((AglExpressionDefVar)o);
      }
      else
      if(o instanceof AglInstructionDefVar)
      {
	//System.err.println("Literal Variable: "+s);
	_expr.literal=new AglLiteralVariable((AglInstructionDefVar)o);
      }
      else
      if(o instanceof AglAttribut)
      {
	//System.err.println("Literal Attribut: "+s);
	_expr.literal=new AglLiteralAttribut((AglAttribut)o);
      }
      /*
      else
      {
        System.err.println("Unreconized id '"+s+"' ["
			   +fichier()+":"+ligne()+"]");
      }
      */
    }
    else
    if((l>0)&&Character.isDigit(c0))
    {
      if(s.startsWith("0x"))
      {
	boolean simple=true;
	if(s.endsWith("l")) { simple=false; s=s.substring(0,l-1); }
	s=s.substring(2);
	//System.err.println("Literal Hexa: "+s);
	_expr.literal=new AglLiteralEntier
	    (Long.parseLong(s,16),simple,16);
      }
      else
      if(s.endsWith("f"))
      {
	//System.err.println("Literal Float: "+s);
	_expr.literal=new AglLiteralReel
	    (new Double(s).doubleValue(),true);
      }
      else
      if(s.endsWith("d")||(s.indexOf('.')>=0)||(s.indexOf('e')>=1))
      {
	//System.err.println("Literal Double: "+s);
	_expr.literal=new AglLiteralReel
	    (new Double(s).doubleValue(),false);
      }
      else
      {
	boolean simple=true;
	if(s.endsWith("l")) { simple=false; s=s.substring(0,l-1); }
	//System.err.println("Literal Decimal: "+s);
	_expr.literal=new AglLiteralEntier
	    (Long.parseLong(s,10),simple,10);
      }
    }
    else
      avertissement("Unreconized literal "+s
			 +" ["+fichier()+":"+ligne()+"]");
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite()
      throws Exception
  {
    String        op=get();

    trace("Unaire     : "+op);
    AglExpression ed=parseExpression();

    AglExpressionUnaireDroite ae=new AglExpressionUnaireDroite(op,ed);
    ae.ligne=st_.lineno();
    return ae;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    AglExpression eg=creeExpressionGenerique(get());
    String        op=get();

    trace("Binaire    : "+op);
    AglExpression ed=parseExpression();

    AglExpressionBinaire ae=new AglExpressionBinaire(op,eg,ed);
    ae.ligne=st_.lineno();
    return ae;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    String t="";

    String nom=get();
    AglVecteur le =parseArgs();
    trace("Fonction    : "+nom+" ("+le.size()+" arguments)");

    AglExpressionFonction ae=new AglExpressionFonction(nom);
    ae.ligne=st_.lineno();
    for(Enumeration e=le.elements(); e.hasMoreElements(); )
      ae.arguments.addElement(e.nextElement());
    return ae;
  }

  public AglExpression parseExpressionInitArray() throws Exception
  {
    if(!peek().equals("{")) syntaxe("{ needed and not "+peek());
    get();

    AglExpressionInitArray ae=new AglExpressionInitArray();

    while(!peek().equals("}"))
    {
      if(peek().equals("{"))
	ae.initiales.addElement(parseExpressionInitArray());
      else
	ae.initiales.addElement(parseExpression());

      if(!peek().equals(",")&&!peek().equals("}"))
	syntaxe("} or , needed and not "+peek());
      if(peek().equals(",")) get();
    }

    if(!peek().equals("}")) syntaxe("} needed and not "+peek());
    get();

    return ae;
  }

  public AglExpression parseExpressionNew() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("new")) syntaxe("new needed and not "+t);

    AglType c=parseType();

    AglExpression ae;

    if(peek().equals("("))
    {
      AglVecteur le=parseArgs();
      trace("New         : "+c.nom.dernier()+" ("+le.size()+" arguments)");

      if(!peek().equals("{"))
      {
	AglExpressionNew aen=new AglExpressionNew(c);
	aen.ligne=st_.lineno();
	for(Enumeration e=le.elements(); e.hasMoreElements(); )
	  aen.arguments.addElement(e.nextElement());
	ae=aen;
      }
      else
      {
	trace("anonymous class not yet well reconized: "+t);
	anonymes_++;

	/*
	AglNom nc=new AglNom("Anonyme_"+anonymes_);
	if(unite_.paquet!=null)
	  nc=new AglNom(unite_.paquet.nom,nc);
	*/
	AglNom nc=new AglNom("Anonyme_"+anonymes_);
	if(classe_!=null)
	  nc=new AglNom(classe_.nom,nc);
	else
	if(unite_.paquet!=null)
	  nc=new AglNom(unite_.paquet.nom,nc);

	AglClasse ac=creeClasse(nc);
	ac.elementaire=false;
	ac.ligne=st_.lineno();
	ac.meres.noms.addElement(c.nom);
	parseBodyClass(ac);

	AglExpressionNew aen=new AglExpressionNew(ac);
	aen.ligne=st_.lineno();
	for(Enumeration e=le.elements(); e.hasMoreElements(); )
	  aen.arguments.addElement(e.nextElement());
	ae=aen;
      }
    }
    else
    {
      trace("NewArray    : "+c.nom.dernier());

      AglExpressionNewArray aen=new AglExpressionNewArray(c);
      aen.ligne=st_.lineno();
      ae=aen;

      if(peek().equals("{"))
      {
	aen.initiale=parseExpressionInitArray();
      }
      else
      {
	t=get(); if(!t.equals("[")) syntaxe("[ needed and not "+t);
	AglExpression ne=parseExpression();
	t=get(); if(!t.equals("]")) syntaxe("] needed and not "+t);
	aen.nombre=ne;

	while(peek().equals("["))
	{
	  t=get(); if(!t.equals("[")) syntaxe("[ needed and not "+t);
	  AglExpression nex=parseExpression();
	  t=get(); if(!t.equals("]")) syntaxe("] needed and not "+t);
	  aen.type=new AglTypeTableau(aen.type,nex);
	}
	while(peek().equals("[]"))
	{
	  get();
	  aen.type=new AglTypeTableau(aen.type);
	}
      }
    }

    return ae;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpressionParenthesee ae=new AglExpressionParenthesee(parseExpression());

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    ae.ligne=st_.lineno();
    return ae;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglType type=parseType();
    trace("Typage      : "+type);

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglExpressionTypage ae=new AglExpressionTypage(type,parseExpression());
    ae.ligne=st_.lineno();
    return ae;
  }

// Instructions

  public void skipInstructions() throws Exception
  {
    String  t="";
    int     na=0;
    boolean pv=true;

    do
    {
      t=get();
      if(t.equals("" )) break;
      if(t.equals("}")) na--;
      if(t.equals("{")) na++;
    } while(na>0);
  }

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    String  t="";
    int     na=0;
    int     np=0;
    boolean pv=true;

    String body="";

    do
    {
      t=get();
      // trace("WI>>> "+t);

      if(t.equals("}")) na--;
      if(t.equals("{")) na++;
      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      // if(na>0)
      {
             if(t.equals("{")) { body+="\n" +indente(na)+"{\n"; pv=true; }
        else if(t.equals("}")) { body+=indente(na+1)+"}\n"; pv=true; }
        else if(t.equals(";")) { if(np==0) { body+=";\n"; pv=true; } else body+="; "; }
        else if(pv) { body+=indente(na+1)+t; pv=false; }
        else
	{
          if(  (body.length()>0)
	     &&(  Character.isJavaIdentifierPart(body.charAt(body.length()-1))
		||(body.charAt(body.length()-1)==')'))
	     &&(  Character.isJavaIdentifierStart(t.charAt(0))
		||(t.charAt(0)=='\"'))) body+=" ";
	  body+=t;
          pv=false;
        }
      }
    } while((na>0)||(!t.equals(";")));

    avertissement("instruction '"+body.trim()+"' not reconized");
    trace("Instruction : "+body);
    AglInstructionGenerique ai=new AglInstructionGenerique(body);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionAffectation parseInstructionAffectation
    (AglExpression eg) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("=")) syntaxe("= needed and not "+t);

    AglExpression ed=parseExpression();

    t=get();
    while(t.equals("="))
    {
      ed=new AglExpressionAffectation(ed,parseExpression());
      t=get();
    }
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    trace("Affectation: "+eg+" <-- "+ed);

    AglInstructionAffectation ai=new AglInstructionAffectation(eg,ed);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    AglBloc ab=parseBlocMultiple();
    AglInstructionBloc ai=new AglInstructionBloc(ab);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionBreak parseInstructionBreak() throws Exception
  {
    String t="";
    String l=null;

    t=get();
    if(!t.equals("break")) syntaxe("break needed and not "+t);

    if(!peek().equals(";"))
      l=get();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglInstructionBreak ai=new AglInstructionBreak(l);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("catch")) syntaxe("catch needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglType type=parseType();
    AglExpression ae=
      new AglExpressionDefVar(get(),type);
      //new AglExpressionGenerique(type.nom.dernier()+" "+get());
      
    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    if(AglOptions.definirLiteraux)
      champs_.ajoute(((AglExpressionDefVar)ae).nom,ae);

    AglBloc ab=parseBloc();

    if(AglOptions.definirLiteraux)
      champs_.enleve(((AglExpressionDefVar)ae).nom,ae);

    AglInstructionCatch ai=new AglInstructionCatch(ae,ab);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionContinue parseInstructionContinue() throws Exception
  {
    String t="";
    String l=null;

    t=get();
    if(!t.equals("continue")) syntaxe("continue needed and not "+t);

    if(!peek().equals(";"))
      l=get();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglInstructionContinue ai=new AglInstructionContinue(l);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglVecteur parseInstructionDefVar() throws Exception
  {
    String     t ="";
    int        nv=0;
    AglVecteur r =new AglVecteur();

    AglType type=parseType();

    do 
    {
      t=get();

      if(!t.equals(";"))
      {
        // ancienne forme
        while(peek().equals("[]")) { get(); type=new AglTypeTableau(type); }

        nv++;
        trace("Variable  "+nv+" : "+t+" ["+type+"]");
        AglInstructionDefVar ai=new AglInstructionDefVar(t,type);
        ai.ligne=st_.lineno();

        t=get();
        if(t.equals("="))
        {
          if(peek().equals("{")) ai.initiale=parseExpressionInitArray();
	  else                   ai.initiale=parseExpression();
	  trace("Initiale   : "+ai.initiale);
	  t=get();
        }
        r.addElement(ai);

        if(!(t.equals(",")||t.equals(";")))
	  syntaxe(", or ; needed and not "+t);
      }

    } while(!t.equals(";"));

    return r;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("do")) syntaxe("do needed and not "+t);

    AglBloc ab=parseBloc();

    t=get();
    if(!t.equals("while")) syntaxe("while needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);
    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglInstructionDoWhile ai=new AglInstructionDoWhile(ae,ab);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    trace("InsExpr   : "+eg);

    AglInstructionExpression ai=new AglInstructionExpression(eg);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionFinally parseInstructionFinally() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("finally")) syntaxe("finally needed and not "+t);

    AglBloc ab=parseBloc();

    AglInstructionFinally ai=new AglInstructionFinally(ab);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionFor parseInstructionFor() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("for")) syntaxe("for needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    /*
    AglInstruction ad=(AglInstruction)
      (parseBlocSimple().instructions.elementAt(0));
    if(ad instanceof AglInstructionDefVar)
	champs_.ajoute(((AglInstructionDefVar)ad).nom,ad);
    */

    t=get();
    String u=peek();
    put(t);
    AglBloc    bd  =null;
    AglVecteur vars=null;

    if(AglOptions.definirLiteraux)
      vars=new AglVecteur();

    if(estDeclaration(t,u))
    {
	bd=new AglBloc();
        for(Enumeration e=parseInstructionDefVar().elements();
	    e.hasMoreElements(); )
        {
          AglInstructionDefVar ai=(AglInstructionDefVar) e.nextElement();
          bd.instructions.addElement(ai);
	  if(AglOptions.definirLiteraux)
	  {
	    vars.addElement(ai);
	    champs_.ajoute(ai.nom,ai);
	  }
        }
    }
    else
    //if((chercheType(t)==null)||(u.equals(".")))
    {
	bd=parseBlocVirgule();
	t=get();
	if(!t.equals(";")) syntaxe("; needed and not "+t);
    }

    AglInstruction ad=null;

    switch(bd.instructions.size())
    {
    case 0 : ad=new AglInstructionVide(); break;
    case 1 : ad=(AglInstruction)bd.instructions.elementAt(0); break;
    default: ad=new AglInstructionBloc(bd,true); break;
    }

    AglExpression at=null;
    if(!peek().equals(";")) at=parseExpression();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglInstruction af=null;
    AglBloc        bf=parseBlocVirgule();

    switch(bf.instructions.size())
    {
    case 0 : af=new AglInstructionVide(); break;
    case 1 : af=(AglInstruction)bf.instructions.elementAt(0); break;
    default: af=new AglInstructionBloc(bf,true); break;
    }

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();

    /*
      if(ad instanceof AglInstructionDefVar)
      champs_.enleve(((AglInstructionDefVar)ad).nom,ad);
    */

    if(AglOptions.definirLiteraux)
    {
      Enumeration ev=vars.reverseElements();
      while(ev.hasMoreElements())
      {
        AglInstructionDefVar ai=(AglInstructionDefVar)ev.nextElement();
        champs_.enleve(ai.nom,ai);
      }
    }

    AglInstructionFor ai=new AglInstructionFor(ad,at,af,ab);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("if")) syntaxe("if needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);
    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc aoui=parseBloc();
    AglBloc anon=null;

    t=peek();

    // System.err.println("$$$"+ligne()+"$$$$$$$$$$$$$$ IF="+t);

    if(t.equals("else"))
    {
      t=get();
      anon=parseBloc();
    }
    
    AglInstructionIf ai=new AglInstructionIf(ae,aoui,anon);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    erreur("Il n'y a pas de procedure en Java ("+get()+")");
    return null;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("return")) syntaxe("return needed and not "+t);

    AglExpression ae=null;
    if(!peek().equals(";")) ae=parseExpression();

    AglInstructionReturn ai=new AglInstructionReturn(ae);
    ai.ligne=st_.lineno();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    return ai;
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("switch")) syntaxe("switch needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    AglTableOrdonnee ac  =new AglTableOrdonnee();
    AglBloc          ad  =new AglBloc();
    AglVecteur       vars=null;

    if(AglOptions.definirLiteraux)
      vars=new AglVecteur();

    while(peek().equals("default")||peek().equals("case"))
    {
      t=get();
      if(!t.equals("default")&&!t.equals("case"))
	syntaxe("default or case needed and not "+t);

      AglExpression ce=null;
      if(t.equals("case")) ce=parseExpression();

      t=get();
      if(!t.equals(":")) syntaxe(": needed and not "+t);

      AglBloc cb=new AglBloc();
      t=peek();
      while(  !t.equals("case" )&&!t.equals("default")
	    /*&&!t.equals("break")*/
	    &&!t.equals("}"))
      {
	if(t.equals("{"))
	  cb.instructions.addElement(parseInstructionBloc());
	else
	{
	  AglBloc bi=parseBlocSimple();
	  AglInstruction ai=(AglInstruction)bi.instructions.elementAt(0);
	  cb.instructions.addElement(ai);
	  if(  AglOptions.definirLiteraux
	     &&(ai instanceof AglInstructionDefVar))
	  {
	    // Ajout variable
	    AglInstructionDefVar aidv=(AglInstructionDefVar)ai;
	    vars.addElement(aidv);
	    champs_.ajoute(aidv.nom,aidv);
	  }
	}
	t=peek();
      }

      /*
      if(t.equals("break"))
      {
	get();

	t=get();
	if(!t.equals(";"))
	{
	  // BUG @GDX
	  trace("!!! label for break is ignored.");
	  trace("!!! This bug will be corrected later.");
	  t=get();
	}

	if(!t.equals(";")) syntaxe("; needed and not "+t);
      }
      */

      if(ce!=null) ac.put(ce,cb);
      else         ad=cb;
    }

    // Retrait variable
    if(AglOptions.definirLiteraux)
    {
      Enumeration ev=vars.reverseElements();
      while(ev.hasMoreElements())
      {
        AglInstructionDefVar aidv=(AglInstructionDefVar)ev.nextElement();
	champs_.enleve(aidv.nom,aidv);
      }
    }

    t=get();
    if(!t.equals("}")) syntaxe("} needed and not "+t);

    AglInstructionSwitch ai=new AglInstructionSwitch(ae,ac,ad);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionSynchronized parseInstructionSynchronized()
    throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("synchronized")) syntaxe("synchronized needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();
    AglInstructionSynchronized ai=new AglInstructionSynchronized(ae,ab);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("throw")) syntaxe("throw needed and not "+t);

    AglExpression ae=parseExpression();

    AglInstructionThrow ai=new AglInstructionThrow(ae);
    ai.ligne=st_.lineno();

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    return ai;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("try")) syntaxe("try needed and not "+t);

    AglBloc ab=parseBloc();

    AglInstructionTry ai=new AglInstructionTry(ab);
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionVide parseInstructionVide() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglInstructionVide ai=new AglInstructionVide();
    ai.ligne=st_.lineno();
    return ai;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("while")) syntaxe("while needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();
    AglInstructionWhile ai=new AglInstructionWhile(ae,ab);
    ai.ligne=st_.lineno();
    return ai;
  }
}
