/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseJsp.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;
import java.io.*;

public class AglParseJsp extends AglParse
{
  protected boolean JSERV=
    AglPreferences.AGL.getBooleanProperty("jsp.parse.jserv",false);

  protected String EXTENDS=
    AglPreferences.AGL.getStringProperty("jsp.parse.extends","JspPageImplBase");

  private InputStream in_;
  private int         lineno_;
  private char[]      buf_;
  private int         nbc_;
 
  public AglParseJsp()
  {
    if(AglType.cherche("String")==null)
      (new AglClasse(new AglNom("java.lang.String"))).elementaire=true;
  }

  public String extension()
  {
    return "jsp";
  }

  public String[] extensions()
  {
    return new String[] { "jsp", "JSP" };
  }

  public String syntaxe()
  {
    return "Html";
  }

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    fichier=_fichier;
    in_    =_input;
    lineno_=1;
    buf_   =new char[1024];
    nbc_   =0;
   }

  private char readChar()
  {
    char r;

    if(nbc_>0)
    {
      nbc_--;
      r=buf_[nbc_];
    }
    else
    {
      try { r=(char)in_.read(); }
      catch(IOException ex) { r=(char)-1; }
      if(r=='\n') lineno_++;
    }

    return r;
  }

  private char peekChar()
  {
    char r=readChar();
    if(r!=-1) putChar(r);
    return r;
  }
    
  private void putChar(char _c)
  {
    int l=buf_.length;
    if(nbc_>=l)
    {
      char[] b=new char[l+1024];
      System.arraycopy(buf_,0,b,0,l);
      buf_=b;
    }

    buf_[nbc_]=_c;
    nbc_++;
  }

  private String get0()
  {
    String r="";
    char   c=readChar();

    if(c==(char)-1) return "";
    r+=c;
    char d=peekChar();

    if((c=='<')&&(d=='%'))
    {
      r+=readChar();
      while(((c=readChar())!='%')||(peekChar()!='>'))
	r+=c;
      r+=c;
      r+=readChar();
    }
    else
    if(c=='<')
    {
      while(((c=readChar())!='>')&&(c!=(char)-1)&&(peekChar()!='<'))
	r+=c;
      if(c=='>') r+=c;
    }
    else
    {
      while(((c=readChar())!='<')&&(c!=(char)-1)&&(peekChar()!='%'))
	r+=c;
      if(c=='<') putChar(c);
    }
    
    trace("TOK="+r);
    return r;
  }

  private Stack pile_=new Stack();

  public String get()
  {
    String r="";

    if(!pile_.empty())
    {
      r=(String)pile_.pop();
    }
    else
    {
      String s;
      while(  !(s=get0()).equals("")
	    &&!s.startsWith("<%")
	    &&(JSERV||!s.startsWith("<jsp:"))
	    &&(JSERV||!s.startsWith("</jsp:"))) r+=s;
      put(s);
    }

    System.err.println("FULL TOKEN="+FuLib.replace(r,"\n","\\n"));
    return r;
  }

  public void put(String _s)
  {
    pile_.push(_s);
    //throw new RuntimeException("No put in Jsp");
  }

  private int pp=0;
  public int ligne()
  {
    int r=lineno_;
    if(MaxLines!=0)
    {
      int p=100*r/MaxLines;
      if(p!=pp) progres(p);
      pp=p;
    }
    return r;
  }

  private String fichier="";
  public String fichier()
  {
    int i=fichier.lastIndexOf(File.separator);
    return fichier.substring(i+1);
  }

  // Parsing

  public void parseFichier() throws Exception
  {
    String  extends_     =EXTENDS;
    String  import_      ="";
    boolean session_     =true;
    int     buffer_      =8192;
    boolean autoflush_   =true;
    boolean isthreadsafe_=true;
    String  info_        ="";
    String  errorpage_   ="errorpage.jsp";
    boolean iserrorpage_ =false;
    String  contenttype_ ="text/html";
    String  name_        ="_"+fichier().replace('.','_');

    if(JSERV)
    {
      extends_="HttpServlet";
      name_   =fichier().substring(0,fichier().length()-4);
    }

    import_+="import javax.servlet.*;\n";
    import_+="import javax.servlet.http.*;\n";
    import_+="import java.io.*;\n";
    import_+="import java.util.*;\n";

    String a=""; // declaration
    String b=""; // scriptlet
    String r=""; // result

    String s;
    while(!(s=get()).equals(""))
    {
      if(s.startsWith("<%!"))
	  a+="  "+s.substring(3,s.length()-2).trim()+"\n";
      else
      if(s.startsWith("<%="))
	  b+="      out.print("+FuLib.replace
	      (s.substring(3,s.length()-2),"\n","\\n").trim()+");\n";
      else
      if(s.startsWith("<%@"))
      {
	  s=s.substring(3,s.length()-2).trim();
	  if(s.startsWith("page"))
	  {
	    s=s.substring(4).trim();
	    while(!"".equals(s))
	    {
	      int i=s.indexOf("=\"");
	      int j=s.indexOf('"',i+2);
	      String k=s.substring(0  ,i).trim();
	      String v=s.substring(i+2,j).trim();
	      //v=v.substring(1,v.length()-1);
	      s=s.substring(j+1).trim();
	      suivi("Page        : "+k+" = "+v);

	      if("extends"     .equals(k)) extends_     =v;
	      else
	      if("info"        .equals(k)) info_        =v;
	      else
	      if("errorPage"   .equals(k)) errorpage_   =v;
	      else
	      if("contentType" .equals(k)) contenttype_ =v;
	      else
	      if("session"     .equals(k)) session_     =("true".equals(v));
	      else
	      if("autoFlush"   .equals(k)) autoflush_   =("true".equals(v));
	      else
	      if("isThreadSafe".equals(k)) isthreadsafe_=("true".equals(v));
	      else
	      if("isErrorPage" .equals(k)) iserrorpage_ =("true".equals(v));
	      else
	      if("buffer".equals(k))
	      {
		try
		{
		  if("none".equals(v)) buffer_=0;
		  else buffer_=1024*Integer.parseInt(v);
		}
		catch(Exception ex) { buffer_=8192; }
	      }
	      else
	      if("import".equals(k))
	      {
	        int p;
	        while((p=v.indexOf(','))>=0)
	        {
		  import_+="import "+v.substring(0,p)+";\n";
		  v=v.substring(p+1).trim();
		} 
	      }
	      else
	        // ajouter les autres directives
	        avertissement("attribute "+k+" not yet implemented:"+
			      v+" ignored");
	    }
	  }
	  else
	  if(s.startsWith("include"))
	  {
	    // lire le fichier et l'inclure dans le flux
	    //avertissement("include directive not yet implemented");
	    s=s.substring(7).trim();
	    int i=s.indexOf('=');
	    String k=s.substring(0,i).trim();
	    String v=s.substring(i+1).trim();
	    v=v.substring(1,v.length()-1); // enleve les guillemets

	    suivi("Include     : "+v);
	    try
	    {
	      File   f=new File(v);
	      int    t=(int)f.length();
	      char[] c=new char[t];
	      InputStream in=new FileInputStream(f);
	      for(int j=0;j<t;j++)
		  c[j]=(char)in.read();
	      for(int j=t-1;j>=0;j--)
		  putChar(c[j]);
	      in.close();
	    }
	    catch(IOException ex)
	      { erreur("problem with "+v); }
	  }
	  else
	  if(s.startsWith("taglib"))
	  {
	    avertissement("taglib directive not implemented");
	  }
	  else
	    erreur("unknown directive:"+s);
      }
      else
      if(s.startsWith("<%"))
      {
	b+="      "+s.substring(2,s.length()-2).trim()+"\n";
      }
      else
      if(!JSERV&&s.startsWith("<jsp:include"))
      {
	String page="";
	boolean flush=true;

	s=s.substring(12,s.length()-1).trim();
	while(!"".equals(s)&&!"/".equals(s))
	{
	  int i=s.indexOf("=\"");
	  int j=s.indexOf('"',i+2);
	  String k=s.substring(0  ,i).trim();
	  String v=s.substring(i+2,j).trim();
	  //v=v.substring(1,v.length()-1);
	  s=s.substring(j+1).trim();
	  suivi("jsp:include : "+k+" = "+v);
	       if("page".equals(k)) page=v;
	  else if("flush".equals(k)) flush=("true".equals(b));
	  else avertissement("attribute "+k+" not yet implemented:"+
			      v+" ignored");
	}

	boolean params=false;
	if(!s.equals("/"))
	{
	  String name="",value="";

	  while(!(s=get().trim()).equals("</jsp:include>"))
	  {
	    if(s.equals("")) continue;
	    if(!s.startsWith("<param")) syntaxe("<param> needed and not "+s);

	    s=s.substring(6,s.length()-2).trim();
	    while(!"".equals(s))
	    {
	      int i=s.indexOf("=\"");
	      int j=s.indexOf('"',i+2);
	      String k=s.substring(0  ,i).trim();
	      String v=s.substring(i+2,j).trim();
	      //v=v.substring(1,v.length()-1);
	      s=s.substring(j+1).trim();
	      suivi("jsp:param : "+k+" = "+v);
	           if("name".equals(k)) name=v;
	      else if("value".equals(k)) value=v;
	      else avertissement("attribute "+k+" not yet implemented:"+
			      v+" ignored");
	    }

	    if(params) page+="&";
	    else       page+="?";
	    page+=name;
	    page+="=";
	    page+=value;
	    params=true;
	  }    
	}

	b+="      pageContext.include(\""+page+"\");\n";
      }
      else
      if(!JSERV&&s.startsWith("<jsp:forward"))
      {
	String page="";

	s=s.substring(12,s.length()-1).trim();
	while(!"".equals(s)&&!"/".equals(s))
	{
	  int i=s.indexOf("=\"");
	  int j=s.indexOf('"',i+2);
	  String k=s.substring(0  ,i).trim();
	  String v=s.substring(i+2,j).trim();
	  //v=v.substring(1,v.length()-1);
	  s=s.substring(j+1).trim();
	  suivi("jsp:forward : "+k+" = "+v);
	       if("page".equals(k)) page=v;
	  else avertissement("attribute "+k+" not yet implemented:"+
			      v+" ignored");
	}

	boolean params=false;
	if(!s.equals("/"))
	{
	  String name="",value="";

	  while(!(s=get().trim()).equals("</jsp:forward>"))
	  {
	    if(s.equals("")) continue;
	    if(!s.startsWith("<param")) syntaxe("<param> needed and not "+s);

	    s=s.substring(6,s.length()-2).trim();
	    while(!"".equals(s))
	    {
	      int i=s.indexOf("=\"");
	      int j=s.indexOf('"',i+2);
	      String k=s.substring(0  ,i).trim();
	      String v=s.substring(i+2,j).trim();
	      //v=v.substring(1,v.length()-1);
	      s=s.substring(j+1).trim();
	      suivi("jsp:param : "+k+" = "+v);
	           if("name".equals(k)) name=v;
	      else if("value".equals(k)) value=v;
	      else avertissement("attribute "+k+" not yet implemented:"+
			      v+" ignored");
	    }

	    if(params) page+="&";
	    else       page+="?";
	    page+=name;
	    page+="=";
	    page+=value;
	    params=true;
	  }    
	}

	b+="      pageContext.forward(\""+page+"\");\n";
      }
      else
      if(!JSERV&&s.startsWith("<jsp:useBean"))
      {
	String beanId   ="";
	String beanScope="";
	String beanClass="";
	String beanName ="";
	String beanType ="";

	s=s.substring(12,s.length()-1).trim();
	while(!"".equals(s)&&!"/".equals(s))
	{
	  int i=s.indexOf("=\"");
	  int j=s.indexOf('"',i+2);
	  String k=s.substring(0  ,i).trim();
	  String v=s.substring(i+2,j).trim();
	  //v=v.substring(1,v.length()-1);
	  s=s.substring(j+1).trim();
	  suivi("jsp:useBean : "+k+" = "+v);
	       if("id"      .equals(k)) beanId   =v;
	  else if("scope"   .equals(k)) beanScope=v;
	  else if("class"   .equals(k)) beanClass=v;
	  else if("beanName".equals(k)) beanName =v;
	  else if("type"    .equals(k)) beanType =v;
	  else avertissement("attribute "+k+" not yet implemented:"+
			      v+" ignored");
	}

	if(!s.equals("/"))
	{
	  while(!(s=get().trim()).equals("</jsp:useBean>"))
	  {
	    if(s.equals("")) continue;
	    syntaxe("</jsp:useBean> needed and not "+s);
	  }    
	}

	b+="      out.print(\"<!-- jsp:useBean not yet implemented : "
	   +beanId+" -->\\n\");\n";
      }
      else
      if(!JSERV&&s.startsWith("<jsp:setProperty"))
      {
	String name    ="";
	String property="";
	String param   ="";
	String value   ="";

	s=s.substring(16,s.length()-1).trim();
	while(!"".equals(s)&&!"/".equals(s))
	{
	  int i=s.indexOf("=\"");
	  int j=s.indexOf('"',i+2);
	  String k=s.substring(0  ,i).trim();
	  String v=s.substring(i+2,j).trim();
	  //v=v.substring(1,v.length()-1);
	  s=s.substring(j+1).trim();
	  suivi("jsp:setProperty : "+k+" = "+v);
	       if("name"    .equals(k)) name    =v;
	  else if("property".equals(k)) property=v;
	  else if("param"   .equals(k)) param   =v;
	  else if("value"   .equals(k)) value   =v;
	  else avertissement("attribute "+k+" not yet implemented:"+
			      v+" ignored");
	}

	if(!s.equals("/"))
	{
	  while(!(s=get().trim()).equals("</jsp:setProperty>"))
	  {
	    if(s.equals("")) continue;
	    syntaxe("</jsp:setProperty> needed and not "+s);
	  }    
	}

	b+="      out.print(\"<!-- jsp:setProperty not yet implemented : "
	   +name+"."+property+" -->\\n\");\n";
      }
      else
      if(!JSERV&&s.startsWith("<jsp:getProperty"))
      {
	String name    ="";
	String property="";

	s=s.substring(16,s.length()-1).trim();
	while(!"".equals(s)&&!"/".equals(s))
	{
	  int i=s.indexOf("=\"");
	  int j=s.indexOf('"',i+2);
	  String k=s.substring(0  ,i).trim();
	  String v=s.substring(i+2,j).trim();
	  //v=v.substring(1,v.length()-1);
	  s=s.substring(j+1).trim();
	  suivi("jsp:setProperty : "+k+" = "+v);
	       if("name"    .equals(k)) name    =v;
	  else if("property".equals(k)) property=v;
	  else avertissement("attribute "+k+" not yet implemented:"+
			      v+" ignored");
	}

	if(!s.equals("/"))
	{
	  while(!(s=get().trim()).equals("</jsp:getProperty>"))
	  {
	    if(s.equals("")) continue;
	    syntaxe("</jsp:getProperty> needed and not "+s);
	  }    
	}

	b+="      out.print(\"<!-- jsp:getProperty not yet implemented : "
	   +name+"."+property+" -->\\n\");\n";
      }
      else
      if(!JSERV&&s.startsWith("<jsp:"))
      {
	int i;
	String t=s;
	i=t.indexOf(" ");
	if(i<0) i=t.length()-1;
	t=t.substring(1,i);
	i=t.indexOf("/");
	if(i>=0) t=t.substring(0,i);
	b+="      out.print(\"<!-- Unknown tag : "+t+" -->\\n\");\n";
	if(i>=0)
	{
	  String endtag="</"+t+">";
	  while(!(s=get()).equals(endtag)&&!s.equals("")) ;
	}
      }
      else
      {
	s=FuLib.replace(s,"\n","\\n");
	s=FuLib.replace(s,"\"","\\\"");
	b+="      out.print(\""+s.trim()+"\");\n";
      }
    }

    r+=import_;
    r+="\n";
    r+="public class "+name_+"\n";
    r+="       extends "+extends_+"\n";
    r+="{\n";
    r+=a;
    r+="\n";

    if(!JSERV)
    {
      r+="  public void _jspService\n";
      r+="    (HttpServletRequest request, HttpServletResponse response)\n";
      r+="    throws IOException, ServletException\n";
      r+="  {\n";
      r+="    JspFactory     _jspxFactory=null;\n";
      r+="    PageContext    pageContext =null;\n";
      r+="    HttpSession    session     =null;\n";
      r+="    ServletContext application =null;\n";
      r+="    ServletConfig  config      =null;\n";
      r+="    JspWriter      out         =null;\n";
      r+="    Object         page        =this;\n";
      r+="    String         _value      =null;\n";
      r+="    try {\n";
      r+="      if(_jspx_inited==false) {\n";
      r+="        _jspx_init();\n";
      r+="        _jspx_inited=true;\n";
      r+="      }\n";
      r+="      _jspxFactory=JspFactory.getDefaultFactory();\n";
      r+="      response.setContentType(\""+contenttype_+"\");\n";
      r+="      pageContext=_jspxFactory.getPageContext\n";
      r+="        (this,request,response,\""+errorpage_+"\","+
	session_+","+buffer_+","+autoflush_+");\n";
      r+="      application=pageContext.getServletContext();\n";
      r+="      config     =pageContext.getServletConfig();\n";
      r+="      session    =pageContext.getSession();\n";
      r+="      out        =pageContext.getOut();\n";
    }
    else
    {
      r+="  public void service\n";
      r+="    (HttpServletRequest request, HttpServletResponse response)\n";
      r+="    throws IOException, ServletException\n";
      r+="  {\n";
      r+="    response.setContentType(\""+contenttype_+"\");\n";
      r+="    PrintWriter out=response.getWriter();\n";
    }

    r+="\n";
    r+=b;

    if(!JSERV)
    {
      r+="    }\n";
      r+="    catch(Exception ex) {\n";
      r+="      if(out.getBufferSize()!=0)\n";
      r+="        out.clear();\n";
      r+="      pageContext.handlePageException(ex);\n";
      r+="    }\n";
      r+="    finally {\n";
      r+="      out.flush();\n";
      r+="      _jspxFactory.releasePageContext(pageContext);\n";
      r+="    }\n";
    }

    r+="  }\n";
    r+="}\n";

    // TRACE TMP CODE: System.err.println(r);

    AglParseJava p=new AglParseJava();
    p.initStream(new ByteArrayInputStream(r.getBytes()),fichier);
    p.parseFichier();
  }

  public AglUnite parseUnite() throws Exception
  {
    return null;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    return null;
  }

  public AglNom parseNom() throws Exception
  {
    return null;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    return null;
  }

  public AglType parseType() throws Exception
  {
    return null;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    return null;
  }
  
  public AglImport parseImport() throws Exception
  {
    return null;
  }

  public AglTypeAlias parseTypeDef() throws Exception
  {
    return null;
  }

  public AglStruct parseStruct() throws Exception
  {
    return null;
  }

  public AglClasse parseClasse() throws Exception
  {
    return null;
  }

  public AglListeNoms parseMothers() throws Exception
  {
    return null;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    return null;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    return null;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    return null;
  }

  public AglMethode parseMethode() throws Exception
  {
    return null;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    return null;
  }

  public AglVecteur parseParametres() throws Exception
  {
    return null;
  }

  public AglVecteur parseArgs() throws Exception
  {
    return null;
  }

  public AglBloc parseBloc() throws Exception
  {
    erreur("Il n'y a pas de bloc en Jsp ("+get()+")");
    return null;
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    erreur("Il n'y a pas de bloc-multiple en Jsp ("+get()+")");
    return null;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    erreur("Il n'y a pas de bloc-simple en Jsp ("+get()+")");
    return null;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    erreur("Il n'y a pas de bloc-virgule en Jsp ("+get()+")");
    return null;
  }

// Expressions

  public AglExpression parseExpression() throws Exception
  {
    erreur("Il n'y a pas d'expression en Jsp ("+get()+")");
    return null;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    erreur("Il n'y a pas d'expression generique en Jsp ("+get()+")");
    return null;
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite() throws Exception
  {
    erreur("Il n'y a pas d'expression unaire droite en Jsp ("+get()+")");
    return null;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    erreur("Il n'y a pas d'expression binaire en Jsp ("+get()+")");
    return null;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    erreur("Il n'y a pas d'expression de fonction en Jsp ("+get()+")");
    return null;
  }

  public AglExpression parseExpressionNew() throws Exception
  {
    erreur("Il n'y a pas d'expression new en Jsp ("+get()+")");
    return null;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    erreur("Il n'y a pas d'expression parenthse en Jsp ("+get()+")");
    return null;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    erreur("Il n'y a pas d'expression de typage en Jsp ("+get()+")");
    return null;
  }

// Instructions

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    erreur("Il n'y a pas d'instruction gnrique en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionAffectation parseInstructionAffectation(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction d'affectation en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    erreur("Il n'y a pas d'instruction de bloc en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    erreur("Il n'y a pas d'instruction catch en Jsp ("+get()+")");
    return null;
  }

  public AglVecteur parseInstructionDefVar() throws Exception
  {
    erreur("Il n'y a pas d'instruction de dfinition de variable en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction do/while en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction expressive en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    erreur("Il n'y a pas d'instruction if en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    erreur("Il n'y a pas de procedure en Java ("+get()+")");
    return null;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    erreur("Il n'y a pas d'instruction return en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    erreur("Il n'y a pas d'instruction switch en Jsp ("+get()+")");
    return null;
  }  

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    erreur("Il n'y a pas d'instruction throw en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    erreur("Il n'y a pas d'instruction try en Jsp ("+get()+")");
    return null;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction while en Jsp ("+get()+")");
    return null;
  }
}
