/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseMdl.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;
import java.io.*;

public class AglParseMdl extends AglParse
{
  FuStreamTokenizer st;
  Stack              pile;

  AglUnite _unite;
  AglNom   _nom;
  AglType  _type;

//  AglPaquet  paquet;
//  AglClasse     classe;
//  AglMethode    methode;
//  AglAttribut   attribut;

  public AglParseMdl()
  {
    // considere Mdl long as Java int
    new AglTypeAlias("long",AglTypePrimitif.tinteger,"Mdl");

    if(AglType.cherche("String")==null)
      (new AglClasse(new AglNom("java.lang.String"))).elementaire=true;

    if(AglType.cherche("string")==null)
      new AglTypeAlias("string",AglType.cherche("String"),"Mdl");
    if(AglType.cherche("wstring")==null)
      new AglTypeAlias("wstring",AglType.cherche("String"),"Mdl");
  }

  public String extension()
  {
    return "mdl";
  }

  public String[] extensions()
  {
    return new String[] { "mdl", "MDL", "ptl", "PTL" };
  }

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    st=new FuStreamTokenizer(_input,_fichier);
    st.resetSyntax();
    st.eolIsSignificant(false);
    st.quoteChar('"');
    st.quoteChar('\'');
    st.commentChar('|');
    st.slashSlashComments(true);
    st.slashStarComments(true);
    st.wordChars('a','z');
    st.wordChars('A','Z');
    st.wordChars('0','9');
    st.wordChars('_','_');
    st.wordChars('@','@');
    st.whitespaceChars(' ',' ');
    st.whitespaceChars('\t','\t');
    st.whitespaceChars('\n','\n');
    st.whitespaceChars('\r','\r');

    pile=new Stack();
  }

  public String get()
  {
    String r="";

    if(pile.empty())
    {
      try
      {
        st.nextToken();

        switch(st.ttype)
        {
          case StreamTokenizer.TT_EOF   : r="";                break;
          case StreamTokenizer.TT_WORD  : r=st.sval;           break;
          case StreamTokenizer.TT_NUMBER: r=""+st.nval;        break;
          case '"'                      : r="\""+ascii2C(st.sval)+"\""; break;
          case '\''                     : r="'" +ascii2C(st.sval)+"'" ; break;
          default                       : r=""+(char)st.ttype; break;
        }
      }
      catch(Exception e) { erreur(""+e+" [AglParse.get]"); }

      if((!r.equals(""))&&Character.isDigit(r.charAt(0)))
      {
        if(peek().equals("."))
        {
          r+=get();
          if(Character.isDigit(peek().charAt(0)))
          {
            r+=get();
            if((peek().equals("E"))||(peek().equals("e")))
            {
	      r+=get();
              if((peek().equals("+"))||(peek().equals("-")))
                r+=get();
              r+=get();
            }
          }
        }
      }

      if((r.equals(":" ))&&(peek().equals(":"))) r+=get();
      if((r.equals("}" ))&&(peek().equals(";"))) r+=get();

      if((r.equals("[" ))&&(peek().equals("]"))) r+=get();

      if((r.equals("=" ))&&(peek().equals("="))) r+=get();
      if((r.equals("!" ))&&(peek().equals("="))) r+=get();

      if((r.equals("+" ))&&(peek().equals("="))) r+=get();
      if((r.equals("-" ))&&(peek().equals("="))) r+=get();
      if((r.equals("*" ))&&(peek().equals("="))) r+=get();
      if((r.equals("/" ))&&(peek().equals("="))) r+=get();
      if((r.equals("%" ))&&(peek().equals("="))) r+=get();

      if((r.equals("+" ))&&(peek().equals("+"))) r+=get();
      if((r.equals("-" ))&&(peek().equals("-"))) r+=get();

      if((r.equals(">" ))&&(peek().equals(">"))) r+=get();
      if((r.equals("<" ))&&(peek().equals("<"))) r+=get();
      if((r.equals(">" ))&&(peek().equals("="))) r+=get();
      if((r.equals("<" ))&&(peek().equals("="))) r+=get();
      if((r.equals(">>"))&&(peek().equals(">"))) r+=get();
      if((r.equals("<<"))&&(peek().equals("<"))) r+=get();

      if((r.equals("&" ))&&(peek().equals("&"))) r+=get();
      if((r.equals("|" ))&&(peek().equals("|"))) r+=get();

      if((r.equals("&" ))&&(peek().equals("="))) r+=get();
      if((r.equals("|" ))&&(peek().equals("="))) r+=get();
      if((r.equals("^" ))&&(peek().equals("="))) r+=get();

      trace(">>> "+r);
    }
    else r=(String)pile.pop();

    return r;
  }

  public void put(String ih)
  {
    pile.push(ih);
  }

  private int pp=0;
  public int ligne()
  {
    int r=st.lineno();
    if(MaxLines!=0)
    {
      int p=100*r/MaxLines;
      if(p!=pp) progres(p);
      pp=p;
    }
    return r;
  }

  public String fichier()
  {
    String r=st.origin();
    int i=r.lastIndexOf(File.separator);
    if(i>=0) r=r.substring(i+1);
    return r;
  }

  // Parse

  public void parseFichier() throws Exception
  {
    parseUnite();
  }

  public AglUnite parseUnite() throws Exception
  {
    AglUnite au=new AglUnite(st.origin());
    au.langage="Mdl";

    _unite=au;
    String t="",u="";

    do
    {
      while(peek().equals(")")) get();

      if(peek().equals("")) break;

      t=get(); u=peek(); 
      trace("WU>"+t+"   PEEK="+u);
      put(t);

      if(!t.equals("(")) syntaxe("( needed and not "+t);
      
      if(u.equals("object"))
	parseObject();
      else
      if(u.equals("list"))
	parseList();
      else
      if(!u.equals("")) syntaxe("unknown keyword: "+u);

    } while(!t.equals(""));

    return au;
  }

  private AglNom getAglNom(String v)
  {
    AglNom nom=new AglNom();

    v=v.substring(v.indexOf("::")+2,v.length()-1);

    while(!v.equals(""))
    {
      int i=v.indexOf("::");
      if(i>=0)
      {
	nom.identificateurs.addElement(v.substring(0,i));
	v=v.substring(i+2);
      }
      else
      {
	nom.identificateurs.addElement(v);
	v="";
      }
    }

    return nom;
  }

  public Hashtable parseObject()
  {
    String    t    ="";
    String    type ="";
    String    name ="";
    Hashtable table=new Hashtable();

    t=get();
    if(!t.equals("("     )) syntaxe("( needed and not "+t);

    t=get();
    if(!t.equals("object")) syntaxe("object needed and not "+t);

    t=get();
    trace("WO>"+t);
    type=t;

    String u=peek();
    if(u.startsWith("\""))
      { name=get(); table.put("__name",name); }

    while(peek().startsWith("\"")) get();
    while(peek().startsWith("@" )) get();

    while(!peek().equals(")"))
    {
      t=get();
      u=get();

      trace("T="+t);

      if(!u.equals("("))
      {
	table.put(t,u);
	trace("U="+u);
      }
      else
      {
	u=peek();
	put("(");
	trace("U="+u);

	Object o="";

	if(u.equals("object")) o=parseObject();
	else
	if(u.equals("list"  )) o=parseList();
	else
	{
	  String r="";
	  // System.err.println("ROSE: unknown key "+u);
	  while(!peek().equals(")"))
	    { r+=get(); }
	  r+=get();
	  // System.err.println("      "+r);
	  o=r;
	}

	table.put(t,o);
      }
    }

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    if(type.equals("Class"))
    {
      name=name.substring(1,name.length()-1);

      AglVisibilite visibilite=new AglVisibilite();
      AglNom        nom=new AglNom(name);

      Object o;

      o=table.get("module");
      if(o instanceof String) nom=getAglNom((String)o);

      AglClasse c=new AglClasse(nom);
      c.ligne      =st.lineno();
      c.elementaire=false;
      c.visibilite =visibilite;
      // System.err.println("CLASSE="+c.nom);

      o=table.get("superclasses");
      if(o!=null)
      {
	AglVecteur vv=(AglVecteur)o;
	for(Enumeration e=((AglVecteur)o).elements(); e.hasMoreElements(); )
	{
	  Object h=e.nextElement();
	  if(h instanceof Hashtable)
	  {
	    String i=(String)((Hashtable)h).get("supplier");
	    // System.err.println("mere "+i);
	    c.meres.noms.addElement(getAglNom(i));
	  }
	}
      }

      o=table.get("realized_interfaces");
      if(o!=null)
      {
	AglVecteur vv=(AglVecteur)o;
	for(Enumeration e=((AglVecteur)o).elements(); e.hasMoreElements(); )
	{
	  Object h=e.nextElement();
	  if(h instanceof Hashtable)
	  {
	    String i=(String)((Hashtable)h).get("supplier");
	    // System.err.println("interface "+i);
	    c.interfaces.noms.addElement(getAglNom(i));
	  }
	}
      }

      o=table.get("abstract");
      if("TRUE".equals(o)) visibilite.vabstract=true;

      o=table.get("stereotype");
      if("Interface".equals(o)) c.normale=false;

      o=table.get("exportControl");
      if(o instanceof String)
      {
	String v=(String)o;
	v=v.substring(1,v.length()-1);
	if("Public"        .equals(v)) visibilite.vpublic   =true;
	if("Implementation".equals(v)) visibilite.vprotected=true;
      }      

      o=table.get("class_attributes");
      if(o!=null)
      {
	AglVecteur vv=(AglVecteur)o;
	for(Enumeration e=((AglVecteur)o).elements(); e.hasMoreElements(); )
	{
	  Object h=e.nextElement();
	  if(h instanceof Hashtable)
	  {
	    String an=(String)((Hashtable)h).get("__name");
	    String at=(String)((Hashtable)h).get("type");

	    // System.err.println("AN="+an);
	    // System.err.println("AT="+at);
	    an=an.substring(1,an.length()-1);
	    at=at.substring(1,at.length()-1);
	    System.err.println("attribute "+an+": "+at);

	    AglType ac=AglType.cherche(at);
	    if(ac==null) ac=new AglClasse(new AglNom(at));
	    AglAttribut aa=new AglAttribut(an,ac);
	    c.attributs.put(aa.signature(),aa);
	  }
	}
      }
    }

    return table;
  }

  public AglVecteur parseList()
  {
    String t="";
    AglVecteur r=new AglVecteur();

    t=get();
    if(!t.equals("("     )) syntaxe("( needed and not "+t);

    t=get();
    if(!t.equals("list")) syntaxe("list needed and not "+t);

    t=get();
    while(!t.equals(")"))
    {
      String u=peek();
      put(t);

      Object o="";

           if(u.equals("object")) o=parseObject();
      else if(u.equals("list"  )) o=parseList();
      else if(peek().equals("("))
      {
	String l="";
	// System.err.println("ROSE: unknown key "+u);
	while(!peek().equals(")"))
	  { l+=get(); }
	l+=get();
	// System.err.println("      "+r);
	o=l;
      }
      else { o=get(); }

      r.addElement(o);
      t=get();
    }

    return r;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    AglVisibilite av=new AglVisibilite();
    String        t ="";
    boolean       e =false;

    av.vpublic=true;
    do
    {
      e=false;
      t=get();
      trace("WV>"+t);
           if(t.equals("const"))    e=av.vfinal=av.vstatic=true;
      else if(t.equals("readonly")) e=av.vreadonly=true;
      else if(t.equals("oneway"  )) e=av.voneway=true;
    } while(e);

    put(t);
    return av;
  }

  public AglNom parseNom() throws Exception
  {
    AglNom an=new AglNom();

    String t="";

    t=peek();
    if(t.equals("::")) get();

    do
    {
      trace("WN>"+peek());
      an.identificateurs.addElement(get());
      t=peek();
      if(t.equals("::")) get();
    }
    while(t.equals("::"));

    an.ligne=st.lineno();
    return an;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    AglListeNoms al=new AglListeNoms();

    String t="";
    do
    {
      trace("WLN>"+peek());
      al.noms.addElement(parseNom());
      t=peek();
      if(t.equals(",")) get();
    }
    while(t.equals(","));

    al.ligne=st.lineno();
    return al;
  }

  public AglType parseType() throws Exception
  {
    boolean tableau=false;
    String t="";

    if(peek().equals("sequence"))
      {
	tableau=true;
	get();
	t=get();
	if(!t.equals("<")) syntaxe("< needed and not "+t);
      }

    trace("WT>"+peek());
    AglNom base=parseNom();
    AglType r=AglType.cherche(base);
    if(r==null) r=new AglClasse(base);

    if(tableau)
      {
	r=new AglTypeTableau(r);
	t=get();
	if(!t.equals(">")) syntaxe("> needed and not "+t);
      }

    // while(peek().equals("[]")) { trace("WT>"+peek()); r=new AglTypeTableau(r); get(); }
    return r;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("module")) syntaxe("module needed and not "+t);

    AglNom np=parseNom();
    trace("Module      : "+np);

    // t=get();
    // if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglPaquet ap=new AglPaquet(np);
    ap.ligne=st.lineno();
    return ap;
  }
  
  public AglImport parseImport() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("include")) syntaxe("include needed and not "+t);

    AglNom ni=parseNom();
    trace("#include    : "+ni);

    // t=get();
    // if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglImport ai=new AglImport(ni);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglTypeAlias parseTypeDef() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("typedef")) syntaxe("typedef needed and not "+t);

    AglType at=parseType();
    AglNom  an=parseNom();
    AglTypeAlias al=new AglTypeAlias(an.dernier(),at); // "Mdl"

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);
    return al;
  }

  public AglClasse parseEnum() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("enum")) syntaxe("enum needed and not "+t);

    trace("WENUM>"+t);
    AglNom nc=parseNom();
    if(_unite.paquet!=null) nc=new AglNom(_unite.paquet.nom,nc);
    trace("Enum        : "+nc);

    AglClasse ac=(AglClasse)AglType.cherche(nc);
    if(ac==null) ac=new AglClasse(nc);
    ac.ligne=st.lineno();
    // ac.elementaire=false;

    if(peek().equals(";"))
      {
	get();
	return ac;
      }
    
    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    // Body

    do
    {
      t=peek();
      trace("WBE>"+t);

      if(!t.equals("};"))
      {
	// _type=AglTypePrimitif.tinteger;
	_nom =parseNom();

	// AglAttribut aa=new AglAttribut(_nom.dernier(),_type);
	// aa.ligne=st.lineno();
	// aa.visibilite=new AglVisibilite();
	// aa.visibilite.vpublic=true;
	// aa.visibilite.vstatic=true;
	// aa.visibilite.vfinal=true;
	// ac.attributs.addElement(aa);

	if(peek().equals(",")) get();
      }
    } while(!peek().equals("};"));

    AglAttribut aa=new AglAttribut("value",AglTypePrimitif.tinteger);
    aa.ligne=st.lineno();
    aa.visibilite=new AglVisibilite();
    aa.visibilite.vpublic=true;
    ac.attributs.put(aa.signature(),aa);

    get();
    return ac;
  }

  public AglStruct parseStruct() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("struct")) syntaxe("struct needed and not "+t);

    trace("WS>"+t);
    AglNom nc=parseNom();
    if(_unite.paquet!=null) nc=new AglNom(_unite.paquet.nom,nc);
    trace("Struct      : "+nc);

    AglStruct ac=(AglStruct)AglType.cherche(nc);
    if(ac==null) ac=new AglStruct(nc);
    ac.ligne=st.lineno();
    // ac.elementaire=false;

    if(peek().equals(";"))
      {
	get();
	return ac;
      }
    
    t=get();
    if(!t.equals("{")) syntaxe("{ needed and not "+t);

    // Body

    do
    {
      t=peek();
      trace("WBS>"+t);

      if(!t.equals("};"))
      {
	_type=parseType();
	_nom =parseNom();

	AglVecteur ava=parseAttribut();
	for(Enumeration e=ava.elements(); e.hasMoreElements(); )
	{
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  aa.visibilite=new AglVisibilite();
	  aa.visibilite.vpublic=true;
	  ac.attributs.put(aa.signature(),aa);
        }
      }
    } while(!peek().equals("};"));

    get();
    return ac;
  }

  public AglClasse parseClasse() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("interface")) syntaxe("interface needed and not "+t);

    trace("WC>"+t);
    AglNom nc=parseNom();
    if(_unite.paquet!=null) nc=new AglNom(_unite.paquet.nom,nc);
    trace("Classe      : "+nc);

    AglClasse ac=(AglClasse)AglType.cherche(nc);
    if(ac==null) ac=new AglClasse(nc);
    ac.proprietaire=_unite;
    ac.ligne       =st.lineno();
    ac.elementaire =false;

    if(peek().equals(";"))
      {
	get();
	return ac;
      }
    
    do
    {
      t=get();
      trace("WCB>"+t);
           if(t.equals(":"))    ac.meres     =parseMothers();
	   // else if(t.equals("implements")) ac.interfaces=parseInterfaces();
      else if(t.equals("raises"))     ac.exceptions=parseExceptions();
    } while(!t.equals("{"));

  // Body

    do
    {
      if(peek().equals("};")) break;

      if(peek().equals("typedef"))
      {
	parseTypeDef();
      }

      AglVisibilite visibilite=parseVisibilite();

      trace("WBC>"+peek());

      t=get();

      if(t.equals(ac.nom.dernier())&&peek().equals("(")&&!visibilite.vstatic)
      {
        put(t);

        AglConstructeur am=parseConstructeur();
        am.visibilite=visibilite;
        ac.constructeurs.put(am.signature(),am);
      }
      else if(t.equals("attribute"))
      {
        _type=parseType();
        _nom =parseNom();

	AglVecteur ava=parseAttribut();
	for(Enumeration e=ava.elements(); e.hasMoreElements(); )
        {
	  AglAttribut aa=(AglAttribut)e.nextElement();
	  aa.visibilite=visibilite;
	  ac.attributs.put(aa.signature(),aa);
        }
      }
      else if(!t.equals("};"))
      {
        put(t);

        _type=parseType();
        _nom =parseNom();
        String v=peek();

        if(!v.equals("(")) syntaxe("( needed and not "+t);

	AglMethode am=parseMethode();
	am.visibilite=visibilite;
	ac.methodes.put(am.signature(),am);
      }

    } while(!peek().equals("};"));

    get();
    return ac;
  }

  public AglListeNoms parseMothers() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    trace("Mere        : "+ln);
    return ln;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    trace("Interfaces  : "+ln);
    return ln;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    AglListeNoms ln=parseListeNoms();
    trace("Exceptions  : "+ln);
    return ln;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    erreur("Il n'y a pas de constructeurs en Mdl ("+get()+")");
    return null;
  }

  public AglMethode parseMethode() throws Exception
  {
    trace("Methode     : "+_nom+" ["+_type+"]");

    AglMethode m=new AglMethode(_nom.dernier(),_type);
    m.ligne=st.lineno();

    m.parametres=parseParametres();
    if(peek().equals("raises")) { get(); m.exceptions=parseExceptions(); }

    String t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    // m.corps=parseBlocMultiple();

    return m;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    AglVecteur r=new AglVecteur();
    String t="";

    // t=get();
    // if(!t.equals("attribute")) syntaxe("attribute needed and not "+t);

    do
    {
      trace("Attribut    : "+_nom+" ["+_type+"]");

      AglAttribut a=new AglAttribut(_nom.dernier(),_type);
      a.ligne=st.lineno();
      r.addElement(a);

      t=get();
      if(t.equals("="))
      {
        a.initiale=parseExpression();
        trace("Initiale    : "+a.initiale);
      }
      else
      if(t.equals(","))
        _nom=parseNom();
      else
      if(!t.equals(";"))
        syntaxe(", or ; needed and not "+t);

    } while(!t.equals(";"));

    return r;
  }

  public AglVecteur parseParametres() throws Exception
  {
    int        np=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    do 
    {
      if(!peek().equals(")"))
      {
	String io=get();
	int passage=0;
             if(io.equals("in"   )) passage=AglParametre.IN;
        else if(io.equals("out"  )) passage=AglParametre.OUT;
        else if(io.equals("inout")) passage=AglParametre.INOUT;
	else syntaxe("in/out needed and not "+io);

        AglType type=parseType();
        String  nom =get();
        np++;

        // ancienne forme
        while(peek().equals("[]")) { get(); type=new AglTypeTableau(type); }

        trace("Parametre "+np+" : "+nom+" ["+type+"]");

        AglParametre p=new AglParametre(nom,type);
	p.passage=passage;
        r.addElement(p);

             if(peek().equals(",")) get();
        else if(!peek().equals(")")) syntaxe(", or ; needed and not "+t);
      }

    } while(!peek().equals(")"));

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    return r;
  }

  public AglVecteur parseArgs() throws Exception
  {
    int        na=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    t=get();
    if(!(t.equals("("))) syntaxe("( needed and not "+t);

    do 
    {
      t=peek();

      if(!t.equals(")"))
      {
        AglExpression ae=parseExpression();
        na++;
        trace("Argument  "+na+" : "+ae);
        r.addElement(ae);
      }

      t=get();
      if(!(t.equals(",")||t.equals(")"))) syntaxe(", or ) needed and not "+t);

    } while(!t.equals(")"));

    return r;
  }

  public AglBloc parseBloc() throws Exception
  {
    erreur("Il n'y a pas de bloc en Mdl ("+get()+")");
    return null;
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    erreur("Il n'y a pas de bloc-multiple en Mdl ("+get()+")");
    return null;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    erreur("Il n'y a pas de bloc-simple en Mdl ("+get()+")");
    return null;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    erreur("Il n'y a pas de bloc-virgule en Mdl ("+get()+")");
    return null;
  }

// Expressions

  public AglExpression parseExpression() throws Exception
  {
    erreur("Il n'y a pas d'expression en Mdl ("+get()+")");
    return null;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    erreur("Il n'y a pas d'expression generique en Mdl ("+get()+")");
    return null;
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite() throws Exception
  {
    erreur("Il n'y a pas d'expression unaire droite en Mdl ("+get()+")");
    return null;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    erreur("Il n'y a pas d'expression binaire en Mdl ("+get()+")");
    return null;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    erreur("Il n'y a pas d'expression de fonction en Mdl ("+get()+")");
    return null;
  }

  public AglExpression parseExpressionNew() throws Exception
  {
    erreur("Il n'y a pas d'expression new en Mdl ("+get()+")");
    return null;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    erreur("Il n'y a pas d'expression parenthse en Mdl ("+get()+")");
    return null;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    erreur("Il n'y a pas d'expression de typage en Mdl ("+get()+")");
    return null;
  }

// Instructions

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    erreur("Il n'y a pas d'instruction gnrique en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionAffectation parseInstructionAffectation(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction d'affectation en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    erreur("Il n'y a pas d'instruction de bloc en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    erreur("Il n'y a pas d'instruction catch en Mdl ("+get()+")");
    return null;
  }

  public AglVecteur parseInstructionDefVar() throws Exception
  {
    erreur("Il n'y a pas d'instruction de dfinition de variable en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction do/while en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction expressive en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    erreur("Il n'y a pas d'instruction if en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    erreur("Il n'y a pas de procedure en Java ("+get()+")");
    return null;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    erreur("Il n'y a pas d'instruction return en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    erreur("Il n'y a pas d'instruction switch en Mdl ("+get()+")");
    return null;
  }  

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    erreur("Il n'y a pas d'instruction throw en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    erreur("Il n'y a pas d'instruction try en Mdl ("+get()+")");
    return null;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction while en Mdl ("+get()+")");
    return null;
  }
}
