/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParsePascal.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;
//import java.net.*;

public class AglParsePascal extends AglParse
{
  FuStreamTokenizer st;
  Stack              pile;

  boolean  implantation_;
  boolean  program_;
  AglUnite unite_;

  AglClasse classe_;

  //AglNom   nom_;
  //AglType  type_;
  AglVecteur   commentaires_;

//  AglPaquet  paquet;
//  AglClasse     classe;
//  AglMethode    methode;
//  AglAttribut   attribut;

  public AglParsePascal()
  {
    if(AglType.cherche("String")==null)
      (new AglClasse(new AglNom("java.lang.String"))).elementaire=true;

    if(AglType.cherche("integer")==null)
      new AglTypeAlias("integer",AglTypePrimitif.tinteger ,"Pascal");
    if(AglType.cherche("real"   )==null)
      new AglTypeAlias("real"   ,AglTypePrimitif.tdouble  ,"Pascal");
    if(AglType.cherche("string" )==null)
      new AglTypeAlias("string" ,AglType.cherche("String"),"Pascal");
  }

  public String extension()
  {
    return "pas";
  }

  public String[] extensions()
  {
    return new String[] { "pas", "PAS" };
  }

  public boolean optionAnalyserCommentaires()
    { return true; }

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    st=new FuStreamTokenizer(_input,_fichier);
    st.resetSyntax();
    st.eolIsSignificant(false);
    st.quoteChar('"');
    st.quoteChar('\'');
    st.slashSlashComments(false);
    st.slashStarComments (false);
    st.wordChars('a','z');
    st.wordChars('A','Z');
    st.wordChars('0','9');
    st.wordChars('_','_');
    st.wordChars(129,255);
    st.whitespaceChars(' ',' ');
    st.whitespaceChars('\t','\t');
    st.whitespaceChars('\n','\n');
    st.whitespaceChars('\r','\r');

    pile=new Stack();

    commentaires_=new AglVecteur();
  }

  private static void addCommentKey(AglTableTriee t, String k, String v)
  {
    v=v.trim();

    /*
    int i;
    do
    {
      i=v.indexOf(" $ *");
      if(i>=0) v=v.substring(0,i)+v.substring(i+4);
    } while(i>=0);
    */

    v=FuLib.replace(v," $ *","");

    while(v.endsWith(" $")) v=v.substring(0,v.length()-2);

    if(k.startsWith("@alma_")||k.startsWith("@alma-"))
	k="@"+k.substring(6);

    if("@pre"          .equals(k)) k="@require";
    if("@precondition" .equals(k)) k="@require";
    if("@post"         .equals(k)) k="@ensure";
    if("@postcondition".equals(k)) k="@ensure";
    if("@inv"          .equals(k)) k="@invariant";
    if("@cond"         .equals(k)) k="@check";
    if("@condition"    .equals(k)) k="@check";

    if("@main".equals(k))
    {
      t.put(k,v);
    }
    else
    {
      AglVecteur l=(AglVecteur)t.get(k);
      if(l==null) l=new AglVecteur(1,1);
      l.addElement(v);
      t.put(k,l);
    }
  }

  private void addCommentaire(Object _c)
  {
    commentaires_.addElement(_c);
  }

  private AglVecteur getCommentaires()
  {
    AglVecteur r=commentaires_;
    commentaires_=new AglVecteur();
    return r;
  }

  private boolean hook_number=false;
  private boolean hook_comment=false;

  public String get()
  {
    String r="";

    if(pile.empty())
    {
      try
      {
        st.nextToken();

        switch(st.ttype)
        {
          case StreamTokenizer.TT_EOL   : r="$";        break;
          case StreamTokenizer.TT_EOF   : r="";         break;
          case StreamTokenizer.TT_WORD:
	    r=st.sval;
	    if(!hook_comment) r=r.toLowerCase();
	    break;
          case StreamTokenizer.TT_NUMBER: r=""+st.nval; break;
	  case '"':
	    if(st.sval!=null) r="\""+ascii2C(st.sval)+"\"";
	    else              r="\"";
	    break;
          case '\'':
	    if(st.sval!=null) r="\"" +ascii2C(st.sval)+"\"";
	    else              r="'";
	    break;
          default                       : r=""+(char)st.ttype; break;
        }
      }
      catch(Exception ex)
      {
        ex.printStackTrace();
	erreur(""+ex+" [AglParse.get]");
      }

      // Nombres reels
      if((!r.equals(""))&&Character.isDigit(r.charAt(0)))
      {
	hook_number=true;
        if(peek().equals("."))
        {
          r+=get();
          if(Character.isDigit(peek().charAt(0)))
          {
            r+=get();
            if((peek().equals("E"))||(peek().equals("e")))
            {
	      get();
	      r+="E";
              if((peek().equals("+"))||(peek().equals("-")))
                r+=get();
              r+=get();
            }
          }

	  /*
	       if(peek().equals("d")) r+=get();
	  else if(peek().equals("f")) r+=get();
	  */
        }
	hook_number=false;
      }

      // .####
      if(  (r.equals("."))
	 &&(!hook_number)
	 &&(!peek().equals(""))
	 &&(Character.isDigit(peek().charAt(0))))
      {
	// System.err.println("~~~ R="+r);
	// System.err.println("~~~ P="+peek());
	r="0."+get();
      }

      if((r.equals("."))&&(peek().equals("." ))) r+=get();
      if((r.equals(":"))&&(peek().equals("=" ))) r+=get();
      if((r.equals("<"))&&(peek().equals(">" ))) r+=get();
      if((r.equals(">"))&&(peek().equals("=" ))) r+=get();
      if((r.equals("<"))&&(peek().equals("=" ))) r+=get();

      if(!hook_comment&&
	 (  (r.equals("(")&&peek().equals("*" ))
	  ||r.equals("{")))
      {
	hook_comment=true;
	int l=st.lineno();
	boolean accolade=r.equals("{");

	if(!accolade) get();
	r="";
	AglTableTriee clefs=new AglTableTriee();

	//st.eolIsSignificant(true);
	st.ordinaryChar('"');
	st.ordinaryChar('\'');
	st.wordChars('@','@');
	st.wordChars('.','.');
	st.wordChars(',',',');

	boolean special=false;
	//while(peek().equals("*")) { special=true; get(); }

	String k="@main";
	String v="";
	while(true)
	{
	  String t=get();

	  if(t.equals("")) erreur("Unclosed comment");

	  if(  (accolade&&t.equals("}"))
	     ||(t.equals("*")&&peek().equals(")")))
	  {
	    if(!accolade) get();
	    addCommentKey(clefs,k,v);
	    break;
	  }
	  if(t.startsWith("@"))
	  {
	    if(!"".equals(v)) addCommentKey(clefs,k,v);
	    k=t;
	    if("@".equals(k)) k+=get();
	    v="";
	  }
	  else
	  {
	    if(  (v.length()>0)
	       &&Character.isLetterOrDigit(v.charAt(v.length()-1))
	       &&Character.isLetterOrDigit(t.charAt(0)))
	      v+=" "+t;
	    else
	      v+=t;
	  }
	}

	hook_comment=false;
	//st.eolIsSignificant(false);
	st.quoteChar('"');
	st.quoteChar('\'');
	st.ordinaryChar('@');
	st.ordinaryChar('.');
	st.ordinaryChar(',');

	/*
	for(Enumeration e=clefs.keys(); e.hasMoreElements(); )
	{
	  k=(String)e.nextElement();
	  v=(String)clefs.get(k);
	  System.err.println("CM: "+k+"="+v+"=");
	}
	*/

	AglCommentaire cm;
	if(  (clefs.size()==1)
	   &&(clefs.get("@main")!=null)
	   &&(((String)clefs.get("@main")).length()<78))
	  cm=new AglCommentaireSimple((String)clefs.get("@main"));
	else
	  cm=new AglCommentaireMulti(clefs,special);
	cm.ligne=l;

	if(AglOptions.analyserCommentaires)
	  addCommentaire(cm);

	//while(peek().equals("$")) get();
	r=get();
      }

     trace(">>> "+r);
    }
    else r=(String)pile.pop();

    return r;
  }

  public void put(String ih)
  {
    pile.push(ih);
  }

  private int pp=0;
  public int ligne()
  {
    int r=st.lineno();
    if(MaxLines!=0)
    {
      int p=100*r/MaxLines;
      if(p!=pp) progres(p);
      pp=p;
    }
    return r;
  }

  public String fichier()
  {
    String r=st.origin();
    int i=r.lastIndexOf(File.separator);
    if(i>=0) r=r.substring(i+1);
    return r;
  }

  // Parse

  public void parseFichier() throws Exception
  {
    parseUnite();
  }

  public AglUnite parseUnite() throws Exception
  {
    AglUnite au=new AglUnite(st.origin());
    au.langage="Pascal";

    classe_=(AglClasse)AglType.cherche("Global");
    if(classe_==null) classe_=new AglClasse(new AglNom("Global"));
    classe_.elementaire=true;

    implantation_=true;
    program_=false;
    unite_=au;
    String t="";

    do
    {
      AglVisibilite visibilite  =parseVisibilite();

      t=peek();
      trace("WU>"+t);
      
      if(t.equals("unit")||t.equals("program"))
      {
	if(t.equals("program")) program_=true;
        classe_=parseClasse();
	classe_.commentaires=getCommentaires();
      }
      else
      if(t.equals("uses"))
      {
        parseImports();
      }
      else
      if(t.equals("interface"))
      {
	get();
	implantation_=false;
      }
      else
      if(t.equals("implementation"))
      {
	get();
	implantation_=true;
      }
      else
      if(t.equals("function")||t.equals("procedure"))
      {
	AglMethode m=parseMethode();
	classe_.methodes.put(m.signature(),m);
	classe_.elementaire=false;
      }
      else
      if(t.equals("var"))
      {
	AglVecteur v=parseVar();
	for(Enumeration e=v.elements(); e.hasMoreElements(); )
	{
	  AglAttribut a=(AglAttribut)e.nextElement();
	  classe_.attributs.put(a.signature(),a);
	  classe_.elementaire=false;
	}
      }
      else
      if(t.equals("const"))
      {
	AglVecteur v=parseConst();
	for(Enumeration e=v.elements(); e.hasMoreElements(); )
	{
	  AglAttribut a=(AglAttribut)e.nextElement();
	  classe_.attributs.put(a.signature(),a);
	  classe_.elementaire=false;
	}
      }
      else
      if(t.equals("type"))
      {
	AglVecteur v=parseTypeDef();
	for(Enumeration e=v.elements(); e.hasMoreElements(); )
	{
	  AglType at=(AglType)e.nextElement();
	  unite_.types.put(""+at,at);
	}
      }
      else
      if(t.equals("begin"))
      {
	if(program_)
	{
	  AglVecteur v=new AglVecteur(1,1);
	  v.addElement(new AglParametre("_args",new AglTypeTableau
					(AglType.cherche("string"))));

	  AglMethode m=new AglMethode("main",AglTypePrimitif.tvoid);
	  m.visibilite.vpublic=true;
	  m.visibilite.vfinal =true;
	  m.visibilite.vstatic=true;
	  m.parametres=v;
	  m.corps=parseBloc();
	  classe_.methodes.put(m.signature(),m);
	  classe_.elementaire=false;
	}
	else
	{
	  classe_.clinit=parseBloc();
	  classe_.elementaire=false;
	}
      }
      else if(!t.equals("")) syntaxe("unknown keyword: "+t);

    } while(!t.equals(""));

    return au;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    AglVisibilite av=new AglVisibilite();
    av.ligne=st.lineno();
    return av;
  }

  public AglNom parseNom() throws Exception
  {
    AglNom an=new AglNom(get());
    an.ligne=st.lineno();
    return an;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    AglListeNoms al=new AglListeNoms();

    String t="";
    do
    {
      trace("WLN>"+peek());
      al.noms.addElement(parseNom());
      t=peek();
      if(t.equals(",")) get();
    }
    while(t.equals(","));

    al.ligne=st.lineno();
    return al;
  }

  public AglVecteur parseTypeDef() throws Exception
  {
    AglVecteur r=new AglVecteur(1,1);

    String t="";
    t=get();
    if(!t.equals("type")) syntaxe("type needed and not "+t);

    String nom=get();
    while(peek().equals("="))
    {
      nom=nom.substring(0,1).toUpperCase()+nom.substring(1);

      t=get();
      if(!t.equals("=")) syntaxe("= needed and not "+t);

      AglType at=parseType();

      if(at.getNom()!=null)
      {
	AglTypeAlias al=new AglTypeAlias(nom,at);
	r.addElement(al);
      }
      else
      {
	at.setNom(new AglNom(nom));
	r.addElement(at);
      }

      t=get();
      if(!t.equals(";")) syntaxe("; needed and not "+t);

      nom=get();
    }

    put(nom);
    return r;
  }

  public AglType parseType() throws Exception
  {
    AglType r=null;

    if(peek().equals("array"))
    {
      get();
      String t=get();
      if(!t.equals("[")) syntaxe("[ needed after array and not "+t);

      AglTypeTableau tt=new AglTypeTableau
	(AglTypePrimitif.tvoid,parseExpression());

      r=tt;
      while(peek().equals(","))
      {
	get();
	r=new AglTypeTableau(r,parseExpression());
      }

      t=get();
      if(!t.equals("]")) syntaxe("] needed after array and not "+t);
      t=get();
      if(!t.equals("of")) syntaxe("of needed after array and not "+t);

      tt.type=parseType();
    }
    else
    if(peek().equals("file"))
    {
      get();
      String t=get();
      if(!t.equals("of")) syntaxe("of needed after file and not "+t);
      parseType();
      r=AglType.cherche("File");
      if(r==null) r=new AglClasse(new AglNom("PascalBinaryFile"));
    }
    else
    if(peek().equals("text"))
    {
      get();
      r=AglType.cherche("TextFile");
      if(r==null) r=new AglClasse(new AglNom("PascalTextFile"));
    }
    else
    if(peek().equals("record"))
    {
      get();
      put("var");

      AglStruct s=new AglStruct(null);
      s.visibilite.vpublic =!implantation_;
      s.visibilite.vprivate= implantation_;
      s.visibilite.vfinal  =true;

      AglVecteur v=parseVar();
      for(Enumeration e=v.elements(); e.hasMoreElements(); )
      {
	AglAttribut a=(AglAttribut)e.nextElement();
	a.visibilite.vstatic=false;
	s.attributs.put(a.signature(),a);
      }

      r=s;
      String t=get();
      if(!t.equals("end")) syntaxe("end needed after file and not "+t);
    }
    else
    if(peek().equals("string"))
    {
      get();
      r=AglType.cherche("string");
      if(peek().equals("["))
      {
	get();
	parseExpression();
	String t=get();
	if(!t.equals("]")) syntaxe("] needed and not "+t);
      }
    }
    else
    if(peek().equals("(")) // enum
    {
      AglStruct s=new AglStruct(null);
      s.visibilite.vpublic=true;
      s.visibilite.vfinal =true;

      int n=0;
      do
      {
	get();
	AglAttribut a=new AglAttribut(get(),AglTypePrimitif.tinteger);
	a.visibilite.vpublic=true;
	a.visibilite.vfinal =true;
	a.visibilite.vstatic=true;
	a.initiale=new AglExpressionGenerique(""+n);
	s.attributs.put(a.signature(),a);
	n++;
      }
      while(peek().equals(","));

      get();
      r=s;
    }
    else
    {
      trace("WT>"+peek());
      String v=get();

      if(peek().equals("..")) // range
      {
	get();
	get();
	r=AglTypePrimitif.tinteger; //new AglClasse(null);
      }
      else
      {
	String nom=v;
	r=AglType.cherche(nom);
	if(r==null)
	{
	  nom=nom.substring(0,1).toUpperCase()+nom.substring(1);
	  r=AglType.cherche(nom);
	}
	if(r==null) r=new AglType(new AglNom(nom));
      }
    }

    return r;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    erreur("Il n'y a pas de paquet en Pascal ("+get()+")");
    return null;
  }
  
  public void parseImports() throws Exception
  {
    String t="";

    t=peek();
    if(!t.equals("uses")) syntaxe("uses needed and not "+t);

    do
    {
      get();
      parseImport();
    }
    while(peek().equals(","));

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);
  }
   
  public AglImport parseImport() throws Exception
  {
    String nom=get();
    //nom=nom.substring(0,1).toUpperCase()+nom.substring(1);
    AglNom ni=new AglNom(nom);
    trace("Import      : "+ni);

    AglImport ai=new AglImport(ni);
    ai.ligne=st.lineno();
    ai.commentaires=getCommentaires();

    unite_.imports.put(ai.nom.formeJava(),ai);

    return ai;
  }
  
  public AglClasse parseClasse() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("unit")&&!t.equals("program"))
      syntaxe("unit or program needed and not "+t);

    String nom=get();
    nom=nom.substring(0,1).toUpperCase()+nom.substring(1);
    AglNom np=new AglNom(nom);
    trace("Classe   : "+np);

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglType at=AglClasse.cherche(np);

    AglClasse ac=null;
    if(at==null) ac=new AglClasse(np);
    if(at instanceof AglClasse) ac=(AglClasse)at;
    if(ac==null) erreur(np.formeJava()+" is already defined");

    ac.visibilite=new AglVisibilite();
    ac.visibilite.vpublic=true;
    ac.visibilite.vfinal=true;

    ac.proprietaire=unite_;
    ac.ligne=st.lineno();
    ac.elementaire=false;
    return ac;
  }

  public AglListeNoms parseMothers() throws Exception
  {
    erreur("Il n'y a pas de meres en Pascal ("+get()+")");
    return null;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    erreur("Il n'y a pas d'interfaces en Pascal ("+get()+")");
    return null;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    erreur("Il n'y a pas d'exceptions en Pascal ("+get()+")");
    return null;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    erreur("Il n'y a pas de constructeur en Pascal ("+get()+")");
    return null;
  }

  public AglMethode parseMethode() throws Exception
  {
    String t=get();

    if(!t.equals("function")&&!t.equals("procedure"))
      syntaxe("function or procedure needed and not "+t);

    String nom=get();

    trace("Methode     : "+nom);
    AglMethode m=new AglMethode(nom,AglTypePrimitif.tvoid);
    m.ligne=st.lineno();
    m.commentaires=getCommentaires();

    if(peek().equals("("))
      m.parametres=parseParametres();

    if(peek().equals(":"))
    {
      get();
      m.type=parseType();
    }

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    if(implantation_)
    {
      m.visibilite.vpublic=true;
      m.visibilite.vfinal =true;
      m.visibilite.vstatic=true;

      boolean encore=true;
      while(encore)
      {
	t=peek();

	if(t.equals("const"))
	{
	  AglVecteur v=parseConst();
	  for(Enumeration e=v.elements(); e.hasMoreElements(); )
	  {
	    AglAttribut a=(AglAttribut)e.nextElement();
	    classe_.attributs.put(a.signature(),a);
	  }
	}
	else
	if(t.equals("var"))
	{
	  AglVecteur v=parseVar();
	  for(Enumeration e=v.elements(); e.hasMoreElements(); )
	  {
	    AglAttribut a=(AglAttribut)e.nextElement();
	    classe_.attributs.put(a.signature(),a);
	  }
	}
	else
	if(t.equals("type"))
	  parseTypeDef();
	else
	if(t.equals("procedure")||t.equals("function"))
	{
	  AglMethode sm=parseMethode();
	  classe_.methodes.put(sm.signature(),sm);
	}
	else
	  encore=false;
      }

      t=peek();
      if(!peek().equals("begin")) syntaxe("begin needed and not "+t);
      m.corps=parseBlocMultiple();
    }

    return m;
  }

  public AglVecteur parseVar() throws Exception
  {
    String t=get();
    if(!t.equals("var")) syntaxe("var needed and not "+t);

    AglVecteur r=parseAttribut();

    String v=get();
    while(peek().equals(":")||peek().equals(","))
    {
      put(v);
      AglVecteur la=parseAttribut();
      for(Enumeration e=la.elements(); e.hasMoreElements(); )
      {
	AglAttribut aa=(AglAttribut)e.nextElement();
	aa.commentaires=getCommentaires();
	r.addElement(aa);
      }
      v=get();
    }
    put(v);

    return r;
  }

  public AglVecteur parseConst() throws Exception
  {
    AglVecteur r=new AglVecteur(1,1);
    
    String t=get();
    if(!t.equals("const")) syntaxe("const needed and not "+t);

    String nom=get();
    while(peek().equals("="))
    {
      t=get();
      if(!t.equals("=")) syntaxe("= needed and not "+t);

      AglExpression vc=parseExpression();

      t=get();
      if(!t.equals(";")) syntaxe("; needed and not "+t);

      AglType type=AglTypePrimitif.tvoid;
      if(vc instanceof AglExpressionGenerique)
      {
	String s=((AglExpressionGenerique)vc).body;
	if((s!=null)&&s.startsWith("\"")) type=AglType.cherche("string");
	else
	{
	  try
	  {
	    new Double(s);  type=AglTypePrimitif.tdouble;
	    new Integer(s); type=AglTypePrimitif.tinteger;
	  }
	  catch(Exception ex) { }
	}
      }

      AglAttribut aa=new AglAttribut(nom,type);
      aa.visibilite.vpublic=true;
      aa.visibilite.vfinal =true;
      aa.visibilite.vstatic=true;
      aa.initiale=vc;
      aa.commentaires=getCommentaires();
      r.addElement(aa);
      nom=get();
    }

    put(nom);
    return r;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    AglVecteur r=new AglVecteur();

    AglVecteur noms=new AglVecteur(1,1);
    put(",");
    do
    {
      get();
      noms.addElement(get());
    } while(peek().equals(","));

    String t=get();
    if(!t.equals(":")) syntaxe(": needed and not "+t);

    AglType type=parseType();

    AglExpression initiale=null;
    if(peek().equals("="))
    {
      get();
      initiale=parseExpression();
    }

    AglVisibilite v=new AglVisibilite();
    v.vstatic=true;
    if(!implantation_) v.vpublic=true;

    for(int i=0; i<noms.size(); i++)
    {
      String nom=(String)noms.elementAt(i);
      trace("Attribut    : "+nom+" ["+type+"]");
      AglAttribut a=new AglAttribut(nom,type);
      a.initiale=initiale;
      a.visibilite=v;
      a.ligne=st.lineno();
      r.addElement(a);
    }

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    return r;
  }

  public AglVecteur parseParametres() throws Exception
  {
    int        np=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    do 
    {
      if(!peek().equals(")"))
      {
	AglVisibilite visibilite=parseVisibilite();

	int passage=AglParametre.IN;

	if(peek().equals("var"))
	{
	  get();
	  passage=AglParametre.INOUT;
	}

        AglVecteur noms=new AglVecteur();
	put(",");
	do
	{
	  get();
	  noms.addElement(get());
	} while(peek().equals(","));

	t=get();
	if(!t.equals(":")) syntaxe(": needed and not "+t);

        AglType type=parseType();

	for(int i=0; i<noms.size(); i++)
        {
	  np++;
	  String nom=(String)noms.elementAt(i);
	  trace("Parametre "+np+" : "+nom+" ["+type+"]");
	  AglParametre p=new AglParametre(nom,type);
	  p.visibilite=visibilite;
	  p.passage=passage;
	  p.ligne=st.lineno();
	  r.addElement(p);
	}

             if(peek().equals(";")) get();
        else if(!peek().equals(")")) syntaxe("; needed and not "+t);
      }

    } while(!peek().equals(")"));

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    return r;
  }

  public AglVecteur parseArgs() throws Exception
  {
    int        na=0;
    String     t ="";
    AglVecteur r =new AglVecteur();

    t=get();
    if(!(t.equals("("))) syntaxe("( needed and not "+t);

    do 
    {
      t=peek();

      if(!t.equals(")"))
      {
        AglExpression ae=parseExpression();
        na++;
        trace("Argument  "+na+" : "+ae);
        r.addElement(ae);
      }

      t=get();
      if(t.equals(":")) t=",";
      if(!(t.equals(",")||t.equals(")"))) syntaxe(", or ) needed and not "+t);

    } while(!t.equals(")"));

    return r;
  }

  public AglBloc parseBloc() throws Exception
  {
    return peek().equals("begin") ? parseBlocMultiple() : parseBlocSimple();
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    AglBloc ab=new AglBloc();
    ab.commentaires=getCommentaires();

    String  t="";
    t=get();
    if(!t.equals("begin")) syntaxe("begin needed and not "+t);

    while(!peek().equals("end"))
    {
      trace("WBM>>> "+peek());
      t=peek();

      AglInstruction ai=null;
      AglVecteur commentaires=getCommentaires();
        
           if(t.equals("begin"))        ai=parseInstructionBloc        ();
      else if(t.equals("procedure"))    syntaxe("procedure is not wanted there");
      else if(t.equals("function"))     syntaxe("function is not wanted there");
      else if(t.equals(";"))            ai=parseInstructionVide        ();
      else if(t.equals("for"))          ai=parseInstructionFor         ();
      else if(t.equals("if"))           ai=parseInstructionIf          ();
      else if(t.equals("return"))       ai=parseInstructionReturn      ();
      else if(t.equals("case"))         ai=parseInstructionSwitch      ();
      else if(t.equals("while"))        ai=parseInstructionWhile       ();
      else if(t.equals("with"))         ai=parseInstructionWith        ();
      else
      {
	AglExpression gauche=parseExpression();

	t=peek();
	     if(isEndExpression(t)) ai=parseInstructionExpression (gauche);
	else if(t.equals(":="))     ai=parseInstructionAffectation(gauche);
	//else if(t.equals("("))    ai=parseInstructionProcedure  (gauche);
	else                        ai=parseInstructionGenerique  ();
      }

      ai.commentaires=commentaires;
      ab.instructions.addElement(ai);
    }

    t=get();
    if(!t.equals("end")) syntaxe("end needed and not "+t);

    if(peek().equals(";")||peek().equals(".")) get();

    return ab;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    AglBloc ab=new AglBloc();
    String  t=get();

    trace("WBS>>> "+t+" | "+peek());

    String u=peek();
    put(t);
    AglInstruction ai=null;
    AglVecteur commentaires=getCommentaires();

    {
           if(t.equals("begin"))        syntaxe("begin is not wanted there");
      else if(t.equals("procedure"))    syntaxe("procedure is not wanted there");
      else if(t.equals("function"))     syntaxe("function is not wanted there");
      else if(t.equals(";"))            ai=parseInstructionVide        ();
      else if(t.equals("for"))          ai=parseInstructionFor         ();
      else if(t.equals("if"))           ai=parseInstructionIf          ();
      else if(t.equals("return"))       ai=parseInstructionReturn      ();
      else if(t.equals("case"))         ai=parseInstructionSwitch      ();
      else if(t.equals("while"))        ai=parseInstructionWhile       ();
      else if(t.equals("with"))         ai=parseInstructionWith        ();
      else
      {
	AglExpression gauche=parseExpression();

	t=peek();
	//System.err.println("$$$ "+ligne()+" $$$$$$$$ parseBS: "+t);

	     if(isEndExpression(t)) ai=parseInstructionExpression (gauche);
	else if(t.equals(":="))     ai=parseInstructionAffectation(gauche);
	//else if(t.equals("("))    ai=parseInstructionProcedure  (gauche);
	else                        ai=parseInstructionGenerique  ();
      }

      ai.commentaires=commentaires;
      ab.instructions.addElement(ai);
    }

    return ab;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    erreur("Il n'y a pas de bloc-virgule en Pascal ("+get()+")");
    return null;
  }

// Expressions

  private final static boolean isEndExpression(String p)
  {
    return
         p.equals(""  )||p.equals("end" )||p.equals(";"   )||p.equals("," )
       ||p.equals("do")||p.equals("then")||p.equals("else")||p.equals("of")
       ||p.equals("to")||p.equals("downto");
  }

  public AglExpression parseExpression() throws Exception
  {
    String p=peek();
    trace("WE>>> "+p);
    if(isEndExpression(p))
      syntaxe("missing expression: "+p);

    AglExpression ae;

         if(p.equals("new")) ae=parseExpressionNew();
    else if(p.equals("+"  )) ae=parseExpressionUnaireDroite();
    else if(p.equals("-"  )) ae=parseExpressionUnaireDroite();
    else if(p.equals("not")) ae=parseExpressionUnaireDroite();
    else if(p.equals("("))
    {
      ae=parseExpressionParenthesee();
    }
    else
    {
      String  t,u;
      t=get();
      u=peek();
      put(t);

      trace("WE>>> "+t+" | "+u);

           if(u.equals("("))      ae=new AglExpressionMethode(null,parseExpressionFonction());
      else if(u.equals("."))      ae=new AglExpressionAttribut(null,get());
      else if(u.equals("["))
      {
	ae=new AglExpressionGenerique(get());
	u=get(); if(!u.equals("[")) syntaxe("[ needed and not "+u);
	ae=new AglExpressionIndice(ae,parseExpression());
	while(peek().equals(","))
	{
	  get();
	  ae=new AglExpressionIndice(ae,parseExpression());
	  ae.ligne=st.lineno();
	}
	u=get();
	if(!u.equals("]")) syntaxe("] needed and not "+u);
      }
      else ae=new AglExpressionGenerique(get());
      ae.ligne=st.lineno();
    }

    boolean encore=true;
    while(encore)
    {
      encore=false;
      String u;

      trace("Expression  : "+ae+" ["+ae.getClass()+"]");
      u=peek();
      trace("WEA>>> "+u);

      if(isEndExpression(u))
	encore=false;
      else
      if(u.equals(":="))
      {
	get();
	ae=new AglExpressionAffectation(ae,parseExpression());
	ae.ligne=st.lineno();
	encore=true;
      }
      else
      if(  (u.equals("+"  ))||(u.equals("-"))
         ||(u.equals("*"  ))||(u.equals("/"))
	 ||(u.equals(".." ))
	 ||(u.equals("div"))||(u.equals("mod"))
         ||(u.equals("="  ))||(u.equals("<>"))
         ||(u.equals("<"  ))||(u.equals(">"))
         ||(u.equals("<=" ))||(u.equals(">="))
         ||(u.equals("and"))||(u.equals("or")))
      {
	ae=new AglExpressionBinaire(get(),ae,parseExpression());
	ae.ligne=st.lineno();
	encore=true;
      }
      else
      if(u.equals("["))
      {
	get();
	ae=new AglExpressionIndice(ae,parseExpression());
	while(peek().equals(","))
        {
	  get();
	  ae=new AglExpressionIndice(ae,parseExpression());
	  ae.ligne=st.lineno();
	}
	u=get();
	if(!u.equals("]")) syntaxe("] needed and not "+u);
	encore=true;
      }
      else
      if(u.equals("."))
      {
	get();
	String v=get();
	String w=peek();
	put(v);
	
	if(w.equals("(")) ae=new AglExpressionMethode(ae,parseExpressionFonction());
	else ae=new AglExpressionAttribut(ae,get());
	ae.ligne=st.lineno();
	encore=true;
      }
    }

    return ae;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    String t="";
    int    np=0;
    int    nt=0;

    String body="";

    do
    {
      t=get();
      // trace("WEG>>> "+t);

      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      if(t.equals("]")) nt--;
      if(t.equals("[")) nt++;
      
           if((np<0)&&(t.equals(")"))) ;
      else if((nt<0)&&(t.equals("]"))) ;
      else if((np==0)&&(nt==0)&&isEndExpression(t)) ;
      else
      {
        if(  (body.length()>0)
           &&(  Character.isJavaIdentifierPart(body.charAt(body.length()-1))
	      ||(body.charAt(body.length()-1)==')'))
	   &&(  Character.isJavaIdentifierStart(t.charAt(0))
	      ||(t.charAt(0)=='\"'))) body+=" ";
	body+=t;
      }
    } while(  (np>0)
            ||(nt>0)
	    ||((np==0)&&(nt==0)&&!isEndExpression(t)));

    if(isEndExpression(t)) put(t);
    if((np<0)&&t.equals(")")) put(t);
    if((nt<0)&&t.equals("]")) put(t);

    AglExpressionGenerique ae=new AglExpressionGenerique(body);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite() throws Exception
  {
    String        op=get();

    trace("Unaire     : "+op);
    AglExpression ed=parseExpression();

    AglExpressionUnaireDroite ae=new AglExpressionUnaireDroite(op,ed);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    AglExpression eg=new AglExpressionGenerique(get());
    String        op=get();

    trace("Binaire    : "+op);
    AglExpression ed=parseExpression();

    AglExpressionBinaire ae=new AglExpressionBinaire(op,eg,ed);
    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    String t="";

    String nom=get();
    AglVecteur le =parseArgs();
    trace("Fonction    : "+nom+" ("+le.size()+" arguments)");

    AglExpressionFonction ae=new AglExpressionFonction(nom);
    ae.ligne=st.lineno();
    for(Enumeration e=le.elements(); e.hasMoreElements(); )
      ae.arguments.addElement(e.nextElement());
    return ae;
  }

  public AglExpression parseExpressionInitArray() throws Exception
  {
    /*
    if(!peek().equals("{")) syntaxe("{ needed and not "+peek());
    get();

    AglExpressionInitArray ae=new AglExpressionInitArray();
    do
    {
      if(peek().equals("{")) ae.initiales.addElement(parseExpressionInitArray());
      else                   ae.initiales.addElement(parseExpression());

      if(!peek().equals(",")&&!peek().equals("}"))
	syntaxe("} or , needed and not "+peek());
      if(peek().equals(",")) get();

    } while(!peek().equals("}"));

    if(!peek().equals("}")) syntaxe("} needed and not "+peek());
    get();

    return ae;
    */

    return null;
  }

  public AglExpression parseExpressionNew() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("new")) syntaxe("new needed and not "+t);

    AglType c=parseType();

    AglExpression ae;

    if(peek().equals("("))
    {
      AglVecteur  le=parseArgs();
      trace("New         : "+c.nom.dernier()+" ("+le.size()+" arguments)");

      AglExpressionNew aen=new AglExpressionNew(c);
      aen.ligne=st.lineno();
      for(Enumeration e=le.elements(); e.hasMoreElements(); )
	aen.arguments.addElement(e.nextElement());
      ae=aen;
    }
    else
    {
      trace("NewArray    : "+c.nom.dernier());

      AglExpressionNewArray aen=new AglExpressionNewArray(c);
      aen.ligne=st.lineno();
      ae=aen;

      /*
      if(peek().equals("{"))
      {
	aen.initiale=parseExpressionInitArray();
      }
      else
      */
      {
	t=get(); if(!t.equals("[")) syntaxe("[ needed and not "+t);
	AglExpression ne=parseExpression();
	t=get(); if(!t.equals("]")) syntaxe("] needed and not "+t);
	aen.nombre=ne;

	while(peek().equals("["))
	{
	  t=get(); if(!t.equals("[")) syntaxe("[ needed and not "+t);
	  AglExpression nex=parseExpression();
	  t=get(); if(!t.equals("]")) syntaxe("] needed and not "+t);
	  aen.type=new AglTypeTableau(aen.type,nex);
	}
	while(peek().equals("[]"))
	{
	  get();
	  aen.type=new AglTypeTableau(aen.type);
	}
      }
    }

    return ae;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpressionParenthesee ae=new AglExpressionParenthesee(parseExpression());

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    ae.ligne=st.lineno();
    return ae;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    String t="";

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglType type=parseType();
    trace("Typage      : "+type);

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglExpressionTypage ae=new AglExpressionTypage(type,parseExpression());
    ae.ligne=st.lineno();
    return ae;
  }

// Instructions

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    String  t="";
    int     na=0;
    int     np=0;
    boolean pv=true;

    String body="";

    do
    {
      t=get();
      // trace("WI>>> "+t);

      if(t.equals(")")) np--;
      if(t.equals("(")) np++;
      {
	if(t.equals(";")) { if(np==0) { body+=";\n"; pv=true; } else body+="; "; }
        else if(pv) { body+=indente(na+1)+t; pv=false; }
        else
	{
          if(  (body.length()>0)
	     &&(  Character.isJavaIdentifierPart(body.charAt(body.length()-1))
		||(body.charAt(body.length()-1)==')'))
	     &&(  Character.isJavaIdentifierStart(t.charAt(0))
		||(t.charAt(0)=='\"'))) body+=" ";
	  body+=t;
          pv=false;
        }
      }
    } while((na>0)||(!t.equals(";")));

    trace("Instruction : "+body);
    AglInstructionGenerique ai=new AglInstructionGenerique(body);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionAffectation parseInstructionAffectation
    (AglExpression eg) throws Exception
  {
    String t="";

    t=get();
    if(!t.equals(":=")) syntaxe(":= needed and not "+t);

    AglExpression ed=parseExpression();

    //t=get();
    //if(!t.equals(";")) syntaxe("; needed and not "+t);
    if(peek().equals(";")) get();

    trace("Affectation: "+eg+" <-- "+ed);

    AglInstructionAffectation ai=new AglInstructionAffectation(eg,ed);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    AglBloc ab=parseBlocMultiple();
    AglInstructionBloc ai=new AglInstructionBloc(ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("catch")) syntaxe("catch needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglType type=parseType();
    AglExpression ae=new AglExpressionGenerique(type.nom.dernier()+" "+get());

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();

    AglInstructionCatch ai=new AglInstructionCatch(ae,ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglVecteur parseInstructionDefVar() throws Exception
  {
    String     t ="";
    int        nv=0;
    AglVecteur r =new AglVecteur();

    AglType type=parseType();

    do 
    {
      t=get();

      if(!t.equals(";"))
      {
        // ancienne forme
        while(peek().equals("[]")) { get(); type=new AglTypeTableau(type); }

        nv++;
        trace("Variable  "+nv+" : "+t+" ["+type+"]");
        AglInstructionDefVar ai=new AglInstructionDefVar(t,type);
        ai.ligne=st.lineno();

        t=get();
        if(t.equals("="))
        {
          /*
	    if(peek().equals("{")) ai.initiale=parseExpressionInitArray();
	    else
	  */
	  ai.initiale=parseExpression();
	  trace("Initiale   : "+ai.initiale);
	  t=get();
        }
        r.addElement(ai);

        if(!(t.equals(",")||t.equals(";")))
	  syntaxe(", or ; needed and not "+t);
      }

    } while(!t.equals(";"));

    return r;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction do-while en Pascal");
    return null;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    String t="";

    //t=get();
    //if(!t.equals(";")) syntaxe("; needed and not "+t);
    if(peek().equals(";")) get();

    trace("InsExpr   : "+eg);

    AglInstructionExpression ai=new AglInstructionExpression(eg);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionFinally parseInstructionFinally() throws Exception
  {
    erreur("Il n'y a pas d'instruction finally en Pascal ("+get()+")");
    return null;
  }

  public AglInstructionFor parseInstructionFor() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("for")) syntaxe("for needed and not "+t);

    String variable=get();
    
    t=get();
    if(!t.equals(":=")) syntaxe (":= needed and not "+t);

    AglExpression ed=parseExpression();

    t=get();
    if(!t.equals("to")&&!t.equals("downto"))
      syntaxe("to or downto needed and not "+t);

    String op=(t.equals("to") ? "<=" : ">=");

    AglExpression ef=parseExpression();

    t=get();
    if(!t.equals("do")) syntaxe("do needed and not "+t);

    AglInstruction ad=new AglInstructionAffectation
      (new AglExpressionGenerique(variable),ed);
    AglExpression  at=new AglExpressionBinaire
      (op,new AglExpressionGenerique(variable),ef);
    AglInstruction af=new AglInstructionExpression
	(new AglExpressionUnaireDroite
	    ("++",new AglExpressionGenerique(variable)));

    AglBloc ab=parseBloc();
    AglInstructionFor ai=new AglInstructionFor(ad,at,af,ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("if")) syntaxe("if needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals("then")) syntaxe("then needed and not "+t);

    AglBloc aoui=parseBloc();
    AglBloc anon=null;

    t=peek();

    //System.err.println("$$$"+ligne()+"$$$$$$$$$$$$$$ IF="+t);

    if(t.equals("else"))
    {
      t=get();
      anon=parseBloc();
    }
    
    AglInstructionIf ai=new AglInstructionIf(ae,aoui,anon);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    erreur("Il n'y a pas de procedure en Pascal ("+get()+")");
    return null;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals("return")) syntaxe("return needed and not "+t);

    AglExpression ae=null;
    if(!peek().equals(";")) ae=parseExpression();

    AglInstructionReturn ai=new AglInstructionReturn(ae);
    ai.ligne=st.lineno();

    //t=get();
    //if(!t.equals(";")) syntaxe("; needed and not "+t);
    if(peek().equals(";")) get();

    return ai;
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("case")) syntaxe("case needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals("of")) syntaxe("of needed and not "+t);

    AglTableOrdonnee ac=new AglTableOrdonnee();
    AglBloc             ad=new AglBloc();

    while(!peek().equals("end")&&!peek().equals("else"))
    {
      String ln=get();
      while(peek().equals(",")) { get(); ln+=","+get(); }
      AglExpression ce=new AglExpressionGenerique(ln);
      trace("SWITCH="+ln);

      t=get();
      if(!t.equals(":")) syntaxe(": needed and not "+t);
      
      trace("PEEK="+peek());
      AglBloc cb=parseBloc();
      ac.put(ce,cb);
    }

    if(peek().equals("else"))
    {
      get();
      ad=parseBloc();
    }

    t=get();
    if(!t.equals("end")) syntaxe("end needed and not "+t);

    //t=get();
    //if(!t.equals(";")) syntaxe("; needed and not "+t);
    if(peek().equals(";")) get();

    AglInstructionSwitch ai=new AglInstructionSwitch(ae,ac,ad);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionSynchronized parseInstructionSynchronized()
    throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("synchronized")) syntaxe("synchronized needed and not "+t);

    t=get();
    if(!t.equals("(")) syntaxe("( needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals(")")) syntaxe(") needed and not "+t);

    AglBloc ab=parseBloc();
    AglInstructionSynchronized ai=new AglInstructionSynchronized(ae,ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    erreur("Il n'y a pas d'instruction throw en Pascal ("+get()+")");
    return null;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    erreur("Il n'y a pas d'instruction try en Pascal ("+get()+")");
    return null;
  }

  public AglInstructionVide parseInstructionVide() throws Exception
  {
    String     t="";

    t=get();
    if(!t.equals(";")) syntaxe("; needed and not "+t);

    AglInstructionVide ai=new AglInstructionVide();
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("while")) syntaxe("while needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals("do")) syntaxe("do needed and not "+t);

    AglBloc ab=parseBloc();
    AglInstructionWhile ai=new AglInstructionWhile(ae,ab);
    ai.ligne=st.lineno();
    return ai;
  }

  public AglInstructionBloc parseInstructionWith() throws Exception
  {
    String     t="";
    int        nv=0;

    t=get();
    if(!t.equals("with")) syntaxe("with needed and not "+t);

    AglExpression ae=parseExpression();

    t=get();
    if(!t.equals("do")) syntaxe("do needed and not "+t);

    AglBloc ab=parseBloc();
    AglInstructionBloc ai=new AglInstructionBloc(ab);
    ai.ligne=st.lineno();
    return ai;
  }
}

