/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseTaglibHandler.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;
import com.memoire.mst.*;

import java.util.*;
import java.io.*;

public class AglParseTaglibHandler
       extends MstHandlerBase
       implements MstXmlHandler
{
  Properties     options_ =new Properties();
  String         fichier_ =null;
  AglParseTaglib parse_   =null;
  String         data_    ="";

  AglUnite       unite_;
  AglClasse      classe_;
  AglAttribut    attribut_;

  public AglParseTaglibHandler(AglParseTaglib _parse, String _fichier)
  {
    fichier_=_fichier;
    parse_  =_parse;
  }

  public void trace(String _s)
  {
    parse_.trace(_s);
  }

  public void error
  (String message, String systemId, int line, int column)
       throws Exception
  {
    parse_.syntaxe(message);
    super.error(message,systemId,line,column);
  }

  public void charData(char ch[], int start, int length) throws Exception
  {
    for(int i=0; i<length; i++)
      data_+=ch[start+i];
    // trace("Data: "+data_+"\n");
  }

  public void startElement(String _element) throws Exception
  {
    // trace(_element+":");
    for(Enumeration e1=options_.keys(); e1.hasMoreElements(); )
    {
      String k1=(String)e1.nextElement();
      Object v1=options_.get(k1);

      // trace(" "+k1+"=");
      // trace(""+v1);
    }
    // trace("\n");

    if("taglib".equals(_element))
    {
      unite_=new AglUnite(fichier_);
      unite_.langage="Taglib";
      // trace("Unite: "+fichier_+"\n");

      AglNom n;
      n=new AglNom("javax.servlet.*");
      unite_.imports.put(n.formeJava(),new AglImport(n));
      n=new AglNom("javax.servlet.http.*");
      unite_.imports.put(n.formeJava(),new AglImport(n));
      n=new AglNom("javax.servlet.jsp.*");
      unite_.imports.put(n.formeJava(),new AglImport(n));
      n=new AglNom("javax.servlet.jsp.tagext.*");
      unite_.imports.put(n.formeJava(),new AglImport(n));
    }
    else
    if("tag".equals(_element))
    {
      classe_=new AglClasse(new AglNom("TaglibTmpClass"));
      classe_.proprietaire=unite_;
      classe_.elementaire=false;
      classe_.visibilite.vpublic=true;
      classe_.meres.noms.addElement(new AglNom("TagSupport"));

      //public final int doStartTag() throws JspException
      AglMethode dst=new AglMethode
	("doStartTag",AglTypePrimitif.tvoid);
      dst.genere=true;
      dst.visibilite.vfinal=true;
      dst.visibilite.vpublic=true;
      dst.exceptions.noms.addElement(new AglNom("JspException"));
      dst.corps.instructions.addElement
	(new AglInstructionReturn
	 (new AglExpressionGenerique("EVAL_BODY_INCLUDE")));
      classe_.methodes.put(dst.nom,dst);
      
      //public final int doEndTag() throws JspException
      AglMethode det=new AglMethode
	("doEndTag",AglTypePrimitif.tvoid);
      det.genere=true;
      det.visibilite.vfinal=true;
      det.visibilite.vpublic=true;
      det.exceptions.noms.addElement(new AglNom("JspException"));
      det.corps.instructions.addElement
	(new AglInstructionReturn
	 (new AglExpressionGenerique("EVAL_PAGE")));
      classe_.methodes.put(det.nom,det);

    }
    else
    if("attribute".equals(_element))
    {
      attribut_=new AglAttribut("TaglibTmpAttribute",AglType.cherche("String"));
      attribut_.visibilite.vprivate=true;
    }

    options_.clear();
  }
    
  public void endElement(String _element) throws Exception
  {
    data_=cleanData(data_);

    if("taglib".equals(_element))
    {
      unite_=null;
    }
    else
    if("tag".equals(_element))
    {
      classe_=null;
    }
    else
    if("tagclass".equals(_element))
    {
      AglNom cn=new AglNom(data_);
      classe_.setNom(cn); //new AglNom(cn.dernier());
      if(unite_.paquet==null)
	unite_.paquet=new AglPaquet(cn.parent());
    }
    else
    if("attribute".equals(_element))
    {
      if((attribut_!=null)&&(classe_!=null))
      {
	String n=attribut_.nom;
	attribut_.nom=n+"_";
	classe_.attributs.put(n+"_",attribut_);

	//public String get()
	AglMethode get=new AglMethode
	  ("get"+n.substring(0,1).toUpperCase()+
	   n.substring(1),AglType.cherche("String"));
	get.genere=true;
	get.visibilite.vpublic=true;
	get.corps.instructions.addElement
	  (new AglInstructionReturn
	   (new AglExpressionGenerique(n+"_")));
	classe_.methodes.put(get.nom,get);

	//public void set(String)
	AglMethode set=new AglMethode
	  ("set"+n.substring(0,1).toUpperCase()+
	   n.substring(1),AglTypePrimitif.tvoid);
	set.genere=true;
	set.visibilite.vpublic=true;
	set.parametres.addElement
	  (new AglParametre("_"+n,AglType.cherche("String")));
	set.corps.instructions.addElement
	  (new AglInstructionAffectation
	   (new AglExpressionGenerique(n+"_"),
	    new AglExpressionGenerique("_"+n)));
	classe_.methodes.put(set.nom,set);

      }
      attribut_=null;
    }
    else
    if("name".equals(_element))
    {
      if((attribut_!=null)&&("TaglibTmpAttribute".equals(attribut_.nom)))
	attribut_.nom=data_;
      else
      if(classe_!=null)
      {
	AglVecteur v=new AglVecteur(1,1);
	v.addElement(data_);
	getCommentaire(classe_).clefs.put("@jsp-name",v);
      }
    }
    else
    if("info".equals(_element))
    {
      if(attribut_!=null)
	getCommentaire(attribut_).clefs.put("@main",data_);
      else
      if(classe_!=null)
	getCommentaire(classe_  ).clefs.put("@main",data_);
      else
      if(unite_!=null)
	getCommentaire(unite_   ).clefs.put("@main",data_);
    }
    else
    if("bodycontent".equals(_element))
    {
      if(attribut_!=null)
      {
	AglVecteur v=new AglVecteur(1,1);
	v.addElement(data_);
	getCommentaire(attribut_).clefs.put("@jsp-bodycontent",v);
      }
    }
    else
    if(  ("required"   .equals(_element))
       ||("rtexprvalue".equals(_element)))
    {
      if(attribut_!=null)
      {
	AglVecteur v=new AglVecteur(1,1);
	v.addElement(data_);
	getCommentaire(attribut_).clefs.put("@jsp-"+_element,v);
      }
    }
    else
    if(  ("tlibversion".equals(_element))
       ||("jspversion" .equals(_element))
       ||("shortname"  .equals(_element))
       ||("uri"        .equals(_element)))
    {
      if(unite_!=null)
      {
	AglVecteur v=new AglVecteur(1,1);
	v.addElement(data_);
	getCommentaire(unite_).clefs.put("@jsp-"+_element,v);
      }
    }

    data_="";
  }

  public void attribute(String _attribut, String _valeur, boolean _specifie)
       throws Exception
  {
    if(_specifie)
      options_.put(_attribut,_valeur);
  }

  private String cleanData(String _s)
  {
    _s=_s.trim();
    _s=FuLib.replace(_s,"\n"," ");
    _s=FuLib.replace(_s,"\r"," ");
    _s=FuLib.replace(_s,"\t"," ");
    while(_s.indexOf("    ")>=0) _s=FuLib.replace(_s,"    "," ");
    while(_s.indexOf("   " )>=0) _s=FuLib.replace(_s,"   " ," ");
    while(_s.indexOf("  "  )>=0) _s=FuLib.replace(_s,"  "  ," ");
    return _s;
  }

  private AglCommentaireMulti getCommentaire(AglObjet _o)
  {
    AglCommentaireMulti r=null;

    for(int i=0;i<_o.commentaires.size();i++)
    {
      Object c=_o.commentaires.elementAt(i);
      if(c instanceof AglCommentaireMulti)
	r=(AglCommentaireMulti)c;
    }

    if(r==null)
    {
      r=new AglCommentaireMulti("");
      r.clefs.remove("@main");
      _o.commentaires.addElement(r);
    }

    return r;
  }
}
