/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseXmiHandler.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;
import com.memoire.mst.*;

import java.util.*;
import java.io.*;

public class AglParseXmiHandler
       extends MstHandlerBase
       implements MstXmlHandler
{
  Properties  options_ =new Properties();
  String      fichier_ =null;
  AglParseXmi parse_   =null;
  String      data_    ="";

  AglUnite      unite_;
  AglClasse     classe_;
  AglMethode    methode_;
  AglAttribut   attribut_;
  AglParametre  parametre_;
  AglVisibilite visibilite_;
  String        reference_;
  AglTypeAlias  datatype_;
  AglType       subtype_,supertype_;
  AglRelationBiOrientee relation_;

  public AglParseXmiHandler(AglParseXmi _parse, String _fichier)
  {
    fichier_=_fichier;
    parse_  =_parse;
  }

  private int ligne()
  {
    return parse_.ligne();
  }

  private void trace(String _s)
  {
    parse_.trace(_s);
  }

  public void error
  (String message, String systemId, int line, int column)
       throws Exception
  {
    parse_.syntaxe(message);
    super.error(message,systemId,line,column);
  }

  public void charData(char ch[], int start, int length) throws Exception
  {
    for(int i=0; i<length; i++)
      data_+=ch[start+i];
    // trace("Data: "+data_+"\n");
  }

  public void startElement(String _element) throws Exception
  {
    // trace(_element+":");
    for(Enumeration e1=options_.keys(); e1.hasMoreElements(); )
    {
      String k1=(String)e1.nextElement();
      Object v1=options_.get(k1);

      // trace(" "+k1+"=");
      // trace(""+v1);
    }
    // trace("\n");

    if("Model".equals(_element))
    {
      unite_=new AglUnite(fichier_);
      unite_.langage="Xmi";
      unite_.ligne=ligne();
      // trace("Unite: "+fichier_+"\n");
    }
    else
    if("Class".equals(_element))
    {
      classe_=new AglClasse(new AglNom("XmiTmpClass"));
      classe_.proprietaire=unite_;
      classe_.elementaire =false;
      classe_.ligne       =ligne();

      String ref=(String)options_.get("XMI.id");
      ref=ref.replace('.','_');
      AglType at=AglType.cherche(ref);
      if(at==null) new AglTypeAlias(ref,classe_);
      else ((AglTypeAlias)at).type=classe_;
    }
    else
    if("Interface".equals(_element))
    {
      classe_=new AglClasse(new AglNom("XmiTmpClass"));
      classe_.proprietaire=unite_;
      classe_.normale     =false;
      classe_.elementaire =false;
      classe_.ligne=ligne();

      String ref=(String)options_.get("XMI.id");
      ref=ref.replace('.','_');
      AglType at=AglType.cherche(ref);
      if(at==null) new AglTypeAlias(ref,classe_);
      else ((AglTypeAlias)at).type=classe_;
    }
    else
    if("DataType".equals(_element))
    {
      String ref=(String)options_.get("XMI.id");
      ref=ref.replace('.','_');
      AglType at=AglType.cherche(ref);
      if(at==null) datatype_=new AglTypeAlias(ref,AglTypePrimitif.tvoid);
      else datatype_=(AglTypeAlias)at;
      datatype_.ligne=ligne();
    }
    else
    if("Association".equals(_element))
    {
      relation_=new AglRelationBiOrientee
	("XmiTmpRelation","","");
      relation_.cardinaliteDebut="";
      relation_.cardinaliteFin  ="";
      relation_.ligne=ligne();
    }
    else
    if("Attribute".equals(_element))
    {
      attribut_=new AglAttribut("XmiTmpAttribute",AglTypePrimitif.tvoid);
      attribut_.ligne=ligne();
    }
    else
    if("Operation".equals(_element))
    {
      methode_=new AglMethode("XmiTmpOperation",AglTypePrimitif.tvoid);
      methode_.ligne=ligne();
    }
    else
    if("visibility".equals(_element))
    {
      visibilite_=new AglVisibilite();
      visibilite_.ligne=ligne();
      if("public".equals(options_.get("XMI.value")))
	visibilite_.vpublic=true;
      if("private".equals(options_.get("XMI.value")))
	visibilite_.vprivate=true;
      if("protected".equals(options_.get("XMI.value")))
	visibilite_.vprotected=true;
    }
    else
    if("isAbstract".equals(_element))
    {
      visibilite_.vabstract=("true".equals(options_.get("XMI.value")));
    }
    else
    if("isLeaf".equals(_element))
    {
      visibilite_.vfinal=("true".equals(options_.get("XMI.value")));
    }
    else
    if("ownerScope".equals(_element))
    {
      visibilite_.vstatic=!("instance".equals(options_.get("XMI.value")));
    }
    else
    if("concurrency".equals(_element))
    {
      visibilite_.voneway=!("sequential".equals(options_.get("XMI.value")));
    }
    else
    if("isPolymorphic".equals(_element))
    {
      visibilite_.vfinal=!("true".equals(options_.get("XMI.value")));
    }
    else
    if("XMI.reference".equals(_element))
    {
      reference_=(String)options_.get("target");
      if(reference_!=null) reference_=reference_.replace('.','_');
    }

    options_.clear();
  }
    
  public void endElement(String _element) throws Exception
  {
    data_=data_.trim();

    if("Model".equals(_element))
    {
      unite_=null;
    }
    else
    if("Class".equals(_element))
    {
      classe_=null;
    }
    else
    if("Interface".equals(_element))
    {
      classe_=null;
    }
    else
    if("DataType".equals(_element))
    {
      datatype_=null;
    }
    else
    if("Association".equals(_element))
    {
      relation_=null;
    }
    else
    if("Attribute".equals(_element))
    {
      if(classe_!=null)
	classe_.attributs.put(attribut_.signature(),attribut_);
      attribut_=null;
    }
    else
    if("Operation".equals(_element))
    {
      if(classe_!=null)
	classe_.methodes.put(methode_.signature(),methode_);
      methode_=null;
    }
    else
    if("generalization".equals(_element))
    {
      if((reference_!=null)&&(classe_!=null))
      {
	// System.err.println("### REF=|"+reference_+"|");
	AglType at=AglType.cherche(reference_);
	while(at instanceof AglTypeAlias)
	  at=((AglTypeAlias)at).type;

	if((classe_!=null)&&(at instanceof AglClasse))
	  classe_.meres.noms.addElement(at.nom);
      }      
    }
    else
    if("specification".equals(_element))
    {
      if((reference_!=null)&&(classe_!=null))
      {
	// System.err.println("### REF=|"+reference_+"|");
	AglType at=AglType.cherche(reference_);
	while(at instanceof AglTypeAlias)
	  at=((AglTypeAlias)at).type;

	if((classe_!=null)&&(at instanceof AglClasse))
	  classe_.interfaces.noms.addElement(at.nom);
      }      
    }
    /*
    else
    if("realization".equals(_element))
    {
      if((reference_!=null)&&(classe_!=null))
      {
	// System.err.println("### REF=|"+reference_+"|");
	AglType at=AglType.cherche(reference_);
	while(at instanceof AglTypeAlias)
	  at=((AglTypeAlias)at).type;

	if((classe_!=null)&&(at instanceof AglClasse))
	  ((AglClasse)at).interfaces.noms.addElement(classe_.nom);
      }      
    }
    else
    if("Generalization".equals(_element))
    {
      while(subtype_ instanceof AglTypeAlias)
	subtype_=((AglTypeAlias)subtype_).type;
      while(supertype_ instanceof AglTypeAlias)
	supertype_=((AglTypeAlias)supertype_).type;

      if(  (subtype_ instanceof AglClasse)
	 &&(supertype_ instanceof AglClasse))
      {
	AglListeNoms m=((AglClasse)subtype_).meres;
	m.noms.addElement(supertype_.nom);
      }

      subtype_  =null;
      supertype_=null;
    }
    */
    else
    if("name".equals(_element))
    {
      if((attribut_!=null)&&("XmiTmpAttribute".equals(attribut_.nom)))
	attribut_.nom=data_;
      else
      if((methode_!=null)&&("XmiTmpOperation".equals(methode_.nom)))
	methode_.nom=data_;
      else
      if((classe_!=null)&&(new AglNom("XmiTmpClass").equals(classe_.nom)))
	classe_.setNom(new AglNom(data_));
      else
      if(datatype_!=null)
      {
	AglType at=AglType.cherche(data_);
	if(at==null)
	{
	  at=new AglTypePrimitif(data_);
	  at.ligne=ligne();
	}
	datatype_.type=at;	
      }
      else
      if(relation_!=null)
      {
	if("XmiTmpRelation".equals(relation_.verbe))
	  relation_.verbe=data_;
	else
	if("".equals(relation_.roleDebut))
	  relation_.roleDebut=data_;
	else
	if("".equals(relation_.roleFin))
	  relation_.roleFin=data_;
      }
    }
    else
    if("initialValue".equals(_element))
    {
      if(attribut_!=null)
	if(!"".equals(data_))
	{
	  attribut_.initiale=new AglExpressionGenerique(data_);
	  attribut_.initiale.ligne=ligne();
	}
    }
    else
    if("type".equals(_element))
    {
      if(reference_!=null)
      {
	AglType at=AglType.cherche(reference_);

	if(at==null)
	{
	  at=new AglTypeAlias(reference_,AglTypePrimitif.tvoid);
	  at.ligne=ligne();
	}
	else
	{
	  while(at instanceof AglTypeAlias)
	    at=((AglTypeAlias)at).type;
	}

	if(attribut_!=null)
	  attribut_.type=at;
	else
	if(methode_!=null)
	  methode_.type=at;
	else
	if(relation_!=null)
        {
	  if(at instanceof AglTypeAlias)
	  {
	    AglRelation.table.removeElement(relation_);
	    relation_=null;
	  }
	  else
	  {
	    if("".equals(relation_.debut))
	      relation_.debut=at.nom.formeJava();
	    else
	    if("".equals(relation_.fin))
	      relation_.fin=at.nom.formeJava();
	  }
	}
      }
    }
    else
    if("subtype".equals(_element))
    {
      if(reference_!=null)
      {
	AglType at=AglType.cherche(reference_);
	if(at==null)
	{
	  at=new AglTypeAlias(reference_,AglTypePrimitif.tvoid);
	  at.ligne=ligne();
	}
	subtype_=at;
      }
    }
    else
    if("supertype".equals(_element))
    {
      if(reference_!=null)
      {
	AglType at=AglType.cherche(reference_);
	if(at==null)
	{
	  at=new AglTypeAlias(reference_,AglTypePrimitif.tvoid);
	  at.ligne=ligne();
	}
	supertype_=at;
      }
    }
    else
    if("multiplicity".equals(_element))
    {
      if(relation_!=null)
      {
	if("".equals(relation_.cardinaliteDebut))
	  relation_.cardinaliteDebut=data_;
	else
	if("".equals(relation_.cardinaliteFin))
	  relation_.cardinaliteFin=data_;
      }
    }
    else
    if("visibility".equals(_element))
    {
      if(attribut_!=null) attribut_.visibilite=visibilite_;
      else
      if(methode_!=null) methode_.visibilite=visibilite_;
      else
      if(classe_!=null) classe_.visibilite=visibilite_;
    }

    data_="";
  }

  public void attribute(String _attribut, String _valeur, boolean _specifie)
       throws Exception
  {
    if(_specifie)
      options_.put(_attribut,_valeur);
  }
}
