/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseXml.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.xml.*;

import java.util.*;
import java.io.*;

public class AglParseXml extends AglParse
{
  public AglParseXml()
  {
    if(AglType.cherche("String")==null)
      (new AglClasse(new AglNom("java.lang.String"))).elementaire=true;
  }

  public String extension()
  {
    return "xml";
  }

  public String[] extensions()
  {
    return new String[] { "xml", "XML" };
  }

  public String syntaxe()
  {
    return "Xml";
  }

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    fichier=_fichier;
  }

  public String get()
  {
    return "";
  }

  public void put(String _s)
  {
  }

  public int ligne()
  {
    return -1;
  }

  private String fichier="";
  public String fichier()
  {
    int i=fichier.lastIndexOf(File.separator);
    return fichier.substring(i+1);
  }

  protected String name(String _s)
  {
    if(_s.equals(_s.toUpperCase())) _s=_s.toLowerCase();
    if(_s.equals(_s.toLowerCase())) _s=_s.toLowerCase();
    _s=_s.replace('-','_');

    String  r="";
    boolean b=false;
    for(int i=0;i<_s.length();i++)
    {
      char c=_s.charAt(i);
      if(c=='_')
      {
	b=true;
      }
      else
      {
	if(b) r+=(""+c).toUpperCase();
	else  r+=c;
	b=false;
      }
    }
    return r;
  }

  protected String attributeName(String _s)
  {
    _s=name(_s.replace(':','_'));
    return _s.substring(0,1).toLowerCase()+_s.substring(1);
  }

  protected AglNom className(String _s)
  {
    _s=name(_s);
    int i=_s.lastIndexOf(':')+1;
    return new AglNom(_s.substring(0,i)
		      +_s.substring(i,i+1).toUpperCase()
		      +_s.substring(i+1),":");
  }

  protected AglClasse analyse(XmlNode _n, AglUnite _u)
  {
    AglNom cn=className(_n.tag_);

    AglClasse c=(AglClasse)AglType.cherche(cn);
    if(c==null)
    {
      c=new AglClasse(cn);
      c.elementaire=false;
      c.proprietaire=_u;
      //System.err.println("CREE "+c);
    }

    String s=_n.getText().trim();
    if(!"".equals(s))
    {
      AglType t=AglType.cherche("String");
      try { new Integer(s); t=AglTypePrimitif.tinteger; }
      catch(Exception ex) { }
      try { new Double(s); t=AglTypePrimitif.tdouble; }
      catch(Exception ex) { }
      AglAttribut aa=new AglAttribut("value",t);
      aa.visibilite.vpublic=true;
      c.attributs.put(aa.nom,aa);
    }

    XmlNode[] f=_n.getNodes();
    if(f.length>0)
    {
      for(int i=0;i<f.length;i++)
      {
	int array=0;
	for(int j=0;j<f.length;j++)
	  if(f[j].tag_.equals(f[i].tag_))
	    array++;

	AglType t=analyse(f[i],_u);
	String an=attributeName(f[i].tag_);
	if(array>1) t=new AglTypeTableau(t,null);
	AglAttribut aa=new AglAttribut(an,t);
	aa.visibilite.vpublic=true;
	c.attributs.put(aa.nom,aa);
      }
    }

    return c;
  }

  // Parsing

  public void parseFichier() throws Exception
  {
    XmlParser p=new XmlParser(fichier,XmlParser.XML);
    XmlReader r=new XmlReader(XmlParser.XML);
    p.setXmlListener(r);
    p.parse();
    AglUnite u=new AglUnite(fichier);
    analyse(r.getTree(),u).visibilite.vpublic=true;
  }

  public AglUnite parseUnite() throws Exception
  {
    return null;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    return null;
  }

  public AglNom parseNom() throws Exception
  {
    return null;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    return null;
  }

  public AglType parseType() throws Exception
  {
    return null;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    return null;
  }
  
  public AglImport parseImport() throws Exception
  {
    return null;
  }

  public AglTypeAlias parseTypeDef() throws Exception
  {
    return null;
  }

  public AglStruct parseStruct() throws Exception
  {
    return null;
  }

  public AglClasse parseClasse() throws Exception
  {
    return null;
  }

  public AglListeNoms parseMothers() throws Exception
  {
    return null;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    return null;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    return null;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    return null;
  }

  public AglMethode parseMethode() throws Exception
  {
    return null;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    return null;
  }

  public AglVecteur parseParametres() throws Exception
  {
    return null;
  }

  public AglVecteur parseArgs() throws Exception
  {
    return null;
  }

  public AglBloc parseBloc() throws Exception
  {
    erreur("Il n'y a pas de bloc en Xml ("+get()+")");
    return null;
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    erreur("Il n'y a pas de bloc-multiple en Xml ("+get()+")");
    return null;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    erreur("Il n'y a pas de bloc-simple en Xml ("+get()+")");
    return null;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    erreur("Il n'y a pas de bloc-virgule en Xml ("+get()+")");
    return null;
  }

// Expressions

  public AglExpression parseExpression() throws Exception
  {
    erreur("Il n'y a pas d'expression en Xml ("+get()+")");
    return null;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    erreur("Il n'y a pas d'expression generique en Xml ("+get()+")");
    return null;
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite() throws Exception
  {
    erreur("Il n'y a pas d'expression unaire droite en Xml ("+get()+")");
    return null;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    erreur("Il n'y a pas d'expression binaire en Xml ("+get()+")");
    return null;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    erreur("Il n'y a pas d'expression de fonction en Xml ("+get()+")");
    return null;
  }

  public AglExpression parseExpressionNew() throws Exception
  {
    erreur("Il n'y a pas d'expression new en Xml ("+get()+")");
    return null;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    erreur("Il n'y a pas d'expression parenthse en Xml ("+get()+")");
    return null;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    erreur("Il n'y a pas d'expression de typage en Xml ("+get()+")");
    return null;
  }

// Instructions

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    erreur("Il n'y a pas d'instruction gnrique en Xml ("+get()+")");
    return null;
  }

  public AglInstructionAffectation parseInstructionAffectation(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction d'affectation en Xml ("+get()+")");
    return null;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    erreur("Il n'y a pas d'instruction de bloc en Xml ("+get()+")");
    return null;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    erreur("Il n'y a pas d'instruction catch en Xml ("+get()+")");
    return null;
  }

  public AglVecteur parseInstructionDefVar() throws Exception
  {
    erreur("Il n'y a pas d'instruction de dfinition de variable en Xml ("+get()+")");
    return null;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction do/while en Xml ("+get()+")");
    return null;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction expressive en Xml ("+get()+")");
    return null;
  }

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    erreur("Il n'y a pas d'instruction if en Xml ("+get()+")");
    return null;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    erreur("Il n'y a pas de procedure en Java ("+get()+")");
    return null;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    erreur("Il n'y a pas d'instruction return en Xml ("+get()+")");
    return null;
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    erreur("Il n'y a pas d'instruction switch en Xml ("+get()+")");
    return null;
  }  

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    erreur("Il n'y a pas d'instruction throw en Xml ("+get()+")");
    return null;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    erreur("Il n'y a pas d'instruction try en Xml ("+get()+")");
    return null;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction while en Xml ("+get()+")");
    return null;
  }
}
