/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseXsd.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.mst.*;

import java.util.*;
import java.io.*;

public class AglParseXsd extends AglParse
{
  private MstXmlParser parser_;

  //AglParseXsdHandler handler_;

  public AglParseXsd()
  {
    if(AglType.cherche("Boolean")==null)
      (new AglClasse(new AglNom("java.lang.Boolean"))).elementaire=true;
    if(AglType.cherche("Byte")==null)
      (new AglClasse(new AglNom("java.lang.Byte"))).elementaire=true;
    if(AglType.cherche("Double")==null)
      (new AglClasse(new AglNom("java.lang.Double"))).elementaire=true;
    if(AglType.cherche("Float")==null)
      (new AglClasse(new AglNom("java.lang.Float"))).elementaire=true;
    if(AglType.cherche("Integer")==null)
      (new AglClasse(new AglNom("java.lang.Integer"))).elementaire=true;
    if(AglType.cherche("Long")==null)
      (new AglClasse(new AglNom("java.lang.Long"))).elementaire=true;
    if(AglType.cherche("Short")==null)
      (new AglClasse(new AglNom("java.lang.Short"))).elementaire=true;

    if(AglType.cherche("Date")==null)
      (new AglClasse(new AglNom("java.util.Date"))).elementaire=true;
    if(AglType.cherche("Integer")==null)
      (new AglClasse(new AglNom("java.lang.Integer"))).elementaire=true;
    if(AglType.cherche("Locale")==null)
      (new AglClasse(new AglNom("java.util.Locale"))).elementaire=true;
    if(AglType.cherche("Number")==null)
      (new AglClasse(new AglNom("java.lang.Number"))).elementaire=true;
    if(AglType.cherche("Object")==null)
      (new AglClasse(new AglNom("java.lang.Object"))).elementaire=true;
    if(AglType.cherche("String")==null)
      (new AglClasse(new AglNom("java.lang.String"))).elementaire=true;
    if(AglType.cherche("URL")==null)
      (new AglClasse(new AglNom("java.net.URL"))).elementaire=true;
    if(AglType.cherche("Vector")==null)
      (new AglClasse(new AglNom("java.util.Vector"))).elementaire=true;

    // considere les types Xsd comme leur equivalent Java

    createAlias("xsd_anyType"           ,"Object" );
    createAlias("xsd_string"            ,"String" );
    createAlias("xsd_normalizedString"  ,"String" );
    createAlias("xsd_token"             ,"String" );
    createAlias("xsd_byte"              ,"Byte"   );
    createAlias("xsd_unsignedByte"      ,"Byte"   );
    //createAlias("xsd_base64Binary"    ,""       );
    //createAlias("xsd_hexBinary"       ,""       );
    createAlias("xsd_integer"           ,"Integer");
    createAlias("xsd_positiveInteger"   ,"Integer");
    createAlias("xsd_negativeInteger"   ,"Integer");
    createAlias("xsd_nonNegativeInteger","Integer");
    createAlias("xsd_nonPositiveInteger","Integer");
    createAlias("xsd_int"               ,"Integer");
    createAlias("xsd_unsignedInt"       ,"Integer");
    createAlias("xsd_long"              ,"Integer");
    createAlias("xsd_unsignedLong"      ,"Integer");
    createAlias("xsd_short"             ,"Integer");
    createAlias("xsd_unsignedShort"     ,"Integer");
    createAlias("xsd_decimal"           ,"Float"  );
    createAlias("xsd_number"            ,"Float"  ); // deprecated
    createAlias("xsd_float"             ,"Float"  );
    createAlias("xsd_double"            ,"Double" );
    createAlias("xsd_boolean"           ,"Boolean");
    createAlias("xsd_time"              ,"Date"   );
    createAlias("xsd_timeDate"          ,"Date"   );
    createAlias("xsd_duration"          ,"Date"   );
    createAlias("xsd_date"              ,"Date"   );
    //createAlias("xsd_g*"              ,""       );
    //createAlias("xsd_*Name"           ,""       );
    createAlias("xsd_anyURI"            ,"URL"    );
    createAlias("xsd_language"          ,"Locale" );
    createAlias("xsd_ID"                ,"String" );
    createAlias("xsd_IDREF"             ,"String" );
    createAlias("xsd_IDREFS"            ,"Vector" );
    createAlias("xsd_ENTITY"            ,"String" );
    createAlias("xsd_ENTITIES"          ,"Vector" );
    //createAlias("xsd_NOTATION"        ,""       );
    createAlias("xsd_NMTOKEN"           ,"String" );
    createAlias("xsd_NMTOKENS"          ,"Vector" );
  }

  private void createAlias(String _xsd,String _agl)
  {
    if(AglType.cherche(_xsd)==null)
      new AglTypeAlias(_xsd,AglType.cherche(_agl),"Xsd");
  }

  public String extension()
  {
    return "xsd";
  }

  public String[] extensions()
  {
    return new String[] { "xsd", "XSD" };
  }

  public String syntaxe()
  {
    return "Xml";
  }

  /*
  private int _lineno_=1;
  public int st_lineno()
  {
    return _lineno_++;
  }
  */

  public void initStream(InputStream _input, String _fichier) throws Exception
  {
    fichier=_fichier;
  }

  public String get()
  {
    return "";
  }

  public void put(String _s)
  {
  }

  private int pp=0;
  public int ligne()
  {
    int r=-1;
    if(parser_!=null) r=parser_.getLineNumber();

    if(MaxLines!=0)
    {
      int p=100*r/MaxLines;
      if(p!=pp) progres(p);
      pp=p;
    }
    return r;
  }

  private String fichier="";
  public String fichier()
  {
    int i=fichier.lastIndexOf(File.separator);
    return fichier.substring(i+1);
  }

  // Parsing

  public void parseFichier() throws Exception
  {
    AglParseXsdHandler handler=new AglParseXsdHandler(this,fichier);
    parser_=new MstXmlParser();
    parser_.setHandler(handler);

    String url=fichier;
    if(url.indexOf(':')<0) url="file://"+new File(url).getAbsolutePath();
    try { parser_.parse(url,null,null,null); }
    catch(FileNotFoundException ex)
    {
      erreur("Fichier "+ex.getMessage()+" introuvable");
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      erreur(ex.getClass().getName()+" "+ex.getMessage());
    }
  }

  public AglUnite parseUnite() throws Exception
  {
    return null;
  }

  public AglVisibilite parseVisibilite() throws Exception
  {
    return null;
  }

  public AglNom parseNom() throws Exception
  {
    return null;
  }

  public AglListeNoms parseListeNoms() throws Exception
  {
    return null;
  }

  public AglType parseType() throws Exception
  {
    return null;
  }

  public AglPaquet parsePaquet() throws Exception
  {
    return null;
  }
  
  public AglImport parseImport() throws Exception
  {
    return null;
  }

  public AglTypeAlias parseTypeDef() throws Exception
  {
    return null;
  }

  public AglStruct parseStruct() throws Exception
  {
    return null;
  }

  public AglClasse parseClasse() throws Exception
  {
    return null;
  }

  public AglListeNoms parseMothers() throws Exception
  {
    return null;
  }

  public AglListeNoms parseInterfaces() throws Exception
  {
    return null;
  }

  public AglListeNoms parseExceptions() throws Exception
  {
    return null;
  }

  public AglConstructeur parseConstructeur() throws Exception
  {
    return null;
  }

  public AglMethode parseMethode() throws Exception
  {
    return null;
  }

  public AglVecteur parseAttribut() throws Exception
  {
    return null;
  }

  public AglVecteur parseParametres() throws Exception
  {
    return null;
  }

  public AglVecteur parseArgs() throws Exception
  {
    return null;
  }

  public AglBloc parseBloc() throws Exception
  {
    erreur("Il n'y a pas de bloc en Xsd ("+get()+")");
    return null;
  }

  public AglBloc parseBlocMultiple() throws Exception
  {
    erreur("Il n'y a pas de bloc-multiple en Xsd ("+get()+")");
    return null;
  }

  public AglBloc parseBlocSimple() throws Exception
  {
    erreur("Il n'y a pas de bloc-simple en Xsd ("+get()+")");
    return null;
  }

  public AglBloc parseBlocVirgule() throws Exception
  {
    erreur("Il n'y a pas de bloc-virgule en Xsd ("+get()+")");
    return null;
  }

// Expressions

  public AglExpression parseExpression() throws Exception
  {
    erreur("Il n'y a pas d'expression en Xsd ("+get()+")");
    return null;
  }

  public AglExpressionGenerique parseExpressionGenerique() throws Exception
  {
    erreur("Il n'y a pas d'expression generique en Xsd ("+get()+")");
    return null;
  }

  public AglExpressionUnaireDroite parseExpressionUnaireDroite() throws Exception
  {
    erreur("Il n'y a pas d'expression unaire droite en Xsd ("+get()+")");
    return null;
  }

  public AglExpressionBinaire parseExpressionBinaire() throws Exception
  {
    erreur("Il n'y a pas d'expression binaire en Xsd ("+get()+")");
    return null;
  }

  public AglExpressionFonction parseExpressionFonction() throws Exception
  {
    erreur("Il n'y a pas d'expression de fonction en Xsd ("+get()+")");
    return null;
  }

  public AglExpression parseExpressionNew() throws Exception
  {
    erreur("Il n'y a pas d'expression new en Xsd ("+get()+")");
    return null;
  }

  public AglExpressionParenthesee parseExpressionParenthesee() throws Exception
  {
    erreur("Il n'y a pas d'expression parenthse en Xsd ("+get()+")");
    return null;
  }

  public AglExpression parseExpressionTypage() throws Exception
  {
    erreur("Il n'y a pas d'expression de typage en Xsd ("+get()+")");
    return null;
  }

// Instructions

  public AglInstructionGenerique parseInstructionGenerique() throws Exception
  {
    erreur("Il n'y a pas d'instruction gnrique en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionAffectation parseInstructionAffectation(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction d'affectation en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionBloc parseInstructionBloc() throws Exception
  {
    erreur("Il n'y a pas d'instruction de bloc en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionCatch parseInstructionCatch() throws Exception
  {
    erreur("Il n'y a pas d'instruction catch en Xsd ("+get()+")");
    return null;
  }

  public AglVecteur parseInstructionDefVar() throws Exception
  {
    erreur("Il n'y a pas d'instruction de dfinition de variable en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionDoWhile parseInstructionDoWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction do/while en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionExpression parseInstructionExpression(AglExpression eg) throws Exception
  {
    erreur("Il n'y a pas d'instruction expressive en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionIf parseInstructionIf() throws Exception
  {
    erreur("Il n'y a pas d'instruction if en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionProcedure parseInstructionProcedure() throws Exception
  {
    erreur("Il n'y a pas de procedure en Java ("+get()+")");
    return null;
  }  

  public AglInstructionReturn parseInstructionReturn() throws Exception
  {
    erreur("Il n'y a pas d'instruction return en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionSwitch parseInstructionSwitch() throws Exception
  {
    erreur("Il n'y a pas d'instruction switch en Xsd ("+get()+")");
    return null;
  }  

  public AglInstructionThrow parseInstructionThrow() throws Exception
  {
    erreur("Il n'y a pas d'instruction throw en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionTry parseInstructionTry() throws Exception
  {
    erreur("Il n'y a pas d'instruction try en Xsd ("+get()+")");
    return null;
  }

  public AglInstructionWhile parseInstructionWhile() throws Exception
  {
    erreur("Il n'y a pas d'instruction while en Xsd ("+get()+")");
    return null;
  }
}
