/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglParseXsdHandler.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.fu.*;
import com.memoire.mst.*;

import java.io.*;
import java.util.*;

public class AglParseXsdHandler
       extends MstHandlerBase
       implements MstXmlHandler
{
  Properties  options_ =new Properties();
  String      fichier_ =null;
  AglParseXsd parse_   =null;
  String      data_    ="";

  AglUnite              unite_;
  AglClasse             classe_;
  AglAttribut           attribut_;

  //AglVisibilite visibilite_;
  //String        reference_;
  //AglTypeAlias  datatype_;
  //AglType               subtype_,supertype_;
  //AglRelationBiOrientee relation_;

  int anonymes=0;
  Hashtable refs    =new Hashtable();
  Stack     classes_=new Stack();

  public AglParseXsdHandler(AglParseXsd _parse, String _fichier)
  {
    fichier_=_fichier;
    parse_  =_parse;
  }

  private int ligne()
  {
    return parse_.ligne();
  }

  private void trace(String _s)
  {
    parse_.trace(_s);
  }

  public void error
  (String message, String systemId, int line, int column)
       throws Exception
  {
    parse_.syntaxe(message);
    super.error(message,systemId,line,column);
  }

  public void charData(char ch[], int start, int length) throws Exception
  {
    for(int i=0; i<length; i++)
      data_+=ch[start+i];

    if(!"".equals(data_.trim()))
      trace("DATA "+data_.trim());
  }

  public void startElement(String _element) throws Exception
  {
    int i=_element.lastIndexOf(':');
    if(i>=0) _element=_element.substring(i+1);

    String t="OPEN TAG IS "+_element;
    for(Enumeration e1=options_.keys(); e1.hasMoreElements(); )
    {
      String k1=(String)e1.nextElement();
      Object v1=options_.get(k1);

      t+=" "+k1+"="+v1;
    }
    trace(t);

    if("schema".equals(_element))
    {
      unite_=new AglUnite(fichier_);
      unite_.langage="Xsd";
      unite_.ligne=ligne();
      trace("Unite: "+fichier_);
    }
    else
    if("complexType".equals(_element))
    {
      String name=(String)options_.get("name");
      if(name==null) name="anonymous"+(++anonymes);

      classe_=new AglClasse(new AglNom(name));
      classe_.proprietaire=unite_;
      classe_.elementaire =false;
      classe_.ligne       =ligne();
      classe_.visibilite.vpublic=true;
      classes_.push(classe_);

      if((attribut_!=null)&&(attribut_.type==AglTypePrimitif.tvoid))
	attribut_.type=classe_;
    }
    else
    if("simpleType".equals(_element))
    {
      String name=(String)options_.get("name");
      if(name==null) name="anonymous"+(++anonymes);

      classe_=new AglClasse(new AglNom(name));
      classe_.proprietaire=unite_;
      classe_.elementaire =false;
      classe_.complexe    =false;
      classe_.ligne       =ligne();
      classe_.visibilite.vpublic=true;
      classes_.push(classe_);

      if((attribut_!=null)&&(attribut_.type==AglTypePrimitif.tvoid))
	attribut_.type=classe_;
    }
    else
    if("restriction".equals(_element))
    {
      String base=(String)options_.get("base");

      if(base!=null) base=base.replace(':','_');
      else           base="void";

      AglType at=AglType.cherche(base);
      if((at!=null)&&(classe_!=null))
	classe_.meres.noms.addElement(at.nom);
    }
    else
    if("pattern".equals(_element))
    {
      String value=(String)options_.get("value");

      if(classe_!=null)
      {
	attribut_=new AglAttribut("PATTERN",AglType.cherche("String"));
	attribut_.ligne=ligne();
	attribut_.visibilite.vpublic=true;
	attribut_.visibilite.vstatic=true;
	attribut_.visibilite.vfinal=true;
	attribut_.initiale=new AglExpressionGenerique('"'+value+'"');
	classe_.attributs.put(attribut_.signature(),attribut_);
      }
    }
    else
    if("element".equals(_element)||"attribute".equals(_element))
    {
      String name=(String)options_.get("name");
      String type=(String)options_.get("type");

      if(type!=null) type=type.replace(':','_');
      else           type="void";

      String ref=(String)options_.get("ref");
      if(ref!=null)
      {
	name=ref;
	type=(String)refs.get(ref);
      }

      if(name!=null)
      {
	if(classe_==null)
	{
	  refs.put(name,type);
	}
	else
	{
	  AglType at=AglType.cherche(type);
	  if(at==null) at=new AglClasse(new AglNom(type));
	  attribut_=new AglAttribut(name,at);
	  attribut_.ligne=ligne();
	  attribut_.visibilite.vpublic=true;
	  classe_.attributs.put(attribut_.signature(),attribut_);

	  String fixed=(String)options_.get("fixed");
	  if(fixed!=null)
	  {
	    attribut_.initiale=new AglExpressionGenerique('"'+fixed+'"');
	    attribut_.visibilite.vfinal=true;
	  }
	}
      }
    }

    options_.clear();
  }
    
  public void endElement(String _element) throws Exception
  {
    data_=data_.trim();

    int i=_element.lastIndexOf(':');
    if(i>=0) _element=_element.substring(i+1);

    String t="CLOSE TAG IS "+_element;
    trace(t);

    if("schema".equals(_element))
    {
      unite_=null;
    }
    else
    if("complexType".equals(_element))
    {
      classes_.pop();

      if(classes_.size()>0) classe_=(AglClasse)classes_.peek();
      else                  classe_=null;
    }
    else
    if("simpleType".equals(_element))
    {
      classes_.pop();

      if(classes_.size()>0) classe_=(AglClasse)classes_.peek();
      else                  classe_=null;
    }
    else
    if("element".equals(_element)||"attribute".equals(_element))
    {
      attribut_=null;
    }

    data_="";
  }

  public void attribute(String _attribut, String _valeur, boolean _specifie)
       throws Exception
  {
    if(_specifie)
      options_.put(_attribut,_valeur);
  }
}
