/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglPatron.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.io.*;
import java.util.*;

public class AglPatron extends AglObjet
{
  public static AglTableTriee table;

  public static void reinit(boolean _nouveau)
  {
    if(_nouveau||(table==null))
    {
      AglOptions.terminal.trace("$$$ AglPatron");
      table=new AglTableTriee();
    }
  }

  private static String genereNom(String _root)
  {
    reinit(false);

    String r="";
    int    n=1;
    do
    {
      r=_root+"_"+n;
      n++;
    } while(table.get(r)!=null);

    return r;
  }

  public String              nom;
  public AglPatronType       type;
  public AglTableOrdonnee roles;

  public AglPatron() // yapod
  {
    this(null,"<<patron>>");
  }

  public AglPatron(AglPatronType _type)
  {
    this(_type,genereNom(_type.nom));
  }

  public AglPatron(AglPatronType _type,String _nom)
  {
    nom  =_nom;
    type =_type;
    roles=new AglTableOrdonnee();

    AglOptions.terminal.trace("Create pattern: "+nom);
    if(table.get(nom)!=null)
      AglOptions.terminal.trace("Conflict    : "+nom);

    reinit(false);
    table.put(nom,this);
  }

  public String toString()
  {
    return "PATRON("+nom+")";
  }

  private final static class Lecteur
  {
    private StreamTokenizer st;

    Lecteur(String _text)
    {
      try
      {
	st=new StreamTokenizer(new StringReader(_text));

	st.resetSyntax();
	st.eolIsSignificant(false);
	st.quoteChar('"');
	st.quoteChar('\'');
	st.slashSlashComments(true);
	st.slashStarComments(true);
	st.wordChars('a','z');
	st.wordChars('A','Z');
	st.wordChars('0','9');
	st.wordChars('_','_');
	st.wordChars(129,255);
	st.whitespaceChars(' ',' ');
	st.whitespaceChars('\t','\t');
	st.whitespaceChars('\n','\n');
	st.whitespaceChars('\r','\r');
      }
      catch(Exception ex)
      {
	st=null;
 	AglOptions.terminal.erreur(""+ex+" [AglPatron.Lecteur.init]");
      }
    }

    String get()
    {
      String r="";

      if(st!=null)
      {
	try
	{
	  st.nextToken();

	  switch(st.ttype)
	  {
            case StreamTokenizer.TT_EOL   : r="$";               break;
            case StreamTokenizer.TT_EOF   : r="";                break;
            case StreamTokenizer.TT_WORD  : r=st.sval;           break;
            case StreamTokenizer.TT_NUMBER: r=""+st.nval;        break;
            default                       : r=""+(char)st.ttype; break;
	  }
	}
	catch(Exception ex)
	{
	  AglOptions.terminal.erreur(""+ex+" [AglPatron.Lecteur.get]");
	}
      }

      //System.err.println("### R="+r);
      return r;
    }

    void syntaxe(String s)
      { AglOptions.terminal.syntaxe(s+" [line "+st.lineno()+"]"); }

    void trace(String s)
      { AglOptions.terminal.trace(s+" [line "+st.lineno()+"]"); }
  }

  public static AglPatron construirePatron(String _s)
  {
    AglPatron r=null;

    try
    {
      Lecteur         lect =new Lecteur(_s);
      String          nt   =lect.get();
      AglPatronType   type =(AglPatronType)AglPatronType.table.get(nt);

      AglTableOrdonnee roles=new AglTableOrdonnee();
      AglTableOrdonnee props=new AglTableOrdonnee();

      if(type==null)
	throw new Exception("Unreconized pattern name: "+nt);

      String s=lect.get();

      if(!"(".equals(s))
	throw new Exception("missing (");

      while(!")".equals(s))
      {
	String nc=lect.get();
	if("".equals(nc))
	  throw new Exception("missing class name");

	if(!":".equals(lect.get()))
	  throw new Exception("missing :");

	String nr=lect.get();
	if("".equals(nr))
	  throw new Exception("missing role name");

	s=lect.get();
	if(!(",".equals(s)||")".equals(s)))
	  throw new Exception("missing , or )");

	roles.put(nr,nc);
      }

      String nom=null;

      s=lect.get();
      if(!"".equals(s)&&!"{".equals(s))
	throw new Exception("missing {");
      if("{".equals(s))
      {
	while(!"}".equals(s))
	{
	  String np=lect.get();
	  if("".equals(np))
	    throw new Exception("missing property name");

	  if(!"=".equals(lect.get()))
	    throw new Exception("missing =");

	  String vp=lect.get();
	  if("".equals(vp))
	    throw new Exception("missing property value");

	  if(!";".equals(lect.get()))
	    throw new Exception("missing ;");

	  props.put(np,vp);
	  s=lect.get();
	}

	nom=(String)props.get("name");
      }

      if(nom==null) nom=genereNom(type.nom);
      r=new AglPatron(type,nom);
      r.roles=roles;
    }
    catch(Exception ex)
    {
      AglOptions.terminal.syntaxe(ex.getMessage());
      r=null;
    }

    return r;
  }

  public static void construirePatrons()
  {
    int p=0;
    for(Enumeration e1=AglType.table.elements(); e1.hasMoreElements(); )
    {
      p++;
      AglOptions.terminal.progres(100*p/AglType.table.size());

      AglType ot=(AglType)e1.nextElement();

      if(ot.commentaires!=null)
      	for(Enumeration e5=ot.commentaires.elements();
	    e5.hasMoreElements(); )
	{
	  Object ocm=e5.nextElement();
	  if(ocm instanceof AglCommentaireMulti)
	  {
	    AglCommentaireMulti cm=(AglCommentaireMulti)ocm;
	    AglVecteur pc=(AglVecteur)cm.clefs.get("@pattern");
	    if(pc!=null)
	    {
	      for(int j=0;j<pc.size();j++)
	      {
		String sc=(String)pc.elementAt(j);;
		construirePatron(sc);
	      }
	    }
	  }
	}
    }
  }
}
