/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglPatronType.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;
import java.io.*;

public class AglPatronType extends AglObjet
{
  private final static class Chargeur
  {
    private StreamTokenizer st;

    Chargeur(String _fichier)
    {
      try
      {
	st=new StreamTokenizer(new InputStreamReader
			       (AglPatronType.class.getResourceAsStream
				("patterns.txt")));
	st.resetSyntax();
	st.eolIsSignificant(false);
	st.quoteChar('"');
	st.quoteChar('\'');
	st.slashSlashComments(true);
	st.slashStarComments(true);
	st.wordChars('a','z');
	st.wordChars('A','Z');
	st.wordChars('0','9');
	st.wordChars('_','_');
	st.wordChars(129,255);
	st.whitespaceChars(' ',' ');
	st.whitespaceChars('\t','\t');
	st.whitespaceChars('\n','\n');
	st.whitespaceChars('\r','\r');
      }
      catch(Exception ex)
      {
	st=null;
 	System.err.println("Can n't find patterns.txt");
      }
    }

    String get()
    {
      String r="";

      if(st!=null)
      {
	try
	{
	  st.nextToken();

	  switch(st.ttype)
	  {
            case StreamTokenizer.TT_EOL   : r="$";               break;
            case StreamTokenizer.TT_EOF   : r="";                break;
            case StreamTokenizer.TT_WORD  : r=st.sval;           break;
            case StreamTokenizer.TT_NUMBER: r=""+st.nval;        break;
            default                       : r=""+(char)st.ttype; break;
	  }
	}
	catch(Exception ex)
	{
	  System.err.println("Error in pattern.txt");
	}
      }

      return r;
    }

    void syntaxe(String s)
      { AglOptions.terminal.syntaxe(s+" [line "+st.lineno()+"]"); }

    void trace(String s)
      { AglOptions.terminal.trace(s+" [line "+st.lineno()+"]"); }
  }

  public static AglTableTriee table;
  public static String[]      categories;

  static
  {
    categories=new String[]
    {
      "Design","GRASP","GUI Design","Organizational Coding",
      "Optimization Coding","Robustness Coding","Testing",
      "Others"
    };

    Chargeur chargeur=new Chargeur("patterns.txt");
    chargeur.trace("$$$ AglPatronType");
    table=new AglTableTriee();

    String r=chargeur.get();
    while(!"".equals(r))
    {
      String nom=r;
      r=chargeur.get();
      if(!"(".equals(r)) chargeur.syntaxe("( needed and not "+r);
      AglVecteur v=new AglVecteur(0,1);
      do
      {
	r=chargeur.get();
	if(")".equals(r)) break;
	if(",".equals(r))
	  chargeur.syntaxe("<id> needed and not ,");
	v.addElement(r);

	r=chargeur.get();
	if(!",".equals(r)&&!")".equals(r))
	  chargeur.syntaxe(", or ) needed and not "+r);

      } while(!")".equals(r));

      int    categorie  =categories.length-1;
      String description="";

      r=chargeur.get();
      if("{".equals(r))
      {
	r=chargeur.get();
	while(!"}".equals(r))
	{
	  if("category".equals(r))
	  {
	    r=chargeur.get();
	    if(!"=".equals(r))
	      chargeur.syntaxe("= needed and not "+r);
	    r=chargeur.get();
	    try
	      { categorie=new Integer(r).intValue(); }
	    catch(Exception ex)
	      { chargeur.syntaxe("<integer> needed and not "+r); }
	    r=chargeur.get();
	    if(!";".equals(r))
	      chargeur.syntaxe("; needed and not "+r);
	  }
	  if("description".equals(r))
	  {
	    r=chargeur.get();
	    if(!"=".equals(r))
	      chargeur.syntaxe("= needed and not "+r);
	    r=chargeur.get();
	    description=r;
	    r=chargeur.get();
	    if(!";".equals(r))
	      chargeur.syntaxe("; needed and not "+r);
	  }

	  r=chargeur.get();
	}
      }
      else
      if(!";".equals(r))
	chargeur.syntaxe("; needed and not "+r);

      String[] roles=new String[v.size()];
      for(int i=0;i<v.size();i++)
	roles[i]=(String)v.elementAt(i);

      AglPatronType m=new AglPatronType(nom,roles);
      m.categorie=categorie;
      m.description=description;

      r=chargeur.get();
    }
  }

  public String   nom;
  public String[] roles;
  public int      categorie;
  public String   description;

  public AglPatronType() // yapod
  {
    this("<<patron-type>>",new String[0]);
  }

  public AglPatronType(String _nom, String[] _roles)
  {
    nom  =_nom;
    roles=_roles;
    AglOptions.terminal.trace("Create pattern-type: "+nom+
			      " ("+roles.length+" roles)");

    if(table.get(nom)!=null)
      AglOptions.terminal.trace("Conflict   : "+nom);

    table.put(nom,this);
  }

  public String toString()
  {
    return "PATRONTYPE("+nom+")";
  }
}
