/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglProjet.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;
import com.memoire.yapod.*;

import java.io.*;
import java.util.*;

public class AglProjet
       implements Serializable
{
  public String        nom;
  public AglTableTriee types;
  public AglTableTriee patrons;
  public AglVecteur    relations;
  public AglTableTriee unites;
  public AglTableTriee paquets;

  public static void reinit(boolean _nouveau)
  {
    AglType    .reinit(_nouveau);
    AglPatron  .reinit(_nouveau);
    AglRelation.reinit(_nouveau);
    AglUnite   .reinit(_nouveau);
    AglPaquet  .reinit(_nouveau);
  }

  public AglProjet()
  {
    types    =AglType.table;
    patrons  =AglPatron.table;
    relations=AglRelation.table;
    unites   =AglUnite.table;
    paquets  =AglPaquet.table;

    /*
    if(unites.size()==1)
    {
      AglUnite u=(AglUnite)(unites.elements().nextElement());
      String   s=u.fichier;
      int      i=s.lastIndexOf(File.separator);
      if(i>=0) s=s.substring(i+1);
      int      j=s.lastIndexOf('.');
      if(j>=0) s=s.substring(0,j);
      nom=s;      
    }
    else
    */

    nom="projet";
    AglOptions.terminal.trace("Create project: "+nom);
  }

  public String toString()
  {
    return "PROJET("+nom+")";
  }

  public static void save(AglProjet _p, String _f)
       throws IOException
  {
         if(_f.toLowerCase().endsWith(".ser")) saveYapodSer(_p,_f);
    else if(_f.toLowerCase().endsWith(".sgz")) saveYapodSgz(_p,_f);
    else if(_f.toLowerCase().endsWith(".xml")) saveYapodXml(_p,_f);
    else                                       saveYapodXgz(_p,_f);
  }
  
  public static AglProjet load(String _f)
       throws IOException, ClassNotFoundException
  {
    AglProjet r=null;

         if(_f.toLowerCase().endsWith(".ser")) r=loadYapodSer(_f);
    else if(_f.toLowerCase().endsWith(".sgz")) r=loadYapodSgz(_f);
    else if(_f.toLowerCase().endsWith(".xml")) r=loadYapodXml(_f);
    else                                       r=loadYapodXgz(_f);

    return r;
  }

  /*
  public static void saveJavaSer(AglProjet _p, String _f)
       throws IOException
  {
    ObjectOutputStream out=new ObjectOutputStream(new FileOutputStream(_f));
    out.writeObject(_p);
    out.close();
  }

  public static AglProjet loadJavaSer(String _f)
       throws IOException, ClassNotFoundException
  {
    AglProjet r=null;

    ObjectInputStream out=new ObjectInputStream(new FileInputStream(_f));
    r=(AglProjet)out.readObject();
    out.close();
    installe(r);

    return r;
  }
  */

  public static void saveYapodSer(AglProjet _p, String _f)
       throws IOException
  {
    /*GCJ-BEGIN*/
    YapodSerSerializer out=new YapodSerSerializer();
    out.open(new FileOutputStream(_f));
    out.write(_p);
    out.close();
    /*GCJ-END*/
  }

  public static AglProjet loadYapodSer(String _f)
       throws IOException, ClassNotFoundException
  {
    AglProjet r=null;

    /*GCJ-BEGIN*/
    YapodSerDeserializer in=new YapodSerDeserializer();
    in.open(new FileInputStream(_f));
    r=(AglProjet)in.read();
    in.close();
    installe(r);
    /*GCJ-END*/

    return r;
  }

  public static void saveYapodSgz(AglProjet _p, String _f)
       throws IOException
  {
    /*GCJ-BEGIN*/
    YapodSgzSerializer out=new YapodSgzSerializer();
    out.open(new FileOutputStream(_f));
    out.write(_p);
    out.close();
    /*GCJ-END*/
  }

  public static AglProjet loadYapodSgz(String _f)
       throws IOException, ClassNotFoundException
  {
    AglProjet r=null;

    /*GCJ-BEGIN*/
    YapodSgzDeserializer in=new YapodSgzDeserializer();
    in.open(new FileInputStream(_f));
    r=(AglProjet)in.read();
    in.close();
    installe(r);
    /*GCJ-END*/

    return r;
  }

  public static void saveYapodXml(AglProjet _p, String _f)
       throws IOException
  {
    /*GCJ-BEGIN*/
    YapodXmlSerializer out=new YapodXmlSerializer();
    out.open(new FileOutputStream(_f));
    out.write(_p);
    out.close();
    /*GCJ-END*/
  }

  public static AglProjet loadYapodXml(String _f)
       throws IOException, ClassNotFoundException
  {
    AglProjet r=null;

    /*GCJ-BEGIN*/
    YapodXmlDeserializer in=new YapodXmlDeserializer();
    in.open(new FileInputStream(_f));
    r=(AglProjet)in.read();
    in.close();
    installe(r);
    /*GCJ-END*/

    return r;
  }

  public static void saveYapodXgz(AglProjet _p, String _f)
       throws IOException
  {
    /*GCJ-BEGIN*/
    YapodXgzSerializer out=new YapodXgzSerializer();
    out.open(new FileOutputStream(_f));
    out.write(_p);
    out.close();
    /*GCJ-END*/
  }

  public static AglProjet loadYapodXgz(String _f)
       throws IOException, ClassNotFoundException
  {
    AglProjet r=null;

    /*GCJ-BEGIN*/
    YapodXgzDeserializer in=new YapodXgzDeserializer();
    in.open(new FileInputStream(_f));
    r=(AglProjet)in.read();
    in.close();
    installe(r);
    /*GCJ-END*/

    return r;
  }

  private static void installe(AglProjet _p)
  {
    reinit(true);
    AglType.table    =_p.types;
    AglPatron.table  =_p.patrons;
    AglRelation.table=_p.relations;
    AglUnite.table   =_p.unites;
    AglPaquet.table  =_p.paquets;
  }
}



