/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglPublicAPI.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglPublicAPI
{
  // general

  public static String hello()
  { return "Alma "+AglOptions.VERSION+": hello!"; }

  public static boolean has_gui()
  { return (AglOptions.application!=null); }

  public static Class get_class(Object _o)
  { return (_o==null ? null : _o.getClass()); }

  public static boolean is_a(Object _o, Class _c)
  { return _c.isInstance(_o); }

  public static String code(Object _o)
  { return AglOptions.code().code(_o); }

  // objets

  public static AglCommentaire[] get_comments(AglObjet _o)
  {
    if(_o.commentaires==null) return new AglCommentaire[0];

    AglCommentaire[] r=new AglCommentaire[_o.commentaires.size()];

    int i=0;
    for(Enumeration e=_o.commentaires.elements(); e.hasMoreElements(); )
    {
      r[i]=(AglCommentaire)e.nextElement();
      i++;
    }

    return r;
  }

  // types

  public static AglType[] defined_types()
  {
    AglType[] r=new AglType[AglType.table.size()];

    int i=0;
    for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
    {
      r[i]=(AglType)e.nextElement();
      i++;
    }

    return r;
  }

  public static String get_type_name(AglType _t)
    { return code(_t.nom); }

  public static AglType type_by_name(String _n)
    { return AglType.cherche(_n); }

  // classes

  public static AglClasse[] defined_classes()
  {
    AglType[] t=defined_types();
    int i,n;

    for(n=i=0; i<t.length; i++)
    {
      if(!(t[i] instanceof AglClasse))
	t[i]=null;
      else
	n++;
    }

    AglClasse[] r=new AglClasse[n];
    for(n=i=0; i<t.length; i++)
      if(t[i]!=null)
	{ r[n]=(AglClasse)t[i]; n++; }

    return r;
  }

  public static AglClasse create_class(String _n)
  {
    AglType r=AglType.cherche(_n);
    if(r==null) r=new AglClasse(new AglNom(_n));
    if(!(r instanceof AglClasse)) r=null;
    return (AglClasse)r;
  }

  public static boolean is_elementary(AglClasse _c)
    { return _c.elementaire; }

  public static void set_elementary(AglClasse _c, boolean _state)
    { _c.elementaire=_state; }

  public static boolean is_interface(AglClasse _c)
    { return !_c.normale; }

  public static void set_interface(AglClasse _c, boolean _state)
    { _c.normale=!_state; }

  public static AglClasse[] get_class_superclasses(AglClasse _c)
  {
    AglClasse[] r=new AglClasse[_c.meres.noms.size()];

    int i=0;
    for(Enumeration e=_c.meres.noms.elements(); e.hasMoreElements(); )
    {
      r[i]=create_class(((AglNom)e.nextElement()).formeJava());
      i++;
    }

    return r;
  }

  public static AglAttribut[] get_class_attributes(AglClasse _c)
  {
    AglAttribut[] r=new AglAttribut[_c.attributs.size()];

    int i=0;
    for(Enumeration e=_c.attributs.elements(); e.hasMoreElements(); )
    {
      r[i]=(AglAttribut)e.nextElement();
      i++;
    }

    return r;
  }

  public static AglConstructeur[] get_class_constructors(AglClasse _c)
  {
    AglConstructeur[] r=new AglConstructeur[_c.constructeurs.size()];

    int i=0;
    for(Enumeration e=_c.constructeurs.elements(); e.hasMoreElements(); )
    {
      r[i]=(AglConstructeur)e.nextElement();
      i++;
    }

    return r;
  }

  public static AglMethode[] get_class_methods(AglClasse _c)
  {
    AglMethode[] r=new AglMethode[_c.methodes.size()];

    int i=0;
    for(Enumeration e=_c.methodes.elements(); e.hasMoreElements(); )
    {
      r[i]=(AglMethode)e.nextElement();
      i++;
    }

    return r;
  }

  public static AglAttribut find_class_attribute(AglClasse _c, String _s)
  {
    return _c.chercheAttributParSignature(_s);
  }

  public static AglConstructeur find_class_constructor(AglClasse _c, String _s)
  {
    return _c.chercheConstructeurParSignature(_s);
  }

  public static AglMethode find_class_method(AglClasse _c, String _s)
  {
    return _c.chercheMethodeParSignature(_s);
  }

  // structs

  public static AglStruct[] defined_structs()
  {
    AglType[] t=defined_types();
    int i,n;

    for(n=i=0; i<t.length; i++)
    {
      if(!(t[i] instanceof AglStruct))
	t[i]=null;
      else
	n++;
    }

    AglStruct[] r=new AglStruct[n];
    for(n=i=0; i<t.length; i++)
      if(t[i]!=null)
	{ r[n]=(AglStruct)t[i]; n++; }

    return r;
  }

  public static AglStruct create_struct(String _n)
  {
    AglType r=AglType.cherche(_n);
    if(r==null) r=new AglStruct(new AglNom(_n));
    if(!(r instanceof AglStruct)) r=null;
    return (AglStruct)r;
  }

  // methods

  public static AglParametre[] get_method_parameters(AglMethode _m)
  {
    AglParametre[] r=new AglParametre[_m.parametres.size()];

    int i=0;
    for(Enumeration e=_m.parametres.elements(); e.hasMoreElements(); )
    {
      r[i]=(AglParametre)e.nextElement();
      i++;
    }

    return r;
  }

  public static AglNom[] get_method_exceptions(AglMethode _m)
  {
    AglNom[] r=new AglNom[_m.exceptions.noms.size()];

    int i=0;
    for(Enumeration e=_m.exceptions.noms.elements(); e.hasMoreElements(); )
    {
      r[i]=(AglNom)e.nextElement();
      i++;
    }

    return r;
  }
}
