/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglRelation.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public abstract class AglRelation extends AglObjet
{
  public static AglVecteur table;

  public static void reinit(boolean _nouveau)
  {
    if(_nouveau||(table==null))
    {
      AglOptions.terminal.trace("$$$ AglRelation");
      table=new AglVecteur(0,1);
    }
  }

  public AglRelation()
  {
    AglOptions.terminal.trace("Create relation");
    reinit(false);
    table.addElement(this);
  }

  public abstract String getIntituleCourt();
  public abstract String getIntituleLong();
  public abstract String toString();

  public static void detruireRelationsGenerees()
  {
    for(int i=table.size()-1; i>=0; i--)
      if(((AglRelation)table.elementAt(i)).genere)
	table.removeElementAt(i);
  }

  public static void construireRelationsHeritage()
  {
    int p=0;
    for(Enumeration e1=AglType.table.elements(); e1.hasMoreElements(); )
    {
      p++;
      AglOptions.terminal.progres(100*p/AglType.table.size());

      Object o=e1.nextElement();
      if(o instanceof AglClasse)
      {
	AglClasse c=(AglClasse)o;
	for(Enumeration e2=c.meres.noms.elements(); e2.hasMoreElements(); )
	{
	  AglNom  mn=(AglNom)e2.nextElement();
	  AglType mt=AglTypeAlias.simplifie((AglType)AglType.cherche(mn));

	  if((mt!=null)&&(mt instanceof AglClasse))
	  {
	    AglClasse mc=(AglClasse)mt;

	    AglRelationBiOrientee r1=new AglRelationBiOrientee("hrite de",c,mc);
	    r1.genere=true;
	    c .relations.addElement(r1);
	    AglOptions.terminal.trace(r1.getIntituleLong());

	    AglRelationBiOrientee r2=new AglRelationBiOrientee("est hrit par",mc,c);
	    r2.genere=true;
	    mc.relations.addElement(r2);
	    AglOptions.terminal.trace(r2.getIntituleLong());
	  }
	}
      }
    }
  }

  public static void construireRelationsImplantation()
  {
    int p=0;
    for(Enumeration e1=AglType.table.elements(); e1.hasMoreElements(); )
    {
      p++;
      AglOptions.terminal.progres(100*p/AglType.table.size());

      Object o=e1.nextElement();
      if(o instanceof AglClasse)
      {
	AglClasse c=(AglClasse)o;
	for(Enumeration e2=c.interfaces.noms.elements(); e2.hasMoreElements(); )
	{
	  AglNom  mn=(AglNom)e2.nextElement();
	  AglType mt=AglTypeAlias.simplifie((AglType)AglType.cherche(mn));

	  if((mt!=null)&&(mt instanceof AglClasse))
	  {
	    AglClasse mc=(AglClasse)mt;

	    AglRelationBiOrientee r1=new AglRelationBiOrientee("implante",c,mc);
	    r1.genere=true;
	    c .relations.addElement(r1);
	    AglOptions.terminal.trace(r1.getIntituleLong());

	    AglRelationBiOrientee r2=new AglRelationBiOrientee("est implant par",mc,c);
	    r2.genere=true;
	    mc.relations.addElement(r2);
	    AglOptions.terminal.trace(r2.getIntituleLong());
	  }
	}
      }
    }
  }

  public static void construireRelationsContenance()
  {
    int p=0;
    for(Enumeration e1=AglType.table.elements(); e1.hasMoreElements(); )
    {
      p++;
      AglOptions.terminal.progres(100*p/AglType.table.size());

      AglType o=(AglType)e1.nextElement();

      Enumeration e2=null;
      if(o instanceof AglClasse) e2=((AglClasse)o).attributs.elements();
      if(o instanceof AglStruct) e2=((AglStruct)o).attributs.elements();

      if(e2!=null)
      {
	for( ; e2.hasMoreElements(); )
	{
	  AglAttribut a=(AglAttribut)e2.nextElement();
	  if(!a.visibilite.vstatic)
	  {
	    AglType at=AglTypeAlias.simplifie(a.type);

	    String cardinalite="1";
	    while(at instanceof AglTypeTableau)
	    {
	      cardinalite="0..N";
	      at=AglTypeAlias.simplifie(((AglTypeTableau)at).type);
	    }

	    if(!(at instanceof AglTypePrimitif))
	    {
	      if(cardinalite.equals("1"))
		cardinalite="0..1";
	      
	      AglRelationBiOrientee r1=new AglRelationBiOrientee("contient",o,at);
	      r1.roleFin=a.nom;
	      r1.cardinaliteFin=cardinalite;
	      r1.genere=true;
	      o .relations.addElement(r1);
	      AglOptions.terminal.trace(r1.getIntituleLong());

	      AglRelationBiOrientee r2=new AglRelationBiOrientee("est contenu par",at,o);
	      r2.roleDebut=a.nom;
	      r2.cardinaliteDebut=cardinalite;
	      r2.genere=true;
	      at.relations.addElement(r2);
	      AglOptions.terminal.trace(r2.getIntituleLong());
	    }
	  }
	}
      }
    }
  }

  public static void construireRelationsUtilisation()
  {
    int p=0;
    for(Enumeration e1=AglType.table.elements(); e1.hasMoreElements(); )
    {
      p++;
      AglOptions.terminal.progres(100*p/AglType.table.size());

      AglType o=(AglType)e1.nextElement();
      if(o instanceof AglClasse)
      {
	AglClasse c=(AglClasse)o;

	for(Enumeration e2=c.methodes.elements(); e2.hasMoreElements(); )
	{
	  AglType at;
	  String  cardinalite;
	  AglRelationBiOrientee r1,r2;

	  AglMethode am=(AglMethode)e2.nextElement();
	  at=AglTypeAlias.simplifie(am.type);
	  cardinalite="1";
	  while(at instanceof AglTypeTableau)
	  {
	    cardinalite="0..N";
	    at=AglTypeAlias.simplifie(((AglTypeTableau)at).type);
	  }

	  /*
	  if(cardinalite.equals("1")&&!(at instanceof AglTypePrimitif))
	    cardinalite="0..1";
          */

	  if(!(at instanceof AglTypePrimitif))
	  {
	    r1=new AglRelationBiOrientee("utilise",c,at);
	    r1.roleFin=am.nom;
	    r1.cardinaliteFin=cardinalite;
	    r1.genere=true;
	    c .relations.addElement(r1);
	    AglOptions.terminal.trace(r1.getIntituleLong());

	    r2=new AglRelationBiOrientee("est utilis par",at,c);
	    r2.roleDebut=am.nom;
	    r2.cardinaliteDebut=cardinalite;
	    r2.genere=true;
	    at.relations.addElement(r2);
	    AglOptions.terminal.trace(r2.getIntituleLong());
	  }

	  for(Enumeration e3=am.parametres.elements(); e3.hasMoreElements(); )
	  {
	    AglParametre ap=(AglParametre)e3.nextElement();
	    at=AglTypeAlias.simplifie(ap.type);
	    cardinalite="1";
	    while(at instanceof AglTypeTableau)
	    {
	      cardinalite="0..N"; 
	      at=AglTypeAlias.simplifie(((AglTypeTableau)at).type);
	    }

	    /*
	    if(cardinalite.equals("1")&&!(at instanceof AglTypePrimitif))
	      cardinalite="0..1";
	    */

	    if(!(at instanceof AglTypePrimitif))
	    {
	      r1=new AglRelationBiOrientee("utilise",c,at);
	      r1.roleFin=am.nom;
	      r1.cardinaliteFin=cardinalite;
	      r1.genere=true;
	      c .relations.addElement(r1);
	      AglOptions.terminal.trace(r1.getIntituleLong());

	      r2=new AglRelationBiOrientee("est utilis par",at,c);
	      r2.roleDebut=am.nom;
	      r2.cardinaliteDebut=cardinalite;
	      r2.genere=true;
	      at.relations.addElement(r2);
	      AglOptions.terminal.trace(r2.getIntituleLong());
	    }
	  }
	}
      }
    }
  }
}
