/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AglType.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.agl;
import com.memoire.agl.*;


import java.util.*;

public class AglType extends AglDeclaration
{
  public static AglTableTriee table;

  public static void reinit(boolean _nouveau)
  {
    if(_nouveau||(table==null))
    {
      AglOptions.terminal.trace("$$$ AglType");
      table=new AglTableTriee();
      AglTypePrimitif.reinit();
    }
  }

  public AglNom     nom;
  public AglVecteur relations;

  public AglType() // yapod
  {
    this(new AglNom(AglLib.genereNom("type")));
  }

  public AglType(AglNom _nom)
  {
    relations=new AglVecteur(0,1);
    setNom(_nom);
  }

  public AglNom getNom()
  {
    return nom;
  }

  public final void setNom(AglNom _nom)
  {
    if(  (_nom!=null)&&(nom!=null)
       &&_nom.formeJava().equals(nom.formeJava())) return;

    reinit(false);

    if(nom!=null)
    {
      AglOptions.terminal.trace("Destroy type: "+nom);
      table.remove(nom.dernier());
    }

    nom=_nom;

    if(nom!=null)
    {
      AglOptions.terminal.trace("Create type: "+nom);

      if(table.get(nom.dernier())!=null)
	AglOptions.terminal.trace("Conflict   : "+nom);

      table.put(nom.dernier(),this);
    }
  }

  public String toString()
  {
    return "TYPE("+nom+")";
  }

  public static AglType cherche(String nc)
  {
    return cherche(new AglNom(nc));
  }

  public static AglType cherche(AglNom nc)
  {
    AglType t=(AglType)table.get(nc.dernier());
    if(t==null) AglOptions.terminal.trace("Unknown type: "+nc);
    return t;
  }

  public final String signature()
  {
    return signature(this);
  }

  public final static String signature(AglType _t)
  {
    String r;

         if(_t==AglTypePrimitif.tvoid   ) r="V";
    else if(_t==AglTypePrimitif.tboolean) r="Z";
    else if(_t==AglTypePrimitif.tchar   ) r="C";
    else if(_t==AglTypePrimitif.tbyte   ) r="B";
    else if(_t==AglTypePrimitif.tshort  ) r="S";
    else if(_t==AglTypePrimitif.tinteger) r="I";
    else if(_t==AglTypePrimitif.tlong   ) r="J";
    else if(_t==AglTypePrimitif.tfloat  ) r="F";
    else if(_t==AglTypePrimitif.tdouble ) r="D";
    else if(_t instanceof AglTypeTableau)
      r="["+signature(((AglTypeTableau)_t).type);
    else if(_t instanceof AglTypeAlias)
      r=signature(((AglTypeAlias)_t).type);
    else if(_t==null) r="";
    else r="L"+_t.nom.formeJava().replace('.','/')+";";

    return r;
  }
}
