/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         Alma.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/04 18:29:49 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;
import com.memoire.fu.*;

import java.util.*;
import javax.swing.*;

public class Alma
  implements Runnable
{
  private static String[] args_;

  private Alma(String[] _args)
  {
    args_=_args;
  }

  private final void step1()
  {
    // guess();
    
    if(!FuLib.verifyMemory(8))
    {
      String t="You need at least 8M to run safely Alma.\n"+
	       "Please use the -mx option of java.";
      boolean ok=FuLib.verifyMemory(1);
      if(ok)
      {
	try { new BuDialogMessage(null,null,t).activate();}
	catch(Throwable e) { ok=false; }
      }
      if(!ok) System.err.println(t);
      System.exit(-1);
    }

    /*
    String jv=System.getProperty("java.version");
    if(jv.compareTo("1.1.2")<0)
    {
      System.err.println("JVM >1.1.2 needed for Swing");
      System.exit(-1);
    }
    */
  }

  private final void step2()
  {
    BuPreferences.BU.applyLookAndFeel();

    /*
    try { UIManager.setLookAndFeel
	    ("BuBasicLookAndFeel"); }
    catch(Exception ex) { ex.printStackTrace(); }
    */

    /*
    // Mac Look and feel
    System.getProperties().put("os.name", "Mac OS");
    try { UIManager.setLookAndFeel
	    ("com.sun.java.swing.plaf.mac.MacLookAndFeel"); }
    catch(Exception ex) { ex.printStackTrace(); }
    */
    
    /*
    try { UIManager.setLookAndFeel
	    ("javax.swing.plaf.organic.OrganicLookAndFeel"); }
    catch(Exception ex) { ex.printStackTrace(); }
    */

    /*
    try { UIManager.setLookAndFeel
	    ("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"); }
    catch(Exception ex) { ex.printStackTrace(); }
    */
  }

  private final void step3()
  {
    BuInformationsSoftware il=AlmaImplementation.informationsSoftware();

    if(BuPreferences.BU.getStringProperty("locale.language").equals(""))
    {
      BuLanguagePreferencesPanel pp=
	new BuLanguagePreferencesPanel(il.languages);

      new BuDialogMessage(null,il,pp).activate();
      pp.validatePreferences();
      Locale.setDefault(new Locale(pp.getLocaleString(),""));
    }

    BuPreferences.BU.applyLanguage(il.languages);
  }

  
    // Update

    /*
    try
    {
      BuUpdate update=new BuUpdate(il);
      if(update.scanServer())
      {
        BuRegistry.register(update);
        update.start();
        BuRegistry.unregister(update);
	return;
      }
    }
    catch(java.net.MalformedURLException ex) { }
    */

  private final void step4()
  {
    BuInformationsSoftware il=AlmaImplementation.informationsSoftware();

    // Splash screen

    BuSplashScreen ss=new BuSplashScreen
      (il,1000,new String[][]
       { FuLib.JAVA_CLASSES, BuLib.SWING_CLASSES, BuLib.BU_CLASSES,
	 //ScriptLib.SCRIPT_CLASSES,
	 AglLib.AGL_CLASSES, AlmaLib.ALMA_CLASSES });
    // AlmaApplication.class.getName()

    if(BuPreferences.BU.getBooleanProperty("splashscreen.visible",true))
    {
      // Exception dans Mac L&F
      try { ss.start(); } catch(Exception ex) { }
    }

    ss.setText("Cration de l'application");
    ss.setProgression(0);
    AlmaApplication app=new AlmaApplication();

    if(BuPreferences.BU.getBooleanProperty("camera.visible",false))
    {
      ss.setText("Cration de l'appareil photo");
      ss.setProgression(10);
      new AlmaScreenCopy(app);
    }

    app.getImplementation().setSplashScreen(ss);
    app.init();

    BuGlassPaneStop glasspane=new BuGlassPaneStop(); //(app);
    app.setGlassPane(glasspane);
    glasspane.setVisible(true);

    ss.setProgression(100);
    ss.setText("Dmarrage de l'application");

    try { Thread.sleep(30); }
    catch(Exception ex) { }

    app.start();
    
    AlmaDnd.installIcons();
    glasspane.setVisible(false);

    if(args_.length>0)
      ((AlmaImplementation)app.getImplementation()).cmdArguments(args_);
  }

  public final void run()
  {
    //System.err.println("STEP1");
    step1();
    //System.err.println("STEP2");
    step2();
    //System.err.println("STEP3");
    step3();
    //System.err.println("STEP4");
    step4();
    //System.err.println("STEP5");
  }

  public final static void main(String _args[])
  {
    Thread.currentThread().setName("Alma main thread");
    Thread.currentThread().setPriority(Thread.NORM_PRIORITY-1);

    /*
    Runnable runnable=new Alma(_args);
    if(!SwingUtilities.isEventDispatchThread())
      SwingUtilities.invokeLater(runnable);
    else
      runnable.run();
    */

    new Alma(_args).run();

    /*
    try
    {
      Class c=FuClassLoaderDedicated.load("com.memoire.alma.Alma");
      ((Runnable)c.newInstance()).run();
    }
    catch(Exception ex) { ex.printStackTrace(); }
    */

  }
}
