/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaDjaClasse.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.dja.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

public class AlmaDjaClasse
       extends DjaForm
       implements AlmaDjaValeur
{
  private transient Dimension size_;
  private transient AglClasse classe_;

  public AlmaDjaClasse()
  {
    this((String)null);
  }

  public AlmaDjaClasse(AglClasse _classe)
    { this(_classe.nom.dernier()); }

  public AlmaDjaClasse(String _text)
  {
    super(null);

    DjaText t0=new DjaText(this,0,_text,false,0,0,
			   RELATIVE_NW,0,MIDDLE,null,
			   defaultBoldFont,false,false);
    setTextArray(new DjaText[] { t0 });

    super.setWidth (0);
    super.setHeight(0);
    set("depliee"    ,false);
    set("visibilites",true);
    set("types"      ,true);
    set("parametres" ,false);
    set("balises"    ,true);
    textChanged(t0);
  }

  public AglClasse getClasse()
  { return classe_; }

  public AglObjet getObjet()
  { return getClasse(); }

  public boolean is(String _k)
  { return "vrai".equals(getProperty(_k)); }

  public void set(String _k, boolean _v)
  {
    putProperty(_k,_v ? "vrai" : "faux");
  }

  public void putProperty(String _k,String _v)
  {
    super.putProperty(_k,_v);
    size_=optimalSize();
  }

  public int getWidth()
  {
    if(size_==null) size_=optimalSize();
    return Math.max(size_.width,super.getWidth());
  }

  public int getHeight()
  {
    if(size_==null) size_=optimalSize();
    return Math.max(size_.height,super.getHeight());
  }

  public void textChanged(DjaText _text)
  {
    if(_text.getNum()==0)
    {
      classe_=null;
    
      String n=_text.getText();
      if(n!=null)
      {
	try
	{
	  AglType at=AglType.cherche(n);
	  if(at instanceof AglClasse) classe_=(AglClasse)at;
	}
	catch(Exception ex) { }
      }
      // System.err.println("CLASSE="+classe_);
    }

    size_=optimalSize();
  }

  public DjaText[] getTexts()
  {
    DjaText[] r=super.getTexts();
    r[0].setW(getWidth()-4);
    r[0].setX(2);
    r[0].setY(getFont().getSize());
    return r;
  }

  public DjaAnchor[] getAnchors()
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    int hs=getFont().getSize();

    int a=0;
    int m=0;

    if(is("depliee")&&(classe_!=null))
    {
      a=classe_.attributs.size();
      m=classe_.methodes.size ();
    }

    int l=8+2*(a+m);

    DjaAnchor[] r=new DjaAnchor[l];
    r[0]=new DjaAnchor(this,0,NORTH,x+w/4  ,y);
    r[1]=new DjaAnchor(this,1,NORTH,x+w/2  ,y);

    if(is("moule"))
      r[2]=new DjaAnchor(this,2,NORTH,x+3*w/8,y);
    else
      r[2]=new DjaAnchor(this,2,NORTH,x+3*w/4,y);

    r[3]=new DjaAnchor(this,3,SOUTH,x+w/4  ,y+h);
    r[4]=new DjaAnchor(this,4,SOUTH,x+w/2  ,y+h);
    r[5]=new DjaAnchor(this,5,SOUTH,x+3*w/4,y+h);

    y+=hs;
    int h0=getTextArray()[0].getSize().height;
    r[6]=new DjaAnchor(this,6,WEST,x  ,y+h0/2+1);
    r[7]=new DjaAnchor(this,7,EAST,x+w,y+h0/2+1);
    y+=h0;

    y+=5+hs/2;
    for(int i=0;i<a;i++)
    {
      y+=hs;
      r[8+2*i]=new DjaAnchor(this,8+2*i,WEST,x  ,y+1);
      r[9+2*i]=new DjaAnchor(this,9+2*i,EAST,x+w,y+1);
    }

    y+=5;
    for(int i=a;i<a+m;i++)
    {
      y+=hs;
      r[8+2*i]=new DjaAnchor(this,8+2*i,WEST,x  ,y+1);
      r[9+2*i]=new DjaAnchor(this,9+2*i,EAST,x+w,y+1);
    }

    return r;
  }

  public DjaHandle[] getHandles()
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    DjaHandle[] r=new DjaHandle[3];
    r[0]=new DjaHandle(this,EAST      ,x+w-1+deltaX,y+h/2       );
    r[1]=new DjaHandle(this,SOUTH_EAST,x+w-1+deltaX,y+h-1+deltaY);
    r[2]=new DjaHandle(this,SOUTH     ,x+w/2       ,y+h-1+deltaY);

    return r;
  }

  public void paintObject(Graphics _g)
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    Font        ft=getFont();
    FontMetrics fm=_g.getFontMetrics(ft);

    int  hs=ft.getSize();
    int  ws;

    Color bg=getBackground();
    if(bg!=null)
    {
      DjaGraphics.setColor(_g,bg);
      DjaGraphics.fillRect(_g,x,y,w+1,h+1);
    }

    if(classe_!=null)
      set("abstraite",classe_.visibilite.vabstract);

    if(is("abstraite"))
      putProperty("trace","3.2");
    else
      putProperty("trace","1.0");
      
    DjaGraphics.BresenhamParams bp=DjaGraphics.getBresenhamParams(this);

    Color fg=getForeground();
    if(fg!=null)
    {
      DjaGraphics.setColor(_g,fg);
      DjaGraphics.drawRect(_g,x,y,w,h,bp);
    }

    if(is("moule"))
    {
      DjaGraphics.BresenhamParams bpx=new DjaGraphics.BresenhamParams(5,5);
      bpx.epaisseur_=(bp!=null ? bp.epaisseur_ : 1);
      int e=(bpx!=null ? bpx.epaisseur_ : 1);
 
      DjaGraphics.setColor(_g,(bg!=null ? bg : Color.white));
      DjaGraphics.fillRect(_g,x+w/2+5,y-5,w/2+e,15+e);

      if(fg!=null)
      {
	DjaGraphics.setColor(_g,fg);
	DjaGraphics.drawRect(_g,x+w/2+5,y-5,w/2,15,bpx);
      }
    }

    Color tc=getColor();
    if(tc!=null)
    {
      _g.setColor(tc);
      _g.setFont(ft);

      y+=hs;
      if((classe_!=null)&&!classe_.normale)
      {
	String s="<<interface>>";
	ws=fm.stringWidth(s);
	_g.drawString(s,x+(w-ws)/2,y);
      }

      /*
      setTextBounds(new Rectangle(x+4,y+2,w-8,hs-1));
      y+=hs;
      if(nom!=null)
      {
	ws=fm.stringWidth(nom);
	_g.drawString(nom,x+(w-ws)/2,y);
      }
      */
      y+=getTextArray()[0].getSize().height;

      y+=hs;
      if(classe_!=null)
      {
	String s=codeTags(classe_);
	if(!"".equals(s))
	{
	  ws=fm.stringWidth(s);
	  _g.drawString(s,x+(w-ws)/2,y);
	}
      }

      if(fg!=null)
      {
	DjaGraphics.setColor(_g,fg);
	DjaGraphics.drawLine(_g,x,y+3,x+w-1,y+3,bp);
	DjaGraphics.setColor(_g,tc);
      }

      y+=5;

      if((classe_!=null)&&is("depliee"))
      for(Enumeration e=classe_.attributs.elements();
	  e.hasMoreElements(); )
      {
	AglAttribut a=(AglAttribut)e.nextElement();
	y+=hs;
	_g.drawString(code(a.visibilite),x+4,y);
	_g.drawString(code(a),x+14,y);

	if(a.visibilite.vstatic)
	{
	  ws=Math.min(w-18,fm.stringWidth(a.nom));
	  _g.drawLine(x+14,y+1,x+14+ws-1,y+1);
	}
      }

      if(fg!=null)
      {
	DjaGraphics.setColor(_g,fg);
	DjaGraphics.drawLine(_g,x,y+3,x+w-1,y+3,bp);
	DjaGraphics.setColor(_g,tc);
      }
      y+=5;

      if((classe_!=null)&&is("depliee"))
      for(Enumeration e=classe_.methodes.elements();
	  e.hasMoreElements(); )
      {
	AglMethode m=(AglMethode)e.nextElement();
	y+=hs;
	_g.drawString(code(m.visibilite),x+4,y);
	_g.drawString(code(m),x+14,y);

	// wmin_=Math.max(wmin_,18+fm.stringWidth(code(m)));

	if(m.visibilite.vstatic)
	{
	  ws=Math.min(w-18,fm.stringWidth(m.nom));
	  _g.drawLine(x+14,y+1,x+14+ws-1,y+1);
	}
      }
    }

    super.paintObject(_g);
  }

  public Dimension optimalSize()
  {
    int w=Math.max(0,2*deltaX-8);
    int h=0;

    Font        ft=getFont();
    FontMetrics fm=DjaGraphics.getFontMetrics(this,ft);
    int         hs=ft.getSize();

    int ws;

    h+=2*hs;
    if((classe_!=null)&&!classe_.normale)
    {
      String s;

      s="<<interface>>";
      w=Math.max(w,8+fm.stringWidth(s));
      s=codeTags(classe_);
      w=Math.max(w,8+fm.stringWidth(s));
    }

    DjaText t=getTextArray()[0];
    Dimension d=t.optimalSize();
    w=Math.max(w,d.width+2);
    h+=d.height;

    h+=5;

    if((classe_!=null)&&is("depliee"))
    for(Enumeration e=classe_.attributs.elements();
	e.hasMoreElements(); )
    {
      AglAttribut a=(AglAttribut)e.nextElement();
      h+=hs;
      w=Math.max(w,18+fm.stringWidth(code(a)));
    }

    h+=5;

    if((classe_!=null)&&is("depliee"))
    for(Enumeration e=classe_.methodes.elements();
	e.hasMoreElements(); )
    {
      AglMethode m=(AglMethode)e.nextElement();
      h+=hs;
      w=Math.max(w,18+fm.stringWidth(code(m)));
    }

    return new Dimension(Math.max(81,w),h+4);
  }

  // Temporaire

  public String codeTags(AglClasse o)
  {
    String s="";
    if(is("balises"))
    {
      if(o.visibilite.vfinal)    s+=",leaf";
      if(o.meres.noms.size()==0) s+=",root";
      if(!"".equals(s))
	s="{"+s.substring(1)+"}";
    }
    return s;
  }

  public String code(AglType o)
  {
    String r="";
    if(o instanceof AglTypeTableau) r+=code(((AglTypeTableau)o).type)+"[]";
    else                            r+=o.nom.dernier();
    return r;
  }

  private String code(AglVisibilite o)
  {
    String r="";
    if(o.vpublic)    r+="+";
    if(o.vprivate)   r+="-";
    if(o.vprotected) r+="#";
    //if(o.vabstract)  r+="0";
    return r;
  }

  private String code(AglAttribut o)
  {
    String r=o.nom;
    if(is("types")) r+=":"+code(o.type);
    if(is("balises")&&o.visibilite.vfinal) r+=" {frozen}";
    return r;
  }

  private String code(AglMethode o)
  {
    String r=o.nom;
    if(is("parametres"))
    {
      r+="(";
      boolean premier=true;
      for(Enumeration e=o.parametres.elements(); e.hasMoreElements(); )
      {
	AglParametre p=(AglParametre)e.nextElement();
	if(!premier) r+=", ";
	else         premier=false;
	r+=p.nom;
	if(is("types")) r+=":"+code(p.type);
      }
      r+=")";
    }
    else r+="()";
    if(is("types")  &&o.type!=AglTypePrimitif.tvoid) r+=":"+code(o.type);
    if(is("balises")&&o.visibilite.vfinal          ) r+=" {leaf}";
    return r;
  }
}
