/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaDjaPaquet.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.dja.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

public class AlmaDjaPaquet
       extends DjaForm//UmlLargePackage
       implements AlmaDjaValeur
{
  private transient Dimension size_;
  private transient AglPaquet paquet_;

  public AlmaDjaPaquet()
  {
    this((String)null);
  }

  public AlmaDjaPaquet(AglPaquet _paquet)
  {
    this(_paquet.nom.formeJava());
  }

  public AlmaDjaPaquet(String _text)
  {
    super(null);

    DjaText t0=new DjaText(this,0,_text,false,0,0,
			   RELATIVE_NW,0,MIDDLE,null,
			   defaultBoldFont,false,false);
    setTextArray(new DjaText[] { t0 });

    super.setWidth (0);
    super.setHeight(0);
    set("depliee"    ,true);
    set("visibilites",true);
    textChanged(t0);
  }

  public AglPaquet getPaquet()
  { return paquet_; }

  public AglObjet getObjet()
  { return getPaquet(); }

  public boolean is(String _k)
  { return "vrai".equals(getProperty(_k)); }

  public void set(String _k, boolean _v)
  {
    putProperty(_k,_v ? "vrai" : "faux");
  }

  public void putProperty(String _k,String _v)
  {
    super.putProperty(_k,_v);
    size_=optimalSize();
  }

  public int getWidth()
  {
    if(size_==null) size_=optimalSize();
    return Math.max(size_.width,super.getWidth());
  }

  public int getHeight()
  {
    if(size_==null) size_=optimalSize();
    return Math.max(size_.height,super.getHeight());
  }

  public void textChanged(DjaText _text)
  {
    if(_text.getNum()==0)
    {
      paquet_=null;
    
      String n=_text.getText();
      if(n!=null)
      {
	try
	{
	  AglPaquet ap=AglPaquet.cherche(n);
	  if(ap instanceof AglPaquet) paquet_=(AglPaquet)ap;
	}
	catch(Exception ex) { }
      }
      // System.err.println("PAQUET="+paquet_);
    }

    size_=optimalSize();
  }

  public boolean contains(int _x,int _y)
  {
    Rectangle b=getBounds();
    boolean   r=b.contains(_x,_y);

    if(r)
    {
      Dimension d=getTextArray()[0].getSize();
      int hs=d.height+2;
      int ws=d.width+2;

      if(new Rectangle(b.x+ws,b.y,b.width-ws,hs).contains(_x,_y))
	r=false;
    }

    return r;
  }

  public DjaText[] getTexts()
  {
    DjaText[] r=super.getTexts();
    //r[0].setW(getWidth()-4);
    r[0].setX(2);
    r[0].setY(2);//getFont().getSize());
    return r;
  }

  public DjaAnchor[] getAnchors()
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    Dimension d=getTextArray()[0].getSize();
    int hs=d.height+2;
    int ws=d.width+2;

    int l=5;
    if((paquet_!=null)&&is("depliee"))
    {
      for(Enumeration e=AglType.table.elements();
	e.hasMoreElements(); )
      {
        AglType at=(AglType)e.nextElement();
	if(  !(at instanceof AglTypeTableau)
	   //&&!estElementaire(at)
	   &&paquet_.nom.equals(at.nom.parent()))
	  l+=2;
      }
      
    }
    else l+=2;

    DjaAnchor[] r=new DjaAnchor[l];
    r[0]=new DjaAnchor(this,0,NORTH,x+ws/2    ,y     );
    r[1]=new DjaAnchor(this,1,NORTH,x+(w+ws)/2,y+hs+2);
    r[2]=new DjaAnchor(this,2,SOUTH,x+w/4     ,y+h   );
    r[3]=new DjaAnchor(this,3,SOUTH,x+w/2     ,y+h   );
    r[4]=new DjaAnchor(this,4,SOUTH,x+3*w/4   ,y+h   );

    if((paquet_!=null)&&is("depliee"))
    {
      y+=hs+10;

      Font        ft=getFont();
      FontMetrics fm=DjaGraphics.getFontMetrics(this,ft);

      hs=ft.getSize();

      for(int i=5;i<l;i+=2)
      {
	r[i  ]=new DjaAnchor(this,i  ,WEST,x  ,y  );
	r[i+1]=new DjaAnchor(this,i+1,EAST,x+w,y  );
	y+=hs;
      }
    }
    else
    {
      r[5]=new DjaAnchor(this,5,WEST,x  ,y+(h+hs)/2+1  );
      r[6]=new DjaAnchor(this,6,EAST,x+w,y+(h+hs)/2+1  );
    }
    //r[2]=new DjaAnchor(this,2,NORTH,x+(w-10)  ,y+hs+8);

    return r;
  }

  public DjaHandle[] getHandles()
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    DjaHandle[] r=new DjaHandle[3];
    r[0]=new DjaHandle(this,EAST      ,x+w-1+deltaX,y+h/2       );
    r[1]=new DjaHandle(this,SOUTH_EAST,x+w-1+deltaX,y+h-1+deltaY);
    r[2]=new DjaHandle(this,SOUTH     ,x+w/2       ,y+h-1+deltaY);

    return r;
  }

  public void paintObject(Graphics _g)
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    Dimension d=getTextArray()[0].getSize();
    int hs=d.height+2;
    int ws=d.width+2;

    DjaGraphics.BresenhamParams bp=DjaGraphics.getBresenhamParams(this);

    Color bg=getBackground();
    if(bg!=null)
    {
      DjaGraphics.setColor(_g,bg);
      DjaGraphics.fillRect(_g,x,y,ws+3,hs+3);
      if((paquet_!=null)&&is("depliee"))
	DjaGraphics.fillRect(_g,x,y+hs+3,w,h-hs-3);
    }

    Color fg=getForeground();
    if(fg!=null)
    {
      DjaGraphics.setColor(_g,fg);
      DjaGraphics.drawLine(_g,x+ws+2,y+hs+2,x+ws+2,y,bp);
      DjaGraphics.drawLine(_g,x+ws+2,y,x,y,bp);
      DjaGraphics.drawLine(_g,x,y,x,y+hs+2,bp);
      DjaGraphics.drawRect(_g,x,y+hs+2,w,h-hs-2,bp);

      if((paquet_!=null)&&is("depliee"))
      {
	Font        ft=getFont();
	FontMetrics fm=_g.getFontMetrics(ft);

	_g.setFont(ft);
	hs=ft.getSize();
	y+=hs+9;

	for(Enumeration e=AglType.table.elements();
	  e.hasMoreElements(); )
	{
	  AglType at=(AglType)e.nextElement();
	  if(  !(at instanceof AglTypeTableau)
	     //&&!estElementaire(at)
	     &&paquet_.nom.equals(at.nom.parent()))
          {
	    y+=hs;
	    _g.drawString(codeVisibilite(at),x+4,y);
	    _g.drawString(codeNom(at),x+14,y);
	    h+=hs;
	    if(estStatique(at))
	    {
	      ws=Math.min(w-18,fm.stringWidth(codeNom(at)));
	      _g.drawLine(x+14,y+1,x+14+ws-1,y+1);
	    }
	  }
	}
      }
    }

    super.paintObject(_g);
  }

  public Dimension optimalSize()
  {
    int w=Math.max(0,2*deltaX-8);
    int h=0;

    Font        ft=getFont();
    FontMetrics fm=DjaGraphics.getFontMetrics(this,ft);
    int         hs=ft.getSize();

    int ws;

    DjaText t=getTextArray()[0];
    Dimension d=t.optimalSize();
    w=Math.max(w,d.width+4);
    h+=d.height;

    h+=5;

    if((paquet_!=null)&&is("depliee"))
    for(Enumeration e=AglType.table.elements();
	e.hasMoreElements(); )
    {
      AglType at=(AglType)e.nextElement();
      if(  !(at instanceof AglTypeTableau)
	 //&&!estElementaire(at)
	 &&paquet_.nom.equals(at.nom.parent()))
      {
        h+=hs;
        w=Math.max(w,18+fm.stringWidth(codeNom(at)));
      }
    }

    return new Dimension(Math.max(41,w),
			 Math.max(d.height+8+hs,h+4));
  }

  public String codeNom(AglType o)
  {
    return o.nom.dernier();
  }

  private String codeVisibilite(AglType o)
  {
    AglVisibilite v;
         if(o instanceof AglClasse) v=((AglClasse)o).visibilite;
    else if(o instanceof AglStruct) v=((AglStruct)o).visibilite;
    else return "";

    String r="";
    if(v.vpublic)    r+="+";
    if(v.vprivate)   r+="-";
    if(v.vprotected) r+="#";
    //if(o.vabstract)  r+="0";
    return r;
  }

  private boolean estElementaire(AglType o)
  {
    boolean r=true;

         if(o instanceof AglClasse) r=((AglClasse)o).elementaire;
    else if(o instanceof AglStruct) r=((AglStruct)o).elementaire;

    return r;
  }

  private boolean estStatique(AglType o)
  {
    boolean r=false;

         if(o instanceof AglClasse) r=((AglClasse)o).visibilite.vstatic;
    else if(o instanceof AglStruct) r=((AglStruct)o).visibilite.vstatic;

    return r;
  }
}


