/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaDjaRelations.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.dja.*;
import com.memoire.fu.*;

import java.awt.*;
import java.util.*;

public class AlmaDjaRelations
{
  public final static DjaObject dejaPresent(DjaGrid _grille, AglObjet _p)
  {
    if(_p==null) return null;

    Enumeration e=_grille.enumerate(AlmaDjaValeur.class);
    while(e.hasMoreElements())
    {
      AlmaDjaValeur q=(AlmaDjaValeur)e.nextElement();
      if(q.getObjet()==_p) return (DjaObject)q;
    }

    return null;
  }

  public final static void creeTout(DjaGrid _grille, DjaObject _o)
  {
    if(_o instanceof AlmaDjaClasse)
    {
      AlmaDjaClasse c=(AlmaDjaClasse)_o;

      creeGeneralisation(_grille,c);
      creeDerivation    (_grille,c);
      creeRealisation   (_grille,c);
      creeSpecification (_grille,c);
      creeNotes         (_grille,c);
      creeContenance    (_grille,c);
      creeContenu       (_grille,c);
      creeUtilisation   (_grille,c);
      creeDependance    (_grille,c);

      //_grille.avoidOverlap();
      _grille.recenterIfNeeded();
      _grille.revalidate();
    }
    else
    if(_o instanceof AlmaDjaMessage)
    {
      AlmaDjaMessage c=(AlmaDjaMessage)_o;
      AglMethode     m=c.getMethode();

      if(m!=null)
      {
	AglType recepteur=null;
	if(m.proprietaire instanceof AglType) recepteur=(AglType)m.proprietaire;
	c.setEndObject(dejaPresent(_grille,recepteur));
	c.setEndPosition(7);
	c.setBeginX(c.getEndX()-100);
	c.setBeginY(c.getEndY()    );

	//_grille.avoidOverlap();
	_grille.recenterIfNeeded();
	_grille.revalidate();
      }
    }
  }

  public final static void creeGeneralisation(DjaGrid grid, AlmaDjaClasse o)
  {
    AglClasse c=o.getClasse();
    if(c==null) return;

    for(Enumeration e=grid.enumerate(AlmaDjaClasse.class);
	e.hasMoreElements(); )
    {
      AlmaDjaClasse p=(AlmaDjaClasse)e.nextElement();
      if(p==o) continue;

      AglClasse d=p.getClasse();
      if(d==null) continue;

      if(c.meres.noms.contains(d.nom))
      {
	// System.err.println("CREATE GENERALIZATION");

	DjaBrokenArrow arrow=new DjaBrokenArrow();
	grid.add(arrow);
	arrow.setBegin(o,1,0);
	arrow.setEnd(p,4,6);
	o.setY(p.getY()+p.getHeight()+4*DjaOptions.deltaY);
	o.setX(p.getX()+(p.getWidth()-o.getWidth())/2);
      }
    }
  }

  public final static void creeDerivation(DjaGrid grid, AlmaDjaClasse o)
  {
    AglClasse c=o.getClasse();
    if(c==null) return;

    for(Enumeration e=grid.enumerate(AlmaDjaClasse.class);
	e.hasMoreElements(); )
    {
      AlmaDjaClasse p=(AlmaDjaClasse)e.nextElement();
      if(p==o) continue;

      AglClasse d=p.getClasse();
      if(d==null) continue;

      if(d.meres.noms.contains(c.nom))
      {
	// System.err.println("CREATE DERIVATION");

	DjaBrokenArrow arrow=new DjaBrokenArrow();
	grid.add(arrow);
	arrow.setBegin(p,1,0);
	arrow.setEnd(o,4,6);
	o.setY(p.getY()-p.getHeight()-4*DjaOptions.deltaY);
	o.setX(p.getX()+(p.getWidth()-o.getWidth())/2);
      }
    }
  }

  public final static void creeRealisation(DjaGrid grid, AlmaDjaClasse o)
  {
    AglClasse c=o.getClasse();
    if(c==null) return;

    for(Enumeration e=grid.enumerate(AlmaDjaClasse.class);
	e.hasMoreElements(); )
    {
      AlmaDjaClasse p=(AlmaDjaClasse)e.nextElement();
      if(p==o) continue;

      AglClasse d=p.getClasse();
      if(d==null) continue;

      if(c.interfaces.noms.contains(d.nom))
      {
	// System.err.println("CREATE REALIZATION");

	DjaBrokenArrow arrow=new DjaBrokenArrow();
	arrow.putProperty("trace","3.2");
	grid.add(arrow);
	arrow.setBegin(o,1,0);
	arrow.setEnd(p,4,6);
	o.setY(p.getY()+p.getHeight()+4*DjaOptions.deltaY);
	o.setX(p.getX()+(p.getWidth()-o.getWidth())/2);
      }
    }
  }

  public final static void creeSpecification(DjaGrid grid, AlmaDjaClasse o)
  {
    AglClasse c=o.getClasse();
    if(c==null) return;

    for(Enumeration e=grid.enumerate(AlmaDjaClasse.class);
	e.hasMoreElements(); )
    {
      AlmaDjaClasse p=(AlmaDjaClasse)e.nextElement();
      if(p==o) continue;

      AglClasse d=p.getClasse();
      if(d==null) continue;

      if(d.interfaces.noms.contains(c.nom))
      {
	// System.err.println("CREATE SPECIFICATION");

	DjaBrokenArrow arrow=new DjaBrokenArrow();
	arrow.putProperty("trace","3.2");
	grid.add(arrow);
	arrow.setBegin(p,1,0);
	arrow.setEnd(o,4,6);
	o.setY(p.getY()-p.getHeight()-4*DjaOptions.deltaY);
	o.setX(p.getX()+(p.getWidth()-o.getWidth())/2);
      }
    }
  }

  public final static void creeNotes(DjaGrid grid, AlmaDjaClasse o)
  {
    AglClasse c=o.getClasse();
    if((c==null)||(c.commentaires==null)) return;

    for(Enumeration e=c.commentaires.elements();
	e.hasMoreElements(); )
    {
      Object p=e.nextElement();
      if(p instanceof AglCommentaireMulti)
      {
	AglCommentaireMulti cm=(AglCommentaireMulti)p;
	if(!cm.special) continue;

	String t=(String)cm.clefs.get("@main");
	if(t!=null)
	{
	  t=FuLib.replace(t,". ",".\n");

	  AlmaDjaNote note=new AlmaDjaNote(t);
	  note.setX(o.getX()+o.getWidth()+4*DjaOptions.deltaX);
	  note.setY(o.getY());
	  grid.add(note);
	  
	  DjaDirectArrow arrow=new DjaDirectArrow();
	  arrow.setBegin(note,7,0);
	  arrow.setEnd(o,7,0);
	  arrow.putProperty("trace","1.1");
	  grid.add(arrow);
	}
      }
    }
  }

  public static void creeContenance(DjaGrid grid, AlmaDjaClasse o)
  {
    AglClasse c=o.getClasse();
    if(c==null) return;

    int p=8;
    for(Enumeration e=c.attributs.elements(); e.hasMoreElements(); p+=2)
    {
      AglAttribut a=(AglAttribut)e.nextElement();

      //if(!a.visibilite.vstatic)
      {
	AglType at=AglTypeAlias.simplifie(a.type);

	//String cardinalite="1";
	while(at instanceof AglTypeTableau)
	{
	  //cardinalite="0..N";
	  at=AglTypeAlias.simplifie(((AglTypeTableau)at).type);
	}

	DjaObject t=dejaPresent(grid,at);

	if(t!=null)
	{
	  //if(cardinalite.equals("1"))
	  //cardinalite="0..1";

	  int q=0;
	  if(t instanceof AlmaDjaClasse)
	  {
	    q=6;
	    o.set("depliee",true);
	    //((AlmaDjaClasse)t).set("depliee",true);
	  }

	  DjaBrokenArrow arrow=new DjaBrokenArrow();
	  grid.add(arrow);
	  arrow.setBegin(o,((t.getX()>o.getX()) ? p+1 : p  ),9);
	  arrow.setEnd  (t,((t.getX()>o.getX()) ? q   : q+1),0);
	}
      }
    }
  }

  public static void creeContenu(DjaGrid grid, AlmaDjaClasse o)
  {
    Enumeration e1=grid.enumerate(AlmaDjaClasse.class);
    while(e1.hasMoreElements())
    {
      AlmaDjaClasse t=(AlmaDjaClasse)e1.nextElement();
      AglClasse     c=t.getClasse();
      if(c==null)   continue;

      int p=8;
      for(Enumeration e2=c.attributs.elements(); e2.hasMoreElements(); p+=2)
      {
	AglAttribut a=(AglAttribut)e2.nextElement();

	//if(!a.visibilite.vstatic)
	{
	  AglType at=AglTypeAlias.simplifie(a.type);

	  //String cardinalite="1";
	  while(at instanceof AglTypeTableau)
	  {
	    //cardinalite="0..N";
	    at=AglTypeAlias.simplifie(((AglTypeTableau)at).type);
	  }

	  if(o==dejaPresent(grid,at))
	  {
	    int q=0;
	    if(t instanceof AlmaDjaClasse)
	    {
	      q=6;
	      t.set("depliee",true);
	    }

	    DjaBrokenArrow arrow=new DjaBrokenArrow();
	    grid.add(arrow);
	    arrow.setBegin(t,((t.getX()>o.getX()) ? p   : p+1),9);
	    arrow.setEnd  (o,((t.getX()>o.getX()) ? q+1 : q  ),0);
	  }
	}
      }
    }
  }

  public static void creeDependance(DjaGrid grid, AlmaDjaClasse o)
  {
    AglClasse c=o.getClasse();
    if(c==null) return;

    int p=8+c.attributs.size()*2;
    for(Enumeration e=c.methodes.elements(); e.hasMoreElements(); p+=2)
    {
      AglMethode a=(AglMethode)e.nextElement();

      //if(!a.visibilite.vstatic)
      {
	AglType at=AglTypeAlias.simplifie(a.type);

	//String cardinalite="1";
	while(at instanceof AglTypeTableau)
	{
	  //cardinalite="0..N";
	  at=AglTypeAlias.simplifie(((AglTypeTableau)at).type);
	}

	DjaObject t=dejaPresent(grid,at);

	if(t!=null)
	{
	  //if(cardinalite.equals("1"))
	  //cardinalite="0..1";

	  int q=0;
	  if(t instanceof AlmaDjaClasse)
	  {
	    q=6;
	    o.set("depliee",true);
	    //((AlmaDjaClasse)t).set("depliee",true);
	  }

	  DjaBrokenArrow arrow=new DjaBrokenArrow();
	  grid.add(arrow);
	  arrow.setBegin(o,((t.getX()>o.getX()) ? p+1 : p  ),0);
	  arrow.setEnd  (t,((t.getX()>o.getX()) ? q   : q+1),5);
	}
      }
    }
  }

  public static void creeUtilisation(DjaGrid grid, AlmaDjaClasse o)
  {
    Enumeration e1=grid.enumerate(AlmaDjaClasse.class);
    while(e1.hasMoreElements())
    {
      AlmaDjaClasse t=(AlmaDjaClasse)e1.nextElement();
      AglClasse     c=t.getClasse();
      if(c==null)   continue;

      int p=8+c.attributs.size()*2;
      for(Enumeration e2=c.methodes.elements(); e2.hasMoreElements(); p+=2)
      {
	AglMethode a=(AglMethode)e2.nextElement();

	//if(!a.visibilite.vstatic)
	{
	  AglType at=AglTypeAlias.simplifie(a.type);

	  //String cardinalite="1";
	  while(at instanceof AglTypeTableau)
	  {
	    //cardinalite="0..N";
	    at=AglTypeAlias.simplifie(((AglTypeTableau)at).type);
	  }

	  if(o==dejaPresent(grid,at))
	  {
	    int q=0;
	    if(t instanceof AlmaDjaClasse)
	    {
	      q=6;
	      t.set("depliee",true);
	    }

	    DjaBrokenArrow arrow=new DjaBrokenArrow();
	    grid.add(arrow);
	    arrow.setBegin(t,((t.getX()>o.getX()) ? p   : p+1),0);
	    arrow.setEnd  (o,((t.getX()>o.getX()) ? q+1 : q  ),5);
	  }
	}
      }
    }
  }
}
