/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaDjaStruct.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;
import com.memoire.dja.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

public class AlmaDjaStruct
       extends DjaForm
       implements AlmaDjaValeur
{
  private transient Dimension size_;
  private transient AglStruct struct_;

  public AlmaDjaStruct()
    { this((String)null); }

  public AlmaDjaStruct(AglStruct _struct)
    { this(_struct.nom.dernier()); }

  public AlmaDjaStruct(String _text)
  {
    super(null);

    DjaText t0=new DjaText(this,0,_text,false,0,0,
			   RELATIVE_NW,0,MIDDLE,null,
			   defaultBoldFont,false,false);
    setTextArray(new DjaText[] { t0 });

    super.setWidth (0);
    super.setHeight(0);
    set("depliee"    ,false);
    set("visibilites",true);
    set("types"      ,true);
    set("balises"    ,true);
    textChanged(t0);
    //if(getWidth()<81) setWidth(81);
  }

  public AglStruct getStruct()
  { return struct_; }

  public AglObjet getObjet()
  { return getStruct(); }

  public boolean is(String _k)
  { return "vrai".equals(getProperty(_k)); }

  public void set(String _k, boolean _v)
  {
    putProperty(_k,_v ? "vrai" : "faux");
  }

  public void putProperty(String _k,String _v)
  {
    super.putProperty(_k,_v);
    size_=optimalSize();
  }

  public int getWidth()
  {
    if(size_==null) size_=optimalSize();
    return Math.max(size_.width,super.getWidth());
  }

  public int getHeight()
  {
    if(size_==null) size_=optimalSize();
    return Math.max(size_.height,super.getHeight());
  }

  public void textChanged(DjaText _text)
  {
    if(_text.getNum()==0)
    {
      struct_=null;

      String n=_text.getText();
      if(n!=null)
      {
	try
	{
	  AglType at=AglType.cherche(n);
	  if(at instanceof AglStruct) struct_=(AglStruct)at;
	}
	catch(Exception ex) { }
      }
    }

    size_=optimalSize();
  }

  public DjaText[] getTexts()
  {
    DjaText[] r=super.getTexts();
    r[0].setW(getWidth()-4);
    r[0].setX(2);
    r[0].setY(getFont().getSize());
    return r;
  }

  public DjaAnchor[] getAnchors()
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    int hs=getFont().getSize();

    int a=0;

    if(!is("depliee")&&(struct_!=null))
    {
      a=struct_.attributs.size();
    }

    int l=8+2*(a);

    DjaAnchor[] r=new DjaAnchor[l];
    r[0]=new DjaAnchor(this,0,NORTH,x+w/4  ,y);
    r[1]=new DjaAnchor(this,1,NORTH,x+w/2  ,y);
    r[2]=new DjaAnchor(this,2,NORTH,x+3*w/4,y);
    r[3]=new DjaAnchor(this,3,SOUTH,x+w/4  ,y+h);
    r[4]=new DjaAnchor(this,4,SOUTH,x+w/2  ,y+h);
    r[5]=new DjaAnchor(this,5,SOUTH,x+3*w/4,y+h);

    y+=hs;
    r[6]=new DjaAnchor(this,6,WEST,x  ,y+hs/2+1);
    r[7]=new DjaAnchor(this,7,EAST,x+w,y+hs/2+1);

    y+=hs;
    int h0=getTextArray()[0].getSize().height;
    r[6]=new DjaAnchor(this,6,WEST,x  ,y+h0/2+1);
    r[7]=new DjaAnchor(this,7,EAST,x+w,y+h0/2+1);
    y+=h0;

    y+=5+hs/2;
    for(int i=0;i<a;i++)
    {
      y+=hs;
      r[8+2*i]=new DjaAnchor(this,8+2*i,WEST,x  ,y+1);
      r[9+2*i]=new DjaAnchor(this,9+2*i,EAST,x+w,y+1);
    }

    return r;
  }

  public DjaHandle[] getHandles()
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    DjaHandle[] r=new DjaHandle[3];
    r[0]=new DjaHandle(this,EAST      ,x+w-1+deltaX,y+h/2       );
    r[1]=new DjaHandle(this,SOUTH_EAST,x+w-1+deltaX,y+h-1+deltaY);
    r[2]=new DjaHandle(this,SOUTH     ,x+w/2       ,y+h-1+deltaY);

    return r;
  }

  public void paintObject(Graphics _g)
  {
    int x=getX();
    int y=getY();
    int w=getWidth();
    int h=getHeight();

    Font ft=getFont();
    FontMetrics fm=_g.getFontMetrics(ft);

    int  hs=ft.getSize();
    int  ws;

    Color bg=getBackground();
    if(bg!=null)
    {
      DjaGraphics.setColor(_g,bg);
      DjaGraphics.fillRect(_g,x,y,w+1,h+1);
    }

    DjaGraphics.BresenhamParams bp=DjaGraphics.getBresenhamParams(this);

    Color fg=getForeground();
    if(fg!=null)
    {
      DjaGraphics.setColor(_g,fg);
      DjaGraphics.drawRect(_g,x,y,w,h,bp);
    }

    Color tc=getColor();
    if(tc!=null)
    {
      DjaGraphics.setColor(_g,tc);
      _g.setFont(ft);

      y+=hs;
      {
	String s="<<structure>>";
	ws=fm.stringWidth(s);
	_g.drawString(s,x+(w-ws)/2,y);
      }

      /*
      setTextBounds(new Rectangle(x+4,y+2,w-8,hs-1));
      y+=hs;
      if(nom!=null)
      {
	ws=fm.stringWidth(nom);
	_g.drawString(nom,x+(w-ws)/2,y);
      }
      */

      y+=getTextArray()[0].getSize().height;

      y+=hs;

      /*
      if((struct!=null)&&is("balises"))
      {
	String s="";
	// if(struct.visibilite.vfinal)    s+=",leaf";
	// if(struct.meres.noms.size()==0) s+=",root";
	if(!"".equals(s))
	{
	  s="{"+s.substring(1)+"}";
	  ws=fm.stringWidth(s);
	  _g.drawString(s,x+(w-ws)/2,y);
	}
      }
      */

      if(fg!=null)
      {
	DjaGraphics.setColor(_g,fg);
	DjaGraphics.drawLine(_g,x,y+3,x+w-1,y+3,bp);
	DjaGraphics.setColor(_g,tc);
      }

      y+=5;

      // System.err.println("STRUCT="+struct);
      // System.err.println("depliee="+is("depliee"));

      if((struct_!=null)&&is("depliee"))
      for(Enumeration e=struct_.attributs.elements();
	  e.hasMoreElements(); )
      {
	AglAttribut a=(AglAttribut)e.nextElement();
	y+=hs;
	_g.drawString(code(a.visibilite),x+4,y);
	_g.drawString(code(a),x+14,y);

	if(a.visibilite.vstatic)
	{
	  ws=Math.min(w-18,fm.stringWidth(a.nom));
	  _g.drawLine(x+14,y+1,x+14+ws-1,y+1);
	}
      }
    }

    super.paintObject(_g);
  }

  public Dimension optimalSize()
  {
    Font        ft=getFont();
    FontMetrics fm=DjaGraphics.getFontMetrics(this,ft);
    int         hs=ft.getSize();
    int         w =Math.max(0,2*deltaX-8);
    int         h =0;

    int ws;

    h+=2*hs;
    {
      String s="\"structure\"";
      w=Math.max(w,8+fm.stringWidth(s));
    }

    DjaText t=getTextArray()[0];
    Dimension d=t.getSize();
    w=Math.max(w,d.width+2);
    h+=d.height;

    h+=5;

    if((struct_!=null)&&is("depliee"))
    for(Enumeration e=struct_.attributs.elements();
	e.hasMoreElements(); )
    {
      AglAttribut a=(AglAttribut)e.nextElement();
      h+=hs;
      w=Math.max(w,18+fm.stringWidth(code(a)));
    }

    return new Dimension(Math.max(81,w),h+4);
  }

  // Temporaire

  public String code(AglType o)
  {
    String r="";
         if(o instanceof AglTypeAlias)   r+=code(((AglTypeAlias)o).type);
    else if(o instanceof AglTypeTableau) r+=code(((AglTypeTableau)o).type)+"[]";
    else r+=o.nom.dernier();
    return r;
  }

  private String code(AglVisibilite o)
  {
    String r="";
    if(o.vpublic)    r+="+";
    if(o.vprivate)   r+="-";
    if(o.vprotected) r+="#";
    if(o.vabstract)  r+="$";
    return r;
  }

  private String code(AglAttribut o)
  {
    String r=o.nom;
    if(is("types")) r+=":"+code(o.type);
    if(is("balises")&&o.visibilite.vfinal) r+=" {frozen}";
    return r;
  }
}

