/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaEditorPreferencesPanel.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.alma.*;
import com.memoire.bu.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * editor.type
 * editor.exec
 */

public class AlmaEditorPreferencesPanel
       extends BuAbstractPreferencesPanel
       implements ActionListener
{
  BuCommonInterface appli_;
  AlmaPreferences   options_;

  JRadioButton      cbInterne;
  JRadioButton      cbExterne;
  JRadioButton      cbPerso;
  JComboBox         chInterne;
  JComboBox         chExterne;
  JTextField        tfPerso;

  public String getTitle()
    { return BuResource.BU.getString("Editeur"); }

  // Constructeur

  public AlmaEditorPreferencesPanel(BuCommonInterface _appli)
  {
    super();
    appli_=_appli;

    options_=AlmaPreferences.ALMA;

    BuGridLayout lo=new BuGridLayout();
    lo.setColumns(2);
    lo.setVgap(5);
    lo.setHgap(5);

    JPanel p=new JPanel();
    p.setBorder
      (new CompoundBorder
       (new TitledBorder(BuResource.BU.getString("Editeur")),
	new EmptyBorder(5,5,5,5)));
    p.setLayout(lo);

    ButtonGroup bg=new ButtonGroup();

    cbInterne=new JRadioButton(BuResource.BU.getString("interne"  ));
    cbExterne=new JRadioButton(BuResource.BU.getString("externe"  ));
    cbPerso  =new JRadioButton(BuResource.BU.getString("personnel"));
    cbInterne.setName("cbEDITOR_INTERNAL" );
    cbExterne.setName("cbEDITOR_EXTERNAL" );
    cbPerso  .setName("cbEDITOR_PERSONNAL");
    bg.add(cbInterne);
    bg.add(cbExterne);
    bg.add(cbPerso);

    p.add(cbInterne);
    cbInterne.addActionListener(this);

    chInterne=new JComboBox();
    chInterne.setName("chINTERNE");
    chInterne.addItem("butextarea");
    chInterne.addItem("butextpane");
    chInterne.addItem("jedittextarea");
    p.add(chInterne);

    p.add(cbExterne);
    cbExterne.addActionListener(this);

    chExterne=new JComboBox();
    chExterne.setName("chEXTERNE");
    chExterne.addItem("jext");
    p.add(chExterne);

    p.add(cbPerso);
    cbPerso.addActionListener(this);

    tfPerso=new JTextField(20);
    tfPerso.setName("tfPERSO");
    tfPerso.setText("emacs");
    p.add(tfPerso);

    setLayout(new BorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    add("Center",p);

    chInterne.setEnabled(false);
    chExterne.setEnabled(false);
    tfPerso.setEnabled  (false);

    updateComponents();
  }

  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    setAbility();
  }
  
  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
  }

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
    int    type=1;
    String exec="";

    if(cbInterne.isSelected())
    {
      type=1;
      exec=(String)chInterne.getSelectedItem();
    }
    if(cbExterne.isSelected())
    {
      type=2;
      exec=(String)chExterne.getSelectedItem();
    }
    if(cbPerso.isSelected())
    {
      type=3;
      exec=tfPerso.getText();
    }

    // System.err.println("TYPE="+type);
    // System.err.println("EXEC="+exec);

    options_.putIntegerProperty("editor.type",type);
    options_.putStringProperty ("editor.exec",exec);
  }

  private void updateComponents()
  {
    int    type=options_.getIntegerProperty("editor.type");
    String exec=options_.getStringProperty("editor.exec");

    switch(type)
    {
      case 2:
        cbExterne.setSelected(true);
        chExterne.setSelectedItem(exec);
        break;
      case 3:
	cbPerso.setSelected(true);
	tfPerso.setText(exec);
	break;
      default:
	cbInterne.setSelected(true);
        chInterne.setSelectedItem(exec);
	break;
    }

    setAbility();
  }

  private void setAbility()
  {
    chInterne.setEnabled(cbInterne.isSelected());
    chExterne.setEnabled(cbExterne.isSelected());
    tfPerso  .setEnabled(cbPerso  .isSelected());
  }
}
