/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaFilleArbre.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;

public class AlmaFilleArbre
       extends BuInternalFrame
       implements ActionListener,
		  TreeSelectionListener, TreeExpansionListener,
		  BuCutCopyPasteInterface, /*BuPrintable,*/
		  BuSelectFindReplaceInterface
{
  private AlmaImplementation app_;

  private JComponent         content_;
  // private BuHorizontalLayout layout_;

  private BuButton       btRafraichir,btCacher,btEditer,btDessiner;
  private BuMenuItem     miCacher;
  private BuMenu         menu_;
  private AlmaArbre      trArbre_;
  private JScrollPane    spArbre_;
  private BuTextArea     taSource_;
  private JScrollPane    spSource_;

  public AlmaFilleArbre(AlmaImplementation _app)
  {
    super("",true,true,true,true);
    app_=_app;

    setName("ifARBRE");
    app_.installContextHelp(getRootPane(),"alma/p-arbre.html");

    /*
    content_=(JComponent)getContentPane();
    layout_=new BuHorizontalLayout();
    layout_.setHgap(5);
    layout_.setHfilled(true);
    layout_.setVfilled(true);
    content_.setLayout(layout_);
    content_.setBorder(new EmptyBorder(5,5,5,5));
    */

    btRafraichir=new BuButton(AlmaResource.ALMA.getString("Rafrachir")); 
    btRafraichir.setActionCommand("RAFRAICHIR");
    btRafraichir.setToolTipText
      (AlmaResource.ALMA.getString("Rafrachir l'arbre")); 
    btRafraichir.setIcon(BuResource.BU.loadToolCommandIcon("RAFRAICHIR"));
    btRafraichir.setName("btRAFRAICHIR_ARBRE");
    btRafraichir.setRequestFocusEnabled(false);
    btRafraichir.addActionListener(this);

    btCacher=new BuButton(AlmaResource.ALMA.getString("Cacher"));
    btCacher.setActionCommand("CACHER");
    btCacher.setToolTipText
      (AlmaResource.ALMA.getString("Cacher le code")); 
    btCacher.setIcon(BuResource.BU.loadToolCommandIcon("CACHER"));
    btCacher.setName("btCACHER_CODE");
    btCacher.setRequestFocusEnabled(false);
    btCacher.addActionListener(this);

    btEditer=new BuButton(AlmaResource.ALMA.getString("Editer"));
    btEditer.setActionCommand("EDITER");
    btEditer.setToolTipText
      (AlmaResource.ALMA.getString("Editer la slection")); 
    btEditer.setIcon(BuResource.BU.loadToolCommandIcon("EDITER"));
    btEditer.setName("btEDITER_OBJET");
    btEditer.setRequestFocusEnabled(false);
    btEditer.addActionListener(this);

    btDessiner=new BuButton(AlmaResource.ALMA.getString("Dessiner"));
    btDessiner.setActionCommand("DESSINER");
    btDessiner.setToolTipText
      (AlmaResource.ALMA.getString("Dessiner la slection")); 
    btDessiner.setName("btDESSINER_OBJET");
    btDessiner.setIcon(BuResource.BU.loadToolCommandIcon("GRAPHE"));
    btDessiner.setRequestFocusEnabled(false);
    btDessiner.addActionListener(this);

    menu_=new BuMenu(AlmaResource.ALMA.getString("Arbre"),"MENU_ARBRE"); 
    menu_.addMenuItem(AlmaResource.ALMA.getString("Rafrachir"),"RAFRAICHIR",this);
    miCacher=menu_.addMenuItem(AlmaResource.ALMA.getString("Cacher"    ),"CACHER"    ,this);
    menu_.addMenuItem(AlmaResource.ALMA.getString("Editer"    ),"EDITER"    ,this);
    menu_.addMenuItem(AlmaResource.ALMA.getString("Dessiner"),
		      "DESSINER",
		      BuResource.BU.loadMenuCommandIcon("GRAPHE"),
		      true,0,this);
    menu_.computeMnemonics();
    menu_.setMnemonic('X');//menu_.getText().charAt(0));

    trArbre_=new AlmaArbre();
    //trArbre_.setName("trARBRE_MODELE");
    trArbre_.addTreeSelectionListener(this);
    trArbre_.addTreeExpansionListener(this);
    spArbre_=new BuScrollPane(trArbre_);
    // spArbre_.setPreferredSize(new Dimension(200,50));
    AlmaDnd.SOURCE.add(trArbre_);

    taSource_=new BuTextArea();
    taSource_.setName("taSOURCE");
    taSource_.setEditable(false);
    taSource_.setOpaque(false);
    spSource_=new BuScrollPane(taSource_);
    // spSource_.setPreferredSize(new Dimension(150,150));

    /*
    content_.add(spArbre_);
    content_.add(spSource_);
    */

    content_=new BuSplit2Pane(spArbre_,spSource_);
    setContentPane(content_);

    setTitle(AlmaResource.ALMA.getString("Arbre du modle"));
    setFrameIcon(BuResource.BU.getBarIcon("arbre"));

    Dimension ps=new Dimension(370,296); // 370
    setSize(ps);
    setPreferredSize(ps);
  }

  public void rafraichir()
  {
    trArbre_.rafraichir();
    spArbre_.revalidate();
  }

  public void cacher()
  {
    boolean t=!spSource_.isVisible();
    spSource_.setVisible(t);

    if(!t)
    {
      taSource_.setText("");
      content_.remove(spSource_);

      String s=AlmaResource.ALMA.getString("Voir");
      btCacher.setText(s);
      miCacher.setText(s);

      BuIcon i=BuResource.BU.loadToolCommandIcon("VOIR");
      btCacher.setIcon(i);
      miCacher.setIcon(i);
    }
    else
    {
      content_.add(spSource_);

      String s=AlmaResource.ALMA.getString("Cacher");
      btCacher.setText(s);
      miCacher.setText(s);

      BuIcon i=BuResource.BU.loadToolCommandIcon("CACHER");
      btCacher.setIcon(i);
      miCacher.setIcon(i);
    }

    btCacher.revalidate();
    relayout();
  }

  public void editer()
  {
    TreePath[] path=trArbre_.getSelectionPaths();;
    if((path!=null)&&(path.length>0))
    {
      Object o=path[0].getLastPathComponent();
      if(o instanceof AlmaNoeud)
      {
	AlmaNoeud n=(AlmaNoeud)o;
	Object    v=n.getValue();
	if(v!=null)
	{
	  AlmaEditeur ed=null;

	  if(v instanceof AglPatron)
	    ed=new AlmaEditeurPatron(app_,(AglPatron)v);

	  if(v instanceof AglCondition)
	    ed=new AlmaEditeurCondition(app_,(AglCondition)v);

	  if(ed!=null)
	  {
	    ed.activate();
	    String t=updateSource(v,n);
	    taSource_.setText(t);
	    taSource_.setCaretPosition(0);
	    relayout();
	  }
	}
      }
    }
  }

  public void dessiner()
  {
    TreePath[] path=trArbre_.getSelectionPaths();;
    if(path==null) return;

    Object[] objets=new Object[path.length];
    for(int j=0; j<path.length; j++)
    {
      Object p=path[j].getLastPathComponent();
      if(p instanceof AlmaNoeud)
	objets[j]=((AlmaNoeud)p).getValue();
    }

    AlmaTacheDessiner tache=new AlmaTacheDessiner(app_,objets);
    tache.setPriority(Thread.MIN_PRIORITY);
    tache.start();

    /*
    TreePath[] path=trArbre_.getSelectionPaths();;
    if(path==null) return;

    DjaGrid grille =new DjaInteractiveGrid();
    String  fichier="SansTitre.dja";
      
    for(int j=0; j<path.length; j++)
    {
      Object p=path[j].getLastPathComponent();
      if(p instanceof AlmaNoeud)
      {
	AlmaNoeud n=(AlmaNoeud)p;
	Object    v=n.getValue();
	if(v instanceof AglObjet) // !=null
	{
	  DjaObject o=AlmaDjaRelations.alreadyThere(grille,(AglObjet)v);
	  if(o!=null) continue;

	  if(v instanceof AglClasse)
	    o=new AlmaDjaClasse((AglClasse)v);
	  else
	  if(v instanceof AglStruct)
	    o=new AlmaDjaStruct((AglStruct)v);
	  else
	  if(v instanceof AglType)
	    o=new DjaBox(((AglType)v).nom.dernier());
	  else
	  if(v instanceof AglPaquet)
	    o=new AlmaDjaPaquet((AglPaquet)v);
	  else
	  if(v instanceof AglRelationBiOrientee)
	    o=new AlmaDjaRelationBiOrientee((AglRelationBiOrientee)v);
	  else
	  if(v instanceof AglCommentaire)
	    o=new AlmaDjaCommentaire((AglCommentaire)v);

	  if(o!=null) grille.add(o);

	  if(o instanceof AlmaDjaClasse)
	  {
	    AlmaDjaClasse c=(AlmaDjaClasse)o;
	    AlmaDjaRelations.createGeneralization(grille,c);
	    AlmaDjaRelations.createDerivation    (grille,c);
	    AlmaDjaRelations.createRealization   (grille,c);
	    AlmaDjaRelations.createSpecification (grille,c);
	    AlmaDjaRelations.createNotes         (grille,c);
	  }

	  grille.recenter();

	  if(v instanceof AglPatron)
	  {
	    AglPatron     patron=(AglPatron)v;
	    AglPatronType type  =patron.type;

	    try
	    {
	      InputStream in=getClass().getResourceAsStream
		("Pattern"+type.nom+".dja");
	      LineNumberReader rin=new LineNumberReader
		(new InputStreamReader(in));

	      String l,r;
	      r="";
	      while(true)
	      {
		l=rin.readLine();
		if(l==null) break;
		r+=l+"\n";
	      }

	      for(int i=0;i<type.roles.length; i++)
	      {
		String role=type.roles[i];
		String acteur=(String)patron.roles.get(role);

		if(acteur!=null)
		  while(true)
		  {
		    int k=r.indexOf("%24"+role);
		    if(k<0) break;
		    r=r.substring(0,k)+acteur+r.substring(k+role.length()+3);
		  }
	      }

	      in=new StringBufferInputStream(r);
	      AlmaFilleDiagramme fille=new AlmaFilleDiagramme
		(app_,patron.nom+".dja",DjaLoader.loadAsText(in,true));
	      app_.ajouteFenetre(fille);
	    }
	    catch(Exception ex) { }
	  }
	}
      }
    }

    if(grille.getObjects().size()>0)
    {
      AlmaFilleDiagramme fille=new AlmaFilleDiagramme
	(app_,fichier,grille);
      app_.ajouteFenetre(fille);
    }
    */
  }

  public void setSelected(boolean _state) throws PropertyVetoException
  {
    super.setSelected(_state);
    if(isSelected()) trArbre_.requestFocus();
   }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
         if(action.equals("RAFRAICHIR")) rafraichir();
    else if(action.equals("CACHER"    )) cacher();
    else if(action.equals("EDITER"    )) editer();
    else if(action.equals("DESSINER"  )) dessiner();
    else super.actionPerformed(_evt);
  }

  public void valueChanged(TreeSelectionEvent _evt)
  {
    if(!spSource_.isVisible()) return;
    
    TreePath path=_evt.getPath();

    String t="";
    if(_evt.isAddedPath(path))
    {
      Object o=path.getLastPathComponent();
      if(o instanceof AlmaNoeud)
      {
	AlmaNoeud n=(AlmaNoeud)o;
	Object    v=n.getValue();

	if(v!=null) t=updateSource(v,n);
      }
    }

    taSource_.setText(t);
    taSource_.setCaretPosition(0);
    relayout();
  }

  private String updateSource(Object v, AlmaNoeud n)
  {
    String t;

    try
    {
      AglCode c=AglOptions.code();
      t=c.code(v);
      if(AglOptions.appliquerExpReg) t=c.appliqueExpReg(t);
    }
    catch(Exception ex)
    {
      t="N'a pu gnrer le code de "+n.getTitle()
	+"\n"+ex.getMessage();
      // System.err.println(t);
      ex.printStackTrace();
    }

    return t;
  }

  public void treeCollapsed(TreeExpansionEvent _evt)
    { relayout(); }

  public void treeExpanded(TreeExpansionEvent _evt)
    { relayout(); }

  private void relayout()
  {
    taSource_.setSize(taSource_.getPreferredSize());
    /*
    taSource_.invalidate();
    taSource_.doLayout();
    spSource_.invalidate();
    spSource_.doLayout();
    */
    content_.doLayout();
    content_.validate();
    doLayout();
    validate();
  }

  // BuCutCopyPasteInterface

  // public String getFichier() { return fichier; }
  public String getSource()  { return taSource_.getText(); }

  public void cut()       { } // { taSource_.cut();   }
  public void copy()      { taSource_.copy();  }
  public void paste()     { } // { taSource_.paste(); }
  public void duplicate() { }

  // BuSelectFindReplaceInterface

  public void find()    { }
  public void replace() { }
  public void select()  { taSource_.select(0,taSource_.getLength()); }

  // BuPrintable

  /*
  public void print(PrintJob _job,Graphics _g)
  {
    BuPrinter.INFO_DOC     =new BuInformationsDocument();
    BuPrinter.INFO_DOC.name=getTitle();
    BuPrinter.INFO_DOC.logo=BuResource.BU.getIcon("texte",24);

    BuPrinter.printString(_job,_g,getSource());
  }
  */

  // BuInternalFrame

  public String[] getEnabledActions()
  {
    String[] r=new String[]
      {
	//"IMPRIMER",
	//"PREVISUALISER",
	"COPIER",
	"TOUTSELECTIONNER",
      };
    return r;
  }

  public String[] getDisabledActions()
  {
    String[] r=new String[] { "ARBRE_MODELE" };
    return r;
  }

  public JMenu[] getSpecificMenus()
  {
    JMenu[] r=new JMenu[1];
    r[0]=menu_;
    return r;
  }

  public JComponent[] getSpecificTools()
  {
    JComponent[] r=new JComponent[4];
    r[0]=btRafraichir;
    r[1]=btCacher;
    r[2]=btEditer;
    r[3]=btDessiner;
    return r;
  }
}
