/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaFilleMessages.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.beans.*;
import java.io.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class AlmaFilleMessages
       extends BuInternalFrame
       implements ActionListener,
                  ListSelectionListener, ListDataListener,
                  AglTerminal,
                  BuCutCopyPasteInterface, BuPrintable
{
  private AlmaImplementation app_;
  private JComponent         content_;

  private BuButton         btEffacer_,btAller_;
  private BuMenu           menu_;
  private BuScrollPane     spTexte;
  private BuEmptyList      lsTexte;
  private DefaultListModel texte_;

  public AlmaFilleMessages(AlmaImplementation _app)
  {
    super("",true,true,true,true);
    app_=_app;

    setName("ifMESSAGES");
    app_.installContextHelp(getRootPane(),"alma/p-messages.html");

    content_=(JComponent)getContentPane();
    content_.setLayout(new BuBorderLayout());
    content_.setBorder(new EmptyBorder(5,5,5,5));

    /*
    JPanel     bar=new JPanel();
    FlowLayout barlo=new FlowLayout(FlowLayout.LEFT);
    barlo.setHgap(0);
    barlo.setVgap(0);
    bar.setLayout(barlo);
    bar.setBorder(new EmptyBorder(2,2,2,2));
    */

    btEffacer_=new BuButton(AlmaResource.ALMA.getString("Effacer"));
    btEffacer_.setActionCommand("EFFACER");
    btEffacer_.setToolTipText
      (AlmaResource.ALMA.getString("Effacer tous les messages")); 
    btEffacer_.setIcon(BuResource.BU.loadToolCommandIcon("EFFACER"));
    btEffacer_.setName("btEFFACER_TOUSMESSAGES");
    btEffacer_.setRequestFocusEnabled(false);
    btEffacer_.addActionListener(this);

    btAller_=new BuButton(AlmaResource.ALMA.getString("Aller"));
    btAller_.setActionCommand("ALLER");
    btAller_.setToolTipText
      (AlmaResource.ALMA.getString("Aller dans le source")); 
    btAller_.setIcon(BuResource.BU.loadToolCommandIcon("ALLER"));
    btAller_.setName("btALLER_LIGNE");
    btAller_.setRequestFocusEnabled(false);
    btAller_.addActionListener(this);

    menu_=new BuMenu (AlmaResource.ALMA.getString("Messages"),"MENU_MESSAGES"); 
    menu_.addMenuItem(AlmaResource.ALMA.getString("Effacer"),"EFFACER",this);
    menu_.addMenuItem(AlmaResource.ALMA.getString("Aller"  ),"ALLER"  ,this);
    menu_.computeMnemonics();
    menu_.setMnemonic('X'); //menu_.getText().charAt(0));

    // bar.add(btEffacer_);
    // content_.add(bar,BuBorderLayout.NORTH);

    texte_=new DefaultListModel();
    texte_.addListDataListener(this);

    lsTexte=new BuEmptyList();
    lsTexte.setModel(texte_);
    lsTexte.setEmptyText(AlmaResource.ALMA.getString
			 ("Aucun message"));
    lsTexte.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    // lsTexte.setBackground(new Color(224,224,255));
    lsTexte.setBorder(new EmptyBorder(0,0,0,0));
    lsTexte.setCellRenderer(new CR());//
    //CellRenderer(lsTexte.getCellRenderer()));
    lsTexte.addListSelectionListener(this);

    spTexte=new BuScrollPane(lsTexte);
    content_.add(spTexte,BuBorderLayout.CENTER);

    setTitle(AlmaResource.ALMA.getString("Messages"));
    setFrameIcon(BuResource.BU.getBarIcon("message"));

    Dimension ps=new Dimension(370,185);
    setSize(ps);
    setPreferredSize(ps);
  }

  public void setSelected(boolean _state) throws PropertyVetoException
  {
    super.setSelected(_state);

    if(isSelected())
      lsTexte.requestFocus();
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
         if(action.equals("EFFACER")) effacer();
    else if(action.equals("ALLER"  )) aller();
    else super.actionPerformed(_evt);
  }

  public void valueChanged(ListSelectionEvent _evt)
  { adjustActions(); }

  public void contentsChanged(ListDataEvent _evt)
  { adjustActions(); }

  public void intervalAdded(ListDataEvent _evt)
  { adjustActions(); }

  public void intervalRemoved(ListDataEvent _evt)
  { adjustActions(); }

  // Actions

  public void effacer()
  {
    texte_.removeAllElements();
  }

  public void aller()
  {
    try
    {
      String s=""+lsTexte.getSelectedValue();
      int i=s.lastIndexOf("ligne ");
      int j=s.lastIndexOf("fichier ");
      int k=s.lastIndexOf("]");

      String fichier=s.substring(j+8,k);

      String ligne;
      if(i<0) ligne="1";
      else    ligne=s.substring(i+6,j-1);

      //System.err.println("ALLER -"+fichier+":"+ligne+"-");
      app_.montrerErreur(fichier,Integer.parseInt(ligne)-1);
    }
    catch(Exception ex) { }
  }

  // AglTerminal

  private int cpt_=0;

  private /*synchronized*/ void texte(String s, boolean m)
  {
    try
    {
      cpt_++;
      texte_.addElement(s);
      if(m) app_.getMainPanel().setMessage(s);
      if(m||(cpt_%20==0))
      {
	int t=texte_.size();
	/*
	if(AglOptions.trace&&(t>=100))
	{
	  for(int i=19;i>=0;i--)
	    if(((String)texte_.elementAt(i)).startsWith("Trace        : "))
	      texte_.removeElementAt(i);
	  t=texte_.size();
	}
	*/
	lsTexte.ensureIndexIsVisible(t-1);
      }
    }
    catch(Exception ex) { ex.printStackTrace(); }
  }

  public void avertissement(String s)
  {
    if(AglOptions.avertissement)
      texte("Avertissement: "+s,true);
  }

  public void syntaxe(String s)
  {
    texte("Syntaxe      : "+s,true);
    Thread t=Thread.currentThread();
    if((t instanceof AlmaTacheAnalyser)||(t instanceof AlmaTacheGenerer))
    {
      progres(0);
      if(t instanceof BuTask)
	app_.getMainPanel().getTaskView().removeTask((BuTask)t);
      t.stop();
    }
  }

  public void erreur(String s)
  {
    texte("Erreur       : "+s,true);
    Thread t=Thread.currentThread();
    if((t instanceof AlmaTacheAnalyser)||(t instanceof AlmaTacheGenerer))
    {
      progres(0);
      if(t instanceof BuTask)
	app_.getMainPanel().getTaskView().removeTask((BuTask)t);
      t.stop();
    }
  }

  public void suivi(String s)
  {
    if(AglOptions.suivi)
      texte("Suivi        : "+s,false);
  }

  public void trace(String s)
  {
    if(AglOptions.trace)
      texte("Trace        : "+s,false);
  }

  public void message(String s)
  {
    texte(s,true);
  }

  public void sortie(String s)
  {
    texte(s,true);
  }

  public void progres(int _v)
  {
    app_.getMainPanel().setProgression(_v);
  }

  public InputStream  getInputStream()
  {
    return System.in;
  }

  public OutputStream getOutputStream()
  {
    return System.out;
  }

  public OutputStream getErrorStream()
  {
    return System.err;
  }

  // BuInternalFrame

  public String[] getEnabledActions()
  {
    String[] r=new String[]
      {
	"IMPRIMER", "PREVISUALISER", 
	"COUPER", "COPIER",
      };
    return r;
  }

  public String[] getDisabledActions()
  {
    String[] r=new String[] { "MESSAGE" };
    return r;
  }

  public JMenu[] getSpecificMenus()
  {
    JMenu[] r=new JMenu[1];
    r[0]=menu_;
    return r;
  }


  public JComponent[] getSpecificTools()
  {
    JComponent[] r=new JComponent[2];
    r[0]=btEffacer_;
    r[1]=btAller_;
    return r;
  }

  public void adjustActions()
  {
    boolean s=(lsTexte.getSelectedValues().length>0);
    app_.setEnabledForAction("COUPER",s);
    app_.setEnabledForAction("COPIER",s);

    boolean e=(texte_.getSize()>0);
    btEffacer_.setEnabled(e);
    BuActionEnabler.setEnabledForAction(menu_,"EFFACER",e);

    boolean a=(lsTexte.getSelectedValues().length==1);
    btAller_.setEnabled(a);
    BuActionEnabler.setEnabledForAction(menu_,"ALLER",a);
  }

  // BuPrintable

  public void print(PrintJob _job,Graphics _g)
  {
    BuPrinter.INFO_DOC     =new BuInformationsDocument();
    BuPrinter.INFO_DOC.name=getTitle();
    BuPrinter.INFO_DOC.logo=BuResource.BU.getIcon("message",24);

    String s="";
    int    n=texte_.getSize();
    for(int i=0;i<n;i++)
      s+=texte_.elementAt(i).toString()+System.getProperty("line.separator");

    s=FuLib.replace(s,"\t","        ");
    BuPrinter.printString(_job,_g,s);
  }

  // BuCutCopyPasteInterface

  public void cut()
  {
    copy();

    Object[] sv=lsTexte.getSelectedValues();

    for(int i=0;i<sv.length;i++)
      texte_.removeElement(sv[i]);

    lsTexte.setSelectedValue(null,true);
  }

  public void copy()
  {
    Object[] sv=lsTexte.getSelectedValues();

    if(sv.length>0)
    {
      String r="";

      for(int i=0;i<sv.length;i++)
	r+=sv[i].toString()+System.getProperty("line.separator");

      Clipboard clipboard=getToolkit().getSystemClipboard();
      StringSelection contents=new StringSelection(r);
      clipboard.setContents(contents,null);
    }
  }

  public void paste    () { }
  public void duplicate() { }

  private final static Color ERREUR =new Color(128,  0,  0);
  private final static Color SYNTAXE=new Color(128,  0,128);
  private final static Color SUIVI  =new Color(  0,128,128);
  private final static Color TRACE  =new Color(128,128,128);

  private static class CR
    extends BuAbstractCellRenderer
  {
    public CR()
    {
      super(BuAbstractCellRenderer.LIST);
    }

    public Component getListCellRendererComponent(
         JList   _list,
         Object  _value,
         int     _index,
         boolean _selected,
         boolean _focus)
    {
      BuLabel r=(BuLabel)super.getListCellRendererComponent
	  (_list,_value,_index,_selected,_focus);

      String v=r.getText();

      if(v.startsWith("Erreur"))
	r.setForeground(ERREUR);
      else
      if(v.startsWith("Syntaxe"))
	r.setForeground(SYNTAXE);
      else
      if(v.startsWith("Suivi"))
	r.setForeground(SUIVI);
      else
      if(v.startsWith("Trace"))
	r.setForeground(TRACE);

      return r;
    }
  }
}
