/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaFilleProjet.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

public class AlmaFilleProjet
       extends BuInternalFrame
       implements ActionListener,
                  //TableModelListener,
                  BuCutCopyPasteInterface,
                  BuSelectFindReplaceInterface,
                  BuPrintable
{
  private AlmaImplementation app_;

  private JComponent         content_;
  private BuHorizontalLayout layout_;

  private BuButton           btAjouter_,btEnlever_,btVoir_;
  private BuMenu             menu_;
  private AlmaTableauProjet  tbProjet_;
  private BuScrollPane       spProjet_;

  public AlmaFilleProjet(AlmaImplementation _app)
  {
    super("",true,true,true,true);
    app_=_app;

    setName("ifPROJET");
    app_.installContextHelp(getRootPane(),"alma/p-projet.html");

    content_=(JComponent)getContentPane();
    layout_=new BuHorizontalLayout();
    layout_.setHgap(0);
    layout_.setHfilled(true);
    layout_.setVfilled(true);
    content_.setLayout(layout_);
    content_.setBorder(new EmptyBorder(5,5,5,5));

    btAjouter_=new BuButton(AlmaResource.ALMA.getString("Ajouter"));
    btAjouter_.setActionCommand("AJOUTER");
    btAjouter_.setToolTipText
      (AlmaResource.ALMA.getString("Ajouter une unit")); 
    btAjouter_.setIcon(BuResource.BU.loadToolCommandIcon("AJOUTER"));
    btAjouter_.setName("btAJOUTER_UNITE");
    btAjouter_.setRequestFocusEnabled(false);
    btAjouter_.addActionListener(this);

    btEnlever_=new BuButton(AlmaResource.ALMA.getString("Enlever"));
    btEnlever_.setActionCommand("ENLEVER");
    btEnlever_.setToolTipText
      (AlmaResource.ALMA.getString("Enlever les units slectionnes")); 
    btEnlever_.setIcon(BuResource.BU.loadToolCommandIcon("ENLEVER"));
    btEnlever_.setName("btENLEVER_UNITE");
    btEnlever_.setRequestFocusEnabled(false);
    btEnlever_.addActionListener(this);

    btVoir_=new BuButton(AlmaResource.ALMA.getString("Voir"));
    btVoir_.setActionCommand("VOIR");
    btVoir_.setToolTipText
      (AlmaResource.ALMA.getString("Voir les options")); 
    btVoir_.setIcon(BuResource.BU.loadToolCommandIcon("VOIR"));
    btVoir_.setName("btVOIR_PROJET");
    btVoir_.setRequestFocusEnabled(false);
    btVoir_.addActionListener(this);

    menu_=new BuMenu (AlmaResource.ALMA.getString("Projet" ),"MENU_PROJET"); 
    menu_.addMenuItem(AlmaResource.ALMA.getString("Ajouter"),"AJOUTER",this);
    menu_.addMenuItem(AlmaResource.ALMA.getString("Enlever"),"ENLEVER",this);
    menu_.addMenuItem(AlmaResource.ALMA.getString("Voir"   ),"VOIR"   ,this);
    menu_.computeMnemonics();
    menu_.setMnemonic('X'); //menu_.getText().charAt(0));

    tbProjet_=new AlmaTableauProjet();
    //tbProjet_.getModel().addTableModelListener(this);
    spProjet_=new BuScrollPane(tbProjet_);
    tbProjet_.addRowHeaders(spProjet_,true);
    content_.add(spProjet_);

    setTitle(AlmaResource.ALMA.getString("Projet"));
    setFrameIcon(AlmaResource.ALMA.getBarIcon("projet"));

    Dimension ps=new Dimension(444,185);
    setSize(ps);
    setPreferredSize(ps);
  }

  private String contenuAlmaProjet()
  {
    String      l=null;
    String      r="";

    if(AglOptions.applet) r+="-a\n";
    if(AglOptions.suivi ) r+="-v\n";
    if(AglOptions.trace ) r+="-vv\n";

    if(AglOptions.classesFondamentales) r+="-f\n";
    if(AglOptions.analyserCommentaires) r+="--comments\n";
    if(AglOptions.definirLiteraux     ) r+="--literals\n";

    if(!AglOptions.entete        ) r+="--no-header\n";
    if(AglOptions.rappelClasses  ) r+="-lc\n";
    if(AglOptions.rappelPatrons  ) r+="-lp\n";
    if(AglOptions.rappelRelations) r+="-lr\n";
    //if(AglOptions.simplifie    ) r+="-s\n";
    if(AglOptions.appliquerExpReg) r+="--regexp\n";

    r+="-i\n"+AglOptions.formatImage+"\n";
    r+="--css\n"+AglOptions.styleHtml+"\n";
    if(!"".equals(AglOptions.module)) r+="-fm\n"+AglOptions.module+"\n";
    if(!"".equals(AglOptions.classe)) r+="-fc\n"+AglOptions.classe+"\n";

    l="";
    for(Enumeration e=AglUnite.table.elements(); e.hasMoreElements(); )
    {
      AglUnite u=(AglUnite)e.nextElement();
      if(!u.cible)
      {
	if(!l.equals(u.langage))
	  { l=u.langage; r+="-p\n"+l+"\n"; }
	File f=new File(u.fichier);
	if(f.exists()) r+=f.getAbsolutePath()+"\n";
	else r+=u.fichier+"\n";
      }
    }

    l="";
    for(Enumeration e=AglUnite.table.elements(); e.hasMoreElements(); )
    {
      AglUnite u=(AglUnite)e.nextElement();
      if(u.cible)
      {
	if(!l.equals(u.langage))
	  { l=u.langage; r+="-g\n"+l+"\n"; }
	File f=new File(u.fichier);
	r+="-o\n";
	if(f.exists()) r+=f.getAbsolutePath()+"\n";
	else r+=u.fichier+"\n";
      }
    }

    return r;
  }

  public void rafraichir()
  {
    tbProjet_.rafraichir();
  }

  public void ajouter()
  {
  }

  public void enlever()
  {
    System.err.println("AlmaFilleProjet: enlever");

    int[] rows=tbProjet_.getSelectedRows();
    AglUnite[] units=new AglUnite[rows.length];

    for(int j=0;j<rows.length;j++)
    {
      int i=0;
      for(Enumeration e=AglUnite.table.elements(); e.hasMoreElements(); )
      {
	AglUnite u=(AglUnite)e.nextElement();
	if(i==rows[j])
	{
	  System.err.println("enlever: "+u.fichier);
	  AglUnite.table.remove(u.fichier);
	  break;
	}
	i++;
      }
    }

    rafraichir();
    /*
    for(int j=0;j<rows.length;j++)
    {
      tbProjet_.tableChanged
	(new TableModelEvent(tbProjet_.getModel(),rows[j],rows[j],
			     TableModelEvent.ALL_COLUMNS,
			     TableModelEvent.DELETE));
    }
    */
  }

  public void voir()
  {
    String        s=contenuAlmaProjet();
    AlmaFilleSource f=new AlmaFilleSource(app_,"projet.acf",s);
    f.setFrameIcon(AlmaResource.ALMA.getIcon("alma"));
    app_.addInternalFrame(f);
  }

  public void setSelected(boolean _state) throws PropertyVetoException
  {
    super.setSelected(_state);
    if(isSelected()) tbProjet_.requestFocus();
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
         if(action.equals("AJOUTER")) ajouter();
    else if(action.equals("ENLEVER")) enlever();
    else if(action.equals("VOIR"   )) voir   ();
    else super.actionPerformed(_evt);
  }

  /*
  public void tableChanged(TableModelEvent _evt)
  {
    revalidate();
  }
  */

  // BuCutCopyPasteInterface

  // public String getFichier() { return fichier; }
  // public String getSource()  { return taSource_.getText(); }

  public void copy     () { tbProjet_.copy(); }
  public void cut      () { }
  public void duplicate() { }
  public void paste    () { }

  // BuSelectFindReplaceInterface
  public void find     () { }
  public void replace  () { }
  public void select   () { tbProjet_.select(); }

  // BuPrintable

  public void print(PrintJob _job, Graphics _g)
  {
    BuPrinter.INFO_DOC     =new BuInformationsDocument();
    BuPrinter.INFO_DOC.name=getTitle();
    BuPrinter.INFO_DOC.logo=BuResource.BU.getIcon("liste",24);

    BuPrinter.printTable(_job,_g,tbProjet_);
  }

  // BuInternalFrame

  public String[] getEnabledActions()
  {
    String[] r=new String[]
    {
      "IMPRIMER",
      "PREVISUALISER",
      "COPIER",
      "TOUTSELECTIONNER",
    };
    return r;
  }

  public String[] getDisabledActions()
  {
    String[] r=new String[] { "PROJET_TABLE" };
    return r;
  }

  public JMenu[] getSpecificMenus()
  {
    JMenu[] r=new JMenu[1];
    r[0]=menu_;
    return r;
  }

  public JComponent[] getSpecificTools()
  {
    JComponent[] r=new JComponent[3];
    r[0]=btAjouter_;
    r[1]=btEnlever_;
    r[2]=btVoir_;
    return r;
  }

  public void adjustActions()
  {
    boolean a=false;
    btAjouter_.setEnabled(a);
    BuActionEnabler.setEnabledForAction(menu_,"AJOUTER",a);
  }
}
