/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaFilleSource.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/04 18:29:49 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;
import com.memoire.editor.*;
import com.memoire.fu.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

public class AlmaFilleSource
       extends AlmaFilleDocument //BuInternalFrame
       implements ActionListener, BuCutCopyPasteInterface,
                  BuPrintable, BuSelectFindReplaceInterface,
                  BuUndoRedoInterface
{
  private AlmaImplementation app_;
  private String             fichier_;

  private JComponent      content;
  private BuBorderLayout  layout;

  private BuButton        btColorier_,btAller_;
  private BuMenu          menu_;
  private EditorInterface ei_;
  private Component       gui_;

  public AlmaFilleSource
    (AlmaImplementation _app, String _fichier, String _source)
  {
    //super("",true,true,true,true);

    app_    =_app;
    fichier_=_fichier;
    ei_     =null;

    setName("ifSOURCE");
    app_.installContextHelp(getRootPane(),"alma/p-source.html");

    content=(JComponent)getContentPane();
    layout=new BuBorderLayout();
    content.setLayout(layout);
    content.setBorder(new EmptyBorder(0,0,0,0));

    String editor=AlmaPreferences.ALMA.getStringProperty("editor.exec");
    int type=AlmaPreferences.ALMA.getIntegerProperty("editor.type");

    if(type==2)
      AglOptions.terminal.message("Starting "+editor+"...");

    //AglOptions.terminal.progres(10);

    ei_=Editor.editor(editor,type==3);
    ei_.start();
    gui_=(Component)ei_.getGUI();

    //AglOptions.terminal.progres(30);

    if(!(gui_ instanceof JComponent))
    {
      ei_=new EditorLinkExternal(ei_);
      ei_.start();
      gui_=(Component)ei_.getGUI();

      if(app_.getApp() instanceof Frame)
	((Frame)app_.getApp()).toFront();
    }

    //AglOptions.terminal.progres(60);

    if(ei_.needScrollBars())
      content.add(new JScrollPane(gui_),BuBorderLayout.CENTER);
    else
      content.add(gui_,BuBorderLayout.CENTER);

    if(gui_ instanceof JComponent)
    {
      ((JComponent)gui_).setBorder(new EmptyBorder(2,2,2,2));
      ((JComponent)gui_).putClientProperty("BU_IMPLEMENTATION",app_);
      AlmaDnd.TARGET.add((JComponent)gui_);
    }

    //AglOptions.terminal.progres(70);

    AglParse parse=AglOptions.parse();
    if(parse!=null) ei_.setSyntax(parse.syntaxe());
    ei_.setText(_source);
    ei_.run();

    //AglOptions.terminal.progres(90);

    btColorier_=new BuButton
      (AlmaResource.ALMA.getString("Colorier")); 
    btColorier_.setActionCommand("COLORIER");
    btColorier_.setToolTipText
      (AlmaResource.ALMA.getString("Colorier le texte")); 
    btColorier_.setIcon(BuResource.BU.loadToolCommandIcon("COLORIER"));
    btColorier_.setName("btCOLORIER");
    btColorier_.setRequestFocusEnabled(false);
    btColorier_.addActionListener(this);

    btAller_=new BuButton
      (AlmaResource.ALMA.getString("Aller")); 
    btAller_.setActionCommand("ALLER");
    btAller_.setToolTipText
      (AlmaResource.ALMA.getString("Aller  la ligne")); 
    btAller_.setIcon(BuResource.BU.loadToolCommandIcon("ALLER"));
    btAller_.setName("btALLER");
    btAller_.setRequestFocusEnabled(false);
    btAller_.addActionListener(this);

    menu_=new BuMenu (AlmaResource.ALMA.getString("Document"),"MENU_DOCUMENT"); 
    menu_.addMenuItem(AlmaResource.ALMA.getString("Colorier"),"COLORIER",this);
    menu_.addMenuItem(AlmaResource.ALMA.getString("Aller"   ),"ALLER"   ,this);
    menu_.computeMnemonics();
    menu_.setMnemonic('X'); //menu_.getText().charAt(0));

    int i=fichier_.lastIndexOf(System.getProperty("file.separator"));
    setTitle(fichier_.substring(i+1));
    setFrameIcon(BuResource.BU.getBarIcon("texte"));

    setSize(getPreferredSize());

    //AglOptions.terminal.progres(0);
  }

  public void setLangage(String _l)
  {
    String t=fichier_;
    int i=t.lastIndexOf(System.getProperty("file.separator"));
    t=t.substring(i+1);
    if(_l!=null) t+=" ["+
		   AlmaResource.ALMA.getString("Source")+
		   "/"+_l+"]";
    setTitle(t);
  }

  public void setSelected(boolean _state) throws PropertyVetoException
  {
    super.setSelected(_state);
    if(isSelected()) gui_.requestFocus();
   }

  public void setClosed(boolean _state) throws PropertyVetoException
  {
    super.setClosed(_state);
    if(isClosed()) ei_.exit();
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
         if(action.equals("COLORIER")) colorize();
    else if(action.equals("ALLER"   )) go();
    else super.actionPerformed(_evt);
  }

  public Component    getComponent() { return gui_; }
  public BuFileFilter getFiltre()    { return null; }

  public String getFichier() { return fichier_; }
  public Object getDonnee () { return getSource(); }
  public String getSource () { return ei_.getText(); }

  public void cut      () { ei_.act("cut"      ); }
  public void copy     () { ei_.act("copy"     ); }
  public void paste    () { ei_.act("paste"    ); }
  public void duplicate() { ei_.act("duplicate"); }
  public void redo     () { ei_.act("redo"     ); }
  public void undo     () { ei_.act("undo"     ); }

  private BuDialogFind    df=null;
  private BuDialogReplace dr=null;
  private BuDialogInput   di=null;

  public void colorize()
  {
    btColorier_.setEnabled(false);
    ei_.act("colorize");
    btColorier_.setEnabled(true);
  }

  public void find()
  {
    if(gui_ instanceof BuTextComponentInterface)
    {
      if(df==null)
	df=new BuDialogFind
	  (app_,app_.getInformationsSoftware(),(BuTextComponentInterface)gui_);
      df.activate();
    }
    else ei_.act("find");
  }

  public void go()
  {
    if(gui_ instanceof BuTextComponentInterface)
    {
      if(di==null)
	di=new BuDialogInput
	  (app_,app_.getInformationsSoftware(),
	   "Aller","Aller  la ligne:","");
      di.activate();

      String r=di.getValue();
      if(r!=null)
      {
	try
	{
	  int n=Integer.parseInt(r);
	  ((BuTextComponentInterface)gui_).go(n-1);
	}
	catch(Exception ex) { }
      }
    }
  }

  public void print(PrintJob _job,Graphics _g)
  {
    BuPrinter.INFO_DOC     =new BuInformationsDocument();
    BuPrinter.INFO_DOC.name=getTitle();
    BuPrinter.INFO_DOC.logo=BuResource.BU.getIcon("texte",24);

    if(gui_ instanceof BuTextPane)
    {
      BuPrinter.printStyledDocument
	(_job,_g,(StyledDocument)((BuTextPane)gui_).getDocument());
    }
    else
    {
      String s=getSource();
      s=FuLib.replace(s,"\t","        ");
      BuPrinter.printString(_job,_g,s);
    }
  }

  public void replace()
  {
    if(gui_ instanceof BuTextComponentInterface)
    {
      if(dr==null)
	dr=new BuDialogReplace
	  (app_,app_.getInformationsSoftware(),(BuTextComponentInterface)gui_);
      dr.activate();
    }
    else ei_.act("replace"); 
  }

  public void select()
  {
    if(gui_ instanceof BuTextComponentInterface)
      ((BuTextComponentInterface)gui_).select
	(0,((BuTextComponentInterface)gui_).getLength());
    else ei_.act("select");
  }

  // BuInternalFrame

  public String[] getEnabledActions()
  {
    String[] r=new String[]
      {
	"ENREGISTRER", "ENREGISTRERSOUS",
	"IMPRIMER", "PREVISUALISER",
	ei_.can("undo"     ) ? "DEFAIRE"          : null,
	ei_.can("redo"     ) ? "REFAIRE"          : null,
	ei_.can("cut"      ) ? "COUPER"           : null,
	ei_.can("copy"     ) ? "COPIER"           : null,
	ei_.can("paste"    ) ? "COLLER"           : null,
	ei_.can("duplicate") ? "DUPLIQUER"        : null,
	ei_.can("select"   ) ? "TOUTSELECTIONNER" : null,
	ei_.can("find"     ) ? "RECHERCHER"       : null,
	ei_.can("replace"  ) ? "REMPLACER"        : null,
	"ANALYSER"
      };
    return r;
  }

  public JMenu[] getSpecificMenus()
  {
    JMenu[] r=super.getSpecificMenus();

    int l=r.length;
    JMenu[] c=new JMenu[l+1];
    System.arraycopy(r,0,c,0,l);
    r=c;
    r[l]=menu_;

    return r;
  }

  public JComponent[] getSpecificTools()
  {
    JComponent[] r=super.getSpecificTools();

    int l=r.length;
    JComponent[] c=new JComponent[l+2];
    System.arraycopy(r,0,c,0,l);
    r=c;
    r[l  ]=btColorier_;
    r[l+1]=btAller_;

    return r;
  }

  public void adjustActions()
  {
    boolean c=ei_.can("colorize");
    BuActionEnabler.setEnabledForAction(menu_,"COLORIER",c);
    btColorier_.setEnabled(c);

    boolean a=(gui_ instanceof BuTextComponentInterface);
    BuActionEnabler.setEnabledForAction(menu_,"ALLER",a);
    btAller_.setEnabled(a);
  }
}

