/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaFilleTerminal.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/07 18:28:12 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;
import com.memoire.script.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.*;

public class AlmaFilleTerminal
       extends BuInternalFrame
       implements ActionListener, BuCutCopyPasteInterface
{
  private AlmaImplementation app_;
  private ScriptTerminal     term_;

  private JComponent     content;
  private BuBorderLayout layout;
  private BuButton       btEffacer_,btExecuter_;
  private BuMenu         menu_;

  public AlmaFilleTerminal(AlmaImplementation _app, String _script)
  {
    super("",false,true,false,true);

    app_   =_app;
    term_  =new ScriptTerminal();

    setName("ifTERMINAL");
    app_.installContextHelp(getRootPane(),"alma/p-terminal.html");

    content=(JComponent)getContentPane();
    layout=new BuBorderLayout();
    content.setLayout(layout);
    content.setBorder(new EmptyBorder(0,0,0,0));
    content.add(term_,BuBorderLayout.CENTER);

    btEffacer_=new BuButton(AlmaResource.ALMA.getString("Effacer"));
    btEffacer_.setActionCommand("EFFACER");
    btEffacer_.setToolTipText
      (AlmaResource.ALMA.getString("Effacer le terminal"));
    btEffacer_.setIcon(BuResource.BU.loadToolCommandIcon("EFFACER"));
    btEffacer_.setName("btEFFACER");
    btEffacer_.setRequestFocusEnabled(false);
    btEffacer_.addActionListener(this);

    btExecuter_=new BuButton(AlmaResource.ALMA.getString("Excuter"));
    btExecuter_.setActionCommand("EXECUTER");
    btExecuter_.setToolTipText
      (AlmaResource.ALMA.getString("Excuter un script"));
    btExecuter_.setIcon(BuResource.BU.loadToolCommandIcon("EXECUTER"));
    btExecuter_.setName("btEXECUTER");
    btExecuter_.setRequestFocusEnabled(false);
    btExecuter_.addActionListener(this);

    menu_=new BuMenu(AlmaResource.ALMA.getString("Terminal"),"MENU_TERMINAL"); 
    menu_.addMenuItem(AlmaResource.ALMA.getString("Effacer" ),"EFFACER" ,this);
    menu_.addMenuItem(AlmaResource.ALMA.getString("Excuter"),"EXECUTER",this);
    menu_.computeMnemonics();
    menu_.setMnemonic('X'); //menu_.getText().charAt(0));

    setTitle(_script);
    setFrameIcon(BuResource.BU.getBarIcon("terminal"));
    pack();
  }

  public ScriptTerminal getScriptTerminal()
    { return term_; }

  public void setClosed(boolean _state)
  {
    try
    {
      if(isClosed()!=_state) super.setClosed(_state);

      if(isClosed()) term_.stopThread();
      else           term_.startThread();
    }
    catch(PropertyVetoException ex) { ex.printStackTrace(); }
  }

  private static File openDirectory_=null;

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
         if(action.equals("EFFACER" )) effacer();
    else if(action.equals("EXECUTER")) executer();
    else super.actionPerformed(_evt);
  }

  public void effacer()
  {
    term_.getDisplay().clear_screen();
  }

  public void executer()
  {
    if(app_.getApp() instanceof BuApplet) return;

    ScriptInterpreterInterface si=term_.getScriptInterpreter();
    if(si!=null)
    {
      BuFileChooser chooser=new BuFileChooser();
      chooser.setDialogTitle(BuResource.BU.getString("Scripts"));
      chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
      chooser.setFileHidingEnabled(true);
      chooser.setMultiSelectionEnabled(true);
      chooser.resetChoosableFileFilters();

      if(openDirectory_==null)
      {
	String d=ScriptPreferences.SCRIPT.
	  getStringProperty("script.directory");
	if(!d.equals(""))
	{
	  File f=new File(BuLib.expandedPath(d));
	  if(f.canRead()) openDirectory_=f;
	}
      }

      if(openDirectory_!=null)
	chooser.setCurrentDirectory(openDirectory_);

      BuFileFilter filter=new BuFileFilter
	(si.getExtensions(),
	 si.getName()+" files");
      chooser.setFileFilter(filter);

      int returnVal=chooser.showDialog((BuApplication)app_.getApp(),
				       BuResource.BU.getString("Excuter"));

      if(returnVal == JFileChooser.APPROVE_OPTION)
      {
	File f=chooser.getSelectedFile();
	if(f!=null) openDirectory_=f;

	File[] files=chooser.getSelectedFiles();

	// bug swing. selectedFiles is always null!
	if((files.length==0)&&(f!=null))
	  files=new File[] { f };

	for(int i=0;i<files.length;i++)
	{
	  String r=files[i].getPath();
	  try
	  {
	    LineNumberReader lin=new LineNumberReader
		(new FileReader(r));
	    String s="";
	    while(true)
	    {
	      String l=lin.readLine();
	      if(l==null) break;
	      s+=l+'\n';
	    }
	    if(!"".equals(s)) si.eval(s);
	  }
	  catch(Exception ex)
	  {
	    AglOptions.terminal.erreur
	      ("Le script "+r+" n'a pu tre excut");
	  }
	}
      }
    }
  }

  // BuCutCopyPaste

  public void cut      () { }
  public void copy     () { }
  public void duplicate() { }

  public void paste()
  {
    Clipboard    clipboard=getToolkit().getSystemClipboard();
    Transferable content=clipboard.getContents(this/*null*/);

    if(content!=null)
    {
      try
      {
	String s=(String)content.getTransferData(DataFlavor.stringFlavor);
	if(s!=null) term_.paste(s);
      }
      catch(Exception ex1)
      {
	getToolkit().beep();
	//try { term_.error_.write(ex1.getMessage().getBytes()); }
	//catch(IOException ex2) { }
      }
    }
  }

  // BuInternalFrame

  public String[] getEnabledActions()
  {
    String[] r=new String[] { "COLLER" };
    return r;
  }

  public JMenu[] getSpecificMenus()
  {
    JMenu[] r=new JMenu[1];
    r[0]=menu_;
    return r;
  }

  public JComponent[] getSpecificTools()
  {
    JComponent[] r=new JComponent[2];
    r[0]=btEffacer_;
    r[1]=btExecuter_;
    return r;
  }
}
