/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaImplementation.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/04 18:29:49 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;
import com.memoire.dja.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.script.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class AlmaImplementation
       extends BuCommonImplementation
       implements AglTransfert //ListSelectionListener
{
  private AlmaPaquetsModel    lmPaquets;
  private AlmaTypesModel      lmTypes;
  private BuEmptyList         ltClasses;
  private BuScrollPane        spClasses;

  private BuAssistant         assistant_;
  private BuExplorer          explorer_;
  private BuTaskView          ltTaches;
  private BuScrollPane        spTaches;

  private AlmaFilleArbre      afArbre       =null; 
  private AlmaFilleMessages   afMessages    =null;
  private AlmaFilleOptions    afOptions     =null;
  private AlmaFilleProjet     afProjet      =null;
  private AlmaFillePatronType afPatronType  =null;

  private BuPreferencesFrame  ifPreferences =null;
  private BuHelpFrame         ifAide        =null;

  // InformationsSoftware

  private final static BuInformationsSoftware isAlma_
    =new BuInformationsSoftware();

  static
  {
    isAlma_.name     ="Alma";
    isAlma_.version  =AglOptions.VERSION;
    isAlma_.date     =AglOptions.DATE;
    isAlma_.rights   ="(c)1998-2001 Guillaume Desnoix";
    isAlma_.contact  ="guillaume@desnoix.com";
    isAlma_.license  ="GNU General Public License 2 (GPL)";
    isAlma_.languages="fr,de,en,eo,es,hu,it,pg,ru";
    isAlma_.logo     =AlmaResource.ALMA.getIcon("logo-alma");
    isAlma_.banner   =AlmaResource.ALMA.getIcon("banner-alma");
    // isAlma_.ftp=null;
    isAlma_.http     ="http://www.memoire.com/guillaume-desnoix/alma/";
    isAlma_.man      =AlmaPreferences.ALMA.getStringProperty
                      ("doc.mirror",
		       "http://www.memoire.com/guillaume-desnoix/alma/doc/");
    isAlma_.update   ="http://www.memoire.com/guillaume-desnoix/alma/update/";

    isAlma_.citation=
      "Axel von Arnim\n  pour son installeur";

    isAlma_.authors     =new String[]
      {
	"Guillaume Desnoix"
      };
    isAlma_.contributors=new String[]
      {
	"Arona Kane (delphi)",
	"Matthias Beil (ant)",
	"Axel von Arnim (dochtml)",
	"Romain Guy (diteur)",
        "Herv Agnoux (script)",
	"Jan Arne Petersen (ruby)"
      };
    isAlma_.documentors =new String[]
      {
	"Guillaume Desnoix"
      };
    isAlma_.translators =new String[]
      {
	"Zsuzsanna Geczki (hongrois)",
	"Norbert Marchl (allemand)",
	"Norbert Batfai (hongrois)",
	"Joeseph Zeldin (russe)",
	"Thomas Fehr (allemand)",
	"Guillaume Desnoix (anglais,esperanto)",
	"Systran/Altavista (espagnol,italien,portuguais)"
      };
    isAlma_.testers     =new String[]
      {
	"Herv Agnoux",
	"Gilles Gaillard",
	"Vincent Dupain",
	"Axel von Arnim",
	"Romain Guy"
      };
    isAlma_.libraries   =new String[]
      {
	"Encodeurs: Jef Poskanzer\n    (http://www.acme.com/)",
	"Expressions rgulires: Wes Biggs\n    (http://www.cacas.org/~wes/java/)",
	"Silk: Peter Norvig\n    (http://www.norvig.com/)",
	"Editeur: Slava Pestov\n    (http://jedit.sourceforge.net/)",
	"Aelfred: David Megginson\n    (http://www.ncf.carleton.ca/~ak117/)",
	"Icnes 20x20: Dean S. Jones\n    (http://www.javalobby.org/jfa/projects/icons/)",
	"Icnes 22x22: Rivyn\n    (http://rivyn.derkarl.org/kti/)",
	"Icnes 24x24: Tuomas Kuosmanen\n    (http://tigert.net/)",
      };
    isAlma_.thanks      =new String[]
      {
	"Programmez!\n    (magazine informatique, juin 1999)",
	"Exolab sessions #1\n    (http://www.exoffice.com/)",
	"Michel Casabianca\n    (http://www.sdv.fr/pages/casa)",
	"Linux Expo\n    (http://www.linux-expo.com/)",
	"Login:\n    (magazine informatique, avril 2000)", 
        "Brave GNU World\n    (http://www.brave-gnu-world.org/)",
      };
  }

  public BuInformationsSoftware getInformationsSoftware()
  { return isAlma_; }

  public static BuInformationsSoftware informationsSoftware()
  { return isAlma_; }

  // Constructeur

  public void init()
  {
    super.init();

    try
    {
      if(AglOptions.terminal==null)
	AglOptions.terminal=new AglTerminalSystem();
      if(AglOptions.parse==null)
	AglOptions.parse="Java";
      if(AglOptions.code==null)
	AglOptions.code="Java";
      AglProjet.reinit(true);

      BuSplashScreen ss=getSplashScreen();

      if(ss!=null)
      {
	ss.setText("Ajustements");
	ss.setProgression(50);
      }

      BuInformationsSoftware il=getInformationsSoftware();
      BuInformationsDocument id=new BuInformationsDocument();

      setTitle(il.name+" "+il.version);
      BuPrinter.INFO_LOG=il;
      BuPrinter.INFO_DOC=id;
      
      BuMenuBar mb=getMainMenuBar();
      mb.addMenu(buildProjetMenu());
      mb.addMenu(buildModeleMenu());
      if(AlmaPreferences.ALMA.getBooleanProperty("module.script",true))
	mb.addMenu(buildScriptMenu());

      BuMenu mt=new BuMenuTools(AlmaPreferences.ALMA,AlmaResource.ALMA);
      mb.addMenu(mt);

      BuMenu       ma=(BuMenu)mb.getMenu("WWW_INTERNET");
      if(ma==null) ma=(BuMenu)mb.getMenu("MENU_AIDE");
      boolean rg=!"".equals(AlmaPreferences.ALMA.getStringProperty("user.id"));
      ma.addMenuItem(BuResource.BU.getString("Support..."),"WWW_SUPPORT",rg);
      ma.addMenuItem(BuResource.BU.getString("Bogue..."  ),"WWW_BOGUE",true);
      if(!rg) ma.addMenuItem(BuResource.BU.getString("Achat..."),"WWW_ACHAT",true);

      BuToolBar tb=getMainToolBar();
      tb.setTitle(il.name+" "+il.version+" [TB]");
      tb.setIconImage(il.logo.getImage());

      /*
      tb.putClientProperty("BACKGROUND_PAINTER",
			   new BuBackgroundPainter
			   (BuResource.BU.getIcon
			    ("background_toolbar.gif")));
      */

      tb.addSeparator();
      tb.addToolButton(AlmaResource.ALMA.getString("Analyser"       ),
		       AlmaResource.ALMA.getString("Analyser"       ),"ANALYSER"    ,true );
      tb.addToolButton(AlmaResource.ALMA.getString("Gnrer"        ),
		       AlmaResource.ALMA.getString("Gnrer"        ),"GENERER"     ,true );
      tb.addSeparator();
      tb.addToolButton(AlmaResource.ALMA.getString("Configurer"     ),
		       AlmaResource.ALMA.getString("Configurer"     ),"CONFIGURER"  ,true );
      tb.addToolButton(AlmaResource.ALMA.getString("Messages"       ),
		       AlmaResource.ALMA.getString("Messages"       ),"MESSAGE"     ,true );
      tb.addToolButton(AlmaResource.ALMA.getString("Arbre"),
		       AlmaResource.ALMA.getString("Arbre du modle"),"ARBRE_MODELE",true );

      boolean b=(getApp() instanceof BuApplication);
      getApp().setEnabledForAction("CREER"              ,true);
      getApp().setEnabledForAction("OUVRIR"             ,b);
      getApp().setEnabledForAction("QUITTER"            ,b);
      getApp().setEnabledForAction("PREFERENCE"         ,true);
      getApp().setEnabledForAction("OUVRIRPROJET"       ,b);
      getApp().setEnabledForAction("ENREGISTRERPROJET"  ,b);
      getApp().setEnabledForAction("EXECUTERPROJET"     ,b);
      getApp().setEnabledForAction("ANALYSER"           ,false);
      getApp().setEnabledForAction("GENERER"            ,false);
      getApp().setEnabledForAction("CONFIGURER"         ,false);
      getApp().setEnabledForAction("MESSAGE"            ,false);
      getApp().setEnabledForAction("ARBRE_MODELE"       ,false);
      getApp().setEnabledForAction("AIDE_THEMA"         ,false);
      getApp().setEnabledForAction("ASTUCE"             ,false);

      getApp().removeAction("IMPORTER");
      getApp().removeAction("EXPORTER");
      getApp().removeAction("PROPRIETE");
      getApp().removeAction("MISEENPAGE");
      getApp().removeAction("MAJ");

      mb.computeMnemonics();

      if(ss!=null)
      {
	ss.setText("Bureau et assistant");
	ss.setProgression(60);
      }

      BuMainPanel mp=getMainPanel();
      BuColumn    lc=mp.getLeftColumn();
      BuColumn    rc=mp.getRightColumn();

      if(  AlmaPreferences.ALMA.getBooleanProperty("module.explorer",true)
         &&!(getApp() instanceof BuApplet))
      {
	explorer_=new BuExplorer(this);
	explorer_.addExtraFileCommand
	  (AlmaResource.ALMA.getString("Analyser"),"ANALYSER",
	   AlmaResource.ALMA.loadMenuCommandIcon("ANALYSER"));
	explorer_.addActionListener(this);
	lc.addToggledComponent
	  (BuResource.BU.getString("Explorateur"),
	   "EXPLORATEUR",explorer_,this);
      }
      /*
      else
      {
	lc.setVisible(false);
	getApp().removeAction("VISIBLE_LEFTCOLUMN");
      }
      */

      if(AlmaPreferences.ALMA.getBooleanProperty("module.assistant",true))
      {
	assistant_=new AlmaAssistant();
	assistant_.setBorder(null);
	rc.addToggledComponent(AlmaResource.ALMA.getString("Assistant"),
			       "ASSISTANT",assistant_,this);

	if(getApp() instanceof BuApplet)
	{
	  rc.toggleComponent("ASSISTANT");
	  setCheckedForAction("ASSISTANT",
			      rc.isToggleComponentVisible("ASSISTANT"));
	}
      }

      ltTaches=new BuTaskView();
      spTaches=new BuScrollPane(ltTaches);
      spTaches.setPreferredSize(new Dimension(120,70));
      rc.addToggledComponent(AlmaResource.ALMA.getString("Tches"),
			     "TACHE",spTaches,this);

      if(ss!=null)
      {
	ss.setText("Navigation");
	ss.setProgression(70);
      }

      lmPaquets=new AlmaPaquetsModel();
      lmTypes  =new AlmaTypesModel();

      ltClasses=new BuEmptyList();
      ltClasses.setName("ltCLASSES");
      ltClasses.setModel(lmTypes);
      ltClasses.setEmptyText(AlmaResource.ALMA.getString
			     ("Aucune classe en mmoire"));
      // ltClasses.addListSelectionListener(this);
      spClasses=new BuScrollPane(ltClasses);
      AlmaDnd.SOURCE.add(ltClasses);

      BuPanel pnClasses=new BuPanel();
      pnClasses.setLayout(new BuVerticalLayout(2,true,true));
      pnClasses.setPreferredSize(new Dimension(120,160));
      BuPanel pnButtons=new BuPanel();
      pnButtons.setLayout(new BuHorizontalLayout(2,false,true));

      Border border=new LineBorder(UIManager.getColor("Label.foreground"),1);

      final BuToggleButton btElementaires=new BuToggleButton();
      btElementaires.setIcon(AlmaResource.ALMA.getIcon("type-primitif",16));
      btElementaires.setRequestFocusEnabled(false);
      btElementaires.setMargin(new Insets(0,0,0,0));
      btElementaires.setBorder(border);
      btElementaires.setToolTipText
	(AlmaResource.ALMA.getString("Elmentaires"));
      btElementaires.setSelected(lmTypes.isElementaires());

      btElementaires.addActionListener
	(new ActionListener()
	 {
	   public void actionPerformed(ActionEvent _evt)
	   {
	     lmTypes.setElementaires(btElementaires.isSelected());
	   }
	 }
	);

      final BuToggleButton btClasses=new BuToggleButton();
      btClasses.setIcon(AlmaResource.ALMA.getIcon("classe",16));
      btClasses.setRequestFocusEnabled(false);
      btClasses.setMargin(new Insets(0,0,0,0));
      btClasses.setBorder(border);
      btClasses.setToolTipText(AlmaResource.ALMA.getString("Classes"));
      btClasses.setSelected(lmTypes.isClasses());

      btClasses.addActionListener
	(new ActionListener()
	 {
	   public void actionPerformed(ActionEvent _evt)
	   {
	     lmTypes.setClasses(btClasses.isSelected());
	   }
	 }
	);

      final BuToggleButton btInterfaces=new BuToggleButton();
      btInterfaces.setIcon(AlmaResource.ALMA.getIcon("interface",16));
      btInterfaces.setRequestFocusEnabled(false);
      btInterfaces.setMargin(new Insets(0,0,0,0));
      btInterfaces.setBorder(border);
      btInterfaces.setToolTipText(AlmaResource.ALMA.getString("Interfaces"));
      btInterfaces.setSelected(lmTypes.isInterfaces());

      btInterfaces.addActionListener
	(new ActionListener()
	 {
	   public void actionPerformed(ActionEvent _evt)
	   {
	     lmTypes.setInterfaces(btInterfaces.isSelected());
	   }
	 }
	);

      final BuToggleButton btStructures=new BuToggleButton();
      btStructures.setIcon(AlmaResource.ALMA.getIcon("structure",16));
      btStructures.setRequestFocusEnabled(false);
      btStructures.setMargin(new Insets(0,0,0,0));
      btStructures.setBorder(border);
      btStructures.setToolTipText(AlmaResource.ALMA.getString("Structures"));
      btStructures.setSelected(lmTypes.isStructures());

      btStructures.addActionListener
	(new ActionListener()
	 {
	   public void actionPerformed(ActionEvent _evt)
	   {
	     lmTypes.setStructures(btStructures.isSelected());
	   }
	 }
	);

      pnButtons.add(btElementaires);
      pnButtons.add(btClasses);
      pnButtons.add(btInterfaces);
      pnButtons.add(btStructures);
      pnClasses.add(pnButtons);
      pnClasses.add(spClasses);
      rc.addToggledComponent(AlmaResource.ALMA.getString("Classes"),
			     "CLASSE",pnClasses,this);

      /*
      JPanel px=new JPanel();
      px.setOpaque(false);
      rc.add(px);
      */

      if(ss!=null)
      {
	ss.setText("Fichiers rcents");
	ss.setProgression(80);
      }

      BuMenuRecentFiles mr=(BuMenuRecentFiles)mb.getMenu("REOUVRIR");
      if(mr!=null)
      {
	mr.setPreferences(AlmaPreferences.ALMA);
	mr.setResource(AlmaResource.ALMA);
      }

      mp.getStatusBar().setApp(this);
      mp.setLogo(il.logo);
      mp.setTaskView(ltTaches);
      mp.setAssistant(assistant_);
    }
    catch(Throwable t)
      { t.printStackTrace(); }
  }

  public void start()
  {
    super.start();

    BuInformationsSoftware il=getInformationsSoftware();

    //@GDX TMP?
    //if(AlmaPreferences.ALMA.getBooleanProperty("module.dja",true))
    //AlmaDjaRegistre.init(il);
    DjaLib.SOFTWARE=il;

    if(assistant_!=null)
      assistant_.actionPerformed
	(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,
			 "DEBUTER("+il.name.toUpperCase()+")"));

    //BuPreferences.BU.applyOn(getApp());
    if(!AlmaPreferences.ALMA.getBooleanProperty("module.explorer",true))
    {
      getMainPanel().getLeftColumn().setVisible(false);
      //lc.setVisible(false);
      getApp().removeAction("VISIBLE_LEFTCOLUMN");
    }

    // about();
    if(assistant_!=null)
      assistant_.changeAttitude(BuAssistant.PAROLE,
				BuResource.BU.getString("Bienvenue")+
				"!\n"+il.name+" "+il.version);

    /*
    BuMenuBar mb=getMainMenuBar();
    for(int i=1;i<=4;i++)
    {
      String fp=AlmaPreferences.ALMA.getStringProperty
        ("file.recent."+i+".path");
      String fi=AlmaPreferences.ALMA.getStringProperty
        ("file.recent."+i+".icon");
      if(  !"".equals(fp)
         &&(new File(BuLib.expandedPath(fp)).canRead()))
	{
	  BuIcon icon=BuResource.BU.getIcon("texte");
	  if(!"".equals(icon)) icon=BuResource.BU.getMenuIcon(fi);
	  mb.addReopenItem(fp,icon);
	  getApp().setEnabledForAction("REOUVRIR",true);
	}
    }
    */

    new BuTaskOperation(this,AlmaResource.ALMA.getString("Mise en place"))
      { public void act()
	{ setPriority(Thread.MIN_PRIORITY+1); miseEnPlace(); } }
      .start();
  }

  public void miseEnPlace()
  {
    try
    {
      BuMainPanel mp=getMainPanel();
      mp.setProgression(5);

      if(afMessages==null)
      {
	afMessages=new AlmaFilleMessages(this);
	afMessages.setShortcut(KeyEvent.VK_F6);
	AglOptions.terminal=afMessages;

	Dimension dd=mp.getDesktop().getSize();
	Dimension df=afMessages.getSize();
	afMessages.setLocation(444,370);
	//dd.width-df.width,dd.height-df.height-37);
      }

      if(AlmaPreferences.ALMA.getBooleanProperty
	 ("window.messages.visible",true))
	cmdMessages();
      else getApp().setEnabledForAction("MESSAGE",true);
      mp.setProgression(20);

      if(AlmaPreferences.ALMA.getBooleanProperty
	 ("window.configuration.visible",true))
	cmdConfigurer();
      else getApp().setEnabledForAction("CONFIGURER",true);
      mp.setProgression(40);

      if(AlmaPreferences.ALMA.getBooleanProperty
	 ("window.modeltree.visible",true))
	cmdArbreModele();
      else getApp().setEnabledForAction("ARBRE_MODELE",true);
      mp.setProgression(60);

      if(AlmaPreferences.ALMA.getBooleanProperty
	 ("window.project.visible",true))
	ajouteFenetreProjet();
      else getApp().setEnabledForAction("PROJET_TABLE",true);
      mp.setProgression(80);

      if(assistant_!=null)
      {
	assistant_.addEmitters((Container)getApp());
	assistant_.changeAttitude(BuAssistant.ATTENTE,"");
	mp.setProgression(90);
      }

      BuMenuBar mb=getMainMenuBar();
      mb.removeActionListener(this);
      mb.addActionListener(this);
      mp.setProgression(95);

      menusOrganisationFenetres();
      mp.setProgression(100);

      new DndTarget()
	.accept(File.class,
		DndTransferObject.SINGLETON,
		new DndManageData()
		{
		  public boolean manage
		    (Object _data,JComponent _component,Point _location)
		    {
		      Object[] o;

		      if(_data instanceof Object[])
			o=(Object[])_data;
		      else
			o=new Object[] { _data };

		      for(int i=0;i<o.length;i++)
			action("OUVRIR("+o[i]+")");

		      return true;
		    }
		})
	.add(mp.getDesktop());

      mp.setProgression(0);
    }
    catch(Exception ex)
      { ex.printStackTrace(); }
  }

  public void cmdArguments(final String[] _args)
  {
    //System.err.println("PARSING ARGS...");
    new BuTaskOperation(this,AlmaResource.ALMA.getString
			("Traitement des arguments"))
      { public void act() { opArguments(_args); } }
      .start();
  }

  public void opArguments(String[] _args)
  {
    if(_args.length>0)
    {
      while(afMessages==null)
      {
	try { Thread.sleep(100); }
	catch(Exception ex) { }
      }

      AglOptions.terminal.message
	(AlmaResource.ALMA.getString("Traitement des arguments"));

      String[] na=new String[_args.length+2];
      String   l =AglOptions.code;//.langage();
      for(int i=0;i<_args.length;i++)
      {
	if("-g".equals(_args[i])||"--generator".equals(_args[i]))
	{
	  na[i  ]=_args[i  ];
	  na[i+1]=_args[i+1];
	  l=na[i+1];
	  i++;
	}
	else
	if("-o".equals(_args[i])||"--output".equals(_args[i]))
	{
	  na[i  ]=_args[i  ];
	  na[i+1]=_args[i+1];
	  AglUnite u=new AglUnite(na[i+1]);
	  u.langage=l;
	  u.cible=true;
	  i++;
	}
	else na[i]=_args[i];
      }

      na[_args.length  ]="-o";
      na[_args.length+1]="null";

      try
      {
	new Agl(na).run();
      }
      catch(Throwable th)
      {
	th.printStackTrace();
	new BuDialogError
	  (this,null,""+th.getClass().getName()+": "
	   +th.getMessage()).activate();
      }

      for(int i=0;i<_args.length;i++)
        if(  (!_args[i].startsWith("-"))
           &&(new File(_args[i]).canRead()))
	  getMainMenuBar().addRecentFile(_args[i],"texte");

      rafraichir();

      getApp().setEnabledForAction("GENERER",AglUnite.table.size()>0);
    }
  }

// Evenements

  public synchronized void actionPerformed(ActionEvent _evt)
  {
    // JComponent source=(JComponent)_evt.getSource();
    // System.err.println("AlmaApplication : "+source);

    String action=_evt.getActionCommand();
    String arg   ="";

    int i=action.indexOf('(');
    if(i>=0)
    {
      arg   =action.substring(i+1,action.length()-1);
      action=action.substring(0,i);
    }

    // System.err.println("AlmaImplementation: ACTION="+action); 
    // System.err.println("AlmaImplementation: ARG   ="+arg); 

         if(action.equals("CREER"                )) cmdCreer();
    else if(action.equals("OUVRIR"               )) cmdOuvrir(arg);
    else if(action.equals("REOUVRIR"             )) cmdReouvrir(arg);
    else if(action.equals("ENREGISTRER"          )) cmdEnregistrer();
    else if(action.equals("ENREGISTRERSOUS"      )) cmdEnregistrerSous();

    else if(action.equals("PREFERENCE"           )) cmdPreference();

    else if(action.equals("OUVRIRPROJET"         )) cmdOuvrirProjet();
    else if(action.equals("ENREGISTRERPROJET"    )) cmdEnregistrerProjet();
    else if(action.equals("EXECUTERPROJET"       )) cmdExecuterProjet();
    else if(action.equals("REINITIALISERPROJET"  )) cmdReinitialiserProjet();
    else if(action.equals("ANALYSER"             )) cmdAnalyser(arg);
    else if(action.equals("GENERER"              )) cmdGenerer();
    else if(action.equals("CONFIGURER"           )) cmdConfigurer();
    else if(action.equals("MESSAGE"              )) cmdMessages();
    else if(action.equals("ARBRE_MODELE"         )) cmdArbreModele();
    else if(action.equals("PROJET_TABLE"         )) cmdProjetTable();

    else if(action.equals("PATRON_CREER"         )) cmdCreerPatron();
    else if(action.equals("RELATION_CREER"       )) cmdCreerRelation();
    else if(action.equals("CONDITION_CREER"      )) cmdCreerCondition();
    else if(action.equals("PATRON_CONSTRUIRE"    )) cmdConstruirePatrons();
    else if(action.equals("RELATION_CONSTRUIRE"  )) cmdConstruireRelations();
    else if(action.equals("CONDITION_CONSTRUIRE" )) cmdConstruireConditions();

    else if(action.equals("GRAPHE_CREER"         )) cmdCreerGraphe();

    else if(action.equals("TERMINAL"             )) cmdTerminal(arg);
    else if(action.equals("SCRIPT"               )) cmdScript(arg);
    else if(action.equals("OUTIL"                )) cmdOutil(arg);

    else if(action.equals("WWW_SUPPORT"          ))
           displayURL("mailto:alma-support@memoire.com");
    else if(action.equals("WWW_BOGUE"            ))
           displayURL("mailto:alma-bug@memoire.com");
    else if(action.equals("WWW_ACHAT"            ))
           displayURL("https://www.yaskifo.com/order.asp?ID=101697");

    else if(action.equals("EXPLORATEUR"))
    {
      BuColumn lc=getMainPanel().getLeftColumn();
      
      lc.toggleComponent(action);
      setCheckedForAction(action,lc.isToggleComponentVisible(action));
    }
    else if(  action.equals("ASSISTANT")
	    ||action.equals("TACHE"    )
	    ||action.equals("CLASSE"   ))
    {
      BuColumn rc=getMainPanel().getRightColumn();
      
      rc.toggleComponent(action);
      setCheckedForAction(action,rc.isToggleComponentVisible(action));

      // if(action.equals("TACHE")) Thread.currentThread().getThreadGroup().list();
    }

    else super.actionPerformed(_evt);
  }

  // Rafraichir

  public void rafraichir()
  {
    lmPaquets.update();
    lmTypes.update();

    if(afArbre!=null)
      afArbre.rafraichir();

    if(afProjet!=null)
      afProjet.rafraichir();
  }

  // Acces

  public BuExplorer getExplorer()
  { return explorer_; }

  // Menu Projet

  public static BuMenu buildProjetMenu()
  {
    BuMenu r=new BuMenu(AlmaResource.ALMA.getString("Projet"),"MENU_PROJET");
    r.addMenuItem(BuResource.BU.getString    ("Ouvrir..."      ),"OUVRIRPROJET"       ,true);
    r.addMenuItem(BuResource.BU.getString    ("Enregistrer..." ),"ENREGISTRERPROJET"  ,true);
    r.addMenuItem(AlmaResource.ALMA.getString("Excuter..."    ),"EXECUTERPROJET"     ,true);
    r.addMenuItem(AlmaResource.ALMA.getString("Rinitialiser"  ),"REINITIALISERPROJET",true);
    r.addSeparator();
    r.addMenuItem(AlmaResource.ALMA.getString("Analyser"       ),"ANALYSER"    ,true )
    .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F8,0));
    r.addMenuItem(AlmaResource.ALMA.getString("Gnrer"        ),"GENERER"     ,true )
    .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F9,0));
    r.addSeparator();
    r.addMenuItem(AlmaResource.ALMA.getString("Configurer"     ),"CONFIGURER"  ,true );
    r.addMenuItem(AlmaResource.ALMA.getString("Messages"       ),"MESSAGE"     ,true );
    r.addMenuItem(AlmaResource.ALMA.getString("Arbre du modle"),"ARBRE_MODELE",true );
    r.addMenuItem(AlmaResource.ALMA.getString("Projet"         ),"PROJET_TABLE",true );
    r.addSeparator();

    if(AlmaPreferences.ALMA.getBooleanProperty("module.explorer",true))
      r.addCheckBox(BuResource.BU.getString("Explorateur"),"EXPLORATEUR" ,true ,true );

    r.addCheckBox(AlmaResource.ALMA.getString("Tches"         ),"TACHE"       ,true ,true );
    r.addCheckBox(AlmaResource.ALMA.getString("Classes"        ),"CLASSE"      ,true ,true );
    r.computeMnemonics();
    return r;
  }

  // Menu Modele

  public static BuMenu buildModeleMenu()
  {
    BuMenu r=new BuMenu(AlmaResource.ALMA.getString("Modle"),"MENU_MODELE");

    if(AlmaPreferences.ALMA.getBooleanProperty("module.dja",true))
    {
      r.addMenuItem(AlmaResource.ALMA.getString("Crer un diagramme" ),"GRAPHE_CREER"   ,true );
      r.addSeparator();
    }

    r.addMenuItem(AlmaResource.ALMA.getString("Construire les patrons"    ),"PATRON_CONSTRUIRE"  ,true);
    r.addMenuItem(AlmaResource.ALMA.getString("Construire les relations" ),"RELATION_CONSTRUIRE" ,true );
    r.addMenuItem(AlmaResource.ALMA.getString("Construire les conditions"),"CONDITION_CONSTRUIRE",true );
    r.addSeparator();
    r.addMenuItem(AlmaResource.ALMA.getString("Crer un patron"     ),"PATRON_CREER"  ,true );
    r.addMenuItem(AlmaResource.ALMA.getString("Crer une relation" ),"RELATION_CREER" ,false);
    r.addMenuItem(AlmaResource.ALMA.getString("Crer une condition"),"CONDITION_CREER",false);
    r.computeMnemonics();
    return r;
  }

  // Menu Script

  public static BuMenu buildScriptMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Scripts"),"MENU_SCRIPTS");

    r.addSubMenu(new ScriptMenuInterpreters(),true);

    String[] scripts=null;
    File     base   =null;

    String d=ScriptPreferences.SCRIPT.getStringProperty("script.directory");
    if(!"".equals(d))
    {
      base=new File(BuLib.expandedPath(d));
      if(base.isDirectory()&&base.canRead())
      {
	scripts=base.list();
	if((scripts!=null)&&(scripts.length==0))
	  scripts=null;
      }
    }

    Vector v=new Vector();

    if(scripts!=null)
    {
      r.addSeparator();
      FuSort.sort(scripts);
      int n=0;

      for(int i=0;i<scripts.length;i++)
      {
	File f=new File(base,scripts[i]);
	if(f.isFile()&&!scripts[i].endsWith("~"))
	{
	  String p=f.getAbsolutePath();
	  String t=f.getName();
	  //int    j=t.lastIndexOf('.');
	  //if(j>0) t=t.substring(0,j);
	       if(t.startsWith("agl-")) t=t.substring(4);
	  else if(t.startsWith("gui-")) t=t.substring(4);

	  v.addElement
	    (r.addMenuItem
	      (/*""+n+" "+*/ t,"SCRIPT("+p+")",
	       BuResource.BU.getMenuIcon("texte"),
	       (ScriptInterpreter.guess(p)!=null)));
	  n++;
	}
      }
    }

    r.computeMnemonics();

    Enumeration e=v.elements();
    while(e.hasMoreElements())
    {
      BuMenuItem mi=(BuMenuItem)e.nextElement();
      int k=mi.getMnemonic();
      mi.setAccelerator
	  (KeyStroke.getKeyStroke
	   (k,KeyEvent.SHIFT_MASK|KeyEvent.ALT_MASK));
    }

    return r;
  }

// FILTRES

  private BuFileFilter filterDja=new BuFileFilter
      (new String[] { "dja","DJA" },
       AlmaResource.ALMA.getString("Diagramme"));
  private final static BuFileFilter filterSer=new BuFileFilter
      (new String[] { "ser","SER" },
       AlmaResource.ALMA.getString("Projet"));
  private final static BuFileFilter filterAlm=new BuFileFilter
      (new String[] { "alma", "ALMA", "alm", "ALM" },
       AlmaResource.ALMA.getString("Projet"));
  private final static BuFileFilter filterXml=new BuFileFilter
      (new String[] { "xml","XML" },
       AlmaResource.ALMA.getString("Projet"));
  private final static BuFileFilter filterXgz=new BuFileFilter
      (new String[] { "xgz","XGZ" },
       AlmaResource.ALMA.getString("Projet"));

// COMMANDES

  public void cmdCreer()
  {
    addInternalFrame(new AlmaFilleSource
		  (this,AlmaResource.ALMA.getString("SansTitre"),""));
    // setEnabledForAction("GENERER",false);
  }

  private static File openDirectory_=null;
  private static File saveDirectory_=null;

  public void cmdOuvrir(String _arg)
  {
    if(getApp() instanceof BuApplet) return;

    if(!"".equals(_arg))
    {
      File f=new File(_arg);
      //System.err.println("### f="+f);
      AlmaTacheOuverture tache=new AlmaTacheOuverture(this,f.getPath());
      tache.setPriority(Thread.MIN_PRIORITY);
      tache.start();
      return;
    }

    BuFileChooser chooser=new BuFileChooser();
    chooser.setFileSelectionMode(BuFileChooser.FILES_ONLY);
    chooser.setFileHidingEnabled(true);
    chooser.setMultiSelectionEnabled(true);
    chooser.resetChoosableFileFilters();

    chooser.setDialogTitle(getInformationsSoftware().name+": "+
			   BuResource.BU.getString("Ouvrir..."));

    if(openDirectory_==null)
    {
      String d=BuPreferences.BU.getStringProperty("directory.open");
      if(!d.equals(""))
      {
	File f=new File(BuLib.expandedPath(d));
	if(f.canRead()) openDirectory_=f;
      }
    }

    if(openDirectory_!=null)
      chooser.setCurrentDirectory(openDirectory_);

    if(AlmaPreferences.ALMA.getBooleanProperty("module.dja",true))
      chooser.addChoosableFileFilter(filterDja);

    if(!AglOptions.code.equals(AglOptions.parse))
    {
      BuFileFilter filterCode=new BuFileFilter
	(AglOptions.code().extensions(),
	 AglOptions.code+" files");
      chooser.addChoosableFileFilter(filterCode);
    }

    BuFileFilter filterParse=new BuFileFilter
      (AglOptions.parse().extensions(),
       AglOptions.parse+" files");
    chooser.setFileFilter(filterParse);

    int returnVal=chooser.showOpenDialog((BuApplication)getApp());

    if(returnVal == BuFileChooser.APPROVE_OPTION)
    {
      File f=chooser.getSelectedFile();
      //System.err.println("### f="+f);
      if(f!=null) openDirectory_=f;

      File[] files=chooser.getSelectedFiles();

      // bug swing. selectedFiles is always null!
      if((files.length==0)&&(f!=null))
	files=new File[] { f };

      BuMenuBar mb=getMainMenuBar();
      for(int i=0;i<files.length;i++)
      {
	String r=files[i].getPath();

	if(filterDja.accept(r))
	{
	  if(AlmaPreferences.ALMA.getBooleanProperty("module.dja",true))
	  {
	    try
	    {
	      DjaGridInteractive grille=
		new DjaGridInteractive
		(true,DjaLoader.loadAsText(new FileInputStream(r)));
	      addInternalFrame(new AlmaFilleDiagramme(this,r,grille));
	      mb.addRecentFile(r,"graphe");
	    }
	    catch(FileNotFoundException ex)
	      { AglOptions.terminal.erreur(r+" introuvable"); }
	  }
	}
	else
	{
	  AlmaTacheOuverture tache=new AlmaTacheOuverture(this,r);
	  tache.setPriority(Thread.MIN_PRIORITY);
	  tache.start();
	  mb.addRecentFile(r,"texte");
	}
      }
    }
  }

  public void cmdReouvrir(String _arg)
  {
    AlmaTacheOuverture tache=new AlmaTacheOuverture(this,_arg);
    tache.setPriority(Thread.MIN_PRIORITY);
    tache.start();
  }

  public void cmdEnregistrer()
  {
    cmdEnregistrerSous();
  }

  public void cmdEnregistrerSous()
  {
    if(getApp() instanceof BuApplet) return;

    JInternalFrame frame=getCurrentInternalFrame();
    if(  (frame!=null)
       &&(  (frame instanceof AlmaFilleSource)
	  ||(frame instanceof AlmaFilleCible)
	  ||(frame instanceof AlmaCodeUmlHierarchieFrame)
	  ||(frame instanceof AlmaCodeUmlRelationsFrame)
	  ||(frame instanceof AlmaFilleDiagramme)))
    {
      String r="";

      if(frame instanceof AlmaFilleSource)
	r=((AlmaFilleSource)frame).getFichier();
      if(frame instanceof AlmaFilleCible)
	r=((AlmaFilleCible)frame).getFichier();
      if(frame instanceof AlmaCodeUmlHierarchieFrame)
	r=((AlmaCodeUmlHierarchieFrame)frame).getFichier();
      if(frame instanceof AlmaCodeUmlRelationsFrame)
	r=((AlmaCodeUmlRelationsFrame)frame).getFichier();
      if(frame instanceof AlmaFilleDiagramme)
	r=((AlmaFilleDiagramme)frame).getFile();

      // System.err.println("### FD begin : "+r);

      BuFileChooser chooser=new BuFileChooser();
      chooser.setFileSelectionMode(BuFileChooser.FILES_ONLY);
      chooser.setFileHidingEnabled(true);
      chooser.setMultiSelectionEnabled(false);
      chooser.resetChoosableFileFilters();

      chooser.setDialogTitle(getInformationsSoftware().name+": "+
			     BuResource.BU.getString("Enregistrer..."));

      if(saveDirectory_==null)
      {
	String d=BuPreferences.BU.getStringProperty("directory.save");
	if(!d.equals(""))
	{
	  File f=new File(BuLib.expandedPath(d));
	  if(f.canRead()) saveDirectory_=f;
	}
      }

      if(saveDirectory_==null)
	saveDirectory_=openDirectory_;

      if(saveDirectory_!=null)
	chooser.setCurrentDirectory(saveDirectory_);

      BuFileFilter filterCode=null;
      BuFileFilter filterImage=null;

      if(frame instanceof AlmaFilleDiagramme)
      {
	filterCode=filterDja;
	chooser.setFileFilter(filterCode);
      }
      else
      if(frame instanceof AlmaFilleDocument)
      {
	filterCode=((AlmaFilleDocument)frame).getFiltre();
	if(filterCode!=null)
	  chooser.setFileFilter(filterCode);
      }

      if(  (frame instanceof AlmaFilleDiagramme)
	 ||(frame instanceof AlmaCodeUmlHierarchieFrame)
	 ||(frame instanceof AlmaCodeUmlRelationsFrame))
      {
	filterImage=new BuFileFilter
	  (new String[]
	      { AglOptions.formatImage.toLowerCase(),
		AglOptions.formatImage.toUpperCase()  },
	   "Images");
	chooser.addChoosableFileFilter(filterImage);
      }

      chooser.setSelectedFile(new File(r));

      int returnVal=chooser.showSaveDialog((BuApplication)getApp());
      if(returnVal==BuFileChooser.APPROVE_OPTION)
      {
	saveDirectory_=chooser.getSelectedFile();
	r=saveDirectory_.getPath(); // .getName();
	// System.out.println("### FD chosen "+r);
      }
      else r=null;

      // System.err.println("### FD end");

      // FDSave.setFilenameFilter(new AlmaCodeFilenameFilter());

      if(r!=null)
      {
	String t=null;
	Image  i=null;

	if(frame instanceof AlmaFilleSource)
	  t=((AlmaFilleSource)frame).getSource();
	if(frame instanceof AlmaFilleCible)
	  t=((AlmaFilleCible)frame).getSource();
	if(frame instanceof AlmaCodeUmlHierarchieFrame)
	  i=((AlmaCodeUmlHierarchieFrame)frame).getImage();
	if(frame instanceof AlmaCodeUmlRelationsFrame)
	  i=((AlmaCodeUmlRelationsFrame)frame).getImage();
	if(frame instanceof AlmaFilleDiagramme)
	{
	  if(filterImage.accept(r))
	  {
	    i=((AlmaFilleDiagramme)frame).getImage();
	    //((AlmaFilleDiagramme)frame).setFile(r);
	  }
	  else
	  {
	    t=((AlmaFilleDiagramme)frame).getSource();
	    ((AlmaFilleDiagramme)frame).setFile(r);
	  }
	}

	AlmaTacheEnregistrement tache=null;
	if(t!=null) tache=new AlmaTacheEnregistrement(this,r,t);
	if(i!=null) tache=new AlmaTacheEnregistrement(this,r,i);
	tache.setPriority(Thread.MIN_PRIORITY);
	tache.start();
      }
    }
  }

  public void cmdPreference()
  {
    new BuTaskOperation(this,AlmaResource.ALMA.getString("Prfrences"))
      { public void act() { opPreference(); } }
      .start();
  }

  public void opPreference()
  {
    BuMainPanel mp=getMainPanel();
    mp.setProgression(5);

    if(ifPreferences==null)
    {
      ifPreferences=new BuPreferencesFrame(this);
      mp.setProgression(5);

      ifPreferences.addTab(new BuLookPreferencesPanel    (this));
      mp.setProgression(10);
      ifPreferences.addTab(new BuCommandPreferencesPanel (this));
      mp.setProgression(15);
      ifPreferences.addTab(new BuDesktopPreferencesPanel (this));
      mp.setProgression(20);
      ifPreferences.addTab(new BuLanguagePreferencesPanel(this));
      mp.setProgression(25);
      ifPreferences.addTab(new BuMonitorPreferencesPanel (this));
      mp.setProgression(30);
      ifPreferences.addTab(new BuBrowserPreferencesPanel (this));
      mp.setProgression(35);
      if(!(getApp() instanceof BuApplet))
	ifPreferences.addTab(new BuDirWinPreferencesPanel(this));
      mp.setProgression(40);
      if(AlmaPreferences.ALMA.getBooleanProperty("module.script",true))
	ifPreferences.addTab(new ScriptPreferencesPanel(this));
      mp.setProgression(45);

      /*
      ifPreferences.addTab(new BuGenericPreferencesPanel
	(this,BuResource.BU.getString("Outils"),
	 AlmaPreferences.ALMA,"tool",true,true));
      */

      ifPreferences.addTab(new BuToolsPreferencesPanel
	  (this,AlmaPreferences.ALMA));
      mp.setProgression(50);
      ifPreferences.addTab(new BuUserPreferencesPanel    (this));
      mp.setProgression(55);
      ifPreferences.addTab(new AlmaPreferencesPanel      (this));
      mp.setProgression(60);
      ifPreferences.addTab(new AlmaEditorPreferencesPanel(this));
      mp.setProgression(65);

      /*
      ifPreferences.addTab
	(new BuGenericPreferencesPanel
	 (this,AglPreferences.AGL,"Agl"));
      mp.setProgression(70);
      */
    }

    mp.setProgression(75);
    addInternalFrame(ifPreferences);
    mp.setProgression(0);
  }

  // Projet

  private void ajouteFenetreProjet()
  {
    if(afProjet==null)
    {
      afProjet=new AlmaFilleProjet(this);
      afProjet.setShortcut(KeyEvent.VK_F5);
      Dimension dd=getMainPanel().getDesktop().getSize();
      Dimension df=afProjet.getSize();
      afProjet.setLocation(0,370); //dd.height-df.height-37);
    }
    addInternalFrame(afProjet);
  }

  public void cmdOuvrirProjet()
  {
    if(getApp() instanceof BuApplet) return;

    ajouteFenetreProjet();

    //System.err.println("### FD begin");
    BuFileChooser chooser=new BuFileChooser();
    chooser.setFileSelectionMode(BuFileChooser.FILES_ONLY);
    chooser.setFileHidingEnabled(true);
    chooser.setMultiSelectionEnabled(false);
    chooser.resetChoosableFileFilters();

    if(openDirectory_==null)
    {
      String d=BuPreferences.BU.getStringProperty("directory.open");
      if(!d.equals(""))
      {
	File f=new File(BuLib.expandedPath(d));
	if(f.canRead()) openDirectory_=f;
      }
    }

    if(openDirectory_!=null)
      chooser.setCurrentDirectory(openDirectory_);

    chooser.addChoosableFileFilter(filterSer);
    chooser.addChoosableFileFilter(filterXgz);
    chooser.addChoosableFileFilter(filterXml);
    chooser.setFileFilter         (filterAlm);
    // chooser.removeChoosableFileFilter(chooser.getAcceptAllFileFilter());

    int returnVal=chooser.showOpenDialog((BuApplication)getApp());
    //System.err.println("### FD end");

    if(returnVal == BuFileChooser.APPROVE_OPTION)
    {
      File f=chooser.getSelectedFile();
      //System.err.println("### f="+f);
      if(f!=null)
      {
	openDirectory_=f;

	System.err.println("OPENING PROJECT "+f);
	AglOptions.terminal.message
	  (AlmaResource.ALMA.getString("Ouverture")+
	   ": "+BuLib.reducedPath(f.getPath()));
	AglOptions.terminal.progres(5);

	try { AglProjet.load(f.getAbsolutePath()); }
	catch(Exception ex)
	{
	  AglOptions.terminal.erreur("Can not read "+f);
	  AglOptions.terminal.erreur(""+ex);
	}
      }
    }

    rafraichir();
  }

  public void cmdEnregistrerProjet()
  {
    if(getApp() instanceof BuApplet) return;

    // System.err.println("### FD begin : "+r);

    BuFileChooser chooser=new BuFileChooser();
    chooser.setFileSelectionMode(BuFileChooser.FILES_ONLY);
    chooser.setFileHidingEnabled(true);
    chooser.setMultiSelectionEnabled(false);
    chooser.resetChoosableFileFilters();

    if(saveDirectory_==null)
    {
      String d=BuPreferences.BU.getStringProperty("directory.save");
      if(!d.equals(""))
      {
	File f=new File(BuLib.expandedPath(d));
	if(f.canRead()) saveDirectory_=f;
      }
    }

    if(saveDirectory_==null)
      saveDirectory_=openDirectory_;

    if(saveDirectory_!=null)
      chooser.setCurrentDirectory(saveDirectory_);

    chooser.addChoosableFileFilter(filterSer);
    chooser.addChoosableFileFilter(filterXml);
    chooser.addChoosableFileFilter(filterXgz);
    chooser.setFileFilter         (filterAlm);
    // chooser.removeChoosableFileFilter(chooser.getAcceptAllFileFilter());

    // chooser.setSelectedFile(new File(r));

    int returnVal=chooser.showSaveDialog((BuApplication)getApp());
    if(returnVal==BuFileChooser.APPROVE_OPTION)
    {
      File f=chooser.getSelectedFile();
      // System.err.println("### f="+f);
      if(f!=null)
      {
	saveDirectory_=f;

	System.err.println("SAVING PROJECT "+f);
	AglOptions.terminal.message
	  (AlmaResource.ALMA.getString("Enregistrement")+
	   ": "+BuLib.reducedPath(f.getPath()));
	AglOptions.terminal.progres(5);

	try { AglProjet.save(new AglProjet(), f.getAbsolutePath()); }
	catch(Exception ex)
	{
	  AglOptions.terminal.erreur("Can not write "+f);
	  AglOptions.terminal.erreur(""+ex);
	}
      }
    }
  }

  public void cmdExecuterProjet()
  {
    if(getApp() instanceof BuApplet) return;

    ajouteFenetreProjet();

    //System.err.println("### FD begin");
    BuFileChooser chooser=new BuFileChooser();
    chooser.setFileSelectionMode(BuFileChooser.FILES_ONLY);
    chooser.setFileHidingEnabled(true);
    chooser.setMultiSelectionEnabled(false);
    chooser.resetChoosableFileFilters();

    if(openDirectory_==null)
    {
      String d=BuPreferences.BU.getStringProperty("directory.open");
      if(!d.equals(""))
      {
	File f=new File(BuLib.expandedPath(d));
	if(f.canRead()) openDirectory_=f;
      }
    }

    if(openDirectory_!=null)
      chooser.setCurrentDirectory(openDirectory_);

    BuFileFilter filter=new BuFileFilter
      (new String[] { "acf", "ACF" }," Alma configuration files");
    chooser.setFileFilter(filter);
    // chooser.removeChoosableFileFilter(chooser.getAcceptAllFileFilter());

    int returnVal=chooser.showOpenDialog((BuApplication)getApp());
    //System.err.println("### FD end");

    if(returnVal == BuFileChooser.APPROVE_OPTION)
    {
      File f=chooser.getSelectedFile();
      //System.err.println("### f="+f);
      if(f!=null)
      {
	openDirectory_=f;

	System.err.println("OPENING CONFIGURATION FILE "+f);
	AglOptions.terminal.message
	  (AlmaResource.ALMA.getString("Ouverture")+
	   ": "+BuLib.reducedPath(f.getPath()));
	AglOptions.terminal.progres(5);

	Vector v=new Vector(10,10);
	try
	{
	  LineNumberReader rin=new LineNumberReader(new FileReader(f));
	  while(rin.ready())
	  {
	    String l=rin.readLine().trim();
	    if(l==null) break;
	    v.addElement(l);
	  }

	  AglOptions.terminal.progres(65);
	  final String[] args=new String[v.size()];
	  for(int i=0;i<v.size();i++)
	    args[i]=(String)v.elementAt(i);

	  AglOptions.terminal.progres(100);
	  System.err.println("PARSING ARGS...");
	  new BuTaskOperation(this,AlmaResource.ALMA.getString
			      ("Traitement des arguments"))
	    { public void act() { opArguments(args); } }
	    .start();
	}
	catch(Exception ex)
	  { AglOptions.terminal.erreur(""+ex); }
      }
    }

    rafraichir();
  }

  public void cmdReinitialiserProjet()
  {
    if(getApp() instanceof BuApplet) return;

    ajouteFenetreProjet();
    AglProjet.reinit(true);
    rafraichir();
    getApp().setEnabledForAction("GENERER",false);
  }

  // Divers

  public void cmdAnalyser(String _arg)
  {
    if(!"".equals(_arg))
    {
      File f=new File(_arg);
      //System.err.println("### f="+f);
      AlmaTacheAnalyser tache=new AlmaTacheAnalyser(this,f.getPath());
      tache.setPriority(Thread.MIN_PRIORITY);
      tache.start();
      return;
    }

    JInternalFrame frame=getCurrentInternalFrame();

    if((frame!=null)&&(frame instanceof AlmaFilleSource))
    {
      AlmaFilleSource f=(AlmaFilleSource)frame;
      String r=f.getFichier();
      String t=f.getSource();

      AglParse parse=AglOptions.parse();
      if(parse!=null)
      {
	f.setLangage(parse.langage());
	//f.setSyntaxe(parse.syntaxe());

	AglOptions.terminal.trace("Analyse de "+r+"...");
	AglOptions.terminal.trace("Avec "+parse.langage());
	AlmaTacheAnalyser tache=new AlmaTacheAnalyser(this,r,t);
	tache.setPriority(Thread.MIN_PRIORITY);
	tache.start();
      }
      else
      {
	String msg="Invalid parser: "+AglOptions.parse;
	new BuDialogError
	  (this,getInformationsSoftware(),msg)
	    .activate();
      }
    }
  }

  public void cmdGenerer()
  {
    AglCode code=AglOptions.code();
    if(code!=null)
    {
      AglProjet projet=new AglProjet();

      AglOptions.terminal.trace("Gnration de "+projet.nom+"...");
      AglOptions.terminal.trace("Avec "+code.langage());
      AlmaTacheGenerer tache=new AlmaTacheGenerer(this,projet);
      tache.setPriority(Thread.MIN_PRIORITY);
      tache.start();
    }
    else
    {
      String msg="Invalid generator: "+AglOptions.code;
      new BuDialogError
	(this,getInformationsSoftware(),msg)
	  .activate();
    }
  }

  public void cmdConfigurer()
  {
    if(afOptions==null)
    {
      afOptions=new AlmaFilleOptions(this);
      afOptions.setShortcut(KeyEvent.VK_F3);
      afOptions.setLocation(0,74);
    }
    addInternalFrame(afOptions);
    // setEnabledForAction("CONFIGURER",true);
  }

  public void cmdMessages()
  {
    /*
    if(afMessages==null)
    {
      afMessages=new AlmaFilleMessages(this);
      afMessages.setShortcut(KeyEvent.VK_F6);
      AglOptions.terminal=afMessages;

      Dimension dd=getMainPanel().getDesktop().getSize();
      Dimension df=afMessages.getSize();
      afMessages.setLocation(dd.width-df.width-20,dd.height-df.height-60);
    }
    */

    addInternalFrame(afMessages);

    // setEnabledForAction("MESSAGE",true);
  }

  public void cmdArbreModele()
  {
    if(afArbre==null)
    {
      afArbre=new AlmaFilleArbre(this);
      afArbre.setShortcut(KeyEvent.VK_F4);
      Dimension dd=getMainPanel().getDesktop().getSize();
      Dimension df=afArbre.getSize();
      afArbre.setLocation(444,74); //dd.width-df.width,0
    }
    addInternalFrame(afArbre);
    // setEnabledForAction("ARBRE_MODELE",true);
  }

  public void cmdProjetTable()
  {
    ajouteFenetreProjet();
  }

  // Menu Modele

  public void cmdCreerPatron()
  {
    if(afPatronType==null)
      afPatronType=new AlmaFillePatronType(this);

    addInternalFrame(afPatronType);
  }

  public void cmdCreerRelation()
  {
  }

  public void cmdCreerCondition()
  {
    AglCondition c=new AglCondition(new AglExpressionGenerique(""));
    AlmaEditeur ed=new AlmaEditeurCondition(this,c);
    if(ed!=null) ed.activate();
    System.out.println("CONDITION\n"+AglOptions.code().code(c));
  }

  public void cmdConstruirePatrons()
  {
    boolean r=AglOptions.analyserCommentaires;

    if(!r)
      r=(JOptionPane.YES_OPTION==
	 (new BuDialogConfirmation
	  (this,getInformationsSoftware(),
	   "Les commentaires semblent ne pas avoir t analyss.\n"+
	   "Il n'y a donc surement aucune information disponible\n"+
	   "pour construire les patrons.\n"+
	   "Voulez-vous continuer ?").activate()));

    if(r)
    {
      AlmaTacheConstructionPatrons tache=
	new AlmaTacheConstructionPatrons(this);
      tache.setPriority(Thread.MIN_PRIORITY);
      tache.start();
    }
  }

  public void cmdConstruireRelations()
  {
    AlmaTacheConstructionRelations tache=
      new AlmaTacheConstructionRelations(this);
    tache.setPriority(Thread.MIN_PRIORITY);
    tache.start();
  }

  public void cmdConstruireConditions()
  {
    boolean r=AglOptions.analyserCommentaires;

    if(!r)
      r=(JOptionPane.YES_OPTION==
	 (new BuDialogConfirmation
	  (this,getInformationsSoftware(),
	   "Les commentaires semblent ne pas avoir t analyss.\n"+
	   "Il n'y a donc surement aucune information disponible\n"+
	   "pour construire les conditions.\n"+
	   "Voulez-vous continuer ?").activate()));

    if(r)
    {
      AlmaTacheConstructionConditions tache=
	new AlmaTacheConstructionConditions(this);
      tache.setPriority(Thread.MIN_PRIORITY);
      tache.start();
    }
  }

  // Graphe

  public void cmdCreerGraphe()
  {
    DjaGridInteractive grille=new DjaGridInteractive();
    grille.setInteractive(true);
    addInternalFrame(new AlmaFilleDiagramme(this,"SansTitre.dja",grille));
  }

  // Montrer erreur

  public void montrerErreur(String _fichier, int _ligne)
  {
    JInternalFrame[] frames=getAllInternalFrames();

    for(int i=0;i<frames.length;i++)
    {
      if(frames[i] instanceof AlmaFilleSource)
      {
	AlmaFilleSource frame=(AlmaFilleSource)frames[i];
	String fichier=frame.getFichier();
	if(  fichier.equals(_fichier)
	   ||fichier.endsWith(File.separator+_fichier))
	{
	  Component gui=frame.getComponent();

	  if(gui instanceof BuTextArea)
	    ((BuTextArea)gui).go(_ligne);
	  else
	  if(gui instanceof BuTextPane)
	    ((BuTextPane)gui).go(_ligne);

	  activateInternalFrame(frame);
	}
      }
    }
  }

  // Script

  public void cmdTerminal(String _interpreteur)
  {
    final String s=_interpreteur.substring(0,1).toUpperCase()+
                   _interpreteur.substring(1).toLowerCase();

    new BuTaskOperation(this,AlmaResource.ALMA.getString("Terminal")+" "+s)
      { public void act() { terminal(s); } }
      .start();
  }

  public void terminal(String _interpreteur)
  {
    BuMainPanel mp=getMainPanel();
    mp.setProgression(10);

    AlmaFilleTerminal afScript=new AlmaFilleTerminal(this,_interpreteur);
    addInternalFrame(afScript);
    mp.setProgression(20);

    ScriptTerminal term=afScript.getScriptTerminal();
    term.setTaskView(ltTaches);
    term.reset(_interpreteur);
    ScriptInterpreterInterface si=term.getScriptInterpreter();
    mp.setProgression(30);

    if(si==null)
    {
      AglOptions.terminal.erreur
	(_interpreteur+" n'est pas un interprteur disponible");
      mp.setProgression(100);
      mp.setProgression(0);
      return;
    }

    // term.setScriptInterpreter(si);

    try
    {
      AglOptions.terminal.message(_interpreteur+" lanc...");
      mp.setProgression(40);
      // si.start();
      // term.run();
      term.startThread();

      mp.setProgression(60);
      // si.addVariable("interpreter",si);
      si.addVariable("alma",this);

      try
      {
	Method m;

	// m=getClass().getMethod("scr_exit",new Class[0]);
	// si.addFunction("exit",m);

	m=AglPublicAPI.class.getMethod("hello",new Class[0]);
	si.addFunction("hello",m);
      }
      catch(Exception ex) { ex.printStackTrace(); }

      mp.setProgression(80);
      si.eval("\"Hello from "+_interpreteur+"\"");
      mp.setProgression(100);
      mp.setProgression(0);

      // si.run();
      /*
	AglOptions.terminal.erreur
	(_script+" arret!");
	*/
    }
    catch(Exception ex)
    {
      mp.setProgression(0);
      term.stopThread();
      ex.printStackTrace();
      AglOptions.terminal.erreur
	(_interpreteur+" arret! "+ex.getMessage());
    }
  }

  public void cmdScript(final String _fichier)
  {
    new BuTaskOperation(this,AlmaResource.ALMA.getString
			("Script")+" "+_fichier)
      { public void act() { script(_fichier); } }
      .start();
  }

  public void cmdOutil(final String _exec)
  {
    new BuTaskOperation(this,AlmaResource.ALMA.getString
			("Outil")+" "+_exec)
      { public void act()
	  { BuLib.runShellCommand(AlmaImplementation.this,_exec); } }
      .start();
  }

  public void script(String _fichier)
  {
    ScriptInterpreterInterface si=ScriptInterpreter.guess(_fichier);

    if(si!=null)
    {
      ByteArrayOutputStream out=new ByteArrayOutputStream();

      try
      {
	LineNumberReader lin=new LineNumberReader
	  (new FileReader(_fichier));
	String s="";
	while(true)
	{
	  String l=lin.readLine();
	  if(l==null) break;
	  s+=l+'\n';
	}
	if(!"".equals(s))
	{
	  si.setOutputStream(out);
	  si.setErrorStream(out);
	  si.start();
	  si.addVariable("alma",this);
	  si.eval(s);
	  si.exit();

	  String msg=new String(out.toByteArray());
	  if(!"".equals(msg))
	    new BuDialogMessage
	      (this,getInformationsSoftware(),msg)
	      .activate();
	}
      }
      catch(Exception ex)
      {
	AglOptions.terminal.erreur
	  ("Le script "+_fichier+" n'a pu tre excut");

	PrintWriter pw=new PrintWriter(out);
	pw.println();
	ex.printStackTrace(pw);
	pw.flush();
	String msg=new String(out.toByteArray());
	new BuDialogError
	  (this,getInformationsSoftware(),msg)
	  .activate();
      }
    }
    else
      AglOptions.terminal.erreur
	("Aucun interprteur trouv pour "+_fichier);
  }

  public void exit()
  {
    BuInformationsSoftware il=getInformationsSoftware();

    if(assistant_!=null)
      assistant_.actionPerformed
	(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,
			 "QUITTER("+il.name.toUpperCase()+")"));

    if(!(getApp() instanceof BuApplet))
    {
      AglPreferences .AGL .writeIniFile();
      AlmaPreferences.ALMA.writeIniFile();
    }

    super.exit();
  }

  public void displayURL(String _url)
  {
    if(_url==null)
    {
      BuInformationsSoftware il=getInformationsSoftware();
      _url=il.http;
    }

    String ll=Locale.getDefault().getLanguage();
    //System.err.println("LOCALE LL="+ll);
    if(ll.equals("en")||ll.equals("de"))
    {
      BuInformationsSoftware il=getInformationsSoftware();
      if(_url.equals(il.http))
	_url+="index-"+ll+".html";
      else
      if(_url.equals(il.baseManUrl()))
        _url+="index-"+ll+".html";
      else
      if(_url.endsWith("/alphabetique.html"))
        _url=FuLib.replace
            (_url,"/alphabetique.html","/alphabetique-"+ll+".html");
      else
      if(_url.endsWith("/thematique.html"))
        _url=FuLib.replace
            (_url,"/thematique.html","/thematique-"+ll+".html");
    }

    if(  (BuPreferences.BU.getIntegerProperty("browser.type",1)!=1)
       ||(  (_url!=null)
	  &&(_url.startsWith("https:")||_url.startsWith("mailto:"))))
    {
      BuBrowserControl.displayURL(_url);
    }
    else
    {
      if(ifAide==null)
	ifAide=new BuHelpFrame(this);
      addInternalFrame(ifAide);
      ifAide.setDocumentUrl(_url);
    }
  }

  // AglTransfert

  public void transfert(AglCodeUmlHierarchie _o)
  {
    AlmaCodeUmlHierarchieFrame f=new AlmaCodeUmlHierarchieFrame(_o);
    ((AlmaImplementation)AglOptions.application).addInternalFrame(f);
  }

  public void transfert(AglCodeUmlRelations  _o)
  {
    AlmaCodeUmlRelationsFrame f=new AlmaCodeUmlRelationsFrame(_o);
    ((AlmaImplementation)AglOptions.application).addInternalFrame(f);
  }
}
