/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaPreferencesPanel.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.alma.*;
import com.memoire.bu.*;

/**
 * @creation     30/08/99
 * @modification 30/08/99
 * @statut       instable
 */

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A panel where the user can choose which windows will
 * appear and which modules will be loaded at startup.
 */

/**
 * window.messages.visible
 * ...
 * module.dja
 * ...
 */

public class AlmaPreferencesPanel
       extends BuAbstractPreferencesPanel
       implements ActionListener
{
  private final static String[] INTITULES=new String[]
  {
    AlmaResource.ALMA.getString("Configuration"),
    AlmaResource.ALMA.getString("Projet"),
    AlmaResource.ALMA.getString("Arbre du modle"),
    AlmaResource.ALMA.getString("Messages"),
    AlmaResource.ALMA.getString("Assistant"),
    AlmaResource.ALMA.getString("Dja"),
    AlmaResource.ALMA.getString("Explorateur"),
    AlmaResource.ALMA.getString("Scripts"),
  };

  private final static String[] NOMS=new String[]
  {
    "CONFIGURATION","PROJECT","MODELTREE","MESSAGES",
    "ASSISTANT", "DJA", "EXPLORER", "SCRIPT",
  };

  AlmaImplementation appli_;
  AlmaPreferences    options_;

  BuCheckBox[]    cbFenetres;
  BuCheckBox[]    cbModules;
  
  public String getTitle()
    { return "Alma"; }

  // Constructeur

  public AlmaPreferencesPanel(AlmaImplementation _appli)
  {
    super();
    appli_=_appli;
    init();
  }

  private void init()
  {
    options_=AlmaPreferences.ALMA;

    JPanel p1=new JPanel();
    p1.setBorder
      (new CompoundBorder
       (new TitledBorder(BuResource.BU.getString("Fentres")),
	new EmptyBorder(5,5,5,5)));
    p1.setLayout(new GridLayout(2,0,5,5));

    cbFenetres=new BuCheckBox[4];
    for(int i=0;i<4;i++)
    {
      cbFenetres[i]=new BuCheckBox(INTITULES[i]);
      cbFenetres[i].setName("cb"+NOMS[i]+"_VISIBLE");
      cbFenetres[i].setActionCommand(NOMS[i]);
      p1.add(cbFenetres[i]);
      cbFenetres[i].addActionListener(this);
    }

    JPanel p2=new JPanel();
    p2.setBorder
      (new CompoundBorder
       (new TitledBorder(BuResource.BU.getString("Modules")),
	new EmptyBorder(5,5,5,5)));
    p2.setLayout(new GridLayout(2,0,5,5));

    cbModules=new BuCheckBox[4];
    for(int i=0;i<4;i++)
    {
      cbModules[i]=new BuCheckBox(INTITULES[i+4]);
      cbModules[i].setName("cb"+NOMS[i+4]+"_MODULE");
      cbModules[i].setActionCommand(NOMS[i+4]);
      p2.add(cbModules[i]);
      cbModules[i].addActionListener(this);
    }

    setLayout(new BuVerticalLayout(5,true,false));
    setBorder(new EmptyBorder(5,5,5,5));
    add(p1);
    add(p2);

    updateComponents();
  }

  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    /*
    JComponent source=(JComponent)_evt.getSource();
    System.err.println("SOURCE="+source.getName());
    System.err.println("ACTION="+_evt.getActionCommand());
    */
  }
  
  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  /*
  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
    options_.applyOn(appli_);
  }
  */

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
    for(int i=0;i<4;i++)
      options_.putBooleanProperty
	  ("window."+NOMS[i].toLowerCase()+".visible",
	   cbFenetres[i].isSelected());
    for(int i=0;i<4;i++)
      options_.putBooleanProperty
	  ("module."+NOMS[i+4].toLowerCase(),
	   cbModules[i].isSelected());
  }

  private void updateComponents()
  {
    for(int i=0;i<4;i++)
      cbFenetres[i].setSelected
	(options_.getBooleanProperty
	 ("window."+NOMS[i].toLowerCase()+".visible",true));
    for(int i=0;i<4;i++)
      cbModules[i].setSelected
	(options_.getBooleanProperty
	 ("module."+NOMS[i+4].toLowerCase(),true));
  }
}
