/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaTableauProjet.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;

public class AlmaTableauProjet
       extends BuTable
{
  public AlmaTableauProjet()
  {
    super(new AlmaTableauProjetModel());
    setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

    int[]     tailles  =new int[]     { 22,160,66,22,66 };
    boolean[] variables=new boolean[] { false,true,true,false,true };

    //TableCellRenderer tcr=new BuTableCellRenderer();
    for(int i=0;i<getColumnCount();i++)
    {
      TableColumn c=getColumn(getColumnName(i));
      //c.setCellRenderer(tcr);
      c.setPreferredWidth(tailles[i]);
      if(!variables[i])
      {
	c.setResizable(false);
	c.setMinWidth (tailles[i]);
	c.setMaxWidth (tailles[i]);
      }
    }
  }

  public void rafraichir()
  {
    ((AlmaTableauProjetModel)getModel()).fireTableChanged();
    invalidate();
    repaint();
  }
}

class AlmaTableauProjetModel
      implements TableModel
{
  Vector listeners_;

  public AlmaTableauProjetModel()
  {
    listeners_=new Vector();
  }

  public Class getColumnClass(int column)
  {
    Class r=null;
    switch(column)
    {
    case  0: r=Icon.class;    break; // source/cible
    case  1: r=String.class;  break; // fichier
    case  2: r=String.class;  break; // langage
    case  3: r=Boolean.class; break; // validite
    case  4: r=Long.class;    break; // taille
    default: r=Object.class;  break;
    }
    return r;
  }

  public int getColumnCount()
  { return 5; }

  public String getColumnName(int column)
  {
    String r="";

    switch(column)
    {
    case  0: r=AlmaResource.ALMA.getString("T");       break;
    case  1: r=AlmaResource.ALMA.getString("Fichier"); break;
    case  2: r=AlmaResource.ALMA.getString("Langage"); break;
    case  3: r=AlmaResource.ALMA.getString("V");       break;
    case  4: r=AlmaResource.ALMA.getString("Taille");  break;
    default: r="undefined"; break;
    }
    
    return r;
  }

  public int getRowCount()
  {
    return AglUnite.table.size();
  }

  public AglUnite getUnite(int _index)
  {
    AglUnite r=null;
    int      j=0;

    for(Enumeration e=AglUnite.table.elements(); e.hasMoreElements(); )
    {
      AglUnite u=(AglUnite)e.nextElement();
      if(_index==j) r=u;
      j++;
    }

    return r;
  }

  public Object getValueAt(int row, int column)
  {
    Object   r=null;
    AglUnite u=getUnite(row);

    switch(column)
    {
    case 0:
      BuIcon icon=BuResource.BU.getIcon(u.cible ? "cible" : "texte",16);
      icon.setDescription(u.cible ? "cible" : "source");
      r=icon;
      break;
    case 1:
      String s=BuLib.fileName(u.fichier);
      int i=s.lastIndexOf('.');
      if(i>=0) s=s.substring(0,i);
      r=s;
      break;
    case 2:
      r=u.langage;
      break;
    case 3:
      boolean b=new File(u.fichier).exists();
      // r=BuResource.BU.getIcon(b ? "valider" : "annuler",16);
      r=new Boolean(b); // ""+b;
      break;
    case 4:
      r=new Long(new File(u.fichier).length());
      break;
    default:
      r="default";
      break;
    }

    return r;
  }

  public boolean isCellEditable(int row, int column)
  { return false; }

  public void setValueAt(Object value, int row, int column)
  { }

  public void fireRowChanged(int row)
  {
    TableModelEvent e=new TableModelEvent(this,row);
    Enumeration     l=listeners_.elements();
    while(l.hasMoreElements())
      ((TableModelListener)l.nextElement()).tableChanged(e);
  }

  public void fireTableChanged()
  {
    TableModelEvent e=new TableModelEvent(this);
    Enumeration     l=listeners_.elements();
    while(l.hasMoreElements())
      ((TableModelListener)l.nextElement()).tableChanged(e);
  }

  public void addTableModelListener(TableModelListener _l)
  { listeners_.addElement(_l); }

  public void removeTableModelListener(TableModelListener _l)
  { listeners_.removeElement(_l); }
}
