/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaTacheAnalyser.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;

import java.io.*;

public class AlmaTacheAnalyser
       extends BuTask
{
  private String             path_;
  private AlmaImplementation app_;
  private String             source_;

  public AlmaTacheAnalyser
  (AlmaImplementation _app, String _path)
  {
    this(_app,_path,null);
  }

  public AlmaTacheAnalyser
  (AlmaImplementation _app, String _path, String _source)
  {
    app_   =_app;
    path_  =_path;
    source_=_source;

    setName(AlmaResource.ALMA.getString("Analyse")+
	    ": "+BuLib.fileName(path_));
    setTaskView(app_.getMainPanel().getTaskView());
  }

  public void start()
  {
    if(source_!=null) app_.setEnabledForAction("ANALYSER",false);
    super.start();
  }

  public void run()
  {
    AglOptions.terminal.message(AlmaResource.ALMA.getString("Analyse")+
				": "+BuLib.reducedPath(path_));
    AglOptions.terminal.message(AlmaResource.ALMA.getString("Langage")+
				": "+AglOptions.parse);
    AglOptions.terminal.progres(0);

    try
    {
      int n=0;
      if(source_!=null)
	for(int i=0;i<source_.length();i++)
	  if(source_.charAt(i)=='\n') n++;

      InputStream sin;
      if(source_!=null) sin=new ByteArrayInputStream(source_.getBytes());
      else              sin=new FileInputStream(path_);

      // duplicate parser
      AglParse p=AglOptions.parse();
      p.MaxLines=n;
      p.initStream(sin,path_);
      p.parseFichier();

      AglOptions.terminal.message(BuLib.reducedPath(path_)+" "+
				  AlmaResource.ALMA.getString("analys"));
    }
    catch(Exception ex)
    {
      AglOptions.terminal.erreur(path_+" "+ex.getMessage());
      ex.printStackTrace();
    }

    AglOptions.terminal.progres(90);
    app_.rafraichir();

    AglOptions.terminal.progres(100);
    if(source_!=null) app_.setEnabledForAction("ANALYSER",true);
    app_.setEnabledForAction("GENERER",true);
    AglOptions.terminal.progres(0);
    super.run();
  }
}

