/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaTacheDessiner.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;
import com.memoire.bu.*;
import com.memoire.dja.*;

import java.io.*;

public class AlmaTacheDessiner
       extends BuTask
{
  private AlmaImplementation app_;
  private Object[]           objets_;

  public AlmaTacheDessiner(AlmaImplementation _app, Object[] _objets)
  {
    app_   =_app;
    objets_=_objets;

    setName(AlmaResource.ALMA.getString("Dessin"));
    setTaskView(app_.getMainPanel().getTaskView());
  }

  public void start()
  {
    super.start();
  }

  public void run()
  {
    int nb=objets_.length;
    AglOptions.terminal.message(AlmaResource.ALMA.getString("Dessin")+": "+nb);
    AglOptions.terminal.progres(0);

    DjaGrid grille =new DjaGridInteractive();
    String  fichier=BuResource.BU.getString("SansTitre")+".dja";
  
    for(int j=0; j<nb; j++)
    {
      Object v=objets_[j];
      AglOptions.terminal.progres(5+75*j/nb);

      if(v instanceof AglObjet) // !=null
      {
	DjaObject o=AlmaDjaRelations.dejaPresent(grille,(AglObjet)v);
	if(o!=null) continue;

	if(v instanceof AglClasse)
	  o=new AlmaDjaClasse((AglClasse)v);
	else
	if(v instanceof AglStruct)
	  o=new AlmaDjaStruct((AglStruct)v);
	else
	if(v instanceof AglType)
	  o=new DjaBox(((AglType)v).nom.dernier());
	else
	if(v instanceof AglPaquet)
	  o=new AlmaDjaPaquet((AglPaquet)v);
	else
	if(v instanceof AglMethode)
	  o=new AlmaDjaMessage((AglMethode)v);
	else
	if(v instanceof AglRelationBiOrientee)
	  o=new AlmaDjaRelationBiOrientee((AglRelationBiOrientee)v);
	else
	if(v instanceof AglCommentaire)
	  o=new AlmaDjaCommentaire((AglCommentaireSimple)v);

	if(o!=null)
	{
	  grille.add(o);
	  grille.avoidOverlap(o);
	  AlmaDjaRelations.creeTout(grille,o);
	  grille.recenter();
	}

	if(v instanceof AglPatron)
	{
	  AglPatron     patron=(AglPatron)v;
	  AglPatronType type  =patron.type;
	  
	  try
	  {
	    InputStream in=getClass().getResourceAsStream
	      ("Pattern"+type.nom+".dja");
	    LineNumberReader rin=new LineNumberReader
	      (new InputStreamReader(in));

	    String l,r;
	    r="";
	    while(true)
	    {
	      l=rin.readLine();
	      if(l==null) break;
	      r+=l+"\n";
	    }

	    for(int i=0;i<type.roles.length; i++)
	    {
	      String role=type.roles[i];
	      String acteur=(String)patron.roles.get(role);

	      if(acteur!=null)
		while(true)
		{
		  int k=r.indexOf("%24"+role);
		  if(k<0) break;
		  r=r.substring(0,k)+acteur+r.substring(k+role.length()+3);
		}
	    }

	    //in=new StringBufferInputStream(r);
	    in=new ByteArrayInputStream(r.getBytes());
	    AlmaFilleDiagramme fille=new AlmaFilleDiagramme
	      (app_,patron.nom+".dja",
	       new DjaGridInteractive(true,DjaLoader.loadAsText(in)));
	    app_.addInternalFrame(fille);
	  }
	  catch(Exception ex) { }
	}
      }
    }

    AglOptions.terminal.progres(85);
    if(grille.getObjects().size()>0)
    {
      AlmaFilleDiagramme fille=new AlmaFilleDiagramme
	(app_,fichier,grille);
      app_.addInternalFrame(fille);
    }

    AglOptions.terminal.progres(0);
    super.run();
  }
}

