/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         AlmaTypesModel.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:06 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.alma;
import com.memoire.agl.*;
import com.memoire.alma.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

public class AlmaTypesModel
       implements ComboBoxModel
{
  private boolean elementaires_;
  private boolean classes_;
  private boolean interfaces_;
  private boolean structures_;
  private Vector  listeners_;
  private Vector  items_;
  private Object  selection_;
  private int     size_;

  public AlmaTypesModel()
  { this(false,true,true,true); }

  public AlmaTypesModel(boolean _e, boolean _c, boolean _i, boolean _s)
  {
    listeners_   =new Vector(2,2);
    items_       =new Vector(0,11);
    selection_   =null;
    elementaires_=_e;
    classes_     =_c;
    interfaces_  =_i;
    structures_  =_s;
    size_        =-1;
    updateItems();
  }

  public void update()
  {
    updateItems();
    fireContentsChanged();
  }

  private void fireContentsChanged()
  {
    ListDataEvent evt=new ListDataEvent
      (this,ListDataEvent.CONTENTS_CHANGED,-1,-1);

    synchronized (listeners_)
    {
      for(Enumeration e=listeners_.elements(); e.hasMoreElements(); )
      {
	try
	{
	  ((ListDataListener)e.nextElement()).
	    contentsChanged(evt);
	}
	catch(Exception ex) { }
      }
    }
  }

  public boolean isElementaires()
  { return elementaires_; }

  public void setElementaires(boolean _b)
  {
    if(elementaires_!=_b)
    {
      elementaires_=_b;
      update();
    }
  }

  public boolean isClasses()
  { return classes_; }

  public void setClasses(boolean _b)
  {
    if(classes_!=_b)
    {
      classes_=_b;
      update();
    }
  }

  public boolean isInterfaces()
  { return interfaces_; }

  public void setInterfaces(boolean _b)
  {
    if(interfaces_!=_b)
    {
      interfaces_=_b;
      update();
    }
  }

  public boolean isStructures()
  { return structures_; }

  public void setStructures(boolean _b)
  {
    if(structures_!=_b)
    {
      structures_=_b;
      update();
    }
  }

  public Object getSelectedItem()
  {
    /*
    Object r=null;
    if((selection_!=null)&&items_.contains(selection_)) r=selection_;
    return r;
    */
    return selection_;
  }

  public void setSelectedItem(Object _o)
  {
    if(selection_!=_o) // if(items_.contains(_o))
    {
      selection_=_o;
      fireContentsChanged();
    }
  }

  public int getSize()
  {
    updateItemsIfNeeded();
    return items_.size();
  }

  public Object getElementAt(int _i)
  {
    Object r=null;
    if((_i>=0)&&(_i<items_.size())) r=items_.elementAt(_i);
    return r;
  }
    
  public void addListDataListener(ListDataListener _l)
  { listeners_.addElement(_l); }
    
  public void removeListDataListener(ListDataListener _l)
  { listeners_.removeElement(_l); }

  private void updateItemsIfNeeded()
  {
    if(size_!=AglType.table.size())
      updateItems();
  }

  private void updateItems()
  {
    size_=AglType.table.size();

    synchronized (items_)
    {
      items_.removeAllElements();

      for(Enumeration e=AglType.table.elements(); e.hasMoreElements(); )
      {
	AglType at=(AglType)e.nextElement();
	if(at instanceof AglClasse)
	{
	  AglClasse ac=(AglClasse)at;
	  if(  (classes_     &&(elementaires_||!ac.elementaire)&&ac.normale )
	     ||(interfaces_  &&(elementaires_||!ac.elementaire)&&!ac.normale))
	    items_.addElement(ac.nom.dernier());
	}
	else
	if(at instanceof AglStruct)
	{
	  AglStruct as=(AglStruct)at;
	  if(structures_&&(elementaires_||!as.elementaire))
	    items_.addElement(as.nom.dernier());
	}
	else
	if(at instanceof AglTypeTableau)
	{
	}
	else
        {
	  if(elementaires_)
	    items_.addElement(at.nom.dernier());
	}
      }

      if(items_.size()==0)
	items_.addElement("");
    }
  }
}
