/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuApplet.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * A root class for applets.
 * Compatible with Bu et Swing.
 * Delegates quite everything to BuCommonImplementation.
 */

public class BuApplet
       extends JApplet
       implements BuCommonInterface, WindowListener, InternalFrameListener
{
  public BuApplet()
  {
    super();
    getRootPane().putClientProperty
      ("defeatSystemEventQueueCheck",Boolean.TRUE);
    setImplementation(new BuCommonImplementation());
  }

  public void init()
  {
    implementation_.init();
  }

  public void start()
  {
    implementation_.start();
  }

  // Implementation

  protected BuCommonImplementation implementation_;

  public BuCommonImplementation getImplementation()
  { return implementation_; }

  public void setImplementation(BuCommonImplementation _implementation)
  {
    implementation_=_implementation;
    implementation_.setApp(this);
  }

  // App

  public BuCommonInterface getApp()
  { return this; }

  // Main Panel

  public BuMainPanel getMainPanel()
  { return implementation_.getMainPanel(); }

  public void setMainPanel(BuMainPanel _p)
  { implementation_.setMainPanel(_p); }

  // Main MenuBar

  public BuMenuBar getMainMenuBar()
  { return implementation_.getMainMenuBar(); }

  public void setMainMenuBar(BuMenuBar _mb)
  { implementation_.setMainMenuBar(_mb); }

  // Main ToolBar

  public BuToolBar getMainToolBar()
  { return implementation_.getMainToolBar(); }

  public void setMainToolBar(BuToolBar _tb)
  { implementation_.setMainToolBar(_tb); }

  // Specific Bar

  public BuSpecificBar getSpecificBar()
  { return implementation_.getSpecificBar(); }

  public void setSpecificBar(BuSpecificBar _sb)
  { implementation_.setSpecificBar(_sb); }

  // Action

  public void actionPerformed(ActionEvent _evt)
  { implementation_.actionPerformed(_evt); }

  // Divers

  public BuInformationsSoftware getInformationsSoftware()
  { return implementation_.getInformationsSoftware(); }

  public void setTitle(String _s)
  {
    System.out.println("setTitle("+_s+")");
    AppletContext ac=getAppletContext();
    if(ac!=null) ac.showStatus(_s);
  }

  public Frame getFrame()
  {
    Frame r=null;

    Component c=this;
    do
    {
      c=c.getParent();
      if(c instanceof Frame) { r=(Frame)c; break; }
    }
    while(c!=null);

    return r;
  }

  public void setLookAndFeel(String _lnf)
  { implementation_.setLookAndFeel(_lnf); }

  public void removeAction(String _cmd)
  { implementation_.removeAction(_cmd); }

  public void setEnabledForAction(String _cmd, boolean _enabled)
  { implementation_.setEnabledForAction(_cmd,_enabled); }

  public void setCheckedForAction(String _cmd, boolean _enabled)
  { implementation_.setCheckedForAction(_cmd,_enabled); }

  public void showOrHide(String _cmd, JComponent _c)
  { implementation_.showOrHide(_cmd,_c); }

  public boolean confirmExit()
  { return implementation_.confirmExit(); }

  public void displayURL(String _url)
  {
    if(_url!=null)
      getAppletContext().showDocument(getDocumentBase(),_url);
  }

  public void setVisible(boolean _visible)
  {
    if(_visible)
    {
      BuLib.setDoubleBuffered(getMainMenuBar(),false);
      BuLib.setDoubleBuffered(getMainToolBar(),false);
      BuLib.setDoubleBuffered(getMainPanel  (),false);
    }
    super.setVisible(_visible);
  }

  // Commands

  public void about()
  { implementation_.about(); }

  public void cut()
  { implementation_.cut(); }

  public void copy()
  { implementation_.copy(); }

  public void exit()
  { 
    BuInformationsSoftware il=getInformationsSoftware();
    BuDialogConfirmation   bd=new BuDialogConfirmation
      (this,il,"Voulez-vous vraiment quitter ce logiciel ?");

    int r=bd.activate();
    if(r==JOptionPane.YES_OPTION)
      getAppletContext().showDocument(getDocumentBase(),"quitter.html");
  }

  public void license()
  { implementation_.license(); }

  public void duplicate()
  { implementation_.duplicate(); }

  public void find()
  { implementation_.find(); }

  public void paste()
  { implementation_.paste(); }

  public void print()
  {
    BuInformationsSoftware il=getInformationsSoftware();
    BuDialogError          bd=new BuDialogError
      (this,il,"L'impression n'est pas disponible dans une appliquette.");
  }

  public void redo()
  { implementation_.redo(); }

  public void replace()
  { implementation_.replace(); }

  public void select()
  { implementation_.select(); }

  public void undo()
  { implementation_.undo(); }

  // Window

  public void windowActivated(WindowEvent _evt)
  { implementation_.windowActivated(_evt); }

  public void windowClosed(WindowEvent _evt)
  { implementation_.windowClosed(_evt); }

  public void windowClosing(WindowEvent _evt)
  { implementation_.windowClosing(_evt);  }

  public void windowDeactivated(WindowEvent _evt)
  { implementation_.windowDeactivated(_evt); }

  public void windowDeiconified(WindowEvent _evt)
  { implementation_.windowDeiconified(_evt); }

  public void windowIconified(WindowEvent _evt)
  { implementation_.windowIconified(_evt); }

  public void windowOpened(WindowEvent _evt)
  { implementation_.windowOpened(_evt); }

  // InternalFrame

  public void internalFrameActivated(InternalFrameEvent _evt)
  { implementation_.internalFrameActivated(_evt); }

  public void internalFrameDeactivated(InternalFrameEvent _evt)
  { implementation_.internalFrameDeactivated(_evt); }

  public void internalFrameClosed(InternalFrameEvent _evt)
  { implementation_.internalFrameClosed(_evt); }

  public void internalFrameClosing(InternalFrameEvent _evt)
  { implementation_.internalFrameClosing(_evt); }

  public void internalFrameDeiconified(InternalFrameEvent _evt)
  { implementation_.internalFrameDeiconified(_evt); }

  public void internalFrameIconified(InternalFrameEvent _evt)
  { implementation_.internalFrameIconified(_evt); }

  public void internalFrameOpened(InternalFrameEvent _evt)
  { implementation_.internalFrameOpened(_evt); }
}

