/**
 * @modification $Date: 2001/12/07 18:28:12 $
 * @statut       unstable
 * @file         BuApplication.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * A root class for stand-alone applications.
 * Compatible with Bu et Swing.
 * Delegates quite everything to BuCommonImplementation.
 */

public class BuApplication
       extends JFrame
       implements BuCommonInterface, WindowListener,
                  InternalFrameListener
{
  /*
  protected void processKeyEvent(KeyEvent _evt)
  {
    if(  (_evt.getKeyCode()==KeyEvent.VK_F1)
       &&(_evt.getID()==KeyEvent.KEY_PRESSED))
    {
      System.err.println("Aide");
      // System.err.println("     sur "+xmouse_+","+ymouse_);
      // System.err.println("     sur "+help_);
      implementation_.displayURL(null);
    }
    else super.processKeyEvent(_evt);
  }
  */

  /*
  protected void processMouseEvent(MouseEvent _evt)
  {
    System.err.println("ME="+_evt);

    if(  ((_evt.getModifiers()&MouseEvent.SHIFT_MASK)!=0)
       &&(_evt.getID()==MouseEvent.MOUSE_CLICKED))
    {
      System.err.println("Aide pointe");
      // System.err.println("     sur "+xmouse_+","+ymouse_);
      // System.err.println("     sur "+help_);
    }
    else super.processMouseEvent(_evt);
  }
  */

  /*
  int xmouse_,ymouse_;
  Component help_;
  */

  public BuApplication()
  {
    super();
    BuRegistry.register(this);
    setImplementation(new BuCommonImplementation());
  }

  public void init()
  {
    Dimension ecran=getToolkit().getScreenSize();
    setLocation(ecran.width/10,ecran.height/10);
    setSize(ecran.width*4/5,ecran.height*4/5);

    setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    addWindowListener(this);

    implementation_.init();
  }

  public void start()
  {
    BuInformationsSoftware il=getInformationsSoftware();

    if(getName()==null)
      setName("fr"+il.name.toUpperCase());

    if(getRootPane().getName()==null)
      getRootPane().setName("rp"+il.name.toUpperCase());

    implementation_.installContextHelp(getRootPane(),""); // il.man);

    addNotify();
    /*
    pack();
    getRootPane().invalidate();
    getRootPane().revalidate();
    getRootPane().repaint(0);
    */
    //getRootPane().revalidate();
    //getRootPane().repaint(0);

    BuSplashScreen ss=implementation_.getSplashScreen();

    if(ss!=null)
    {
      ss.setVisible(false);
      ss=null;
      implementation_.setSplashScreen(null);
    }

    BuPreferences.BU.applyOn(getApp());
    setVisible(true);
    implementation_.start();
  }

  public Image getIconImage()
  {
    BuIcon l=getInformationsSoftware().logo;
    Image  r=null;
    if(l!=null) r=l.getImage();
    else        r=BuResource.BU.getImage("buapplication");
    return r;
  }

  // App

  public BuCommonInterface getApp()
  { return this; }

  // Implementation

  protected BuCommonImplementation implementation_;

  public BuCommonImplementation getImplementation()
  { return implementation_; }

  public void setImplementation(BuCommonImplementation _implementation)
  {
    implementation_=_implementation;
    implementation_.setApp(this);
  }

  // Main Panel

  public BuMainPanel getMainPanel()
  { return implementation_.getMainPanel(); }

  public void setMainPanel(BuMainPanel _p)
  { implementation_.setMainPanel(_p); }

  // Main MenuBar

  public BuMenuBar getMainMenuBar()
  { return implementation_.getMainMenuBar(); }

  public void setMainMenuBar(BuMenuBar _mb)
  { implementation_.setMainMenuBar(_mb); }

  // Main ToolBar

  public BuToolBar getMainToolBar()
  { return implementation_.getMainToolBar(); }

  public void setMainToolBar(BuToolBar _tb)
  { implementation_.setMainToolBar(_tb); }

  // Specific Bar

  public BuSpecificBar getSpecificBar()
  { return implementation_.getSpecificBar(); }

  public void setSpecificBar(BuSpecificBar _sb)
  { implementation_.setSpecificBar(_sb); }

  // Action

  public void actionPerformed(ActionEvent _evt)
  { implementation_.actionPerformed(_evt); }

  // Divers

  public BuInformationsSoftware getInformationsSoftware()
  { return implementation_.getInformationsSoftware(); }

  public Frame getFrame()
  { return this; }

  public void setLookAndFeel(String _lnf)
  { implementation_.setLookAndFeel(_lnf); }

  public void removeAction(String _cmd)
  { implementation_.removeAction(_cmd); }

  public void setEnabledForAction(String _cmd, boolean _enabled)
  { implementation_.setEnabledForAction(_cmd,_enabled); }

  public void setCheckedForAction(String _cmd, boolean _enabled)
  { implementation_.setCheckedForAction(_cmd,_enabled); }

  public void showOrHide(String _cmd, JComponent _c)
  { implementation_.showOrHide(_cmd,_c); }

  public boolean confirmExit()
  { return implementation_.confirmExit(); }

  public void displayURL(String _url)
  {
    BuInformationsSoftware il=getInformationsSoftware();
    if(_url==null) _url=il.man;

    BuDialogMessage bd=new BuDialogMessage(this,il,_url);
    bd.activate();
  }

  public void setVisible(boolean _visible)
  {
    if(_visible)
    {
      //RepaintManager.currentManager(this).setDoubleBufferingEnabled(false);
      //BuLib.setDoubleBuffered(this,false);
      BuLib.setDoubleBuffered(getMainMenuBar(),false);
      BuLib.setDoubleBuffered(getMainToolBar(),false);
      BuLib.setDoubleBuffered(getMainPanel  (),false);
    }
    super.setVisible(_visible);
  }

  // Commands

  public void about()
  { implementation_.about(); }

  public void cut()
  { implementation_.cut(); }

  public void copy()
  { implementation_.copy(); }

  public void exit()
  {
    if(confirmExit())
    {
      if(BuPreferences.BU.getIntegerProperty("window.size")==2)
      {
	Point     p=getLocation();
	Dimension d=getSize();
	BuPreferences.BU.putIntegerProperty("window.x",p.x);
	BuPreferences.BU.putIntegerProperty("window.y",p.y);
	BuPreferences.BU.putIntegerProperty("window.w",d.width);
	BuPreferences.BU.putIntegerProperty("window.h",d.height);
      }

      BuPreferences.BU.writeIniFile();
      BuRegistry.unregister(this);
      // System.exit(0);
    }
  }

  public void duplicate()
  { implementation_.duplicate(); }

  public void find()
  { implementation_.find(); }

  public void license()
  { implementation_.license(); }

  public void paste()
  { implementation_.paste(); }

  public void print()
  {
    JInternalFrame frame=getMainPanel().getCurrentInternalFrame();

    if(frame instanceof BuPrintable)
    {
      BuPrinter.print(this,
		      frame.getTitle(),
		      (BuPrintable)frame);
    }
    else
    {
      BuInformationsSoftware il=getInformationsSoftware();
      BuDialogError          bd=new BuDialogError
	(this,il,BuResource.BU.getString
	 ("L'impression n'est pas disponible pour")+
	"\""+frame.getTitle()+"\".");
    }
  }

  public void redo()
  { implementation_.redo(); }

  public void replace()
  { implementation_.replace(); }

  public void select()
  { implementation_.select(); }

  public void undo()
  { implementation_.undo(); }

  // Window

  public void windowActivated(WindowEvent _evt)
  { implementation_.windowActivated(_evt); }

  public void windowClosed(WindowEvent _evt)
  { implementation_.windowClosed(_evt); }

  public void windowClosing(WindowEvent _evt)
  { implementation_.windowClosing(_evt);  }

  public void windowDeactivated(WindowEvent _evt)
  { implementation_.windowDeactivated(_evt); }

  public void windowDeiconified(WindowEvent _evt)
  { implementation_.windowDeiconified(_evt); }

  public void windowIconified(WindowEvent _evt)
  { implementation_.windowIconified(_evt); }

  public void windowOpened(WindowEvent _evt)
  { implementation_.windowOpened(_evt); }

  // InternalFrame

  public void internalFrameActivated(InternalFrameEvent _evt)
  { implementation_.internalFrameActivated(_evt); }

  public void internalFrameDeactivated(InternalFrameEvent _evt)
  { implementation_.internalFrameDeactivated(_evt); }

  public void internalFrameClosed(InternalFrameEvent _evt)
  { implementation_.internalFrameClosed(_evt); }

  public void internalFrameClosing(InternalFrameEvent _evt)
  { implementation_.internalFrameClosing(_evt); }

  public void internalFrameDeiconified(InternalFrameEvent _evt)
  { implementation_.internalFrameDeiconified(_evt); }

  public void internalFrameIconified(InternalFrameEvent _evt)
  { implementation_.internalFrameIconified(_evt); }

  public void internalFrameOpened(InternalFrameEvent _evt)
  { implementation_.internalFrameOpened(_evt); }

  // Anti-aliasing
  /*
  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }
  */

  // Main

  public static void main(String[] _args)
  {
    BuApplication app=new BuApplication();
    app.init();
    app.start();
  }
}
