/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuBackgroundPainter.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

/**
 * An utility class to paint the background of a component.
 * Accepts texture and gradient.
 */
public class BuBackgroundPainter
       extends Object
{

  public BuBackgroundPainter()
  { this(null,false); }

  /*
  public BuBackgroundPainter(Icon _icon)
  { this(_icon,false); }

  public BuBackgroundPainter(boolean _gradient)
  { this(null,_gradient); }
  */

  public BuBackgroundPainter(Icon _icon, boolean _gradient)
  {
    super();
    icon_=_icon;
    gradient_=_gradient;
  }

  public void paintBackground(JComponent _c, Graphics _g)
  {
    Rectangle clip=_g.getClipBounds();
    Dimension dc  =_c.getSize();
    Icon      icon=getIcon();

    String ui=_c.getUIClassID();
    ui=ui.substring(0,ui.length()-2);
    if(ui.equals("DesktopPane")) ui="Desktop";
    ui+=".texture";
    Image image=(Image)UIManager.get(ui);
    //System.err.println("$$$ UI="+ui);
    //System.err.println("$$$ image="+image);
    //System.err.println("$$$ icon="+icon);
    if((image!=null)&&(icon==null))
      icon=new ImageIcon(image);

    if(icon!=null)
    {
      Point p,q;
      // component must be showing on the screen to determine its location
      try
      {
	p=_c.getLocationOnScreen();
	q=_c.getTopLevelAncestor().getLocationOnScreen();
	p.x-=q.x;
	p.y-=q.y;
      }
      catch(IllegalComponentStateException ex) { p=new Point(0,0); }

      int wi=icon.getIconWidth();
      int hi=icon.getIconHeight();

      int i,j,ni,nj,x,y;

      ni=clip.width/wi+2;
      nj=clip.height/hi+2;
      x =clip.x/wi*wi-(p.x%wi);
      y =clip.y/hi*hi-(p.y%hi);

      for(i=0;i<ni;i++)
	for(j=0;j<nj;j++)
	  icon.paintIcon(_c,_g,x+i*wi,y+j*hi);
    }
    else
    if(isGradient())
    {
      Color c1=_c.getBackground();
      //BuLib.getColor(_c.getBackground(),true);
      int   r1=c1.getRed();
      int   v1=c1.getGreen();
      int   b1=c1.getBlue();

      Color c2=c1.brighter().brighter();
      //BuLib.getColor(c1.brighter().brighter(),true);
      int   r2=c2.getRed();
      int   v2=c2.getGreen();
      int   b2=c2.getBlue();

      int y0=clip.y;
      int y1=Math.min(clip.y+clip.height,dc.height-128);
      int y2=clip.y+clip.height;

      int x0=clip.x;
      int x2=clip.x+clip.width;

      _g.setColor(c1);

      if(c1.equals(c2))
      {
        _g.fillRect(x0,y0,x2,y2);
      }
      else
      {
        _g.fillRect(x0,y0,x2,y1);

        for(int y=y1;y<y2;y++)
        {
	  int r=r1+(r2-r1)*(y-dc.height+128)/128;
	  int v=v1+(v2-v1)*(y-dc.height+128)/128;
	  int b=b1+(b2-b1)*(y-dc.height+128)/128;
	  _g.setColor(new Color(r,v,b));
	  _g.drawLine(x0,y,x2,y);
	}
      }
    }
    else
    {
      Color c1=_c.getBackground();
      if(c1!=UIManager.getColor(_c.getUIClassID()+".background"))
      {
	_g.setColor(c1);
	_g.fillRect(clip.x,clip.y,clip.width,clip.height);
      }
    }

    if(isGrid())
    {
      int wi=37;
      int hi=37;

      int i,j,ni,nj,x,y;

      ni=clip.width/wi+2;
      nj=clip.height/hi+2;
      x =clip.x/wi*wi;
      y =clip.y/hi*hi;

      _g.setColor(_c.getBackground());
      for(i=0;i<ni;i++)
	for(j=0;j<nj;j++)
	{
	  int xg=x+i*wi-1;
	  int yg=y+j*hi-1;
	  _g.draw3DRect(xg,yg,wi-1,hi-1,false);
	}
    }

    if(isDots())
    {
      int wi=37;
      int hi=37;

      int i,j,ni,nj,x,y;

      ni=clip.width/wi+2;
      nj=clip.height/hi+2;
      x =clip.x/wi*wi;
      y =clip.y/hi*hi;

      _g.setColor(_c.getBackground());
      for(i=0;i<ni;i++)
	for(j=0;j<nj;j++)
	{
	  int xg=x+i*wi-2;
	  int yg=y+j*hi-2;
	  _g.draw3DRect(xg,yg,1,1,false);
	}
    }
  }

  // Proprietes

  private Icon icon_;
  public Icon getIcon() { return icon_; }
  public void setIcon(Icon _icon) { icon_=_icon; }

  private boolean gradient_;
  public boolean isGradient() { return gradient_; }
  public void setGradient(boolean _gradient) { gradient_=_gradient; }

  private boolean grid_;
  public boolean isGrid() { return grid_; }
  public void setGrid(boolean _grid) { grid_=_grid; }

  private boolean dots_;
  public boolean isDots() { return dots_; }
  public void setDots(boolean _dots) { dots_=_dots; }
}
