/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuBorderLayout.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.io.*;
import java.util.*;

/**
 * A better BorderLayout.
 * @see java.awt.BorderLayout
 */
public class BuBorderLayout
       implements LayoutManager2, Serializable
{
  private Hashtable table_;

  public static final String NORTH ="North";
  public static final String SOUTH ="South";
  public static final String EAST  ="East";
  public static final String WEST  ="West";
  public static final String CENTER="Center";

  // private static final long serialVersionUID = -8658291919501921765L;

  public BuBorderLayout()
  {
    this(0,0);
  }

  public BuBorderLayout(int _hgap, int _vgap)
  {
    hgap_=_hgap;
    vgap_=_vgap;
    table_=new Hashtable(5);
  }

  private int hgap_;
  public int getHgap() { return hgap_; }
  public void setHgap(int _hgap) { hgap_=_hgap; }

  private int vgap_;
  public int getVgap() { return vgap_; }
  public void setVgap(int _vgap) { vgap_=_vgap; }

  public void addLayoutComponent(Component _comp, Object _constraints)
  {
    synchronized(_comp.getTreeLock())
    {
      if((_constraints==null)||(_constraints instanceof String))
	addLayoutComponent2((String)_constraints,_comp);
      else
	throw new IllegalArgumentException
	  ("Can not add to layout: constraint must be a string (or null)");
    }
  }

  public void addLayoutComponent(String _name, Component _comp)
  {
    if(_name==null) _name=CENTER;

    System.err.println("@deprecated add(String,Component)");
    System.err.print(" use add(Component,BuBorderLayout."+
		       _name.toUpperCase()+" instead at ");
    StringWriter out=new StringWriter();
    new RuntimeException("").printStackTrace(new PrintWriter(out));
    String s=out.getBuffer().toString();
    while(s.indexOf("BuBorderLayout")>0)
      s=s.substring(s.indexOf('\n')+1);
    while(s.indexOf("Container")>0)
      s=s.substring(s.indexOf('\n')+1);
    s=s.substring(s.indexOf('(')+1,s.indexOf(')'));
    System.err.println(s);

    addLayoutComponent2(_name,_comp);
  }

  private void addLayoutComponent2(String _name, Component _comp)
  {
    if(_name==null) _name=CENTER;

    synchronized(_comp.getTreeLock())
    {
      if(CENTER.equals(_name)||NORTH.equals(_name)||
	 SOUTH.equals(_name)||EAST.equals(_name)||WEST.equals(_name))
      {
	// table_.put(name,comp);
	table_.put(_comp,_name);
      }
      else
	throw new IllegalArgumentException
	  ("Can not add to layout: unknown constraint: " + _name);
    }
  }

  public void removeLayoutComponent(Component _comp)
  {
    synchronized(_comp.getTreeLock())
    {
      // Object name=table_.get(_comp);
      // if(name!=null)
      // {
	table_.remove(_comp);
      //   table_.remove(name);
      // }
    }
  }

  private Dimension getMinimumSize(Component _c)
  {
    Dimension r=_c.getMinimumSize();
    if(r==null) r=new Dimension(0,0);
    return r;
  }

  private Dimension getPreferredSize(Component _c)
  {
    Dimension r=_c.getPreferredSize();
    if(r==null) r=new Dimension(0,0);
    return r;
  }

  private void adjustSize(Dimension _r, Dimension _d, String _s)
  {
    if(CENTER.equals(_s))
    {
      _r.width +=_d.width;
      _r.height+=_d.height;
    }
    else
    if(EAST.equals(_s))
    {
      _r.width +=_d.width+hgap_;
      _r.height =Math.max(_d.height,_r.height);
    }
    else
    if(WEST.equals(_s))
    {
      _r.width +=_d.width + hgap_;
      _r.height =Math.max(_d.height,_r.height);
    }
    else
    if(NORTH.equals(_s))
    {
      _r.width  =Math.max(_d.width,_r.width);
      _r.height+=_d.height+vgap_;
    }
    else
    if(SOUTH.equals(_s))
    {
      _r.width  =Math.max(_d.width,_r.width);
      _r.height+=_d.height+vgap_;
    }
  }

  public Dimension minimumLayoutSize(Container _parent)
  {
    synchronized(_parent.getTreeLock())
    {
      Dimension r=new Dimension(0,0);
      int nbc=_parent.getComponentCount();

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	String    s=(String)table_.get(c);

	if(c.isVisible())
	{
	  if(s==null) s=CENTER;

	  if(CENTER.equals(s))
	    adjustSize(r,getMinimumSize(c),s);
	}
      }

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	String    s=(String)table_.get(c);

	if(c.isVisible())
	{
	  if(s==null) s=CENTER;

	  if(!CENTER.equals(s))
	    adjustSize(r,getMinimumSize(c),s);
	}
      }

      Insets insets=_parent.getInsets();
      r.width +=insets.left+insets.right;
      r.height+=insets.top+insets.bottom;
      
      return r;
    }
  }
    
  public Dimension preferredLayoutSize(Container _parent)
  {
    synchronized(_parent.getTreeLock())
    {
      Dimension r=new Dimension(0,0);
      int nbc=_parent.getComponentCount();
      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	String    s=(String)table_.get(c);

	if(c.isVisible())
	{
	  if(s==null) s=CENTER;

	  if(CENTER.equals(s))
	    adjustSize(r,getPreferredSize(c),s);
	}
      }

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	String    s=(String)table_.get(c);

	if(c.isVisible())
	{
	  if(s==null) s=CENTER;

	  if(!CENTER.equals(s))
	    adjustSize(r,getPreferredSize(c),s);
	}
      }

      Insets insets=_parent.getInsets();
      r.width +=insets.left+insets.right;
      r.height+=insets.top+insets.bottom;
      
      return r;
    }
  }

  public Dimension maximumLayoutSize(Container _parent)
  {
    return new Dimension(Integer.MAX_VALUE,Integer.MAX_VALUE);
  }

  public float getLayoutAlignmentX(Container _parent)
    { return 0.5f; }

  public float getLayoutAlignmentY(Container _parent)
    { return 0.5f; }

  public void invalidateLayout(Container _parent)
    { }
				      
  public void layoutContainer(Container _parent)
  {
    synchronized(_parent.getTreeLock())
    {
      Insets insets=_parent.getInsets();

      int top   =insets.top;
      int bottom=_parent.getSize().height-insets.bottom;
      int left  =insets.left;
      int right =_parent.getSize().width-insets.right;

      int nbc=_parent.getComponentCount();
      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	String    s=(String)table_.get(c);

	if(c.isVisible())
	{
	  Dimension d=getPreferredSize(c);

	  if(s==null) s=CENTER;

	  if(NORTH.equals(s))
	  {
	    c.setSize(right-left,d.height);
	    d=getPreferredSize(c);
	    c.setBounds(left,top,right-left,d.height);
	    top+=d.height+vgap_;
	  }
	  else
	  if(SOUTH.equals(s))
	  {
	    c.setSize(right-left,d.height);
	    d=getPreferredSize(c);
	    c.setBounds(left,bottom-d.height,right-left,d.height);
	    bottom-=d.height+vgap_;
	  }
	}
      }

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	String    s=(String)table_.get(c);

	if(c.isVisible())
	{
	  Dimension d=getPreferredSize(c);

	  if(s==null) s=CENTER;

	  if(EAST.equals(s))
	  {
	    c.setSize(d.width,bottom-top);
	    d=getPreferredSize(c);
	    c.setBounds(right-d.width,top,d.width,bottom-top);
	    right-=d.width+hgap_;
	  }
	  else
	  if(WEST.equals(s))
	  {
	    c.setSize(d.width,bottom-top);
	    d=getPreferredSize(c);
	    c.setBounds(left,top,d.width,bottom-top);
	    left+=d.width+hgap_;
	  }
	}
      }

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	String    s=(String)table_.get(c);

	if(c.isVisible())
	{
	  if(s==null) s=CENTER;

	  if(CENTER.equals(s))
	    c.setBounds(left,top,right-left,bottom-top);
	}
      }
    }
  }

  public String toString()
  {
    return getClass().getName()+"[hgap="+hgap_+",vgap="+vgap_+"]";
  }
}
