/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuBrowserFrame.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.net.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*;

/**
 * A tiny HTML browser.
 * Is an internal frame on the desktop and an alternative
 * to an external browser as lynx ;-)
 */

public class BuBrowserFrame
       extends BuInternalFrame 
       implements ActionListener, BuCutCopyPasteInterface
{
  private BuCommonImplementation app_;
  private JComponent             content_;
  private BuLabel                lbUrl_;
  private BuLabel                lbLink_;
  private XHtmlPane              pane_;
  private BuButton               btAvancer_;
  private BuButton               btReculer_;
  private BuButton               btRafraichir_;
  private BuButton               btMaison_;

  public BuBrowserFrame()
  { this(null); }

  public BuBrowserFrame(BuCommonImplementation _app)
  { this(_app,null); }

  public BuBrowserFrame(BuCommonImplementation _app, String _source)
  {
    super("",true,true,true,true);
    app_=_app;

    setName("ifNAVIGATEUR");
    if(app_!=null)
      app_.installContextHelp(getRootPane(),"bu/p-navigateur.html");

    content_=(JComponent)getContentPane();
    content_.setBorder(new EmptyBorder(5,5,5,5));

    lbUrl_=new BuLabel("-");
    lbUrl_.setFont(BuLib.deriveFont("Label",Font.PLAIN,-2));
    lbUrl_.setBorder(new EmptyBorder(2,2,2,2));
    content_.add(lbUrl_,BuBorderLayout.NORTH);

    lbLink_=new BuLabel("-");
    lbLink_.setFont(BuLib.deriveFont("Label",Font.PLAIN,-2));
    lbLink_.setBorder(new EmptyBorder(2,2,2,2));
    content_.add(lbLink_,BuBorderLayout.SOUTH);

    pane_=new XHtmlPane();
    content_.add(pane_,BuBorderLayout.CENTER);

    btAvancer_=new BuButton();
    btAvancer_.setActionCommand("AVANCER");
    btAvancer_.setToolTipText
      (BuResource.BU.getString("Avancer  la page suivante"));
    btAvancer_.setIcon(BuResource.BU.getIcon("avancer"));
    btAvancer_.setName("btAVANCER");
    btAvancer_.setRequestFocusEnabled(false);
    btAvancer_.addActionListener(this);

    btReculer_=new BuButton();
    btReculer_.setActionCommand("RECULER");
    btReculer_.setToolTipText
      (BuResource.BU.getString("Revenir  la page prcdente"));
    btReculer_.setIcon(BuResource.BU.getIcon("reculer"));
    btReculer_.setName("btRECULER");
    btReculer_.setRequestFocusEnabled(false);
    btReculer_.addActionListener(this);

    btRafraichir_=new BuButton();
    btRafraichir_.setActionCommand("RAFRAICHIR");
    btRafraichir_.setToolTipText
      (BuResource.BU.getString("Rafrachir la page"));
    btRafraichir_.setIcon(BuResource.BU.getIcon("rafraichir"));
    btRafraichir_.setName("btRAFRAICHIR");
    btRafraichir_.setRequestFocusEnabled(false);
    btRafraichir_.addActionListener(this);

    btMaison_=new BuButton();
    btMaison_.setActionCommand("MAISON");
    btMaison_.setToolTipText
      (BuResource.BU.getString("Aller  la page principale"));
    btMaison_.setIcon(BuResource.BU.getIcon("www"));
    btMaison_.setName("btMAISON");
    btMaison_.setRequestFocusEnabled(false);
    btMaison_.addActionListener(this);

    btAvancer_.setEnabled(false);
    btReculer_.setEnabled(false);
    //if(app_==null) btMaison_.setEnabled(false);

    /*
    JRootPane rootpane=getRootPane();
    KeyStroke ksAvancer=KeyStroke.getKeyStroke
      (KeyEvent.VK_RIGHT,KeyEvent.CTRL_MASK|KeyEvent.ALT_MASK,false);
    KeyStroke ksReculer=KeyStroke.getKeyStroke
      (KeyEvent.VK_LEFT ,KeyEvent.CTRL_MASK|KeyEvent.ALT_MASK,false);
    ActionListener alAvancer=new ActionListener()
    {
      public void actionPerformed(ActionEvent _evt)
      {
	BuBrowserFrame.this.actionPerformed
	  (new ActionEvent(BuBrowserFrame.this,
			   ActionEvent.ACTION_PERFORMED,"AVANCER"));
      }
    };
    ActionListener alReculer=new ActionListener()
    {
      public void actionPerformed(ActionEvent _evt)
      {
	BuBrowserFrame.this.actionPerformed
	  (new ActionEvent(BuBrowserFrame.this,
			   ActionEvent.ACTION_PERFORMED,"RECULER"));
      }
    };
    rootpane.unregisterKeyboardAction(ksAvancer);
    rootpane.unregisterKeyboardAction(ksReculer);
    rootpane.registerKeyboardAction
      (alAvancer,ksAvancer,JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    rootpane.registerKeyboardAction
      (alReculer,ksReculer,JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    */

    if(_source!=null) setHtmlSource(_source);
    
    setTitle(BuResource.BU.getString("Navigateur"));
    setFrameIcon(BuResource.BU.getBarIcon("www"));
    
    Dimension ps=new Dimension(400,500);
    setSize(ps);
    setPreferredSize(ps);
  }

  public void setSelected(boolean _state) throws PropertyVetoException
  {
    super.setSelected(_state);
    if(isSelected()) pane_.requestFocus();
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
    //System.err.println("BuBrowserFrame : "+action);

    if("RECULER".equals(action))
    {
      pane_.back();
    }
    else
    if("AVANCER".equals(action))
    {
      pane_.forward();
    }
    else
    if("RAFRAICHIR".equals(action))
    {
      pane_.reload();
    }
    else
    if("MAISON".equals(action))
    {
      if(app_!=null)
        setDocumentUrl(app_.getInformationsSoftware().baseManUrl());
      else
	setHtmlSource(INTRO,"Introduction");
    }
  }

  public void setDocumentUrl(URL _url)
  {
    setDocumentUrl(""+_url);
  }

  public void setDocumentUrl(String _s)
  {
    try
    {
      if(_s.startsWith("mailto:")) throw new MalformedURLException(); 
      if(_s.startsWith("https:")) throw new MalformedURLException(); 
      URL url=new URL(_s);
      pane_.linkActivated(url);
    }
    catch (MalformedURLException ex)
    {
      setHtmlSource
	  ("<HTML><BODY><P>This URL can not be displayed:</P><SMALL><TT>"+
	   _s+" </TT></SMALL><P>If it is correct, please use an external"+
	   " browser.</P></BODY></HTML>","Error");
      // System.err.println("Malformed URL: " + ex);
    }
  }

  public String getHtmlSource()
    { return pane_.getHtmlSource(); }

  public void setHtmlSource(String _source)
    { setHtmlSource(_source,null); }

  public void setHtmlSource(String _source, String _titre)
  {
    if((_titre==null)||("".equals(_titre))) _titre=" ";
    pane_.setHtmlSource(_source,_titre);
  }

  /**
   * @deprecated
   */
  public JEditorPane getEditorPane()
    { return pane_.getEditorPane(); }

  class XHtmlPane extends BuScrollPane
                  implements HyperlinkListener
  {
    private BuEditorPane html_;
    private Cursor       cursor_;
    private Vector       avant_,apres_;

    public XHtmlPane()
    {
      html_=new BuEditorPane();
      html_.setEditable(false);
      html_.addHyperlinkListener(this);
      cursor_=html_.getCursor();
      avant_ =new Vector();
      apres_ =new Vector();

      JViewport vp=getViewport();
      vp.add(html_);
    }

    public JEditorPane getEditorPane()
      { return html_; }

    public void back()
    {
      int l=avant_.size();
      if(l>1)
      {
	URL url=(URL)avant_.elementAt(l-2);
	URL cur=(URL)avant_.elementAt(l-1);
	System.err.println("BACK "+url);
	avant_.removeElementAt(l-1);
	apres_.insertElementAt(cur,0);
        linkActivated(url);
      }
    } 

    public void forward()
    {
      if(apres_.size()>0)
      {
	int l=avant_.size();
	URL url=(URL)apres_.elementAt(0);
	URL cur=(URL)avant_.elementAt(l-1);
	System.err.println("FORWARD "+url);
	apres_.removeElementAt(0);
        linkActivated(url);
      }
    }

    public void reload()
    {
      int l=avant_.size();
      if(l>0)
      {
	URL url=(URL)avant_.elementAt(l-1);
	System.err.println("RELOAD "+url);
	avant_.removeElementAt(l-1);
	setHtmlSource("Reload "+url,"Reload");
        linkActivated(url);
      }
    } 

    public void hyperlinkUpdate(HyperlinkEvent _evt)
    {
      if(_evt.getEventType()==HyperlinkEvent.EventType.ACTIVATED)
      {
	linkActivated(_evt.getURL());
      }
      else
      if(_evt.getEventType()==HyperlinkEvent.EventType.ENTERED)
      {
	lbLink_.setText(""+_evt.getURL());
      }
      else
      if(_evt.getEventType()==HyperlinkEvent.EventType.EXITED)
      {
	lbLink_.setText("-");
      }
    }

    private Runnable current_=null;

    public void linkActivated(final URL _url)
    {
      try
      {
        String f=_url.getFile();
        if(f==null) f="";
	f=f.toLowerCase();
	if(f.endsWith(".gif")||f.endsWith(".jpg")||f.endsWith(".jpeg"))
	{
	  setHtmlSource("<IMG SRC=\""+_url+"\">",f);
	}
	else
        {
	  String h=_url.getHost();
	  if((h==null)||"".equals(h)) h="localhost";
	  lbUrl_.setText("Connexion: "+h+" ...");
	  /*
	    new BuTaskOperation
	    (app_,getTitle(),
	      new XPageLoader(_url)).start();
	  */
	  Runnable runnable=new XPageLoader(_url);

	  if(!SwingUtilities.isEventDispatchThread())
            SwingUtilities.invokeLater(runnable);
	  else
	    runnable.run();
        }
      }
      catch(Exception ex)
      {
	lbUrl_.setText("Bad URL: "+_url);
	BuBrowserFrame.this.getToolkit().beep();
      }
    }

    public String getHtmlSource()
    {
      return html_.getText();
    }

    public final void setHtmlSource(String _source,String _titre)
    {
      setHtmlSource0(_source,_titre);
    }

    public void setHtmlSource0(final String _source,final String _titre)
    {
      Runnable runnable=new Runnable()
      {
        public void run()
        {
          lbUrl_.setText(_titre);

          try
          {
	    EditorKit ek=html_.getEditorKitForContentType("text/html");
	    Document  dk=ek.createDefaultDocument();
	    ek.read(new StringReader(_source),dk,0);
	    html_.setEditorKit(ek);
	    html_.setDocument(dk);
	  }
	  catch(Exception ex)
	  {
	    html_.setText("Couldn't understand html source.\n\n"+_source);
	    ex.printStackTrace();
	  }
	}
      };

      if(!SwingUtilities.isEventDispatchThread())
        SwingUtilities.invokeLater(runnable);
      else
	runnable.run();
    }

    class XPageLoader implements Runnable
    {
      private URL url_;

      XPageLoader(URL _url)
      {
	url_=_url;
      }

      public void run()
      {
	boolean encore=true;

	while((current_!=null)&&(current_!=this))
	{
	  try { Thread.sleep(400); }
	  catch(Exception ex) { }
	}
	  
	current_=this;
	html_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

	while(encore)
	{
	  if(url_==null)
	  {
	    encore=false;
	  }
	  else
	  {
	    Document doc=html_.getDocument();
	    try 
	    {
	      html_.setPage(url_);
	      avant_.addElement(url_);
	      btAvancer_.setEnabled(apres_.size()>0);
	      btReculer_.setEnabled(avant_.size()>1);

	      encore=false;
	    }
	    catch (Exception ex)
	    {
	      System.err.println("%%% "+ex);
	      html_.setDocument(doc);
	      lbUrl_.setText("Bad URL: "+url_);
	      BuBrowserFrame.this.getToolkit().beep();
	      encore=true;
	    }
	    finally
	    {
	      lbUrl_.setText("Page: "+url_.getFile());
	      url_=null;
	    }
	  }

	  Container parent=html_.getParent();
	  parent.repaint();
	}

	if(current_==this)
	{
	  html_.setCursor(cursor_);
	  current_=null;
	}
      }
    }
  }

  // BuInternalFrame

  public String[] getEnabledActions()
  {
    String[] r=new String[] { "COPIER" };
    return r;
  }

  public JComponent[] getSpecificTools()
  {
    JComponent[] r=new JComponent[4];
    r[0]=btReculer_;
    r[1]=btAvancer_;
    r[2]=btRafraichir_;
    r[3]=btMaison_;
    return r;
  }

  // BuCutCopyPasteInterface

  public void cut      () { }
  public void copy     () { pane_.getEditorPane().copy(); }
  public void paste    () { }
  public void duplicate() { }

  // Test

  private final static String INTRO=
      "<HTML><BODY><H1>Bienvenue ;-)</H1>"+
      "<P>Liens<UL>"+
      //"<LI><A HREF=\"http://localhost/\"      >Local host</A></LI>"+
      "<LI><A HREF=\"http://www.memoire.com/\">Memoire.com</A></LI>"+
      "<LI><A HREF=\"http://www.debian.org/\" >Debian</A></LI>"+
      //"<LI><A HREF=\"file:///home/desnoix/conges.gif\">Test GIF</A></LI>"+
      //"<LI><A HREF=\"file:///home/desnoix/DELL.txt\">Test TXT</A></LI>"+
      "</UL></P></BODY></HTML>";

  public static void main(String[] _args)
  {
    JFrame         f=new JFrame("Browser");
    BuBrowserFrame b=new BuBrowserFrame();
    BuPanel        p=new BuPanel();
    JComponent[]   c=b.getSpecificTools();
    JComponent     x=(JComponent)b.getContentPane();

    x.setPreferredSize(new Dimension(690,650));
    p.setLayout(new BuHorizontalLayout());
    for(int i=0;i<c.length;i++) p.add(c[i]);
    f.getContentPane().add("Center",x);
    f.getContentPane().add("North" ,p);
    f.pack();    
    f.show();
    f.setLocation(200,100);

    b.setHtmlSource(INTRO,"Introduction");
  }
}
