/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuBrowserPreferencesPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A panel where the user can choose his favorite browser.
 * @creation     1999-01-14
 * @version      0.02, 2001-06-18
 * @author       Guillaume Desnoix
 */

/**
 * browser.type
 * browser.exec
 */

public class BuBrowserPreferencesPanel
       extends BuAbstractPreferencesPanel
       implements ActionListener
{
  BuCommonInterface appli_;
  BuPreferences     options_;

  BuRadioButton     cbInterne_;
  BuRadioButton     cbExterne_;
  BuRadioButton     cbPerso_;
  BuComboBox        chBrowser_;
  BuTextField       tfPerso_;

  public String getTitle()
    { return BuResource.BU.getString("Navigateur"); }

  // Constructeur

  public BuBrowserPreferencesPanel(BuCommonInterface _appli)
  {
    super();
    appli_=_appli;

    options_=BuPreferences.BU;

    BuGridLayout lo=new BuGridLayout();
    lo.setColumns(2);
    lo.setVgap(5);
    lo.setHgap(5);

    BuPanel p=new BuPanel();
    p.setBorder
      (new CompoundBorder
       (new TitledBorder(BuResource.BU.getString("Navigateur")),
	new EmptyBorder(5,5,5,5)));
    p.setLayout(lo);

    ButtonGroup bg=new ButtonGroup();

    cbInterne_=new BuRadioButton(BuResource.BU.getString("interne"  ));
    cbExterne_=new BuRadioButton(BuResource.BU.getString("externe"  ));
    cbPerso_  =new BuRadioButton(BuResource.BU.getString("personnel"));
    cbInterne_.setName("cbBROWSING_INTERNAL" );
    cbExterne_.setName("cbBROWSING_EXTERNAL" );
    cbPerso_  .setName("cbBROWSING_PERSONNAL");
    bg.add(cbInterne_);
    bg.add(cbExterne_);
    bg.add(cbPerso_);

    p.add(cbInterne_);
    cbInterne_.addActionListener(this);
    BuComboBox chInterne=new BuComboBox();
    chInterne.addItem("   ");
    chInterne.setVisible(false);
    p.add(chInterne);

    p.add(cbExterne_);
    cbExterne_.addActionListener(this);

    chBrowser_=new BuComboBox();
    chBrowser_.setName("chBrowser_");

    for(int i=0;i<BuBrowserControl.LIST.length;i++)
    {
      // System.err.println(BuBrowserControl.LIST[i][0]);
      chBrowser_.addItem(BuBrowserControl.LIST[i][0]);
    }

    p.add(chBrowser_);
    chBrowser_.setVisible(BuLib.isUnix());
    // chBrowser_.addActionListener(this);

    p.add(cbPerso_);
    cbPerso_.addActionListener(this);

    tfPerso_=new BuTextField(25);
    tfPerso_.setName("tfBROWSER");
    p.add(tfPerso_);
    // tfPerso_.addActionListener(this);

    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    add(p,BuBorderLayout.CENTER);

    cbInterne_.setSelected(true);
    updateComponents();
  }

  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();
    // System.err.println("SOURCE="+source.getName());
    // System.err.println("ACTION="+_evt.getActionCommand());

    chBrowser_.setEnabled(cbExterne_.isSelected());
    tfPerso_  .setEnabled(cbPerso_  .isSelected());
  }
  
  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
  }

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
    int    type=1;
    String exec="";

    if(cbInterne_.isSelected())
    {
      type=1;
      exec=options_.getStringProperty("browser.exec");
    }
    if(cbExterne_.isSelected())
    {
      type=2;
      exec=(String)chBrowser_.getSelectedItem();
    }
    if(cbPerso_.isSelected())
    {
      type=3;
      exec=tfPerso_.getText();
    }

    options_.putIntegerProperty("browser.type",type);
    options_.putStringProperty ("browser.exec",exec);
  }

  private void updateComponents()
  {
    int    type=options_.getIntegerProperty("browser.type");
    String exec=options_.getStringProperty("browser.exec");

    switch(type)
    {
      case 2:
        cbExterne_.setSelected(true);
        chBrowser_.setSelectedItem(exec);
        break;
      case 3:
	cbPerso_.setSelected(true);
	tfPerso_.setText(exec);
	break;
      default:
	cbInterne_.setSelected(true);
	break;
    }

    chBrowser_.setEnabled(type==2);
    tfPerso_  .setEnabled(type==3);
  }
}
