/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuButton.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.*;

/**
 * Like JButton but with better management of icons.
 */

public class BuButton
       extends JButton
{
  public BuButton()
  {
    this((BuIcon)null,null);
  }

  public BuButton(BuIcon _icon)
  {
    this(_icon,null);
  }

  public BuButton(String _label)
  {
    this(null,_label);
  }

  public BuButton(BuIcon _icon, String _label)
  {
    super();

    setCursor(new Cursor(Cursor.HAND_CURSOR));
    setIcon(_icon);
    setText(_label);
  }

  /**
   * @deprecated
   */
  public BuButton(Icon _icon)
  {
    this(null,_icon);
  }

  /**
   * @deprecated
   */
  public BuButton(String _text,Icon _icon)
  {
    super();
    setCursor(new Cursor(Cursor.HAND_CURSOR));
    setIcon(_icon);
    setText(_text);
  }

  // Anti-aliasing

  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  public boolean isOpaque()
  {
    boolean r=super.isOpaque();

    Container parent=getParent();

    if(parent instanceof JComponent)
    {
      Object p=((JComponent)parent)
	  .getClientProperty("JToolBar.isRollover");
      if(Boolean.TRUE.equals(p))
	  r&=isEnabled()&&getModel().isRollover();
    }

    return r;
  }

  public void setText(String _text)
  {
    if("".equals(_text)) _text=null;
    super.setText(_text);

    if(  !BuPreferences.BU.getBooleanProperty("button.text",true)
       &&(getToolTipText()==null))
      super.setToolTipText(_text);
  }

  public String getText()
  {
    if(  BuPreferences.BU.getBooleanProperty("button.text",true)
       ||(super.getIcon()==null))
      return super.getText();
    return null;
  }

  public void setIcon(BuIcon _icon)
    { BuLib.setIcon(this,_icon); }

  public Icon getIcon()
  {
    if(  BuPreferences.BU.getBooleanProperty("button.icon",true)
       ||(super.getText()==null))
      return super.getIcon();
    return null;
  }

  public Point getToolTipLocation(MouseEvent _evt)
  {
    Point r=super.getToolTipLocation(_evt);
    if(r==null) r=new Point(0,getHeight()+1);
    return r;
  }

  public void updateUI()
  {
    super.updateUI();
    setRolloverEnabled(true);
  }

  public Color getBackground()
  {
    Color r;
    if(isEnabled()&&getModel().isRollover()&&BuLib.isMetal())
      r=new ColorUIResource(192,192,208);
    else
      r=super.getBackground();
    return r;
  }
}
