/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuCharValidator.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


/**
 * An abstract root class to specify a validator for input chars.
 * Provides standard ones for ints, doubles, dates, ...
 */

public abstract class BuCharValidator
{
  public abstract boolean isCharValid(char _char);

  // Instances

  public final static BuCharValidator INTEGER=new BuCharValidator()
  {
    public boolean isCharValid(char _char)
      { return Character.isDigit(_char); }
  };

  public final static BuCharValidator LONG=new BuCharValidator()
  {
    public boolean isCharValid(char _char)
      { return Character.isDigit(_char); }
  };

  public final static BuCharValidator FLOAT=new BuCharValidator()
  {
    public boolean isCharValid(char _char)
    {
      return   Character.isDigit(_char)
	     ||(".,+-Ee".indexOf(_char)>=0);
    }
  };

  public final static BuCharValidator DOUBLE=new BuCharValidator()
  {
    public boolean isCharValid(char _char)
    {
      return   Character.isDigit(_char)
	     ||(".,+-Ee".indexOf(_char)>=0);
    }
  };

  public final static BuCharValidator ID=new BuCharValidator()
  {
    public boolean isCharValid(char _char)
      { return Character.isJavaIdentifierPart(_char); }
  };

  public final static BuCharValidator FILE=new BuCharValidator()
  {
    public boolean isCharValid(char _char)
    {
      return Character.isLetterOrDigit(_char)
	||((System.getProperty("file.separator")+".").indexOf(_char)>=0);
    }
  };

  public final static BuCharValidator DATE=new BuCharValidator()
  {
    public boolean isCharValid(char _char)
    {
      return   Character.isDigit(_char)
	     ||(_char=='/');
    }
  };

  public final static BuCharValidator TIME=new BuCharValidator()
  {
    public boolean isCharValid(char _char)
    {
      return   Character.isDigit(_char)
	     ||(_char==':');
    }
  };
}


