/**
 * @modification $Date: 2001/12/07 18:28:12 $
 * @statut       unstable
 * @file         BuCommonImplementation.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.plaf.metal.*;

/**
 * Main part of the code to manage BuApplets and BuApplications.
 * Use to avoid redundant code.
 */

public class BuCommonImplementation
       implements BuCommonInterface,
                  ContainerListener, WindowListener, InternalFrameListener
{
  protected BuCommonInterface app_;
  protected JComponent        content_;
  protected BuMainPanel       main_panel_;
  protected BuMenuBar         main_menubar_;
  protected BuToolBar         main_toolbar_;
  protected BuSplashScreen    splash_screen_;

  public BuCommonImplementation()
  {
    super();

    app_         =null;
    content_     =null;
    main_panel_  =null;
    main_menubar_=null;
    main_toolbar_=null;
  }

  // Implementation

  public BuCommonImplementation getImplementation()
  { return this; }

  // App (Application ou Applet)

  public BuCommonInterface getApp()
  { return app_; }

  public void setApp(BuCommonInterface _app)
  {
    app_=_app;
  }

  /**
   * Mise en place des elements.
   * (Panneau principal, barres de menu et de boutons).
   */
  public void init()
  {
    BuSplashScreen ss=getSplashScreen();

    if(ss!=null)
    {
      ss.setText("Initialisation de l'application");
      ss.setProgression(20);
    }

    content_=new BuPanel();
    content_.setLayout(new BuBorderLayout());
    setContentPane(content_);

    BuMainPanel mp=new BuMainPanel();
    setMainPanel(mp);

    if(ss!=null)
    {
      ss.setText("Menus");
      ss.setProgression(30);
    }

    BuMenuBar mb=BuMenuBar.buildBasicMenuBar();
    setMainMenuBar(mb);
    mb.computeMnemonics();
    mb.addActionListener(this);

    if(ss!=null)
    {
      ss.setText("Barre d'outils");
      ss.setProgression(40);
    }

    BuToolBar tb=BuToolBar.buildBasicToolBar();
    setMainToolBar(tb);
    tb.addActionListener(this);

    if(ss!=null)
    {
      ss.setText("Liste des fentres");
      ss.setProgression(45);
    }

    BuMenuInternalFrames mw=(BuMenuInternalFrames)mb.getMenu("LISTE_FENETRES");
    if(mw!=null) mw.setDesktop(mp.getDesktop());
  }

  public void start()
  {
    getMainPanel().updateSplits();
    menusOrganisationFenetres();
  }

  public String toString()
  { return "implementation "+getInformationsSoftware().name; }

  // CommonInterface

  public void doLayout()
  { app_.doLayout(); }

  public void validate()
  { app_.validate(); }

  public void setTitle(String _s)
  { app_.setTitle(_s); }

  public BuInformationsSoftware getInformationsSoftware()
  { return new BuInformationsSoftware(); }

  public void setJMenuBar(JMenuBar _mb)
  { app_.setJMenuBar(_mb); }

  public void setContentPane(Container _c)
  { app_.setContentPane(_c); }

  public JRootPane getRootPane()
  { return app_.getRootPane(); }

  public Frame getFrame()
  { return app_.getFrame(); }

  // Main Panel

  public BuMainPanel getMainPanel()
  { return main_panel_; }

  public void setMainPanel(BuMainPanel _p)
  {
    main_panel_=_p; 
    content_.add(main_panel_,BuBorderLayout.CENTER);
    content_.doLayout();
    content_.validate();
    main_panel_.getDesktop().addContainerListener(this);
  }

  // Main MenuBar

  public BuMenuBar getMainMenuBar()
  { return main_menubar_; }

  public void setMainMenuBar(BuMenuBar _mb)
  {
    main_menubar_=_mb; 
    setJMenuBar(main_menubar_);
  }

  // Main ToolBar

  public BuToolBar getMainToolBar()
  { return main_toolbar_; }

  public void setMainToolBar(BuToolBar _tb)
  {
    main_toolbar_=_tb;
    if(!(getApp() instanceof BuSubApp))
    {
      if((content_.getComponentCount()>1)&&(content_.getComponent(1)!=null))
	content_.remove(1);

      if(main_toolbar_!=null)
      {
	content_.add(main_toolbar_,BuBorderLayout.NORTH);
	main_toolbar_.revalidate();
	// content_.doLayout();
	// content_.validate();
      }
    }
  }

  // Specific Bar

  public BuSpecificBar getSpecificBar()
  { return getMainPanel().getSpecificBar(); }

  public void setSpecificBar(BuSpecificBar _sb)
  { getMainPanel().setSpecificBar(_sb); }

  // Splash Screen

  public BuSplashScreen getSplashScreen()
  { return splash_screen_; }

  public void setSplashScreen(BuSplashScreen _ss)
  { splash_screen_=_ss; }

  // Internal Frames

  public void addInternalFrame(final JInternalFrame _f)
  {
    final InternalFrameListener l=this;

    Runnable runnable=new Runnable()
    {
      public void run()
      {
	_f.removeInternalFrameListener(l);
	_f.addInternalFrameListener(l);

	BuMainPanel mp=getMainPanel();
	mp.addInternalFrame(_f);

	menusOrganisationFenetres();
      }
    };

    if(!SwingUtilities.isEventDispatchThread())
      SwingUtilities.invokeLater(runnable);
    else
      runnable.run();
  }

  public void removeInternalFrame(final JInternalFrame _f)
  {
    final InternalFrameListener l=this;

    Runnable runnable=new Runnable()
    {
      public void run()
      {
	BuMainPanel mp=getMainPanel();
	mp.removeInternalFrame(_f);

	_f.removeInternalFrameListener(l);

	menusOrganisationFenetres();
      }
    };

    if(!SwingUtilities.isEventDispatchThread())
      SwingUtilities.invokeLater(runnable);
    else
      runnable.run();
  }

  public void activateInternalFrame(final JInternalFrame _f)
  {
    final InternalFrameListener l=this;

    Runnable runnable=new Runnable()
    {
      public void run()
      {
	BuMainPanel mp=getMainPanel();
	mp.activateInternalFrame(_f);

	menusOrganisationFenetres();
      }
    };

    if(!SwingUtilities.isEventDispatchThread())
      SwingUtilities.invokeLater(runnable);
    else
      runnable.run();
  }

  public JInternalFrame getCurrentInternalFrame()
  {
    return getMainPanel().getCurrentInternalFrame();
  }
  
  public JInternalFrame[] getAllInternalFrames()
  {
    return getMainPanel().getAllInternalFrames();
  }
  
  // Action

  public void action(String _action)
  {
    try
    {
      if(_action!=null)
	actionPerformed(new ActionEvent
			(this,ActionEvent.ACTION_PERFORMED,_action));
    }
    catch(Exception ex) { }
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String                 action=_evt.getActionCommand();
    String                 arg   ="";
    BuMainPanel            mp    =getMainPanel();
    BuInformationsSoftware il    =getInformationsSoftware();

    int i=action.indexOf('(');
    if(i>=0)
    {
      arg   =action.substring(i+1,action.length()-1);
      action=action.substring(0,i);
    }

    // System.err.println("BuImplementation: ACTION="+action); 
    // System.err.println("BuImplementation: ARG   ="+arg); 

         if(action.equals("FERMER"             )) close();
    else if(action.equals("IMPRIMER"           )) print();
    else if(action.equals("PREVISUALISER"      )) preview();
    else if(action.equals("QUITTER"            )) exit();

    else if(action.equals("DEFAIRE"            )) undo();
    else if(action.equals("REFAIRE"            )) redo();

    else if(action.equals("COUPER"             )) cut();
    else if(action.equals("COPIER"             )) copy();
    else if(action.equals("COLLER"             )) paste();
    else if(action.equals("DUPLIQUER"          )) duplicate();

    else if(action.equals("TOUTSELECTIONNER"   )) select();
    else if(action.equals("RECHERCHER"         )) find();
    else if(action.equals("REMPLACER"          )) replace();

    else if(action.equals("PLEINECRAN"         )) fullscreen();
    else if(action.equals("CASCADE"            )) mp.waterfall();
    else if(action.equals("MOSAIQUE"           )) mp.tile();
    else if(action.equals("RANGERICONES"       )) mp.arrangeIcons();
    else if(action.equals("RANGERPALETTES"     )) mp.arrangePalettes();
    else if(action.equals("ECHANGER_COLONNES"  )) mp.swapColumns();

    else if(action.equals("ASPECT_AMIGA"       )) setLookAndFeel("swing.addon.plaf.threeD.ThreeDLookAndFeel");
 // else if(action.equals("ASPECT_BASIC"       )) setLookAndFeel("BuBasicLookAndFeel");
    else if(action.equals("ASPECT_KUNSTSTOFF"  )) setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    else if(action.equals("ASPECT_MAC"         )) setLookAndFeel("com.sun.java.swing.plaf.mac.MacLookAndFeel");
    else if(action.equals("ASPECT_METAL"       )) setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
    else if(action.equals("ASPECT_MOTIF"       )) setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
 // else if(action.equals("ASPECT_MULTI"       )) setLookAndFeel("javax.swing.plaf.multi.MultiLookAndFeel");
    else if(action.equals("ASPECT_NEXT"        )) setLookAndFeel("nextlf.plaf.NextLookAndFeel");
    else if(action.equals("ASPECT_ORGANIC"     )) setLookAndFeel("javax.swing.plaf.organic.OrganicLookAndFeel");
    else if(action.equals("ASPECT_OYOAHA"      )) setLookAndFeel("com.oyoaha.swing.plaf.oyoaha.OyoahaLookAndFeel");
    else if(action.equals("ASPECT_SKINLF"      )) setLookAndFeel("com.l2fprod.gui.plaf.skin.SkinLookAndFeel");
    else if(action.equals("ASPECT_SLAF"        )) setLookAndFeel("com.memoire.slaf.SlafLookAndFeel");
    else if(action.equals("ASPECT_WINDOWS"     )) setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");

    else if(action.startsWith("THEME_METAL"    )) setMetalTheme(arg);
    else if(action.startsWith("THEME_SLAF"     )) setSlafTheme (arg);

    else if(action.equals("VISIBLE_SPECIFICBAR")) showOrHide(action,getSpecificBar());
    else if(action.equals("VISIBLE_LEFTCOLUMN" )) showOrHide(action,mp.getLeftColumn());
    else if(action.equals("VISIBLE_RIGHTCOLUMN")) showOrHide(action,mp.getRightColumn());
    else if(action.equals("VISIBLE_STATUSBAR"  )) showOrHide(action,mp.getStatusBar());

    else if(action.equals("AIDE"               )) displayURL(null);
    else if(action.equals("AIDE_INDEX"         )) displayURL(il.baseManUrl());
    else if(action.equals("INDEX_ALPHA"        )) displayURL(il.baseManUrl()+"alphabetique.html");
    else if(action.equals("INDEX_THEMA"        )) displayURL(il.baseManUrl()+"thematique.html");
    else if(action.equals("TEXTE_LICENCE"      )) license();

    else if(action.equals("WWW_ACCUEIL"        )) displayURL(il.http);
    // else if(action.equals("FTP_ACCUEIL"        )) displayURL(il.ftp);

    else if(action.startsWith("AIDE_CONTEXTUELLE")) contextHelp(arg);
    else if(action.equals("POINTEURAIDE"         )) pointerHelp();
    else if(action.equals("APROPOSDE"            )) about();

    else if(action.equals("DEBUTER"    )) ;
    else if(action.equals("QUITTER"    )) exit();
    else if(action.startsWith("FILLE_" )) daughter(action.substring(6),arg);

    else System.err.println("UNKNOWN ACTION: "+action);
  }

  public void showOrHide(String _a, JComponent _c)
  {
    getMainPanel().showOrHide(_c);
    setCheckedForAction(_a,_c.isVisible());
  }

  public void removeAction(String _cmd)
  {
    BuActionRemover.removeAction(getMainMenuBar(),_cmd);
    BuActionRemover.removeAction(getMainToolBar(),_cmd);
    //BuActionRemover.removeAction(getSpecificBar(),_cmd);
  }

  public void setEnabledForAction(String _cmd, boolean _enabled)
  {
    BuActionEnabler.setEnabledForAction(getMainMenuBar(),_cmd,_enabled);
    BuActionEnabler.setEnabledForAction(getMainToolBar(),_cmd,_enabled);
    //BuActionEnabler.setEnabledForAction(getSpecificBar(),_cmd,_enabled);
  }

  public void setCheckedForAction(String _cmd, boolean _checked)
  {
    BuActionChecker.setCheckedForAction(getMainMenuBar(),_cmd,_checked);
    BuActionChecker.setCheckedForAction(getMainToolBar(),_cmd,_checked);
    //BuActionChecker.setCheckedForAction(getSpecificBar(),_cmd,_checked);
  }

  // Divers

  public void daughter(String _action, String _name)
  {
    JInternalFrame[] frames=getMainPanel().getAllInternalFrames();

    JInternalFrame f=null;

    for(int i=0; i<frames.length; i++)
    {
      String n=frames[i].getName();
      if(  _name.equals(n)
	 ||("if"+_name).equals(n)
	 ||_name.equals("*")
	 ||_name.equals(frames[i].getTitle()))
      {
	f=frames[i];
	try
	{
	  if("ACTIVER".equals(_action))
	  {
	    if(f.isIcon()) f.setIcon(false);
	    if(!f.isSelected()) f.setSelected(true);
	  }
	  else if("FERMER".equals(_action))
	  {
	    if(!f.isClosed()) f.setClosed(true);
	  }
	  else if("ICONIFIER"  .equals(_action))
	  {
	    if(!f.isIcon()) f.setIcon(true);
	  }
	  else if("DEICONIFIER".equals(_action))
	  {
	    if(f.isIcon()) f.setIcon(false);
	  }
	  else System.err.println("UNKNOWN SUB-ACTION: "+_action);
	}
	catch(PropertyVetoException ex) { }
      }
    }

    if(f==null) System.err.println("UNKNOWN FRAME: "+_name);
  }

  private BuGlassPaneHelp glasspane_=null;

  public void pointerHelp()
  {
    BuCommonInterface app=getApp();
    if(app instanceof BuApplication)
    {
      BuApplication appli=(BuApplication)app;

      if(glasspane_==null)
      {
	glasspane_=new BuGlassPaneHelp(appli);
	glasspane_.setVisible(false);
      }

      if(appli.getGlassPane()!=glasspane_)
	appli.setGlassPane(glasspane_);

      // System.err.println("SET-GLASSPANE");
      glasspane_.setVisible(true);
      appli.invalidate();
      appli.doLayout();
      appli.validate();
      glasspane_.repaint();
    }
  }

  public void contextHelp(String _url)
  {
    /*
    if(_url.startsWith("AIDE_CONTEXTUELLE"))
      _url=_url.substring(_url.indexOf('(')+1,
			  _url.indexOf(')'));
    */

    try
    {
      URL    url=new URL(new URL(getInformationsSoftware().man),_url);
      String ll =Locale.getDefault().getLanguage();

      _url=url.toString();
      if(!ll.equals("fr"))
      {
	ll="en";
	int i=_url.lastIndexOf(".html");
	if(i>=0)
	  _url=_url.substring(0,i)+"-"+ll+_url.substring(i);
      }
      System.err.println("URL = "+_url);
      displayURL(_url);
    }
    catch (Exception ex) { }
  }

  public void contextHelp(JComponent _c)
  {
    if(_c instanceof JInternalFrame)
      _c=((JInternalFrame)_c).getRootPane();

    JComponent cp=_c;
    String param=(String)cp.getClientProperty("HELP_URL");

    Container pt=cp;
    while((pt!=null)&&(param==null))
    {
      pt=pt.getParent();
      if(pt instanceof JComponent)
	param=(String)((JComponent)pt).getClientProperty("HELP_URL");
    }

    JComponent focus=_c;
    if(focus!=null)
    {
      String ancre=focus.getName();
      if(ancre!=null)
      {
	int i=0;
	while(i<ancre.length())
	{
	  int c=ancre.charAt(i);
	  if((c<'A')||(c>'Z')) i++;
	  else break;
	}
	ancre=ancre.substring(i).toLowerCase().replace('_','-');
	if(!"".equals(ancre)) param+="#"+ancre;
      }

      /*
	else
	System.err.println("NOT NAMED = "+focus);
      */
    }

    System.err.println("PARAM = "+param);
    contextHelp(param);
  }

  private ActionListener al_=null;
  private KeyStroke      f1_=null;

  public void installContextHelp(JComponent _cp)
  {
    installContextHelp(_cp,null);
  }

  public void installContextHelp(JComponent _cp, String _url)
  {
    if(_url!=null) _cp.putClientProperty("HELP_URL",_url);

    if(al_==null)
    {
      al_=new ActionListener()
      {
	public void actionPerformed(ActionEvent _evt)
	{
	  Object source=_evt.getSource();
	  if(source instanceof JComponent)
	  {
	    JComponent cp=(JComponent)source;
	    // System.err.println("SOURCE = "+cp);

	    String param=(String)cp.getClientProperty("HELP_URL");

	    Container pt=cp;
	    while((pt!=null)&&(param==null))
	    {
	      pt=pt.getParent();
	      if(pt instanceof JComponent)
		param=(String)((JComponent)pt).getClientProperty("HELP_URL");
	    }

	    JComponent focus=null;
	    for(Enumeration e=BuLib.getAllSubComponents(cp).elements();
		e.hasMoreElements(); )
	    {
	      Object o=e.nextElement();
	      if(  (o instanceof JComponent)
		 &&((JComponent)o).hasFocus())
	      {
		focus=(JComponent)o;
		break;
	      }
	    }

	    if(focus!=null)
	    {
	      String ancre=focus.getName();
	      if(ancre!=null)
	      {
		int i=0;
		while(i<ancre.length())
		{
		  int c=ancre.charAt(i);
		  if((c<'A')||(c>'Z')) i++;
		  else break;
		}
		ancre=ancre.substring(i).toLowerCase().replace('_','-');
		if(!"".equals(ancre)) param+="#"+ancre;
	      }
	      else
		System.err.println("NOT NAMED = "+focus);
	    }

	    System.err.println("PARAM = "+param);
	    param="("+param+")";

	    /*
	      if(cp instanceof JInternalFrame)
	      cp=((JInternalFrame)cp).getRootPane();
	    */

	    ActionEvent evt=new ActionEvent
	      (cp,_evt.getID(),
	       "AIDE_CONTEXTUELLE"+param,_evt.getModifiers());
	    getImplementation().actionPerformed(evt);
	  }
	}
      };
    }

    if(f1_==null)
      f1_=KeyStroke.getKeyStroke(KeyEvent.VK_F1,0,false);

    _cp.unregisterKeyboardAction(f1_);
    _cp.registerKeyboardAction
      (al_,f1_,JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
  }

  /**
   * Modification du LookAndFeel.
   */
  public void setLookAndFeel(String _lnf)
  {
    // System.err.println("LNF="+_lnf);

    /*
    if(_lnf.equals(UIManager.getLookAndFeel().getClass().getName()))
    {
      String tm=UIManager.getLookAndFeel().getName()+" est dj install!";

      BuDialogMessage dm=new BuDialogMessage
	(app_,getInformationsSoftware(),tm);
      dm.activate();
      return;
    }
    */

    String os=System.getProperty("os.name");
    if("com.sun.java.swing.plaf.mac.MacLookAndFeel".equals(_lnf))
      System.getProperties().put("os.name","Mac OS");
    if("com.sun.java.swing.plaf.windows.WindowsLookAndFeel".equals(_lnf))
      System.getProperties().put("os.name","Windows");

    boolean error=false;

    try
    {
      UIManager.setLookAndFeel(_lnf);
    }
    catch (Throwable ex)
    {
      error=true;
      // ex.printStackTrace();

      String tm;
      int i;

      tm=_lnf;
      i=tm.lastIndexOf('.');
      if(i>=0) tm=tm.substring(i+1);
      i=tm.lastIndexOf("LookAndFeel");
      if(i>=0) tm=tm.substring(0,i);
      tm=BuResource.BU.getString("L'aspect n'a pu tre trouv")+
	". ["+tm+"]";

      BuDialogMessage dm=new BuDialogMessage
	(app_,getInformationsSoftware(),tm);
      dm.activate();
    }

    System.getProperties().put("os.name",os);

    if(!error)
    {
      try
      {
	SwingUtilities.updateComponentTreeUI((Component)app_);
	getMainPanel().arrangeIcons();
      }
      catch (Throwable ex)
      {
	// ex.printStackTrace();
      
	String tm;
	int i;
	
	tm=_lnf;
	i=tm.lastIndexOf('.');
	if(i>=0) tm=tm.substring(i+1);
	i=tm.lastIndexOf("LookAndFeel");
	if(i>=0) tm=tm.substring(0,i);
	tm=BuResource.BU.getString
	  ("L'aspect n'a pu mettre  jour l'arbre des composants")+
	  ". ["+tm+"]";

	BuDialogMessage dm=new BuDialogMessage
	  (app_,getInformationsSoftware(),tm);
	dm.activate();
      }
    }
  }

  public void setMetalTheme(String _arg)
  {
    try
    {
      int i=Integer.parseInt(_arg);
      BuMetalCustomTheme[] themes=BuMetalCustomTheme.getList();
      MetalLookAndFeel.setCurrentTheme(themes[i]);
      setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
    }
    catch (Exception ex)
    {
      BuDialogMessage dm=new BuDialogMessage
	(app_,getInformationsSoftware(),
	 BuResource.BU.getString
	 ("Le thme pour Metal n'a pu tre mis en place")+".");
      dm.activate();
    }
  }

  public void setSlafTheme(String _arg)
  {
    try
    {
      int i=Integer.parseInt(_arg);
      BuSlafCustomTheme[] themes=BuSlafCustomTheme.getList();

      System.err.println("SLAF THEME"+i+" "+themes[i]);
      Class   c=Class.forName("com.memoire.slaf.SlafLookAndFeel");
      Method  m=c.getMethod("setCurrentTheme",new Class[] { String.class } );
      m.invoke(null,new Object[] { themes[i].getKey() } );

      setLookAndFeel("com.memoire.slaf.SlafLookAndFeel");
    }
    catch (Exception ex)
    {
      ex.printStackTrace();

      BuDialogMessage dm=new BuDialogMessage
	(app_,getInformationsSoftware(),
	 BuResource.BU.getString
	 ("Le thme pour Slaf n'a pu tre mis en place")+".");
      dm.activate();
    }
  }

  public void fullscreen()
  {
    if(!(getApp() instanceof JFrame)) return;
    final JInternalFrame frame=getCurrentInternalFrame();
    if(frame==null) return;

    final JComponent     content=(JComponent)frame.getContentPane();
    final JWindow        win    =new JWindow();
    final JComponent     cwin   =(JComponent)win.getContentPane();
    final JFrame         app    =(JFrame)getApp();
    final Dimension      de     =app.getToolkit().getScreenSize();

    final BuLabel        label  =new BuLabel();
    final BuButton       button =new BuButton();
    final BuPanel        top    =new BuPanel();

    top.setLayout(new BuBorderLayout());

    label.setText(app.getTitle()+": "+frame.getTitle());
    label.setBorder(new EmptyBorder(1,1,1,1));
    label.setHorizontalAlignment(JLabel.CENTER);
    top.add(label,BorderLayout.CENTER);

    button.setBorder(new EmptyBorder(1,1,1,1));
    button.setRequestFocusEnabled(false);
    Icon icon=UIManager.getIcon("InternalFrameTitlePane.closeIcon");
    if(icon==null) icon=UIManager.getIcon("InternalFrame.closeIcon");
    button.setIcon(icon);
    button.setIcon(icon);
    top.add(button,BorderLayout.EAST);

    frame.setContentPane(new JPanel());

    cwin.add(content,BuBorderLayout.CENTER);
    cwin.add(top    ,BuBorderLayout.NORTH);
    cwin.setBorder(new LineBorder(Color.black,2));

    win.setSize(de.width,de.height);
    win.setLocation(0,0);
    app.setVisible(false);
    win.doLayout();
    win.validate();
    win.setVisible(true);
    win.setLocation(0,0);

    button.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent _ae)
        {
	  win.setVisible(false);
	  cwin.remove(content);
	  frame.setContentPane(content);
	  app.setVisible(true);
	}
      }
    );
  }

  public boolean confirmExit()
  {
    BuInformationsSoftware il=getInformationsSoftware();
    BuDialogConfirmation   bd=new BuDialogConfirmation
      (getApp(),il,BuResource.BU.getString
       ("Voulez-vous vraiment quitter ce logiciel ?"));

    int r=bd.activate();
    return (r==JOptionPane.YES_OPTION);
  }

  public void displayURL(String _url)
  {
    getApp().displayURL(_url);
  }

  public boolean isCloseFrameMode()
  { return true; }

  // commands

  public void about()
  {
    BuInformationsSoftware il=getInformationsSoftware();
    BuDialogMessage        bd=new BuDialogMessage(getApp(),il,il.about());
    bd.activate();
  }

  public void cut()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuCutCopyPasteInterface)
      ((BuCutCopyPasteInterface)frame).cut();
    else
      System.err.println
	(frame+" doesn't implement BuCutCopyPasteInterface");
  }

  public void copy()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuCutCopyPasteInterface)
      ((BuCutCopyPasteInterface)frame).copy();
    else
      System.err.println
	(frame+" doesn't implement BuCutCopyPasteInterface");
  }

  public void duplicate()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuCutCopyPasteInterface)
      ((BuCutCopyPasteInterface)frame).duplicate();
    else
      System.err.println
	(frame+" doesn't implement BuCutCopyPasteInterface");
  }

  public void find()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuSelectFindReplaceInterface)
      ((BuSelectFindReplaceInterface)frame).find();
    else
      System.err.println
	(frame+" doesn't implement BuSelectFindReplaceInterface");
  }

  public void license()
  {
    BuInformationsSoftware il=getInformationsSoftware();
    BuDialogMessage        bd=new BuDialogMessage(getApp(),il,il.license());
    bd.activate();
  }

  public void paste()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuCutCopyPasteInterface)
      ((BuCutCopyPasteInterface)frame).paste();
    else
      System.err.println
	(frame+" doesn't implement BuCutCopyPasteInterface");
  }

  public void undo()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuUndoRedoInterface)
      ((BuUndoRedoInterface)frame).undo();
    else
      System.err.println
	(frame+" doesn't implement BuUndoRedoInterface");
  }

  public void redo()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuUndoRedoInterface)
      ((BuUndoRedoInterface)frame).redo();
    else
      System.err.println
	(frame+" doesn't implement BuUndoRedoInterface");
  }

  public void replace()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuSelectFindReplaceInterface)
      ((BuSelectFindReplaceInterface)frame).replace();
    else
      System.err.println
	(frame+" doesn't implement BuSelectFindReplaceInterface");
  }

  public void select()
  {
    JInternalFrame frame=getCurrentInternalFrame();

    if(frame instanceof BuSelectFindReplaceInterface)
      ((BuSelectFindReplaceInterface)frame).select();
    else
      System.err.println
	(frame+" doesn't implement BuSelectFindReplaceInterface");
  }

  public void close()
  {
    if(isCloseFrameMode())
    {
      JInternalFrame frame=getCurrentInternalFrame();

      if(!frame.isClosed()&&frame.isClosable())
      {
	try { frame.setClosed(true); }
	catch(PropertyVetoException ex) { }
      }
    }
  }

  public void exit()
  {
    getApp().exit();
  }

  public void print()
  {
    getApp().print();
  }

  public void preview()
  {
    JInternalFrame frame=getMainPanel().getCurrentInternalFrame();

    if(frame instanceof BuPrintable)
    {
      BuPrinter.preview(this,
			frame.getTitle(),
			(BuPrintable)frame);
    }
    else
    {
      BuInformationsSoftware il=getInformationsSoftware();
      BuDialogError          bd=new BuDialogError
	(this,il,BuResource.BU.getString
	 ("La prvisualisation n'est pas disponible pour")+
	"\""+frame.getTitle()+"\".");
    }
  }

  // Window

  public void windowActivated(WindowEvent _evt)
  { }

  public void windowClosed(WindowEvent _evt)
  { }

  public void windowClosing(WindowEvent _evt)
  { exit(); }

  public void windowDeactivated(WindowEvent _evt)
  { }

  public void windowDeiconified(WindowEvent _evt)
  { }

  public void windowIconified(WindowEvent _evt)
  { }

  public void windowOpened(WindowEvent _evt)
  { }

  // InternalFrame

  public void internalFrameActivated(InternalFrameEvent _evt)
  {
    ajouterActionsOutils(_evt);
    menusOrganisationFenetres();

    BuDesktop      desktop=getMainPanel().getDesktop();
    JInternalFrame frame  =desktop.getCurrentInternalFrame();
    desktop.showFrame(frame);
  }

  public void internalFrameDeactivated(InternalFrameEvent _evt)
  {
    // System.err.println("DEACTIVATED "+_evt.getSource());
    enleverActionsOutils(_evt);
    menusOrganisationFenetres();
  }

  public void internalFrameClosed(InternalFrameEvent _evt)
  {
    // System.err.println("CLOSED "+_evt.getSource());
    /*
    try
    {
      if(((BuInternalFrame)_evt.getSource()).isSelected()) 
      {
	enleverActionsOutils(_evt);
      }
      catch(PropertyVetoException ex) { }
    }
    */
    //enleverActionsOutils(_evt);
    //menusOrganisationFenetres();
    selectionnerPremiere(_evt);
  }

  public void internalFrameClosing(InternalFrameEvent _evt)
  {
    // System.err.println("CLOSING "+_evt.getSource());
    // enleverActionsOutils(_evt);
  }

  public void internalFrameDeiconified(InternalFrameEvent _evt)
  {
    // System.err.println("DEICONIFIED "+_evt.getSource());
    menusOrganisationFenetres();
    try
    {
      Object source=_evt.getSource();
      if(source instanceof BuInternalFrame)
	((BuInternalFrame)source).setSelected(true);
    }
    catch(PropertyVetoException ex) { }
  }

  public void internalFrameIconified(InternalFrameEvent _evt)
  {
    // System.err.println("ICONIFIED "+_evt.getSource());
    menusOrganisationFenetres();
    selectionnerPremiere(_evt);
  }

  public void internalFrameOpened(InternalFrameEvent _evt)
  {
    // System.err.println("OPENED "+_evt.getSource());
    // ajouterActionsOutils(_evt);
    menusOrganisationFenetres();
  }

  public void componentAdded(ContainerEvent _evt)
  {
    menusOrganisationFenetres();
  }

  public void componentRemoved(ContainerEvent _evt)
  {
    menusOrganisationFenetres();
  }

  protected void ajouterActionsOutils(InternalFrameEvent _evt)
  {
    Object source=_evt.getSource();
    if(source instanceof BuInternalFrame)
    {
      BuInternalFrame f =(BuInternalFrame)source;
      BuMenuBar       mb=getMainMenuBar();
      BuSpecificBar   sb=getSpecificBar();

      // System.err.println(""+f+" activated!");

      int i;

      String[] ea=f.getEnabledActions();
      for(i=0;i<ea.length;i++)
	setEnabledForAction(ea[i],true);

      String[] da=f.getDisabledActions();
      for(i=0;i<da.length;i++)
	setEnabledForAction(da[i],false);

      mb.addMenus(f.getSpecificMenus());
      sb.addTools(f.getSpecificTools());

      BuMainPanel mp=getMainPanel();
      if(mp.getAssistant()!=null)
	  mp.getAssistant().addEmitters(sb);

      f.adjustActions();
    }
  }

  protected void enleverActionsOutils(InternalFrameEvent _evt)
  {
    Object source=_evt.getSource();
    if(source instanceof BuInternalFrame)
    {
      BuInternalFrame f =(BuInternalFrame)source;
      BuMenuBar       mb=getMainMenuBar();
      BuSpecificBar   sb=getSpecificBar();

      // System.err.println(""+f+" deactivated!");
      int i;

      String[] ea=f.getEnabledActions();
      for(i=0;i<ea.length;i++)
	setEnabledForAction(ea[i],false);

      String[] da=f.getDisabledActions();
      for(i=0;i<da.length;i++)
	setEnabledForAction(da[i],true);

      mb.removeMenus(f.getSpecificMenus());
      sb.removeTools(f.getSpecificTools());
    }
  }

  protected void selectionnerPremiere(InternalFrameEvent _evt)
  {
    Object source=_evt.getSource();
    if(source instanceof BuInternalFrame)
    {
      BuInternalFrame f=(BuInternalFrame)source;
      BuDesktop       d=getMainPanel().getDesktop();

      if((f==d.getCurrentInternalFrame())||f.isIcon())
      {
	try
	{
	  f.setSelected(false);

	  JInternalFrame[] frames;
	  boolean          ok;

	  ok=false;
	  frames=d.getNormalFrames();
	  if(frames.length>0)
	  {
	    for(int i=0;i<frames.length;i++)
	      if(frames[i]!=f)
	      {
		try
		{
		  frames[i].setSelected(true); 
		  // System.err.println("select: "+frames[i]);
		  ok=true;
		  break;
		}
		catch(PropertyVetoException ex) { }
	      }
	  }

	  if(!ok)
	  {
	    frames=d.getIconifiedFrames();
	    for(int i=0;i<frames.length;i++)
	      if(frames[i]!=f)
	      {
		try
		{
		  frames[i].setSelected(true);
		  // System.err.println("select: "+frames[i]);
		  ok=true;
		  break;
		}
		catch(PropertyVetoException ex) { }
	      }
	  }
	}
	catch(PropertyVetoException ex) { }
      }
    }
  }

  protected void menusOrganisationFenetres()
  {
    BuDesktop      desktop=getMainPanel().getDesktop();
    boolean        appli  =(getApp() instanceof JFrame);
    JInternalFrame frame  =desktop.getCurrentInternalFrame();
    boolean b;

    b=(desktop.getNormalFramesCount()>0);
    setEnabledForAction("CASCADE",b);
    setEnabledForAction("MOSAIQUE",b);

    b=(desktop.getIconifiedFramesCount()>0);
    setEnabledForAction("RANGERICONES",b);
    
    b=(desktop.getPalettesCount()>0);
    setEnabledForAction("RANGERPALETTES",b);
    
    b=(frame!=null);
    setEnabledForAction("PLEINECRAN",b&&appli);

    if(isCloseFrameMode())
    {
      b=(frame!=null)&&frame.isClosable();
      setEnabledForAction("FERMER",b);
    }

    // TMP
    // desktop.adjustSize();
  }
}
