/**
 * @modification $Date: 2001/12/07 18:28:12 $
 * @statut       unstable
 * @file         BuDesktop.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
//import com.memoire.dnd.*;
import com.memoire.fu.*;
//import com.memoire.re.*;

import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

/**
 * A DesktopPane with a few additional services.
 * Auto-resizing, optionnaly tabbed, custom-friendly, icons management, ...
 */
public class BuDesktop
       extends JDesktopPane
       implements MouseListener, ActionListener
{
  public final static int LEFT_MARGIN=31;

  public final static boolean isPalette(JComponent _f)
  {
    return (Boolean.TRUE.equals
	    (_f.getClientProperty("JInternalFrame.isPalette")));
  }

  // Anti-aliasing

  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  public final void snapXY(JComponent _f)
  {
    if(  BuPreferences.BU.getBooleanProperty("desktop.snap",false)
       &&!isTabbed()
       &&!isPalette(_f)
       &&(_f instanceof JInternalFrame))
    {
      Point   p=_f.getLocation();
      boolean b=false;
      if(p.x%37!=0) { p.x=p.x-p.x%37; b=true; }
      if(p.y%37!=0) { p.y=p.y-p.y%37; b=true; }
      if(b) _f.setLocation(p);
    }
  }

  public final void snapWH(JComponent _f)
  {
    if(  BuPreferences.BU.getBooleanProperty("desktop.snap",false)
       &&!isTabbed()
       &&!isPalette(_f)
       &&(_f instanceof JInternalFrame))
    {
      Dimension d=_f.getSize();
      boolean   b=false;
      if(((JInternalFrame)_f).isResizable())
      {
        if(d.width %37!=0) { d.width =d.width +37-d.width %37; b=true; }
        if(d.height%37!=0) { d.height=d.height+37-d.height%37; b=true; }
      }
      if(b) _f.setSize(d);
    }
  }

  public BuDesktop()
  {
    super();

    putClientProperty("JDesktopPane.dragMode","outline");

    setName("buDESKTOP");
    setDesktopManager(new BuDesktopManager(this));
    setBackgroundPainter(new BuBackgroundPainter());

    addMouseListener(this);

    if(isTabbed())
    {
      setToolTipText("ToolTip");
      setLayout(new Layout());
    }
  }

  public boolean isFocusCycleRoot()
  { return true; }

  public synchronized void moveToFront(JInternalFrame _f)
  {
    _f.moveToFront();

    JInternalFrame[] frames=getAllFrames();
    int l=frames.length;
    for(int i=0;i<l;i++)
      if(isPalette(frames[i])&&frames[i].isVisible())
	frames[i].moveToFront();
  }

  public boolean isOpaque()
  {
    return Boolean.TRUE.equals(UIManager.get("Desktop.opaque"));
  }

  // Paint

  public synchronized void paintComponent(Graphics _g)
  {
    JInternalFrame[] frames=getAllFrames();

    if(!isTabbed()||(frames.length==0))
    {
      Rectangle clip=_g.getClipBounds();
      Dimension dd  =getSize();

      BuBackgroundPainter bp=getBackgroundPainter();
      if(bp!=null) bp.paintBackground(this,_g);

      if(logo_!=null)
      {
	Rectangle ri=new Rectangle(5,
				   // dd.width-5-logo_.getIconWidth(),
				   5,
				   logo_.getIconWidth(),
				   logo_.getIconHeight());
	if(ri.intersects(clip))
	  logo_.paintIcon(this,_g,ri.x,ri.y);
      }
    }
    else
    {
      Dimension dd=getSize();
      Color pbg=UIManager.getColor("Panel.background");
      Color dbg=getBackground();

      _g.setColor(pbg);
      _g.fillRect(0,0,dd.width,dd.height); // LM+4

      // JInternalFrame[] frames=getAllFrames();
      int y,l;

      sortFramesByTitle(frames);

      l=frames.length;
      y=0;
      for(int i=0; i<l; i++)
      {
	if( frames[i].isClosed() ) continue;
	if(!frames[i].isVisible()) continue;

	// if(isPalette(frames[i])) continue;

	_g.setColor(pbg);
	_g.draw3DRect(0,y,LEFT_MARGIN-1,LEFT_MARGIN-1,false);
	
	     if(isPalette(frames[i]))   _g.setColor(pbg); 
	else if(frames[i].isSelected()) _g.setColor(dbg);
	else                            _g.setColor(dbg.brighter());
	_g.fill3DRect(1,y+1,LEFT_MARGIN-2,LEFT_MARGIN-2,true);

	Icon icon=frames[i].getFrameIcon();
	if(icon!=null)
	{
	  int w=icon.getIconWidth();
	  int h=icon.getIconHeight();
	  icon.paintIcon(this,_g,(LEFT_MARGIN-w)/2,y+(LEFT_MARGIN-h)/2);
	}

	y+=LEFT_MARGIN;
      }
    }
  }

  public void paintChildren(Graphics _g)
  {
    /*
    if(isTabbed())
    {
      JInternalFrame[] frames=getAllFrames();
    }
    else
    */

    super.paintChildren(_g);
  }

  public String getToolTipText()
    { return null; }

  public synchronized String getToolTipText(MouseEvent _evt)
  {
    if(isTabbed())
    {
      int xe=_evt.getX();
      int ye=_evt.getY();

      JInternalFrame[] frames=getAllFrames();
      int l=frames.length;

      if((xe<LEFT_MARGIN)&&(ye<LEFT_MARGIN*l))
      {
	sortFramesByTitle(frames);
	JInternalFrame f=frames[ye/LEFT_MARGIN];
	String r=f.getTitle();
	if("".equals(r)) r=null;
	if(r==null) r=""+(1+ye/LEFT_MARGIN);
	return r;
      }
    }

    return super.getToolTipText(_evt);
  }

  public synchronized Point getToolTipLocation(MouseEvent _evt)
  {
    if(isTabbed())
    {
      int xe=_evt.getX();
      int ye=_evt.getY();

      JInternalFrame[] frames=getAllFrames();
      int l=frames.length;

      if((xe<LEFT_MARGIN)&&(ye<LEFT_MARGIN*l))
      {
	sortFramesByTitle(frames);
	// return new Point(0,(ye/LEFT_MARGIN+1)*LEFT_MARGIN);
	return new Point(LEFT_MARGIN+2,(ye/LEFT_MARGIN)*LEFT_MARGIN+1);
      }
    }

    return super.getToolTipLocation(_evt);
  }

  // Proprietes

  private boolean tabbed_=
    BuPreferences.BU.getBooleanProperty("desktop.tabbed");
  public boolean isTabbed() { return tabbed_; }
  // public void setTabbed(boolean _tabbed) { tabbed_=_tabbed; }

  private Icon logo_;
  public Icon getLogo() { return logo_; }
  public void setLogo(Icon _logo)
  {
    logo_=_logo;
    repaint();
  }

  private BuBackgroundPainter bp_;
  public BuBackgroundPainter getBackgroundPainter() { return bp_; }
  public void setBackgroundPainter(BuBackgroundPainter _bp)
  {
    bp_=_bp;
    repaint();
  }

  // Filles

  public void addInternalFrame(JInternalFrame _f)
  {
    if(!SwingUtilities.isEventDispatchThread())
      throw new RuntimeException
	("Not in swing thread. "+
	 "Use Implementation.addInternalFrame() instead");

    //System.err.println(_f.getName()+":"+_f.getDesktopPane());

    if(_f.getDesktopPane()==null)
    {
      if(_f.isSelected())
      {
	try { _f.setSelected(false); }
	catch(PropertyVetoException ex) { }
      }

      Point pf=_f.getLocation();
      if((pf.x==0)&&(pf.y==0))
      {
	Container cp=getParent();
	if(cp instanceof JViewport)
	{
	  Rectangle vr=((JViewport)cp).getViewRect();
	  Dimension df=_f.getSize();

	  pf.x=vr.x+(vr.width-df.width  )/2;
	  pf.y=vr.y+(vr.height-df.height)/2;
	  _f.setLocation(pf);
	}
      }

      SwingUtilities.updateComponentTreeUI(_f);
      add(_f);
      //doLayout();
      //validate();

      snapXY(_f);
      snapWH(_f);
      //repaint(); // @GDX ???
    }
    activateInternalFrame(_f);
  }

  public void removeInternalFrame(JInternalFrame _f)
  {
    if(!SwingUtilities.isEventDispatchThread())
      throw new RuntimeException
	("Not in swing thread. "+
	 "Use Implementation.removeInternalFrame() instead");

    if(_f.getDesktopPane()==this)
    {
      deactivateInternalFrame(_f);
      remove(_f);
      //doLayout();
      //validate();
      //repaint(); // @GDX ???
    }

    adjustSize();
  }

  public void checkInternalFrame(JInternalFrame _f)
  {
    if(!SwingUtilities.isEventDispatchThread())
      throw new RuntimeException
	("Not in swing thread.");

    Dimension dd=getSize();
    Point     pf=_f.getLocation();
    Dimension df=_f.getSize();

    if(_f.isResizable())
    {
      if(df.width >dd.width ) df.width=dd.width;
      if(df.height>dd.height) df.height=dd.height;
      if(!df.equals(getSize())) _f.setSize(df);
    }

    if(pf.x+df.width >dd.width ) pf.x=dd.width-df.width;
    if(pf.y+df.height>dd.height) pf.y=dd.height-df.height;
    if(pf.x<0) pf.x=0;
    if(pf.y<0) pf.y=0;

    if(isTabbed()&&isPalette(_f)&&(pf.x<LEFT_MARGIN+4))
      pf.x=LEFT_MARGIN+4;

    if(!pf.equals(getLocation())) _f.setLocation(pf);

    adjustSize();
  }

  public void activateInternalFrame(JInternalFrame _f)
  {
    if(!SwingUtilities.isEventDispatchThread())
      throw new RuntimeException
	("Not in swing thread. "+
	 "Use Implementation.activateInternalFrame() instead");

    if(!_f.isVisible())
    {
      _f.setVisible(true);
    }

    if(_f.isClosed())
    {
      try { _f.setClosed(false); }
      catch(PropertyVetoException ex) { }
    }

    checkInternalFrame(_f);

    if(_f.isIcon())
    {
      try { _f.setIcon(false); }
      catch(PropertyVetoException ex) { }
    }
    moveToFront(_f);
    if(!_f.isSelected())
    {
      try { _f.setSelected(true); }
      catch(PropertyVetoException ex) { }
    }
  }

  public void deactivateInternalFrame(JInternalFrame _f)
  {
    if(!SwingUtilities.isEventDispatchThread())
      throw new RuntimeException
	("Not in swing thread.");

    checkInternalFrame(_f);

    if(_f.isSelected())
    {
      try { _f.setSelected(false); }
      catch(PropertyVetoException ex) { }
    }
  }

  public synchronized JInternalFrame getCurrentInternalFrame()
  {
    JInternalFrame[] frames=getAllFrames();
    JInternalFrame   r=null;

    for(int i=0; i<frames.length; i++)
      if(frames[i].isSelected())
	{ r=frames[i]; break; }

    return r;
  }

  public synchronized JInternalFrame[] getNormalFrames()
  {
    JInternalFrame[] frames=getAllFrames();
    int l,n,i,j;

    l=frames.length;
    n=0;

    for(i=0; i<l; i++)
      if(!frames[i].isIcon()&&!isPalette(frames[i])) n++;

    JInternalFrame[] r=new JInternalFrame[n];

    j=0;
    for(i=0; i<l; i++)
      if(!frames[i].isIcon()&&!isPalette(frames[i])) { r[j]=frames[i]; j++; }

    return r;
  }

  public synchronized JInternalFrame[] getNotIconifiedFrames()
  {
    JInternalFrame[] frames=getAllFrames();
    int l,n,i,j;

    l=frames.length;
    n=0;

    for(i=0; i<l; i++)
      if(!frames[i].isIcon()) n++;

    JInternalFrame[] r=new JInternalFrame[n];

    j=0;
    for(i=0; i<l; i++)
      if(!frames[i].isIcon()) { r[j]=frames[i]; j++; }

    return r;
  }

  public synchronized JInternalFrame[] getIconifiedFrames()
  {
    JInternalFrame[] frames=getAllFrames();
    int l,n,i,j;

    l=frames.length;
    n=0;

    for(i=0; i<l; i++)
      if(frames[i].isIcon()) n++;

    JInternalFrame[] r=new JInternalFrame[n];

    j=0;
    for(i=0; i<l; i++)
      if(frames[i].isIcon()) { r[j]=frames[i]; j++; }

    return r;
  }

  public synchronized JInternalFrame[] getPalettes()
  {
    JInternalFrame[] frames=getAllFrames();
    int l,n,i,j;

    l=frames.length;
    n=0;

    for(i=0; i<l; i++)
      if(isPalette(frames[i])) n++;

    JInternalFrame[] r=new JInternalFrame[n];

    j=0;
    for(i=0; i<l; i++)
      if(isPalette(frames[i])) { r[j]=frames[i]; j++; }

    return r;
  }

  public int getNormalFramesCount()
  { return getNormalFrames().length; }

  public int getNotIconifiedFramesCount()
  { return getNotIconifiedFrames().length; }

  public int getIconifiedFramesCount()
  { return getIconifiedFrames().length; }

  public int getPalettesCount()
  { return getPalettes().length; }

  // Reorganisation

  protected void sortFramesByTitle(JInternalFrame[] _frames)
  {
    JInternalFrame tmp;
    int i,l;

    l=_frames.length;
    for(i=0; i+1<l; i++)
    {
      String t1=_frames[i].getTitle();
      String t2=_frames[i+1].getTitle();
      if((t1!=null)&&(t2!=null)&&(t1.compareTo(t2)>0))
      {
	tmp         =_frames[i];
	_frames[i]  =_frames[i+1];
	_frames[i+1]=tmp;
	i--; if(i>=0) i--;
      }
    }
  }

  protected void sortFramesByHeight(JInternalFrame[] _frames)
  {
    sortFramesByTitle(_frames);

    JInternalFrame tmp;
    int i,l;

    l=_frames.length;
    for(i=0; i+1<l; i++)
      if(_frames[i].getHeight()>_frames[i+1].getHeight())
      {
	tmp         =_frames[i];
	_frames[i]  =_frames[i+1];
	_frames[i+1]=tmp;
	i--; if(i>=0) i--;
      }
  }

  protected void sortFramesByWidth(JInternalFrame[] _frames)
  {
    sortFramesByTitle(_frames);

    JInternalFrame tmp;
    int i,l;

    l=_frames.length;
    for(i=0; i+1<l; i++)
      if(_frames[i].getWidth()<_frames[i+1].getWidth())
      {
	tmp         =_frames[i];
	_frames[i]  =_frames[i+1];
	_frames[i+1]=tmp;
	i--; if(i>=0) i--;
      }
  }

  public synchronized void waterfall()
  {
    if(isTabbed()) return;

    JInternalFrame[] frames=getNormalFrames();
    int i,l,x,y;

    l=frames.length;
    if(l>0)
    {
      sortFramesByTitle(frames);

      x=74;
      y=0; //y=5;
      for(i=l-1;i>=0;i--)
      {
	frames[i].setLocation(x,y);
	moveToFront(frames[i]);
	checkInternalFrame(frames[i]);
	x+=37;//x+=25;
	y+=37;//y+=25;
      }
    }
  }

  public synchronized void tile()
  {
    if(isTabbed()) return;

    JInternalFrame[] frames=getNormalFrames();
    int i,l,x,y,h,wd,hd;

    l=frames.length;
    if(l>0)
    {
      sortFramesByHeight(frames);

      wd=getWidth();  // getSize().width;
      hd=getHeight(); // getSize().height;
      wd=(wd<200 ? 200 : wd);
      hd=(hd<200 ? 200 : hd-60);

      x=0;
      y=0;
      h=0;
      for(i=0;i<l;i++)
      {
	int wf=frames[i].getWidth();  // Size().width;
	int hf=frames[i].getHeight(); // Size().height;
	if(x+wf>wd)
	  { x=0; y+=h; h=0; }
	frames[i].setLocation(x,y);
	x+=wf;
	h=Math.max(h,hf);
	moveToFront(frames[i]);
	checkInternalFrame(frames[i]);
      }
    }
  }

  public synchronized void arrangeIcons()
  {
    if(isTabbed()) return;

    JInternalFrame[] frames;
    int i,l,x,y,wd,hd;

    wd=getWidth();  // getSize().width;
    hd=getHeight(); // getSize().height;

    frames=getIconifiedFrames();
    l=frames.length;

    if(l>0)
    {
      sortFramesByTitle(frames);

      x=0; y=hd;
      for(i=0;i<l;i++)
      {
	JInternalFrame.JDesktopIcon dti=frames[i].getDesktopIcon();
	Dimension cs;

	dti.setSize(dti.getPreferredSize());
	cs=dti.getSize();

	if(x+cs.width<wd)
	{
	  dti.setLocation(x,y-cs.height);
	  x+=cs.width;
	}
	else
	{
	  x=0;
	  y-=cs.height;
	  dti.setLocation(x,y-cs.height);
	}
      }
    }

    /*
    frames=getNotIconifiedFrames();
    l=frames.length;

    if(l>0)
    {
      sortFramesByWidth(frames);

      y=0;
      for(i=l-1;i>=0;i--)
      {
	Dimension d=frames[i].getSize();
	if(d.height<32)
	{
	  frames[i].setLocation(0,y);
	  y+=d.height;
	}
      }
    }
    */

    adjustSize();
  }

  public synchronized void arrangePalettes()
  {
    // if(isTabbed()) return;

    JInternalFrame[] frames;
    int i,l,x,y,wd,hd,wmax;

    wd=getWidth();  // getSize().width;
    hd=getHeight(); // getSize().height;

    frames=getAllFrames();
    l=frames.length;

    if(l>0)
    {
      sortFramesByHeight(frames);

      wmax=0;
      x=(isTabbed() ? LEFT_MARGIN+1 : 0);
      y=(isTabbed() ? 17 : 0);

      for(i=0;i<l;i++)
      {
	JInternalFrame dti=frames[i];
	if(!isPalette(dti)) continue;

	// dti.setSize(dti.getPreferredSize());
	Dimension cs=dti.getSize();

	if(y+cs.height>=hd)
	{
	  x+=wmax;
	  y=(isTabbed() ? 17 : 0);
	  wmax=0;
	}

	dti.setLocation(x,y);
	y+=cs.height;
	wmax=Math.max(wmax,cs.width);
      }
    }

    adjustSize();
  }

  // Taille

  public synchronized Dimension getPreferredSize()
  {
    Dimension r=super.getPreferredSize();

    // if(isTabbed()) return r;

    JInternalFrame[] frames=getAllFrames();
    Point            qmin  =new Point(0,0);
    int i,l,n,x;

    l=frames.length;
    for(i=l-1;i>=0;i--)
    {
      if(isTabbed()&&!isPalette(frames[i])) continue;

      Point q=null;
      if(frames[i].isIcon()) q=frames[i].getDesktopIcon().getLocation();
      else                   q=frames[i].getLocation();
      if(qmin.x>q.x) qmin.x=q.x;
      if(qmin.y>q.y) qmin.y=q.y;
    }

    for(i=l-1;i>=0;i--)
    {
      if(isTabbed()&&!isPalette(frames[i])) continue;

      if(frames[i].isIcon())
      {
	Point p=frames[i].getDesktopIcon().getLocation();
	p.x-=qmin.x;
	p.y-=qmin.y;
	frames[i].getDesktopIcon().setLocation(p);

	Dimension d=frames[i].getDesktopIcon().getSize();
	if(p.x+d.width >r.width ) r.width =p.x+d.width;
	if(p.y+d.height>r.height) r.height=p.y+d.height;
      }
      else
      {
	Point p=frames[i].getLocation();
	p.x-=qmin.x;
	p.y-=qmin.y;
	frames[i].setLocation(p);

	Dimension d=frames[i].getSize();
	if(p.x+d.width >r.width ) r.width =p.x+d.width;
	if(p.y+d.height>r.height) r.height=p.y+d.height;
      }
    }

    // System.err.println("PSIZE="+r);
    return r;
  }

  public void adjustSize()
  {
    SwingUtilities.invokeLater
      (new Runnable()
       {
	 public void run()
	 {
	   Dimension dd=getPreferredSize();
	   Dimension ds=getParent().getSize();
	   if(dd.width <ds.width ) dd.width =ds.width;
	   if(dd.height<ds.height) dd.height=ds.height;
	   setSize(dd);

	   /*
	   getParent().doLayout();
	   getParent().validate();
	   doLayout();
	   validate();
	   */
	   revalidate();

	   /*
	   Container cp=getParent();
	   if(cp instanceof JViewport)
	   {
	     JInternalFrame sf=getCurrentInternalFrame();
	     if((sf!=null)&&sf.isShowing())
	       ((JViewport)cp).scrollRectToVisible(sf.getBounds());
	   }
	   */
	 }
       });
  }

  public void showFrame(final JInternalFrame _f)
  {
    SwingUtilities.invokeLater
      (new Runnable()
       {
	 public void run()
	 {
	   Container cp=getParent();
	   if(  (cp instanceof JViewport)
	      &&(_f!=null)
	      &&_f.isShowing())
	   {
	     JViewport vp=(JViewport)cp;
	     Point     vo=vp.getViewPosition();
	     Dimension es=vp.getExtentSize();
	     Dimension vs=vp.getViewSize();
	     Point     fo=_f.getLocation();
	     Dimension fs=_f.getSize();

	     if(fo.x<vo.x) vo.x=fo.x;
	     if(fo.y<vo.y) vo.y=fo.y;
	     if(fo.x+fs.width >vo.x+es.width ) vo.x=Math.min(fo.x,vs.width -es.width );
	     if(fo.y+fs.height>vo.y+es.height) vo.y=Math.min(fo.y,vs.height-es.height);
	     vp.setViewPosition(vo);
	   }
	 }
       });
  }

  // Layout

  private final class Layout implements LayoutManager2
  {
    public void addLayoutComponent(String _s, Component _c)
    { addLayoutComponent(_c,_s); }

    public void addLayoutComponent(Component _c, Object _o)
    {
      // System.out.println("ADD "+_c);
      if(  (_c instanceof JComponent)
	 &&isPalette((JComponent)_c))
	return;

      if(_c instanceof BuInternalFrame)
	((BuInternalFrame)_c).simplifyTop();
      
      /*
      if(_c instanceof JComponent)
	((JComponent)_c).setBorder(UIManager.getBorder("TitleBorder.border"));
      */

      if(_c instanceof JInternalFrame)
      {
	JInternalFrame f=(JInternalFrame)_c;
	f.setIconifiable(false);
	f.setMaximizable(false);
	// f.setVisible(f.isSelected());
      }
    }

    public void removeLayoutComponent(Component _c)
    { }

    public float getLayoutAlignmentX(Container _p)
    { return 0.5f; }
    public float getLayoutAlignmentY(Container _p)
    { return 0.5f; }
    public Dimension minimumLayoutSize(Container _p)
    { return _p.getSize(); }
    public Dimension preferredLayoutSize(Container _p)
    { return _p.getSize(); }
    public Dimension maximumLayoutSize(Container _p)
    { return _p.getSize(); }

    public void invalidateLayout(Container _p)
    {
      Dimension ds=_p.getSize();
      // int l=p.getComponentCount();
      repaint(0,0,LEFT_MARGIN,ds.height); // l*LEFT_MARGIN;
    }

    public void layoutContainer(Container _p)
    {
      Dimension ds=_p.getSize();
      int l=_p.getComponentCount();
      for(int i=0;i<l;i++)
      {
	Component c=getComponent(i);
	if(  (c instanceof JComponent)
	   &&isPalette((JComponent)c))
	  ;
	else
	  c.setBounds(LEFT_MARGIN,0,
		      ds.width-LEFT_MARGIN,ds.height);
      }

      repaint(0,0,LEFT_MARGIN,ds.height);
    }
  }

  // Mouse

  public void mouseDown(MouseEvent _evt)
  { }

  public void mouseEntered(MouseEvent _evt)
  { }

  public void mouseExited(MouseEvent _evt)
  { }

  public void mousePressed(MouseEvent _evt)
  {
    if(  _evt.isPopupTrigger()
       ||((_evt.getModifiers()&MouseEvent.BUTTON3_MASK)!=0))
      popupMenu(_evt);
  }

  public void mouseReleased(MouseEvent _evt)
  { }

  public void mouseUp(MouseEvent _evt)
  { }

  public void mouseClicked(MouseEvent _evt)
  {
    if(isTabbed())
    {
      int xe=_evt.getX();
      int ye=_evt.getY();

      JInternalFrame[] frames=getAllFrames();
      int l=frames.length;

      if((xe<LEFT_MARGIN)&&(ye<LEFT_MARGIN*l))
      {
	sortFramesByTitle(frames);
	activateInternalFrame(frames[ye/LEFT_MARGIN]);
      }
    }
  }

  protected BuPopupMenu menu=null;

  public void popupMenu(MouseEvent _evt)
  {
    // System.err.println("EVT="+_evt);

    int xe=_evt.getX();
    int ye=_evt.getY();

    BuBackgroundPainter bp=getBackgroundPainter();

    if(menu==null)
    {
      menu=new BuPopupMenu(true,BuResource.BU.getString("Bureau"));

      menu.addCheckBoxMenuItem
	  (BuResource.BU.getString("Grille"),
	   "GRILLE",true,
	   bp.isGrid());
      menu.addCheckBoxMenuItem
	  (BuResource.BU.getString("Points"),
	   "POINTS",true,
	   bp.isDots());
      menu.addCheckBoxMenuItem
	  (BuResource.BU.getString("Magntisme"),
	   "MAGNETISME",true,
	   BuPreferences.BU.getBooleanProperty("desktop.snap",false));
      menu.addSeparator();
      menu.addMenuItem
	  (BuResource.BU.getString("Uniforme"),
	   "DESKTOP_UNIFORME",BuResource.BU.getIcon("uniforme"),true);
    menu.addMenuItem
	(BuResource.BU.getString("Dgrad"),
	 "DESKTOP_DEGRADE",BuResource.BU.getIcon("degrade"),true);
    menu.addSeparator();

    menu.addMenuItem
	(BuResource.BU.getString("Texture")+" 1",
	 "DESKTOP_TEXTURE1",new BuTextureIcon(BuResource.BU.getImage("background_desktop_1")),true);
    menu.addMenuItem
	(BuResource.BU.getString("Texture")+" 2",
	 "DESKTOP_TEXTURE2",new BuTextureIcon(BuResource.BU.getImage("background_desktop_2")),true);
    menu.addMenuItem
	(BuResource.BU.getString("Texture")+" 3",
	 "DESKTOP_TEXTURE3",new BuTextureIcon(BuResource.BU.getImage("background_desktop_3")),true);
    menu.addMenuItem
	(BuResource.BU.getString("Texture")+" 4",
	 "DESKTOP_TEXTURE4",new BuTextureIcon
	     (new ImageIcon(BuPreferences.BU.getStringProperty("desktop.image")).getImage()),true);
    menu.addSeparator();

    menu.addMenuItem
	(BuResource.BU.getString("Rouge"),
	 "DESKTOP_ROUGE"     ,new BuColorIcon(new Color(128, 64, 64)),true);
    menu.addMenuItem
	(BuResource.BU.getString("Vert"),
	 "DESKTOP_VERT"      ,new BuColorIcon(new Color( 64,129, 64)),true);
    menu.addMenuItem
	(BuResource.BU.getString("Bleu"),
	 "DESKTOP_BLEU"      ,new BuColorIcon(new Color( 64, 64,128)),true);
    menu.addMenuItem
	(BuResource.BU.getString("Orange"),
	 "DESKTOP_ORANGE"    ,new BuColorIcon(new Color(192,128, 96)),true);
    menu.addMenuItem
	(BuResource.BU.getString("Similaire"),
	 "DESKTOP_SIMILAIRE" ,new BuColorIcon(UIManager.getColor("Panel.background"  )),true);
    menu.addMenuItem
	(BuResource.BU.getString("Dfaut"),
	 "DESKTOP_DEFAUT"    ,new BuColorIcon(UIManager.getColor("Desktop.background")),true);
    }

    BuActionChecker.setCheckedForAction(menu,"GRILLE",bp.isGrid());
    BuActionChecker.setCheckedForAction(menu,"POINTS",bp.isDots());
    BuActionChecker.setCheckedForAction
	(menu,"MAGNETISME",
	 BuPreferences.BU.getBooleanProperty("desktop.snap",false));

    // xe=20-menu.getSize().width;
    // ye-=5;
    menu.show((JComponent)_evt.getSource(),xe,ye);
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
    // System.err.println("BuDesktop ACTION="+action); 

    BuBackgroundPainter bp=getBackgroundPainter();

    if(action.equals("GRILLE"))
    {
      bp.setGrid(!bp.isGrid());
    }
    else if(action.equals("POINTS"))
    {
      bp.setDots(!bp.isDots());
    }
    else if(action.equals("MAGNETISME"))
    {
      BuPreferences.BU.putBooleanProperty
	("desktop.snap",
	 !BuPreferences.BU.getBooleanProperty("desktop.snap",false));
    }
    else if(action.equals("DESKTOP_UNIFORME"))
    {
      bp.setIcon(null);
      bp.setGradient(false);
    }
    else if(action.equals("DESKTOP_DEGRADE"))
    {
      bp.setIcon(null);
      bp.setGradient(true);
    }
    else if(action.equals("DESKTOP_TEXTURE4"))
    {
      bp.setIcon(new ImageIcon(BuPreferences.BU.getStringProperty("desktop.image")));
    }
    else if(action.startsWith("DESKTOP_TEXTURE"))
    {
      bp.setIcon(BuResource.BU.getIcon("background_desktop_"+action.substring(15)));
    }
    else if(action.equals("DESKTOP_ROUGE"))
    { 
      setBackground(new Color(128,64,64));
      bp.setIcon(null);
    }
    else if(action.equals("DESKTOP_VERT"))
    {
      setBackground(new Color(64,128,64));
      bp.setIcon(null);
    }
    else if(action.equals("DESKTOP_BLEU"))
    { 
      setBackground(new Color(64,64,128));
      bp.setIcon(null);
    }
    else if(action.equals("DESKTOP_ORANGE"))
    { 
      setBackground(new Color(192,128,96));
      bp.setIcon(null);
    }
    else if(action.equals("DESKTOP_SIMILAIRE"))
    { 
      setBackground(UIManager.getColor("Panel.background"));
      bp.setIcon(null);
    }
    else if(action.equals("DESKTOP_DEFAUT"))
    { 
      setBackground(UIManager.getColor("Desktop.background"));
      bp.setIcon(null);
    }

    setBackgroundPainter(bp);
  }

  public void updateUI()
  {
    if(isShowing()) setBackground(null);
    super.updateUI();
  }
}
