/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuDesktopPreferencesPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * A panel where the user can choose which elements to
 * put on his desktop.
 */

/*
 * see below.
 */

public class BuDesktopPreferencesPanel
       extends BuAbstractPreferencesPanel
{
  private final static int NBL=18;

  private final static String[] CODES=new String[]
  {
    "leftcolumn.visible",
    "specificbar.visible",
    "rightcolumn.visible",
    "statusbar.visible",
    "columns.swapped",
    "assistant.visible",
    "button.icon",
    "tool.icon",
    "button.text",
    "tool.text",
    "toolbar.floatable",
    "splashscreen.visible",
    "toolbar.rollover",
    "desktop.tabbed",
    "desktop.grid",
    "desktop.snap",
    "desktop.dots",
    "antialias.all",
  };

  private final static boolean[] DEFAUTS=new boolean[]
  {
    true,
    true,
    true,
    true,
    true,
    true,
    false,
    true,
    true,
    true,
    true,
    true,
    true,
    true,
    false,
    false,
    false,
    false,
    false,
  };

  private final static String[] INTITULES=new String[]
  {
    "Colonne gauche",
    "Outils spcifiques",
    "Colonne droite",
    "Barre d'tat",
    "Colonnes inverses",
    "Assistant",
    "Icne de bouton",
    "Icne d'outil",
    "Texte de bouton",
    "Texte d'outil",
    "Dtachement des barres",
    "Ecran d'accueil",
    "Survol des barres",
    "Bureau  onglets",
    "Grille",
    "Magntisme",
    "Points",
    "Antialias",
  };

  BuCommonInterface appli_;
  BuPreferences     options_;
  BuCheckBox[]      cbEtats_;

  public String getTitle()
    { return BuResource.BU.getString("Bureau"); }

  // Constructeur
  
  public BuDesktopPreferencesPanel(BuCommonInterface _appli)
  {
    super();
    appli_     =_appli;
    options_   =BuPreferences.BU;

    BuPanel q=new BuPanel();
    q.setLayout(new BuGridLayout(2,5,2));
    q.setBorder(new EmptyBorder(0,5,0,5));

    cbEtats_=new BuCheckBox[NBL];
    for(int i=0;i<NBL;i++)
    {
      cbEtats_[i]=new BuCheckBox(BuResource.BU.getString(INTITULES[i]));
      cbEtats_[i].setName("cb"+CODES[i].replace('.','_').toUpperCase());
      cbEtats_[i].setActionCommand(CODES[i].replace('.','_').toUpperCase());
      q.add(cbEtats_[i]);
    }

    BuPanel p=new BuPanel();
    p.setBorder
      (new CompoundBorder
       (new TitledBorder(getTitle()),
	new EmptyBorder(5,5,5,5)));
    p.setLayout(new BuBorderLayout());
    p.add(new BuScrollPane(q),BuBorderLayout.CENTER);

    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    setPreferredSize(new Dimension(200,200));
    add(p,BuBorderLayout.CENTER);

    for(int i=0;i<NBL;i++)
      cbEtats_[i].setSelected(DEFAUTS[i]);

    updateComponents();
  }
  
  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  /*
  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
    options_.applyOn(appli_);
  }
  */

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }
  
  // Methodes privees
  
  private void fillTable()
  {
    for(int i=0;i<NBL;i++)
      options_.putBooleanProperty(CODES[i],cbEtats_[i].isSelected());
  }

  private void updateComponents()
  {
    for(int i=0;i<NBL;i++)
      cbEtats_[i].setSelected
	(options_.getBooleanProperty(CODES[i],DEFAUTS[i]));
  }
}

