/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuDialog.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

/**
 * An abstract class for dialogs.
 * Better to use than JOptionPane.
 */

public abstract class BuDialog
       extends JDialog
       implements ActionListener
{
  protected Frame          frame_;
  protected JComponent     content_;
  protected BuBorderLayout layout_;
  protected int            reponse_;
  protected Object         message_;

  public BuDialog(BuCommonInterface      _parent,
		  BuInformationsSoftware _isoft,
		  String                 _titre)
  {
    this(_parent,_isoft,_titre,null);
  }

  public BuDialog(BuCommonInterface      _parent,
		  BuInformationsSoftware _isoft,
		  String                 _titre,
		  Object                 _message)
  {
    super(_parent==null ? null : _parent.getFrame());

    if(_parent!=null) frame_=_parent.getFrame();
    reponse_=JOptionPane.CANCEL_OPTION;

    BuBorderLayout mlo=new BuBorderLayout();

    JComponent mct=(JComponent)getContentPane();
    mct.setLayout(mlo);

    if((_parent!=null)&&(_isoft==null))
      _isoft=_parent.getInformationsSoftware();

    if(_isoft!=null)
    {
      BuPanel mpi=new BuPanel();
      mpi.setLayout(new BuVerticalLayout());
      mpi.setBorder(new EmptyBorder(5,5,5,5)); //10,10,10,10));
      mpi.setBackground(mpi.getBackground().darker());
      mpi.setForeground(Color.white);
      
      BuPicture ml_logo=null;
      if(_isoft.logo!=null)
      {
	ml_logo=new BuPicture(_isoft.logo);
	// setIconImage(BuResource.BU.getImage("dialog"));
      }
      
      BuLabelMultiLine ml_isoft=new BuLabelMultiLine
	(_isoft.name+"\nversion: "+_isoft.version+"\ndate: "+_isoft.date);
      ml_isoft.setFont(BuLib.deriveFont("Label",Font.PLAIN,-2));
      ml_isoft.setForeground(Color.white);

      if(ml_logo!=null) mpi.add(ml_logo);
      mpi.add(ml_isoft);
      mct.add(mpi,BuBorderLayout.WEST);
    }

    layout_=new BuBorderLayout();
    layout_.setHgap(5);
    layout_.setVgap(5);
    content_=new BuPanel();
    content_.setBorder(new EmptyBorder(10,10,5,10));
    content_.setLayout(layout_);

    JComponent lbMessage=null;

    if(_message==null)
      _message=getComponent();
    if(_message==null)
      _message="Please overload getComponent() in BuDialog.";

    message_=_message;

    if(_message instanceof JComponent)
      lbMessage=(JComponent)_message;
    else
    {
      String text=_message.toString();
      if(text.startsWith("<HTML>"))
      {
	BuEditorPane html=new BuEditorPane();
	EditorKit   ek  =html.getEditorKitForContentType("text/html");
	Document    dk  =ek.createDefaultDocument();

	try { ek.read(new StringReader(text),dk,0); }
	catch(Exception ex) { ex.printStackTrace(); }

	html.setOpaque(false);
	html.setBorder(new EmptyBorder(0,0,0,0));
	html.setEditable(false);
	html.setEditorKit(ek);
	html.setDocument(dk);
	lbMessage=html;
      }
      else
      {
	lbMessage=new BuLabelMultiLine(text);
	lbMessage.setFont(BuLib.deriveFont("Label",Font.PLAIN, 0));
	lbMessage.setBorder(new EmptyBorder(5,5,5,5));
      }
    }

    BuScrollPane spm=new BuScrollPane(lbMessage);
    int h=lbMessage.getFont().getSize();
    spm.getHorizontalScrollBar().setUnitIncrement (150);
    spm.getVerticalScrollBar()  .setUnitIncrement (h+1);
    content_.add(spm,BuBorderLayout.CENTER);

    mct.add(content_,BuBorderLayout.CENTER);

    if(lbMessage.isFocusTraversable())
      lbMessage.requestFocus();
    else
      transferFocus();

    setResizable(false);
    setTitle(_titre);
    setModal(true);
  }

  // Anti-aliasing

  public final void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  public abstract JComponent getComponent();

  public abstract void actionPerformed(ActionEvent _evt);

  public final int activate()
  {
    JComponent mct=(JComponent)getContentPane();

    Dimension d=mct.getPreferredSize();
    Dimension ecran=getToolkit().getScreenSize();

    //d.width+=30; d.height+=30;

    if(!(message_ instanceof JComponent))
    {
      if(d.width <450) d.width =450;
      if(d.width >550) d.width =550;
      if(d.height<150) d.height=150;
      if(d.height>250) d.height=250;
    }

    if(d.width >ecran.width *3/4) d.width=ecran.width  *3/4;
    if(d.height>ecran.height*3/4) d.height=ecran.height*3/4;

    boolean resizable=isResizable();
    setResizable(true);
    //setSize(d);
    mct.setPreferredSize(d);
    pack();
    setLocation((ecran.width-d.width)/2,(ecran.height-d.height)/2);
    setResizable(resizable);

    // SWING 1.03: LightweightDispatcher.retargetMouseEvent
    // try { setVisible(true); }
    // catch(NullPointerException ex) { }
    show();

    return reponse_;
  }

  public void show()
  {
    BuLib.computeMnemonics(getRootPane(),this);
    super.show();
  }
}

