/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuDirWinPreferencesPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.*;

/**
 * A panel where the user can choose your directories and the
 * size of the main window.
 * @version      0.01, 24/06/99
 * @author       Guillaume Desnoix
 */

/**
 * directory.open
 * directory.save
 * window.size
 */

public class BuDirWinPreferencesPanel
       extends BuAbstractPreferencesPanel
       implements ActionListener
{
  protected BuCommonInterface appli_;
  protected BuPreferences     options_;

  protected JTextField        tfDirOpen_,tfDirSave_;
  protected JButton           btDirOpen_,btDirSave_;
  protected JComboBox         chWinSize_;
  
  public String getTitle()
    { return BuResource.BU.getString("Position"); }

  // Constructeur

  public BuDirWinPreferencesPanel(BuCommonInterface _appli)
  {
    super();
    appli_=_appli;
    options_=BuPreferences.BU;

    BuVerticalLayout lo=new BuVerticalLayout();
    lo.setVgap(5);

    BuPanel p=new BuPanel();
    p.setLayout(lo);

    BuBorderLayout lo1=new BuBorderLayout();
    lo1.setVgap(5);
    lo1.setHgap(5);
    BuPanel p1=new BuPanel();
    p1.setLayout(lo1);
    p1.setBorder(new EmptyBorder(5,5,5,5));
    
    tfDirOpen_=new BuTextField(25);
    tfDirOpen_.setName("tfDIRECTORY_OPEN");
    tfDirOpen_.setEditable(false);
    btDirOpen_=new BuButton();
    btDirOpen_.setRequestFocusEnabled(false);
    btDirOpen_.setIcon(getDirectoryIcon());
    btDirOpen_.setName("btDIRECTORY_OPEN");
    btDirOpen_.setMargin(new Insets(0,0,0,0));
    btDirOpen_.addActionListener(this);
    p1.add(new BuLabel(BuResource.BU.getString("Ouverture:")),
	   BuBorderLayout.NORTH);
    p1.add(tfDirOpen_,BuBorderLayout.CENTER);
    p1.add(btDirOpen_,BuBorderLayout.EAST);

    BuBorderLayout lo2=new BuBorderLayout();
    lo2.setVgap(5);
    lo2.setHgap(5);
    BuPanel p2=new BuPanel();
    p2.setLayout(lo2);
    p2.setBorder(new EmptyBorder(5,5,5,5));

    tfDirSave_=new BuTextField(25);
    tfDirSave_.setName("tfDIRECTORY_SAVE");
    tfDirSave_.setEditable(false);
    btDirSave_=new BuButton();
    btDirSave_.setRequestFocusEnabled(false);
    btDirSave_.setIcon(getDirectoryIcon());
    btDirSave_.setName("btDIRECTORY_SAVE");
    btDirSave_.setMargin(new Insets(0,0,0,0));
    btDirSave_.addActionListener(this);
    p2.add(new BuLabel(BuResource.BU.getString("Enregistrement:")),
	   BuBorderLayout.NORTH);
    p2.add(tfDirSave_,BuBorderLayout.CENTER);
    p2.add(btDirSave_,BuBorderLayout.EAST);

    BuVerticalLayout lo3=new BuVerticalLayout();
    lo3.setVgap(5);
    BuPanel p3=new BuPanel();
    p3.setLayout(lo3);
    p3.setBorder(new TitledBorder(BuResource.BU.getString("Rpertoires")));
    p3.add(p1);
    p3.add(p2);

    chWinSize_=new BuComboBox();
    chWinSize_.setName("chWINDOW_SIZE");
    chWinSize_.addItem(BuResource.BU.getString("normale" ));
    chWinSize_.addItem(BuResource.BU.getString("maximale"));
    chWinSize_.addItem(BuResource.BU.getString("dernire"));
    chWinSize_.addItem(BuResource.BU.getString("minimale"));

    BuBorderLayout lo4=new BuBorderLayout();
    lo4.setVgap(5);
    lo4.setHgap(5);
    BuPanel p4=new BuPanel();
    p4.setLayout(lo4);
    p4.add(new BuLabel(BuResource.BU.getString("Taille:")),
	   BuBorderLayout.NORTH);
    p4.add(chWinSize_,BuBorderLayout.CENTER);
    p4.setBorder
      (new CompoundBorder
       (new TitledBorder(BuResource.BU.getString("Fentre principale")),
	new EmptyBorder(5,5,5,5)));

    p.add(p3);
    p.add(p4);
    // p.add(new BuPanel());

    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    add(p,BuBorderLayout.CENTER);

    chWinSize_.setSelectedIndex(1);
    updateComponents();
  }

  private final static Icon getDirectoryIcon()
  {
    Icon r=UIManager.getIcon("FileView.directoryIcon");
    if(r==null) r=BuResource.BU.getIcon("ouvrir");
    return r;
  }

  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();
    System.err.println("SOURCE="+source.getName());
    System.err.println("ACTION="+_evt.getActionCommand());

    if(source==btDirOpen_)
      tfDirOpen_.setText(getDirectory(tfDirOpen_.getText()));
	
    if(source==btDirSave_)
      tfDirSave_.setText(getDirectory(tfDirSave_.getText()));
	
    // chDirWin.setEnabled(cbExterne.isSelected());
  }
  
  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
    options_.applyOn(appli_);
  }

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
    options_.putStringProperty("directory.open",tfDirOpen_.getText());
    options_.putStringProperty("directory.save",tfDirSave_.getText());
    options_.putIntegerProperty
	("window.size",chWinSize_.getSelectedIndex());
  }

  private void updateComponents()
  {
    tfDirOpen_.setText(options_.getStringProperty("directory.open"));
    tfDirSave_.setText(options_.getStringProperty("directory.save"));
    chWinSize_.setSelectedIndex(options_.getIntegerProperty("window.size"));
  }

  private String getDirectory(String _default)
  {
    String r=BuLib.expandedPath(_default);

    // System.err.println("### FD begin");
    BuFileChooser chooser=new BuFileChooser();
    chooser.setDialogTitle(BuResource.BU.getString("Repertoire"));
    chooser.setFileSelectionMode(BuFileChooser.DIRECTORIES_ONLY);
    chooser.setFileHidingEnabled(true);
    chooser.setMultiSelectionEnabled(false);
    chooser.resetChoosableFileFilters();

    File f=new File(r);
    if(f.exists()) chooser.setCurrentDirectory(f);

    int returnVal=chooser.showDialog(this,BuResource.BU.getString("Slectionner"));
    // System.err.println("### FD end");

    if(returnVal == BuFileChooser.APPROVE_OPTION)
    {
      f=chooser.getSelectedFile();
      // System.err.println("### f="+f);
      if(f!=null) r=f.getPath();
    }

    return BuLib.reducedPath(r);
  }
}
