/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuExplorer.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class BuExplorer
       extends BuPanel
    implements ActionListener, ListSelectionListener
{
  Icon i_disk     ;
  Icon i_floppy   ;
  Icon i_directory;
  Icon i_file     ;
  Icon i_home     ;
  Icon i_parent   ;

  BuCommonInterface app_;
  Vector            listeners_;

  BuComboBox  roots_;
  BuLabel     current_;
  BuEmptyList dirs_;
  BuComboBox  filters_;
  BuEmptyList files_;

  BuPopupMenu pmCurrent_;
  BuPopupMenu pmDirs_;
  BuPopupMenu pmFiles_;

  BuMenuItem  miCvsAdd_;
  BuMenuItem  miCvsUpdate_;
  BuMenuItem  miCvsCommit_;
  BuMenuItem  miAntBuild_;
  BuMenuItem  miMake_;

  public BuExplorer()
  {
    this(null);
  }

  public BuExplorer(BuCommonInterface _app)
  {
    app_      =_app;
    listeners_=new Vector(1,1);

    MouseListener mh=new MouseHandler();

    roots_=new BuComboBox();
    roots_.setMaximumRowCount(20);
    roots_.addActionListener(this);

    current_=new BuLabel();
    current_.setBorder(new EmptyBorder(0,6,0,6));
    current_.addMouseListener(mh);

    dirs_=new BuEmptyList();
    dirs_.setEmptyText("No directory");
    //dirs_.setBorder(new EmptyBorder(3,3,3,3));
    dirs_.addMouseListener(mh);
    dirs_.addListSelectionListener(this);

    filters_=new BuComboBox();
    filters_.setMaximumRowCount(20);
    filters_.addActionListener(this);

    files_=new BuEmptyList();
    files_.setEmptyText("No file");
    //files_.setBorder(new EmptyBorder(3,3,3,3));
    files_.addMouseListener(mh);
    files_.addListSelectionListener(this);
    
    pmCurrent_=new BuPopupMenu(true,BuResource.BU.getString("Rpertoire"));
    pmCurrent_.addMenuItem(BuResource.BU.getString("Rafrachir"),"RAFRAICHIR"      ,true);
    pmCurrent_.addMenuItem(BuResource.BU.getString("Crer..."  ),"CREER_REPERTOIRE",true);
    pmCurrent_.addSeparator();
    miCvsUpdate_=pmCurrent_.addMenuItem("Update","CVS_UPDATE",false);
    miCvsCommit_=pmCurrent_.addMenuItem("Commit","CVS_COMMIT",false);
    miAntBuild_ =pmCurrent_.addMenuItem("Build" ,"ANT_UPDATE",false);
    miMake_     =pmCurrent_.addMenuItem("Make"  ,"MAKE"      ,false);

    pmDirs_=new BuPopupMenu(true,BuResource.BU.getString("Rpertoires"));
    pmDirs_.addMenuItem(BuResource.BU.getString("Ouvrir"     ),"OUVRIR_REPERTOIRE"  ,true);
    pmDirs_.addSeparator();
    pmDirs_.addMenuItem(BuResource.BU.getString("Renommer..."),"RENOMMER_REPERTOIRE",true);
    pmDirs_.addMenuItem(BuResource.BU.getString("Dtruire..."),"DETRUIRE_REPERTOIRE",true);

    pmFiles_=new BuPopupMenu(true,BuResource.BU.getString("Fichiers"));
    pmFiles_.addMenuItem(BuResource.BU.getString("Ouvrir"     ),"OUVRIR_FICHIER"  ,true);
    pmFiles_.addSeparator();
    pmFiles_.addMenuItem(BuResource.BU.getString("Renommer..."),"RENOMMER_FICHIER",true);
    pmFiles_.addMenuItem(BuResource.BU.getString("Dtruire..."),"DETRUIRE_FICHIER",true);
    pmFiles_.addSeparator();
    miCvsAdd_=pmFiles_.addMenuItem("Add","CVS_ADD",false);

    setLayout(new BuVerticalLayout(2,true,true));

    add(roots_);
    add(current_);

    BuScrollPane spDirs=new BuScrollPane(dirs_);
    spDirs.setPreferredSize(new Dimension(170,220));
    add(spDirs);

    add(filters_);

    BuScrollPane spFiles=new BuScrollPane(files_);
    spFiles.setPreferredSize(new Dimension(170,220));
    add(spFiles);

    addMtab();
    addDefaultFilters();

    new DndSource(DndRequestSelection.SINGLETON).add(files_);

    updateLnf();
  }

  public void updateUI()
  {
    super.updateUI();
    try
    {
      updateLnf();
      pmCurrent_.updateUI();
      pmDirs_   .updateUI();
      pmFiles_  .updateUI();
    }
    catch(Exception ex) { }
  }

  private Font getETF()
  {
    return BuLib.deriveFont("List.font",Font.PLAIN,-2);
  }

  private void updateLnf()
  {
    i_disk     =UIManager.getIcon("FileView.hardDriveIcon");
    i_floppy   =UIManager.getIcon("FileView.floppyDriveIcon");
    i_directory=UIManager.getIcon("FileView.directoryIcon");
    i_file     =UIManager.getIcon("FileView.fileIcon");
    i_home     =UIManager.getIcon("FileChooser.homeFolderIcon");
    i_parent   =UIManager.getIcon("FileChooser.upFolderIcon");

    if(i_file!=null) DndIcons.put(File.class,i_file);

    Font ft=getETF();
    roots_  .setFont(ft);
    current_.setFont(ft);
    dirs_   .setFont(ft);
    filters_.setFont(ft);
    files_  .setFont(ft);

    roots_  .setRenderer    (new FileCR(true));
    dirs_   .setCellRenderer(new FileCR(false));
    files_  .setCellRenderer(new FileCR(false));
    filters_.setRenderer    (new FilterCR());
  }

  protected void updateMenuCurrent()
  {
    miCvsUpdate_.setEnabled
      (new File(getCurrentDirectory(),"CVS"      ).exists());
    miCvsCommit_.setEnabled
      (new File(getCurrentDirectory(),"CVS"      ).exists());
    miAntBuild_ .setEnabled
      (new File(getCurrentDirectory(),"build.xml").exists());
    miMake_     .setEnabled
      (new File(getCurrentDirectory(),"Makefile" ).exists());
  }

  protected void updateMenuDirs()
  {
    int l=dirs_.getSelectedValues().length;
    BuActionEnabler.setEnabledForAction(pmDirs_,"OUVRIR_REPERTOIRE"  ,l==1);
    BuActionEnabler.setEnabledForAction(pmDirs_,"DETRUIRE_REPERTOIRE",l==1);
    BuActionEnabler.setEnabledForAction(pmDirs_,"RENOMMER_REPERTOIRE",l==1);
  }

  protected void updateMenuFiles()
  {
    int l=files_.getSelectedValues().length;
    BuActionEnabler.setEnabledForAction(pmFiles_,"OUVRIR_FICHIER"  ,l>0);
    BuActionEnabler.setEnabledForAction(pmFiles_,"DETRUIRE_FICHIER",l==1);
    BuActionEnabler.setEnabledForAction(pmFiles_,"RENOMMER_FICHIER",l==1);
    miCvsAdd_.setEnabled(new File(getCurrentDirectory(),"CVS").exists());
  }

  public void addActionListener(ActionListener _l)
  { listeners_.addElement(_l); }

  public void removeActionListener(ActionListener _l)
  { listeners_.removeElement(_l); }

  private class MouseHandler extends MouseAdapter
  {
    public void mouseClicked(MouseEvent _evt)
    {
      Object source=_evt.getSource();

      if(_evt.getClickCount()==2)
      {
	if(  (source==dirs_)
	   &&(dirs_.getSelectedValues().length==1))
	  openDirectory();
	else
	if(  (source==files_)
	   &&(files_.getSelectedValues().length==1))
	  openFile();
      }
      else
      if((_evt.getModifiers()&InputEvent.BUTTON3_MASK)!= 0)
      {
	if(source==current_)
	{
	  updateMenuCurrent();
	  Point p=SwingUtilities.convertPoint
	    (current_,new Point(0,current_.getHeight()),BuExplorer.this);
	  pmCurrent_.show(BuExplorer.this,4,p.y+10);//p.x,p.y);
	}
	else
	if(source==dirs_)
	{
	  updateMenuDirs();
	  Point p=SwingUtilities.convertPoint
	    (dirs_,new Point(_evt.getX(),_evt.getY()),BuExplorer.this);
	  pmDirs_.show(BuExplorer.this,4,p.y+10);//p.x,p.y);
	}
	else
	if(source==files_)
	{
	  updateMenuFiles();
	  Point p=SwingUtilities.convertPoint
	    (files_,new Point(_evt.getX(),_evt.getY()),BuExplorer.this);
	  pmFiles_.show(BuExplorer.this,4,p.y+10);//p.x,p.y);
	}
      }
    }
  }

  private class Root extends File
  {
    //private String device_;
    private String type_;

    public Root(String _mount, String _device, String _type)
    {
      super(_mount);
      //device_=_device;
      type_  =_type;
    }

    public String getText()
    {
      String r;

      if("..".equals(type_)) r="..";
      else r=BuLib.reducedPath(getAbsolutePath());

      return r;
    }

    public Icon getIcon()
    {
      Icon r=i_disk;

      if(".."       .equals(type_         )) r=i_parent;
      else
      if("home"     .equals(type_         )) r=i_home;
      else
      if("directory".equals(type_         )) r=i_directory;
      else
      if("floppy"   .equals(this.getName())) r=i_floppy;
      /*
      else
      if("nfs".equals(type_)) r=BuResource.BU.getIcon("cacher",16);
      */

      return r;
    }
  }

  /*
  private final class FileCR
    implements ListCellRenderer
  {
    private boolean cb_;
    private JLabel  dl_;

    public FileCR(boolean _cb)
    {
      cb_=_cb;
      dl_=new BuLabel();
    }

    public Component getListCellRendererComponent
      (JList   _list,
       Object  _value,
       int     _index,
       boolean _selected,
       boolean _focus)
    {
      JLabel r=null;

      Object o=(cb_
		? UIManager.get("ComboBox.renderer")
		: UIManager.get("List.cellRenderer"));

      if(o instanceof ListCellRenderer)
      {
	o=((ListCellRenderer)o).
	  getListCellRendererComponent(_list,_value,_index,_selected,_focus);
	if(o instanceof JLabel) r=(JLabel)o;
      }

      if(r==null)
      {
        r=dl_;
      
	if(_selected)
        {
	  r.setOpaque(true);
	  r.setBackground(_list.getSelectionBackground());
	  r.setForeground(_list.getSelectionForeground());
	  if(cb_)
	    r.setBorder(UIManager.getBorder("ComboBox.selectedCellBorder"));
	  else
	    r.setBorder(UIManager.getBorder("List.selectedCellBorder"));
	}
	else
        {
	  r.setOpaque(false);
	  r.setBackground(_list.getBackground());
	  r.setForeground(_list.getForeground());
	  if(cb_)
	    r.setBorder(UIManager.getBorder("ComboBox.cellBorder"));
	  else
	    r.setBorder(UIManager.getBorder("List.cellBorder"));
	}
      }

      r.setFont(getETF());

      File f=(File)_value;

      Icon   icon=BuResource.BU.getIcon("detruire",16);
      String text=f.getName();

      if(f instanceof Root)
      {
	icon=((Root)f).getIcon();
	text=((Root)f).getText();
      }
      else
      if(f.isDirectory())
      {
	icon=i_directory;
      }
      else
      if(f.isFile())
      {
	icon=i_file;
      }
      else
      {
	r.setForeground(Color.red);
      }
	
      r.setText(text);
      r.setIcon(icon);
      r.setDisabledIcon(icon);

      return r;
    }
  }
  */

  private final class FileCR
    extends BuAbstractCellRenderer
  {
    public FileCR(boolean _cb)
    {
      super(_cb
	    ? BuAbstractCellRenderer.COMBOBOX
	    : BuAbstractCellRenderer.LIST);
    }

    public Component getListCellRendererComponent
      (JList   _list,
       Object  _value,
       int     _index,
       boolean _selected,
       boolean _focus)
    {
      BuLabel r=(BuLabel)super.getListCellRendererComponent
	  (_list,_value,_index,_selected,_focus);

      r.setFont(getETF());

      File f=(File)_value;

      Icon   icon=BuResource.BU.getIcon("detruire",16);
      String text=f.getName();

      if(f instanceof Root)
      {
	icon=((Root)f).getIcon();
	text=((Root)f).getText();
      }
      else
      if(f.isDirectory())
      {
	icon=i_directory;
      }
      else
      if(f.isFile())
      {
	icon=i_file;
      }
      else
      {
	r.setForeground(Color.red);
      }
	
      r.setText(text);
      r.setIcon(icon);
      r.setDisabledIcon(icon);

      return r;
    }
  }

  /*
  private final class FilterCR
    implements ListCellRenderer
  {
    private JLabel  dl_;

    public FilterCR()
    {
      dl_=new BuLabel();
    }

    public Component getListCellRendererComponent
      (JList   _list,
       Object  _value,
       int     _index,
       boolean _selected,
       boolean _focus)
    {
      JLabel r=null;

      Object o=UIManager.get("ComboBox.renderer");

      if(o instanceof ListCellRenderer)
      {
	o=((ListCellRenderer)o).
	  getListCellRendererComponent(_list,_value,_index,_selected,_focus);
	if(o instanceof JLabel) r=(JLabel)o;
      }

      if(r==null)
      {
        r=dl_;
      
	if(_selected)
        {
	  r.setOpaque(true);
	  r.setBackground(_list.getSelectionBackground());
	  r.setForeground(_list.getSelectionForeground());
	  r.setBorder(UIManager.getBorder("ComboBox.selectedCellBorder"));
	}
	else
        {
	  r.setOpaque(false);
	  r.setBackground(_list.getBackground());
	  r.setForeground(_list.getForeground());
	  r.setBorder(UIManager.getBorder("ComboBox.cellBorder"));
	}
      }

      BuFileFilter f=(BuFileFilter)_value;
      r.setText(f.getDescription());
      r.setFont(getETF());
      return r;
    }
  }
  */

  private final class FilterCR
    extends BuAbstractCellRenderer
  {
    public FilterCR()
    {
      super(BuAbstractCellRenderer.COMBOBOX);
    }

    public Component getListCellRendererComponent
      (JList   _list,
       Object  _value,
       int     _index,
       boolean _selected,
       boolean _focus)
    {
      BuLabel r=(BuLabel)super.getListCellRendererComponent
	  (_list,_value,_index,_selected,_focus);

      BuFileFilter f=(BuFileFilter)_value;
      r.setText(f.getDescription());
      r.setFont(getETF());
      return r;
    }
  }

  protected static String clean(String _dir)
  {
    String r=_dir;
    if(r!=null)
    {
      r=r.trim();
      if("".equals(r))
      {
	r=null;
      }
      else
      {
        r=FuLib.expandedPath(r);
        if((r.length()>1)&&r.endsWith(File.separator))
	  r=r.substring(0,r.length()-1);
      }
    }
    return r;
  }

  protected void addMtab()
  {
    String od=clean(BuPreferences.BU.getStringProperty("directory.open",null));
    if(od!=null)
      roots_.addItem(new Root(od,od,"directory"));

    String sd=clean(BuPreferences.BU.getStringProperty("directory.save",null));
    if((sd!=null)&&!sd.equals(od))
      roots_.addItem(new Root(sd,sd,"directory"));

    String ud=clean(FuLib.getSystemProperty("user.dir"));
    if((ud!=null)&&!ud.equals(sd)&&!ud.equals(od))
      roots_.addItem(new Root(ud,ud,"directory"));

    String hd=clean(FuLib.getSystemProperty("user.home"));
    if((hd!=null)&&!hd.equals(ud)&&!hd.equals(sd)&&!hd.equals(od))
      roots_.addItem(new Root(hd,hd,"home"));

    if(FuLib.isUnix())
    {
      try
      {
	LineNumberReader rin=new LineNumberReader
	  (new FileReader("/etc/mtab"));

	String l;
	while((l=rin.readLine())!=null)
	{
	  l=l.trim();
	  if(l.startsWith("#")) continue;
	
	  try
	  {
	    StringTokenizer st=new StringTokenizer(l);
	    String device=st.nextToken();
	    String mount =st.nextToken();
	    String type  =st.nextToken();
	    if(  !"none"    .equals(mount)
	       &&!"swap"    .equals(type)
	       &&!"proc"    .equals(type)
	       &&!"devpts"  .equals(type)
	       &&!"usbdevfs".equals(type))
	      roots_.addItem(new Root(mount,device,type));
	  }
	  catch(Exception ex) { }
	}
      }
      catch(Throwable th) { } //IOException ex
    }

    try
    {
      Method m=File.class.getMethod("listRoots",new Class[0]);
      File[] l=(File[])m.invoke(null,new Object[0]);
      for(int i=0;i<l.length;i++)
      {
	String p=l[i].getAbsolutePath();
	if(!p.equals("/"))
	  roots_.addItem
	    (new Root(p,p,"disk"));
      }
    }
    catch(NoSuchMethodException ex1) { }
    catch(Exception ex2) { ex2.printStackTrace(); }
  }

  public void addDefaultFilters()
  {
    addFilter(new BuFileFilterAll());
    //addFilter(new BuFileFilter("java","Java files"));
  }

  private Vector filters_list=new Vector(5);

  public void addFilter(BuFileFilter _ff)
  {
    //_ff.setExtensionListInDescription(false);
    if(!filters_list.contains(_ff))
    {
      filters_.addItem(_ff);
      filters_list.addElement(_ff);
    }
  }

  private class Model
          implements ListModel
  {
    private File[] els_;
    private int    nb_;
    private Vector ldl_;

    public Model(File[] _els, int _nb)
    {
      els_=_els;
      nb_ =_nb;
      ldl_=new Vector();
    }

    public int getSize() { return nb_; }
    public Object getElementAt(int _i) { return els_[_i]; }

    public void addListDataListener(ListDataListener _l)
    { ldl_.addElement(_l); }

    public void removeListDataListener(ListDataListener _l)
    { ldl_.removeElement(_l); }
  }

  public File getCurrentRoot()
  {
    return (File)roots_.getSelectedItem();
  }

  public File getCurrentDirectory()
  {
    return new File(BuLib.expandedPath(current_.getText()));
  }

  public File getSelectedDirectory()
  {
    return (File)dirs_.getSelectedValue();
  }

  public File getSelectedFile()
  {
    return (File)files_.getSelectedValue();
  }

  // actions

  public void refresh()
  {
    updateLists(getCurrentDirectory());
  }
  
  public void openDirectory()
  {
    updateLists(getSelectedDirectory());
  }

  public void openFile()
  {
    Object[] selected=files_.getSelectedValues();
    if(selected==null) return;

    for(int i=0;i<selected.length;i++)
      if(((File)selected[i]).canRead())
      {
	ActionEvent evt=new ActionEvent
	  (this,ActionEvent.ACTION_PERFORMED,
	   "OUVRIR("+((File)selected[i]).getAbsolutePath()+")");
	for(Enumeration e=listeners_.elements(); e.hasMoreElements(); )
	  ((ActionListener)e.nextElement()).actionPerformed(evt);
      }
  }

  public void addExtraFileCommand(String _s,String _cmd,BuIcon _icon)
  {
    pmFiles_.addSeparator();
    pmFiles_.addMenuItem(_s,"EXTRA_FICHIER("+_cmd+")",_icon,true);
  }

  public void extraFile(String _cmd)
  {
    Object[] selected=files_.getSelectedValues();
    if(selected==null) return;

    for(int i=0;i<selected.length;i++)
      if(((File)selected[i]).canRead())
      {
	ActionEvent evt=new ActionEvent
	  (this,ActionEvent.ACTION_PERFORMED,
	   _cmd+"("+((File)selected[i]).getAbsolutePath()+")");
	for(Enumeration e=listeners_.elements(); e.hasMoreElements(); )
	  ((ActionListener)e.nextElement()).actionPerformed(evt);
      }
  }

  public void createDirectory()
  {
    BuDialogInput d=new BuDialogInput
      (null,null,"Crer","Nom du sous-rpertoire:","");
    if(d.activate()==JOptionPane.OK_OPTION)
    {
      String name   =d.getValue();
      File   current=getCurrentDirectory();

      if(new File(current,name).mkdir())
	refresh();
      else
      {
	new BuDialogError
	  (null,null,
	   "Impossible de crer le sous-rpertoire "+name+"\n"+
	   "dans le rpertoire "+current.getAbsolutePath())
	  .activate();
      }
    }
  }

  public void renameDirectory()
  {
    File selected=getSelectedDirectory();

    BuDialogInput d=new BuDialogInput
      (null,null,"Renommer","Nouveau nom du rpertoire:",selected.getName());
    if(d.activate()==JOptionPane.OK_OPTION)
    {
      String name  =d.getValue();
      File   target=new File(selected.getParent(),name);
      
      if(selected.renameTo(target))
	refresh();
      else
      {
	new BuDialogError
	  (null,null,
	   "Impossible de renommer le rpertoire "+selected.getName()+"\n"+
	   "en "+name)
	  .activate();
      }
    }
  }

  public void renameFile()
  {
    File selected=getSelectedFile();

    BuDialogInput d=new BuDialogInput
      (null,null,"Renommer","Nouveau nom du fichier:",selected.getName());
    if(d.activate()==JOptionPane.OK_OPTION)
    {
      String name  =d.getValue();
      File   target=new File(selected.getParent(),name);
      
      if(selected.renameTo(target))
	refresh();
      else
      {
	new BuDialogError
	  (null,null,
	   "Impossible de renommer le fichier "+selected.getName()+"\n"+
	   "en "+name)
	  .activate();
      }
    }
  }

  public void deleteDirectory()
  {
    File selected=getSelectedDirectory();

    BuDialogConfirmation d=new BuDialogConfirmation
      (null,null,"Voulez-vous vraiment dtruire\n"+
       "le rpertoire "+selected.getName()+" ?");
    
    if(d.activate()==JOptionPane.OK_OPTION)
    {
      if(selected.delete())
	refresh();
      else
      {
	new BuDialogError
	  (null,null,
	   "Le rpertoire "+selected.getName()+" n'a pas pu\n"+
	   "tre dtruit.")
	  .activate();
      }
    }
  }

  public void deleteFile()
  {
    File selected=getSelectedFile();

    BuDialogConfirmation d=new BuDialogConfirmation
      (null,null,"Voulez-vous vraiment dtruire\n"+
       "le fichier "+selected.getName()+" ?");
    
    if(d.activate()==JOptionPane.OK_OPTION)
    {
      if(selected.delete())
	refresh();
      else
      {
	new BuDialogError
	  (null,null,
	   "Le fichier "+selected.getName()+" n'a pas pu\n"+
	   "tre dtruit.")
	  .activate();
      }
    }
  }

  // utils

  private void updateLists(final File _dir)
  {
    try
    {

    if((_dir==null)||!_dir.isDirectory()) return;

    //BuLib.setRecursiveEnabled(BuExplorer.this,false);
    new BuTaskOperation(app_,_dir.getName())
      { public void act()
	{
	  setPriority(Thread.MIN_PRIORITY);
	  updateLists0(_dir);
	  //BuLib.setRecursiveEnabled(BuExplorer.this,true);
	}
      }
      .start();

    }
    catch(Throwable th) { }
  }

  private final void setProgression(int _p)
  {
    if(app_!=null)
      app_.getMainPanel().setProgression(_p);
  }

  private final static Cursor WAIT_CURSOR=Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);

  private void updateLists0(final File _dir)
  {
    if((_dir==null)||!_dir.isDirectory()) return;

    setProgression(10);
    Cursor old_cursor=getCursor();
    if(!WAIT_CURSOR.equals(old_cursor))
      setCursor(WAIT_CURSOR);

    current_.setText(BuLib.reducedPath(_dir.getAbsolutePath()));
    current_.setToolTipText(_dir.getAbsolutePath());
    updateMenuCurrent();

    //System.err.println("read");
    String[] els=_dir.list();
    if(els==null) els=new String[0];

    setProgression(20);
    int l=els.length;

    File[] dirs =new File[l+1];
    File[] files=new File[l];
    int nb_dirs=0;
    int nb_files=0;

    if(_dir.getParent()!=null)
    {
      String p=new File(_dir.getParent()).getAbsolutePath();
      dirs[0]=new Root(p,p,"..");
      nb_dirs++;
    }

    setProgression(30);
    //System.err.println("filter");
    BuFileFilter filter=(BuFileFilter)filters_.getSelectedItem();

    for(int i=0;i<l;i++)
    {
      if(els[i].startsWith(".")) continue;
      if("CVS".equals(els[i]))   continue;

      File f=new File(_dir.getAbsolutePath(),els[i]);
      if(f.isDirectory())
      {
	dirs[nb_dirs]=f;
	nb_dirs++;
      }
      else
      if(f.isFile()&&((filter==null)||filter.accept(f)))
      {
	files[nb_files]=f;
	nb_files++;
      }
    }

    //System.err.println("sort");
    els=null;
    setProgression(40);
    FuSort.sort(dirs ,0,nb_dirs);
    setProgression(50);
    FuSort.sort(files,0,nb_files);

    //System.err.println("model");
    dirs_.setModel(new Model(dirs,nb_dirs));
    dirs_.revalidate();

    files_.setModel(new Model(files,nb_files));
    files_.revalidate();

    setProgression(60);
    //System.err.println("end");
    repaint();
    if(!WAIT_CURSOR.equals(old_cursor))
      setCursor(old_cursor);

    setProgression(100);
  }

  public void valueChanged(ListSelectionEvent _evt)
  {
    Object source=_evt.getSource();

         if(source==dirs_ ) updateMenuDirs();
    else if(source==files_) updateMenuFiles();
  }

  public void actionPerformed(ActionEvent _evt)
  {
    //System.err.println("SOURCE="+_evt.getSource());

    Object source=_evt.getSource();

    if(source==roots_)
    {
      File r=getCurrentRoot();
      if(r!=null) updateLists(r);
      else System.err.println("CURRENT_ROOT=null");
    }
    else
    if(source==filters_)
    {
      File r=getCurrentDirectory();
      if(r!=null) updateLists(r);
      else System.err.println("CURRENT_DIRECTORY=null");
    }
    else
    {
      String action=_evt.getActionCommand();
      String arg   ="";

      int i=action.indexOf('(');
      if(i>=0)
      {
	arg   =action.substring(i+1,action.length()-1);
	action=action.substring(0,i);
      }

      if("RAFRAICHIR"         .equals(action)) refresh();
      else
      if("OUVRIR_REPERTOIRE"  .equals(action)) openDirectory();
      else
      if("OUVRIR_FICHIER"     .equals(action)) openFile();
      else
      if("CREER_REPERTOIRE"   .equals(action)) createDirectory();
      else
      if("RENOMMER_REPERTOIRE".equals(action)) renameDirectory();
      else
      if("RENOMMER_FICHIER"   .equals(action)) renameFile();
      else
      if("DETRUIRE_REPERTOIRE".equals(action)) deleteDirectory();
      else
      if("DETRUIRE_FICHIER"   .equals(action)) deleteFile();
      else
      if("EXTRA_FICHIER"      .equals(action)) extraFile(arg);
      else
      if("CVS_UPDATE"         .equals(action)) runDirCmd("cvs update");
      else
      if("CVS_COMMIT"         .equals(action)) runDirCmd("cvs commit");
      else
      if("ANT_BUILD"          .equals(action)) runDirCmd("ant");
      else
      if("MAKE"               .equals(action)) runDirCmd("make");
      else
      if("CVS_ADD"            .equals(action)) runFileCmd("cvs add");
      else
	System.err.println("UNKNOWN ACTION="+action);
    }
  }

  private void runDirCmd(String _cmd)
  {
    _cmd="cd "+getCurrentDirectory().getAbsolutePath()+" ; "+_cmd;

    BuLib.runShellCommand(app_,_cmd);
  }

  private void runFileCmd(String _cmd)
  {
    _cmd="cd "+getCurrentDirectory().getAbsolutePath()+" ; "+_cmd;

    Object[] selected=files_.getSelectedValues();
    if(selected==null) return;

    for(int i=0;i<selected.length;i++)
      if(((File)selected[i]).canRead())
        _cmd+=" "+((File)selected[i]).getName();

    BuLib.runShellCommand(app_,_cmd);
  }

  /*
  private void runCmd0(String _cmd)
  {
    if(JOptionPane.YES_OPTION==
       (new BuDialogConfirmation
	(app_,null,
	 _cmd+"\n"+
	 "Voulez-vous continuer ?").activate()));
    {
      ByteArrayOutputStream out=new ByteArrayOutputStream();

      try
      {
	Process p=Runtime.getRuntime().exec
	  ( new String[]
	    { "/bin/sh", "-c",
	      "cd "+getCurrentDirectory().getAbsolutePath()+" ; "+_cmd } );

	BufferedReader in=new BufferedReader
	  (new InputStreamReader(p.getInputStream()));
	BufferedReader err=new BufferedReader
	  (new InputStreamReader(p.getErrorStream()));

	String lin =null;
	String lerr=null;

	while(  ((lin=in.readLine())!=null)
	      ||((lerr=err.readLine())!=null))
	{
	  if(lin!=null)
	  {
	    out.write(lin.getBytes());
	    out.write('\n');
	  }
	  if(lerr!=null)
	  {
	    out.write(lerr.getBytes());
	    out.write('\n');
	  }
	}
	in .close();
	err.close();

	p.waitFor();
      }
      catch(Throwable th)
      {
	PrintWriter pw=new PrintWriter(out);
	th.printStackTrace(pw);
	pw.close();
      }
      finally
      {
	new BuDialogMessage
	  (app_,null,
	   new String(out.toByteArray())).activate();
      }
    }
  }
  */

  public static void main(String[] _args)
  {
    JFrame f=new JFrame("Explorer");
    f.getContentPane().add(new BuExplorer());
    f.pack();
    f.show();
  }
}
