/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuFileFilter.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.*;

/**
 * A simple implementation of FileFilter.
 */


/*
 *     BuFileChooser chooser = new BuFileChooser();
 *     ExampleFileFilter filter = new ExampleFileFilter(
 *                   new String{"gif", "jpg"}, "JPEG & GIF Images")
 *     chooser.addChoosableFileFilter(filter);
 *     chooser.showOpenDialog(this);
 */

public class BuFileFilter
       extends javax.swing.filechooser.FileFilter
       implements FilenameFilter
{
  private   Hashtable filters_                   =null;
  private   boolean   useExtensionsInDescription_=true;
  protected String    description_               =null;
  protected String    fullDescription_           =null;
  
  /**
   * Creates a file filter that accepts files with the given extension.
   * <CODE>new BuFileFilter("jpg");</CODE>
   * @see #addExtension
   */
  public BuFileFilter(String extension)
  {
    this(extension,null);
  }

  /**
   * Creates a file filter that accepts the given file type.
   * <CODE>new BuFileFilter("jpg","JPEG Images");</CODE>
   * @see #addExtension
   */
  public BuFileFilter(String _extension, String _description)
  {
    this(new String[] { _extension }, _description);
  }

  /**
   * Creates a file filter from the given string array.
   * <CODE>new BuFileFilter(String {"gif", "jpg"});</CODE>
   * @see #addExtension
   */
  public BuFileFilter(String[] _filters)
  {
    this(_filters,null);
  }

  /**
   * Creates a file filter from the given string array and description.
   * <CODE>new BuFileFilter(String {"gif","jpg"},"Gif and JPG Images");</CODE>
   * @see #addExtension
   */
  public BuFileFilter(String[] _filters, String _description)
  {
    filters_=new Hashtable(_filters.length);
    for(int i=0; i<_filters.length; i++)
      addExtension(_filters[i]); // add filters one by one
    setDescription(_description);
  }

  /**
   * Return true if this file should be shown in the directory pane,
   * false if it shouldn't.
   * Files that begin with "." are ignored.
   * @see #getExtension
   * @see FileFilter#accepts
   */ 
  public boolean accept(String _f)
  {
    return (_f==null ? false : accept(new File(_f)));
  }

  /**
   * Return true if this file should be shown in the directory pane,
   * false if it shouldn't.
   * Files that begin with "." are ignored.
   * @see #getExtension
   * @see FileFilter#accepts
   */ 
  public boolean accept(File _f)
  {
    if(_f!=null)
    {
      if(_f.isDirectory())
	return true;
      String extension=getExtension(_f);
      if((extension!=null)&&(filters_.get(extension)!=null))
	return true;
    } 
    return false;
  }

  /**
   * Return true if this file matches one of the extensions.
   * Files that begin with "." are ignored.
   * @see #getExtension
   * @see FilenameFilter#accepts
   */ 
  public boolean accept(File _d,String _fn)
  {
    File f=new File(_fn);
    if(f!=null)
    {
      if(f.isDirectory())
	return false;
      String extension=getExtension(f);
      if((extension!=null)&&(filters_.get(extension)!=null))
	return true;
    }
    return false;
  }

  /**
   * Return the extension portion of the file's name . 
   *
   * @see #getExtension
   * @see FileFilter#accept
   */
  public String getExtension(File _f)
  {
    if(_f!=null)
    {
      String filename=_f.getName();
      int i=filename.lastIndexOf('.');
      if((i>0)&&(i<filename.length()-1))
	return filename.substring(i+1).toLowerCase();
    } 
    return null;
  }

  /**
   * Adds a filetype "dot" extension to filter against. 
   * the following code will create a filter that filters
   * out all files except those that end in ".jpg" and ".tif":
   *
   *   BuFileFilter filter = new BuFileFilter();
   *   filter.addExtension("jpg");
   *   filter.addExtension("tif");
   */
  public void addExtension(String _extension)
  {
    if(filters_==null) filters_=new Hashtable(5);
    filters_.put(_extension.toLowerCase(),this);
    fullDescription_=null;
  }

  /**
   * Returns the human readable description of this filter.
   * JPEG and GIF Image Files (*.jpg, *.gif)
   * @see setDescription
   * @see setExtensionListInDescription
   * @see isExtensionListInDescription
   * @see FileFilter#getDescription
   */
  public String getDescription()
  {
    if(fullDescription_==null)
    {
      if((description_==null)||isExtensionListInDescription())
      {
	if(description_!=null)
	  fullDescription_=description_;

	fullDescription_+=" (";
	// build the description from the extension list
	Enumeration extensions=filters_.keys();
	if(extensions!=null)
	{
	  fullDescription_+="*."+(String)extensions.nextElement();
	  while(extensions.hasMoreElements())
	    fullDescription_+=", *."+(String)extensions.nextElement();
	}
	fullDescription_+=")";
      }
      else
      {
	fullDescription_=description_;
      }
    } 
    return fullDescription_;
  }

  /**
   * Sets the human readable description of this filter.
   * filter.setDescription("Gif and JPG Images");
   *
   * @see setDescription
   * @see setExtensionListInDescription
   * @see isExtensionListInDescription
   */
  public void setDescription(String _description)
  {
    description_=_description;
    fullDescription_ = null;
  }

  /**
   * Determines whether the extension list (.jpg, .gif, etc) should
   * show up in the human readable description.
   *
   * Only relevent if a description was provided in the constructor
   * or using setDescription();
   *
   * @see getDescription
   * @see setDescription
   * @see isExtensionListInDescription
   */
  public void setExtensionListInDescription(boolean _b)
  {
    useExtensionsInDescription_=_b;
    fullDescription_=null;
  }
  
  /**
   * Returns whether the extension list (.jpg, .gif, etc) should
   * show up in the human readable description.
   *
   * Only relevent if a description was provided in the constructor
   * or using setDescription();
   *
   * @see getDescription
   * @see setDescription
   * @see setExtensionListInDescription
   */
  public boolean isExtensionListInDescription()
  {
    return useExtensionsInDescription_;
  }

  public int hashCode()
  {
    return toString().hashCode();
  }

  public boolean equals(Object _o)
  {
    boolean r=false;

    if(_o instanceof BuFileFilter)
      r=toString().equals(_o.toString());

    return r;
  }

  public String toString()
  {
    return getDescription();
  }
}
