/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuFontChooser.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * A simple font chooser.
 */

public class BuFontChooser
       extends BuDialog
       implements ActionListener, ListSelectionListener
{
  protected BuButton    btValider_;
  protected BuButton    btAnnuler_;
  protected BuEmptyList ltFamille_;
  protected BuEmptyList ltTaille_;
  protected BuCheckBox  cbGras_;
  protected BuCheckBox  cbItalique_;
  protected BuCheckBox  cbSouligne_;
  protected BuTextField tfExemple_;

  public BuFontChooser(BuCommonInterface      _parent,
		       BuInformationsSoftware _isoft,
		       String                 _title,
		       Font                   _value)
  {
    super(_parent,_isoft,_title);

    tfExemple_.setText("ABC abc 123");

    if(_value==null) _value=new Font("SansSerif",Font.PLAIN,12);
    valeur_=_value;

    String f=valeur_.getName();
    String s=""+valeur_.getSize();
    if(f==null) { f="SansSerif"; s="12"; }
    ltFamille_.setSelectedValue(f,true);
    ltTaille_ .setSelectedValue(s,true);
    
    BuPanel pnb=new BuPanel();
    pnb.setLayout(new FlowLayout(FlowLayout.RIGHT));

    btValider_=new BuButton(BuLib.loadCommandIcon("VALIDER"),
		       BuResource.BU.getString("Valider"));
    btValider_.addActionListener(this);
    getRootPane().setDefaultButton(btValider_);
    pnb.add(btValider_);

    btAnnuler_=new BuButton(BuLib.loadCommandIcon("ANNULER"),
		       BuResource.BU.getString("Annuler"));
    btAnnuler_.addActionListener(this);
    pnb.add(btAnnuler_);

    content_.add(pnb,BuBorderLayout.SOUTH);
  }

  public JComponent getComponent()
  {
    String[] FAMILLES=new String[]
    {"SansSerif","Serif","Monospaced" }; //,"Dialog", "Input" };

    String[] TAILLES =new String[]
    { "6","8","10","12","14","16","18","20","24","28","32","36","48","72" };

    ltFamille_=new BuEmptyList();
    ltTaille_ =new BuEmptyList();
    ltFamille_.setListData(FAMILLES);
    ltTaille_ .setListData(TAILLES );
    ltFamille_.addListSelectionListener(this);
    ltTaille_ .addListSelectionListener(this);

    cbGras_    =new BuCheckBox("Bold");
    cbItalique_=new BuCheckBox("Italic");
    cbSouligne_=new BuCheckBox("Underlined");
    cbGras_    .addActionListener(this);
    cbItalique_.addActionListener(this);
    cbSouligne_.addActionListener(this);
    cbSouligne_.setEnabled(false);

    BuPanel s=new BuPanel();
    s.setLayout(new BuVerticalLayout(5,true,false));
    s.add(cbGras_);
    s.add(cbItalique_);
    s.add(cbSouligne_);

    BuPanel q=new BuPanel();
    q.setLayout(new BuGridLayout(3,5,5,true,true));
    q.add(new BuScrollPane(ltFamille_));
    q.add(s);
    q.add(new BuScrollPane(ltTaille_));

    tfExemple_=new BuTextField();
    tfExemple_.setPreferredSize(new Dimension(200,100));

    BuPanel p=new BuPanel();
    p.setLayout(new BuVerticalLayout(5,true,true));
    p.setBorder(new EmptyBorder(5,5,5,5));
    p.add(q);
    p.add(tfExemple_);
    p.add(new BuPanel());
    return p;
  }

  private final Font value()
  {
    Font r=null;

    try
    {
      r=new Font(""+ltFamille_.getSelectedValue(),
		 Font.PLAIN
		 | (cbGras_    .isSelected() ? Font.BOLD : 0)
		 | (cbItalique_.isSelected() ? Font.ITALIC : 0),
		 Integer.parseInt(""+ltTaille_.getSelectedValue()));
    }
    catch(Exception ex) { }

    return r;
  }

  public void valueChanged(ListSelectionEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();

    if((source==ltFamille_)||(source==ltTaille_))
    {
      valeur_=value();
      tfExemple_.setFont(valeur_);
    }
  }

  public void actionPerformed(ActionEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();

    if((source==cbGras_)||(source==cbItalique_))
    {
      valeur_=value();
      tfExemple_.setFont(valeur_);
    }

    if(source==btValider_)
    {
      reponse_=JOptionPane.OK_OPTION;
      //valeur_ =tfValeur_.getText();
      setVisible(false);
    }

    if(source==btAnnuler_)
    {
      reponse_=JOptionPane.CANCEL_OPTION;
      valeur_ =null;
      setVisible(false);
    }
  }

  private Font valeur_;

  public Font getValue()
    { return valeur_; }

  public void setValue(Font _value)
  {
    tfExemple_.setFont(_value);
    valeur_=_value;
  }

  public static void main(String[] _args)
  {
    new BuFontChooser(null,null,"test BuFontChooser",null)
      .activate();
  }
}
