/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuFrameImage.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.net.*;
import java.io.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import javax.swing.event.*;

/**
 * A small image displayer.
 */

public class BuFrameImage
       extends BuInternalFrame 
       implements BuPrintable
{
  private BuCommonImplementation app_;
  private JComponent             content_;
  private BuPicture              pane_;
  private BuScrollPane           sp_;

  public BuFrameImage()
  { this(null,null,null); }

  public BuFrameImage(BuCommonImplementation _app, String _file, Image _img)
  {
    super("",true,true,true,true);
    app_=_app;

    setName("ifIMAGEUR");
    if(app_!=null) app_.installContextHelp(getRootPane(),"bu/p-imageur.html");

    content_=(JComponent)getContentPane();
    content_.setBorder(new EmptyBorder(0,0,0,0));

    pane_=new BuPicture();
    pane_.setBorder(new EmptyBorder(5,5,5,5));
    pane_.setMode(BuPicture.LEFT);

    sp_=new BuScrollPane(pane_);
    sp_.setBorder(new EmptyBorder(0,0,0,0));
    content_.add(sp_,BuBorderLayout.CENTER);

    if(_file!=null) setTitle(new File(_file).getName());
    else            setTitle(BuResource.BU.getString("Image"));

    setFrameIcon(BuResource.BU.getBarIcon("image"));

    setImage(_img);
  }

  public Image getImage()
  {
    return pane_.getImage();
  }

  public void setImage(Image _img)
  {
    pane_.setImage(_img);
    Dimension ps=getPreferredSize();
    if(ps.width >370) ps.width =296;
    if(ps.height>370) ps.height=296;
    if(ps.width <148) ps.width =148;
    if(ps.height< 74) ps.height= 74;
    setSize(ps);
  }

  public void setBounds(int _x,int _y,int _w,int _h)
  {
    super.setBounds(_x,_y,_w,_h);

    if(pane_!=null)
    {
      pane_.computePreferredSize();
      Dimension ps=pane_.getPreferredSize();
      sp_.getViewport().setViewPosition(new Point(0,0));
      sp_.getViewport().setViewSize    (ps);
    }
  }

  /*
  public void setSelected(boolean _state) throws PropertyVetoException
  {
    super.setSelected(_state);
    if(isSelected()) pane_.requestFocus();
  }
  */

  // BuInternalFrame

  public String[] getEnabledActions()
  {
    String[] r=new String[]
      {
	"IMPRIMER","PREVISUALISER",
	//"COPIER"
      };
    return r;
  }

  // BuPrintable

  public void print(PrintJob _job, Graphics _g)
  {
    BuPrinter.INFO_DOC     =new BuInformationsDocument();
    BuPrinter.INFO_DOC.name=getTitle();
    BuPrinter.INFO_DOC.logo=BuResource.BU.getIcon("image",24);

    BuPrinter.printImage(_job,_g,getImage());
  }

  // Test

  public static void main(String[] _args)
  {
    JFrame         f=new JFrame("test BuFrameImage");
    BuFrameImage   b=new BuFrameImage();
    BuFrameImage   c=new BuFrameImage();

    BuDesktop d=new BuDesktop();
    d.add(b);
    d.add(c);

    f.getContentPane().add("Center",d);
    f.setSize(300,400);
    f.show();
    f.setLocation(200,100);

    b.setImage(BuResource.BU.getImage("background_desktop_1.gif"));
    c.setImage(BuResource.BU.getImage("ouvrir_24.gif"));
    b.setLocation(70,30);
  }
}
