/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuGridLayout.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;

import java.awt.* ;
import java.util.* ;

/**
 * Lays out in a grid a set of components.
 * Not like the java.awt.GridLayout, rows may have different heigths
 * and columns may have different widths.
 */

public class BuGridLayout
       implements LayoutManager2
{
  public BuGridLayout()
  {
    this(3,0,0,true,true);
  }

  public BuGridLayout(int _columns)
  {
    this(_columns,0,0,true,true);
  }

  public BuGridLayout(int _columns, int _hgap, int _vgap)
  {
    this(_columns,_hgap,_vgap,true,true);
  }

  public BuGridLayout(int _columns,
		      int _hgap, int _vgap,
		      boolean _hfilled, boolean _vfilled)
  {
    this(_columns,_hgap,_vgap,_hfilled,_vfilled,true,false);
  }

  public BuGridLayout(int _columns,
		      int _hgap, int _vgap,
		      boolean _hfilled, boolean _vfilled,
		      boolean _cfilled, boolean _rfilled)
  {
    columns_=_columns;
    hgap_=_hgap;
    vgap_=_vgap;
    hfilled_=_hfilled;
    vfilled_=_vfilled;
    cfilled_=_cfilled;
    rfilled_=_rfilled;
    xalign_=0.5f;
    yalign_=0.5f;
  }

  protected int columns_;
  public int getColumns() { return columns_; }
  public void setColumns(int _columns) { columns_=_columns; }

  protected int hgap_;
  public int getHgap() { return hgap_; }
  public void setHgap(int _hgap) { hgap_=_hgap; }

  protected int vgap_;
  public int getVgap() { return vgap_; }
  public void setVgap(int _vgap) { vgap_=_vgap; }

  protected boolean hfilled_;
  public boolean getHfilled() { return hfilled_; }
  public void setHfilled(boolean _hfilled) { hfilled_=_hfilled; }

  protected boolean vfilled_;
  public boolean getVfilled() { return vfilled_; }
  public void setVfilled(boolean _vfilled) { vfilled_=_vfilled; }

  protected boolean cfilled_;
  public boolean getCfilled() { return cfilled_; }
  public void setCfilled(boolean _cfilled) { cfilled_=_cfilled; }

  protected boolean rfilled_;
  public boolean getRfilled() { return rfilled_; }
  public void setRfilled(boolean _rfilled) { rfilled_=_rfilled; }

  protected float xalign_;
  public float getXalign() { return xalign_; }
  public void setXalign(float _xalign) { xalign_=_xalign; }

  protected float yalign_;
  public float getYalign() { return yalign_; }
  public void setYalign(float _yalign) { yalign_=_yalign; }

  public void addLayoutComponent(Component _comp, Object _cstr)
  {
  }

  public void addLayoutComponent(String _name, Component _comp)
  {
  }

  public void removeLayoutComponent(Component _comp)
  {
  }

  public Dimension minimumLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      int nbc=_parent.getComponentCount();
      int w  =0;
      int h  =0;
    
      int nbcol=columns_;
      int nbrow=(nbc+nbcol-1)/nbcol;

      int[] wcol=new int[nbcol];
      int[] hrow=new int[nbrow];

      int i;
      
      for(i=0;i<nbc;i++)
      {
	Dimension d=_parent.getComponent(i).getMinimumSize();
	wcol[i%nbcol]=Math.max(wcol[i%nbcol],d.width);
	hrow[i/nbcol]=Math.max(hrow[i/nbcol],d.height);
      }

      for(i=0;i<nbcol;i++)
      {
	if(i>0) w+=hgap_;
	w+=wcol[i];
      }

      for(i=0;i<nbrow;i++)
      {
	if(i>0) h+=vgap_;
	h+=hrow[i];
      }

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
                         h+insets.top+insets.bottom);
    }
  }

  public Dimension preferredLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      int nbc=_parent.getComponentCount();
      int w  =0;
      int h  =0;

      int nbcol=columns_;
      int nbrow=(nbc+nbcol-1)/nbcol;

      int[] wcol=new int[nbcol];
      int[] hrow=new int[nbrow];

      int i;

      for(i=0;i<nbc;i++)
      {
	Dimension d=_parent.getComponent(i).getPreferredSize();
	wcol[i%nbcol]=Math.max(wcol[i%nbcol],d.width);
	hrow[i/nbcol]=Math.max(hrow[i/nbcol],d.height);
      }

      for(i=0;i<nbcol;i++)
      {
	if(i>0) w+=hgap_;
	w+=wcol[i];
      }

      for(i=0;i<nbrow;i++)
      {
	if(i>0) h+=vgap_;
	h+=hrow[i];
      }

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
			   h+insets.top+insets.bottom);
    }
  }

  public Dimension maximumLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      int nbc=_parent.getComponentCount();
      int w  =Integer.MAX_VALUE;
      int h  =0;

      int nbcol=columns_;
      int nbrow=(nbc+nbcol-1)/nbcol;

      int[] wcol=new int[nbcol];
      int[] hrow=new int[nbrow];

      int i;

      for(i=0;i<nbc;i++)
      {
	Dimension d=_parent.getComponent(i).getMaximumSize();
	wcol[i%nbcol]=Math.max(wcol[i%nbcol],d.width);
	hrow[i/nbcol]=Math.max(hrow[i/nbcol],d.height);
      }

      for(i=0;i<nbcol;i++)
      {
	if(i>0) w+=hgap_;
	w+=wcol[i];
      }

      for(i=0;i<nbrow;i++)
      {
	if(i>0) h+=vgap_;
	h+=hrow[i];
      }

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
			   h+insets.top+insets.bottom);
    }
  }

  public void invalidateLayout(Container _parent)
    { }

  public float getLayoutAlignmentX(Container _parent)
    { return 0.5f; }

  public float getLayoutAlignmentY(Container _parent)
    { return 0.5f; }

  public void layoutContainer(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      Insets    insets=_parent.getInsets();
      Dimension size  =_parent.getSize();
      Rectangle bounds=new Rectangle
	(insets.left,
	 insets.top,
	 size.width-(insets.left + insets.right),
	 size.height-(insets.top + insets.bottom));

      int nbc=_parent.getComponentCount();
      int x  =0;
      int y  =0;
      int w  =0;
      int h  =0;

      int nbcol=columns_;
      int nbrow=(nbc+nbcol-1)/nbcol;

      int[] wcol=new int[nbcol];
      int[] hrow=new int[nbrow];

      int i;

      for(i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	Dimension d=c.getPreferredSize();
	wcol[i%nbcol]=Math.max(wcol[i%nbcol],d.width);
	hrow[i/nbcol]=Math.max(hrow[i/nbcol],d.height);
      }

      for(i=0;i<nbcol;i++)
      {
	if(i>0) w+=hgap_;
	w+=wcol[i];
      }

      for(i=0;i<nbrow;i++)
      {
	if(i>0) h+=vgap_;
	h+=hrow[i];
      }

      for(i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	Dimension d=c.getPreferredSize();
	int wmax=wcol[i%nbcol];
	int hmax=hrow[i/nbcol];

	if((i>0)&&(i%nbcol==0)) y+=vgap_;
	if(i%nbcol!=0) x+=hgap_;

	w=(hfilled_ ? wmax : Math.min(wmax,d.width ));
	h=(vfilled_ ? hmax : Math.min(hmax,d.height));

	// last
	if(cfilled_&&(i%nbcol==nbcol-1)&&(x+w<bounds.width ))
	  w=wmax=bounds.width -x;
	if(rfilled_&&(i/nbcol==nbrow-1)&&(y+h<bounds.height))
	  h=hmax=bounds.height-y;
	
	// center
	int dx=0;
	int dy=0;
	if(!hfilled_) dx=(int)((wmax-w)*xalign_);
	if(!vfilled_) dy=(int)((hmax-h)*yalign_);

	c.setBounds(bounds.x+x+dx,bounds.y+y+dy,w,h);
	// System.err.println("-- "+i%nbcol+","+i/nbcol+": "+c.getBounds());

	if((i+1)%nbcol!=0) x+=wmax;
	else { x=0; y+=hmax; }
      }
    }
  }
}
