/**
 * @modification $Date: 2001/12/07 18:28:12 $
 * @statut       unstable
 * @file         BuInternalFrame.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.*;

/**
 * An internal frame having specific menus, tools and actions.
 */

public class BuInternalFrame
       extends JInternalFrame
       implements ActionListener
{
  public static final String SHORTCUT="SHORTCUT";

  public BuInternalFrame()
  {
    this("",true,true,true,true);
  }

  public BuInternalFrame(String _title,
			 boolean _resizable, boolean _closable,
			 boolean _maximizable, boolean _iconifiable)
  {
    super(_title,_resizable,_closable,_maximizable,_iconifiable);
    getRootPane().putClientProperty
      ("defeatSystemEventQueueCheck",Boolean.TRUE);
    getGlassPane().setName(null);

    //setLocation(10,10);
    setSize(111,111);
  }

  public void setVisible(boolean _visible)
  {
    boolean maximum=isMaximum();

    //TMP @GDX
    if(_visible)
    {
      BuLib.setDoubleBuffered(this,false);
      setDoubleBuffered(true);
      if(!isVisible())
      {
	try { setMaximum(false); }
	catch(PropertyVetoException ex) { }
      }
    }

    super.setVisible(_visible);

    if(isVisible()&&!isMaximum()&&maximum)
    {
      try { setMaximum(true); }
      catch(PropertyVetoException ex) { }
    }
  }

  public void setClosed(boolean _closed) throws PropertyVetoException
  {
    super.setClosed(_closed);
    if(isClosed()&&isSelected()) setSelected(false);
    // should be done before ?
  }

  private boolean simplified_=false;

  /*
  public void paint(Graphics _g)
  {
    if((!simplified_||isSelected())&&super.isVisible())
    {
      BuLib.setAntialiasing(_g);
      super.paint(_g);
    }
  }
  */

  public void simplifyTop()
  {
    // if(simplified_) return;

    simplified_=true;
    setRootPaneCheckingEnabled(false);

    /*
    for(int i=0;i<getComponentCount();i++)
    {
      Component c=getComponent(i);
      System.err.println("C["+i+"]="+c.getClass());
      if(c instanceof BasicInternalFrameTitlePane)
	remove(c);
      // add(new JLabel(getTitle()),BorderLayout.NORTH);
    }
    */

    Component c=getRootPane();

    BuPanel top=new BuPanel();
    top.setLayout(new BuBorderLayout());

    BuLabel l=new BuLabel();
    l.setText(getTitle());
    l.setBorder(new EmptyBorder(1,1,1,1));
    // l.setFont(BuLib.deriveFont("Label",-2));
    l.setHorizontalAlignment(JLabel.CENTER);

    // l.setHorizontalTextPosition(JLabel.RIGHT);
    // l.setBackground(new Color(128,224,192));
    // l.setForeground(Color.black);
    // l.setRequestFocusEnabled(false);

    top.add(l,BorderLayout.CENTER);

    if(isClosable())
    { 
      Icon icon=UIManager.getIcon("InternalFrameTitlePane.closeIcon");
      if(icon==null) icon=UIManager.getIcon("InternalFrame.closeIcon");
      if(icon==null) icon=BuResource.BU.getIcon("fermer");

      BuButton b=new BuButton();
      b.setBorder(new EmptyBorder(1,1,1,1));
      b.setRequestFocusEnabled(false);
      b.setIcon(icon);
      top.add(b,BorderLayout.EAST);

      b.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent _ae)
        {
	  try { setClosed(true); }
	  catch(PropertyVetoException ex) { }
	}
      }
      );
    }

    setBorder(UIManager.getBorder("TitleBorder.border"));

    removeAll();
    setLayout(new BuBorderLayout());
    add(c,BuBorderLayout.CENTER);
    add(top,BuBorderLayout.NORTH);
    getLayout().layoutContainer(this);
    // setOpaque(true);
    repaint();
  }

  public static int getShortcut(JInternalFrame _f)
  {
    int r=0;
    if(_f!=null)
    {
      Object o=_f.getClientProperty(SHORTCUT);
      if(o!=null) r=((Integer)o).intValue();
    }
    return r;
  }

  public int getShortcut()
    { return getShortcut(this); }

  public void setShortcut(int _key)
  {
    putClientProperty(SHORTCUT,new Integer(_key));
  }

  /*
  public boolean isFocusCycleRoot()
    { return true; }
  */

  public String toString()
  {
    String      r=getName();
    if(r==null) r=getTitle();
    if(r==null) r=super.toString();
    return r;
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
    new IllegalArgumentException
      ("Illegal action on "+getClass().getName()+": "+action)
	.printStackTrace();
  }

  public String[] getEnabledActions()
  {
    String[] r=new String[0];
    return r;
  }

  public String[] getDisabledActions()
  {
    String[] r=new String[0];
    return r;
  }

  public JMenu[] getSpecificMenus()
  {
    JMenu[] r=new JMenu[0];
    return r;
  }

  public JComponent[] getSpecificTools()
  {
    JComponent[] r=new JComponent[0];
    return r;
  }

  public void adjustActions()
  {
  }

  public void updateUI()
  {
    super.updateUI();

    if(!isResizable())
    {
      setResizable(true);
      SwingUtilities.updateComponentTreeUI(getContentPane());
      pack();
      setResizable(false);
    }
    else SwingUtilities.updateComponentTreeUI(getContentPane());

    if(simplified_) simplifyTop();
    BuLib.setDoubleBuffered(this,false);
  }

  public final void paintBorder(Graphics _g) { }

  protected void paintSuperBorder(Graphics _g)
  {
    super.paintBorder(_g);
  }

  public void paint(Graphics _g)
  {
    super.paint(_g);
    paintSuperBorder(_g);
  }
}
