/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuLanguagePreferencesPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


/**
 * @creation     25/01/99
 * @modification 10/09/99
 * @statut       instable
 */

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A panel where the user can choose his language
 * among 9 (fr,de,en,es,eo,it,hu,pg,ru).
 */

/**
 * locale.language
 */

public class BuLanguagePreferencesPanel
       extends BuAbstractPreferencesPanel
{
  private final static int NBL=9;

  private final static String[] CODES=new String[]
  {
    "fr","de","en","es",
    "eo","it","hu","pg",
    "ru"
  };

  private final static String[] INTITULES=new String[]
  {
    "franais","deutsch","english","espa\u00F1ol",
    "esperanto","italiano","magyarul","portuguese",
    "russian"
  };

  private final static String[] NOMS=new String[]
  {
    "FRENCH","GERMAN","ENGLISH","SPANISH",
    "ESPERANTO","ITALIAN","HUNGARIAN","PORTUGUESE",
    "RUSSIAN"
  };


  BuCommonInterface appli_;
  BuPreferences     options_;
  String            dispo_;

  JRadioButton[]    cbLangues_;
  
  public String getTitle()
    { return BuResource.BU.getString("Langue"); }

  // Constructeur

  public BuLanguagePreferencesPanel(BuCommonInterface _appli)
  {
    super();
    appli_=_appli;
    dispo_=appli_.getInformationsSoftware().languages;
    init();
  }

  public BuLanguagePreferencesPanel(String _dispo)
  {
    super();
    appli_=null;
    dispo_=_dispo;
    init();
  }

  private void init()
  {
    options_=BuPreferences.BU;

    BuGridLayout lo=new BuGridLayout();
    lo.setColumns(2);
    lo.setVgap(5);
    lo.setHgap(5);

    BuPanel p=new BuPanel();
    p.setBorder
      (new CompoundBorder
       (new TitledBorder(getTitle()),
	new EmptyBorder(5,5,5,5)));
    p.setLayout(lo);

    ButtonGroup bg=new ButtonGroup();

    cbLangues_=new BuRadioButton[NBL];
    for(int i=0;i<NBL;i++)
    {
      cbLangues_[i]=new BuRadioButton(INTITULES[i]);
      cbLangues_[i].setName("cbLOCALE_LANGUAGE_"+NOMS[i]);
      cbLangues_[i].setActionCommand(NOMS[i]);
      bg.add(cbLangues_[i]);
      p.add(cbLangues_[i]);
    }

    p.add(new BuPanel());
    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    add(p,BuBorderLayout.CENTER);

    String l=Locale.getDefault().toString();
    for(int i=0;i<NBL;i++)
    {
      if(dispo_.indexOf(CODES[i])<0)
	cbLangues_[i].setEnabled(false);
      if(l.startsWith(CODES[i]))
	cbLangues_[i].setSelected(true);
    }
    updateComponents();
  }

  // Methodes publiques

  public String getLocaleString()
  {
    String r="";

    for(int i=0;i<NBL;i++)
      if(cbLangues_[i].isSelected())
	r=CODES[i];

    return r;
  }

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  /*
  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
    options_.applyOn(appli_);
  }
  */

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
    for(int i=0;i<NBL;i++)
      if(cbLangues_[i].isSelected())
        options_.putStringProperty("locale.language",CODES[i]);
  }

  private void updateComponents()
  {
    String l=options_.getStringProperty("locale.language");
    for(int i=0;i<NBL;i++)
      if(l.startsWith(CODES[i]))
	cbLangues_[i].setSelected(true);
  }
}


