/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuLib.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.text.*;

import javax.swing.*;
import javax.swing.plaf.*;

/**
 * Utility class with only static methods.
 * For small and useful services.
 */
public class BuLib extends FuLib /* @GDX TMP */
{
  public final static Frame HELPER=new Frame();

  static { HELPER.addNotify(); }

  /**
   * @param _comp a component if available, can be null
   * @param _font a font
   * @return the metrics of the font
   */
  public static FontMetrics getFontMetrics(JComponent _comp, Font _font)
  {
    Graphics g=null;
    if(_comp!=null) g=_comp.getGraphics();
    if(g==null) g=HELPER.getGraphics();
    return g.getFontMetrics(_font);
  }

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand.
   */
  public static BuIcon loadCommandIcon(String _cmd)
  {
    BuIcon r=null;
    String c=_cmd.toLowerCase();

    int i=c.indexOf('_');
    if(i>=0) c=c.substring(0,i);

    r=BuResource.BU.getIcon(c);
    if(r==null) r=BuResource.BU.getIcon("aucun");
    return r;
  }

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand and reduce it according to the
   * menu icon size.
   * @deprecated use BuResource.BU.loadMenuCommandIcon(String)
   */
  public static BuIcon loadMenuCommandIcon(String _cmd)
  {
    BuIcon r=null;
    if(BuPreferences.BU.getBooleanProperty("icons.menu",true))
      r=BuResource.BU.reduceMenuIcon(loadCommandIcon(_cmd));
    return r;
  }

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand and reduce it according to the
   * tool icon size.
   * @deprecated use BuResource.BU.loadToolCommandIcon(String)
   */
  public static BuIcon loadToolCommandIcon(String _cmd)
  {
    return BuResource.BU.reduceToolIcon(loadCommandIcon(_cmd));
  }

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand and reduce it according to the
   * bar icon size.
   * @deprecated use BuResource.BU.loadBarCommandIcon(String)
   */
  public static BuIcon loadBarCommandIcon(String _cmd)
  {
    return BuResource.BU.reduceBarIcon(loadCommandIcon(_cmd));
  }

  /**
   * To get a font of the same family than an other but
   * a little differente in size.
   * @return a derivated font
   */
  public static Font deriveFont(String _key, int _size)
  {
    Font ft=UIManager.getFont(_key+".font");
    if(ft==null) ft=new Font("SansSerif",Font.PLAIN,12);
    int  sz=Math.max(ft.getSize()+_size,8);
    // UIResource
    return new Font(ft.getFamily(),ft.getStyle(),sz);
  }
  
  /**
   * To get a font of the same family than an other but
   * a little differente in size or style. 
   * @return a derivated font
   */
  public static Font deriveFont(String _key, int _style, int _size)
  {
    Font ft=UIManager.getFont(_key+".font");
    if(ft==null) ft=new Font("SansSerif",Font.PLAIN,12);
    int  sz=Math.max(ft.getSize()+_size,8);
    // UIResource
    return new Font(ft.getFamily(),_style,sz);
  }
  
  /**
   * To get a font of the same family than an other but
   * a little differente in size. 
   * @return a derivated font
   */
  public static Font deriveFont(Font _ft, int _size)
  {
    int sz=Math.max(_ft.getSize()+_size,8);
    return new Font(_ft.getFamily(),_ft.getStyle(),sz);
  }
  
  /**
   * To get a font of the same family than an other but
   * a little difference in size or style. 
   * @return a derivated font
   */
  public static Font deriveFont(Font _ft, int _style, int _size)
  {
    int sz=Math.max(_ft.getSize()+_size,8);
    return new Font(_ft.getFamily(),_style,sz);
  }
  
  /**
   * To get the tab placement customized by the user.
   * (SwingConstants.LEFT, TOP, ...)
   * @return a tab placement
   */
  public static int getTabPlacement()
  {
    int r=BuPreferences.BU.getIntegerProperty
      ("tabbedpane.tabplacement");

    //if(BuLib.isMetal()||BuLib.isSlaf())
    if((r<1)||(r>4)) r=2; // LEFT

    if(UIManager.getLookAndFeel().getClass()
       .getName().endsWith(".MacLookAndFeel"))
	r=1; // TOP for Mac

    return r;
  }

  /**
   * Internal use.
   */
  private static void getAllSubComponents(Vector _v, Container _parent)
  {
    Component[] c=_parent.getComponents();

    for(int i=0;i<c.length;i++)
    {
      if(!_v.contains(c[i]))
	_v.addElement(c[i]);
      if(c[i] instanceof Container)
	getAllSubComponents(_v,(Container)c[i]);
    }
  }

  /**
   * Returns all subcomponents of a container.
   */
  public static Vector getAllSubComponents(Container _parent)
  {
    Vector r=new Vector();
    getAllSubComponents(r,_parent);
    return r;
  }

  /**
   * Set/unset double-buffering for a tree of components.
   * Except JPopupMenu and Window.
   */
  public static void setDoubleBuffered(Container _parent, boolean _b)
  {
    if(_parent!=null)
    {
      //System.err.println("NO-DOUBLE-BUFFER: "+_parent.getName());
      Enumeration e=getAllSubComponents(_parent).elements();
      while(e.hasMoreElements())
      {
	Component c=(Component)e.nextElement();
	if(c instanceof JPopupMenu) continue;
	if(c instanceof Window    ) continue;

	if(c instanceof JComponent)
	  ((JComponent)c).setDoubleBuffered(_b);
      }
    }
  }

  /**
   * Simplifies a component name by removing any lowercase letter
   * at the beginning. btOPEN -> OPEN
   */
  public static String simplifyComponentName(String _name)
  {
    if(_name==null) return "";

    int i=0;
    while(i<_name.length())
    {
      int c=_name.charAt(i);
      if((c<'A')||(c>'Z')) i++;
      else break;
    }

    return _name.substring(i);
  }

  /**
   * Find a named component.
   */
  public static Component findNamedComponent(Container _parent, String _name)
  {
    Component r=null;

    Enumeration e=getAllSubComponents(_parent).elements();
    while(e.hasMoreElements())
    {
      Component c=(Component)e.nextElement();
      String    n=c.getName();
      if(_name.equals(n)||_name.equals(simplifyComponentName(n)))
	  { r=c; break; }
    }

    return r;
  }

  private static void enableRecursively(Component _r, boolean _b)
  {
    _r.setEnabled(_b);

    if(_r instanceof Container)
    {
      Component[] c=((Container)_r).getComponents();

      for(int i=0;i<c.length;i++)
	setRecursiveEnabled(c[i],_b);
    }
  }

  /**
   * Enables a component and all his children, grand-children...
   */
  public static void setRecursiveEnabled(Component _r, boolean _b)
  {
    enableRecursively(_r,_b);
    _r.repaint();
  }

  private final static ImageIcon filter
    (JComponent _comp, ImageIcon _icon, ImageFilter _filter)
  {
    return new ImageIcon
	(_comp.createImage(new FilteredImageSource
	    (_icon.getImage().getSource(),_filter)));
  }

  /**
   * Sets icons for an abstract button (button, menuitem, ...).
   */
  public static void setIcon(AbstractButton _button, BuIcon _icon)
  {
    if(_icon!=null)
    {
      if(BuPreferences.BU.getBooleanProperty("icons.grey",false))
      {
	if(_button instanceof JMenuItem)
	    _button.setIcon
		(filter(_button,
			filter(_button,_icon,BuFilters.GREY),
			BuFilters.getEnabled()));
	else
	    _button.setIcon
		(filter(_button,_icon,BuFilters.getEnabled()));

	_button.setPressedIcon
	    (filter(_button,_icon,BuFilters.getPressed()));
	_button.setRolloverIcon
	    (filter(_button,_icon,BuFilters.getEnabled()));
	_button.setDisabledIcon
	    (filter(_button,
		    filter(_button,_icon,BuFilters.EMBOSS),
		    BuFilters.getDisabled()));

	if(_button instanceof BuToggleButton)
	{
	  _button.setSelectedIcon
	    (filter(_button,
		    filter(_button,_icon,BuFilters.GREY),
		    BuFilters.getSelected()));
	  _button.setRolloverSelectedIcon
	      (filter(_button,_icon,BuFilters.getSelected()));
	  _button.setDisabledSelectedIcon
	    (filter(_button,
		    filter(_button,_icon,BuFilters.getSelected()),
		    BuFilters.getDisabled()));
	}
      }
      else
      {
	ImageProducer ip=_icon.getImage().getSource();

	_button.setIcon
	    (filter(_button,_icon,BuFilters.getEnabled ()));
	_button.setPressedIcon
	    (filter(_button,_icon,BuFilters.getPressed ()));
	_button.setRolloverIcon
	    (filter(_button,_icon,BuFilters.getRollover()));
	_button.setDisabledIcon
	    (filter(_button,_icon,BuFilters.getDisabled()));
	
	if(_button instanceof BuToggleButton)
	{
	  _button.setSelectedIcon
	      (filter(_button,_icon,BuFilters.getSelected()));
	  _button.setRolloverSelectedIcon
	      (filter(_button,_icon,BuFilters.getRollover()));
	  _button.setDisabledSelectedIcon
	      (filter(_button,_icon,BuFilters.getDisabled()));
	}
      }
    }
    else _button.setIcon(null);
  }

  public static int mixColors(int _rgb1, int _rgb2)
  {
    int a1=(_rgb1 & 0xff000000) >>24;
    int r1=(_rgb1 & 0x00ff0000) >>16;
    int g1=(_rgb1 & 0x0000ff00) >> 8;
    int b1=(_rgb1 & 0x000000ff)     ;

    int a2=(_rgb2 & 0xff000000) >>24;
    int r2=(_rgb2 & 0x00ff0000) >>16;
    int g2=(_rgb2 & 0x0000ff00) >> 8;
    int b2=(_rgb2 & 0x000000ff)     ;

    int a=(a1+a2)/2;
    int r=(r1+r2)/2;
    int g=(g1+g2)/2;
    int b=(b1+b2)/2;

    return (a<<24) | (r<<16) | (g<<8) | b;
  }

  public static Color mixColors(Color _a, Color _b)
  {
    return new Color(mixColors(_a.getRGB(),_b.getRGB()));
  }

  public static void setColor(Graphics _g, Color _c)
  {
    _g.setColor(getColor(_c,true));
  }

  /**
   * Sets color to a graphics.
   */
  public static void setColor(Graphics _g, Color _c, boolean _e)
  {
    _g.setColor(getColor(_c,_e));
  }

  public static Color getColor(Color _c)
  {
    return getColor(_c,true);
  }

  /**
   * Adapts a color according to the theme filters.
   */
  public static Color getColor(Color _c, boolean _e)
  {
    Color r=_c;

    Object o=null;
    if(!_e)     o=UIManager.get("Theme.disabledColorFilter");
    if(o==null) o=UIManager.get("Theme.colorFilter");
    if(o instanceof RGBImageFilter)
      r=new Color(((RGBImageFilter)o).filterRGB(0,0,_c.getRGB()));

    return r;
  }

  /**
   * Activates antialiasing.
   */
  public static void setAntialiasing(Graphics _g)
  {
    if(BuPreferences.BU.getBooleanProperty("antialias.all",false))
      setAntialiasing(_g,true);
  }

  private static Object key_      =null;
  private static Object value_on_ =null;
  private static Object value_off_=null;
  private static Method set_rh_   =null;

  /**
   * Enables or disables antialiasing.
   */
  public static void setAntialiasing(Graphics _g, boolean _e)
  {
    if(FuLib.jdk()<1.2) return;

    try
    {
      if(key_==null)
      {
        Class rh  =Class.forName("java.awt.RenderingHints");
        key_      =rh.getField("KEY_ANTIALIASING"   ).get(rh);
        value_on_ =rh.getField("VALUE_ANTIALIAS_ON" ).get(rh);
        value_off_=rh.getField("VALUE_ANTIALIAS_OFF").get(rh);
      
	Class  g2=Class.forName("java.awt.Graphics2D");
	set_rh_=g2.getMethod
	    ("setRenderingHint",
	     new Class[] { Class.forName("java.awt.RenderingHints$Key"),
			   Object.class } );
      }

      set_rh_.invoke(_g,new Object[]
	  { key_, (_e ? value_on_ : value_off_) });
    }
    catch(Throwable th) { }
  }

  // Mnemonics

  private final static String NO_AUTO_MNEMONIC="NO_AUTO_MNEMONIC";

  public static void setAutoMnemonic(JComponent _c,boolean _b)
  {
    _c.putClientProperty(NO_AUTO_MNEMONIC,(_b ? Boolean.TRUE : Boolean.FALSE));
  }

  public static boolean getAutoMnemonic(JComponent _c)
  {
    return (_c.getClientProperty(NO_AUTO_MNEMONIC)!=Boolean.FALSE);
  }

  public static void computeMnemonics(Container _rootpane)
    { computeMnemonics(_rootpane,null); }

  public static void computeMnemonics(Container _rootpane, ActionListener _al)
  {
    Vector    v=BuLib.getAllSubComponents(_rootpane);
    Hashtable t=new Hashtable();

    for(int i=0; i+1<v.size();i++)
    {
      Object c=v.elementAt(i);
      if(c instanceof JLabel)
      {
	JLabel lb=(JLabel)c;
	if(lb.getLabelFor()==null)
	{
	  Component tg=(Component)v.elementAt(i+1);
	  if(!tg.isFocusTraversable()&&(tg instanceof Container))
	  {
	    Vector w=BuLib.getAllSubComponents((Container)tg);
	    tg=null;
	    for(int j=0;j<w.size();j++)
	    {
	      Component d=(Component)w.elementAt(j);
	      if(d.isFocusTraversable())
		{ tg=d; break; }
	    }
	  }

	  if(tg!=null) lb.setLabelFor(tg);
	}
      }
    }

    for(Enumeration e=v.elements(); e.hasMoreElements(); )
    {
      Object c=e.nextElement();
      if(c instanceof AbstractButton)
      {
	AbstractButton ab  =(AbstractButton)c;
	boolean        auto=getAutoMnemonic(ab);
	int            mn  =ab.getMnemonic();

	if(auto&&(mn<=0)&&ab.isRequestFocusEnabled())
	{
	  String tx=candidateMnemonics(ab.getText());
	  if(tx!=null)
	    for(int j=0;j<tx.length();j++)
	    {
	      mn=(int)tx.charAt(j);
	      if(t.get(new Integer(mn))==null)
	      {
		t.put(new Integer(mn),ab);
		// System.err.println("### MNEMO "+(char)mn+" TO '"+tx+"'");
		ab.setMnemonic(mn);
		if(_al!=null)
		  ab.registerKeyboardAction
		    (_al,KeyStroke.getKeyStroke(mn,KeyEvent.ALT_MASK),
		     JComponent.WHEN_IN_FOCUSED_WINDOW);
		break;
	      }
	    }
	}
      }
      if(c instanceof JLabel)
      {
	JLabel  lb  =(JLabel)c;
	boolean auto=getAutoMnemonic(lb);
	int     mn  =lb.getDisplayedMnemonic();

	if(auto&&(mn<=0)&&(lb.getLabelFor()!=null))
	{
	  String tx=candidateMnemonics(lb.getText());
	  if(tx!=null)
	    for(int j=0;j<tx.length();j++)
	    {
	      mn=(int)tx.charAt(j);
	      if(t.get(new Integer(mn))==null)
	      {
		t.put(new Integer(mn),lb);
		lb.setDisplayedMnemonic(mn);
		break;
	      }
	    }
	}
      }
    }
  }

  public static String candidateMnemonics(String _s)
  {
    if(_s==null) return null;

    String s=_s;

    s=s.trim();
    if(s.endsWith(":"  )) s=s.substring(0,s.length()-1);
    if(s.endsWith("...")) s=s.substring(0,s.length()-3);
    s=s.trim();
    if("".equals(s)) return s;

    StringBuffer r=new StringBuffer(30);
    int i;

    if((s.length()>2)&&(s.charAt(1)==' '))
    {
      char c=s.charAt(0);
      if((c>='0')&&(c<='9'))
	r.append(c);
    }

    //if(s.indexOf(' ')<0)
    {
      char c=s.charAt(0);
      if((c>='A')&&(c<='Z'))
	r.append(c);
    }

    s=s.toUpperCase();

    for(i=s.length()-1;i>=0;i--)
      if(s.charAt(i)==' ')
      {
	char c=s.charAt(i+1);
	int  j=s.indexOf(' ',i+1);
	if((c>='A')&&(c<='Z')&&((j<0)||(j>i+5)))
	  r.append(c);
      }

    //for(i=s.length()-1;i>=0;i--)
    for(i=0;i<=s.length()-1;i++)
      if(s.charAt(i)=='-')
      {
	char c=s.charAt(i+1);
	int  j=s.indexOf('-',i+1);
	if((c>='A')&&(c<='Z')&&((j<0)||(j>i+1)))
	  r.append(c);
      }

    if(r.length()==0)
    {
      char c=s.charAt(0);
      if((c>='A')&&(c<='Z'))
	r.append(c);
    }

    while(!"".equals(s))
    {
      String t=s;
      int j=s.lastIndexOf(' ');
      if(j>=0) { t=s.substring(j+1); s=s.substring(0,j); }
      else     { t=s; s=""; }

      if(t.length()>4)
      {
	t=t.substring(0,t.length()-1);

	for(i=0;i<t.length();i++)
	{
	  char c=t.charAt(i);
	  if((c>='A')&&(c<='Z')&&("AEIOUY".indexOf(c)<0))
	  {
	    r.append(c);
	    /*
	    if(s.indexOf(c)<0) r.append(c);
	    else for(int k=0;k<r.length();k++)
	           if(r.charAt(k)==c) r.setCharAt(k,'=');
	    */
	  }
	}
	for(i=0;i<t.length();i++)
	{
	  char c=t.charAt(i);
	  if((c>='A')&&(c<='Z')&&("AEIOUY".indexOf(c)>=0))
	  {
	    r.append(c);
	    /*
	    if(s.indexOf(c)<0) r.append(c);
	    else for(int k=0;k<r.length();k++)
	           if(r.charAt(k)==c) r.setCharAt(k,'=');
	    */
	  }
	}
      }
    }

    char c,o;
    int l=r.length();
    StringBuffer f=new StringBuffer(l);

    o=0;
    for(i=0;i<l;i++)
    {
      c=r.charAt(i);
      if((c!='=')&&(c!=o)) { f.append(c); o=c; }
    }
    // System.err.println("CANDIDATE="+f+" FOR "+_s);

    return f.toString();
  }

  /*
  private static int getLabelMnemonic(JLabel _lb)
  {
    int r=0;
    try
    {
      Method m=_lb.getClass().getMethod("getMnemonic",new Class[0]);
      Object o=m.invoke(_lb,new Object[0]);
      r=((Integer)o).intValue();
    }
    catch(Exception ex) { }
    return r;
  }
  */

  /*
  private static void setLabelMnemonic(JLabel _lb, int _mn)
  {
    System.err.println("SET-MNEMO "+(char)_mn+" ON LABEL '"+_lb.getText()+"'");
    try
    {
      Method m=_lb.getClass().getMethod("setMnemonic",
					new Class[] { Integer.class });
      Object o=m.invoke(_lb,new Object[] { new Integer(_mn) } );
    }
    catch(Exception ex) { }
  }
  */

  public static void runShellCommand(BuCommonInterface _app, String _cmd)
  {
    runShellCommand
      (_app,_cmd,BuPreferences.BU.getBooleanProperty("tool.confirm",true));
  }

  public static void runShellCommand
    (BuCommonInterface _app, String _cmd, boolean _confirm)
  {
    if(  (!_confirm)
       ||(JOptionPane.YES_OPTION==
           (new BuDialogConfirmation
	     (_app,null,
	      FuLib.replace(_cmd,"; ",";\n").trim()+"\n"+
	      "Voulez-vous continuer ?").activate())))
    {
      String s=FuLib.runShellCommand(_cmd);
      if(!"".equals(s))
	new BuDialogMessage(_app,null,s).activate();
    }
  }

  /**
   * @return true if lnf is metal
   */
  public static boolean isMetal()
  {
    return UIManager.getLookAndFeel().getClass()
      .getName().endsWith(".MetalLookAndFeel");
  }

  /**
   * @return true if lnf is motif
   */
  public static boolean isMotif()
  {
    return UIManager.getLookAndFeel().getClass()
      .getName().endsWith(".MotifLookAndFeel");
  }

  /**
   * @return true if lnf is motif
   */
  public static boolean isSlaf()
  {
    return UIManager.getLookAndFeel().getClass()
      .getName().endsWith(".SlafLookAndFeel");
  }

  /**
   * @return true if lnf is kunststoff
   */
  public static boolean isKunststoff()
  {
    return UIManager.getLookAndFeel().getClass()
      .getName().endsWith(".KunststoffLookAndFeel");
  }

  /**
   * @return true if swing 1.0 is available
   */
  public static boolean isSwing10()
  {
    // + needed because of the patch
    return UIManager.class.getName().equals("com.sun."+"java.swing.UIManager");
  }

  /**
   * @return true if swing 1.1 is available
   */
  public static boolean isSwing11()
  {
    return UIManager.class.getName().equals("javax.swing.UIManager");
  }

  public final static String[] BU_CLASSES=new String[]
  {
    "com.memoire.bu.BuValueValidator",
    "com.memoire.bu.BuStringValidator",
    "com.memoire.bu.BuCharValidator",
    "com.memoire.bu.BuTextField",
    "com.memoire.bu.BuGridLayout",
    "com.memoire.bu.BuInternalFrame",
    "com.memoire.bu.BuTask",
    "com.memoire.bu.BuTaskOperation",
    "com.memoire.bu.BuScrollPane",
    "com.memoire.bu.BuEmptyList",
    "com.memoire.bu.BuTaskView",
    "com.memoire.bu.BuApplet",
    "com.memoire.bu.BuToggleButton",
    //"BuAssistant",
    "com.memoire.bu.BuActionRemover",
    "com.memoire.bu.BuPrinter",
    "com.memoire.bu.BuInformationsDocument",
    "com.memoire.bu.BuActionEnabler",
    "com.memoire.bu.BuSubApp",
    "com.memoire.bu.BuButton",
    "com.memoire.bu.BuToolBar",
    "com.memoire.bu.BuCheckBoxMenuItem",
    "com.memoire.bu.BuSeparator",
    "com.memoire.bu.BuDynamicMenu",
    "com.memoire.bu.BuMenuInternalFrames",
    "com.memoire.bu.BuSlafCustomTheme",
    "com.memoire.bu.BuMetalCustomTheme",
    "com.memoire.bu.BuMenuItem",
    "com.memoire.bu.BuFilters",
    "com.memoire.bu.BuMenu",
    "com.memoire.bu.BuMenuBar",
    "com.memoire.bu.BuColumn",
    "com.memoire.bu.BuBackgroundPainter",
    "com.memoire.bu.BuDesktopManager",
    "com.memoire.bu.BuDesktop",
    "com.memoire.bu.BuMemoryMonitor",
    "com.memoire.bu.BuStatusBar",
    "com.memoire.bu.BuSpecificBar",
    "com.memoire.bu.BuMainPanel",
    "com.memoire.bu.BuPanel",
    "com.memoire.bu.BuOverlayLayout",
    "com.memoire.bu.BuProgressBar",
    "com.memoire.bu.BuLabelMultiLine",
    "com.memoire.bu.BuLabel",
    "com.memoire.bu.BuLightBorder",
    "com.memoire.bu.BuVerticalLayout",
    "com.memoire.bu.BuPicture",
    "com.memoire.bu.BuApplication",
    "com.memoire.bu.BuPopupWindow",
    "com.memoire.bu.BuSplashScreen",
    "com.memoire.bu.BuIcon",
    "com.memoire.bu.BuResource",
    "com.memoire.bu.BuInformationsSoftware",
    "com.memoire.bu.BuSelectFindReplaceInterface",
    "com.memoire.bu.BuUndoRedoInterface",
    "com.memoire.bu.BuCutCopyPasteInterface",
    "com.memoire.bu.BuCommonInterface",
    "com.memoire.bu.BuCommonImplementation",
    "com.memoire.bu.BuPreferences",
  };

  public final static String[] SWING_CLASSES=new String[]
  {
    "javax.swing.MenuSelectionManager",
    "javax.swing.event.InternalFrameEvent",
    "javax.swing.TimerQueue",
    "javax.swing.event.ListSelectionEvent",
    "javax.swing.event.ListDataEvent",
    "javax.swing.JComboBox",
    "javax.swing.JCheckBox",
    "javax.swing.DefaultListModel",
    "javax.swing.event.ListDataListener",
    "javax.swing.ListCellRenderer",
    "javax.swing.Autoscroller",
    "javax.swing.ListSelectionModel",
    "javax.swing.DefaultListSelectionModel",
    "javax.swing.ListModel",
    "javax.swing.AbstractListModel",
    "javax.swing.JList",
    "javax.swing.JApplet",
    "javax.swing.JToggleButton",
    "javax.swing.ToolTipManager",
    "javax.swing.JCheckBoxMenuItem",
    "javax.swing.JSeparator",
    "javax.swing.event.PopupMenuListener",
    "javax.swing.JMenuBar",
    "javax.swing.SizeRequirements",
    "javax.swing.Scrollable",
    "javax.swing.ViewportLayout",
    "javax.swing.JViewport",
    "javax.swing.Timer",
    "javax.swing.Action",
    "javax.swing.AbstractAction",
    "javax.swing.OverlayLayout",
    "javax.swing.ButtonModel",
    "javax.swing.DefaultButtonModel",
    "javax.swing.JButton",
    "javax.swing.JScrollBar",
    "javax.swing.ScrollPaneLayout",
    "javax.swing.ScrollPaneConstants",
    "javax.swing.JScrollPane",
    "javax.swing.DesktopManager",
    "javax.swing.DefaultDesktopManager",
    "javax.swing.JDesktopPane",
    "javax.swing.BoxLayout",
    "javax.swing.JToolBar",
    "javax.swing.JToolBar",
    "javax.swing.JDesktopPane",
    "javax.swing.DesktopManager",
    "javax.swing.DefaultDesktopManager",
    "javax.swing.JMenuBar",
    "javax.swing.JSeparator",
    "javax.swing.JCheckBoxMenuItem",
    "javax.swing.JToolBar",
    "javax.swing.JButton",
    "javax.swing.JToggleButton",
    "javax.swing.JApplet",
    "javax.swing.JList",
    "javax.swing.ScrollPaneConstants",
    "javax.swing.JScrollPane",
    "javax.swing.text.EditorKit",
    "javax.swing.text.DefaultEditorKit",
    "javax.swing.Action",
    "javax.swing.AbstractAction",
    "javax.swing.text.TextAction",
    "javax.swing.text.Keymap",
    "javax.swing.Scrollable",
    "javax.swing.text.JTextComponent",
    "javax.swing.JTextField",
    "javax.swing.event.ChangeEvent",
    "javax.swing.SwingGraphics",
    "javax.swing.DebugGraphicsInfo",
    "javax.swing.DebugGraphics",
    "javax.swing.JWindow",
    "javax.swing.AbstractButton",
    "javax.swing.JMenuItem",
    "javax.swing.JMenu",
    "javax.swing.JInternalFrame",
    "javax.swing.JLayeredPane",
    "javax.swing.JRootPane",
    "javax.swing.CellRendererPane",
    "javax.swing.event.ChangeListener",
    "javax.swing.BoundedRangeModel",
    "javax.swing.DefaultBoundedRangeModel",
    "javax.swing.JProgressBar",
    "javax.swing.JLabel",
    "javax.swing.RepaintManager",
    "javax.swing.SystemEventQueueUtilities",
    "javax.swing.JPanel",
    "javax.swing.SingleSelectionModel",
    "javax.swing.DefaultSingleSelectionModel",
    "javax.swing.event.EventListenerList",
    "javax.swing.RootPaneContainer",
    "javax.swing.WindowConstants",
    "javax.swing.JFrame",
    "javax.swing.MenuElement",
    "javax.swing.JComponent",
    "javax.swing.JPopupMenu",
    "javax.swing.border.MatteBorder",
    "javax.swing.border.CompoundBorder",
    "javax.swing.border.EmptyBorder",
    "javax.swing.border.EtchedBorder",
    "javax.swing.BorderFactory",
    "javax.swing.border.LineBorder",
    "javax.swing.KeyStroke",
    "javax.swing.border.AbstractBorder",
    "javax.swing.border.BevelBorder",
    "javax.swing.border.Border",
    "javax.swing.LookAndFeel",
    "javax.swing.MultiUIDefaults",
    "javax.swing.UIDefaults",
    "javax.swing.SwingConstants",
    "javax.swing.SwingUtilities",
    "javax.swing.UIManager",
    "javax.swing.Icon",
    "javax.swing.ImageIcon",
    "javax.swing.event.ListSelectionListener",
    "javax.swing.event.InternalFrameListener",
  };
}
