/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuLookPreferencesPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * A panel where the user can choose his favorite look.
 * (background, look and feel, icons, ...)
 */

/**
 * desktop.decor
 * desktop.texture
 * desktop.background
 * lookandfeel.name
 * lookandfeel.class
 * lookandfeel.theme
 * icons.size
 */

public class BuLookPreferencesPanel
       extends BuAbstractPreferencesPanel
       implements ActionListener
{
  private final class OptionItem
  {
    public String texte;
    public Icon icone;
    public boolean activee;
  
    public OptionItem(String _texte, Icon _icone, boolean _activee)
    {
      texte=_texte;
      activee=_activee;
      icone=_icone;

      if(!activee&&(_icone instanceof ImageIcon))
	_icone=new ImageIcon
	  (BuLookPreferencesPanel.this.createImage
	   (new FilteredImageSource
	    (((ImageIcon)_icone).getImage().getSource(),
	     BuFilters.getDisabled())));

      int s=BuResource.BU.getDefaultSize();
      if((_icone!=null)&&!(_icone instanceof BuFixedSizeIcon))
      {
	int wi=_icone.getIconWidth();
	int hi=_icone.getIconHeight();
	if((wi<s)||(hi<s))
	  icone=new BuFixedSizeIcon
	    (_icone,Math.max(wi,s),Math.max(hi,s));
      }
    }

    public OptionItem(String _texte, Icon _icone)
    {
      this(_texte,_icone,true);
    }
  }

  /*

  private final class XCBCR
    implements ListCellRenderer
  {
    public Component getListCellRendererComponent
      (JList list,
       Object value,
       int index,
       boolean isSelected,
       boolean cellHasFocus)
    {
      return new XCBCC((OptionItem)value, isSelected, cellHasFocus, list);
    }
  }

  private final class XCBCC
    extends JLabel
  {
    public XCBCC
      (OptionItem _option, boolean _selected, boolean _focus, JList _list)
    {
      super("",null,SwingConstants.LEFT);
      if(_option!=null)
      {
	this.setText(_option.texte);
	this.setEnabled(_option.activee);
	this.setIcon(_option.icone);
      }
      else
      {
	this.setText("Erreur");
	this.setEnabled(false);
      }

      if(_selected)
      {
	this.setOpaque(true);
	this.setBackground(_list.getSelectionBackground());
	this.setForeground(_list.getSelectionForeground());
	this.setBorder(UIManager.getBorder("ComboBox.selectedCellBorder"));
      }
      else
      {
	this.setOpaque(false);
	this.setBackground(_list.getBackground());
	this.setForeground(_list.getForeground());
	this.setBorder(UIManager.getBorder("ComboBox.cellBorder"));
      }
    }
  }
  */

  /*
  private final class XCBCR
    extends BuLabel
    implements ListCellRenderer
  {
    public Component getListCellRendererComponent
      (JList _list,
       Object _value,
       int _index,
       boolean _selected,
       boolean _focus)
    {
      OptionItem _option=(OptionItem)_value;

      setHorizontalAlignment(LEFT);

      if(_option!=null)
      {
	this.setText(_option.texte);
	this.setEnabled(_option.activee&&_list.isEnabled());
	this.setIcon(_option.icone);
	this.setDisabledIcon(_option.icone);
      }
      else
      {
	this.setText("Erreur");
	this.setEnabled(false);
	this.setIcon(null);
      }

      if(_selected)
      {
	this.setOpaque(true);
	this.setBackground(_list.getSelectionBackground());
	this.setForeground(_list.getSelectionForeground());
	this.setBorder(UIManager.getBorder("ComboBox.selectedCellBorder"));
      }
      else
      {
	this.setOpaque(false);
	this.setBackground(_list.getBackground());
	this.setForeground(_list.getForeground());
	this.setBorder(UIManager.getBorder("ComboBox.cellBorder"));
      }

      return this;
    }
  }
  */

  private final class XCBCR
    extends BuAbstractCellRenderer
  {
    public XCBCR()
    {
      super(BuAbstractCellRenderer.COMBOBOX);
    }

    public Component getListCellRendererComponent
      (JList _list,
       Object _value,
       int _index,
       boolean _selected,
       boolean _focus)
    {
      BuLabel r=(BuLabel)super.getListCellRendererComponent
	  (_list,_value,_index,_selected,_focus);

      OptionItem _option=(OptionItem)_value;

      if(_option!=null)
      {
	r.setText(_option.texte);
	r.setEnabled(_option.activee&&_list.isEnabled());
	r.setIcon(_option.icone);
	r.setDisabledIcon(_option.icone);
      }
      else
      {
	r.setText(BuResource.BU.getString("Erreur"));
	r.setEnabled(false);
	r.setIcon(null);
      }

      return r;
    }
  }

  BuCommonInterface     appli_;
  BuPreferences         options_;
  Hashtable             optionsStr_;
  XCBCR                 cbRenderer_;

  // Panneau des options generales
  BuGridLayout   loGen_;
  BuPanel        pnGenDesk_;
  AbstractBorder boGenDesk_;
  BuGridLayout   loGenDesk_;
  BuLabel        lbGenDeskDecor_, lbGenDeskTexture_, lbGenDeskCouleur_;
  BuComboBox     liGenDeskDecor_, liGenDeskTexture_, liGenDeskCouleur_;
  BuLabel        lbGenAspAspect_, lbGenAspSlaf_, lbGenAspMetal_;
  BuComboBox     liGenAspAspect_, liGenAspSlaf_, liGenAspMetal_;
  BuLabel        lbGenAspSkinlf_, lbGenAspOyoaha_;
  BuTextField    tfGenAspSkinlf_, tfGenAspOyoaha_;

  BuLabel        lbIcones_;
  BuComboBox     chIcones_;

  public String getTitle()
    { return BuResource.BU.getString("Aspect"); }

  // Constructeur
  
  public BuLookPreferencesPanel(BuCommonInterface _appli)
  {
    super();
    appli_     =_appli;
    options_   =BuPreferences.BU;
    cbRenderer_=new XCBCR();
    optionsStr_=new Hashtable();

    OptionItem itemOpt;
    String     itemStr;

    optionsStr_.put(itemStr="16",
		    itemOpt=new OptionItem(BuResource.BU.getString("16x16 (standard)"),
					   new BuFixedSizeIcon
					   (BuResource.BU.getIcon("ouvrir",16),26,26)));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="20",
		    itemOpt=new OptionItem(BuResource.BU.getString("20x20 (dean)"),
					   new BuFixedSizeIcon
					   (BuResource.BU.getIcon("ouvrir",20),26,26)));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="22",
		    itemOpt=new OptionItem(BuResource.BU.getString("22x22 (kde)"),
					   new BuFixedSizeIcon
					   (BuResource.BU.getIcon("ouvrir",22),26,26)));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="24",
		    itemOpt=new OptionItem(BuResource.BU.getString("24x24 (gnome)"),
					   new BuFixedSizeIcon
					   (BuResource.BU.getIcon("ouvrir",24),26,26)));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="26",
		    itemOpt=new OptionItem(BuResource.BU.getString("26x26 (swing)"),
					   new BuFixedSizeIcon
					   (BuResource.BU.getIcon("ouvrir",26),26,26)));
    optionsStr_.put(itemOpt, itemStr);

    optionsStr_.put(itemStr="DESKTOP_UNIFORME",
		    itemOpt=new OptionItem(BuResource.BU.getString("Uniforme"),
					   BuResource.BU.getIcon("uniforme")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_DEGRADE",
		    itemOpt=new OptionItem(BuResource.BU.getString("Dgrad"),
					   BuResource.BU.getIcon("degrade")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_TEXTURE",
		    itemOpt=new OptionItem(BuResource.BU.getString("Textur"),
					   BuResource.BU.getIcon("texture")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_TEXTURE1",
		    itemOpt=new OptionItem(BuResource.BU.getString("Texture")+" 1",
					   new BuTextureIcon(BuResource.BU.getImage("background_desktop_1"))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_TEXTURE2",
		    itemOpt=new OptionItem(BuResource.BU.getString("Texture")+" 2",
					   new BuTextureIcon(BuResource.BU.getImage("background_desktop_2"))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_TEXTURE3",
		    itemOpt=new OptionItem(BuResource.BU.getString("Texture")+" 3",
					   new BuTextureIcon(BuResource.BU.getImage("background_desktop_3"))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_TEXTURE4",
		    itemOpt=new OptionItem
		    (BuResource.BU.getString("Texture")+" 4",
		     new BuTextureIcon(new ImageIcon
				       (BuPreferences.BU.getStringProperty("desktop.image")).getImage())));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_ROUGE",
		    itemOpt=new OptionItem(BuResource.BU.getString("Rouge"),
					   new BuColorIcon(new Color(128, 64, 64))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_VERT",
		    itemOpt=new OptionItem(BuResource.BU.getString("Vert"),
					   new BuColorIcon(new Color( 64,129, 64))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_BLEU",
		    itemOpt=new OptionItem(BuResource.BU.getString("Bleu"),
					   new BuColorIcon(new Color( 64, 64,128))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_ORANGE",
		    itemOpt=new OptionItem(BuResource.BU.getString("Orange"),
					   new BuColorIcon(new Color(192,128, 96))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_SIMILAIRE",
		    itemOpt=new OptionItem(BuResource.BU.getString("Similaire"),
					   new BuColorIcon(UIManager.getColor("Panel.background"))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="DESKTOP_DEFAUT",
		    itemOpt=new OptionItem(BuResource.BU.getString("Dfaut"),
					   new BuColorIcon(UIManager.getColor("Desktop.background"))));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_DEFAUT",
		    itemOpt=new OptionItem(BuResource.BU.getString("Dfaut"),
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_DEFAUT")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_AMIGA",
		    itemOpt=new OptionItem("Amiga",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_AMIGA")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_KUNSTSTOFF",
		    itemOpt=new OptionItem("Kunststoff",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_KUNSTSTOFF")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_MAC",
		    itemOpt=new OptionItem("Mac",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_MAC")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_METAL",
		    itemOpt=new OptionItem("Metal",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_METAL")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_MOTIF",
		    itemOpt=new OptionItem("Motif",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_MOTIF")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_NEXT",
		    itemOpt=new OptionItem("Next",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_NEXT")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_ORGANIC",
		    itemOpt=new OptionItem("Organic",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_ORGANIC")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_OYOAHA",
		    itemOpt=new OptionItem("Oyoaha",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_OYOAHA")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_SKINLF",
		    itemOpt=new OptionItem("SkinLF",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_SKINLF")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_SLAF",
		    itemOpt=new OptionItem("Slaf",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_SLAF")));
    optionsStr_.put(itemOpt, itemStr);
    optionsStr_.put(itemStr="ASPECT_WINDOWS",
		    itemOpt=new OptionItem("Windows",
					   BuLib.loadCommandIcon(itemStr),
					   options_.isEnabled("ASPECT_WINDOWS")));
    optionsStr_.put(itemOpt, itemStr);

    Object[] items;
    int nGen, nGenDesk;
    
    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));

    nGen=0;

    // Desktop
    pnGenDesk_=new BuPanel();
    boGenDesk_=new CompoundBorder
      (new TitledBorder
       (BuResource.BU.getString("Aspect")), new EmptyBorder(5,5,5,5));
    pnGenDesk_.setBorder(boGenDesk_);
    loGenDesk_=new BuGridLayout();
    loGenDesk_.setColumns(2);
    loGenDesk_.setVgap(5);
    loGenDesk_.setHgap(5);
    pnGenDesk_.setLayout(loGenDesk_);
    nGenDesk=0;

    // Aspect
    lbGenAspAspect_=new BuLabel(BuResource.BU.getString("Aspect:"),JLabel.RIGHT);
    items=new OptionItem[12];
    items[ 0]=optionsStr_.get("ASPECT_DEFAUT");
    items[ 1]=optionsStr_.get("ASPECT_AMIGA");
    items[ 2]=optionsStr_.get("ASPECT_KUNSTSTOFF");
    items[ 3]=optionsStr_.get("ASPECT_MAC");
    items[ 4]=optionsStr_.get("ASPECT_METAL");
    items[ 5]=optionsStr_.get("ASPECT_MOTIF");
    items[ 6]=optionsStr_.get("ASPECT_ORGANIC");
    items[ 7]=optionsStr_.get("ASPECT_NEXT");
    items[ 8]=optionsStr_.get("ASPECT_OYOAHA");
    items[ 9]=optionsStr_.get("ASPECT_SKINLF");
    items[10]=optionsStr_.get("ASPECT_SLAF");
    items[11]=optionsStr_.get("ASPECT_WINDOWS");
    liGenAspAspect_=new BuComboBox(items);
    liGenAspAspect_.setRenderer(cbRenderer_);
    liGenAspAspect_.setEditable(false);
    liGenAspAspect_.addActionListener(this);
    pnGenDesk_.add(lbGenAspAspect_, nGenDesk++);
    pnGenDesk_.add(liGenAspAspect_, nGenDesk++);

    // Metal theme
    lbGenAspMetal_=new BuLabel(BuResource.BU.getString
			      ("Thme Metal:"),JLabel.RIGHT);
    BuMetalCustomTheme[] metalThemes=BuMetalCustomTheme.getList();
    items=new OptionItem[metalThemes.length-4];
    for(int i=0,j=0;i<metalThemes.length;i++)
    {
      BuMetalCustomTheme s=metalThemes[i];
      if(s!=null)
      {
	optionsStr_.put(itemStr="THEME_METAL"+i,
			itemOpt=new OptionItem(s.getName(),s.getIcon()));
	optionsStr_.put(itemOpt, itemStr);
	items[j]=optionsStr_.get("THEME_METAL"+i);
	j++;
      }
    }
    liGenAspMetal_=new BuComboBox(items);
    liGenAspMetal_.setRenderer(cbRenderer_);
    liGenAspMetal_.setEditable(false);
    liGenAspMetal_.addActionListener(this);
    pnGenDesk_.add(lbGenAspMetal_, nGenDesk++);
    pnGenDesk_.add(liGenAspMetal_, nGenDesk++);

    // Slaf theme
    lbGenAspSlaf_=new BuLabel(BuResource.BU.getString
			     ("Thme Slaf:"),JLabel.RIGHT);
    BuSlafCustomTheme[] slafThemes=BuSlafCustomTheme.getList();
    items=new OptionItem[slafThemes.length-3];
    for(int i=0,j=0;i<slafThemes.length;i++)
    {
      BuSlafCustomTheme s=slafThemes[i];
      if(s!=null)
      {
	optionsStr_.put(itemStr=s.getKey(),
			itemOpt=new OptionItem(s.getName(),s.getIcon()));
	optionsStr_.put(itemOpt, itemStr);
	items[j]=optionsStr_.get(s.getKey());
	j++;
      }
    }
    liGenAspSlaf_=new BuComboBox(items);
    liGenAspSlaf_.setRenderer(cbRenderer_);
    liGenAspSlaf_.setEditable(false);
    liGenAspSlaf_.addActionListener(this);
    pnGenDesk_.add(lbGenAspSlaf_, nGenDesk++);
    pnGenDesk_.add(liGenAspSlaf_, nGenDesk++);

    // Skinlf theme
    lbGenAspSkinlf_=new BuLabel(BuResource.BU.getString
			     ("Thme Skinlf:"),JLabel.RIGHT);
    tfGenAspSkinlf_=new BuTextField(25);
    tfGenAspSkinlf_.addActionListener(this);
    pnGenDesk_.add(lbGenAspSkinlf_, nGenDesk++);
    pnGenDesk_.add(tfGenAspSkinlf_, nGenDesk++);

    // Oyoaha theme
    lbGenAspOyoaha_=new BuLabel(BuResource.BU.getString
				("Thme Oyoaha:"),JLabel.RIGHT);
    tfGenAspOyoaha_=new BuTextField(25);
    tfGenAspOyoaha_.addActionListener(this);
    pnGenDesk_.add(lbGenAspOyoaha_, nGenDesk++);
    pnGenDesk_.add(tfGenAspOyoaha_, nGenDesk++);

    // Icones
    lbIcones_=new BuLabel(BuResource.BU.getString("Icnes:"),JLabel.RIGHT);
    items=new OptionItem[5];
    items[0]=optionsStr_.get("16");
    items[1]=optionsStr_.get("20");
    items[2]=optionsStr_.get("22");
    items[3]=optionsStr_.get("24");
    items[4]=optionsStr_.get("26");
    chIcones_=new BuComboBox(items);
    chIcones_.setRenderer(cbRenderer_);
    chIcones_.setEditable(false);
    chIcones_.addActionListener(this);
    pnGenDesk_.add(lbIcones_, nGenDesk++);
    pnGenDesk_.add(chIcones_, nGenDesk++);

    // Decor
    lbGenDeskDecor_=new BuLabel(BuResource.BU.getString("Dcor:"),JLabel.RIGHT);
    items=new OptionItem[3];
    items[0]=optionsStr_.get("DESKTOP_UNIFORME");
    items[1]=optionsStr_.get("DESKTOP_DEGRADE"); 
    items[2]=optionsStr_.get("DESKTOP_TEXTURE"); 
    liGenDeskDecor_=new BuComboBox(items);
    liGenDeskDecor_.setRenderer(cbRenderer_);
    liGenDeskDecor_.setEditable(false);
    liGenDeskDecor_.addActionListener(this);
    pnGenDesk_.add(lbGenDeskDecor_, nGenDesk++);
    pnGenDesk_.add(liGenDeskDecor_, nGenDesk++);

    // Texture
    lbGenDeskTexture_=new BuLabel(BuResource.BU.getString("Texture:"),JLabel.RIGHT);
    items=new OptionItem[4];
    items[0]=optionsStr_.get("DESKTOP_TEXTURE1");
    items[1]=optionsStr_.get("DESKTOP_TEXTURE2");
    items[2]=optionsStr_.get("DESKTOP_TEXTURE3");
    items[3]=optionsStr_.get("DESKTOP_TEXTURE4");
    liGenDeskTexture_=new BuComboBox(items);
    liGenDeskTexture_.setRenderer(cbRenderer_);
    liGenDeskTexture_.setEditable(false);
    liGenDeskTexture_.addActionListener(this);
    pnGenDesk_.add(lbGenDeskTexture_, nGenDesk++);
    pnGenDesk_.add(liGenDeskTexture_, nGenDesk++);
    
    // Couleur
    lbGenDeskCouleur_=new BuLabel(BuResource.BU.getString("Couleur:"),JLabel.RIGHT);
    items=new OptionItem[6];
    items[0]=optionsStr_.get("DESKTOP_ROUGE");
    items[1]=optionsStr_.get("DESKTOP_VERT");
    items[2]=optionsStr_.get("DESKTOP_BLEU");
    items[3]=optionsStr_.get("DESKTOP_ORANGE");
    items[4]=optionsStr_.get("DESKTOP_SIMILAIRE");
    items[5]=optionsStr_.get("DESKTOP_DEFAUT");
    liGenDeskCouleur_=new BuComboBox(items);
    liGenDeskCouleur_.setRenderer(cbRenderer_);
    liGenDeskCouleur_.setEditable(false);
    liGenDeskCouleur_.addActionListener(this);
    pnGenDesk_.add(lbGenDeskCouleur_, nGenDesk++);
    pnGenDesk_.add(liGenDeskCouleur_, nGenDesk++);

    add(pnGenDesk_,BuBorderLayout.CENTER);

    updateComponents();
  }
  
  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    setAbility();
  }

  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
    options_.applyOn(appli_);
  }

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }
  
  // Methodes privees
  
  private void fillTable()
  {
    Object o;

    o=liGenDeskDecor_  .getSelectedItem(); if(o==null) o="DESKTOP_DEGRADE";
    options_.putStringProperty("desktop.decor"      , ""+optionsStr_.get(o));

    o=liGenDeskTexture_.getSelectedItem(); if(o==null) o="DESKTOP_TEXTURE1";
    options_.putStringProperty("desktop.texture"    , ""+optionsStr_.get(o));

    o=liGenDeskCouleur_.getSelectedItem(); if(o==null) o="DESKTOP_DEFAUT";
    options_.putStringProperty("desktop.background" , ""+optionsStr_.get(o));

    o=liGenAspAspect_.getSelectedItem(); if(o==null) o="ASPECT_DEFAUT";
    options_.putStringProperty("lookandfeel.name"   , ""+optionsStr_.get(o));

    o=liGenAspMetal_.getSelectedItem(); if(o==null) o="THEME_METAL0";
    options_.putStringProperty("metal.theme"        , ""+optionsStr_.get(o));

    o=liGenAspSlaf_.getSelectedItem(); if(o==null) o="default";
    options_.putStringProperty("slaf.theme"         , ""+optionsStr_.get(o));

    o=chIcones_.getSelectedItem(); if(o==null) o="16";
    options_.putStringProperty("icons.size"         , ""+optionsStr_.get(o));

    options_.putStringProperty("skinlf.theme",tfGenAspSkinlf_.getText());
    options_.putStringProperty("oyoaha.theme",tfGenAspOyoaha_.getText());
  }

  private void updateComponents()
  {
    liGenDeskDecor_      .setSelectedItem(optionsStr_.get(options_.getStringProperty("desktop.decor"     ,"DESKTOP_DEGRADE"   )));
    liGenDeskTexture_    .setSelectedItem(optionsStr_.get(options_.getStringProperty("desktop.texture"   ,"DESKTOP_TEXTURE1"  )));
    liGenDeskCouleur_    .setSelectedItem(optionsStr_.get(options_.getStringProperty("desktop.background","DESKTOP_DEFAUT"    )));
    liGenAspAspect_      .setSelectedItem(optionsStr_.get(options_.getStringProperty("lookandfeel.name"  ,"ASPECT_DEFAUT"     )));
    liGenAspMetal_       .setSelectedItem(optionsStr_.get(options_.getStringProperty("metal.theme"       ,"THEME_METAL0"      )));
    liGenAspSlaf_        .setSelectedItem(optionsStr_.get(options_.getStringProperty("slaf.theme"        ,"default"           )));
    chIcones_            .setSelectedItem(optionsStr_.get(options_.getStringProperty("icons.size"        ,"16"                )));

    tfGenAspSkinlf_.setText
	(options_.getStringProperty
	 ("skinlf.theme","/usr/share/themes/Chrome/gtk/gtkrc"));
    tfGenAspOyoaha_.setText
	(options_.getStringProperty
	 ("oyoaha.theme","slushy.otm"));
    setAbility();
  }

  private void setAbility()
  {
    boolean b;

    b=(liGenDeskDecor_.getSelectedItem()==optionsStr_.get("DESKTOP_TEXTURE"));
    liGenDeskTexture_.setEnabled(b);
    liGenDeskCouleur_.setEnabled(!b);

    b=(liGenAspAspect_.getSelectedItem()==optionsStr_.get("ASPECT_METAL"));
    liGenAspMetal_.setEnabled(b);

    b=(liGenAspAspect_.getSelectedItem()==optionsStr_.get("ASPECT_SLAF"));
    liGenAspSlaf_.setEnabled(b);

    b=(liGenAspAspect_.getSelectedItem()==optionsStr_.get("ASPECT_SKINLF"));
    tfGenAspSkinlf_.setEnabled(b);

    b=(liGenAspAspect_.getSelectedItem()==optionsStr_.get("ASPECT_OYOAHA"));
    tfGenAspOyoaha_.setEnabled(b);
  }
}
