/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuMainPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * The main panel.
 * Contains a status bar, a specific button bar, two columns of
 * components and a desktop with internal frames.
 */

public class BuMainPanel
       extends BuPanel
{
  private BuSpecificBar specificbar_;
  private BuPanel       specificpanel_;
  private BuDesktop     desktop_;
  private BuStatusBar   statusbar_;
  private BuSplit3Pane  splitpane_;
  private BuColumn      leftcolumn_;
  private BuColumn      rightcolumn_;
  private boolean       swapped_=false;

  public BuMainPanel()
  {
    super();

    setName  ("buMAINPANEL");
    setLayout(new BuBorderLayout());
    setBorder(null);


    specificpanel_=new BuPanel();
    specificpanel_.setLayout(new BuBorderLayout());
    // specificpanel_.setName("pnSPECIFIC");
    add(specificpanel_,BuBorderLayout.CENTER);

    setSpecificBar(new BuSpecificBar());
    setStatusBar  (new BuStatusBar());

    desktop_=new BuDesktop();

    JComponent middle=desktop_;
    if(!desktop_.isTabbed())
    {
      BuScrollPane sp=new BuScrollPane(desktop_);
      sp.setVerticalScrollBarPolicy
	(BuScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
      sp.setHorizontalScrollBarPolicy
	(BuScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
      middle=sp;
    }

    leftcolumn_=new BuColumn();
    leftcolumn_.setName("buLEFTCOLUMN");
    rightcolumn_=new BuColumn();
    rightcolumn_.setName("buRIGHTCOLUMN");
    // specificpanel_.add(leftcolumn_ ,BuBorderLayout.WEST);
    // specificpanel_.add(rightcolumn_,BuBorderLayout.EAST);
    splitpane_=new BuSplit3Pane(leftcolumn_,middle,rightcolumn_);
    specificpanel_.add(splitpane_,BuBorderLayout.CENTER);
  }

  // Specific ToolBar

  public BuSpecificBar getSpecificBar()
  { return specificbar_; }

  public void setSpecificBar(BuSpecificBar _specificbar)
  {
    if(specificbar_!=null) remove(specificbar_);
    specificbar_=_specificbar;
    add(specificbar_,BuBorderLayout.NORTH);
  }

  // StatusBar

  public BuStatusBar getStatusBar()
  { return statusbar_; }

  public void setStatusBar(BuStatusBar _statusbar)
  {
    if(statusbar_!=null) specificpanel_.remove(statusbar_);
    statusbar_=_statusbar;
    specificpanel_.add(statusbar_,BuBorderLayout.SOUTH);
  }

  public void setMessage(String _s)
  { statusbar_.setMessage(_s); }

  public void setProgression(int _v)
  {
    statusbar_.setProgression(_v);

    Thread th=Thread.currentThread();
    if(th instanceof BuTask)
      ((BuTask)th).setProgression(_v);
  }

  // Desktop

  public BuDesktop getDesktop()
  { return desktop_; }

  public Icon getLogo()
  { return desktop_.getLogo(); }

  public void setLogo(Icon _icon)
  { desktop_.setLogo(_icon); }

  void addInternalFrame(JInternalFrame _f)
  {
    desktop_.addInternalFrame(_f);
    if(getAssistant()!=null)
      getAssistant().addEmitters(_f);
  }

  void removeInternalFrame(JInternalFrame _f)
  { desktop_.removeInternalFrame(_f); }

  void activateInternalFrame(JInternalFrame _f)
  { desktop_.activateInternalFrame(_f); }

  void deactivateInternalFrame(JInternalFrame _f)
  { desktop_.deactivateInternalFrame(_f); }

  JInternalFrame getCurrentInternalFrame()
  { return desktop_.getCurrentInternalFrame(); }

  public JInternalFrame[] getAllInternalFrames()
  { return desktop_.getAllFrames(); }

  public void waterfall()
  { desktop_.waterfall(); }

  public void tile()
  { desktop_.tile(); }

  public void arrangeIcons()
  { desktop_.arrangeIcons(); }

  public void arrangePalettes()
  { desktop_.arrangePalettes(); }

  // Columns

  public BuColumn getLeftColumn()
  { return leftcolumn_; }

  public void setLeftColumn(BuColumn _leftcolumn)
  {
    if(leftcolumn_!=_leftcolumn)
    {
      leftcolumn_=_leftcolumn;
      splitpane_.setLeftComponent(leftcolumn_);
      splitpane_.updateDisplay();
    }
  }

  public BuColumn getRightColumn()
  { return rightcolumn_; }

  public void setRightColumn(BuColumn _rightcolumn)
  {
    if(rightcolumn_!=_rightcolumn)
    {
      rightcolumn_=_rightcolumn;
      splitpane_.setRightComponent(_rightcolumn);
      splitpane_.updateDisplay();
    }
  }

  public boolean  isColumnsSwapped()
  { return swapped_; }

  public void swapColumns()
  {
    splitpane_.swapColumns();
    swapped_=!swapped_;

    /*
    specificpanel_.remove(leftcolumn_);
    specificpanel_.remove(rightcolumn_);

    swapped_=!swapped_;
    if(swapped_)
    {
      specificpanel_.add(leftcolumn_ ,BuBorderLayout.EAST);
      specificpanel_.add(rightcolumn_,BuBorderLayout.WEST);
    }
    else
    {
      specificpanel_.add(leftcolumn_ ,BuBorderLayout.WEST);
      specificpanel_.add(rightcolumn_,BuBorderLayout.EAST);
    }

    specificpanel_.revalidate();
    */
  }

  public void updateSplits()
  {
    splitpane_.updateSplits();
  }

  // Assistant

  private BuAssistant assistant_;
  public BuAssistant getAssistant() { return assistant_; }
  public void setAssistant(BuAssistant _assistant) { assistant_=_assistant; }

  // TaskView

  private BuTaskView task_view_;
  public BuTaskView getTaskView() { return task_view_; }
  public void setTaskView(BuTaskView _tv) { task_view_=_tv; }

  // Divers

  public void showOrHide(JComponent _c)
  {
    if(_c!=null)
    {
      boolean b=!_c.isVisible();
      _c.setVisible(b);
      revalidate();
      repaint();
    }
  }
}
