/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuMenu.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 * A menu with simplified adding methods.
 * A few static methods are available for standard menus as
 * File, Edition, Windows and Help.
 */

public class BuMenu
       extends JMenu
       implements ActionListener
{
  public BuMenu()
  { this("Menu","MENU",true,0); }

  public BuMenu(String _s, String _cmd)
  { this(_s,_cmd,null,false,0); }

  public BuMenu(String _s, String _cmd, BuIcon _icon)
  { this(_s,_cmd,_icon,false,0); }

  public BuMenu(String _s, String _cmd, boolean _tearoff)
  { this(_s,_cmd,null,_tearoff,0); }

  public BuMenu(String _s, String _cmd, boolean _tearoff, int _key)
  { this(_s,_cmd,null,_tearoff,0); }

  public BuMenu
  (String _s, String _cmd, BuIcon _icon, boolean _tearoff, int _key)
  {
    super(_s,_tearoff);

    if(_icon==null) _icon=BuResource.BU.loadMenuCommandIcon(_cmd);

    setName("mn"+_cmd);
    setActionCommand(_cmd);
    setHorizontalTextPosition(JButton.RIGHT);
    setIcon(_icon);

    // TMP
    // getPopupMenu().setLightWeightPopupEnabled(false);
  }

  // Anti-aliasing

  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  public void setIcon(BuIcon _icon)
    { BuLib.setIcon(this,_icon); }

  // Mnemonics

  public void computeMnemonics()
  {
    Component[] c=null;
    Hashtable   t=new Hashtable();

    try { c=getMenuComponents(); }
    catch(Exception ex) { } // swing 1.03

    {
      int mn=getMnemonic();
      if(mn>0) t.put(new Integer(mn),this);
    }

    for(int i=0; i<c.length; i++)
    {
      if(c[i] instanceof JMenuItem)
      {
	JMenuItem mi=(JMenuItem)c[i];
	int mn=mi.getMnemonic();
	if(mn<=0)
	{
	  String tx=BuLib.candidateMnemonics(mi.getText());
	  if(tx!=null)
	    for(int j=0;j<tx.length();j++)
	    {
	      mn=(int)tx.charAt(j);
	      if(t.get(new Integer(mn))==null)
	      {
		t.put(new Integer(mn),mi);
		mi.setMnemonic(mn);
		break;
	      }
	    }
	}
	if(mi instanceof BuMenu)
	  ((BuMenu)mi).computeMnemonics();
      }
    }
  }

  // Separator

  public BuSeparator addSeparator(String _text)
  {
    BuSeparator r=new BuSeparator(_text);
    this.add(r);
    return r;
  }

  // SubMenu

  public void addSubMenu(JMenu _m, boolean _enabled)
  {
    _m.setEnabled(_enabled);
    _m.addActionListener(this);
    this.add(_m);
  }

  // Menu Item

  public BuMenuItem addMenuItem(String _s, String _cmd)
  {
    return addMenuItem(_s,_cmd,true);
  }

  public BuMenuItem addMenuItem(String _s, String _cmd,
				ActionListener _al)
  {
    BuIcon icon=BuResource.BU.loadMenuCommandIcon(_cmd);
    return addMenuItem(_s,_cmd,icon,true,0,_al);
  }

  public BuMenuItem addMenuItem(String _s, String _cmd,
				boolean _enabled)
  {
    BuIcon icon=BuResource.BU.loadMenuCommandIcon(_cmd);
    return addMenuItem(_s,_cmd,icon,_enabled,0,this);
  }

  public BuMenuItem addMenuItem(String _s, String _cmd,
				boolean _enabled, int _key)
  {
    BuIcon icon=BuResource.BU.loadMenuCommandIcon(_cmd);
    return addMenuItem(_s,_cmd,icon,_enabled,_key,this);
  }

  public BuMenuItem addMenuItem(String _s, String _cmd, BuIcon _icon,
				boolean _enabled)
  {
    return addMenuItem(_s,_cmd,_icon,_enabled,0,this);
  }

  public BuMenuItem addMenuItem(String _s, String _cmd, BuIcon _icon,
				boolean _enabled, int _key)
  {
    return addMenuItem(_s,_cmd,_icon,_enabled,_key,this);
  }

  public BuMenuItem addMenuItem(String _s, String _cmd, BuIcon _icon,
				boolean _enabled, int _key, ActionListener _al)
  {
    BuMenuItem r=new BuMenuItem();
    r.setText(_s);
    r.setName("mi"+_cmd);
    r.setActionCommand(_cmd);
    r.setIcon(_icon);
    r.setHorizontalTextPosition(JButton.RIGHT);
    r.addActionListener(_al);
    r.setEnabled(_enabled);
    if(_key!=0)
    {
      if((_key>=KeyEvent.VK_F1)&&(_key<=KeyEvent.VK_F12))
	r.setAccelerator(KeyStroke.getKeyStroke(_key,KeyEvent.ALT_MASK));
      else
	r.setAccelerator(KeyStroke.getKeyStroke(_key,KeyEvent.CTRL_MASK));
    }
    this.add(r);
    return r;
  }

  // CheckBoxMenuItem

  public BuCheckBoxMenuItem addCheckBox
    (String _s, String _cmd, boolean _enabled, boolean _checked)
  {
    BuIcon icon=BuResource.BU.loadMenuCommandIcon(_cmd);
    return addCheckBox(_s,_cmd,icon,_enabled,_checked);
  }

  public BuCheckBoxMenuItem addCheckBox
    (String _s, String _cmd, BuIcon _icon, boolean _enabled, boolean _checked)
  {
    BuCheckBoxMenuItem r=new BuCheckBoxMenuItem();
    r.setText(_s);
    r.setName("mi"+_cmd);
    r.setActionCommand(_cmd);
    r.setIcon(_icon);
    r.setHorizontalTextPosition(JButton.RIGHT);
    r.addActionListener(this);
    r.setEnabled(_enabled);
    r.setSelected(_checked);
    this.add(r);
    return r;
  }

  // RadioButtonMenuItem

  public BuRadioButtonMenuItem addRadioButton
    (String _s, String _cmd, boolean _enabled, boolean _checked)
  {
    BuIcon icon=BuResource.BU.loadMenuCommandIcon(_cmd);
    return addRadioButton(_s,_cmd,icon,_enabled,_checked);
  }

  public BuRadioButtonMenuItem addRadioButton
    (String _s, String _cmd, BuIcon _icon, boolean _enabled, boolean _checked)
  {
    BuRadioButtonMenuItem r=new BuRadioButtonMenuItem();
    r.setText(_s);
    r.setName("mi"+_cmd);
    r.setActionCommand(_cmd);
    r.setIcon(_icon);
    r.setHorizontalTextPosition(JButton.RIGHT);
    r.addActionListener(this);
    r.setEnabled(_enabled);
    r.setSelected(_checked);
    this.add(r);
    return r;
  }

  /*
  public void removeMenuItem(String _cmd)
  {
    MenuElement[] cm=getSubElements();
    for(int i=0; i<cm.length; i++)
      if(cm[i] instanceof JPopupMenu)
      {
	MenuElement[] cp=((JPopupMenu)cm[i]).getSubElements();
	for(int j=0; j<cp.length; j++)
	  if(cp[j] instanceof JMenuItem)
	  {
	    JMenuItem m=(JMenuItem)cp[j];
	    if(m.getActionCommand().equals(_cmd))
	      remove(m);
	  }
      }
  }
  */

  // Menus standards

  public static BuMenu buildImportMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Importer"),"IMPORTER");
    return r;
  }

  public static BuMenu buildExportMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Exporter"),"EXPORTER");
    return r;
  }

  public static BuMenu buildFileMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Fichier"),"MENU_FICHIER");
    r.addMenuItem(BuResource.BU.getString("Crer"              ),"CREER"            ,false,KeyEvent.VK_N);
    r.addMenuItem(BuResource.BU.getString("Ouvrir..."          ),"OUVRIR"           ,false,KeyEvent.VK_O);

    r.addSubMenu(new BuMenuRecentFiles(),true);
    r.addMenuItem(BuResource.BU.getString("Proprits..."      ),"PROPRIETE"        ,false);
    r.addSeparator();

    r.addMenuItem(BuResource.BU.getString("Enregistrer"        ),"ENREGISTRER"      ,false,KeyEvent.VK_S);
    r.addMenuItem(BuResource.BU.getString("Enregistrer sous..."),"ENREGISTRERSOUS"  ,false)
      .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,KeyEvent.CTRL_MASK|KeyEvent.SHIFT_MASK));
    r.addMenuItem(BuResource.BU.getString("Fermer"             ),"FERMER"           ,false,KeyEvent.VK_W);
    r.addSeparator();

    r.addSubMenu(buildImportMenu(),false);
    r.addSubMenu(buildExportMenu(),false);
    //r.addSeparator();

    r.addMenuItem(BuResource.BU.getString("Imprimer..."        ),"IMPRIMER"         ,false,KeyEvent.VK_P);
    r.addMenuItem(BuResource.BU.getString("Prvisualiser"      ),"PREVISUALISER"    ,false);
    r.addMenuItem(BuResource.BU.getString("Mise en page..."    ),"MISEENPAGE"       ,false);
    r.addSeparator();

    r.addMenuItem(BuResource.BU.getString("Quitter..."         ),"QUITTER"          ,false,KeyEvent.VK_Q);
    return r;
  }

  public static BuMenu buildEditionMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Edition"),"MENU_EDITION");
    r.addMenuItem(BuResource.BU.getString("Dfaire"            ),"DEFAIRE"          ,false,KeyEvent.VK_Z);
    r.addMenuItem(BuResource.BU.getString("Refaire"            ),"REFAIRE"          ,false)
      .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,KeyEvent.CTRL_MASK|KeyEvent.SHIFT_MASK));
    // normalement Ctrl-Y
    r.addSeparator();

    r.addMenuItem(BuResource.BU.getString("Copier"             ),"COPIER"           ,false,KeyEvent.VK_C);
    r.addMenuItem(BuResource.BU.getString("Couper"             ),"COUPER"           ,false,KeyEvent.VK_X);
    r.addMenuItem(BuResource.BU.getString("Coller"             ),"COLLER"           ,false,KeyEvent.VK_V);
    r.addMenuItem(BuResource.BU.getString("Dupliquer"          ),"DUPLIQUER"        ,false)
      .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,KeyEvent.CTRL_MASK|KeyEvent.SHIFT_MASK));
    r.addSeparator();
    r.addMenuItem(BuResource.BU.getString("Tout slectionner"  ),"TOUTSELECTIONNER" ,false,KeyEvent.VK_A);
    r.addMenuItem(BuResource.BU.getString("Rechercher..."      ),"RECHERCHER"       ,false,KeyEvent.VK_F);
    r.addMenuItem(BuResource.BU.getString("Remplacer..."       ),"REMPLACER"        ,false,KeyEvent.VK_R);
    r.addSeparator();
    r.addMenuItem(BuResource.BU.getString("Prfrences"        ),"PREFERENCE"       ,false,KeyEvent.VK_F2);
    return r;
  }

  public static BuMenu buildMetalThemeMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Thmes pour Metal"),"THEME_METAL");

    BuMetalCustomTheme[] themes=BuMetalCustomTheme.getList();
    for(int i=0;i<themes.length;i++)
    {
      if(themes[i]==null)
	r.addSeparator();
      else
      {
	JMenuItem mi;
	mi=r.addMenuItem(BuResource.BU.getString(themes[i].getName()),"THEME_METAL("+i+")",null,true);
	mi.setIcon(themes[i].getIcon());
      }
    }

    return r;
  }

  public static BuMenu buildSlafThemeMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Thmes pour Slaf"),"THEME_SLAF");

    BuSlafCustomTheme[] themes=BuSlafCustomTheme.getList();
    for(int i=0;i<themes.length;i++)
    {
      if(themes[i]==null)
	r.addSeparator();
      else
      {
	JMenuItem mi;
	mi=r.addMenuItem(BuResource.BU.getString(themes[i].getName()),"THEME_SLAF("+i+")",null,true);
	mi.setIcon(themes[i].getIcon());
      }
    }

    return r;
  }

  public static BuMenu buildLookAndFeelMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Aspects"),"ASPECT");

    r.addSubMenu(buildMetalThemeMenu(),BuLib.classExists("javax.swing.plaf.metal.MetalLookAndFeel"));

    if(!BuLib.isSwing10())
    r.addSubMenu(buildSlafThemeMenu(),BuLib.classExists("com.memoire.slaf.SlafLookAndFeel"));

    r.addSeparator();

    if(!BuLib.isSwing10())
    r.addMenuItem("Amiga"             ,"ASPECT_AMIGA"       ,BuLib.classExists("swing.addon.plaf.threeD.ThreeDLookAndFeel"));

    if(!BuLib.isSwing10())
    r.addMenuItem("Kunststoff"        ,"ASPECT_KUNSTSTOFF"  ,BuLib.classExists("com.incors.plaf.kunststoff.KunststoffLookAndFeel"));

 // r.addMenuItem("Basic"             ,"ASPECT_BASIC"       ,BuLib.classExists("BuBasicLookAndFeel"));
    r.addMenuItem("Mac"               ,"ASPECT_MAC"         ,BuLib.classExists("com.sun.java.swing.plaf.mac.MacLookAndFeel"));
    r.addMenuItem("Metal"             ,"ASPECT_METAL"       ,BuLib.classExists("javax.swing.plaf.metal.MetalLookAndFeel"));
    r.addMenuItem("Motif"             ,"ASPECT_MOTIF"       ,BuLib.classExists("com.sun.java.swing.plaf.motif.MotifLookAndFeel"));
 // r.addMenuItem("Multi"             ,"ASPECT_MULTI"       ,BuLib.classExists("javax.swing.plaf.multi.MultiLookAndFeel"));
    r.addMenuItem("Next"              ,"ASPECT_NEXT"        ,BuLib.classExists("nextlf.plaf.NextLookAndFeel"));
    r.addMenuItem("Organic"           ,"ASPECT_ORGANIC"     ,BuLib.classExists("javax.swing.plaf.organic.OrganicLookAndFeel"));
    r.addMenuItem("Oyoaha"            ,"ASPECT_OYOAHA"      ,BuLib.classExists("com.oyoaha.swing.plaf.oyoaha.OyoahaLookAndFeel"));

    if(!BuLib.isSwing10())
    r.addMenuItem("SkinLF"            ,"ASPECT_SKINLF"      ,BuLib.classExists("com.l2fprod.gui.plaf.skin.SkinLookAndFeel"));

    if(!BuLib.isSwing10())
    r.addMenuItem("Slaf"              ,"ASPECT_SLAF"        ,BuLib.classExists("com.memoire.slaf.SlafLookAndFeel"));

    r.addMenuItem("Windows"           ,"ASPECT_WINDOWS"     ,BuLib.classExists("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"));

    return r;
  }

  public static BuMenu buildWindowMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Fentres"),"MENU_FENETRES");

    r.addSubMenu(buildLookAndFeelMenu(),true);
    r.addSubMenu(new BuMenuInternalFrames(),true);

    BuMenu sm;

    if(BuPreferences.BU.getBooleanProperty("menu.short",false))
    {
      sm=new BuMenu(BuResource.BU.getString("Organisation"),"CASCADE_ORGANISATION");
      r.addSubMenu(sm,true);
    }
    else
    {
      sm=r;
      r.addSeparator(BuResource.BU.getString("Organisation"));
    }
    sm.addMenuItem (BuResource.BU.getString("Plein cran"          ),"PLEINECRAN"          ,true);
    sm.addMenuItem (BuResource.BU.getString("Cascade"              ),"CASCADE"             ,true);
    sm.addMenuItem (BuResource.BU.getString("Mosaique"             ),"MOSAIQUE"            ,true);
    sm.addMenuItem (BuResource.BU.getString("Ranger les icnes"    ),"RANGERICONES"        ,true);
    sm.addMenuItem (BuResource.BU.getString("Ranger les palettes"  ),"RANGERPALETTES"      ,true);
    sm.addMenuItem (BuResource.BU.getString("Echanger les colonnes"),"ECHANGER_COLONNES"   ,true);

    if(BuPreferences.BU.getBooleanProperty("menu.short",false))
    {
      sm=new BuMenu(BuResource.BU.getString("Elments"),"VISIBLE_ELEMENTS");
      r.addSubMenu(sm,true);
    }
    else
    {
      sm=r;
      r.addSeparator(BuResource.BU.getString("Elments"));
    }
    sm.addCheckBox (BuResource.BU.getString("Outils spcifiques"),"VISIBLE_SPECIFICBAR",true ,true );
    sm.addCheckBox (BuResource.BU.getString("Colonne gauche"    ),"VISIBLE_LEFTCOLUMN" ,true ,true )
      .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F11,KeyEvent.ALT_MASK));
    sm.addCheckBox (BuResource.BU.getString("Colonne droite"    ),"VISIBLE_RIGHTCOLUMN",true ,true )
      .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F12,KeyEvent.ALT_MASK));
    sm.addCheckBox (BuResource.BU.getString("Barre d'tat"      ),"VISIBLE_STATUSBAR"  ,true ,true );

    return r;
  }

  public static BuMenu buildHelpMenu()
  {
    BuMenu r=new BuMenu(BuResource.BU.getString("Aide"),"MENU_AIDE");

    r.addCheckBox (BuResource.BU.getString("Assistant"          ),"ASSISTANT"      ,true ,true );

    BuMenu sm;

    if(BuPreferences.BU.getBooleanProperty("menu.short",false))
    {
      sm=new BuMenu(BuResource.BU.getString("Documentation"),"AIDE_DOCUMENTATION");
      r.addSubMenu(sm,true);
    }
    else
    {
      sm=r;
      r.addSeparator(BuResource.BU.getString("Documentation")     );
    }
    sm.addMenuItem (BuResource.BU.getString("Page principale"    ),"AIDE_INDEX"     ,true ,KeyEvent.VK_F1);
    sm.addMenuItem (BuResource.BU.getString("Pointeur"           ),"POINTEURAIDE"   ,true )
      .setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1,KeyEvent.SHIFT_MASK));
    sm.addMenuItem (BuResource.BU.getString("Index alphabtique" ),"INDEX_ALPHA"    ,true );
    sm.addMenuItem (BuResource.BU.getString("Index thmatique"   ),"INDEX_THEMA"    ,true );

    if(BuPreferences.BU.getBooleanProperty("menu.short",false))
    {
      sm=new BuMenu(BuResource.BU.getString("Informations"),"TEXTE_INFORMATIONS");
      r.addSubMenu(sm,true);
    }
    else
    {
      sm=r;
      r.addSeparator(BuResource.BU.getString("Informations")     );
    }
    sm.addMenuItem (BuResource.BU.getString("Astuces..."         ),"ASTUCE"         ,true );
    sm.addMenuItem (BuResource.BU.getString("A propos de..."     ),"APROPOSDE"      ,true );
    sm.addMenuItem (BuResource.BU.getString("Licence..."         ),"TEXTE_LICENCE"  ,true );

    if(BuPreferences.BU.getBooleanProperty("menu.short",false))
    {
      sm=new BuMenu(BuResource.BU.getString("Internet"),"WWW_INTERNET");
      r.addSubMenu(sm,true);
    }
    else
    {
      sm=r;
      r.addSeparator(BuResource.BU.getString("Internet")     );
    }
    sm.addMenuItem (BuResource.BU.getString("Site WWW"           ),"WWW_ACCUEIL"    ,true );
    // sm.addMenuItem (BuResource.BU.getString("Site FTP"),"FTP_ACCUEIL",true );
    sm.addMenuItem (BuResource.BU.getString("Mise  jour"        ),"MAJ"            ,false);
    // sm.addMenuItem (BuResource.BU.getString("Bogue..."           ),"WWW_BOGUE"      ,false);
    // sm.addMenuItem (BuResource.BU.getString("Achat..."           ),"WWW_ACHAT"      ,false);

    return r;
  }

  // Action

  private Vector actionListeners_=new Vector();

  public void addActionListener(ActionListener _l)
  { actionListeners_.addElement(_l); }

  public void removeActionListener(ActionListener _l)
  { actionListeners_.removeElement(_l); }

  public void actionPerformed(ActionEvent _evt)
  {
    // JComponent source=(JComponent)_evt.getSource();
    // System.err.println(""+getClass()+": "+source);

    for(Enumeration e=actionListeners_.elements(); e.hasMoreElements(); )
      ((ActionListener)e.nextElement()).actionPerformed(_evt);
  }
}
