/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuMenuBar.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 * A menu bar with simplified adding methods.
 */

public class BuMenuBar
       extends JMenuBar
       implements ActionListener
{
  public BuMenuBar()
  {
    super();
    setName("mbMAIN_MENUBAR");
  }

  // Anti-aliasing

  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  // Mnemonics

  public void computeMnemonics()
  {
    Hashtable t=new Hashtable();

    synchronized(this)
    {
      MenuElement[] c=getSubElements();

      for(int i=0; i<c.length; i++)
      {
	if(c[i] instanceof JMenu)
	{
	  JMenu mi=(JMenu)c[i];
	  int mn=mi.getMnemonic();
	  if(mn<=0)
	  {
	    String tx=BuLib.candidateMnemonics(mi.getText());
	    if(tx!=null)
	      for(int j=0;j<tx.length();j++)
	      {
		mn=(int)tx.charAt(j);
		if(t.get(new Integer(mn))==null)
		{
		  t.put(new Integer(mn),mi);
		  mi.setMnemonic(mn);
		  break;
		}
	      }
	  }
	  if(mi instanceof BuMenu)
	    ((BuMenu)mi).computeMnemonics();
	}
      }
    }
  }

  public void insertMenu(JMenu _menu)
  {
    SwingUtilities.updateComponentTreeUI(_menu);

    synchronized (this)
    {
      Component[] c=getComponents();
      for(int i=0;i<c.length;i++)
	remove(c[i]);
      add(_menu);
      for(int i=0;i<c.length;i++)
	add(c[i]);
    }
  }

  public void addMenu(JMenu _menu)
  {
    SwingUtilities.updateComponentTreeUI(_menu);

    synchronized (this)
    {
      int n=getSubElements().length;
      if(n>=2)
      {
	n-=2;
	Component fm=getComponent(n  ); // Fenetres
	Component hm=getComponent(n+1); // Aide

	remove(hm);
	remove(fm);
	add(_menu);
	add(fm);
	add(hm);

	/*
	add(_menu,n  );
	add(fm   ,n+1);
	add(hm   ,n+2);
	*/
      }
      else add(_menu);
    }

    _menu.setIcon(null);
    _menu.addActionListener(this);
  }

  public void addMenus(JMenu[] _menus)
  {
    if((_menus==null)||(_menus.length==0)) return;

    for(int i=0;i<_menus.length;i++)
      addMenu(_menus[i]);

    revalidate();

    if(isShowing())
      for(int i=0;i<_menus.length;i++)
	_menus[i].repaint(0);
  }

  public void removeMenu(JMenu _menu)
  {
    synchronized(this)
    {
      MenuElement[] c=getSubElements();
      for(int i=0; i<c.length; i++)
	if(c[i]==_menu)
	  remove(_menu);
    }
  }

  public void removeMenus(JMenu[] _menus)
  {
    if((_menus==null)||(_menus.length==0)) return;

    for(int i=0;i<_menus.length;i++)
      removeMenu(_menus[i]);

    revalidate();
  }

  public static JMenu getMenu(MenuElement _menu, String _name)
  {
    JMenu r=null;

    synchronized(_menu)
    {
      MenuElement[] c=_menu.getSubElements();
      for(int i=0; i<c.length; i++)
      {
	if(c[i] instanceof BuMenu)
	{
	  JMenu m=(JMenu)c[i];
	  // System.err.println(m.getName());

	  if(m.getName().equals("mn"+_name))
	    { r=m; break; }
	}

	r=getMenu(c[i],_name);
	if(r!=null) break;
      }
    }

    return r;
  }

  public JMenu getMenu(String _name)
  {
    return getMenu(this,_name);
  }

  public static JMenuItem getMenuItem(MenuElement _menu, String _name)
  {
    JMenuItem r=null;

    synchronized(_menu)
    {
      MenuElement[] c=_menu.getSubElements();

      for(int i=0; i<c.length; i++)
      {
	if(c[i] instanceof JMenuItem)
	{
	  JMenuItem m=(JMenuItem)c[i];
	  // System.err.println(m.getName());

	  if(m.getName().equals("mi"+_name))
	    { r=m; break; }
	}

	r=getMenuItem(c[i],_name);
	if(r!=null) break;
      }
    }

    return r;
  }

  public JMenuItem getMenuItem(String _name)
  {
    return getMenuItem(this,_name);
  }

  /*
  public void removeMenu(String _name)
  {
    MenuElement[] c=getSubElements();
    for(int i=0; i<c.length; i++)
      if(c[i] instanceof JMenu)
      {
	JMenu m=(JMenu)c[i];
	if(m.getName().equals("mn"+_name))
	  remove(m);
      }
    doLayout();
    validate();
  }
  */

  public void addRecentFile(String _path, String _icon)
  {
    BuMenu m=(BuMenu)getMenu("REOUVRIR");
    if(m instanceof BuMenuRecentFiles)
      ((BuMenuRecentFiles)m).addRecentFile(_path,_icon);
  }

  public static BuMenuBar buildBasicMenuBar()
  {
    BuMenuBar r=new BuMenuBar();

    BuMenu fm=BuMenu.buildFileMenu    ();
    BuMenu em=BuMenu.buildEditionMenu ();
    BuMenu wm=BuMenu.buildWindowMenu  ();
    BuMenu hm=BuMenu.buildHelpMenu    ();

    fm.setIcon(null);
    em.setIcon(null);
    wm.setIcon(null);
    hm.setIcon(null);

    fm.addActionListener(r);
    em.addActionListener(r);
    wm.addActionListener(r);
    hm.addActionListener(r);

    r.add(fm);
    r.add(em);
    r.add(wm);
    try
    {
      r.setHelpMenu(hm);
    }
    catch(Error ex)
    {
      // r.add(Box.createHorizontalGlue());
      r.add(hm); 
    }

    return r;
  }

  // Action

  private Vector actionListeners_=new Vector();

  public void addActionListener(ActionListener _l)
  { actionListeners_.addElement(_l); }

  public void removeActionListener(ActionListener _l)
  { actionListeners_.removeElement(_l); }

  public void actionPerformed(ActionEvent _evt)
  {
    // JComponent source=(JComponent)_evt.getSource();
    // System.err.println(""+getClass()+": "+source);

    for(Enumeration e=actionListeners_.elements(); e.hasMoreElements(); )
      ((ActionListener)e.nextElement()).actionPerformed(_evt);
  }
}
