/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuMenuRecentFiles.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

public class BuMenuRecentFiles
       extends BuMenu
       implements PropertyChangeListener
{
  private int     number_;
  private boolean valid_;

  public BuMenuRecentFiles()
  {
    this(null,null);
  }

  public BuMenuRecentFiles(FuPreferences _preferences, BuResource _resource)
  {
    super(BuResource.BU.getString("Rouvrir"),"REOUVRIR");
    preferences_=_preferences;
    resource_   =_resource;
    number_     =0;
    valid_      =false;
    if(preferences_!=null) preferences_.addPropertyChangeListener(this);
  }

  private FuPreferences preferences_;

  public FuPreferences getPreferences()
    { return preferences_; }

  public void setPreferences(FuPreferences _preferences)
  {
    if(preferences_!=null) preferences_.removePropertyChangeListener(this);
    preferences_=_preferences;
    if(preferences_!=null) preferences_.addPropertyChangeListener(this);
    valid_=false;
  }

  private BuResource resource_;

  public BuResource getResource()
    { return resource_; }

  public void setResource(BuResource _resource)
  {
    resource_=_resource;
    valid_=false;
  }

  public void addRecentFile(String _path, String _icon)
  {
    if(preferences_==null) return;

    for(int i=8; i>=1; i--)
    {
      String path=preferences_.getStringProperty("file.recent."+i+".path");
      String icon=preferences_.getStringProperty("file.recent."+i+".icon");
      preferences_.putStringProperty("file.recent."+(i+1)+".path",path);
      preferences_.putStringProperty("file.recent."+(i+1)+".icon",icon);
    }

    preferences_.putStringProperty("file.recent.1.path",_path);
    preferences_.putStringProperty("file.recent.1.icon",_icon);

    //valid_=false;
  }

  public void propertyChange(PropertyChangeEvent _evt)
  {
    valid_=false;
  }

  public final void setSelected(boolean _state)
  {
    if(_state&&!isSelected()&&!valid_) build();
    super.setSelected(_state);
  }

  protected void build()
  {
    if(isPopupMenuVisible()||valid_) return;

    valid_=true;
    removeAll();
    number_=0;
    if(preferences_==null) return;

    Hashtable table=new Hashtable(11);
    
    for(int i=1; i<=9; i++)
    {
      String path=preferences_.getStringProperty("file.recent."+i+".path");
      String icon=preferences_.getStringProperty("file.recent."+i+".icon");

      if(  !"".equals(path)
         &&(new File(BuLib.expandedPath(path)).canRead())
	 &&(table.get(path)==null))
      {
	String t=BuLib.reducedPath(path);
	if(t.length()>20)
	{
	  String FS=System.getProperty("file.separator");

	  int j0=t.indexOf(FS);
	  if(j0>=0)
	  {
	    int j1=t.indexOf(FS,j0+1);
	    int j2=t.lastIndexOf(FS);
	    if((j1>=0)&&(j2>=0)&&(j1<j2))
	      t=t.substring(0,j1+1)+"..."+t.substring(j2);
	  }
	}

	BuMenuItem mi=addMenuItem
	  (""+number_+" "+t,
	   "REOUVRIR("+BuLib.expandedPath(path)+")",
	   true);
	if(resource_!=null) mi.setIcon(resource_.getIcon(icon));
	number_++;
	table.put(path,icon);
      }
    }

    computeMnemonics();
  }

  public void paintComponent(Graphics _g)
  {
    boolean r=isEnabled();
    if(r!=super.isEnabled()) setEnabled(r);
    super.paintComponent(_g);
  }

  public boolean isEnabled()
  {
    if(!valid_) build();
    return (preferences_!=null)&&(number_>0);
  }
}
