/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuMenuTools.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

public class BuMenuTools
       extends BuMenu
       implements PropertyChangeListener
{
  private int     number_;
  private boolean valid_;

  public BuMenuTools()
  {
    this(null,null);
  }

  public BuMenuTools(FuPreferences _preferences, BuResource _resource)
  {
    super(BuResource.BU.getString("Outils"),"MENU_OUTILS");
    preferences_=_preferences;
    resource_   =_resource;
    number_     =0;
    valid_      =false;
    if(preferences_!=null) preferences_.addPropertyChangeListener(this);
  }

  private FuPreferences preferences_;

  public FuPreferences getPreferences()
    { return preferences_; }

  public void setPreferences(FuPreferences _preferences)
  {
    if(preferences_!=null) preferences_.removePropertyChangeListener(this);
    preferences_=_preferences;
    if(preferences_!=null) preferences_.addPropertyChangeListener(this);
    valid_=false;
  }

  private BuResource resource_;

  public BuResource getResource()
    { return resource_; }

  public void setResource(BuResource _resource)
  {
    resource_=_resource;
    valid_=false;
  }

  public void propertyChange(PropertyChangeEvent _evt)
  {
    valid_=false;
  }

  public final void setSelected(boolean _state)
  {
    if(_state&&!isSelected()&&!valid_) build();
    super.setSelected(_state);
  }

  protected void build()
  {
    if(isPopupMenuVisible()||valid_) return;

    valid_=true;
    removeAll();
    number_=0;
    if(preferences_==null) return;

    Vector v=new Vector();

    int i=1;
    while(true)
    {
      String name=preferences_.getStringProperty("tool."+i+".name");
      String exec=preferences_.getStringProperty("tool."+i+".exec");
      String icon=preferences_.getStringProperty("tool."+i+".icon");

      if("".equals(exec)) break;

      if("-".equals(exec))
      {
	addSeparator(name);
      }
      else
      {
	if("".equals(name)) name=exec;

        BuMenuItem mi=addMenuItem
	  (name,
	   "OUTIL("+exec+")",
	   true);
	if(resource_!=null) mi.setIcon(resource_.getMenuIcon(icon));
	/*
	  if(number_<10)
	  mi.setAccelerator
	  (KeyStroke.getKeyStroke
	  (KeyEvent.VK_A+number_,KeyEvent.CTRL_MASK|KeyEvent.ALT_MASK));
	*/
	v.addElement(mi);

	number_++;
      }

      i++;
    }

    /*
    addSeparator();
    addMenuItem("Ajouter","AJOUTER_OUTIL");
    */

    computeMnemonics();

    Enumeration e=v.elements();
    while(e.hasMoreElements())
    {
      BuMenuItem mi=(BuMenuItem)e.nextElement();
      int k=mi.getMnemonic();
      mi.setAccelerator
	  (KeyStroke.getKeyStroke
	   (k,KeyEvent.CTRL_MASK|KeyEvent.ALT_MASK));
    }
  }

  public void paintComponent(Graphics _g)
  {
    boolean r=isEnabled();
    if(r!=super.isEnabled()) setEnabled(r);
    super.paintComponent(_g);
  }

  public boolean isEnabled()
  {
    if(!valid_) build();
    return (preferences_!=null)&&(number_>0);
  }
}
