/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuMetalCustomTheme.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import javax.swing.plaf.*;
import javax.swing.plaf.metal.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;

/**
 * A simple customizable theme for Metal.
 */

public class BuMetalCustomTheme
       extends DefaultMetalTheme
       implements Icon
{
  private boolean defaut_;
  private String  name_;
  private int     size_;
  // private Color   primary_;
  // private Color   secondary_;

  public BuMetalCustomTheme()
  {
    defaut_   =true;
    name_     ="Traditionnel";
    size_     =0;

    controlFont_=super.getControlTextFont();
    systemFont_ =super.getSystemTextFont();
    userFont_   =super.getUserTextFont();
    smallFont_  =super.getSubTextFont();

    primary1_=super.getPrimary1();
    primary2_=super.getPrimary2();
    primary3_=super.getPrimary3();

    secondary1_=super.getSecondary1();
    secondary2_=super.getSecondary2();
    secondary3_=super.getSecondary3();
  }

  public BuMetalCustomTheme(String _name, int _size, Color _primary, Color _secondary)
  {
    this(_name,true,_size,_primary,_secondary);
  }

  public BuMetalCustomTheme(String _name, boolean _bold, int _size, Color _primary, Color _secondary)
  {
    defaut_   =false;
    name_     =_name;
    size_     =_size;
    // primary_  =_primary;
    // secondary_=_secondary;

    controlFont_=new FontUIResource
      ("SansSerif", (_bold ? Font.BOLD : Font.PLAIN), size_);
    systemFont_ =new FontUIResource("SansSerif", Font.PLAIN,size_);
    userFont_   =new FontUIResource("Monospaced",Font.PLAIN,size_);
    smallFont_  =new FontUIResource("SansSerif", Font.PLAIN,7+(size_/4));

    primary1_  =new ColorUIResource(BuLib.mixColors(_primary  ,Color.black));
    primary2_  =new ColorUIResource(_primary                               );
    primary3_  =new ColorUIResource(BuLib.mixColors(_primary  ,Color.white));

    secondary1_=new ColorUIResource(BuLib.mixColors(_secondary,Color.black));
    secondary2_=new ColorUIResource(_secondary                             );
    secondary3_=new ColorUIResource(BuLib.mixColors(_secondary,Color.white));
  }

  public BuMetalCustomTheme(String _name, int _size)
  {
    defaut_   =false;
    name_     =_name;
    size_     =_size;
    // primary_  =_primary;
    // secondary_=_secondary;

    controlFont_=new FontUIResource("SansSerif", Font.PLAIN, size_);
    systemFont_ =new FontUIResource("SansSerif", Font.PLAIN,size_);
    userFont_   =new FontUIResource("Monospaced",Font.PLAIN,size_);
    smallFont_  =new FontUIResource("SansSerif", Font.PLAIN,7+(size_/4));

    primary1_=super.getPrimary1();
    primary2_=super.getPrimary2();
    primary3_=super.getPrimary3();

    secondary1_=super.getSecondary1();
    secondary2_=super.getSecondary2();
    secondary3_=super.getSecondary3();
  }

  public String getName() { return name_; }
  public Icon   getIcon() { return this; }

  private FontUIResource controlFont_;
  private FontUIResource systemFont_;
  private FontUIResource userFont_;
  private FontUIResource smallFont_;

  private ColorUIResource primary1_;
  private ColorUIResource primary2_;
  private ColorUIResource primary3_;

  private ColorUIResource secondary1_;
  private ColorUIResource secondary2_;
  private ColorUIResource secondary3_;
  
  protected ColorUIResource getPrimary1() { return primary1_; }  
  protected ColorUIResource getPrimary2() { return primary2_; } 
  protected ColorUIResource getPrimary3() { return primary3_; } 

  protected ColorUIResource getSecondary1() { return secondary1_; }  
  protected ColorUIResource getSecondary2() { return secondary2_; } 
  protected ColorUIResource getSecondary3() { return secondary3_; } 

  public FontUIResource getControlTextFont() { return controlFont_; }
  public FontUIResource getSystemTextFont()  { return systemFont_;  }
  public FontUIResource getUserTextFont()    { return userFont_;    }
  public FontUIResource getMenuTextFont()    { return controlFont_; }
  public FontUIResource getWindowTitleFont() { return controlFont_; }
  public FontUIResource getSubTextFont()     { return smallFont_;   }

  public void addCustomEntriesToTable(UIDefaults _table)
  {
    super.addCustomEntriesToTable(_table);

    if(!defaut_)
    {
      final int internalFrameIconSize=Math.max(12,size_-2);
      _table.put("InternalFrameTitlePane.closeIcon",       MetalIconFactory.getInternalFrameCloseIcon(internalFrameIconSize));
      _table.put("InternalFrameTitlePane.maximizeIcon",    MetalIconFactory.getInternalFrameMaximizeIcon(internalFrameIconSize));
      _table.put("InternalFrameTitlePane.iconizeIcon",     MetalIconFactory.getInternalFrameMinimizeIcon(internalFrameIconSize));
      _table.put("InternalFrameTitlePane.altMaximizeIcon", MetalIconFactory.getInternalFrameAltMaximizeIcon(internalFrameIconSize));
      _table.put("ScrollBar.width",new Integer(5+size_));
    }
  }

  public int getIconWidth()  { return 44; }
  public int getIconHeight() { return 18; }

  public void paintIcon(Component _c, Graphics _g, int _x, int _y)
  {
    Color cold=_g.getColor();
    Font  fold=_g.getFont();

    FontMetrics fm=_g.getFontMetrics(controlFont_);
    int ws=fm.stringWidth("ab12");

    int w=getIconWidth();
    int h=getIconHeight();

    _g.setColor(getSecondary2());
    _g.fill3DRect(_x,_y,w,h,true);

    _g.setColor(getPrimary2());
    _g.fill3DRect(_x+2,_y+2,w/2,h-3,true);

    _g.setColor(getSecondary3());
    _g.fill3DRect(_x+w/2+2,_y+2,w/2-3,h-3,true);

    _g.setColor(getBlack());
    _g.setFont(controlFont_);
    
    _g.drawString("ab12",_x+(w-ws)/2+1,_y+h-4);

    _g.setColor(cold);
    _g.setFont(fold);
  }

  public static BuMetalCustomTheme[] getList()
  {
    return new BuMetalCustomTheme[]
    {
      new BuMetalCustomTheme(),
      new BuMetalCustomTheme
	(BuResource.BU.getString("Personnel"),
	 BuPreferences.BU.getBooleanProperty("metal.bold"   ,false),
	 BuPreferences.BU.getIntegerProperty("metal.size"   ,10),
	 BuPreferences.BU.getColorProperty("metal.primary"  ,new Color(128,128,192)),  //128,160,224
	 BuPreferences.BU.getColorProperty("metal.secondary",new Color(160,160,160))), //224,192,160
      null,
      new BuMetalCustomTheme("Allg, petit"   ,10),
      new BuMetalCustomTheme("Allg, moyen"   ,12),
      new BuMetalCustomTheme("Allg, grand"   ,14),
      null,
      new BuMetalCustomTheme("Brique, petit"   ,10,new Color(192,128,128),new Color(176,160,160)),
      new BuMetalCustomTheme("Brique, moyen"   ,12,new Color(192,128,128),new Color(176,160,160)),
      new BuMetalCustomTheme("Brique, grand"   ,14,new Color(192,128,128),new Color(176,160,160)),
      null,
      new BuMetalCustomTheme("Ocre, petit"     ,10,new Color(192,160,128),new Color(176,160,144)),
      new BuMetalCustomTheme("Ocre, moyen"     ,12,new Color(192,160,128),new Color(176,160,144)),
      new BuMetalCustomTheme("Ocre, grand"     ,14,new Color(192,160,128),new Color(176,160,144)),
      null,
      new BuMetalCustomTheme("Turquoise, petit",10,new Color(96,160,160),new Color(128,160,160)),
      new BuMetalCustomTheme("Turquoise, moyen",12,new Color(96,160,160),new Color(128,160,160)),
      new BuMetalCustomTheme("Turquoise, grand",14,new Color(96,160,160),new Color(128,160,160)),
    };
  }
}
