/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuMonitorPreferencesPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


/**
 * @creation     2001-01-09
 * @modification 2001-01-09
 * @statut       instable
 */

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A panel where the user can choose which monitors will
 * appear in the status bar.
 */

/**
 * monitor.memory
 * ...
 */

public class BuMonitorPreferencesPanel
       extends BuAbstractPreferencesPanel
       implements ActionListener
{
  private final static int NBF=4;

  private final static String[] INTITULES=new String[]
  {
    BuResource.BU.getString("Mmoire"),
    BuResource.BU.getString("Processeur"),
    BuResource.BU.getString("Horloge"),
    BuResource.BU.getString("Temps"),
  };

  private final static String[] NOMS=new String[]
  {
    "MEMORY","CPU","CLOCK","TIME"
  };

  BuCommonImplementation appli_;
  BuPreferences          options_;
  BuCheckBox[]           cbMoniteurs;
  
  public String getTitle()
    { return BuResource.BU.getString("Moniteur"); }

  // Constructeur

  public BuMonitorPreferencesPanel(BuCommonImplementation _appli)
  {
    super();
    appli_=_appli;
    init();
  }

  private void init()
  {
    options_=BuPreferences.BU;

    BuGridLayout lo=new BuGridLayout();
    lo.setColumns(2);
    lo.setVgap(5);
    lo.setHgap(5);

    BuPanel p=new BuPanel();
    p.setBorder
      (new CompoundBorder
       (new TitledBorder(getTitle()),
	new EmptyBorder(5,5,5,5)));
    p.setLayout(lo);

    cbMoniteurs=new BuCheckBox[NBF];
    for(int i=0;i<NBF;i++)
    {
      cbMoniteurs[i]=new BuCheckBox(INTITULES[i]);
      cbMoniteurs[i].setName("cb"+NOMS[i]+"_VISIBLE");
      cbMoniteurs[i].setActionCommand(NOMS[i]);
      p.add(cbMoniteurs[i]);
      cbMoniteurs[i].addActionListener(this);
    }

    p.add(new BuPanel());
    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    add(p,BuBorderLayout.CENTER);

    updateComponents();
  }

  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    /*
    JComponent source=(JComponent)_evt.getSource();
    System.err.println("SOURCE="+source.getName());
    System.err.println("ACTION="+_evt.getActionCommand());
    */
  }
  
  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  /*
  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
    options_.applyOn(appli_);
  }
  */

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
    for(int i=0;i<NBF;i++)
      options_.putBooleanProperty
	  ("monitor."+NOMS[i].toLowerCase(),
	   cbMoniteurs[i].isSelected());
  }

  private void updateComponents()
  {
    for(int i=0;i<NBF;i++)
      cbMoniteurs[i].setSelected
	(options_.getBooleanProperty
	 ("monitor."+NOMS[i].toLowerCase(),(i==0)));
  }
}
