/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuNumericValueValidator.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.util.Date;

/**
 * A validator for numerical input values with a min/max control.
 * Provides standard ones for ints, doubles, positive ints and
 * positive doubles.
 */

public class BuNumericValueValidator
  extends BuValueValidator
{
  public BuNumericValueValidator()
  {
    this(false,-Double.MAX_VALUE,Double.MAX_VALUE);
  }

  public BuNumericValueValidator
    (boolean _integer_only, double _min, double _max)
  {
    setIntegerOnly(_integer_only);
    setMin(_min);
    setMax(_max);
  }
  
  private boolean integer_only_;
  public boolean isIntegerOnly()
    { return integer_only_; }
  public void    setIntegerOnly(boolean _integer_only)
    { integer_only_=_integer_only; }

  private double min_;
  public double getMin() { return min_; }
  public void setMin(double _min) { min_=_min; }

  private double max_;
  public double getMax() { return max_; }
  public void setMax(double _max) { max_=_max; }

  public boolean isValueValid(Object _value)
  {
    boolean r=false;

    if(   (integer_only_&&(_value instanceof Integer))
        &&(((Integer)_value).intValue()>=getMin())
        &&(((Integer)_value).intValue()<=getMax())) r=true;

    if(   (!integer_only_&&(_value instanceof Double))
	&&(((Double)_value).doubleValue()>=getMin())
	&&(((Double)_value).doubleValue()<=getMax())) r=true;

    return r;
  }

  // Instances

  public final static BuNumericValueValidator INTEGER
    =new BuNumericValueValidator(true,-Integer.MAX_VALUE,Integer.MAX_VALUE);

  public final static BuNumericValueValidator DOUBLE
    =new BuNumericValueValidator();

  public final static BuNumericValueValidator POSITIVE_INTEGER
    =new BuNumericValueValidator(true,0,Integer.MAX_VALUE);

  public final static BuNumericValueValidator POSITIVE_DOUBLE
    =new BuNumericValueValidator(false,0.,Double.MAX_VALUE);
}


