/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuOverlayLayout.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.* ;
import java.util.* ;

/**
 * An overlay layout where all the components have the same size.
 */

public class BuOverlayLayout
       implements LayoutManager2
{
  // Vector components=new Vector(0,1);

  public BuOverlayLayout()
  {
  }

  public void addLayoutComponent(Component _comp, Object _cstr)
  {
    // components.addElement(_comp);
  }

  public void addLayoutComponent(String _name, Component _comp)
  {
    // components.addElement(_comp);
  }

  public void removeLayoutComponent(Component _comp)
  {
    // components.removeElement(_comp);
  }

  public Dimension minimumLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      // int nbc=components.size();
      int nbc=_parent.getComponentCount();
      int w  =0;
      int h  =0;

      for(int i=0;i<nbc;i++)
      {
	// Dimension d=((Component)components.elementAt(i)).getMinimumSize();
	Dimension d=_parent.getComponent(i).getMinimumSize();
	w=Math.max(w,d.width);
	h=Math.max(h,d.height);
      }

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
			   h+insets.top+insets.bottom);
    }
  }

  public Dimension preferredLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      // int nbc=components.size();
      int nbc=_parent.getComponentCount();
      int w  =0;
      int h  =0;

      for(int i=0;i<nbc;i++)
      {
	// Dimension d=((Component)components.elementAt(i)).getPreferredSize();
	Dimension d=_parent.getComponent(i).getPreferredSize();
	w=Math.max(w,d.width);
	h=Math.max(h,d.height);
      }

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
			   h+insets.top+insets.bottom);
    }
  }

  public Dimension maximumLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      // int nbc=components.size();
      int nbc=_parent.getComponentCount();
      int w  =0;
      int h  =0;

      for(int i=0;i<nbc;i++)
      {
	// Dimension d=((Component)components.elementAt(i)).getMaximumSize();
	Dimension d=_parent.getComponent(i).getMaximumSize();
	w=Math.max(w,d.width);
	h=Math.max(h,d.height);
      }

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
			   h+insets.top+insets.bottom);
    }
  }

  public void invalidateLayout(Container _parent)
    { }

  public float getLayoutAlignmentX(Container _parent)
    { return 0.5f; }

  public float getLayoutAlignmentY(Container _parent)
    { return 0.5f; }

  public void layoutContainer(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      Insets insets = _parent.getInsets();
      Rectangle bounds = new Rectangle
	(insets.left,
	 insets.top,
	 _parent.getSize().width  - (insets.left + insets.right),
	 _parent.getSize().height - (insets.top + insets.bottom));
      
      // int nbc=components.size();
      int nbc=_parent.getComponentCount();

      for(int i=0;i<nbc;i++)
      {
	// Component c=(Component)components.elementAt(i);
	Component c=_parent.getComponent(i);

	Rectangle actual=new Rectangle
	  (bounds.x,
	   bounds.y,
	   bounds.width,
	   bounds.height);
	c.setBounds(actual.x,actual.y,actual.width,actual.height);
      }
    }
  }
}
