/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuPicture.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A component to display images.
 * Enabled scaling.
 */

public class BuPicture
       extends JComponent
{
  final public static int SCALE =0;
  final public static int ZOOM  =1;
  final public static int LEFT  =2;
  final public static int MIDDLE=3;
  final public static int RIGHT =4;

  public BuPicture()
  {
    mode_ =MIDDLE;
    icon_ =null;
  }

  public BuPicture(BuIcon _icon)
  {
    this();
    setIcon(_icon);
  }

  public BuPicture(Image _image)
  {
    this();
    setImage(_image);
  }

  public void paintComponent(Graphics _g)
  {
    super.paintComponent(_g);

    if(isOpaque())
    {
      Rectangle clip=_g.getClipBounds();
      if(clip!=null)
      {
	_g.setColor(getBackground());
	_g.fillRect(clip.x,clip.y,clip.width,clip.height);
      }
    }

    if(icon_!=null)
    {
      Image     image =icon_.getImage();
      Dimension size  =getSize();
      Insets    insets=getInsets();

      int xp=insets.left;
      int yp=insets.top;
      int wp=size.width-insets.left-insets.right;
      int hp=size.height-insets.top-insets.bottom;

      switch(getMode())
      {
	case SCALE:
	{
	  _g.drawImage(image,xp,yp,wp,hp,this);
	  break;
	}
        case ZOOM:
	{
	  int    wi=icon_.getIconWidth();
	  int    hi=icon_.getIconHeight();
	  double wr=(double)wp/(double)wi;
	  double hr=(double)hp/(double)hi;
	  double r =Math.min(wr,hr);
	  int    wf=(int)(r*wi);
	  int    hf=(int)(r*hi);
	  _g.drawImage(image,xp,yp,wf,hf,this);
	  break;
	}
        case LEFT:
	{
	  int    wi=icon_.getIconWidth();
	  int    hi=icon_.getIconHeight();
	  _g.drawImage(image,xp,yp,this);
	  break;
	}
        case MIDDLE:
	{
	  int    wi=icon_.getIconWidth();
	  int    hi=icon_.getIconHeight();
	  _g.drawImage(image,xp+(wp-wi)/2,yp+(hp-hi)/2,this);
	  break;
	}
        case RIGHT:
	{
	  int    wi=icon_.getIconWidth();
	  int    hi=icon_.getIconHeight();
	  _g.drawImage(image,xp+wp-wi,yp+hp-hi,this);
	  break;
	}
      }
    }
  }

  public void computePreferredSize()
  {
    Dimension size;
    if(icon_!=null)
      size=new Dimension(icon_.getIconWidth(), icon_.getIconHeight());
    else
      size=new Dimension(0,0);

    Insets insets=getInsets();
    size.width +=insets.left+insets.right ;
    size.height+=insets.top +insets.bottom;
    setPreferredSize(size);
  }

  public void setBorder(Border _border)
  {
    super.setBorder(_border);
    computePreferredSize();
  }

  // Properties

  private int mode_;
  public int getMode() { return mode_; }
  public void setMode(int _v)
  {
    if(mode_!=_v)
    {
      int vp=mode_;
      mode_=_v%(RIGHT+1);
      firePropertyChange("mode",vp,_v);
      repaint();
    }
  }

  private BuIcon icon_;
  public BuIcon getIcon() { return icon_; }
  public void setIcon(BuIcon _v)
  {
    if(icon_!=_v)
    {
      BuIcon vp=icon_;
      icon_=_v;
      computePreferredSize();
      // setMinimumSize(size);
      // setSize(size);

      firePropertyChange("icon",vp,_v);
      revalidate();
      repaint();
    }
  }

  public Image getImage()
  {
    return (icon_!=null ? icon_.getImage() : null);
  }

  public void setImage(Image _v)
  {
    setIcon(_v!=null ? new BuIcon(_v) : null);
  }

  public static void main(String[] _args)
  {
    BuPicture p=new BuPicture
      (BuResource.BU.getIcon("banner-alma.gif"));
    p.setMode(SCALE);
    p.setBackground(Color.red);
    p.setBorder(new EmptyBorder(50,0,0,20));

    JFrame frame=new JFrame("Test BuPicture");
    frame.getContentPane().add(p);
    frame.addWindowListener( new WindowAdapter() {
      public void windowClosing( WindowEvent e ) {
	System.exit(0);
      }
    });

    frame.setSize(200,200);
    frame.setVisible(true);
  }
}
