/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuPopupMenu.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.fu.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;

/**
 * A popup menu which follows action events and has
 * simplified adding methods.
 */

public class BuPopupMenu
       extends JPopupMenu
    implements ActionListener, ChangeListener,
	       MouseListener, MouseMotionListener
{
  private final static Vector menus_=new Vector();

  protected BuButton b1;
  protected BuToggleButton b2;
  protected BuPanel p;

  private boolean sticky_=false;

  private final Point location_=new Point(0,0);

  public BuPopupMenu()
  {
    this(false);
  }

  public BuPopupMenu(boolean _stickable)
  {
    this(_stickable,null);
  }

  public BuPopupMenu(boolean _stickable, String _title)
  {
    super();

    // TMP
    // setLightWeightPopupEnabled(false);

    if(_stickable)
    {
      BuLabel l=new BuLabel(_title);
      l.setOpaque(false);
      l.setBorder(new EmptyBorder(2,2,2,2));
      l.setForeground
	  (BuLib.getColor
	   (UIManager.getColor("MenuItem.background").brighter(),true));
      l.setFont(BuLib.deriveFont("MenuItem",Font.PLAIN,-2));
      l.setPreferredSize(l.getPreferredSize());

      b1=new BuButton();
      b1.setActionCommand("CLOSE_MENU");
      b1.setToolTipText(BuResource.BU.getString("Fermer"));
      Icon i1=UIManager.getIcon("InternalFrameTitlePane.closeIcon");
      if(i1!=null)
      {
        b1.setIcon(i1);
	b1.setBorder(new EmptyBorder(0,0,0,0));
	b1.setRolloverEnabled(false);
	b1.putClientProperty("SLAF_NO_TEXTURE",Boolean.TRUE);
      }
      else
      {
        b1.setPreferredSize(new Dimension(12,12));
      }
      b1.setRequestFocusEnabled(false);
      b1.addActionListener(this);

      b2=new BuToggleButton();
      b2.setActionCommand("STICK_MENU");
      b2.setToolTipText(BuResource.BU.getString("Epingler"));
      Icon i2=UIManager.getIcon("RadioButton.unselectedIcon");
      if(i2!=null)
      {
        b2.setIcon(i2);
	b2.setBorder(new EmptyBorder(0,0,0,0));
	b2.setRolloverEnabled(false);
	b2.putClientProperty("SLAF_NO_TEXTURE",Boolean.TRUE);
	b2.setSelectedIcon(UIManager.getIcon
			   ("RadioButton.selectedIcon"));
      }
      else
      {
        b2.setPreferredSize(new Dimension(12,12));
      }
      b2.setRequestFocusEnabled(false);
      b2.addActionListener(this);

      p=new BuPanel();
      p.setOpaque(new JMenuItem().isOpaque());
      if(p.isOpaque())
      {
	Color bg=BuLib.getColor(p.getBackground().darker(),true);
	if(bg.equals(p.getForeground()))
	  bg=UIManager.getColor("Label.foreground");
	p.setBackground(bg);
      }

      p.setLayout(new BuBorderLayout()); //FlowLayout(FlowLayout.RIGHT));
      p.setBorder(new EmptyBorder(0,0,0,0));
      p.setToolTipText(BuResource.BU.getString("Dplacer"));
      p.setCursor(new Cursor(Cursor.MOVE_CURSOR));
      p.addMouseMotionListener(this);
      p.add(b2,BuBorderLayout.WEST  );
      p.add(l ,BuBorderLayout.CENTER);
      p.add(b1,BuBorderLayout.EAST  );
      this.add(p);
      this.addSeparator();

      setSticky(false);

      /*
      addMouseMotionListener
        (new MouseMotionListener()
	  {
	    public void mouseMoved  (MouseEvent _evt) { }
	    public void mouseDragged(MouseEvent _evt) { }
	  }
        );
      */
    }

    addMouseListener(this);
    MenuSelectionManager.defaultManager().addChangeListener(this);
  }

  /*
  protected void processMouseMotionEvent(MouseEvent _evt)
  {
    System.err.println(_evt);
    repaint();
    super.processMouseMotionEvent(_evt);
  }
  */

  public void mouseClicked (MouseEvent _evt) { }
  public void mouseReleased(MouseEvent _evt) { }
  public void mousePressed (MouseEvent _evt) { }

  public void mouseEntered (MouseEvent _evt)
  { 
    MenuElement[] me=new MenuElement[1];
    me[0]=this;//(MenuElement)this;
    //me[1]=(MenuElement)source;
    MenuSelectionManager.defaultManager().setSelectedPath(me);
    repaint(200);
  }

  public void mouseExited  (MouseEvent _evt)
  {
    //MenuSelectionManager.defaultManager().clearSelectedPath();
  }

  public void stateChanged(ChangeEvent _evt)
  {
    if(sticky_)
    {
      MenuSelectionManager sm=MenuSelectionManager.defaultManager();

      if(_evt.getSource()==sm)
      {
	MenuElement[] me=sm.getSelectedPath();

	boolean r=false;
	Vector  c=BuLib.getAllSubComponents(this);
	for(int i=0; i<me.length;i++)
	  if(c.contains(me[i]))
	  {
	    r=true; break;
	  }

	if(r) repaintAllPopupMenus();
      }
    }
  }

  private final static void repaintAllPopupMenus()
  {
    for(int i=0; i<menus_.size(); i++)
    {
      BuPopupMenu pm=(BuPopupMenu)menus_.elementAt(i);
      pm.repaint();
    }
  }

  public void mouseMoved(MouseEvent _evt)
  {
  }

  public void mouseDragged(MouseEvent _evt)
  {
    int   x=_evt.getX();
    int   y=_evt.getY();
    Point p=new Point(x,y);
    SwingUtilities.convertPointToScreen(p,this);
    Dimension s=getSize();
    setLocation(p.x-s.width/2,p.y-5);

    if(!isSticky()) setSticky(true);
  }

  public void setLocation(int _x, int _y)
  {
    Dimension s=getSize();
    Dimension screen=getToolkit().getScreenSize();
	
    // Inside the top-level container
 
    Component invoker=getInvoker();
    if(invoker instanceof JComponent)
    {
      Container parent=((JComponent)invoker).getTopLevelAncestor();
      Point     q=parent.getLocationOnScreen();
      Dimension t=parent.getSize();
      Insets    i=parent.getInsets();
      if(_x+s.width >q.x+t.width ) _x=q.x+t.width -s.width -i.right;
      if(_y+s.height>q.y+t.height) _y=q.y+t.height-s.height-i.bottom;
      if(_x<q.x) _x=q.x+i.left;
      if(_y<q.y) _y=q.y+i.top;
    }

    // Inside the screen
 
    if(_x+s.width >screen.width ) _x=screen.width -s.width ;
    if(_y+s.height>screen.height) _y=screen.height-s.height;
    if(_x<0) _x=0;
    if(_y<0) _y=0;

    location_.x=_x;
    location_.y=_y;
    super.setLocation(_x,_y);
  }

  public void setVisible(boolean _visible)
  {
    if(_visible)
    {
      setSize(getPreferredSize());
      BuLib.setDoubleBuffered(this,false);

      if(!isVisible())
      {
	SwingUtilities.updateComponentTreeUI(this);
	setLocation(location_.x,location_.y);
      }

      if((p!=null)&&(UIManager.get("PopupMenu.alpha")!=null))
	p.setOpaque(false);

      if(b1!=null)
	b1.setOpaque(!Boolean.TRUE.equals
		     (b1.getClientProperty("SLAF_NO_TEXTURE")));
      if(b2!=null)
	b2.setOpaque(!Boolean.TRUE.equals
		     (b2.getClientProperty("SLAF_NO_TEXTURE")));
    }

    super.setVisible(_visible);
  }

  public boolean isSticky()
  {
    return sticky_;
  }

  public void setSticky(boolean _sticky)
  {
    sticky_=_sticky;

    if(sticky_)
    {
      if(b2!=null) b2.setSelected(true);
      //b2.setEnabled (false);
      //if(!b2.isOpaque()) b2.setVisible(false);
      //b1.setVisible(true);
      if(!menus_.contains(this)) menus_.addElement(this);
      setOpaque(false);

      /*
      // ESSAI DANS FRAME EXTERNE
      Point xy=getLocationOnScreen();
      Container a=getTopLevelAncestor();
      JDialog d=new JDialog((a instanceof Frame) ? (Frame)a : null);
      d.setTitle(" [Menu]");
      d.getContentPane().add(this);
      d.setModal(false);
      d.pack();
      d.setLocation(xy);
      d.show();
      */
    }
    else
    {
      if(b2!=null) b2.setSelected(false);
      //b2.setEnabled (true);
      //b2.setVisible (true);
      //b1.setVisible (false);
      if(menus_.contains(this)) menus_.removeElement(this);
    }
  }

  public void menuSelectionChanged(boolean _included)
  {
    if(!sticky_||_included)
      super.menuSelectionChanged(_included);
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
    //System.err.println("ACTION="+action);

    if("CLOSE_MENU".equals(action))
    {
      //sticky_=false;
      super.setVisible(false);
    }
    else
    if("STICK_MENU".equals(action))
    {
      setSticky(b2.isSelected());
    }
    else
    {
      Component c=getInvoker();
      if(c instanceof ActionListener)
        ((ActionListener)c).actionPerformed(_evt);

      if(sticky_)
      {
	Object source=_evt.getSource();

	if(source instanceof MenuElement)
	{
	  MenuElement[] me=new MenuElement[2];
	  me[0]=this;//(MenuElement)this;
	  me[1]=(MenuElement)source;
	  MenuSelectionManager.defaultManager().setSelectedPath(me);
	}
      }
    }
  }

  // Separator

  public BuSeparator addSeparator(String _text)
  {
    BuSeparator r=new BuSeparator(_text);
    this.add(r);
    return r;
  }

  // SubMenu

  public void addSubMenu(JMenu _m, boolean _enabled)
  {
    _m.setEnabled(_enabled);
    _m.addActionListener(this);
    this.add(_m);
  }

  // Menu Item

  public BuCheckBoxMenuItem addCheckBoxMenuItem
    (String _s, String _cmd, boolean _enabled, boolean _value)
  {
    BuIcon             icon=BuResource.BU.loadMenuCommandIcon(_cmd);
    BuCheckBoxMenuItem r   =new BuCheckBoxMenuItem();
    r.setName("cb"+_cmd);
    r.setActionCommand(_cmd);
    r.setText(_s);
    r.setIcon(icon);
    r.setHorizontalTextPosition(JButton.RIGHT);
    r.addActionListener(this);
    r.setEnabled(_enabled);
    r.setSelected(_value);
    this.add(r);
    return r;
  }

  public BuMenuItem addMenuItem(String _s, String _cmd, boolean _enabled)
  {
    BuIcon icon=BuResource.BU.loadMenuCommandIcon(_cmd);
    return addMenuItem(_s,_cmd,icon,_enabled);
  }

  public BuMenuItem addMenuItem
    (String _s, String _cmd, Icon _icon, boolean _enabled, int _key)
  {
    BuMenuItem r=addMenuItem(_s,_cmd,_icon,_enabled);
    if(_key!=0)
    {
      if((_key>=KeyEvent.VK_F1)&&(_key<=KeyEvent.VK_F12))
	r.setAccelerator(KeyStroke.getKeyStroke(_key,KeyEvent.ALT_MASK));
      else
	r.setAccelerator(KeyStroke.getKeyStroke(_key,KeyEvent.CTRL_MASK));
    }
    return r;
  }

  public BuMenuItem addMenuItem
    (String _s, String _cmd, Icon _icon, boolean _enabled)
  {
    BuMenuItem r=new BuMenuItem();
    r.setName("mi"+_cmd);
    r.setActionCommand(_cmd);
    r.setText(_s);
    r.setIcon(_icon);
    r.setHorizontalTextPosition(JButton.RIGHT);
    r.addActionListener(this);
    r.setEnabled(_enabled);
    this.add(r);
    return r;
  }
}
