/**
 * @modification $Date: 2001/12/04 18:29:49 $
 * @statut       unstable
 * @file         BuPreferences.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;

import java.util.*;
import java.awt.*;
import java.lang.reflect.*;
import java.io.*;

import javax.swing.*;
import javax.swing.plaf.metal.*;

/**
 * A class to manage user preferences for the standard GUI.
 */

public class BuPreferences
       extends FuPreferences
{
  public final static BuPreferences BU=new BuPreferences();
  
  protected BuPreferences() { }

  // Specific

  public void applyLookAndFeel()
  {
    String os=System.getProperty("os.name");

    String lnfname =getStringProperty("lookandfeel.name");
    String lnfclass=getStringProperty("lookandfeel.class");

    if(BuLib.isSwing10()&&lnfname.equals("ASPECT_KUNSTSTOFF"))
      lnfname="ASPECT_METAL";
    if(BuLib.isSwing10()&&lnfname.equals("ASPECT_SLAF"))
      lnfname="ASPECT_METAL";
    if((FuLib.jdk()<=1.1)&&lnfname.equals("ASPECT_SKINLF"))
      lnfname="ASPECT_METAL";

    if(lnfname.equals("ASPECT_DEFAUT"))
      lnfclass="";
    else
    if(lnfname.equals("ASPECT_AMIGA"))
      lnfclass="swing.addon.plaf.threeD.ThreeDLookAndFeel";
    else
    if(lnfname.equals("ASPECT_KUNSTSTOFF"))
      lnfclass="com.incors.plaf.kunststoff.KunststoffLookAndFeel";
    else
    if(lnfname.equals("ASPECT_MAC"))
    {
      System.getProperties().put("os.name","Mac OS");
      lnfclass="com.sun.java.swing.plaf.mac.MacLookAndFeel";
    }
    else
    if(lnfname.equals("ASPECT_METAL"))
    {
      lnfclass="javax.swing.plaf.metal.MetalLookAndFeel";
      try
      {
	Class  c=Class.forName(lnfclass);
	Method m=c.getMethod
	  ("setCurrentTheme",new Class[] { MetalTheme.class } );
	int i=Integer.parseInt
	  (getStringProperty("metal.theme","THEME_METAL0").substring(11));
	m.invoke(null,new Object[] { BuMetalCustomTheme.getList()[i] } );
      }
      catch(Exception ex) { }
    }
    else
    if(lnfname.equals("ASPECT_MOTIF"))
      lnfclass="com.sun.java.swing.plaf.motif.MotifLookAndFeel";
    else
    if(lnfname.equals("ASPECT_NEXT"))
      lnfclass="nextlf.plaf.NextLookAndFeel";
    else
    if(lnfname.equals("ASPECT_ORGANIC"))
      lnfclass="javax.swing.plaf.organic.OrganicLookAndFeel";
    else
    if(lnfname.equals("ASPECT_OYOAHA"))
    {
      lnfclass="com.oyoaha.swing.plaf.oyoaha.OyoahaLookAndFeel";
      try
      {
        Class  c=Class.forName(lnfclass);
	Object o=c.newInstance();
	Method m=c.getMethod("setOyoahaTheme",new Class[] { File.class } );
	m.invoke(o,new Object[]
	    { new File(getStringProperty("oyoaha.theme")) } );

	UIManager.setLookAndFeel((LookAndFeel)o);
      }
      catch(Exception ex) { ex.printStackTrace(); }
    }
    else
    if(lnfname.equals("ASPECT_SKINLF"))
    {
      lnfclass="com.l2fprod.gui.plaf.skin.SkinLookAndFeel";
      try
      {
	String t=getStringProperty("skinlf.theme");
	Class  c=Class.forName(lnfclass);
	Method m=null;

	if(t.toLowerCase().endsWith(".zip"))
	  m=c.getMethod("loadThemePack",new Class[] { String.class } );
	else
	  m=c.getMethod("loadSkin"     ,new Class[] { String.class } );

	Object s=m.invoke(null,new Object[] { t } );
	  
	m=c.getMethod("setSkin",new Class[]
		      { Class.forName("com.l2fprod.gui.plaf.skin.Skin") } );
	m.invoke(null,new Object[] { s } );
      }
      catch(Exception ex) { ex.printStackTrace(); }
    }
    else
    if(lnfname.equals("ASPECT_SLAF"))
    {
      lnfclass="com.memoire.slaf.SlafLookAndFeel";
      try
      {
	Class   c=Class.forName(lnfclass);
	Method  m=c.getMethod("setCurrentTheme",new Class[] { String.class } );
	m.invoke(null,new Object[] { getStringProperty("slaf.theme") } );
      }
      catch(Exception ex) { }
    }
    else
    if(lnfname.equals("ASPECT_WINDOWS"))
    {
      System.getProperties().put("os.name","Windows");
      lnfclass="com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
    }

    // if(lnfclass.equals("")) lnfclass="com.memoire.slaf.SlafLookAndFeel";

    // this test is important under JDK1.2
    if(  !"".equals(lnfclass)
       &&!UIManager.getLookAndFeel().getClass().getName().equals(lnfclass))
    {
      try { UIManager.setLookAndFeel(lnfclass); }
      catch (Throwable ex) { }
    }

    System.getProperties().put("os.name",os);
  }

  public void applyLanguage(String _dispo)
  {
    String l=BuPreferences.BU.getStringProperty("locale.language");
    if(_dispo.indexOf(l)>=0)
    {
      // System.err.println("LANG="+l);
      if(!l.equals("")) Locale.setDefault(new Locale(l,""));
    }
    else
    {
      System.err.println("LANG=fr ("+l+" is not available)");
      Locale.setDefault(Locale.FRENCH);
    }
  }

  public void applyOn(Object _o)
  {
    if(!(_o instanceof BuCommonInterface))
      throw new RuntimeException(""+_o+" is not a BuCommonInterface.");

    BuCommonInterface _appli=(BuCommonInterface)_o;

    BuMainPanel main=_appli.getMainPanel();
    BuDesktop desktop=main.getDesktop();


    // Window
    BuCommonInterface a=_appli.getApp();
    if(a instanceof JFrame)
    {
      final JFrame f=(JFrame)a;

      f.getRootPane().invalidate();

      Dimension d=f.getPreferredSize();
      Dimension e=f.getToolkit().getScreenSize();

      int left  =getIntegerProperty("screen.left"  ,0);
      int right =getIntegerProperty("screen.right" ,0);
      int top   =getIntegerProperty("screen.top"   ,0);
      int bottom=getIntegerProperty("screen.bottom",0);

      switch(getIntegerProperty("window.size"))
      {
      case 0:
	f.setLocation(Math.max(left,e.width/10),
		      Math.max(top,e.height/10));
	f.setSize(Math.min(e.width*4/5,e.width-left-right),
		  Math.min(e.height*4/5,e.height-top-bottom));
	break;
      case 1:
	f.setLocation(left,top);
	f.setSize(Math.min(e.width -f.getLocation().x,e.width-left-right),
		  Math.min(e.height-f.getLocation().y,e.height-top-bottom));
	break;
      case 2:
	int x=getIntegerProperty("window.x",e.width /10);
	int y=getIntegerProperty("window.y",e.height/10);
	int w=getIntegerProperty("window.w");
	int h=getIntegerProperty("window.h");

	if(x<0) x=0;
	if(y<0) y=0;
	f.setLocation(x,y);
	if((w>0)&&(h>0)) f.setSize(w,h);
	break;
      case 3:
	try
	{
	  Method  m=JFrame.class.getMethod("setState",
					   new Class[] { Integer.class } );
	  m.invoke(f,new Object[] { new Integer(1) } );
	}
	catch(Exception ex) { }
	// f.setState(Frame.ICONIFIED);
	// Frame.ICONIFIED = 1;
	break;
      }

      /*
      if(System.getProperty("java.version").startsWith("1.3"))
      {
	Runnable runnable=new Runnable()
	{
	  public void run()
	  {
	    f.doLayout();
	    f.validate();
	  }
	};

	try
	{
	  if(!SwingUtilities.isEventDispatchThread())
	    SwingUtilities.invokeAndWait(runnable);
	  else
	    runnable.run();
	}
	catch(Exception ex) { }
      }
      */

      f.getRootPane().revalidate();
    }

    // Aspect

    String lnfname =getStringProperty("lookandfeel.name");
    String lnfclass=getStringProperty("lookandfeel.class");

    if(BuLib.isSwing10()&&lnfname.equals("ASPECT_KUNSTSTOFF"))
      lnfname="ASPECT_METAL";
    if(BuLib.isSwing10()&&lnfname.equals("ASPECT_SLAF"))
      lnfname="ASPECT_METAL";
    if((FuLib.jdk()<=1.1)&&lnfname.equals("ASPECT_SKINLF"))
      lnfname="ASPECT_METAL";

         if(lnfname.equals("ASPECT_AMIGA"     )) lnfclass="swing.addon.plaf.threeD.ThreeDLookAndFeel";
 // else if(lnfname.equals("ASPECT_BASIC"     )) lnfclass="com.memoire.bu.BuBasicLookAndFeel";
    else if(lnfname.equals("ASPECT_KUNSTSTOFF")) lnfclass="com.incors.plaf.kunststoff.KunststoffLookAndFeel";
    else if(lnfname.equals("ASPECT_MAC"       )) lnfclass="com.sun.java.swing.plaf.mac.MacLookAndFeel";
    else if(lnfname.equals("ASPECT_METAL"     )) lnfclass="javax.swing.plaf.metal.MetalLookAndFeel";
    else if(lnfname.equals("ASPECT_MOTIF"     )) lnfclass="com.sun.java.swing.plaf.motif.MotifLookAndFeel";
    else if(lnfname.equals("ASPECT_NEXT"      )) lnfclass="nextlf.plaf.NextLookAndFeel";
    else if(lnfname.equals("ASPECT_ORGANIC"   )) lnfclass="com.sun.java.swing.plaf.organic.OrganicLookAndFeel";
    else if(lnfname.equals("ASPECT_OYOAHA"    )) lnfclass="com.oyoaha.swing.plaf.oyoaha.OyoahaLookAndFeel";
    else if(lnfname.equals("ASPECT_SKINLF"    )) lnfclass="com.l2fprod.gui.plaf.skin.SkinLookAndFeel";
    else if(lnfname.equals("ASPECT_SLAF"      )) lnfclass="com.memoire.slaf.SlafLookAndFeel";
    else if(lnfname.equals("ASPECT_WINDOWS"   )) lnfclass="com.sun.java.swing.plaf.windows.WindowsLookAndFeel";

    // this test is important under JDK1.2
    if(!lnfclass.equals(""))
      if(!UIManager.getLookAndFeel().getClass().getName().equals(lnfclass))
	_appli.setLookAndFeel(lnfclass);

    // Desktop
    BuBackgroundPainter bp=desktop.getBackgroundPainter();
    bp.setGrid(getBooleanProperty("desktop.grid",false));
    bp.setDots(getBooleanProperty("desktop.dots",false));

    if(getStringProperty("desktop.background").equals("DESKTOP_ROUGE"))
    {
      desktop.setBackground(new Color(128,64,64));
      bp.setIcon(null);
    }
    else
    if(getStringProperty("desktop.background").equals("DESKTOP_VERT"))
    {
      desktop.setBackground(new Color(64,128,64));
      bp.setIcon(null);
    }
    else
    if(getStringProperty("desktop.background").equals("DESKTOP_BLEU"))
    { 
      desktop.setBackground(new Color(64,64,128));
      bp.setIcon(null);
    }
    else
    if(getStringProperty("desktop.background").equals("DESKTOP_ORANGE"))
    { 
      desktop.setBackground(new Color(192,128,96));
      bp.setIcon(null);
    }
    else
    if(getStringProperty("desktop.background").equals("DESKTOP_SIMILAIRE"))
    { 
      desktop.setBackground(UIManager.getColor("Panel.background"));
      bp.setIcon(null);
    }
    else
    if(getStringProperty("desktop.background").equals("DESKTOP_DEFAUT"))
    { 
      desktop.setBackground(UIManager.getColor("Desktop.background"));
      bp.setIcon(null);
    }
    
    if(getStringProperty("desktop.decor").equals("DESKTOP_UNIFORME"))
    {
      bp.setIcon(null);
      bp.setGradient(false);
    }
    else
    if(getStringProperty("desktop.decor").equals("DESKTOP_DEGRADE"))
    {
      bp.setIcon(null);
      bp.setGradient(true);
    }
    else
    if(getStringProperty("desktop.decor").equals("DESKTOP_TEXTURE"))
    {
      if(getStringProperty("desktop.texture").equals("DESKTOP_TEXTURE1"))
        bp.setIcon(BuResource.BU.getIcon("background_desktop_1"));
      else
      if(getStringProperty("desktop.texture").equals("DESKTOP_TEXTURE2"))
        bp.setIcon(BuResource.BU.getIcon("background_desktop_2"));
      else
      if(getStringProperty("desktop.texture").equals("DESKTOP_TEXTURE3"))
        bp.setIcon(BuResource.BU.getIcon("background_desktop_3"));
      else
      {
	String file=BuPreferences.BU.getStringProperty("desktop.image");
	if(!"".equals(file)) bp.setIcon(new ImageIcon(file));
      }
    }

    desktop.setBackgroundPainter(bp);

    // Visible
    main.getSpecificBar().setVisible(getBooleanProperty("specificbar.visible",true));
    main.getLeftColumn() .setVisible(getBooleanProperty("leftcolumn.visible" ,true));
    main.getRightColumn().setVisible(getBooleanProperty("rightcolumn.visible",true));
    main.getStatusBar()  .setVisible(getBooleanProperty("statusbar.visible"  ,true));

    boolean va=getBooleanProperty("assistant.visible",true);
    main.getLeftColumn() .toggleComponent("ASSISTANT",va);
    main.getRightColumn().toggleComponent("ASSISTANT",va);

    if(getBooleanProperty("columns.swapped",false)^main.isColumnsSwapped())
      main.swapColumns();

    main.revalidate();
  }

  public boolean isEnabled(String _option)
  {
    boolean res=true;
    if(_option.startsWith("ASPECT_"))
    {
	   if(_option.endsWith("AMIGA"     )) res=BuLib.classExists("swing.addon.plaf.threeD.ThreeDLookAndFeel");
      else if(_option.endsWith("KUNSTSTOFF")) res=BuLib.classExists("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
      else if(_option.endsWith("MAC"       )) res=BuLib.classExists("com.sun.java.swing.plaf.mac.MacLookAndFeel");
      else if(_option.endsWith("METAL"     )) res=BuLib.classExists("javax.swing.plaf.metal.MetalLookAndFeel");
      else if(_option.endsWith("MOTIF"     )) res=BuLib.classExists("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
      else if(_option.endsWith("NEXT"      )) res=BuLib.classExists("nextlf.plaf.NextLookAndFeel");
      else if(_option.endsWith("ORGANIC"   )) res=BuLib.classExists("com.sun.java.swing.plaf.organic.OrganicLookAndFeel");
      else if(_option.endsWith("OYOAHA"    )) res=BuLib.classExists("com.oyoaha.swing.plaf.oyoaha.OyoahaLookAndFeel");
      else if(_option.endsWith("SKINLF"    )) res=BuLib.classExists("com.l2fprod.gui.plaf.skin.SkinLookAndFeel");
      else if(_option.endsWith("SLAF"      )) res=BuLib.classExists("com.memoire.slaf.SlafLookAndFeel");
      else if(_option.endsWith("WINDOWS"   )) res=BuLib.classExists("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
    }
    return res;
  }

  // Accessors
  
  public Color getColorProperty(String _key)
    { return getColorProperty(_key,Color.black); }

  public Color getColorProperty(String _key, Color _default)
  {
    String  s=get(_key);
    Color   r=_default;

    try { r=new Color(Integer.parseInt(s,16)); }
    catch(Exception ex) { }

    return r;
  }
}
