/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuPreferencesFrame.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * An internal frame which groups all the panels to set
 * user preferences.
 */

public class BuPreferencesFrame
       extends BuInternalFrame
       implements ActionListener, ChangeListener
{
  private BuTabbedPane tpMain_;
  private BuPanel      pnButtons_;
  private BuButton     btValider_, btAppliquer_, btAnnuler_, btFermer_;

  // Constructeur
  
  public BuPreferencesFrame()
  { this(null); }

  public BuPreferencesFrame(BuCommonImplementation _app)
  {
    super("",false,true,false,true);

    setName("ifPREFERENCES");
    if(_app!=null)
      _app.installContextHelp(getRootPane(),"bu/p-preferences.html");
    setShortcut(KeyEvent.VK_F2);

    tpMain_=new BuTabbedPane();
    tpMain_.setTabPlacement(BuLib.getTabPlacement());
    tpMain_.addChangeListener(this);

    // Panneau de validation
    pnButtons_=new BuPanel();
    pnButtons_.setLayout(new FlowLayout(FlowLayout.RIGHT));

    btAppliquer_=new BuButton(BuResource.BU.getString("Appliquer"));
    btAppliquer_.setIcon(BuResource.BU.getIcon("appliquer"));
    btAppliquer_.setActionCommand("APPLIQUER");
    btAppliquer_.addActionListener(this);
    pnButtons_.add(btAppliquer_,0);

    btAnnuler_=new BuButton(BuResource.BU.getString("Annuler"));
    btAnnuler_.setIcon(BuResource.BU.getIcon("annuler"));
    btAnnuler_.setActionCommand("ANNULER");
    btAnnuler_.addActionListener(this);
    pnButtons_.add(btAnnuler_,1);

    btValider_=new BuButton(BuResource.BU.getString("Ecrire"));
    btValider_.setIcon(BuResource.BU.getIcon("valider"));
    btValider_.setActionCommand("VALIDER");
    btValider_.addActionListener(this);
    getRootPane().setDefaultButton(btValider_);
    pnButtons_.add(btValider_,2);
    
    btFermer_=new BuButton(BuResource.BU.getString("Fermer"));
    btFermer_.setIcon(BuResource.BU.getIcon("fermer"));
    btFermer_.setActionCommand("FERMER");
    btFermer_.addActionListener(this);
    pnButtons_.add(btFermer_,3);
    
    JComponent content=(JComponent)getContentPane();
    content.setLayout(new BuBorderLayout());
    content.setBorder(new EmptyBorder(5,5,5,5));
    content.add(tpMain_ ,BuBorderLayout.CENTER);
    content.add(pnButtons_,BuBorderLayout.SOUTH);

    setTitle(BuResource.BU.getString("Prfrences de dmarrage"));
    setFrameIcon(BuResource.BU.getBarIcon("preference"));

    BuLib.computeMnemonics(pnButtons_,this);
    pack();
    updateButtons();
  }
  
  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    String actionCmd=_evt.getActionCommand();
    if(actionCmd.equals("VALIDER"))
    {
      /*
      for(int i=0; i<tpMain_.getTabCount(); i++)
      {
	BuAbstractPreferencesPanel p=
	  (BuAbstractPreferencesPanel)tpMain_.getComponentAt(i);
	if(p.isPreferencesValidable()) p.validatePreferences();
      }
      */
      // valider();
      Component c=tpMain_.getSelectedComponent();
      if(c instanceof BuAbstractPreferencesPanel)
      {
	BuAbstractPreferencesPanel pp=(BuAbstractPreferencesPanel)c;
	if(pp.isPreferencesValidable()) pp.validatePreferences();
      }
    }
    else
    if(actionCmd.equals("APPLIQUER"))
    {
      Component c=tpMain_.getSelectedComponent();
      if(c instanceof BuAbstractPreferencesPanel)
      {
	BuAbstractPreferencesPanel pp=(BuAbstractPreferencesPanel)c;
	if(pp.isPreferencesApplyable()) pp.applyPreferences();
      }
      // appliquer();
    }
    else
    if(actionCmd.equals("ANNULER"))
    {
      Component c=tpMain_.getSelectedComponent();
      if(c instanceof BuAbstractPreferencesPanel)
      {
	BuAbstractPreferencesPanel pp=(BuAbstractPreferencesPanel)c;
	if(pp.isPreferencesCancelable())
	{
	  pp.cancelPreferences();
	  if(pp.isPreferencesApplyable()) pp.applyPreferences();
	}
      }
      // annuler();
    }
    else
    if(actionCmd.equals("FERMER"))
    {
      try { setClosed(true); }
      catch(PropertyVetoException ex) { }
    }

    updateButtons();
  }

  public void setSelected(boolean _state) throws PropertyVetoException
  {
    super.setSelected(_state);
    if(isSelected()) tpMain_.requestFocus();
   }

  public void stateChanged(ChangeEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();
    // System.err.println("Change : "+source.getName());
    updateButtons();
    //BuLib.computeMnemonics(pnButtons_,this);
  }

  private void updateButtons()
  {
    Component c=tpMain_.getSelectedComponent();
    if(c instanceof BuAbstractPreferencesPanel)
    {
      BuAbstractPreferencesPanel pp=(BuAbstractPreferencesPanel)c;
      btAppliquer_.setEnabled(pp.isPreferencesApplyable ());
      btAnnuler_  .setEnabled(pp.isPreferencesCancelable());
      btValider_  .setEnabled(pp.isPreferencesValidable ());
    }
  }

  // Methodes publiques

  public void selectTab(int _index)
  {
    tpMain_.setSelectedIndex(_index);
  }
  
  public void addTab(BuAbstractPreferencesPanel _nouveau)
  {
    if(!(_nouveau instanceof JComponent))
      throw new RuntimeException(""+_nouveau+" is not a JComponent");

    //if(_nouveau instanceof Container)
    //BuLib.computeMnemonics((Container)_nouveau,null);

    String title  =_nouveau.getTitle();
    String tooltip=_nouveau.getToolTipText();

    tpMain_.addTab(title, null, (JComponent)_nouveau, tooltip);
    doLayout();
    validate();
  }
  
  // Methodes privees
  
  private void valider()
  {
    try { setClosed(true); }
    catch(PropertyVetoException ex) { }
  }

  private void appliquer()
  {
    // pack();
  }
  
  private void annuler()
  {
    try { setClosed(true); }
    catch(PropertyVetoException ex) { }
  }
}
