/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuPrinter.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.text.*;

/**
 * An utility class to print components, strings and tables.
 */

public class BuPrinter
{
  public final static Font FONTBASE=new Font("SansSerif",Font.PLAIN,10);

  public static BuInformationsDocument INFO_DOC;
  public static BuInformationsSoftware INFO_LOG;

  public static void print
    (Frame _parent, String _name, BuPrintable _composant)
  {
    System.err.println("IMPRESSION - Debut: "+_name);

    String  xxx=_name.toLowerCase().trim();
    if(INFO_LOG!=null) xxx+="_"+INFO_LOG.name.toLowerCase();

    xxx=xxx.replace('','a');
    xxx=xxx.replace('','e');
    xxx=xxx.replace('','e');
    xxx=xxx.replace('','e');
    xxx=xxx.replace('','e');
    xxx=xxx.replace('','i');
    xxx=xxx.replace('','o');
    xxx=xxx.replace('','u');
    xxx=xxx.replace('','u');
    xxx=xxx.replace('','c');
    
    String  tmp="";
    boolean us =true;
    for(int i=0;i<xxx.length();i++)
    {
      char c=xxx.charAt(i);
      if(Character.isLetterOrDigit(c)&&(c>='0')&&(c<='z'))
	{ tmp+=c; us=false; }
      else
	if(!us) {tmp+='_'; us=true; }
    }
    while(tmp.endsWith("_")) tmp=tmp.substring(0,tmp.length()-1);
    if("".equals(tmp))       tmp="job_"+FuLib.currentTimeSeconds();

    boolean win=BuLib.isWindows();
    Properties pp=new Properties();
    pp.put("awt.print.destination",(win ? "printer" : "file"));
    pp.put("awt.print.fileName",(win ? "" : "/tmp/"+tmp+".ps"));
    pp.put("awt.print.paperSize","a4");

    PrintJob pjob=_parent.getToolkit().getPrintJob(_parent,_name,pp);
    if(pjob!=null)
    {
      Graphics pgph=pjob.getGraphics();

      System.err.println("  Largeur   ="+pjob.getPageDimension().width);
      System.err.println("  Hauteur   ="+pjob.getPageDimension().height);
      System.err.println("  Rsolution="+pjob.getPageResolution());

      _composant.print(pjob,pgph);
      pgph.dispose();
      pjob.end();
    }
    else System.err.println("  ANNULEE");

    System.err.println("IMPRESSION - Fin  : "+_name);
  }

  public static void preview
    (BuCommonImplementation _app, String _name, BuPrintable _composant)
  {
    System.err.println("PREVISUALISATION - Debut: "+_name);

    PrintJob pjob=new BuPreviewJob(_app,_name);
    if(pjob!=null)
    {
      Graphics pgph=pjob.getGraphics();

      System.err.println("  Largeur   ="+pjob.getPageDimension().width);
      System.err.println("  Hauteur   ="+pjob.getPageDimension().height);
      System.err.println("  Rsolution="+pjob.getPageResolution());

      _composant.print(pjob,pgph);
      pgph.dispose();
      pjob.end();
    }
    else System.err.println("  ANNULEE");

    System.err.println("PREVISUALISATION - Fin  : "+_name);
  }

  public static void printHeader(PrintJob _job,Graphics _g)
  {
    printHeader(_job,_g,-1,-1);
  }

  public static void printHeader(PrintJob _job,Graphics _g, int _page)
  {
    printHeader(_job,_g,_page,-1);
  }

  public static void printHeader(PrintJob _job,Graphics _g, int _page, int _nbpages)
  {
    if(INFO_DOC==null) return;

    int pw  =_job.getPageDimension().width;
    int ph  =_job.getPageDimension().height;
    int pres=_job.getPageResolution();

    int marge=pres*2/5; // 1 cm

    int x=0;
    int y=0;

    if(INFO_DOC.logo!=null)
    {
      INFO_DOC.logo.paintIcon(null,_g,0,0);
      x+=INFO_DOC.logo.getIconWidth()+5;
    }

    Font ft;
    FontMetrics fm;

    _g.setColor(Color.black);

    ft=new Font(FONTBASE.getFamily(),Font.BOLD,FONTBASE.getSize());
    _g.setFont(ft);
    fm=_g.getFontMetrics();
    y+=fm.getMaxAscent();
    _g.drawString(INFO_DOC.name+" "+INFO_DOC.version,x,y);
    y+=fm.getMaxDescent();

    ft=FONTBASE;
    _g.setFont(ft);
    fm=_g.getFontMetrics();
    y+=fm.getMaxAscent();
    _g.drawString(INFO_DOC.date,x,y);
    y+=fm.getMaxDescent();

    ft=new Font(FONTBASE.getFamily(),Font.PLAIN,FONTBASE.getSize()-2);
    _g.setFont(ft);
    fm=_g.getFontMetrics();
    y+=fm.getMaxAscent();
    _g.drawString(INFO_DOC.author+" "+INFO_DOC.contact,x,y);
    y+=fm.getMaxDescent();

    if(_page>0)
    {
      String s="Page: "+_page;
      if(_nbpages>0) s+="/"+_nbpages;

      ft=FONTBASE;
      _g.setFont(ft);
      fm=_g.getFontMetrics();
      y=fm.getMaxAscent();
      x=pw-2*marge-fm.stringWidth(s);
      _g.drawString(s,x,y);
    }
  }

  public static void printFooter(PrintJob _job,Graphics _g)
  {
    if(INFO_LOG==null) return;

    int pw  =_job.getPageDimension().width;
    int ph  =_job.getPageDimension().height;
    int pres=_job.getPageResolution();

    int marge=pres*2/5; // 1 cm

    int x=0;
    int y=0;

    if(INFO_LOG.logo!=null)
    {
      INFO_LOG.logo.paintIcon(null,_g,0,0);
      x+=INFO_LOG.logo.getIconWidth()+5;
    }

    Font ft;
    FontMetrics fm;

    _g.setColor(Color.black);

    ft=new Font(FONTBASE.getFamily(),Font.BOLD,FONTBASE.getSize());
    _g.setFont(ft);
    fm=_g.getFontMetrics();
    y+=fm.getMaxAscent();
    _g.drawString(INFO_LOG.name+" "+INFO_LOG.version,x,y);
    y+=fm.getMaxDescent();

    /*
    ft=FONTBASE;
    _g.setFont(ft);
    fm=_g.getFontMetrics();
    y+=fm.getMaxAscent();
    _g.drawString(INFO_LOG.date,x,y);
    y+=fm.getMaxDescent();
    */

    ft=new Font(FONTBASE.getFamily(),Font.PLAIN,FONTBASE.getSize()-2);
    _g.setFont(ft);
    fm=_g.getFontMetrics();
    y+=fm.getMaxAscent();
    _g.drawString(INFO_LOG.http,x,y);
    y+=fm.getMaxDescent();
  }

  public static void printComponent(PrintJob _job,Graphics _g, Component _c)
  {
    int pw  =_job.getPageDimension().width;
    int ph  =_job.getPageDimension().height;
    int pres=_job.getPageResolution();

    int marge=pres*2/5; // 1 cm

    _g.setFont(FONTBASE);
    _g.setColor(Color.black);

    FontMetrics fm=_g.getFontMetrics();
    int x,y;
    int h=fm.getMaxAscent()+fm.getMaxDescent();

    x=marge;
    y=marge;
    _g.translate(x,y);
    printHeader(_job,_g);
    _g.translate(-x,-y);
    
    x=marge;
    y=ph-marge-64;
    _g.translate(x,y);
    printFooter(_job,_g);
    _g.translate(-x,-y);

    x=marge;
    y=marge+64+2*h;
    _g.translate(x,y);

    Shape     s=_g.getClip();
    Dimension d=_c.getSize();
    _g.setClip(0,0,d.width,d.height);
    _c.paint(_g);
    _g.setClip(s);
    _g.translate(-x,-y);
  }
    
  public static void printImage(PrintJob _job, Graphics _g, Image _c)
  {
    int pw  =_job.getPageDimension().width;
    int ph  =_job.getPageDimension().height;
    int pres=_job.getPageResolution();

    int marge=pres*2/5; // 1 cm

    _g.setFont(FONTBASE);
    _g.setColor(Color.black);

    FontMetrics fm=_g.getFontMetrics();
    int x,y;
    int h=fm.getMaxAscent()+fm.getMaxDescent();

    x=marge;
    y=marge;
    _g.translate(x,y);
    printHeader(_job,_g);
    _g.translate(-x,-y);
    
    x=marge;
    y=ph-marge-64;
    _g.translate(x,y);
    printFooter(_job,_g);
    _g.translate(-x,-y);

    x=marge;
    y=marge+64+2*h;
    _g.translate(x,y);

    Shape   s      =_g.getClip();
    int     wi     =_c.getWidth(null);
    int     hi     =_c.getHeight(null);
    boolean reduite=false;

    int wmax=pw-2*marge;
    int hmax=ph-2*(marge+128+2*h);

    if(wi>wmax)
    {
      hi=hi*wmax/wi;
      wi=wmax;
      reduite=true;
    }
    if(hi>hmax)
    {
      wi=wi*hmax/hi;
      hi=hmax;
      reduite=true;
    }
    _g.setClip(0,0,wi,hi);
    if(reduite) _g.drawImage(_c,0,0,wi,hi,null);
    else        _g.drawImage(_c,0,0,null);
    _g.setClip(s);

    _g.translate(-x,-y);
  }
    
  public static void printString(PrintJob _job, Graphics _g, String _t)
  {
    int pw  =_job.getPageDimension().width;
    int ph  =_job.getPageDimension().height;
    int pres=_job.getPageResolution();

    int marge=pres*2/5; // 1 cm
    int page =1;

    _g.setFont(FONTBASE);
    _g.setColor(Color.black);

    FontMetrics fm=_g.getFontMetrics();
    int x,y;
    int h=fm.getMaxAscent()+fm.getMaxDescent();

    x=marge;
    y=marge;
    _g.translate(x,y);
    printHeader(_job,_g,page);
    _g.translate(-x,-y);

    x=marge;
    y=ph-marge-64;
    _g.translate(x,y);
    printFooter(_job,_g);
    _g.translate(-x,-y);

    x=marge;
    y=marge+64+h;
    
    while(!_t.equals(""))
    {
      int    i=_t.indexOf("\n");
      String s;
      if(i>=0) { s=_t.substring(0,i); _t=_t.substring(i+1); }
      else     { s=_t; _t=""; }

      _g.drawString(s,x,y);
      y+=h;

      if(y>ph-marge-64-h)
      {
	_g.dispose();
	_g=_job.getGraphics();
	_g.setFont(FONTBASE);
	_g.setColor(Color.black);

	fm=_g.getFontMetrics();
	h=fm.getMaxAscent()+fm.getMaxDescent();
	page++;

	x=marge;
	y=marge;
	_g.translate(x,y);
	printHeader(_job,_g,page);
	_g.translate(-x,-y);

	x=marge;
	y=ph-marge-64;
	_g.translate(x,y);
	printFooter(_job,_g);
	_g.translate(-x,-y);

	x=marge;
	y=marge+64+h;
      }
    }
  }

  public static void printStyledDocument(PrintJob _job, Graphics _g, StyledDocument _doc)
  {
    int pw  =_job.getPageDimension().width;
    int ph  =_job.getPageDimension().height;
    int pres=_job.getPageResolution();

    int marge=pres*2/5; // 1 cm
    int page =1;

    _g.setFont(FONTBASE);
    _g.setColor(Color.black);

    FontMetrics fm=_g.getFontMetrics();
    int x,y;
    int h=fm.getMaxAscent()+fm.getMaxDescent();

    x=marge;
    y=marge;
    _g.translate(x,y);
    printHeader(_job,_g,page);
    _g.translate(-x,-y);

    x=marge;
    y=ph-marge-64;
    _g.translate(x,y);
    printFooter(_job,_g);
    _g.translate(-x,-y);

    x=marge;
    y=marge+64+h;
    
    
    Element root=_doc.getDefaultRootElement();
    int     nbl =root.getElementCount();

    for(int i=0;i<nbl;i++)
    {
      Element line=root.getElement(i);
      int     nbw =line.getElementCount();
      int     hl  =0;

      for(int j=0;j<nbw;j++)
      {	
	Element word=line.getElement(j);
	String t="";
	try
	{
	  int start=word.getStartOffset();
	  int end  =word.getEndOffset();
	  t=_doc.getText(start,end-start);
	  t=FuLib.replace(t,"\t","        ");
	}
	catch(BadLocationException ex) { }
	//System.err.println(word+" ["+replace(t,"\n","")+"]");

	AttributeSet attributes=word.getAttributes();
	_g.setColor(_doc.getForeground(attributes));
	_g.setFont (BuLib.deriveFont(_doc.getFont(attributes),-2));
	_g.drawString(t,x,y);

	fm=_g.getFontMetrics();
	x+=fm.stringWidth(t);
	hl=Math.max(hl,fm.getMaxAscent()+fm.getMaxDescent());
      }

      x=marge;
      y+=hl;

      if(y>ph-marge-64-h)
      {
	_g.dispose();
	_g=_job.getGraphics();
	_g.setFont(FONTBASE);
	_g.setColor(Color.black);

	fm=_g.getFontMetrics();
	h=fm.getMaxAscent()+fm.getMaxDescent();
	page++;

	x=marge;
	y=marge;
	_g.translate(x,y);
	printHeader(_job,_g,page);
	_g.translate(-x,-y);

	x=marge;
	y=ph-marge-64;
	_g.translate(x,y);
	printFooter(_job,_g);
	_g.translate(-x,-y);

	x=marge;
	y=marge+64+h;
      }
    }
  }

  private static void printCell
    (Graphics _g, FontMetrics _fm, String _s, int _x, int _y, int _w1)
  {
    int ws=_fm.stringWidth(_s)+4;
    
    if(ws>_w1)
    {
      while((!"".equals(_s))&&(_fm.stringWidth(_s+"...")+4>_w1))
	_s=_s.substring(0,_s.length()-1);
      _s+="...";
    }

    _g.drawString(_s,_x+2,_y+2+_fm.getMaxAscent());
  }
    
  public static void printTable(PrintJob _job, Graphics _g, JTable _t)
  { printTable(_job,_g,_t,-1); }

  public static void printTable(PrintJob _job, Graphics _g, JTable _t, int _height)
  {
    int pw  =_job.getPageDimension().width;
    int ph  =_job.getPageDimension().height;
    int pres=_job.getPageResolution();

    int marge=pres*2/5; // 1 cm
    int page =1;

    _g.setFont(FONTBASE);

    FontMetrics fm=_g.getFontMetrics();
    int x,y;

    int h1=fm.getMaxAscent()+fm.getMaxDescent()+4;
    int w1=(pw-2*marge)/_t.getColumnCount();

    x=marge;
    y=marge;
    _g.translate(x,y);
    printHeader(_job,_g,page);
    _g.translate(-x,-y);

    x=marge;
    y=ph-marge-64;
    _g.translate(x,y);
    printFooter(_job,_g);
    _g.translate(-x,-y);

    x=marge;
    y=marge+64+h1;

    final Color GRIS1=new Color(192,192,192);
    final Color GRIS2=new Color(224,224,224);

    for(int j=0;j<_t.getColumnCount();j++)
    {
      _g.setColor(GRIS1);
      _g.fillRect(x,y,w1,h1);
      _g.setColor(Color.black);
      _g.drawRect(x,y,w1,h1);
	
      String s=_t.getColumnName(j);
      printCell(_g,fm,s,x,y,w1);
      x+=w1;
    }

    x=marge;
    y+=h1;

    for(int i=0;i<_t.getRowCount();i++)
    {
      h1=Math.max(h1,_height);

      for(int j=0;j<_t.getColumnCount();j++)
      {
	_g.setColor(i%2==1 ? GRIS2 : Color.white);
	_g.fillRect(x,y,w1,h1);
	_g.setColor(Color.black);
	_g.drawRect(x,y,w1,h1);
	
	Object s=_t.getValueAt(i,j);
 	if(s instanceof Icon)
 	  ((Icon)s).paintIcon(_t,_g,x+2,y+2);
 	else
	if(s!=null)
 	  printCell(_g,fm,s.toString(),x,y,w1);

	x+=w1;
      }

      x=marge;
      y+=h1;

      if(y>ph-marge-64-h1)
      {
	_g.dispose();
	_g=_job.getGraphics();
	_g.setFont(FONTBASE);
	_g.setColor(Color.black);

	fm=_g.getFontMetrics();
	h1=fm.getMaxAscent()+fm.getMaxDescent()+4;
	w1=(pw-2*marge)/_t.getColumnCount();
	page++;

	x=marge;
	y=marge;
	_g.translate(x,y);
	printHeader(_job,_g,page);
	_g.translate(-x,-y);

	x=marge;
	y=ph-marge-64;
	_g.translate(x,y);
	printFooter(_job,_g);
	_g.translate(-x,-y);

	x=marge;
	y=marge+64+h1;

	for(int j=0;j<_t.getColumnCount();j++)
	{
	  _g.setColor(GRIS1);
	  _g.fillRect(x,y,w1,h1);
	  _g.setColor(Color.black);
	  _g.drawRect(x,y,w1,h1);
	
	  String s=_t.getColumnName(j);
	  printCell(_g,fm,s,x,y,w1);
	  x+=w1;
	}

	x=marge;
	y+=h1;
      }
    }
  }
}
