/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuResource.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;

import javax.swing.*;

/**
 * Utility class to manage resources.
 * As images, icons, localized strings, ...
 */
public class BuResource extends FuResource
{
  public final static BuResource BU=new BuResource(); //FuResource.FU);

  //  private BuResource(FuResource _parent) { setParent(_parent); }

  // pour les classes derivees
  // public BuResource() { }
  // public BuResource(BuResource _parent) { setParent(_parent); }

  public int getDefaultSize()
  {
    return BuPreferences.BU.getIntegerProperty
      ("icons.size",16);
  }

  public void setDefaultSize(int _v)
  {
    if(_v<=0) BuPreferences.BU.removeProperty("icons.size");
    else      BuPreferences.BU.putIntegerProperty("icons.size",_v);
  }

  // Preference icons.menuSize

  public int getDefaultMenuSize()
  {
    return BuPreferences.BU.getIntegerProperty
      ("icons.menuSize",getDefaultSize());
  }

  public void setDefaultMenuSize(int _v)
  {
    if(_v<=0) BuPreferences.BU.removeProperty("icons.menuSize");
    else      BuPreferences.BU.putIntegerProperty("icons.menuSize",_v);
  }

  // Preference icons.toolSize

  public int getDefaultToolSize()
  {
    return BuPreferences.BU.getIntegerProperty
      ("icons.toolSize",getDefaultSize());
  }

  public void setDefaultToolSize(int _v)
  {
    if(_v<=0) BuPreferences.BU.removeProperty("icons.toolSize");
    else      BuPreferences.BU.putIntegerProperty("icons.toolSize",_v);
  }

  // Preference icons.barSize

  public int getDefaultBarSize()
  {
    return BuPreferences.BU.getIntegerProperty
      ("icons.barSize",getDefaultSize());
  }

  public void setDefaultBarSize(int _v)
  {
    if(_v<=0) BuPreferences.BU.removeProperty("icons.barSize");
    else      BuPreferences.BU.putIntegerProperty("icons.barSize",_v);
  }

  // Preference cursors.size

  public int getDefaultCursorSize()
  {
    return BuPreferences.BU.getIntegerProperty
      ("cursors.size",getDefaultSize());
  }

  public void setDefaultCursorSize(int _v)
  {
    if(_v<=0) BuPreferences.BU.removeProperty("cursors.size");
    else      BuPreferences.BU.putIntegerProperty("cursors.size",_v);
  }

  // Property defaultBase

  private URL defaultBase_=null;
  public URL  getDefaultBase()       { return defaultBase_; }
  public void setDefaultBase(URL _v) { defaultBase_=_v; }

  // Property parent

  private BuResource parent_=null;
  public BuResource  getParent()                   { return parent_; }
  public void        setParent(BuResource _parent) { parent_=_parent; }

  // URL

  public URL getURL(String _path)
  {
    URL url=null;
    String p=_path;

    try
    {
      p=adjust(p);
      // System.out.println("~~~ P0="+p);

      if(getDefaultBase()!=null)
      {
	try { url=new URL(p); }
	catch(MalformedURLException ex) { url=null; }
	//System.out.println("H="+url.getHost());
      }
      else
      {
	url=getClass().getResource(p);
	if(url==null)
	{
	  int i,j;

	  i=p.lastIndexOf('_');
	  j=p.lastIndexOf('.');

	  p=(i>=0 ? p.substring(0,i) : p.substring(0,j))
	    +"_"+getDefaultSize()+p.substring(j);
	  // System.out.println("~~~ P1="+p);
	  url=getClass().getResource(p);
	}
	if((url==null)&&(getDefaultSize()!=16))
	{
	  int i,j;

	  i=p.lastIndexOf('_');
	  j=p.lastIndexOf('.');

	  p=(i>=0 ? p.substring(0,i) : p.substring(0,j))
	    +"_16"+p.substring(j);
	  // System.out.println("~~~ P2="+p);
	  url=getClass().getResource(p);
	}

	//if(url==null) System.out.println("~~~ INTROUVABLE: "+p);
      }
    }
    catch(Exception ex)
    {
      url=null;
    }

    //System.out.println("U="+url);

    if((url==null)&&(getParent()!=null))
      url=getParent().getURL(_path);

    return url;
  }

  public String adjust(String _path)
  {
    String r=_path;

    // System.out.println("base:  "+getDefaultBase());

    if(getDefaultBase()!=null)
    {
      // System.out.println("avant: "+_path);

      String c=getClass().getName();
      int i=c.lastIndexOf('.');
      if(i>=0) c=c.substring(0,i+1);
      c=c.replace('.','/');

      r=""+getDefaultBase()+c+_path;
      // System.out.println("apres: "+r);
    }

    return r;
  }

  // Stream

  public InputStream getStream(String _path)
  {
    InputStream r=null;

    URL url=getURL(_path);
    if(url!=null)
    {
      try { r=url.openStream(); }
      catch(IOException ex) { }
    }

    return r;
  }

  // Image

  public Image getImage(String _path, int _size)
  {
    return getIcon(_path,_size).getImage();
  }

  public Image getImage(String _path)
  {
    return getIcon(_path).getImage();
  }

  // BuIcon

  private Hashtable icons_=new Hashtable();

  public void clear_cache()
  {
    if(icons_!=null) icons_.clear();
    //if(pp_   !=null) pp_.clear();
  }

  public BuIcon getIcon(String _path, int _size)
  {
    String p=_path.toLowerCase();

    if(p.endsWith(".gif"))
      p=p.substring(0,p.length()-4)+"_"+_size+".gif";
    else
    if(p.endsWith(".jpg"))
      p=p.substring(0,p.length()-4)+"_"+_size+".jpg";
    else
      p+="_"+_size+".gif";

    BuIcon r=(BuIcon)icons_.get(p);
    if(r==null)
    {
      r=getIcon(getURL(p));
      icons_.put(p,r);
    }
    return r;
  }

  public BuIcon getIcon(String _path)
  {
    String p=_path.toLowerCase();
    if(  !p.endsWith(".gif")
       &&!p.endsWith(".jpg")) p+=".gif";

    BuIcon r=(BuIcon)icons_.get(p);
    if(r==null)
    {
      r=getIcon(getURL(p));
      icons_.put(p,r);
    }
    return r;
  }

  public BuIcon getMenuIcon(String _path)
  {
    BuIcon r=null;
    if(BuPreferences.BU.getBooleanProperty("icons.menu",true))
    {
      BuIcon k=(BuIcon)getIcon(_path);
      r=(BuIcon)icons_.get(k);
      if(r==null)
      {
	r=reduceMenuIcon(k);
	icons_.put(k,r);
      }
    }
    return r;
  }

  public BuIcon getToolIcon(String _path)
  {
    return reduceToolIcon(getIcon(_path));
  }

  public BuIcon getBarIcon(String _path)
  {
    return reduceBarIcon(getIcon(_path));
  }

  public BuIcon getTabIcon(String _path)
  {
    return reduceTabIcon(getIcon(_path));
  }

  public BuIcon reduceMenuIcon(BuIcon _icon)
  {
    return resizeIcon(_icon,getDefaultMenuSize());
  }

  public BuIcon reduceToolIcon(BuIcon _icon)
  {
    return resizeIcon(_icon,getDefaultToolSize());
  }

  public BuIcon reduceBarIcon(BuIcon _icon)
  {
    return resizeIcon(_icon,getDefaultBarSize());
  }

  public BuIcon reduceTabIcon(BuIcon _icon)
  {
    return resizeIcon(_icon,16);
  }

  public final static BuIcon resizeIcon(BuIcon _icon, int _size)
  {
    BuIcon r=_icon;
    if((r!=null)&&((r.getIconWidth()!=_size)||(r.getIconHeight()!=_size)))
    {
      r=new BuIcon(r.getImage().
		   getScaledInstance(_size,_size,Image.SCALE_SMOOTH));
    }
    return r;
  }
    
  public BuIcon getIcon(URL _url)
  {
    if(_url==null) return new BuIcon();
  
    BuIcon      r=new BuIcon(_url);
    ImageFilter f=(ImageFilter)UIManager.get("Theme.iconFilter");

    // SMOOTH ICONS TMP @GDX
    // TOO MANY EXCEPTIONS
    /*
    try
    {
      int   w=r.getIconWidth();
      int   h=r.getIconHeight();
      if((w>0)&&(h>0))
      {
	r=new BuIcon(r.getImage()
		     .getScaledInstance(2*w-1,2*h-1,Image.SCALE_SMOOTH));
	r=new BuIcon(r.getImage()
		     .getScaledInstance(w    ,h    ,Image.SCALE_SMOOTH));
      }
    }
    catch(Throwable th) { }
    */

    if(  (BuLib.jdk()>=1.2)
       &&BuPreferences.BU.getBooleanProperty("antialias.all",false))
    {
      r=new BuIcon
	(BuLib.HELPER.getToolkit().createImage
	 (new FilteredImageSource
	   (r.getImage().getSource(),BuFilters.ANTIALIAS)));
    }

    if(f!=null)
    {
      r=new BuIcon
	(BuLib.HELPER.getToolkit().createImage
	 (new FilteredImageSource
	  (r.getImage().getSource(),f)));
    }

    return r;
  }

  // Command

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand.
   */
  public BuIcon loadCommandIcon(String _cmd)
  {
    BuIcon r=null;
    String c=_cmd.toLowerCase();

    int i=c.indexOf('_');
    if(i>=0) c=c.substring(0,i);

    r=getIcon(c);
    if(r==null) r=getIcon("aucun");
    return r;
  }

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand and reduce it according to the
   * menu icon size.
   */
  public BuIcon loadMenuCommandIcon(String _cmd)
  {
    BuIcon r=null;
    if(BuPreferences.BU.getBooleanProperty("icons.menu",true))
      r=reduceMenuIcon(loadCommandIcon(_cmd));
    return r;
  }

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand and reduce it according to the
   * tool icon size.
   */
  public BuIcon loadToolCommandIcon(String _cmd)
  {
    return reduceToolIcon(loadCommandIcon(_cmd));
  }

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand and reduce it according to the
   * bar icon size.
   */
  public BuIcon loadBarCommandIcon(String _cmd)
  {
    return reduceBarIcon(loadCommandIcon(_cmd));
  }

  /**
   * Load an icon from a file which the name is given
   * by an actionCommand and reduce it according to the
   * tab icon size.
   */
  public BuIcon loadTabCommandIcon(String _cmd)
  {
    return reduceTabIcon(loadCommandIcon(_cmd));
  }

  // Cursors

  public Cursor getCursor(String _path, int _dx, int _dy)
  {
    return getCursor(_path,getDefaultCursorSize(),new Point(_dx,_dy));
  }

  public Cursor getCursor(String _path, int _size, Point _spot)
  {
    return getCursor(_path,_size,_spot,Cursor.DEFAULT_CURSOR);
  }

  public Cursor getCursor(String _path, int _size, Point _spot, int _default)
  {
    Cursor r=null;

    if(System.getProperty("java.version").compareTo("1.2")<0)
    {
      r=Cursor.getPredefinedCursor(_default);
    }
    else
    {
      try
      {
	Image i=getImage(_path,_size);

	int size=64;
	if(BuLib.isWindows()) size=32;

	Toolkit tk=BuLib.HELPER.getToolkit();

	i=tk.createImage
	  (new FilteredImageSource
	   (i.getSource(),new CropImageFilter(0,0,size,size)));

	if(BuPreferences.BU.getBooleanProperty
	   ("cursors.monochrome",BuLib.isUnix()))
	  i=tk.createImage
	    (new FilteredImageSource
	     (i.getSource(),BuFilters.BW));
	
	Method m=tk.getClass().getMethod
	  ("createCustomCursor",new Class[]
	   { Image.class, Point.class, String.class });

	r=(Cursor)m.invoke(tk,new Object[] { i,_spot,_path });
      }
      catch(Exception ex)
	{ r=Cursor.getPredefinedCursor(_default); }
    }

    return r;
  }

  // Strings

  public String getString(String _s)
  {
    String r=super.getString(_s);

    if(_s.equals(r))
    {
      if(getParent()!=null)
	r=getParent().getString(_s);
      else
	r=FuResource.FU.getString(_s);
    }

    return r;
  }

  public static void main(String[] argv)
  {
    JFrame frame=new JFrame("test curseur");
    frame.setBounds(0,0,200,200);
    frame.setVisible(true);
    //BuPreferences.BU.put("cursors.size","26");
    //BuPreferences.BU.put("cursors.monochrome","true");
    frame.setCursor(BU.getCursor("voir",10,10));
  }
}
