/**
 * @modification $Date: 2001/12/04 18:29:50 $
 * @statut       unstable
 * @file         BuSplashScreen.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.fu.*;

import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;

/**
 * An easy-to-use splash screen which takes a list of classes
 * to load.
 */

public class BuSplashScreen
       extends BuPopupWindow
       // implements Runnable
{
  /*
  class XLabel extends BuLabel
  {
    protected void paintComponent(Graphics _g)
    {
      Color old=getForeground();
      setForeground(Color.white);
      for(int i=-1;i<=1;i++)
	for(int j=-1;j<=1;j++)
	{
	  _g.translate(i,j);
	  super.paintComponent(_g);
	  _g.translate(-i,-j);
	}
      setForeground(old);
      super.paintComponent(_g);
    }
  }
  */

  /*
  class XMultiLabel extends BuLabelMultiLine
  {
    protected void paintComponent(Graphics _g)
    {
      Color old=getForeground();
      setForeground(Color.white);
      for(int i=-1;i<=1;i++)
	for(int j=-1;j<=1;j++)
	{
	  _g.translate(i,j);
	  super.paintComponent(_g);
	  _g.translate(-i,-j);
	}
      setForeground(old);
      super.paintComponent(_g);
    }
  }
  */

  private BuInformationsSoftware info_;
  private long                   delai_;
  private String[][]             classes_;
  // private String                 base_;

  protected BuPanel              panel_;
  protected BuPanel              left_;
  protected BuProgressBar        progress_;
  protected BuLabel              load_;
  protected BuLabel              name_;

  /** @deprecated */
  public BuSplashScreen(BuInformationsSoftware _info, long _delai,
			String[] _classes, String _base)
  {
    this(_info,_delai,new String[][]{_classes});
  }

  /** @deprecated */
  public BuSplashScreen(BuInformationsSoftware _info, long _delai,
			String[][] _classes, String _base)
  {
    this(_info,_delai,_classes);
  }

  public BuSplashScreen(BuInformationsSoftware _info, long _delai, String[][] _classes)
  {
    super();

    info_   =_info;
    classes_=_classes;
    delai_  =_delai;
    // base_   =_base;

    setLayout(new BuBorderLayout());
    setBorder(new LineBorder(Color.black,1));
    build();
    add(panel_,BuBorderLayout.CENTER);
  }

  public void build()
  {
    panel_=new BuPanel();
    panel_.setLayout(new BuBorderLayout());

    BuPicture ml_logo=null;
    if(info_.logo!=null) ml_logo=new BuPicture(info_.logo);

    BuVerticalLayout leftlo=new BuVerticalLayout();
    leftlo.setVfilled(false);
    leftlo.setVgap(5);

    left_=new BuPanel();
    left_.setLayout(leftlo);
    left_.setBorder(new BuLightBorder(BuLightBorder.RAISED,10));
    left_.setBackground(left_.getBackground().darker());
    if(ml_logo!=null)  left_.add(ml_logo);
    
    BuLabel          l1=new BuLabel();
    BuLabel          l2=new BuLabel();
    BuLabel          l3=new BuLabel();
    BuLabelMultiLine l4=new BuLabelMultiLine();

    name_=l1;

    l1.setText(info_.name+" "+info_.version);
    l2.setText(info_.date);
    l3.setText(info_.authors[0]);
    l4.setText(info_.contact+"\n"+info_.http);

    l1.setHorizontalAlignment(BuLabel.CENTER);
    l2.setHorizontalAlignment(BuLabel.CENTER);
    l3.setHorizontalAlignment(BuLabel.RIGHT );
    l4.setHorizontalAlignment(BuLabel.RIGHT );

    l1.setFont(BuLib.deriveFont("Label",Font.BOLD, +6));
    l2.setFont(BuLib.deriveFont("Label",Font.BOLD, +2));
    l3.setFont(BuLib.deriveFont("Label",Font.PLAIN, 0));
    l4.setFont(BuLib.deriveFont("Label",Font.PLAIN,-2));

    BuVerticalLayout lol=new BuVerticalLayout();
    lol.setVgap(5);
    lol.setVfilled(false);

    BuPanel mpl=new BuPanel();
    mpl.setLayout(lol);
    mpl.setBorder(new CompoundBorder
		  (new BuLightBorder(BuLightBorder.RAISED,5),
		   new EmptyBorder(5,5,5,5)));

    progress_=new BuProgressBar();
    /*
    progress_.setBorder(new CompoundBorder
			(new LineBorder(Color.white,1),
			 progress_.getBorder()));
    */

    load_=new BuLabel();
    load_.setText("Chargement...");
    load_.setHorizontalAlignment(BuLabel.LEFT);
    load_.setFont(BuLib.deriveFont("Label",Font.BOLD,-2));

    BuPanel px=new BuPanel();
    px.setOpaque(false);
    // px.setPreferredSize(new Dimension(0,0));

    mpl.add(px);
    mpl.add(l1);
    mpl.add(l2);
    mpl.add(l3);
    mpl.add(l4);
    mpl.add(progress_);
    mpl.add(load_);

    px.setPreferredSize(new Dimension
			(0,Math.max(0,384-mpl.getPreferredSize().height)));

    BuPicture ml_banner=null;
    if(info_.banner!=null)
    {
      ml_banner=new BuPicture(info_.banner);
      Dimension d=new Dimension(384,384);
      ml_banner.setPreferredSize(d);
      ml_banner.setSize(d);
      ml_banner.setMode(BuPicture.SCALE);
    }

    if(ml_banner!=null)
    {
      mpl.setOpaque(false);
      BuPanel p=new BuPanel();
      p.setLayout(new BuOverlayLayout());
      p.add(mpl);
      p.add(ml_banner);
      mpl=p;
    }
    panel_.add(left_,BuBorderLayout.WEST);
    panel_.add(mpl,BuBorderLayout.CENTER);
  }

  public void start()
  {
    Dimension dc=getPreferredSize();
    Dimension de=getToolkit().getScreenSize();

    int x=(de.width -dc.width )/2;
    int y=(de.height-dc.height)/2;

    addNotify();
    setLocation(x,y);
    setInvoker(BuLib.HELPER); //new JFrame();
    pack();
    BuLib.setDoubleBuffered(panel_,false);
    panel_.setDoubleBuffered(true);
    setDoubleBuffered(false);
    setVisible(true);

    run();
  }

  public void run()
  {
    long avant,apres,diff;
    avant=System.currentTimeMillis();

    BuUpdateGUI.repaintNow(this);

    int i,j,p,pp,m,l;

    /*
    String paquet=base_;
    i=paquet.lastIndexOf('.');
    if(i>=0) paquet=paquet.substring(0,i+1);
    else     paquet="";
    */
    // System.err.println("PAQUET="+paquet);

    /*
    boolean jdk12=false;
    if(System.getProperty("java.version").startsWith("1.2"))
      jdk12=true;
    */

    pp=-1;

    l=0;
    for(i=0; i<classes_.length;i++)
      l+=classes_[i].length;

    m=0;
    for(i=0; i<classes_.length;i++)
      for(j=0; j<classes_[i].length;j++)
      {
	String n=classes_[i][j];
	int k=n.lastIndexOf('.');
	setText(k>=0 ? n.substring(k+1) : n);
	// if(k<0) n=paquet+n;
	try
	{
	  Class c=Class.forName(n);
	  FuClassLoaderDedicated.putGlobal(c);
	}
	catch(Exception ex)
	  { /*System.err.println("Error loading class "+n);*/ }
      
	Thread.yield();
	/*
	try { Thread.sleep(5); }
	catch(Exception ex) { }
	*/

	m++;
	p=100*m/l;
	if(p!=pp)
	{
	  setProgression(p);
	  pp=p;
	}
      }

    setText("");
    apres=System.currentTimeMillis();
    if(apres<avant+delai_)
    {
      delai_=avant+delai_-apres;
      avant=apres;
      do
      {
        try { Thread.sleep(50); }
	catch(Exception ex) { }

	diff=System.currentTimeMillis()-avant;
	p=(int)(diff*100/delai_);
	setProgression(p);
      }
      while(delai_>diff);
    }
  }

  public void setText(String _text)
  {
    load_.setText(_text);
    BuUpdateGUI.repaintNow(load_);
  }

  public void setProgression(int _v)
  {
    progress_.setValue(_v);
    BuUpdateGUI.repaintNow(progress_);
  }
}
