/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuSplit3Pane.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * A split panel into 3 (resizable by the user).
 * Because the JSplitPane does not work fine.
 */

public class BuSplit3Pane
       extends BuPanel
       implements LayoutManager2, MouseListener, MouseMotionListener
{
  // private final int RAPPORT=10000;

  class Divider extends JComponent
  {
    private int cursor_;
    public  int p_;
    public  int q_;

    Divider()
    {
      cursor_=Cursor.MOVE_CURSOR;
      this.setCursor(new Cursor(cursor_));
      this.p_=0;
      configure();
    }

    public void updateUI()
    {
      super.updateUI();
      configure();
    }

    private void configure()
    {
      Color bg=UIManager.getColor("Divider.background");
      if(bg==null) bg=UIManager.getColor("Panel.background");
      if(bg!=null) this.setBackground(bg);
      Color fg=UIManager.getColor("Divider.foreground");
      if(fg==null) fg=UIManager.getColor("Panel.foreground");
      if(fg!=null) this.setForeground(fg);
      Border bd=UIManager.getBorder("Divider.border");
      /*if(bd!=null)*/ this.setBorder(bd);

      Object op=UIManager.get("Divider.opaque");
      //System.err.println("OPAQUE="+op);
      if(op instanceof Boolean)
	this.setOpaque(((Boolean)op).booleanValue());

      Object wi=UIManager.get("Divider.width");
      if(wi instanceof Integer)
      {
	int wd=((Integer)wi).intValue();
	this.setPreferredSize(new Dimension(wd,wd));
      }
      else
	this.setPreferredSize(new Dimension(9,9));
    }

    protected void paintComponent(Graphics _g)
    {
      int w=this.getWidth();
      int h=this.getHeight();
      int x=w/2-2;
      int y=h/2-2;
      Color fg=this.getForeground();
      Color bg=this.getBackground();

      if(this.isOpaque())
      {
	_g.setColor(bg);
	_g.fillRect(0,0,w,h);
      }

      String p=(String)UIManager.get("Divider.drawing");
      //System.err.println("DRAWING="+p);

      if("double".equals(p)||"double_raised_line".equals(p))
      {
	_g.setColor(bg.darker());
	_g.drawLine(x  ,2,x  ,h-5);
	_g.drawLine(x+3,2,x+3,h-5);
	_g.setColor(bg.brighter());
	_g.drawLine(x+1,2,x+1,h-5);
	_g.drawLine(x+4,2,x+4,h-5);
      }
      else
      if("square".equals(p))
      {
	_g.setColor(fg);
	_g.drawRect(x-1,y-1,6,6);
      }
      else
      if("filled_square".equals(p))
      {
	_g.setColor(fg);
	_g.fillRect(x-1,y-1,5,5);
      }
      else
      if("lowered_square".equals(p))
      {
	_g.setColor(bg);
	_g.draw3DRect(x-1,y-1,6,6,false);
      }
      else
      if("raised_square".equals(p))
      {
	_g.setColor(bg);
	_g.draw3DRect(x-1,y-1,6,6,true);
      }
      else
      if("line".equals(p))
      {
	_g.setColor(fg);
	_g.drawLine(x+2,2,x+2,h-5);
      }
      else
      if("thick_line".equals(p))
      {
	_g.setColor(fg);
	_g.drawLine(x+1,2,x+1,h-5);
	_g.drawLine(x+2,2,x+2,h-5);
      }
      else
      if("double_line".equals(p))
      {
	_g.setColor(fg);
	_g.drawLine(x  ,2,x  ,h-5);
	_g.drawLine(x+3,2,x+3,h-5);
      }
      else
      if("".equals(p)||"nothing".equals(p))
      {
      }
      else
      if(BuLib.isMetal())
      {
	_g.setColor(bg.darker());
	_g.drawLine(x+2,0,x+2,h-1);
      }
      else
      {
	_g.setColor(bg);
	_g.draw3DRect(x+1,2,2,h-5,true);
      }
    }
  }

  private JComponent left_,middle_,right_;
  private Divider    divider1_,divider2_;

  public BuSplit3Pane(JComponent _left, JComponent _middle, JComponent _right)
  {
    putClientProperty("BuSplit3Pane.dragMode","outline");

    divider1_=new Divider();
    divider1_.addMouseMotionListener(this);
    divider1_.addMouseListener(this);

    divider2_=new Divider();
    divider2_.addMouseMotionListener(this);
    divider2_.addMouseListener(this);

    setLayout(this);
    add(divider1_);
    add(divider2_);

    setLeftComponent  (_left);
    setMiddleComponent(_middle);
    setRightComponent (_right);

    updateSplits();

    /*
    if(_left  ==null) { _left  =new JPanel(); _left  .setVisible(false); }
    if(_middle==null) { _middle=new JPanel(); _middle.setVisible(false); }
    if(_right ==null) { _right =new JPanel(); _right .setVisible(false); }

    left_  =_left;
    middle_=_middle;
    right_ =_right;

    updateSplits();

    setLayout(this);
    add(left_);
    add(divider1_);
    add(middle_);
    add(divider2_);
    add(right_);
    */
  }

  public void setLeftComponent(JComponent _left)
  {
    if(left_!=null) remove(left_);

    if(_left==null) { _left=new JPanel(); _left.setVisible(false); }
    left_=_left;
    add(left_);
  }

  public void setMiddleComponent(JComponent _middle)
  {
    if(middle_!=null) remove(middle_);

    if(_middle==null) { _middle=new JPanel(); _middle.setVisible(false); }
    middle_=_middle;
    add(middle_);
  }

  public void setRightComponent(JComponent _right)
  {
    if(right_!=null) remove(right_);

    if(_right==null) { _right=new JPanel(); _right.setVisible(false); }
    right_=_right;
    add(right_);
  }

  // Divers

  public void swapColumns()
  {
    JComponent ctmp=left_;
    Divider    dtmp=divider1_;

    left_ =right_;
    right_=ctmp;
    divider1_=divider2_;
    divider2_=dtmp;

    updateDisplay();
  }

  public void updateSplits()
  {
    int wl=left_ .getPreferredSize().width;
    int wr=right_.getPreferredSize().width;

    if((divider1_.p_!=wl)||(divider2_.p_!=-wr))
    {
      divider1_.p_=wl;
      divider2_.p_=wr;
      updateDisplay();
    }
  }

  public void updateDisplay()
  {
    revalidate();
    //repaint();
  }

  private boolean isOutlineDragMode()
  {
    return "outline".equals(getClientProperty("BuSplit3Pane.dragMode"));
  }

  // Mouse

  public void mouseDown(MouseEvent _evt)
  { }

  public void mouseEntered(MouseEvent _evt)
  { }

  public void mouseExited(MouseEvent _evt)
  { }

  public void mousePressed(MouseEvent _evt)
  {
    Divider divider=(Divider)_evt.getSource();

    if(isOutlineDragMode())
      paintDraggedDivider(divider);
  }

  public void mouseReleased(MouseEvent _evt)
  {
    Divider divider=(Divider)_evt.getSource();
    divider.cursor_=Cursor.MOVE_CURSOR;
    divider.setCursor(new Cursor(divider.cursor_));

    if(isOutlineDragMode())
    {
      paintDraggedDivider(divider);
      updateDisplay();
    }
  }

  public void mouseUp(MouseEvent _evt)
  { }

  public void mouseClicked(MouseEvent _evt)
  {
    Divider divider=(Divider)_evt.getSource();
    boolean refresh=false;

    if((_evt.getModifiers()&MouseEvent.BUTTON3_MASK)==0)
    {
      refresh=(divider.p_!=divider.q_);

      if(divider.p_!=0)
      {
	divider.q_=divider.p_;
	divider.p_=0;
      }
      else
      {
	divider.p_=divider.q_;
	divider.q_=0;
      }
    }
    else
    if(divider==divider1_)
    {
      int wl=left_.getPreferredSize().width;
      if(divider1_.p_!=wl)
      {
	divider1_.p_=wl;
	refresh=true;
      }
    }
    else
    if(divider==divider2_)
    {
      int wr=right_.getPreferredSize().width;
      if(divider2_.p_!=wr)
      {
	divider2_.p_=wr;
	refresh=true;
      }
    }

    if(refresh) updateDisplay();
  }

  // MouseMotion

  private void paintDraggedDivider(Divider _d)
  {
    if(isOutlineDragMode())
    {
      Insets   i=getInsets();
      Graphics g=getGraphics();
      int x;

      if(_d==divider1_) x=i.left+_d.p_;
      else              x=getWidth()-i.right-_d.p_-_d.getWidth();

      g.setXORMode(new Color(64,64,64));
      g.fillRect(x+1,i.top,_d.getWidth()-1,
		 getHeight()-i.top-i.bottom-1);
      g.setXORMode(Color.gray);
      g.drawRect(x,i.top,_d.getWidth()-1,
		 getHeight()-i.top-i.bottom-1);
    }
  }

  public void mouseDragged(MouseEvent _evt)
  {
    int ex=_evt.getX();
    // System.err.println("DRAG "+ex);

    boolean refresh=false;
    Insets  insets=getInsets();
    int     w=getWidth()-insets.left-insets.right;

    if(_evt.getSource()==divider1_)
    {
      int pp=divider1_.p_;

      int x1=ex+divider1_.getLocation().x-insets.left;
      int x2=divider2_.getLocation().x-insets.left;
      if(x1<0) x1=0;
      if(x1+divider1_.getWidth()>=x2) x1=x2-divider1_.getWidth();
      if(x1+divider1_.getWidth()>=w ) x1=w-divider1_.getWidth();

      int nc=-1;
      if(x1!=pp)
      {
	if(x1<pp) nc=Cursor.W_RESIZE_CURSOR;
	else      nc=Cursor.E_RESIZE_CURSOR;

	paintDraggedDivider(divider1_);
	divider1_.p_=x1;
	paintDraggedDivider(divider1_);
	refresh=true;
      }

      if(nc>0)
      {
	divider1_.cursor_=nc;
	divider1_.setCursor(new Cursor(divider1_.cursor_));
      }
    }

    if(_evt.getSource()==divider2_)
    {
      int pp=divider2_.p_;

      int x1=divider1_.getLocation().x-insets.left;
      int x2=ex+divider2_.getLocation().x-insets.left;
      if(x2<0) x2=0;
      if(x1+divider1_.getWidth()>=x2-divider2_.getWidth())
	x2=x1+divider1_.getWidth()+divider2_.getWidth();
      if(x2>=w) x2=w;

      x2=w-x2;
      int nc=-1;
      if(x2!=pp)
      {
	if(x2<pp) nc=Cursor.W_RESIZE_CURSOR;
	else      nc=Cursor.E_RESIZE_CURSOR;

	paintDraggedDivider(divider2_);
	divider2_.p_=x2;
	paintDraggedDivider(divider2_);
	refresh=true;
      }

      if(nc>0)
      {
	divider2_.cursor_=nc;
	divider2_.setCursor(new Cursor(divider2_.cursor_));
      }
    }

    if(refresh&&!isOutlineDragMode())
      updateDisplay();
  }

  public void mouseMoved(MouseEvent _evt)
  { }

  // Layout

  public void addLayoutComponent(String _p, Component _c) { }
  public void addLayoutComponent(Component _c, Object _o) { }
  public void removeLayoutComponent(Component _c) { }

  public Dimension minimumLayoutSize(Container _c)
  {
    int w=left_.getMinimumSize().width+
      divider1_.getPreferredSize().width+
      middle_.getMinimumSize().width+
      divider2_.getPreferredSize().width+
      right_.getMinimumSize().width;

    int h=Math.max(Math.max(left_.getMinimumSize().height,
			    middle_.getMinimumSize().height),
		   right_.getMinimumSize().height);

    return new Dimension(w,h);
  }

  public Dimension preferredLayoutSize(Container _c)
  {
    int w=left_.getPreferredSize().width+
      divider1_.getPreferredSize().width+
      middle_.getPreferredSize().width+
      divider2_.getPreferredSize().width+
      right_.getPreferredSize().width;
    int h=Math.max(Math.max(left_.getPreferredSize().height,
			    middle_.getPreferredSize().height),
		   right_.getPreferredSize().height);
    return new Dimension(w,h);
  }

  public Dimension maximumLayoutSize(Container _c)
  { return preferredLayoutSize(_c); }

  public float getLayoutAlignmentX(Container _c)
  { return 0.5f; }

  public float getLayoutAlignmentY(Container _c)
  { return 0.5f; }

  public void invalidateLayout(Container _c)
  {
    // System.out.println("LEFT ="+left_ .isVisible());
    // System.out.println("RIGHT="+right_.isVisible());
    divider1_.setVisible
      (  (left_.getParent()  ==this)&&left_ .isVisible()
       &&(middle_.getParent()==this)&&middle_.isVisible());
    divider2_.setVisible
      ((right_.getParent()==this)&&right_.isVisible());
  }
  
  public void layoutContainer(Container _c)
  {
    Insets insets=getInsets();
    int d1=divider1_.getPreferredSize().width;
    int d2=divider2_.getPreferredSize().width;
    int w=_c.getSize().width -insets.left-insets.right;
    int h=_c.getSize().height-insets.top-insets.bottom;

    int x1=divider1_.p_;
    int x2=w-divider2_.p_;

    if(!divider1_.isVisible()) { x1=0; d1=0; }
    if(!divider2_.isVisible()) { x2=w; d2=0; }

    int wm=(x2-d2-x1-d1);
    if(!middle_.isVisible()&&left_.isVisible())
      { x1=x2; wm=0; }

    left_    .setBounds(insets.left      ,insets.top,x1  ,h);
    divider1_.setBounds(insets.left+x1   ,insets.top,d1  ,h);
    middle_  .setBounds(insets.left+x1+d1,insets.top,wm  ,h);
    divider2_.setBounds(insets.left+x2-d2,insets.top,d2  ,h);
    right_   .setBounds(insets.left+x2   ,insets.top,w-x2,h);
  }

  public static void main(String[] _args)
  {
    JFrame f=new JFrame("test BuSplit3Pane");

    JButton c1=new JButton("1");
    JButton c2=new JButton("2");
    JButton c3=new JButton("3");

    c1.setBorder(new LineBorder(Color.black,2));
    c2.setBorder(new LineBorder(Color.black,2));
    c3.setBorder(new LineBorder(Color.black,2));

    BuSplit3Pane sp=new BuSplit3Pane(c1,c2,c3);
    sp.setBorder(new LineBorder(Color.red,12));

    f.getContentPane().add("Center",sp);
    f.setSize(300,200);
    f.show();
    f.setLocation(500,100);
  }
}
