/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuStatusBar.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A status bar which contains a memory monitor, a text line
 * and a progress bar.
 */

public class BuStatusBar
       extends BuPanel
{
  private BuPanel         monitors_;
  private BuLabel         message_;
  private BuProgressBar   progression_;

  public BuStatusBar()
  {
    super();
    setName("buSTATUSBAR");

    monitors_=new BuPanel();
    monitors_.setBorder(new EmptyBorder(0,0,0,0));
    monitors_.setLayout(new BuHorizontalLayout(2,true,true));

    if(BuPreferences.BU.getBooleanProperty("monitor.memory",true))
      addMonitor(new BuMonitorMemory());

    if(BuPreferences.BU.getBooleanProperty("monitor.cpu" ,false))
      addMonitor(new BuMonitorCpu());

    if(BuPreferences.BU.getBooleanProperty("monitor.clock" ,false))
      addMonitor(new BuMonitorClock());

    if(BuPreferences.BU.getBooleanProperty("monitor.time"  ,false))
      addMonitor(new BuMonitorTime());

    message_=new BuLabel(BuResource.BU.getString("Bienvenue"));
    message_.setName("lbMESSAGESTATUS");

    progression_=new BuProgressBar();
    progression_.setMinimum(0);
    progression_.setMaximum(100);
    progression_.setValue(0);
    progression_.setName("pgTASKPROGRESS");
    progression_.setPreferredSize(new Dimension(120,14));

    BuBorderLayout layout=new BuBorderLayout();
    layout.setHgap(5);
    setLayout(layout);

    add(monitors_   ,BuBorderLayout.WEST);
    add(message_    ,BuBorderLayout.CENTER);
    add(progression_,BuBorderLayout.EAST);
  }

  public void addMonitor(JComponent _monitor)
  {
    monitors_.add(_monitor);
    _monitor.revalidate();
  }

  public void removeMonitor(JComponent _monitor)
  {
    monitors_.remove(_monitor);
    _monitor.revalidate();
  }

  public void setApp(BuCommonInterface _app)
  {
    Component[] c=monitors_.getComponents();
    for(int i=0;i<c.length;i++)
      if(c[i] instanceof BuMonitorAbstract)
	((BuMonitorAbstract)c[i]).setApp(_app);
  }

  public void updateUI()
  {
    super.updateUI();

    Border b=UIManager.getBorder("StatusBar.border");
    if(b!=null) setBorder(b);
    Color bg=UIManager.getColor("StatusBar.background");
    if(bg!=null) setBackground(bg);
    Color fg=UIManager.getColor("StatusBar.foreground");
    if(fg!=null) setForeground(fg);
    Font ft=UIManager.getFont("StatusBar.font");
    if(ft!=null) setFont(ft);
  }

  public void setMessage(String _s)
  {
    if(_s==null) _s="";

    if(!_s.equals(message_.getText()))
    {
      message_.setText(_s);
      BuUpdateGUI.repaintNow(message_);
    }
  }

  public void setProgression(int _v)
  {
    if(_v<  0) _v=0;
    if(_v>100) _v=100;

    if(progression_.getValue()!=_v)
    {
      progression_.setValue(_v);
      BuUpdateGUI.repaintNow(progression_);
    }
  }
}
