/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuStringValidator.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.util.*;
import java.text.*;

/**
 * An abstract root class to specify a validator for input strings.
 * Provides standard ones for ints, doubles, dates, ...
 */

public abstract class BuStringValidator
{
  public abstract boolean isStringValid(String _string);
  public abstract Object  stringToValue(String _string);

  public String valueToString(Object _value)
    { return _value.toString(); }

  // Instances

  public final static BuStringValidator INTEGER=new BuStringValidator()
  {
    public boolean isStringValid(String _string)
    {
      boolean r=true;
      try { new Integer(_string); }
      catch(Exception ex) { r=false; }
      return r;
    }
    public Object stringToValue(String _string)
      { return new Integer(_string); }
  };

  public final static BuStringValidator LONG=new BuStringValidator()
  {
    public boolean isStringValid(String _string)
    {
      boolean r=true;
      try { new Long(_string); }
      catch(Exception ex) { r=false; }
      return r;
    }
    public Object stringToValue(String _string)
      { return new Long(_string); }
  };

  public final static BuStringValidator FLOAT=new BuStringValidator()
  {
    public boolean isStringValid(String _string)
    {
      boolean r=true;
      try { new Float(_string.replace(',','.')); }
      catch(Exception ex) { r=false; }
      return r;
    }
    public Object stringToValue(String _string)
      { return new Float(_string.replace(',','.')); }
  };

  public final static BuStringValidator DOUBLE=new BuStringValidator()
  {
    public boolean isStringValid(String _string)
    {
      boolean r=true;
      try { new Double(_string.replace(',','.')); }
      catch(Exception ex) { r=false; }
      return r;
    }
    public Object stringToValue(String _string)
      { return new Double(_string.replace(',','.')); }
  };

  public final static BuStringValidator ID=new BuStringValidator()
  {
    public boolean isStringValid(String _string)
      { return true; }
    public Object stringToValue(String _string)
      { return _string; }
  };

  public final static BuStringValidator FILE=new BuStringValidator()
  {
    public boolean isStringValid(String _string)
      { return (_string.length()>0); }
    public Object stringToValue(String _string)
      { return _string; }
  };

  public final static BuStringValidator DATE=new BuStringValidator()
  {
    public boolean isStringValid(String _string)
    {
      boolean r=true;
      try
      {
	DateFormat.getDateInstance
	  (DateFormat.SHORT,Locale.getDefault()).parse(_string);
      }
      catch(Exception ex) { r=false; }
      return r;
    }
    public Object stringToValue(String _string)
    {
      Date r=null;
      try
      {
	r=DateFormat.getDateInstance
	  (DateFormat.SHORT,Locale.getDefault()).parse(_string);
      }
      catch(Exception ex) { r=null; }
      // System.err.println(""+r);
      return r;
    }
    public String valueToString(Object _value)
    {
      return DateFormat.getDateInstance
	(DateFormat.SHORT,Locale.getDefault()).format(_value);      
    }
  };

  public final static BuStringValidator TIME=new BuStringValidator()
  {
    public boolean isStringValid(String _string)
    {
      boolean r=true;

      try
      {
	DateFormat.getTimeInstance
	  (DateFormat.MEDIUM,Locale.getDefault()).parse(_string);
      }
      catch(Exception ex) { r=false; }
      
      if(r==false)
      {
	r=true;
	if((_string.length()!=5)&&(_string.length()!=8)) r=false;
	else
	{
	  try { new SimpleDateFormat("hh:mm:ss").parse(_string); }
	  catch(ParseException ex1)
	  {
	    try { new SimpleDateFormat("hh:mm").parse(_string); }
	    catch(ParseException ex2)
	      { r=false; }
	  }
	}
      }
      return r;
    }
    public Object stringToValue(String _string)
    {
      Date r=null;

      try
      {
	r=DateFormat.getTimeInstance
	  (DateFormat.MEDIUM,Locale.getDefault()).parse(_string);
      }
      catch(ParseException ex) { r=null; }

      if(r==null)
      {
	try  { r=new SimpleDateFormat("hh:mm:ss").parse(_string); }
	catch(ParseException ex1)
	{
	  try { r=new SimpleDateFormat("hh:mm").parse(_string); }
	  catch(ParseException ex2)
	    { r=null; }
	}
      }

      // System.err.println(""+r);
      if(r!=null) return new Long(r.getTime());
      return r;
    }
    public String valueToString(Object _value)
    {
      return DateFormat.getTimeInstance
	(DateFormat.MEDIUM,Locale.getDefault()).format(_value);      
    }
  };
}


