/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuTable.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;

/**
 * Similar to JTable but add copy/select operations and row headers.
 * @see javax.swing.JTable
 */
public class BuTable
  extends JTable
  implements ClipboardOwner
{
  public BuTable()
  {
    super();
  }

  public BuTable(TableModel _model)
  {
    super(_model);
  }

  // Anti-aliasing

  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  private final static TableCellRenderer tcr_=new BuTableCellRenderer();

  public TableCellRenderer getDefaultRenderer(Class _columnClass)
  {
    return tcr_;
  }

  public Object[][] getAllValues()
  {
    int nr =getRowCount();
    int nc =getColumnCount();

    Object[][] r=new Object[nr][nc];

    int i,j;
    for(i=0;i<nr;i++)
      for(j=0;j<nc;j++)
	r[i][j]=getValueAt(i,j);

    return r;
  }

  public Object[][] getSelectedValues()
  {
    int[]   sr =getSelectedRows();
    int[]   sc =getColumnModel().getSelectedColumns();
    boolean rsa=getRowSelectionAllowed();
    boolean csa=getColumnSelectionAllowed();

    Object[][] r=new Object[rsa ? sr.length : getRowCount()]
                           [csa ? sc.length : getColumnCount()];

    int i,j;
    for(i=0;i<r.length;i++)
      for(j=0;j<r[i].length;j++)
	r[i][j]=getValueAt(rsa ? sr[i] : i, csa ? sc[j] : j);

    return r;
  }

  public String[] getSelectedColumnNames()
  {
    int[] sc=getColumnModel().getSelectedColumns();

    String[] r=new String[sc.length];
    for(int i=0;i<sc.length;i++)
      r[i]=getColumnName(sc[i]);
    return r;
  }

  public void lostOwnership(Clipboard clipboard, Transferable contents)
  {
  }

  public String convertAllValues(String _separator)
  {
    return convert0(getAllValues(),_separator);
  }

  public String convertSelectedValues(String _separator)
  {
    return convert0(getSelectedValues(),_separator);
  }

  private String convert0(Object[][] _values,String _separator)
  {
    String r="";

    for(int i=0;i<_values.length;i++)
    {
      for(int j=0;j<_values[i].length;j++)
      {
	if(j!=0) r+=_separator;
	if(_values[i][j]!=null) r+=_values[i][j].toString();
      }
      r+="\n"; //System.getProperty("line.separator");
    }

    return r;
  }

  public void copy()
  {
    String r=convertSelectedValues("\t");

    if(!r.equals(""))
    {
      Clipboard clipboard=getToolkit().getSystemClipboard();
      StringSelection contents=new StringSelection(r);
      clipboard.setContents(contents,this);
    }
  }

  public void select()
  {
    if(getColumnSelectionAllowed())
      setColumnSelectionInterval(0,getColumnCount()-1);
    if(getRowSelectionAllowed())
      setRowSelectionInterval(0,getRowCount()-1);
  }

  public JList buildRowHeaders(boolean _letters)
  {
    /*
    JList r=new JList
      (new AbstractListModel()
       {
	 public int getSize() { return getModel().getRowCount(); }
	 public Object getElementAt(int index) { return null; }
       });
    r.setFixedCellWidth(40);
    r.setFixedCellHeight(getRowHeight()+
			 getIntercellSpacing().height);
    r.setCellRenderer(new BuTableRowHeaderRenderer(this,_letters));
    r.setRequestFocusEnabled(false);
    return r;
    */
    return new BuTableRowHeader(this,_letters);
  }

  public void addRowHeaders(JScrollPane _pane, boolean _letters)
  {
    _pane.setRowHeaderView(buildRowHeaders(_letters));
  }

  public static void main(String[] _args)
  {
    JFrame frame=new JFrame("Test BuTable");

    TableModel tm=new AbstractTableModel()
      {
	public int getRowCount() { return 101; }
	public int getColumnCount() { return 4; }
	public Object getValueAt(int row, int col)
	  { return new Integer(row*col); }
      };

    final BuTable tb=new BuTable(tm);

    tb.setColumnSelectionAllowed(true);
    tb.setRowSelectionAllowed(true);

    /*
      tb.getColumnModel().getSelectionModel().
      setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      tb.getSelectionModel().
      setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    */

    TableCellRenderer tcr=new BuTableCellRenderer();
    for(int i=0;i<tb.getColumnCount();i++)
    {
      TableColumn c=tb.getColumn(tb.getColumnName(i));
      c.setCellRenderer(tcr);
    }

    JScrollPane sp=new JScrollPane(tb);
    tb.addRowHeaders(sp,false);
    frame.getContentPane().add(sp);

    frame.addWindowListener( new WindowAdapter() {
      public void windowClosing( WindowEvent e ) {
	System.err.println(tb.convertSelectedValues(","));
	System.exit(0);
      }
    });

    frame.setSize(300,200);
    frame.setVisible(true);
  }
}

