/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuTableCellRenderer.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

/**
 * A displayer for table cells, which darkens every second line and
 * accepts different formats for numbers and dates.
 */

public class BuTableCellRenderer
       extends BuLabel implements TableCellRenderer
{
  private NumberFormat numberFormat_;
  public NumberFormat getNumberFormat() { return numberFormat_; }
  public void setNumberFormat(NumberFormat _f) { numberFormat_=_f; }

  private DateFormat dateFormat_;
  public DateFormat getDateFormat() { return dateFormat_; }
  public void setDateFormat(DateFormat _f) { dateFormat_=_f; }

  public Component getTableCellRendererComponent
    (JTable _table, Object _value, boolean _selected,
     boolean _hasFocus, int _row, int _column) 
  {
    Color  bg=_table.getBackground();
    Color  fg=_table.getForeground();
    int    a =LEFT;
    String t ="";
    Icon   i =null;
    Border bd=UIManager.getBorder("Table.cellBorder");
    Font   ft=_table.getFont();

    if(_selected)
    {
      _selected=false;
      int[] sr=_table.getSelectedRows();
      int[] sc=_table.getColumnModel().getSelectedColumns();
      boolean rsa=_table.getRowSelectionAllowed();
      boolean csa=_table.getColumnSelectionAllowed();
      int j,k;
      for(j=0;j<sr.length;j++)
	for(k=0;k<sc.length;k++)
	if((!rsa||(_row==sr[j]))&&(!csa||(_column==sc[k])))
	  { _selected=true; break; }
    }

    if(_selected)
    {
      bg=_table.getSelectionBackground();
      fg=_table.getSelectionForeground();
    }

    if(_value instanceof Number)
    {
      a=RIGHT;
      if(((Number)_value).doubleValue()<0)
	fg=blueify(fg);
      if(numberFormat_!=null)
	t=numberFormat_.format(((Number)_value).doubleValue());
      else
	t=_value.toString();
    }
    else
    if(_value instanceof Date)
    {
      if(dateFormat_!=null)
	t=dateFormat_.format((Date)_value);
      else
	t=_value.toString();
    }
    else
    if(_value instanceof Boolean)
    {
      if(_selected)
	bg=((Boolean)_value).booleanValue()
	  ? BuLib.getColor(new Color(128,255,128))
	  : BuLib.getColor(new Color(255,128,128));
      else
	bg=((Boolean)_value).booleanValue()
	  ? BuLib.getColor(new Color(192,255,192))
	  : BuLib.getColor(new Color(255,192,192));
    }
    else
    if(_value instanceof BuIcon)
    {
      i=BuResource.resizeIcon((BuIcon)_value,Math.max(16,ft.getSize()));
    }
    else
    if(_value instanceof Icon)
    {
      i=(Icon)_value;
    }
    else
    if(_value==null)
    {
      t="";
    }
    else
    {
      t=_value.toString();
    }
    
    if(_hasFocus)
    {
      bd=UIManager.getBorder("Table.focusCellHighlightBorder");
      if(_table.isCellEditable(_row,_column))
      {
	fg=UIManager.getColor("Table.focusCellForeground");
	bg=UIManager.getColor("Table.focusCellBackground");
      }
    }

    if((_row%2)==1) bg=darken(bg);

    setOpaque(true);
    setBorder(bd);
    setFont(ft);
    setForeground(fg);
    setBackground(bg);
    setHorizontalAlignment(a);
    setText(t);
    setIcon(i);
    return this;
  }

  private static final Color darken(Color _c)
  {
    int r=_c.getRed();
    int g=_c.getGreen();
    int b=_c.getBlue();

    r=r*19/20;
    g=g*19/20;
    b=b*19/20;

    return new Color(r,g,b);
  }

  private static final Color blueify(Color _c)
  {
    int r=_c.getRed();
    int g=_c.getGreen();
    int b=_c.getBlue();

    b+=(255-b)/2;

    return new Color(r,g,b);
  }
}

