/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuTask.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.io.*;
import java.lang.reflect.*;
import javax.swing.*;

/**
 * Simple extension of thread more easily viewable.
 */
public class BuTask
       extends Thread
{
  public BuTask()
  {
    this(null,"Bu task");
  }

  public BuTask(String _name)
  {
    this(null,_name);
  }

  public BuTask(ThreadGroup _group)
  {
    this(_group,null);
  }

  public BuTask(ThreadGroup _group,String _name)
  {
    super(_group,null,_name);
  }

  // Estimate Time

  private long delay_=-1;
  private long begin_=-1;
  private long end_  =-1;

  public long getEstimateDelay() { return delay_; }
  public void setEstimateDelay(long _delay) { delay_=_delay; }

  public long getEstimateEnd()
  {
    if(begin_==-1) throw new RuntimeException("Task not yet started");
    return begin_+delay_;
  }

  public long getEstimateTime()
  {
    // if(begin_==-1) throw new RuntimeException("Task not yet started");
    return Math.max(0,getEstimateEnd()-System.currentTimeMillis());
  }

  public long getRealTime()
  {
    if(begin_==-1) throw new RuntimeException("Task not yet started");
    if(end_  ==-1) throw new RuntimeException("Task not yet ended");
    return end_-begin_;
  }

  private Thread thread_=null;
  private long   bip_   =500;

  // Thread

  public void start()
  {
    BuTaskView tv=getTaskView();
    if(tv!=null) tv.addTask(this);

    if(delay_!=-1)
    {
      begin_=System.currentTimeMillis();
      end_  =-1;
      bip_  =Math.max(250l,delay_/100l);

      thread_=new Thread("Bu task estimate time")
      {
	public void run()
	{
	  long t;
	  int  p=0,q=-1;
	  while((t=System.currentTimeMillis())<begin_+delay_)
	  {
	    p=(int)(100l*(t-begin_)/delay_);
	    if(p!=q) { setProgression(p); q=p; }
	    try { Thread.sleep(bip_); }
	    catch(InterruptedException ex) { }
	  }
	  setProgression(100);
	}
      };
      thread_.setPriority(Math.min(getPriority()+1,Thread.MAX_PRIORITY));
      thread_.start();
    }

    super.start();
  }

  public void run()
  {
    BuTaskView tv=getTaskView();
    if(tv!=null) tv.removeTask(this);

    end_=System.currentTimeMillis();
    super.run();
  }

  // Divers

  public String toString()
  {
    String r=getName();
    int p=getProgression();
    if(p>0) r=""+p+"% "+r;
    return r;
  }

  // Proprits

  private BuTaskView task_view_;
  public BuTaskView getTaskView() { return task_view_; }
  public void setTaskView(BuTaskView _tv) { task_view_=_tv; }

  private int progression_;
  public int getProgression() { return progression_; }
  public void setProgression(int _v)
  {
    if(_v!=progression_)
    {
      int vp=progression_;
      progression_=_v;

      BuTaskView tv=getTaskView();
      if(tv!=null) BuUpdateGUI.repaintLater(tv);
    }
  }
}
