/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuTaskView.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.bu;

import com.memoire.bu.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;

/**
 * A simple list to display running tasks.
 */
public class BuTaskView
       extends BuEmptyList
{
  DefaultListModel model_;

  public BuTaskView()
  {
    super();
    setName("buTASKVIEW");
    model_=new DefaultListModel();
    setModel(model_);
    setEmptyText(BuResource.BU.getString("Aucune tche en cours"));
    setCellRenderer(new XCR());
    // setFont(BuLib.deriveFont("List",Font.PLAIN,-2));
  }

  public void setFont(Font _f)
  {
    super.setFont(getETF());
  }

  public void addTask(BuTask _t)
  {
    model_.addElement(_t);
    revalidate();
    repaint(200);
  }

  public void removeTask(BuTask _t)
  {
    model_.removeElement(_t);
    revalidate();
    repaint(200);
  }

  private final static class Cell extends BuLabel
  {
    private BuTask task_;

    public Cell(BuTask _task)
    {
      super();

      task_=_task;

      String p;
      switch(task_.getPriority())
      {
      case Thread.MIN_PRIORITY:    p="-"; break;
      case Thread.MIN_PRIORITY+1:  p="-"; break;
      case Thread.NORM_PRIORITY-1: p="="; break;
      case Thread.NORM_PRIORITY:   p="="; break;
      case Thread.MAX_PRIORITY:    p="+"; break;
      default:                     p="?"; break;
      }
      if(task_.isInterrupted())	p="~";
      if(!task_.isAlive())      p="!";

      setText(" "+p+" "+task_.toString());
    }

    /*
    public boolean isOpaque()
    {
      return false;
    }
    */

    public final void paintComponent(Graphics _g)
    {
      //super.paintComponent(_g);

      Insets insets=this.getInsets();
      Color fg=UIManager.getColor("ProgressBar.foreground");
      Color bg=UIManager.getColor("ProgressBar.background");
      if(fg==null) fg=this.getForeground();
      if(bg==null) bg=this.getBackground();

      int v=task_.getProgression();
      int vmin=0;
      int vmax=100;
      String s=this.getText();
      int    xs=insets.left;
      int    ys=insets.top+this.getFont().getSize();

      Shape     old=_g.getClip();
      Rectangle rcb=_g.getClipBounds();
      Rectangle r;
      int wb=this.getWidth ()-insets.left-insets.right;
      int hb=this.getHeight()-insets.top-insets.bottom;
      int xb=(v-vmin)*wb/(vmax-vmin);

      r=rcb.intersection(new Rectangle(insets.left+xb,insets.top,wb-xb,hb));
      _g.setClip(r);
      _g.setColor(this.getBackground());
      _g.fillRect(r.x,r.y,r.width,r.height);
      _g.setColor(this.getForeground());
      _g.drawString(s,xs,ys);

      r=rcb.intersection(new Rectangle(insets.left,insets.top,xb,hb));
      _g.setClip(r);
      _g.setColor(fg);
      _g.fillRect(r.x,r.y,r.width,r.height);
      _g.setColor(bg);
      _g.drawString(s,xs+1,ys);

      _g.setClip(old);
    }

    /*
    public void paint(Graphics _g)
    {
      super.paint(_g);

      Dimension size=this.getSize();
      int w=task_.getProgression()*size.width/100;

      Color c=UIManager.getColor("ProgressBar.foreground");
      if(c==null) c=this.getForeground();
      _g.setXORMode(c);
      _g.setColor(this.getBackground());

      int cl=1,cs=1;
      try
      {
	cl=((Integer)UIManager.get("ProgressBar.cellLength")).intValue();
        cs=((Integer)UIManager.get("ProgressBar.cellSpacing")).intValue();
      }
      catch(Exception ex) { }

      if(cl==1)
	_g.fillRect(0,0,w,size.height);
      else
	for(int x=0;x+cl<=w;x+=cl+cs+1)
	  _g.fillRect(x,0,cl+1,size.height);

      if((cs==0)&&(BuLib.jdk()<1.2))
      {
	Rectangle old=_g.getClipBounds();
	_g.setClip(0,0,w,size.height);
	super.paint(_g);
	_g.setClip(old);
	_g.setPaintMode();
      }
      else
      {
	_g.setPaintMode();
	super.paint(_g);
      }
    }
    */
  }

  private final static class XCR implements ListCellRenderer
  {
    public final Component getListCellRendererComponent
    (JList _list, Object _value, int _index, boolean _selected, boolean _cellHasFocus)
    {
      Cell r=new Cell((BuTask)_value);

      r.setFont(_list.getFont());
      r.setBackground(_list.getBackground());
      r.setForeground(_list.getForeground());

      return r;
    }
  }
}
