/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuTerminal.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * A small terminal (mainly for script interpreters).
 */
public class BuTerminal
       extends JPanel
       implements ActionListener
{
  protected BuTerminalDisplay           display_;
  protected BuTerminalDisplay.InStream  input_;
  protected BuTerminalDisplay.OutStream output_;
  protected BuTerminalDisplay.OutStream error_;
  protected BuPanel                     scp_;

  public BuTerminal()
  {
    display_=new BuTerminalDisplay();
    scp_    =null;

    setLayout(new BuBorderLayout());
    add(new JScrollPane(display_),BuBorderLayout.CENTER);
    setShortcuts(new String[0]);
  }

  public BuTerminalDisplay getDisplay()
    { return display_; }

  public void paste(String _s)
  {
    if(input_!=null) input_.add(_s);
  }

  public void setShortcuts(String[] _ba)
  {
    if(scp_==null)
    {
      scp_=new BuPanel();
      scp_.setLayout(new BuVerticalLayout(0,true,false));
      scp_.setBorder(new EmptyBorder(2,2,2,2));
    }
    else scp_.removeAll();

    // String[]  ba=new String[] { "1+2\r",null,"5+6\r","a\tbbbb\tc" };
    JButton[] b =new JButton[11]; // ba.length;

    for(int i=0;i<11;i++)
    {
      b[i]=new BuButton("F"+(i+2))
	{
	  public boolean isFocusTraversable() { return false; }
	};
      b[i].setRequestFocusEnabled(false);
      b[i].setMargin(new Insets(1,1,1,1));
      scp_.add(b[i]);

      if((i>=_ba.length)||(_ba[i]==null)||"".equals(_ba[i]))
	b[i].setEnabled(false);
      else
      {
	Shortcut sc=new Shortcut(this,_ba[i]);
	registerKeyboardAction
	  (sc,
	   KeyStroke.getKeyStroke(KeyEvent.VK_F1+(i+1),0,false),
	   JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
	b[i].addActionListener(sc);
	b[i].setActionCommand(_ba[i]);
	b[i].setToolTipText(_ba[i].replace('\t',' '));
      }
    }

    add(scp_,BuBorderLayout.WEST);
    revalidate();
    scp_.repaint();
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
    // System.err.println("action="+action);
    if(action!=null)
    {
      input_.add(action);
      display_.requestFocus();
    }
  }

  class Shortcut implements ActionListener
  {
    String         action_;
    ActionListener listener_;

    public Shortcut(ActionListener _listener, String _action)
    {
      action_=_action;
      listener_=_listener;
    }

    public void actionPerformed(ActionEvent _evt)
    {
      listener_.actionPerformed(new ActionEvent
				(_evt.getSource(),_evt.getID(),action_));
    }
  }
}
