/**
 * @modification $Date: 2001/12/07 18:28:12 $
 * @statut       unstable
 * @file         BuTerminalDisplay.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * A component to display the content of a terminal.
 */
public class BuTerminalDisplay
       extends JComponent
       implements ActionListener, FocusListener
{
  public final static byte DEFAULT   =-1;
  public final static byte NORMAL    = 0;
  public final static byte INVERSE   = 1;
  public final static byte UNDERLINED= 2;
  public final static byte BOLD      = 4;

  public final static int DELAY=100;

  private char [][]  text_;
  private int  [][]  fgc_;
  private byte [][]  inv_;
  private BuTimer    timer_;
  private boolean    cursor_;
  private int        cpos_;

  private int        nbl_,nbc_;
  private int        wc_,hc_;

  private Repainter repainter_;

  public BuTerminalDisplay()
  { this(80,24); }

  public BuTerminalDisplay(int _nbc,int _nbl)
  {
    nbc_=_nbc;
    nbl_=_nbl;

    setFont(new Font("Monospaced", Font.PLAIN, 10));
    setForeground(new Color(64,255,128));
    setBackground(new Color(0,32,0));

    cursor_=true;
    cpos_=0;
    text_=new char[nbl_][nbc_];
    fgc_ =new int [nbl_][nbc_];
    inv_ =new byte[nbl_][nbc_];
    for(int i=0;i<nbl_;i++)
      for(int j=0;j<nbc_;j++)
      {
	text_[i][j]=' ';
	fgc_ [i][j]=getForeground().getRGB();
	inv_ [i][j]=NORMAL;
      }

    setSize(getPreferredSize());

    addFocusListener(this);
    
    addMouseListener(new MouseAdapter()
      {
	public void mouseClicked(MouseEvent _evt)
	  { requestFocus(); }
      });

    repainter_=new Repainter();
    repainter_.setPriority(Thread.NORM_PRIORITY-1);
    repainter_.start();
  }

  private final void repainter(int _x,int _y,int _w,int _h)
  {
      repainter_.update(_x,_y,_w,_h);
  }

  private final void repainter(int _d,int _x,int _y,int _w,int _h)
  {
      repainter_.update(_x,_y,_w,_h);
  }

  private final void repainter(int _d)
  {
      repainter_.update(0,0,getWidth(),getHeight());
  }

  public void setFont(Font _f)
  {
    super.setFont(_f);
    FontMetrics fm=BuLib.getFontMetrics(this,getFont());
    wc_=fm.charWidth('m');
    hc_=fm.getHeight()+1;
    revalidate();
  }

  public void focusGained(FocusEvent _evt)
  {
    if(timer_==null)
    {
      timer_=new BuTimer(350,this);
      timer_.start();
    }
    repainter(c_to_x(cpos_),l_to_y(nbl_-1),c_to_x(1),l_to_y(1));
  }

  public void focusLost(FocusEvent _evt)
  {
    if(timer_!=null)
    {
      timer_.stop();
      timer_.removeActionListener(this);
      timer_=null;
    }
    repainter(c_to_x(cpos_),l_to_y(nbl_-1),c_to_x(1),l_to_y(1));
  }

  public void actionPerformed(ActionEvent _evt)
  {
    if(_evt.getSource()==timer_)
    {
      cursor_=!cursor_;
      repainter(200,c_to_x(cpos_),l_to_y(nbl_-1),c_to_x(1),l_to_y(1));
    }
    else
    {
      String action=_evt.getActionCommand();
      System.err.println("unknown action: "+action);
    }
  }

  final public int x_to_c(int _x)
    { return _x/wc_; }

  final public int c_to_x(int _c)
    { return _c*wc_; }

  final public int y_to_l(int _y)
    { return _y/hc_; }

  final public int l_to_y(int _l)
    { return _l*hc_; }

  public Dimension getPreferredSize()
    { return new Dimension(c_to_x(nbc_),l_to_y(nbl_)); }

  public boolean isFocusTraversable()
    { return true; }

  public synchronized void paintComponent(Graphics _g)
  {
    _g.setColor(getBackground());
    _g.fillRect(0,0,getWidth(),getHeight());
    _g.setColor(Color.gray);
    _g.drawLine(0,l_to_y(nbl_),c_to_x(nbc_),l_to_y(nbl_));
    _g.drawLine(c_to_x(nbc_),0,c_to_x(nbc_),l_to_y(nbl_));
    _g.setColor(getForeground());

    int imin=0,imax=nbl_;
    int jmin=0,jmax=nbc_;
    Rectangle clip=_g.getClipBounds();
    int h=getFont().getSize();

    if(clip!=null)
    {
      jmin=x_to_c(clip.x);
      jmax=Math.min(nbc_,x_to_c(clip.x+clip.width)+1);
      imin=y_to_l(clip.y);
      imax=Math.min(nbl_,y_to_l(clip.y+clip.height)+1);
    }

    int    x,y;
    String s;
    Color  c;

    for(int i=imin;i<imax;i++)
    {
      y=l_to_y(i)+h;
      for(int j=jmin;j<jmax;j++)
      {
	x=c_to_x(j);
	s=""+text_[i][j];
	c=new Color(fgc_[i][j]);
	_g.setColor(c);

	if((inv_[i][j]&INVERSE)!=0)
	{
	  _g.fillRect(x,y-h,c_to_x(1),l_to_y(1));
	  _g.setColor(getBackground());
	}

	if((inv_[i][j]&BOLD)!=0)
	  _g.setFont(new Font(getFont().getFamily(),
			      Font.BOLD,getFont().getSize()));
	else
	  _g.setFont(getFont());

	_g.drawString(s,x,y);

	if((inv_[i][j]&UNDERLINED)!=0)
	  _g.drawLine(x,y+1,x+c_to_x(1),y+1);
      }
    }

    _g.setColor(Color.yellow);
    x=c_to_x(cpos_)+1;
    y=l_to_y(nbl_-1);
         if(!hasFocus()) _g.drawRect(x,y+2,4,h-2);
    else if(cursor_)     _g.fillRect(x,y+2,5,h-1);
  }

  public synchronized void scroll()
  {
    char[] text0=text_[0];
    int [] fgc0=fgc_[0];
    byte[] inv0=inv_[0];

    for(int i=1;i<nbl_;i++)
    {
	text_[i-1]=text_[i];
	fgc_ [i-1]=fgc_ [i];
	inv_ [i-1]=inv_ [i];
    }
    
    text_[nbl_-1]=text0;
    fgc_ [nbl_-1]=fgc0;
    inv_ [nbl_-1]=inv0;
    
    /*
    for(int i=0;i<nbl_-1;i++)
      for(int j=0;j<nbc_;j++)
      {
	text_[i][j]=text_[i+1][j];
	fgc_ [i][j]=fgc_ [i+1][j];
	inv_ [i][j]=inv_ [i+1][j];
      }
    */

    int color=getForeground().getRGB();

    for(int j=0;j<nbc_;j++)
    {
      text_[nbl_-1][j]=' ';
      fgc_ [nbl_-1][j]=color;
      inv_ [nbl_-1][j]=(byte)0;
    }

    repainter(DELAY);
  }

  public synchronized void insert(char _ch,int _color, byte _style)
  {
    switch(_ch)
    {
      case '\n':
	cpos_=nbc_;
	break;
      case '\007':
	getToolkit().beep();
	break;
      case '\t':
	do
	{
	  text_[nbl_-1][cpos_]=' ';
	  fgc_ [nbl_-1][cpos_]=_color;
	  inv_ [nbl_-1][cpos_]=_style;
	  repainter(DELAY,c_to_x(cpos_),l_to_y(nbl_-1),c_to_x(2),l_to_y(1));
	  cpos_++;
	} while(cpos_%8!=0);
	break;
      default:
	text_[nbl_-1][cpos_]=_ch;
	fgc_ [nbl_-1][cpos_]=_color;
	inv_ [nbl_-1][cpos_]=_style;
	repainter(DELAY,c_to_x(cpos_),l_to_y(nbl_-1),c_to_x(2),l_to_y(1));
	cpos_++;
	break;
    }

    if(cpos_>=nbc_)
      { scroll(); cpos_=0; }
  }

  public synchronized void clear_char()
  {
    int color=getForeground().getRGB();

    text_[nbl_-1][cpos_]=' ';
    fgc_ [nbl_-1][cpos_]=color;
    inv_ [nbl_-1][cpos_]=(byte)0;
    repainter(DELAY,c_to_x(cpos_),l_to_y(nbl_-1),c_to_x(2),l_to_y(1));
  }

  public synchronized void clear_eol()
  {
    int color=getForeground().getRGB();

    for(int i=cpos_;i<nbc_;i++)
    {
      text_[nbl_-1][i]=' ';
      fgc_ [nbl_-1][i]=color;
      inv_ [nbl_-1][i]=(byte)0;
    }

    repainter(DELAY,c_to_x(cpos_),l_to_y(nbl_-1),c_to_x(nbc_-cpos_),l_to_y(1));
  }

  public synchronized void clear_screen()
  {
    int color=getForeground().getRGB();

    for(int j=0;j<nbl_;j++)
      for(int i=0;i<nbc_;i++)
      {
	text_[j][i]=' ';
	fgc_ [j][i]=color;
	inv_ [j][i]=(byte)0;
      }

    cpos_=0;
    repainter(DELAY);
  }

  public BuTerminalKey createKey
    (String _word, int _color)
    { return createKey(_word,_color,NORMAL,BuTerminalKey.WHOLE); }

  public BuTerminalKey createKey
    (String _word, int _color, byte _style)
    { return createKey(_word,_color,_style,BuTerminalKey.WHOLE); }

  public BuTerminalKey createKey
    (String _word, int _color, byte _style, String _til)
    { return new BuTerminalKey(_word,_color,_style,_til); }

  public OutStream createOutStream()
    { return createOutStream(getForeground()); }

  public InStream createInStream()
    { return createInStream(getForeground(),true); }

  public OutStream createOutStream(Color _color)
    { return createOutStream(_color,NORMAL); }

  public InStream createInStream(Color _color,boolean _editable)
    { return createInStream(_color,NORMAL,_editable); }

  public OutStream createOutStream(Color _color,byte _style)
    { return new OutStream(_color.getRGB(),_style); }

  public InStream createInStream(Color _color,byte _style,boolean _editable)
  {
    InStream r=null;
    if(_editable) r=new EditableInStream(_color.getRGB(),_style);
    else          r=new SimpleInStream(_color.getRGB(),_style);
    return r;
  }

  public final class OutStream extends OutputStream
  {
    private int  color_;
    private byte style_;
    private int  left_,right_;

    private BuTerminalKey[] keys_;

    /*
    private String[] words_;
    private int[]    colors_;
    private byte[]   styles_;
    */

    public OutStream(int _color,byte _style)
    {
      color_=_color;
      style_=_style;
      left_ =0;
      right_=0;
      keys_ =null;
    }

    public void setKeys(BuTerminalKey[] _keys)
      { keys_=_keys; }

    public void setLeftMargin(int _left)
      { left_=_left; }

    public void setRightMargin(int _right)
      { right_=_right; }

    private void write(char _char,int _color,byte _style)
    {
      while(cpos_<left_)
	insert(' ',_color,NORMAL);
      if(_color==DEFAULT) _color=color_;
      if(_style==DEFAULT) _style=style_;
      insert(_char,_color,(byte)(_style|style_));
      if(cpos_>nbc_-right_)
	{ scroll(); cpos_=0; }
    }

    public void write(int _b) throws IOException
    {
      write((char)_b,color_,style_);
    }

    public void write(byte _b[]) throws IOException
    {
      String text=new String(_b);
      int    l   =text.length();

      if(keys_==null)
      {
	for(int i=0;i<l;i++)
	  write(text.charAt(i));
      }
      else
      {
	int [] color=new int [l];
	byte[] style=new byte[l];
	for(int i=0;i<l;i++)
	{
	  color[i]=color_;
	  style[i]=style_;
	}

	for(int j=0;j<keys_.length;j++)
	{
	  BuTerminalKey k=keys_[j];
	  int p=0;
	  while((p=text.indexOf(k.word_,p))>=0)
	  {
	    int qmin,qmax;

	    if(BuTerminalKey.WHOLE.equals(k.til_))
	      { qmin=p; qmax=p+k.word_.length(); }
	    else
	    if(BuTerminalKey.LEFT.equals(k.til_))
	      { qmin=0; qmax=p+k.word_.length(); }
	    else
	    if(BuTerminalKey.RIGHT.equals(k.til_))
	      { qmin=p; qmax=l-p; }
	    else
	      { qmin=p; qmax=Math.max(l,1+text.indexOf(k.til_)); }

	    for(int q=qmin; q<qmax; q++)
	      { color[q]=k.color_; style[q]=k.style_; }
	    p++;
	  }
	}

	for(int i=0;i<l;i++)
	  write(text.charAt(i),color[i],style[i]);
      }
    }

    public void write(byte _b[],int _d,int _l) throws IOException
    {
      byte[] b=new byte[_l];
      System.arraycopy(_b,_d,b,0,_l);
      write(b);
    }
  }

  public abstract class InStream extends InputStream
  {
    public final void add(String _s)
    {
      for(int i=0;i<_s.length();i++)
	add(_s.charAt(i));
    }

    public abstract void add(char _clef);

    public void setKeys(BuTerminalKey[] _keys)    { }
    public void setFastCompletionChar(char _char) { }
  }

  public final static int BUFFER_SIZE=128;

  public final class SimpleInStream extends InStream
  {
    char[] buf_   =new char[BUFFER_SIZE];
    int    bbegin_=0;
    int    bend_  =0;
    Object lock_  =new Object();
    
    private int         color_;
    private byte        style_;
    private KeyListener kl_;

    protected SimpleInStream(int _color,byte _style)
    {
      color_ =_color;
      style_ =_style;

      kl_=new KeyAdapter()
	{
	  public void keyTyped(KeyEvent _evt)
	  {
	    char clef=_evt.getKeyChar();

	         if(clef=='\r'  ) clef='\n';
            else if(clef=='\007') ;
	    else if(clef<=27    ) clef='\000';
	    if(clef!='\000') add(clef);
	    _evt.consume();
	  }
	};
      addKeyListener(kl_);
    }

    public int read()
    {
      int i=-1;

      try
      {
	if(bbegin_==bend_)
	  synchronized(lock_)
	  {
	    lock_.wait();
	  }

	i=(int)buf_[bbegin_];
	bbegin_++;
	if(bbegin_==BUFFER_SIZE) bbegin_=0;
      }
      catch (Exception ex)
	{ System.err.println("BuTerminalDisplay.read: "+ex); }

      return i;
    }

    public void add(char _clef)
    {
      insert(_clef,color_,style_);

      int     old_bend=bend_;
      boolean unlock  =(bend_==bbegin_);

      buf_[bend_]=_clef;
      bend_++;
      if(bend_==BUFFER_SIZE) bend_=0;

      if(bend_==bbegin_)
      {
	System.err.println("BuTerminal overflow");
	bend_=old_bend;
      }

      if(unlock)
	synchronized(lock_)
	{
	  lock_.notifyAll();
	}
    }
    
    public int available()
    {
      int r=bend_-bbegin_;
      if(r<0) r=(BUFFER_SIZE-bbegin_)+bend_;
      return r;
    }

    public void close()            { }
    public void mark(int _p)       { }
    public boolean markSupported() { return false; }
    public void reset()            { }
    public int read(byte[] _b)     { return read(_b,0,_b.length); }
  
    public int read(byte[] _b, int _d, int _l)
    {
      if(_l==0) return 0;

      /*
	int a=available();
	if(a>_l) a=_l;
	for(int i=_d;i<_d+a;i++)
	_b[i]=(byte)read();
	return a;
	*/

      _b[_d]=(byte)read();
      return 1;
    }

    public long skip(long _l)
    {
      System.err.println("skip");

      long i;
      for(i=0; i<_l; i++)
	if(read()==-1) break;
      return i;
    }
  }

  public final static int HISTORY_SIZE=10;

  public final class EditableInStream extends InStream
  {
    private Object lock_=new Object();
    
    private BuTerminalKey[]   keys_;
    private char              fcc_;
    private int               color_;
    private byte              style_;
    
    private String            buf_;
    private String[]          history_;
    private int               previous_;
    private KeyListener       kl_;

    protected void addHistory(String _s)
    {
      for(int i=HISTORY_SIZE-1;i>0;i--)
	history_[i]=history_[i-1];
      history_[0]=_s;
    }

    protected String previous()
    {
      return history_[0];
    }

    protected EditableInStream(int _color,byte _style)
    {
      buf_     ="";
      history_ =new String[HISTORY_SIZE];
      previous_=0;

      color_=_color;
      style_=_style;
      keys_ =null;
      fcc_  ='';

      for(int i=0;i<HISTORY_SIZE;i++)
	history_[i]="";

      kl_=new KeyAdapter()
	{
	  public void keyTyped(KeyEvent _evt)
	  {
	    traite(_evt.getKeyChar());
	    _evt.consume();
	  }

	  public void keyPressed(KeyEvent _evt)
	  {
	    char clef=0;
	    int  code=_evt.getKeyCode();

	    if(code==KeyEvent.VK_LEFT     ) clef=2;  // B
	    if(code==KeyEvent.VK_RIGHT    ) clef=6;  // F
	    if(code==KeyEvent.VK_UP       ) clef=16; // P
	    if(code==KeyEvent.VK_DOWN     ) clef=14; // N
	    if(code==KeyEvent.VK_PAGE_UP  ) clef=16; // P
	    if(code==KeyEvent.VK_PAGE_DOWN) clef=14; // N
	    if(code==KeyEvent.VK_HOME     ) clef=1;  // A
	    if(code==KeyEvent.VK_END      ) clef=5;  // E

	    if(clef!=0)
	    {
	      traite(clef);
	      _evt.consume();
	    }
	  }

	  private void traite(char _clef)
	  {
	    // System.err.print(""+(int)_clef);

	    if((_clef==1)||(_clef==27)) // A
	    {
	      synchronized(buf_)
	      {
		cpos_=(cpos_+nbc_-buf_.length())%nbc_;
		clear_eol();
		buf_="";
	      }
	      _clef=0;
	      previous_=0;
	    }

	    if(_clef==2) // B
	    {
	      if(buf_.length()>0)
	      {
		synchronized(buf_)
		{
		  cpos_=(cpos_+nbc_-1)%nbc_;
		  clear_char();
		  buf_=buf_.substring(0,buf_.length()-1);
		}
	      }
	      _clef=0;
	    }

	    if((_clef==3)||(_clef==fcc_)) // C
	    {
	      complete();
	      _clef=0;
	    }

	    if(_clef==4) // D
	      _clef=0;

	    if(_clef==5) // E
	    {
	      if(buf_.length()<previous().length())
		add(previous().substring(buf_.length(),previous().length()));
	      _clef=0;
	    }

	    if(_clef==6) // F
	    {
	      if(buf_.length()<previous().length())
		add(previous().substring(buf_.length(),buf_.length()+1));
	      _clef=0;
	    }

	    if(_clef==7) // G
	    {
	      getToolkit().beep();
	      _clef=0;
	    }

	    if(_clef==8) // H 
	    {
	      if(buf_.length()>0)
	      {
		synchronized(buf_)
		{
		  cpos_=(cpos_+nbc_-1)%nbc_;
		  clear_char();
		  buf_=buf_.substring(0,buf_.length()-1);
		}
	      }
	      _clef=0;
	    }

	    // never 9 I \t

	    if(_clef==11) // K
	      _clef=0;

	    if(_clef==12) // L
	      _clef=0;

	    if(_clef==13) // M \r-->\n
	      _clef=10;

	    if(_clef==10) // J
	    {
	      /*
	      if(buf_.startsWith("!ls"))
	      {
		OutStream out=createOutStream(getForeground().darker());
		BuFileFilter filtre=null;
		String ff=buf_.substring(3).trim();
		if(ff.startsWith("*.")) ff=ff.substring(2);
		if(!"".equals(ff)) filtre=new BuFileFilter(ff);
		try
		{
		  File dir=new File(System.getProperty("user.dir"));
		  String[] files=dir.list(filtre);
		  for(int i=0;i<files.length;i++)
		  {
		    out.write('\n');
		    out.write(files[i].getBytes());
		  }
		}
		catch(Exception ex)
		{
		  getToolkit().beep();
		  System.err.println(ex.getMessage());
		}
		buf_="";
	      }
	      */
	      /*
	      else
	      if(buf_.startsWith("!more"))
	      {
		OutStream out=createOutStream(getForeground().darker());
		String f=buf_.substring(6);
		try
		{
		  Properties pp=new Properties();
		  pp.load(new FileInputStream("java.stx"));
		  BuTerminalKey[] keys=new BuTerminalKey[pp.size()];
		  int i=0;
		  for(Enumeration e=pp.keys(); e.hasMoreElements(); )
		  {
		    String k=(String)e.nextElement();
		    try
		    {
		      String v    =(String)pp.get(k);
		      int    comma=v.indexOf(',');
		      int    color=Integer.parseInt(v.substring(0,comma),16);
		      String til  =v.substring(comma+1);
		      keys[i]=createKey(k,color,NORMAL,til);
		    }
		    catch(Exception ex)
		    {
		      keys[i]=createKey(k,Color.white.getRGB(),
					NORMAL,BuTerminalKey.WHOLE);
		    }
		    i++;
		  }
		  out.setKeys(keys);
		}
		catch(Exception ex)
		{
		  getToolkit().beep();
		  System.err.println(ex.getMessage());
		  return;
		}
		try
		{
		  LineNumberReader lin=new LineNumberReader(new FileReader(f));
		  while(true)
		  {
		    String l=lin.readLine();
		    if(l==null) break;
		    out.write(l.getBytes());
		    out.write('\n');
		  }
		}
		catch(Exception ex)
		{
		  getToolkit().beep();
		  System.err.println(ex.getMessage());
		}
		buf_="";
	      }
	      */
	    }

	    if(_clef==15) // O
	      _clef=0;

	    if(_clef==16) // P
	    {
	      if(previous_<0) previous_=0;
	      if((previous_+1>=HISTORY_SIZE)||("".equals(history_[previous_])))
		getToolkit().beep();
	      else
	      {
		if(buf_.equals(history_[previous_]))
		  previous_++;
		synchronized(buf_)
		{
		  cpos_=(cpos_+nbc_-buf_.length())%nbc_;
		  clear_eol();
		  buf_="";
		}
		add(history_[previous_]);
		previous_++;
	      }
	      _clef=0;
	    }
	    
	    if(_clef==14) // N
	    {
	      if(previous_<=0)
		getToolkit().beep();
	      else
	      {
		previous_--;
		if((previous_>=0)&&(buf_.equals(history_[previous_])))
		  previous_--;
		synchronized(buf_)
		{
		  cpos_=(cpos_+nbc_-buf_.length())%nbc_;
		  clear_eol();
		  buf_="";
		}
		if(previous_>=0) add(history_[previous_]);
		else             previous_=0;
	      }
	      _clef=0;
	    }
	    
	    if((_clef>=17)&&(_clef<=26)) // Q-Z
	      _clef=0;

	    if(_clef==127) // Suppr
	      _clef=0;

	    // System.err.print("("+(int)_clef+")");
	    if(_clef!=0) add(_clef);
	  }
	};
      addKeyListener(kl_);
    }

    public void setKeys(BuTerminalKey[] _keys)
    {
      keys_=_keys;
    }

    public void setFastCompletionChar(char _char)
    {
      fcc_=_char;
    }

    private void complete()
    {
      String w;

      synchronized(buf_)
      {
	int i=buf_.length()-1;
	while((i>=0)&&Character.isJavaIdentifierPart(buf_.charAt(i))) i--;
	i++;
	w=buf_.substring(i);
      }

      boolean beep=true;

      if((keys_!=null)&&!"".equals(w))
      {
	int n=0;
	int f=-1;

	for(int j=0;j<keys_.length;j++)
	  if(keys_[j].word_.startsWith(w))
	    { f=j; n++; }

	if(n==1)
	{
	  if(keys_[f].word_.equals(w)) add(' ');
	  else add(keys_[f].word_.substring(w.length()));
	  beep=false;
	}
      }

      if(beep) getToolkit().beep();
    }

    private boolean eol()
    {
      boolean r=false;
      synchronized(buf_)
      {
	if(buf_.length()!=0)
	  r=(buf_.charAt(buf_.length()-1)=='\n');
      }
      return r;
    }

    public int read()
    {
      int i=-1;

      if(eol())
      {
	try
	{
	  if(buf_.length()==0)
	    synchronized(lock_)
	    {
	      lock_.wait();
	    }

	  synchronized (buf_)
	  {
	    i=(int)buf_.charAt(0);
	    buf_=buf_.substring(1,buf_.length());
	  }
	}
	catch (Exception e)
	{ System.err.println("\n### "+e+" ["+buf_+" ("+buf_.length()+") ]"); }
      }

      return i;
    }

    public void add(char _clef)
    {
      insert(_clef,color_,style_);

      synchronized(lock_)
      {
	synchronized (buf_)
	{
	  if(_clef=='\n')
	  {
	    if(!"".equals(buf_)) addHistory(buf_);
	    previous_=-1;
	  }
	  buf_+=_clef;
	}
	lock_.notifyAll();
      }
    }
    
    public int available()
    {
      if(!eol()) return 0;
      return buf_.length();
    }

    public void close()            { }
    public void mark(int _p)       { }
    public boolean markSupported() { return false; }
    public void reset()            { }
    public int read(byte[] _b)     { return read(_b,0,_b.length); }
  
    public int read(byte[] _b, int _d, int _l)
    {
      if(_l==0) return 0;
      if(!eol()) return 0;

      _b[_d]=(byte)read();
      return 1;
    }

    public long skip(long _l)
    {
      System.err.println("skip");

      long i;
      for(i=0; i<_l; i++)
	if(read()==-1) break;
      return i;
    }
  }

  class Repainter extends Thread
  {
      Rectangle r=null;
      Object lock=new Object();

      public void update(int _x,int _y,int _w, int _h)
      {
	  synchronized (lock)
	  {
	      if(r==null)  r=new Rectangle(_x,_y,_w,_h);
	      else r=r.union(new Rectangle(_x,_y,_w,_h));
	  }
      }

      public void run()
      {
	  while(true)
	  {
	      try { Thread.sleep(DELAY); }
	      catch(InterruptedException ex) { }
	      if((r!=null)&&isShowing())
	      {
		  synchronized (lock)
		  {
		      //repaint(DELAY,r.x,r.y,r.width,r.height);
		      repaint(0,r.x,r.y,r.width,r.height);
		      r=null;
		  }
	      }
	  }
	  //super.run();
      }
  }
}
