/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuTextArea.java
 * @version      0.36
 * @author       Romain Guy
 * @email        guy.romain@bigfoot.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


/*
 * 19:13:20 13/06/99
 *
 * JextTextAreaLite.java - A 'lite' extended JTextArea
 * Copyright (C) 1998-1999 Romain Guy
 * Especially designed for Nicolas Warneck
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * A few changes and adaptations 25/5/2000 (C) Guillaume Desnoix
 */

import java.io.*;
import java.awt.*;
// import gnu.regexp.*;
import java.util.zip.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.undo.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * This component extends a JTextArea, providing methods to
 * undo/redo, find/replace and read/save.
 *
 * This component extends a JTextArea. This component provides
 * its own methods to read and save files (even to zip them).
 * @author  Romain Guy
 * @email   guy.romain@bigfoot.com
 * @version 1.1
 */
public class BuTextArea extends JTextArea
       implements UndoableEditListener, DocumentListener,
                  BuTextComponentInterface
{
  /** This constant defines an open dialog box. */
  public static final int OPEN = 0;
  /** This constant defines a save dialog box. */
  public static final int SAVE = 1;

  // Private declaration

  private RE regexp;
  private CompoundEdit compoundEdit;
  private String fontName, currentFile;
  private int anchor, fontSize, fontStyle;
  private boolean dirty, newText, operation;
  private UndoManager undo = new UndoManager();

  /** This constant defines the size of the buffer used to read files */
  private static final int BUFFER_SIZE = 32768;

  /**
   * The constructor add the necessary listeners, set some stuffs
   * (caret color, borers, fonts...).
   */

  public BuTextArea()
  {
    super(15,40);

    // setBorder(null);
    // setCaretColor(Color.red);

    getDocument().addDocumentListener(this);
    getDocument().addUndoableEditListener(this);

    Font defaultFont = getFont(); // new Font("Monospaced", Font.PLAIN, 12);
    fontName = defaultFont.getName();
    fontSize = defaultFont.getSize();
    fontStyle = defaultFont.getStyle();
    // setFont(defaultFont);
  }

  // Anti-aliasing

  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  public void select()
  {
    select(0,getLength());
  }

  public void duplicate()
  {
    int s=getSelectionStart();
    int e=getSelectionEnd();
    if(s<e)
    {
      copy();
      setCaretPosition(e);
      paste();
      setSelectionEnd(e+(e-s));
      setSelectionStart(e);
    }
  }

  public void go(int _line)
  {
    Element map =getDocument().getDefaultRootElement();
    Element line=map.getElement(_line);
    select(line.getStartOffset(),line.getEndOffset());
  }

  /**
   * Display a file chooser dialog box.
   * @param owner <code>Component</code> which 'owns' the dialog
   * @param mode Can be either <code>LOAD</code> or <code>SAVE</code>
   * @return The path to selected file, null otherwise
   */

  public static String chooseFile(Component owner, int mode)
  {
    BuFileChooser chooser = new BuFileChooser();
    if (mode == OPEN)
      chooser.setDialogType(BuFileChooser.OPEN_DIALOG);
    else if (mode == SAVE)
      chooser.setDialogType(BuFileChooser.SAVE_DIALOG);
    chooser.setFileSelectionMode(BuFileChooser.FILES_ONLY);
    chooser.setFileHidingEnabled(true);
    if (chooser.showDialog(owner, null) == BuFileChooser.APPROVE_OPTION)
      return chooser.getSelectedFile().getAbsolutePath();
    return null;
  }

  /**
   * Display a sample message in a dialog box.
   * @param message The message to display
   */

  public static void showMessage(String message)
  {
    JOptionPane.showMessageDialog(null, message, "Message", JOptionPane.INFORMATION_MESSAGE);
  }

  /**
   * Display an error message in a dialog box.
   * @param message The message to display
   */

  public static void showError(String message)
  {
    JOptionPane.showMessageDialog(null,message,"Error",
				  JOptionPane.ERROR_MESSAGE);
  }
  /**
   * Return current font's name
   */

  public String getFontName()
  {
    return fontName;
  }

  /**
   * Return current font's size
   */

  public int getFontSize()
  {
    return fontSize;
  }

  /**
   * Return current font's style (bold, italic...)
   */

  public int getFontStyle()
  {
    return fontStyle;
  }

  /**
   * Set the font which has to be used.
   * @param name The name of the font
   */

  public void setFontName(String name)
  {
    fontName = name;
    changeFont();
  }

  /**
   * Set the size of the font.
   * @param size The new font's size
   */

  public void setFontSize(int size)
  {
    fontSize = size;
    changeFont();
    repaint();
  }

  /**
   * Set the style of the font.
   * @param style The new style to apply
   */

  public void setFontStyle(int style)
  {
    fontStyle = style;
    changeFont();
    repaint();
  }

  /**
   * Set the new font.
   */

  private void changeFont()
  {
    setFont(new Font(fontName, fontStyle, fontSize));
  }

  /**
   * Return the full path of the opened file.
   */

  public String getCurrentFile()
  {
    return currentFile;
  }

  /**
   * When an operation has began, setChanged() cannot be called.
   * This is very important when we need to insert or remove some
   * parts of the texte without turning on the 'to_be_saved' flag.
   */

  public void beginOperation()
  {
    operation = true;
  }

  /**
   * Calling this will allow the DocumentListener to use setChanged().
   */

  public void endOperation()
  {
    operation = false;
  }

  /**
   * Return true if we can use the setChanged() method,
   * false otherwise.
   */

  public boolean getOperation()
  {
    return operation;
  }

  /**
   * Set a new file. We first ask the user if he'd like to save its
   * changes (if some have been made).
   */

  public void newFile()
  {
    beginOperation();                            // we don't want to see a 'modified' message
    if (isDirty() && !isEmpty())
    {
      int response = JOptionPane.showConfirmDialog(null, "Do you want to save your changes ?", "Save", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
      switch (response)
      {
        case 0:
          saveContent();
          break;
        case 1:
          break;
        case 2:
          endOperation();
          return;
        default:
          endOperation();
          return;
      }
    }
    // we have to create a new document, so we remove
    // old listeners and create new ones below
    getDocument().removeUndoableEditListener(this);
    getDocument().removeDocumentListener(this);
    clean();
    newText = true;
    currentFile = null;
    setEditable(true);
    setText("");
    setAnchor(0);
    discard();
    getDocument().addUndoableEditListener(this);
    getDocument().addDocumentListener(this);
    endOperation();
  }

  /**
   * This is just to reduce code size of other classes.
   * @param off The line index
   * @return The offset in the text where the line begins
   */

  public int getLineStartOffset(int off)
  {
    return getDocument().getDefaultRootElement().getElement(off).getStartOffset();
  }

  /**
   * Called to save current content in specified zip file.
   * Call zip(String file) but asks user for overwriting if
   * file already exists.
   */

  public void zipContent()
  {
    if (getText().equals("")) return;
    String zipFile = chooseFile(this, SAVE);
    if (zipFile != null)
    {
      if (!zipFile.endsWith(".zip")) zipFile += ".zip";
      if (!(new File(zipFile)).exists())
        zip(zipFile);
      else
      {
        int response = JOptionPane.showConfirmDialog(null, "File already exists, overwrite it ?", "Save", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
        switch (response)
        {
          case 0:
            zip(zipFile);
            break;
          case 1:
            break;
          default:
            return;
        }
      }
    }
  }

  /**
   * Zip text area content into specified file.
   * @param zipFile The file name where to zip the text
   */

  public void zip(String zipFile)
  {
    try
    {
      ZipOutputStream out = new ZipOutputStream(new FileOutputStream(zipFile));
      out.putNextEntry(new ZipEntry((new File(currentFile)).getName()));
      // we ensure we use system's carriage return char
      String newline = System.getProperty("line.separator");
      Element map = getDocument().getDefaultRootElement();
      // we zip the text line by line
      for (int i = 0; i < map.getElementCount(); i++)
      {
        Element line = map.getElement(i);
        int start = line.getStartOffset();
        byte[] buf = (getText(start, line.getEndOffset() - start - 1) + newline).getBytes();
        out.write(buf, 0, buf.length);
      }
      out.closeEntry();
      out.close();
    } catch(IOException ioe) {
      showError("Error has occured while ziping");
    } catch(BadLocationException ble) {
      showError("Error has occured while ziping");
    }
  }

  /**
   * Called to save this component's content.
   * Call save(String file) but let the user choosing a file name.
   * In the case the user choosed an existing file, we ask him if
   * he really wants to overwrite it.
   */

  public void saveContent()
  {
    String fileToSave = chooseFile(this, SAVE);
    if (fileToSave != null)
    {
      if (!(new File(fileToSave)).exists())
        save(fileToSave);
      else
      {
        int response = JOptionPane.showConfirmDialog(null, "File already exists, overwrite it ?", "Save", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
        switch (response)
        {
          case 0:
            save(fileToSave);
            break;
          case 1:
            break;
          default:
            return;
        }
      }
    }
  }

  /**
   * Store the text in a specified file.
   * @param file The file in wich we'll write the text
   */

  public void save(String file)
  {
    try
    {
      OutputStream outs = new FileOutputStream(new File(file));
      BufferedWriter out = new BufferedWriter(new OutputStreamWriter(outs), BUFFER_SIZE);
      // we ensure we use system's carriage return char
      String newline = System.getProperty("line.separator");
      Element map = getDocument().getDefaultRootElement();
      // we write the file line by line
      for (int i = 0; i < map.getElementCount(); i++)
      {
        Element line = map.getElement(i);
        int start = line.getStartOffset();
        out.write(getText(start, line.getEndOffset() - start - 1));
        out.write(newline);
      }
      out.close();
      outs.close();
      if (!file.equals(currentFile)) currentFile = file;
    } catch (IOException ioe) {
      showError("Error has occured while saving");
    } catch (BadLocationException ble) {
      showError("Error has occured while saving");
    }
  }

  /**
   * Called to load a new file in the text area.
   * Determines which line separator (\n, \r\n...) are used in the
   * file to open. Convert'em into Swing line separator (\n).
   * @param path The path of the file to be loaded
   */

  public void open(String path)
  {
    beginOperation();
    // we do the same thing as in newFile() for the listeners
    getDocument().removeUndoableEditListener(this);
    getDocument().removeDocumentListener(this);
    clean();
    discard();
    InputStream is;
    StringBuffer buffer = new StringBuffer();
    try
    {
      File toLoad = new File(path);
      // we check if the file is read only or not
      if (!toLoad.canWrite())
        setEditable(false);
      else if (!isEditable())
        setEditable(true);
      is = new FileInputStream(toLoad);
      InputStreamReader in = new InputStreamReader(is);
      char[] buf = new char[BUFFER_SIZE];
      int len;
      int lineCount = 0;
      boolean CRLF = false;
      boolean CROnly = false;
      boolean lastWasCR = false;

      // we read the file until the end of it (amazing, hu ?)
      while ((len = in.read(buf, 0, buf.length)) != -1)
      {
        int lastLine = 0;
        for (int i = 0; i < len; i++)
        {
          switch(buf[i])
          {
            // and we convert system's carriage return char into \n
            case '\r':
              if (lastWasCR)
              {
                CROnly = true;
                CRLF = false;
              } else
                lastWasCR = true;
              buffer.append(buf, lastLine, i - lastLine);
              buffer.append('\n');
              lastLine = i + 1;
              break;
            case '\n':
              if (lastWasCR)
              {
                CROnly = false;
                CRLF = true;
                lastWasCR = false;
                lastLine = i + 1;
              } else {
                CROnly = false;
                CRLF = false;
                buffer.append(buf, lastLine, i - lastLine);
                buffer.append('\n');
                lastLine = i + 1;
              }
              break;
            default:
              if (lastWasCR)
              {
                CROnly = true;
                CRLF = false;
                lastWasCR = false;
              }
              break;
          }
        }
        buffer.append(buf, lastLine, len - lastLine);
      }
      in.close();
      if (buffer.length() != 0 && buffer.charAt(buffer.length() - 1) == '\n')
        buffer.setLength(buffer.length() - 1);

      // we clear the area
      getDocument().remove(0, getLength());
      // we put the text in it
      getDocument().insertString(0, buffer.toString(), null);
      currentFile = path;
      setCaretPosition(0);
      newText = false;
      getDocument().addUndoableEditListener(this);
      getDocument().addDocumentListener(this);
    } catch(BadLocationException bl) {
      bl.printStackTrace();
    } catch(FileNotFoundException fnf) {
      showError(path + " not found !");
    } catch(IOException io) {
      showError(io.toString());
    }
    endOperation();
  }

  /**
   * Return true if current text is new, false otherwise.
   */

  public boolean isNew()
  {
    return newText;
  }

  /**
   * Return true if area is empty, false otherwise.
   */

  public boolean isEmpty()
  {
    if (getLength() == 0)
      return true;
    else
      return false;
  }


  /**
   * Return true if area content has changed, false otherwise.
   */

  public boolean isDirty()
  {
    return dirty;
  }

  /**
   * Called when the content of the area has changed.
   */

  public void setDirty()
  {
    dirty = true;
  }

  /**
   * Called after having saved or created a new document to ensure
   * the content isn't 'dirty'.
   */

  public void clean()
  {
    dirty = false;
  }

  /**
   * Discard all edits contained in the UndoManager.
   */

  public void discard()
  {
    undo.discardAllEdits();
  }

  /**
   * Useful for the GUI.
   */
  public UndoManager getUndo()
  {
    return undo;
  }

  /**
   * undo the last operation
   */
  public void undo()
  {
    // System.err.println("==== UNDO");
    if(undo.canUndo()) undo.undo();
  }

  /**
   * redo the last operation
   */
  public void redo()
  {
    // System.err.println("==== REDO");
    if(undo.canRedo()) undo.redo();
  }

  /**
   * Return the anchor position.
   */

  public int getAnchor()
  {
    return anchor;
  }

  /**
   * Set the anchor postion.
   * @param offset The new anchor's position
   */

  public void setAnchor(int offset)
  {
    anchor = offset;
  }

  /**
   * Return the lentgh of the text in the area.
   */

  public int getLength()
  {
    return getDocument().getLength();
  }

  /**
   * Used for ReplaceAll.
   * This merges all text changes made between the beginCompoundEdit()
   * and the endCompoundEdit() calls into only one undo event.
   */

  public void beginCompoundEdit()
  {
    if (compoundEdit == null)
      compoundEdit = new CompoundEdit();
  }

  /**
   * See beginCompoundEdit().
   */

  public void endCompoundEdit()
  {
    if (compoundEdit != null)
    {
      compoundEdit.end();
      if (compoundEdit.canUndo())
        undo.addEdit(compoundEdit);
      compoundEdit = null;
    }
  }

  /**
   * Return the result of a string search.
   * @param searchStr The string to be found
   * @param start The search's start offset
   * @param ignoreCase Set to true, we'll ignore the text case
   * @return True if <code>searchStr</code> has been found, false otherwise
   */

  public boolean find(String searchStr, int start, boolean ignoreCase)
  {
    try
    {
      if (searchStr.equals("") || searchStr == null) return false;
      regexp = new RE(searchStr, (ignoreCase == true ? RE.REG_ICASE : 0) | RE.REG_MULTILINE, RESyntax.RE_SYNTAX_PERL5);
      if (regexp == null)
      {
        getToolkit().beep();
        return false;
      }
      String text = getText(start, getLength() - start);
      REMatch match = regexp.getMatch(text);
      if (match != null)
      {
        select(start + match.getStartIndex(), start + match.getEndIndex());
        return true;
      }
    } catch(Exception e) { }
    return false;
  }

  public boolean replace(String searchStr, String replaceStr,
			 int start, int end, boolean ignoreCase)
  {
    boolean r=false;
    try { r=replaceAll(searchStr,replaceStr,start,end,ignoreCase); }
    catch(Exception ex) { }
    return r;
  }

  /**
   * Return the result of a string replace.
   * @param searchStr The string to be found
   * @param replaceStr The string which will replace <code>searchStr</code>
   * @param start The search's start offset
   * @param end The search's end offset
   * @param ignoreCase Set to true, we'll ignore the text case
   * @return True if the replace has been successfully done, false otherwise
   */

  public boolean replaceAll(String searchStr, String replaceStr, int start, int end, boolean ignoreCase) throws REException
  {
    boolean found = false;
    beginCompoundEdit();
    try
    {
      if (searchStr.equals("") || searchStr == null) return false;
      regexp = new RE(searchStr, (ignoreCase == true ? RE.REG_ICASE : 0) | RE.REG_MULTILINE, RESyntax.RE_SYNTAX_PERL5);
      if (regexp == null)
      {
        endCompoundEdit();
        return false;
      }

      if (replaceStr == null)
        replaceStr = "";

      REMatch match;

      Element map = getDocument().getDefaultRootElement();
      int startLine = map.getElementIndex(start);
      int endLine = map.getElementIndex(end);

      for (int i = startLine; i <= endLine; i++)
      {
        Element lineElement = map.getElement(i);
        int lineStart;
        int lineEnd;

        if (i == startLine)
          lineStart = start;
        else
          lineStart = lineElement.getStartOffset();
        if (i == endLine)
          lineEnd = end;
        else
          lineEnd = lineElement.getEndOffset() - 1;

        lineEnd -= lineStart;
        String line = getText(lineStart, lineEnd);
        String newLine = regexp.substituteAll(line, replaceStr);
        if (line.equals(newLine)) continue;
        getDocument().remove(lineStart, lineEnd);
        getDocument().insertString(lineStart, newLine, null);

        end += (newLine.length() - lineEnd);
        found = true;
      }
    } catch(Exception e) {
      found = false;
    }
    endCompoundEdit();
    return found;
  }

  /**
   * When an undoable event is fired, we add it to the undo/redo list.
   */

  public void undoableEditHappened(UndoableEditEvent e)
  {
    if (!getOperation())
    {
      if (compoundEdit == null)
        undo.addEdit(e.getEdit());
      else
        compoundEdit.addEdit(e.getEdit());
    }
  }

  /**
   * When a modification is made in the text, we turn
   * the 'to_be_saved' flag to true.
   */

  public void changedUpdate(DocumentEvent e)
  {
    if (!getOperation() && !isDirty()) setDirty();
  }

  /**
   * When a modification is made in the text, we turn
   * the 'to_be_saved' flag to true.
   */

  public void insertUpdate(DocumentEvent e)
  {
    if (!getOperation() && !isDirty()) setDirty();
  }

  /**
   * When a modification is made in the text, we turn
   * the 'to_be_saved' flag to true.
   */

  public void removeUpdate(DocumentEvent e)
  {
    if (!getOperation() && !isDirty()) setDirty();
  }

  /**
   * Return a String representation of this object.
   */

  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("BuTextArea: ");
    buf.append("[filename: " + getCurrentFile() + ";");
    buf.append(" filesize: " + getLength() + "] -");
    buf.append("[anchor: " + getAnchor() + "] -");
    buf.append(" [font-name: " + getFontName() + ";");
    buf.append(" font-style: " + getFontStyle() + ";");
    buf.append(" font-size: " + getFontSize() + "]");
    return buf.toString();
  }


  public final static void main(String argv[])
  {
    JFrame frame=new JFrame("Test BuTextArea");
    JComponent content=(JComponent)frame.getContentPane();
    BuTextArea ta1=new BuTextArea();
    BuTextArea ta2=new BuTextArea();
    ta2.setDocument(ta1.getDocument());

    content.setLayout(new GridLayout(1,2));
    content.add(ta1);
    content.add(ta2);
    frame.pack();
    frame.show();
    frame.setLocation(200,200);
  }
}

