/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuTextField.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * An extended text field which takes validators.
 */

public class BuTextField
       extends JTextField
{
  private Color   textFg_;
  private Color   errorFg_;
  private Format  displayFormat_  =null;
  private boolean focusSelection_ =false;
  private String  realText_       =null;

  private BuCharValidator   charValidator_;
  private BuStringValidator stringValidator_;
  private BuValueValidator  valueValidator_;

  public BuTextField()
  {
    this(null,-1);
  }

  public BuTextField(int _cols)
  {
    this(null,_cols);
  }

  public BuTextField(String _text)
  {
    this(_text,-1);
  }

  public BuTextField(String _text, int _cols)
  {
    super();

    if(_cols>0) setColumns(_cols);
    
    if(_text==null) _text="";
    setText(_text);

    errorFg_=UIManager.getColor("TextField.errorForeground");
    if(errorFg_==null) errorFg_=Color.red;
  }

  // Property Display Format

  public Format getDisplayFormat()
  {
    return displayFormat_;
  }

  public void setDisplayFormat(Format _format)
  {
    if(_format==null) realText_=null;
    displayFormat_=_format;
  }

  // Property Selection Mode

  public boolean isFocusSelection()
  {
    return focusSelection_;
  }

  public void setFocusSelection(boolean _focusSelection)
  {
    focusSelection_=_focusSelection;
  }

  // Property Text

  public String getText()
  {
    String r;
    if((displayFormat_!=null)&&!hasFocus()) r=realText_;
    else r=super.getText();
    return r;
  }

  public void setText(String _text)
  {
    boolean ok=true;

    String s=_text;
    if(!isStringValid(s))
      ok=false;
    else
      if(!isValueValid(stringToValue(s)))
	ok=false;

    if(ok&&!hasFocus())
    {
      Object v=stringToValue(s);

      s=valueToString(v);

      if(displayFormat_!=null)
      {
	realText_=s;
	s=displayFormat_.format(v);
      }
    }

    if(!ok) super.setForeground(errorFg_);
    else    super.setForeground(textFg_);
    super.setText(s);
  }

  // Properties Validators

  public BuCharValidator getCharValidator()
    { return charValidator_; }

  public void setCharValidator(BuCharValidator _validator)
    { charValidator_=_validator; }

  public BuStringValidator getStringValidator()
    { return stringValidator_; }

  public void setStringValidator(BuStringValidator _validator)
    { stringValidator_=_validator; }

  public BuValueValidator getValueValidator()
    { return valueValidator_; }

  public void setValueValidator(BuValueValidator _validator)
    { valueValidator_=_validator; }


  public void setForeground(Color _c)
  {
    textFg_=_c;
    super.setForeground(_c);
  }

  public void setErrorForeground(Color _c)
  {
    errorFg_=_c;
  }

  public void replaceSelection(String _s)
  {
    boolean b=true;

    for(int i=0;i<_s.length();i++)
      b&=isCharValid(_s.charAt(i));

    if(b) super.replaceSelection(_s);
    else  getToolkit().beep();
  }

  protected void processKeyEvent(KeyEvent _evt)
  { 
    if(_evt.getID()==KeyEvent.KEY_TYPED)
      if(!isCharValid(_evt.getKeyChar()))
	return;

    super.setForeground(textFg_);
    super.processKeyEvent (_evt);
  }

  protected void processFocusEvent(FocusEvent _evt)
  {
    boolean ok=true;

    if(_evt.getID()==FocusEvent.FOCUS_LOST)
    {
      String s=super.getText();
      if(!isStringValid(s))
	ok=false;
      else
	if(!isValueValid(stringToValue(s)))
	  ok=false;

      if(displayFormat_!=null)
	realText_=s;

      if(ok)
      {
	Object v=stringToValue(s);

	s=valueToString(v);

	if(displayFormat_!=null)
	{
	  realText_=s;
	  s=displayFormat_.format(v);
	}

	super.setText(s);
      }
      else
	getToolkit().beep();
    }

    if(_evt.getID()==FocusEvent.FOCUS_GAINED)
    {
      ok=true;

      String s=getText();

      if(isStringValid(s))
      {
	Object v=stringToValue(s);
	if(isValueValid(v))
	{
	  s=valueToString(v);
	  super.setText(s);
          if(isFocusSelection())
	  {
	    setSelectionStart(0);
	    setSelectionEnd(s.length());
	  }
        }
      }
    }
    
    if(!ok) super.setForeground(errorFg_);
    else    super.setForeground(textFg_);

    repaint();
    super.processFocusEvent(_evt);
  } 

  public Object stringToValue(String _string)
  {
    Object r=_string;
    if((r!=null)&&(stringValidator_!=null))
    {
      if(isStringValid(_string))
      {
	r=stringValidator_.stringToValue(_string);
	if(!isValueValid(r)) r=null;
      }
      else
	r=null;
    }
    return r;
  }

  public String valueToString(Object _value)
  {
    String r=null;
    
    if(isValueValid(_value))
      if(stringValidator_!=null)
	r=stringValidator_.valueToString(_value);

         if(_value==null) r="";
    else if(r==null) r=""+_value;
    return r;
  }

  private boolean isCharValid(char _char)
  {
    boolean r=true;

         if(_char==  8) r=true;
    else if(_char==255) r=false;
    else if(charValidator_!=null)
           r=charValidator_.isCharValid(_char);

    return r;
  }

  private boolean isStringValid(String _value)
  {
    boolean r=true;
    if(stringValidator_!=null) r=stringValidator_.isStringValid(_value);
    return r;
  }

  private boolean isValueValid(Object _value)
  {
    boolean r=true;
    if(valueValidator_!=null) r=valueValidator_.isValueValid(_value);
    return r;
  }

  public Object getValue()
    { return stringToValue(getText()); }

  public void setValue(Object _value)
    { setText(valueToString(_value)); }

  // Factory

  public static BuTextField createIntegerField()
  {
    BuTextField r=new BuTextField();
    r.setCharValidator(BuCharValidator.INTEGER);
    r.setStringValidator(BuStringValidator.INTEGER);
    r.setValueValidator(BuValueValidator.INTEGER);
    return r;
  }

  public static BuTextField createLongField()
  {
    BuTextField r=new BuTextField();
    r.setCharValidator(BuCharValidator.LONG);
    r.setStringValidator(BuStringValidator.LONG);
    r.setValueValidator(BuValueValidator.LONG);
    return r;
  }

  public static BuTextField createFloatField()
  {
    BuTextField r=new BuTextField();
    r.setCharValidator(BuCharValidator.FLOAT);
    r.setStringValidator(BuStringValidator.FLOAT);
    r.setValueValidator(BuValueValidator.FLOAT);
    return r;
  }

  public static BuTextField createDoubleField()
  {
    BuTextField r=new BuTextField();
    r.setCharValidator(BuCharValidator.DOUBLE);
    r.setStringValidator(BuStringValidator.DOUBLE);
    r.setValueValidator(BuValueValidator.DOUBLE);
    return r;
  }

  public static BuTextField createIdField()
  {
    BuTextField r=new BuTextField();
    r.setCharValidator(BuCharValidator.ID);
    r.setStringValidator(BuStringValidator.ID);
    r.setValueValidator(BuValueValidator.ID);
    return r;
  }

  public static BuTextField createFileField()
  {
    BuTextField r=new BuTextField();
    r.setCharValidator(BuCharValidator.FILE);
    r.setStringValidator(BuStringValidator.FILE);
    r.setValueValidator(BuValueValidator.FILE);
    return r;
  }

  public static BuTextField createDateField()
  {
    BuTextField r=new BuTextField();
    r.setCharValidator(BuCharValidator.DATE);
    r.setStringValidator(BuStringValidator.DATE);
    r.setValueValidator(BuValueValidator.DATE);
    return r;
  }

  public static BuTextField createTimeField()
  {
    BuTextField r=new BuTextField();
    r.setCharValidator(BuCharValidator.TIME);
    r.setStringValidator(BuStringValidator.TIME);
    r.setValueValidator(BuValueValidator.TIME);
    return r;
  }

  // Test

  public final static JComponent launch(String argv[])
  {
    JComponent content=new JPanel();

    BuVerticalLayout lo=new BuVerticalLayout();
    lo.setVgap(5);
    lo.setHfilled(true);
    lo.setVfilled(false);
    content.setLayout(lo);
    content.setBorder(new EmptyBorder(5,5,5,5));

    JTextField  tf0=new JTextField("Rfrence");

    BuTextField tf1=createIntegerField();
    BuTextField tf2=createDoubleField();
    BuTextField tf3=createIdField();
    BuTextField tf4=createDateField();
    BuTextField tf5=createTimeField();
    BuTextField tf6=createDoubleField();
    BuTextField tf7=new BuTextField();

    DecimalFormat fmt=new DecimalFormat();
    fmt.setMinimumFractionDigits(2);
    fmt.setMaximumFractionDigits(2);
    // fmt.setMinimumIntegerDigits(1);
    // fmt.setMaximumIntegerDigits(6);
    tf6.setDisplayFormat(fmt);
    tf6.setValueValidator
      (BuValueValidator.MULTI
       (BuValueValidator.DOUBLE,BuValueValidator.MINMAX(0.,1.)));

    tf1.setValue(new Integer(123));
    tf2.setValue(new Double(456.789));
    tf3.setValue("abc_def");
    tf4.setValue(new Date());
    tf5.setValue(new Long(new Date().getTime()));
    tf6.setValue(new Double(-123456.78901));

    content.add(tf0);
    content.add(tf1);
    content.add(tf2);
    content.add(tf3);
    content.add(tf4);
    content.add(tf5);
    content.add(tf6);
    content.add(tf7);

    return content;
  }

  public final static void main(String argv[])
  {
    Locale.setDefault(Locale.FRANCE);

    JFrame     frame  =new JFrame("Test BuTextField");
    JComponent content=launch(argv);

    frame.setContentPane(content);
    frame.setSize(220,220);
    frame.show();
    frame.setLocation(200,200);
  }
}
