/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuToolBar.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * A tool bar with simplified adding methods.
 */

public class BuToolBar
       extends JToolBar
       implements ActionListener, PropertyChangeListener
{
  private final static int ML=7;
  private final static int MR=7;

  private int dx_=ML;
  private int dy_=ML;

  private class MA implements MouseListener
  {
    public void mouseClicked(MouseEvent _evt)
    {
      Insets    insets=getInsets();
      Dimension size  =getSize();
      boolean   done  =false;

      if(isHorizontal())
      {
	int x=_evt.getX();
	if(  ((x>=insets.left)&&(x<insets.left+ML))
	   ||(  (x>=size.width-insets.right-MR)
	      &&(x<size.width-insets.right)))
	{
	  dy_=ML;
	  if(dx_!=ML)
	    dx_=ML;
	  else
	    dx_=Math.min(ML,getWidth()-getPreferredSize().width-MR);
	  done=true;
	}
      }
      else
      {
	int y=_evt.getY();
	if(  ((y>=insets.top)&&(y<insets.top+ML))
	   ||(  (y>=size.height-insets.bottom-MR)
	      &&(y<size.height-insets.bottom)))
	{
	  dx_=ML;
	  if(dy_!=ML)
	    dy_=ML;
	  else
	    dy_=Math.min(ML,getHeight()-getPreferredSize().height-MR);
	  done=true;
	}
      }

      if(done)
      {
	doLayout();
	repaint();
      }
    }

    public void mousePressed (MouseEvent _evt) { }
    public void mouseReleased(MouseEvent _evt) { }
    public void mouseEntered (MouseEvent _evt) { }
    public void mouseExited  (MouseEvent _evt) { }
  }

  public BuToolBar()
  {
    super();
    setName("tbMAIN_TOOLBAR");
    setFloatable
      (BuPreferences.BU.getBooleanProperty("toolbar.floatable",true));
    putClientProperty
      ("JToolBar.isRollover",
       !BuLib.isKunststoff()
       &&(UIManager.get("Button.textureType")==null)
       &&BuPreferences.BU.getBooleanProperty("toolbar.rollover",true)
       ? Boolean.TRUE : Boolean.FALSE);

    addPropertyChangeListener(this);

    MA ma=new MA();
    addMouseListener(ma);
    //addMouseMotionListener(ma);
  }

  // Anti-aliasing

  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  private String title_="Main toolbar";
  public String getTitle() { return title_; }
  public void setTitle(String _title) { title_=_title; }

  private Image icon_=null;
  public Image getIconImage() { return icon_; }
  public void setIconImage(Image _icon) { icon_=_icon; }

  public void propertyChange(PropertyChangeEvent _evt)
  {
    if("ancestor".equals(_evt.getPropertyName()))
    {
      Container a=getTopLevelAncestor();
      //System.err.println("ANCESTOR="+a);
      if((a!=null)&&(a.getClass()==JFrame.class))
      {
	try // Korte
	{
	  JFrame f=(JFrame)a;
	  if(getTitle()!=null) f.setTitle(getTitle());
	  if(getIconImage()!=null) f.setIconImage(getIconImage());
	  f.setResizable(true);
	  f.pack();
	}
	catch(Throwable th) { }
      }

      /*
      invalidate();
      doLayout();
      validate();
      */
      revalidate();
    }
    else
    {
      //System.err.println(_evt.getPropertyName());
    }
  }

  public boolean isFocusCycleRoot()
  { return true; }

  public BuToolButton addToolButton
    (String _tip, String _cmd, boolean _enabled)
  {
    BuIcon icon=BuResource.BU.loadToolCommandIcon(_cmd);
    return addToolButton(_tip,_cmd,icon,_enabled);
  }

  public BuToolButton addToolButton
    (String _text, String _tip, String _cmd, boolean _enabled)
  {
    BuIcon icon=BuResource.BU.loadToolCommandIcon(_cmd);
    return addToolButton(_text,_tip,_cmd,icon,_enabled);
  }

  public BuToolButton addToolButton
    (String _tip, String _cmd, BuIcon _icon, boolean _enabled)
  {
    String s=_tip;
    if(s.endsWith("...")) s=s.substring(0,s.length()-3);
    //if(s.length()>9) s="...";
    return addToolButton(s,_tip,_cmd,_icon,_enabled);
  }

  public BuToolButton addToolButton
    (String _text, String _tip, String _cmd, BuIcon _icon, boolean _enabled)
  {
    BuToolButton r=new BuToolButton();
    r.setName("tb"+_cmd);
    r.setActionCommand(_cmd);
    r.setText(_text);
    r.setIcon(_icon);
    r.setToolTipText(_tip);
    r.setRequestFocusEnabled(false);
    r.addActionListener(this);
    r.setEnabled(_enabled);
    r.setMargin(new Insets(1,1,1,1));

    add(r);
    //revalidate();

    r.setRolloverEnabled(true);
    return r;
  }

  /*
  public void removeToolButton(String _cmd)
  {
    Component[] c=_bar.getComponents();
    for(int i=0; i<c.length; i++)
      if(c[i] instanceof JButton)
      {
	JButton b=(JButton)c[i];
	if(b.getActionCommand().equals(_cmd))
	  remove(b);
      }
    doLayout();
    validate();
  }
  */

  // Paint

  private final static int[] hlax=new int[] { 0,6,6 };
  private final static int[] hlay=new int[] { 3,6,0 };
  private final static int[] hrax=new int[] { 0,0,6 };
  private final static int[] hray=new int[] { 0,6,3 };

  private final static int[] vtax=new int[] { 6,3,0 };
  private final static int[] vtay=new int[] { 6,0,6 };
  private final static int[] vbax=new int[] { 0,3,6 };
  private final static int[] vbay=new int[] { 0,6,0 };

  public void paintComponent(Graphics _g)
  {
    BuBackgroundPainter bp=(BuBackgroundPainter)
                           getClientProperty("BACKGROUND_PAINTER");
    if(bp!=null)
      bp.paintBackground(this,_g);

    super.paintComponent(_g);

    Insets insets=getInsets();
    Dimension size=getSize();
    Color c=UIManager.getColor("Desktop.background");

    if(isHorizontal())
    {
      if(isTooBigX())
      {
	int x,y;
	boolean b=(dx_==ML);
	
	/*
	_g.setColor(b?c.brighter():c);
	_g.fill3DRect(insets.left,insets.top,ML,
		      size.height-insets.top-insets.bottom,!b);
	_g.setColor(!b?c.brighter():c);
	_g.fill3DRect(size.width-insets.right-MR,insets.top,MR,
		      size.height-insets.top-insets.bottom,b);
	*/

	_g.setColor(UIManager.getColor("ToolBar.foreground"));
	y=(size.height-6)/2;

	x=insets.left;
	_g.translate(x,y);
	if(b) _g.fillPolygon(hrax,hray,3);
	else  _g.fillPolygon(hlax,hlay,3);
	_g.translate(-x,-y);

	x=size.width-insets.right-5;
	_g.translate(x,y);
	if(b) _g.fillPolygon(hrax,hray,3);
	else  _g.fillPolygon(hlax,hlay,3);
	_g.translate(-x,-y);
      }
    }
    else
    {
      if(isTooBigY())
      {
	int x,y;
	boolean b=(dy_==ML);

	/*
	_g.setColor(b?c.brighter():c);
	_g.fill3DRect(insets.left,insets.top,
		      size.width-insets.left-insets.right,ML,!b);
	_g.setColor(!b?c.brighter():c);
	_g.fill3DRect(insets.left,size.height-insets.bottom-MR,
		      size.width-insets.left-insets.right,MR,b);
	*/


	_g.setColor(UIManager.getColor("ToolBar.foreground"));
	x=(size.width-6)/2;

	y=insets.top;
	_g.translate(x,y);
	if(b) _g.fillPolygon(vbax,vbay,3);
	else  _g.fillPolygon(vtax,vtay,3);
	_g.translate(-x,-y);

	y=size.height-insets.bottom-5;
	_g.translate(x,y);
	if(b) _g.fillPolygon(vbax,vbay,3);
	else  _g.fillPolygon(vtax,vtay,3);
	_g.translate(-x,-y);
      }
    }
  }

  public void paintChildren(Graphics _g)
  {
    Shape old=_g.getClip();

    Insets insets=getInsets();
    Dimension size=getSize();
    Rectangle clip=new Rectangle
      (insets.left,insets.top,size.width-insets.left-insets.right,
       size.height-insets.top-insets.bottom);
    _g.setClip(clip.intersection(old.getBounds()));
    super.paintChildren(_g);
    _g.setClip(old);
  }

  // TMP
  private final boolean isHorizontal()
  {
    return (getWidth()>getHeight());
  }

  private final boolean isTooBigX()
  {
    return ((getWidth()-getPreferredSize().width-MR-ML)<0);
  }

  private final boolean isTooBigY()
  {
    return ((getHeight()-getPreferredSize().height-MR-ML)<0);
  }

  public void doLayout()
  {
    super.doLayout();

    Insets      b=getInsets();
    Dimension   s=getSize();
    boolean     h=isHorizontal();
    Component[] c=getComponents();
    for(int i=0;i<c.length;i++)
    {
      Rectangle r=c[i].getBounds();
      if(h)
      {
	r.x+=dx_;
	if(r.x<b.left+ML) r.x=-r.width;
	if(r.x+r.width>s.width-b.right-MR) r.x=s.width;
      }
      else
      {
	r.y+=dy_;
	if(r.y<b.top+ML) r.y=-r.height;
	if(r.y+r.height>s.height-b.bottom-MR) r.y=s.height;
      }
      c[i].setBounds(r);
    }
    //System.err.println("LAYOUT:"+dx_+","+dy_);
  }

  // Barre standarde

  private final static String getString(String _s)
  { return BuResource.BU.getString(_s); }

  private final static boolean isThere(String _s, boolean _d)
  { return BuPreferences.BU.getBooleanProperty("command."+_s,_d); }

  public static BuToolBar buildBasicToolBar()
  {
    BuToolBar r=new BuToolBar();

    String[][] codes=BuCommandPreferencesPanel.CODES;
    for(int i=0;i<codes.length;i++)
    {
      if(isThere(codes[i][0],"true".equals(codes[i][1])))
      {
	if("-".equals(codes[i][2]))
	  r.addSeparator();
	else
	  r.addToolButton(getString(codes[i][2]),getString(codes[i][3]),
			  codes[i][4],"true".equals(codes[i][5]));
      }
    }

    /*
    if(isThere("create"          ,true )) r.addToolButton(getString("Crer"        ),getString("Crer..."             ),"CREER"           ,false);
    if(isThere("open"            ,true )) r.addToolButton(getString("Ouvrir"       ),getString("Ouvrir..."            ),"OUVRIR"          ,false);
    if(isThere("save"            ,true )) r.addToolButton(getString("Enregistrer"  ),getString("Enregistrer"          ),"ENREGISTRER"     ,false);
    if(isThere("save_as"         ,false)) r.addToolButton(getString("Enregistrer"  ),getString("Enregistrer sous..."  ),"ENREGISTRERSOUS" ,false);
    if(isThere("close"           ,false)) r.addToolButton(getString("Fermer"       ),getString("Fermer..."            ),"FERMER"          ,false);
    if(isThere("separator1"      ,true )) r.addSeparator();
    if(isThere("print"           ,true )) r.addToolButton(getString("Imprimer"     ),getString("Imprimer..."          ),"IMPRIMER"        ,false);
    if(isThere("preview"         ,false)) r.addToolButton(getString("Prvisualiser"),getString("Prvisualiser..."     ),"PREVISUALISER"   ,false);
    if(isThere("separator2"      ,true )) r.addSeparator();
    if(isThere("undo"            ,true )) r.addToolButton(getString("Dfaire"      ),getString("Dfaire"              ),"DEFAIRE"         ,false);
    if(isThere("redo"            ,true )) r.addToolButton(getString("Refaire"      ),getString("Refaire"              ),"REFAIRE"         ,false);
    if(isThere("separator3"      ,true )) r.addSeparator();
    if(isThere("copy"            ,true )) r.addToolButton(getString("Copier"       ),getString("Copier"               ),"COPIER"          ,false);
    if(isThere("cut"             ,true )) r.addToolButton(getString("Couper"       ),getString("Couper"               ),"COUPER"          ,false);
    if(isThere("paste"           ,true )) r.addToolButton(getString("Coller"       ),getString("Coller"               ),"COLLER"          ,false);
    if(isThere("duplicate"       ,false)) r.addToolButton(getString("Dupliquer"    ),getString("Dupliquer"            ),"COLLER"          ,false);
    if(isThere("separator4"      ,true )) r.addSeparator();
    if(isThere("select_all"      ,true )) r.addToolButton(getString("Slectionner" ),getString("Tout slectionner"    ),"TOUTSELECTIONNER",false);
    if(isThere("find"            ,true )) r.addToolButton(getString("Rechercher"   ),getString("Rechercher..."        ),"RECHERCHER"      ,false);
    if(isThere("replace"         ,true )) r.addToolButton(getString("Remplacer"    ),getString("Remplacer..."         ),"REMPLACER"       ,false);
    if(isThere("separator5"      ,true )) r.addSeparator();
    if(isThere("fullscreen"      ,false)) r.addToolButton(getString("Ecran"        ),getString("Plein cran"          ),"PLEINECRAN"      ,true );
    if(isThere("waterfall"       ,false)) r.addToolButton(getString("Cascade"      ),getString("Cascade"              ),"CASCADE"         ,true );
    if(isThere("arrange_icons"   ,true )) r.addToolButton(getString("Ranger"       ),getString("Ranger les icnes"    ),"RANGERICONES"   ,true );
    if(isThere("arrange_palettes",true )) r.addToolButton(getString("Ranger"       ),getString("Ranger les palettes"  ),"RANGERPALETTES" ,true );
    if(isThere("swap_columns"    ,false)) r.addToolButton(getString("Echanger"     ),getString("Echanger les colonnes"),"ECHANGER_COLONNES",true );
    if(isThere("separator6"      ,true )) r.addSeparator();
    if(isThere("help"            ,false)) r.addToolButton(getString("Aide"         ),getString("Aide en-ligne"        ),"AIDE"            ,true );
    if(isThere("help_pointer"    ,true )) r.addToolButton(getString("Pointeur"     ),getString("Pointeur d'aide"      ),"POINTEURAIDE"    ,true );
    */

    return r;
  }

  // Action

  private Vector actionListeners_=new Vector();

  public void addActionListener(ActionListener _l)
  { actionListeners_.addElement(_l); }

  public void removeActionListener(ActionListener _l)
  { actionListeners_.removeElement(_l); }

  public void actionPerformed(ActionEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();
    // System.err.println(""+getClass()+": "+source);

    for(Enumeration e=actionListeners_.elements(); e.hasMoreElements(); )
      ((ActionListener)e.nextElement()).actionPerformed(_evt);
  }
}


