/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuToolButton.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;

/**
 * Like BuButton but for toolbars.
 */

public class BuToolButton
       extends JButton
{
  public BuToolButton()
  {
    this((BuIcon)null,null);
  }

  public BuToolButton(BuIcon _icon)
  {
    this(_icon,null);
  }

  public BuToolButton(String _label)
  {
    this(null,_label);
  }

  public BuToolButton(BuIcon _icon, String _label)
  {
    super();

    setCursor(new Cursor(Cursor.HAND_CURSOR));
    setIcon(_icon);
    setText(_label);
  }

  // Anti-aliasing

  public void paint(Graphics _g)
  {
    BuLib.setAntialiasing(_g);
    super.paint(_g);
  }

  public boolean isOpaque()
  {
    boolean r=super.isOpaque();

    Container parent=getParent();

    if(parent instanceof JComponent)
    {
      Object p=((JComponent)parent)
	  .getClientProperty("JToolBar.isRollover");
      if(Boolean.TRUE.equals(p))
	  r&=isEnabled()&&getModel().isRollover();
    }

    return r;
  }

  private final boolean isHorizontal()
  {
    boolean r=true;
    Container parent=getParent();

    if(parent!=null)
    {
      Dimension size=parent.getSize();
      r=(size.width>size.height);
    }

    return r;
  }

  public Font getFont()
  {
    Font r=super.getFont();
    if((r instanceof UIResource)&&isHorizontal()&&(getIcon()!=null))
    {
      r=BuLib.deriveFont("Button",Font.PLAIN,-2);
      //System.err.println("FAMILY="+r.getFamily());
      if("dialog.bold".equals(r.getFamily()))
	r=new Font("SansSerif",Font.PLAIN,r.getSize());
      r=new FontUIResource(r);
    }
    return r;
  }

  public int getHorizontalTextPosition()
  {
    return (isHorizontal() ? CENTER : RIGHT);
  }

  public int getVerticalTextPosition()
  {
    return (isHorizontal() ? BOTTOM : CENTER);
  }

  public Dimension getPreferredSize()
  {
    Dimension r=super.getPreferredSize();
    if(isHorizontal()&&(getIcon()!=null))
      r.width=r.height;
    //40;+=39-(r.width%40);
    //else r.width=3*r.height;
    return r;
  }

  public Dimension getMinimumSize()
  {
    return getPreferredSize();
  }

  public Dimension getMaximumSize()
  {
    return getPreferredSize();
  }

  public void setText(String _text)
  {
    if("".equals(_text)) _text=null;
    super.setText(_text);

    if(  !BuPreferences.BU.getBooleanProperty("tool.text",true)
       &&(getToolTipText()==null))
      super.setToolTipText(_text);
  }

  public String getText()
  {
    if(  BuPreferences.BU.getBooleanProperty("tool.text",true)
       ||(super.getIcon()==null))
    {
      String r=super.getText();
      if((getIcon()!=null)&&(r!=null)&&isHorizontal())
      {
	FontMetrics fm=BuLib.getFontMetrics(this,getFont());
	Insets in=getInsets();
	int w=getWidth()-in.left-in.right;
	
	if(fm.stringWidth(r)>w)
	{
	  while((r.length()>0)&&(fm.stringWidth(r+".")>w))
	    r=r.substring(0,r.length()-1);
	  while(  (r.length()>0)
		&&("aeiouy".indexOf(r.charAt(r.length()-1))>0))
	    r=r.substring(0,r.length()-1);
	  r+=".";
	}
      }
      return r;
    }

    return null;
  }

  public void setIcon(BuIcon _icon)
    { BuLib.setIcon(this,_icon); }

  public Icon getIcon()
  {
    if(  BuPreferences.BU.getBooleanProperty("tool.icon",true)
       ||(super.getText()==null))
      return super.getIcon();
    return null;
  }

  public Point getToolTipLocation(MouseEvent _evt)
  {
    Point r;

    Container parent=getParent();
    if((parent==null)||isHorizontal())
      r=new Point(0,getHeight()+1);
    else
      r=new Point(-getX()+parent.getSize().width-1,0);

    return r;
  }

  public void updateUI()
  {
    super.updateUI();
    setRolloverEnabled(true);
  }

  public Color getBackground()
  {
    Color r;
    if(isEnabled()&&getModel().isRollover()&&BuLib.isMetal())
      r=new ColorUIResource(192,192,208);
    else
      r=super.getBackground();
    return r;
  }
}
