/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuToolsPreferencesPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;

/**
 * A panel where the user can choose his favorite tools.
 * @creation     2001-10-24
 * @version      0.01, 2001-10-24
 * @author       Guillaume Desnoix
 */

/**
 * tool.#.name
 * tool.#.exec
 * tool.#.icon
 */

public class BuToolsPreferencesPanel
       extends BuAbstractPreferencesPanel
       implements ActionListener
{
  protected BuCommonInterface appli_;
  protected FuPreferences     options_;
  protected BuTable           table_;
  protected BuToolsModel      model_;
  protected BuButton          btInsert_,btRemove_,btUp_,btDown_;

  public String getTitle()
    { return BuResource.BU.getString("Outils"); }

  // Constructeur

  public BuToolsPreferencesPanel
    (BuCommonInterface _appli, FuPreferences _options)
  {
    super();
    appli_  =_appli;
    options_=_options;

    BuPanel p=new BuPanel();
    p.setBorder
      (new CompoundBorder
       (new TitledBorder(BuResource.BU.getString("Outils")),
	new EmptyBorder(5,5,5,5)));
    p.setLayout(new BuBorderLayout(5,5));

    model_=new BuToolsModel(_options);
    table_=new BuTable();
    table_.setModel(model_);
    table_.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN); // OFF
    table_.setSelectionMode (ListSelectionModel.SINGLE_SELECTION);

    int[]     tailles  =new int[]     { 26,100,184 };
    boolean[] variables=new boolean[] { false,true,true };

    for(int i=0;i<table_.getColumnCount();i++)
    {
      TableColumn c=table_.getColumn(table_.getColumnName(i));
      c.setPreferredWidth(tailles[i]);
      if(!variables[i])
      {
	c.setResizable(false);
	c.setMinWidth (tailles[i]);
	c.setMaxWidth (tailles[i]);
      }
    }

    p.add(new BuScrollPane(table_),BuBorderLayout.CENTER);

    BuPanel q=new BuPanel();
    q.setLayout(new BuGridLayout(4,5,5,true,true,false,false));

    btInsert_=new BuButton(BuLib.loadCommandIcon("AJOUTER"),
			   BuResource.BU.getString("Insrer"));
    btInsert_.setActionCommand("AJOUTER");
    btInsert_.addActionListener(this);
    q.add(btInsert_);

    btRemove_=new BuButton(BuLib.loadCommandIcon("ENLEVER"),
			   BuResource.BU.getString("Enlever"));
    btRemove_.setActionCommand("ENLEVER");
    btRemove_.addActionListener(this);
    q.add(btRemove_);

    btUp_    =new BuButton(BuLib.loadCommandIcon("RECULER"),
			   BuResource.BU.getString("Reculer"));
    btUp_.setActionCommand("RECULER");
    btUp_.addActionListener(this);
    q.add(btUp_);

    btDown_    =new BuButton(BuLib.loadCommandIcon("AVANCER"),
			     BuResource.BU.getString("Avancer"));
    btDown_.setActionCommand("AVANCER");
    btDown_.addActionListener(this);
    q.add(btDown_);

    BuLib.computeMnemonics(q);
    p.add(q,BuBorderLayout.SOUTH);

    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    add(p,BuBorderLayout.CENTER);

    //Dimension qs=q.getPreferredSize();
    setPreferredSize(new Dimension(100,100));
    //(Math.max(200,qs.width),Math.max(200,100+qs.height)));

    //updateComponents();
  }

  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();
    String     action=_evt.getActionCommand();
    // System.err.println("SOURCE="+source.getName());
    // System.err.println("ACTION="+action);

    int index=1+table_.getSelectedRow();

    if("AJOUTER".equals(action))
	insertTool(index);
    else
    if("ENLEVER".equals(action))
	removeTool(index);
    else
    if("AVANCER".equals(action))
	downTool(index);
    else
    if("RECULER".equals(action))
	upTool(index);
  }
  
  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
  }

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
  }

  private void updateComponents()
  {
  }

  // Manipulation

  protected int nbTools()
  {
    int i=1;
    while(true)
    {
      String name=options_.getStringProperty("tool."+i+".name");
      String exec=options_.getStringProperty("tool."+i+".exec");
      String icon=options_.getStringProperty("tool."+i+".icon");

      if("".equals(exec)) break;
      i++;
    }

    return i;
  }

  protected void setSelection(int _row)
  {
    table_.removeEditor();
    table_.clearSelection();
    table_.setColumnSelectionInterval(0,table_.getColumnCount()-1);
    table_.setRowSelectionInterval(_row-1,_row-1);
  }

  public void insertTool(int _index)
  {
    if((options_==null)||(_index<1)||(_index>=nbTools())) return;

    int i=nbTools()-1;
    for(int j=i;j>=_index;j--)
    {
      options_.putStringProperty
	  ("tool."+(j+1)+".name",
	   options_.getStringProperty("tool."+j+".name"));
      options_.putStringProperty
	  ("tool."+(j+1)+".exec",
	   options_.getStringProperty("tool."+j+".exec"));
      options_.putStringProperty
	  ("tool."+(j+1)+".icon",
	   options_.getStringProperty("tool."+j+".icon"));
    }

    i=_index;
    options_.putStringProperty("tool."+i+".name","");
    options_.putStringProperty("tool."+i+".exec","-");
    options_.putStringProperty("tool."+i+".icon","executer");
    setSelection(i);
  }

  public void removeTool(int _index)
  {
    if((options_==null)||(_index<1)||(_index>=nbTools())) return;

    int i=nbTools()-1;
    for(int j=_index;j<i;j++)
    {
      options_.putStringProperty
	  ("tool."+j+".name",
	   options_.getStringProperty("tool."+(j+1)+".name"));
      options_.putStringProperty
	  ("tool."+j+".exec",
	   options_.getStringProperty("tool."+(j+1)+".exec"));
      options_.putStringProperty
	  ("tool."+j+".icon",
	   options_.getStringProperty("tool."+(j+1)+".icon"));
    }

    options_.removeProperty("tool."+i+".name");
    options_.removeProperty("tool."+i+".exec");
    options_.removeProperty("tool."+i+".icon");
    setSelection(_index);
  }

  public void upTool(int _index)
  {
    if((options_==null)||(_index<2)||(_index>=nbTools())) return;

    int i=_index;
    int j=_index-1;

    String namei=options_.getStringProperty("tool."+i+".name");
    String execi=options_.getStringProperty("tool."+i+".exec");
    String iconi=options_.getStringProperty("tool."+i+".icon");
    String namej=options_.getStringProperty("tool."+j+".name");
    String execj=options_.getStringProperty("tool."+j+".exec");
    String iconj=options_.getStringProperty("tool."+j+".icon");

    options_.putStringProperty("tool."+i+".name",namej);
    options_.putStringProperty("tool."+i+".exec",execj);
    options_.putStringProperty("tool."+i+".icon",iconj);
    options_.putStringProperty("tool."+j+".name",namei);
    options_.putStringProperty("tool."+j+".exec",execi);
    options_.putStringProperty("tool."+j+".icon",iconi);
    setSelection(j);
  }

  public void downTool(int _index)
  {
    if((options_==null)||(_index<1)||(_index+1>=nbTools())) return;

    int i=_index;
    int j=_index+1;

    String namei=options_.getStringProperty("tool."+i+".name");
    String execi=options_.getStringProperty("tool."+i+".exec");
    String iconi=options_.getStringProperty("tool."+i+".icon");
    String namej=options_.getStringProperty("tool."+j+".name");
    String execj=options_.getStringProperty("tool."+j+".exec");
    String iconj=options_.getStringProperty("tool."+j+".icon");

    options_.putStringProperty("tool."+i+".name",namej);
    options_.putStringProperty("tool."+i+".exec",execj);
    options_.putStringProperty("tool."+i+".icon",iconj);
    options_.putStringProperty("tool."+j+".name",namei);
    options_.putStringProperty("tool."+j+".exec",execi);
    options_.putStringProperty("tool."+j+".icon",iconi);
    setSelection(j);
  }

  public void addTool(String _name, String _exec, String _icon)
  {
    if(options_==null) return;

    int i=nbTools();

    options_.putStringProperty("tool."+i+".name",_name);
    options_.putStringProperty("tool."+i+".exec",_exec);
    options_.putStringProperty("tool."+i+".icon",_icon);
  }
}
