/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuUserPreferencesPanel.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.*;

/**
 * A panel where the user can enter informations about
 * himself.
 * @version      0.01, 12/11/99
 * @author       Guillaume Desnoix
 */

/**
 * user.*
 */

public class BuUserPreferencesPanel
       extends BuAbstractPreferencesPanel
{
  BuCommonInterface appli_;
  BuPreferences     options_;

  JTextField[]      text_;

  public String getTitle()
    { return BuResource.BU.getString("Utilisateur"); }

  private final static int NB=7;

  private final static String[] PROP=new String[]
    { "lastname","firstname","nickname","email",
      "title","organization","department" };

  private final static String[] LABEL=new String[]
    { "Nom","Prnom","Surnom","Ml",
      "Titre","Organisme","Dpartement" };

  // Constructeur

  public BuUserPreferencesPanel(BuCommonInterface _appli)
  {
    super();
    appli_  =_appli;
    options_=BuPreferences.BU;
    text_   =new BuTextField[NB];

    BuPanel p=new BuPanel();
    BuGridLayout lo=new BuGridLayout(2,5,5,true,true);
    p.setLayout(lo);

    int n=0;
    for(int i=0;i<NB;i++)
    {
      BuLabel lb=new BuLabel
	(BuResource.BU.getString(LABEL[i])+":",JLabel.RIGHT);
      p.add(lb,n++);
      text_[i]=new BuTextField();
      text_[i].setName("tf"+PROP[i].toUpperCase());
      p.add(text_[i],n++);
    }

    p.setBorder
      (new CompoundBorder
       (new TitledBorder(getTitle()),
	new EmptyBorder(5,5,5,5)));

    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    add(p,BuBorderLayout.CENTER);

    updateComponents();
  }

  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  public boolean isPreferencesApplyable()
    { return true; }
  
  public void applyPreferences()
  {
    fillTable();
    options_.applyOn(appli_);
  }

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
    for(int i=0;i<NB;i++)
      options_.putStringProperty("user."+PROP[i],text_[i].getText());
  }

  private void updateComponents()
  {
    for(int i=0;i<NB;i++)
      text_[i].setText(options_.getStringProperty("user."+PROP[i]));
  }
}
