/**
 * @modification $Date: 2001/12/03 16:28:07 $
 * @statut       unstable
 * @file         BuValueValidator.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */
package com.memoire.bu;
import com.memoire.bu.*;
import com.memoire.dnd.*;
import com.memoire.fu.*;
import com.memoire.re.*;


import java.util.Date;

/**
 * An abstract root class to specify a validator for input values.
 * Provides standard ones for ints, doubles, dates, ...
 */

public abstract class BuValueValidator
{
  public abstract boolean isValueValid(Object _value);

  // Instances

  public final static BuValueValidator INTEGER=new BuValueValidator()
  {
    public boolean isValueValid(Object _value)
      { return (_value instanceof Integer); }
  };

  public final static BuValueValidator LONG=new BuValueValidator()
  {
    public boolean isValueValid(Object _value)
      { return (_value instanceof Long); }
  };

  public final static BuValueValidator FLOAT=new BuValueValidator()
  {
    public boolean isValueValid(Object _value)
      { return (_value instanceof Float); }
  };

  public final static BuValueValidator DOUBLE=new BuValueValidator()
  {
    public boolean isValueValid(Object _value)
      { return (_value instanceof Double); }
  };

  public final static BuValueValidator ID=new BuValueValidator()
  {
    public boolean isValueValid(Object _value)
      { return (_value instanceof String); }
  };

  public final static BuValueValidator FILE=new BuValueValidator()
  {
    public boolean isValueValid(Object _value)
      { return (_value instanceof String); }
  };

  public final static BuValueValidator DATE=new BuValueValidator()
  {
    public boolean isValueValid(Object _value)
      { return (_value instanceof Date); }
  };

  public final static BuValueValidator TIME=new BuValueValidator()
  {
    public boolean isValueValid(Object _value)
      { return (_value instanceof Long); }
  };

  public final static BuValueValidator MIN(final double _min)
  {
    return new BuValueValidator()
    {
      public boolean isValueValid(Object _value)
        { return  (_value instanceof Number)
	        &&(((Number)_value).doubleValue()>=_min); }
    };
  }

  public final static BuValueValidator MAX(final double _max)
  {
    return new BuValueValidator()
    {
      public boolean isValueValid(Object _value)
        { return  (_value instanceof Number)
	        &&(((Number)_value).doubleValue()<=_max); }
    };
  }

  public final static BuValueValidator MINMAX(final double _min, final double _max)
  {
    return new BuValueValidator()
    {
      public boolean isValueValid(Object _value)
        { return  (_value instanceof Number)
	        &&(((Number)_value).doubleValue()>=_min)
	        &&(((Number)_value).doubleValue()<=_max); }
    };
  }

  public final static BuValueValidator MULTI
    (final BuValueValidator _vv1, final BuValueValidator _vv2)
  {
    return new BuValueValidator()
    {
      public boolean isValueValid(Object _value)
	{ return  _vv1.isValueValid(_value)
	        &&_vv2.isValueValid(_value); }
    };
  }
}


