/**
 * @modification $Date: 2001/12/07 18:28:12 $
 * @statut       unstable
 * @file         DjaGraphics.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.dja;

import com.memoire.fu.*;
import com.memoire.bu.*;
import com.memoire.dja.*;

import java.awt.*;

/**
 * Implements some features of Graphics2D for JDK1.1.
 */
public class DjaGraphics
       implements DjaOptions
{
  public final static class BresenhamParams
  {
    public int trait_, vide_;
    public int epaisseur_;

    private int     compteur_;
    private boolean trace_;
  
    public BresenhamParams()
    {
      this(1,0,1);
    }

    public BresenhamParams(int _trait, int _vide)
    {
      this(_trait,_vide,1);
    }

    public BresenhamParams(int _trait, int _vide, int _epaisseur)
    {
      trait_    =_trait;
      vide_     =_vide;
      epaisseur_=_epaisseur;
      compteur_ =0;
      trace_    =true;
    }
  }

  public static BresenhamParams getBresenhamParams(DjaObject _o)
  {
    String trace    =_o.getProperty("trace");
    String epaisseur=_o.getProperty("epaisseur");

    BresenhamParams bp=new BresenhamParams();

    if(trace!=null)
    {
      int i    =trace.indexOf(".");
      bp.trait_=Integer.parseInt(trace.substring(0,i));
      bp.vide_ =Integer.parseInt(trace.substring(i+1));
    }

    if(epaisseur!=null)
    {
      bp.epaisseur_=Integer.parseInt(epaisseur);
    }

    return bp;
  }

  // color
  
  public static Color getColor(Graphics _g)
  {
    return _g.getColor();
  }

  public static void setColor(Graphics _g, Color _c)
  {
    _g.setColor(_c);
  }

  // text

  public static FontMetrics getFontMetrics(DjaObject _o, Font _font)
  {
    DjaGrid g=null;
    if(_o!=null) g=_o.getGrid();
    return BuLib.getFontMetrics(g,_font);
  }

  public static Dimension getTextSize(String _text, Font _font)
  {
    if((_text==null)||"".equals(_text)) return new Dimension(23,23);

    FontMetrics fm=getFontMetrics(null,_font);
    int         w =0;
    int         h =0;
    int         hs=_font.getSize();

    while(!_text.equals(""))
    {
      int    i=_text.indexOf("\n");
      String s;
      if(i>=0) { s=_text.substring(0,i); _text=_text.substring(i+1); }
      else     { s=_text; _text=""; }
      w=Math.max(w,fm.stringWidth(s));
      h+=hs;
    }

    return new Dimension(w+2,h+4); // (w+9,h+7);
  }

  public static void drawText(Graphics _g, String _text, int _x, int _y)
  {
    drawText(_g,_text,_x,_y,0,0,false);
  }

  public static void drawText
    (Graphics _g, String _text, int _x, int _y, int _a, int _w)
  {
    drawText(_g,_text,_x,_y,_a,_w,false);
  }

  public static void drawText
    (Graphics _g, String _text, int _x, int _y,
     int _a, int _w, boolean _underlined)
  {
    if((_text==null)||"".equals(_text)) return;

    FontMetrics fm=_g.getFontMetrics();
    Font        ft=_g.getFont();
    int         x =_x;
    int         hs=ft.getSize();
    int         y =_y+hs;

    while(!_text.equals(""))
    {
      int    i,j;
      String s;

      i=_text.indexOf("\n");
      if(i>=0) { s=_text.substring(0,i); _text=_text.substring(i+1); }
      else     { s=_text; _text=""; }

      int ws=fm.stringWidth(s);

      x=_x;
      if(_a==0) x+=2;
      if(_a==1) x+=(_w-ws)/2;
      if(_a==2) x+=(_w-ws)-2;

      _g.drawString(s,x,y);
      if(_underlined) _g.drawLine(x,y+1,x+ws,y+1);
      y+=hs;
    }
  }

  // draw

  public static void drawPoint
    (Graphics _g, int _x, int _y, BresenhamParams _bp)
  {
    drawPoint(_g,_x,_y,_x,_y,_bp);
  }

  public static void drawPoint
    (Graphics _g, int _x1, int _y1, int _x2, int _y2, BresenhamParams _bp)
  {
    if(_bp.epaisseur_==1)
      _g.drawLine(_x1,_y1,_x2,_y2);
    else
    if(_bp.epaisseur_==2)
    {
      _g.drawLine(_x1  ,_y1  ,_x2  ,_y2  );
      _g.drawLine(_x1+1,_y1  ,_x2+1,_y2  );
      _g.drawLine(_x1  ,_y1+1,_x2  ,_y2+1);
      _g.drawLine(_x1+1,_y1+1,_x2+1,_y2+1);
    }
    else
    if(_bp.epaisseur_==3)
    {
      _g.drawLine(_x1  ,_y1  ,_x2  ,_y2  );
      _g.drawLine(_x1+1,_y1  ,_x2+1,_y2  );
      _g.drawLine(_x1-1,_y1  ,_x2-1,_y2  );
      _g.drawLine(_x1  ,_y1+1,_x2  ,_y2+1);
      _g.drawLine(_x1  ,_y1-1,_x2  ,_y2-1);
    }
    else
    if(_bp.epaisseur_>2)
    {
      /*
      int e=_bp.epaisseur_;
      _g.fillOval(_x1-e/2,_y1-e/2,e,e);
      _g.drawLine(_x1,_y1,_x2,_y2);
      _g.fillOval(_x2-e/2,_y2-e/2,e,e);
      */
      int e=_bp.epaisseur_;
      for(int i=0;i<e;i++)
	for(int j=0;j<e;j++)
	  if(((i!=0)&&(i!=e-1))||((j!=0)&&(j!=e-1)))
	    _g.drawLine(_x1+i-e/2,_y1+j-e/2,_x2+i-e/2,_y2+j-e/2);
    }
  }

  public static void drawLine
    (Graphics _g, int _x1, int _y1, int _x2, int _y2, BresenhamParams _bp)
  {
    bresenham(_g,_x1,_y1,_x2,_y2,_bp);
    bresenham(_g,_x2,_y2,_x2,_y2,_bp);
  }

  public static void drawRect
    (Graphics _g, int _x, int _y, int _w, int _h, BresenhamParams _bp)
  {
    bresenham(_g,_x   ,_y+_h,_x   ,_y   ,_bp);
    bresenham(_g,_x   ,_y   ,_x+_w,_y   ,_bp);
    bresenham(_g,_x+_w,_y   ,_x+_w,_y+_h,_bp);
    bresenham(_g,_x+_w,_y+_h,_x   ,_y+_h,_bp);
  }

  public static void fillRect
    (Graphics _g, int _x, int _y, int _w, int _h)
  {
    _g.fillRect(_x,_y,_w,_h);
  }

  private static Polygon getRoundRect
    (int _x, int _y, int _w, int _h, int _rh, int _rv)
  {
    double CX=2.;
    double CY=2.;
    Polygon p,r;

    if(_rh<1) _rh=1;
    if(_rv<1) _rv=1;
    r=new Polygon();

    p=DjaMatrixHermite.arc2polyline
      (_x,_y+_rv,_x+_rh,_y,0.,-_rv*CX,_rh*CY,0.);
    for(int i=0;i<p.npoints;i++) r.addPoint(p.xpoints[i],p.ypoints[i]);

    p=DjaMatrixHermite.arc2polyline
      (_x+_w-_rh,_y,_x+_w,_y+_rv,_rh*CX,0.,0.,_rv*CY);
    for(int i=0;i<p.npoints;i++) r.addPoint(p.xpoints[i],p.ypoints[i]);

    p=DjaMatrixHermite.arc2polyline
      (_x+_w,_y+_h-_rv,_x+_w-_rh,_y+_h,0.,_rv*CX,-_rh*CY,0.);
    for(int i=0;i<p.npoints;i++) r.addPoint(p.xpoints[i],p.ypoints[i]);

    p=DjaMatrixHermite.arc2polyline
      (_x+_rh,_y+_h,_x,_y+_h-_rv,-_rh*CX,0.,0.,-_rv*CY);
    for(int i=0;i<p.npoints;i++) r.addPoint(p.xpoints[i],p.ypoints[i]);

    return r;
  }

  public static void drawRoundRect
    (Graphics _g, int _x, int _y, int _w, int _h,
     int _rh, int _rv, BresenhamParams _bp)
  {
    DjaGraphics.drawPolygon(_g,getRoundRect(_x,_y,_w,_h,_rh,_rv),_bp);
  }

  public static void fillRoundRect
    (Graphics _g, int _x, int _y, int _w, int _h,
     int _rh, int _rv)
  {
    DjaGraphics.fillPolygon(_g,getRoundRect(_x,_y,_w,_h,_rh,_rv));
  }

  private static Polygon getOval
    (int _x, int _y, int _w, int _h)
  {
    double CX=1.58;
    double CY=1.58;
    Polygon p,r;

    int _rv=_h/2;
    int _rh=_w/2;
    r=new Polygon();

    p=DjaMatrixHermite.arc2polyline
      (_x,_y+_rv,_x+_rh,_y,0.,-_rv*CX,_rh*CY,0.);
    for(int i=0;i<p.npoints;i++) r.addPoint(p.xpoints[i],p.ypoints[i]);

    p=DjaMatrixHermite.arc2polyline
      (_x+_w-_rh,_y,_x+_w,_y+_rv,_rh*CX,0.,0.,_rv*CY);
    for(int i=0;i<p.npoints;i++) r.addPoint(p.xpoints[i],p.ypoints[i]);

    p=DjaMatrixHermite.arc2polyline
      (_x+_w,_y+_h-_rv,_x+_w-_rh,_y+_h,0.,_rv*CX,-_rh*CY,0.);
    for(int i=0;i<p.npoints;i++) r.addPoint(p.xpoints[i],p.ypoints[i]);

    p=DjaMatrixHermite.arc2polyline
      (_x+_rh,_y+_h,_x,_y+_h-_rv,-_rh*CX,0.,0.,-_rv*CY);
    for(int i=0;i<p.npoints;i++) r.addPoint(p.xpoints[i],p.ypoints[i]);

    return r;
  }

  public static void drawOval
    (Graphics _g, int _x, int _y, int _w, int _h, BresenhamParams _bp)
  {
    DjaGraphics.drawPolygon(_g,getOval(_x,_y,_w,_h),_bp);
  }

  public static void fillOval
    (Graphics _g, int _x, int _y, int _w, int _h)
  {
    DjaGraphics.fillPolygon(_g,getOval(_x,_y,_w,_h));
  }

  public static void drawPolyline
    (Graphics _g, Polygon _p, BresenhamParams _bp)
  {
    drawPolyline(_g,_p.xpoints,_p.ypoints,_p.npoints,_bp);
  }

  public static void drawPolyline
    (Graphics _g, int[] _xpoints, int[] _ypoints, int _npoints,
     BresenhamParams _bp)
  {
    int n=_npoints;

    for(int i=1;i<n; i++)
      bresenham(_g,_xpoints[i-1],_ypoints[i-1],
		   _xpoints[i  ],_ypoints[i  ],_bp);

    if(n>0) drawPoint(_g,_xpoints[n-1],_ypoints[n-1],_bp);
  }

  public static void drawPolygon
    (Graphics _g, Polygon _p, BresenhamParams _bp)
  {
    drawPolygon(_g,_p.xpoints,_p.ypoints,_p.npoints,_bp);
  }

  public static void drawPolygon
    (Graphics _g, int[] _xpoints, int[] _ypoints, int _npoints,
     BresenhamParams _bp)
  {
    int i;
    for(i=1;i<_npoints; i++)
      bresenham(_g,_xpoints[i-1],_ypoints[i-1],
		   _xpoints[i  ],_ypoints[i  ],_bp);
    if(i>=2) bresenham(_g,_xpoints[i-1],_ypoints[i-1],
		          _xpoints[0  ],_ypoints[0  ],_bp);
  }

  public static void fillPolygon
    (Graphics _g, Polygon _p)
  {
    _g.fillPolygon(_p);
  }

  public static void fillPolygon
    (Graphics _g, int[] _xpoints, int[] _ypoints, int _npoints)
  {
    _g.fillPolygon(_xpoints,_ypoints,_npoints);
  }

  public static void bresenham
    (Graphics _g, int x1, int y1, int x2, int y2, BresenhamParams bp)
  {
    if((bp.vide_==0)&&(bp.epaisseur_==1))
    {
      _g.drawLine(x1,y1,x2,y2);
      return;
    }

    int xprec=x1;
    int yprec=y1;
    int c, deltaX, deltaY, x, y, incrX, incrY, i;
    deltaX=Math.abs(x2-x1);
    deltaY=Math.abs(y2-y1);
    if( (y2-y1)>0 ) incrY=1;
    else incrY=-1;
    if( (x2-x1)>0 ) incrX=1;
    else incrX=-1;
    if( deltaX>deltaY ) {
      c=2*deltaY-deltaX;
      y=y1;
      x=x1;
      for(i=0; i<deltaX; i++) {
        if( bp.trace_ ) {
          if( bp.compteur_<bp.trait_ ) {
            // _g.drawLine(xprec, yprec, x, y);
	    drawPoint(_g,xprec,yprec,x,y,bp);
            bp.compteur_++;
          }
          else {
            bp.compteur_=0;
            bp.trace_=(bp.vide_<=0); // @GDX false;
          }
        } else {
          if( bp.compteur_<bp.vide_ ) bp.compteur_++;
          else {
            bp.compteur_=0;
            xprec=x;
            yprec=y;
            bp.trace_=true;
          }
        }
        x+=incrX;
        if( c<0 ) c+=(2*deltaY);
        else {
          y+=incrY;
          c+=(2*deltaY-2*deltaX);
        }
      }
    } else {
      c=2*deltaX-deltaY;
      x=x1;
      y=y1;
      for(i=0; i<deltaY; i++) {
        if( bp.trace_ ) {
          if( bp.compteur_<bp.trait_ ) {
            // _g.drawLine(xprec, yprec, x, y);
	    drawPoint(_g,xprec,yprec,x,y,bp);
            bp.compteur_++;
          }
          else {
            bp.compteur_=0;
            bp.trace_=(bp.vide_<=0); // @GDX false;
          }
        } else {
          if( bp.compteur_<bp.vide_ ) bp.compteur_++;
          else {
            bp.compteur_=0;
            xprec=x;
            yprec=y;
            bp.trace_=true;
          }
        }
        y+=incrY;
        if( c<0 ) c+=(2*deltaX);
        else {
          x+=incrX;
          c+=(2*deltaX-2*deltaY);
        }
      }
    }
  }
}
