/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         DjaLib.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.dja;

import com.memoire.fu.*;
import com.memoire.dnd.*;
import com.memoire.bu.*;
import com.memoire.dja.*;
import com.memoire.mst.*;


import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.*;

public class DjaLib
       implements DjaOptions
{
  public static BuInformationsSoftware SOFTWARE=null;

  public final static boolean close(Point p,Point q)
  {
    return (Math.abs(p.x-q.x)<=closeX)&&(Math.abs(p.y-q.y)<=closeY);
  }

  public final static boolean close(int x1, int y1, int x2, int y2)
  {
    return (Math.abs(x1-x2)<=closeX)&&(Math.abs(y1-y2)<=closeY);
  }

  public final static void snap(Point _p)
  {
    if(!snap) return;

    int x=_p.x+deltaX/2;
    int y=_p.y+deltaY/2;
    _p.x=x-x%deltaX;
    _p.y=y-y%deltaY;
  }

  public final static void snap(DjaObject _f)
  {
    if(!snap) return;

    int x=_f.getX()+deltaX/2;
    int y=_f.getY()+deltaY/2;
    x-=x%deltaX;
    y-=y%deltaY;
    _f.setX(x);
    _f.setY(y);

    int w=_f.getWidth() +deltaX/2;
    int h=_f.getHeight()+deltaY/2;
    int dw=x%deltaX; //_f.getX()%deltaX;
    int dh=y%deltaY; //_f.getY()%deltaY;
    if(dw>0) dw=deltaX-dw;
    if(dh>0) dh=deltaY-dh;
    _f.setWidth (w-w%deltaX+dw);
    _f.setHeight(h-h%deltaY+dh);
  }

  public final static boolean close
    (int ox,int oy,int ex, int ey, int px,int py)
  {
    int bx=ex-ox;
    int by=ey-oy;

    if((bx==0)&&(by==0)) return close(ox,oy,px,py);

    int t=0;

    if(Math.abs(bx)>Math.abs(by))
      t=(12800*(px-ox))/bx;
    else
      t=(12800*(py-oy))/by;

    if(t<0) t=0;
    if(t>12800) t=12800;
    int x=ox+(t*bx)/12800;
    int y=oy+(t*by)/12800;
    
    return close(x,y,px,py);
  }

  public final static DjaVector clone(DjaVector s)
  {
    FuHashtableFast c=new FuHashtableFast(s.size());

    for(Enumeration e=s.elements(); e.hasMoreElements(); )
    {
      DjaObject o=(DjaObject)e.nextElement();
      try { c.put(o,o.clone()); }
      catch(CloneNotSupportedException ex) { }
    }

    for(Enumeration e=c.elements(); e.hasMoreElements(); )
    {
      DjaObject o=(DjaObject)e.nextElement();
      if(o instanceof DjaLink)
      {
	DjaLink   a=(DjaLink)o;
	DjaObject t;

	t=a.getBeginObject();
	if(t!=null) t=(DjaObject)c.get(t);
	a.setBeginObject(t);

	t=a.getEndObject();
	if(t!=null) t=(DjaObject)c.get(t);
	a.setEndObject(t);
      }
    }

    DjaVector r=new DjaVector();
    for(Enumeration e=c.elements(); e.hasMoreElements(); )
    {
      DjaObject o=(DjaObject)e.nextElement();
      r.addElement(o);
    }

    return r;
  }

  public final static void setForeground(DjaVector _v, Color _c)
  {
    if(_v==null) return;

    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      f.setForeground(_c);
    }
  }

  public final static void setBackground(DjaVector _v, Color _c)
  {
    if(_v==null) return;

    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      f.setBackground(_c);
    }
  }

  public final static void setColor(DjaVector _v, Color _c)
  {
    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      DjaText[] texts=f.getTexts();
      int n=0;
      for(int i=0;i<texts.length;i++)
	if(texts[i].isSelected())
	  { n++; texts[i].setColor(_c); }
      if(n==0) f.setColor(_c);
    }
  }

  public final static void setFont(DjaVector _v, Font _f)
  {
    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      DjaText[] texts=f.getTexts();
      int n=0;
      for(int i=0;i<texts.length;i++)
	if(texts[i].isSelected())
	  { n++; texts[i].setFont(_f); }
      if(n==0) f.setFont(_f);
    }
  }

  public final static void resizeSelection(DjaGridInteractive _g)
  {
    DjaVector s=_g.getSelection();
    if(s.size()>0)
    {
      for(Enumeration e=s.elements(); e.hasMoreElements(); )
      {
	DjaObject f=(DjaObject)e.nextElement();
	f.setWidth(2*deltaX);
	f.setHeight(2*deltaY);
      }
    }
  }

  public final static void setForeground(DjaGridInteractive _g)
  {
    DjaVector s=_g.getSelection();
    if(s.size()>0)
    {
      Color fg=Color.black;
      fg=JColorChooser.showDialog(_g,"Foreground",fg);
      if(fg!=null)
	  setForeground(s,fg);
    }
  }

  public final static void setBackground(DjaGridInteractive _g)
  {
    DjaVector s=_g.getSelection();
    if(s.size()>0)
    {
      Color bg=Color.white;
      bg=JColorChooser.showDialog(_g,"Background",bg);
      if(bg!=null)
	  setBackground(s,bg);
    }
  }

  public final static void setColor(DjaGridInteractive _g)
  {
    DjaVector s=_g.getSelection();
    if(s.size()>0)
    {
      Color tc=Color.black;
      tc=JColorChooser.showDialog(_g,"Text Color",tc);
      if(tc!=null)
	  setColor(s,tc);
    }
  }

  public final static void setFont(DjaGridInteractive _g)
  {
    DjaVector s=_g.getSelection();
    if(s.size()>0)
    {
      Font tf=defaultPlainFont;
      BuFontChooser d=new BuFontChooser
	  (null,DjaLib.SOFTWARE,"Text Font",tf);
      d.activate();
      tf=d.getValue();
      if(tf!=null)
	  setFont(s,tf);
    }
  }

  public final static void addText(DjaGridInteractive _g)
  {
    DjaVector s=_g.getSelection();
    if(s.size()>0)
    {
      for(Enumeration e=s.elements(); e.hasMoreElements(); )
      {
	DjaObject f=(DjaObject)e.nextElement();
	f.addText((String)null);
      }
    }
  }

  public final static void removeText(DjaGridInteractive _g)
  {
    DjaVector s=_g.getSelection();
    if(s.size()>0)
    {
      for(Enumeration e=s.elements(); e.hasMoreElements(); )
      {
	DjaObject f=(DjaObject)e.nextElement();
	DjaText[] t=f.getTextArray();
	int n=0;
	for(int i=0;i<t.length;i++)
	    if(t[i].isSelected()) n++;
	DjaText[] r=new DjaText[t.length-n];
	n=0;
	for(int i=0;i<t.length;i++)
	    if(!t[i].isSelected())
	    {
	      r[n]=t[i];
	      r[n].setNum(n);
	      n++;
	    }
	f.setTextArray(r);
      }
    }
  }

  public final static void setProperty(DjaVector _v, String _n, String _s)
  {
    if(_v==null) return;

    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      f.putProperty(_n,_s);
    }
  }

  public final static void setBeginType(DjaVector _v, int _t)
  {
    if(_v==null) return;

    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      if(f instanceof DjaLink)
	((DjaLink)f).setBeginType(_t);
    }
  }

  public final static void setEndType(DjaVector _v, int _t)
  {
    if(_v==null) return;

    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      if(f instanceof DjaLink)
	((DjaLink)f).setEndType(_t);
    }
  }

  public final static void setBeginO(DjaVector _v, int _o)
  {
    if(_v==null) return;

    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      if(f instanceof DjaLink)
	((DjaLink)f).setBeginO(_o);
    }
  }

  public final static void setEndO(DjaVector _v, int _o)
  {
    if(_v==null) return;

    for(Enumeration e=_v.elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();
      if(f instanceof DjaLink)
	((DjaLink)f).setEndO(_o);
    }
  }

  /*
    // test contain
    _g.setColor(new Color(224,192,128));
    int xx=getX();
    int yy=getY();
    for(int i=0;i<getWidth();i++)
      for(int j=0;j<getHeight(); j++)
	if(contains(xx+i,yy+j))
	  _g.drawLine(xx+i,yy+j,xx+i,yy+j);
  */


  final public static String[] DJA_CLASSES=new String[]
  {
    "com.memoire.dja.DjaObject",
    "com.memoire.dja.DjaLink",
    "com.memoire.dja.DjaPaletteBracket",
    "com.memoire.dja.DjaPaletteThickness",
    "com.memoire.dja.DjaGraphics",
    "com.memoire.dja.DjaPaletteStroke",
    "com.memoire.dja.DjaPaletteUml",
    "com.memoire.dja.DjaPaletteLink",
    "com.memoire.dja.DjaPaletteForm",
    "com.memoire.dja.DjaPreferences",
    "com.memoire.dja.DjaResource",
    "com.memoire.dja.DjaResource",
    "com.memoire.dja.DjaPreferences",
    "com.memoire.dja.DjaPaletteForm",
    "com.memoire.dja.DjaPaletteLink",
    "com.memoire.dja.DjaPaletteBracket",
    "com.memoire.dja.DjaGridInteractive",
    "com.memoire.dja.DjaGrid",
    "com.memoire.dja.DjaMouseListener",
    "com.memoire.dja.DjaKeyListener",
    "com.memoire.dja.DjaFrame",
    "com.memoire.dja.DjaLoader",
    "com.memoire.dja.DjaLink",
    "com.memoire.dja.DjaBrokenArrow",
    "com.memoire.dja.DjaGroup",
    "com.memoire.dja.DjaDiamond",
    "com.memoire.dja.DjaObject",
    "com.memoire.dja.DjaForm",
    "com.memoire.dja.DjaBox",
    "com.memoire.dja.DjaAnchor",
    "com.memoire.dja.DjaOptions",
    "com.memoire.dja.DjaLib",
    "com.memoire.dja.DjaImplementation",
    "com.memoire.dja.Dja",
  };
}
