/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         DjaLoadSaveDia.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.dja;

import com.memoire.fu.*;
import com.memoire.dnd.*;
import com.memoire.bu.*;
import com.memoire.dja.*;
import com.memoire.mst.*;


import java.awt.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;

public class DjaLoadSaveDia
       implements DjaLoadSaveInterface
{
  public boolean canLoad() { return true;  }
  public boolean canSave() { return false; }

  public DjaVector load(File _file)
    throws Exception
  {
    InputStream input=new BufferedInputStream
      (new FileInputStream(_file));

    /*
    Enumeration e=System.getProperties().keys();
    while(e.hasMoreElements())
    {
      String k=(String)e.nextElement();
      System.err.println(k+"="+System.getProperty(k));
    }
    System.err.println("ENCODING="+System.getProperty("file.encoding"));
    */

    if(FuLib.isGziped(_file.getAbsolutePath()))
    {
      //System.err.println("the file "+_file.getPath()+" is gziped.");
      input=new GZIPInputStream(input);
    }
    //else System.err.println("the file "+_file.getPath()+" is NOT gziped.");

    String url=_file.getAbsolutePath();
    if(url.indexOf(':')<0) url="file://"+new File(url).getAbsolutePath();

    DjaConvertDia handler=new DjaConvertDia(_file.getAbsolutePath());

    MstXmlParser parser=new MstXmlParser();
    parser.setHandler(handler);
    parser.parse(url,null,input,null); 
    //try { parser.parse(url,null,input,null); }
    //catch (Exception ex) { ex.printStackTrace(); }

    DjaVector r=handler.getVector();
    if(r!=null) connect(r);
    return r;
  }

  public void connect(DjaVector _g)
  {
    Enumeration eo=_g.enumerate(DjaObject.class);
    Enumeration el=_g.enumerate(DjaLink.class);

    while(el.hasMoreElements())
    {
      DjaLink     l=(DjaLink)el.nextElement();
      DjaAttach[] t=l.getAttachs();

      while(eo.hasMoreElements())
      {
	DjaObject   o=(DjaObject)eo.nextElement();
	DjaAnchor[] a=o.getAnchors(); 
	if(o!=l)
	{
	  for(int i=0;i<t.length;i++)
	    for(int j=0;j<a.length;j++)
	      if(DjaLib.close(t[i].getX(),t[i].getY(),a[j].getX(),a[j].getY()))
	      {
		if(t[i].getP()==0)
		{
		  l.setBeginObject  (o);
		  l.setBeginPosition(a[j].getPosition());
		}
		else
		{
		  l.setEndObject  (o);
		  l.setEndPosition(a[j].getPosition());
		}
	      }
	}
      }
    }
  }

  public void save(File _file, DjaVector _vector)
    throws Exception
  {
    throw new Exception("Not yet implemented");
  }

  public BuFileFilter getFilter()
  {
    return new BuFileFilter
      (new String[] { "dia","DIA" },
      "Gnome Dia files");
  }
}
