/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         DjaMouseListener.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.dja;

import com.memoire.fu.*;
import com.memoire.bu.*;
import com.memoire.dja.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

public class DjaMouseListener
       implements DjaOptions,
                  MouseListener, MouseMotionListener,
	          KeyListener
                  // not serializable
{
  private int xm_,ym_;
  private boolean ctrl_,shift_;

  private DjaGridInteractive grid_;
  private DjaObject          last_;
  private int                lastdx_,lastdy_;
  private int                lastx_,lasty_,lastw_,lasth_;
  private DjaHandle          handle_;
  private DjaControl         control_;
  private DjaText            text_;
  private boolean            linkbegin_;
  private boolean            linkend_;

  public DjaMouseListener(DjaGridInteractive _grid)
  {
    grid_=_grid;
  }

  public void keyTyped   (KeyEvent _evt) { }
  public void keyPressed (KeyEvent _evt)
  {
      if(_evt.isControlDown()) ctrl_ =true;
      if(_evt.isShiftDown  ()) shift_=true;
      adjustCursor();
  }
  public void keyReleased(KeyEvent _evt)
  {
      if(_evt.isControlDown()) ctrl_ =false;
      if(_evt.isShiftDown  ()) shift_=false;
      adjustCursor();
  }

  public void mouseClicked(MouseEvent _evt)
  {
    Object o=DjaLibDetect.detect(grid_,_evt.getX(),_evt.getY());

    if(o instanceof DjaHandle ) return;
    if(o instanceof DjaControl) return;
    if(o instanceof DjaAttach ) return;
    if(o instanceof DjaAnchor ) return;
    if(linkbegin_||linkend_   ) return;

    if(o instanceof DjaText)
    {
	DjaText t=(DjaText)o;

	if(_evt.isControlDown())
	{
	}
	else
	if(_evt.isShiftDown())
	{
	    t.setSelected(!t.isSelected());
	}
	else
	{
	    String s=t.getText();

	    if(t.isMultiline())
	    {
		BuDialogMulti d=new BuDialogMulti
		    (null,DjaLib.SOFTWARE,"Text","Edit the text:",s);
		d.activate();
		s=d.getValue();
	    }
	    else
	    {
		BuDialogInput d=new BuDialogInput
		    (null,DjaLib.SOFTWARE,"Text","Edit the text:",s);
		d.activate();
		s=d.getValue();
	    }

	    if(s!=null)
	    {
		if("".equals(s)) s=null;
		/*
		  if(t.getNum()==-1)
		  t.getParent().setText(s,0);
		  else
		*/
		t.setText(s);
	    }
	}

	last_=null;
	adjustCursor(_evt);
	grid_.repaint();
	return;
    }

    if(last_!=null)
    {
      if(_evt.isShiftDown()||_evt.isControlDown())
      {
	if(!last_.isSelected()) grid_.addSelection(last_);
	else                    grid_.removeSelection(last_);
      }
      else
      {
	if(!last_.isSelected()) grid_.setSelection(last_);
	else                    grid_.clearSelection();
      }

      last_=null;
    }

    adjustCursor(_evt);
  }

  public void mousePressed(MouseEvent _evt)
  {
    int x=_evt.getX();
    int y=_evt.getY();
    Object o=DjaLibDetect.detect(grid_,x,y);

    grid_.requestFocus();

    last_     =null;
    handle_   =null;
    control_  =null;
    text_     =null;
    linkbegin_=false;
    linkend_  =false;

    if(o instanceof DjaHandle)
    {
      handle_=(DjaHandle)o;
      last_=handle_.getParent();
      lastdx_=x;
      lastdy_=y;
      lastx_=last_.getX();
      lasty_=last_.getY();
      lastw_=last_.getWidth();
      lasth_=last_.getHeight();
      return;
    }
    else
    if(o instanceof DjaControl)
    {
      control_=(DjaControl)o;
      last_=control_.getParent();
      lastdx_=x;
      lastdy_=y;
      lastx_=last_.getX();
      lasty_=last_.getY();
      lastw_=last_.getWidth();
      lasth_=last_.getHeight();
      return;
    }
    else
    if(o instanceof DjaAnchor)
    {
      return;
    }
    else
    if(o instanceof DjaText)
    {
      text_=(DjaText)o;
      Point q=text_.getLocation();
      last_=text_.getParent();
      lastx_=last_.getX();
      lasty_=last_.getY();
      lastw_=last_.getWidth();
      lasth_=last_.getHeight();
      lastdx_=x-q.x+lastx_;
      lastdy_=y-q.y+lasty_;
      return;
    }
    else
    if(o instanceof DjaAttach)
    {
      DjaAttach a=(DjaAttach)o;
      DjaObject w=a.getParent();
      if(w instanceof DjaLink)
      {
	  DjaLink l=(DjaLink)w;
	  if(a.getP()==0)
	  {
	    last_=l;
	    lastdx_=a.getX()-x;
	    lastdy_=a.getY()-y;
	    linkbegin_=true;
	  }
	  else
	  if(a.getP()==1)
	  {
	    last_=l;
	    lastdx_=a.getX()-x;
	    lastdy_=a.getY()-y;
	    linkend_  =true;
	  }
      }

      adjustCursor(_evt);
      return;
    }
    else
    if(o instanceof DjaObject)
    {
	DjaObject f=(DjaObject)o;
	last_=f;
	lastx_=f.getX();
	lasty_=f.getY();
	lastw_=f.getWidth();
	lasth_=f.getHeight();
	lastdx_=f.getX()-x;
	lastdy_=f.getY()-y;
    }

    if(last_==null)
    {
      lastx_=x;
      lasty_=y;
      lastw_=0;
      lasth_=0;

      Graphics g=grid_.getGraphics();
      g.setXORMode(selectionZone);
      g.drawRect(lastx_,lasty_,lastw_,lasth_);
      g.setPaintMode();
    }

    adjustCursor(_evt);
  }

  public void mouseReleased(MouseEvent _evt)
  {
    if(last_!=null)
    {
      int x,y;

      if(last_ instanceof DjaLink)
      {
	DjaLink a=(DjaLink)last_;

	if(linkbegin_)
	{
	  x=a.getBeginX();
	  y=a.getBeginY();
	  Object o=DjaLibDetect.detect(grid_,x,y,a);
	  if(o instanceof DjaAnchor)
	  {
	    DjaAnchor n=(DjaAnchor)o;
	    a.setBeginObject  (n.getParent());
	    a.setBeginPosition(n.getPosition());
	  }
	  else
	  {
	    Point p=new Point(x,y);
	    DjaLib.snap(p);
	    a.setBeginX(p.x);
	    a.setBeginY(p.y);
	  }
	}
	else
	if(linkend_)
	{
	  x=a.getEndX();
	  y=a.getEndY();
	  Object o=DjaLibDetect.detect(grid_,x,y,a);
	  if(o instanceof DjaAnchor)
	  {
	    DjaAnchor n=(DjaAnchor)o;
	    a.setEndObject  (n.getParent());
	    a.setEndPosition(n.getPosition());
	  }
	  else
	  {
	    Point p=new Point(x,y);
	    DjaLib.snap(p);
	    a.setEndX(p.x);
	    a.setEndY(p.y);
	  }
	}
	else
	  DjaLib.snap(a);
      }
      else
	DjaLib.snap(last_);

      /*
      x=last_.getX()+deltaX/2;
      y=last_.getY()+deltaY/2;
      x-=x%deltaX;
      y-=y%deltaY;
      last_.setX(x);
      last_.setY(y);

      int w=last_.getWidth()+deltaX/2;
      int h=last_.getHeight()+deltaY/2;
      w-=w%deltaX;
      h-=h%deltaY;
      last_.setWidth(w);
      last_.setHeight(h);
      */

      grid_.repaint();
    }
    else
    {
      Graphics g=grid_.getGraphics();
      g.setXORMode(selectionZone);
      g.drawRect(lastx_,lasty_,lastw_,lasth_);
      g.setPaintMode();

      Rectangle z=new Rectangle(lastx_,lasty_,lastw_,lasth_);
      DjaVector v=new DjaVector();
      for(Enumeration e=grid_.getObjects().elements(); e.hasMoreElements(); )
      {
	DjaObject f=(DjaObject)e.nextElement();
	if(z.union(f.getBounds()).equals(z))
	  v.addElement(f);
      }
      grid_.setSelection(v);
    }

    adjustCursor(_evt);
  }

  public void mouseEntered(MouseEvent _evt)
  { }

  public void mouseExited(MouseEvent _evt)
  { }

  public void mouseMoved(MouseEvent _evt)
  { adjustCursor(_evt); }

  public void mouseDragged(MouseEvent _evt)
  {
    boolean erase=true;

    int x=_evt.getX();
    int y=_evt.getY();

    if(handle_!=null)
    {
      DjaForm f=(DjaForm)handle_.getParent();

      int w=f.getWidth();
      int h=f.getHeight();

      // System.err.println("EVT_X="+x);
      // System.err.println("EVT_Y="+y);
      // System.err.println("LST_X="+lastdx_);
      // System.err.println("LST_Y="+lastdy_);
      // System.err.println("");

      switch(handle_.getO())
      {
      case NORTH:
	f.setY(lasty_+Math.min(lasth_,y-lastdy_));
	f.setHeight(Math.max(0,lasth_-y+lastdy_));
	break;
      case WEST:
	f.setX(lastx_+Math.min(lastw_,x-lastdx_));
	f.setWidth(Math.max(0,lastw_-x+lastdx_));
	break;
      case EAST:
	f.setWidth(Math.max(0,lastw_+x-lastdx_));
	break;
      case SOUTH:
	f.setHeight(Math.max(0,lasth_+y-lastdy_));
	break;
      case NORTH_WEST:
	f.setX(lastx_+Math.min(lastw_,x-lastdx_));
	f.setY(lasty_+Math.min(lasth_,y-lastdy_));
	f.setWidth (Math.max(0,lastw_-x+lastdx_));
	f.setHeight(Math.max(0,lasth_-y+lastdy_));
	break;
      case NORTH_EAST:
	f.setY(lasty_+Math.min(lasth_,y-lastdy_));
	f.setWidth (Math.max(0,lastw_+x-lastdx_));
	f.setHeight(Math.max(0,lasth_-y+lastdy_));
	break;
      case SOUTH_EAST:
	f.setWidth (Math.max(0,lastw_+x-lastdx_));
	f.setHeight(Math.max(0,lasth_+y-lastdy_));
	break;
      case SOUTH_WEST:
	f.setX(lastx_+Math.min(lastw_,x-lastdx_));
	f.setWidth (Math.max(0,lastw_-x+lastdx_));
	f.setHeight(Math.max(0,lasth_+y-lastdy_));
	break;
      }
      grid_.repaint();
    }
    else
    if(control_!=null)
    {
      control_.draggedTo(x,y);
      grid_.repaint();
    }
    else
    if(text_!=null)
    {
	if(_evt.isControlDown())
	{
	  if(text_.isMoveable())
	  {
	    Point p=new Point(lastx_+x-lastdx_,lasty_+y-lastdy_);
	    //DjaLib.snap(p);
	    text_.setLocation(p);
	    text_.setSelected(true);
	    grid_.repaint();
	  }
	}
    }
    else
    if(last_!=null)
    {
      if(last_ instanceof DjaForm)
      {
	if(!last_.isSelected())
	{
	  DjaForm f=(DjaForm)last_;
	  f.setX(x+lastdx_);
	  f.setY(y+lastdy_);
	}
	else
	{
	  grid_.moveSelection(x-lastx_+lastdx_,y-lasty_+lastdy_);
	  lastx_=last_.getX();
	  lasty_=last_.getY();
	}
      }
      else
      if(last_ instanceof DjaLink)
      {
	DjaLink a=(DjaLink)last_;
	if(linkend_)
	{
	  a.setEndObject(null);
	  a.setEndX(x+lastdx_);
	  a.setEndY(y+lastdy_);
	}
	else
	if(linkbegin_)
	{
	  a.setBeginObject(null);
	  a.setBeginX(x+lastdx_);
	  a.setBeginY(y+lastdy_);
	}
	else
	{
	  a.setX(x+lastdx_);
	  a.setY(y+lastdy_);
	}
      }

      // essai
      /*
      if(!last_.isSelected()&&!_evt.isShiftDown()&&!_evt.isControlDown())
	grid_.setSelection(last_);
      */

      grid_.repaint();
    }
    else
      erase=true;

    Graphics g=grid_.getGraphics();
    g.setXORMode(selectionZone);

    if(last_==null)
    {
      if(erase) g.drawRect(lastx_,lasty_,lastw_,lasth_);
      lastw_=x-lastx_;
      lasth_=y-lasty_;
      g.drawRect(lastx_,lasty_,lastw_,lasth_);
      g.setPaintMode();
    }
  }

  /*
  protected DjaAnchor detectAnchor(MouseEvent _evt)
  { return detectAnchor(_evt.getX(),_evt.getY(),null); }

  protected DjaAnchor detectAnchor(int x, int y, DjaObject _notthis)
  {
    for(Enumeration e=grid_.getObjects().elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();

      if(f==_notthis) continue;
      if(f instanceof DjaLink) continue;

      DjaAnchor[] l=f.getAnchors();

      for(int i=0;i<l.length;i++)
      {
	if(DjaLib.close(l[i].getX(),l[i].getY(),x,y))
	  return l[i];
      }
    }

    return null;
  }

  protected DjaAttach detectAttach(MouseEvent _evt)
  { return detectAttach(_evt.getX(),_evt.getY(),null); }

  protected DjaAttach detectAttach(int x, int y, DjaObject _notthis)
  {
    for(Enumeration e=grid_.getObjects().elements(); e.hasMoreElements(); )
    {
      DjaObject f=(DjaObject)e.nextElement();

      if(f==_notthis) continue;
      //if(f instanceof DjaLink) continue;

      DjaAttach[] l=f.getAttachs();

      for(int i=0;i<l.length;i++)
      {
	if(DjaLib.close(l[i].getX(),l[i].getY(),x,y))
	  return l[i];
      }
    }

    return null;
  }

  protected DjaHandle detectHandle(MouseEvent _evt)
  {
    int x=_evt.getX();
    int y=_evt.getY();

    for(Enumeration e=grid_.getSelection().elements(); e.hasMoreElements(); )
    {
      DjaObject   f=(DjaObject)e.nextElement();
      DjaHandle[] l=f.getHandles();

      for(int i=0;i<l.length;i++)
      {
	if(DjaLib.close(l[i].getX(),l[i].getY(),x,y))
	{
	  last_=f;
	  lastdx_=_evt.getX();
	  lastdy_=_evt.getY();
	  lastx_=f.getX();
	  lasty_=f.getY();
	  lastw_=f.getWidth();
	  lasth_=f.getHeight();
	  return l[i];
	}
      }
    }

    return null;
  }

  protected DjaControl detectControl(MouseEvent _evt)
  {
    int x=_evt.getX();
    int y=_evt.getY();

    for(Enumeration e=grid_.getSelection().elements(); e.hasMoreElements(); )
    {
      DjaObject    f=(DjaObject)e.nextElement();
      DjaControl[] l=f.getControls();

      for(int i=0;i<l.length;i++)
      {
	if(DjaLib.close(l[i].getX(),l[i].getY(),x,y))
	{
	  last_=f;
	  lastdx_=_evt.getX();
	  lastdy_=_evt.getY();
	  lastx_=f.getX();
	  lasty_=f.getY();
	  lastw_=f.getWidth();
	  lasth_=f.getHeight();
	  return l[i];
	}
      }
    }

    return null;
  }

  protected DjaObject detectText(MouseEvent _evt)
  {
    int x=_evt.getX();
    int y=_evt.getY();

    for(Enumeration e=grid_.getObjects().elements(); e.hasMoreElements(); )
    {
      DjaObject o=(DjaObject)e.nextElement();
      if(o.textContains(x,y)) return o;
    }

    return null;
  }

  protected DjaObject detectObject(MouseEvent _evt)
  {
    int x=_evt.getX();
    int y=_evt.getY();

    for(Enumeration e=grid_.getObjects().elements(); e.hasMoreElements(); )
    {
      DjaObject o=(DjaObject)e.nextElement();
      if(o.contains(x,y)) return o;
    }

    return null;
  }
  */

  protected void adjustCursor()
  {
    adjustCursor(xm_,ym_,shift_,ctrl_);
  }

  protected void adjustCursor(MouseEvent _evt)
  {
    xm_=_evt.getX();
    ym_=_evt.getY();
    boolean shift=_evt.isShiftDown();
    boolean ctrl =_evt.isControlDown();
    adjustCursor(xm_,ym_,shift,ctrl);
  }

  protected void adjustCursor(int _x,int _y,boolean _shift,boolean _ctrl)
  {
    int r=Cursor.CROSSHAIR_CURSOR;

    Object o=DjaLibDetect.detect(grid_,_x,_y);

    if(o instanceof DjaHandle)
    {
      DjaHandle h=(DjaHandle)o;
      switch(h.getO())
      {
      case NORTH:      r=Cursor.N_RESIZE_CURSOR; break;
      case EAST:       r=Cursor.E_RESIZE_CURSOR; break;
      case SOUTH:      r=Cursor.S_RESIZE_CURSOR; break;
      case WEST:       r=Cursor.W_RESIZE_CURSOR; break;
      case NORTH_EAST: r=Cursor.NE_RESIZE_CURSOR; break;
      case SOUTH_EAST: r=Cursor.SE_RESIZE_CURSOR; break;
      case SOUTH_WEST: r=Cursor.SW_RESIZE_CURSOR; break;
      case NORTH_WEST: r=Cursor.NW_RESIZE_CURSOR; break;
      }
    }
    else
    if(o instanceof DjaControl)
    {
	r=Cursor.MOVE_CURSOR;
    }
    else
    if(o instanceof DjaAttach)
    {
	r=Cursor.MOVE_CURSOR;
    }
    else
    if(o instanceof DjaAnchor)
    {
    }
    else
    if(o instanceof DjaText)
    {
	if(_ctrl)
	{
	  if(((DjaText)o).isMoveable())
	    r=Cursor.MOVE_CURSOR;
	  else
	    r=Cursor.CROSSHAIR_CURSOR;
	}
	else
	if(_shift)
	    r=Cursor.CROSSHAIR_CURSOR;
	else
	    r=Cursor.TEXT_CURSOR;
    }
    else
    if(o instanceof DjaObject)
    {
      //if(((DjaObject)o).isSelected())
      r=Cursor.MOVE_CURSOR;
    }

    Cursor c=Cursor.getPredefinedCursor(r);
    if(grid_.getCursor()!=c) grid_.setCursor(c);
  }
}
