/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         DndSource.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.dnd;
import com.memoire.dnd.*;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;

public class DndSource
implements MouseListener, MouseMotionListener
{
  private final static int WI=26+12;
  private final static int HI=26+12;

  private DndRequestData request_;

  private int            counter_;
  private Cursor         cursor_;
  private boolean        dragging_;
  private DndTarget      target_;
  private JComponent     over_;
  private Point          loc_;
  private Object[]       values_;

  public DndSource(DndRequestData _request)
  {
    request_ =_request;

    dragging_=false;
    target_  =null;
    over_    =null;
    loc_     =null;
    values_  =null;
  }

  public void add(JComponent _comp)
  {
    if(_comp.getClientProperty("DND_SOURCE")==null)
    {
      _comp.addMouseListener(this);
      _comp.addMouseMotionListener(this);
      _comp.putClientProperty("DND_SOURCE",this);
    }
    else
      throw new RuntimeException
	("Already a Dnd Source");
  }

  /*
    public void remove(JComponent _comp)
    {
    _comp.removeMouseListener(this);
    _comp.removeMouseMotionListener(this);
    }
    */

  public void mouseClicked(MouseEvent _evt)
  {
  }
    
  public void mouseEntered(MouseEvent _evt)
  {
  }
    
  public void mouseExited(MouseEvent _evt)
  {
  }
    
  public void mousePressed(MouseEvent _evt)
  {
    counter_=0;
  }
    
  public void mouseReleased(MouseEvent _evt)
  {
    JComponent s=(JComponent)_evt.getSource();
    s.setCursor(cursor_);
    s.putClientProperty("DND_EVENTS",null);

    if((over_!=null)&&(target_!=null)&&(values_!=null)&&(loc_!=null))
      target_.drop(values_,over_,
		   SwingUtilities.convertPoint
		   (s,_evt.getX(),_evt.getY(),over_));

    Component top=s.getTopLevelAncestor();
    if(top instanceof JFrame)
      top=((JFrame)top).getContentPane();
    if(top instanceof JComponent)
      ((JComponent)top).paintImmediately(ox,oy,WI,HI);

    over_=null;
    target_=null;
    values_=null;
    loc_=null;
    dragging_=false;
    _evt.consume();

    /*
      if(top instanceof BuApplication)
      {
      BuApplication a=(BuApplication)top;
      a.setGlassPane(glass_);
      }
      */
  }
    
  public void mouseMoved(MouseEvent _evt)
  {
  }    

  private int ox,oy;  //,ow,oh;
  private Icon icon_; //new ImageIcon("texte_24.gif");

  //private static Icon noDropIcon =new ImageIcon("dnd-nodrop_24.gif");
  //private static Icon canDropIcon=new ImageIcon("dnd-candrop_24.gif");

  //private Component glass_=null;

  public void mouseDragged(MouseEvent _evt)
  {
    counter_++;
    if(counter_<5) return;

    /*
      JComponent s=(JComponent)_evt.getSource();
      if(counter_==5)
      {
      Component top=s.getTopLevelAncestor();
      if(top instanceof BuApplication)
      {
      BuApplication a=(BuApplication)top;
      System.err.println("PUT DND GLASS");
      glass_=a.getGlassPane();
      BuGlassPaneStop gp=new BuGlassPaneStop(a);
      a.setGlassPane(gp);
      gp.setVisible(true);
      gp.revalidate();
      }
      }
      */

    JComponent s=(JComponent)_evt.getSource();
    int x=_evt.getX();
    int y=_evt.getY();

    if(!dragging_)
    {
      dragging_=true;
      cursor_=s.getCursor();
      s.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
      values_=request_.request(s,new Point(x,y));
      if(DndIcons.isIconDisplayed())
	icon_=DndIcons.getIcon(values_);
      // ow=icon_.getIconWitdh ();
      // oh=icon_.getIconHeight();
      //if(values_.length>0)
      //  System.err.println("VALUE="+values_[0]);
      /*
	for(int i=0;i<values_.length;i++)
	System.err.println("VALUE["+i+"]="+values_[i]);
	*/

      s.putClientProperty("DND_EVENTS",this);
    }

    Component top=s.getTopLevelAncestor();
    if(top instanceof JFrame) top=((JFrame)top).getContentPane();

    Point p=SwingUtilities.convertPoint(s,x,y,top);

    if(!top.getBounds().contains(p))
      over_=null;
    else
    {
      //Component c=SwingUtilities.getDeepestComponentAt(top,p.x,p.y);
      //Point     q=SwingUtilities.convertPoint(top,p.x,p.y,c);
      Component c=top;
      Point     q=p;
      boolean   again=true;
      
      while(again)
      {
	Component d=c;
	again=false;
	
	if(c instanceof JRootPane)
	  c=((JRootPane)c).getContentPane();
	else
	if(c instanceof JTabbedPane)
	  c=((JTabbedPane)c).getSelectedComponent();
	else
	if(c instanceof Container)
	  c=((Container)c).getComponentAt(q.x,q.y);
	
	if((c==null)||!c.isVisible())
	  c=d;
	
	if(c!=d)
	{
	  q=SwingUtilities.convertPoint(d,q.x,q.y,c);
	  again=true;
	}
	
	if(c instanceof JInternalFrame)
	{
	  JInternalFrame frame  =(JInternalFrame)c;
	  JDesktopPane   desktop=frame.getDesktopPane();
	  if((desktop!=null)&&(desktop.getPosition(frame)!=0))
	    frame.moveToFront();
	}

	JComponent jc=((c instanceof JComponent)
		       ? (JComponent)c
		       : null);

	if((jc!=null)&&(jc.getClientProperty("DND_TARGET")!=null))
	{
	  /*
	  if(jc!=over_)
	  {
	    System.err.println("Over "+c.getName()+
			       ":"+c.getClass().getName()+
			       " xy="+q.x+","+q.y);
	  }
	  */

	  over_  =jc;
	  target_=(DndTarget)over_.getClientProperty("DND_TARGET");

	  if(  !target_.isAcceptable(values_)
	     ||!over_.isEnabled())
	  {
	    loc_ =null;
	    again=(c!=d);
	  }
	  else
	  {
	    loc_ =q;
	    again=false;
	  }
	}
	else
	if(c instanceof JInternalFrame.JDesktopIcon)
	{
	  JInternalFrame frame=((JInternalFrame.JDesktopIcon)c)
	    .getInternalFrame();
	  try { frame.setIcon(false); }
	  catch(PropertyVetoException ex) { }
	}
      }
    }
      
      //System.err.println("XY="+q.x+","+q.y);
      //System.err.println("Over "+c.getClass().getName());

      /*
      if(c!=over_)
      {
	if(c instanceof JComponent)
	  over_=(JComponent)c;
	else
	  over_=null;

	  System.err.println("Over "+c.getName()+
	  ":"+c.getClass().getName()+
	  " xy="+q.x+","+q.y);
	
	if(over_!=null)
	{
	  target_=(DndTarget)
	    over_.getClientProperty("DND_TARGET");
	  if(target_!=null)
	  {
	    if(  !target_.isAcceptable(values_)
	       ||!over_.isEnabled())
	    {
	      // over_=null;
	      // target_=null;
	      loc_=null;
	      again=true;
	      //System.err.println("reject drop !");
	    }
	    else
	    {
	      loc_=q;
	      //SwingUtilities.convertPoint(s,p.x,p.y,over_);
	      //System.err.println("can drop !");
	    }
	  }
	  
	  //if(target_==null)
	  //s.setCursor(new Cursor(Cursor.HAND_CURSOR));
	  //else
	  //s.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	}
      }
      */
	
    if(  DndIcons.isIconDisplayed()
       &&(top instanceof JComponent))
    {
      try
      {
	((JComponent)top).paintImmediately(ox,oy,WI,HI);
	ox=p.x; oy=p.y;

	Graphics g=top.getGraphics();
	if(icon_!=null)
	  icon_.paintIcon(top,g,ox,oy);
	
	if((over_!=null)&&(target_!=null))
	{
	  if(loc_==null)
	  {
	    Icon icon=DndIcons.getNoDropIcon();
	    if(icon!=null) icon.paintIcon(top,g,ox,oy);
	  }
	  else
	  {
	    Icon icon=DndIcons.getCanDropIcon();
	    if(icon!=null) icon.paintIcon(top,g,ox,oy);
	  }
	}
      }
      catch(Exception ex) { }
    }

    _evt.consume();
  }
}
