/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         EditorInterface.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:08 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.editor;
import com.memoire.editor.*;


import java.io.*;
import java.lang.reflect.*;

public interface EditorInterface
       extends Runnable
{
  /**
   * Returns the common name, for example: "Emacs", "JExt", ...
   */
  String getName();

  /**
   * Returns a short description, for tooltips.
   */
  String getShortDescription();

  /**
   * Returns a long description.
   */
  String getLongDescription();

  /**
   * Returns an Icon.
   */
  Object getIcon();


  /**
   * Returns the GUI if available.
   */
  Object getGUI();

  /**
   * Returns true if scrollbars should be added.
   */
  boolean needScrollBars();

  /**
   * Returns true if action is available.
   */
  boolean can(String _action);

  /**
   * Do the action.
   */
  void act(String _action);

  /**
   * Initialises the editor.
   */
  void start();

  /**
   * Runs the editor.
   */
  void run();

  /**
   * Exits the editor (if possible)
   */
  void exit();

  /**
   * Loads some files.
   */
  // void load(InputStream _in);
  // void load(String      _filename);
  // void load(File        _file);
  // void load(URL         _url);

  /**
   * Set/get the text.
   */
  void setText(String _text);
  String getText();

  /**
   * Set/get the syntax.
   */
  void setSyntax(String _syntax);
  String getSyntax();

  /**
   * Set/is editable.
   */
  void setEditable(boolean _editable);
  boolean isEditable();

}
