/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         EditorJedittextarea.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:08 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.editor;
import com.memoire.editor.*;
import com.memoire.jedit.*;

import java.awt.*;
import java.io.*;

import javax.swing.*;
import javax.swing.text.*;

// import org.gjt.sp.jedit.syntax.*;
// import org.gjt.sp.jedit.textarea.*;

public class EditorJedittextarea
       implements EditorInterface
{
  private JEditTextArea ta_ =null;

  public String getName()
  {
    return "JEditTextArea";
  }

  public String getShortDescription()
  {
    return "JEditTextArea is a powerfull text editor component";
  }

  public String getLongDescription()
  {
    return "JEditTextArea is a powerfull text editor component "+
           "written by Slava Pestov.";
  }

  public Object getIcon()
  {
    return null;
  }

  public boolean can(String _action)
  {
    boolean r=false;
    if("copy"   .equals(_action)) r=true;
    if("cut"    .equals(_action)) r=true;
    if("paste"  .equals(_action)) r=true;
    return r;
  }

  public void act(String _action)
  {
    if("copy"   .equals(_action)) ta_.copy();
    if("cut"    .equals(_action)) ta_.cut();
    if("paste"  .equals(_action)) ta_.paste();
  }

  public boolean needScrollBars()
  {
    return false;
  }

  public Object getGUI()
  {
    return ta_;
  }

  public void start()
  {
    if(ta_==null)
    {
      ta_=new JEditTextArea();
      ta_.setDocument(new JEditSyntaxDocument());
      // ta_.setCaretBlinkEnabled(true);
      JEditTextAreaPainter pt=ta_.getPainter();
      pt.setFont(new Font("Monospaced",Font.PLAIN,12));
      pt.setBlockCaretEnabled(true);
      pt.setLineHighlightColor(new Color(255,255,224));
      pt.setBracketHighlightColor(new Color(128,255,0));
      pt.setEOLMarkerColor(new Color(64,192,128));
      pt.setEOLMarkerEnabled(true);
    }
  }

  public void run()
  {
  }

  public void exit()
  {
  }

  /**
   * Loads some files.
   */
  // public void load(InputStream _in);
  // public void load(String      _filename);
  // public void load(File        _file);
  // public void load(URL         _url);

  public void setText(String _text)
  {
    ta_.setText(_text);
  }

  public String getText()
  {
    return ta_.getText();
  }

  private String syntax_=null;

  public void setSyntax(String _syntax)
  {
    syntax_=_syntax;

    String stx=syntax_;

    try
    {
      if("Cpp"       .equals(stx)) stx="CC";
      if("Hpp"       .equals(stx)) stx="CC";
      if("Html"      .equals(stx)) stx="HTML";
      if("Idl"       .equals(stx)) stx="IDL";
      if("Javascript".equals(stx)) stx="JavaScript";
      if("Sql2"      .equals(stx)) stx="SQL";
      if("Sql3"      .equals(stx)) stx="SQL";
      if("Svg"       .equals(stx)) stx="XML";
      if("Uxf"       .equals(stx)) stx="XML";
      if("Xml"       .equals(stx)) stx="XML";
      if("Xmi"       .equals(stx)) stx="XML";

      String pkg=JEditTokenMarker.class.getName();

      int    i=pkg.lastIndexOf('.');
      if(i>=0) pkg=pkg.substring(0,i+1);
      else     pkg="";

      System.err.println("TOKENMARKER="+pkg+"JEdit"+stx+"TokenMarker");

      Class  tmc=Class.forName(pkg+"JEdit"+stx+"TokenMarker");
      Object tmi=tmc.newInstance();
      ta_.setTokenMarker((JEditTokenMarker)tmi);

      System.err.println("OK for "+stx);
    }
    catch(Exception ex) { System.err.println("ERROR for "+stx); }
  }

  public String getSyntax()
  {
    return syntax_;
  }

  public void setEditable(boolean _editable)
  {
    ta_.setEditable(_editable);
  }

  public boolean isEditable()
  {
    return ta_.isEditable();
  }

  public static void main(String[] args)
  {
    EditorJedittextarea buta=new EditorJedittextarea();
    buta.start();

    StringBuffer s=new StringBuffer();
    try
    {
      LineNumberReader lnr=new LineNumberReader
	(new FileReader("../index.html"));

      while(true)
      {
	String l=lnr.readLine();
	if(l==null) break;
	s.append(l);
	s.append('\n');
      }
    }
    catch(Exception ex) { }
    // System.out.println(s);
    buta.setText(s.toString());
    buta.setSyntax("Html");

    JFrame f=new JFrame("EditorJedittextarea");
    f.getContentPane().add("Center",
			   (Component)buta.getGUI());
    f.setSize(400,300);
    f.show();

    buta.run();

    /*
    try { Thread.sleep(30000); }
    catch(Exception ex) { }
    System.out.println(""+buta.getData());
    buta.exit();
    System.exit(0);
    */
  }
}
