/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         EditorLinkExternal.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:08 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.editor;
import com.memoire.editor.*;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;

public class EditorLinkExternal
       extends JPanel
       implements EditorInterface, ActionListener
{
  private EditorInterface ei_=null;
  private JTextArea       ta_=null;
  private JButton         bt_=null;
  private JLabel          lb_=null;

  public EditorLinkExternal(EditorInterface _ei)
  {
    ei_=_ei;

    BorderLayout lo=new BorderLayout();
    lo.setVgap(5);
    lo.setHgap(5);
    setLayout(lo);
    setBorder(new EmptyBorder(5,5,5,5));

    ta_=new JTextArea(6,40);
    ta_.setBorder(new EmptyBorder(0,0,0,0));
    add("Center",new JScrollPane(ta_));

    JPanel pn=new JPanel();
    GridLayout lopn=new GridLayout(1,2);
    lopn.setVgap(0);
    lopn.setHgap(5);
    pn.setLayout(lopn);

    lb_=new JLabel();
    pn.add(lb_,0);

    bt_=new JButton("Edit with "+ei_.getName());
    bt_.setCursor(new Cursor(Cursor.HAND_CURSOR));
    bt_.setActionCommand("EDITER");
    bt_.addActionListener(this);
    pn.add(bt_,1);

    add("South",pn);
  }

  public String getName()
  {
    return ei_.getName();
  }

  public String getShortDescription()
  {
    return ei_.getShortDescription();
  }

  public String getLongDescription()
  {
    return ei_.getLongDescription();
  }

  public Object getIcon()
  {
    return ei_.getIcon();
  }

  public boolean can(String _action)
  {
    return ei_.can(_action);
  }

  public void act(String _action)
  {
    ei_.act(_action);
  }

  public boolean needScrollBars()
  {
    return false;
  }

  public Object getGUI()
  {
    Object o=ei_.getGUI();
    if(!(o instanceof JComponent)) o=this;
    return o;
  }


  public void start()
  {
    ei_.start();
  }

  public void run()
  {
  }

  public void exit()
  {
    ei_.exit();
  }

  /**
   * Loads some files.
   */
  // public void load(InputStream _in);
  // public void load(String      _filename);
  // public void load(File        _file);
  // public void load(URL         _url);

  public boolean acceptBuInformationsDocument()
  {
    return true;
  }

  public void setText(String _text)
  {
    ta_.setText(_text);
    lb_.setText(_text.length()+" chars");
    lb_.revalidate();
  }

  public String getText()
  {
    return ta_.getText();
  }

  public void setSyntax(String _syntax)
  {
    ei_.setSyntax(_syntax);
  }

  public String getSyntax()
  {
    return ei_.getSyntax();
  }

  public void setEditable(boolean _editable)
  {
    ta_.setEditable(_editable);
    ei_.setEditable(_editable);
  }

  public boolean isEditable()
  {
    return ta_.isEditable();
  }

  public void actionPerformed(ActionEvent _evt)
  {
    String action=_evt.getActionCommand();
    System.out.println("action="+action);

    if("EDITER".equals(action))
    {
      Thread t=new Thread("Editor link external")
      {
	public void run()
	{
	  bt_.setEnabled (false);
	  ta_.setEditable(false);
	  ta_.setBackground(UIManager.getColor("Panel.background"));
	  ta_.repaint();
	  ei_.start();
	  ei_.setText(getText());

	  if(ei_.getGUI() instanceof Frame)
	    ((Frame)ei_.getGUI()).toFront();

	  ei_.run();
	  setText(ei_.getText());
	  ei_.exit();

	  if(ei_.getGUI() instanceof Frame)
	    ((Frame)ei_.getGUI()).toBack();

	  ta_.setBackground(UIManager.getColor("TextArea.background"));
	  ta_.repaint();
	  ta_.setEditable(true);
	  bt_.setEnabled (true);
	}
      };
      t.setPriority(Thread.MIN_PRIORITY);
      t.start();
    }
  }

  public static void main(String[] args)
  {
    EditorExternal     emacs=new EditorExternal("emacs");
    EditorLinkExternal buta =new EditorLinkExternal(emacs);
    buta.start();
    buta.setText("hello");

    JFrame f=new JFrame("EditorLinkExternal");
    f.getContentPane().add("Center",(Component)buta.getGUI());
    f.pack();
    f.show();

    buta.run();
  }
}
