/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FooArray.java
 * @version      0.08
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2001 Guillaume Desnoix
 */

package com.memoire.foo;
import com.memoire.foo.*;

import java.lang.reflect.*;
import java.util.*;

public class FooArray
{
  private static FooCategory pkg_=null;

  public final static FooCategory init()
  {
    if(pkg_==null)
    {
      pkg_=FooCategory.create("&array");
      FooCategory.create(FooLib.getArrayclass(Boolean.TYPE)).extend(pkg_);
      FooCategory.create(FooLib.getArrayclass(Byte   .TYPE)).extend(pkg_);
      FooCategory.create(FooLib.getArrayclass(Short  .TYPE)).extend(pkg_);
      FooCategory.create(FooLib.getArrayclass(Integer.TYPE)).extend(pkg_);
      FooCategory.create(FooLib.getArrayclass(Long   .TYPE)).extend(pkg_);
      FooCategory.create(FooLib.getArrayclass(Float  .TYPE)).extend(pkg_);
      FooCategory.create(FooLib.getArrayclass(Double .TYPE)).extend(pkg_);
      FooCategory.create(Object[].class                    ).extend(pkg_);

      pkg_.setMessage("length"  ,FooArray.class,"length");
      pkg_.setMessage("at"      ,FooArray.class,"at"); 
      pkg_.setMessage("empty"   ,FooArray.class,"empty"); 
      pkg_.setMessage("slice"   ,FooArray.class,"slice"); 
      pkg_.setMessage("map"     ,FooArray.class,"map"); 
      pkg_.setMessage("each"    ,FooArray.class,"each",false); 
      pkg_.setMessage("convert" ,FooArray.class,"convert"); 
      pkg_.setMessage("[]"      ,FooArray.class,"extractII");
      pkg_.setMessage("[["      ,FooArray.class,"extractIX");
      pkg_.setMessage("]]"      ,FooArray.class,"extractXI");
      pkg_.setMessage("]["      ,FooArray.class,"extractXX");
      pkg_.setMessage("toString",FooArray.class,"toStringMsg");
    }

    return pkg_;
  }

  public static int length(Object _o)
  {
    return Array.getLength(_o);
  }

  public static Object at(Object _o, int _i)
  {
    Object r=null;
    if((_i>=0)&&(_i<length(_o)))
      r=Array.get(_o,_i);
    if(r==FooNull.NULL) r=null;
    return r;
  }

  public static boolean empty(Object _o)
  {
    return (length(_o)==0);
  }

  public static Object slice(Object _o, int _begin, int _end)
  {
    int imin=Math.max(0,_begin);
    int imax=Math.min(length(_o),_end);
    int l   =Math.max(0,imax-imin);

    Object r=Array.newInstance(_o.getClass().getComponentType(),l);
    for(int i=imin; i<imax; i++)
      Array.set(r,i-imin,Array.get(_o,i));
    return r;
  }

  public static Object map(Object _o, Object _msg)
  {
    FooList r=new FooList();
    for(int i=0;i<length(_o);i++)
    {
      Object o=at(_o,i);
      FooList call=new FooList().append(new Object[] { o,_msg });
      r.add(call.eval());
    }
    return r;
  }

  public static Object each(Object[] _o)
  {
    FooLib.checkArrayArgument(_o[0],0);
    FooLib.checkClassArgument(FooSymbol.class,_o[1],1);

    FooList r   =new FooList();
    String  name=((FooSymbol)_o[1]).getName();
    Object  old =FooSymbol.get_local(name);

    for(int i=0;i<length(_o[0]);i++)
    {
      FooSymbol.create_local(name,Array.get(_o[0],i));
      r.add(FooLib.evalBlock(_o,2));
      FooSymbol.forget_local(name,old);
    }

    return r;
  }

  public static Object convert(Object _o, Class _c)
  {
    int    l=length(_o);
    Object r=Array.newInstance(_c,l);

    for(int i=0;i<l;i++)
      Array.set(r,i,Array.get(_o,i));

    return r;
  }

  public static Object extractII(Object[] _o)
  {
    FooLib.checkArrayArgument(_o[0],0);

    Object r=null;

    switch(_o.length)
    {
    case 2:
      FooLib.checkClassArgument(Number.class,_o[1],1);
      r=at(_o[0],((Number)_o[1]).intValue());
      break;
    case 3:
      FooLib.checkClassArgument(Number.class,_o[1],1);
      FooLib.checkClassArgument(Number.class,_o[2],2);
      r=slice(_o[0],
	      ((Number)_o[1]).intValue()+0,
	      ((Number)_o[2]).intValue()+1);
      break;
    default:
      FooLib.checkNumberArgument(2,_o.length);
      break;
    }

    return r;
  }

  public static Object extractIX(Object[] _o)
  {
    FooLib.checkArrayArgument(_o[0],0);

    Object r=null;

    switch(_o.length)
    {
    case 3:
      FooLib.checkClassArgument(Number.class,_o[1],1);
      FooLib.checkClassArgument(Number.class,_o[2],2);
      r=slice(_o[0],
	      ((Number)_o[1]).intValue()+0,
	      ((Number)_o[2]).intValue()+0);
      break;
    default:
      FooLib.checkNumberArgument(2,_o.length);
      break;
    }

    return r;
  }

  public static Object extractXI(Object[] _o)
  {
    FooLib.checkArrayArgument(_o[0],0);

    Object r=null;

    switch(_o.length)
    {
    case 3:
      FooLib.checkClassArgument(Number.class,_o[1],1);
      FooLib.checkClassArgument(Number.class,_o[2],2);
      r=slice(_o[0],
	      ((Number)_o[1]).intValue()+1,
	      ((Number)_o[2]).intValue()+1);
      break;
    default:
      FooLib.checkNumberArgument(2,_o.length);
      break;
    }

    return r;
  }

  public static Object extractXX(Object[] _o)
  {
    FooLib.checkArrayArgument(_o[0],0);

    Object r=null;

    switch(_o.length)
    {
    case 3:
      FooLib.checkClassArgument(Number.class,_o[1],1);
      FooLib.checkClassArgument(Number.class,_o[2],2);
      r=slice(_o[0],
	      ((Number)_o[1]).intValue()+1,
	      ((Number)_o[2]).intValue()+0);
      break;
    default:
      FooLib.checkNumberArgument(2,_o.length);
      break;
    }

    return r;
  }

  public static String toStringMsg(Object _o)
  {
    return FooLib.stringValue(_o);
  }
}
