/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FooFix.java
 * @version      0.08
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2001 Guillaume Desnoix
 */

package com.memoire.foo;
import com.memoire.foo.*;

public class FooFix
{
  private static FooCategory pkg_=null;

  public final static FooCategory init()
  {
    if(pkg_==null)
    {
      pkg_=FooCategory.create("&fix");
      FooCategory.create(Integer.class).alias("&integer").extend(pkg_);
      FooCategory.create(Byte   .class).alias("&byte"   ).extend(pkg_);
      FooCategory.create(Short  .class).alias("&short"  ).extend(pkg_);
      FooCategory.create(Long   .class).alias("&long"   ).extend(pkg_);

      pkg_.setMessage("abs"  ,Math.class,"abs(int)");
      pkg_.setMessage("max"  ,Math.class,"max(int,int)");
      pkg_.setMessage("min"  ,Math.class,"min(int,int)");
      
      pkg_.setMessage("+" ,FooFix.class,"addMsg");
      pkg_.setMessage("-" ,FooFix.class,"subMsg");
      pkg_.setMessage("*" ,FooFix.class,"mulMsg");
      pkg_.setMessage("/" ,FooFix.class,"divMsg");
      pkg_.setMessage("<" ,FooFix.class,"ltMsg");
      pkg_.setMessage("<=",FooFix.class,"leMsg");
      pkg_.setMessage(">" ,FooFix.class,"gtMsg");
      pkg_.setMessage(">=",FooFix.class,"geMsg");
      pkg_.setMessage("==",FooFix.class,"eqMsg");
      pkg_.setMessage("!=",FooFix.class,"neMsg");
      pkg_.setMessage("~" ,FooFix.class,"mtMsg");
      pkg_.setMessage("!~",FooFix.class,"nmMsg");
      pkg_.setMessage("" ,FooFix.class,"sqrMsg");
      pkg_.setMessage("**",FooFix.class,"powMsg");

      pkg_.setMessage("times",FooFix.class,"times",false);
      pkg_.setMessage(".."   ,FooFix.class,"range");
    }

    return pkg_;
  }

  private final static Integer[] INTS;

  static
  {
    INTS=new Integer[21];
    for(int i=0;i<21;i++) INTS[i]=new Integer(i);
  }
  //private final static Integer INT_ONE =new Integer(1);
  //private final static Integer INT_TWO =new Integer(2);

  public static Integer create(int _value)
  {
    return (_value<21 ? INTS[_value] : new Integer(_value));
  }
    
  public static Number addMsg(Object[] _n)
  {
    return operation(_n,0);
  }

  public static Number subMsg(Object[] _n)
  {
    return operation(_n,1);
  }

  public static Number mulMsg(Object[] _n)
  {
    return operation(_n,2);
  }

  public static Number divMsg(Object[] _n)
  {
    return operation(_n,3);
  }
  
  private static Number operation(Object[] _n, int _op)
  {
    FooLib.checkClassArgument(Number.class,_n[0],0);
    int r=((Number)_n[0]).intValue();
    for(int i=1;i<_n.length;i++)
    {
      FooLib.checkClassArgument(Number.class,_n[i],i);
      int a=((Number)_n[i]).intValue();
      switch(_op)
      {
      case 0: r+=a; break;
      case 1: r-=a; break;
      case 2: r*=a; break;
      case 3: r/=a; break;
      }
    }
    return create(r);
  }    

  public static Boolean ltMsg(Object[] _n)
  {
    return comparison(_n,0);
  }
  
  public static Boolean leMsg(Object[] _n)
  {
    return comparison(_n,1);
  }
  
  public static Boolean gtMsg(Object[] _n)
  {
    return comparison(_n,2);
  }
  
  public static Boolean geMsg(Object[] _n)
  {
    return comparison(_n,3);
  }
  
  public static Boolean eqMsg(Object[] _n)
  {
    return comparison(_n,4);
  }
  
  public static Boolean neMsg(Object[] _n)
  {
    return comparison(_n,5);
  }

  public static Boolean mtMsg(Object[] _n)
  {
    return comparison(_n,6);
  }
  
  public static Boolean nmMsg(Object[] _n)
  {
    return comparison(_n,7);
  }
  
  private static Boolean comparison(Object[] _n, int _op)
  {
    FooLib.checkClassArgument(Number.class,_n[0],0);
    boolean r=true;
    int v=((Number)_n[0]).intValue();
    for(int i=1;i<_n.length;i++)
    {
      FooLib.checkClassArgument(Number.class,_n[i],i);
      int a=((Number)_n[i]).intValue();
      switch(_op)
      {
      case 0: r&=(v< a); break;
      case 1: r&=(v<=a); break;
      case 2: r&=(v> a); break;
      case 3: r&=(v>=a); break;
      case 4: r&=(v==a); break;
      case 5: r&=(v!=a); break;
      case 6: r&=(v==a); break;
      case 7: r&=(v!=a); break;
      }
    }
    return FooBoolean.create(r);
  }    
  
  public static Number sqrMsg(Number _a)
  {
    int a=_a.intValue();
    return create(a*a);
  }
  
  public static Number powMsg(Number _a,Integer _b)
  {
    int a=_a.intValue();
    int b=_b.intValue();
    return create((int)Math.pow(a,b));
  }

  //tmp
  public static Object times(Object[] _body)
  {
    int    n=((Number)_body[0]).intValue();
    Object r=null;
    while(n>0) { r=FooLib.evalBlock(_body,1); n--; }
    return r;
  }

  public static FooList range(Number _a, Number _b)
  {
    int a=_a.intValue();
    int b=_b.intValue();

    FooList r=new FooList();
    while(a<=b) { r.add(new Integer(a)); a++; }
    return r;
  }
}
