/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FooLib.java
 * @version      0.08
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2001 Guillaume Desnoix
 */

package com.memoire.foo;
import com.memoire.foo.*;

import java.io.*;
import java.lang.reflect.*;

public class FooLib
{

  // Native members

  public static boolean isEllipsis(Method _method)
  {
    Class[] p=_method.getParameterTypes();
    return isEllipsis(p);
  }
  
  public static boolean isEllipsis(Constructor _constructor)
  {
    Class[] p=_constructor.getParameterTypes();
    return isEllipsis(p);
  }
  
  public static boolean isEllipsis(Class[] _p)
  {
    return   (_p.length==1)
           &&(_p[0].isArray())
           &&(_p[0].getComponentType()==Object.class);
  }

  public static Method getMethod(Class _clazz, String _signature)
  { return getMethod(_clazz,_signature,-1); }

    public static Method getMethod
    (Class _clazz, String _signature, int _params)
    {
	Method r=null;
	try
	{
	    Method[] methods=_clazz.getMethods();

	    for(int i=0;i<methods.length;i++)
	    {
		Method  m=methods[i];
		Class[] p=m.getParameterTypes();
		StringBuffer d=new StringBuffer(m.getName() + "(");
		boolean isStatic=((m.getModifiers() & Modifier.STATIC) != 0);

		for(int j = 0; j < p.length; j++)
		{
		    if(j>0) d.append(',');
		    d.append(getClassName(p[j]));
		}
		d.append(')');
		// System.out.println(d.toString());

		if(_signature.equals(d.toString()))
		{
		    r=methods[i];
		    break;
		}

		if(_signature.equals(m.getName())
		   &&(  (_params==-1)
		      ||isEllipsis(m)
		      ||(_params==p.length)
		      ||(isStatic&&(_params==p.length+1))))
	        {
		    if(r!=null)
			throw new RuntimeException
			    ("method " + _signature + " is ambiguous");
		    r = methods[i];
		}
	    }
	}
	catch(SecurityException ex)
	{
	    throw new RuntimeException("security violation");
	}

	return r;
    }

    public static Constructor getConstructor(Class _clazz, String _signature)
    { return getConstructor(_clazz,_signature,-1); }

    public static Constructor getConstructor
	(Class _clazz, String _signature, int _params)
    {
	Constructor r=null;
	try
        {
	    Constructor[] constructors=_clazz.getConstructors();

	    for(int i=0;i<constructors.length;i++)
	    {
		Constructor  m=constructors[i];
		Class[] p=m.getParameterTypes();
		StringBuffer d=new StringBuffer(m.getName() + "(");
		boolean isStatic=((m.getModifiers() & Modifier.STATIC) != 0);

		for(int j=0;j<p.length;j++)
		{
		    if(j>0) d.append(',');
		    d.append(getClassName(p[j]));
		}
		d.append(')');

		if(_signature.equals(d.toString()))
		{
		    r=constructors[i];
		    break;
		}

		if(_signature.equals(m.getName())
		   &&(  (_params==-1)
		      ||isEllipsis(m)
		      ||(_params==p.length)
		      ||(isStatic&&(_params==p.length+1))))
		{
		    if(r!=null)
			throw new RuntimeException
			    ("constructor " + _signature + " is ambiguous");
		    r=constructors[i];
		}
	    }
	}
	catch(SecurityException ex)
	{
	    throw new RuntimeException("security violation");
	}
	
	return r;
    }

    public static Field getField(Class _clazz, String _name)
    {
	Field r=null;
	try
	{
	    Field[] fields=_clazz.getFields();

	    for(int i=0;i<fields.length;i++)
	    {
		if(_name.equals(fields[i].getName()))
		{
		    r=fields[i];
		    break;
		}
	    }
	}
	catch(SecurityException ex)
	{
	    throw new RuntimeException("security violation");
	}

	return r;
    }

    // Invokations

    private static Object convert(Object _o, Class _t)
    {
	Object r=_o;
	
	if(_t==Boolean.TYPE)
	    r=toBoolean(r);
        else
	if(_t==Integer.TYPE)
	    r=toInteger(r);
	else
	if(_t==Double.TYPE)
	    r=toDouble(r);
	else
	if(_t==Byte.TYPE)
	    r=toByte(r);
	else
	if(_t==Short.TYPE)
	    r=toShort(r);
	else
	if(_t==Float.TYPE)
	    r=toFloat(r);
	else
	if(_t==Character.TYPE)
            r=toCharacter(r);
	else
	if((r==null)||(r==FooNull.NULL)||(r==FooVoid.VOID))
	    r=null;
        else
	if(_t==String.class)
	    r=toString(r);
	else
	if(_t==Boolean.class)
	    r=toBoolean(r);
        else
	if(_t==Integer.class)
	    r=toInteger(r);
	else
	if(_t==Double.class)
	    r=toDouble(r);
	else
	if(_t==Byte.class)
	    r=toByte(r);
	else
	if(_t==Short.class)
	    r=toShort(r);
	else
	if(_t==Float.class)
	    r=toFloat(r);
	else
	if(_t==Character.class)
	    r=toCharacter(r);
	else
	if(_t.isArray())
	  r=toArray(r,_t);

	return r;
    }

  public static Object invokeField
    (Object _self, Field _field, Object[] _params)
  {
    Object r=null;
    String s=_field.getName();
    try
    {
	switch(_params.length)
	{
	case 0:
	    r=_field.get(_self);
	    break;
	case 1:
	    _field.set(_self,convert(_params[0],_field.getType()));
	    r=_params[0]; break;
	default:
	    throw new RuntimeException
		("too many arguments for attribute "+s);
	}
    }
    catch(IllegalAccessException ex)
    {
	throw new RuntimeException
	    ("illegal access to attribute "+s);  
    }

    return r;
  }

  public static Object invokeMethod
    (Object _self, Method _method, Object[] _params)
  {
    Object  r = null;
    
    Method   m = _method;
    Class[]  t = m.getParameterTypes();
    boolean  isStatic = ((m.getModifiers() & Modifier.STATIC) != 0);
    Object[] p = _params;

    if(isEllipsis(t))
	p=new Object[] { p };

    if(t.length>p.length)
	throw new RuntimeException
	    ("missing arguments: "+t.length+" needed");
    if(t.length<p.length)
	throw new RuntimeException
	    ("too many arguments: "+t.length+" needed");

    try
    {
      for(int i = 0; i < p.length; i++)
	  p[i]=convert(p[i],t[i]);

      r = m.invoke((isStatic ? null : _self),p);

      if((r==null)&&(m.getReturnType()==Void.TYPE))
	  r=FooVoid.VOID;
    }
    catch(ThreadDeath ex)
    {
      throw ex;
    }
    catch(IllegalAccessException ex)
    {
      throw new RuntimeException("illegal access");
    }
    catch(InvocationTargetException ex)
    {
      Throwable th=ex.getTargetException();
      if(th instanceof RuntimeException)
	throw (RuntimeException)th;
      else
      if(th instanceof ThreadDeath)
	throw (ThreadDeath)th;
      else
	throw new RuntimeException
	  (getClassName(th.getClass())+":"+th.getMessage());
    }
    
    return r;
  }

  public static Object invokeConstructor
    (Constructor _constructor, Object[] _params)
  {
    Object  r = null;
    
    Constructor m = _constructor;
    Class[]     t = m.getParameterTypes();
    Object[]    p = _params;

    if(isEllipsis(t))
	p=new Object[] { p };

    if(t.length>p.length)
	throw new RuntimeException
	    ("missing arguments: "+t.length+" needed");
    if(t.length<p.length)
	throw new RuntimeException
	    ("too many arguments: "+t.length+" needed");

    try
    {
      for(int i = 0; i < p.length; i++)
	  p[i]=convert(p[i],t[i]);

      r = m.newInstance(p);
      //System.out.println("RESULT = "+r);
    }
    catch(IllegalAccessException ex)
    {
      throw new RuntimeException("illegal access");
    }
    catch(InstantiationException ex)
    {
      throw new RuntimeException("instantiation failed");
    }
    catch(InvocationTargetException ex)
    {
      Throwable th=ex.getTargetException();
      if(th instanceof RuntimeException)
	throw (RuntimeException)th;
      else
      throw new RuntimeException
	(getClassName(th.getClass())+":"+th.getMessage());
    }
    
    return r;
  }

  public static Object invokeBlock
    (Object _self,Object[] _block,Object[] _params)
  {
    // save old values
    Object   old_self=FooSymbol.get_local("$0");
    Object   old_star=FooSymbol.get_local("$*");
    Object[] old_params=new Object[9];
    for(int i=0;i<9;i++)
      old_params[i]=FooSymbol.get_local("$"+(i+1));

    // set new values
    FooSymbol.create_local("$0",_self);
    FooSymbol.create_local("$*",new FooList().append(_params));
    for(int i=0;i<9;i++)
    {
      if(i>=_params.length)
	FooSymbol.create_local("$"+(i+1),null);
      else
      if(_params[i]==null)
	FooSymbol.create_local("$"+(i+1),null);
      else
	FooSymbol.create_local("$"+(i+1),_params[i]);
    }
    
    Object r=evalBlock(_block,0);
    
    // restore previous values
    FooSymbol.forget_local("$0",old_self);
    FooSymbol.forget_local("$*",old_star);
    for(int i=0;i<9;i++)
      FooSymbol.forget_local("$"+(i+1),old_params[i]);

    return r;
  }

  public static Object invokeBlock
    (Object _self,Object[] _block,Object[] _params,String[] _argnames)
  {
    if(_argnames==null)
      return invokeBlock(_self,_block,_params);

    // save old values
    Object old_self  =FooSymbol.get_local("self");
    int la=_argnames.length;
    Object[] old_params=new Object[la];
    for(int i=0;i<la;i++)
      old_params[i]=FooSymbol.get_local(_argnames[i]);

    // set new values
    FooSymbol.create_local("self"  ,_self);
    for(int i=0;i<la;i++)
    {
      if(i>=_params.length)
	FooSymbol.create_local(_argnames[i],null);
      else
      if(_params[i]==null)
	FooSymbol.create_local(_argnames[i],null);
      else
	FooSymbol.create_local(_argnames[i],_params[i]);
    }
    
    Object r=invokeBlock(_self,_block,_params);

    // restore previous values
    FooSymbol.forget_local("self"  ,old_self);
    for(int i=0;i<la;i++)
      FooSymbol.forget_local(_argnames[i],old_params[i]);
    
    return r;
  }

    // Checks

    public static String getClassName(Object _o)
    {
	Class clazz;
	if(_o instanceof Class) clazz=(Class)_o;
	else                    clazz=_o.getClass();

	String r;
	if(clazz.isArray()) r=getClassName(clazz.getComponentType())+"[]";
	else                r=clazz.getName();

	/*
	if(r.startsWith("com.memoire.foo.Foo"))
	r=r.substring(19).toLowerCase();
	else
	if(r.indexOf('.')>=0) r=r.substring(r.lastIndexOf('.')+1);
	*/

	return r;
    }

    public static void checkNumberArgument(int _needed, int _provided)
    {
	if(_needed>_provided)
	    throw new RuntimeException("missing arguments");
	if(_needed<_provided)
	    throw new RuntimeException("too many arguments");
    }

    public static void checkClassArgument(Class _clazz, Object _o, int _i)
    {
	if(!_clazz.isInstance(_o))
	{
	    String s=""+FooLib.toString(_o)+
	      " is not a "+getClassName(_clazz);
	         if(_i==0) s+=" [object]";
	    else if(_i> 0) s+=" [argument "+_i+"]";
	    throw new RuntimeException(s);
	}
    }

    public static void checkArrayArgument(Object _o, int _i)
    {
      if((_o==null)||!_o.getClass().isArray())
      {
	String s=""+FooLib.toString(_o)+" is not an array";
	     if(_i==0) s+=" [object]";
	else if(_i> 0) s+=" [argument "+_i+"]";
	throw new RuntimeException(s);
      }
    }

    public static void checkValueArgument(Object _value, Object _o, int _i)
    {
	if(!_value.equals(_o))
	{
	    String s=""+FooLib.toString(_value)+
	      " is needed and not "+FooLib.toString(_o);
	         if(_i==0) s+=" [object]";
	    else if(_i> 0) s+=" [argument "+_i+"]";
	    throw new RuntimeException(s);
	}
    }

  /*
    public static void checkBracket(Object _o, int _i)
    {
	if(!"]".equals(_o)&&!"[".equals(_o))
	{
	    String s="] or [ needed and not "+_o;
	         if(_i==0) s+=" [object]";
	    else if(_i> 0) s+=" [argument "+_i+"]";
	    throw new RuntimeException(s);
	}
    }
  */

    // Global variables

    public static FooParser getParser()
    {
	Object o=FooSymbol.getValue("?");
	if(!(o instanceof FooParser)) o=new FooParser();
	return (FooParser)o;
    }

    public static PrintStream getOutput()
    {
	Object o=FooSymbol.getValue(":output");
	if(!(o instanceof PrintStream)) o=System.out;
	return (PrintStream)o;
    }

    public static PrintStream getError()
    {
	Object o=FooSymbol.getValue(":error");
	if(!(o instanceof PrintStream)) o=System.err;
	return (PrintStream)o;
    }

    // Convert to primitive types

    public static void convertException(Object _o,Class _c)
    {
	throw new RuntimeException
	    ("can not convert "+_o+" to "+getClassName(_c));
    }

    public static boolean booleanValue(Object _o)
    {
	boolean r=false;

	if((_o==null)||(_o==FooNull.NULL)||(_o==FooVoid.VOID))
	    r=false;
	else
	if(_o instanceof Boolean)
	    r=((Boolean)_o).booleanValue();
	else
	if(_o instanceof Number)
	    r=(((Number)_o).doubleValue()!=0.0);
	else
	if(_o instanceof String)
	    r=!("".equals(_o));
	else
	if(_o instanceof FooList)
	    r=!((FooList)_o).empty();
	else
	    convertException(_o,Boolean.TYPE);

	return r;
    }

    public static byte byteValue(Object _o)
    {
	return (byte)intValue(_o);
    }

    public static short shortValue(Object _o)
    {
	return (short)intValue(_o);
    }

    public static int intValue(Object _o)
    {
	int r=0;

	if((_o==null)||(_o==FooNull.NULL)||(_o==FooVoid.VOID))
	    r=0;
	else
	if(_o instanceof Boolean)
	    r=(((Boolean)_o).booleanValue() ? 1 : 0);
	else
	if(_o instanceof Number)
	    r=((Number)_o).intValue();
	else
	if(_o instanceof String)
	{
	    try { r=Integer.parseInt((String)_o); }
	    catch(Exception ex) { convertException(_o,Integer.TYPE); }
	}
	else
	if(_o instanceof FooList)
	    r=((FooList)_o).length();
	else
	    convertException(_o,Integer.TYPE);

	return r;
    }

    public static long longValue(Object _o)
    {
	return (long)intValue(_o);
    }

    public static float floatValue(Object _o)
    {
	return (float)doubleValue(_o);
    }

    public static double doubleValue(Object _o)
    {
	double r=0.0;

	if((_o==null)||(_o==FooNull.NULL)||(_o==FooVoid.VOID))
	    r=0.0;
	else
	if(_o instanceof Number)
	    r=((Number)_o).doubleValue();
	else
	if(_o instanceof String)
	{
	    try { r=new Double((String)_o).doubleValue(); }
	    catch(Exception ex) { convertException(_o,Double.TYPE); }
	}
	else
	    convertException(_o,Double.TYPE);

	return r;
    }

    public static char charValue(Object _o)
    {
	char r=' ';

	if(_o instanceof Character)
            r=((Character)_o).charValue();
	else
	if(_o instanceof Number)
	    r=(char)((Number)_o).intValue();
	else
	if((_o instanceof String)&&(((String)_o).length()==1))
	    r=((String)_o).charAt(0);
	else
	    convertException(_o,Character.TYPE);

	return r;
    }

    public static String stringValue(Object _o)
    {
	if(_o==null)
	  return "null";

	Class  c=_o.getClass();
	String r=_o.toString();

	if(c.isArray())
	{
	  c=c.getComponentType();
	  r="["+Array.getLength(_o)+"]";
	  while(c.isArray())
	    { c=c.getComponentType(); r+="[]"; }
	  r=FooLib.getClassName(c)+r;	    
	}
	else
	if(r.startsWith(_o.getClass().getName()+'@'))
	  r="<"+FooLib.getClassName(_o.getClass())+">";

	if(_o instanceof Class)
	{
	  Class z=(Class)_o;
	       if(z.isPrimitive()) r="type "     +z.toString();
	  else if(z.isArray()    ) r="array "    +getClassName(z);
	  else if(z.isInterface()) r="interface "+getClassName(z);
	  else                     r="class "    +getClassName(z);
	}

	if(_o instanceof File)
	{
	    File f=(File)_o;
	    if(f.isDirectory())
		r="directory ";
	    else
		r="file ";
	    try { r+=f.getCanonicalPath(); }
	    catch(IOException ex) { r+=f.getAbsolutePath(); }
	}

	if(_o instanceof FooMessage)
	    r="message "+r;
	if(_o instanceof FooSymbol)
	    r="symbol "+r;

	return r;
    }

    /*
	Class  c=_o.getClass();
	String r=_o.toString();

	if(r.startsWith(c.getName()+'@'))
	    r="instance "+FooLib.getClassName(c);

	if(_o instanceof File)
	{
	    File f=(File)_o;
	    if(f.isDirectory())
		r="directory ";
	    else
		r="file ";
	    try { r+=f.getCanonicalPath(); }
	    catch(IOException ex) { r+=f.getAbsolutePath(); }
	}
	
	if(_o instanceof FooMessage)
	    r="message "+r;
	if(_o instanceof FooSymbol)
	    r="symbol "+r;

	return r;
    */

    public static Boolean toBoolean(Object _o)
    {
	Boolean r;
	if(_o instanceof Boolean) r=(Boolean)_o;
	else r=FooBoolean.create(booleanValue(_o));
	return r;
    }

    public static Byte toByte(Object _o)
    {
	Byte r;
	if(_o instanceof Byte) r=(Byte)_o;
	else r=new Byte(byteValue(_o));
	return r;
    }

    public static Short toShort(Object _o)
    {
	Short r;
	if(_o instanceof Short) r=(Short)_o;
	else r=new Short(shortValue(_o));
	return r;
    }

    public static Integer toInteger(Object _o)
    {
	Integer r;
	if(_o instanceof Integer) r=(Integer)_o;
	else r=FooFix.create(intValue(_o));
	return r;
    }

    public static Long toLong(Object _o)
    {
	Long r;
	if(_o instanceof Long) r=(Long)_o;
	else r=new Long(longValue(_o));
	return r;
    }

    public static Float toFloat(Object _o)
    {
	Float r;
	if(_o instanceof Float) r=(Float)_o;
	else r=new Float(floatValue(_o));
	return r;
    }

    public static Double toDouble(Object _o)
    {
	Double r;
	if(_o instanceof Double) r=(Double)_o;
	else r=FooReal.create(doubleValue(_o));
	return r;
    }

    public static Character toCharacter(Object _o)
    {
	Character r;
	if(_o instanceof Character) r=(Character)_o;
	else r=new Character(charValue(_o));
	return r;
    }

    public static String toString(Object _o)
    {
	String r;
	if(_o instanceof String) r=(String)_o;
	else r=stringValue(_o);
	return r;
    }

    public static Object toArray(Object _o, Class _t)
    {
      Object r=null;
      Object v=null;
      int    l=0;
      Class  c=_t;

      if(c.isArray()) c=c.getComponentType();

      if(_o instanceof FooList)
	v=((FooList)_o).array();
      else
      if(!_o.getClass().isArray())
	v=new Object[] { _o };
      else
	v=_o;

      l=Array.getLength(v);
      r=Array.newInstance(c,l);

      for(int i=0;i<l;i++)
	Array.set(r,i,convert(Array.get(v,i),c));

      return r;
    }

    // Eval

    /*
    public static boolean isNull(Object _o)
    {
	return ((_o==null)||(_o==FooNull.NULL));
    }
    */

  private static Thread   thread_=null;
  private static int      level_ =0;
  private static boolean  trace_ =false;

  public final static void setTrace(boolean _trace)
  {
    thread_=Thread.currentThread();
    level_ =0;
    trace_ =_trace;
  }

  public final static boolean isTrace()
  {
    return trace_;
  }

  public static Object eval(Object _o)
  {
    Thread t=Thread.currentThread();
    t.yield();
    //try { t.sleep(50); }
    //catch(InterruptedException ex) { }
    if((t instanceof FooThread)&&((FooThread)t).isKilled())
      throw new ThreadDeath();

    if(trace_&&(t==thread_))
    {
      String s="\033[42m\033[30m";
      for(int i=0;i<level_;i++) s+="  ";
      System.err.println(s+"\033[43m\033[30m\033[K"+toSource(_o)+"\033[00m");
      level_++;
    }
    
    Object r=_o;
    if(_o instanceof FooEval)
      r=((FooEval)_o).eval();

    if(trace_&&(t==thread_))
      level_--;

    return r;
  }

  public static Object evalBlock(Object[] _body, int _begin)
  {
    Object r=null;
    for(int i=_begin;i<_body.length;i++)
      r=eval(_body[i]);
    return r;
  }

    public static Class getSuperclass(Class _c)
    {
      if((_c==null)||(_c==Object.class)) return null;

      Class r=null;

      if(!_c.isArray())
      {
	r=_c.getSuperclass();
      }
      else
      {
	_c=_c.getComponentType();
	if(_c.isPrimitive()) r=Object.class;
	else                 r=getArrayclass(getSuperclass(_c));
      }

      return r;
    }

    public static Class getArrayclass(Class _c)
    {
      if(_c==null)         return null;
      if(_c==Object.class) return Object[].class;

      return Array.newInstance(_c,0).getClass();
    }

    public static String toSource(Object _o)
    {
      if(_o==null)
	return "null";

      if(_o instanceof String)
	return FooString.toSource((String)_o);

      if(_o instanceof FooList)
	return ((FooList)_o).toSource();

      if(_o instanceof FooMessage)
	return FooString.toSource(((FooMessage)_o).getName());

      if(_o instanceof FooSymbol)
	return ((FooSymbol)_o).toSource();

      String r=_o.toString();
      Class  c=_o.getClass();

      if(c.isArray()||r.startsWith(c.getName()+'@'))
	r="<"+getClassName(_o)+">";

      return r;
    }
}
