/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FooParser.java
 * @version      0.08
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2001 Guillaume Desnoix
 */

package com.memoire.foo;
import com.memoire.foo.*;

import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class FooParser
{
  private static FooCategory pkg_=null;

  public final static FooCategory init()
  {
    if(pkg_==null)
    {
      pkg_=FooCategory.create(FooParser.class);
      pkg_.alias("&parser");

      pkg_.setMessage("."       ,FooParser.class,"dot");
      pkg_.setMessage("?"       ,FooParser.class,"debug",false);
      //pkg_.setMessage("import",FooParser.class,"importMsg");

      pkg_.setMessage("parallel",FooParser.class,"parallel",false);
      pkg_.setMessage("serial"  ,FooParser.class,"serial"  ,false);
      pkg_.setMessage("any"     ,FooParser.class,"any"     ,false);
      pkg_.setMessage("some"    ,FooParser.class,"some"    ,false);
      pkg_.setMessage("first"   ,FooParser.class,"first"   ,false);
      pkg_.setMessage("last"    ,FooParser.class,"last"    ,false);
      pkg_.setMessage("fork"    ,FooParser.class,"fork"    ,false);
      pkg_.setMessage("timeout" ,FooParser.class,"timeout" ,false);

      pkg_.setMessage("trace"   ,FooParser.class,"trace"   ,false);
      pkg_.setMessage("try"     ,FooParser.class,"tryMsg"  ,false);
      pkg_.setMessage("throw"   ,FooParser.class,"throwMsg");
      pkg_.setMessage("loop"    ,FooParser.class,"loop"    ,false);
      pkg_.setMessage("break"   ,FooParser.class,"breakMsg");

      //pkg_.setMessage("class"   ,FooParser.class,"classMsg",false);
    }

    return pkg_;
  }

  public FooParser()
  {
    FooSymbol.setValue("?"      ,this);
    FooSymbol.setValue(":input" ,System.in );
    FooSymbol.setValue(":output",System.out);
    FooSymbol.setValue(":error" ,System.err);
    
    install();
  }

  private boolean ansi_;
  public  void    setAnsi(boolean _ansi) { ansi_=_ansi; }
  public  boolean isAnsi() { return ansi_; }

  private boolean exception_;
  public  void    setException(boolean _exception) { exception_=_exception; }
  public  boolean isException() { return exception_; }

  private boolean step_;
  public  void    setStep(boolean _step) { step_=_step; }
  public  boolean isStep() { return step_; }

  public  void    setTrace(boolean _trace) { FooLib.setTrace(_trace); }
  public  boolean isTrace() { return FooLib.isTrace(); }

  public InputStream getInput()
  { return (InputStream)FooSymbol.getValue(":input"); }

  public PrintStream getOutput()
  { return (PrintStream)FooSymbol.getValue(":output"); }

  public PrintStream getError()
  { return (PrintStream)FooSymbol.getValue(":error"); }

  public void setInput(InputStream _input)
  { FooSymbol.setValue(":input" ,_input); }
  
  public void setOutput(PrintStream _output)
  { FooSymbol.setValue(":output",_output); }
  
  public void setError(PrintStream _error)
  { FooSymbol.setValue(":error" ,_error); }

  private String stack(String _s)
  {
    Object c=FooSymbol.get_raw(_s);
    String r="";

    if(c!=null)
    {
      r+=FooLib.toSource(c);
      if(c instanceof FooList)
      {
	FooList l=(FooList)c;
	String  o=l.origin_;
	if(o!=null)
	  r+="  ; "+o+":"+l.lineno_;
      }
    }

    return r;
  }

  public synchronized void error(Exception _ex)
  {
    if(ansi_) getError().print("\033[41m\033[33m\033[K");
    getError().println
      ("error:"+
       (ft_!=null
	? ft_.origin()+":"+ft_.lineno()+":"+ft_.charno()+":"
	: ":::")+
       _ex.getMessage());
    if(ansi_) getError().print("\033[00m\033[K");

    if(ansi_) getError().print("\033[43m\033[31m\033[K");
    getError().println("<  "+stack("[call-1]")+"   ");
    getError().println("<<  "+stack("[call-2]")+"  ");
    getError().println("<<<  "+stack("[call-3]")+" ");
    if(ansi_) getError().print("\033[00m\033[K");

    if(isException())
    {
      if(ansi_) getError().print("\033[41m\033[33m\033[K");
      ByteArrayOutputStream out=new ByteArrayOutputStream();
      _ex.printStackTrace(new PrintStream(out));
      LineNumberReader in=new LineNumberReader
	(new InputStreamReader
	  (new ByteArrayInputStream(out.toByteArray())));
      try
      {
	String l;
	int    n;

	n=0;
	while(((l=in.readLine())!=null)&&(n<6))
	{
	  getError().println(l);
	  n++;
	}
	if(l!=null)
	{
	  n++;
	  while(in.readLine()!=null) n++;
	  getError().println("        ... ["+n+" more lines]");
	}
      }
      catch(IOException ex) { }
      if(ansi_) getError().print("\033[00m\033[K");
    }
  }
  
  public synchronized void warning(String _s)
  {
    if(ansi_) getError().print("\033[41m\033[33m\033[K");
    getError().println
      ("warning:"+
       (ft_!=null
	? ft_.origin()+":"+ft_.lineno()+":"+ft_.charno()+":"
	: ":::")+_s);
    if(ansi_) getError().print("\033[00m\033[K");
  }
  
  private FooTokenizer ft_=null;
  void setTokenizer(FooTokenizer _ft) { ft_=_ft; }
  FooTokenizer getTokenizer() { return ft_; }
  
  private boolean standAlone_;
  public boolean isStandAlone()
  { return standAlone_; }
  public void setStandAlone(boolean _standAlone)
  { standAlone_=_standAlone; }
  
  private boolean exited_=false;

  public void exit()
  {
    exited_=true;
    if(ansi_) { getOutput().print("\033[00m\033[K"); getOutput().flush(); }
    if(ansi_) { getError() .print("\033[00m\033[K"); getError() .flush(); }
    if(isStandAlone()) System.exit(0);
  }
  
  public Object parseMain(Reader _reader, String _origin, boolean _prompt)
       throws RuntimeException
  {
    Object r=null;
    FooTokenizer old=ft_;
    if(old!=null)
      synchronized(old)
      {
	r=parseMain1(_reader,_origin,_prompt);
      }
    else
      r=parseMain1(_reader,_origin,_prompt);
    
    ft_=old;
    return r;
  }

  private Object parseMain1
  (Reader _reader, String _origin, boolean _prompt)
       throws RuntimeException
  {
    Object old_call3 =FooSymbol.get_local("[call-3]");
    Object old_call2 =FooSymbol.get_local("[call-2]");
    Object old_call1 =FooSymbol.get_local("[call-1]");
    Object old_caller=FooSymbol.get_local("[caller]");

    FooSymbol.create_local("[call-3]",null);
    FooSymbol.create_local("[call-2]",null);
    FooSymbol.create_local("[call-1]",null);
    FooSymbol.create_local("[caller]",null);

    Object r=parseMain0(_reader,_origin,_prompt);

    FooSymbol.forget_local("[call-3]",old_call3);
    FooSymbol.forget_local("[call-2]",old_call2);
    FooSymbol.forget_local("[call-1]",old_call1);
    FooSymbol.forget_local("[caller]",old_caller);

    return r;
  }

  private Object parseMain0
  (Reader _reader, String _origin, boolean _prompt)
       throws RuntimeException
  {
    Object r=null;
    
    ft_=new FooTokenizer(_reader,_origin);

    try
    {
      while(!exited_)
      {
	Object value=parseExpr(_prompt,false);

	if(  (value instanceof FooToken)
	   &&(((FooToken)value).type_==FooToken.PARRIGHT))
	{
	  warning("superflous parenthesis:)");
	  continue;
	}

	if(isStep())
	{
	  if(ansi_) getOutput().print("\033[44m\033[37m   ");
	  else      getOutput().print("\033[44m\033[37m?? ");
	  getOutput().print(FooLib.toSource(value));
	  if(ansi_) getOutput().print("\033[K\033[00m");
	  getOutput().println();
	}

	value=FooLib.eval(value);
	r=value;
	if(_prompt&&(value!=FooVoid.VOID))
	{
	  if(ansi_) getOutput().print("\033[31m\033[47m");
	  getOutput().print("   "+FooLib.toString(value));
	  if(ansi_) getOutput().print("\033[K\033[00m");
	  getOutput().println();
	}
      }
    }
    catch(IOException ex) { }
    
    return r;
  }
  
  public Object parseExpr(boolean _prompt)
       throws IOException
  { return parseExpr(_prompt,true); }

    public Object parseExpr(boolean _prompt,boolean _inside)
	throws IOException
    {
	Object r=null;

	try
	{
	    if(_prompt&&!ft_.ready())
	    {
		getOutput().print(_inside ? ".. " : ">> ");
		// if(ansi_) getOutput().print("\033[K");
		getOutput().flush();
	    }

	    // if(ansi_) getOutput().print("\033[00m");
	    FooToken token=ft_.nextToken();
	    // if(ansi_) getOutput().print("\033[34m");

	    switch(token.type_)
	    {
	    case FooToken.INTEGER:
	    case FooToken.REAL:
	    case FooToken.STRING:
	    case FooToken.IDENTIFIER:
	    case FooToken.EXEC:
	    case FooToken.PATTERN:
		r=token.value_;
		break;
	    
	    case FooToken.PARLEFT:
		r=parseList(_prompt);
		break;
		
	    case FooToken.PARRIGHT:
		r=token;
		break;

	    case FooToken.QUOTE:
		r=parseQuote(_prompt);
		break;

	    case FooToken.COMMA:
		r=parseComma(_prompt);
		break;

		//case FooToken.UNKNOWN: warning("unknown token"); break;
		//case FooToken.COMMENT: warning("comment"); break;
	    default:
		throw new RuntimeException("syntax error:"+token);
	    }
	}
	catch(IOException ex)
	{
	    if(_inside)
		throw new RuntimeException("syntax error:unexpected end");
	    else
		throw new IOException(ex.getMessage());
	}

	return r;
    }

    public FooQuote parseQuote(boolean _prompt)
	throws IOException
    {
	return new FooQuote(parseExpr(_prompt));
    }

    public Object parseComma(boolean _prompt)
	throws IOException
    {
	return new FooComma(parseExpr(_prompt));
    }

    public FooList parseList(boolean _prompt)
	throws IOException
    {
	FooList r=new FooList();

	r.origin_=ft_.origin();
	r.lineno_=ft_.lineno();

	while(true)
	{
	    Object o=parseExpr(_prompt);
	    if(  (o instanceof FooToken)
	       &&(((FooToken)o).type_==FooToken.PARRIGHT))
		break;
	    r.add(o);
	}
	
	return r;
    }

    public void dot(Object[] _p)
    {
	for(int i=0;i<_p.length;i++)
	    getOutput().print((i>0 ? " " : "")+_p[i]);
	getOutput().println();
    }

    public Object debug(Object _o)
    {
	Object v=FooLib.eval(_o);
	getOutput().println(FooLib.toSource(_o)+" --> "+
			    FooLib.toSource(v));
	return v;
    }

    /*
    public void importMsg(Class _class)
    {
      String name=_class.getName();
      int i=name.lastIndexOf('.');
      if(i>=0)
      {
	name=name.substring(i+1);

	Object v=FooSymbol.get_raw(name);

	if(v==null)
	  FooSymbol.create(name).setValue(_class);
	else
	{
	  if(_class!=v)
	  {
	    if(v instanceof Class)
	      throw new RuntimeException
		("ambiguous import:"+_class+":"+v);
	    else
	      throw new RuntimeException
		(name+" is already bound to "+v);
	  }
	}
      }
    }
    */

  public void parseStdin()
  {
    getOutput().println();
    if(ansi_) getOutput().print("\033[30m\033[43m\033[K");
    getOutput().println
      ("Foo interpreter version "+Foo.VERSION+" ["+Foo.DATE+"]");
    if(ansi_) getOutput().print("\033[33m\033[41m\033[K");
    getOutput().println
      ("(c)2000-2001 Guillaume Desnoix <guillaume@desnoix.com>");
    getOutput().println
      ("Distributed under the terms of the GNU General Public "+
       "License 2");
    if(ansi_) getOutput().print("\033[00m\033[K");
    getOutput().println();
    
    while(!exited_)
    {
      try
      {
	parseMain(new InputStreamReader(getInput()),"stdin",true);
      }
      catch(RuntimeException ex)
	{ if(!exited_) error(ex); }
    }
  }

  public Object parsePackage(String _pkg)
  {
    Object r=null;
    try
    {
      InputStream in=getClass().
	getResourceAsStream(_pkg+".foo");
      // if(in==null)
      // throw new RuntimeException("package not found:"+_pkg);

      r=parseMain(new InputStreamReader(in),
		  "package "+_pkg,false);
    }
    catch(Exception ex) { error(ex); }
    return r;
  }

    public Object parseInline(String _code)
    {
	Object r=null;
	try { r=parseMain(new StringReader(_code),"inline",false); }
	catch(Exception ex) { error(ex); }
	return r;
    }

    public void parseString(String _code)
    {
	try { parseMain(new StringReader(_code),"string",false); }
	catch(Exception ex) { error(ex); }
    }

    public void include(Object _o)
    {
      try
      {
	Reader rin=null;

	if(_o instanceof File)
	  rin=new FileReader((File)_o);
	else
	if(_o instanceof URL)
	  rin=new InputStreamReader(((URL)_o).openStream());
	else
	  throw new RuntimeException
	    (FooLib.toString(_o)+" is not a file or an url");

	try { parseMain(rin,FooLib.toString(_o),false); }
	catch(Exception ex) { error(ex); }
      }
      catch(IOException ex)
      {
	throw new RuntimeException
	  ("Error in opening "+FooLib.toString(_o));
      }
    }

  public FooList parallel(Object[] _body)
  {
    int         l=_body.length;
    Object[]    r=new Object[l];
    FooThread[] t=new FooThread[l];

    for(int i=0;i<l;i++)
      t[i]=new FooThread(_body[i]);

    for(int i=0;i<l;i++)
      t[i].start();

    for(int i=0;i<l;i++)
      r[i]=t[i].eval();

    return new FooList().append(r);
  }
  
  public FooList serial(Object[] _body)
  {
    int      l=_body.length;
    Object[] r=new Object[l];

    for(int i=0;i<l;i++)
      r[i]=FooLib.eval(_body[i]);

    return new FooList().append(r);
  }
  
  public FooList any(Object[] _body)
  {
    int         l=_body.length;
    FooThread[] t=new FooThread[l];
    Object      r=null;

    for(int i=0;i<l;i++)
      t[i]=new FooThread(_body[i]);

    for(int i=0;i<l;i++)
      t[i].start();

    boolean again=true;
    while(again)
    {
      Thread.yield();
      //try { Thread.sleep(50); }
      //catch(InterruptedException ex) { }

      for(int i=0;i<l;i++)
	if(!t[i].isAlive())
	{
	  r=t[i].eval();
	  if(r!=FooVoid.VOID)
	  {
	    again=false;
	    break;
	  }
	}
    }

    for(int i=0;i<l;i++)
      t[i].kill();

    FooList v=new FooList();
    v.add(r);
    return v;
  }
  
  public FooList some(Object[] _body)
  {
    int         l=_body.length;
    FooThread[] t=new FooThread[l];
    //Object[]    r=new Object[l];

    //for(int i=0;i<l;i++)
    //r[i]=FooVoid.VOID;

    for(int i=0;i<l;i++)
      t[i]=new FooThread(_body[i]);

    for(int i=0;i<l;i++)
      t[i].start();

    long delay=FooLib.toLong(FooSymbol.get_raw("[some]:timeout")).longValue();
    //System.err.println("DELAY="+delay);
    long before=System.currentTimeMillis();
    long after =before+delay;

    while(System.currentTimeMillis()<after)
    {
      try { Thread.sleep(50); }
      catch(InterruptedException ex) { }
    }

    FooList v=new FooList();

    //for(int i=0;i<l;i++)
      //t[i].kill();

    for(int i=0;i<l;i++)
    {
      if(t[i].isEnded())
	v.add(t[i].eval());
      else
	System.err.println("NOT FINISHED "+t[i]);
    }

    System.err.println("$$$ V="+v);
    System.err.println("END SOME");

    if((l>0)&&v.empty()) v.add(null);
    return v;
  }

  public FooThread fork(Object _body)
  {
    Thread t=Thread.currentThread();
    if(t instanceof FooThread)
	t=((FooThread)t).parent();

    FooThread r=new FooThread(_body,t,FooVoid.VOID);
    r.start();
    return r;
  }

  public Object timeout(int _d, Object _o)
  {
    //FooLib.checkClassArgument(Number.class,_o[0],0);

    Object    r=null;
    int       d=Math.max(0,_d);//((Number)_o[0]).intValue());

    /*
    int      l=_o.length;
    Object[] b=new Object[l-1];
    for(int i=1;i<l;i++) b[i-1]=_o[i];
    FooThread t=new FooThread(b);
    */

    FooThread t=new FooThread(_o);

    long before=System.currentTimeMillis();
    long after =before+d;
    long now;

    t.start();
    //System.err.println("$$$ T.alive="+t.isAlive());
    while(t.isAlive()&&((now=System.currentTimeMillis())<after))
    {
      try { Thread.sleep(20); }
      catch(InterruptedException ex) { }
    }

    if(!t.isAlive())
      r=t.eval();
    else
    {
      //System.err.println("$$$ T.kill");
      //r=new Long(System.currentTimeMillis()-before);
      t.kill();
    }
    //System.err.println("$$$ R      ="+r);
    return r;
  }

  public Object first(Object[] _body)
  {
    int    l=_body.length;
    Object r=null;

    if(l>=1)
    {
      r=FooLib.eval(_body[0]);
      for(int i=1;i<l;i++)
	FooLib.eval(_body[i]);
    }

    return r;
  }
  
  public Object last(Object[] _body)
  {
    int    l=_body.length;
    Object r=null;

    for(int i=0;i<l;i++)
      r=FooLib.eval(_body[i]);

    return r;
  }
  
  public Object trace(Object[] _body)
  {
    setTrace(true);
    Object r=FooLib.evalBlock(_body,0);
    setTrace(false);
    return r;
  }
  
  public Object tryMsg(Object[] _body)
  {
    Object r=null;

    try { r=FooLib.evalBlock(_body,0); }
    catch(RuntimeException ex)
    {
      //error(ex);
      r=FooVoid.VOID;
    } //r=ex.getMessage(); }

    return r;
  }
  
  public void throwMsg(String _s)
  {
    throw new RuntimeException(_s);
  }
  
  public void loop(Object[] _body)
  {
    try
    {
      while(true)
      {
	FooLib.evalBlock(_body,0);
      }
    }
    catch(FooBreakException ex)
    {
    }
  }
  
  public void breakMsg()
  {
    throw new FooBreakException();
  }
  
  public double random()
  {
    return Math.random();
  }
  
  public String toString()
  {
    return "interpreter Foo";
  }

  public String toSource()
  {
    return "?";
  }

  public void install()
  {
    FooCategory   .init();
    FooList       .init();
    FooParser     .init();
    FooSymbol     .init();

    FooArray      .init();
    FooBoolean    .init();
    FooClass      .init();
    FooObject     .init();
    FooFile       .init();
    FooInputStream.init();
    FooFix    .init();
    FooNumber     .init();
    FooPrintStream.init();
    FooReal       .init();
    FooString     .init();
    FooURL        .init();

      /*
	Hashtable registry=FooCategory.classes_;

	registry.put(Boolean     .class,FooBoolean     .init());
	registry.put(Byte        .class,FooFix     .init());
	registry.put(Short       .class,FooFix     .init());
	registry.put(Integer     .class,FooFix     .init());
	registry.put(Long        .class,FooFix     .init());
	registry.put(Float       .class,FooReal        .init());
	registry.put(Double      .class,FooReal        .init());

	registry.put(FooLib.getArrayclass(Boolean.TYPE),FooArray.init());
	registry.put(FooLib.getArrayclass(Byte   .TYPE),FooArray.init());
	registry.put(FooLib.getArrayclass(Short  .TYPE),FooArray.init());
	registry.put(FooLib.getArrayclass(Integer.TYPE),FooArray.init());
	registry.put(FooLib.getArrayclass(Long   .TYPE),FooArray.init());
	registry.put(FooLib.getArrayclass(Float  .TYPE),FooArray.init());
	registry.put(FooLib.getArrayclass(Double .TYPE),FooArray.init());
	registry.put(Object[]    .class                ,FooArray.init());

	registry.put(Class       .class,FooClass       .init());
	registry.put(Object      .class,FooObject      .init());
	registry.put(File        .class,FooFile        .init());
	registry.put(InputStream .class,FooInputStream .init());
	registry.put(Number      .class,FooNumber      .init());
	registry.put(PrintStream .class,FooPrintStream .init());
	registry.put(String      .class,FooString      .init());
	registry.put(URL         .class,FooURL         .init());

	registry.put(FooCategory .class,FooCategory    .init());
	registry.put(FooList     .class,FooList        .init());
	registry.put(FooParser   .class,FooParser      .init());
	registry.put(FooSymbol   .class,FooSymbol      .init());
	*/

    parsePackage("array");
    parsePackage("boolean");
    parsePackage("fix");
    parsePackage("list");
    parsePackage("object");
    parsePackage("symbol");
  }

  /*
  public Class classMsg(FooSymbol _name,FooSymbol _mother,FooList _fields)
       throws Exception
  {
    String   n=_name  .getName();
    String   m=_mother.getName();
    int      l=_fields.length();

    String[] f=new String[l];
    for(int i=0;i<l;i++)
      f[i]=((FooSymbol)_fields.at(i)).getName();

    Class r=FooBytecode.create_class(n,m,f);
    return r;
  }
  */
}
