/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FooReal.java
 * @version      0.08
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2001 Guillaume Desnoix
 */

package com.memoire.foo;
import com.memoire.foo.*;

public class FooReal
{
  private static FooCategory pkg_=null;

  public final static FooCategory init()
  {
    if(pkg_==null)
    {
      pkg_=FooCategory.create("&real");
      FooCategory.create(Double.class).alias("&double").extend(pkg_);
      FooCategory.create(Float.class ).alias("&float" ).extend(pkg_);

      pkg_.setMessage("+" ,FooReal.class,"addMsg");
      pkg_.setMessage("-" ,FooReal.class,"subMsg");
      pkg_.setMessage("*" ,FooReal.class,"mulMsg");
      pkg_.setMessage("/" ,FooReal.class,"divMsg");
      pkg_.setMessage("<" ,FooReal.class,"ltMsg");
      pkg_.setMessage("<=",FooReal.class,"leMsg");
      pkg_.setMessage(">" ,FooReal.class,"gtMsg");
      pkg_.setMessage(">=",FooReal.class,"geMsg");
      pkg_.setMessage("==",FooReal.class,"eqMsg");
      pkg_.setMessage("!=",FooReal.class,"neMsg");
      pkg_.setMessage("~" ,FooReal.class,"mtMsg");
      pkg_.setMessage("!~",FooReal.class,"nmMsg");
      pkg_.setMessage("" ,FooReal.class,"sqrMsg");
      pkg_.setMessage("**",Math.class   ,"pow(double,double)");
    }

    return pkg_;
  }

    private final static Double ZERO=new Double(0.);
    private final static Double ONE =new Double(1.);
    private final static Double TWO =new Double(2.);

    public static Double create(double _value)
    {
	Double r;
	     if(_value==0.) r=ZERO;
	else if(_value==1.) r=ONE;
	else if(_value==2.) r=TWO;
	else r=new Double(_value);
	return r;
    }
    
    public static Number addMsg(Object[] _n)
    {
	return operation(_n,0);
    }

    public static Number subMsg(Object[] _n)
    {
	return operation(_n,1);
    }

    public static Number mulMsg(Object[] _n)
    {
	return operation(_n,2);
    }

    public static Number divMsg(Object[] _n)
    {
	return operation(_n,3);
    }

    private static Number operation(Object[] _n, int _op)
    {
	FooLib.checkClassArgument(Number.class,_n[0],0);
	double r=((Number)_n[0]).doubleValue();
	for(int i=1;i<_n.length;i++)
	{
	    FooLib.checkClassArgument(Number.class,_n[i],i);
	    double a=((Number)_n[i]).doubleValue();
	    switch(_op)
	    {
	    case 0: r+=a; break;
	    case 1: r-=a; break;
	    case 2: r*=a; break;
	    case 3: r/=a; break;
	    }
	}
	return create(r);
    }    

    public static Boolean ltMsg(Object[] _n)
    {
	return comparison(_n,0);
    }

    public static Boolean leMsg(Object[] _n)
    {
	return comparison(_n,1);
    }

    public static Boolean gtMsg(Object[] _n)
    {
	return comparison(_n,2);
    }

    public static Boolean geMsg(Object[] _n)
    {
	return comparison(_n,3);
    }

    public static Boolean eqMsg(Object[] _n)
    {
	return comparison(_n,4);
    }

    public static Boolean neMsg(Object[] _n)
    {
	return comparison(_n,5);
    }

    public static Boolean mtMsg(Object[] _n)
    {
	return comparison(_n,6);
    }

    public static Boolean nmMsg(Object[] _n)
    {
	return comparison(_n,7);
    }

    private static Boolean comparison(Object[] _n, int _op)
    {
	FooLib.checkClassArgument(Number.class,_n[0],0);
	boolean r=true;
	double v=((Number)_n[0]).doubleValue();
	for(int i=1;i<_n.length;i++)
	{
	    FooLib.checkClassArgument(Number.class,_n[i],i);
	    double a=((Number)_n[i]).doubleValue();
	    switch(_op)
	    {
	    case 0: r&=(v< a);  break;
	    case 1: r&=(v<=a);  break;
	    case 2: r&=(v> a); break;
	    case 3: r&=(v>=a); break;
	    case 4: r&=(v==a); break;
	    case 5: r&=(v!=a); break;
	    case 6: r&=(Math.round(v-a)==0.); break;
	    case 7: r&=(Math.round(v-a)!=0.); break;
	    }
	}
	return FooBoolean.create(r);
    }    

    public static Number sqrMsg(Number _a)
    {
	double a=_a.doubleValue();
	return create(a*a);
    }
}
