/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FooSymbol.java
 * @version      0.08
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2001 Guillaume Desnoix
 */

package com.memoire.foo;
import com.memoire.foo.*;

import com.memoire.re.*;
//import gnu.regexp.*;

import java.io.*;
import java.net.*;
import java.util.*;

public class FooSymbol
       implements FooEval
{
  private static FooCategory pkg_=null;

  public final static FooCategory init()
  {
    if(pkg_==null)
    {
      pkg_=FooCategory.create(FooSymbol.class);
      pkg_.alias("&symbol");

      pkg_.setMessage("=",FooSymbol.class,
		      "setValue(java.lang.Object)");
      pkg_.setMessage("=:",FooSymbol.class,
		      "setDynamicValue(java.lang.Object)");
      pkg_.setMessage("bound?",FooSymbol.class,"boundMsg");
      pkg_.setMessage(":",FooSymbol.class,"localMsg",false);
    }
    
    return pkg_;
  }

  private static Hashtable SYMBOLS=new Hashtable(21);
  //static Hashtable types_   =new Hashtable(21);
  //static Hashtable readonly_=new Hashtable(21);

  private static Hashtable THREADS=new Hashtable(11);

    static
    {
	put_raw("boolean",Boolean.TYPE);
	put_raw("byte"   ,Byte   .TYPE);
	put_raw("short"  ,Short  .TYPE);
	put_raw("int"    ,Integer.TYPE);
	put_raw("long"   ,Long   .TYPE);
	put_raw("float"  ,Float  .TYPE);
	put_raw("double" ,Double .TYPE);

	put_raw("true"   ,Boolean.TRUE);
	put_raw("false"  ,Boolean.FALSE);
	put_raw("null"   ,FooNull.NULL);
	put_raw("void"   ,FooVoid.VOID);
	put_raw("default",FooDefault.DEFAULT);

	put_raw("Lib",FooLib.class);
	
	put_raw("{home}" ,"(java.io.File new user.home)");
	put_raw("{pwd}"  ,"(java.io.File new user.dir)");
	put_raw("~"      ,"(java.io.File new user.home)");

	//put_raw("{list}","FooList new");
	//put_raw("{agent}","com.memoire.foo.FooAgent");
	//put_raw("dix"    ,new Integer(10));
	//put_raw("desnoix","guillaume");
    }

  private String name_;

  private FooSymbol(String _name)
  {
    name_=_name;
  }

  public int hashCode()
  {
    return getName().hashCode();
  }

  public boolean equals(Object _o)
  {
    if(_o==this) return true;
    if(_o==null) return false;

    return   (_o instanceof FooSymbol)
           &&getName().equals(((FooSymbol)_o).getName());
  }

  public void setValue(Object _value)
  {
    setValue(name_,_value);
  }

  public void setDynamicValue(Object _value)
  {
    setValue(name_,FooDynamic.create(_value));
  }

  public Object getValue()
  {
    return getValue(name_);
  }
  
  public Object eval()
  {
    return getValue();
  }

  public boolean boundMsg()
  {
    return (get_raw(name_)!=null);
  }
  
  public Object localMsg(Object[] _body)
  {
    Object r=null;
    String n=name_;
    Object old_var=FooSymbol.get_local(n);
    
    create_local(n,null);
    r=FooLib.evalBlock(_body,0);
    forget_local(n,old_var);
    
    return r;
  }

  public String getName()
  {
    return name_;
  }
  
  public String toString()
  {
    return name_;
  }

  // static

  public static FooSymbol create(String _name)
  {
    return new FooSymbol(_name);
  }
  
  static synchronized Object get_raw(String _name)
  {
    Thread t=Thread.currentThread();

    while(t instanceof FooThread)
    {
      Hashtable s=(Hashtable)THREADS.get(t);
      Object    r=null;

      if(s!=null) r=s.get(_name);

      if(r!=null)
      {
	if(r==FooNull.NULL) r=null;
	//System.err.println("GET "+_name+" "+r+" "+t);
	return r;
      }

      t=((FooThread)t).parent();
    }
    
    return SYMBOLS.get(_name);
  }

  static synchronized void put_raw(String _name, Object _value)
  {
    Thread t=Thread.currentThread();

    while(t instanceof FooThread)
    {
      Hashtable s=(Hashtable)THREADS.get(t);
      Object    r=null;

      if(s==null)
      {
	s=new Hashtable(11);
	THREADS.put(t,s);
      }

      r=s.get(_name);

      if(r!=null)
      {
	//System.err.println("PUT "+_name+"="+_value+" "+t);
	if(_value==null) s.put(_name,FooNull.NULL);
	else             s.put(_name,_value);
	return;
      }

      t=((FooThread)t).parent();
    }

    if(_value==null) SYMBOLS.remove(_name);
    else             SYMBOLS.put(_name,_value);
  }

  static synchronized void create_local(String _name,Object _new)
  {
    Thread t=Thread.currentThread();

    if(!(t instanceof FooThread))
    {
      if(_new==null) SYMBOLS.remove(_name);
      else           SYMBOLS.put(_name,_new);
    }
    else
    {
      Hashtable s=(Hashtable)THREADS.get(t);
      
      if(s==null)
      {
	s=new Hashtable(11);
	THREADS.put(t,s);
      }

      //System.err.println("CREATE "+_name+" "+t);
      if(_new==null) _new=FooNull.NULL;
      s.put(_name,_new);
    }

    return;
  }

  static synchronized void forget_local(String _name, Object _old)
  {
    Thread t=Thread.currentThread();

    if(!(t instanceof FooThread))
    {
      if(_old==null) SYMBOLS.remove(_name);
      else           SYMBOLS.put(_name,_old);
    }
    else
    {
      Hashtable s=(Hashtable)THREADS.get(t);
      
      if(s==null)
	throw new RuntimeException
	  ("forget_local:internal error");

      //System.err.println("FORGET "+_name+" "+t);
      if(_old==null) s.remove(_name);
      else           s.put(_name,_old);
    }

    return;
  }

  static synchronized Object get_local(String _name)
  {
    Thread t=Thread.currentThread();

    if(t instanceof FooThread)
    {
      Hashtable s=(Hashtable)THREADS.get(t);
      Object    r=null;

      if(s!=null) r=s.get(_name);
      return r;
    }
    
    return SYMBOLS.get(_name);
  }

  static synchronized Object get_global(String _name)
  {
    return SYMBOLS.get(_name);
  }

  static synchronized void set_global(String _name,Object _new)
  {
    if(_new==null) SYMBOLS.remove(_name);
    else           SYMBOLS.put(_name,_new);
  }

  static void list_raw()
  {
    Enumeration e=SYMBOLS.keys();
    while(e.hasMoreElements())
    {
      String n=(String)e.nextElement();
      System.err.println(n+"="+SYMBOLS.get(n));
    }
  }

  // public

  public static void setValue(String _name, Object _value)
  {
    if(_name==null)
      throw new RuntimeException
	("can not define null symbol:"+
	 _name+":"+_value);
    
    if(_name.startsWith("[")&&_name.endsWith("]"))
      throw new RuntimeException
	("can not define meta-symbol:"+
	 _name+":"+_value);

	Object previous=get_raw(_name);

	if(previous!=null)
	{
	    if(previous instanceof FooCategory)
	    {
		/*
		  if(_value instanceof FooCategory)
		  FooLib.getParser().warning
		  ("suspicious categoryname change:"
		  +_name+":"+_value);
		  else
		*/
		throw new RuntimeException
		    ("can not change category"+
		     //" name for a non-category value"+
		     ":"+_name+":"+_value);
	    }
	}
	else // previous is null
        {
	    try { previous=Class.forName(_name); }
	    catch(Exception ex) { }
	    if(previous!=null)
	    {
		if(_value instanceof Class)
		    FooLib.getParser().warning
			("suspicious classname change:"
			 +_name+":"+_value);
		else
		    throw new RuntimeException
			("can not change classname for a non-class value:"+
			 _name+":"+_value);
	    }

	    try
	    {
		previous=System.getProperty(_name);
		if(previous!=null)
		{
		    if(_value instanceof String)
		    {
			System.getProperties().put(_name,_value);
			return;
		    }
		    else
			throw new RuntimeException
			    ("can not change system property for a "+
			     "non-string value:"+_name+":"+_value);
		}
	    }
	    catch(SecurityException ex)
	    {
		throw new RuntimeException
		    ("can not access system properties:"+
		     _name);
	    }

	    if(_name.startsWith("./")||_name.startsWith("/"))
	    {
		File f=new File(_name);
		if(f.exists())
		    throw new RuntimeException
			("can not change a filename:"+
			 _name);
	    }
	}

	if(_value==null) _value=FooNull.NULL;
	put_raw(_name,_value);
    }

  private final static Object getValue0(String _name)
  {
    Object r=get_raw(_name);

    if(_name.endsWith("[]"))
    {
      String s=_name.substring(0,_name.length()-2);
      r=getValue0(s);
      if(r instanceof Class)
	r=FooLib.getArrayclass((Class)r);
      else
	r=null;
    }

    if(r==null) // &&(_name.indexOf('/')<0))
    {
      try { r=Class.forName(_name); }
      catch(Exception ex) { }
    }

    return r;
  }

  public static Object getValue(String _name)
  {
    Object r=getValue0(_name);

    if(r==null)
    {
      r=FooCategory.get_raw(_name);
    }

    if(r==null)
    {
      try { r=System.getProperty(_name); }
      catch(SecurityException ex) { }
    }

    if((r==null)&&(_name.startsWith("./")||_name.startsWith("/")))
    {
      File f=new File(_name);
      if(f.exists()&&f.canRead())
	r=f;
    }
    
    if(  (r==null)&&_name.startsWith("/")
       &&(_name.endsWith("/")||_name.endsWith("/i")))
    {
      boolean ignoreCase=_name.endsWith("i");
      if(ignoreCase) _name=_name.substring(1,_name.length()-2);
      else           _name=_name.substring(1,_name.length()-1);

      try
      {
	r=new RE(_name,
		 (ignoreCase==true ? RE.REG_ICASE : 0) | RE.REG_MULTILINE,
		 RESyntax.RE_SYNTAX_PERL5);
      }
      catch(REException ex)
      {
	r=null;
	throw new RuntimeException
	  ("unvalid regexp:"+_name);
      }
    }

    if(r==null)
    {
      if(_name.indexOf(':')>0)
      {
	try { r=new URL(_name); }
	catch(MalformedURLException ex) { }
      }
    }

    if(r==null)
      throw new RuntimeException
	("symbol "+_name+" is not bound");
    
    if(r instanceof FooDynamic)
    {
      r=((FooDynamic)r).dynamic();
    }

    if(r==FooNull.NULL) r=null;
    return r;
  }

  public String toSource()
  {
    String  r=getName();
    int     l=r.length();
    boolean b=false;

    for(int i=0;i<l;i++)
    {
      char c=r.charAt(i);
           if(FooTokenizer.isSpace (c)) b=true;
      else if(FooTokenizer.isSingle(c)) b=true;
      else if(FooTokenizer.isDouble(c)) b=true;
    }

    if(b) r="|"+r+"|";
    return r;
  }
}
