/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FooTokenizer.java
 * @version      0.08
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2001 Guillaume Desnoix
 */

package com.memoire.foo;
import com.memoire.foo.*;

import java.io.*;

public class FooTokenizer
{
  private Reader in_;
  private char[] buf_;
  private int    nbc_;
  private int    lineno_,charno_;
  private String origin_;

  public FooTokenizer(Reader _in, String _origin)
  {
    in_=_in;
    buf_=new char[1024];
    nbc_=0;
    origin_=_origin;
    lineno_=1;
    charno_=1;
  }

  public boolean ready() throws IOException
  {
      while(in_.ready()&&isSpace(peek())) read();
      return in_.ready();
  }

  public String origin() { return origin_; }
  public int    lineno() { return lineno_; }
  public int    charno() { return charno_; }

  private char read0() throws IOException
  {
    char r;

    if(nbc_>0)
    {
      nbc_--;
      r=buf_[nbc_];
    }
    else
    {
      int c=in_.read();
      if(c==-1) throw new EOFException();
      r=(char)c;
      if(r=='\n') { charno_=1; lineno_++; }
      else charno_++;
    }

    return r;
  }

  private char read() throws IOException
  {
    char r=read0();
    while(r==12)
    {
      FooSymbol.list_raw();
      r=read0();
    }
    return r;
  }

  private char peek()
  {
    char r;

    try { r=read(); put(r); }
    catch(IOException ex) { r=(char)-1; }

    return r;
  }
    
  private void put(char _c)
  {
     buf_[nbc_]=_c;
     nbc_++;
  }

  // test

  final static boolean isSpace(char c)
  {
    return Character.isWhitespace(c);
  }

  final static boolean isSingle(char c)
  {
    return
	// (c=='.')||
	(c==',')||(c=='\'')||
	(c=='(')||(c==')')||
        //(c=='[')||(c==']')||
	(c=='}');
  }

  final static boolean isDouble(char c)
  {
    return
	(c=='"')||(c=='|')||(c=='`');
  }

  // tokenize

  private boolean first_=true;

  public FooToken nextToken() throws IOException
  {
    if(first_&&peek()=='#')
    {
	try { while(read()!='\n') ; }
	catch(IOException ex) { }
    }
    first_=false;

    char c;
    while(isSpace(c=read())) ;

    while(c==';')
    {
      while((c=read())!='\n') ;
      while(isSpace(c=read())) ;
    }

    FooSymbol.create_local("[origin]",origin_);    
    FooSymbol.create_local("[lineno]",FooFix.create(lineno_));
    FooSymbol.create_local("[charno]",FooFix.create(charno_));

    int    t=FooToken.UNKNOWN;
    Object r=null;

    StringBuffer sb=new StringBuffer(10);

    if(isSingle(c))
    {
      r=""+c; // new Character(c);
           if(c=='(' ) t=FooToken.PARLEFT;
      else if(c==')' ) t=FooToken.PARRIGHT;
      else if(c=='\'') t=FooToken.QUOTE;
      else if(c==',' ) t=FooToken.COMMA;
      //else if(c=='[' ) t=FooToken.STRING;
      //else if(c==']' ) t=FooToken.STRING;
      else             t=FooToken.UNKNOWN;
    }
    else
    if(isDouble(c))
    {
      char d=c;
      while((c=read())!=d)
      {
	if(c=='\\')
        {
	  c=read();
	       if(c=='g' ) c='\007';
	  else if(c=='n' ) c='\n';
	  else if(c=='r' ) c='\r';
	  else if(c=='t' ) c='\t';
	  else if(c=='\\') c='\\';
	  else if(c=='\"') c='\"';
	  else if(c=='\'') c='\'';
	  else if(c=='`' ) c='`' ;
	  else
	    FooLib.getParser().warning("suspicious antislah:\\"+c);
	}
	sb.append(c);
      }

      if(d=='"')
      {
	  r=sb.toString();
	  t=FooToken.STRING;
      }
      else
      if(d=='|')
      {
	  r=FooSymbol.create(sb.toString());
	  t=FooToken.IDENTIFIER;
      }
      else
      {
	  r=FooExec.create(sb.toString());
	  t=FooToken.EXEC;
      }
    }
    else
    if(c=='{')
    {
      while((c=read())!='}')
      {
	sb.append(c);
      }
      Object o=null;
      try { o=FooSymbol.getValue("{"+sb+"}"); }
      catch(RuntimeException ex) { }

      if(o instanceof String)
      {
	  String s=(String)o;
	  for(int i=s.length()-1;i>=0;i--) put(s.charAt(i));
	  return nextToken();
      }
      else
      if(o==null)
	  throw new RuntimeException("unknown abbreviation:"+sb);
      else
	  throw new RuntimeException("bad abbreviation:"+sb+":not a string");
    }
    else
    if((c=='0')&&(peek()=='b'))
    {
      read();
      while(((c=peek())>='0')&&(c<='1')) sb.append(read());
      r=new Integer(Integer.parseInt(sb.toString(),2));
      t=FooToken.INTEGER;
    }
    else
    if((c=='0')&&(peek()=='o'))
    {
      read();
      while(((c=peek())>='0')&&(c<='7')) sb.append(read());
      r=new Integer(Integer.parseInt(sb.toString(),8));
      t=FooToken.INTEGER;
    }
    else
    if((c=='0')&&(peek()=='d'))
    {
      read();
      while(((c=peek())>='0')&&(c<='9')) sb.append(read());
      r=new Integer(Integer.parseInt(sb.toString(),10));
      t=FooToken.INTEGER;
    }
    else
    if((c=='0')&&(peek()=='x'))
    {
      read();
      while(  (((c=peek())>='0')&&(c<='9'))
	    ||((c>='A')&&(c<='F'))
	    ||((c>='a')&&(c<='f'))) sb.append(read());
      r=new Integer(Integer.parseInt(sb.toString(),16));
      t=FooToken.INTEGER;
    }
    else
    if(  Character.isDigit(c)
	 ||((c=='-')&&Character.isDigit(peek()))
	 ||((c=='+')&&Character.isDigit(peek())))
    {
      sb.append(c);
      while(Character.isDigit(peek())) sb.append(read());
      t=FooToken.INTEGER;
      if(peek()=='.')
      {
	sb.append(read());
	while(Character.isDigit(peek())) sb.append(read());
	t=FooToken.REAL;
      }
      if(peek()=='E')
      {
	sb.append(read());
	if(peek()=='+') sb.append(read());
	else if(peek()=='-') sb.append(read());
	while(Character.isDigit(peek())) sb.append(read());
	t=FooToken.REAL;
      }
      if(t==FooToken.REAL) r=new Double(sb.toString());
      else                 r=new Integer(sb.toString());
    }
    else
    {
      while(true)
      {
	sb.append(c);
	c=peek();
	if(isSpace(c)||isSingle(c)) break;
	read();
      }
      r=FooSymbol.create(sb.toString());
      t=FooToken.IDENTIFIER;
    }

    return new FooToken(t,r);
  }
}
