/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FuLib.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.fu;

import com.memoire.fu.*;


import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.text.*;

/**
 * Utility class with only static methods.
 * For small and useful services.
 */
public /*final*/ class FuLib
{
  /**
   * replace parts of a string.
   * @param _s the initial string
   * @param _a the string to be found
   * @param _b the string which will replace
   * @return the modified string
   */
  public final static String replace(String _s, String _a, String _b)
  {
    String r=_s;
    int i=0;
    while((i=r.indexOf(_a,i))>=0)
    {
      r=r.substring(0,i)+_b+r.substring(i+_a.length());
      i=i+_b.length();
    }
    return r;
  }

  /**
   * replace parts of a string, ignoring case.
   * @param _s the initial string
   * @param _a the string to be found
   * @param _b the string which will replace
   * @return the modified string
   */
  public final static String replaceIgnoreCase
    (String _s, String _a, String _b)
  {
    String r=_s;
    int i=0;

    _a=_a.toLowerCase();
    while((i=r.toLowerCase().indexOf(_a,i))>=0)
    {
      r=r.substring(0,i)+_b+r.substring(i+_a.length());
      i=i+_b.length();
    }
    return r;
  }

  /**
   * @return the current time in seconds.
   */
  public final static int currentTimeSeconds()
  {
    return (int)(System.currentTimeMillis()/1000l);
  }

  /**
   * @return the current time in milliseconds.
   */
  public final static long currentTimeMillis()
  {
    return System.currentTimeMillis();
  }

  public final static String getSystemProperty(String _property)
  {
    String r=null;

    try { r=System.getProperty(_property); }
    catch(Throwable th)
    {
      /*
      if("user.home".equals(_property)) r="/";
      else if("user.dir" .equals(_property)) r="/";
      else
      */
      //System.err.println("Can not read system property "+_property);
    }

    return r;
  }

  public final static String getUserHome()
  {
    String r=getSystemProperty("user.home");
    if(r==null) r="";
    return r;
  }

  /**
   * Creates a calendar CET.
   * @return the calendar.
   */
  public final static Calendar calendarCET()
  {
    String[]       ids=TimeZone.getAvailableIDs(+1*60*60*1000);
    SimpleTimeZone cet=new SimpleTimeZone(+1*60*60*1000,ids[0]);
    cet.setStartRule(Calendar.APRIL,   1,Calendar.SUNDAY,2*60*60*1000);
    cet.setEndRule  (Calendar.OCTOBER,-1,Calendar.SUNDAY,2*60*60*1000);
    return new GregorianCalendar(cet);
  }

  /**
   * @return a nicely formated date (today).
   */
  public final static String date()
  {
    return date(System.currentTimeMillis());
  }

  /**
   * @return a nicely formated date.
   */
  public final static String date(long _time)
  {
    Date           now=new Date(_time);
    Calendar       cal=calendarCET();
    DateFormat     fmt;

    cal.setTime(now);
    fmt=DateFormat.getDateInstance(DateFormat.LONG,Locale.getDefault());
    fmt.setCalendar(cal);

    return fmt.format(now);
  }

  /**
   * @return a nicely formated time (today).
   */
  public final static String time()
  {
    return time(System.currentTimeMillis());
  }

  /**
   * @return a nicely formated date.
   */
  public final static String time(long _time)
  {
    Date           now=new Date(_time);
    Calendar       cal=calendarCET();
    DateFormat     fmt;

    cal.setTime(now);
    fmt=DateFormat.getTimeInstance(DateFormat.MEDIUM,Locale.US);
    //getDefault());
    fmt.setCalendar(cal);

    String r=fmt.format(now);
    int    i=r.indexOf(' ');
    if(i>=0) r=r.substring(0,i);
    return r;
  }

  /**
   * @return a nicely formated duration.
   */
  public final static String duration(long _d)
  {
    _d=_d/1000;
    long s=_d%60; _d/=60;
    long m=_d%60; _d/=60;
    long h=_d%24; _d/=24;

    long j=_d%30; _d/=30;
    long n=_d%12; _d/=12;
    long a=_d;

    String r="";

    r+=h; r+=':';
    if(m<10) r+="0"; r+=m; r+=':';
    if(s<10) r+="0"; r+=s;

    if((j>0)||(n>0)||(a>0))
    {
      r=j+"j "+r;
      if(a>0) r=n+"m "+r;
      if(a>0) r=a+"a "+r;
    }

    return r;
  }

  /**
   * Runs an external command inside a shell.
   * @param _cmd the command line
   * @return the output of the command execution
   */

  public static String runShellCommand(String _cmd)
  {
    ByteArrayOutputStream out=new ByteArrayOutputStream();
    int                   res=0;
    String                r  =null;

    try
    {
      Process p=Runtime.getRuntime().exec
	(new String[] { "/bin/sh", "-c", _cmd } );

      BufferedReader in=new BufferedReader
	(new InputStreamReader(p.getInputStream()));
      BufferedReader err=new BufferedReader
	(new InputStreamReader(p.getErrorStream()));

      String lin =null;
      String lerr=null;

      while(  ((lin=in.readLine())!=null)
	    ||((lerr=err.readLine())!=null))
      {
	if(lin!=null)
	{
	  out.write(lin.getBytes());
	  out.write('\n');
	}
	if(lerr!=null)
	{
	  out.write(lerr.getBytes());
	  out.write('\n');
	}
      }
      in .close();
      err.close();

      res=p.waitFor();
    }
    catch(Throwable th)
    {
      PrintWriter pw=new PrintWriter(out);
      th.printStackTrace(pw);
      pw.close();
    }
    finally
    {
      r=new String(out.toByteArray());
      if(res!=0)
      {
	if(!"".equals(r)) r+="\n--\n";
	r+="Erreur  l'excution ["+res+"]\n";
      }
    }

    return r;
  }

  /**
   * Reduce a path by replacing the user's home dir.
   * @return the reduced path.
   */
  public final static String reducedPath(String _p)
  {
    String r=_p;

    String h=getSystemProperty("user.home");
    //int    i=r.lastIndexOf(h);
    //if(i>=0) r="~"+r.substring(i+h.length());
    if(r.startsWith(h)) r="~"+r.substring(h.length());

    return r;
  }

  /**
   * Expand a path with the full user's home dir.
   * @return the expanded path.
   */
  public final static String expandedPath(String _p)
  {
    String r=_p;

    String h=getSystemProperty("user.home");
    //int    i=r.lastIndexOf('~');
    //if(i==0) r=h+r.substring(1);
    if(r.startsWith("~")) r=h+r.substring(1);

    return r;
  }

  /**
   * @return the filename part of a path.
   */
  public final static String fileName(String _p)
  {
    String r=_p;
    String s=getSystemProperty("file.separator");

    int    i=r.lastIndexOf(s);
    if(i>=0) r=r.substring(i+1);

    return r;
  }

  /**
   * @return true if the file is gziped.
   */
  public final static boolean isGziped(String _p)
  {
    boolean r=false;
    try
    {
      FileInputStream fis=new FileInputStream(_p);
      int c1=fis.read();
      int c2=fis.read();
      //System.err.println("C1="+c1+" C2="+c2);
      r=(c1==31)&&(c2==139);
      fis.close();
    }
    catch(IOException ex) { }
    return r;
  }

  /**
   * @return true if the class of the given name exists.
   */
  public final static boolean classExists(String _s)
  {
    boolean r=false;

    try
    {
      Class c=Class.forName(_s);
      r=(c!=null);
    }
    catch(ClassNotFoundException ex) { }

    return r;
  }

  /**
   * Tests if a class exists. This method is better because
   * it doesn't load the class but it doesn't test the dependencies.
   * @param _base a class in the same package
   * @param _name the name of the class (no package)
   * @return true if the class exists.
   */
  /*
  public final static boolean classExists(Class _base, String _name)
  {
    return (_base.getResourceAsStream(_name+".class")!=null);
  }
  */

  /**
   * @return null or an instance of a class.
   */
  public final static Object classInstance(String _s)
  {
    Object r=null;

    try
    {
      Class c=Class.forName(_s);
      r=c.newInstance();
    }
    catch(Exception ex) { }

    return r;
  }
  /**
   * @return the double release number of the JVM.
   */
  public final static double jdk()
  {
    double r=1.1;
    try
    {
      r=new Double(getSystemProperty("java.version")
		   .substring(0,3)).doubleValue();
    }
    catch(Exception ex)
    {
      System.err.println("Unknown JDK version. Switched to 1.1.");
    }
    return r;
  }

  /**
   * @return true if os is windows
   */
  public final static boolean isWindows()
  {
    return (getSystemProperty("os.name").startsWith("Win"));
  }

  /**
   * @return true if os is mac
   */
  public final static boolean isMachintosh()
  {
    return (getSystemProperty("os.name").startsWith("Mac"));
  }

  /**
   * @return true if os is unix
   */
  public final static boolean isUnix()
  {
    return !isWindows()&&!isMachintosh();
  }

  /**
   * Verifies an amount of available memory.
   * @param  _megas number of requested mega-bytes
   * @return true if memory is available
   */
  public final static boolean verifyMemory(int _megas)
  {
    int     m=1048576*_megas;
    boolean r=true;

    /*
    try { System.gc(); }
    catch(Throwable ex) { }
    */

    try
    {
      byte[] b=new byte[m];
      b=null;
    }
    catch(OutOfMemoryError ex) { r=false; }

    /*
    try { System.gc(); }
    catch(Throwable ex) { }
    */

    return r;
  }

  /*
  public static void guess()
  {
    int r=BuPreferences.BU.getIntegerProperty("memory");
    if(r>0) return;

    new BuDialogMessage
      (null,null,
       "I will test your system memory.\n"+
       "It will take a few seconds.")
      .activate();

    System.err.println("Welcome to Alma!");
    System.err.println("For the first time, I will test your system memory.");
    System.err.print("Testing: ");

    int m=1048576;
    try
    {
      while(true)
      {
	m=(m*7)/5;
	byte[] b=new byte[m];
	b=null;
	// System.gc();
	System.err.print("#");
      }
    }
    catch(OutOfMemoryError ex) { }

    m/=1048576;
    m++;
    System.err.println(" "+m+"M");

    if(m<8)
    {
      String t="Found "+m+"M of memory.\n"+
	"You need at least 8M to run safely Alma.\n"+
	"Please use the -mx option of java.";

      System.err.println(t);
      if(m>1) new BuDialogMessage(null,null,t).activate();
      System.exit(0);
    }

    BuPreferences.BU.put("memory",""+m);
  }
  */

  final public static String[] FU_CLASSES=new String[]
  {
    "com.memoire.fu.FuLib",
  };

  public final static String[] JAVA_CLASSES=new String[]
  {
    "java.lang.ClassNotFoundException",
    "java.lang.Exception",
    "java.lang.NullPointerException",
    "java.lang.NoSuchMethodException",
    "java.lang.Object",
    "java.lang.SecurityException",
    "java.lang.VerifyError",
    "java.io.InputStreamReader",
    "java.io.Reader",
    "java.io.StringReader",
    "java.io.Writer",
    "java.io.PrintWriter",
  };

  public static void main(String[] _args)
  {
    System.out.println(duration(9920018678l));
  }
}
