/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FuPreferences.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.fu;

import com.memoire.fu.*;

import java.beans.*;
import java.io.*;
import java.util.*;

/**
 * A class to manage user preferences in a file of properties.
 */

public class FuPreferences
{
  public final static FuPreferences FU=new FuPreferences();

  protected Properties values_;
  protected Vector     listeners_;
    
  // Constructeur "interdit"
  
  protected FuPreferences()
  {
    values_   =new Properties();
    listeners_=new Vector();

    readIniFile();
  }

  public final String getRoot()
  {
    Class  c=getClass();
    String f=c.getName();

    int i=f.lastIndexOf('.');
    if(i>=0) f=f.substring(i+1);

    if(f.endsWith("Preferences")) f=f.substring(0,f.length()-11);
    f=f.toLowerCase();

    return f;
  }

  public final String getPath()
  {
    String f=getRoot();
    String p=FuLib.getUserHome();

    if(!"".equals(p)) p+=File.separator;

         if(FuLib.isWindows()) f=p+f+".ini";
    else if(FuLib.isUnix   ()) f=p+"."+f+"rc";
    else                       f=p+f+".preferences";

    // System.err.println("getPath -> "+f);
    return f;
  }

  public final synchronized void readIniFile()
  {
    // System.err.println(getRoot()+": reading preferences");

    try
    {
      values_.load(new FileInputStream(getPath()));
    }
    catch(Throwable th) //IOException ex
    {
      // System.err.println(getPath()+" not found");
    }
    addDefaultValues();
  }
  
  public final synchronized void writeIniFile()
  {
    // System.err.println(getRoot()+": writing preferences");

    removeDefaultValues();
    try
    {
      values_.save(new FileOutputStream(getPath()),
		  "Preferences for "+getRoot());
    }
    catch(Throwable th) //IOException ex
    {
      System.err.println("Couldn't write "+getPath());
    }
    addDefaultValues();
  }

  protected void addDefaultValues() { }
  protected void removeDefaultValues() { }


  // Basic
  public final Enumeration keys()
  {
    return values_.keys();
  }

  public final int size()
  {
    return values_.size();
  }

  /**
   * -deprecated- use getStringProperty instead
   */
  public final String get(String _key)
  {
    return values_.getProperty(_key);
  }

  /**
    @deprecated use removeProperty instead
  */
  public final void remove(String _key)
  {
    String old=get(_key);
    values_.remove(_key);
    firePropertyChange(_key,old,null);
  }

  /**
    @deprecated use putStringProperty instead
  */
  public final void put(String _key, String _value)
  {
    String old=get(_key);
    if(_value==null) values_.remove(_key);
    else             values_.put(_key,_value);
    firePropertyChange(_key,old,_value);
  }

  protected void firePropertyChange(String _p, String _o, String _n)
  {
    PropertyChangeEvent e=new PropertyChangeEvent(this,_p,_o,_n);
    Enumeration         l=listeners_.elements();
    while(l.hasMoreElements())
      ((PropertyChangeListener)l.nextElement()).propertyChange(e);
  }

  public void addPropertyChangeListener(PropertyChangeListener _l)
  { listeners_.addElement(_l); }

  public void removePropertyChangeListener(PropertyChangeListener _l)
  { listeners_.removeElement(_l); }

  // GET
  
  /*
  public final String getProperty(String _key)
    { return getProperty(_key,""); }

  public final String getProperty(String _key, String _default)
  {
    String r=get(_key);
    if(r==null) r=_default;
    return r;
  }
  */

  /**
    @deprecated use getStringProperty instead
  */
  public final String getProperty(String _key)
    { return getStringProperty(_key); }

  /**
    @deprecated use getStringProperty instead
  */
  public final String getProperty(String _key, String _default)
    { return getStringProperty(_key, _default); }
  
  public final String getStringProperty(String _key)
    { return getStringProperty(_key,""); }

  public final String getStringProperty(String _key, String _default)
  {
    String r=get(_key);
    if(r==null) r=_default;
    return r;
  }
  
  public final boolean getBooleanProperty(String _key)
    { return getBooleanProperty(_key,false); }

  public final boolean getBooleanProperty(String _key, boolean _default)
  {
    String  s=get(_key);
    boolean r=_default;

    if("false".equals(s)) r=false;
    if("true" .equals(s)) r=true;
    if("no"   .equals(s)) r=false;
    if("yes"  .equals(s)) r=true;
    if("0"    .equals(s)) r=false;
    if("1"    .equals(s)) r=true;

    return r;
  }

  public final int getIntegerProperty(String _key)
    { return getIntegerProperty(_key,0); }

  public final int getIntegerProperty(String _key, int _default)
  {
    String  s=get(_key);
    int     r=_default;

    try { r=Integer.parseInt(s); }
    catch(Exception ex) { }

    return r;
  }

  public final double getDoubleProperty(String _key)
    { return getDoubleProperty(_key,0.); }

  public final double getDoubleProperty(String _key, double _default)
  {
    String s=get(_key);
    double r=_default;

    try { r=new Double(s).doubleValue(); } // Double.parseDouble
    catch(Exception ex) { }

    return r;
  }

  // PUT

  public void putStringProperty(String _key, String _value)
  {
    put(_key,_value);
  }

  public void putBooleanProperty(String _key, boolean _value)
  {
    put(_key,""+_value);
  }

  public void putIntegerProperty(String _key, int _value)
  {
    put(_key,""+_value);
  }

  public void putDoubleProperty(String _key, double _value)
  {
    put(_key,""+_value);
  }

  // REMOVE

  public final void removeProperty(String _key)
  {
    if(_key!=null) remove(_key);
  }
}

