/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         FuResource.java
 * @version      0.36
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.fu;

import com.memoire.fu.*;

import java.io.*;
import java.util.*;

/**
 * Utility class to manage resources.
 * As localized strings, ...
 */

public class FuResource
{
  public final static FuResource FU=new FuResource();

  // Strings

  private Properties pp_=null;
  private Locale     pl_=null;

  public String getString(String _s)
  {
    Locale l=Locale.getDefault();

    if("fr".equals(l.getLanguage())) return _s;

    String  r=_s;
    boolean points_suspension=false;
    boolean deux_points=false;

    if(r.endsWith(":"))
    {
      r=r.substring(0,r.length()-1);
      deux_points=true;
    }

    if(r.endsWith("..."))
    {
      r=r.substring(0,r.length()-3);
      points_suspension=true;
    }

    r=r.replace(' ','_');

    if(l!=pl_) pp_=null;

    if(pp_==null)
    {
      Class  c=getClass();
      String f=c.getName();

      int i=f.lastIndexOf('.');
      if(i>=0) f=f.substring(i+1);

      if(f.endsWith("Resource")) f=f.substring(0,f.length()-8);
      f=f.toLowerCase()+"_"+l.getLanguage()+".txt";
      // System.err.println("loading "+f);

      pp_=new Properties();
      pl_=l;

      try
      {
	InputStream sin=c.getResourceAsStream(f);
	// System.err.println("SIN="+sin);
	pp_.load(sin);
      }
      catch(Exception ex)
      {
	// System.err.println(f+" not found");
      }
    }

    String v=pp_.getProperty(r);
    if(v!=null) r=v;

    if(l.getLanguage().equals("eo"))
      r=r.replace('^','x');

    if(l.getLanguage().equals("hu"))
    {
      v=r;
      r="";

      int lv=v.length();
      for(int i=0;i<lv;i++)
      {
	char c=v.charAt(i);
	if((i+1<lv)&&(v.charAt(i+1)=='\''))
	{
	       if(c=='a') r+='\u00E1';
	  else if(c=='o') r+='\u00F3';
	  else if(c=='u') r+='\u00FA';
	  else            { r+=c; r+='\''; }
	  i++;
	}
	else r+=c;
      }
    }

    if(l.getLanguage().equals("ru"))
    {
      byte[] b=null;
      try { b=r.getBytes("ISO8859_1"); }
      catch(UnsupportedEncodingException ex1)
	{ System.err.println("ISO8859_1 Unsupported encoding"); }
      try { r=new String(b,"KOI8_R"); }
      catch(UnsupportedEncodingException ex1)
	{ System.err.println("KOI8_R Unsupported encoding"); }
    }

    r=r.replace('_',' ');
    if(points_suspension) r+="...";
    if(deux_points) r+=":";


    /*
      // BuResource
      if(_s.equals(r)&&getParent()!=null)
      r=getParent().getString(_s);
    */

    return r;
  }
}
