
package com.memoire.jedit;
import com.memoire.jedit.*;

/*
 * JEditBatchFileTokenMarker.java - Batch file token marker
 * Copyright (C) 1998, 1999 Slava Pestov
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */


import javax.swing.text.Segment;

/**
 * Batch file token marker.
 *
 * @author Slava Pestov
 * @version $Id: JEditBatchFileTokenMarker.java,v 1.3 2001/07/25 17:39:05 desnoix Exp $
 */
public class JEditBatchFileTokenMarker extends JEditTokenMarker
{
	public byte markJEditTokensImpl(byte token, Segment line, int lineIndex)
	{
		char[] array = line.array;
		int offset = line.offset;
		int lastOffset = offset;
		int length = line.count + offset;

		if(JEditSyntaxUtilities.regionMatches(true,line,offset,"rem"))
		{
			addJEditToken(line.count,JEditToken.COMMENT1);
			return JEditToken.NULL;
		}

loop:		for(int i = offset; i < length; i++)
		{
			int i1 = (i+1);

			switch(token)
			{
			case JEditToken.NULL:
				switch(array[i])
				{
				case '%':
					addJEditToken(i - lastOffset,token);
					lastOffset = i;
					if(length - i <= 3 || array[i+2] == ' ')
					{
						addJEditToken(2,JEditToken.KEYWORD2);
						i += 2;
						lastOffset = i;
					}
					else
						token = JEditToken.KEYWORD2;
					break;
				case '"':
					addJEditToken(i - lastOffset,token);
					token = JEditToken.LITERAL1;
					lastOffset = i;
					break;
				case ':':
					if(i == offset)
					{
						addJEditToken(line.count,JEditToken.LABEL);
						lastOffset = length;
						break loop;
					}
					break;
				case ' ':
					if(lastOffset == offset)
					{
						addJEditToken(i - lastOffset,JEditToken.KEYWORD1);
						lastOffset = i;
					}
					break;
				}
				break;
			case JEditToken.KEYWORD2:
				if(array[i] == '%')
				{
					addJEditToken(i1 - lastOffset,token);
					token = JEditToken.NULL;
					lastOffset = i1;
				}
				break;
			case JEditToken.LITERAL1:
				if(array[i] == '"')
				{
					addJEditToken(i1 - lastOffset,token);
					token = JEditToken.NULL;
					lastOffset = i1;
				}
				break;
			default:
				throw new InternalError("Invalid state: " + token);
			}
		}

		if(lastOffset != length)
		{
			if(token != JEditToken.NULL)
				token = JEditToken.INVALID;
			else if(lastOffset == offset)
				token = JEditToken.KEYWORD1;
			addJEditToken(length - lastOffset,token);
		}
		return JEditToken.NULL;
	}

	public boolean supportsMultilineJEditTokens()
	{
		return false;
	}
}

/*
 * ChangeLog:
 * $Log: JEditBatchFileTokenMarker.java,v $
 * Revision 1.3  2001/07/25 17:39:05  desnoix
 * *** empty log message ***
 *
 * Revision 1.2  2001/06/21 18:50:04  desnoix
 * *** empty log message ***
 *
 * Revision 1.1  2001/06/13 19:48:18  desnoix
 * *** empty log message ***
 *
 * Revision 1.19  1999/07/16 23:45:49  sp
 * 1.7pre6 BugFree version
 *
 * Revision 1.18  1999/07/05 04:38:39  sp
 * Massive batch of changes... bug fixes, also new text component is in place.
 * Have fun
 *
 * Revision 1.17  1999/06/20 02:15:45  sp
 * Syntax coloring optimizations
 *
 * Revision 1.16  1999/06/05 00:22:58  sp
 * LGPL'd syntax package
 *
 * Revision 1.15  1999/06/03 08:24:13  sp
 * Fixing broken CVS
 *
 * Revision 1.16  1999/05/31 08:11:10  sp
 * Syntax coloring updates, expand abbrev bug fix
 *
 * Revision 1.15  1999/05/31 04:38:51  sp
 * Syntax optimizations, HyperSearch for Selection added (Mike Dillon)
 *
 * Revision 1.14  1999/04/19 05:38:20  sp
 * Syntax API changes
 *
 * Revision 1.13  1999/03/26 05:13:04  sp
 * Enhanced menu item updates
 *
 * Revision 1.12  1999/03/13 08:50:39  sp
 * Syntax colorizing updates and cleanups, general code reorganizations
 *
 * Revision 1.11  1999/03/13 00:09:07  sp
 * Console updates, uncomment removed cos it's too buggy, cvs log tags added
 *
 * Revision 1.10  1999/03/12 23:51:00  sp
 * Console updates, uncomment removed cos it's too buggy, cvs log tags added
 *
 */
