
package com.memoire.jedit;
import com.memoire.jedit.*;

/*
 * JEditSQLTokenMarker.java - Generic SQL token marker
 * Copyright (C) 1999 mike dillon
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import javax.swing.text.Segment;

/**
 * SQL token marker.
 *
 * @author mike dillon
 * @version $Id: JEditSQLTokenMarker.java,v 1.3 2001/07/25 17:39:05 desnoix Exp $
 */
public class JEditSQLTokenMarker extends JEditTokenMarker
{
	private int offset, lastOffset, lastKeyword, length;

	// public members
	public JEditSQLTokenMarker(JEditKeywordMap k)
	{
		this(k, false);
	}

	public JEditSQLTokenMarker(JEditKeywordMap k, boolean tsql)
	{
		keywords = k;
		isTSQL = tsql;
	}

	public byte markJEditTokensImpl(byte token, Segment line, int lineIndex)
	{
		offset = lastOffset = lastKeyword = line.offset;
		length = line.count + offset;

loop:
		for(int i = offset; i < length; i++)
		{
			switch(line.array[i])
			{
			case '*':
				if(token == JEditToken.COMMENT1 && length - i >= 1 && line.array[i+1] == '/')
				{
					token = JEditToken.NULL;
					i++;
					addJEditToken((i + 1) - lastOffset,JEditToken.COMMENT1);
					lastOffset = i + 1;
				}
				else if (token == JEditToken.NULL)
				{
					searchBack(line, i);
					addJEditToken(1,JEditToken.OPERATOR);
					lastOffset = i + 1;
				}
				break;
			case '[':
				if(token == JEditToken.NULL)
				{
					searchBack(line, i);
					token = JEditToken.LITERAL1;
					literalChar = '[';
					lastOffset = i;
				}
				break;
			case ']':
				if(token == JEditToken.LITERAL1 && literalChar == '[')
				{
					token = JEditToken.NULL;
					literalChar = 0;
					addJEditToken((i + 1) - lastOffset,JEditToken.LITERAL1);
					lastOffset = i + 1;
				}
				break;
			case '.': case ',': case '(': case ')':
				if (token == JEditToken.NULL) {
					searchBack(line, i);
					addJEditToken(1, JEditToken.NULL);
					lastOffset = i + 1;
				}
				break;
			case '+': case '%': case '&': case '|': case '^':
			case '~': case '<': case '>': case '=':
				if (token == JEditToken.NULL) {
					searchBack(line, i);
					addJEditToken(1,JEditToken.OPERATOR);
					lastOffset = i + 1;
				}
				break;
			case ' ': case '\t':
				if (token == JEditToken.NULL) {
					searchBack(line, i, false);
				}
				break;
			case ':':
				if(token == JEditToken.NULL)
				{
					addJEditToken((i+1) - lastOffset,JEditToken.LABEL);
					lastOffset = i + 1;
				}
				break;
			case '/':
				if(token == JEditToken.NULL)
				{
					if (length - i >= 2 && line.array[i + 1] == '*')
					{
						searchBack(line, i);
						token = JEditToken.COMMENT1;
						lastOffset = i;
						i++;
					}
					else
					{
						searchBack(line, i);
						addJEditToken(1,JEditToken.OPERATOR);
						lastOffset = i + 1;
					}
				}
				break;
			case '-':
				if(token == JEditToken.NULL)
				{
					if (length - i >= 2 && line.array[i+1] == '-')
					{
						searchBack(line, i);
						addJEditToken(length - i,JEditToken.COMMENT1);
						lastOffset = length;
						break loop;
					}
					else
					{
						searchBack(line, i);
						addJEditToken(1,JEditToken.OPERATOR);
						lastOffset = i + 1;
					}
				}
				break;
			case '!':
				if(isTSQL && token == JEditToken.NULL && length - i >= 2 &&
				(line.array[i+1] == '=' || line.array[i+1] == '<' || line.array[i+1] == '>'))
				{
					searchBack(line, i);
					addJEditToken(1,JEditToken.OPERATOR);
					lastOffset = i + 1;
				}
				break;
			case '"': case '\'':
				if(token == JEditToken.NULL)
				{
					token = JEditToken.LITERAL1;
					literalChar = line.array[i];
					addJEditToken(i - lastOffset,JEditToken.NULL);
					lastOffset = i;
				}
				else if(token == JEditToken.LITERAL1 && literalChar == line.array[i])
				{
					token = JEditToken.NULL;
					literalChar = 0;
					addJEditToken((i + 1) - lastOffset,JEditToken.LITERAL1);
					lastOffset = i + 1;
				}
				break;
			default:
				break;
			}
		}
		if(token == JEditToken.NULL)
			searchBack(line, length, false);
		if(lastOffset != length)
			addJEditToken(length - lastOffset,token);
		return token;
	}

	// protected members
	protected boolean isTSQL = false;

	// private members
	private JEditKeywordMap keywords;
	private char literalChar = 0;

	private void searchBack(Segment line, int pos)
	{
		searchBack(line, pos, true);
	}

	private void searchBack(Segment line, int pos, boolean padNull)
	{
		int len = pos - lastKeyword;
		byte id = keywords.lookup(line,lastKeyword,len);
		if(id != JEditToken.NULL)
		{
			if(lastKeyword != lastOffset)
				addJEditToken(lastKeyword - lastOffset,JEditToken.NULL);
			addJEditToken(len,id);
			lastOffset = pos;
		}
		lastKeyword = pos + 1;
		if (padNull && lastOffset < pos)
			addJEditToken(pos - lastOffset, JEditToken.NULL);
	}
}

/*
 * ChangeLog:
 * $Log: JEditSQLTokenMarker.java,v $
 * Revision 1.3  2001/07/25 17:39:05  desnoix
 * *** empty log message ***
 *
 * Revision 1.2  2001/06/21 18:50:04  desnoix
 * *** empty log message ***
 *
 * Revision 1.1  2001/06/13 19:48:18  desnoix
 * *** empty log message ***
 *
 * Revision 1.6  1999/04/19 05:38:20  sp
 * Syntax API changes
 *
 * Revision 1.5  1999/03/15 03:40:23  sp
 * Search and replace updates, TSQL mode/token marker updates
 *
 * Revision 1.4  1999/03/13 00:09:07  sp
 * Console updates, uncomment removed cos it's too buggy, cvs log tags added
 *
 * Revision 1.3  1999/03/12 23:51:00  sp
 * Console updates, uncomment removed cos it's too buggy, cvs log tags added
 *
 */
