
package com.memoire.jedit;
import com.memoire.jedit.*;

/*
 * JEditTSQLTokenMarker.java - Transact-SQL token marker
 * Copyright (C) 1999 mike dillon
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */


import javax.swing.text.Segment;

/**
 * Transact-SQL token marker.
 *
 * @author mike dillon
 * @version $Id: JEditTSQLTokenMarker.java,v 1.3 2001/07/25 17:39:05 desnoix Exp $
 */
public class JEditTSQLTokenMarker extends JEditSQLTokenMarker
{
	// public members
	public JEditTSQLTokenMarker()
	{
		super(getJEditKeywordMap(), true);
	}

	public static JEditKeywordMap getJEditKeywordMap()
	{
		if (tsqlKeywords == null) {
			tsqlKeywords = new JEditKeywordMap(true);
			addKeywords();
			addDataTypes();
			addSystemFunctions();
			addOperators();
	  		addSystemStoredProcedures();
			addSystemTables();
		}
		return tsqlKeywords;
	}	

	private static void addKeywords()
	{
		tsqlKeywords.add("ADD",JEditToken.KEYWORD1);
		tsqlKeywords.add("ALTER",JEditToken.KEYWORD1);
		tsqlKeywords.add("ANSI_NULLS",JEditToken.KEYWORD1);
		tsqlKeywords.add("AS",JEditToken.KEYWORD1);
		tsqlKeywords.add("ASC",JEditToken.KEYWORD1);
		tsqlKeywords.add("AUTHORIZATION",JEditToken.KEYWORD1);
		tsqlKeywords.add("BACKUP",JEditToken.KEYWORD1);
		tsqlKeywords.add("BEGIN",JEditToken.KEYWORD1);
		tsqlKeywords.add("BREAK",JEditToken.KEYWORD1);
		tsqlKeywords.add("BROWSE",JEditToken.KEYWORD1);
		tsqlKeywords.add("BULK",JEditToken.KEYWORD1);
		tsqlKeywords.add("BY",JEditToken.KEYWORD1);
		tsqlKeywords.add("CASCADE",JEditToken.KEYWORD1);
		tsqlKeywords.add("CHECK",JEditToken.KEYWORD1);
		tsqlKeywords.add("CHECKPOINT",JEditToken.KEYWORD1);
		tsqlKeywords.add("CLOSE",JEditToken.KEYWORD1);
		tsqlKeywords.add("CLUSTERED",JEditToken.KEYWORD1);
		tsqlKeywords.add("COLUMN",JEditToken.KEYWORD1);
		tsqlKeywords.add("COMMIT",JEditToken.KEYWORD1);
		tsqlKeywords.add("COMMITTED",JEditToken.KEYWORD1);
		tsqlKeywords.add("COMPUTE",JEditToken.KEYWORD1);
		tsqlKeywords.add("CONFIRM",JEditToken.KEYWORD1);
		tsqlKeywords.add("CONSTRAINT",JEditToken.KEYWORD1);
		tsqlKeywords.add("CONTAINS",JEditToken.KEYWORD1);
		tsqlKeywords.add("CONTAINSTABLE",JEditToken.KEYWORD1);
		tsqlKeywords.add("CONTINUE",JEditToken.KEYWORD1);
		tsqlKeywords.add("CONTROLROW",JEditToken.KEYWORD1);
		tsqlKeywords.add("CREATE",JEditToken.KEYWORD1);
		tsqlKeywords.add("CURRENT",JEditToken.KEYWORD1);
		tsqlKeywords.add("CURRENT_DATE",JEditToken.KEYWORD1);
		tsqlKeywords.add("CURRENT_TIME",JEditToken.KEYWORD1);
		tsqlKeywords.add("CURSOR",JEditToken.KEYWORD1);
		tsqlKeywords.add("DATABASE",JEditToken.KEYWORD1);
		tsqlKeywords.add("DBCC",JEditToken.KEYWORD1);
		tsqlKeywords.add("DEALLOCATE",JEditToken.KEYWORD1);
		tsqlKeywords.add("DECLARE",JEditToken.KEYWORD1);
		tsqlKeywords.add("DEFAULT",JEditToken.KEYWORD1);
		tsqlKeywords.add("DELETE",JEditToken.KEYWORD1);
		tsqlKeywords.add("DENY",JEditToken.KEYWORD1);
		tsqlKeywords.add("DESC",JEditToken.KEYWORD1);
		tsqlKeywords.add("DISK",JEditToken.KEYWORD1);
		tsqlKeywords.add("DISTINCT",JEditToken.KEYWORD1);
		tsqlKeywords.add("DISTRIBUTED",JEditToken.KEYWORD1);
		tsqlKeywords.add("DOUBLE",JEditToken.KEYWORD1);
		tsqlKeywords.add("DROP",JEditToken.KEYWORD1);
		tsqlKeywords.add("DUMMY",JEditToken.KEYWORD1);
		tsqlKeywords.add("DUMP",JEditToken.KEYWORD1);
		tsqlKeywords.add("ELSE",JEditToken.KEYWORD1);
		tsqlKeywords.add("END",JEditToken.KEYWORD1);
		tsqlKeywords.add("ERRLVL",JEditToken.KEYWORD1);
		tsqlKeywords.add("ERROREXIT",JEditToken.KEYWORD1);
		tsqlKeywords.add("ESCAPE",JEditToken.KEYWORD1);
		tsqlKeywords.add("EXCEPT",JEditToken.KEYWORD1);
		tsqlKeywords.add("EXEC",JEditToken.KEYWORD1);
		tsqlKeywords.add("EXECUTE",JEditToken.KEYWORD1);
		tsqlKeywords.add("EXIT",JEditToken.KEYWORD1);
		tsqlKeywords.add("FETCH",JEditToken.KEYWORD1);
		tsqlKeywords.add("FILE",JEditToken.KEYWORD1);
		tsqlKeywords.add("FILLFACTOR",JEditToken.KEYWORD1);
		tsqlKeywords.add("FLOPPY",JEditToken.KEYWORD1);
		tsqlKeywords.add("FOR",JEditToken.KEYWORD1);
		tsqlKeywords.add("FOREIGN",JEditToken.KEYWORD1);
		tsqlKeywords.add("FREETEXT",JEditToken.KEYWORD1);
		tsqlKeywords.add("FREETEXTTABLE",JEditToken.KEYWORD1);
		tsqlKeywords.add("FROM",JEditToken.KEYWORD1);
		tsqlKeywords.add("FULL",JEditToken.KEYWORD1);
		tsqlKeywords.add("GOTO",JEditToken.KEYWORD1);
		tsqlKeywords.add("GRANT",JEditToken.KEYWORD1);
		tsqlKeywords.add("GROUP",JEditToken.KEYWORD1);
		tsqlKeywords.add("HAVING",JEditToken.KEYWORD1);
		tsqlKeywords.add("HOLDLOCK",JEditToken.KEYWORD1);
		tsqlKeywords.add("IDENTITY_INSERT",JEditToken.KEYWORD1);
		tsqlKeywords.add("IDENTITYCOL",JEditToken.KEYWORD1);
		tsqlKeywords.add("ID",JEditToken.KEYWORD1);
		tsqlKeywords.add("IF",JEditToken.KEYWORD1);
		tsqlKeywords.add("INDEX",JEditToken.KEYWORD1);
		tsqlKeywords.add("INNER",JEditToken.KEYWORD1);
		tsqlKeywords.add("INSERT",JEditToken.KEYWORD1);
		tsqlKeywords.add("INTO",JEditToken.KEYWORD1);
		tsqlKeywords.add("IS",JEditToken.KEYWORD1);
		tsqlKeywords.add("ISOLATION",JEditToken.KEYWORD1);
		tsqlKeywords.add("KEY",JEditToken.KEYWORD1);
		tsqlKeywords.add("KILL",JEditToken.KEYWORD1);
		tsqlKeywords.add("LEVEL",JEditToken.KEYWORD1);
		tsqlKeywords.add("LINENO",JEditToken.KEYWORD1);
		tsqlKeywords.add("LOAD",JEditToken.KEYWORD1);
		tsqlKeywords.add("MAX",JEditToken.KEYWORD1);
		tsqlKeywords.add("MIN",JEditToken.KEYWORD1);
		tsqlKeywords.add("MIRROREXIT",JEditToken.KEYWORD1);
		tsqlKeywords.add("NATIONAL",JEditToken.KEYWORD1);
		tsqlKeywords.add("NOCHECK",JEditToken.KEYWORD1);
		tsqlKeywords.add("NONCLUSTERED",JEditToken.KEYWORD1);
		tsqlKeywords.add("OF",JEditToken.KEYWORD1);
		tsqlKeywords.add("OFF",JEditToken.KEYWORD1);
		tsqlKeywords.add("OFFSETS",JEditToken.KEYWORD1);
		tsqlKeywords.add("ON",JEditToken.KEYWORD1);
		tsqlKeywords.add("ONCE",JEditToken.KEYWORD1);
		tsqlKeywords.add("ONLY",JEditToken.KEYWORD1);
		tsqlKeywords.add("OPEN",JEditToken.KEYWORD1);
		tsqlKeywords.add("OPENDATASOURCE",JEditToken.KEYWORD1);
		tsqlKeywords.add("OPENQUERY",JEditToken.KEYWORD1);
		tsqlKeywords.add("OPENROWSET",JEditToken.KEYWORD1);
		tsqlKeywords.add("OPTION",JEditToken.KEYWORD1);
		tsqlKeywords.add("ORDER",JEditToken.KEYWORD1);
		tsqlKeywords.add("OVER",JEditToken.KEYWORD1);
		tsqlKeywords.add("PERCENT",JEditToken.KEYWORD1);
		tsqlKeywords.add("PERM",JEditToken.KEYWORD1);
		tsqlKeywords.add("PERMANENT",JEditToken.KEYWORD1);
		tsqlKeywords.add("PIPE",JEditToken.KEYWORD1);
		tsqlKeywords.add("PLAN",JEditToken.KEYWORD1);
		tsqlKeywords.add("PRECISION",JEditToken.KEYWORD1);
		tsqlKeywords.add("PREPARE",JEditToken.KEYWORD1);
		tsqlKeywords.add("PRIMARY",JEditToken.KEYWORD1);
		tsqlKeywords.add("PRINT",JEditToken.KEYWORD1);
		tsqlKeywords.add("PRIVILEGES",JEditToken.KEYWORD1);
		tsqlKeywords.add("PROC",JEditToken.KEYWORD1);
		tsqlKeywords.add("PROCEDURE",JEditToken.KEYWORD1);
		tsqlKeywords.add("PROCESSEXIT",JEditToken.KEYWORD1);
		tsqlKeywords.add("PUBLIC",JEditToken.KEYWORD1);
		tsqlKeywords.add("QUOTED_IDENTIFIER",JEditToken.KEYWORD1);
		tsqlKeywords.add("RAISERROR",JEditToken.KEYWORD1);
		tsqlKeywords.add("READ",JEditToken.KEYWORD1);
		tsqlKeywords.add("READTEXT",JEditToken.KEYWORD1);
		tsqlKeywords.add("RECONFIGURE",JEditToken.KEYWORD1);
		tsqlKeywords.add("REFERENCES",JEditToken.KEYWORD1);
		tsqlKeywords.add("REPEATABLE",JEditToken.KEYWORD1);
		tsqlKeywords.add("REPLICATION",JEditToken.KEYWORD1);
		tsqlKeywords.add("RESTORE",JEditToken.KEYWORD1);
		tsqlKeywords.add("RESTRICT",JEditToken.KEYWORD1);
		tsqlKeywords.add("RETURN",JEditToken.KEYWORD1);
		tsqlKeywords.add("REVOKE",JEditToken.KEYWORD1);
		tsqlKeywords.add("ROLLBACK",JEditToken.KEYWORD1);
		tsqlKeywords.add("ROWGUIDCOL",JEditToken.KEYWORD1);
		tsqlKeywords.add("RULE",JEditToken.KEYWORD1);
		tsqlKeywords.add("SAVE",JEditToken.KEYWORD1);
		tsqlKeywords.add("SCHEMA",JEditToken.KEYWORD1);
		tsqlKeywords.add("SELECT",JEditToken.KEYWORD1);
		tsqlKeywords.add("SERIALIZABLE",JEditToken.KEYWORD1);
		tsqlKeywords.add("SET",JEditToken.KEYWORD1);
		tsqlKeywords.add("SETUSER",JEditToken.KEYWORD1);
		tsqlKeywords.add("SHUTDOWN",JEditToken.KEYWORD1);
		tsqlKeywords.add("STATISTICS",JEditToken.KEYWORD1);
		tsqlKeywords.add("TABLE",JEditToken.KEYWORD1);
		tsqlKeywords.add("TAPE",JEditToken.KEYWORD1);
		tsqlKeywords.add("TEMP",JEditToken.KEYWORD1);
		tsqlKeywords.add("TEMPORARY",JEditToken.KEYWORD1);
		tsqlKeywords.add("TEXTIMAGE_ON",JEditToken.KEYWORD1);
		tsqlKeywords.add("THEN",JEditToken.KEYWORD1);
		tsqlKeywords.add("TO",JEditToken.KEYWORD1);
		tsqlKeywords.add("TOP",JEditToken.KEYWORD1);
		tsqlKeywords.add("TRAN",JEditToken.KEYWORD1);
		tsqlKeywords.add("TRANSACTION",JEditToken.KEYWORD1);
		tsqlKeywords.add("TRIGGER",JEditToken.KEYWORD1);
		tsqlKeywords.add("TRUNCATE",JEditToken.KEYWORD1);
		tsqlKeywords.add("TSEQUAL",JEditToken.KEYWORD1);
		tsqlKeywords.add("UNCOMMITTED",JEditToken.KEYWORD1);
		tsqlKeywords.add("UNION",JEditToken.KEYWORD1);
		tsqlKeywords.add("UNIQUE",JEditToken.KEYWORD1);
		tsqlKeywords.add("UPDATE",JEditToken.KEYWORD1);
		tsqlKeywords.add("UPDATETEXT",JEditToken.KEYWORD1);
		tsqlKeywords.add("USE",JEditToken.KEYWORD1);
		tsqlKeywords.add("VALUES",JEditToken.KEYWORD1);
		tsqlKeywords.add("VARYING",JEditToken.KEYWORD1);
		tsqlKeywords.add("VIEW",JEditToken.KEYWORD1);
		tsqlKeywords.add("WAITFOR",JEditToken.KEYWORD1);
		tsqlKeywords.add("WHEN",JEditToken.KEYWORD1);
		tsqlKeywords.add("WHERE",JEditToken.KEYWORD1);
		tsqlKeywords.add("WHILE",JEditToken.KEYWORD1);
		tsqlKeywords.add("WITH",JEditToken.KEYWORD1);
		tsqlKeywords.add("WORK",JEditToken.KEYWORD1);
		tsqlKeywords.add("WRITETEXT",JEditToken.KEYWORD1);
	}

	private static void addDataTypes()
	{
		tsqlKeywords.add("binary",JEditToken.KEYWORD1);
		tsqlKeywords.add("bit",JEditToken.KEYWORD1);
		tsqlKeywords.add("char",JEditToken.KEYWORD1);
		tsqlKeywords.add("character",JEditToken.KEYWORD1);
		tsqlKeywords.add("datetime",JEditToken.KEYWORD1);
		tsqlKeywords.add("decimal",JEditToken.KEYWORD1);
		tsqlKeywords.add("float",JEditToken.KEYWORD1);
		tsqlKeywords.add("image",JEditToken.KEYWORD1);
		tsqlKeywords.add("int",JEditToken.KEYWORD1);
		tsqlKeywords.add("integer",JEditToken.KEYWORD1);
		tsqlKeywords.add("money",JEditToken.KEYWORD1);
		tsqlKeywords.add("name",JEditToken.KEYWORD1);
		tsqlKeywords.add("numeric",JEditToken.KEYWORD1);
		tsqlKeywords.add("nchar",JEditToken.KEYWORD1);
		tsqlKeywords.add("nvarchar",JEditToken.KEYWORD1);
		tsqlKeywords.add("ntext",JEditToken.KEYWORD1);
		tsqlKeywords.add("real",JEditToken.KEYWORD1);
		tsqlKeywords.add("smalldatetime",JEditToken.KEYWORD1);
		tsqlKeywords.add("smallint",JEditToken.KEYWORD1);
		tsqlKeywords.add("smallmoney",JEditToken.KEYWORD1);
		tsqlKeywords.add("text",JEditToken.KEYWORD1);
		tsqlKeywords.add("timestamp",JEditToken.KEYWORD1);
		tsqlKeywords.add("tinyint",JEditToken.KEYWORD1);
		tsqlKeywords.add("uniqueidentifier",JEditToken.KEYWORD1);
		tsqlKeywords.add("varbinary",JEditToken.KEYWORD1);
		tsqlKeywords.add("varchar",JEditToken.KEYWORD1);
	}

	private static void addSystemFunctions()
	{
		tsqlKeywords.add("@@CONNECTIONS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@CPU_BUSY",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@CURSOR_ROWS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@DATEFIRST",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@DBTS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@ERROR",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@FETCH_STATUS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@IDENTITY",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@IDLE",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@IO_BUSY",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@LANGID",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@LANGUAGE",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@LOCK_TIMEOUT",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@MAX_CONNECTIONS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@MAX_PRECISION",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@NESTLEVEL",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@OPTIONS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@PACK_RECEIVED",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@PACK_SENT",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@PACKET_ERRORS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@PROCID",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@REMSERVER",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@ROWCOUNT",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@SERVERNAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@SERVICENAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@SPID",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@TEXTSIZE",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@TIMETICKS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@TOTAL_ERRORS",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@TOTAL_READ",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@TOTAL_WRITE",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@TRANCOUNT",JEditToken.KEYWORD2);
		tsqlKeywords.add("@@VERSION",JEditToken.KEYWORD2);
		tsqlKeywords.add("ABS",JEditToken.KEYWORD2);
		tsqlKeywords.add("ACOS",JEditToken.KEYWORD2);
		tsqlKeywords.add("APP_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("ASCII",JEditToken.KEYWORD2);
		tsqlKeywords.add("ASIN",JEditToken.KEYWORD2);
		tsqlKeywords.add("ATAN",JEditToken.KEYWORD2);
		tsqlKeywords.add("ATN2",JEditToken.KEYWORD2);
		tsqlKeywords.add("CASE",JEditToken.KEYWORD2);
		tsqlKeywords.add("CAST",JEditToken.KEYWORD2);
		tsqlKeywords.add("CEILING",JEditToken.KEYWORD2);
//				tsqlKeywords.add("CHAR",JEditToken.KEYWORD2);
		tsqlKeywords.add("CHARINDEX",JEditToken.KEYWORD2);
		tsqlKeywords.add("COALESCE",JEditToken.KEYWORD2);
		tsqlKeywords.add("COL_LENGTH",JEditToken.KEYWORD2);
		tsqlKeywords.add("COL_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("COLUMNPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("CONVERT",JEditToken.KEYWORD2);
		tsqlKeywords.add("COS",JEditToken.KEYWORD2);
		tsqlKeywords.add("COT",JEditToken.KEYWORD2);
		tsqlKeywords.add("CURRENT_TIME",JEditToken.KEYWORD2);
		tsqlKeywords.add("CURRENT_DATE",JEditToken.KEYWORD2);
		tsqlKeywords.add("CURRENT_TIMESTAMP",JEditToken.KEYWORD2);
		tsqlKeywords.add("CURRENT_USER",JEditToken.KEYWORD2);
		tsqlKeywords.add("CURSOR_STATUS",JEditToken.KEYWORD2);
		tsqlKeywords.add("DATABASEPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("DATALENGTH",JEditToken.KEYWORD2);
		tsqlKeywords.add("DATEADD",JEditToken.KEYWORD2);
		tsqlKeywords.add("DATEDIFF",JEditToken.KEYWORD2);
		tsqlKeywords.add("DATENAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("DATEPART",JEditToken.KEYWORD2);
		tsqlKeywords.add("DAY",JEditToken.KEYWORD2);
		tsqlKeywords.add("DB_ID",JEditToken.KEYWORD2);
		tsqlKeywords.add("DB_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("DEGREES",JEditToken.KEYWORD2);
		tsqlKeywords.add("DIFFERENCE",JEditToken.KEYWORD2);
		tsqlKeywords.add("EXP",JEditToken.KEYWORD2);
		tsqlKeywords.add("FILE_ID",JEditToken.KEYWORD2);
		tsqlKeywords.add("FILE_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("FILEGROUP_ID",JEditToken.KEYWORD2);
		tsqlKeywords.add("FILEGROUP_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("FILEGROUPPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("FILEPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("FLOOR",JEditToken.KEYWORD2);
		tsqlKeywords.add("FORMATMESSAGE",JEditToken.KEYWORD2);
		tsqlKeywords.add("FULLTEXTCATALOGPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("FULLTEXTSERVICEPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("GETANSINULL",JEditToken.KEYWORD2);
		tsqlKeywords.add("GETDATE",JEditToken.KEYWORD2);
		tsqlKeywords.add("HOST_ID",JEditToken.KEYWORD2);
		tsqlKeywords.add("HOST_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("IDENT_INCR",JEditToken.KEYWORD2);
		tsqlKeywords.add("IDENT_SEED",JEditToken.KEYWORD2);
//				tsqlKeywords.add("IDENTITY",JEditToken.KEYWORD2);
		tsqlKeywords.add("IDENTITY_INSERT",JEditToken.KEYWORD2);
		tsqlKeywords.add("INDEX_COL",JEditToken.KEYWORD2);
		tsqlKeywords.add("INDEXPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("IS_MEMBER",JEditToken.KEYWORD2);
		tsqlKeywords.add("IS_SRVROLEMEMBER",JEditToken.KEYWORD2);
		tsqlKeywords.add("ISDATE",JEditToken.KEYWORD2);
		tsqlKeywords.add("ISNULL",JEditToken.KEYWORD2);
		tsqlKeywords.add("ISNUMERIC",JEditToken.KEYWORD2);
		tsqlKeywords.add("LEFT",JEditToken.KEYWORD2);
		tsqlKeywords.add("LEN",JEditToken.KEYWORD2);
		tsqlKeywords.add("LOG",JEditToken.KEYWORD2);
		tsqlKeywords.add("LOG10",JEditToken.KEYWORD2);
		tsqlKeywords.add("LOWER",JEditToken.KEYWORD2);
		tsqlKeywords.add("LTRIM",JEditToken.KEYWORD2);
		tsqlKeywords.add("MONTH",JEditToken.KEYWORD2);
//				tsqlKeywords.add("NCHAR",JEditToken.KEYWORD2);
		tsqlKeywords.add("NEWID",JEditToken.KEYWORD2);
		tsqlKeywords.add("NULLIF",JEditToken.KEYWORD2);
		tsqlKeywords.add("OBJECT_ID",JEditToken.KEYWORD2);
		tsqlKeywords.add("OBJECT_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("OBJECTPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("PARSENAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("PATINDEX",JEditToken.KEYWORD2);
		tsqlKeywords.add("PERMISSIONS",JEditToken.KEYWORD2);
		tsqlKeywords.add("PI",JEditToken.KEYWORD2);
		tsqlKeywords.add("POWER",JEditToken.KEYWORD2);
		tsqlKeywords.add("QUOTENAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("RADIANS",JEditToken.KEYWORD2);
		tsqlKeywords.add("RAND",JEditToken.KEYWORD2);
		tsqlKeywords.add("REPLACE",JEditToken.KEYWORD2);
		tsqlKeywords.add("REPLICATE",JEditToken.KEYWORD2);
		tsqlKeywords.add("REVERSE",JEditToken.KEYWORD2);
		tsqlKeywords.add("RIGHT",JEditToken.KEYWORD2);
		tsqlKeywords.add("ROUND",JEditToken.KEYWORD2);
		tsqlKeywords.add("RTRIM",JEditToken.KEYWORD2);
		tsqlKeywords.add("SESSION_USER",JEditToken.KEYWORD2);
		tsqlKeywords.add("SIGN",JEditToken.KEYWORD2);
		tsqlKeywords.add("SIN",JEditToken.KEYWORD2);
		tsqlKeywords.add("SOUNDEX",JEditToken.KEYWORD2);
		tsqlKeywords.add("SPACE",JEditToken.KEYWORD2);
		tsqlKeywords.add("SQRT",JEditToken.KEYWORD2);
		tsqlKeywords.add("SQUARE",JEditToken.KEYWORD2);
		tsqlKeywords.add("STATS_DATE",JEditToken.KEYWORD2);
		tsqlKeywords.add("STR",JEditToken.KEYWORD2);
		tsqlKeywords.add("STUFF",JEditToken.KEYWORD2);
		tsqlKeywords.add("SUBSTRING",JEditToken.KEYWORD2);
		tsqlKeywords.add("SUSER_ID",JEditToken.KEYWORD2);
		tsqlKeywords.add("SUSER_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("SUSER_SID",JEditToken.KEYWORD2);
		tsqlKeywords.add("SUSER_SNAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("SYSTEM_USER",JEditToken.KEYWORD2);
		tsqlKeywords.add("TAN",JEditToken.KEYWORD2);
		tsqlKeywords.add("TEXTPTR",JEditToken.KEYWORD2);
		tsqlKeywords.add("TEXTVALID",JEditToken.KEYWORD2);
		tsqlKeywords.add("TYPEPROPERTY",JEditToken.KEYWORD2);
		tsqlKeywords.add("UNICODE",JEditToken.KEYWORD2);
		tsqlKeywords.add("UPPER",JEditToken.KEYWORD2);
		tsqlKeywords.add("USER_ID",JEditToken.KEYWORD2);
		tsqlKeywords.add("USER_NAME",JEditToken.KEYWORD2);
		tsqlKeywords.add("USER",JEditToken.KEYWORD2);
		tsqlKeywords.add("YEAR",JEditToken.KEYWORD2);
	}

	private static void addOperators()
	{
		tsqlKeywords.add("ALL",JEditToken.KEYWORD1);
		tsqlKeywords.add("AND",JEditToken.KEYWORD1);
		tsqlKeywords.add("ANY",JEditToken.KEYWORD1);
		tsqlKeywords.add("BETWEEN",JEditToken.KEYWORD1);
		tsqlKeywords.add("CROSS",JEditToken.KEYWORD1);
		tsqlKeywords.add("EXISTS",JEditToken.KEYWORD1);
		tsqlKeywords.add("IN",JEditToken.KEYWORD1);
		tsqlKeywords.add("INTERSECT",JEditToken.KEYWORD1);
		tsqlKeywords.add("JOIN",JEditToken.KEYWORD1);
		tsqlKeywords.add("LIKE",JEditToken.KEYWORD1);
		tsqlKeywords.add("NOT",JEditToken.KEYWORD1);
		tsqlKeywords.add("NULL",JEditToken.KEYWORD1);
		tsqlKeywords.add("OR",JEditToken.KEYWORD1);
		tsqlKeywords.add("OUTER",JEditToken.KEYWORD1);
		tsqlKeywords.add("SOME",JEditToken.KEYWORD1);
	}

	private static void addSystemStoredProcedures()
	{
		tsqlKeywords.add("sp_add_agent_parameter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_agent_profile",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_alert",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_category",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_data_file_recover_suspect_db",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_job",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_jobschedule",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_jobserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_jobstep",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_log_file_recover_suspect_db",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_notification",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_operator",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_targetservergroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_add_targetsvrgrp_member",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addalias",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addapprole",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addarticle",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_adddistpublisher",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_adddistributiondb",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_adddistributor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addextendedproc",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addgroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addlinkedserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addlinkedsrvlogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addlinkedsrvlogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addlogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addmergearticle",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addmergefilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addmergepublication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addmergepullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addmergepullsubscription_agent",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addmergesubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addmessage",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addpublication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addpublication_snapshot",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addpublisher70",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addpullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addpullsubscription_agent",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addremotelogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addrole",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addrolemember",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addsrvrolemember",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addsubscriber",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addsubscriber_schedule",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addsynctriggers",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addtabletocontents",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addtask",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addtype",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_addumpdevice",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_adduser",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_altermessage",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_apply_job_to_targets",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_approlepassword",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_article_validation",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_articlecolumn",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_articlefilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_articlesynctranprocs",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_articleview",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_attach_db",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_attach_single_file_db",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_autostats",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_bindefault",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_bindrule",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_bindsession",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_browsereplcmds",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_catalogs",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_certify_removable",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_change_agent_parameter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_change_agent_profile",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_change_subscription_properties",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_change_users_login",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changearticle",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changedbowner",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changedistpublisher",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changedistributiondb",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changedistributor_password",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changedistributor_property",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changegroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changemergearticle",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changemergefilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changemergepublication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changemergepullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changemergesubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changeobjectowner",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changepublication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changesubscriber",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changesubscriber_schedule",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_changesubstatus",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_check_for_sync_trigger",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_column_privileges",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_column_privileges_ex",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_columns",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_columns_ex",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_configure",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_create_removable",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_createorphan",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_createstats",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursor_list",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursorclose",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursorexecute",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursorfetch",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursoropen",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursoroption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursorprepare",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cursorunprepare",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_cycle_errorlog",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_databases",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_datatype_info",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dbcmptlevel",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dbfixedrolepermission",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dboption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_defaultdb",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_defaultlanguage",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_alert",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_backuphistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_category",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_job",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_jobschedule",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_jobserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_jobstep",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_notification",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_operator",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_targetserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_targetservergroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_delete_targetsvrgrp_member",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_deletemergeconflictrow",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_denylogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_depends",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_describe_cursor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_describe_cursor_columns",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_describe_cursor_tables",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_detach_db",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_drop_agent_parameter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_drop_agent_profile",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropalias",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropapprole",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droparticle",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropdevice",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropdistpublisher",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropdistributiondb",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropdistributor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropextendedproc",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropgroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droplinkedsrvlogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droplinkedsrvlogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droplogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropmergearticle",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropmergefilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropmergepublication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropmergepullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropmergesubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropmessage",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droporphans",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droppublication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droppullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropremotelogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droprole",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droprolemember",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropsrvrolemember",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropsubscriber",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droptask",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_droptype",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropuser",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dropwebtask",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dsninfo",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_dumpparamcmd",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_enumcodepages",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_enumcustomresolvers",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_enumdsn",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_enumfullsubscribers",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_execute",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_executesql",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_expired_subscription_cleanup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_fkeys",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_foreignkeys",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_fulltext_catalog",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_fulltext_column",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_fulltext_database",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_fulltext_service",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_fulltext_table",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_generatefilters",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_get_distributor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_getbindtoken",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_getmergedeletetype",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_grant_publication_access",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_grantdbaccess",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_grantlogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_agent_default",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_agent_parameter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_agent_profile",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_alert",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_category",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_downloadlist",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_fulltext_catalogs",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_fulltext_catalogs_cursor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_fulltext_columns",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_fulltext_columns_cursor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_fulltext_tables",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_fulltext_tables_cursor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_job",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_jobhistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_jobschedule",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_jobserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_jobstep",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_notification",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_operator",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_publication_access",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_targetserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_help_targetservergroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helparticle",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helparticlecolumns",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpconstraint",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpdb",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpdbfixedrole",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpdevice",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpdistpublisher",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpdistributiondb",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpdistributor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpextendedproc",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpfile",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpfilegroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpgroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helphistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpindex",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helplanguage",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helplinkedsrvlogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helplogins",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpmergearticle",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpmergearticleconflicts",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpmergeconflictrows",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpmergedeleteconflictrows",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpmergefilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpmergepublication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpmergepullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpmergesubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpntgroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helppublication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helppullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpremotelogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpreplicationdboption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helprole",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helprolemember",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helprotect",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpsort",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpsrvrole",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpsrvrolemember",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpsubscriberinfo",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpsubscription_properties",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helptask",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helptext",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helptrigger",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_helpuser",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_indexes",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_indexoption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_link_publication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_linkedservers",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_lock",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_makewebtask",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_manage_jobs_by_login",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_mergedummyupdate",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_mergesubscription_cleanup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_monitor",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_msx_defect",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_msx_enlist",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_OACreate",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_OADestroy",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_OAGetErrorInfo",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_OAGetProperty",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_OAMethod",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_OASetProperty",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_OAStop",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_password",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_pkeys",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_post_msx_operation",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_prepare",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_primarykeys",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_processmail",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_procoption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_publication_validation",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_purge_jobhistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_purgehistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_reassigntask",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_recompile",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_refreshsubscriptions",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_refreshview",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_reinitmergepullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_reinitmergesubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_reinitpullsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_reinitsubscription",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_remoteoption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_remove_job_from_targets",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_removedbreplication",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_rename",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_renamedb",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_replcmds",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_replcounters",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_repldone",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_replflush",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_replication_agent_checkup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_replicationdboption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_replsetoriginator",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_replshowcmds",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_repltrans",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_reset_connection",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_resync_targetserver",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_revoke_publication_access",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_revokedbaccess",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_revokelogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_runwebtask",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_script_synctran_commands",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_scriptdelproc",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_scriptinsproc",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_scriptmappedupdproc",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_scriptupdproc",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_sdidebug",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_server_info",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_serveroption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_serveroption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_setapprole",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_setnetname",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_spaceused",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_special_columns",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_sproc_columns",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_srvrolepermission",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_start_job",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_statistics",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_stop_job",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_stored_procedures",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_subscription_cleanup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_table_privileges",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_table_privileges_ex",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_table_validation",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_tableoption",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_tables",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_tables_ex",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_unbindefault",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_unbindrule",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_unprepare",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_agent_profile",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_alert",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_category",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_job",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_jobschedule",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_jobstep",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_notification",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_operator",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_update_targetservergroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_updatestats",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_updatetask",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_validatelogins",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_validname",JEditToken.KEYWORD3);
		tsqlKeywords.add("sp_who",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_cmdshell",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_deletemail",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_enumgroups",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_findnextmsg",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_findnextmsg",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_grantlogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_logevent",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_loginconfig",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_logininfo",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_msver",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_readmail",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_revokelogin",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_sendmail",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_sprintf",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_sqlinventory",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_sqlmaint",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_sqltrace",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_sscanf",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_startmail",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_stopmail",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_addnewqueue",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_deletequeuedefinition",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_destroyqueue",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_enumqueuedefname",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_enumqueuehandles",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_eventclassrequired",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_flushqueryhistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_generate_event",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getappfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getconnectionidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getcpufilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getdbidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getdurationfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_geteventfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_geteventnames",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getevents",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_gethostfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_gethpidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getindidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getntdmfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getntnmfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getobjidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getqueueautostart",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getqueuedestination",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getqueueproperties",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getreadfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getserverfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getseverityfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getspidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getsysobjectsfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_gettextfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getuserfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_getwritefilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_loadqueuedefinition",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_pausequeue",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_restartqueue",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_savequeuedefinition",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setappfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setconnectionidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setcpufilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setdbidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setdurationfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_seteventclassrequired",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_seteventfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_sethostfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_sethpidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setindidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setntdmfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setntnmfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setobjidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setqueryhistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setqueueautostart",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setqueuecreateinfo",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setqueuedestination",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setreadfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setserverfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setseverityfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setspidfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setsysobjectsfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_settextfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setuserfilter",JEditToken.KEYWORD3);
		tsqlKeywords.add("xp_trace_setwritefilter",JEditToken.KEYWORD3);
	}

	private static void addSystemTables()
	{
		tsqlKeywords.add("backupfile",JEditToken.KEYWORD3);
		tsqlKeywords.add("backupmediafamily",JEditToken.KEYWORD3);
		tsqlKeywords.add("backupmediaset",JEditToken.KEYWORD3);
		tsqlKeywords.add("backupset",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSagent_parameters",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSagent_profiles",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSarticles",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSdistpublishers",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSdistribution_agents",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSdistribution_history",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSdistributiondbs",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSdistributor",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSlogreader_agents",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSlogreader_history",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSmerge_agents",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSmerge_contents",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSmerge_delete_conflicts",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSmerge_genhistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSmerge_history",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSmerge_replinfo",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSmerge_subscriptions",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSmerge_tombstone",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSpublication_access",JEditToken.KEYWORD3);
		tsqlKeywords.add("Mspublications",JEditToken.KEYWORD3);
		tsqlKeywords.add("Mspublisher_databases",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSrepl_commands",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSrepl_errors",JEditToken.KEYWORD3);
		tsqlKeywords.add("Msrepl_originators",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSrepl_transactions",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSrepl_version",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSreplication_objects",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSreplication_subscriptions",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSsnapshot_agents",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSsnapshot_history",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSsubscriber_info",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSsubscriber_schedule",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSsubscription_properties",JEditToken.KEYWORD3);
		tsqlKeywords.add("MSsubscriptions",JEditToken.KEYWORD3);
		tsqlKeywords.add("restorefile",JEditToken.KEYWORD3);
		tsqlKeywords.add("restorefilegroup",JEditToken.KEYWORD3);
		tsqlKeywords.add("restorehistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysalerts",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysallocations",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysaltfiles",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysarticles",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysarticleupdates",JEditToken.KEYWORD3);
		tsqlKeywords.add("syscacheobjects",JEditToken.KEYWORD3);
		tsqlKeywords.add("syscategories",JEditToken.KEYWORD3);
		tsqlKeywords.add("syscharsets",JEditToken.KEYWORD3);
		tsqlKeywords.add("syscolumns",JEditToken.KEYWORD3);
		tsqlKeywords.add("syscomments",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysconfigures",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysconstraints",JEditToken.KEYWORD3);
		tsqlKeywords.add("syscurconfigs",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysdatabases",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysdatabases",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysdepends",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysdevices",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysdownloadlist",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysfilegroups",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysfiles",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysforeignkeys",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysfulltextcatalogs",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysindexes",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysindexkeys",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysjobhistory",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysjobs",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysjobschedules",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysjobservers",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysjobsteps",JEditToken.KEYWORD3);
		tsqlKeywords.add("syslanguages",JEditToken.KEYWORD3);
		tsqlKeywords.add("syslockinfo",JEditToken.KEYWORD3);
		tsqlKeywords.add("syslogins",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysmembers",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysmergearticles",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysmergepublications",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysmergeschemachange",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysmergesubscriptions",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysmergesubsetfilters",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysmessages",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysnotifications",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysobjects",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysobjects",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysoledbusers",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysoperators",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysperfinfo",JEditToken.KEYWORD3);
		tsqlKeywords.add("syspermissions",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysprocesses",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysprotects",JEditToken.KEYWORD3);
		tsqlKeywords.add("syspublications",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysreferences",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysremotelogins",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysreplicationalerts",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysservers",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysservers",JEditToken.KEYWORD3);
		tsqlKeywords.add("syssubscriptions",JEditToken.KEYWORD3);
		tsqlKeywords.add("systargetservergroupmembers",JEditToken.KEYWORD3);
		tsqlKeywords.add("systargetservergroups",JEditToken.KEYWORD3);
		tsqlKeywords.add("systargetservers",JEditToken.KEYWORD3);
		tsqlKeywords.add("systaskids",JEditToken.KEYWORD3);
		tsqlKeywords.add("systypes",JEditToken.KEYWORD3);
		tsqlKeywords.add("sysusers",JEditToken.KEYWORD3);
	}

	private static JEditKeywordMap tsqlKeywords;
}

/*
 * ChangeLog:
 * $Log: JEditTSQLTokenMarker.java,v $
 * Revision 1.3  2001/07/25 17:39:05  desnoix
 * *** empty log message ***
 *
 * Revision 1.2  2001/06/21 18:50:04  desnoix
 * *** empty log message ***
 *
 * Revision 1.1  2001/06/13 19:48:18  desnoix
 * *** empty log message ***
 *
 * Revision 1.8  1999/06/05 02:13:22  sp
 * LGPL'd remaining syntax files
 *
 * Revision 1.7  1999/03/19 07:12:11  sp
 * JOptionPane changes, did a fromdos of the source
 *
 * Revision 1.6  1999/03/15 03:40:23  sp
 * Search and replace updates, TSQL mode/token marker updates
 *
 */
