
package com.memoire.jedit;
import com.memoire.jedit.*;

/*
 * JEditTeXTokenMarker.java - TeX/LaTeX/AMS-TeX token marker
 * Copyright (C) 1998 Slava Pestov
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */


import javax.swing.text.Segment;

/**
 * TeX token marker.
 *
 * @author Slava Pestov
 * @version $Id: JEditTeXTokenMarker.java,v 1.3 2001/07/25 17:39:05 desnoix Exp $
 */
public class JEditTeXTokenMarker extends JEditTokenMarker
{
	// public members
	public static final byte BDFORMULA = JEditToken.INTERNAL_FIRST;
	public static final byte EDFORMULA = (byte)(JEditToken.INTERNAL_FIRST + 1);
	
	public byte markJEditTokensImpl(byte token, Segment line, int lineIndex)
	{
		char[] array = line.array;
		int offset = line.offset;
		int lastOffset = offset;
		int length = line.count + offset;
		boolean backslash = false;
loop:		for(int i = offset; i < length; i++)
		{
			int i1 = (i+1);

			char c = array[i];
			// if a backslash is followed immediately
			// by a non-alpha character, the command at
			// the non-alpha char. If we have a backslash,
			// some text, and then a non-alpha char,
			// the command ends before the non-alpha char.
			if(Character.isLetter(c))
			{
				backslash = false;
			}
			else
			{
				if(backslash)
				{
					// \<non alpha>
					// we skip over this character,
					// hence the `continue'
					backslash = false;
					if(token == JEditToken.KEYWORD2 || token == EDFORMULA)
						token = JEditToken.KEYWORD2;
					addJEditToken(i1 - lastOffset,token);
					lastOffset = i1;
					if(token == JEditToken.KEYWORD1)
						token = JEditToken.NULL;
					continue;
				}
				else
				{
					//\blah<non alpha>
					// we leave the character in
					// the stream, and it's not
					// part of the command token
					if(token == BDFORMULA || token == EDFORMULA)
						token = JEditToken.KEYWORD2;
					addJEditToken(i - lastOffset,token);
					if(token == JEditToken.KEYWORD1)
						token = JEditToken.NULL;
					lastOffset = i;
				}
			}
			switch(c)
			{
			case '%':
				if(backslash)
				{
					backslash = false;
					break;
				}
				addJEditToken(i - lastOffset,token);
				addJEditToken(length - i,JEditToken.COMMENT1);
				lastOffset = length;
				break loop;
			case '\\':
				backslash = true;
				if(token == JEditToken.NULL)
				{
					token = JEditToken.KEYWORD1;
					addJEditToken(i - lastOffset,JEditToken.NULL);
					lastOffset = i;
				}
				break;
			case '$':
				backslash = false;
				if(token == JEditToken.NULL) // singe $
				{
					token = JEditToken.KEYWORD2;
					addJEditToken(i - lastOffset,JEditToken.NULL);
					lastOffset = i;
				}
				else if(token == JEditToken.KEYWORD1) // \...$
				{
					token = JEditToken.KEYWORD2;
					addJEditToken(i - lastOffset,JEditToken.KEYWORD1);
					lastOffset = i;
				}
				else if(token == JEditToken.KEYWORD2) // $$aaa
				{
					if(i - lastOffset == 1 && array[i-1] == '$')
					{
						token = BDFORMULA;
						break;
					}
					token = JEditToken.NULL;
					addJEditToken(i1 - lastOffset,JEditToken.KEYWORD2);
					lastOffset = i1;
				}
				else if(token == BDFORMULA) // $$aaa$
				{
					token = EDFORMULA;
				}
				else if(token == EDFORMULA) // $$aaa$$
				{
					token = JEditToken.NULL;
					addJEditToken(i1 - lastOffset,JEditToken.KEYWORD2);
					lastOffset = i1;
				}
				break;
			}
		}
		if(lastOffset != length)
			addJEditToken(length - lastOffset,token == BDFORMULA
				|| token == EDFORMULA ? JEditToken.KEYWORD2 :
				token);
		return (token != JEditToken.KEYWORD1 ? token : JEditToken.NULL);
	}
}

/*
 * ChangeLog:
 * $Log: JEditTeXTokenMarker.java,v $
 * Revision 1.3  2001/07/25 17:39:05  desnoix
 * *** empty log message ***
 *
 * Revision 1.2  2001/06/21 18:50:04  desnoix
 * *** empty log message ***
 *
 * Revision 1.1  2001/06/13 19:48:18  desnoix
 * *** empty log message ***
 *
 * Revision 1.15  1999/06/05 00:22:58  sp
 * LGPL'd syntax package
 *
 * Revision 1.14  1999/06/03 08:24:14  sp
 * Fixing broken CVS
 *
 * Revision 1.15  1999/05/31 08:11:10  sp
 * Syntax coloring updates, expand abbrev bug fix
 *
 * Revision 1.14  1999/05/31 04:38:51  sp
 * Syntax optimizations, HyperSearch for Selection added (Mike Dillon)
 *
 * Revision 1.13  1999/04/19 05:38:20  sp
 * Syntax API changes
 *
 * Revision 1.12  1999/04/01 04:13:00  sp
 * Bug fixing for 1.5final
 *
 * Revision 1.11  1999/03/27 00:44:15  sp
 * Documentation updates, various bug fixes
 *
 * Revision 1.10  1999/03/13 00:09:07  sp
 * Console updates, uncomment removed cos it's too buggy, cvs log tags added
 *
 * Revision 1.9  1999/03/12 23:51:00  sp
 * Console updates, uncomment removed cos it's too buggy, cvs log tags added
 *
 */
