/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         ScriptInterpreterActo.java
 * @version      0.29
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.script;

import com.memoire.script.*;

import java.io.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;

public class ScriptInterpreterActo
       implements ScriptInterpreterInterface // , ActionListener
{
  ActionListener actor;
  Hashtable      syms;

  public ScriptInterpreterActo()
  {
    actor     =null;
    syms      =new Hashtable();
  }

  public String getName()
  {
    return "Acto";
  }

  public String getShortDescription()
  {
    return "An acto interpreter";
  } 

  public String getLongDescription()
  {
    return "Acto is a simple action-event interpreter.";
  }

  public Object getIcon()
  {
    return null;
  }

  public String getExtension()
  {
    return "acto";
  }

  public String[] getExtensions()
  {
    return new String[] { "acto", "ACTO" };
  }

  public Object getGUI()
  {
    return null;
  }

  public String toString()
  {
    return "interpreter "+getName();
  }

  private InputStream  in_;
  private OutputStream out_,err_;

  private LineNumberReader input;
  private PrintWriter      output,error;

  public void setInputStream(InputStream _in)
  {
    in_=_in;
    input=new LineNumberReader(new InputStreamReader(_in));
  }

  public InputStream getInputStream()
  {
    return in_;
  }

  public void setOutputStream(OutputStream _out)
  {
    out_=_out;
    output=new PrintWriter(_out,true);
  }

  public OutputStream getOutputStream()
  {
    return out_;
  }

  public void setErrorStream(OutputStream _err)
  {
    err_=_err;
    error=new PrintWriter(_err,true);
  }

  public OutputStream getErrorStream()
  {
    return err_;
  }

  public void start()
  {
    // System.err.println("Start Acto");
    if(in_ ==null) setInputStream (System.in);
    if(out_==null) setOutputStream(System.out);
    if(err_==null) setErrorStream (System.err);
  }

  public void run()
  {
    // System.err.println("Run Acto [begin]");
    try
    {
      while(true)
      {
	error.flush();
	output.print("= ");
	output.flush();

	while(!input.ready())
	{
	  try { Thread.sleep(100); }
	  catch(InterruptedException ex) { }
	  if(exited_) throw new IOException("Exited");
	}

	String newline=input.readLine();
	if(newline==null) break;
	eval(newline);
      }
    }
    catch(IOException ex)
    { 
      System.err.println("Acto.run: "+ex.getMessage());
    }
    // System.err.println("Run Acto [end]");
  }

  private boolean exited_=false;

  public void exit()
  {
    exited_=true;

    // System.err.println("Exit Acto [begin]");

    /*
    try
    {
      input .close();
      output.close();
      error .close();
      in_   .close();
      out_  .close();
      err_  .close();
    }
    catch(Exception ex) { System.err.println(ex.getMessage()); }
    */

    actor =null;
    syms  =null;
    in_   =null;
    out_  =null;
    err_  =null;
    input =null;
    error =null;
    output=null;

    // System.err.println("Exit Acto [end]");
    // Thread.currentThread().getThreadGroup().list();
  }

  public void eval(String _script)
  {
    String r="";

    while(!"".equals(_script))
    {
      try
      {
	int i=_script.indexOf('\n');
	String e;
	if(i<0) { e=_script; _script=""; }
	else    { e=_script.substring(0,i); _script=_script.substring(i+1); }
	e=e.trim();
	if(!"".equals(e)) r=eval1(e);
      }
      catch(Throwable t)
      {
	error.println(t.getMessage());
	r="";
      }
    }

    if(!"".equals(r)) output.println(r);
  }

  private String eval1(String _expr) throws Throwable
  {
    String r="ok";

      String s=_expr.trim();
      Object o=syms.get(s);

      if(o!=null)
      {
	if(o instanceof Method)
	  r=""+((Method)o).invoke(null,new Object[0]);
	else
	if(o instanceof ActionListener)
	  { r="actor is now "+s; actor=(ActionListener)o; }
	else
	  r=o.toString();

	if("null".equals(r)) r="ok";
      }
      else
      if((s.length()>=2)&&(s.charAt(0)=='\"')&&(s.charAt(s.length()-1)=='\"'))
	r=s.substring(1,s.length()-1);
      else
      if("".equals(s))
	r="";
      else
      if("actor".equals(s))
      {
	r=""+actor;
	if("null".equals(r)) r="actor is not defined";
      }
      else
      {
	if(actor!=null)
	  actor.actionPerformed
	    (new ActionEvent(this,ActionEvent.ACTION_PERFORMED,s));
	else
	  throw new Exception("ACTOR UNDEFINED");
	r="";
      }

    return r;
  }

  public void addVariable(String _name, Object _object)
  {
    if((actor==null)&&(_object instanceof ActionListener))
      actor=(ActionListener)_object;
    syms.put(_name,_object);
  }

  public void removeVariable(String _name)
  {
    if(actor==syms.get(_name))
      actor=null;
    syms.remove(_name);
  }

  public void addFunction(String _name, Method _method)
  {
    syms.put(_name,_method);
  }

  public void removeFunction(String _name)
  {
    syms.remove(_name);
  }

  /*
  public void actionPerformed(ActionEvent _evt)
  {
    System.err.println("received: "+_evt.getActionCommand());
  }
  */

  //////////////// Main Loop

  public static void main(String[] files)
  {
    ScriptInterpreterActo acto=new ScriptInterpreterActo();
    acto.start();
    // acto.addVariable("interpreter",acto);

    /*
    try
    {
      for(int i=0; i<files.length;i++) acto.load(files[i]);
    }
    catch (RuntimeException ex) { }
    */

    acto.run();
  }
}
