/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         ScriptInterpreterBeanshell.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/04 18:29:50 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.script;

import com.memoire.agl.*;
import com.memoire.script.*;

import java.io.*;
import java.lang.reflect.*;
import bsh.*;

public class ScriptInterpreterBeanshell
       implements ScriptInterpreterInterface
{
  private Interpreter interpreter;

  public ScriptInterpreterBeanshell()
  {
    try
      { Class.forName("bsh.Interpreter"); }
    catch(ClassNotFoundException ex)
      { throw new RuntimeException("Beanshell not found"); }
  }

  public String getName()
  {
    return "Beanshell";
  }

  public String getShortDescription()
  {
    return "A Beanshell interpreter";
  } 

  public String getLongDescription()
  {
    return "Beanshell is a java-like interpreter.";
  }

  public Object getIcon()
  {
    return null;
  }

  public String getExtension()
  {
    return "bsh";
  }

  public String[] getExtensions()
  {
    return new String[] { "bsh", "BSH" };
  }

  public Object getGUI()
  {
    return null;
  }

  public String toString()
  {
    return "interpreter "+getName();
  }

  private InputStream  in_;
  private Reader       input;
  private OutputStream out_,err_;
  private PrintStream  output,error;

  public void setInputStream(InputStream _in)
  {
    in_=_in;
    input=new InputStreamReader(_in);
  }

  public InputStream getInputStream()
  {
    return in_;
  }

  public void setOutputStream(OutputStream _out)
  {
    out_=_out;
    output=new PrintStream(_out,true);
  }

  public OutputStream getOutputStream()
  {
    return out_;
  }

  public void setErrorStream(OutputStream _err)
  {
    err_=_err;
    error=new PrintStream(_err,true);
  }

  public OutputStream getErrorStream()
  {
    return err_;
  }

  public void start()
  {
    if(in_ ==null) setInputStream (System.in);
    if(out_==null) setOutputStream(System.out);
    if(err_==null) setErrorStream (System.err);

    interpreter=new Interpreter(input,output,error,true,null);
  }

  public void run()
  {
    interpreter.run();
  }

  public void exit()
  {
    // interpreter.eval("exit();");
  }

  public void eval(String _script)
  {
    try
      { output.println(interpreter.eval(_script)); }
    catch(Exception ex)
      { error.println(ex.getMessage()); }
  }

  public void silent_eval(String _script)
  {
    try
      { interpreter.eval(_script); }
    catch(Exception ex)
      { error.println(ex.getMessage()); }
  }

  public void load(String _file)
  {
    try
    {
      interpreter.source(_file);
    }
    catch(FileNotFoundException ex)
      { error.println("couldn't find file "+_file); }
    catch(Exception ex)
      { error.println("couldn't load file "+_file+": "+ex.getMessage()); }
  }

  public void addVariable(String _nom, Object _objet)
  {
    interpreter.setVariable(_nom,_objet);    
  }

  public void removeVariable(String _nom)
  {
    interpreter.setVariable(_nom,null);    
  }

  public void addFunction(String _name, Method _method)
  {
    int l=_method.getParameterTypes().length;

    String p="(";
    for(int i=0;i<l;i++)
    {
      if(i>0) p+=",";
      p+="p"+i;
    }
    p+=")";

    String d=_name+p+
      " { return "+
      _method.getDeclaringClass().getName()+"."+
      _method.getName()+p+"; }";

    System.err.println("method: "+d);
    silent_eval(d);
  }

  public void removeFunction(String _name)
  {
    String d=_name+"()"+
      " { throw new bsh.EvalError(\"Function "+
      _name+"() is undefined.\"); }";

    System.err.println("method: "+d);
    silent_eval(d);
  }

  //////////////// Main Loop

  public static void main(String[] files)
  {
    ScriptInterpreterBeanshell beanshell=new ScriptInterpreterBeanshell();
    beanshell.start();

    for(int i=0; i<files.length;i++)
      beanshell.load(files[i]);

    beanshell.addVariable("hello",new Integer(1));
    try
    {
      Method m=AglPublicAPI.class.getMethod("hello",new Class[0]);
      beanshell.addFunction("hellox",m);
    }
    catch(Exception ex) { ex.printStackTrace(); }
    beanshell.removeFunction("helloy");

    beanshell.run();
  }
}
