/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         ScriptInterpreterDawn.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/07 18:28:12 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.script;

import com.memoire.script.*;
import org.jext.dawn.*;
import java.io.*;
import java.lang.reflect.*;

public class ScriptInterpreterDawn
       implements ScriptInterpreterInterface
{
  private static class XFunction extends Function
  {
    private Method method_;

    public XFunction(String _name, Method _method)
    {
      super(_name);
      method_=_method;
    }

    public void invoke(DawnParser _parser) throws DawnRuntimeException
    {
      try
      {
	//System.err.println("INVOKE "+getName()+"()");
	Object r=method_.invoke(null,new Object[0]);
	if(r!=null) _parser.push(r);
      }
      catch(Exception ex)
      {
	throw new DawnRuntimeException(this,_parser,ex.getMessage());
      }
    }
  }

  public ScriptInterpreterDawn()
  {
    try
      { Class.forName("org.jext.dawn.DawnParser"); }
    catch(ClassNotFoundException ex)
      { throw new RuntimeException("Dawn not found"); }
  }

  public String getName()
  {
    return "Dawn";
  }

  public String getShortDescription()
  {
    return "A RPN interpreter";
  } 

  public String getLongDescription()
  {
    return "Dawn is a small and RPN interpreter similar to Forth.";
  }

  public Object getIcon()
  {
    return null;
  }

  public String getExtension()
  {
    return "dawn";
  }

  public String[] getExtensions()
  {
    return new String[] { "dawn", "daw", "DAWN" };
  }

  public Object getGUI()
  {
    return null;
  }

  public String toString()
  {
    return "interpreter "+getName();
  }

  private InputStream  in_;
  private OutputStream out_,err_;

  private LineNumberReader input;
  private PrintStream      output,error;

  public void setInputStream(InputStream _in)
  {
    in_=_in;
    input=new LineNumberReader(new InputStreamReader(_in));
  }

  public InputStream getInputStream()
  {
    return in_;
  }

  public void setOutputStream(OutputStream _out)
  {
    out_=_out;
    output=new PrintStream(_out,true);
  }

  public OutputStream getOutputStream()
  {
    return out_;
  }

  public void setErrorStream(OutputStream _err)
  {
    err_=_err;
    error=new PrintStream(_err,true);
  }

  public OutputStream getErrorStream()
  {
    return err_;
  }

  DawnParser parser=null;

  public void start()
  {
    if(in_ ==null) setInputStream (System.in);
    if(out_==null) setOutputStream(System.out);
    if(err_==null) setErrorStream (System.err);

    if(!DawnParser.isInitialized())
    {
      DawnParser.init();
      // DawnParser.installPackage(Jext.class, "dawn-jext.scripting");
    }

    // parser.setProperty("JEXT.JEXT_FRAME", parent);
    // parser.exec();
  }

  private static final String CONSOLE="\"\" command ->\n" +
  "while inputLine dup command -> \"exit\" same not repeat\n" +
  "command rcl eval consoleDump \"\\n>\" print\nwend";

  public void run()
  {
    try
    {
      while(true)
      {
	output.print("\n>");
	String l=input.readLine();
	if(l==null) break;
	if(l.equals("exit")) break;

	DawnParser parser=new DawnParser(new StringReader(l));
	//parser.setIn(input);
	parser.setOut(output);
	parser.setErr(error);

	try
	{
	  parser.exec();
	  output.print(parser.dump());
	}
	catch(DawnRuntimeException drex)
	{
	  error.print(drex.getMessage()+'\n');
	}
      }
    }
    catch(IOException ex) { }
    // eval(CONSOLE);
  }

  public void exit()
  {
    if(parser!=null) parser.stop();
  }

  public void eval(String _script)
  {
    try
    {
      DawnParser parser=new DawnParser(new StringReader(_script));
      //parser.setIn(in_);
      parser.setOut(output);
      parser.setErr(error);
      parser.exec();
    }
    catch(DawnRuntimeException drex)
    {
      error.print(drex.getMessage()+'\n');
    }
  }

  public void load(String _file)
  {
    /*
    try
      { dawn_.load(_file); }
    catch(RuntimeException ex)
      { dawn_.error.println(ex.getMessage()); }
    */
  }

  public void addVariable(String _name, Object _object)
  {
    DawnParser.setGlobalVariable(_name,_object);
  }

  public void removeVariable(String _name)
  {
    DawnParser.setGlobalVariable(_name,null);
  }

  public void addFunction(String _name, Method _method)
  {
    DawnParser.addGlobalFunction(new XFunction(_name,_method));
  }

  public void removeFunction(String _name)
  {
  }

  //////////////// Main Loop

  /** Create a new Dawn interpreter, passing in the command line args
   * as files to load, and then enter a read eval write loop. **/
  public static void main(String[] files)
  {
    ScriptInterpreterDawn dawn=new ScriptInterpreterDawn();
    dawn.start();

    for(int i=0; i<files.length;i++)
      dawn.load(files[i]);

    /*
    try
    {
      Method m=com.memoire.agl.AglPublicAPI.class
        .getMethod("hello",new Class[0]);
      dawn.addFunction("hellox",m);
    }
    catch(Exception ex) { ex.printStackTrace(); }
    */

    dawn.run();
  }
}

