/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         ScriptInterpreterFiji.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/07 18:28:12 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.script;
import com.memoire.script.*;


import java.io.*;
import java.lang.reflect.*;
import com.SoftWoehr.FIJI.base.desktop.shell.*;

public class ScriptInterpreterFiji
       implements ScriptInterpreterInterface
{
  public ScriptInterpreterFiji()
  {
    try
      { Class.forName("com.SoftWoehr.FIJI.base.desktop.shell.interpreter"); }
    catch(ClassNotFoundException ex)
      { throw new RuntimeException("Fiji not found"); }
  }

  public String getName()
  {
    return "Fiji";
  }

  public String getShortDescription()
  {
    return "A forth interpreter";
  } 

  public String getLongDescription()
  {
    return "Fiji is a forth interpreter.";
  }

  public Object getIcon()
  {
    return null;
  }

  public String getExtension()
  {
    return "forth";
  }

  public String[] getExtensions()
  {
    return new String[] { "forth", "fth", "FTH" };
  }

  public Object getGUI()
  {
    return null;
  }

  public String toString()
  {
    return "interpreter "+getName();
  }

  private interpreter  fiji_;
  private InputStream  in_;
  private OutputStream out_,err_;
  private PrintWriter  error;

  public void setInputStream(InputStream _in)
  {
    in_=_in;
  }

  public InputStream getInputStream()
  {
    return in_;
  }

  public void setOutputStream(OutputStream _out)
  {
    out_=_out;
  }

  public OutputStream getOutputStream()
  {
    return out_;
  }

  public void setErrorStream(OutputStream _err)
  {
    err_=_err;
    error=new PrintWriter(err_,true);
  }

  public OutputStream getErrorStream()
  {
    return err_;
  }

  public void start()
  {
    if(in_ ==null)  setInputStream (System.in);
    if(out_==null)  setOutputStream(System.out);
    if(err_==null)  setErrorStream (System.err);

    fiji_=new interpreter();
    fiji_.setInput(in_);
    fiji_.setOutput(out_);
    // fiji_.setError(err_);
  }

  public void run()
  {
    BufferedReader rin=new BufferedReader
      (new InputStreamReader(fiji_.getInput()));

    while(!fiji_.getKillFlag()&&!fiji_.getQuitFlag())
    {
      try
      {
	fiji_.prompt();

	while(!rin.ready())
	{
	  try { Thread.sleep(100); }
	  catch(InterruptedException ex) { }
	  if(exited_) return;
	}

	String l=rin.readLine();
	if(l==null) break;
	fiji_.interpret(l);
      }
      catch(Exception ex) { error.println(ex.getMessage()); }
    }
  }

  private boolean exited_=false;

  public void exit()
  {
    // eval("\u0004");
    exited_=true;
  }

  public void eval(String _script)
  {
    fiji_.interpret(_script);
  }

  public void load(String _file)
  {
    try
      { fiji_.load(_file); }
    catch(RuntimeException ex)
      { error.println(ex.getMessage()); }
  }

  public void addVariable(String _nom, Object _objet)
  {
    // fiji_.globalFijiEnvironment.define(_nom,_objet);
  }

  public void removeVariable(String _nom)
  {
    // silent_eval("(define "+_nom+")");
  }

  public void addFunction(String _name, Method _method)
  {
    /*
      String d="(define "+_name+" (method \""+
      _method.getName()+"\" \""+
      _method.getDeclaringClass().getName()+"\"";

    Class[] p=_method.getParameterTypes();
    for(int i=0;i<p.length;i++)
      d+=" \""+p[i].getName()+"\"";

    d+="))";
    System.err.println("method: "+d);
    silent_eval(d);
    */
  }

  public void removeFunction(String _name)
  {
    // silent_eval("(define "+_name+")");
  }

  //////////////// Main Loop

  /** Create a new FijiScheme interpreter, passing in the command line args
   * as files to load, and then enter a read eval write loop. **/
  public static void main(String[] files)
  {
    ScriptInterpreterFiji fiji=new ScriptInterpreterFiji();
    fiji.start();

    for(int i=0; i<files.length;i++)
      fiji.load(files[i]);

    fiji.run();
  }
}
