/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         ScriptInterpreterInterface.java
 * @version      0.29
 * @author       Guillaume Desnoix, Herve Agnoux
 * @email        guillaume@desnoix.com, hagnoux@mail.club-internet.fr
 * @license      Distributed under the terms of the GNU
 *               General Public License 2 (GPL2)
 *               (c)1998-2001 Guillaume Desnoix, Herve Agnoux
 */
package com.memoire.script;

import com.memoire.script.*;
import java.io.*;
import java.lang.reflect.*;

/**
 * @author Guillaume Desnoix
 * @email  guillaume@desnoix.com
 * @author Herve Agnoux
 * @email  hagnoux@mail.club-internet.fr
 */

public interface ScriptInterpreterInterface
       extends Runnable
{
  /**
   * Returns the common name, for example: "Pyhton", "Peanuts", ...
   */
  String getName();

  /**
   * Returns a short description, for tooltips.
   */
  String getShortDescription();

  /**
   * Returns a long description.
   */
  String getLongDescription();

  /**
   * Returns an Icon.
   */
  Object getIcon();

  /**
   * Returns the most common file extension.
   */
  String getExtension();

  /**
   * Returns the possible file extensions.
   */
  String[] getExtensions();

  /**
   * Returns the GUI if available.
   */
  Object getGUI();

  /**
   * Initialises the interpreter, load scripts.
   */
  void start();

  /**
   * Runs the interpreter loop.
   */
  void run();

  /**
   * Exits the interpreter (needed ?)
   */
  void exit();

  /**
   * Evals some script provided by a string.
   * Convenient way, usefull for initialisation.
   */
  void eval(String _script);

  /**
   * Loads some files.
   */
  // void load(InputStream _in);
  // void load(String      _filename);
  // void load(File        _file);
  // void load(URL         _url);

  // set/get streams
  void setInputStream(InputStream _in);
  InputStream getInputStream();
  void setOutputStream(OutputStream _output);
  OutputStream getOutputStream();
  void setErrorStream(OutputStream _error);
  OutputStream getErrorStream();

  // add/remove variables in the context
  void addVariable(String _name, Object _object);
  void removeVariable(String _name);

  // add/remove functions in the context
  void addFunction(String _name, Method _method);
  void removeFunction(String _name);
}
