/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         ScriptInterpreterPnuts.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/04 18:29:50 $
 * @author       Herve Agnoux,Guillaume Desnoix
 * @email        hagnoux@mail.club-internet.fr,guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Herve Agnoux,Guillaume Desnoix.
 */

package com.memoire.script;

import com.memoire.script.*;

import java.io.*;
import java.lang.reflect.*;
import pnuts.lang.*;

/**
 * @author <A HREF="mailto:hagnoux@mail.club-internet.fr">Herve Agnoux</A>
 * @author <A HREF="mailto:guillaume@desnoix.com">Guillaume Desnoix</A>
 */

public class ScriptInterpreterPnuts
       implements ScriptInterpreterInterface
{
  Context context;

  public ScriptInterpreterPnuts()
  {
    try
      { Class.forName("pnuts.lang.Context"); }
    catch(ClassNotFoundException ex)
      { throw new RuntimeException("Pnuts not found"); }
  }

  public String getName()
  {
    return "Pnuts";
  }

  public String getShortDescription()
  {
    return "A pnuts interpreter";
  } 

  public String getLongDescription()
  {
    return "Pnuts is an interpreter.";
  }

  public Object getIcon()
  {
    return null;
  }

  public String getExtension()
  {
    return "pnut";
  }

  public String[] getExtensions()
  {
    return new String[] { "pnut", "PNUT", "pnu", "PNU" };
  }

  public Object getGUI()
  {
    return null;
  }

  public String toString()
  {
    return "interpreter "+getName();
  }

  private InputStream  in_;
  private OutputStream out_,err_;

  private LineNumberReader input;
  private PrintWriter      output,error;

  public void setInputStream(InputStream _in)
  {
    in_=_in;
    input=new LineNumberReader(new InputStreamReader(_in));
  }

  public InputStream getInputStream()
  {
    return in_;
  }

  public void setOutputStream(OutputStream _out)
  {
    out_=_out;
    output=new PrintWriter(_out,true);
  }

  public OutputStream getOutputStream()
  {
    return out_;
  }

  public void setErrorStream(OutputStream _err)
  {
    err_=_err;
    error=new PrintWriter(_err,true);
  }

  public OutputStream getErrorStream()
  {
    return err_;
  }

  public void start()
  {
    context=new Context();

    if(in_ ==null) setInputStream (System.in);
    if(out_==null) setOutputStream(System.out);
    if(err_==null) setErrorStream (System.err);

    context.setOutputStream  (getOutputStream());
    context.setErrorStream   (getErrorStream ());
    context.setTerminalStream(getOutputStream());

    load("init.pnut");
  }

  public void run()
  {
    try
      { Pnuts.load(getInputStream(),true,context); }
    catch(Exception ex)
      { error.println(ex.getMessage()); }
  }

  public void exit()
  {
    // context.exit();
  }

  public void eval(String _script)
  {
    try
      { output.println(Pnuts.eval(_script,context)); }
    catch(Exception ex)
      { error.println(ex.getMessage()); }
  }

  public void load(String _file)
  {
    try
    {
      Pnuts.load(_file,context);
    }
    catch(FileNotFoundException ex)
      { error.println("couldn't find file "+_file); }
    catch(Exception ex)
      { error.println("couldn't load file "+_file+": "+ex.getMessage()); }
  }

  public void addVariable(String _nom, Object _objet)
  {
    Pnuts.set(_nom,_objet);    
  }

  public void removeVariable(String _nom)
  {
    Pnuts.set(_nom,null);    
  }

  public void addFunction(String _name, Method _method)
  {
    /*
      String d="(define "+_name+" (method \""+
      _method.getName()+"\" \""+
      _method.getDeclaringClass().getName()+"\"";

    Class[] p=_method.getParameterTypes();
    for(int i=0;i<p.length;i++)
      d+=" \""+p[i].getName()+"\"";

    d+="))";
    System.err.println("method: "+d);
    silent_eval(d);
    */
  }

  public void removeFunction(String _name)
  {
    // silent_eval("(define "+_name+")");
  }

  //////////////// Main Loop

  public static void main(String[] files)
  {
    ScriptInterpreterPnuts pnuts=new ScriptInterpreterPnuts();
    pnuts.start();

    for(int i=0; i<files.length;i++)
      pnuts.load(files[i]);

    pnuts.run();
  }
}


