/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         ScriptInterpreterPython.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:08 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.script;
import com.memoire.script.*;


import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import org.python.util.*;
import org.python.core.*;

public class ScriptInterpreterPython
       implements ScriptInterpreterInterface
{
  private InteractiveInterpreter interp;

  public ScriptInterpreterPython()
  {
    try
      { Class.forName("org.python.util.InteractiveInterpreter"); }
    catch(ClassNotFoundException ex)
      { throw new RuntimeException("Python not found"); }
  }

  public String getName()
  {
    return "Jython";
  }

  public String getShortDescription()
  {
    return "A python interpreter";
  } 

  public String getLongDescription()
  {
    return "Jython is a python interpreter.";
  }

  public Object getIcon()
  {
    return null;
  }

  public String getExtension()
  {
    return "py";
  }

  public String[] getExtensions()
  {
    return new String[] { "py", "PY" };
  }

  public Object getGUI()
  {
    return null;
  }

  public String toString()
  {
    return "interpreter "+getName();
  }

  private InputStream  in_;
  private OutputStream out_,err_;

  private LineNumberReader input;
  private PrintWriter      output,error;

  public void setInputStream(InputStream _in)
  {
    in_=_in;
    input=new LineNumberReader(new InputStreamReader(_in));
  }

  public InputStream getInputStream()
  {
    return in_;
  }

  public void setOutputStream(OutputStream _out)
  {
    out_=_out;
    output=new PrintWriter(_out,true);
  }

  public OutputStream getOutputStream()
  {
    return out_;
  }

  public void setErrorStream(OutputStream _err)
  {
    err_=_err;
    error=new PrintWriter(_err,true);
  }

  public OutputStream getErrorStream()
  {
    return err_;
  }

  public void start()
  {
    if(in_ ==null) setInputStream (System.in);
    if(out_==null) setOutputStream(System.out);
    if(err_==null) setErrorStream (System.err);

    Options.classBasedExceptions=false;
    Options.showJavaExceptions=false;

    interp=new InteractiveInterpreter();
    interp.setOut(output);
    interp.setErr(error);
  }

  public void run()
  {
    boolean more=false;

    try
    {
      while(true)
      {
	error.flush();
	output.print(more ? "... " : ">>> ");
	output.flush();

	while(!input.ready())
	{
	  try { Thread.sleep(100); }
	  catch(InterruptedException ex) { }
	  if(exited_) throw new IOException("Exited");
	}

	String newline=input.readLine();
	more=interp.runsource(newline);
      }
    }
    catch(IOException ex)
    {
      error.println("Python.run: "+ex.getMessage());
    }
  }

  private boolean exited_=false;

  public void exit()
  {
    exited_=true;
    // interp.exit();
  }

  public void eval(String _script)
  {
    try
    {
      interp.exec(_script);
    }
    catch (Throwable t)
    {
      Py.printException(t);
    }
  }

  public void addVariable(String _nom, Object _objet)
  {
    interp.set(_nom,_objet);
  }

  public void removeVariable(String _nom)
  {
    interp.set(_nom,null);
  }

  public void addFunction(String _name, Method _method)
  {
    /*
      String d="(define "+_name+" (method \""+
      _method.getName()+"\" \""+
      _method.getDeclaringClass().getName()+"\"";

    Class[] p=_method.getParameterTypes();
    for(int i=0;i<p.length;i++)
      d+=" \""+p[i].getName()+"\"";

    d+="))";
    System.err.println("method: "+d);
    silent_eval(d);
    */
  }

  public void removeFunction(String _name)
  {
    // silent_eval("(define "+_name+")");
  }

  //////////////// Main Loop

  public static void main(String[] files)
  {
    ScriptInterpreterPython python=new ScriptInterpreterPython();
    python.start();

    /*
    try
    {
      for(int i=0; i<files.length;i++) python.load(files[i]);
    }
    catch (RuntimeException ex) { }
    */

    python.run();
  }
}
