/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         ScriptInterpreterRhino.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:08 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.script;
import com.memoire.script.*;


import java.io.*;
import java.lang.reflect.*;
import org.mozilla.javascript.*;

public class ScriptInterpreterRhino
       implements ScriptInterpreterInterface
{
  private Context    context;
  private Scriptable scope;

  public ScriptInterpreterRhino()
  {
    try
      { Class.forName("org.mozilla.javascript.Context"); }
    catch(ClassNotFoundException ex)
      { throw new RuntimeException("Rhino not found"); }
  }

  public String getName()
  {
    return "Rhino";
  }

  public String getShortDescription()
  {
    return "A javascript interpreter";
  } 

  public String getLongDescription()
  {
    return "Rhino is a javascript interpreter.";
  }

  public Object getIcon()
  {
    return null;
  }

  public String getExtension()
  {
    return "js";
  }

  public String[] getExtensions()
  {
    return new String[] { "js", "JS" };
  }

  public Object getGUI()
  {
    return null;
  }

  public String toString()
  {
    return "interpreter "+getName();
  }

  private InputStream  in_;
  private OutputStream out_,err_;

  private LineNumberReader input;
  private PrintWriter      output,error;

  public void setInputStream(InputStream _in)
  {
    in_=_in;
    input=new LineNumberReader(new InputStreamReader(_in));
  }

  public InputStream getInputStream()
  {
    return in_;
  }

  public void setOutputStream(OutputStream _out)
  {
    out_=_out;
    output=new PrintWriter(_out,true);
  }

  public OutputStream getOutputStream()
  {
    return out_;
  }

  public void setErrorStream(OutputStream _err)
  {
    err_=_err;
    error=new PrintWriter(_err,true);
  }

  public OutputStream getErrorStream()
  {
    return err_;
  }

  public void start()
  {
    context=Context.enter();

    if(in_ ==null) setInputStream (System.in);
    if(out_==null) setOutputStream(System.out);
    if(err_==null) setErrorStream (System.err);

    scope=context.initStandardObjects(null);
  }

  public void run()
  {
    boolean hitEOF=false;
    while(!hitEOF)
    {
      int startline=input.getLineNumber();
      error.flush();
      output.print("js> ");
      output.flush();
      try
      {
	String source="";
	while(true)
	{

	  while(!input.ready())
	  {
	    try { Thread.sleep(100); }
	    catch(InterruptedException ex) { }
	    if(exited_)
	    {
	      hitEOF=true;
	      break;
	    }
	  }

	  String newline=input.readLine();

	  if(newline==null)
	  {
	    hitEOF=true;
	    break;
	  }
	  source=source+newline+"\n";
	  if(context.stringIsCompilableUnit(source))
	    break;
	}
	Object result=context.evaluateString(scope,source,
					     "<stdin>",startline,
					     null);
	if(result!=context.getUndefinedValue())
	  output.println(context.toString(result));
      }
      catch(Exception ex)
      {
	error.println(ex.getMessage());
      }
    }
  }

  private boolean exited_=false;

  public void exit()
  {
    exited_=true;
    context.exit();
  }

  public void eval(String _script)
  {
    try
    {
      Object r=context.evaluateString(scope,_script,"<script>",1,null);
      if(r!=context.getUndefinedValue())
	output.println(context.toString(r));
    }
    catch(Exception ex)
    {
      error.println(ex.getMessage());
    }
  }

  public void addVariable(String _nom, Object _objet)
  {
    scope.put(_nom,scope,_objet);
  }

  public void removeVariable(String _nom)
  {
    scope.delete(_nom);
  }

  public void addFunction(String _name, Method _method)
  {
    /*
      String d="(define "+_name+" (method \""+
      _method.getName()+"\" \""+
      _method.getDeclaringClass().getName()+"\"";

    Class[] p=_method.getParameterTypes();
    for(int i=0;i<p.length;i++)
      d+=" \""+p[i].getName()+"\"";

    d+="))";
    System.err.println("method: "+d);
    silent_eval(d);
    */
  }

  public void removeFunction(String _name)
  {
    // silent_eval("(define "+_name+")");
  }

  //////////////// Main Loop

  public static void main(String[] files)
  {
    ScriptInterpreterRhino rhino=new ScriptInterpreterRhino();
    rhino.start();

    /*
    try
    {
      for(int i=0; i<files.length;i++) rhino.load(files[i]);
    }
    catch (RuntimeException ex) { }
    */

    rhino.run();
  }
}
