/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         ScriptInterpreterSilk.java
 * @version      0.29
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.script;

import com.memoire.script.*;
import com.memoire.silk.*;

import java.io.*;
import java.lang.reflect.*;

public class ScriptInterpreterSilk
       implements ScriptInterpreterInterface
{
  public ScriptInterpreterSilk()
  {
  }

  public String getName()
  {
    return "Silk";
  }

  public String getShortDescription()
  {
    return "A scheme interpreter";
  } 

  public String getLongDescription()
  {
    return "Silk is a small and efficient scheme interpreter.";
  }

  public Object getIcon()
  {
    return null;
  }

  public String getExtension()
  {
    return "scm";
  }

  public String[] getExtensions()
  {
    return new String[] { "scm", "SCM" };
  }

  public Object getGUI()
  {
    return null;
  }

  public String toString()
  {
    return "interpreter "+getName();
  }

  private SilkScheme   silk_;
  private InputStream  in_;
  private OutputStream out_,err_;

  public void setInputStream(InputStream _in)
  {
    in_=_in;
  }

  public InputStream getInputStream()
  {
    return in_;
  }

  public void setOutputStream(OutputStream _out)
  {
    out_=_out;
  }

  public OutputStream getOutputStream()
  {
    return out_;
  }

  public void setErrorStream(OutputStream _err)
  {
    err_=_err;
  }

  public OutputStream getErrorStream()
  {
    return err_;
  }

  public void start()
  {
    if(in_ ==null) setInputStream (System.in);
    if(out_==null) setOutputStream(System.out);
    if(err_==null) setErrorStream (System.err);

    silk_=new SilkScheme();
    silk_.globalSilkEnvironment=new SilkEnvironment();
    silk_.input =new SilkInputPort(in_);
    silk_.output=new PrintWriter(out_,true);
    silk_.error =new PrintWriter(err_,true);
    SilkPrimitive.installSilkPrimitives(silk_.globalSilkEnvironment); 

    try
    {
      silk_.load(new SilkInputPort
		 (new StringReader(SilkSchemePrimitives.CODE)));

      addVariable("silk-vars",silk_.globalSilkEnvironment.vars);
    }
    catch(RuntimeException ex) { }
  }

  public void run()
  {
    silk_.readEvalWriteLoop();
  }

  public void exit()
  {
    silk_.exit();
  }

  public void eval(String _script)
  {
    SilkInputPort in=new SilkInputPort(new StringReader(_script));
    Object        x =null;

    while(true)
    {
      if(in.isEOF(x=in.read())) return;
      silk_.write(silk_.eval(x),silk_.output,true); 
      silk_.output.println();
      silk_.output.flush();
    }
  }

  private void silent_eval(String _script)
  {
    SilkInputPort in=new SilkInputPort(new StringReader(_script));
    Object        x =null;

    while(true)
    {
      if(in.isEOF(x=in.read())) return;
      silk_.eval(x);
    }
  }

  public void load(String _file)
  {
    try
      { silk_.load(_file); }
    catch(RuntimeException ex)
      { silk_.error.println(ex.getMessage()); }
  }

  public void addVariable(String _name, Object _object)
  {
    silk_.globalSilkEnvironment.define(_name,_object);
  }

  public void removeVariable(String _name)
  {
    silent_eval("(define "+_name+")");
  }

  public void addFunction(String _name, Method _method)
  {
    String d="(define "+_name+" (method \""+
      _method.getName()+"\" \""+
      _method.getDeclaringClass().getName()+"\"";

    Class[] p=_method.getParameterTypes();
    for(int i=0;i<p.length;i++)
      d+=" \""+p[i].getName()+"\"";

    d+="))";
    System.err.println("method: "+d);
    silent_eval(d);
  }

  public void removeFunction(String _name)
  {
    silent_eval("(define "+_name+")");
  }

  //////////////// Main Loop

  /** Create a new SilkScheme interpreter, passing in the command line args
   * as files to load, and then enter a read eval write loop. **/
  public static void main(String[] files)
  {
    ScriptInterpreterSilk silk=new ScriptInterpreterSilk();
    silk.start();

    for(int i=0; i<files.length;i++)
      silk.load(files[i]);

    /*
    try
    {
      Method m=AglPublicAPI.class.getMethod("hello",new Class[0]);
      silk.addFunction("hellox",m);
    }
    catch(Exception ex) { ex.printStackTrace(); }
    */

    silk.run();
  }
}

