/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         ScriptPreferencesPanel.java
 * @version      0.36
 * @www          http://www.memoire.com/guillaume-desnoix/alma/
 * @modification $Date: 2001/12/03 16:28:08 $
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.script;
import com.memoire.script.*;
import com.memoire.bu.*;
import com.memoire.fu.*;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.*;

/**
 * A panel where the user can choose which script interpreters
 * are available and what is the the scripts directory.
 * @version      0.02, 2001-03-14
 * @author       Guillaume Desnoix
 */

/**
 * script.directory
 * script.interpreters
 */

public class ScriptPreferencesPanel
       extends BuAbstractPreferencesPanel
       implements ActionListener
{
  protected BuCommonInterface appli_;
  protected FuPreferences     options_;

  protected JTextField        tfDir_;
  protected JButton           btDir_;
  protected JTextField        tfInt_;
  
  public String getTitle()
    { return BuResource.BU.getString("Script"); }

  // Constructeur

  public ScriptPreferencesPanel(BuCommonInterface _appli)
  {
    super();
    appli_=_appli;
    options_=ScriptPreferences.SCRIPT;

    BuGridLayout lo=new BuGridLayout();
    lo.setColumns(2);
    lo.setVgap(5);
    lo.setHgap(5);

    JPanel p=new JPanel();
    p.setBorder
      (new CompoundBorder
       (new TitledBorder(BuResource.BU.getString("Script")),
	new EmptyBorder(5,5,5,5)));
    p.setLayout(lo);

    BuBorderLayout lo1=new BuBorderLayout();
    lo1.setHgap(5);
    JPanel p1=new JPanel();
    p1.setLayout(lo1);

    tfDir_=new JTextField(25);
    tfDir_.setName("tfDIRECTORY");
    tfDir_.setEditable(false);
    btDir_=new BuButton();
    btDir_.setRequestFocusEnabled(false);
    btDir_.setIcon(getDirectoryIcon());
    btDir_.setName("btDIRECTORY");
    btDir_.setMargin(new Insets(0,0,0,0));
    btDir_.addActionListener(this);

    p1.add(tfDir_,BuBorderLayout.CENTER);
    p1.add(btDir_,BuBorderLayout.EAST);
    p.add(new JLabel(BuResource.BU.getString("Rpertoire:"),JLabel.RIGHT));
    p.add(p1);

    p.add(new JLabel(BuResource.BU.getString("Interprteurs:"),JLabel.RIGHT));
    tfInt_=new JTextField(25);
    tfInt_.setName("tfINTERPRETERS");
    p.add(tfInt_);

    setLayout(new BuBorderLayout());
    setBorder(new EmptyBorder(5,5,5,5));
    add(p,BuBorderLayout.CENTER);

    updateComponents();
  }

  private final static Icon getDirectoryIcon()
  {
    Icon r=UIManager.getIcon("FileView.directoryIcon");
    if(r==null) r=BuResource.BU.getIcon("ouvrir");
    return r;
  }

  // Evenements
  
  public void actionPerformed(ActionEvent _evt)
  {
    JComponent source=(JComponent)_evt.getSource();
    System.err.println("SOURCE="+source.getName());
    System.err.println("ACTION="+_evt.getActionCommand());

    if(source==btDir_)
      tfDir_.setText(getDirectory(tfDir_.getText()));
	
    // chScript.setEnabled(cbExterne.isSelected());
  }
  
  // Methodes publiques

  public boolean isPreferencesValidable()
    { return true; }

  public void validatePreferences()
  {
    fillTable();
    options_.writeIniFile();
  }

  public boolean isPreferencesCancelable()
    { return true; }

  public void cancelPreferences()
  {
    options_.readIniFile();
    updateComponents();
  }

  // Methodes privees
  
  private void fillTable()
  {
    options_.putStringProperty("script.directory",tfDir_.getText());
    options_.putStringProperty("script.interpreters",tfInt_.getText());
  }

  private void updateComponents()
  {
    tfDir_.setText(options_.getStringProperty("script.directory"));
    tfInt_.setText(options_.getStringProperty("script.interpreters",
					      ScriptPreferences.INTERPRETERS));
  }

  private String getDirectory(String _default)
  {
    String r=BuLib.expandedPath(_default);

    // System.err.println("### FD begin");
    BuFileChooser chooser=new BuFileChooser();
    chooser.setDialogTitle(BuResource.BU.getString("Repertoire"));
    chooser.setFileSelectionMode(BuFileChooser.DIRECTORIES_ONLY);
    chooser.setFileHidingEnabled(true);
    chooser.setMultiSelectionEnabled(false);
    chooser.resetChoosableFileFilters();

    File f=new File(r);
    if(f.exists()) chooser.setCurrentDirectory(f);

    int returnVal=chooser.showDialog(this,BuResource.BU.getString("Slectionner"));
    // System.err.println("### FD end");

    if(returnVal == BuFileChooser.APPROVE_OPTION)
    {
      f=chooser.getSelectedFile();
      // System.err.println("### f="+f);
      if(f!=null) r=f.getPath();
    }

    return BuLib.reducedPath(r);
  }
}
