/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         ScriptTerminal.java
 * @version      0.29
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.script;

import com.memoire.script.*;
import com.memoire.bu.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.border.*;

public class ScriptTerminal
       extends BuTerminal
{
  ScriptInterpreterInterface interpreter_;

  public ScriptTerminal()
  {
    input_ =display_.createInStream(new Color(128,160,255),true);
    output_=display_.createOutStream();
    error_ =display_.createOutStream(new Color(255,128,0),
				     BuTerminalDisplay.INVERSE);

    // exemples

    /*
    BuTerminalKey[] keys=new BuTerminalKey[]
      {
	display_.createKey("hello" ,-1,BuTerminalDisplay.BOLD      ), 
	display_.createKey("heaven",-1,BuTerminalDisplay.BOLD      ), 
	display_.createKey("world" ,-1,BuTerminalDisplay.BOLD      ), 
	display_.createKey("js>"   ,-1,BuTerminalDisplay.INVERSE   ), 
	display_.createKey("not"   ,-1,BuTerminalDisplay.UNDERLINED),
      };
    input_ .setKeys(keys);
    output_.setKeys(keys);
    error_ .setKeys(keys);
    */

    error_ .setLeftMargin(4);
    error_ .setRightMargin(4);

    /*
    setShortcuts(new String[]
		 { "java.lang.System.out.println(\"",
		   "\");\n",
		   "1+2\n",
		   null,
		   null,
		   null,
		   null,
		   null,
		   null,
		   null,
                   null,
		 });
     */

    // pas necessaire normalement...
    // System.setIn (input_);
    // System.setOut(new PrintStream(output_,true));
    // System.setErr(new PrintStream(error_ ,true));
  }

  /*
  public ScriptTerminal(ScriptInterpreterInterface _interpreter)
  {
    this();
    setScriptInterpreter(_interpreter);
  }
  */

  public void reset(String n)
  {
    setScriptInterpreter(ScriptInterpreter.language(n));
    setShortcuts(ScriptInterpreter.shortcuts(n));
  }

  public ScriptInterpreterInterface getScriptInterpreter()
  {
    return interpreter_;
  }

  public void setScriptInterpreter(ScriptInterpreterInterface _interpreter)
  {
    interpreter_=_interpreter;
  }

  private ScriptThread thread_  =null;
  private BuTaskView   taskview_=null;

  // public ScriptThread getThread() { return thread_; }

  public void setTaskView(BuTaskView _tv)
  {
    taskview_=_tv;
  }

  public void startThread()
  {
    if(interpreter_==null) return;
    if(thread_!=null)      return;

    thread_=new ScriptThread(interpreter_,input_,output_,error_);
    // System.err.println("ScriptTerminal: startThread: "+thread_);
    thread_.setTaskView(taskview_);
    thread_.start();
  }

  public void stopThread()
  {
    // System.err.println("ScriptTerminal: stopThread: "+thread_);
    if(thread_!=null) thread_.shouldStop();
    thread_=null;
  }

  public static void main(String args[]) throws IOException
  {
    String interpreter="Foo";

    if(args.length>0) interpreter=args[0];

    final JFrame f=new JFrame("Terminal "+interpreter);
    final ScriptTerminal t=new ScriptTerminal();
    t.reset(interpreter);

    f.getContentPane().setLayout(new BuBorderLayout());
    f.getContentPane().add(t,BuBorderLayout.CENTER);
    f.pack();

    f.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    f.addWindowListener
      (new WindowAdapter()
       {
	 public void windowClosing(WindowEvent _evt)
	   { t.stopThread(); BuRegistry.unregister(f); }
       });

    BuRegistry.register(f);
    f.show();
    t.startThread();
    // Thread.currentThread().getThreadGroup().list();
  }
}

