/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         ScriptThread.java
 * @version      0.29
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.script;

import com.memoire.bu.*;
import com.memoire.script.*;

import java.io.*;

public class ScriptThread extends BuTask // Thread
{
  ScriptInterpreterInterface interpreter_;
  InputStream                in_;
  OutputStream               out_,err_;
   
  public ScriptThread(ScriptInterpreterInterface _interpreter,
		      InputStream _in,OutputStream _out,OutputStream _err)
  {
    super();

    String n=""+_interpreter;
    if(n.length()>=1) n=n.substring(0,1).toUpperCase()+n.substring(1);
    setName(n);

    setPriority(Thread.MIN_PRIORITY);
    interpreter_=_interpreter;
    in_=_in;
    out_=_out;
    err_=_err;
  }

  public void start()
  {
    ScriptInterpreterInterface si=interpreter_;

    synchronized(si)
    {
      si.setInputStream(in_);
      si.setOutputStream(out_);
      si.setErrorStream(err_);
      si.start();
      si.addVariable("interpreter",si);
    }

    super.start();
  }

  public void run()
  {
    ScriptInterpreterInterface si=interpreter_;

    try
    {
      si.run();
      new PrintStream(err_,true).println
	(si.getName()+" finished.");
    }
    catch(Exception ex)
    {
      new PrintStream(err_,true).println
	(si.getName()+" stopped! "+ex.getMessage());
    }
    
    new PrintStream(err_,true).println("");

    si.exit();
    super.run();
  }

  public void shouldStop()
  {
    ScriptInterpreterInterface si=interpreter_;
    System.err.println("ScriptThread: shouldStop: "+si);
    if(si!=null) si.exit();
    super.run();
  }
}
