

package com.memoire.silk;
import com.memoire.silk.*;



/** A closure is a user-defined procedure.  It is "closed" over the
 * environment in which it was created.  To apply the procedure, bind
 * the parameters to the passed in variables, and evaluate the body.
 * @author Peter Norvig, peter@norvig.com http://www.norvig.com 
 * Copyright 1998 Peter Norvig, see http://www.norvig.com/license.html **/

public class SilkClosure extends SilkProcedure {

    Object parms;
    Object body;
    SilkEnvironment env;
    
    /** Make a closure from a parameter list, body, and environment. **/
    public SilkClosure (Object parms, Object body, SilkEnvironment env) {
        this.parms = parms;
	this.env = env;
	this.body = (body instanceof SilkPair && rest(body) == null)
	    ? first(body)
	    : cons("begin", body);
    }

    /** Apply a closure to a list of arguments.  **/
    public Object apply(SilkScheme interpreter, Object args) {
	return interpreter.eval(body, new SilkEnvironment(parms, args, env));
    }
}
