/**
 * @modification 2001-10-25
 * @statut       unstable
 * @file         XmlBrowser.java
 * @version      0.04
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.xml;

import com.memoire.fu.*;
import com.memoire.bu.*;
import com.memoire.xml.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

public class XmlBrowser
       extends BuPanel
       implements MouseListener
{
  public final static String FONT_FAMILY="SansSerif";
  public final static int    FONT_SIZE  =12;

  public int   width_    =580;
  public int   height_   =580;
  public URL   base_     =null;

  private Color linkColor_;
  private int   cellspacing_;
  private int   cellpadding_;
  private int   cellborder_ ;
  private Color trbg_;

  private Stack panels_;
  private Stack fonts_;
  private Stack fgs_;
  private Stack cursors_;
  private Stack actions_;
  //private Stack tables_;

  public XmlBrowser()
  {
    clean();
  }

  private void clean()
  {
    removeAll();

    linkColor_  =Color.blue;
    cellspacing_=2;
    cellpadding_=2;
    cellborder_ =0;
    trbg_       =null;

    setBorder(new EmptyBorder(5,5,5,5));
    setBackground(UIManager.getColor("Panel.background"));
    setForeground(UIManager.getColor("Panel.foreground"));
    setOpaque(true);
    setLayout(new XmlLayout(0));

    panels_ =new Stack();
    fonts_  =new Stack();
    fgs_    =new Stack();
    cursors_=new Stack();
    actions_=new Stack();
    //tables_ =new Stack();

    pushPanel(this);
    pushFont(new Font(FONT_FAMILY,Font.PLAIN,FONT_SIZE));
    pushFg(getForeground());
    pushCursor(new Cursor(Cursor.TEXT_CURSOR));
  }

  private boolean done_=false;

  /*
  public void setBounds(int _x, int _y, int _w, int _h)
  {
    dontpaint_=true;
    super.setBounds(_x,_y,_w,_h);
    dontpaint_=false;
  }
  */

  /*
  public Dimension getPreferredSize()
  {
    Dimension r=super.getPreferredSize();
    if(r.width <width_ ) r.width =width_ ;
    if(r.height<height_) r.height=height_;
    return r;
  }
  */

  /*
  public Dimension getSize()
  {
    Dimension r=super.getSize();
    if(r.width>width_ ) r.width =width_;
    //if(r.height<height_) r.height=height_;
    return r;
  }
  */

  public void doLayout()
  {
    if(!done_) layout();
  }

  public void layout()
  {
    if(!done_)
    {
      //System.err.println("LAYOUT");
      setPreferredSize(null);
      setSize(width_,height_);
      super.layout();
      done_=true;
      Dimension ps=getPreferredSize();
      if(ps.width <width_ ) ps.width =width_;
      if(ps.height<height_) ps.height=height_;
      setSize(ps);
      setPreferredSize(ps);
      System.err.println("PS  ="+ps);
    }
  }

  public void invalidate()
  {
    //System.err.println("INVALIDATE");
    if(!done_) super.invalidate();
  }

  /*
  public void validate()
  {
    //System.err.println("VALIDATE");
    if(!done_) super.validate();
  }

  public void revalidate()
  {
    //System.err.println("REVALIDATE");
    if(!done_) super.revalidate();
  }
  */

  public void paint(Graphics _g)
  {
    //System.err.println("PAINT "+done_);
    if(done_) super.paint(_g);
    //System.err.println("END PAINT "+done_);
  }

  public void repaint()
  {
    //System.err.println("REPAINT "+done_);
    if(done_) super.repaint();
    //System.err.println("END REPAINT "+done_);
  }

  public void mouseClicked (MouseEvent _evt)
  {
    //System.err.println("EVT="+_evt);
    JComponent source=(JComponent)_evt.getSource();
    String     action=(String)source.getClientProperty("XML_ACTION");
    System.err.println("ACTION="+action);

    URL u=null;
    try
    {
      u=new URL(base_,action);
      System.err.println("URL="+u);
      setPage(u);
    }
    catch(Exception ex) { }
  }

  public void mousePressed (MouseEvent _evt) { }
  public void mouseReleased(MouseEvent _evt) { }
  public void mouseEntered (MouseEvent _evt) { }
  public void mouseExited  (MouseEvent _evt) { }

  public void setPage(URL _u)
  {
    clean();

    try
    {
      int m=XmlParser.HTML;
      //if(f.endsWith(".html")||f.endsWith(".htm")) m=XmlParser.HTML;
      //if(f.endsWith(".txt"))                      m=XmlParser.TXT;
      System.err.println("URL: "+_u+" MODE: "+m);

      Reader in=new InputStreamReader(_u.openStream());

      XmlParser p=new XmlParser(in,_u.toString(),m);
      XmlReader r=new XmlReader(m);
      p.setXmlListener(r);
      p.parse();
      XmlNode t=r.getTree();
      //System.out.println(t);

      done_=false;
      base_=_u;
      element(t);
      revalidate();
      //((JComponent)getParent()).revalidate();
    }
    catch(Exception ex) { ex.printStackTrace(); }
  }

  private Font   peekFont()            { return (Font)fonts_.peek(); }
  private void   pushFont(Font _f)     { fonts_.push(_f); }
  private void   popFont()             { fonts_.pop(); }

  private Color  peekFg()              { return (Color)fgs_.peek(); }
  private void   pushFg(Color _f)      { fgs_.push(_f); }
  private void   popFg()               { fgs_.pop(); }

  private Cursor peekCursor()          { return (Cursor)cursors_.peek(); }
  private void   pushCursor(Cursor _c) { cursors_.push(_c); }
  private void   popCursor()           { cursors_.pop(); }

  private JPanel peekPanel()           { return (JPanel)panels_.peek(); }
  private void   pushPanel(JPanel _c)  { panels_.push(_c); }
  private void   popPanel()            { panels_.pop(); }

  private String peekAction()          
  {
    String r=null;
    if(!actions_.empty()) r=(String)actions_.peek();
    return r;
  }
  private void   pushAction(String _f) { actions_.push(_f); }
  private void   popAction()           { actions_.pop(); }

  public String getText(XmlNode _n)
  {
    return getText(_n.getText());
  }

  public String getText(String s)
  {
    if(s==null) return null;

    s=XmlParser.replace(s,"&lt;"  ,"<" );
    s=XmlParser.replace(s,"&gt;"  ,">" );
    s=XmlParser.replace(s,"&quot;","\"");
    s=XmlParser.replace(s,"&amp;" ,"&" );

    s=XmlParser.replace(s,"&agrave;","");
    s=XmlParser.replace(s,"&egrave;","");
    s=XmlParser.replace(s,"&ograve;","\u00f2");
    s=XmlParser.replace(s,"&ugrave;","");

    s=XmlParser.replace(s,"&eacute;","");
    s=XmlParser.replace(s,"&oacute;","\u00f3");

    s=XmlParser.replace(s,"&acirc;" ,"");
    s=XmlParser.replace(s,"&ecirc;" ,"");
    s=XmlParser.replace(s,"&icirc;" ,"");
    s=XmlParser.replace(s,"&ocirc;" ,"");
    s=XmlParser.replace(s,"&ucirc;" ,"");

    s=XmlParser.replace(s,"&euml;"  ,"");
    s=XmlParser.replace(s,"&iuml;"  ,"");
    s=XmlParser.replace(s,"&ouml;"  ,"");
    s=XmlParser.replace(s,"&uuml;"  ,"");

    s=XmlParser.replace(s,"&ccedil;","");

    s=XmlParser.replace(s,"&nbsp;"  ,"\u00a0");
    s=XmlParser.replace(s,"&copy;"  ,"\u00a9");
    s=XmlParser.replace(s,"&deg;"   ,"\u00b0");

    s=XmlParser.replace(s,"&ntilde;","\u00f1");
    s=XmlParser.replace(s,"&otilde;","\u00f5");

    s=XmlParser.replace(s,"&Agrave;","\u00c0");
    s=XmlParser.replace(s,"&Egrave;","\u00c8");
    s=XmlParser.replace(s,"&Eacute;","\u00c9");

    return s;
  }

  private String getString(XmlNode _n, String _attr)
  {
    return (String)_n.attributes_.get(_attr);
  }

  private int getInt(XmlNode _n, String _attr, int _default)
  {
    int    r=_default;
    String s=getString(_n,_attr);
    if(s!=null)
    {
      try { r=Integer.parseInt(s); }
      catch(Exception ex) { }
    }
    return r;
  }

  private int getWidth(XmlNode _n, int _default)
  {
    int    r=_default;
    String s=getString(_n,"WIDTH");
    if(s!=null)
    {
      if(s.endsWith("%"))
      {
        try { r=-Integer.parseInt(s.substring(0,s.length()-1)); }
        catch(Exception ex) { }
      }
      else
      {
        try { r=Integer.parseInt(s); }
        catch(Exception ex) { }
      }
    }
    return r;
  }

  private int getHeight(XmlNode _n, int _default)
  {
    int    r=_default;
    String s=getString(_n,"HEIGHT");
    if(s!=null)
    {
      if(s.endsWith("%"))
      {
        try { r=-Integer.parseInt(s.substring(0,s.length()-1)); }
        catch(Exception ex) { }
      }
      else
      {
        try { r=Integer.parseInt(s); }
        catch(Exception ex) { }
      }
    }
    return r;
  }

  private Color getColor(XmlNode _n, String _attr, Color _default)
  {
    Color  r=_default;
    String s=getString(_n,_attr);
    if(s!=null)
    {
      if(s.startsWith("#"))
      {
	try { r=new Color(Integer.parseInt(s.substring(1),16)); }
	catch(Exception ex) { }
      }
    }
    return r;
  }

  private int getAlign(XmlNode _n, int _align)
  {
    int    r=_align;
    String s=getString(_n,"ALIGN");
    if(s!=null)
    {
      s=s.toUpperCase();
      if("LEFT"   .equals(s))                     r=0;
      if("MIDDLE" .equals(s)||"CENTER".equals(s)) r=1;
      if("RIGHT"  .equals(s))                     r=2;
      if("JUSTIFY".equals(s))                     r=3;
    }
    return r;
  }

  private void insert(JComponent _c)
  {
    peekPanel().add(_c);
  }

  public void body(XmlNode _n)
  {
    Color bg=getColor(_n,"BGCOLOR",null);
    Color fg=getColor(_n,"TEXT"   ,null);
    Color lk=getColor(_n,"LINK"   ,null);

    if(bg!=null) setBackground(bg);
    if(fg!=null) { setForeground(fg); popFg(); pushFg(fg); }
    if(lk!=null) linkColor_=lk;

    children(_n);
  }

  public void big(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font(f.getFamily(),f.getStyle(),f.getSize()+2));
    children(_n);
    popFont();
  }

  public void bold(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font(f.getFamily(),Font.BOLD,f.getSize()));
    children(_n);
    popFont();
  }

  public void italic(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font(f.getFamily(),Font.ITALIC,f.getSize()));
    children(_n);
    popFont();
  }

  public void small(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font(f.getFamily(),f.getStyle(),f.getSize()-2));
    children(_n);
    popFont();
  }

  public void tt(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font("Monospaced",f.getStyle(),f.getSize()));
    children(_n);
    popFont();
  }

  public void font(XmlNode _n)
  {
    Color color=getColor(_n,"COLOR",null);
    if(color!=null) pushFg(color);

    String size =getString(_n,"SIZE");
    if(size!=null)
    {
      Font f=peekFont();
      int taille=f.getSize();
	   if("+1".equals(size)) taille=(taille*12)/10;
      else if("+2".equals(size)) taille=(taille*14)/10;
      else if("+3".equals(size)) taille=(taille*16)/10;
      else if("+4".equals(size)) taille=(taille*18)/10;
      else if("+5".equals(size)) taille=(taille*20)/10;
      else if("+6".equals(size)) taille=(taille*22)/10;
      else if("+7".equals(size)) taille=(taille*24)/10;
      else if("-1".equals(size)) taille=(taille*10)/12;
      else if("-2".equals(size)) taille=(taille*10)/14;
      else if("-3".equals(size)) taille=(taille*10)/16;
      else if("-4".equals(size)) taille=(taille*10)/18;
      else if("-5".equals(size)) taille=(taille*10)/20;
      else if("-6".equals(size)) taille=(taille*10)/22;
      else if("-7".equals(size)) taille=(taille*10)/24;
      else if("1" .equals(size)) taille=(FONT_SIZE*10)/14;
      else if("2" .equals(size)) taille=(FONT_SIZE*10)/12;
      else if("3" .equals(size)) taille=FONT_SIZE;
      else if("4" .equals(size)) taille=(FONT_SIZE*12)/10;
      else if("5" .equals(size)) taille=(FONT_SIZE*14)/10;
      else if("6" .equals(size)) taille=(FONT_SIZE*16)/10;
      else if("7" .equals(size)) taille=(FONT_SIZE*18)/10;
      pushFont(new Font(f.getFamily(),f.getStyle(),taille));
    }

    children(_n);

    if(size!=null)
    {
      popFont();
    }

    if(color!=null) popFg();
  }

  public void strong(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font(f.getFamily(),Font.BOLD,f.getSize()));
    children(_n);
    popFont();
  }

  public void em(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font(f.getFamily(),Font.ITALIC,f.getSize()));
    children(_n);
    popFont();
  }

  public void cite(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font(FONT_FAMILY,Font.PLAIN,8));
    pushFg(new Color(128,0,0));
    children(_n);
    popFg();
    popFont();
  }

  public void code(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font("Monospaced",Font.PLAIN,f.getSize()));
    pushFg(new Color(128,0,0));
    children(_n);
    popFg();
    popFont();
  }

  public void kbd(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font("Monospaced",Font.PLAIN,f.getSize()));
    pushFg(new Color(128,0,0));
    children(_n);
    popFg();
    popFont();
  }

  public void br(XmlNode _n)
  {
    BuLabel r=new BuLabel();
    r.setText("");
    r.setFont(peekFont());
    r.setForeground(peekFg());
    r.setCursor(peekCursor());
    r.putClientProperty("XML_BREAKAFTER",Boolean.TRUE);
    insert(r);
  }

  public void nobr(XmlNode _n)
  {
    BuPanel r=new BuPanel();
    r.setLayout(new FlowLayout(FlowLayout.LEFT));
    pushPanel(r);
    children(_n);
    popPanel();
    insert(r);
  }

  public void hr(XmlNode _n)
  {
    BuPanel r=new BuPanel();
    r.setPreferredSize(new Dimension(1,1));

    r.setBackground(Color.black);
    //r.setCursor(peekCursor());
    r.putClientProperty("XML_BREAKBEFORE",Boolean.TRUE);
    r.putClientProperty("XML_BREAKAFTER" ,Boolean.TRUE);
    insert(r);
  }

  public XmlBlock block(XmlNode _n, Font _f, int _a, String _t)
  {
    return block(_n,_f,_a,_t,null);
  }

  public XmlBlock block
    (XmlNode _n, Font _f, int _a, String _t, Border _b)
  {
    XmlBlock p=new XmlBlock();
    p.setBorder(_b);
    p.setLayout(new XmlLayout(getAlign(_n,_a)));
    p.setOpaque(false);
    //p.setBorder(new LineBorder(new Color(255,255,128)));
    //p.setPreferredWidth (getWidth (_n,0));
    //p.setPreferredHeight(getHeight(_n,0));

    pushPanel(p);

    if(!"".equals(_t))
    {
      BuLabel r=new BuLabel(_t);
      r.setFont(_f);
      r.setForeground(peekFg());
      r.setCursor(peekCursor());
      //r.putClientProperty("XML_BREAKBEFORE",Boolean.TRUE);
      insert(r);
    }

    pushFont(_f);
    children(_n);
    popFont();

    /*
    r=new BuLabel("");
    r.setFont(_f);
    r.setForeground(peekFg());
    r.setCursor(peekCursor());
    r.putClientProperty("XML_BREAKAFTER",Boolean.TRUE);
    insert(r);
    */

    popPanel();
    p.putClientProperty("XML_BREAKBEFORE",Boolean.TRUE);
    p.putClientProperty("XML_BREAKAFTER",Boolean.TRUE);
    insert(p);
    return p;
  }

  public void header(XmlNode _n, int _s)
  {
    block(_n,new Font(FONT_FAMILY,Font.BOLD,_s),0,"");
  }

  public void p(XmlNode _n)
  {
    block(_n,new Font(FONT_FAMILY,Font.PLAIN,FONT_SIZE),3,"  ");
  }

  public void center(XmlNode _n)
  {
    block(_n,peekFont(),1,"");
  }

  public void div(XmlNode _n)
  {
    block(_n,peekFont(),3,"");
  }

  public void blockquote(XmlNode _n)
  {
    block(_n,peekFont(),3,"",new EmptyBorder(0,50,0,50));
  }

  public void ol(XmlNode _n)
  {
    block(_n,peekFont(),3,"",new EmptyBorder(0,20,0,20));
  }

  public void ul(XmlNode _n)
  {
    block(_n,peekFont(),3,"",new EmptyBorder(0,20,0,20));
  }

  public void li(XmlNode _n)
  {
    block(_n,new Font(FONT_FAMILY,Font.PLAIN,FONT_SIZE),3,"- ");
  }

  public void table(XmlNode _n)
  {
    int nbmaxcols=0;
    int nbmaxrows=0;
    Enumeration er=_n.children_.elements();
    while(er.hasMoreElements())
    {
      Object or=er.nextElement();
      if((or instanceof XmlNode)&&((XmlNode)or).tag_.equals("TR"))
      {
	int nbcols=0;
	Enumeration ec=((XmlNode)or).children_.elements();
	while(ec.hasMoreElements())
	{
          Object oc=ec.nextElement();
	  if(  (oc instanceof XmlNode)
	     &&(  ((XmlNode)oc).tag_.equals("TD")
	        ||((XmlNode)oc).tag_.equals("TH")))
	      nbcols+=getInt((XmlNode)oc,"COLSPAN",1);
	}
	nbmaxrows++;
	nbmaxcols=Math.max(nbmaxcols,nbcols);
      }
    }

    System.err.println("Table with "+nbmaxcols+" columns and "+
		       nbmaxrows+" rows.");
    int old_cellspacing=cellspacing_;
    int old_cellpadding=cellpadding_;
    int old_cellborder =cellborder_;

    cellspacing_=getInt(_n,"CELLSPACING",2);
    cellpadding_=getInt(_n,"CELLPADDING",2);
    cellborder_ =getInt(_n,"BORDER"     ,0);

    XmlBlock p=new XmlBlock();
    p.setLayout(new XmlTableLayout
	(nbmaxcols,nbmaxrows,cellspacing_,cellspacing_));
    //p.setPreferredWidth (getWidth (_n,0));
    //p.setPreferredHeight(getHeight(_n,0));
    p.setOpaque(false);
    //p.setBorder(new BuLightBorder(BuLightBorder.RAISED,1));

    Color bg=getColor(_n,"BGCOLOR",null);
    if(bg!=null)
    {
      p.setOpaque(true);
      p.setBackground(bg);
    }

    pushPanel(p);
    children(_n);
    popPanel();

    cellspacing_=old_cellspacing;
    cellpadding_=old_cellpadding;
    cellborder_ =old_cellborder;

    p.putClientProperty("XML_BREAKBEFORE",Boolean.TRUE);
    p.putClientProperty("XML_BREAKAFTER" ,Boolean.TRUE);
    insert(p);
  }

  public void tr(XmlNode _n)
  {
    Color old=trbg_;
    trbg_=getColor(_n,"BGCOLOR",null);
    children(_n);
    trbg_=old;
  }

  public void th(XmlNode _n)
  {
    Font f=peekFont();
    pushFont(new Font(f.getFamily(),Font.BOLD,f.getSize()));
    XmlBlock p=block(_n,peekFont(),1,"",new XmlBorder
	(XmlBorder.LOWERED,cellborder_,cellpadding_));
    popFont();

    Color bg=getColor(_n,"BGCOLOR",trbg_);
    if(bg!=null)
    {
      p.setOpaque(true);
      p.setBackground(bg);
    }

    int colspan=getInt(_n,"COLSPAN",1);
    int rowspan=getInt(_n,"ROWSPAN",1);
    if(colspan!=1) p.putClientProperty("XML_COLSPAN",new Integer(colspan));
    if(rowspan!=1) p.putClientProperty("XML_ROWSPAN",new Integer(rowspan));
  }

  public void td(XmlNode _n)
  {
    XmlBlock p=block(_n,peekFont(),3,"",new XmlBorder
	(XmlBorder.LOWERED,cellborder_,cellpadding_));

    Color bg=getColor(_n,"BGCOLOR",trbg_);
    if(bg!=null)
    {
      p.setOpaque(true);
      p.setBackground(bg);
    }

    int colspan=getInt(_n,"COLSPAN",1);
    int rowspan=getInt(_n,"ROWSPAN",1);
    if(colspan!=1) p.putClientProperty("XML_COLSPAN",new Integer(colspan));
    if(rowspan!=1) p.putClientProperty("XML_ROWSPAN",new Integer(rowspan));
  }

  public void img(XmlNode _n)
  {
    URL url=null;
    try { url=new URL(base_,getString(_n,"SRC")); }
    catch(MalformedURLException ex) { }

    Image i=null;

    if(url!=null)
    {
      try
      {
	//System.err.println("Loading image "+url);
	InputStream in=new BufferedInputStream(url.openStream(),65536);
	ByteArrayOutputStream out=new ByteArrayOutputStream();
	while(true)
	{
	  int c=in.read();
	  if(c==-1) break;
	  out.write(c);
	}
	i=BuLib.HELPER.getToolkit().createImage(out.toByteArray());
      }
      catch(Exception ex)
      {
	System.err.println("Problem with image "+url);
      }
    }

    BuPicture r=new BuPicture(i);
    r.setForeground(peekFg());
    r.setCursor(peekCursor());

    String a=peekAction();
    int    b=getInt(_n,"BORDER",(a!=null ? 1 : 0));

    int wi=0;
    int hi=0;
    if(i!=null) wi=Math.max(0,i.getWidth (this));
    if(i!=null) hi=Math.max(0,i.getHeight(this));

    /*
    int w=getWidth (_n,Math.max(0,width_-2*b));
    int h=getHeight(_n,Math.max(0,width_-2*b));
    if(w==-1)
    {
      if(h==-1) { w=wi; h=hi; }
      else      { w=wi*h/hi;  }
    }
    else
    {
      if(h==-1) { h=hi*w/wi; }
    }
    */
    r.setPreferredSize(new Dimension(wi+2*b,hi+2*b));

    if(b>0)
    {
      Color c;
      if(a!=null) c=linkColor_;
      else        c=peekFg();
      r.setBorder(new LineBorder(c,b));
    }

    if(a!=null)
    {
      r.addMouseListener(this);
      r.putClientProperty("XML_ACTION",peekAction());
    }

    insert(r);
  }

  public void pre(XmlNode _n)
  {
    JTextArea r=new JTextArea();
    r.setText(getText(_n));
    r.setFont(new Font("Monospaced",Font.PLAIN,FONT_SIZE));
    r.setForeground(new Color(128,0,0));
    r.setCursor(peekCursor());
    r.putClientProperty("XML_BREAKBEFORE",Boolean.TRUE);
    r.putClientProperty("XML_BREAKAFTER",Boolean.TRUE);
    insert(r);
  }

  public void form(XmlNode _n)
  {
    children(_n);
  }

  public void input(XmlNode _n)
  {
    String type=getString(_n,"TYPE");
    if(type==null) type="BUTTON";
    type=type.toUpperCase();

    String value=getText(getString(_n,"VALUE"));
    int    cols =getInt(_n,"SIZE",getInt(_n,"MAXLENGTH",0));

    JComponent r=null;
    if(type.equals("CHECKBOX"))
      r=new BuCheckBox();
    else
    if(type.equals("BUTTON"))
      r=new BuButton(value!=null ? value : "Push");
    else
    if(type.equals("TEXT"))
      r=new JTextField(value!=null ? value : "Text",cols);
    else
    if(type.equals("PASSWORD"))
      r=new JPasswordField(value!=null ? value : "Password",cols);
    else
    if(type.equals("SUBMIT"))
      r=new BuButton(value!=null ? value : "Submit");
    else
    if(type.equals("RESET"))
      r=new BuButton(value!=null ? value : "Reset");

    if(r!=null)
    {
      r.setToolTipText(type.toLowerCase()+" "+getString(_n,"NAME"));
      r.setCursor(new Cursor(Cursor.HAND_CURSOR));
      if(r instanceof BuCheckBox) r.setOpaque(false);
      else                        r.setOpaque(true);
      r.setEnabled(false);

      insert(r);
    }
  }

  public void select(XmlNode _n)
  {
    BuComboBox r=new BuComboBox();

    r.setToolTipText("select "+getString(_n,"NAME"));
    r.setCursor(new Cursor(Cursor.HAND_CURSOR));
    r.setOpaque(true);
    r.setEnabled(false);

    Enumeration e=_n.children_.elements();
    while(e.hasMoreElements())
    {
      Object o=e.nextElement();
      if(  (o instanceof XmlNode)
	 &&((XmlNode)o).tag_.equals("OPTION"))
	r.addItem(getText((XmlNode)o));
    }

    insert(r);
  }

  public void textarea(XmlNode _n)
  {
    int cols=getInt(_n,"COLS",10);
    int rows=getInt(_n,"ROWS",10);

    JTextArea r=new JTextArea(rows,cols);
    r.setText(getText(getString(_n,"VALUE")));
    r.setToolTipText("textarea "+getString(_n,"NAME"));
    r.setCursor(new Cursor(Cursor.TEXT_CURSOR));
    r.setFont(new Font("Monospaced",Font.PLAIN,FONT_SIZE));
    r.putClientProperty("XML_BREAKBEFORE",Boolean.TRUE);
    r.putClientProperty("XML_BREAKAFTER",Boolean.TRUE);
    r.setEnabled(false);
    insert(new BuScrollPane(r));
  }

  public void link(XmlNode _n)
  {
    String href=getString(_n,"HREF");

    if(href!=null)
    {
      pushAction(href);
      pushFg(linkColor_);
      pushCursor(new Cursor(Cursor.HAND_CURSOR));
    }

    children(_n);

    if(href!=null)
    {
      popCursor();
      popFg();
      popAction();
    }
  }

  public void children(XmlNode _n)
  {
    Enumeration e=_n.children_.elements();
    while(e.hasMoreElements())
    {
      Object o=e.nextElement();
      if(o instanceof String)
	text((String)o);
      else
      if(o instanceof XmlNode)
	element((XmlNode)o);
    }
  }

  public void text(String _n)
  {
    String  t=getText(_n);
    boolean espace_avant=
	((t.length()>0)&&(Character.isWhitespace(t.charAt(0))));
    boolean espace_apres=
	((t.length()>1)&&(Character.isWhitespace(t.charAt(t.length()-1))));

    StringTokenizer st=new StringTokenizer(t," \t\n");
    while(st.hasMoreTokens())
    {
      BuLabel r=new BuLabel();
      String  s=st.nextToken();
      if(espace_avant) s=' '+s;
      if(st.hasMoreTokens()||espace_apres) s+=' ';
      r.setText(s);
      r.setFont(peekFont());
      r.setForeground(peekFg());
      r.setCursor(peekCursor());

      String a=peekAction();
      if(a!=null)
      {
	r.addMouseListener(this);
	r.putClientProperty("XML_ACTION",peekAction());
      }

      insert(r);
      espace_avant=false;
    }
  }

  public void element(XmlNode _n)
  {
    //System.err.println(""+_n.tag_);

    if(_n.tag_.equals("HTML"))
	children(_n);
    else
    if(_n.tag_.equals("BODY"))
	body(_n);
    else
    if(_n.tag_.equals("HEAD"))
	;
    else
    if(_n.tag_.equals("BIG"))
	big(_n);
    else
    if(_n.tag_.equals("B"))
	bold(_n);
    else
    if(_n.tag_.equals("I"))
	italic(_n);
    else
    if(_n.tag_.equals("U"))
	;
    else	    
    if(_n.tag_.equals("SMALL"))
	small(_n);
    else	    
    if(_n.tag_.equals("TT"))
	tt(_n);
    else	    
    if(_n.tag_.equals("FONT"))
	font(_n);
    else
    if(_n.tag_.equals("STRONG"))
	strong(_n);
    else
    if(_n.tag_.equals("EM"))
	em(_n);
    else	    
    if(_n.tag_.equals("CITE"))
	cite(_n);
    else	    
    if(_n.tag_.equals("CODE"))
	code(_n);
    else
    if(_n.tag_.equals("KBD"))
	kbd(_n);
    else	    
    if(_n.tag_.equals("H1"))
	header(_n,FONT_SIZE*18/10);
    else	    
    if(_n.tag_.equals("H2"))
	header(_n,FONT_SIZE*16/10);
    else
    if(_n.tag_.equals("H3"))
	header(_n,FONT_SIZE*12/10);
    else
    if(_n.tag_.equals("H4"))
	header(_n,FONT_SIZE);
    else
    if(_n.tag_.equals("H5"))
	header(_n,FONT_SIZE);
    else
    if(_n.tag_.equals("H6"))
	header(_n,FONT_SIZE);
    else
    if(_n.tag_.equals("A"))
        link(_n);
    else
    if(_n.tag_.equals("BR"))
	br(_n);
    else
    if(_n.tag_.equals("NOBR"))
	nobr(_n);
    else
    if(_n.tag_.equals("HR"))
	hr(_n);
    else
    if(_n.tag_.equals("P"))
	p(_n);
    else
    if(_n.tag_.equals("CENTER"))
	center(_n);
    else
    if(_n.tag_.equals("DIV"))
	div(_n);
    else
    if(  _n.tag_.equals("BLOCKQUOTE")
       ||_n.tag_.equals("BQ"))
	blockquote(_n);
    else
    if(_n.tag_.equals("OL"))
	ol(_n);
    else
    if(_n.tag_.equals("UL"))
	ul(_n);
    else
    if(_n.tag_.equals("LI"))
	li(_n);
    else
    if(_n.tag_.equals("TABLE"))
	table(_n);
    else
    if(_n.tag_.equals("TR"))
	tr(_n);
    else
    if(_n.tag_.equals("TD"))
	td(_n);
    else
    if(_n.tag_.equals("TH"))
	th(_n);
    else
    if(_n.tag_.equals("PRE"))
        pre(_n);
    else
    if(_n.tag_.equals("FORM"))
        form(_n);
    else
    if(_n.tag_.equals("INPUT"))
        input(_n);
    else
    if(_n.tag_.equals("SELECT"))
        select(_n);
    else
    if(_n.tag_.equals("OPTION"))
        ;
    else
    if(_n.tag_.equals("TEXTAREA"))
        textarea(_n);
    else
    if(_n.tag_.equals("IMG"))
        img(_n);
    else
    {
      System.err.println("UNKNOWN TAG: "+_n.tag_);
      children(_n);
    }
  }

  public static void main(String[] _args)
  {
    JFrame       fr=new JFrame("XmlBrowser");
    BuTabbedPane tp=new BuTabbedPane();

    for(int i=0;i<_args.length;i++)
    {
      try
      {
        String f=_args[i];
	URL    u=null;

	try { u=new URL(new URL("file://localhost/"),f); }
	catch(MalformedURLException ex) { }

	/*
	int    m=XmlParser.HTML;
	//if(f.endsWith(".html")||f.endsWith(".htm")) m=XmlParser.HTML;
	//if(f.endsWith(".txt"))                      m=XmlParser.TXT;
	System.err.println("URL: "+u+" MODE: "+m);

	Reader in=new InputStreamReader(u.openStream());

	XmlParser p=new XmlParser(in,f,m);
	XmlReader r=new XmlReader();
	p.setXmlListener(r);
	p.parse();
	XmlNode t=r.getTree();
	//System.out.println(t);
	*/

	XmlBrowser   br=new XmlBrowser();
	BuPanel      ip=new BuPanel();
	//ip.add(br);
	BuScrollPane sp=new BuScrollPane(br);

	/*
	br.base_=u;
	br.element(t);
	*/

	sp.setPreferredSize(new Dimension(640,640));
	br.setPage(u);
	//int j=f.lastIndexOf(File.separator);
	//f=f.substring(j+1);
	tp.addTab(""+(i+1),null,sp,""+u);
      }
      catch(Exception ex)
      {
	ex.printStackTrace();
	//System.err.println(ex);
      }
    }

    fr.getContentPane().add("Center",tp);
    fr.pack();    
    fr.show();
    fr.setLocation(200,100);
  }
}
