/**
 * @modification $Date: 2001/12/03 16:28:08 $
 * @statut       unstable
 * @file         XmlLayout.java
 * @version      0.34
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.xml;

import com.memoire.xml.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;

/**
 * Similar to FlowLayout.
 */

public class XmlLayout
       implements LayoutManager2
{
  private int align_;

  public XmlLayout()
  {
    this(3);
  }

  public XmlLayout(int _align)
  {
    align_=_align;
  }

  public void addLayoutComponent(Component _comp, Object _cstr)
  {
  }

  public void addLayoutComponent(String _name, Component _comp)
  {
  }

  public void removeLayoutComponent(Component _comp)
  {
  }

  public Dimension minimumLayoutSize(Container _parent)
  {
    return new Dimension(0,0);//preferredLayoutSize(_parent);
  }

  public Dimension preferredLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      Insets insets=_parent.getInsets();
      int nbc =_parent.getComponentCount();
      int wmin=_parent.getSize().width-(insets.left+insets.right);
      int wopt=0;
      int hmin=0;

      if(  (_parent instanceof XmlBlock)
	   ||(_parent instanceof XmlBrowser))
	if(wmin>580) wmin=580;

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	if(c.isVisible())
	{
	  Dimension d=c.getPreferredSize();
	  wmin=Math.max(wmin,d.width);
	}
      }

      int w=0;
      int h=0;

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	if(c.isVisible())
	{
	  boolean before=false;
	  if(  (c instanceof JComponent)
	     &&Boolean.TRUE.equals
	       (((JComponent)c).getClientProperty("XML_BREAKBEFORE")))
	      before=true;
	  if(before)
	  {
	    hmin+=h;
	    w=0;
	    h=0;
	  }

	  Dimension d=c.getPreferredSize();

	  if((h>0)&&(w+d.width>wmin))
	  {
	    hmin+=h;
	    w=0;
	    h=0;
	  }
	  w+=d.width;
	  h=Math.max(h,d.height);
	  wopt=Math.max(wopt,w);

	  boolean after=false;
	  if(  (c instanceof JComponent)
	     &&Boolean.TRUE.equals
	       (((JComponent)c).getClientProperty("XML_BREAKAFTER")))
	      after=true;
	  if(after)
	  {
	    hmin+=h;
	    w=0;
	    h=0;
	  }
	}
      }

      hmin+=h;

      return new Dimension(wopt+insets.left+insets.right,
			   hmin+insets.top+insets.bottom);
    }
  }

  public Dimension maximumLayoutSize(Container _parent)
  {
    return null;
    //preferredLayoutSize(_parent);
    //return new Dimension(2000,2000);
  }

  public void invalidateLayout(Container _parent)
  {
    layoutContainer(_parent);
  }

  public float getLayoutAlignmentX(Container _parent)
  {
    return (float)0.5;
  }

  public float getLayoutAlignmentY(Container _parent)
  {
    return (float)0.5;
  }

  public void layoutContainer(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      Insets insets=_parent.getInsets();
      Rectangle bounds = new Rectangle
	(insets.left,
	 insets.top,
	 Math.max(0,_parent.getSize().width -(insets.left+insets.right)),
	 Math.max(0,_parent.getSize().height-(insets.top+insets.bottom)));

      if(  (_parent instanceof XmlBlock)
	   ||(_parent instanceof XmlBrowser))
	if(bounds.width>580) bounds.width=580;

      int nbc=_parent.getComponentCount();
      int x  =0;
      int y  =0;
      int h  =0;

      Vector v=new Vector();
      Vector r=new Vector();

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);
	if(c.isVisible())
	{
	  boolean before=false;
	  if(  (c instanceof JComponent)
	     &&Boolean.TRUE.equals
	       (((JComponent)c).getClientProperty("XML_BREAKBEFORE")))
	      before=true;
	  if(before)
	  {
	    computeRow(bounds,v,r,x,h,true);
	    x=0;
	    y+=h;
	    h=0;
	  }

	  Dimension d=c.getPreferredSize();

	  if(x+d.width>bounds.width)
	  {
	    computeRow(bounds,v,r,x,h,false);
	    x=0;
	    y+=h;
	    h=0;
	  }

	  Rectangle actual=new Rectangle
	    (x,
	     y,
	     d.width,
	     d.height);
	  //c.setBounds(actual.x,actual.y,actual.width,actual.height);

	  v.addElement(c);
	  r.addElement(actual);

	  x+=d.width;
	  h=Math.max(h,d.height);

	  boolean after=false;
	  if(  (c instanceof JComponent)
	     &&Boolean.TRUE.equals
	       (((JComponent)c).getClientProperty("XML_BREAKAFTER")))
	      after=true;
	  if(after)
	  {
	    actual.width=bounds.width-(actual.x-actual.width);
	    x=bounds.width;
	    computeRow(bounds,v,r,x,h,true);
	    //computeRow(bounds,v,r,x,h,true);
	    //c.invalidate();
	    //c.doLayout();
	    x=0;
	    y+=h;
	    h=0;
	  }
	}
      }

      computeRow(bounds,v,r,x,h,true);
    }
  }

  public void computeRow
    (Rectangle _bounds, Vector _v, Vector _r, int _w, int _h, boolean _last)
  {
    int lv=_v.size();
    int dw=_bounds.width-_w;

    for(int i=0;i<lv;i++)
    {
	Component c=(Component)_v.elementAt(i);
	Rectangle r=(Rectangle)_r.elementAt(i);
	int       a=align_;

	if((a==3)&&_last) a=0;

	switch(a)
	{
	case 1: r.x+=dw/2;    break;
	case 2: r.x+=dw;      break;
	case 3: r.x+=dw*i/lv; break;
	}

	if((a==3)&&(i+1==lv)) r.x=_bounds.width-r.width;
	if((a==3)&&(i==0))    r.x=0;

	r.x+=_bounds.x;
	r.y+=_h-r.height;
	if(r.width <0) r.width =0;
	if(r.height<0) r.height=0;
	c.setBounds(r);
    }

    _v.removeAllElements();
    _r.removeAllElements();
  }
}


