/**
 * @modification 2001-10-25
 * @statut       unstable
 * @file         XmlReader.java
 * @version      0.04
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1998-2001 Guillaume Desnoix
 */

package com.memoire.xml;
import  com.memoire.xml.*;

import java.util.*;

public class XmlReader
       implements XmlListener
{
  private int       mode_;

  private XmlNode   current_;
  private Stack     nodes_     =new Stack();
  private Hashtable attributes_=new Hashtable(11);

  private String origin_;
  private int    lineno_;
  private int    charno_;

  public XmlReader(int _mode)
  {
    mode_=_mode;
  }
  
  public void location(String _origin, int _lineno, int _charno)
  {
    origin_=_origin;
    lineno_=_lineno;
    charno_=_charno;
  }

  public void startElement(String _tag)
  {
    //System.err.println("START "+_tag);

    XmlNode n=new XmlNode();
    n.tag_=_tag;
    n.attributes_=attributes_;
    attributes_=new Hashtable(11);

    if(current_!=null)
    {
      if(  (mode_==XmlParser.HTML)
	 &&current_.tag_.equals(_tag)&&!_tag.equals("FONT"))
	missingTag(_tag);
    }

    if(current_!=null) current_.children_.addElement(n);
    current_=n;
    nodes_.push(n);
  }

  private void missingTag(String _tag)
  {
    if(  !_tag.equals("P")
       &&!_tag.equals("LI")
       &&!_tag.equals("OPTION")
       &&!_tag.equals("A"))
      System.err.println("MISSING END: "+_tag);

    endElement(_tag);
  }

  public void endElement(String _tag)
  {
    if(nodes_.size()==0)
    {
      System.err.println("END NEEDED "+_tag);
      return;
    }

    XmlNode n=(XmlNode)nodes_.peek();

    if(_tag.equals(n.tag_))
    {
      nodes_.pop();
    }
    else
    {
      if((mode_==XmlParser.HTML)
	 &&(  n.tag_.equals("P")
	    ||n.tag_.equals("LI")
	    ||n.tag_.equals("OPTION")
	    ||_tag.equals("FONT")))
	nodes_.pop(); //System.err.println("=== "+_tag);
      else
      {
	//dump();

	nodes_.pop();
	if(nodes_.size()>0) n=(XmlNode)nodes_.peek();

	if(  !_tag.equals(n.tag_)
	   &&(  _tag.equals("TABLE")
	      ||_tag.equals("TR")
	      ||_tag.equals("TD")))	   
	{
	  nodes_.pop();
	  if(nodes_.size()>0) n=(XmlNode)nodes_.peek();
	}

	if(!_tag.equals(n.tag_))
	{
	  error("block: "+n.tag_+" /"+_tag);
	  //dump();
	}
      }
    }

    if(nodes_.size()>0) current_=(XmlNode)nodes_.peek();
  }

  private void dump()
  {
    String t="";
    while(!nodes_.isEmpty())
    {
      XmlNode n=(XmlNode)nodes_.pop();
      System.err.println(t+n.tag_); t+="  ";
    }
    System.exit(-3);
  }

  public void attribute(String _name,String _value)
  {
    attributes_.put(_name,_value);
  }

  public void text(String _data)
  {
    current_.children_.addElement(_data);
  }

  public void error(String _message)
  {
    System.err.println("ERROR "+_message+" ["+lineno_+":"+charno_+"]");
    //System.exit(1);
  }

  public XmlNode getTree()
  {
    return current_;
  }

  public static void main(String[] _args)
  {
    for(int i=0;i<_args.length;i++)
    {
      try
      {
        String f=_args[i];
	int    m=XmlParser.XML;
	if(f.endsWith(".html")||f.endsWith(".htm")) m=XmlParser.HTML;
	if(f.endsWith(".txt"))                      m=XmlParser.TXT;
	System.err.println("FILE: "+f+" MODE: "+m);
	XmlParser p=new XmlParser(f,m);
	XmlReader r=new XmlReader(m);
	p.setXmlListener(r);
	p.parse();
	System.out.println(r.getTree());
      }
      catch(Exception ex)
      {
	ex.printStackTrace();
	//System.err.println(ex);
      }
    }
  }
}
