/**
 * @product      Alma (Atelier logiciel de modelistion et d'analyse)
 * @file         XmlTableLayout.java
// Version: 0.32
// Java   : 12
// Date   : 14/05/01
 * @author       Guillaume Desnoix
// Mel    : guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.xml;
import com.memoire.xml.*;

import java.awt.* ;
import java.util.* ;
import javax.swing.* ;

/**
 * Lays out in a grid a set of components.
 * Not like the java.awt.GridLayout, rows may have different heigths
 * and columns may have different widths.
 */

public class XmlTableLayout
       implements LayoutManager2
{
  public XmlTableLayout()
  {
      this(3,3,0,0);
  }

  public XmlTableLayout(int _columns, int _rows)
  {
    this(_columns,_rows,0,0);
  }

  public XmlTableLayout(int _columns, int _rows, int _hgap, int _vgap)
  {
    columns_=_columns;
    rows_   =_rows;
    hgap_   =_hgap;
    vgap_   =_vgap;
  }

  protected int columns_;
  public int getColumns() { return columns_; }
  public void setColumns(int _columns) { columns_=_columns; }

  protected int rows_;
  public int getRows() { return rows_; }
  public void setRows(int _rows) { rows_=_rows; }

  protected int hgap_;
  public int getHgap() { return hgap_; }
  public void setHgap(int _hgap) { hgap_=_hgap; }

  protected int vgap_;
  public int getVgap() { return vgap_; }
  public void setVgap(int _vgap) { vgap_=_vgap; }

  public void addLayoutComponent(Component _comp, Object _cstr)
  {
  }

  public void addLayoutComponent(String _name, Component _comp)
  {
  }

  public void removeLayoutComponent(Component _comp)
  {
  }

  public Dimension minimumLayoutSize(Container _parent)
  {
    return new Dimension(0,0);
  }

  private void compute
    (Container _parent, int nbc, int[] wcol, int[] hrow, Component[][] cmp)
  {
      int i=0;
      for(int row=0;row<rows_; row++)
	for(int col=0;col<columns_; col++)
        {
	  if(cmp[col][row]!=null) continue;
	  if(i>=nbc) break;

	  Component c=_parent.getComponent(i);
	  Dimension d=c.getPreferredSize();

	  int cspan=1;
	  int rspan=1;
	  if(c instanceof JComponent)
	  {
	    Integer icp;
	    icp=(Integer)((JComponent)c).getClientProperty("XML_COLSPAN");
	    if(icp!=null) cspan=icp.intValue();
	    icp=(Integer)((JComponent)c).getClientProperty("XML_ROWSPAN");
	    if(icp!=null) rspan=icp.intValue();
	  }

	  if(cspan==1) wcol[col]=Math.max(wcol[col],d.width);
	  if(rspan==1) hrow[row]=Math.max(hrow[row],d.height);

	  for(int k=0;k<cspan;k++)
	    for(int l=0;l<rspan;l++)
	      cmp[Math.min(col+k,columns_-1)]
		 [Math.min(row+l,rows_-1   )]=c;

	  //col+=(cspan-1);
	  i++;
	}

      // Ajustement
      Component[][] tmp=new Component[columns_][rows_];
      i=0;
      for(int row=0;row<rows_; row++)
	for(int col=0;col<columns_; col++)
        {
	  if(tmp[col][row]!=null) continue;
	  if(i>=nbc) break;

	  Component c=_parent.getComponent(i);
	  Dimension d=c.getPreferredSize();

	  int cspan=1;
	  int rspan=1;
	  if(c instanceof JComponent)
	  {
	    Integer icp;
	    icp=(Integer)((JComponent)c).getClientProperty("XML_COLSPAN");
	    if(icp!=null) cspan=icp.intValue();
	    icp=(Integer)((JComponent)c).getClientProperty("XML_ROWSPAN");
	    if(icp!=null) rspan=icp.intValue();
	  }

	  if(cspan>1)
	  {
	    int w=0;
	    for(int k=col;k<Math.min(col+cspan,columns_);k++)
		w+=wcol[k]+((k>col) ? 2*hgap_ : 0);

	    if(w<d.width)
	      wcol[Math.min(col+cspan,columns_)-1]+=(d.width-w);
	  }
	  if(rspan!=1)
	  {
	    int h=0;
	    for(int k=row;k<Math.min(row+rspan,rows_);k++)
		h+=hrow[k]+((k>row) ? 2*vgap_ : 0);

	    if(h<d.height)
	      hrow[Math.min(row+rspan,rows_)-1]+=(d.height-h);
	  }

	  for(int k=0;k<cspan;k++)
	    for(int l=0;l<rspan;l++)
	      tmp[Math.min(col+k,columns_-1)]
		 [Math.min(row+l,rows_-1   )]=c;

	  i++;
	}
  }

  public Dimension preferredLayoutSize(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      int w  =0;
      int h  =0;

      int           nbc =_parent.getComponentCount();
      int[]         wcol=new int[columns_];
      int[]         hrow=new int[rows_   ];
      Component[][] cmp =new Component[columns_][rows_];

      compute(_parent,nbc,wcol,hrow,cmp);

      /*
      int i=0;
      for(int row=0;row<rows_; row++)
	for(int col=0;col<columns_; col++)
        {
	  if(cmp[col][row]!=null) continue;
	  if(i>=nbc) break;

	  Component c=_parent.getComponent(i);
	  Dimension d=c.getPreferredSize();

	  int cspan=1;
	  int rspan=1;
	  if(c instanceof JComponent)
	  {
	    Integer icp;
	    icp=(Integer)((JComponent)c).getClientProperty("XML_COLSPAN");
	    if(icp!=null) cspan=icp.intValue();
	    icp=(Integer)((JComponent)c).getClientProperty("XML_ROWSPAN");
	    if(icp!=null) rspan=icp.intValue();
	  }

	  if(cspan==1) wcol[col]=Math.max(wcol[col],d.width);
	  if(rspan==1) hrow[row]=Math.max(hrow[row],d.height);

	  for(int k=0;k<cspan;k++)
	    for(int l=0;l<rspan;l++)
	      cmp[Math.min(col+k,columns_-1)]
		 [Math.min(row+l,rows_-1   )]=c;

	  //col+=(cspan-1);
	  i++;
	}
      */

      for(int i=0;i<columns_;i++)
      {
	//if(i>0) w+=hgap_;
	w+=2*hgap_+wcol[i];
      }

      for(int i=0;i<rows_;i++)
      {
	//if(i>0) h+=vgap_;
	h+=2*vgap_+hrow[i];
      }

      Insets insets=_parent.getInsets();
      return new Dimension(w+insets.left+insets.right,
			   h+insets.top+insets.bottom);
    }
  }

  public Dimension maximumLayoutSize(Container _parent)
  {
    return new Dimension(2000,2000);
  }

  public void invalidateLayout(Container _parent)
  {
    layoutContainer(_parent);
  }

  public float getLayoutAlignmentX(Container _parent)
  {
    return (float)0.5;
  }

  public float getLayoutAlignmentY(Container _parent)
  {
    return (float)0.5;
  }

  public void layoutContainer(Container _parent)
  {
    synchronized (_parent.getTreeLock())
    {
      Insets    insets=_parent.getInsets();
      Dimension size  =_parent.getSize();
      Rectangle bounds=new Rectangle
	(insets.left,
	 insets.top,
	 size.width-(insets.left + insets.right),
	 size.height-(insets.top + insets.bottom));

      int nbc=_parent.getComponentCount();
      int[] wcol=new int[columns_];
      int[] hrow=new int[rows_   ];
      Component[][] cmp=new Component[columns_][rows_];

      compute(_parent,nbc,wcol,hrow,cmp);

      /*
      int i=0;
      for(int row=0;row<rows_; row++)
	for(int col=0;col<columns_; col++)
        {
	  if(cmp[col][row]!=null) continue;
	  if(i>=nbc) break;

	  Component c=_parent.getComponent(i);
	  Dimension d=c.getPreferredSize();

	  int cspan=1;
	  int rspan=1;
	  if(c instanceof JComponent)
	  {
	    Integer icp;
	    icp=(Integer)((JComponent)c).getClientProperty("XML_COLSPAN");
	    if(icp!=null) cspan=icp.intValue();
	    icp=(Integer)((JComponent)c).getClientProperty("XML_ROWSPAN");
	    if(icp!=null) rspan=icp.intValue();
	  }

	  if(cspan==1) wcol[col]=Math.max(wcol[col],d.width);
	  if(rspan==1) hrow[row]=Math.max(hrow[row],d.height);

	  for(int k=0;k<cspan;k++)
	    for(int l=0;l<rspan;l++)
	      cmp[Math.min(col+k,columns_-1)]
		 [Math.min(row+l,rows_-1   )]=c;

	  //col+=(cspan-1);
	  i++;
	}
      */

      for(int i=0;i<nbc;i++)
      {
	Component c=_parent.getComponent(i);

	for(int col=0;col<columns_; col++)
	  for(int row=0;row<rows_; row++)
	    if(c==cmp[col][row])
	    {
	      int cspan=1;
	      int rspan=1;
	      if(c instanceof JComponent)
	      {
		Integer icp;
		icp=(Integer)((JComponent)c).getClientProperty("XML_COLSPAN");
		if(icp!=null) cspan=icp.intValue();
		icp=(Integer)((JComponent)c).getClientProperty("XML_ROWSPAN");
		if(icp!=null) rspan=icp.intValue();
	      }

	      int x=bounds.x+hgap_;
	      int y=bounds.y+vgap_;
	      int w=0;
	      int h=0;

	      for(int k=0;k<col;k++)
		  x+=2*hgap_+wcol[k];
	      for(int k=0;k<row;k++)
		  y+=2*vgap_+hrow[k];
	      for(int k=col;k<Math.min(col+cspan,columns_);k++)
		  w+=wcol[k]+((k>col) ? 2*hgap_ : 0);
	      for(int k=row;k<Math.min(row+rspan,rows_   );k++)
		  h+=hrow[k]+((k>row) ? 2*vgap_ : 0);
	      
	      c.setBounds(x,y,w,h);
	      row=rows_;
	      col=columns_;
	      // fini
	    }
      }
    }
  }
}
