/**
 * @modification $Date: 2001/12/03 16:28:09 $
 * @statut       unstable
 * @file         YapodStaticDoubleArrayQuery.java
 * @version      0.17
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.yapod;
import  com.memoire.yapod.*;


import java.util.*;

public class YapodStaticDoubleArrayQuery
       implements YapodQuery
{
  private double[] values_;

  public YapodStaticDoubleArrayQuery(double[] _values)
  {
    if(_values==null)
      throw new IllegalArgumentException("_values is null");

    values_=_values;
  }

  public final Enumeration getResult()
    { return new Enumerator(values_); }

  public String toString()
  {
    StringBuffer r=new StringBuffer();

    r.append('[');
    for(int i=0;i<values_.length;i++)
    {
      if(i!=0) r.append(", ");
      r.append(values_[i]);
    }
    r.append(']');

    return r.toString();
  }

  private final class Mutable
                extends Number
  {
    private double value_;
    public final double getValue() { return value_; }
    public final void   setValue(double _value) { value_=_value; }

    public final int    intValue   () { return (int)  value_; }
    public final long   longValue  () { return (long) value_; }
    public final float  floatValue () { return (float)value_; }
    public final double doubleValue() { return        value_; }

    public String toString()
      { return "mutable("+value_+")"; }
    public Object clone()
      { Mutable r=new Mutable(); r.setValue(value_); return r; }
  }

  private final class Enumerator implements Enumeration
  {
    private double[] values_;
    private int      index_;
    private Mutable  mutan_;

    public Enumerator(double[] _values)
    {
      values_=_values;
      index_ =0;
      mutan_ =new Mutable();
    }

    public final boolean hasMoreElements()
      { return index_<values_.length; }

    public final Object nextElement()
    {
      if(index_>=values_.length)
	throw new NoSuchElementException();
      // return new Double(values_[index_++]);
      mutan_.setValue(values_[index_++]);
      return mutan_;
    }
  }
}
