/**
 * @modification $Date: 2001/12/03 16:28:09 $
 * @statut       unstable
 * @file         YapodTestCollector.java
 * @version      0.17
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.yapod;
import  com.memoire.yapod.*;


import java.io.*;
import java.util.*;

public class YapodTestCollector
{
  public static void main(String[] args)
  {
    // System.setSecurityManager(new YapodSecurityManager());

    String         nm="test.xml";
    YapodCollector cl=null;
    
    if(!YapodCollector.exists(nm))
    {
      cl=YapodCollector.create(nm);

      System.err.println("feeding...");
      cl.add("Hello");
      for(int i=0;i<100;i++)
	cl.add(new Integer(i));

      for(int i=0;i<10;i++)
	cl.add(new YapodTestC3(i,2*i));

      cl.close();
      return;
    }

    cl=YapodCollector.open(nm);
    cl.startAutoSave();

    YapodQuery q0,q1,q2,q3,q4,q5,q6,q7,q8;

    q0=cl.createQuery();
    q0=new YapodNumberQuery(q0);
    // YapodLib.print(q0.getResult());

    q1=new YapodCompareQuery("this","<",new Integer(12),q0);
    YapodLib.print(q1.getResult());

    q2=new YapodCompareQuery("doubleValue",">",new Double(92),q0);
    YapodLib.print(q2.getResult());

    q3=new YapodUnionQuery(q1,q2);
    YapodLib.print(q3.getResult());

    q4=new YapodAbstractTestQuery(q3)
      {
	protected boolean test(Object o)
	  { return ((Number)o).intValue()%2==0; }
        public String getText()
	  { return "parity"; }
      };
    YapodLib.print(q4.getResult());

    q4=new YapodMathQuery("this","/",new Integer(2),q4);
    YapodLib.print(q4.getResult());

    Vector v5=new Vector();
    v5.addElement(new Integer(4));
    v5.addElement(new Integer(5));
    v5.addElement(new Integer(6));

    /*
    q5=new YapodStaticVectorQuery(v5);
    YapodLib.print(q5.getResult());
    q5=new YapodCastQuery("double",q5);
    YapodLib.print(q5.getResult());
    */

    q5=new YapodStaticDoubleArrayQuery(new double[] { 3.,4.,5. });
    YapodLib.print(q5.getResult());
    q5=new YapodCastQuery("double",q5);
    YapodLib.print(q5.getResult());

    q6=new YapodDifferenceQuery(q4,q5);
    YapodLib.print(q6.getResult());

    q7=new YapodCountQuery(q6);
    YapodLib.print("count=",q7.getResult());
    q7=new YapodMaxQuery(q6);
    YapodLib.print("max=",q7.getResult());
    q7=new YapodMinQuery(q6);
    YapodLib.print("min=",q7.getResult());
    q7=new YapodAvgQuery(q6);
    YapodLib.print("avg=",q7.getResult());

    YapodLib.print(q7);

    q7=new YapodExternalQuery("com.memoire.yapod.YapodTestCollector.sin",q1);
    YapodLib.print(q7.getResult());

    q7=new YapodCategoriesQuery("this",q1);
    YapodLib.print(q7.getResult());

    // System.err.println(""+new Double(5).equals(new Float(5))); -->false !!
    // System.err.println(""+("YapodTestC3".equals
    //                    (new StringBuffer("YapodTestC3"))));

    q0=cl.createQuery();
    q0=new YapodClassQuery(YapodTestC3.class,q0);
    YapodLib.print(q0.getResult());

    q1=new YapodExplodeQuery(q0);
    YapodLib.print(q1.getResult());

    q2=new YapodAccessQuery("a",q1);
    YapodLib.print(q2.getResult());

    q3=new YapodAccessQuery("b",q1);
    YapodLib.print(q3.getResult());

    q4=new YapodIntervalQuery(3,6,q3);
    YapodLib.print(q4.getResult());

    q5=new YapodSetQuery(q2,q3);
    YapodLib.print(q5.getResult());

    /*
    q6=new YapodConcreteQuery(q5);
    print(q6);
    print(q6.getResult());
    */

    YapodAbstractQuery qp=new YapodMathQuery
      ("this","+",new Integer(1),YapodConstants.DUMMY);
    q7=new YapodMapQuery(qp,q5);
    YapodLib.print(q7.getResult());
    YapodLib.print(q7);

    cl.stopAutoSave();
    cl.close();
    // try { Thread.currentThread().sleep(30000); }
    // catch(Exception ex) { }
  }

  public static Object sin(Object _o)
  {
    Object r=YapodConstants.FAKE;
    if(_o instanceof Number)
      r=new Float(Math.sin(((Number)_o).doubleValue()));
    return r;
  }
}

