/**
 * @modification $Date: 2001/12/03 16:28:09 $
 * @statut       unstable
 * @file         YapodXmlSerializer.java
 * @version      0.17
 * @author       Guillaume Desnoix
 * @email        guillaume@desnoix.com
 * @license      GNU General Public License 2 (GPL2)
 */

package com.memoire.yapod;

import  com.memoire.yapod.*;
import java.lang.reflect.*;
import java.io.*;
import java.util.*;

public class YapodXmlSerializer
       implements YapodSerializer
{
  public final static boolean INDENTATION=true; // false;
  public final static String  ENCODING="UTF-8"; //"ISO-8859-1"

  protected OutputStream out;
  protected Hashtable    ref; // (obj,id)
  protected Hashtable    obj; // (id,obj)
  protected Object       root;
  protected int          indentation_;

  public YapodXmlSerializer()
  {
  }

  public String extension()
  {
    return "xml";
  }

  // property BufferSize

  /*
  private int bufferSize_=4096;

  public int getBufferSize()
  {
    return bufferSize_;
  }

  public void setBufferSize(int _bufferSize)
  {
    //bufferSize_=_bufferSize;
  }
  */

  private final static String SPACES=
    "                                                            "+
    "                                                            "+
    "                                                            ";

  protected String indente()
  {
    /*
    String r="";
    int    i=2*indentation_;

    while(i>=30)
    {
      r+=SPACES;
      i-=30;
    }
    r+=SPACES.substring(0,i);

    return r;
    */

    return SPACES.substring(0,Math.min(indentation_,SPACES.length()));
  }

  public static void warning(String _s)
  {
    System.err.println("Yapod XML warning: "+_s);
  }

  protected static void output(OutputStream _w, String _o)
      throws IOException
  {
    //try { 
    output0(_w,_o.getBytes(ENCODING));
    //}
    //catch(UnsupportedEncodingException ex)
    //{ warning(ex.getMessage()); }
  }

  protected static void output0(OutputStream _w, byte[] _o)
      throws IOException
  {
    _w.write(_o);
  }

  public synchronized void open(OutputStream _out)
      throws IOException
  {
    if(!(_out instanceof BufferedOutputStream))
      _out=new BufferedOutputStream(_out,BUFFER_SIZE);

    // OW+BO ou BW+OW
    //out=new OutputStreamWriter(new BufferedOutputStream(_out,8192));

    idbase_=1;
    ref=new Hashtable();
    obj=new Hashtable();
    out=_out; //new OutputStreamWriter(_out);

    indentation_=0;
    writeHeader(out);
  }

  public synchronized void close()
      throws IOException
  {
    writeFooter(out);
    out.close();
  }

  public synchronized void store
  (Object _o, Object dbcx_, Hashtable _ref, Hashtable _obj)
       throws IOException
  {
    String _dir=(String)dbcx_;
    String k=(String)_ref.get(_o);
    if(k==null)
      throw new IllegalArgumentException
	("object not referenced: "+_o);

    Hashtable old_ref=ref;
    Hashtable old_obj=obj;
    ref=_ref;
    obj=_obj;
    //ref.remove(_o);
    //obj.remove(k);
    root=_o;
    indentation_=0;

    out=new BufferedOutputStream(new FileOutputStream
	(_dir+File.separator+k+".xml"),8192);
    output(out,"<?xml version=\"1.0\" encoding=\""+ENCODING+"\"?>\n");
    output(out,"<?xml-stylesheet href=\"yapod.css\" type=\"text/css\"?>\n");
    output(out,"<!DOCTYPE yapod>\n"); // SYSTEM \"yapod.dtd\"
    output(out,"<yapod>\n");
    write(_o);
    output(out,"</yapod>\n");
    out.close();

    root=null;
    //ref.put(_o,k);
    //obj.put(k,_o);
    ref=old_ref;
    obj=old_obj;
  }

  private int idbase_=1;

  protected String getId(Object _o)
  {
    if(_o==null) return "null";

    String r=(String)ref.get(_o);

    if(r==null)
    {
      do
      {
	r="RO-"+idbase_;
	idbase_++;
      } while(obj.get(r)!=null);

      ref.put(_o,r);
      obj.put(r,_o);
    }

    return r;
  }

  public final synchronized void write(Object _o)
      throws IOException
  {
    indentation_++;
    write(_o,out);
    indentation_--;
  }

  // Write parts into a OutputStream

  protected synchronized void writeHeader(OutputStream _w)
      throws IOException
  {
    output(_w,"<?xml version=\"1.0\" encoding=\""+ENCODING+"\"?>\n");
    output(_w,"<?xml-stylesheet href=\"yapod.css\" type=\"text/css\"?>\n");
    output(_w,"<!DOCTYPE yapod>\n"); // SYSTEM \"yapod.dtd\"
    output(_w,"<yapod>\n");
  }

  protected synchronized void writeFooter(OutputStream _w)
      throws IOException
  {
    output(_w,"</yapod>\n");
  }

  protected synchronized void writeNull(OutputStream _w)
      throws IOException
  {
    if(INDENTATION) output(_w,indente());
    output(_w,"<null/>\n");
  }

  protected synchronized void writeBoolean(Boolean _o, OutputStream _w)
      throws IOException
  {
    if(INDENTATION) output(_w,indente());
    output(_w,"<single");
    output(_w,getTypeAttr(_o));
    output(_w,">");
    output(_w,_o.toString());
    output(_w,"</single>\n");
  }

  protected synchronized void writeNumber(Number _o, OutputStream _w)
      throws IOException
  {
    if(INDENTATION) output(_w,indente());
    output(_w,"<single");
    output(_w,getTypeAttr(_o));
    output(_w,">");
    output(_w,_o.toString());
    output(_w,"</single>\n");
  }

  protected synchronized void writeCharacter(Character _o, OutputStream _w)
      throws IOException
  {
    if(INDENTATION) output(_w,indente());
    output(_w,"<single");
    output(_w,getTypeAttr(_o));
    output(_w,">");
    output(_w,""+(int)_o.charValue());
    output(_w,"</single>\n");
  }

  protected synchronized void writeString(String _o, OutputStream _w)
      throws IOException
  {
    if(INDENTATION) output(_w,indente());
    output(_w,"<single");
    output(_w,getTypeAttr(_o));
    output(_w,">");
    output(_w,toXmlCharset(_o));
    output(_w,"</single>\n");
  }

  protected synchronized void writeBytes(byte[] _o, OutputStream _w)
      throws IOException
  {
    if(INDENTATION) output(_w,indente());
    output(_w,"<array-byte><![CDATA[");
    char[] s=new char[_o.length];
    for(int i=0;i<_o.length;i++) s[i]=(char)_o[i];
    output(_w,new String(s));
    //output(_w,new String(_o,ENCODING));
    output(_w,"]]></array-byte>\n");
  }

  /*
  protected synchronized void writeInts(int[] _o, OutputStream _w)
      throws IOException
  {
    int    l=_o.length;
    char[] o=new char[l];
    System.arraycopy(_o,0,o,0,l);

    if(INDENTATION) output(_w,indente());
    output(_w,"<array-int><![CDATA[");
    output(_w,new String(o));
    output(_w,"]]></array-int>\n");
  }
  */

  protected synchronized void writeArray(Object _o, OutputStream _w)
      throws IOException
  {
    int l=Array.getLength(_o);

    if(INDENTATION) output(_w,indente());
    output(_w,"<array");
    output(_w,getTypeAttr(_o));
    output(_w,getDepthAttr(_o));
    output(_w," id=\"");
    output(_w,getId(_o));
    output(_w,"\" length=\"");
    output(_w,""+l);
    output(_w,"\">\n");
    indentation_++;

    for(int i=0;i<l;i++)
      write(Array.get(_o,i),_w);

    indentation_--;
    if(INDENTATION) output(_w,indente());
    output(_w,"</array>\n");
  }

  protected synchronized void writeObject(Object _o, OutputStream _w)
      throws IOException
  {
    Class c=_o.getClass();

    // if(!(_o instanceof Serializable))
    //  warning("Not serializable "+c.getName());

    try
    {
      c.getMethod
	("writeObject",new Class[] { ObjectOutputStream.class });
      warning("Specific serialization for "+c.getName());
    }
    catch(NoSuchMethodException ex) { }

    synchronized(_o)
    {
      if(INDENTATION) output(_w,indente());
      output(_w,"<object");
      output(_w,getTypeAttr(_o));
      output(_w," id=\"");
      output(_w,getId(_o));
      output(_w,"\">\n");
      indentation_++;

      Field[] fields=YapodLib.getAllFields(_o.getClass());
      for(int i=0;i<fields.length;i++)
	writeField(_o,fields[i],_w);

      indentation_--;
      if(INDENTATION) output(_w,indente());
      output(_w,"</object>\n");
    }
  }

  protected synchronized void writeReference(Object _o, OutputStream _w)
      throws IOException
  {
    if(INDENTATION) output(_w,indente());
    output(_w,"<reference idref=\"");
    output(_w,getId(_o));
    output(_w,"\"/>\n");
  }

  protected void writeField(String _name, Object _value, OutputStream _w)
      throws IOException
  {
    if(INDENTATION) output(_w,indente());
    output(_w,"<field name=\"");
    output(_w,_name);
    output(_w,"\">\n");
    indentation_++;

    write(_value,_w);

    indentation_--;
    if(INDENTATION) output(_w,indente());
    output(_w,"</field>\n");
  }

  // General write

  public synchronized void write(Object _o, OutputStream _w)
      throws IOException
  {
    if(_o==null)
      writeNull(_w);
    else
    if(_o instanceof Boolean)
      writeBoolean((Boolean)_o,_w);
    else
    if(_o instanceof Number)
      writeNumber((Number)_o,_w);
    else
    if(_o instanceof Character)
      writeCharacter((Character)_o,_w);
    else
    if(_o instanceof String)
      writeString((String)_o,_w);
    else
    if(_o instanceof byte[])
      writeBytes((byte[])_o,_w);
    /*
    else
    if(_o instanceof int[])
      writeInts((int[])_o,_w);
    */
    else
    if(_o.getClass().isArray())
      writeArray(_o,_w);
    else
    {
      if((root==_o)||(ref.get(_o)==null))
	writeObject(_o,_w);

      writeReference(_o,_w);
    }
  }

  protected void writeField(Object _o, Field _f, OutputStream _w)
      throws IOException
  {
    // System.err.println(""+_f);
    if(!YapodLib.isValid(_f)) return;

    YapodLib.setAccessible(_f,true);

    Object FAKE_VALUE=new Object();
    Object value=FAKE_VALUE;

    try
    {
      value=_f.get(_o);
    }
    catch(IllegalAccessException ex1)
    {
      String fn=_f.getName();
      // System.err.println("Access error: "+fn+" in "+_o.getClass());

      Method m=null;

      try { m=_o.getClass().getMethod(fn,new Class[0]); }
      catch(NoSuchMethodException ex2) { }

      if((m==null)&&(fn.endsWith("_")))
      {
	fn=fn.substring(0,fn.length()-1);
	try { m=_o.getClass().getMethod(fn,new Class[0]); }
	catch(NoSuchMethodException ex3) { }
      }

      if(m==null)
      {
	fn="get"+fn.substring(0,1).toUpperCase()+fn.substring(1);
	try { m=_o.getClass().getMethod(fn,new Class[0]); }
	catch(NoSuchMethodException ex4) { }
      }

      if(m!=null)
      {
	try
	{
	  value=m.invoke(_o,new Object[0]);
	  // System.err.println("Using method "+fn+"()");
	}
        catch(IllegalAccessException ex5) { }
        catch(InvocationTargetException ex6) { }
      }
    }

    if(value==FAKE_VALUE)
    {
      warning("No access to field "+_f.getName()+
	      " in "+_o.getClass());
      value=null;
    }

    writeField(_f.getName(),value,_w);
  }

  private String getTypeAttr(Object _value)
  {
    if(_value==null) return "";

    StringBuffer r=new StringBuffer(30);
    r.append(" type=\"");

    Class c=_value.getClass();
    while(c.isArray()) c=c.getComponentType();

    r.append(c.getName());
    r.append("\"");
    return r.toString();
  }

  private String getDepthAttr(Object _value)
  {
    if(_value==null) return "";

    Class c=_value.getClass();
    while(!c.isArray()) return "";

    StringBuffer r=new StringBuffer(30);
    r.append(" depth=\"");

    int n=0;
    while(c.isArray()) { c=c.getComponentType(); n++; }
    r.append(""+n);

    r.append("\"");
    return r.toString();
  }

  protected final static String toXmlCharset(char _c)
  {
    String r;

    if(_c=='<') r="&lt;";
    else
    if(_c=='>') r="&gt;";
    else
    if(_c==' ') r="&#32;";
    else
    if(_c=='&') r="&amp;";
    else
    if(_c=='"') r="&quot;";
    else
    if(Character.isISOControl(_c)||(_c=='\'')||(_c=='\"')||(_c>254))
      r="&#"+(int)_c+";";
    else
      r=""+_c;
    return r;
  }
    
  protected final static String toXmlCharset(Character _c)
  {
    return toXmlCharset(_c.charValue());
  }

  protected final static String toXmlCharset(String _s)
  {
    StringBuffer r=new StringBuffer(2*_s.length());
    int          l=_s.length();

    for(int i=0;i<l;i++)
    {
      char c=_s.charAt(i);
      r.append(toXmlCharset(c));
    }
    
    return r.toString();
  }

  public static void main(String[] args)
  {
    try
    {
      YapodSerializer out=new YapodXmlSerializer();
      OutputStream xxx=new FileOutputStream("xxx.xml");
      // OutputStream xxx=System.out;
      out.open(xxx);

      /*Hashtable h=new Hashtable(11);
      h.put("C3",new YapodTestC3(19,70));
      h.put("C2",new YapodTestC2());
      h.put("C1",new YapodTestC1());
      out.write(h);*/
      out.write(new YapodTestC3(19,70));
      out.write(new YapodTestC2());
      out.write(new YapodTestC1());
      out.close();
    }
    catch(IOException ex) { ex.printStackTrace(); }
  }
}
